`timescale 1ps/1ps

module PERFRO16_TSMCN7B1 (
	RO_en, 
	RO_en_all, 
	RO_cntr_reset, 
	RO_cntr_ovflw, 
	RO_cntr, 
	RO_ocv_sel, 
	RO_sel,
	ext_clk,
	clk_sel);

   input [1:0] RO_ocv_sel;
   input [3:0] RO_sel;
   input ext_clk,clk_sel;

   input RO_en;
   input RO_en_all;
   input RO_cntr_reset;
   
   output RO_cntr_ovflw;
   output [28:0] RO_cntr;

   wire RO_MUX_OUT, MUX_OUT;
   wire [15:0] RO_EN; 
   wire [15:0] RO_OUT;


   PERFRO16_TSMCN7B1_RO_CLK_MUX RO_MUX_D2 (
	.in({RO_MUX_OUT,ext_clk}), 
	.clk_sel(clk_sel), 
	.out(MUX_OUT)
   );


   PERFRO16_TSMCN7B1_RO_MUX RO_MUX_D1 (
      	.in(RO_OUT), 
      	.ro_sel(RO_sel), 
      	.out(RO_MUX_OUT)
   );

   PERFRO16_TSMCN7B1_RO_DEMUX RO_DEMUX_D1 (
      	.ro_en_all(RO_en_all),
      	.ro_en(RO_en),
      	.ro_sel(RO_sel), 
      	.out(RO_EN)
   );

   // reset low
   PERFRO16_TSMCN7B1_RIPPLE_COUNTER RIPPLE_COUNTER_D1 (
      	.reset(~RO_cntr_reset), 
      	.D(MUX_OUT), 
      	.Cntr(RO_cntr), 
      	.overflow(RO_cntr_ovflw)
   );

   PERFRO16_TSMCN7B1_RO_CHAIN IRO0   (.RO_Out(RO_OUT[0]),    .En_X(RO_EN[0]));
   PERFRO16_TSMCN7B1_RO_CHAIN IRO1   (.RO_Out(RO_OUT[1]),    .En_X(RO_EN[1]));
   PERFRO16_TSMCN7B1_RO_CHAIN IRO2   (.RO_Out(RO_OUT[2]),    .En_X(RO_EN[2]));
   PERFRO16_TSMCN7B1_RO_CHAIN IRO3   (.RO_Out(RO_OUT[3]),    .En_X(RO_EN[3]));
   PERFRO16_TSMCN7B1_RO_CHAIN IRO4   (.RO_Out(RO_OUT[4]),    .En_X(RO_EN[4]));
   PERFRO16_TSMCN7B1_RO_CHAIN IRO5   (.RO_Out(RO_OUT[5]),    .En_X(RO_EN[5]));
   PERFRO16_TSMCN7B1_RO_CHAIN IRO6   (.RO_Out(RO_OUT[6]),    .En_X(RO_EN[6]));
   PERFRO16_TSMCN7B1_RO_CHAIN_OCV IRO7   (.RO_Out(RO_OUT[7]),    .En_X(RO_EN[7]), .RO_ocv_sel(RO_ocv_sel));
   PERFRO16_TSMCN7B1_RO_CHAIN_OCV IRO8   (.RO_Out(RO_OUT[8]),    .En_X(RO_EN[8]), .RO_ocv_sel(RO_ocv_sel));
   PERFRO16_TSMCN7B1_RO_CHAIN IRO9  (.RO_Out(RO_OUT[9]),    .En_X(RO_EN[9]));
   PERFRO16_TSMCN7B1_RO_CHAIN IRO10  (.RO_Out(RO_OUT[10]),   .En_X(RO_EN[10]));
   PERFRO16_TSMCN7B1_RO_CHAIN IRO11  (.RO_Out(RO_OUT[11]),   .En_X(RO_EN[11]));
   PERFRO16_TSMCN7B1_RO_CHAIN IRO12  (.RO_Out(RO_OUT[12]),   .En_X(RO_EN[12]));
   PERFRO16_TSMCN7B1_RO_CHAIN IRO13  (.RO_Out(RO_OUT[13]),   .En_X(RO_EN[13]));
   PERFRO16_TSMCN7B1_RO_CHAIN IRO14  (.RO_Out(RO_OUT[14]),   .En_X(RO_EN[14]));
   PERFRO16_TSMCN7B1_RO_CHAIN IRO15  (.RO_Out(RO_OUT[15]),   .En_X(RO_EN[15]));

endmodule



module PERFRO16_TSMCN7B1_RIPPLE_COUNTER (reset, D, Cntr, overflow);

   // PINS DECLARE
   input D;
   input reset;

   output [28:0] Cntr;
   output overflow;
  


   reg [28:0] Q = 29'b00000000000000000000000000000;
   reg overflow_temp;

   wire [28:0] c;

   assign overflow = overflow_temp;
   assign Cntr[28:0] = Q[28:0];
   assign c[28:0] = ~Q[28:0];

   always @(posedge D     or negedge (reset) )     if (~reset)  Q[0]  <= 1'b0;    else Q[0] <= ~Q[0];
   always @(posedge c[0]  or negedge (reset) )     if (~reset)  Q[1]  <= 1'b0;    else Q[1] <= ~Q[1];
   always @(posedge c[1]  or negedge (reset) )     if (~reset)  Q[2]  <= 1'b0;    else Q[2] <= ~Q[2];
   always @(posedge c[2]  or negedge (reset) )     if (~reset)  Q[3]  <= 1'b0;    else Q[3] <= ~Q[3];
   always @(posedge c[3]  or negedge (reset) )     if (~reset)  Q[4]  <= 1'b0;    else Q[4] <= ~Q[4];
   always @(posedge c[4]  or negedge (reset) )     if (~reset)  Q[5]  <= 1'b0;    else Q[5] <= ~Q[5];
   always @(posedge c[5]  or negedge (reset) )     if (~reset)  Q[6]  <= 1'b0;    else Q[6] <= ~Q[6];
   always @(posedge c[6]  or negedge (reset) )     if (~reset)  Q[7]  <= 1'b0;    else Q[7] <= ~Q[7];
   always @(posedge c[7]  or negedge (reset) )     if (~reset)  Q[8]  <= 1'b0;    else Q[8] <= ~Q[8];
   always @(posedge c[8]  or negedge (reset) )     if (~reset)  Q[9]  <= 1'b0;    else Q[9] <= ~Q[9];
   always @(posedge c[9]  or negedge (reset) )     if (~reset)  Q[10] <= 1'b0;   else Q[10] <= ~Q[10];
   always @(posedge c[10] or negedge (reset) )     if (~reset)  Q[11] <= 1'b0;   else Q[11] <= ~Q[11];
   always @(posedge c[11] or negedge (reset) )     if (~reset)  Q[12] <= 1'b0;   else Q[12] <= ~Q[12];
   always @(posedge c[12] or negedge (reset) )     if (~reset)  Q[13] <= 1'b0;   else Q[13] <= ~Q[13];
   always @(posedge c[13] or negedge (reset) )     if (~reset)  Q[14] <= 1'b0;   else Q[14] <= ~Q[14];
   always @(posedge c[14] or negedge (reset) )     if (~reset)  Q[15] <= 1'b0;   else Q[15] <= ~Q[15];
   always @(posedge c[15] or negedge (reset) )     if (~reset)  Q[16] <= 1'b0;   else Q[16] <= ~Q[16];
   always @(posedge c[16] or negedge (reset) )     if (~reset)  Q[17] <= 1'b0;   else Q[17] <= ~Q[17];
   always @(posedge c[17] or negedge (reset) )     if (~reset)  Q[18] <= 1'b0;   else Q[18] <= ~Q[18];
   always @(posedge c[18] or negedge (reset) )     if (~reset)  Q[19] <= 1'b0;   else Q[19] <= ~Q[19];
   always @(posedge c[19] or negedge (reset) )     if (~reset)  Q[20] <= 1'b0;   else Q[20] <= ~Q[20];
   always @(posedge c[20] or negedge (reset) )     if (~reset)  Q[21] <= 1'b0;   else Q[21] <= ~Q[21];
   always @(posedge c[21] or negedge (reset) )     if (~reset)  Q[22] <= 1'b0;   else Q[22] <= ~Q[22];
   always @(posedge c[22] or negedge (reset) )     if (~reset)  Q[23] <= 1'b0;   else Q[23] <= ~Q[23];
   always @(posedge c[23] or negedge (reset) )     if (~reset)  Q[24] <= 1'b0;   else Q[24] <= ~Q[24];
   always @(posedge c[24] or negedge (reset) )     if (~reset)  Q[25] <= 1'b0;   else Q[25] <= ~Q[25];
   always @(posedge c[25] or negedge (reset) )     if (~reset)  Q[26] <= 1'b0;   else Q[26] <= ~Q[26];
   always @(posedge c[26] or negedge (reset) )     if (~reset)  Q[27] <= 1'b0;   else Q[27] <= ~Q[27];
   always @(posedge c[27] or negedge (reset) )     if (~reset)  Q[28] <= 1'b0;   else Q[28] <= ~Q[28];
   always @(posedge c[28] or negedge (reset) )     if (~reset)  overflow_temp <= 1'b0;   else overflow_temp <= 1'b1;

endmodule


module PERFRO16_TSMCN7B1_RO_MUX (in, ro_sel, out);

   input [15:0] in;
   input [3:0] ro_sel;
   output out;

   reg out;

   always @(ro_sel[0] or ro_sel[1] or ro_sel[2] or ro_sel[3] or in[0] or in[1] or in[2] or in[3] or in[4] or in[5] or in[6] or in[7] or in[8] or in[9] or in[10] or in[11] or in[12] or in[13] or in[14] or in[15])
   case (ro_sel)
      0: out <= in[0];
      1: out <= in[1];
      2: out <= in[2];
      3: out <= in[3];
      4: out <= in[4];
      5: out <= in[5];
      6: out <= in[6];
      7: out <= in[7];
      8: out <= in[8];
      9: out <= in[9];
      10: out <= in[10];
      11: out <= in[11];
      12: out <= in[12];
      13: out <= in[13];      
      14: out <= in[14];      
      15: out <= in[15];  
      default: out <= in[0];          
   endcase

endmodule


module PERFRO16_TSMCN7B1_RO_DEMUX (ro_en_all, ro_en, ro_sel, out);
  
   // PINS DECLARE
   input [3:0] ro_sel;
   input ro_en_all;
   input ro_en;
   

   output [15:0] out;
   reg [15:0] out;

   always @(ro_sel[0] or ro_sel[1] or ro_sel[2] or ro_sel[3] or ro_en or ro_en_all)
   begin
      if (ro_en_all == 1 & ro_en == 1) 
             out     = 16'b0000000000000000;
      else 
      if (ro_en == 1 & ro_en_all == 0) 
      begin
         case (ro_sel)
            0: out  = 16'b1111111111111110;
            1: out  = 16'b1111111111111101;
            2: out  = 16'b1111111111111011;
            3: out  = 16'b1111111111110111;
            4: out  = 16'b1111111111101111;
            5: out  = 16'b1111111111011111;
            6: out  = 16'b1111111110111111;
            7: out  = 16'b1111111101111111;
            8: out  = 16'b1111111011111111;
            9: out  = 16'b1111110111111111;
            10: out = 16'b1111101111111111;
            11: out = 16'b1111011111111111;
            12: out = 16'b1110111111111111;
            13: out = 16'b1101111111111111;
            14: out = 16'b1011111111111111;
            15: out = 16'b0111111111111111;
	    default:  out = 16'b1111111111111111;
         endcase
      end
      else 
            out = 16'b1111111111111111;
   end

endmodule



module PERFRO16_TSMCN7B1_RO_CHAIN (En_X,RO_Out);
   
   // PINS DECLARE   
   input En_X;
   output RO_Out;


   // REG , WIRES required for oscillation
   reg RO_Out   = 1'b0;
   wire OscFB ;
   reg [3:0] randPeriod = 4'b0001;

   assign OscFB = RO_Out;
   always @* 
   begin
     RO_Out    <=  #(50 + {28'h0, randPeriod[3:0]}) ~(~En_X & OscFB);
   end

endmodule


// NOTES: En_X is ACTIVE LOW!!!!!!!!!!!

module PERFRO16_TSMCN7B1_RO_CHAIN_OCV (En_X,RO_Out,RO_ocv_sel);
   
   // PINS DECLARE   
   input En_X;
   output RO_Out;
   input       [1:0] RO_ocv_sel;

   
   // REG , WIRES required for oscillation
   reg RO_Out = 1'b0;
   wire OscFB;
   reg [3:0] randPeriod = 4'b0001;
 
   assign OscFB = RO_Out;
  
   always @*
   begin
   	case (RO_ocv_sel)
   	    0: RO_Out  <=  #(50 + {28'h0, randPeriod[3:0]}) ~(~En_X & OscFB );
            1: RO_Out  <= #(100 + {28'h0, randPeriod[3:0]}) ~(~En_X & OscFB );
            2: RO_Out  <= #(150 + {28'h0, randPeriod[3:0]}) ~(~En_X & OscFB );
   	    3: RO_Out  <= #(200 + {28'h0, randPeriod[3:0]}) ~(~En_X & OscFB );
   	endcase
   end
 
endmodule

module PERFRO16_TSMCN7B1_RO_CLK_MUX (in, clk_sel, out);

   input [1:0] in;
   input clk_sel;
   output out;


   reg out;

   always @(clk_sel or in[0] or in[1])
   case (clk_sel)
      0: out <= in[0];
      1: out <= in[1];
   endcase

endmodule
