//                              -*- Mode: Verilog -*-

////////////////////////////////////////////////////////////////////////////
/*
 *
 *  Trade secret of ATI Technologies, Inc.
 *  Copyright 2005, ATI Technologies, Inc., (unpublished)
 *
 *  All rights reserved.  This notice is intended as a precaution against
 *  inadvertent publication and does not imply publication or any waiver
 *  of confidentiality.  The year included in the foregoing notice is the
 *  year of creation of the work.
 *
 */
////////////////////////////////////////////////////////////////////////////
//
// Filename    : SRF_lib.v
// Description : Scan Routing Fabric library
//
////////////////////////////////////////////////////////////////////////////

module swapper_gc10 (
  input swap,
  input in_top, in_bot,
  output wire out_top, out_bot
  );
  assign out_top = swap ? in_bot : in_top;
  assign out_bot = swap ? in_top : in_bot;
endmodule

module swapper_demux_gc10 (
  input swap,
  input in,
  output wire out_top, out_bot
  );
  assign out_top = swap ? 1'b0 : in;
  assign out_bot = swap ? in: 1'b0;
endmodule

module swapper_mux_gc10 (
  input swap,
  input in_top, in_bot,
  output wire out
  );
  assign out = swap ? in_bot : in_top;
endmodule


module srf_demux_gc10 #(parameter cntl_size = 1, out_size = 2 ) (
  input [cntl_size-1:0] sel,
  input in,
  output wire [out_size-1:0] out
  );

  assign out[out_size-1:0] = in << sel[cntl_size-1:0];

endmodule

module srf_mux_gc10 #(parameter cntl_size = 1, in_size = 2 ) (
  input [cntl_size-1:0] sel,
  input wire [in_size-1:0] in,
  output out
  );

  assign out = in[sel];

endmodule


module srf_buffer_gc10 (
  input in,
  output wire out
  );
  assign out = in;
endmodule

module extest_mux_gc10 (
  input sel,
  input in_srf, in_hier,
  output wire out
  );
  assign out = sel ? in_hier : in_srf;
endmodule



module FLOP_GATED_gc10 #(parameter
  size = 1
)(
  input C,  // Clock
  input [size-1:0] D, // D -input
  input R,  // Reset
  input G,  // Gating Logic
  output reg [size-1:0] Q 
);
  
// Remove clock gater, convert to re-circulating mux 
// to let synthesis to optimize
//  wire D_latch;
//  wire Q_latch;
//  wire C_gated;


//  assign  D_latch = G|R;
//  // Glitch latch
//  latch_wrapper latch_wrapper(
//                  .CLK(~C),
//                  .D(D_latch),
//                  .Q(Q_latch)
//                  );

//  assign C_gated = C & Q_latch;
  
  // Flop
  always @(posedge C) begin
    if(R)
      Q <= {size{1'b0}};      
    else
      if (G) begin
	 Q <= D ;
      end
  end
endmodule

