// ==============================================================================================
// PLEASE DONOT CHANGES THIS FILE
// THIS FILE IS AUTO_GENERATED
// Source: import/address/src/rtl/gen_one/addr_tile_split_index.v
// ===============================================================================================
//  Trade secret of ATI Technologies, Inc.
//  Copyright 2011, ATI Technologies, Inc., (unpublished)
//
//  All rights reserved.  This notice is intended as a precaution against
//  inadvertent publication and does not imply publication or any waiver
//  of confidentiality.  The year included in the foregoing notice is the
//  year of creation of the work.
//
//  
// 
// Description:  common module for tile_split and macrotile_mode_index
//               derived from $STEM/ld/src/gfx/cb/cmn/cb_addr_tile_index_calc.v


`include "gc__gb_reg.v"
`include "addrlib_defines_gc.vh"

module addr_tile_split_index_gc (
			// outputs
			tile_split,           
			macrotile_mode_index,

			// inputs
			array_mode,
			micro_tile_mode,
			log2_sample_split, 
			depth_tile_split,
			row_size, 
			log2_pixelsize,
			log2_samples  
			);


localparam ARRAY_MODE_WIDTH 		=  4,
           MICRO_TILE_MODE_WIDTH	=  3,
           LOG2_SAMPLE_SPLIT_WIDTH	=  2,
           TILE_SPLIT_WIDTH		=  3,
           LOG2_PIXELSIZE_WIDTH		=  3,
           LOG2_SAMPLES_WIDTH		=  2,
           MACROTILE_MODE_INDEX_WIDTH	=  4;

//GB_ADDR_CONFIG.ROW_SIZE is deprecated and so adding explicit param to support 1KB, 2KB and 4KB row sizes
localparam ROW_SIZE_SIZE = 2;

   parameter LPARAM_TILE_THICKNESS_LOG2_WIDTH=2; // tile thickness in slice log2 log2; max = 8, log2(8)=3, to represent 3 need 2 bits i.e log2(3)
   parameter LPARAM_1KB_LOG2  = 10; // log2(1K)
   parameter LPARAM_64B_LOG2  = 6;  // log2(64)
   parameter LPARAM_256B_LOG2 = 8;  // log2(256)
   parameter LPARAM_4KB_LOG2  = 12; // log2(4kB)

   localparam PARAM_PIXEL_PER_QUAD_IN_XY_LOG2 = 1;
   localparam PARAM_QUAD_PER_TILE_IN_XY_LOG2  = 2;
   localparam PARAM_PIXEL_PER_TILE_IN_XY_LOG2 = (PARAM_PIXEL_PER_QUAD_IN_XY_LOG2 + PARAM_QUAD_PER_TILE_IN_XY_LOG2); 

   //-----------------------------
   // Ports
input  [ARRAY_MODE_WIDTH-1              : 0]          array_mode;
input  [MICRO_TILE_MODE_WIDTH-1         : 0]          micro_tile_mode;        //r11xx: changed to 3-bits

input  [LOG2_SAMPLE_SPLIT_WIDTH-1       : 0]          log2_sample_split;      //0: 1 sample split, 1: 2 sample split, 2: 4 sample split, ...
input  [TILE_SPLIT_WIDTH-1              : 0]          depth_tile_split;       //0: 64B, 1: 128B, 2: 256B, ...

input  [ROW_SIZE_SIZE-1  :0]          row_size;               //0: 1KB, 1: 2KB, 2: 4KB

input  [LOG2_PIXELSIZE_WIDTH-1          : 0]          log2_pixelsize;         //0: 8bpp, 1:16bpp, 2:32bpp, 3:64bpp, 4:128bpp
input  [LOG2_SAMPLES_WIDTH-1            : 0]          log2_samples;           //0: 1-sample, 1: 2-samples, 2; 4-samples, ...

output [TILE_SPLIT_WIDTH-1              : 0]          tile_split;             //0: 64B  split, 1: 128B split, 2: 256B split, ...
output [MACROTILE_MODE_INDEX_WIDTH-1    : 0]          macrotile_mode_index;   //used to index GB_MACROTILE_MODE: {prt_array_mode, tile_bytes[2:0]}
   

   //------------------------------
   // Internal Logic


   //------------------------------
   // find tile thickness
   reg [LPARAM_TILE_THICKNESS_LOG2_WIDTH-1:0]  tile_thickness_log2;
   reg                                         prt_array_mode;
   
   always @* begin
      case (array_mode)
        `ARRAY_1D_TILED_THICK     ,
        `ARRAY_2D_TILED_THICK     ,
        `ARRAY_3D_TILED_THICK     ,
        `ARRAY_PRT_TILED_THICK    ,
        `ARRAY_PRT_2D_TILED_THICK ,
        `ARRAY_PRT_3D_TILED_THICK : begin tile_thickness_log2 = 2; end // thickness = 4 slices

        `ARRAY_2D_TILED_XTHICK    ,
        `ARRAY_3D_TILED_XTHICK    : begin tile_thickness_log2 = 3; end // thickness = 8 slices

        `ARRAY_LINEAR_GENERAL     ,
        `ARRAY_LINEAR_ALIGNED     ,
        `ARRAY_1D_TILED_THIN1     ,
        `ARRAY_2D_TILED_THIN1     ,
        `ARRAY_3D_TILED_THIN1     ,
        `ARRAY_PRT_TILED_THIN1    ,
        `ARRAY_PRT_2D_TILED_THIN1 ,
        `ARRAY_PRT_3D_TILED_THIN1 : begin tile_thickness_log2 = 0; end // thickness = 1 slice
        default                   : begin tile_thickness_log2 = 'bx; end
      endcase

      case (array_mode)

        `ARRAY_LINEAR_GENERAL     ,
        `ARRAY_LINEAR_ALIGNED     ,
        `ARRAY_1D_TILED_THIN1     ,
        `ARRAY_2D_TILED_THIN1     ,
        `ARRAY_3D_TILED_THIN1     ,
        `ARRAY_1D_TILED_THICK     ,
        `ARRAY_2D_TILED_THICK     ,
        `ARRAY_3D_TILED_THICK     ,
        `ARRAY_2D_TILED_XTHICK    ,
        `ARRAY_3D_TILED_XTHICK    : begin prt_array_mode = 0; end // thickness = 8 slices

        `ARRAY_PRT_TILED_THIN1    ,
        `ARRAY_PRT_2D_TILED_THIN1 ,
        `ARRAY_PRT_3D_TILED_THIN1 ,
        `ARRAY_PRT_TILED_THICK    ,
        `ARRAY_PRT_2D_TILED_THICK ,
        `ARRAY_PRT_3D_TILED_THICK : begin prt_array_mode = 1; end // thickness = 4 slices
        default                   : begin prt_array_mode = 'bx; end
      endcase

   end

   //=========================================================
   // This calculation removes the bottom 6 bits (log2(64B)) because minimum tile_split is 64B
   // Basically, calculate two things: tile_split and tile_bytes
   // tile_split : MIN(ROW_SIZE, MAX(256B, tile_size_1xaa * sample_split)) i.e RESTRICTION: ROW_SIZE >= tile_split >= 256
   // tile_bytes : MIN( tile_split, tile_size_1xaa * num_samples)          i.e RESTRICTION: tile_split >= tile_bytes

   parameter LPARAM_MAXBYTE_LOG2 = 17; // max we will go is 16BperSample * 16SamplesPerPixel * 64PixelsPerSlice * 8 SlicesPerTile = 128KB 
   parameter LPARAM_MAX_BYTES_64B_LOG2 = (LPARAM_MAXBYTE_LOG2 - LPARAM_64B_LOG2); // log2 64B aligned max byte value (17-6=11)
   parameter LPARAM_MAX_BYTES_64B_LOG2_WIDTH = 4; // to encode upto 11, we need 4 bits i.e log2(LPARAM_MAX_BYTES_64B_LOG2)
   
   parameter LPARAM_MIN_TILE_SPLIT_64B_LOG2 = (LPARAM_256B_LOG2 - LPARAM_64B_LOG2); // min tile split we allow is 256B for color because fmask/color cacheline size = 256B
   parameter LPARAM_MIN_ROW_SIZE_64B_LOG2   = (LPARAM_1KB_LOG2  - LPARAM_64B_LOG2); // min row size is 1KB

   //============================
   // First calculate tile_split:
   
   // log2 of tile bytes for 1xaa 64B aligned
   wire [LPARAM_MAX_BYTES_64B_LOG2_WIDTH-1:0] tile_byte_1xaa_64B_log2 = log2_pixelsize + tile_thickness_log2;

   // log2 of tile split 64B aligned with lower clamp of 256B
   wire [LPARAM_MAX_BYTES_64B_LOG2_WIDTH-1:0] color_tile_split_64B_log2_tmp = log2_sample_split + tile_byte_1xaa_64B_log2;
   wire [LPARAM_MAX_BYTES_64B_LOG2_WIDTH-1:0] color_tile_split_64B_log2 = color_tile_split_64B_log2_tmp > LPARAM_MIN_TILE_SPLIT_64B_LOG2 ? color_tile_split_64B_log2_tmp : LPARAM_MIN_TILE_SPLIT_64B_LOG2;
   wire [LPARAM_MAX_BYTES_64B_LOG2_WIDTH-1:0] tile_split_64B_log2 = (micro_tile_mode == `ADDR_SURF_DEPTH_MICRO_TILING) ? {1'b0, depth_tile_split} : color_tile_split_64B_log2;

   // log2 of tile_split with upper clamp of ROW_SIZE
   // convert row size enum to log2 of the 64B aligned value, note: min. row_size = 1kB
   wire [LPARAM_MAX_BYTES_64B_LOG2_WIDTH-1:0] row_size_64B_log2 = row_size + LPARAM_MIN_ROW_SIZE_64B_LOG2; 
   
   // clamped at the bottom by 256B, clamped at top by ROW_SIZE
   assign  tile_split = tile_split_64B_log2 > row_size_64B_log2 ? row_size_64B_log2 : tile_split_64B_log2;

   //============================
   // Second calculate tile_bytes
   
   // log2 of tile bytes 64B aligned with upper clamp of tile_split
   wire [LPARAM_MAX_BYTES_64B_LOG2_WIDTH-1:0] tile_bytes_64B_log2_tmp = log2_samples + tile_byte_1xaa_64B_log2;
   wire [LPARAM_MAX_BYTES_64B_LOG2_WIDTH-1:0] tile_bytes_64B_log2 =  tile_bytes_64B_log2_tmp > tile_split ? tile_split : tile_bytes_64B_log2_tmp;

   // the final index is calculated based on tile bytes and array mode:
   // index#0-6  for non PRT array modes
   // index#8-14 for PRT array modes
   // index#7,#15 are invalid (you won't go above 4KB tiles bytes as row_size will clamp to that.
   // move prt bit to MSB to select upper or lower entries in the array
   assign  macrotile_mode_index = prt_array_mode << (MACROTILE_MODE_INDEX_WIDTH-1) | tile_bytes_64B_log2;

   
   
`ifdef SIMULATION

   //=========================================================
   // Calculate properly without taking short cuts and then compare the outputs 
   
   parameter LPARAM_MAX_BYTES_LOG2_WIDTH = 5; // to encode upto 17, we need 5 bits i.e log2(LPARAM_MAXBYTE_LOG2)
   parameter LPARAM_MIN_TILE_SPLIT_BYTES = 1 << LPARAM_256B_LOG2; 

   // tile bytes for 1xaa:   
   // pixel_size_in_bytes * 8 * 8 * tile_thickness
   wire [LPARAM_MAX_BYTES_LOG2_WIDTH-1:0] SIM_tile_bytes_1xaa_log2 = log2_pixelsize + PARAM_PIXEL_PER_TILE_IN_XY_LOG2 + PARAM_PIXEL_PER_TILE_IN_XY_LOG2 + tile_thickness_log2;

   // row_size in bytes:
   // row_size * 1kB
   wire [LPARAM_MAXBYTE_LOG2-1:0]         SIM_row_size_bytes = 1 << (row_size + LPARAM_1KB_LOG2);

   // 1xaa tile bytes times samples in a tile_split to get tile_split_bytes
   // sample_split * tile_bytes_1xaa
   wire [LPARAM_MAXBYTE_LOG2-1:0]         SIM_color_tile_split_bytes = 1 << (log2_sample_split + SIM_tile_bytes_1xaa_log2);
   // color tile split in bytes is a minimum of 256B
   // MAX(256B, color_tile_split)
   wire [LPARAM_MAXBYTE_LOG2-1:0]         SIM_color_tile_split_bytes_with_floor = (SIM_color_tile_split_bytes > LPARAM_MIN_TILE_SPLIT_BYTES) ? SIM_color_tile_split_bytes : LPARAM_MIN_TILE_SPLIT_BYTES;
   // color tile split in bytes cannot go above row_size in bytes
   // MIN(ROW_SIZE, color_tile_split)
   wire [LPARAM_MAXBYTE_LOG2-1:0]         SIM_color_tile_split_bytes_with_floor_and_ceiling = (SIM_color_tile_split_bytes_with_floor > SIM_row_size_bytes) ? SIM_row_size_bytes : SIM_color_tile_split_bytes_with_floor;

   // bytes for a tile split
   wire [LPARAM_MAXBYTE_LOG2-1:0]         SIM_tile_split_c = SIM_color_tile_split_bytes_with_floor_and_ceiling;
   
   // total bytes for all samples of a tile
   // multiply 1xaa tile bytes by samples to get full tile bytes 
   wire [LPARAM_MAXBYTE_LOG2-1:0]         SIM_tile_bytes_total = (1 << SIM_tile_bytes_1xaa_log2 + log2_samples); 

   // figure out which is smaller: total_tile_bytes or tile_split_bytes
   // MIN(total_tile_bytes, tile_split_bytes)
   wire [LPARAM_MAXBYTE_LOG2-1:0]         SIM_tile_bytes =  (SIM_tile_bytes_total > SIM_tile_split_c) ? SIM_tile_split_c : SIM_tile_bytes_total;

   
   // get the enum back from tile_bytes/tile_split
   wire [MACROTILE_MODE_INDEX_WIDTH-1:0] SIM_macrotile_mode_index;
   reg [MACROTILE_MODE_INDEX_WIDTH-1:0]  SIM_macrotile_mode_index_tmp;
   assign                                      SIM_macrotile_mode_index = prt_array_mode << (MACROTILE_MODE_INDEX_WIDTH-1) | SIM_macrotile_mode_index_tmp;

   reg [TILE_SPLIT_WIDTH-1:0]            SIM_tile_split_out; 

   always @* begin
      case (SIM_tile_bytes)
        'h00040  : begin SIM_macrotile_mode_index_tmp = 0; end 
        'h00080  : begin SIM_macrotile_mode_index_tmp = 1; end 
        'h00100  : begin SIM_macrotile_mode_index_tmp = 2; end 
        'h00200  : begin SIM_macrotile_mode_index_tmp = 3; end 
        'h00400  : begin SIM_macrotile_mode_index_tmp = 4; end 
        'h00800  : begin SIM_macrotile_mode_index_tmp = 5; end 
        'h01000  : begin SIM_macrotile_mode_index_tmp = 6; end 
        default  : begin SIM_macrotile_mode_index_tmp = 'bx; end
      endcase

      case (SIM_tile_split_c)
        'h00040  : begin SIM_tile_split_out = `ADDR_SURF_TILE_SPLIT_64B ; end 
        'h00080  : begin SIM_tile_split_out = `ADDR_SURF_TILE_SPLIT_128B; end 
        'h00100  : begin SIM_tile_split_out = `ADDR_SURF_TILE_SPLIT_256B; end 
        'h00200  : begin SIM_tile_split_out = `ADDR_SURF_TILE_SPLIT_512B; end 
        'h00400  : begin SIM_tile_split_out = `ADDR_SURF_TILE_SPLIT_1KB ; end 
        'h00800  : begin SIM_tile_split_out = `ADDR_SURF_TILE_SPLIT_2KB ; end 
        'h01000  : begin SIM_tile_split_out = `ADDR_SURF_TILE_SPLIT_4KB ; end 
        default  : begin SIM_tile_split_out = 'bx; end
      endcase

   end
   

`endif
                               
endmodule // cb_addr_tile_index_calc


 
