//--------------------------------------------------------------------------
//
// $Id: //depot/r600/branches/devel_db/parts_lib/src/common/ati_8rp_state_storage.v#2 $
//
// $Change: 168897 $
//
//
// Notes:	DFF library including plain dff, dff w/reset, dff w/clk enable,
//  		and dff w/reset and clk enable.  Reset causes rval to be loaded
//		into the flop
// Copyright:	Trade secret of ATI Technologies, Inc.
//		Copyright 2001-2002, ATI Technologies, Inc., (unpublished) 
//
//		All rights reserved.  This notice is intended as a precaution against
//		inadvertent publication and does not imply publication or any waiver
//		of confidentiality.  The year included in the foregoing notice is the
//		year of creation of the work.
//
//--------------------------------------------------------------------------
module ati_8rp_state_storage_with_reset (
	                                   st_data0,
	                                   st_data1,
	                                   st_data2,
	                                   st_data3,
	                                   st_data4,
	                                   st_data5,
	                                   st_data6,
	                                   st_data7,
	                                   w_data, 
	                                   w_addr, 
	                                   r_data, 
	                                   r_addr, 
	                                   we, 
	                                   re,
	                                   sel, 
	                                   cpy,
                                           rst,
	                                   clk);
   
   parameter	WIDTH = 1;
   parameter    RESET_ENABLE = 0;
   parameter    RESET_DATA = 0;

   output [WIDTH-1:0] st_data0,
                      st_data1,
                      st_data2,
                      st_data3,
                      st_data4,
                      st_data5,
                      st_data6,
                      st_data7;
   output [WIDTH-1:0] r_data;
   input [WIDTH-1:0]  w_data;
   input [2:0]        w_addr;
   input [2:0]        r_addr;
   input              we;
   input              re;
   input              cpy;
   input              sel;
   input              rst;
   input              clk;
   
   reg [WIDTH-1:0]    state_flop0;
   reg [WIDTH-1:0]    state_flop1;
   reg [WIDTH-1:0]    state_flop2;
   reg [WIDTH-1:0]    state_flop3;
   reg [WIDTH-1:0]    state_flop4;
   reg [WIDTH-1:0]    state_flop5;
   reg [WIDTH-1:0]    state_flop6;
   reg [WIDTH-1:0]    state_flop7;
   
   wire [WIDTH-1:0]   next_state_flop0;
   wire [WIDTH-1:0]   next_state_flop1;
   wire [WIDTH-1:0]   next_state_flop2;
   wire [WIDTH-1:0]   next_state_flop3;
   wire [WIDTH-1:0]   next_state_flop4;
   wire [WIDTH-1:0]   next_state_flop5;
   wire [WIDTH-1:0]   next_state_flop6;
   wire [WIDTH-1:0]   next_state_flop7;

   wire               load_flop0;
   wire               load_flop1;
   wire               load_flop2;
   wire               load_flop3;
   wire               load_flop4;
   wire               load_flop5;
   wire               load_flop6;
   wire               load_flop7;

   reg [WIDTH-1:0]    read_data;

   wire [WIDTH-1:0]   write_data;
   wire [0:0]         write_sel;
   wire [0:0]         read_sel;
   wire [0:0]         write_copy;
   wire [0:0]         enable_copy;
   wire [0:0]         enable_load;
   
   
   // Two different write cases
   // One read case
   assign             read_sel = re & sel;
   assign             write_sel = we & sel;
   assign             write_copy = we & cpy;
   
   // rdata mux for copy and reads
   always @(
	    state_flop0 or 
	    state_flop1 or
	    state_flop2 or
	    state_flop3 or
	    state_flop4 or
	    state_flop5 or
	    state_flop6 or
	    state_flop7 or
	    r_addr )
     begin
	case(r_addr)
	  3'h0 : read_data = state_flop0;
	  3'h1 : read_data = state_flop1;
	  3'h2 : read_data = state_flop2;
	  3'h3 : read_data = state_flop3;
	  3'h4 : read_data = state_flop4;
	  3'h5 : read_data = state_flop5;
	  3'h6 : read_data = state_flop6;
	  3'h7 : read_data = state_flop7;
	  default : read_data = {WIDTH{1'bx}};
	endcase
     end

   // wdata mux
   assign write_data = write_copy ? read_data : w_data;

   // only allow copies from different context id
   assign enable_copy = |(r_addr ^ w_addr);
   
   // load a state register on a copy or write select
   // write_sel and write_copy are mutually exclusive
   assign enable_load = ((write_copy & enable_copy) | write_sel);

   // Use combo logic with address for x propagation
   assign load_flop0 = ~w_addr[2] & ~w_addr[1] & ~w_addr[0] & enable_load;
   assign load_flop1 = ~w_addr[2] & ~w_addr[1] &  w_addr[0] & enable_load;
   assign load_flop2 = ~w_addr[2] &  w_addr[1] & ~w_addr[0] & enable_load;
   assign load_flop3 = ~w_addr[2] &  w_addr[1] &  w_addr[0] & enable_load;
   assign load_flop4 =  w_addr[2] & ~w_addr[1] & ~w_addr[0] & enable_load;
   assign load_flop5 =  w_addr[2] & ~w_addr[1] &  w_addr[0] & enable_load;
   assign load_flop6 =  w_addr[2] &  w_addr[1] & ~w_addr[0] & enable_load;
   assign load_flop7 =  w_addr[2] &  w_addr[1] &  w_addr[0] & enable_load;

   // Select next data for all state flops
   assign next_state_flop0 = load_flop0 ? write_data : state_flop0;
   assign next_state_flop1 = load_flop1 ? write_data : state_flop1;
   assign next_state_flop2 = load_flop2 ? write_data : state_flop2;
   assign next_state_flop3 = load_flop3 ? write_data : state_flop3;
   assign next_state_flop4 = load_flop4 ? write_data : state_flop4;
   assign next_state_flop5 = load_flop5 ? write_data : state_flop5;
   assign next_state_flop6 = load_flop6 ? write_data : state_flop6;
   assign next_state_flop7 = load_flop7 ? write_data : state_flop7;
   
   // register the state data
   integer i;
   always @(posedge clk)
     begin
        if (RESET_ENABLE && rst) begin
           state_flop0 <= RESET_DATA;
           state_flop1 <= RESET_DATA;
           state_flop2 <= RESET_DATA;
           state_flop3 <= RESET_DATA;
           state_flop4 <= RESET_DATA;
           state_flop5 <= RESET_DATA;
           state_flop6 <= RESET_DATA;
           state_flop7 <= RESET_DATA;
        end
        else begin
           state_flop0 <= next_state_flop0;
	   state_flop1 <= next_state_flop1;
	   state_flop2 <= next_state_flop2;
	   state_flop3 <= next_state_flop3;
	   state_flop4 <= next_state_flop4;
	   state_flop5 <= next_state_flop5;
	   state_flop6 <= next_state_flop6;
	   state_flop7 <= next_state_flop7;
        end // else: !if(rst)
     end // always @ (posedge clk)
      
   
   // state data output
   assign st_data0 = state_flop0;
   assign st_data1 = state_flop1;
   assign st_data2 = state_flop2;
   assign st_data3 = state_flop3;
   assign st_data4 = state_flop4;
   assign st_data5 = state_flop5;
   assign st_data6 = state_flop6;
   assign st_data7 = state_flop7;

   
   // r_data output  
   assign r_data = read_sel ? read_data : {WIDTH{1'b0}};

endmodule

