//                              -*- Mode: Verilog -*-
// Filename              : 
// Perforce revision     : $Id: //depot/gpu/main/ld/src/misc/ati_N_st_storage_1_out_no_read.v#1 $
// Perforce changelist   : $Change: 89611 $
//
// Copyright             : Trade secret of ATI Technologies, Inc.
//                         Copyright 2004, ATI Technologies, Inc., (unpublished)
//                         All rights reserved.  This notice is intended as a precaution against
//                         inadvertent publication and does not imply publication or any waiver
//                         of confidentiality.  The year included in the foregoing notice is the
//                         year of creation of the work.
//
// Description           : 

`include "gc_features.vh"

// leda FM_2_4 off // Assignment to X is not recommended (handled differently by simulation and synthesis tools)
module ati_N_st_storage_1_out_no_read (
  st_data,

  st_indx, 
  w_data, 
  w_addr, 
  r_addr, 
  we, 
  sel, 
  cpy, 

  clk);
    
parameter D_WIDTH = 1; // data width
parameter A_WIDTH = 3; // addr width: default for 8 states

output  [D_WIDTH-1:0]   st_data;

input   [D_WIDTH-1:0]   w_data;
input   [A_WIDTH-1:0]   w_addr;     // Should w_addr & r_addr
input   [A_WIDTH-1:0]   r_addr;     // be parameterized ?????
input		[A_WIDTH-1:0]   st_indx;
input                   we;
input                   cpy;
input                   sel;

input                   clk;


reg     [D_WIDTH-1:0]   state_flop[0:`GC__NUM_STATES-1];

wire    [D_WIDTH-1:0]   read_data;

wire    [D_WIDTH-1:0]   write_data;
wire    [0:0]           write_sel;
wire    [0:0]           write_copy;
wire    [0:0]           enable_copy;
wire    [0:0]           enable_load;


// Two different write cases & one read case
assign write_sel  = we & sel;
assign write_copy = we & cpy;
    
// only allow copies from different context id
assign enable_copy = |(r_addr ^ w_addr);

// load a state register on a copy or write select
// write_sel and write_copy are mutually exclusive
assign enable_load = ((write_copy & enable_copy) | write_sel);

// wdata mux
assign write_data = write_copy ? read_data : w_data;

// register the state data
always @(posedge clk)
begin
    if (enable_load) begin
        state_flop[w_addr] <= write_data;
    end
end

// rdata mux for copy and reads
assign read_data = state_flop[r_addr];

// state data output
assign st_data = state_flop[st_indx];

endmodule
// leda FM_2_4 on
