//                              -*- Mode: Verilog -*-
// Filename              : 
// Perforce revision     : $Id: //depot/gpu/main/ld/src/misc/ati_N_st_storage_N_out_with_rst.v#1 $
// Perforce changelist   : $Change: 89611 $
//
// Copyright             : Trade secret of ATI Technologies, Inc.
//                         Copyright 2004, ATI Technologies, Inc., (unpublished)
//                         All rights reserved.  This notice is intended as a precaution against
//                         inadvertent publication and does not imply publication or any waiver
//                         of confidentiality.  The year included in the foregoing notice is the
//                         year of creation of the work.
//
// Description           : 

`include "gc_features.vh"

// leda FM_2_4 off // Assignment to X is not recommended (handled differently by simulation and synthesis tools)
module ati_N_st_storage_N_out_with_rst (
  st_data0,
  st_data1,
  st_data2,
  st_data3,
  st_data4,
  st_data5,
  st_data6,
  st_data7,

  w_data, 
  w_addr,
  r_data, 
  r_addr, 
  we, 
  re,
  sel, 
  cpy, 

  rst,
  clk);
    
parameter D_WIDTH = 1; // data width
parameter A_WIDTH = 3; // addr width: default for 8 states

output  [D_WIDTH-1:0]   st_data0;
output  [D_WIDTH-1:0]   st_data1;
output  [D_WIDTH-1:0]   st_data2;
output  [D_WIDTH-1:0]   st_data3;
output  [D_WIDTH-1:0]   st_data4;
output  [D_WIDTH-1:0]   st_data5;
output  [D_WIDTH-1:0]   st_data6;
output  [D_WIDTH-1:0]   st_data7;
output  [D_WIDTH-1:0]   r_data;

input   [D_WIDTH-1:0]   w_data;
input   [A_WIDTH-1:0]   w_addr;     // Should w_addr & r_addr
input   [A_WIDTH-1:0]   r_addr;     // be parameterized ?????
input                   we;
input                   re;
input                   cpy;
input                   sel;

input                   rst;
input                   clk;


reg     [D_WIDTH-1:0]   state_flop[0:`GC__NUM_STATES-1];

wire    [D_WIDTH-1:0]   read_data;

wire    [D_WIDTH-1:0]   write_data;
wire    [0:0]           write_sel;
wire    [0:0]           read_sel;
wire    [0:0]           write_copy;
wire    [0:0]           enable_copy;
wire    [0:0]           enable_load;

integer states;


// Two different write cases
assign read_sel   = re & sel;
assign write_sel  = we & sel;
assign write_copy = we & cpy;
    
// only allow copies from different context id
assign enable_copy = |(r_addr ^ w_addr);

// load a state register on a copy or write select
// write_sel and write_copy are mutually exclusive
assign enable_load = ((write_copy & enable_copy) | write_sel);

// wdata mux
assign write_data = write_copy ? read_data : w_data;

// register the state data
always @(posedge clk)
begin
    if (rst) begin
        for (states=0; states<`GC__NUM_STATES; states=states+1) begin
            state_flop[states] <= {D_WIDTH{1'b0}};
        end
    end
    else
    if (enable_load) begin
        state_flop[w_addr] <= write_data;
    end
end

// rdata mux for copy and reads
assign read_data = state_flop[r_addr];

// state data output
// Valid options are 1, 2, 4, 8
// There is always at least one state present
assign st_data0 = state_flop[0];

`ifdef GC__NUM_STATES__1_PRESENT
  assign st_data1 = state_flop[1];
`else
  assign st_data1 = {D_WIDTH{1'bX}};
`endif

`ifdef GC__NUM_STATES__3_PRESENT
  assign st_data2 = state_flop[2];
  assign st_data3 = state_flop[3];
`else
  assign st_data2 = {D_WIDTH{1'bX}};
  assign st_data3 = {D_WIDTH{1'bX}};
`endif

`ifdef GC__NUM_STATES__7_PRESENT
  assign st_data4 = state_flop[4];
  assign st_data5 = state_flop[5];
  assign st_data6 = state_flop[6];
  assign st_data7 = state_flop[7];
`else
  assign st_data4 = {D_WIDTH{1'bX}};
  assign st_data5 = {D_WIDTH{1'bX}};
  assign st_data6 = {D_WIDTH{1'bX}};
  assign st_data7 = {D_WIDTH{1'bX}};
`endif

// r_data output  
assign r_data = read_sel ? read_data : {D_WIDTH{1'b0}};

endmodule
// leda FM_2_4 on
