//VCS coverage exclude_file
//--------------------------------------------------------------------------
//        fghodrat -- Adding basic asserts and cover statements for FIFO
//
//--------------------------------------------------------------------------
//`ifndef ATI_BASIC_FIFO_COVER_ASSERT
//`define ATI_BASIC_FIFO_COVER_ASSERT
module ati_basic_fifo_cover_assert(/*AUTOARG*/
   // Inputs
   clk, reset, empty, full, push, pop
   );
  
   parameter ASSERT_FULL_AND_PUSH = 1;
   parameter ASSERT_EMPTY_AND_POP = 1;
   parameter COVER_PUSH_AND_POP = 1;
   parameter COVER_FULL = 1;
   parameter COVER_FULL_EMPTY_FULL = 1;
   parameter ASSERT_EMPTY_EOS = 1;

   
   input clk;
   input reset;
   
   input empty;
   input full;

   input push;
   input pop;

//VCS coverage off
   reg full_encountered, full_empty_encountered;
   always @(posedge clk) 
	if (reset) full_encountered <= 1'b0;
        else if (full)
            full_encountered <= full;
   wire full_empty = full_encountered & empty;
   always @(posedge clk) 
     if (reset) full_empty_encountered <= 1'b0;
     else if (full_empty)
       full_empty_encountered <= full_empty;
   wire full_empty_full = full_empty_encountered && full;


   // ASSERTS
   // psl ati_basic_fifo_cover_assert__empty_and_full: assert never (!reset && empty && full) @(posedge clk);
   // psl ati_basic_fifo_cover_assert__empty_and_pop: assert never (!reset && empty && pop && ASSERT_EMPTY_AND_POP) @(posedge clk);
   // psl ati_basic_fifo_cover_assert__full_and_push: assert never (!reset && full && push && ASSERT_FULL_AND_PUSH) @(posedge clk);

// None of the simulators seem to actually support eventually
   //XX ati_basic_fifo_cover_assert__empty_eos : assert always ((~empty && ASSERT_EMPTY_EOS) |-> eventually! (empty)) @(posedge clk);

   // COVERS
   // psl ati_basic_fifo_cover_assert__empty: cover {!reset && empty} @(posedge clk);
generate
   if (COVER_FULL) begin
     // psl ati_basic_fifo_cover_assert__full: cover {!reset && full} @(posedge clk);
   end
   if (COVER_PUSH_AND_POP) begin
     // psl ati_basic_fifo_cover_assert__push_and_pop: cover {!reset && push && pop } @(posedge clk);
   end
   if (COVER_FULL_EMPTY_FULL) begin
     // psl ati_basic_fifo_cover_assert__full_empty_full: cover {!reset && full_empty_full} @(posedge clk);
   end
endgenerate

//VCS coverage on
endmodule
//`endif

