//                              -*- Mode: Verilog -*-
// Filename              : ati_binary_encoder
//
// Copyright             : Trade secret of ATI Technologies, Inc.
//                         Copyright 2004, ATI Technologies, Inc., (unpublished)
//                         All rights reserved.  This notice is intended as a precaution against
//                         inadvertent publication and does not imply publication or any waiver
//                         of confidentiality.  The year included in the foregoing notice is the
//                         year of creation of the work.
//
// Description           : takes an N bit "one-hot" input (cam search) and converts it to a Log2(N)-bit binary number.


module ati_binary_encoder(/*AUTOARG*/
   // Outputs
   out,
   // Inputs
   sel
   );

   parameter NSEL = 64;

   // Don't need to set this parameter
   parameter LOG2N = ((NSEL <= 2) ? 1 :
                      (NSEL <= 4) ? 2 :
                      (NSEL <= 8) ? 3 :
                      (NSEL <= 16) ? 4 :
                      (NSEL <= 32) ? 5 :
                      (NSEL <= 64) ? 6 :
                      (NSEL <= 128) ? 7 :
                      (NSEL <= 256) ? 8 :
                      (NSEL <= 512) ? 9 :
                      (NSEL <= 1024) ? 10 :
                      (NSEL <= 2048) ? 11 :
                      (NSEL <= 4096) ? 12 :
                      (NSEL <= 8192) ? 13 :
                      (NSEL <= 16384) ? 14 :
                      (NSEL <= 32768) ? 15 :
                      16);  // Bigger than 65536??  i hope not.

   input [NSEL-1:0]   sel;
   output [LOG2N-1:0] out;

   // Pad out to next power of 2.
   parameter        NPOW2   = (1 << LOG2N);
   wire [NPOW2-1:0] selpow2 = sel;

   // N / 2 is the size of the or tree needed to generate each bit.
   parameter NO2 = (NPOW2+1)/2;

   // Pull out the bits that need to go into each output bit's OR tree and or it together.
   integer         i, j;
   reg [NO2-1:0]   or_tree;
   reg [LOG2N-1:0] out;
   always @(/*AS*/selpow2) 
     for (i = 0; i < LOG2N; i=i+1) begin
        for (j = 0; j < NO2; j=j+1) 
          or_tree[j] = selpow2[(NPOW2-1) - (j + ((j>>i)<<i))];
        out[i] = |(or_tree);
     end

   // AUTO_CONSTANT ( NSEL, LOG2N, NO2 )
endmodule
