
module ati_edc_counter (
	regbus_rd,
	regbus_rs,
	regbus_re,
	regbus_a,
    regbus_we,
    regbus_wd,
	sclk,
	reset, // active high
  count_event
);

parameter EDC_PERF_REG_ADDR=0; // grbm address
parameter NUM_CNTR=1; // 1, 2, or 4 are valid
parameter CNTR_WIDTH=4; // up to 32/NUM_CNTR

output [31:0]          regbus_rd;
reg    [31:0]          regbus_rd;
output                 regbus_rs;
input                  regbus_re;
input  [17:2]          regbus_a;
input                  regbus_we;
input  [31:0]          regbus_wd; 
input                  sclk;
input                  reset;
input  [NUM_CNTR-1:0]  count_event;

// river translate_off
  wire regbus_ws;
  // NUM_CNTR counters of width CNTR_WIDTH
  wire [CNTR_WIDTH-1:0] count [NUM_CNTR-1:0];
  reg  [CNTR_WIDTH-1:0] count_wd [NUM_CNTR-1:0]; 
  genvar cnt;
  generate
    for (cnt=0; cnt<NUM_CNTR; cnt=cnt+1) begin : gen_edc_counter
      edc_counter #(CNTR_WIDTH) ucounter (
        .reset(reset),
        .sclk(sclk),
        .count_event(count_event[cnt]),
        .regbus_ws(regbus_ws),
        .count_wd(count_wd[cnt]),
        .regbus_rs(regbus_rs),
        .count(count[cnt])
      );
    end
  endgenerate

  // grbm interface
  wire [17:0] regbus_aa = {regbus_a, 2'h0};
  assign      regbus_rs = regbus_re & (regbus_aa == EDC_PERF_REG_ADDR);
  assign      regbus_ws = regbus_we & (regbus_aa == EDC_PERF_REG_ADDR);
  generate
    if (NUM_CNTR==1) begin
      always @(*) begin
        regbus_rd = 32'b0;
        if (regbus_rs) regbus_rd[CNTR_WIDTH-1:0] = count[0];
      end // always
    end // NUM_CNTR==1
    else if (NUM_CNTR==2) begin
      always @(*) begin
        regbus_rd = 32'b0;
        if (regbus_rs) regbus_rd[CNTR_WIDTH-1:0] = count[0];
        if (regbus_rs) regbus_rd[CNTR_WIDTH-1+16:16] = count[1];
      end // always
    end // NUM_CNTR==2
    else begin // else if (NUM_CNTR==4) begin
      always @(*) begin
        regbus_rd = 32'b0;
        if (regbus_rs) regbus_rd[CNTR_WIDTH-1:0] = count[0];
        if (regbus_rs) regbus_rd[CNTR_WIDTH-1+8:8] = count[1];
        if (regbus_rs) regbus_rd[CNTR_WIDTH-1+16:16] = count[2];
        if (regbus_rs) regbus_rd[CNTR_WIDTH-1+24:24] = count[3];
      end // always
    end // NUM_CNTR==4
  endgenerate

  generate
    if (NUM_CNTR==1) begin
      always @(*) begin
        count_wd[0] = {CNTR_WIDTH{1'b0}};
        if (regbus_ws) count_wd[0] = regbus_wd[CNTR_WIDTH-1:0];
      end // always
    end // NUM_CNTR==1
    else if (NUM_CNTR==2) begin
      always @(*) begin
        count_wd[0] = {CNTR_WIDTH{1'b0}};
        count_wd[1] = {CNTR_WIDTH{1'b0}};
        if (regbus_ws) count_wd[0] = regbus_wd[CNTR_WIDTH-1:0];
        if (regbus_ws) count_wd[1] = regbus_wd[CNTR_WIDTH-1+16:16];
      end // always
    end // NUM_CNTR==2
    else begin // else if (NUM_CNTR==4) begin
      always @(*) begin
        count_wd[0] = {CNTR_WIDTH{1'b0}};
        count_wd[1] = {CNTR_WIDTH{1'b0}};
        count_wd[2] = {CNTR_WIDTH{1'b0}};
        count_wd[3] = {CNTR_WIDTH{1'b0}};
        if (regbus_ws) count_wd[0] = regbus_wd[CNTR_WIDTH-1:0];
        if (regbus_ws) count_wd[1] = regbus_wd[CNTR_WIDTH-1+8:8];
        if (regbus_ws) count_wd[2] = regbus_wd[CNTR_WIDTH-1+16:16];
        if (regbus_ws) count_wd[3] = regbus_wd[CNTR_WIDTH-1+24:24];
      end // always
    end // NUM_CNTR==4
  endgenerate

// river translate_on

endmodule

module edc_counter (
  reset,
  sclk,
  count_event,
  regbus_ws,
  count_wd,  
  regbus_rs,
  count
);

parameter CNTR_WIDTH=4;

  input reset;
  input sclk;
  input count_event;
  input regbus_rs;
  input regbus_ws;
  input [CNTR_WIDTH-1:0] count_wd; 
  // Read un-flopped count value, and we won't lose a clock of count
  output [CNTR_WIDTH-1:0] count;
  reg    [CNTR_WIDTH-1:0] q_count;

  assign count = q_count + (count_event&~(&q_count)); // inc until saturated
  always @(posedge sclk) begin
    if (reset | regbus_rs)
      q_count <= {CNTR_WIDTH{1'b0}};
    else if (regbus_ws)
      q_count <= count_wd;
    else if (count_event)
      q_count <= count;
  end // always

endmodule



