
module ati_edc_counter_new (
	regbus_rd,
	regbus_rs,
	regbus_re,
	regbus_a,
        regbus_we,
        regbus_wd,
	sclk,
	reset, // active high
        count_event
);

parameter EDC_PERF_REG_ADDR=0; // grbm address
parameter NUM_CNTR=1; // 1~16 are valid
parameter CNTR_WIDTH=2; // up to 32/NUM_CNTR
parameter ADDR_WIDTH=18;
    

output [31:0]           regbus_rd;
reg    [31:0]           regbus_rd;
output                  regbus_rs;
input                   regbus_re;
input  [ADDR_WIDTH-1:2] regbus_a;
input                   regbus_we;
input  [31:0]           regbus_wd; 
input                   sclk;
input                   reset;
input  [NUM_CNTR-1:0]   count_event;

// river translate_off
  wire regbus_ws;
  // NUM_CNTR counters of width CNTR_WIDTH
  wire [CNTR_WIDTH-1:0] count [NUM_CNTR-1:0];
  reg  [CNTR_WIDTH-1:0] count_wd [NUM_CNTR-1:0]; 
  genvar cnt;
  generate
    for (cnt=0; cnt<NUM_CNTR; cnt=cnt+1) begin : gen_edc_counter
      edc_counter_new #(CNTR_WIDTH) ucounter (
        .reset(reset),
        .sclk(sclk),
        .count_event(count_event[cnt]),
        .regbus_ws(regbus_ws),
        .count_wd(count_wd[cnt]),
        .regbus_rs(regbus_rs),
        .count(count[cnt])
      );
    end
  endgenerate

  // grbm interface
  wire [ADDR_WIDTH-1:0] regbus_aa = {regbus_a, 2'h0};
  assign                regbus_rs = regbus_re & (regbus_aa == EDC_PERF_REG_ADDR);
  assign                regbus_ws = regbus_we & (regbus_aa == EDC_PERF_REG_ADDR);

  // for read
  generate
    if (NUM_CNTR==1) begin
      always @(*) begin
        regbus_rd = 32'b0;
        if (regbus_rs) regbus_rd[CNTR_WIDTH*1-1:CNTR_WIDTH*0]   = count[0];
      end // always
    end // NUM_CNTR==1

    else if (NUM_CNTR==2) begin
      always @(*) begin
        regbus_rd = 32'b0;
        if (regbus_rs) regbus_rd[CNTR_WIDTH*1-1:CNTR_WIDTH*0]   = count[0];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*2-1:CNTR_WIDTH*1]   = count[1];
      end // always
    end // NUM_CNTR==2

    else if (NUM_CNTR==3) begin
      always @(*) begin
        regbus_rd = 32'b0;
        if (regbus_rs) regbus_rd[CNTR_WIDTH*1-1:CNTR_WIDTH*0]   = count[0];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*2-1:CNTR_WIDTH*1]   = count[1];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*3-1:CNTR_WIDTH*2]   = count[2];
      end // always
    end // NUM_CNTR==3

    else if (NUM_CNTR==4) begin
      always @(*) begin
        regbus_rd = 32'b0;
        if (regbus_rs) regbus_rd[CNTR_WIDTH*1-1:CNTR_WIDTH*0]   = count[0];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*2-1:CNTR_WIDTH*1]   = count[1];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*3-1:CNTR_WIDTH*2]   = count[2];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*4-1:CNTR_WIDTH*3]   = count[3];
      end // always
    end // NUM_CNTR==4

    else if (NUM_CNTR==5) begin
      always @(*) begin
        regbus_rd = 32'b0;
        if (regbus_rs) regbus_rd[CNTR_WIDTH*1-1:CNTR_WIDTH*0]   = count[0];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*2-1:CNTR_WIDTH*1]   = count[1];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*3-1:CNTR_WIDTH*2]   = count[2];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*4-1:CNTR_WIDTH*3]   = count[3];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*5-1:CNTR_WIDTH*4]   = count[4];
      end // always
    end // NUM_CNTR==5

    else if (NUM_CNTR==6) begin
      always @(*) begin
        regbus_rd = 32'b0;
        if (regbus_rs) regbus_rd[CNTR_WIDTH*1-1:CNTR_WIDTH*0]   = count[0];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*2-1:CNTR_WIDTH*1]   = count[1];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*3-1:CNTR_WIDTH*2]   = count[2];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*4-1:CNTR_WIDTH*3]   = count[3];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*5-1:CNTR_WIDTH*4]   = count[4];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*6-1:CNTR_WIDTH*5]   = count[5];
      end // always
    end // NUM_CNTR==6

    else if (NUM_CNTR==7) begin
      always @(*) begin
        regbus_rd = 32'b0;
        if (regbus_rs) regbus_rd[CNTR_WIDTH*1-1:CNTR_WIDTH*0]   = count[0];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*2-1:CNTR_WIDTH*1]   = count[1];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*3-1:CNTR_WIDTH*2]   = count[2];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*4-1:CNTR_WIDTH*3]   = count[3];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*5-1:CNTR_WIDTH*4]   = count[4];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*6-1:CNTR_WIDTH*5]   = count[5];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*7-1:CNTR_WIDTH*6]   = count[6];
      end // always
    end // NUM_CNTR==7

    else if (NUM_CNTR==8) begin
      always @(*) begin
        regbus_rd = 32'b0;
        if (regbus_rs) regbus_rd[CNTR_WIDTH*1-1:CNTR_WIDTH*0]   = count[0];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*2-1:CNTR_WIDTH*1]   = count[1];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*3-1:CNTR_WIDTH*2]   = count[2];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*4-1:CNTR_WIDTH*3]   = count[3];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*5-1:CNTR_WIDTH*4]   = count[4];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*6-1:CNTR_WIDTH*5]   = count[5];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*7-1:CNTR_WIDTH*6]   = count[6];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*8-1:CNTR_WIDTH*7]   = count[7];
      end // always
    end // NUM_CNTR==8

    else if (NUM_CNTR==9) begin
      always @(*) begin
        regbus_rd = 32'b0;
        if (regbus_rs) regbus_rd[CNTR_WIDTH*1-1:CNTR_WIDTH*0]   = count[0];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*2-1:CNTR_WIDTH*1]   = count[1];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*3-1:CNTR_WIDTH*2]   = count[2];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*4-1:CNTR_WIDTH*3]   = count[3];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*5-1:CNTR_WIDTH*4]   = count[4];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*6-1:CNTR_WIDTH*5]   = count[5];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*7-1:CNTR_WIDTH*6]   = count[6];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*8-1:CNTR_WIDTH*7]   = count[7];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*9-1:CNTR_WIDTH*8]   = count[8];
      end // always
    end // NUM_CNTR==9

    else if (NUM_CNTR==10) begin
      always @(*) begin
        regbus_rd = 32'b0;
        if (regbus_rs) regbus_rd[CNTR_WIDTH*1-1:CNTR_WIDTH*0]   = count[0];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*2-1:CNTR_WIDTH*1]   = count[1];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*3-1:CNTR_WIDTH*2]   = count[2];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*4-1:CNTR_WIDTH*3]   = count[3];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*5-1:CNTR_WIDTH*4]   = count[4];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*6-1:CNTR_WIDTH*5]   = count[5];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*7-1:CNTR_WIDTH*6]   = count[6];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*8-1:CNTR_WIDTH*7]   = count[7];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*9-1:CNTR_WIDTH*8]   = count[8];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*10-1:CNTR_WIDTH*9]  = count[9];
      end // always
    end // NUM_CNTR==10

    else if (NUM_CNTR==11) begin
      always @(*) begin
        regbus_rd = 32'b0;
        if (regbus_rs) regbus_rd[CNTR_WIDTH*1-1:CNTR_WIDTH*0]   = count[0];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*2-1:CNTR_WIDTH*1]   = count[1];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*3-1:CNTR_WIDTH*2]   = count[2];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*4-1:CNTR_WIDTH*3]   = count[3];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*5-1:CNTR_WIDTH*4]   = count[4];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*6-1:CNTR_WIDTH*5]   = count[5];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*7-1:CNTR_WIDTH*6]   = count[6];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*8-1:CNTR_WIDTH*7]   = count[7];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*9-1:CNTR_WIDTH*8]   = count[8];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*10-1:CNTR_WIDTH*9]  = count[9];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*11-1:CNTR_WIDTH*10] = count[10];
      end // always
    end // NUM_CNTR==11

    else if (NUM_CNTR==12) begin
      always @(*) begin
        regbus_rd = 32'b0;
        if (regbus_rs) regbus_rd[CNTR_WIDTH*1-1:CNTR_WIDTH*0]   = count[0];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*2-1:CNTR_WIDTH*1]   = count[1];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*3-1:CNTR_WIDTH*2]   = count[2];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*4-1:CNTR_WIDTH*3]   = count[3];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*5-1:CNTR_WIDTH*4]   = count[4];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*6-1:CNTR_WIDTH*5]   = count[5];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*7-1:CNTR_WIDTH*6]   = count[6];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*8-1:CNTR_WIDTH*7]   = count[7];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*9-1:CNTR_WIDTH*8]   = count[8];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*10-1:CNTR_WIDTH*9]  = count[9];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*11-1:CNTR_WIDTH*10] = count[10];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*12-1:CNTR_WIDTH*11] = count[11];
      end // always
    end // NUM_CNTR==12

    else if (NUM_CNTR==13) begin
      always @(*) begin
        regbus_rd = 32'b0;
        if (regbus_rs) regbus_rd[CNTR_WIDTH*1-1:CNTR_WIDTH*0]   = count[0];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*2-1:CNTR_WIDTH*1]   = count[1];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*3-1:CNTR_WIDTH*2]   = count[2];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*4-1:CNTR_WIDTH*3]   = count[3];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*5-1:CNTR_WIDTH*4]   = count[4];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*6-1:CNTR_WIDTH*5]   = count[5];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*7-1:CNTR_WIDTH*6]   = count[6];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*8-1:CNTR_WIDTH*7]   = count[7];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*9-1:CNTR_WIDTH*8]   = count[8];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*10-1:CNTR_WIDTH*9]  = count[9];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*11-1:CNTR_WIDTH*10] = count[10];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*12-1:CNTR_WIDTH*11] = count[11];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*13-1:CNTR_WIDTH*12] = count[12];
      end // always
    end // NUM_CNTR==13

    else if (NUM_CNTR==14) begin
      always @(*) begin
        regbus_rd = 32'b0;
        if (regbus_rs) regbus_rd[CNTR_WIDTH*1-1:CNTR_WIDTH*0]   = count[0];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*2-1:CNTR_WIDTH*1]   = count[1];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*3-1:CNTR_WIDTH*2]   = count[2];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*4-1:CNTR_WIDTH*3]   = count[3];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*5-1:CNTR_WIDTH*4]   = count[4];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*6-1:CNTR_WIDTH*5]   = count[5];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*7-1:CNTR_WIDTH*6]   = count[6];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*8-1:CNTR_WIDTH*7]   = count[7];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*9-1:CNTR_WIDTH*8]   = count[8];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*10-1:CNTR_WIDTH*9]  = count[9];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*11-1:CNTR_WIDTH*10] = count[10];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*12-1:CNTR_WIDTH*11] = count[11];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*13-1:CNTR_WIDTH*12] = count[12];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*14-1:CNTR_WIDTH*13] = count[13];
      end // always
    end // NUM_CNTR==14

    else if (NUM_CNTR==15) begin
      always @(*) begin
        regbus_rd = 32'b0;
        if (regbus_rs) regbus_rd[CNTR_WIDTH*1-1:CNTR_WIDTH*0]   = count[0];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*2-1:CNTR_WIDTH*1]   = count[1];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*3-1:CNTR_WIDTH*2]   = count[2];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*4-1:CNTR_WIDTH*3]   = count[3];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*5-1:CNTR_WIDTH*4]   = count[4];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*6-1:CNTR_WIDTH*5]   = count[5];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*7-1:CNTR_WIDTH*6]   = count[6];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*8-1:CNTR_WIDTH*7]   = count[7];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*9-1:CNTR_WIDTH*8]   = count[8];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*10-1:CNTR_WIDTH*9]  = count[9];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*11-1:CNTR_WIDTH*10] = count[10];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*12-1:CNTR_WIDTH*11] = count[11];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*13-1:CNTR_WIDTH*12] = count[12];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*14-1:CNTR_WIDTH*13] = count[13];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*15-1:CNTR_WIDTH*14] = count[14];
      end // always
    end // NUM_CNTR==15

    else begin // else if (NUM_CNTR==16) begin
      always @(*) begin
        regbus_rd = 32'b0;
        if (regbus_rs) regbus_rd[CNTR_WIDTH*1-1:CNTR_WIDTH*0]   = count[0];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*2-1:CNTR_WIDTH*1]   = count[1];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*3-1:CNTR_WIDTH*2]   = count[2];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*4-1:CNTR_WIDTH*3]   = count[3];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*5-1:CNTR_WIDTH*4]   = count[4];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*6-1:CNTR_WIDTH*5]   = count[5];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*7-1:CNTR_WIDTH*6]   = count[6];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*8-1:CNTR_WIDTH*7]   = count[7];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*9-1:CNTR_WIDTH*8]   = count[8];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*10-1:CNTR_WIDTH*9]  = count[9];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*11-1:CNTR_WIDTH*10] = count[10];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*12-1:CNTR_WIDTH*11] = count[11];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*13-1:CNTR_WIDTH*12] = count[12];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*14-1:CNTR_WIDTH*13] = count[13];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*15-1:CNTR_WIDTH*14] = count[14];
        if (regbus_rs) regbus_rd[CNTR_WIDTH*16-1:CNTR_WIDTH*15] = count[15];
      end // always
    end // NUM_CNTR==16
  endgenerate



  //for write
  generate
    if (NUM_CNTR==1) begin
      always @(*) begin
        count_wd[0]  = {CNTR_WIDTH{1'b0}};
        if (regbus_ws) count_wd[0]  = regbus_wd[CNTR_WIDTH*1-1:CNTR_WIDTH*0];
      end // always
    end // NUM_CNTR==1

    else if (NUM_CNTR==2) begin
      always @(*) begin
        count_wd[0]  = {CNTR_WIDTH{1'b0}};
        count_wd[1]  = {CNTR_WIDTH{1'b0}};
        if (regbus_ws) count_wd[0]  = regbus_wd[CNTR_WIDTH*1-1:CNTR_WIDTH*0];
        if (regbus_ws) count_wd[1]  = regbus_wd[CNTR_WIDTH*2-1:CNTR_WIDTH*1];
      end // always
    end // NUM_CNTR==2

    else if (NUM_CNTR==3) begin
      always @(*) begin
        count_wd[0]  = {CNTR_WIDTH{1'b0}};
        count_wd[1]  = {CNTR_WIDTH{1'b0}};
        count_wd[2]  = {CNTR_WIDTH{1'b0}};
        if (regbus_ws) count_wd[0]  = regbus_wd[CNTR_WIDTH*1-1:CNTR_WIDTH*0];
        if (regbus_ws) count_wd[1]  = regbus_wd[CNTR_WIDTH*2-1:CNTR_WIDTH*1];
        if (regbus_ws) count_wd[2]  = regbus_wd[CNTR_WIDTH*3-1:CNTR_WIDTH*2];
      end // always
    end // NUM_CNTR==3

    else if (NUM_CNTR==4) begin
      always @(*) begin
        count_wd[0]  = {CNTR_WIDTH{1'b0}};
        count_wd[1]  = {CNTR_WIDTH{1'b0}};
        count_wd[2]  = {CNTR_WIDTH{1'b0}};
        count_wd[3]  = {CNTR_WIDTH{1'b0}};
        if (regbus_ws) count_wd[0]  = regbus_wd[CNTR_WIDTH*1-1:CNTR_WIDTH*0];
        if (regbus_ws) count_wd[1]  = regbus_wd[CNTR_WIDTH*2-1:CNTR_WIDTH*1];
        if (regbus_ws) count_wd[2]  = regbus_wd[CNTR_WIDTH*3-1:CNTR_WIDTH*2];
        if (regbus_ws) count_wd[3]  = regbus_wd[CNTR_WIDTH*4-1:CNTR_WIDTH*3];
      end // always
    end // NUM_CNTR==4

    else if (NUM_CNTR==5) begin
      always @(*) begin
        count_wd[0]  = {CNTR_WIDTH{1'b0}};
        count_wd[1]  = {CNTR_WIDTH{1'b0}};
        count_wd[2]  = {CNTR_WIDTH{1'b0}};
        count_wd[3]  = {CNTR_WIDTH{1'b0}};
        count_wd[4]  = {CNTR_WIDTH{1'b0}};
        if (regbus_ws) count_wd[0]  = regbus_wd[CNTR_WIDTH*1-1:CNTR_WIDTH*0];
        if (regbus_ws) count_wd[1]  = regbus_wd[CNTR_WIDTH*2-1:CNTR_WIDTH*1];
        if (regbus_ws) count_wd[2]  = regbus_wd[CNTR_WIDTH*3-1:CNTR_WIDTH*2];
        if (regbus_ws) count_wd[3]  = regbus_wd[CNTR_WIDTH*4-1:CNTR_WIDTH*3];
        if (regbus_ws) count_wd[4]  = regbus_wd[CNTR_WIDTH*5-1:CNTR_WIDTH*4];
      end // always
    end // NUM_CNTR==5

    else if (NUM_CNTR==6) begin
      always @(*) begin
        count_wd[0]  = {CNTR_WIDTH{1'b0}};
        count_wd[1]  = {CNTR_WIDTH{1'b0}};
        count_wd[2]  = {CNTR_WIDTH{1'b0}};
        count_wd[3]  = {CNTR_WIDTH{1'b0}};
        count_wd[4]  = {CNTR_WIDTH{1'b0}};
        count_wd[5]  = {CNTR_WIDTH{1'b0}};
        if (regbus_ws) count_wd[0]  = regbus_wd[CNTR_WIDTH*1-1:CNTR_WIDTH*0];
        if (regbus_ws) count_wd[1]  = regbus_wd[CNTR_WIDTH*2-1:CNTR_WIDTH*1];
        if (regbus_ws) count_wd[2]  = regbus_wd[CNTR_WIDTH*3-1:CNTR_WIDTH*2];
        if (regbus_ws) count_wd[3]  = regbus_wd[CNTR_WIDTH*4-1:CNTR_WIDTH*3];
        if (regbus_ws) count_wd[4]  = regbus_wd[CNTR_WIDTH*5-1:CNTR_WIDTH*4];
        if (regbus_ws) count_wd[5]  = regbus_wd[CNTR_WIDTH*6-1:CNTR_WIDTH*5];
      end // always
    end // NUM_CNTR==6

    else if (NUM_CNTR==7) begin
      always @(*) begin
        count_wd[0]  = {CNTR_WIDTH{1'b0}};
        count_wd[1]  = {CNTR_WIDTH{1'b0}};
        count_wd[2]  = {CNTR_WIDTH{1'b0}};
        count_wd[3]  = {CNTR_WIDTH{1'b0}};
        count_wd[4]  = {CNTR_WIDTH{1'b0}};
        count_wd[5]  = {CNTR_WIDTH{1'b0}};
        count_wd[6]  = {CNTR_WIDTH{1'b0}};
        if (regbus_ws) count_wd[0]  = regbus_wd[CNTR_WIDTH*1-1:CNTR_WIDTH*0];
        if (regbus_ws) count_wd[1]  = regbus_wd[CNTR_WIDTH*2-1:CNTR_WIDTH*1];
        if (regbus_ws) count_wd[2]  = regbus_wd[CNTR_WIDTH*3-1:CNTR_WIDTH*2];
        if (regbus_ws) count_wd[3]  = regbus_wd[CNTR_WIDTH*4-1:CNTR_WIDTH*3];
        if (regbus_ws) count_wd[4]  = regbus_wd[CNTR_WIDTH*5-1:CNTR_WIDTH*4];
        if (regbus_ws) count_wd[5]  = regbus_wd[CNTR_WIDTH*6-1:CNTR_WIDTH*5];
        if (regbus_ws) count_wd[6]  = regbus_wd[CNTR_WIDTH*7-1:CNTR_WIDTH*6];
      end // always
    end // NUM_CNTR==7

    else if (NUM_CNTR==8) begin
      always @(*) begin
        count_wd[0]  = {CNTR_WIDTH{1'b0}};
        count_wd[1]  = {CNTR_WIDTH{1'b0}};
        count_wd[2]  = {CNTR_WIDTH{1'b0}};
        count_wd[3]  = {CNTR_WIDTH{1'b0}};
        count_wd[4]  = {CNTR_WIDTH{1'b0}};
        count_wd[5]  = {CNTR_WIDTH{1'b0}};
        count_wd[6]  = {CNTR_WIDTH{1'b0}};
        count_wd[7]  = {CNTR_WIDTH{1'b0}};
        if (regbus_ws) count_wd[0]  = regbus_wd[CNTR_WIDTH*1-1:CNTR_WIDTH*0];
        if (regbus_ws) count_wd[1]  = regbus_wd[CNTR_WIDTH*2-1:CNTR_WIDTH*1];
        if (regbus_ws) count_wd[2]  = regbus_wd[CNTR_WIDTH*3-1:CNTR_WIDTH*2];
        if (regbus_ws) count_wd[3]  = regbus_wd[CNTR_WIDTH*4-1:CNTR_WIDTH*3];
        if (regbus_ws) count_wd[4]  = regbus_wd[CNTR_WIDTH*5-1:CNTR_WIDTH*4];
        if (regbus_ws) count_wd[5]  = regbus_wd[CNTR_WIDTH*6-1:CNTR_WIDTH*5];
        if (regbus_ws) count_wd[6]  = regbus_wd[CNTR_WIDTH*7-1:CNTR_WIDTH*6];
        if (regbus_ws) count_wd[7]  = regbus_wd[CNTR_WIDTH*8-1:CNTR_WIDTH*7];
      end // always
    end // NUM_CNTR==8

    else if (NUM_CNTR==9) begin
      always @(*) begin
        count_wd[0]  = {CNTR_WIDTH{1'b0}};
        count_wd[1]  = {CNTR_WIDTH{1'b0}};
        count_wd[2]  = {CNTR_WIDTH{1'b0}};
        count_wd[3]  = {CNTR_WIDTH{1'b0}};
        count_wd[4]  = {CNTR_WIDTH{1'b0}};
        count_wd[5]  = {CNTR_WIDTH{1'b0}};
        count_wd[6]  = {CNTR_WIDTH{1'b0}};
        count_wd[7]  = {CNTR_WIDTH{1'b0}};
        count_wd[8]  = {CNTR_WIDTH{1'b0}};
        if (regbus_ws) count_wd[0]  = regbus_wd[CNTR_WIDTH*1-1:CNTR_WIDTH*0];
        if (regbus_ws) count_wd[1]  = regbus_wd[CNTR_WIDTH*2-1:CNTR_WIDTH*1];
        if (regbus_ws) count_wd[2]  = regbus_wd[CNTR_WIDTH*3-1:CNTR_WIDTH*2];
        if (regbus_ws) count_wd[3]  = regbus_wd[CNTR_WIDTH*4-1:CNTR_WIDTH*3];
        if (regbus_ws) count_wd[4]  = regbus_wd[CNTR_WIDTH*5-1:CNTR_WIDTH*4];
        if (regbus_ws) count_wd[5]  = regbus_wd[CNTR_WIDTH*6-1:CNTR_WIDTH*5];
        if (regbus_ws) count_wd[6]  = regbus_wd[CNTR_WIDTH*7-1:CNTR_WIDTH*6];
        if (regbus_ws) count_wd[7]  = regbus_wd[CNTR_WIDTH*8-1:CNTR_WIDTH*7];
        if (regbus_ws) count_wd[8]  = regbus_wd[CNTR_WIDTH*9-1:CNTR_WIDTH*8];
      end // always
    end // NUM_CNTR==9

    else if (NUM_CNTR==10) begin
      always @(*) begin
        count_wd[0]  = {CNTR_WIDTH{1'b0}};
        count_wd[1]  = {CNTR_WIDTH{1'b0}};
        count_wd[2]  = {CNTR_WIDTH{1'b0}};
        count_wd[3]  = {CNTR_WIDTH{1'b0}};
        count_wd[4]  = {CNTR_WIDTH{1'b0}};
        count_wd[5]  = {CNTR_WIDTH{1'b0}};
        count_wd[6]  = {CNTR_WIDTH{1'b0}};
        count_wd[7]  = {CNTR_WIDTH{1'b0}};
        count_wd[8]  = {CNTR_WIDTH{1'b0}};
        count_wd[9]  = {CNTR_WIDTH{1'b0}};
        if (regbus_ws) count_wd[0]  = regbus_wd[CNTR_WIDTH*1-1:CNTR_WIDTH*0];
        if (regbus_ws) count_wd[1]  = regbus_wd[CNTR_WIDTH*2-1:CNTR_WIDTH*1];
        if (regbus_ws) count_wd[2]  = regbus_wd[CNTR_WIDTH*3-1:CNTR_WIDTH*2];
        if (regbus_ws) count_wd[3]  = regbus_wd[CNTR_WIDTH*4-1:CNTR_WIDTH*3];
        if (regbus_ws) count_wd[4]  = regbus_wd[CNTR_WIDTH*5-1:CNTR_WIDTH*4];
        if (regbus_ws) count_wd[5]  = regbus_wd[CNTR_WIDTH*6-1:CNTR_WIDTH*5];
        if (regbus_ws) count_wd[6]  = regbus_wd[CNTR_WIDTH*7-1:CNTR_WIDTH*6];
        if (regbus_ws) count_wd[7]  = regbus_wd[CNTR_WIDTH*8-1:CNTR_WIDTH*7];
        if (regbus_ws) count_wd[8]  = regbus_wd[CNTR_WIDTH*9-1:CNTR_WIDTH*8];
        if (regbus_ws) count_wd[9]  = regbus_wd[CNTR_WIDTH*10-1:CNTR_WIDTH*9];
      end // always
    end // NUM_CNTR==10

    else if (NUM_CNTR==11) begin
      always @(*) begin
        count_wd[0]  = {CNTR_WIDTH{1'b0}};
        count_wd[1]  = {CNTR_WIDTH{1'b0}};
        count_wd[2]  = {CNTR_WIDTH{1'b0}};
        count_wd[3]  = {CNTR_WIDTH{1'b0}};
        count_wd[4]  = {CNTR_WIDTH{1'b0}};
        count_wd[5]  = {CNTR_WIDTH{1'b0}};
        count_wd[6]  = {CNTR_WIDTH{1'b0}};
        count_wd[7]  = {CNTR_WIDTH{1'b0}};
        count_wd[8]  = {CNTR_WIDTH{1'b0}};
        count_wd[9]  = {CNTR_WIDTH{1'b0}};
        count_wd[10] = {CNTR_WIDTH{1'b0}};
        if (regbus_ws) count_wd[0]  = regbus_wd[CNTR_WIDTH*1-1:CNTR_WIDTH*0];
        if (regbus_ws) count_wd[1]  = regbus_wd[CNTR_WIDTH*2-1:CNTR_WIDTH*1];
        if (regbus_ws) count_wd[2]  = regbus_wd[CNTR_WIDTH*3-1:CNTR_WIDTH*2];
        if (regbus_ws) count_wd[3]  = regbus_wd[CNTR_WIDTH*4-1:CNTR_WIDTH*3];
        if (regbus_ws) count_wd[4]  = regbus_wd[CNTR_WIDTH*5-1:CNTR_WIDTH*4];
        if (regbus_ws) count_wd[5]  = regbus_wd[CNTR_WIDTH*6-1:CNTR_WIDTH*5];
        if (regbus_ws) count_wd[6]  = regbus_wd[CNTR_WIDTH*7-1:CNTR_WIDTH*6];
        if (regbus_ws) count_wd[7]  = regbus_wd[CNTR_WIDTH*8-1:CNTR_WIDTH*7];
        if (regbus_ws) count_wd[8]  = regbus_wd[CNTR_WIDTH*9-1:CNTR_WIDTH*8];
        if (regbus_ws) count_wd[9]  = regbus_wd[CNTR_WIDTH*10-1:CNTR_WIDTH*9];
        if (regbus_ws) count_wd[10] = regbus_wd[CNTR_WIDTH*11-1:CNTR_WIDTH*10];
      end // always
    end // NUM_CNTR==11

    else if (NUM_CNTR==12) begin
      always @(*) begin
        count_wd[0]  = {CNTR_WIDTH{1'b0}};
        count_wd[1]  = {CNTR_WIDTH{1'b0}};
        count_wd[2]  = {CNTR_WIDTH{1'b0}};
        count_wd[3]  = {CNTR_WIDTH{1'b0}};
        count_wd[4]  = {CNTR_WIDTH{1'b0}};
        count_wd[5]  = {CNTR_WIDTH{1'b0}};
        count_wd[6]  = {CNTR_WIDTH{1'b0}};
        count_wd[7]  = {CNTR_WIDTH{1'b0}};
        count_wd[8]  = {CNTR_WIDTH{1'b0}};
        count_wd[9]  = {CNTR_WIDTH{1'b0}};
        count_wd[10] = {CNTR_WIDTH{1'b0}};
        count_wd[11] = {CNTR_WIDTH{1'b0}};
        if (regbus_ws) count_wd[0]  = regbus_wd[CNTR_WIDTH*1-1:CNTR_WIDTH*0];
        if (regbus_ws) count_wd[1]  = regbus_wd[CNTR_WIDTH*2-1:CNTR_WIDTH*1];
        if (regbus_ws) count_wd[2]  = regbus_wd[CNTR_WIDTH*3-1:CNTR_WIDTH*2];
        if (regbus_ws) count_wd[3]  = regbus_wd[CNTR_WIDTH*4-1:CNTR_WIDTH*3];
        if (regbus_ws) count_wd[4]  = regbus_wd[CNTR_WIDTH*5-1:CNTR_WIDTH*4];
        if (regbus_ws) count_wd[5]  = regbus_wd[CNTR_WIDTH*6-1:CNTR_WIDTH*5];
        if (regbus_ws) count_wd[6]  = regbus_wd[CNTR_WIDTH*7-1:CNTR_WIDTH*6];
        if (regbus_ws) count_wd[7]  = regbus_wd[CNTR_WIDTH*8-1:CNTR_WIDTH*7];
        if (regbus_ws) count_wd[8]  = regbus_wd[CNTR_WIDTH*9-1:CNTR_WIDTH*8];
        if (regbus_ws) count_wd[9]  = regbus_wd[CNTR_WIDTH*10-1:CNTR_WIDTH*9];
        if (regbus_ws) count_wd[10] = regbus_wd[CNTR_WIDTH*11-1:CNTR_WIDTH*10];
        if (regbus_ws) count_wd[11] = regbus_wd[CNTR_WIDTH*12-1:CNTR_WIDTH*11];
      end // always
    end // NUM_CNTR==12

    else if (NUM_CNTR==13) begin
      always @(*) begin
        count_wd[0]  = {CNTR_WIDTH{1'b0}};
        count_wd[1]  = {CNTR_WIDTH{1'b0}};
        count_wd[2]  = {CNTR_WIDTH{1'b0}};
        count_wd[3]  = {CNTR_WIDTH{1'b0}};
        count_wd[4]  = {CNTR_WIDTH{1'b0}};
        count_wd[5]  = {CNTR_WIDTH{1'b0}};
        count_wd[6]  = {CNTR_WIDTH{1'b0}};
        count_wd[7]  = {CNTR_WIDTH{1'b0}};
        count_wd[8]  = {CNTR_WIDTH{1'b0}};
        count_wd[9]  = {CNTR_WIDTH{1'b0}};
        count_wd[10] = {CNTR_WIDTH{1'b0}};
        count_wd[11] = {CNTR_WIDTH{1'b0}};
        count_wd[12] = {CNTR_WIDTH{1'b0}};
        if (regbus_ws) count_wd[0]  = regbus_wd[CNTR_WIDTH*1-1:CNTR_WIDTH*0];
        if (regbus_ws) count_wd[1]  = regbus_wd[CNTR_WIDTH*2-1:CNTR_WIDTH*1];
        if (regbus_ws) count_wd[2]  = regbus_wd[CNTR_WIDTH*3-1:CNTR_WIDTH*2];
        if (regbus_ws) count_wd[3]  = regbus_wd[CNTR_WIDTH*4-1:CNTR_WIDTH*3];
        if (regbus_ws) count_wd[4]  = regbus_wd[CNTR_WIDTH*5-1:CNTR_WIDTH*4];
        if (regbus_ws) count_wd[5]  = regbus_wd[CNTR_WIDTH*6-1:CNTR_WIDTH*5];
        if (regbus_ws) count_wd[6]  = regbus_wd[CNTR_WIDTH*7-1:CNTR_WIDTH*6];
        if (regbus_ws) count_wd[7]  = regbus_wd[CNTR_WIDTH*8-1:CNTR_WIDTH*7];
        if (regbus_ws) count_wd[8]  = regbus_wd[CNTR_WIDTH*9-1:CNTR_WIDTH*8];
        if (regbus_ws) count_wd[9]  = regbus_wd[CNTR_WIDTH*10-1:CNTR_WIDTH*9];
        if (regbus_ws) count_wd[10] = regbus_wd[CNTR_WIDTH*11-1:CNTR_WIDTH*10];
        if (regbus_ws) count_wd[11] = regbus_wd[CNTR_WIDTH*12-1:CNTR_WIDTH*11];
        if (regbus_ws) count_wd[12] = regbus_wd[CNTR_WIDTH*13-1:CNTR_WIDTH*12];
      end // always
    end // NUM_CNTR==13

    else if (NUM_CNTR==14) begin
      always @(*) begin
        count_wd[0]  = {CNTR_WIDTH{1'b0}};
        count_wd[1]  = {CNTR_WIDTH{1'b0}};
        count_wd[2]  = {CNTR_WIDTH{1'b0}};
        count_wd[3]  = {CNTR_WIDTH{1'b0}};
        count_wd[4]  = {CNTR_WIDTH{1'b0}};
        count_wd[5]  = {CNTR_WIDTH{1'b0}};
        count_wd[6]  = {CNTR_WIDTH{1'b0}};
        count_wd[7]  = {CNTR_WIDTH{1'b0}};
        count_wd[8]  = {CNTR_WIDTH{1'b0}};
        count_wd[9]  = {CNTR_WIDTH{1'b0}};
        count_wd[10] = {CNTR_WIDTH{1'b0}};
        count_wd[11] = {CNTR_WIDTH{1'b0}};
        count_wd[12] = {CNTR_WIDTH{1'b0}};
        count_wd[13] = {CNTR_WIDTH{1'b0}};
        if (regbus_ws) count_wd[0]  = regbus_wd[CNTR_WIDTH*1-1:CNTR_WIDTH*0];
        if (regbus_ws) count_wd[1]  = regbus_wd[CNTR_WIDTH*2-1:CNTR_WIDTH*1];
        if (regbus_ws) count_wd[2]  = regbus_wd[CNTR_WIDTH*3-1:CNTR_WIDTH*2];
        if (regbus_ws) count_wd[3]  = regbus_wd[CNTR_WIDTH*4-1:CNTR_WIDTH*3];
        if (regbus_ws) count_wd[4]  = regbus_wd[CNTR_WIDTH*5-1:CNTR_WIDTH*4];
        if (regbus_ws) count_wd[5]  = regbus_wd[CNTR_WIDTH*6-1:CNTR_WIDTH*5];
        if (regbus_ws) count_wd[6]  = regbus_wd[CNTR_WIDTH*7-1:CNTR_WIDTH*6];
        if (regbus_ws) count_wd[7]  = regbus_wd[CNTR_WIDTH*8-1:CNTR_WIDTH*7];
        if (regbus_ws) count_wd[8]  = regbus_wd[CNTR_WIDTH*9-1:CNTR_WIDTH*8];
        if (regbus_ws) count_wd[9]  = regbus_wd[CNTR_WIDTH*10-1:CNTR_WIDTH*9];
        if (regbus_ws) count_wd[10] = regbus_wd[CNTR_WIDTH*11-1:CNTR_WIDTH*10];
        if (regbus_ws) count_wd[11] = regbus_wd[CNTR_WIDTH*12-1:CNTR_WIDTH*11];
        if (regbus_ws) count_wd[12] = regbus_wd[CNTR_WIDTH*13-1:CNTR_WIDTH*12];
        if (regbus_ws) count_wd[13] = regbus_wd[CNTR_WIDTH*14-1:CNTR_WIDTH*13];
      end // always
    end // NUM_CNTR==14

    else if (NUM_CNTR==15) begin
      always @(*) begin
        count_wd[0]  = {CNTR_WIDTH{1'b0}};
        count_wd[1]  = {CNTR_WIDTH{1'b0}};
        count_wd[2]  = {CNTR_WIDTH{1'b0}};
        count_wd[3]  = {CNTR_WIDTH{1'b0}};
        count_wd[4]  = {CNTR_WIDTH{1'b0}};
        count_wd[5]  = {CNTR_WIDTH{1'b0}};
        count_wd[6]  = {CNTR_WIDTH{1'b0}};
        count_wd[7]  = {CNTR_WIDTH{1'b0}};
        count_wd[8]  = {CNTR_WIDTH{1'b0}};
        count_wd[9]  = {CNTR_WIDTH{1'b0}};
        count_wd[10] = {CNTR_WIDTH{1'b0}};
        count_wd[11] = {CNTR_WIDTH{1'b0}};
        count_wd[12] = {CNTR_WIDTH{1'b0}};
        count_wd[13] = {CNTR_WIDTH{1'b0}};
        count_wd[14] = {CNTR_WIDTH{1'b0}};
        if (regbus_ws) count_wd[0]  = regbus_wd[CNTR_WIDTH*1-1:CNTR_WIDTH*0];
        if (regbus_ws) count_wd[1]  = regbus_wd[CNTR_WIDTH*2-1:CNTR_WIDTH*1];
        if (regbus_ws) count_wd[2]  = regbus_wd[CNTR_WIDTH*3-1:CNTR_WIDTH*2];
        if (regbus_ws) count_wd[3]  = regbus_wd[CNTR_WIDTH*4-1:CNTR_WIDTH*3];
        if (regbus_ws) count_wd[4]  = regbus_wd[CNTR_WIDTH*5-1:CNTR_WIDTH*4];
        if (regbus_ws) count_wd[5]  = regbus_wd[CNTR_WIDTH*6-1:CNTR_WIDTH*5];
        if (regbus_ws) count_wd[6]  = regbus_wd[CNTR_WIDTH*7-1:CNTR_WIDTH*6];
        if (regbus_ws) count_wd[7]  = regbus_wd[CNTR_WIDTH*8-1:CNTR_WIDTH*7];
        if (regbus_ws) count_wd[8]  = regbus_wd[CNTR_WIDTH*9-1:CNTR_WIDTH*8];
        if (regbus_ws) count_wd[9]  = regbus_wd[CNTR_WIDTH*10-1:CNTR_WIDTH*9];
        if (regbus_ws) count_wd[10] = regbus_wd[CNTR_WIDTH*11-1:CNTR_WIDTH*10];
        if (regbus_ws) count_wd[11] = regbus_wd[CNTR_WIDTH*12-1:CNTR_WIDTH*11];
        if (regbus_ws) count_wd[12] = regbus_wd[CNTR_WIDTH*13-1:CNTR_WIDTH*12];
        if (regbus_ws) count_wd[13] = regbus_wd[CNTR_WIDTH*14-1:CNTR_WIDTH*13];
        if (regbus_ws) count_wd[14] = regbus_wd[CNTR_WIDTH*15-1:CNTR_WIDTH*14];
      end // always
    end // NUM_CNTR==15

    else begin // else if (NUM_CNTR==16) begin
      always @(*) begin
        count_wd[0]  = {CNTR_WIDTH{1'b0}};
        count_wd[1]  = {CNTR_WIDTH{1'b0}};
        count_wd[2]  = {CNTR_WIDTH{1'b0}};
        count_wd[3]  = {CNTR_WIDTH{1'b0}};
        count_wd[4]  = {CNTR_WIDTH{1'b0}};
        count_wd[5]  = {CNTR_WIDTH{1'b0}};
        count_wd[6]  = {CNTR_WIDTH{1'b0}};
        count_wd[7]  = {CNTR_WIDTH{1'b0}};
        count_wd[8]  = {CNTR_WIDTH{1'b0}};
        count_wd[9]  = {CNTR_WIDTH{1'b0}};
        count_wd[10] = {CNTR_WIDTH{1'b0}};
        count_wd[11] = {CNTR_WIDTH{1'b0}};
        count_wd[12] = {CNTR_WIDTH{1'b0}};
        count_wd[13] = {CNTR_WIDTH{1'b0}};
        count_wd[14] = {CNTR_WIDTH{1'b0}};
        count_wd[15] = {CNTR_WIDTH{1'b0}};
        if (regbus_ws) count_wd[0]  = regbus_wd[CNTR_WIDTH*1-1:CNTR_WIDTH*0];
        if (regbus_ws) count_wd[1]  = regbus_wd[CNTR_WIDTH*2-1:CNTR_WIDTH*1];
        if (regbus_ws) count_wd[2]  = regbus_wd[CNTR_WIDTH*3-1:CNTR_WIDTH*2];
        if (regbus_ws) count_wd[3]  = regbus_wd[CNTR_WIDTH*4-1:CNTR_WIDTH*3];
        if (regbus_ws) count_wd[4]  = regbus_wd[CNTR_WIDTH*5-1:CNTR_WIDTH*4];
        if (regbus_ws) count_wd[5]  = regbus_wd[CNTR_WIDTH*6-1:CNTR_WIDTH*5];
        if (regbus_ws) count_wd[6]  = regbus_wd[CNTR_WIDTH*7-1:CNTR_WIDTH*6];
        if (regbus_ws) count_wd[7]  = regbus_wd[CNTR_WIDTH*8-1:CNTR_WIDTH*7];
        if (regbus_ws) count_wd[8]  = regbus_wd[CNTR_WIDTH*9-1:CNTR_WIDTH*8];
        if (regbus_ws) count_wd[9]  = regbus_wd[CNTR_WIDTH*10-1:CNTR_WIDTH*9];
        if (regbus_ws) count_wd[10] = regbus_wd[CNTR_WIDTH*11-1:CNTR_WIDTH*10];
        if (regbus_ws) count_wd[11] = regbus_wd[CNTR_WIDTH*12-1:CNTR_WIDTH*11];
        if (regbus_ws) count_wd[12] = regbus_wd[CNTR_WIDTH*13-1:CNTR_WIDTH*12];
        if (regbus_ws) count_wd[13] = regbus_wd[CNTR_WIDTH*14-1:CNTR_WIDTH*13];
        if (regbus_ws) count_wd[14] = regbus_wd[CNTR_WIDTH*15-1:CNTR_WIDTH*14];
        if (regbus_ws) count_wd[15] = regbus_wd[CNTR_WIDTH*16-1:CNTR_WIDTH*15];
      end // always
    end // NUM_CNTR==16
  endgenerate

// river translate_on

endmodule

module edc_counter_new (
  reset,
  sclk,
  count_event,
  regbus_ws,
  count_wd,  
  regbus_rs,
  count
);

parameter CNTR_WIDTH=4;

  input reset;
  input sclk;
  input count_event;
  input regbus_rs;
  input regbus_ws;
  input [CNTR_WIDTH-1:0] count_wd; 
  // Read un-flopped count value, and we won't lose a clock of count
  output [CNTR_WIDTH-1:0] count;
  reg    [CNTR_WIDTH-1:0] q_count;

  assign count = q_count + (count_event&~(&q_count)); // inc until saturated
  always @(posedge sclk) begin
    if (regbus_rs)
      q_count <= {CNTR_WIDTH{1'b0}};
    else if (regbus_ws)
      q_count <= count_wd;
    else if (count_event)
      q_count <= count;
  end // always

endmodule



