//VCS coverage exclude_file
module ati_fifo_cover_assert(/*AUTOARG*/
   // Inputs
   clk, resetb, push, pop
   );

   parameter NUM_ENTRIES = 30; // This does not need to be power of 2
   
   parameter ASSERT_FULL_AND_PUSH = 1;
   parameter ASSERT_EMPTY_AND_POP = 1;
   parameter COVER_PUSH_AND_POP = 1;
   parameter COVER_FULL = 1;
   parameter COVER_FULL_EMPTY_FULL = 1;
   parameter ASSERT_EMPTY_EOS = 1;   
   parameter COVER_HALF_FULL = 1;
   parameter COVER_FULL_MINUS_TWO = 1;
   parameter COVER_FULL_MINUS_ONE = 1;

   parameter NUM_ENTRIES_L2 =   NUM_ENTRIES>1024 ? 11 :
                                NUM_ENTRIES>512 ? 10 :
				NUM_ENTRIES>256 ? 9 : 
                                NUM_ENTRIES>128 ? 8 : 
                                NUM_ENTRIES>64 ? 7 : 
                                NUM_ENTRIES>32 ? 6 : 
                                NUM_ENTRIES>16 ? 5 : 
                                NUM_ENTRIES>8  ? 4 :
                                NUM_ENTRIES>4  ? 3 :
                                NUM_ENTRIES>2  ? 2 : 1;

   input clk;
   input resetb;
   
   input push;
   input pop;
   
//VCS coverage off
   wire  reset = ~resetb;
   
   reg [NUM_ENTRIES_L2:0] count_ff;
   wire [NUM_ENTRIES_L2:0] count = 
	((~push && ~pop) ? count_ff     : {NUM_ENTRIES_L2{1'b0}})
      |	((~push &&  pop) ? count_ff - 1 : {NUM_ENTRIES_L2{1'b0}})
      |	(( push && ~pop) ? count_ff + 1 : {NUM_ENTRIES_L2{1'b0}})
      |	(( push &&  pop) ? count_ff     : {NUM_ENTRIES_L2{1'b0}});
   wire empty = count_ff == {NUM_ENTRIES_L2+1{1'b0}};
   wire full =  count_ff == NUM_ENTRIES;
   wire half_full = count_ff == NUM_ENTRIES>>1;
   wire full_minus_two = count_ff == NUM_ENTRIES-2;
   wire full_minus_one = count_ff == NUM_ENTRIES-1;
   always @(posedge clk) begin
	if (~resetb) 
	   count_ff <= {NUM_ENTRIES_L2+1{1'b0}};
        else 
           count_ff <= count;
   end
       
   ati_basic_fifo_cover_assert #(ASSERT_FULL_AND_PUSH, 
				 ASSERT_EMPTY_AND_POP, 
				 COVER_PUSH_AND_POP,
				 COVER_FULL,
	                         COVER_FULL_EMPTY_FULL,
				 ASSERT_EMPTY_EOS
				 ) 
   ati_basic_fifo_cover_assert
       ( /*AUTOINST*/
	// Inputs
	.clk				(clk),
	.reset				(reset),
	.empty				(empty),
	.full				(full),
	.push				(push),
	.pop				(pop)); 

generate
   if (COVER_HALF_FULL) begin
     //psl half_full_e : cover {resetb && half_full} @(posedge clk);
   end
   if (COVER_FULL_MINUS_TWO) begin
     //psl full_minus_two_e : cover {resetb && full_minus_two} @(posedge clk);
   end
   if (COVER_FULL_MINUS_ONE) begin
     //psl full_minus_one_e : cover {resetb && full_minus_one} @(posedge clk);
   end
endgenerate

//VCS coverage on   

endmodule // ati_fifo_cover_assert





