//--------------------------------------------------------------------------
//
// $Id: $
//
// $Change: $
//
// Notes:     This is a flopped out version of ati_fifo_ctrl_pipe
//
// Copyright: Trade secret of ATI Technologies, Inc.
//            (C) Copyright 2001-2005, ATI Technologies, Inc., (unpublished) 
//
//        All rights reserved.  This notice is intended as a precaution against
//        inadvertent publication and does not imply publication or any waiver
//        of confidentiality.  The year included in the foregoing notice is the
//        year of creation of the work.
//
// Author: Kamal Pillai (ppillai@ati.com)
//
//--------------------------------------------------------------------------
module ati_fifo_ctrl_pipe_flopped(/*AUTOARG*/
   // Outputs
   read_data, full, empty, busy, mem_waddr, mem_we, mem_raddr, mem_re, 
   // Inputs
   we, re, mem_data, clk, reset
   );

   //**************
   //* Parameters *
   //**************
   parameter FIFO_WIDTH = 1;
   parameter FIFO_DEPTH = 2;
   parameter SKID_WORDS = 0;
   parameter SKID_FULL  = 0;
   parameter FIFO_NAME  = "";
   parameter ADDR_WIDTH = ((FIFO_DEPTH <= 4) ? 2 :
                           (FIFO_DEPTH <= 8) ? 3 :
                           (FIFO_DEPTH <= 16) ? 4 :
                           (FIFO_DEPTH <= 32) ? 5 :
                           (FIFO_DEPTH <= 64) ? 6 :
                           (FIFO_DEPTH <= 128) ? 7 :
                           (FIFO_DEPTH <= 256) ? 8 :
                           (FIFO_DEPTH <= 512) ? 9 : 
			   (FIFO_DEPTH <= 1024) ? 10 :
			   (FIFO_DEPTH <= 2048) ? 11 : 12);  // Bigger than 4096???  i hope not.
   
   //***********************
   //* Output Declarations *
   //***********************
   output [FIFO_WIDTH-1:0] read_data;     // Flopped data from Virage RAM
   output 		   full;          // FIFO Full signal
   output 		   empty;         // FIFO Empty signal
   output 		   busy;
   output [ADDR_WIDTH-1:0] mem_waddr;     // Write Address for RAM
   output 		   mem_we;        // Write Enable 
   output [ADDR_WIDTH-1:0] mem_raddr;     // Read Address for RAM
   output 		   mem_re;        // Read Enable for RAM
			      
   //**********************
   //* Input Declarations *
   //**********************
   input 		   we;            // Write Enable
   input 		   re;            // Read Enable
   input [FIFO_WIDTH-1:0]  mem_data;      // Data from Virage MEM  
   input 		   clk;            
   input 		   reset;

   wire            int_busy;
   wire            int_re;
   wire            int_empty;
   wire [FIFO_WIDTH-1:0] int_data;

   ati_fifo_ctrl_pipe #(FIFO_WIDTH, FIFO_DEPTH, SKID_WORDS, SKID_FULL, FIFO_NAME, ADDR_WIDTH)
    uati_fifo_ctrl_pipe(.read_data( int_data ),
                        .full( full ),
                        .empty( int_empty ),
                        .busy( int_busy ),
                        .mem_waddr( mem_waddr ),
                        .mem_we( mem_we ),
                        .mem_raddr( mem_raddr ),
                        .mem_re( mem_re ),

                        .we( we ),
                        .re( int_re ),
                        .mem_data( mem_data ),
                        .clk( clk ),
                        .reset( reset )
                        );

   ati_fifo_flopper #(FIFO_WIDTH)
    uati_fifo_flopper(.clk( clk ),
                      .reset( reset ),
                      .int_empty( int_empty ),
                      .int_data( int_data ),
                      .int_re( int_re ),
                      .ext_empty( empty ),
                      .ext_data( read_data ),
                      .ext_re( re )
                      );

   assign                busy = int_busy | ~empty;

`ifdef SIMULATION
   // Adding cover and asserts
   ati_basic_fifo_cover_assert #(1, 1, 1) ati_shift_basic_cover_assert(.clk(clk), .reset(reset), .empty(empty), .full(full), .push(we), .pop(re));
`endif

endmodule // ati_fifo_ctrl_pipe_flopped

