//--------------------------------------------------------------------------
//
// $Id: $
//
// $Change: $
//
// Notes:     This module takes empty and data signals from a standard
//            fifo controller and flops the empty and data signals going out.
//            This module adds latency, and extra area from the data flops, but
//            maintains throughput of the fifo
//
// Copyright: Trade secret of ATI Technologies, Inc.
//            (C) Copyright 2001-2005, ATI Technologies, Inc., (unpublished) 
//
//        All rights reserved.  This notice is intended as a precaution against
//        inadvertent publication and does not imply publication or any waiver
//        of confidentiality.  The year included in the foregoing notice is the
//        year of creation of the work.
//
// Author: Kamal Pillai (ppillai@ati.com)
//
//--------------------------------------------------------------------------

module ati_fifo_flopper(/*AUTOARG*/
   // Outputs
   int_re, ext_empty, ext_data, 
   // Inputs
   clk, reset, int_empty, int_data, ext_re
   );

   parameter                FIFO_WIDTH = 1;

   input                    clk;
   input                    reset;

   // These are the empty and data signals comign from the fifo controller
   input                    int_empty;
   input [FIFO_WIDTH-1:0]   int_data;

   // This is the read signal going to the fifo controller
   output                   int_re;

   // These are the flpped empty and data signals going to external logic
   output                   ext_empty;
   output [FIFO_WIDTH-1:0]  ext_data;

   // This is the read signal coming from the external logic
   input                    ext_re;

   // These signals get flopped
   reg                      ext_empty;
   reg [FIFO_WIDTH-1:0]     ext_data;

   // We read from the fifo controller if the fifo is not empty and we
   // externally say we are or if the external logic just popped an entry
   assign int_re = ~int_empty & (ext_empty | ext_re);

   always @( posedge clk ) begin
      // if the external logic reads, then we say we are empty if the fifo is empty
      // otherwise, we stay as we are until we read from the fifo, in which case
      // we go back to being not empty
      ext_empty <= (ext_re) ? int_empty : ext_empty & ~int_re;

      // if we read from the fifo, flop the data out
      // otherwise, keep the data we already have
      ext_data <= (int_re) ? int_data : ext_data;

      // reset the empty signal
      if( reset ) ext_empty <= 1'b1;
   end // always @ ( posedge clk )

endmodule // ati_fifo_flopper
