/*
 *  Trade secret of AMD, Inc.
 *  Copyright 2010, AMD, Inc., (unpublished)
 *
 *  All rights reserved.  This notice is intended as a precaution against
 *  inadvertent publication and does not imply publication or any waiver
 *  of confidentiality.  The year included in the foregoing notice is the
 *  year of creation of the work.
 *
 */

module ati_fp_cmp_max_min (
   // Outputs
   cmp,
   max,
   min,

   // Inputs
   allow_input_denorms,
   bc_mode_7x,
   src0,
   src1
   );

   parameter              DOUBLE = 0;

   parameter              SRC_WIDTH = DOUBLE ? 64 : 32;
   parameter              EXP_WIDTH = DOUBLE ? 11 : 8;
   parameter              MAN_WIDTH = DOUBLE ? 52 : 23;

   input                  allow_input_denorms;
   input                  bc_mode_7x;
   input [SRC_WIDTH-1:0]  src0;
   input [SRC_WIDTH-1:0]  src1;

   output                 cmp;
   output [SRC_WIDTH-1:0] max;
   output [SRC_WIDTH-1:0] min;

   wire                   src0_sign = src0[SRC_WIDTH-1];
   wire                   src1_sign = src1[SRC_WIDTH-1];

   wire [EXP_WIDTH-1:0]   src0_exp = src0[SRC_WIDTH-2:MAN_WIDTH];
   wire [EXP_WIDTH-1:0]   src1_exp = src1[SRC_WIDTH-2:MAN_WIDTH];

   wire [MAN_WIDTH-1:0]   src0_man = src0[MAN_WIDTH-1:0];
   wire [MAN_WIDTH-1:0]   src1_man = src1[MAN_WIDTH-1:0];

   wire                   src0_is_nan = &src0_exp && |src0_man;
   wire                   src1_is_nan = &src1_exp && |src1_man;

   wire                   src0_is_snan = src0_is_nan && !src0_man[MAN_WIDTH-1];
   wire                   src1_is_snan = src1_is_nan && !src1_man[MAN_WIDTH-1];

   wire [SRC_WIDTH-1:0]   src0_flush_mask = (!allow_input_denorms && !(|src0_exp)) ? {1'b1, {SRC_WIDTH-1{1'b0}}} : {SRC_WIDTH{1'b1}};
   wire [SRC_WIDTH-1:0]   src1_flush_mask = (!allow_input_denorms && !(|src1_exp)) ? {1'b1, {SRC_WIDTH-1{1'b0}}} : {SRC_WIDTH{1'b1}};

   wire [SRC_WIDTH-1:0]   src0_flushed = src0 & src0_flush_mask;
   wire [SRC_WIDTH-1:0]   src1_flushed = src1 & src1_flush_mask;

   wire [SRC_WIDTH-1:0]   src0_quiet_mask = src0_is_snan ? {1'b0, {EXP_WIDTH{1'b0}}, 1'b1, {MAN_WIDTH-1{1'b0}}} : {SRC_WIDTH{1'b0}};
   wire [SRC_WIDTH-1:0]   src1_quiet_mask = src1_is_snan ? {1'b0, {EXP_WIDTH{1'b0}}, 1'b1, {MAN_WIDTH-1{1'b0}}} : {SRC_WIDTH{1'b0}};
   
   wire [SRC_WIDTH-1:0]   src0_quieted = (bc_mode_7x ? src0 : src0_flushed) | src0_quiet_mask;
   wire [SRC_WIDTH-1:0]   src1_quieted = (bc_mode_7x ? src1 : src1_flushed) | src1_quiet_mask;

   wire                   src0_is_zero = !(|src0_flushed[SRC_WIDTH-2:0]);
   wire                   src1_is_zero = !(|src1_flushed[SRC_WIDTH-2:0]);

   wire                   src0_eq_src1 = (src0_flushed == src1_flushed);
   wire                   src0_ge_src1 = (src0_flushed >= src1_flushed);

   wire                   max_sel = !src0_is_snan && (src1_is_snan || !src1_is_nan && (src0_is_nan || !(!src0_sign && src1_sign) && (src0_sign && !src1_sign || (!src0_sign && !src1_sign && (!src0_ge_src1 || src0_eq_src1) || src0_sign && src1_sign && src0_ge_src1 || src0_is_zero && src1_is_zero))));
   wire                   min_sel = !src0_is_snan && (src1_is_snan || !src1_is_nan && (src0_is_nan || !(src0_sign && !src1_sign) && (!src0_sign && !src1_sign && src0_ge_src1 || src0_sign && src1_sign && (!src0_ge_src1 || src0_eq_src1) || !src0_sign && src1_sign)));

   assign                 cmp = src0_eq_src1 && !src0_is_nan || src0_is_zero && src1_is_zero;   
   assign                 max = max_sel ? src1_quieted : src0_quieted;
   assign                 min = min_sel ? src1_quieted : src0_quieted;
   
endmodule // ati_fp_cmp_max_min
