`include "gc_register_addr_ext.v"
`include "gc__grbm_reg.v"

//
//GRBM_GFX_CNTL Encoder module
//

module ati_grbm_gfx_cntl_encoder (
              sclk, // Clock
              srst, // Active High Hard/Soft Reset
		      
              // register bus inputs
		      reg_send,
		      reg_op,
		      reg_be,
		      reg_addr,
		      reg_wd,
              // restore bus inputs
              restore_be,
              restore_we,
              restore_wd,
		      
              // outputs
              reg_gfx_cntl
		      );
   
//-------------------------------------------------------------    
// IO
//-------------------------------------------------------------
parameter DATA_WIDTH        = 32;
parameter ADDRESS_WIDTH     = 18; 

input                       sclk;
input                       srst;

input                       reg_send;
input                       reg_op;
input   [3:0]               reg_be; // Tie to 0xF if you don't know what this is
input   [ADDRESS_WIDTH+1:2] reg_addr;
input   [DATA_WIDTH-1:0]    reg_wd;

input   [3:0]               restore_be;
input                       restore_we;
input   [DATA_WIDTH-1:0]    restore_wd;

output  [10:0]              reg_gfx_cntl;

reg     [1:0]               meid;
reg     [1:0]               pipeid;
reg     [2:0]               queueid;
reg     [3:0]               vmid;

wire                        dec_grbm_gfx_cntl;
wire                        grbm_gfx_cntl_wen;

//-------------------------------------------------------------    
// Shadow GRBM_GFX_INDEX
//-------------------------------------------------------------

assign dec_grbm_gfx_cntl = ({reg_addr[ADDRESS_WIDTH+1:2], 2'b00}==`GRBM_GFX_CNTL_EXX);
assign grbm_gfx_cntl_wen = dec_grbm_gfx_cntl & reg_op & reg_send;

always @(posedge sclk)
begin
  if (srst)
  begin
    vmid        <= `GRBM_GFX_CNTL_VMID_DEFAULT;
    pipeid      <= `GRBM_GFX_CNTL_PIPEID_DEFAULT;
    meid        <= `GRBM_GFX_CNTL_MEID_DEFAULT;
    queueid     <= `GRBM_GFX_CNTL_QUEUEID_DEFAULT;
  end
  else
  if (restore_we)
  begin
    if (restore_be[1])
      queueid   <= restore_wd[10:8];

    if (restore_be[0])
    begin
      vmid      <= restore_wd[7:4];
      meid      <= restore_wd[3:2];
      pipeid    <= restore_wd[1:0];
    end 
  end
  else if (grbm_gfx_cntl_wen)
  begin
    if (reg_be[1])
      queueid   <= reg_wd[10:8];

    if (reg_be[0])
    begin
      vmid      <= reg_wd[7:4];
      meid      <= reg_wd[3:2];
      pipeid    <= reg_wd[1:0];
    end 
  end
end

//-------------------------------------------------------------    
// Encode gfx cntl
//-------------------------------------------------------------

assign reg_gfx_cntl = {queueid, vmid, meid, pipeid};

endmodule 
