`include "gc_register_addr_ext.v"
`include "gc__grbm_reg.v"

//
//GRBM_GFX_INDEX Encoder module
//

module ati_grbm_index_encoder (
              sclk, // Clock
              srst, // Active High Hard/Soft Reset
		      
              // register bus inputs
		      reg_send,
		      reg_op,
		      reg_be,
		      reg_addr,
		      reg_wd,
		      
              // restore bus inputs
              restore_be,
              restore_we,
              restore_wd,

              // outputs
              reg_index
		      );
   
//-------------------------------------------------------------    
// IO
//-------------------------------------------------------------
parameter DATA_WIDTH        = 32;
parameter ADDRESS_WIDTH     = 18; 

input                       sclk;
input                       srst;

input                       reg_send;
input                       reg_op;
input   [3:0]               reg_be; // Tie to 0xF if you don't know what this is
input   [ADDRESS_WIDTH+1:2] reg_addr;
input   [DATA_WIDTH-1:0]    reg_wd;

input   [3:0]               restore_be;
input                       restore_we;
input   [DATA_WIDTH-1:0]    restore_wd;

output  [13:0]              reg_index;

reg                         se_broadcast_writes;
reg                         instance_broadcast_writes;
reg                         sa_broadcast_writes;
reg     [1:0]               se_index;
reg     [1:0]               sa_index;
reg     [6:0]               instance_index;

wire                        dec_grbm_gfx_index;
wire                        grbm_gfx_index_wen;

//-------------------------------------------------------------    
// Shadow GRBM_GFX_INDEX
//-------------------------------------------------------------

assign dec_grbm_gfx_index = ({reg_addr[ADDRESS_WIDTH+1:2], 2'b00}==`GRBM_GFX_INDEX_EXX);
assign grbm_gfx_index_wen = dec_grbm_gfx_index & reg_op & reg_send;

always @(posedge sclk)
begin
  if (srst)
  begin
    se_broadcast_writes       <= 1'b1;
    instance_broadcast_writes <= 1'b1;
    sa_broadcast_writes       <= 1'b1;
    se_index                  <= 2'b0;
    sa_index                  <= 2'b0;
    instance_index            <= 7'b0;
  end
  else if (restore_we)
  begin
    if (restore_be[3])
    begin
      se_broadcast_writes       <= restore_wd[31];
      instance_broadcast_writes <= restore_wd[30];
      sa_broadcast_writes       <= restore_wd[29];
    end
    
    if (restore_be[2])
      se_index <= restore_wd[17:16];

    if (restore_be[1])
      sa_index <= restore_wd[9:8];

    if (restore_be[0])
      instance_index <= restore_wd[6:0];
  end

  else if (grbm_gfx_index_wen)
  begin
    if (reg_be[3])
    begin
      se_broadcast_writes       <= reg_wd[31];
      instance_broadcast_writes <= reg_wd[30];
      sa_broadcast_writes       <= reg_wd[29];
    end
    
    if (reg_be[2])
      se_index <= reg_wd[17:16];

    if (reg_be[1])
      sa_index <= reg_wd[9:8];

    if (reg_be[0])
      instance_index <= reg_wd[6:0];
  end
end

//-------------------------------------------------------------    
// Encode index
//-------------------------------------------------------------

assign reg_index = {se_broadcast_writes,
                    instance_broadcast_writes,
                    sa_broadcast_writes,
                    se_index,
                    sa_index,
                    instance_index};

endmodule 
