`include "gc_register_addr.v"
`include "gc__grbm_reg.v"
`include "grbm_defs.vh"

//
//Client module for the GRBM Priv Interface.
//
module ati_grbm_mcd_intf (
              sclk,       // Permanent Clock
              sclk_reg,   // Gated Register Clock
              srstb,      // Active Low Hard/Soft Reset (Only used for Serializer)
                          // Hard Resetb AND Soft Resetb
              hard_srstb, // Hard Resetb
		      
              // inputs to your block
              GRBM_block_reg_clken, // Connect these directly to the ports on your block, do not register. 
		      GRBM_block_reg_send,
		      GRBM_block_reg_addr,
		      GRBM_block_reg_wd,
		      GRBM_block_reg_op,
		      GRBM_block_reg_priv,
		      GRBM_block_reg_ssrcid,
		      GRBM_block_reg_vmid,
		      GRBM_block_reg_tmz,
              GRBM_block_reg_meid,
		      GRBM_block_reg_pipeid,
		      GRBM_block_reg_queueid,
		      GRBM_block_reg_index,
		      
              // outputs for use by your block
              reg_clken,
		      reg_send,
		      reg_addr,
		      reg_wd,
		      reg_op,
		      reg_priv,
              reg_ssrcid,
		      reg_vmid,
		      reg_tmz,
              reg_meid,
		      reg_pipeid,
		      reg_queueid,
              reg_instance_select,
		      reg_se_index,
		      reg_sa_index,
		      reg_instance_index,
		      reg_sa_broadcast_writes,
		      reg_se_broadcast_writes,
		      reg_instance_broadcast_writes,

              // repeaters for use by other blocks, do not register
              block_TARG_reg_clken,
		      block_TARG_reg_send,
		      block_TARG_reg_addr,
		      block_TARG_reg_wd,
		      block_TARG_reg_op,
		      block_TARG_reg_priv,
		      block_TARG_reg_vmid,
		      block_TARG_reg_tmz,
		      block_TARG_reg_meid,
		      block_TARG_reg_pipeid,
		      block_TARG_reg_index,
 
              // read return signals
              block_read_valid,  // valid
              block_read_data,   // data
              
              block_GRBM_serread_data, // serialized output, connect it to the port on your block, do not register. 
		      
              // Instance/Shader Engine control signals

              sa_strap,
              se_strap,
              instance_strap

		      );
   
//-------------------------------------------------------------    
// IO
//-------------------------------------------------------------
parameter DISABLE_SE_CHECK = 0;
parameter DISABLE_SA_CHECK = 0;
parameter DISABLE_INSTANCE_CHECK = 0;
   
input                                     sclk;
input                                     sclk_reg;
input                                     srstb;
input                                     hard_srstb;

input                                     GRBM_block_reg_clken;
input [`GRBM_TARGMCD_REG_SEND_RANGE]      GRBM_block_reg_send;
input                                     GRBM_block_reg_op;
input                                     GRBM_block_reg_priv;
input [`GRBM_TARGMCD_REG_SSRCID_RANGE]    GRBM_block_reg_ssrcid;
input [`GRBM_TARGMCD_REG_VMID_RANGE]      GRBM_block_reg_vmid;
input                                     GRBM_block_reg_tmz;
input [`GRBM_TARGMCD_REG_MEID_RANGE]      GRBM_block_reg_meid;
input [`GRBM_TARGMCD_REG_PIPEID_RANGE]    GRBM_block_reg_pipeid;
input [`GRBM_TARGMCD_REG_QUEUEID_RANGE]   GRBM_block_reg_queueid;
input [`GRBM_TARGMCD_REG_INDEX_RANGE]     GRBM_block_reg_index;
input [`GRBM_TARGMCD_REG_ADDR_RANGE]      GRBM_block_reg_addr;
input [`GRBM_TARGMCD_REG_WD_RANGE]        GRBM_block_reg_wd;

input                                     block_read_valid;
input [`TARG_GRBM_READ_DATA_RANGE]        block_read_data;

output 		                              reg_clken;
output [`GRBM_TARGMCD_REG_SEND_RANGE]     reg_send;
output                                    reg_op;
output                                    reg_priv;
output [`GRBM_TARGMCD_REG_SSRCID_RANGE]   reg_ssrcid;
output [`GRBM_TARGMCD_REG_VMID_RANGE]     reg_vmid;
output                                    reg_tmz;
output [`GRBM_TARGMCD_REG_MEID_RANGE]     reg_meid;
output [`GRBM_TARGMCD_REG_PIPEID_RANGE]   reg_pipeid;
output [`GRBM_TARGMCD_REG_QUEUEID_RANGE]  reg_queueid;
wire   [`GRBM_TARGMCD_REG_INDEX_RANGE]    reg_index;
output [`GRBM_TARGMCD_REG_ADDR_RANGE]     reg_addr;
output [`GRBM_TARGMCD_REG_WD_RANGE]       reg_wd;
output                                    reg_instance_select;
output [1:0]                              reg_sa_index;
output [1:0]                              reg_se_index;
output [6:0]                              reg_instance_index;
output 		                              reg_sa_broadcast_writes;
output		                              reg_se_broadcast_writes;
output		                              reg_instance_broadcast_writes;

output 		                              block_TARG_reg_clken;
output 		                              block_TARG_reg_send;
output 		                              block_TARG_reg_op;
output 		                              block_TARG_reg_priv;
output [`GRBM_TARGMCD_REG_VMID_RANGE]     block_TARG_reg_vmid;
output 		                              block_TARG_reg_tmz;
output [`GRBM_TARGMCD_REG_MEID_RANGE]     block_TARG_reg_meid;
output [`GRBM_TARGMCD_REG_PIPEID_RANGE]   block_TARG_reg_pipeid;
output [`GRBM_TARGMCD_REG_INDEX_RANGE]    block_TARG_reg_index;
output [`GRBM_TARGMCD_REG_ADDR_RANGE]     block_TARG_reg_addr;
output [`GRBM_TARGMCD_REG_WD_RANGE]       block_TARG_reg_wd;

output [`TARG_GRBM_SERREAD_DATA_RANGE]    block_GRBM_serread_data;

// Signals for Select Logic

input  [1:0]                              sa_strap;
input  [1:0]                              se_strap;
input  [6:0]                              instance_strap;

wire                                      se_broadcast_writes;
wire                                      instance_broadcast_writes;
wire                                      sa_broadcast_writes;
wire   [1:0]                              se_index;
wire   [1:0]                              sa_index;
wire   [6:0]                              instance_index;

//-------------------------------------------------------------    
// Code 
//-------------------------------------------------------------

ati_dff_in
  #(`GRBM_TARGMCD_REG_CLKEN_WIDTH)
  uati_dff_in
    (.clk     (sclk),
     .d       (GRBM_block_reg_clken),
     .q       (reg_clken));

ati_dff_out
  #(`GRBM_TARGMCD_REG_CLKEN_WIDTH)
  uati_dff_out
    (.clk     (sclk),
     .d       (GRBM_block_reg_clken),
     .q       (block_TARG_reg_clken));

ati_dff_in
  #(`GRBM_TARGMCD_REG_SEND_WIDTH)
  uati_dff_send_in
    (.clk     (sclk_reg),
     .d       (GRBM_block_reg_send),
     .q       (reg_send));

ati_dff_out
  #(`GRBM_TARG_REG_SEND_WIDTH)
  uati_dff_send_out
    (.clk     (sclk_reg),
     .d       (|GRBM_block_reg_send),
     .q       (block_TARG_reg_send));

ati_dff_en_in
  #(`GRBM_TARGMCD_REG_INDEX_WIDTH +
    `GRBM_TARGMCD_REG_QUEUEID_WIDTH +
    `GRBM_TARGMCD_REG_PIPEID_WIDTH +
    `GRBM_TARGMCD_REG_MEID_WIDTH +
    `GRBM_TARGMCD_REG_VMID_WIDTH +
    `GRBM_TARGMCD_REG_TMZ_WIDTH +
    `GRBM_TARGMCD_REG_SSRCID_WIDTH +
    `GRBM_TARGMCD_REG_PRIV_WIDTH +
    `GRBM_TARGMCD_REG_OP_WIDTH +
    `GRBM_TARGMCD_REG_ADDR_WIDTH +
    `GRBM_TARGMCD_REG_WD_WIDTH
    )
  uati_dff_in_gated
    (.clk     (sclk_reg),
     .en      (|GRBM_block_reg_send),
     .d       ({GRBM_block_reg_index,
                GRBM_block_reg_queueid,
                GRBM_block_reg_pipeid,
                GRBM_block_reg_meid,
                GRBM_block_reg_vmid,
                GRBM_block_reg_tmz,
                GRBM_block_reg_ssrcid,
                GRBM_block_reg_priv,
                GRBM_block_reg_op,
                GRBM_block_reg_addr,
                GRBM_block_reg_wd
                }),
     .q       ({reg_index,
                reg_queueid,
                reg_pipeid,
                reg_meid,
                reg_vmid,
                reg_tmz,
                reg_ssrcid,
                reg_priv,
                reg_op,
                reg_addr,
                reg_wd}));

ati_dff_en_out
  #(`GRBM_TARGMCD_REG_INDEX_WIDTH +
    `GRBM_TARGMCD_REG_PIPEID_WIDTH +
    `GRBM_TARGMCD_REG_MEID_WIDTH +
    `GRBM_TARGMCD_REG_VMID_WIDTH +
    `GRBM_TARGMCD_REG_TMZ_WIDTH +
    `GRBM_TARGMCD_REG_PRIV_WIDTH +
    `GRBM_TARGMCD_REG_OP_WIDTH +
    `GRBM_TARGMCD_REG_ADDR_WIDTH +
    `GRBM_TARGMCD_REG_WD_WIDTH
    )
  uati_dff_out_gated
    (.clk     (sclk_reg),
     .en      (|GRBM_block_reg_send),
     .d       ({GRBM_block_reg_index,
                 GRBM_block_reg_pipeid,
                 GRBM_block_reg_meid,
                 GRBM_block_reg_vmid,
                 GRBM_block_reg_tmz,
                 GRBM_block_reg_priv,
                 GRBM_block_reg_op,
                 GRBM_block_reg_addr,
                 GRBM_block_reg_wd}),
     .q       ({block_TARG_reg_index,
                block_TARG_reg_pipeid,
                block_TARG_reg_meid,
                block_TARG_reg_vmid,
                block_TARG_reg_tmz,
                block_TARG_reg_priv,
                block_TARG_reg_op,
                block_TARG_reg_addr,
                block_TARG_reg_wd}));

//-------------------------------------------------------------    
// Read Return Serializer
//-------------------------------------------------------------

gfx_ati_rd_rtn_32toN #(4,3,32) uati_rd_rtn_32toN
  (.iSRST  (~srstb),
   .iSCLK  (sclk_reg),
   .iDATA  (block_read_data),
   .iVALID (block_read_valid),
   .oSERREAD_DATA (block_GRBM_serread_data)); 

//-------------------------------------------------------------    
// Shader Engine/Shader Core/Instance Select
//-------------------------------------------------------------

assign se_broadcast_writes       = reg_index[13];
assign instance_broadcast_writes = reg_index[12];
assign sa_broadcast_writes       = reg_index[11];
assign se_index                  = reg_index[10:9];
assign sa_index                  = reg_index[8:7];
assign instance_index            = reg_index[6:0];

                                         // Does the instance match?
assign reg_instance_select = |reg_send & (((instance_strap == instance_index) | (instance_broadcast_writes & reg_op) | (DISABLE_INSTANCE_CHECK != 0)) & 
                                         // Does the shader core match?
                                         ((sa_strap == sa_index) | (sa_broadcast_writes & reg_op) | (DISABLE_SA_CHECK != 0)) &
                                         // Does the shader engine match?
                                         ((se_strap == se_index) | (se_broadcast_writes & reg_op) | (DISABLE_SE_CHECK != 0)));

// export these because SQG needs them
assign reg_se_index                  = se_index;
assign reg_sa_index                  = sa_index;
assign reg_instance_index            = instance_index;
assign reg_se_broadcast_writes       = se_broadcast_writes;
assign reg_instance_broadcast_writes = instance_broadcast_writes;
assign reg_sa_broadcast_writes       = sa_broadcast_writes;

//-------------------------------------------------------------------------------
// Daisy chain connectivity check
//-------------------------------------------------------------------------------

// psl property daisy_chain_check = always ((hard_srstb==1'b1)->next[1](reg_send^reg_send==0)) @(posedge sclk_reg);
// psl assert daisy_chain_check report "GRBM Daisy chain broken, floating send detected!";

endmodule 
