`include "gc__grbm_reg.v"
`include "grbm_defs.vh"

// This module should only be used by the CP.

module ati_grbm_priv_intf (
              sclk,       // Permanent Clock
              sclk_reg,   // Gated Register Clock
              hard_srstb, // Hard Resetb
		      
              // inputs to your block
              GRBM_block_reg_clken, // Connect these directly to the ports on your block, do not register. 
		      GRBM_block_reg_send,
		      GRBM_block_reg_addr,
		      GRBM_block_reg_wd,
		      GRBM_block_reg_op,
		      GRBM_block_reg_priv,
		      GRBM_block_reg_ssrcid,
		      GRBM_block_reg_vmid,
		      GRBM_block_reg_tmz,
		      GRBM_block_reg_meid,
		      GRBM_block_reg_pipeid,
		      GRBM_block_reg_queueid,
		      GRBM_block_reg_index,
		      
              // outputs for use by your block
              reg_clken,
		      reg_send,
		      reg_addr,
		      reg_wd,
		      reg_op,
		      reg_priv,
		      reg_ssrcid,
		      reg_vmid,
		      reg_tmz,
		      reg_meid,
		      reg_pipeid,
		      reg_queueid,
              reg_index,
		      reg_instance_select_0,
		      reg_instance_select_1,
		      reg_instance_select_2,
		      reg_instance_select_3,

              // repeaters for use by other blocks, do not register
		      block_TARG_reg_clken,
		      block_TARG_reg_send,
		      block_TARG_reg_addr,
		      block_TARG_reg_wd,
		      block_TARG_reg_op,
              block_TARG_reg_priv,
              block_TARG_reg_ssrcid,
              block_TARG_reg_vmid,
              block_TARG_reg_tmz,
		      block_TARG_reg_meid,
		      block_TARG_reg_pipeid,
		      block_TARG_reg_queueid,
		      block_TARG_reg_index

		      );
   
//-------------------------------------------------------------    
// IO
//-------------------------------------------------------------
input                                  sclk;
input                                  sclk_reg;
input                                  hard_srstb;

input                                  GRBM_block_reg_clken;
input                                  GRBM_block_reg_send;
input                                  GRBM_block_reg_op;
input                                  GRBM_block_reg_priv;
input [`GRBM_CPG_REG_SSRCID_RANGE]     GRBM_block_reg_ssrcid;
input [`GRBM_CPG_REG_VMID_RANGE]       GRBM_block_reg_vmid;
input                                  GRBM_block_reg_tmz;
input [`GRBM_CPG_REG_MEID_RANGE]       GRBM_block_reg_meid;
input [`GRBM_CPG_REG_PIPEID_RANGE]     GRBM_block_reg_pipeid;
input [`GRBM_CPF_REG_QUEUEID_RANGE]    GRBM_block_reg_queueid; // Only CPF uses this
input [`GRBM_CPG_REG_INDEX_RANGE]      GRBM_block_reg_index;
input [`GRBM_CPG_REG_ADDR_RANGE]       GRBM_block_reg_addr;
input [`GRBM_CPG_REG_WD_RANGE]         GRBM_block_reg_wd;

output 		                           reg_clken;
output 		                           reg_send;
output 		                           reg_op;
output 		                           reg_priv;
output [`GRBM_CPG_REG_SSRCID_RANGE]    reg_ssrcid;
output [`GRBM_CPG_REG_VMID_RANGE]      reg_vmid;
output 		                           reg_tmz;
output [`GRBM_CPG_REG_MEID_RANGE]      reg_meid;
output [`GRBM_CPG_REG_PIPEID_RANGE]    reg_pipeid;
output [`GRBM_CPF_REG_QUEUEID_RANGE]   reg_queueid;
output [`GRBM_CPG_REG_INDEX_RANGE]     reg_index;
wire   [`GRBM_CPG_REG_INDEX_RANGE]     reg_index;
output [`GRBM_CPG_REG_ADDR_RANGE]      reg_addr;
output [`GRBM_CPG_REG_WD_RANGE]        reg_wd;
output [7:0]	                       reg_instance_select_0;
output [7:0]	                       reg_instance_select_1;
output [7:0]	                       reg_instance_select_2;
output [7:0]	                       reg_instance_select_3;

output 		                           block_TARG_reg_clken;
output 		                           block_TARG_reg_send;
output 		                           block_TARG_reg_op;
output 		                           block_TARG_reg_priv;
output [`GRBM_CPG_REG_SSRCID_RANGE]    block_TARG_reg_ssrcid;
output [`GRBM_CPG_REG_VMID_RANGE]      block_TARG_reg_vmid;
output 		                           block_TARG_reg_tmz;
output [`GRBM_CPG_REG_MEID_RANGE]      block_TARG_reg_meid;
output [`GRBM_CPG_REG_PIPEID_RANGE]    block_TARG_reg_pipeid;
output [`GRBM_CPF_REG_QUEUEID_RANGE]   block_TARG_reg_queueid;
output [`GRBM_CPG_REG_INDEX_RANGE]     block_TARG_reg_index;
output [`GRBM_CPG_REG_ADDR_RANGE]      block_TARG_reg_addr;
output [`GRBM_CPG_REG_WD_RANGE]        block_TARG_reg_wd;

// Signals for Select Logic

wire                                   se_broadcast_writes;
wire                                   instance_broadcast_writes;
wire                                   sa_broadcast_writes;
wire   [1:0]                           se_index;
wire   [1:0]                           sa_index;
wire   [6:0]                           instance_index;

//-------------------------------------------------------------    
// Code 
//-------------------------------------------------------------

ati_dff_in
  #(`GRBM_CPG_REG_CLKEN_WIDTH)
  uati_dff_in
    (.clk     (sclk),
     .d       (GRBM_block_reg_clken),
     .q       (reg_clken));

ati_dff_out
  #(`GRBM_CPG_REG_CLKEN_WIDTH)
  uati_dff_out
    (.clk     (sclk),
     .d       (GRBM_block_reg_clken),
     .q       (block_TARG_reg_clken));

ati_dff_in
  #(`GRBM_CPG_REG_SEND_WIDTH)
  uati_dff_send_in
    (.clk     (sclk_reg),
     .d       (GRBM_block_reg_send),
     .q       (reg_send));

ati_dff_out
  #(`GRBM_CPG_REG_SEND_WIDTH)
  uati_dff_send_out
    (.clk     (sclk_reg),
     .d       (GRBM_block_reg_send),
     .q       (block_TARG_reg_send));

ati_dff_en_in
  #(`GRBM_CPG_REG_INDEX_WIDTH +
    `GRBM_CPF_REG_QUEUEID_WIDTH +
    `GRBM_CPG_REG_PIPEID_WIDTH +
    `GRBM_CPG_REG_MEID_WIDTH +
    `GRBM_CPG_REG_VMID_WIDTH +
    `GRBM_CPG_REG_TMZ_WIDTH +
    `GRBM_CPG_REG_SSRCID_WIDTH +
    `GRBM_CPG_REG_PRIV_WIDTH +
    `GRBM_CPG_REG_OP_WIDTH +
    `GRBM_CPG_REG_ADDR_WIDTH +
    `GRBM_CPG_REG_WD_WIDTH
    )
  uati_dff_in_gated
    (.clk     (sclk_reg),
     .en      (GRBM_block_reg_send),
     .d       ({GRBM_block_reg_index,
                GRBM_block_reg_queueid,
                GRBM_block_reg_pipeid,
                GRBM_block_reg_meid,
                GRBM_block_reg_vmid,
                GRBM_block_reg_tmz,
                GRBM_block_reg_ssrcid,
                GRBM_block_reg_priv,
                GRBM_block_reg_op,
                GRBM_block_reg_addr,
                GRBM_block_reg_wd}),
     .q       ({reg_index,
                reg_queueid,
                reg_pipeid,
                reg_meid,
                reg_vmid,
                reg_tmz,
                reg_ssrcid,
                reg_priv,
                reg_op,
                reg_addr,
                reg_wd}));

ati_dff_en_out
  #(`GRBM_CPG_REG_INDEX_WIDTH +
    `GRBM_CPF_REG_QUEUEID_WIDTH +
    `GRBM_CPG_REG_PIPEID_WIDTH +
    `GRBM_CPG_REG_MEID_WIDTH +
    `GRBM_CPG_REG_VMID_WIDTH +
    `GRBM_CPG_REG_TMZ_WIDTH +
    `GRBM_CPG_REG_SSRCID_WIDTH +
    `GRBM_CPG_REG_PRIV_WIDTH +
    `GRBM_CPG_REG_OP_WIDTH +
    `GRBM_CPG_REG_ADDR_WIDTH +
    `GRBM_CPG_REG_WD_WIDTH
    )
  uati_dff_out_gated
    (.clk     (sclk_reg),
     .en      (GRBM_block_reg_send),
     .d       ({GRBM_block_reg_index,
                GRBM_block_reg_queueid,
                GRBM_block_reg_pipeid,
                GRBM_block_reg_meid,
                GRBM_block_reg_vmid,
                GRBM_block_reg_tmz,
                GRBM_block_reg_ssrcid,
                GRBM_block_reg_priv,
                GRBM_block_reg_op,
                GRBM_block_reg_addr,
                GRBM_block_reg_wd}),
     .q       ({block_TARG_reg_index,
                block_TARG_reg_queueid,
                block_TARG_reg_pipeid,
                block_TARG_reg_meid,
                block_TARG_reg_vmid,
                block_TARG_reg_tmz,
                block_TARG_reg_ssrcid,
                block_TARG_reg_priv,
                block_TARG_reg_op,
                block_TARG_reg_addr,
                block_TARG_reg_wd}));

//-------------------------------------------------------------    
// Shader Engine/Shader Core/Instance Select
//-------------------------------------------------------------

assign se_broadcast_writes       = reg_index[13];
assign instance_broadcast_writes = reg_index[12];
assign sa_broadcast_writes       = reg_index[11];
assign se_index                  = reg_index[10:9];
assign sa_index                  = reg_index[8:7];
assign instance_index            = reg_index[6:0];

//------------
// Instance 0
//------------

//SE0SA0
                                             // Instance 0 match?
assign reg_instance_select_0[0] = reg_send & (((7'h0 == instance_index) | (instance_broadcast_writes & reg_op)) & 
                                             // Shader array 0 match?
                                             ((2'h0 == sa_index) | (sa_broadcast_writes & reg_op)) &
                                             // Shader engine 0 match?
                                             ((2'h0 == se_index) | (se_broadcast_writes & reg_op)));

//SE0SA1
                                             // Instance 0 match?
assign reg_instance_select_0[1] = reg_send & (((7'h0 == instance_index) | (instance_broadcast_writes & reg_op)) & 
                                             // Shader array 1 match?
                                             ((2'h1 == sa_index) | (sa_broadcast_writes & reg_op)) &
                                             // Shader engine 0 match?
                                             ((2'h0 == se_index) | (se_broadcast_writes & reg_op)));

//SE1SA0
                                             // Instance 0 match?
assign reg_instance_select_0[2] = reg_send & (((7'h0 == instance_index) | (instance_broadcast_writes & reg_op)) & 
                                             // Shader array 0 match?
                                             ((2'h0 == sa_index) | (sa_broadcast_writes & reg_op)) &
                                             // Shader engine 1 match?
                                             ((2'h1 == se_index) | (se_broadcast_writes & reg_op)));

//SE1SA1
                                             // Instance 0 match?
assign reg_instance_select_0[3] = reg_send & (((7'h0 == instance_index) | (instance_broadcast_writes & reg_op)) & 
                                             // Shader array 1 match?
                                             ((2'h1 == sa_index) | (sa_broadcast_writes & reg_op)) &
                                             // Shader engine 1 match?
                                             ((2'h1 == se_index) | (se_broadcast_writes & reg_op)));

//SE2SA0
                                             // Instance 0 match?
assign reg_instance_select_0[4] = reg_send & (((7'h0 == instance_index) | (instance_broadcast_writes & reg_op)) & 
                                             // Shader array 0 match?
                                             ((2'h0 == sa_index) | (sa_broadcast_writes & reg_op)) &
                                             // Shader engine 2 match?
                                             ((2'h2 == se_index) | (se_broadcast_writes & reg_op)));

//SE2SA1
                                             // Instance 0 match?
assign reg_instance_select_0[5] = reg_send & (((7'h0 == instance_index) | (instance_broadcast_writes & reg_op)) & 
                                             // Shader array 1 match?
                                             ((2'h1 == sa_index) | (sa_broadcast_writes & reg_op)) &
                                             // Shader engine 2 match?
                                             ((2'h2 == se_index) | (se_broadcast_writes & reg_op)));

//SE3SA0
                                             // Instance 0 match?
assign reg_instance_select_0[6] = reg_send & (((7'h0 == instance_index) | (instance_broadcast_writes & reg_op)) & 
                                             // Shader array 0 match?
                                             ((2'h0 == sa_index) | (sa_broadcast_writes & reg_op)) &
                                             // Shader engine 3 match?
                                             ((2'h3 == se_index) | (se_broadcast_writes & reg_op)));

//SE3SA1
                                             // Instance 0 match?
assign reg_instance_select_0[7] = reg_send & (((7'h0 == instance_index) | (instance_broadcast_writes & reg_op)) & 
                                             // Shader array 1 match?
                                             ((2'h1 == sa_index) | (sa_broadcast_writes & reg_op)) &
                                             // Shader engine 3 match?
                                             ((2'h3 == se_index) | (se_broadcast_writes & reg_op)));

//------------
// Instance 1
//------------

//SE0SA0
                                             // Instance 1 match?
assign reg_instance_select_1[0] = reg_send & (((7'h1 == instance_index) | (instance_broadcast_writes & reg_op)) & 
                                             // Shader array 0 match?
                                             ((2'h0 == sa_index) | (sa_broadcast_writes & reg_op)) &
                                             // Shader engine 0 match?
                                             ((2'h0 == se_index) | (se_broadcast_writes & reg_op)));

//SE0SA1
                                             // Instance 1 match?
assign reg_instance_select_1[1] = reg_send & (((7'h1 == instance_index) | (instance_broadcast_writes & reg_op)) & 
                                             // Shader array 1 match?
                                             ((2'h1 == sa_index) | (sa_broadcast_writes & reg_op)) &
                                             // Shader engine 0 match?
                                             ((2'h0 == se_index) | (se_broadcast_writes & reg_op)));

//SE1SA0
                                             // Instance 1 match?
assign reg_instance_select_1[2] = reg_send & (((7'h1 == instance_index) | (instance_broadcast_writes & reg_op)) & 
                                             // Shader array 0 match?
                                             ((2'h0 == sa_index) | (sa_broadcast_writes & reg_op)) &
                                             // Shader engine 1 match?
                                             ((2'h1 == se_index) | (se_broadcast_writes & reg_op)));

//SE1SA1
                                             // Instance 1 match?
assign reg_instance_select_1[3] = reg_send & (((7'h1 == instance_index) | (instance_broadcast_writes & reg_op)) & 
                                             // Shader array 1 match?
                                             ((2'h1 == sa_index) | (sa_broadcast_writes & reg_op)) &
                                             // Shader engine 1 match?
                                             ((2'h1 == se_index) | (se_broadcast_writes & reg_op)));

//SE2SA0
                                             // Instance 1 match?
assign reg_instance_select_1[4] = reg_send & (((7'h1 == instance_index) | (instance_broadcast_writes & reg_op)) & 
                                             // Shader array 0 match?
                                             ((2'h0 == sa_index) | (sa_broadcast_writes & reg_op)) &
                                             // Shader engine 2 match?
                                             ((2'h2 == se_index) | (se_broadcast_writes & reg_op)));

//SE2SA1
                                             // Instance 1 match?
assign reg_instance_select_1[5] = reg_send & (((7'h1 == instance_index) | (instance_broadcast_writes & reg_op)) & 
                                             // Shader array 1 match?
                                             ((2'h1 == sa_index) | (sa_broadcast_writes & reg_op)) &
                                             // Shader engine 2 match?
                                             ((2'h2 == se_index) | (se_broadcast_writes & reg_op)));

//SE3SA0
                                             // Instance 1 match?
assign reg_instance_select_1[6] = reg_send & (((7'h1 == instance_index) | (instance_broadcast_writes & reg_op)) & 
                                             // Shader array 0 match?
                                             ((2'h0 == sa_index) | (sa_broadcast_writes & reg_op)) &
                                             // Shader engine 3 match?
                                             ((2'h3 == se_index) | (se_broadcast_writes & reg_op)));

//SE3SA1
                                             // Instance 1 match?
assign reg_instance_select_1[7] = reg_send & (((7'h1 == instance_index) | (instance_broadcast_writes & reg_op)) & 
                                             // Shader array 1 match?
                                             ((2'h1 == sa_index) | (sa_broadcast_writes & reg_op)) &
                                             // Shader engine 3 match?
                                             ((2'h3 == se_index) | (se_broadcast_writes & reg_op)));

//------------
// Instance 2
//------------

//SE0SA0
                                             // Instance 2 match?
assign reg_instance_select_2[0] = reg_send & (((7'h2 == instance_index) | (instance_broadcast_writes & reg_op)) & 
                                             // Shader array 0 match?
                                             ((2'h0 == sa_index) | (sa_broadcast_writes & reg_op)) &
                                             // Shader engine 0 match?
                                             ((2'h0 == se_index) | (se_broadcast_writes & reg_op)));

//SE0SA1
                                             // Instance 2 match?
assign reg_instance_select_2[1] = reg_send & (((7'h2 == instance_index) | (instance_broadcast_writes & reg_op)) & 
                                             // Shader array 1 match?
                                             ((2'h1 == sa_index) | (sa_broadcast_writes & reg_op)) &
                                             // Shader engine 0 match?
                                             ((2'h0 == se_index) | (se_broadcast_writes & reg_op)));

//SE1SA0
                                             // Instance 2 match?
assign reg_instance_select_2[2] = reg_send & (((7'h2 == instance_index) | (instance_broadcast_writes & reg_op)) & 
                                             // Shader array 0 match?
                                             ((2'h0 == sa_index) | (sa_broadcast_writes & reg_op)) &
                                             // Shader engine 1 match?
                                             ((2'h1 == se_index) | (se_broadcast_writes & reg_op)));

//SE1SA1
                                             // Instance 2 match?
assign reg_instance_select_2[3] = reg_send & (((7'h2 == instance_index) | (instance_broadcast_writes & reg_op)) & 
                                             // Shader array 1 match?
                                             ((2'h1 == sa_index) | (sa_broadcast_writes & reg_op)) &
                                             // Shader engine 1 match?
                                             ((2'h1 == se_index) | (se_broadcast_writes & reg_op)));

//SE2SA0
                                             // Instance 2 match?
assign reg_instance_select_2[4] = reg_send & (((7'h2 == instance_index) | (instance_broadcast_writes & reg_op)) & 
                                             // Shader array 0 match?
                                             ((2'h0 == sa_index) | (sa_broadcast_writes & reg_op)) &
                                             // Shader engine 2 match?
                                             ((2'h2 == se_index) | (se_broadcast_writes & reg_op)));

//SE2SA1
                                             // Instance 2 match?
assign reg_instance_select_2[5] = reg_send & (((7'h2 == instance_index) | (instance_broadcast_writes & reg_op)) & 
                                             // Shader array 1 match?
                                             ((2'h1 == sa_index) | (sa_broadcast_writes & reg_op)) &
                                             // Shader engine 2 match?
                                             ((2'h2 == se_index) | (se_broadcast_writes & reg_op)));

//SE3SA0
                                             // Instance 2 match?
assign reg_instance_select_2[6] = reg_send & (((7'h2 == instance_index) | (instance_broadcast_writes & reg_op)) & 
                                             // Shader array 0 match?
                                             ((2'h0 == sa_index) | (sa_broadcast_writes & reg_op)) &
                                             // Shader engine 3 match?
                                             ((2'h3 == se_index) | (se_broadcast_writes & reg_op)));

//SE3SA1
                                             // Instance 2 match?
assign reg_instance_select_2[7] = reg_send & (((7'h2 == instance_index) | (instance_broadcast_writes & reg_op)) & 
                                             // Shader array 1 match?
                                             ((2'h1 == sa_index) | (sa_broadcast_writes & reg_op)) &
                                             // Shader engine 3 match?
                                             ((2'h3 == se_index) | (se_broadcast_writes & reg_op)));

//------------
// Instance 3
//------------

//SE0SA0
                                             // Instance 3 match?
assign reg_instance_select_3[0] = reg_send & (((7'h3 == instance_index) | (instance_broadcast_writes & reg_op)) & 
                                             // Shader array 0 match?
                                             ((2'h0 == sa_index) | (sa_broadcast_writes & reg_op)) &
                                             // Shader engine 0 match?
                                             ((2'h0 == se_index) | (se_broadcast_writes & reg_op)));

//SE0SA1
                                             // Instance 3 match?
assign reg_instance_select_3[1] = reg_send & (((7'h3 == instance_index) | (instance_broadcast_writes & reg_op)) & 
                                             // Shader array 1 match?
                                             ((2'h1 == sa_index) | (sa_broadcast_writes & reg_op)) &
                                             // Shader engine 0 match?
                                             ((2'h0 == se_index) | (se_broadcast_writes & reg_op)));

//SE1SA0
                                             // Instance 3 match?
assign reg_instance_select_3[2] = reg_send & (((7'h3 == instance_index) | (instance_broadcast_writes & reg_op)) & 
                                             // Shader array 0 match?
                                             ((2'h0 == sa_index) | (sa_broadcast_writes & reg_op)) &
                                             // Shader engine 1 match?
                                             ((2'h1 == se_index) | (se_broadcast_writes & reg_op)));

//SE1SA1
                                             // Instance 3 match?
assign reg_instance_select_3[3] = reg_send & (((7'h3 == instance_index) | (instance_broadcast_writes & reg_op)) & 
                                             // Shader array 1 match?
                                             ((2'h1 == sa_index) | (sa_broadcast_writes & reg_op)) &
                                             // Shader engine 1 match?
                                             ((2'h1 == se_index) | (se_broadcast_writes & reg_op)));

//SE2SA0
                                             // Instance 3 match?
assign reg_instance_select_3[4] = reg_send & (((7'h3 == instance_index) | (instance_broadcast_writes & reg_op)) & 
                                             // Shader array 0 match?
                                             ((2'h0 == sa_index) | (sa_broadcast_writes & reg_op)) &
                                             // Shader engine 2 match?
                                             ((2'h2 == se_index) | (se_broadcast_writes & reg_op)));

//SE2SA1
                                             // Instance 3 match?
assign reg_instance_select_3[5] = reg_send & (((7'h3 == instance_index) | (instance_broadcast_writes & reg_op)) & 
                                             // Shader array 1 match?
                                             ((2'h1 == sa_index) | (sa_broadcast_writes & reg_op)) &
                                             // Shader engine 2 match?
                                             ((2'h2 == se_index) | (se_broadcast_writes & reg_op)));

//SE3SA0
                                             // Instance 3 match?
assign reg_instance_select_3[6] = reg_send & (((7'h3 == instance_index) | (instance_broadcast_writes & reg_op)) & 
                                             // Shader array 0 match?
                                             ((2'h0 == sa_index) | (sa_broadcast_writes & reg_op)) &
                                             // Shader engine 3 match?
                                             ((2'h3 == se_index) | (se_broadcast_writes & reg_op)));

//SE3SA1
                                             // Instance 3 match?
assign reg_instance_select_3[7] = reg_send & (((7'h3 == instance_index) | (instance_broadcast_writes & reg_op)) & 
                                             // Shader array 1 match?
                                             ((2'h1 == sa_index) | (sa_broadcast_writes & reg_op)) &
                                             // Shader engine 3 match?
                                             ((2'h3 == se_index) | (se_broadcast_writes & reg_op)));

//-------------------------------------------------------------------------------
// Daisy chain connectivity check
//-------------------------------------------------------------------------------

// psl property daisy_chain_check = always ((hard_srstb==1'b1)->next[1](reg_send^reg_send==0)) @(posedge sclk_reg);
// psl assert daisy_chain_check report "GRBM Daisy chain broken, floating send detected!";

endmodule 
