//==============================================================================
//
// Filename: ati_ind_N_state_storage_1_out.v
// Creation Date: Friday, May 2, 2008
//
// Description: Dual Indirect Pointer N state storage block.  This block allows
//              indirect pointers to be allocated to State Register locations.
//
// Revision Control Information:
// $Author: $
// $Date: $
// $Id: $
// $Change: $
//
//==============================================================================
//
// Copyright: Trade secret of ATI Technologies, Inc.
//    Copyright 2003, ATI Technologies, Inc., (unpublished)
//
//    All rights reserved.  This notice is intended as a precaution against
//    inadvertent publication and does not imply publication or any waiver
//    of confidentiality.  The year included in the foregoing notice is the
//    year of creation of the work.
//
//==============================================================================
`include "gc_features.vh"
module ati_ind_N_state_storage_1_out_with_release
  (// Outputs
  read_ptr, write_ptr,
  // Inputs
  sclk, rst, rel, w_addr, r_addr, st_indx, we, cpy, sel, re
  );

input                                 sclk;
input                                 rst;

input                                 rel;

input                                 sel;
// Copy State command
input                                 cpy;
// GRBM write control
input                                 we;
input   [2:0]                         w_addr;
// GRBM read control
input                                 re;
input   [2:0]                         r_addr;
// State register strobe
input   [2:0]                         st_indx;

// Pointer for read address
output  [`GC__STWTPTR_WIDTH-1:0] read_ptr;
// Pointer for write address
output  [`GC__STWTPTR_WIDTH-1:0] write_ptr;


 wire [`GC__STWTPTR_WIDTH-1:0]  addr_ptr_0;
 wire [`GC__STWTPTR_WIDTH-1:0]  addr_ptr_1;
 wire [`GC__STWTPTR_WIDTH-1:0]  addr_ptr_2;
 wire [`GC__STWTPTR_WIDTH-1:0]  addr_ptr_3;
 wire [`GC__STWTPTR_WIDTH-1:0]  addr_ptr_4;
 wire [`GC__STWTPTR_WIDTH-1:0]  addr_ptr_5;
 wire [`GC__STWTPTR_WIDTH-1:0]  addr_ptr_6;
 wire [`GC__STWTPTR_WIDTH-1:0]  addr_ptr_7;

 wire                                cpy;
 wire [2:0]                          rbiu_waddr;
 wire [`GC__STWTPTR_WIDTH-1:0]  rd_write_ptr;

 wire                                wr_sel;
 wire                                wr_or_copy;
 wire [`GC__STWTPTR_WIDTH-1:0]  new_write_ptr;
 wire                                write_only;

// N_st_storage is written by a unique write or a copy
assign wr_or_copy = wr_sel | cpy;

ati_N_st_storage_N_out_with_rst #(`GC__STWTPTR_WIDTH) ustate_reg
 (
   // Outputs
   .st_data0 (addr_ptr_0),
   .st_data1 (addr_ptr_1),
   .st_data2 (addr_ptr_2),
   .st_data3 (addr_ptr_3),
   .st_data4 (addr_ptr_4),
   .st_data5 (addr_ptr_5),
   .st_data6 (addr_ptr_6),
   .st_data7 (addr_ptr_7),
   .r_data   (rd_write_ptr),
   // Inputs
   .we       (wr_or_copy),
   .w_addr   (w_addr),
   .w_data   (write_ptr),
   .re       (re),
   .r_addr   (r_addr),
   .sel      (sel),
   .cpy      (cpy),
   .rst      (rst),
   .clk      (sclk) );

// The read pointer is updated by either the re command or the state index
// The re command is a GRBM request and has priority
`ifdef GC__NUM_STATES__7_PRESENT
assign read_ptr = re ? rd_write_ptr :
                       (st_indx == 3'h0) ? addr_ptr_0 :
                       (st_indx == 3'h1) ? addr_ptr_1 :
                       (st_indx == 3'h2) ? addr_ptr_2 :
                       (st_indx == 3'h3) ? addr_ptr_3 :
                       (st_indx == 3'h4) ? addr_ptr_4 :
                       (st_indx == 3'h5) ? addr_ptr_5 :
                       (st_indx == 3'h6) ? addr_ptr_6 : addr_ptr_7;
`else
`ifdef GC__NUM_STATES__3_PRESENT
assign read_ptr = re ? rd_write_ptr :
                       (st_indx == 3'h0) ? addr_ptr_0 :
                       (st_indx == 3'h1) ? addr_ptr_1 :
                       (st_indx == 3'h2) ? addr_ptr_2 : addr_ptr_3;
`endif
`endif

// Only update the write pointers on a GRBM write operation
assign write_only = we & ~cpy;

ati_state_write_ptr_with_release ustate_write_ptr
   (
// Inputs
    .sclk             (sclk),
    .rst              (rst),
    .we               (write_only),
    .w_addr           (w_addr),
    .rel              (rel),
    .ptr0             (addr_ptr_0),
    .ptr1             (addr_ptr_1),
    .ptr2             (addr_ptr_2),
    .ptr3             (addr_ptr_3),
    .ptr4             (addr_ptr_4),
    .ptr5             (addr_ptr_5),
    .ptr6             (addr_ptr_6),
    .ptr7             (addr_ptr_7),
// Outputs
    .wr_sel           (wr_sel),
    .write_ptr        (new_write_ptr)  );

// The write pointer issues the new write pointer when a unique write
// occurs. Otherwise, it uses the values in the state register.
`ifdef GC__NUM_STATES__7_PRESENT
assign write_ptr = wr_sel ? new_write_ptr :
                   (w_addr == 3'h0) ? addr_ptr_0 :
                   (w_addr == 3'h1) ? addr_ptr_1 :
                   (w_addr == 3'h2) ? addr_ptr_2 :
                   (w_addr == 3'h3) ? addr_ptr_3 :
                   (w_addr == 3'h4) ? addr_ptr_4 :
                   (w_addr == 3'h5) ? addr_ptr_5 :
                   (w_addr == 3'h6) ? addr_ptr_6 : addr_ptr_7;
`else
`ifdef GC__NUM_STATES__3_PRESENT
assign write_ptr = wr_sel ? new_write_ptr :
                   (w_addr == 3'h0) ? addr_ptr_0 :
                   (w_addr == 3'h1) ? addr_ptr_1 :
                   (w_addr == 3'h2) ? addr_ptr_2 : addr_ptr_3;
`endif
`endif

endmodule // ati_ind_N_state_storage_1_out
