//--------------------------------------------------------------------------
//
// $Id: //tools/internal/packages/design/rtllib/branches/rtllib_soc15/src/rtl/gf28a/ati_master_clock_gater.v#1 $
//
// $Change: 1601556 $
//
//
// Notes: ATI master clock gater
//
//              clk_in  - chip global clock input,
//              clk    - block local permanent clock tree,
//              en    - clock enable input for dynamic clock gating,
//              pm_enb - enable dynamic clock gating (driven by
//                      power management block register) input,
//              clk_out   - local clock output.
//
//
// Copyright: Trade secret of ATI Technologies, Inc.
//    (C) Copyright 2001-2002, ATI Technologies, Inc., (unpublished)
//
//    All rights reserved.  This notice is intended as a precaution against
//    inadvertent publication and does not imply publication or any waiver
//    of confidentiality.  The year included in the foregoing notice is the
//    year of creation of the work.
//
//--------------------------------------------------------------------------
`include "rtllib_features.vh"
module ati_master_clock_gater
(
   clk_in,
   clk,
   en,
   TEST_EN,
   pm_enb,
   clk_out
);
   input  clk_in;
   input  clk;
   input  en;
   input  TEST_EN;
   input  pm_enb;
   output clk_out;
   reg clock_enable;
   wire clock_enable_te;
   always @(posedge clk)
      clock_enable <= (en | pm_enb) ;

`ifdef CDC_VERIFICATION
reg clk_en_latch;
always @ (clk_in or clock_enable or TEST_EN) begin
if (!clk_in)
clk_en_latch = (clock_enable | TEST_EN);
end
assign clk_out = clk_in & clk_en_latch;
`else
   HDBLVT08_CKGTPLT_V7Y2_4  d0nt_gater1 (.CLK(clk_in),.E(clock_enable),.TE(TEST_EN),.Z(clk_out));
`endif

`ifdef MONITOR_GATER
`ifdef SIMULATION
`include "msgio_defines.vh"
   initial
     `MSGIO_DISPLAY("*** MONITOR_GATER: %m: initial state: en=%b pm_enb=%b",
              en, pm_enb);
   always @(en)
     `MSGIO_DISPLAY("*** MONITOR_GATER: %m: en changed from %b to %b at %t",
              ~en, en, $time);
   always @(pm_enb)
     `MSGIO_DISPLAY("*** MONITOR_GATER: %m: pm_enb changed from %b to %b at %t",
              ~pm_enb, pm_enb, $time);
`endif
`endif
endmodule
