`include "gc_features.vh"

// Function implementation of state write ptr stwtptr
module ati_state_write_ptr
(/*AUTOARG*/
 // Outputs
 wr_sel, write_ptr,
 // Inputs
 sclk, rst, we, w_addr, ptr0, ptr1, ptr2, ptr3, ptr4, ptr5, ptr6, ptr7
 );

input                               sclk;
input                               rst;
input                               we;
input [2:0]                         w_addr;

input [`GC__STWTPTR_WIDTH-1:0] ptr0;
input [`GC__STWTPTR_WIDTH-1:0] ptr1;
input [`GC__STWTPTR_WIDTH-1:0] ptr2;
input [`GC__STWTPTR_WIDTH-1:0] ptr3;
input [`GC__STWTPTR_WIDTH-1:0] ptr4;
input [`GC__STWTPTR_WIDTH-1:0] ptr5;
input [`GC__STWTPTR_WIDTH-1:0] ptr6;
input [`GC__STWTPTR_WIDTH-1:0] ptr7;

output                              wr_sel;
output [`GC__STWTPTR_WIDTH-1:0] write_ptr;

reg [`GC__NUM_STATES-1:0] 	     loc_used;
reg [`GC__STWTPTR_WIDTH-1:0]    tmp_write_ptr;
wire                                 next_just_out_reset;
reg                                  just_out_reset;

integer                              i;


`ifdef GC__NUM_STATES__3_PRESENT
`ifdef GC__NUM_STATES__7_PRESENT
wire [`GC__STWTPTR_WIDTH-1:0]   constant_value[7:0];
`else
wire [`GC__STWTPTR_WIDTH-1:0]   constant_value[3:0];
`endif
`endif


assign                               constant_value[0] = `GC__STWTPTR_WIDTH'h0;
assign                               constant_value[1] = `GC__STWTPTR_WIDTH'h1;
`ifdef GC__NUM_STATES__3_PRESENT
assign                               constant_value[2] = `GC__STWTPTR_WIDTH'h2;
assign                               constant_value[3] = `GC__STWTPTR_WIDTH'h3;
`endif
`ifdef GC__NUM_STATES__7_PRESENT
assign                               constant_value[4] = `GC__STWTPTR_WIDTH'h4;
assign                               constant_value[5] = `GC__STWTPTR_WIDTH'h5;
assign                               constant_value[6] = `GC__STWTPTR_WIDTH'h6;
assign                               constant_value[7] = `GC__STWTPTR_WIDTH'h7;
`endif


always @(*) begin
for (i = 0; i < `GC__NUM_STATES; i=i+1) begin

`ifdef GC__NUM_STATES__3_PRESENT
`ifdef GC__NUM_STATES__7_PRESENT
loc_used[i] = (ptr0 == constant_value[i]) |
              (ptr1 == constant_value[i]) |
              (ptr2 == constant_value[i]) |
              (ptr3 == constant_value[i]) |
              (ptr4 == constant_value[i]) |
              (ptr5 == constant_value[i]) |
              (ptr6 == constant_value[i]) |
              (ptr7 == constant_value[i]);
`else
loc_used[i] = (ptr0 == constant_value[i]) |
              (ptr1 == constant_value[i]) |
              (ptr2 == constant_value[i]) |
              (ptr3 == constant_value[i]);
`endif
`endif
end
end


always @(*) begin
// create encoder to establish next write pointer
// select the lowest number that is unused
// out of reset select '0'
casez({just_out_reset,loc_used})
`ifdef GC__NUM_STATES__3_PRESENT
`ifdef GC__NUM_STATES__7_PRESENT
9'bzzzzzzzz0 : tmp_write_ptr = constant_value[0];
9'b0zzzzzz01 : tmp_write_ptr = constant_value[1];
9'b0zzzzz011 : tmp_write_ptr = constant_value[2];
9'b0zzzz0111 : tmp_write_ptr = constant_value[3];
9'b0zzz01111 : tmp_write_ptr = constant_value[4];
9'b0zz011111 : tmp_write_ptr = constant_value[5];
9'b0z0111111 : tmp_write_ptr = constant_value[6];
9'b001111111 : tmp_write_ptr = constant_value[7];
`else
5'bzzzz0 : tmp_write_ptr = constant_value[0];
5'b0zz01 : tmp_write_ptr = constant_value[1];
5'b0z011 : tmp_write_ptr = constant_value[2];
5'b00111 : tmp_write_ptr = constant_value[3];
`endif
`endif
default : tmp_write_ptr = constant_value[0];
endcase
end

assign write_ptr = tmp_write_ptr;

// when we start up after reset we want to start off writing to location 0 in memory
// since all the locations of the ati_storage module will be '0' after reset
// the first pointer will be '1', so we have to force the first location to be '0'
assign next_just_out_reset = just_out_reset & ~we;

always @(posedge sclk) begin
if(rst) begin
just_out_reset <= 1'b1;
end else begin
just_out_reset <= next_just_out_reset;
end
end

// Determine the uniqueness of each pointer location
// A pointer is unique if it does not match any other pointer
`ifdef GC__NUM_STATES__7_PRESENT
wire ptr0_unique_b = ( (ptr0 == ptr1) |
                       (ptr0 == ptr2) |
                       (ptr0 == ptr3) |
                       (ptr0 == ptr4) |
                       (ptr0 == ptr5) |
                       (ptr0 == ptr6) |
                       (ptr0 == ptr7) );

wire ptr1_unique_b = ( (ptr1 == ptr0) |
                       (ptr1 == ptr2) |
                       (ptr1 == ptr3) |
                       (ptr1 == ptr4) |
                       (ptr1 == ptr5) |
                       (ptr1 == ptr6) |
                       (ptr1 == ptr7) );

wire ptr2_unique_b = ( (ptr2 == ptr0) |
                       (ptr2 == ptr1) |
                       (ptr2 == ptr3) |
                       (ptr2 == ptr4) |
                       (ptr2 == ptr5) |
                       (ptr2 == ptr6) |
                       (ptr2 == ptr7) );

wire ptr3_unique_b = ( (ptr3 == ptr0) |
                       (ptr3 == ptr1) |
                       (ptr3 == ptr2) |
                       (ptr3 == ptr4) |
                       (ptr3 == ptr5) |
                       (ptr3 == ptr6) |
                       (ptr3 == ptr7) );

wire ptr4_unique_b = ( (ptr4 == ptr0) |
                       (ptr4 == ptr1) |
                       (ptr4 == ptr2) |
                       (ptr4 == ptr3) |
                       (ptr4 == ptr5) |
                       (ptr4 == ptr6) |
                       (ptr4 == ptr7) );

wire ptr5_unique_b = ( (ptr5 == ptr0) |
                       (ptr5 == ptr1) |
                       (ptr5 == ptr2) |
                       (ptr5 == ptr3) |
                       (ptr5 == ptr4) |
                       (ptr5 == ptr6) |
                       (ptr5 == ptr7) );

wire ptr6_unique_b = ( (ptr6 == ptr0) |
                       (ptr6 == ptr1) |
                       (ptr6 == ptr2) |
                       (ptr6 == ptr3) |
                       (ptr6 == ptr4) |
                       (ptr6 == ptr5) |
                       (ptr6 == ptr7) );

wire ptr7_unique_b = ( (ptr7 == ptr0) |
                       (ptr7 == ptr1) |
                       (ptr7 == ptr2) |
                       (ptr7 == ptr3) |
                       (ptr7 == ptr4) |
                       (ptr7 == ptr5) |
                       (ptr7 == ptr6) );
`else
`ifdef GC__NUM_STATES__3_PRESENT
wire ptr0_unique_b = ( (ptr0 == ptr1) |
                       (ptr0 == ptr2) |
                       (ptr0 == ptr3) );

wire ptr1_unique_b = ( (ptr1 == ptr0) |
                       (ptr1 == ptr2) |
                       (ptr1 == ptr3) );

wire ptr2_unique_b = ( (ptr2 == ptr0) |
                       (ptr2 == ptr1) |
                       (ptr2 == ptr3) );

wire ptr3_unique_b = ( (ptr3 == ptr0) |
                       (ptr3 == ptr1) |
                       (ptr3 == ptr2) );
`endif
`endif

// The write select signal indicates that a write has occurred
// to a location that doesn't have a unique pointer
`ifdef GC__NUM_STATES__7_PRESENT
assign wr_sel = we ? (w_addr == 3'h0) ? ptr0_unique_b :
       (w_addr == 3'h1) ? ptr1_unique_b :
       (w_addr == 3'h2) ? ptr2_unique_b :
       (w_addr == 3'h3) ? ptr3_unique_b :
       (w_addr == 3'h4) ? ptr4_unique_b :
       (w_addr == 3'h5) ? ptr5_unique_b :
       (w_addr == 3'h6) ? ptr6_unique_b : ptr7_unique_b : 1'b0;
`else
`ifdef GC__NUM_STATES__3_PRESENT
assign wr_sel = we ? 
       (w_addr == 3'h0) ? ptr0_unique_b :
       (w_addr == 3'h1) ? ptr1_unique_b :
       (w_addr == 3'h2) ? ptr2_unique_b : ptr3_unique_b : 1'b0;
`endif
`endif

endmodule // state_write_ptr
