////
//  
//  Trade secret of ATI Technologies, Inc.
//  Copyright 2001-5, ATI Technologies, Inc., (unpublished)
//
//  All rights reserved.  This notice is intended as a precaution against
//  inadvertent publication and does not imply publication or any waiver
//  of confidentiality.  The year included in the foregoing notice is the
//  year of creation of the work.
//
////
//
// - synchornize one pulse to another pulse regardless of clock ratio
// - input has to be a pulse for one cycle and the max rate of asserting
//   the input pulse is every (1 + (2 * in_freq(MHz)/out_freq(MHz))) input clocks
// - this synchronizer detects a state change (input pulse) and causes
//   a state change (output pulse)
// - the reset logic is not necessary but for simulation
module ati_sync_pulse (ICLK, IRESET, SYNC_IN, OCLK,
        `ifdef ABV   
        InputChkDis, 
        `endif  
SYNC_OUT);
    input ICLK;
    input IRESET;
    input SYNC_IN;
    `ifdef ABV
    input InputChkDis ;
    `endif
    input OCLK;
    output SYNC_OUT;
    reg in_ff_out, out_ff_out; 
    wire in_pulse, in_ff_in;
    assign in_pulse  = SYNC_IN ^ in_ff_out; 
    assign in_ff_in  = ~IRESET & in_pulse; 
    always @(posedge ICLK) begin
        in_ff_out <= in_ff_in; 
    end
    // ati_sdffcdc_4xdly_wrapper sync_r (
    //     .QSRC(), .SRC_CLK(ICLK), .D(in_ff_out), .Q(sync_ff_out),
    //     `ifdef ABV
    //     .InputChkDis(InputChkDis),
    //     `endif
    //     .QDST(), .DST_CLK(OCLK)
    // );
    
techind_sync_icd #(.depth           (4),
                   .sourceFlop      (1),
                   .TransportCycle  (0)
       )
u_techind_sync(
               .SB              (1'b1       ),
               .RB              (1'b1       ),
               .FLUSH           (1'b0       ),
               .CKsrc           (ICLK       ),
               .CKdst           (OCLK       ),
               .EN              (1'b1       ),
               .Dsrc            (in_ff_out  ),
               .Qsrc            (),
               .Qdst            (sync_ff_out),
               `ifdef ABV
               .InputChkDis     (InputChkDis),
               `endif
               .SC1             (1'b0       ),
               .SC2             (1'b0       ),
               .SE              (1'b0       ),
               .SI              (1'b0       ),
               .SO              ()
       ) ;

    always @(posedge OCLK) begin
        out_ff_out <= sync_ff_out; 
    end
    assign SYNC_OUT = out_ff_out ^ sync_ff_out; 
endmodule
