//--------------------------------------------------------------------------
//
// $Id: //depot/xenos/devel/parts_lib/src/common/block_busy_extender.v#4 $
//
// $Change: 159803 $
//
//
// Notes:       Counter to extend dis-assertion of block busy signal
// 
//              clk - block local gated clock (from clock gater controlled by
//                                                    busy_out of this module).
//              rst  - reset 
//              busy_in    - clock enable input for dynamic clock gating,
//              off_latency [5:0] - latency of "busy_in" signal turn off. 
//                                  The clk_out will run for "off_latency" clock
//                                  cycles after "busy_in" is disasserted.
//              busy_out   - busy signal to clock gater.
// 
// 
// Copyright:	Trade secret of ATI Technologies, Inc.
//		(C) Copyright 2001-2002, ATI Technologies, Inc., (unpublished) 
//
//		All rights reserved.  This notice is intended as a precaution against
//		inadvertent publication and does not imply publication or any waiver
//		of confidentiality.  The year included in the foregoing notice is the
//		year of creation of the work.
//
//--------------------------------------------------------------------------
module block_busy_extender 
( 
   clk,
   rst,
   busy_in, 
   off_latency, 
   busy_out
);

   input  clk;
   input  rst;
   input  busy_in;
   input [5:0]  off_latency;
   output busy_out;

   
   reg [5:0]  latency_cnt;
   wire cnt_en;

  always @(posedge clk )
   begin
     if(rst)
       latency_cnt <= 6'h00;
     else if(busy_in)
       latency_cnt <= off_latency;
     else if(cnt_en)
       latency_cnt <= latency_cnt - 1 ;
   end

   assign cnt_en = (~busy_in) & (|latency_cnt) ;

   assign busy_out = busy_in | cnt_en ;



endmodule 
