//gfxip/gfx9/main/src/rtl/shared/bpm/bpm.v#3 - edit change 1477521 (text)

// **********************************************************************************
//                              -*- Mode: Verilog -*-
// Filename        : bpm.v
// Description     : Block Power Manager - Controls clocking and power mgmt for gfxip blocks
// Created by      : John McCardle
// Created On      : 03/22/2011
// **********************************************************************************
`include "gc_features.vh"
`include "mem_features.vh"
`include "gfx_pgfsm_defines.h"
`include "bpm_defs.vh"

`define CMD_CGLS_Enable     5'd16
`define CMD_CGLS_Disable    5'd17
`define CMD_CGLS_On         5'd18
`define CMD_CGLS_Off        5'd19
`define CMD_CGCG_Override_0 5'd20
`define CMD_CGCG_Override_1 5'd21
`define CMD_MGCG_Override_0 5'd22
`define CMD_MGCG_Override_1 5'd23
`define CMD_FGCG_Override_0 5'd24
`define CMD_FGCG_Override_1 5'd25

// For new BPM map
`define BPM_BROADCAST         0:0
`define BPM_WRITE             1:1
`define BPM_LONG_CMD          2:2
`define BPM_ADDRESS           15:3
`define BPM_REG_ADDR          23:16
`define BPM_GROUPCAST_ADDR    6:3
`define BPM_CHAIN_ID          15:8


`define PSM_CURPSM_LENGTH     14


// This module combines following modules and thier features
//   1. PGFSM
//   2. 4x_rst_combine
//   3. Medium grain clock gating and fine grain clock gating overrides

module bpm
  (
`ifdef GC__GFX_ISLAND_POWER_GATING__1
   Cpl_REFCLK,
   TTOP_P1_mother_sleep,
   TTOP_P1_daughter_sleep,
   TTOP_P1_iso_clampn,
   TTOP_P1_scan_iso_clampn,

   TTOP_P2_mother_sleep,
   TTOP_P2_daughter_sleep,
   TTOP_P2_iso_clampn,
   TTOP_P2_scan_iso_clampn,

 `ifdef GC__GNB_DFT
   TST_TARG_miscpfh_tclk,
   PGFSM_TARG_tdr_tresetb,
   PGFSM_TARG_tdr_shift,
   PGFSM_TARG_tdr_capture,
   PGFSM_TARG_tdr_update,
   PGFSM_TARG_tdr_tdi,
   PGFSM_tdr_en_rw,
   PGFSM_tdr_en_ro,
   PGFSM_tdr_tdo_rw,
   PGFSM_tdr_tdo_ro,
   PGFSM2_tdr_en_rw,
   PGFSM2_tdr_en_ro,
   PGFSM2_tdr_tdo_rw,
   PGFSM2_tdr_tdo_ro,
 `else
   DFT_TARG_si,
   DFT_TARG_trstb,
   DFT_TARG_shift,
   DFT_TARG_capture,
   DFT_TARG_update,
   DFT_TARG_tclk,
   DFT_BPM_0_rw_dec,
   DFT_BPM_0_ro_dec,
   DFT_BPM_1_rw_dec,
   DFT_BPM_1_ro_dec,
   DFT_BPM_0_rw_so,
   DFT_BPM_0_ro_so,
   DFT_BPM_1_rw_so,
   DFT_BPM_1_ro_so,
 `endif
   DFT_P1_resetb,
   DFT_P2_resetb,
`endif
  // 1500 interface for AVFS
  WRCK,
  WSI,
  ShiftWR,
  UpdateWR,
  CaptureWR,
  WDREn,
  WRSTN,
  WRO,
 /*autoarg*/
  // Outputs
`ifdef GC__GFX_PIPELINE_CLOCK_GATING__1
  cg3d_Cpl_GFXCLK,
`endif
  cgcg_Cpl_GFXCLK, cg_gfx_sclk, RLC_BPM_status, TTOP_P1_hard_resetb, TTOP_P1_combined_resetb_0,
  TTOP_P1_combined_resetb_1, TTOP_P1_combined_resetb_2,
  TTOP_P1_combined_resetb_3, TTOP_P2_hard_resetb, TTOP_P2_combined_resetb_0,
  TTOP_P2_combined_resetb_1, TTOP_P2_combined_resetb_2,
  TTOP_P2_combined_resetb_3, TTOP_P1_mem_power_ctrl, TTOP_P2_mem_power_ctrl,
  TTOP_FGCG_override, TTOP_MGCG_override, TTOP_MGLS_current_freq_code , TTOP_MGLS_override ,dbg_mux_Data,
  cgcg_en, BPM_BPM_cmd, aon_Cpl_GFXCLK,
  oCUSTOM_MEM_RM, oCUSTOM_MEM_RME, didt_throttle_level, edc_throttle_level,
  // Inputs
`ifdef GC__GFX_PIPELINE_CLOCK_GATING__1
  RLC_TARG_gfx3Den, 
  BPM_gfx3d,
`endif
  Cpl_GFXCLK, TEST_CLK_EN, pg_P1_strap, pg_P2_strap,
  RLC_TARG_gfxen, TARG_BPM_cgcgls_override, RLC_BPM_cmd, BPM_blk_addr0, BPM_blk_addr1, BPM_blk_addr2, BPM_blk_addr3, RSMU_GFX_cold_resetb,
  RSMU_GFX_hard_resetb, xRBM_blk_soft_0_resetb, xRBM_blk_soft_1_resetb, oRM_VALID,
`ifdef GC__DLDO_POWER_GATING__1
  bpm_bpm_resetb,
`endif
  Dft_Clk_Gater_Aon, Dft_Clk_Gater_En, tmon_s0_select, tmon_s1_select, tmon_s_bd0_rd_pr, tmon_s_bd0_rd_pw, tmon_s_bd0_ref_pr, tmon_s_bd0_ref_pw,
  tmon_s_bd1_rd_pr, tmon_s_bd1_rd_pw, tmon_s_bd1_ref_pr, tmon_s_bd1_ref_pw,
  xRBM_blk_soft_2_resetb, xRBM_blk_soft_3_resetb, mem_power_ctrl,dbg_mux_SelLane, DbgBusIn_en, DbgBusIn_dbg_bus, DbgBusOut_en, DbgBusOut_dbg_bus,
  iCUSTOM_MEM_RM, iCUSTOM_MEM_RME, BPM_BPM_status, charz_observ_a, charz_observ_b, droopdetected,
  RLC_GFX_rm_valid, dfs_clk, dft_clk_gate_ctrl, VDDAN_18_TS, VDDCR_SOC, scan_mode
  );


//--------------------------------------------------------------------------------
// Input/Outputs
//--------------------------------------------------------------------------------
// --------- cgtt_tile ------------------
input                                     Cpl_GFXCLK;      // Clock from SPLL
output                                    cgcg_Cpl_GFXCLK; // GFX clock branch for gfx repeaters
`ifdef GC__GFX_PIPELINE_CLOCK_GATING__1
output                                    cg3d_Cpl_GFXCLK; // GFX clock branch for gfx 3d repeaters
`endif
output                                    cg_gfx_sclk;     // GFX clock branch for gfx blocks and gfx repeaters
output                                    cgcg_en;         // GFX coarse grain clock gating enable
output                                    aon_Cpl_GFXCLK;
output  [1:0]                             dft_clk_gate_ctrl;

input                                     VDDAN_18_TS;
input                                     VDDCR_SOC;

input                                     scan_mode;

input                                     dfs_clk;
input                                     tmon_s0_select;
inout                                     tmon_s_bd0_rd_pr;
inout                                     tmon_s_bd0_rd_pw;
inout                                     tmon_s_bd0_ref_pr;
inout                                     tmon_s_bd0_ref_pw;
input                                     tmon_s1_select;
inout                                     tmon_s_bd1_rd_pr;
inout                                     tmon_s_bd1_rd_pw;
inout                                     tmon_s_bd1_ref_pr;
inout                                     tmon_s_bd1_ref_pw;

input                                     TEST_CLK_EN;     // DFT control over clocks - One to all
input                                     Dft_Clk_Gater_Aon;
input                                     Dft_Clk_Gater_En;

input                                     pg_P1_strap;        // P1 = 1, P1 PGFSM is enabled
input                                     pg_P2_strap;        // P2 = 1, P2 PGFSM is enabled

output                                    oRM_VALID;
output  [2:0]                             didt_throttle_level;
output  [2:0]                             edc_throttle_level;

  // 1500 interface for AVFS
input                                     WRCK;
input                                     WSI;
input                                     ShiftWR;
input                                     UpdateWR;
input                                     CaptureWR;
input                                     WDREn;
input                                     WRSTN;
output                                    WRO;

// --------- bpm ------------------
//Gfx clk control from RLC
input                                     RLC_TARG_gfxen;
`ifdef GC__GFX_PIPELINE_CLOCK_GATING__1
input                                     RLC_TARG_gfx3Den;
input                                     BPM_gfx3d;
`endif
input                                     TARG_BPM_cgcgls_override;

// Command signal from RLC
input                                     RLC_BPM_cmd;
// Command signal to next BPM
output                                    BPM_BPM_cmd;
// Status signal to RLC
output                                    RLC_BPM_status;
// Status signal from previous BPM
input                                     BPM_BPM_status;

// Address for a given bpm. Address from command is compared to initiate activity
input [1:0]                                BPM_blk_addr3;   // BPM_blk_addr[12:11]
input                                      BPM_blk_addr2;   // BPM_blk_addr[10:10]
input [4:0]                                BPM_blk_addr1;   // BPM_blk_addr[9:5]
input [4:0]                                BPM_blk_addr0;   // BPM_blk_addr[4:0]

// io_resetb will be used to reset deserializer for SMU-TTOP SerDes link.
input                                     RSMU_GFX_cold_resetb;

`ifdef GC__DLDO_POWER_GATING__1
input                                     bpm_bpm_resetb;
`endif

input                                     RLC_GFX_rm_valid;
input  [5:0]                              iCUSTOM_MEM_RM;
input                                     iCUSTOM_MEM_RME;
output [5:0]                              oCUSTOM_MEM_RM;
output                                    oCUSTOM_MEM_RME;

// --------- rst_combine ------------------
input                                     RSMU_GFX_hard_resetb;         //hard_resetb from RCU
input                                     xRBM_blk_soft_0_resetb;     //soft_resetb from SRBM or GRBM. One per tile
input                                     xRBM_blk_soft_1_resetb;     //soft_resetb from SRBM or GRBM. One per tile
input                                     xRBM_blk_soft_2_resetb;     //soft_resetb from SRBM or GRBM. One per tile
input                                     xRBM_blk_soft_3_resetb;     //soft_resetb from SRBM or GRBM. One per tile

output                                    TTOP_P1_hard_resetb;        //P1 hard resetb
output                                    TTOP_P1_combined_resetb_0;  //P1 combined resetb
output                                    TTOP_P1_combined_resetb_1;  //P1 combined resetb
output                                    TTOP_P1_combined_resetb_2;  //P1 combined resetb
output                                    TTOP_P1_combined_resetb_3;  //P1 combined resetb
output                                    TTOP_P2_hard_resetb;        //P2 hard resetb
output                                    TTOP_P2_combined_resetb_0;  //P2 combined resetb
output                                    TTOP_P2_combined_resetb_1;  //P2 combined resetb
output                                    TTOP_P2_combined_resetb_2;  //P2 combined resetb
output                                    TTOP_P2_combined_resetb_3;  //P2 combined resetb

// --------- PGFSM ------------------
// Refernce clock for counters inside PGFSM

// Daisy chain of mother/daughter buffers
`ifdef GC__GFX_ISLAND_POWER_GATING__1
input                                     Cpl_REFCLK;
// DFT overrides
 `ifdef GC__GNB_DFT
input                                     TST_TARG_miscpfh_tclk;
input                                     PGFSM_TARG_tdr_tresetb;
input                                     PGFSM_TARG_tdr_shift;
input                                     PGFSM_TARG_tdr_capture;
input                                     PGFSM_TARG_tdr_update;
input                                     PGFSM_TARG_tdr_tdi;
input                                     PGFSM_tdr_en_rw;
input                                     PGFSM_tdr_en_ro;
output                                    PGFSM_tdr_tdo_rw;
output                                    PGFSM_tdr_tdo_ro;
input                                     PGFSM2_tdr_en_rw;
input                                     PGFSM2_tdr_en_ro;
output                                    PGFSM2_tdr_tdo_rw;
output                                    PGFSM2_tdr_tdo_ro;
 `else
input                                     DFT_TARG_si;
input                                     DFT_TARG_trstb;
input                                     DFT_TARG_shift;
input                                     DFT_TARG_capture;
input                                     DFT_TARG_update;
input                                     DFT_TARG_tclk;

output                                    DFT_BPM_0_rw_so;
output                                    DFT_BPM_0_ro_so;
input                                     DFT_BPM_0_rw_dec;
input                                     DFT_BPM_0_ro_dec;

output                                    DFT_BPM_1_rw_so;
output                                    DFT_BPM_1_ro_so;
input                                     DFT_BPM_1_rw_dec;
input                                     DFT_BPM_1_ro_dec;
 `endif // !`ifdef GC__GNB_DFT

output                                    DFT_P1_resetb;
output                                    DFT_P2_resetb;

//Mother, daughter, reset, isolation outputs
output                                    TTOP_P1_mother_sleep;
output                                    TTOP_P2_mother_sleep;
output                                    TTOP_P1_daughter_sleep;
output                                    TTOP_P2_daughter_sleep;
output                                    TTOP_P1_iso_clampn;
output                                    TTOP_P2_iso_clampn;
output                                    TTOP_P1_scan_iso_clampn;
output                                    TTOP_P2_scan_iso_clampn;

`endif //  `ifdef GC__GFX_ISLAND_POWER_GATING__1

//Mem_power_ctrl bus from CGTT Tile
input [`MEM__MEM_POWER_CTRL-1:0]    mem_power_ctrl;

//Mem_power_ctrl bus for P1 and P2 power domains
output [`MEM__MEM_POWER_CTRL-1:0]   TTOP_P1_mem_power_ctrl;
output [`MEM__MEM_POWER_CTRL-1:0]   TTOP_P2_mem_power_ctrl;


// Fine grain clock gating override. 1=Disable MGCG, 0=Enable MGCG
output                              TTOP_FGCG_override;

// Medium grain clock gating override. 1=Disable MGCG, 0=Enable MGCG
output                              TTOP_MGCG_override;


output [3:0]        TTOP_MGLS_current_freq_code;
output              TTOP_MGLS_override;

input               DbgBusIn_en;
input  [15:0]       DbgBusIn_dbg_bus;
output              DbgBusOut_en;
output [15:0]       DbgBusOut_dbg_bus;

input  [35:0]       dbg_mux_SelLane;
output [31:0]       dbg_mux_Data;

output              charz_observ_a;
output              charz_observ_b;
output              droopdetected;


parameter enable_pgfsm = 1;
parameter custom_rm_through_bpm = 0;
parameter compiled_rm_through_bpm = 1;
parameter enable_avfs = 0;
parameter enable_dd = 0;
parameter num_rdih = 0;
parameter num_rdiv = 0;
parameter enable_psm_didt_dd = 0;
parameter enable_psm_edc_dd = 0;
parameter DIDT_CTRL_REG0_WIDTH = 32;
parameter DIDT_CTRL_REG1_WIDTH = 32;
parameter EDC_CTRL_REG_WIDTH   = 32;
parameter enable_dummy_avfs_node = 0;

  wire  [12:0]                             BPM_blk_addr = {BPM_blk_addr3, BPM_blk_addr2, BPM_blk_addr1, BPM_blk_addr0};
  wire  [3:0]                              TTOP_MGLS_current_freq_code = 4'h0;
  reg                                      mgmempg_override;          

  reg   [9:0]                              groupcast_cntl;
  reg                                      rm_cntl;
  reg                                      bpm_blk_hard_resetb;
  wire  [25:0]                             rm_synced_v;
  wire  [6:0]                              rm_synced_c;
  wire                                     disable_mgls_v;
  wire                                     disable_mgls_c;

  wire                                     disable_mgcg_v;
  wire                                     disable_mgcg_c;

  wire                                     disable_mgls;
  wire                                     disable_mgcg;

  reg                                      DD_CTRL1;
  reg   [31:0]                             DD_CTRL2;
  reg   [30:0]                             DD_CTRL3;
  reg   [26:0]                             DD_CTRL4;
  reg                                      DD_MASTER_RESET;
  reg                                      DD_STATIC_ENABLE;
  reg                                      DD_STRETCH_ENABLE;
  wire                                     charz_observ_a;
  wire                                     charz_observ_b;
  wire                                     dd_dll_lock;
  wire                                     droopdetected;

//-------------------------------------------------------------------------------
  reg [`MEM__MEM_POWER_CTRL-1:0]     TTOP_P1_mem_power_ctrl, TTOP_P2_mem_power_ctrl;
  wire [`MEM__MEM_POWER_CTRL-1:0]    PGFSM_P1_pwr_ctrl_out, PGFSM_P2_pwr_ctrl_out;
  wire [(`MEM__MEM_POWER_CTRL-1):0]  cgtt_tile_mem_power_ctrl;

  reg                                      cgmempg_en, cgmempg_on;
  reg                                      cgls_ctrl;
  reg                                      rTTOP_FGCG_override;
  reg                                      rTTOP_MGCG_override;
  wire                                     TTOP_MGCG_override;
  wire                                     TTOP_FGCG_override;

  wire                                     Cpl_GFXCLK;
  reg                                      cgcg_override;
  reg                                      stop_clock;
`ifdef GC__GFX_PIPELINE_CLOCK_GATING__1
  reg                                      cg3d_blk_cntl_override;
  reg                                      cg3d_rep_cntl_override;
`endif

  wire                                     hard_resetb;
  wire                                     io_resetb;
`ifdef GC__DLDO_POWER_GATING__1
  wire                                     bpm_bpm_resetb_sync;
  wire                                     hard_resetb_pre;
`endif
  wire                                     io_resetb_sync;
  wire                                     io_reset;

  wire                                     PGFSM_P1_resetb;
  wire                                     PGFSM_P2_resetb;
`ifdef GC__GFX_ISLAND_POWER_GATING__1
  reg                                      DFT_P1_resetb;
  reg                                      DFT_P2_resetb;
`endif
  wire                                     DFT_PD_P1_resetb;
  wire                                     DFT_PD_P2_resetb;

  reg                                      p1_resetb;
  reg                                      p2_resetb;

  wire                                     bpm_daisy_status;
  reg                                      p1_daisy_status_in;
  reg                                      p2_daisy_status_in;
  wire                                     p1_daisy_status_out;
  wire                                     p2_daisy_status_out;
  reg                                      pgfsm_daisy_status;

// SERDES Command wires
  wire [23:0]                              SERDES_cmd;
  wire [31:0]                              SERDES_wdata;
  wire                                     SERDES_new_cmd;

  wire                                     SERDES_p1_new_cmd;

  wire                                     BPM_select;

// DFT Signalling
  reg                                      DFT_BPM_tdr_tdo;

  reg                                      p1_tdr_tdo;
  reg                                      p2_tdr_tdo;

//Mother, daughter, reset, isolation outputs
  wire                                     PGFSM_P1_mother_sleep;
  wire                                     PGFSM_P1_daughter_sleep;
  wire                                     PGFSM_P1_iso_clampn;
  wire                                     PGFSM_P1_scan_iso_clampn;

  wire                                     PGFSM_P2_mother_sleep;
  wire                                     PGFSM_P2_daughter_sleep;
  wire                                     PGFSM_P2_iso_clampn;
  wire                                     PGFSM_P2_scan_iso_clampn;

//--------------------------------------------------------------------------------
// cgtt_tile
//--------------------------------------------------------------------------------
//Flopping constant inputs to get rid of Input -> Flop violations
  wire [12:0]                              BPM_blk_addr_ff;
  wire                                     BPM_BPM_status_ff;

//Clock gate for cg_gfx_sclk
//--------------------------------
  wire                                     RLC_TARG_gfxen_sync;
  reg                                      RLC_TARG_gfxen_ff;
  wire                                     cgcgclk_en;
  wire                                     cgcgclk_en_rep;
`ifdef GC__GFX_PIPELINE_CLOCK_GATING__1
  wire                                     RLC_TARG_gfx3Den_sync;
  reg                                      RLC_TARG_gfx3Den_ff;
  wire                                     cg3dclk_en;
  wire                                     cg3d_Cpl_GFXCLK;
  wire                                     cg3dclk_en_rep;
  wire                                     pre_cg3d_Cpl_GFXCLK;
  wire                                     pre_cgcg_Cpl_GFXCLK;
`endif

  wire                                     rm_valid_sync;
  reg                                      rm_didt_ff;

  wire                                     RLC_cmd_q;
  wire                                     cg_Cpl_GFXCLK;
  wire                                     clk_en;  
   
//BPM Debug Bus    
  wire [7:0]                               bpm_dbus_byte0;
  wire [7:0]                               bpm_dbus_byte1;
  wire [7:0]                               bpm_dbus_byte2;
  wire [7:0]                               bpm_dbus_byte3;
  wire [31:0]                              bpm_dbus;

// 
  wire                                     short_write;
  wire                                     long_write;
  wire                                     bc_short_write;
  wire                                     bc_long_write;
  wire                                     single_read;   // always 32bit data expected
  wire                                     bc_read;

  reg   [DIDT_CTRL_REG0_WIDTH-1:0]         psm_didt_dd_cntl0;
  wire  [DIDT_CTRL_REG0_WIDTH-1:0]         psm_didt_dd_cntl0_out;
  reg   [DIDT_CTRL_REG1_WIDTH-1:0]         psm_didt_dd_cntl1;
  wire  [DIDT_CTRL_REG0_WIDTH-1:0]         psm_didt_dd_cntl1_out;
  reg   [EDC_CTRL_REG_WIDTH-1:0]           psm_edc_dd_cntl;
  wire  [EDC_CTRL_REG_WIDTH-1:0]           psm_edc_dd_cntl_out;
  reg   [`PSM_CURPSM_LENGTH-1:0]           psm_counter;
  wire  [31:0]                             psm_didt_dd_stat;
  wire  [31:0]                             psm_edc_dd_stat;

  wire  [2:0]                              didt_throttle_level;
  wire  [2:0]                              edc_throttle_level;

  reg         n0scan_cgcg_Cpl_GFXCLK_en;
  reg         n0scan_cg3d_Cpl_GFXCLK_en;
  reg         n0scan_aon_Cpl_GFXCLK_en;
  reg         n0scan_Dft_Clk_Gater_Aon_d1;
  reg         n0scan_Dft_Clk_Gater_En_d1;
  wire        aon_Cpl_GFXCLK;
  wire        cgcg_en;
  reg         cgcg_en_ff;

  wire        is_broadcast_addr;

  assign      dft_clk_gate_ctrl = {Dft_Clk_Gater_En, Dft_Clk_Gater_Aon};

always @(posedge Cpl_GFXCLK) begin
  n0scan_Dft_Clk_Gater_Aon_d1 <= Dft_Clk_Gater_Aon;
  n0scan_Dft_Clk_Gater_En_d1  <= Dft_Clk_Gater_En;
  n0scan_aon_Cpl_GFXCLK_en    <= ~n0scan_Dft_Clk_Gater_En_d1 | n0scan_Dft_Clk_Gater_Aon_d1;
end

// For Cpl_GFXCLK repeaters and bpm internal logics
ati_clock_gate bpm_aon_Cpl_gfx_clkgate (
        .clk_out(aon_Cpl_GFXCLK),
        .clk_src(Cpl_GFXCLK),
        .clk_en(n0scan_aon_Cpl_GFXCLK_en),
        .TE(TEST_CLK_EN));

// For cgcg repeaters and bpm internal logics
ati_clock_gate bpm_cgcg_Cpl_gfx_clkgate (
        .clk_out(cgcg_Cpl_GFXCLK),
        .clk_src(Cpl_GFXCLK),
        .clk_en(n0scan_cgcg_Cpl_GFXCLK_en),
        .TE(TEST_CLK_EN));

// For 3dcg repeaters and bpm internal logics
ati_clock_gate bpm_cg3d_Cpl_gfx_clkgate (
        .clk_out(cg3d_Cpl_GFXCLK),
        .clk_src(Cpl_GFXCLK),
        .clk_en(n0scan_cg3d_Cpl_GFXCLK_en),
        .TE(TEST_CLK_EN));

techind_sync_icd #(.depth(3)) io_rst_sync   (.CKdst(aon_Cpl_GFXCLK), .Dsrc(RSMU_GFX_cold_resetb), .Qdst(io_resetb),
                                .SB(1'b1), .RB(1'b1), .FLUSH(1'b0), .CKsrc(1'b0), .EN(1'b1), .Qsrc(), 
                                .SC1(1'b0), .SC2(1'b0), .SE(1'b0), .SI(1'b0), .SO()
`ifdef ABV
                                ,.InputChkDis(1'b0)
`endif
);

`ifdef GC__DLDO_POWER_GATING__1
techind_sync_icd #(.depth(3)) bpm_rst_sync   (.CKdst(aon_Cpl_GFXCLK), .Dsrc(bpm_bpm_resetb), .Qdst(bpm_bpm_resetb_sync),
                                 .SB(1'b1), .RB(1'b1), .FLUSH(1'b0), .CKsrc(1'b0), .EN(1'b1), .Qsrc(), 
                                 .SC1(1'b0), .SC2(1'b0), .SE(1'b0), .SI(1'b0), .SO()
`ifdef ABV
                                 ,.InputChkDis(1'b0)
`endif
);
   
assign io_reset = ~(io_resetb & bpm_bpm_resetb_sync);  
wire   RSMU_GFX_hard_resetb_in = RSMU_GFX_hard_resetb & bpm_bpm_resetb_sync; 
wire   RSMU_GFX_cold_resetb_in = RSMU_GFX_cold_resetb & bpm_bpm_resetb_sync; 
`else
assign io_reset = ~io_resetb;  
`endif
   
techind_sync_icd #(.depth(3)) rlc_gfxen   (.CKdst(aon_Cpl_GFXCLK), .Dsrc(RLC_TARG_gfxen), .Qdst(RLC_TARG_gfxen_sync),
                              .SB(1'b1), .RB(1'b1), .FLUSH(1'b0), .CKsrc(1'b0), .EN(1'b1), .Qsrc(), 
                              .SC1(1'b0), .SC2(1'b0), .SE(1'b0), .SI(1'b0), .SO()
`ifdef ABV
                              ,.InputChkDis(1'b1)
`endif
);

`ifdef GC__GFX_PIPELINE_CLOCK_GATING__1
techind_sync_icd #(.depth(3)) rlc_gfx3d_en   (.CKdst(aon_Cpl_GFXCLK), .Dsrc(RLC_TARG_gfx3Den), .Qdst(RLC_TARG_gfx3Den_sync),
                                 .SB(1'b1), .RB(1'b1), .FLUSH(1'b0), .CKsrc(1'b0), .EN(1'b1), .Qsrc(), 
                                 .SC1(1'b0), .SC2(1'b0), .SE(1'b0), .SI(1'b0), .SO()
`ifdef ABV
                                 ,.InputChkDis(1'b1)
`endif
);
`endif

ati_dff_in  #(1) bpm_cmd_in_ff (.clk(aon_Cpl_GFXCLK), .d(RLC_BPM_cmd), .q(RLC_cmd_q) );
ati_dff_in  #(1) bpm_cmd_out_ff (.clk(aon_Cpl_GFXCLK), .d(RLC_cmd_q), .q(BPM_BPM_cmd) );
ati_dff_out #(1) bpm_status_ff (.clk(aon_Cpl_GFXCLK), .d(bpm_daisy_status | BPM_BPM_status_ff), .q(RLC_BPM_status) );
ati_dff_in  #(13) blk_addr_ff (.clk(aon_Cpl_GFXCLK), .d(BPM_blk_addr), .q(BPM_blk_addr_ff) );
ati_dff_in  #(1) bpm_status_in (.clk(aon_Cpl_GFXCLK), .d(BPM_BPM_status), .q(BPM_BPM_status_ff) );

always @(posedge aon_Cpl_GFXCLK) begin
  RLC_TARG_gfxen_ff <= RLC_TARG_gfxen_sync;
`ifdef GC__GFX_PIPELINE_CLOCK_GATING__1
  RLC_TARG_gfx3Den_ff <= RLC_TARG_gfx3Den_sync;
`endif
end

assign cgcgclk_en = (RLC_TARG_gfxen_ff && ~stop_clock) | cgcg_override | TARG_BPM_cgcgls_override;
assign cgcgclk_en_rep = RLC_TARG_gfxen_ff | cgcg_override | TARG_BPM_cgcgls_override;

`ifdef GC__GFX_PIPELINE_CLOCK_GATING__1
assign cg3dclk_en = (RLC_TARG_gfx3Den_ff && ~stop_clock) | cgcg_override | TARG_BPM_cgcgls_override;
assign cg3dclk_en_rep = RLC_TARG_gfx3Den_ff | cgcg_override | TARG_BPM_cgcgls_override;

always @(posedge aon_Cpl_GFXCLK) begin
  cgcg_en_ff <= (BPM_gfx3d && ~cg3d_blk_cntl_override) ? cg3dclk_en : cgcgclk_en;
  n0scan_cg3d_Cpl_GFXCLK_en <= cg3d_rep_cntl_override ? cgcgclk_en_rep : cg3dclk_en_rep;
end
`else
always @(posedge aon_Cpl_GFXCLK) begin
  cgcg_en_ff <= cgcgclk_en;
end
`endif

assign cgcg_en = cgcg_en_ff;

always @(posedge Cpl_GFXCLK) begin
  n0scan_cgcg_Cpl_GFXCLK_en <= (cgcgclk_en_rep & ~n0scan_Dft_Clk_Gater_En_d1) | n0scan_Dft_Clk_Gater_Aon_d1;
end

//FIXME: need to remove this after dft gater merge is done.
assign cg_gfx_sclk = Cpl_GFXCLK;

//--------------------------------------------------------------------------------
// rst_combine for P1 and P2 power domains
//--------------------------------------------------------------------------------
rst_combine_4 rst_combine_4(
                            .clk                    (aon_Cpl_GFXCLK),
                            // input resetb
                            .RCU_blk_hard_resetb    (RSMU_GFX_hard_resetb),
                            .xRBM_blk_soft_0_resetb (xRBM_blk_soft_0_resetb),
                            .xRBM_blk_soft_1_resetb (xRBM_blk_soft_1_resetb),
                            .xRBM_blk_soft_2_resetb (xRBM_blk_soft_2_resetb),
                            .xRBM_blk_soft_3_resetb (xRBM_blk_soft_3_resetb),
                            // output Combined resetb
`ifdef GC__DLDO_POWER_GATING__1
                            .hard_resetb            (hard_resetb_pre),
`else
                            .hard_resetb            (hard_resetb),
`endif
                            .combined_0_resetb      (combined_resetb_0),
                            .combined_1_resetb      (combined_resetb_1),
                            .combined_2_resetb      (combined_resetb_2),
                            .combined_3_resetb      (combined_resetb_3));


`ifdef GC__DLDO_POWER_GATING__1
// Power domain P1 reset
assign TTOP_P1_hard_resetb       = hard_resetb_pre   & p1_resetb & bpm_bpm_resetb_sync & bpm_blk_hard_resetb;
assign TTOP_P1_combined_resetb_0 = combined_resetb_0 & p1_resetb & bpm_bpm_resetb_sync & bpm_blk_hard_resetb;
assign TTOP_P1_combined_resetb_1 = combined_resetb_1 & p1_resetb & bpm_bpm_resetb_sync & bpm_blk_hard_resetb;
assign TTOP_P1_combined_resetb_2 = combined_resetb_2 & p1_resetb & bpm_bpm_resetb_sync & bpm_blk_hard_resetb;
assign TTOP_P1_combined_resetb_3 = combined_resetb_3 & p1_resetb & bpm_bpm_resetb_sync & bpm_blk_hard_resetb;

// Power domain P2 reset
assign TTOP_P2_hard_resetb       = hard_resetb_pre   & p2_resetb & bpm_bpm_resetb_sync & bpm_blk_hard_resetb;
assign TTOP_P2_combined_resetb_0 = combined_resetb_0 & p2_resetb & bpm_bpm_resetb_sync & bpm_blk_hard_resetb;
assign TTOP_P2_combined_resetb_1 = combined_resetb_1 & p2_resetb & bpm_bpm_resetb_sync & bpm_blk_hard_resetb;
assign TTOP_P2_combined_resetb_2 = combined_resetb_2 & p2_resetb & bpm_bpm_resetb_sync & bpm_blk_hard_resetb;
assign TTOP_P2_combined_resetb_3 = combined_resetb_3 & p2_resetb & bpm_bpm_resetb_sync & bpm_blk_hard_resetb;

assign hard_resetb               = hard_resetb_pre   & bpm_bpm_resetb_sync;
`else

// Power domain P1 reset
assign TTOP_P1_hard_resetb       = hard_resetb       & p1_resetb & bpm_blk_hard_resetb;
assign TTOP_P1_combined_resetb_0 = combined_resetb_0 & p1_resetb & bpm_blk_hard_resetb;
assign TTOP_P1_combined_resetb_1 = combined_resetb_1 & p1_resetb & bpm_blk_hard_resetb;
assign TTOP_P1_combined_resetb_2 = combined_resetb_2 & p1_resetb & bpm_blk_hard_resetb;
assign TTOP_P1_combined_resetb_3 = combined_resetb_3 & p1_resetb & bpm_blk_hard_resetb;

// Power domain P2 reset
assign TTOP_P2_hard_resetb       = hard_resetb       & p2_resetb & bpm_blk_hard_resetb;
assign TTOP_P2_combined_resetb_0 = combined_resetb_0 & p2_resetb & bpm_blk_hard_resetb;
assign TTOP_P2_combined_resetb_1 = combined_resetb_1 & p2_resetb & bpm_blk_hard_resetb;
assign TTOP_P2_combined_resetb_2 = combined_resetb_2 & p2_resetb & bpm_blk_hard_resetb;
assign TTOP_P2_combined_resetb_3 = combined_resetb_3 & p2_resetb & bpm_blk_hard_resetb;
`endif


//--------------------------------------------------------------------------------
// mem_power_ctrl mapping
//--------------------------------------------------------------------------------

// mem_power_ctrl bus goes thru hierarchy of the design and connects to all memories in a tile.
// Mapping is as follows
// Bit 0    = Virage LS    - Light sleep - Controlled by Coarse grain light sleep feature in GFXIP. All other IPs control LS pin locally.
// Bit 1    = AMD DS_D     - Deep sleep daughter
// Bit 2    = AMD DS_M     - Deep sleep mother
// Bit 3    = AMD SD_D     - Shut down daughter
// Bit 4    = AMD SD_M     - Shut down mother
// Bit 5    = Virage DS    - Deep Sleep
// Bit 6    = Virage SD    - Shut down
// Bit 7    = Virage FISO  - FISO pin
// Bit 16:8 = Virage RM    - 9 bits Read margin control. 3 bits for HD, 3 bits for RF, 3 bots for PDP (includes RME)
// Bit 30:8 = AMD RM       - 23 bits Read margin control. 10 bits for RF, 13bits for PDP (includes RME)


always @(*)
begin
  case ( {pg_P2_strap, pg_P1_strap}  )
    2'b00:
      begin
        p1_daisy_status_in  = 1'b0;
        p2_daisy_status_in  = 1'b0;
        pgfsm_daisy_status  = 1'b0;

        p1_resetb           = 1'b1;
        p2_resetb           = 1'b1;
`ifdef GC__GFX_ISLAND_POWER_GATING__1
        DFT_P1_resetb       = 1'b1;
        DFT_P2_resetb       = 1'b1;
`endif
        //LS is controlled by bpm for CGLS
        TTOP_P1_mem_power_ctrl[`MEM__MEM_POWER_CTRL_LS]   = (mem_power_ctrl[`MEM__MEM_POWER_CTRL_LS] | cgls_ctrl);
        TTOP_P2_mem_power_ctrl[`MEM__MEM_POWER_CTRL_LS]   = (mem_power_ctrl[`MEM__MEM_POWER_CTRL_LS] | cgls_ctrl);

        //Outputs of PGFSM go to outputs of bpm
        TTOP_P1_mem_power_ctrl[`MEM__MEM_POWER_CTRL_TDR_SD:`MEM__MEM_POWER_CTRL_DS] = mem_power_ctrl[`MEM__MEM_POWER_CTRL_TDR_SD:`MEM__MEM_POWER_CTRL_DS];
        TTOP_P2_mem_power_ctrl[`MEM__MEM_POWER_CTRL_TDR_SD:`MEM__MEM_POWER_CTRL_DS] = mem_power_ctrl[`MEM__MEM_POWER_CTRL_TDR_SD:`MEM__MEM_POWER_CTRL_DS];
        TTOP_P1_mem_power_ctrl[`GPU__MEM__MEM_POWER_CTRL_RM_END:`GPU__MEM__MEM_POWER_CTRL_RM_START] = rm_synced_v;
        TTOP_P2_mem_power_ctrl[`GPU__MEM__MEM_POWER_CTRL_RM_END:`GPU__MEM__MEM_POWER_CTRL_RM_START] = rm_synced_v;
        TTOP_P1_mem_power_ctrl[`MEM__MEM_POWER_CTRL-1:`GPU__MEM__MEM_POWER_CTRL_RM_END+1] = mem_power_ctrl[`MEM__MEM_POWER_CTRL-1:`GPU__MEM__MEM_POWER_CTRL_RM_END+1];
        TTOP_P2_mem_power_ctrl[`MEM__MEM_POWER_CTRL-1:`GPU__MEM__MEM_POWER_CTRL_RM_END+1] = mem_power_ctrl[`MEM__MEM_POWER_CTRL-1:`GPU__MEM__MEM_POWER_CTRL_RM_END+1];

      end
    2'b01:
      begin
        p1_daisy_status_in  = 1'b0;
        p2_daisy_status_in  = 1'b0;
        pgfsm_daisy_status  = p1_daisy_status_out;

        p1_resetb           = PGFSM_P1_resetb;
        p2_resetb           = PGFSM_P1_resetb;     // If P1 only, P2 is promoted to AON
`ifdef GC__GFX_ISLAND_POWER_GATING__1
        DFT_P1_resetb       = DFT_PD_P1_resetb;
        DFT_P2_resetb       = 1'b1;
`endif
        TTOP_P1_mem_power_ctrl[`MEM__MEM_POWER_CTRL_LS]   = (PGFSM_P1_pwr_ctrl_out[`MEM__MEM_POWER_CTRL_LS] | cgls_ctrl);
        TTOP_P2_mem_power_ctrl[`MEM__MEM_POWER_CTRL_LS]   = (PGFSM_P1_pwr_ctrl_out[`MEM__MEM_POWER_CTRL_LS] | cgls_ctrl);
        TTOP_P1_mem_power_ctrl[`MEM__MEM_POWER_CTRL_TDR_SD:`MEM__MEM_POWER_CTRL_DS] = PGFSM_P1_pwr_ctrl_out[`MEM__MEM_POWER_CTRL_TDR_SD:`MEM__MEM_POWER_CTRL_DS];
        TTOP_P2_mem_power_ctrl[`MEM__MEM_POWER_CTRL_TDR_SD:`MEM__MEM_POWER_CTRL_DS] = PGFSM_P1_pwr_ctrl_out[`MEM__MEM_POWER_CTRL_TDR_SD:`MEM__MEM_POWER_CTRL_DS];
        TTOP_P1_mem_power_ctrl[`GPU__MEM__MEM_POWER_CTRL_RM_END:`GPU__MEM__MEM_POWER_CTRL_RM_START] = rm_synced_v;
        TTOP_P2_mem_power_ctrl[`GPU__MEM__MEM_POWER_CTRL_RM_END:`GPU__MEM__MEM_POWER_CTRL_RM_START] = rm_synced_v;
        TTOP_P1_mem_power_ctrl[`MEM__MEM_POWER_CTRL-1:`GPU__MEM__MEM_POWER_CTRL_RM_END+1] = PGFSM_P1_pwr_ctrl_out[`MEM__MEM_POWER_CTRL-1:`GPU__MEM__MEM_POWER_CTRL_RM_END+1];
        TTOP_P2_mem_power_ctrl[`MEM__MEM_POWER_CTRL-1:`GPU__MEM__MEM_POWER_CTRL_RM_END+1] = PGFSM_P1_pwr_ctrl_out[`MEM__MEM_POWER_CTRL-1:`GPU__MEM__MEM_POWER_CTRL_RM_END+1];

      end
    2'b10:
      begin
        p1_daisy_status_in  = 1'b0;
        p2_daisy_status_in  = 1'b0;
        pgfsm_daisy_status  = p2_daisy_status_out;

        p1_resetb           = 1'b1;                 // If P2 only, P1 is promoted to AON
        p2_resetb           = PGFSM_P2_resetb;
`ifdef GC__GFX_ISLAND_POWER_GATING__1
        DFT_P1_resetb       = 1'b1;
        DFT_P2_resetb       = DFT_PD_P2_resetb;
`endif
        TTOP_P1_mem_power_ctrl[`MEM__MEM_POWER_CTRL_LS]   = (mem_power_ctrl[`MEM__MEM_POWER_CTRL_LS] | cgls_ctrl);
        TTOP_P2_mem_power_ctrl[`MEM__MEM_POWER_CTRL_LS]   = (PGFSM_P2_pwr_ctrl_out[`MEM__MEM_POWER_CTRL_LS] | cgls_ctrl);
        TTOP_P1_mem_power_ctrl[`MEM__MEM_POWER_CTRL_TDR_SD:`MEM__MEM_POWER_CTRL_DS] = mem_power_ctrl[`MEM__MEM_POWER_CTRL_TDR_SD:`MEM__MEM_POWER_CTRL_DS];
        TTOP_P2_mem_power_ctrl[`MEM__MEM_POWER_CTRL_TDR_SD:`MEM__MEM_POWER_CTRL_DS] = PGFSM_P2_pwr_ctrl_out[`MEM__MEM_POWER_CTRL_TDR_SD:`MEM__MEM_POWER_CTRL_DS];
        TTOP_P1_mem_power_ctrl[`GPU__MEM__MEM_POWER_CTRL_RM_END:`GPU__MEM__MEM_POWER_CTRL_RM_START] = rm_synced_v;
        TTOP_P2_mem_power_ctrl[`GPU__MEM__MEM_POWER_CTRL_RM_END:`GPU__MEM__MEM_POWER_CTRL_RM_START] = rm_synced_v;
        TTOP_P1_mem_power_ctrl[`MEM__MEM_POWER_CTRL-1:`GPU__MEM__MEM_POWER_CTRL_RM_END+1] = mem_power_ctrl[`MEM__MEM_POWER_CTRL-1:`GPU__MEM__MEM_POWER_CTRL_RM_END+1];
        TTOP_P2_mem_power_ctrl[`MEM__MEM_POWER_CTRL-1:`GPU__MEM__MEM_POWER_CTRL_RM_END+1] = PGFSM_P2_pwr_ctrl_out[`MEM__MEM_POWER_CTRL-1:`GPU__MEM__MEM_POWER_CTRL_RM_END+1];

      end
    2'b11:
      begin
        p1_daisy_status_in  = p2_daisy_status_out;
        p2_daisy_status_in  = 1'b0;
        pgfsm_daisy_status  = p1_daisy_status_out;

        p1_resetb           = PGFSM_P1_resetb;
        p2_resetb           = PGFSM_P2_resetb;
`ifdef GC__GFX_ISLAND_POWER_GATING__1
        DFT_P1_resetb       = DFT_PD_P1_resetb;
        DFT_P2_resetb       = DFT_PD_P2_resetb;
`endif
        TTOP_P1_mem_power_ctrl[`MEM__MEM_POWER_CTRL_LS]   = (PGFSM_P1_pwr_ctrl_out[`MEM__MEM_POWER_CTRL_LS] | cgls_ctrl);
        TTOP_P2_mem_power_ctrl[`MEM__MEM_POWER_CTRL_LS]   = (PGFSM_P2_pwr_ctrl_out[`MEM__MEM_POWER_CTRL_LS] | cgls_ctrl);
        TTOP_P1_mem_power_ctrl[`MEM__MEM_POWER_CTRL_TDR_SD:`MEM__MEM_POWER_CTRL_DS] = PGFSM_P1_pwr_ctrl_out[`MEM__MEM_POWER_CTRL_TDR_SD:`MEM__MEM_POWER_CTRL_DS];
        TTOP_P2_mem_power_ctrl[`MEM__MEM_POWER_CTRL_TDR_SD:`MEM__MEM_POWER_CTRL_DS] = PGFSM_P2_pwr_ctrl_out[`MEM__MEM_POWER_CTRL_TDR_SD:`MEM__MEM_POWER_CTRL_DS];
        TTOP_P1_mem_power_ctrl[`GPU__MEM__MEM_POWER_CTRL_RM_END:`GPU__MEM__MEM_POWER_CTRL_RM_START] = rm_synced_v;
        TTOP_P2_mem_power_ctrl[`GPU__MEM__MEM_POWER_CTRL_RM_END:`GPU__MEM__MEM_POWER_CTRL_RM_START] = rm_synced_v;
        TTOP_P1_mem_power_ctrl[`MEM__MEM_POWER_CTRL-1:`GPU__MEM__MEM_POWER_CTRL_RM_END+1] = PGFSM_P1_pwr_ctrl_out[`MEM__MEM_POWER_CTRL-1:`GPU__MEM__MEM_POWER_CTRL_RM_END+1];
        TTOP_P2_mem_power_ctrl[`MEM__MEM_POWER_CTRL-1:`GPU__MEM__MEM_POWER_CTRL_RM_END+1] = PGFSM_P2_pwr_ctrl_out[`MEM__MEM_POWER_CTRL-1:`GPU__MEM__MEM_POWER_CTRL_RM_END+1];

      end
    default: begin end
  endcase // case ( {pg_P2_strap, pg_P1_strap}  )
  end // always @ (*)

// Deserializer for control, wdata
wire  [7:0]   data_unused;
gfx_pgfsm_deser #(64,6) deserializer
  (
   .data_out    ({SERDES_wdata,data_unused, SERDES_cmd}),
   .new_data    (SERDES_new_cmd),
   .serial_in   (RLC_cmd_q),
   .iclk        (aon_Cpl_GFXCLK),
   .reset       (io_reset),
   .hard_resetb (1'b1)
   );

//==============================================================
// Update for BPM Serdes Readback
//==============================================================
wire                            bpm_transmitting_pulse;
wire                            bpm_read_cmd;
wire                            bpm_avfs_broadcast_read;
wire                            bpm_avfs_broadcast_read_pulse;

reg                             bpm_transmitting;
reg                             bpm_read_request;
wire  [31:0]                    bpm_read_data;
wire  [31:0]                    bpm_read_data_dbg;
wire  [31:0]                    bpm_read_data_tmp;
wire  [31:0]                    bpm_read_data_avfs;

`ifdef GC__GFX_ISLAND_POWER_GATING__1
wire                            pgfsm_pending;
wire                            pgfsm_started;
wire                            pgfsm_transmitting_pulse;
reg                             pgfsm_transmitting;
reg   [31:0]                    pgfsm_transmit_data;
`endif
// Serializer for read
gfx_pgfsm_ser #(32,5) serializer (
   // outputs
   .ser_busy(),
   .ser_data(bpm_daisy_status),
   // inputs
`ifdef GC__GFX_ISLAND_POWER_GATING__1
   .par_data(pgfsm_transmitting ? pgfsm_transmit_data : bpm_read_data),
   .par_valid(bpm_transmitting_pulse | pgfsm_transmitting_pulse),
`else
   .par_data(bpm_read_data),
   .par_valid(bpm_transmitting_pulse),
`endif
   .short_cmd (1'b0),
   .clk(aon_Cpl_GFXCLK),
   .reset(io_reset)
   );

`ifdef GC__GFX_ISLAND_POWER_GATING__1
wire  [31:0]                    pgfsm_read_data;
wire                            pgfsm_data_valid;
wire  [31:0]                    tmp_pgfsm_read_data;
wire                            tmp_pgfsm_data_valid;
// Deserializer for data from PGFSM
gfx_pgfsm_status_deser #(32,5) fifo_deser (
   // outputs
   .data_out(tmp_pgfsm_read_data),
   .new_data(tmp_pgfsm_data_valid),
   // inputs
   .serial_in(pgfsm_daisy_status),
   .iclk(aon_Cpl_GFXCLK),
   .reset(io_reset),
   .hard_resetb(1'b1)
   ); 

reg                             tmp_pgfsm_pending;
always @(posedge aon_Cpl_GFXCLK) begin
  if (io_reset) begin
    tmp_pgfsm_pending       <= 1'b0;
  end else if (pgfsm_data_valid) begin
    pgfsm_transmit_data <= pgfsm_read_data;
    tmp_pgfsm_pending       <= 1'b1;
  end else if (pgfsm_pending & pgfsm_transmitting) begin
    tmp_pgfsm_pending       <= 1'b0;
  end
end


reg                             tmp_pgfsm_started;
always @(posedge aon_Cpl_GFXCLK) begin
  if (io_reset) 
    tmp_pgfsm_started <= 1'b0;
  else if (pgfsm_data_valid & pgfsm_started) 
    tmp_pgfsm_started <= 1'b0;
  else if (pgfsm_daisy_status)
    tmp_pgfsm_started <= 1'b1;
end

assign  pgfsm_started    = (pg_P1_strap | pg_P2_strap) ? tmp_pgfsm_started : 1'b0;
assign  pgfsm_pending    = (pg_P1_strap | pg_P2_strap) ? tmp_pgfsm_pending : 1'b0;
assign  pgfsm_read_data  = (pg_P1_strap | pg_P2_strap) ? tmp_pgfsm_read_data : 32'h0;
assign  pgfsm_data_valid = (pg_P1_strap | pg_P2_strap) ? tmp_pgfsm_data_valid : 1'h0;

`endif


// Priority arbiter for the read path transmission. 
reg   [5:0]                     count35;
`ifdef GC__GFX_ISLAND_POWER_GATING__1
always @(posedge aon_Cpl_GFXCLK) begin
  if (io_reset) begin
    bpm_transmitting   <= 1'b0;
    pgfsm_transmitting <= 1'b0;
    count35            <= 6'b0;
  end else if (pgfsm_pending & ~pgfsm_transmitting & ~bpm_transmitting & ~(|count35)) begin
    // Highest priority is for fifo 
    pgfsm_transmitting <= 1'b1;
    count35            <= 6'b100011;
  end else if (bpm_read_request & ~pgfsm_transmitting & ~pgfsm_started
               & ~bpm_transmitting & ~(|count35)) begin
    // Last priority for the current pgfsm's data to be sent
    bpm_transmitting   <= 1'b1;
    count35            <= 6'b100011;
  end else if (count35 == 6'b11) begin
    bpm_transmitting   <= 1'b0;
    pgfsm_transmitting <= 1'b0;
    count35            <= count35 - 6'b1;
  end else if (|count35) begin
    count35            <= count35 - 6'b1;
  end
end
`else
always @(posedge aon_Cpl_GFXCLK) begin
  if (io_reset) begin
    bpm_transmitting   <= 1'b0;
    count35            <= 6'b0;
  end else if (bpm_read_request & ~bpm_transmitting & ~(|count35)) begin
    bpm_transmitting   <= 1'b1;
    count35            <= 6'b100011;
  end else if (count35 == 6'b11) begin
    bpm_transmitting   <= 1'b0;
    count35            <= count35 - 6'b1;
  end else if (|count35) begin
    count35            <= count35 - 6'b1;
  end
end
`endif

reg                             bpm_transmitting_ff1;
`ifdef GC__GFX_ISLAND_POWER_GATING__1
reg                             pgfsm_transmitting_ff1;
`endif
always @(posedge aon_Cpl_GFXCLK) begin
  bpm_transmitting_ff1   <= bpm_transmitting;
`ifdef GC__GFX_ISLAND_POWER_GATING__1
  pgfsm_transmitting_ff1 <= pgfsm_transmitting;
`endif
end

assign bpm_transmitting_pulse = bpm_transmitting & ~bpm_transmitting_ff1;
`ifdef GC__GFX_ISLAND_POWER_GATING__1
assign pgfsm_transmitting_pulse = pgfsm_transmitting & ~pgfsm_transmitting_ff1;
`endif


always @(posedge aon_Cpl_GFXCLK) begin
  if(io_reset) begin
    bpm_read_request <= 1'b0;
  end
  else if(bpm_read_cmd || bpm_avfs_broadcast_read_pulse) begin
    bpm_read_request <= 1'b1;
  end
  else if(bpm_transmitting) begin
    bpm_read_request <= 1'b0;
  end
end


assign bpm_read_cmd = SERDES_new_cmd && (SERDES_cmd[`BPM_ADDRESS] == BPM_blk_addr) && ~is_broadcast_addr && ~SERDES_cmd[`BPM_WRITE];

reg   [7:0]                     bpm_read_reg_addr;
always @(posedge aon_Cpl_GFXCLK) begin
  if(io_reset) begin
    bpm_read_reg_addr <= 8'h00;
  end
  else if (bpm_read_cmd || bpm_avfs_broadcast_read) begin

    bpm_read_reg_addr <= SERDES_cmd[`BPM_REG_ADDR];
  end
end


reg  [23:0]      bpm_avfs_cfg;
reg  [15:0]      bpm_avfs_ctl;
reg  [19:0]      bpm_avfs_mm_ctl;
reg  [31:0]      bpm_avfs_mm_wdata_h;
reg  [31:0]      bpm_avfs_mm_wdata_l;

wire [31:0]      bpm_avfs_mm_rdata_h;
wire [31:0]      bpm_avfs_mm_rdata_l;
wire             avfs_mm_cmd_done;

reg          bpm_dbg_en;
reg          dd_dbg_en;
reg          dummy_avfs_node_en;

reg  [31:0]  bpm_dbg_dbus;
wire [31:0]  bpm_dbus_final;
wire [31:0]  psm_didt_dbus;
wire [31:0]  psm_edc_dbus;

generate
if((enable_avfs==1) || (enable_dd==1) || (enable_dummy_avfs_node==1) || (enable_psm_edc_dd==1) || (enable_psm_didt_dd==1)) begin 
   assign bpm_read_data_dbg = (bpm_read_reg_addr == 8'b00000110) ? {29'h0, enable_dummy_avfs_node ? dummy_avfs_node_en : 1'b0, enable_dd ? dd_dbg_en : 1'b0, bpm_dbg_en} : bpm_read_data_tmp;
end else begin
   assign bpm_read_data_dbg = bpm_read_data_tmp;
end
endgenerate


reg         avfs_delay_vld;
reg         avfs_delay_vld_dly;
reg [12:0]  avfs_delay_cnt;

generate
if(enable_avfs == 1) 
  begin
    assign bpm_avfs_broadcast_read = SERDES_new_cmd && ~SERDES_cmd[`BPM_WRITE] && (SERDES_cmd[`BPM_REG_ADDR]>=8'b00010000) && (SERDES_cmd[`BPM_REG_ADDR]<8'b00010111) && is_broadcast_addr;

    always @(posedge aon_Cpl_GFXCLK) begin
      if(io_reset) begin
        avfs_delay_vld            <= 0;
      end else if(avfs_delay_cnt==bpm_avfs_cfg[19:8]) begin
        avfs_delay_vld            <= 0;
      end else if(bpm_avfs_broadcast_read) begin
        avfs_delay_vld            <= 1;
      end
    end

    always @(posedge aon_Cpl_GFXCLK) begin
      if(io_reset) begin
        avfs_delay_cnt            <= 0;
      end else if(bpm_avfs_broadcast_read) begin
        avfs_delay_cnt            <= 0;
      end else if(avfs_delay_vld) begin
        avfs_delay_cnt            <= avfs_delay_cnt + 1;
      end
    end

    always @(posedge aon_Cpl_GFXCLK) begin
      if(io_reset) begin
        avfs_delay_vld_dly       <= 0;
      end else begin
        avfs_delay_vld_dly       <= avfs_delay_vld;
      end
    end

    assign bpm_avfs_broadcast_read_pulse = (bpm_avfs_cfg[19:8]==0) ? bpm_avfs_broadcast_read : (~avfs_delay_vld && avfs_delay_vld_dly);

    assign bpm_read_data_avfs = 
                       (bpm_read_reg_addr == 8'b00010000) ? {8'h0, bpm_avfs_cfg} :
                       (bpm_read_reg_addr == 8'b00010001) ? {16'h0, bpm_avfs_ctl} :
                       (bpm_read_reg_addr == 8'b00010010) ? {12'h0, bpm_avfs_mm_ctl} :
                       (bpm_read_reg_addr == 8'b00010011) ? bpm_avfs_mm_wdata_l :
                       (bpm_read_reg_addr == 8'b00010100) ? bpm_avfs_mm_wdata_h :
                       (bpm_read_reg_addr == 8'b00010101) ? bpm_avfs_mm_rdata_l :
                       (bpm_read_reg_addr == 8'b00010110) ? bpm_avfs_mm_rdata_h : 
                       bpm_read_data_dbg;
  end
else 
  begin
    assign bpm_avfs_broadcast_read = 0;
    assign bpm_avfs_broadcast_read_pulse = 0;
    assign bpm_read_data_avfs = bpm_read_data_dbg;
  end
endgenerate

generate
if(enable_dd == 1) 
  begin
    assign bpm_read_data = 
                       (bpm_read_reg_addr == 8'b00100000) ? {2'h0, droopdetected, dd_dll_lock, 27'h0, DD_CTRL1} :
                       (bpm_read_reg_addr == 8'b00100001) ? DD_CTRL2 :
                       (bpm_read_reg_addr == 8'b00100010) ? {1'h0, DD_CTRL3} :
                       (bpm_read_reg_addr == 8'b00100011) ? {5'h0, DD_CTRL4} :
                       (bpm_read_reg_addr == 8'b00100100) ? {31'h0, DD_MASTER_RESET}:
                       (bpm_read_reg_addr == 8'b00100101) ? {31'h0, DD_STATIC_ENABLE} :
                       (bpm_read_reg_addr == 8'b00100110) ? {31'h0, DD_STRETCH_ENABLE} : 
                       bpm_read_data_avfs;
  end
else 
  begin
    assign bpm_read_data = bpm_read_data_avfs;
  end
endgenerate

assign bpm_read_data_tmp = (bpm_read_reg_addr == 8'b00000000) ? {31'h0, cgmempg_en} :
                       (bpm_read_reg_addr == 8'b00000001) ? {31'h0, cgmempg_on} :
                       (bpm_read_reg_addr == 8'b00000010) ? {30'h0, stop_clock, cgcg_override} :
                       (bpm_read_reg_addr == 8'b00000011) ? {31'h0, TTOP_MGCG_override} :
                       (bpm_read_reg_addr == 8'b00000100) ? {31'h0, TTOP_FGCG_override} :
                       (bpm_read_reg_addr == 8'b00000101) ? {31'h0, mgmempg_override} :
                       (bpm_read_reg_addr == 8'b00000111) ? 32'h0 :
                       (bpm_read_reg_addr == 8'b00001000) ? {22'h0, groupcast_cntl} :

`ifdef GC__GFX_PIPELINE_CLOCK_GATING__1
                       (bpm_read_reg_addr == 8'b00001001) ? {30'h0, cg3d_rep_cntl_override, cg3d_blk_cntl_override} :
`endif
                       (bpm_read_reg_addr == 8'b00001010) ? {31'h0, rm_cntl} :
                       (bpm_read_reg_addr == 8'b00001011) ? {31'h0, bpm_blk_hard_resetb} :

`ifdef GC__GFX_PIPELINE_CLOCK_GATING__1
                       (bpm_read_reg_addr == 8'b00001111) ? {29'h0, cgcg_en, cgls_ctrl, BPM_gfx3d} :   // bpm dbg
`else
                       (bpm_read_reg_addr == 8'b00001111) ? {29'h0, cgcg_en, cgls_ctrl, 1'h0} :   // bpm dbg
`endif
                       (bpm_read_reg_addr == 8'b00110000) ? psm_didt_dd_cntl0_out :
                       (bpm_read_reg_addr == 8'b00110001) ? psm_didt_dd_cntl1_out :
                       (bpm_read_reg_addr == 8'b00110010) ? psm_didt_dd_stat :
                       (bpm_read_reg_addr == 8'b01000000) ? psm_edc_dd_cntl_out :
                       (bpm_read_reg_addr == 8'b01000010) ? psm_edc_dd_stat :
                       32'h0;


//wire groupcast_en = groupcast_cntl[9];
wire [`GFX_PGFSM_GROUP_ID_LO] group_id = {1'b0,groupcast_cntl[2:0]};
wire is_3DCAST         = (SERDES_cmd[`BPM_ADDRESS] == 13'h1FC0) && BPM_gfx3d;
wire is_non3DCAST      = (SERDES_cmd[`BPM_ADDRESS] == 13'h1FC8) && ~BPM_gfx3d;
wire is_AWGPCAST       = (SERDES_cmd[`BPM_ADDRESS] == 13'h1FC1) && (BPM_blk_addr[12:11] == 2'b10);
wire is_SA0CAST        = (SERDES_cmd[`BPM_ADDRESS] == 13'h1FC4) && ~BPM_blk_addr[10] && |BPM_blk_addr[9:5];
wire is_SA1CAST        = (SERDES_cmd[`BPM_ADDRESS] == 13'h1FC5) && BPM_blk_addr[10] && |BPM_blk_addr[9:5];
wire is_AGL2CAST       = (SERDES_cmd[`BPM_ADDRESS] == 13'h1FC6) && ({BPM_blk_addr[12:11], BPM_blk_addr[9:6]} == 6'b000010);
wire is_ARBCAST        = (SERDES_cmd[`BPM_ADDRESS] == 13'h1FC7) && ({BPM_blk_addr[12:11], BPM_blk_addr[9:7]} == 5'b01001);
wire is_CHAINCAST      = SERDES_cmd[`BPM_BROADCAST] && (SERDES_cmd[`BPM_CHAIN_ID] == BPM_blk_addr[12:5]);
wire is_FULLCAST       = SERDES_cmd[`BPM_ADDRESS] == 13'h1FFF;
assign is_broadcast_addr = is_3DCAST | is_non3DCAST | is_AWGPCAST | is_SA0CAST | is_SA1CAST | is_AGL2CAST | is_ARBCAST | is_CHAINCAST | is_FULLCAST;

assign BPM_select = SERDES_new_cmd && (is_broadcast_addr | (SERDES_cmd[`BPM_ADDRESS] == BPM_blk_addr));

assign SERDES_p1_new_cmd = 1'b0;

   //--------------------------------------------------------------------------------
   // PGFSM core
   //--------------------------------------------------------------------------------
generate
  if(enable_pgfsm == 1)
    begin : pgfsm_core

`ifdef GC__GFX_ISLAND_POWER_GATING__1


wire P1_pgfsm_disable = ~pg_P1_strap;
wire P2_pgfsm_disable = ~pg_P2_strap;

   gfx_pgfsm_core #(.autotransmit_default_value(0)) pgfsm_core_P1
     (
      // Inputs
      .TEST_CLK_EN                       (TEST_CLK_EN),
      .PGFSM_sel                         (SERDES_p1_new_cmd),
      .PGFSM_disable_strap               (P1_pgfsm_disable),

      .PGFSM_cmd                         (SERDES_cmd),
      .PGFSM_wdata                       (SERDES_wdata),
      .PGFSM_new_cmd                     (SERDES_p1_new_cmd),
      .is_bcast_addr                     (is_bcast_addr),

      // Clocks/reset
      .PGFSM_tile_ref_clk                (Cpl_REFCLK),
      .PGFSM_tile_sclk                   (aon_Cpl_GFXCLK),
`ifdef GC__DLDO_POWER_GATING__1
      .PGFSM_tile_hard_resetb            (RSMU_GFX_hard_resetb_in),
      .PGFSM_tile_cold_resetb            (RSMU_GFX_cold_resetb_in),
`else
      .PGFSM_tile_hard_resetb            (RSMU_GFX_hard_resetb),
      .PGFSM_tile_cold_resetb            (RSMU_GFX_cold_resetb),
`endif

      // Address for PGFSM in daisy chain
      .PGFSM_tile_local_address          (BPM_blk_addr_ff),

      //SERDES input
      .PGFSM_SD_next_daisy_status        (p1_daisy_status_in),

      .PGFSM_in_MEM_PD_power_ctrl        (mem_power_ctrl),
      .PGFSM_CGLS_override               (1'b0),  //tied off for non GFX tiles

      //DFT
 `ifdef GC__GNB_DFT
      .PGFSM_DFT_tdr_tdi                 (PGFSM_TARG_tdr_tdi),
      .PGFSM_DFT_tdr_tresetb             (PGFSM_TARG_tdr_tresetb),
      .PGFSM_DFT_tdr_shift               (PGFSM_TARG_tdr_shift),
      .PGFSM_DFT_tdr_capture             (PGFSM_TARG_tdr_capture),
      .PGFSM_DFT_tdr_update              (PGFSM_TARG_tdr_update),
      .PGFSM_DFT_tdr_en_rw               (PGFSM_tdr_en_rw),
      .PGFSM_DFT_tdr_en_ro               (PGFSM_tdr_en_ro),
      .PGFSM_DFT_tdr_tck                 (TST_TARG_miscpfh_tclk),
 `else
      .PGFSM_DFT_tdr_tdi                 (DFT_TARG_si),
      .PGFSM_DFT_tdr_tresetb             (DFT_TARG_trstb),
      .PGFSM_DFT_tdr_shift               (DFT_TARG_shift),
      .PGFSM_DFT_tdr_capture             (DFT_TARG_capture),
      .PGFSM_DFT_tdr_update              (DFT_TARG_update),
      .PGFSM_DFT_tdr_en_rw               (DFT_BPM_0_rw_dec),
      .PGFSM_DFT_tdr_en_ro               (DFT_BPM_0_ro_dec),
      .PGFSM_DFT_tdr_tck                 (DFT_TARG_tclk),
`endif
      // Outputs

      .PGFSM_SD_prev_daisy_status        (p1_daisy_status_out),

      .PGFSM_cmd_done                    (),
      .PGFSM_iso_clock_1                 (1'b0),
      .PGFSM_iso_clock_2                 (1'b0),
      .PGFSM_iso_clampn_clock_1     (),
      .PGFSM_iso_clampn_clock_2     (),
      //Mother, Daughter, reset, isolation outputs
      .PGFSM_PD_mother_sleep             (PGFSM_P1_mother_sleep),
      .PGFSM_PD_daughter_sleep           (PGFSM_P1_daughter_sleep),
      .PGFSM_PD_resetb                   (PGFSM_P1_resetb),
      .PGFSM_PD_iso_clampn               (PGFSM_P1_iso_clampn),
      .PGFSM_PD_scan_iso_clampn          (PGFSM_P1_scan_iso_clampn),
      .PGFSM_MEM_PD_power_ctrl           (PGFSM_P1_pwr_ctrl_out),

      //DFT
 `ifdef GC__GNB_DFT
      .PGFSM_DFT_tdr_tdo_rw              (PGFSM_tdr_tdo_rw),
      .PGFSM_DFT_tdr_tdo_ro              (PGFSM_tdr_tdo_ro),
 `else
      .PGFSM_DFT_tdr_tdo_rw              (DFT_BPM_0_rw_so),
      .PGFSM_DFT_tdr_tdo_ro              (DFT_BPM_0_ro_so),
 `endif
      .PGFSM_PFH_PD_resetb               (DFT_PD_P1_resetb)
      );

    assign TTOP_P1_mother_sleep       = pg_P1_strap ? PGFSM_P1_mother_sleep : 1'b0;
    assign TTOP_P1_daughter_sleep     = pg_P1_strap ? PGFSM_P1_daughter_sleep : 1'b0;
    assign TTOP_P1_iso_clampn         = pg_P1_strap ? PGFSM_P1_iso_clampn : 1'b1;
    assign TTOP_P1_scan_iso_clampn    = pg_P1_strap ? PGFSM_P1_scan_iso_clampn : 1'b1;

  `ifdef GC__STATIC_PERCU_POWER_GATING__1
   wire    SERDES_p2_new_cmd;

   assign  SERDES_p2_new_cmd = 1'b0;


gfx_pgfsm_core #(.autotransmit_default_value(0)) pgfsm_core_P2
     (
      // Inputs
      .TEST_CLK_EN                       (TEST_CLK_EN),
      .PGFSM_sel                         (SERDES_p2_new_cmd),
      .PGFSM_disable_strap               (P2_pgfsm_disable),

      .PGFSM_cmd                         (SERDES_cmd),
      .PGFSM_wdata                       (SERDES_wdata),
      .PGFSM_new_cmd                     (SERDES_p2_new_cmd),
      .is_bcast_addr                     (is_bcast_addr),

      // Clocks/reset
      .PGFSM_tile_ref_clk                (Cpl_REFCLK),
      .PGFSM_tile_sclk                   (aon_Cpl_GFXCLK),
`ifdef GC__DLDO_POWER_GATING__1
      .PGFSM_tile_hard_resetb            (RSMU_GFX_hard_resetb_in),
      .PGFSM_tile_cold_resetb            (RSMU_GFX_cold_resetb_in),
`else
      .PGFSM_tile_hard_resetb            (RSMU_GFX_hard_resetb),
      .PGFSM_tile_cold_resetb            (RSMU_GFX_cold_resetb),
`endif
      // Address for PGFSM in daisy chain
      .PGFSM_tile_local_address          (BPM_blk_addr_ff),

      //SERDES input
      .PGFSM_SD_next_daisy_status        (p2_daisy_status_in),

      .PGFSM_in_MEM_PD_power_ctrl        (mem_power_ctrl),
      .PGFSM_CGLS_override               (1'b0),  //tied off for non GFX tiles

      //DFT
    `ifdef GC__GNB_DFT
      .PGFSM_DFT_tdr_tdi                 (PGFSM_TARG_tdr_tdi),
      .PGFSM_DFT_tdr_tresetb             (PGFSM_TARG_tdr_tresetb),
      .PGFSM_DFT_tdr_shift               (PGFSM_TARG_tdr_shift),
      .PGFSM_DFT_tdr_capture             (PGFSM_TARG_tdr_capture),
      .PGFSM_DFT_tdr_update              (PGFSM_TARG_tdr_update),
      .PGFSM_DFT_tdr_en_rw               (PGFSM2_tdr_en_rw),
      .PGFSM_DFT_tdr_en_ro               (PGFSM2_tdr_en_ro),
      .PGFSM_DFT_tdr_tck                 (TST_TARG_miscpfh_tclk),
    `else
      .PGFSM_DFT_tdr_tdi                 (DFT_TARG_si),
      .PGFSM_DFT_tdr_tresetb             (DFT_TARG_trstb),
      .PGFSM_DFT_tdr_shift               (DFT_TARG_shift),
      .PGFSM_DFT_tdr_capture             (DFT_TARG_capture),
      .PGFSM_DFT_tdr_update              (DFT_TARG_update),
      .PGFSM_DFT_tdr_en_rw               (DFT_BPM_1_rw_dec),
      .PGFSM_DFT_tdr_en_ro               (DFT_BPM_1_ro_dec),
      .PGFSM_DFT_tdr_tck                 (DFT_TARG_tclk),
    `endif
      // Outputs

      .PGFSM_SD_prev_daisy_status        (p2_daisy_status_out),

      .PGFSM_cmd_done                    (),
      .PGFSM_iso_clock_1                 (1'b0),
      .PGFSM_iso_clock_2                 (1'b0),
      .PGFSM_iso_clampn_clock_1     (),
      .PGFSM_iso_clampn_clock_2     (),
      //Mother, Daughter, reset, isolation outputs
      .PGFSM_PD_mother_sleep             (PGFSM_P2_mother_sleep),
      .PGFSM_PD_daughter_sleep           (PGFSM_P2_daughter_sleep),
      .PGFSM_PD_resetb                   (PGFSM_P2_resetb),
      .PGFSM_PD_iso_clampn               (PGFSM_P2_iso_clampn),
      .PGFSM_PD_scan_iso_clampn          (PGFSM_P2_scan_iso_clampn),
      .PGFSM_MEM_PD_power_ctrl           (PGFSM_P2_pwr_ctrl_out),

      //DFT
    `ifdef GC__GNB_DFT
      .PGFSM_DFT_tdr_tdo_rw              (PGFSM2_tdr_tdo_rw),
      .PGFSM_DFT_tdr_tdo_ro              (PGFSM2_tdr_tdo_ro),
    `else
      .PGFSM_DFT_tdr_tdo_rw              (DFT_BPM_1_rw_so),
      .PGFSM_DFT_tdr_tdo_ro              (DFT_BPM_1_ro_so),
    `endif
      .PGFSM_PFH_PD_resetb               (DFT_PD_P2_resetb)
      );

  `else // !`ifdef GC__STATIC_PERCU_POWER_GATING__1

      assign p2_daisy_status_out = 1'b0;
      assign PGFSM_P2_mother_sleep = PGFSM_P1_mother_sleep;
      assign PGFSM_P2_daughter_sleep = PGFSM_P1_daughter_sleep;
      assign PGFSM_P2_resetb = PGFSM_P1_resetb;
      assign PGFSM_P2_iso_clampn = PGFSM_P1_iso_clampn;
      assign PGFSM_P2_scan_iso_clampn = PGFSM_P1_scan_iso_clampn;
      assign PGFSM_P2_pwr_ctrl_out = PGFSM_P1_pwr_ctrl_out;

    `ifdef GC__GNB_DFT
      assign PGFSM2_tdr_tdo_rw = 1'b0;
      assign PGFSM2_tdr_tdo_ro = 1'b0;
    `else
      assign DFT_BPM_1_rw_so = 1'b0;
      assign DFT_BPM_1_ro_so = 1'b0;
    `endif

      assign DFT_PD_P2_resetb = DFT_PD_P1_resetb;

  `endif // !`ifdef GC__STATIC_PERCU_POWER_GATING__1

  `ifdef GC__STATIC_PERCU_POWER_GATING__1
      assign TTOP_P2_mother_sleep    = pg_P2_strap ? PGFSM_P2_mother_sleep : 1'b0;
      assign TTOP_P2_daughter_sleep  = pg_P2_strap ? PGFSM_P2_daughter_sleep : 1'b0;
      assign TTOP_P2_iso_clampn      = pg_P2_strap ? PGFSM_P2_iso_clampn : 1'b1;
      assign TTOP_P2_scan_iso_clampn = pg_P2_strap ? PGFSM_P2_scan_iso_clampn : 1'b1;
  `else // !`ifdef GC__STATIC_PERCU_POWER_GATING__1
      assign TTOP_P2_mother_sleep    = PGFSM_P2_mother_sleep;
      assign TTOP_P2_daughter_sleep  = PGFSM_P2_daughter_sleep;
      assign TTOP_P2_iso_clampn      = PGFSM_P2_iso_clampn;
      assign TTOP_P2_scan_iso_clampn = PGFSM_P2_scan_iso_clampn;
  `endif // !`ifdef GC__STATIC_PERCU_POWER_GATING__1


`else // !`ifdef GC__GFX_ISLAND_POWER_GATING__1

assign PGFSM_P1_resetb         = 1'b1;
assign PGFSM_P1_pwr_ctrl_out   = mem_power_ctrl;
assign p1_daisy_status_out     = 1'b0;

assign PGFSM_P2_resetb         = 1'b1;
assign PGFSM_P2_pwr_ctrl_out   = mem_power_ctrl;
assign p2_daisy_status_out     = 1'b0;

`endif // !`ifdef GC__GFX_ISLAND_POWER_GATING__1

    end
    else
    begin : no_pgfsm_core

    assign PGFSM_P1_resetb            = 1'b1;
    assign PGFSM_P2_resetb            = 1'b1;
    assign TTOP_P1_mother_sleep       = 1'b0;
    assign TTOP_P1_daughter_sleep     = 1'b0;
    assign TTOP_P1_iso_clampn         = 1'b1;
    assign TTOP_P1_scan_iso_clampn    = 1'b1;
    assign p1_daisy_status_out        = 1'b0;
    assign PGFSM_P1_pwr_ctrl_out      = mem_power_ctrl;
    assign PGFSM_P2_pwr_ctrl_out      = mem_power_ctrl;
    assign TTOP_P2_mother_sleep       = 1'b0;
    assign TTOP_P2_daughter_sleep     = 1'b0;
    assign TTOP_P2_iso_clampn         = 1'b1;
    assign TTOP_P2_scan_iso_clampn    = 1'b1;
    assign p2_daisy_status_out        = 1'b0;

    assign DFT_PD_P1_resetb           = 1'b1;
    assign DFT_PD_P2_resetb           = 1'b1;

`ifdef GC__GFX_ISLAND_POWER_GATING__1
 `ifdef GC__GNB_DFT
    assign PGFSM_tdr_tdo_rw           = 1'b0;
    assign PGFSM_tdr_tdo_ro           = 1'b0;
    assign PGFSM2_tdr_tdo_rw          = 1'b0;
    assign PGFSM2_tdr_tdo_ro          = 1'b0;
 `else
    assign DFT_BPM_0_rw_so            = 1'b0;
    assign DFT_BPM_0_ro_so            = 1'b0;
    assign DFT_BPM_1_rw_so            = 1'b0;
    assign DFT_BPM_1_ro_so            = 1'b0;
 `endif
`endif
    end
endgenerate


//--------------------------------------------------------------------------------
// CGLS control, CGCG, MGCG overrides
//--------------------------------------------------------------------------------

generate
if(enable_avfs == 1) begin 
  always @(posedge aon_Cpl_GFXCLK) 
  begin
    if (~hard_resetb) begin
      bpm_avfs_cfg          <= 24'h0;
      bpm_avfs_ctl          <= 16'h0;
      bpm_avfs_mm_ctl       <= 20'h0;
      bpm_avfs_mm_wdata_h   <= 32'h0;
      bpm_avfs_mm_wdata_l   <= 32'h0;
    end else begin
      if (BPM_select & SERDES_cmd[`BPM_WRITE]) begin
        case (SERDES_cmd[`BPM_REG_ADDR])
          8'b00010000:  
            bpm_avfs_cfg          <= SERDES_wdata[23:0];
          8'b00010001:  
            bpm_avfs_ctl          <= SERDES_wdata[15:0];
          8'b00010010:  
            bpm_avfs_mm_ctl       <= SERDES_wdata[19:0];
          8'b00010011:  
            bpm_avfs_mm_wdata_l   <= SERDES_wdata[31:0];
          8'b00010100:  
            bpm_avfs_mm_wdata_h   <= SERDES_wdata[31:0];
          default:;
        endcase
      end else if(avfs_mm_cmd_done) begin
        bpm_avfs_mm_ctl[7]       <= 0;
      end
    end
  end
end
endgenerate

generate
  if(enable_dd == 1) begin
    always @(posedge aon_Cpl_GFXCLK) begin
      if (~hard_resetb) begin
        DD_CTRL1              <= 1'h0;
        DD_CTRL2              <= 32'h0;
        DD_CTRL3              <= 31'h0;
        DD_CTRL4              <= 27'ha150;//24'h00142a;
        DD_MASTER_RESET       <= 1'b0;
        DD_STATIC_ENABLE      <= 1'b0;
        DD_STRETCH_ENABLE     <= 1'b0;
      end else begin
        if (BPM_select & SERDES_cmd[`BPM_WRITE]) begin
          case (SERDES_cmd[`BPM_REG_ADDR])
          // DD_CTRL1
            8'b00100000:  
            DD_CTRL1              <= SERDES_wdata[0];
          // DD_CTRL2
            8'b00100001:  
            DD_CTRL2              <= SERDES_wdata[31:0];
          // DD_CTRL3
            8'b00100010:  
            DD_CTRL3              <= SERDES_wdata[30:0];
          // DD_CTRL4
            8'b00100011:  
            DD_CTRL4              <= SERDES_wdata[26:0];
          // DD_MASTER_RESET
            8'b00100100:  
            DD_MASTER_RESET       <= SERDES_wdata[0];
          // DD_STATIC_ENABLE
            8'b00100101:  
            DD_STATIC_ENABLE      <= SERDES_wdata[0];
          // DD_STRETCH_ENABLE
            8'b00100110:  
            DD_STRETCH_ENABLE     <= SERDES_wdata[0];
            default:;
          endcase
        end
      end
    end
  end
endgenerate

always @(posedge aon_Cpl_GFXCLK) begin
  if (~hard_resetb) begin
    cgmempg_en           <= 1'b0;
    cgmempg_on           <= 1'b0;
    cgls_ctrl            <= 1'b0;
    cgcg_override        <= 1'b1;
    stop_clock           <= 1'b0;
    rTTOP_FGCG_override  <= 1'b1;
    rTTOP_MGCG_override  <= 1'b1;
`ifdef GC__GFX_PIPELINE_CLOCK_GATING__1
    cg3d_blk_cntl_override <= 1'b0;
    cg3d_rep_cntl_override <= 1'b0;
`endif
    mgmempg_override       <= 1'b1;
    groupcast_cntl         <= 10'h000;
    rm_cntl                <= 1'b1;
    bpm_blk_hard_resetb    <= 1'b1;
  end else begin
    if (BPM_select & SERDES_cmd[`BPM_WRITE]) begin
      case (SERDES_cmd[`BPM_REG_ADDR])
        8'b00000000:
        //CGLS
          cgmempg_en           <= SERDES_wdata[0];
        8'b00000001:
          cgmempg_on           <= SERDES_wdata[0];
        //CGCG
        8'b00000010:  
          {stop_clock, cgcg_override}  <= SERDES_wdata[1:0];
        //MGCG
        8'b00000011:  
          rTTOP_MGCG_override  <= SERDES_wdata[0];
        //MGCG
        8'b00000100:  
          rTTOP_FGCG_override  <= SERDES_wdata[0];
        //MGLS
        8'b00000101:  
          mgmempg_override     <= SERDES_wdata[0];
        //GROUP_ID
        8'b00001000:  
          groupcast_cntl       <= SERDES_wdata[9:0];
      `ifdef GC__GFX_PIPELINE_CLOCK_GATING__1
        //3DCG
        8'b00001001:
          {cg3d_rep_cntl_override, cg3d_blk_cntl_override} <=SERDES_wdata[1:0];
      `endif
        // rm_cntl
        8'b00001010:  
          rm_cntl               <= SERDES_wdata[0];
        // bpm_blk_hard_resetb
        8'b00001011:  
          bpm_blk_hard_resetb   <= SERDES_wdata[0];
        // Add this to save RLC init time
        8'h0F:
          {cg3d_rep_cntl_override, rTTOP_FGCG_override, cg3d_blk_cntl_override, cgmempg_en
          , cgcg_override, rTTOP_MGCG_override} <= SERDES_wdata[5:0];

        default:;
      endcase
  end
    cgls_ctrl            <= (cgmempg_en & cgmempg_on) && (!TARG_BPM_cgcgls_override); 
  end
end

generate
  if(compiled_rm_through_bpm == 1) begin : dyn_rm_for_compiled_mem
bpm_dyn_rm udyn_rm_v (
  .oRM                 (rm_synced_v        ),
  .oOverride_mgls      (disable_mgls_v     ),
  .oOverride_mgcg      (disable_mgcg_v     ),

  .iRM            (mem_power_ctrl[`GPU__MEM__MEM_POWER_CTRL_RM_END:`GPU__MEM__MEM_POWER_CTRL_RM_START]),
  .BPM_addr_lo    (BPM_blk_addr[3:0]),
  .rm_cntl        (rm_cntl          ),
  .rm_valid       (RLC_GFX_rm_valid ),
  .clk            (aon_Cpl_GFXCLK       ),
`ifdef GC__DLDO_POWER_GATING__1
  .resetn         (hard_resetb_pre  )
`else
  .resetn         (hard_resetb      )
`endif
  );
  end
  else begin : no_compiled_rm_through_bpm
    assign rm_synced_v       = 'h0;
    assign disable_mgls_v    = 'h0;
    assign disable_mgcg_v    = 'h0;
//    assign disable_cgcg_cgls_v = 'h0;
  end
endgenerate

generate
  if(custom_rm_through_bpm == 1) begin : dyn_rm_for_custom_mem
bpm_dyn_rm #(.RM_BIT_WIDTH(7)) udyn_rm_c (
  .oRM                 (rm_synced_c        ),
  .oOverride_mgls      (disable_mgls_c     ),
  .oOverride_mgcg      (disable_mgcg_c     ),

  .iRM            ({iCUSTOM_MEM_RM, iCUSTOM_MEM_RME}),
  .BPM_addr_lo    (BPM_blk_addr[3:0]),
  .rm_cntl        (rm_cntl          ),
  .rm_valid       (RLC_GFX_rm_valid ),
  .clk            (aon_Cpl_GFXCLK       ),
`ifdef GC__DLDO_POWER_GATING__1
  .resetn         (hard_resetb_pre  )
`else
  .resetn         (hard_resetb      )
`endif
  );
  end
  else begin : no_custom_rm_through_bpm
    assign rm_synced_c       = 'h0;
    assign disable_mgls_c    = 'h0;
    assign disable_mgcg_c    = 'h0;
//    assign disable_cgcg_cgls_c = 'h0;
  end
endgenerate

  assign  oCUSTOM_MEM_RM  = rm_synced_c[6:1];
  assign  oCUSTOM_MEM_RME = rm_synced_c[0];

  assign  disable_mgls      = disable_mgls_v | disable_mgls_c;
  assign  disable_mgcg      = disable_mgcg_v | disable_mgcg_c;


assign TTOP_MGLS_override = mgmempg_override | disable_mgls;
assign TTOP_MGCG_override = rTTOP_MGCG_override | disable_mgcg;
assign TTOP_FGCG_override = rTTOP_FGCG_override;

assign  oRM_VALID         = rm_didt_ff;
always @(posedge aon_Cpl_GFXCLK) begin
  rm_didt_ff <= disable_mgcg | rTTOP_FGCG_override ;
end


assign bpm_dbus_byte0 = {TTOP_P2_combined_resetb_0,                                      //bit7
                         TTOP_P2_hard_resetb,                                            //bit6
                         TTOP_P1_combined_resetb_0,                                      //bit5    
                         TTOP_P1_hard_resetb,                                            //bit4
                         TTOP_MGLS_override,                                             //bit3 
                         TTOP_MGCG_override,                                             //bit2 
                         cgcg_override,                                                  //bit1 
                         RLC_TARG_gfxen_ff};                                             //bit0 

`ifdef GC__GFX_ISLAND_POWER_GATING__1
assign bpm_dbus_byte1 = {p2_resetb,                                                      //bit7
                         TTOP_P2_iso_clampn,                                             //bit6
                         TTOP_P2_daughter_sleep,                                         //bit5
                         TTOP_P2_mother_sleep,                                           //bit4
                         p1_resetb,                                                      //bit3
                         TTOP_P1_iso_clampn,                                             //bit2
                         TTOP_P1_daughter_sleep,                                         //bit1
                         TTOP_P1_mother_sleep};                                          //bit0
`else
assign bpm_dbus_byte1 = 8'b0;
`endif

assign bpm_dbus_byte2 = {1'b0,                                                           //bit7
                         1'b0,                                                           //bit6
                         1'b0,                                                           //bit5
                         1'b0,                                                           //bit4
                         TARG_BPM_cgcgls_override,                                       //bit3
                         TTOP_P1_mem_power_ctrl[`MEM__MEM_POWER_CTRL_SD],                //bit2
                         TTOP_P1_mem_power_ctrl[`MEM__MEM_POWER_CTRL_DS],                //bit1
                         TTOP_P1_mem_power_ctrl[`MEM__MEM_POWER_CTRL_LS]};               //bit0
                        
assign bpm_dbus_byte3 = {bpm_blk_hard_resetb,                                            //bit7
                         disable_mgls,                                                   //bit6
                         disable_mgcg,                                                   //bit5
                         RLC_GFX_rm_valid,                                               //bit4
                         rm_cntl,                                                        //bit3
                         TTOP_P2_mem_power_ctrl[`MEM__MEM_POWER_CTRL_SD],                //bit2
                         TTOP_P2_mem_power_ctrl[`MEM__MEM_POWER_CTRL_DS],                //bit1
                         TTOP_P2_mem_power_ctrl[`MEM__MEM_POWER_CTRL_LS]};               //bit0
                        
assign bpm_dbus = {bpm_dbus_byte3,bpm_dbus_byte2,bpm_dbus_byte1,bpm_dbus_byte0};

wire  [15:0]  bpm_avfs_dbus; //need connect to AVFS module

wire  [16:0]  DbgBusIn = {DbgBusIn_en,DbgBusIn_dbg_bus};

`ifdef SIMULATION
reg avfs_WRCK_en;

initial begin
  if($test$plusargs("BPM_avfs_WRCK_enable"))
    avfs_WRCK_en = 1;
  else
    avfs_WRCK_en = 0;
end
`else
`endif

`ifdef SIMULATION
wire avfs_WRCK;
assign avfs_WRCK = avfs_WRCK_en ? aon_Cpl_GFXCLK : WRCK;
`else
`endif


generate
if(enable_avfs == 1) begin
`include "mm_global.d"
  bpm_avfs_wrap ubpm_avfs_wrap(
  // Inputs to AVFS
  .PowerOk_VDD2     (bpm_avfs_ctl[6]),
  .ScanShiftEn      (1'h0),  //??
  .CGater01_CG0     (bpm_avfs_ctl[4]),
  .Reset            (bpm_avfs_ctl[5]),
  .PwrMonEnX        (1'b1),  //1'h0)   // mux between SPMI (0) and p1500 path(1)
`ifdef SIMULATION
  .WRCK             (avfs_WRCK),
`else
  .WRCK             (WRCK),
`endif
  .WSI              (WSI),
  .ShiftWR          (ShiftWR),
  .UpdateWR         (UpdateWR),
  .CaptureWR        (CaptureWR),
  .WDREn            (WDREn),
  .WRSTN            (WRSTN),
  .MeasureWR        (1'h0), //SPMI only
  .CCLK             (aon_Cpl_GFXCLK),
  .TstCcgEn         (1'h0),
  .Cgen_UnitActive  (bpm_avfs_ctl[3]),
  .RCLK_Free_AR     (aon_Cpl_GFXCLK),  //??  // ZAK uses free-running clock for coarse gater flops
  .LBistEn          (scan_mode),
  .PsmEn            (bpm_avfs_ctl[0]),
  .PsmAvfs_Gater    (bpm_avfs_ctl[1]),
  .Avfs_ScanMode    (scan_mode),
  .SkipPhase        (1'h0),  // Not POR
  .Isolate          (bpm_avfs_ctl[2]),
  .DbgBusSel        (bpm_avfs_ctl[10:8]),
  .DbgBusIn         (DbgBusIn),
  .avfs_dbg_en      (bpm_avfs_ctl[7]),
  // Inputs to FSM
  .bpm_avfs_mm_clk      (aon_Cpl_GFXCLK),
  .bpm_avfs_mm_reset    (~hard_resetb),
  .bpm_avfs_cfg         (bpm_avfs_cfg),
  .bpm_avfs_mm_ctl      (bpm_avfs_mm_ctl),
  .bpm_avfs_mm_wdata_h  (bpm_avfs_mm_wdata_h),
  .bpm_avfs_mm_wdata_l  (bpm_avfs_mm_wdata_l),

  // Outputs from FSM
  .bpm_avfs_mm_rdata_h  (bpm_avfs_mm_rdata_h),
  .bpm_avfs_mm_rdata_l  (bpm_avfs_mm_rdata_l),
  .avfs_mm_cmd_done     (avfs_mm_cmd_done),
  // Outputs from AVFS
`ifdef GFX_DROOP_DETECT
  .PsmOut           (psm_counter),
`endif
  .Psm_Alarm_Flag   (),
  .DbgBus_enable    (DbgBusOut_en),
  .DbgBus           (bpm_avfs_dbus),
  .WRO              (WRO)
  );

end
endgenerate

wire  tmon_s_rd_pr;
wire  tmon_s_rd_pw;
wire  tmon_s_ref_pr;
wire  tmon_s_ref_pw;

generate
if(num_rdih == 2) begin
  mardih_n7_ns urdih1 (
    .TMON_S_RD_PR     (tmon_s_bd1_rd_pr   )
    , .TMON_S_RD_PW   (tmon_s_bd1_rd_pw   )
    , .TMON_S_REF_PR  (tmon_s_bd1_ref_pr  )
    , .TMON_S_REF_PW  (tmon_s_bd1_ref_pw  )
    , .TMON_S_SELECT  (tmon_s1_select  )
    , .VDD18_TMON_S   (VDDAN_18_TS)
    , .VDD_CORE       (VDDCR_SOC)
    , .VSS            ()
  );
end

if(num_rdih > 0) begin
  mardih_n7_ns urdih0 (
    .TMON_S_RD_PR     (tmon_s_bd0_rd_pr   )
    , .TMON_S_RD_PW   (tmon_s_bd0_rd_pw   )
    , .TMON_S_REF_PR  (tmon_s_bd0_ref_pr  )
    , .TMON_S_REF_PW  (tmon_s_bd0_ref_pw  )
    , .TMON_S_SELECT  (tmon_s0_select  )
    , .VDD18_TMON_S   (VDDAN_18_TS)
    , .VDD_CORE       (VDDCR_SOC)
    , .VSS            ()
  );
end

if(num_rdiv > 0) begin
  mardiv_n7_ns urdiv0 (
    .TMON_S_RD_PR     (tmon_s_bd0_rd_pr   )
    , .TMON_S_RD_PW   (tmon_s_bd0_rd_pw   )
    , .TMON_S_REF_PR  (tmon_s_bd0_ref_pr  )
    , .TMON_S_REF_PW  (tmon_s_bd0_ref_pw  )
    , .TMON_S_SELECT  (tmon_s0_select  )
    , .VDD18_TMON_S   (VDDAN_18_TS)
    , .VDD_CORE       (VDDCR_SOC)
    , .VSS            ()
  );
end
endgenerate

reg   [31:0]  bpm_dd_dbus;
wire  [7:0]   phase_lead_lag;
wire  [3:0]   dd_debug_c0;
wire  [3:0]   dd_debug_c19;
wire  [3:0]   dd_debug_c29;
wire  [3:0]   dd_debug_c39;

generate
  if (enable_dd == 1) begin : droopdetect
madroopdetect udd (
  .VDD                  (),
  .VSS                  (),

  .ENABLE_DD            (DD_CTRL1),
  .STRETCH_ENABLE       (DD_STRETCH_ENABLE), //DD_CTRL1[1]),
  .staticEnable         (DD_STATIC_ENABLE),
  .masterReset          (DD_MASTER_RESET),
  .dfs_clk              (dfs_clk),
  .CONSERVATIVE_LOCK    (DD_CTRL4[23:22]), //[20:19]),
  .LOW_BW_CONFIG        (DD_CTRL4[21:15]), //[18:12]),
  .LOW_BW_CONFIG_UP     (DD_CTRL3[30:24]),
  .LOW_BW_SKEW_EN       (DD_CTRL4[26:24]), //[23:21]),
  .STRETCH_THRESH       (DD_CTRL4[14:12]), //[11:9]),
  .FSM_WAIT_CYCLES      (DD_CTRL4[7:4]),   //[4:1]),   // New: number of cycles to wait after each delay change
  .RESERVE              (DD_CTRL2[31:16]), //[29:14]), // [7:0]=not used
  .SAMPLE_SIZE          (DD_CTRL4[11:8]),  //[9:6]),   // New: Number of samples to take to determine Lead/Lag
  .NO_EXTRA_COARSE_STEP (DD_CTRL4[0]),     // New: If 1, FSM will increase delay by one coarse step going into FINE_DEC state.  Default = 1.  Default = 0.
  .FREQ_MODE            (DD_CTRL2[5:4]),   //[3:2]),  // Selects the frequency range supported
  .PD_SYNC_SEL          (DD_CTRL2[1:0]),   // Selects the level of synchronization from PD output
  .CHARZ_SEL_A          (DD_CTRL3[3:0]),
  .CHARZ_SEL_B          (DD_CTRL3[7:4]),
  .CHARZ_DIV_A          (DD_CTRL3[9:8]),
  .CHARZ_DIV_B          (DD_CTRL3[11:10]),
  .FSM_STEP_SIZE        (DD_CTRL3[13:12]),
  .FSM_CODE_MARGIN      (DD_CTRL3[15:14]),
  .CHARZ_DEBUG_MODE     (DD_CTRL3[16]),
  .FSM_DEBUG_SEL        (DD_CTRL3[18:17]),
  .FSM_OVERRIDE_EN      (DD_CTRL3[19]),
  .FSM_OVERRIDE_VAL     (DD_CTRL3[23:20]),

  .UNSTRETCH_THRESH     (DD_CTRL2[10:6]),  //[8:4]),  // Specifies trigger threshold when DFSSTRETCH=1. (FREQ_MODE=10, CPU only)
  .DFSSTRETCH           (DD_CTRL2[11]),    //[9]),  // 1 means dfs_clk is stretched    
  .DFSSTRETCH_AMOUNT    (DD_CTRL2[15:12]), //[13:10]),  // specifies stretch percentage seen on dfs_clk. (FREQ_MODE=10, CPU only)
  .CT_CpuPwrOkVDD       (1'b1),            // Isolation for the dfs_clk to be usd by CPU else connected to 1'b1 at higher level

  .charz_observ_a       (charz_observ_a),
  .charz_observ_b       (charz_observ_b),
  .droopdetected        (droopdetected),
  .dd_dll_lock          (dd_dll_lock),
  .phase_lead_lag       (phase_lead_lag),

  .debug_c0             (dd_debug_c0),
  .debug_c19            (dd_debug_c19),
  .debug_c29            (dd_debug_c29),
  .debug_c39            (dd_debug_c39)
);
  end
endgenerate

wire [31:0] dbus_bpm;
generate
if((enable_avfs==1) || (enable_dd==1) || (enable_dummy_avfs_node==1) || (enable_psm_edc_dd==1) || (enable_psm_didt_dd==1)) begin 

  always @(posedge aon_Cpl_GFXCLK) begin
    if(~hard_resetb) 
      bpm_dbg_en           <= 1'b0;
    else if(BPM_select & SERDES_cmd[`BPM_WRITE] & (SERDES_cmd[`BPM_REG_ADDR]==8'b00000110))
      bpm_dbg_en           <= SERDES_wdata[0];
  end

  assign dbus_bpm = bpm_dbus & {32{bpm_dbg_en}};
end else begin
  assign dbus_bpm = bpm_dbus;
end

endgenerate

wire [31:0] dbus_bpm_edc;
generate
if(enable_psm_edc_dd==1) begin 
  assign dbus_bpm_edc = dbus_bpm | psm_edc_dbus;
end else begin
  assign dbus_bpm_edc = dbus_bpm;
end

endgenerate

wire [31:0] dbus_bpm_edc_didt;
generate
if(enable_psm_didt_dd==1) begin 
  assign dbus_bpm_edc_didt = dbus_bpm_edc | psm_didt_dbus;
end else begin
  assign dbus_bpm_edc_didt = dbus_bpm_edc;
end

endgenerate

wire [31:0] dbus_bpm_edc_didt_dd;
generate
if(enable_dd==1) begin 

  always @(posedge aon_Cpl_GFXCLK) begin
    if(~hard_resetb)
      dd_dbg_en   <= 1'b0;
    else if(BPM_select & SERDES_cmd[`BPM_WRITE] & (SERDES_cmd[`BPM_REG_ADDR]==8'b00000110))
      dd_dbg_en   <= SERDES_wdata[1];
  end

  always @(posedge aon_Cpl_GFXCLK) begin
    if(~hard_resetb)   //??? or io_reset
      bpm_dd_dbus    <= 32'h0;
    else if(dd_dbg_en)
      bpm_dd_dbus    <= {4'h0,charz_observ_b,charz_observ_a,droopdetected,dd_dll_lock,phase_lead_lag,dd_debug_c39,dd_debug_c29,dd_debug_c19,dd_debug_c0}; //28 bits
  end

  assign dbus_bpm_edc_didt_dd = dbus_bpm_edc_didt | (bpm_dd_dbus & {32{dd_dbg_en}});
end else begin
  assign dbus_bpm_edc_didt_dd = dbus_bpm_edc_didt;
end

endgenerate

wire [15:0] dbus_bpm_edc_didt_dd_avfs;
generate
if(enable_avfs==1) begin 
  assign dbus_bpm_edc_didt_dd_avfs = dbus_bpm_edc_didt_dd[15:0] | bpm_avfs_dbus;
end else begin
  assign dbus_bpm_edc_didt_dd_avfs = dbus_bpm_edc_didt_dd[15:0];
end

endgenerate

wire [15:0] dbus_bpm_edc_didt_dd_avfs_dummy;
reg  [15:0] DbgBusIn_dly;
generate
if(enable_dummy_avfs_node==1) begin

  always @(posedge aon_Cpl_GFXCLK) begin
    if(~hard_resetb)
      dummy_avfs_node_en   <= 1'b0;
    else if(BPM_select & SERDES_cmd[`BPM_WRITE] & (SERDES_cmd[`BPM_REG_ADDR]==8'b00000110))
      dummy_avfs_node_en   <= SERDES_wdata[2];
  end

  always @(posedge aon_Cpl_GFXCLK) begin
    if(~hard_resetb)
      DbgBusIn_dly           <= 0;
    else if(dummy_avfs_node_en) 
      DbgBusIn_dly           <= DbgBusIn[15:0];
  end

  assign dbus_bpm_edc_didt_dd_avfs_dummy = dbus_bpm_edc_didt_dd_avfs | (DbgBusIn_dly & {16{dummy_avfs_node_en}});

end else begin
  assign dbus_bpm_edc_didt_dd_avfs_dummy = dbus_bpm_edc_didt_dd_avfs;
end

endgenerate

assign bpm_dbus_final = {dbus_bpm_edc_didt_dd[31:16],dbus_bpm_edc_didt_dd_avfs_dummy};
wire [15:0] DbgBusOut_dbg_bus = bpm_dbus_final[15:0];

  reg               didt_cgcg_en;
  reg               edc_cgcg_en;
generate
if(enable_psm_didt_dd == 1) begin
  always @(posedge aon_Cpl_GFXCLK) begin
    if(~hard_resetb) begin
      psm_didt_dd_cntl0 <= 'h0;
      psm_didt_dd_cntl1 <= 'h0;
    end else if(BPM_select & SERDES_cmd[`BPM_WRITE]) begin
      case (SERDES_cmd[`BPM_REG_ADDR])
        8'b00110000: begin
          psm_didt_dd_cntl0 <= SERDES_wdata;
        end
        8'b00110001: begin
          psm_didt_dd_cntl1 <= SERDES_wdata;
        end
      endcase
    end
  end

  always @(posedge aon_Cpl_GFXCLK) begin
    didt_cgcg_en <= cgcgclk_en;
  end

  didt_droop_detection uddd(
    // outputs
    .didt_throttle_level  (didt_throttle_level),
    .didt_dbus            (psm_didt_dbus),
    .psm_didt_dd_stat     (psm_didt_dd_stat),
    // inputs
    .Cpl_GFXCLK           (Cpl_GFXCLK),
    .resetb               (hard_resetb),
    .psm_didt_dd_cntl0    (psm_didt_dd_cntl0),
    .psm_didt_dd_cntl1    (psm_didt_dd_cntl1),
    .cgcg_en              (didt_cgcg_en),
    .psm_counter          (psm_counter),
    .dft_clk_gate_ctrl    ({Dft_Clk_Gater_En, Dft_Clk_Gater_Aon}),
    .dft_shift_clk        (TEST_CLK_EN)
  );

  assign psm_didt_dd_cntl0_out = psm_didt_dd_cntl0;
  assign psm_didt_dd_cntl1_out = psm_didt_dd_cntl1;
end
else begin
  assign psm_didt_dd_cntl0_out = 32'd0;
  assign psm_didt_dd_cntl1_out = 32'd0;
  assign didt_throttle_level = 3'h0;
  assign psm_didt_dd_stat    = 32'd0;
end
endgenerate

generate
if(enable_psm_edc_dd == 1) begin
  always @(posedge aon_Cpl_GFXCLK) begin
    if(~hard_resetb) begin
      psm_edc_dd_cntl <= 'h0;
    end else if(BPM_select & SERDES_cmd[`BPM_WRITE] & (SERDES_cmd[`BPM_REG_ADDR]==8'b01000000)) begin
      psm_edc_dd_cntl <= SERDES_wdata;
    end
  end

  always @(posedge aon_Cpl_GFXCLK) begin
    edc_cgcg_en <= cgcgclk_en;
  end

  edc_droop_detection uedd(
    // outputs
    .edc_throttle_level   (edc_throttle_level),
    .edc_dbus             (psm_edc_dbus),
    .psm_edc_dd_stat      (psm_edc_dd_stat),
    // inputs
    .Cpl_GFXCLK           (Cpl_GFXCLK),
    .resetb               (hard_resetb),
    .psm_edc_dd_cntl      (psm_edc_dd_cntl),
    .cgcg_en              (edc_cgcg_en),
    .psm_counter          (psm_counter),
    .dft_clk_gate_ctrl    ({Dft_Clk_Gater_En, Dft_Clk_Gater_Aon}),
    .dft_shift_clk        (TEST_CLK_EN)
  );

  assign psm_edc_dd_cntl_out = psm_edc_dd_cntl;
end
else begin
  assign edc_throttle_level  = 3'h0;
  assign psm_edc_dd_cntl_out = 32'd0;
  assign psm_edc_dd_stat     = 32'd0;
end
endgenerate



dbg_client_sync_mux_byte #(.LANE_COUNT(4),     // Number of Lanes, only 3 or 4 is currently supported
                           .LANE_WIDTH(8),     // Bit width of each Lane, only 8 or 16 is currently supported
                           .FLOPPED_OUTPUT(0), // flag to add flop after  the mux
                           .FLOPPED_INPUTS(0)) // flag to add flop before the mux
   bpm_dbg_client_sync_mux_byte             
    (.ipclk(1'b0),                             // system clock
     .dbg_active(1'b0),                        // Clock gater control
     .dbg_Mux_SelLane(dbg_mux_SelLane),        // LANE_COUNT * ([8] Lane Enable; [7:6] Byte Select; [5:0] Channel Select;)
     .dbg_Mux_Data(dbg_mux_Data),              // Debug-bus data outputs after muxing
     .idebug(bpm_dbus_final),
     .dbg_rst(1'b0),                           // Combined global reset with command decoded reset
     .TEST_CLK_EN(1'b0));                      // Gater override for test mode


`ifndef GFX_CG_MONITOR_DISABLE

`ifdef SIMULATION

`ifdef PG_ASSERT
bpm_cov #(.enable_dd(enable_dd), .enable_avfs(enable_avfs), .enable_psm_didt_dd(enable_psm_didt_dd), .enable_psm_edc_dd(enable_psm_edc_dd)) bpm_cov
  (
   .cg_gbl_sclk                         (Cpl_GFXCLK),
`ifdef GC__GFX_PIPELINE_CLOCK_GATING__1
   .BPM_gfx3d                           (BPM_gfx3d),
   .cg3d_blk_cntl_override              (cg3d_blk_cntl_override),
   .cg3d_rep_cntl_override              (cg3d_rep_cntl_override),
   .RLC_TARG_gfx3Den                    (RLC_TARG_gfx3Den_ff),
   .cg3d_Cpl_GFXCLK                     (cg3d_Cpl_GFXCLK),
`endif
`ifdef GC__DLDO_POWER_GATING__1
   .bpm_bpm_resetb                      (bpm_bpm_resetb_sync),
`endif
   .cg_Cpl_GFXCLK                       (cgcg_Cpl_GFXCLK),
   /*AUTOINST*/
   // Inputs
   .RLC_TARG_gfxen                      (RLC_TARG_gfxen_ff),
   .stop_clock                          (stop_clock),
   .TARG_BPM_cgcgls_override            (TARG_BPM_cgcgls_override),
   .cgcg_en                             (cgcg_en),
   .cgcgclk_en_rep                      (cgcgclk_en_rep),
   .cg3dclk_en_rep                      (cg3dclk_en_rep),
   .cgls_en                             (cgmempg_en),
   .cgls_on                             (cgmempg_on),
   .cgcg_override                       (cgcg_override),
   .TTOP_MGCG_override                  (TTOP_MGCG_override),
   .TTOP_FGCG_override                  (TTOP_FGCG_override),
   .BPM_blk_addr                        (BPM_blk_addr),
   .bpm_avfs_mm_wdata_l                 (bpm_avfs_mm_wdata_l),
   .bpm_avfs_mm_wdata_h                 (bpm_avfs_mm_wdata_h),
   .bpm_avfs_mm_rdata_l                 (bpm_avfs_mm_rdata_l),
   .bpm_avfs_mm_rdata_h                 (bpm_avfs_mm_rdata_h),
   .TTOP_P1_hard_resetb                 (TTOP_P1_hard_resetb),
   .TTOP_P1_combined_resetb_0           (TTOP_P1_combined_resetb_0),
   .TTOP_P1_combined_resetb_1           (TTOP_P1_combined_resetb_1),
   .TTOP_P1_combined_resetb_2           (TTOP_P1_combined_resetb_2),
   .TTOP_P1_combined_resetb_3           (TTOP_P1_combined_resetb_3),
   .BPM_select                          (BPM_select),
   .SERDES_cmd                          (SERDES_cmd),
   .SERDES_wdata                        (SERDES_wdata),
   .pg_P1_strap                         (pg_P1_strap),
   .pg_P2_strap                         (pg_P2_strap),
   .RSMU_GFX_hard_resetb                (RSMU_GFX_hard_resetb));

bpm_assert inst_bpm_assert
   (/*AUTOINST*/
    // Inputs
`ifdef GC__GFX_ISLAND_POWER_GATING__1
    .TTOP_P1_mother_sleep               (TTOP_P1_mother_sleep),
    .TTOP_P1_daughter_sleep             (TTOP_P1_daughter_sleep),
    .TTOP_P1_iso_clampn                 (TTOP_P1_iso_clampn),
    .TTOP_P2_mother_sleep               (TTOP_P2_mother_sleep),
    .TTOP_P2_daughter_sleep             (TTOP_P2_daughter_sleep),
    .TTOP_P2_iso_clampn                 (TTOP_P2_iso_clampn),
`endif
`ifdef GC__GFX_PIPELINE_CLOCK_GATING__1
    .BPM_gfx3d                          (BPM_gfx3d),
    .RLC_TARG_gfx3Den_ff                (RLC_TARG_gfx3Den_ff),
    .RLC_TARG_gfx3Den                   (RLC_TARG_gfx3Den),
    .cg3d_override                      (cg3d_blk_cntl_override),
`endif
    .disable_mgls                       (disable_mgls),
    .disable_mgcg                       (disable_mgcg),
    .rm_valid                           (RLC_GFX_rm_valid),
    .iCUSTOM_MEM_RM                     (iCUSTOM_MEM_RM),
    .iCUSTOM_MEM_RME                    (iCUSTOM_MEM_RME),
    .oCUSTOM_MEM_RM                     (oCUSTOM_MEM_RM),
    .oCUSTOM_MEM_RME                    (oCUSTOM_MEM_RME),
    .RLC_TARG_gfxen_ff                  (RLC_TARG_gfxen_ff),
    .stop_clock                         (stop_clock),
    .cg_gfx_sclk                        (cgcg_Cpl_GFXCLK),              // FIXME: changed to cgcg_Cpl_GFXCLK temporarily for dft/cg/mg gater merge feature
    .cgcg_override                      (cgcg_override),
    .Cpl_GFXCLK                         (Cpl_GFXCLK),
    .hard_resetb                        (hard_resetb),
    .TEST_CLK_EN                        (TEST_CLK_EN),
    .pg_P1_strap                        (pg_P1_strap),
    .pg_P2_strap                        (pg_P2_strap),
    .RLC_TARG_gfxen                     (RLC_TARG_gfxen),
    .RSMU_GFX_cold_resetb               (RSMU_GFX_cold_resetb),
    .RSMU_GFX_hard_resetb               (RSMU_GFX_hard_resetb),
    .xRBM_blk_soft_0_resetb             (xRBM_blk_soft_0_resetb),
    .xRBM_blk_soft_1_resetb             (xRBM_blk_soft_1_resetb),
    .xRBM_blk_soft_2_resetb             (xRBM_blk_soft_2_resetb),
    .xRBM_blk_soft_3_resetb             (xRBM_blk_soft_3_resetb),
    .dbg_mux_SelLane                    (dbg_mux_SelLane),
    .TTOP_P1_combined_resetb_0          (TTOP_P1_combined_resetb_0),
    .TTOP_P1_combined_resetb_1          (TTOP_P1_combined_resetb_1),
    .TTOP_P1_combined_resetb_2          (TTOP_P1_combined_resetb_2),
    .TTOP_P1_combined_resetb_3          (TTOP_P1_combined_resetb_3),
    .TTOP_P2_hard_resetb                (TTOP_P2_hard_resetb),
    .TTOP_P2_combined_resetb_0          (TTOP_P2_combined_resetb_0),
    .TTOP_P2_combined_resetb_1          (TTOP_P2_combined_resetb_1),
    .TTOP_P2_combined_resetb_2          (TTOP_P2_combined_resetb_2),
    .TTOP_P2_combined_resetb_3          (TTOP_P2_combined_resetb_3),
    .cgcg_en                            (cgcg_en),
    .TARG_BPM_cgcgls_override           (TARG_BPM_cgcgls_override),
    .BPM_blk_addr                       (BPM_blk_addr),
    .RLC_BPM_cmd                        (RLC_BPM_cmd),
    .BPM_select                         (BPM_select),
    .SERDES_new_cmd                     (SERDES_new_cmd),
    .SERDES_cmd                         (SERDES_cmd[23:0]),
    .TTOP_P1_hard_resetb                (TTOP_P1_hard_resetb),
    .TTOP_MGCG_override                 (TTOP_MGCG_override),
    .TTOP_FGCG_override                 (TTOP_FGCG_override),
    .TTOP_MGLS_override                 (TTOP_MGLS_override),
    .TTOP_MGLS_current_freq_code        (4'h0),
    .dbg_mux_Data                       (dbg_mux_Data),
    .mem_power_ctrl                     (mem_power_ctrl[`MEM__MEM_POWER_CTRL-1:0]),
    .TTOP_P1_mem_power_ctrl             (TTOP_P1_mem_power_ctrl[`MEM__MEM_POWER_CTRL-1:0]),
    .TTOP_P2_mem_power_ctrl             (TTOP_P2_mem_power_ctrl[`MEM__MEM_POWER_CTRL-1:0]),
    .PGFSM_P1_pwr_ctrl_out              (PGFSM_P1_pwr_ctrl_out[`MEM__MEM_POWER_CTRL-1:0]),
    .PGFSM_P2_pwr_ctrl_out              (PGFSM_P2_pwr_ctrl_out[`MEM__MEM_POWER_CTRL-1:0]),
    .RLC_BPM_status                     (RLC_BPM_status));

`endif // `ifdef PG_ASSERT

`endif //  `ifdef SIMULATION

`endif //  `ifdef GFX_CG_MONITOR_DISABLE 

endmodule // bpm

// Local Variables:
// verilog-library-directories:("." "./cpl/")
// verilog-library-extensions:(".v" ".sv" ".vh" ".h")
// End:









