`include "mm_global.d"
`define PSM_CURPSM_LENGTH 14

module bpm_avfs_wrap (
  // Inputs to AVFS
  PowerOk_VDD2
  , ScanShiftEn
  , CGater01_CG0
  , Reset
  , PwrMonEnX
  , WRCK
  , WSI
  , ShiftWR
  , UpdateWR
  , CaptureWR
  , WDREn
  , WRSTN
  , MeasureWR
  , CCLK
  , TstCcgEn
  , Cgen_UnitActive
  , RCLK_Free_AR
  , LBistEn
  , PsmEn
  , PsmAvfs_Gater
  , Avfs_ScanMode
  , SkipPhase
  , Isolate
  , DbgBusSel
  , DbgBusIn
  , avfs_dbg_en

  // Inputs to FSM
  , bpm_avfs_cfg
  , bpm_avfs_mm_clk
  , bpm_avfs_mm_reset
  , bpm_avfs_mm_ctl
  , bpm_avfs_mm_wdata_h
  , bpm_avfs_mm_wdata_l

  // Outputs from FSM
  , bpm_avfs_mm_rdata_h
  , bpm_avfs_mm_rdata_l
  , avfs_mm_cmd_done
  // Outputs from AVFS
`ifdef GFX_DROOP_DETECT
  , PsmOut
`endif
  , Psm_Alarm_Flag
  , WRO
  , DbgBus_enable
  , DbgBus
);

// Inputs to AVFS
  input       PowerOk_VDD2;
  input       ScanShiftEn;
  input       CGater01_CG0;
  input       Reset;
  input       PwrMonEnX;
  input       WRCK;
  input       WSI;
  input       ShiftWR;
  input       UpdateWR;
  input       CaptureWR;
  input       WDREn;
  input       WRSTN;
  input       MeasureWR;
  input       CCLK;
  input       TstCcgEn;
  input       Cgen_UnitActive;
  input       RCLK_Free_AR;
  input       LBistEn;
  input       PsmEn;
  input       PsmAvfs_Gater;
  input       Avfs_ScanMode;
  input       SkipPhase;
  input       Isolate;
  input       [3-1:0] DbgBusSel;  // FIXME: replace '3' with 'DBG_CFG_LENGTH' when AVFS is ready
  input       [16:0]  DbgBusIn;
  input       avfs_dbg_en;

// Inputs to FSM
  input  bpm_avfs_mm_clk;
  input  bpm_avfs_mm_reset;
  input  [23:0]      bpm_avfs_cfg;
  input  [19:0]      bpm_avfs_mm_ctl;
  input  [31:0]      bpm_avfs_mm_wdata_h;
  input  [31:0]      bpm_avfs_mm_wdata_l;

// Output from AVFS
  output [31:0]      bpm_avfs_mm_rdata_h;
  output [31:0]      bpm_avfs_mm_rdata_l;
  output             avfs_mm_cmd_done;
// Output from AVFS
`ifdef GFX_DROOP_DETECT
  output [`PSM_CURPSM_LENGTH-1:0] PsmOut;   // FIXME: replace '14' with 'PSM_CURPSM_LENGTH' when AVFS is ready
`endif
  output          DbgBus_enable;
  output [15:0]   DbgBus;  // FIXME: replace '' with 'DBG_LENGTH' when AVFS is ready
  output          Psm_Alarm_Flag;
  output          WRO;

  wire   [`MM_DATA]  MmBusOut_M1;   //8-bit M&M bus output contains local reg addr in M2 and write data in M4
  wire   MmLclRdEn_M2;              //local register read indication
  wire   MmLclWrEn_M2;              //local register write indication
  wire   [`MM_SZ]    MmLclSz_M2;    //local register size indication
  wire   [`MM_STATE] MmState;       //M&M state for read/write byte select
  wire   [`MM_DATA]  MmDatIn_M4;
  wire   [15:0]      DbgBus_tmp;


bpm_avfs_fsm u_bpm_avfs_fsm(
  .clk                  (bpm_avfs_mm_clk),
  .reset                (bpm_avfs_mm_reset),
  .bpm_avfs_cfg         (bpm_avfs_cfg),
  .bpm_avfs_mm_ctl      (bpm_avfs_mm_ctl),
  .bpm_avfs_mm_wdata_h  (bpm_avfs_mm_wdata_h),
  .bpm_avfs_mm_wdata_l  (bpm_avfs_mm_wdata_l),
  .MmDatIn_M4           (MmDatIn_M4),   

  .bpm_avfs_mm_rdata_h  (bpm_avfs_mm_rdata_h),
  .bpm_avfs_mm_rdata_l  (bpm_avfs_mm_rdata_l),
  .avfs_mm_cmd_done     (avfs_mm_cmd_done),
  .MmBusOut_M1          (MmBusOut_M1),   
  .MmLclRdEn_M2         (MmLclRdEn_M2),              
  .MmLclWrEn_M2         (MmLclWrEn_M2),              
  .MmLclSz_M2           (MmLclSz_M2),    
  .MmState              (MmState)
);        

gfx_AVFS3_TOP ugfx_avfs3_top (
  // inputs
  .PowerOk_VDD2     (PowerOk_VDD2     ), // New PowerOk port to drive PSM macro pwrOK pin
  .ScanShiftEn      (ScanShiftEn      ), // Scan shift Enable
  .CGater01_CG0     (CGater01_CG0     ), // Coarse Gater
  .Reset            (Reset            ), // reset signal to reset flops for SOCs which don't use scan-shift-reset
  .PwrMonEnX        (PwrMonEnX        ), // mux between SPM
  .WRCK             (WRCK             ), // override internal p1500
  .WSI              (WSI              ), // p1500
  .ShiftWR          (ShiftWR          ), // p1500
  .UpdateWR         (UpdateWR         ), // p1500
  .CaptureWR        (CaptureWR        ), // p1500
  .WDREn            (WDREn            ), // p1500
  .WRSTN            (WRSTN            ), // p1500
  .MeasureWR        (MeasureWR        ), // p1500 (does not exist in JTAG protocol - can use Idle signal)
`ifdef AVFS_ZAK
  .CCLK             (CCLK             ), // ZAK uses mesh clock
  .TstCcgEn         (TstCcgEn         ), // ZAK test coarse gater enable (non-scannable flops would use 1'b1)
  .Cgen_UnitActive  (Cgen_UnitActive  ), // ZAK coarse gater enable
  .RCLK_Free_AR     (RCLK_Free_AR     ), // ZAK uses free-running clock for coarse gater flops
`endif
  .LBistEn          (LBistEn          ),
  .PsmEn            (PsmEn            ),
  .PsmAvfs_Gater    (PsmAvfs_Gater    ),
  .Avfs_ScanMode    (Avfs_ScanMode    ),
  .SkipPhase        (SkipPhase        ), // from clock stretcher module - needs to be conditionally gated at the PSM when IP is updated
  .Isolate          (Isolate          ), // indicates if tile containing AVFS module is being power-gated
  .DbgBusSel        (DbgBusSel        ),
`ifdef AVFS_ZAK
  .MmBusIn_M1       (MmBusOut_M1      ), //   8-bit M&M bus input contains local reg addr in M2 and write data in M4
  .MmLclRdEn_M2     (MmLclRdEn_M2     ), //   local register read indication
  .MmLclWrEn_M2     (MmLclWrEn_M2     ), //   local register write indication
  .MmLclSz_M2       (MmLclSz_M2       ), //  local register size indication
  .MmState          (MmState          ), //   M&M state for read/write byte select
`endif

  // outputs
`ifdef GFX_DROOP_DETECT
  .PsmOut           (PsmOut           ), // adding new port to direct PSM count out through AVFS3 for GNL purposes
`endif
`ifdef AVFS_ZAK
  .MmDatOut_M4      (MmDatIn_M4       ),   //   8-bit data back to M&M bus for read
`endif
  .Psm_Alarm_Flag   (Psm_Alarm_Flag),
  .WRO              (WRO),   // p1500
  .DbgBus           (DbgBus_tmp)
);

reg  [15:0] DbgBus;
reg  DbgBus_enable;

always @(posedge bpm_avfs_mm_clk) 
begin
  if(bpm_avfs_mm_reset)
    DbgBus_enable           <= 0;
  else
    DbgBus_enable           <= avfs_dbg_en || DbgBusIn[16];
end

always @(posedge bpm_avfs_mm_clk) 
begin
  if(bpm_avfs_mm_reset)
    DbgBus           <= 0;
  else if(DbgBus_enable)
    DbgBus           <= DbgBusIn[15:0] | DbgBus_tmp;
end

endmodule
