module bpm_dyn_rm (
  // outputs
  oRM,
  oOverride_mgls,
  oOverride_mgcg,

  // Inputs
  iRM,
  BPM_addr_lo,
  rm_cntl,
  rm_valid,
  clk,
  resetn
);

parameter RM_BIT_WIDTH      = 26;
parameter BPM_ADDR_WIDTH    = 4;
parameter OVERRIDE_CYCLE    = 7;
parameter REL_MGCG_OVERRIDE = 3;

input   [RM_BIT_WIDTH-1:0]    iRM;
input                         clk;
input                         resetn;
input   [BPM_ADDR_WIDTH-1:0]  BPM_addr_lo;
input                         rm_valid;
input                         rm_cntl;

output  [RM_BIT_WIDTH-1:0]    oRM;
output                        oOverride_mgls;
output                        oOverride_mgcg;

  wire                        rm_valid_sync;
  wire                        oOverride_mgls;
  wire                        oOverride_mgcg;

  rm_f_sync #(.WIDTH(RM_BIT_WIDTH)) urm_f_sync (
    // Inputs
    .clk(clk),
    .rst(~resetn),
    .irm(iRM), 
    .iRLC_GFX_rm_valid(rm_valid),

    //Outputs
    .orm_sync(oRM),
    .oCG_override(rm_valid_sync)
    );

  reg                         rm_valid_sync_d1;
  always @(posedge clk) begin
    rm_valid_sync_d1 <= rm_valid_sync;
  end

  wire  start_rm_sample = rm_valid_sync & ~rm_valid_sync_d1;

  reg   [BPM_ADDR_WIDTH:0]    delay_cnt;
  reg                         rm_didt;
  reg                         override_mgls;
  reg                         override_mgcg;
  wire                        neg_rm_didt;
  always @(posedge clk) begin
    if(rm_cntl) begin
      if(!resetn) begin
        delay_cnt          <= 'h0;
        rm_didt            <= 'h0;
        override_mgls      <= 'h0;
        override_mgcg      <= 'h0;
      end
      else if(start_rm_sample) begin  // rm_didt, BPM_blk_addr[3:0]*2
        delay_cnt <= {BPM_addr_lo, 1'b0};
        rm_didt   <= 'h1;
      end
      else if(neg_rm_didt) begin
        delay_cnt          <= OVERRIDE_CYCLE;
        override_mgls      <= 'h1;
        override_mgcg      <= 'h1;
      end
      else if(override_mgcg && (delay_cnt == REL_MGCG_OVERRIDE)) begin
        override_mgcg <= 'h0;
        delay_cnt     <= delay_cnt - 1'h1;
      end
      else if(override_mgls && ~(|delay_cnt)) begin
        override_mgls <= 'h0;
        rm_didt       <= 'h0;
      end
      else if(~(|delay_cnt)) begin
        rm_didt       <= 'h0;
      end
      else if(|delay_cnt) begin
        delay_cnt <= delay_cnt - 1'h1;
      end
    end
    else begin
      delay_cnt          <= 'h0;
      rm_didt            <= 'h0;
      override_mgls      <= 'h0;
      override_mgcg      <= 'h0;
    end
  end

  reg                       rm_didt_d1;
  always @(posedge clk) begin
      rm_didt_d1 <= rm_didt;
  end

  assign neg_rm_didt = ~rm_didt & rm_didt_d1;

  assign oOverride_mgls    = rm_cntl ? override_mgls : 1'b0;
  assign oOverride_mgcg    = rm_cntl ? override_mgcg : 1'b0;

endmodule
