//                              -*- Mode: Verilog -*-
////////////////////////////////////////////////////////////////////////////
/*
 *
 *  Trade secret of ATI Technologies, Inc.
 *  Copyright 2005, ATI Technologies, Inc., (unpublished)
 *
 *  All rights reserved.  This notice is intended as a precaution against
 *  inadvertent publication and does not imply publication or any waiver
 *  of confidentiality.  The year included in the foregoing notice is the
 *  year of creation of the work.
 *
 */
////////////////////////////////////////////////////////////////////////////
//
// Filename    : bsc.v
// Description : Boundary scan cells
//
////////////////////////////////////////////////////////////////////////////



module bs_bidir_cell
(
    input   SI,           // Boundary scan chain input
    output  SO,          // Boundary scan chain output
    input   CAPTURE,      // = ~dr_shift && dec_bst;
    input   TCK,          // = dec_bst && (dr_shift || dr_capture);
    input   UPDATE,       // = dr_update  && dec_bst;

    input   TEST_DATA_SEL, // 1 - select TSTO, 0 select boundary scan flop data. (default 1)
    input   TEST_OE_SEL,   // 1 - select TSTE, 0 select boundary scan flop OE. (default 1)

    input   OE,		  //Functional Output Enable.
    input   TSTE,         // Test Output Enable
    output  PAD_OE,	// Output Enable to the PAD

    input   ODATA,	  //Functional Data from core
    input   SCAN_OUT_FROM_CORE,         // Scan channel output
    output  PAD_DATA,	// Output Data to Transmitters 

    input   IDATA,	  //Functional Data from Receivers
    output  SCAN_IN_TO_CORE, // Scan channel Input

    input   MODE,          //Extest mode
    input   MODE_INTEST          //Intest mode

);

wire so_rx, so_tx;

bs_tx_cell bs_oe (
		.SI		(so_tx),
		.SO		(SO),
		.CAPTURE	(CAPTURE),
		.TCK		(TCK),
		.UPDATE		(UPDATE),

		.TEST_DATA_SEL	(TEST_OE_SEL),
		.TEST_DATA	(TSTE),
		.FUNC_DATA	(OE),
		.MODE		(MODE),
		.DATA_OUT	(PAD_OE)
		);

bs_tx_cell bs_tx_data (
		.SI		(so_rx),
		.SO		(so_tx),
		.CAPTURE	(CAPTURE),
		.TCK		(TCK),
		.UPDATE		(UPDATE),

		.TEST_DATA_SEL	(TEST_DATA_SEL),
		.TEST_DATA	(SCAN_OUT_FROM_CORE),
		.FUNC_DATA	(ODATA),
		.MODE		(MODE),
		.DATA_OUT	(PAD_DATA)
		);


bs_rx_cell bs_rx_data (
		.SI		  (SI),
		.SO		  (so_rx),
		.CAPTURE	  (CAPTURE),
		.TCK		  (TCK),
		.UPDATE		  (UPDATE),

		.IDATA		  (IDATA),
		.MODE_INTEST	  (MODE_INTEST),
		.SCAN_IN_TO_CORE  (SCAN_IN_TO_CORE)
		);

endmodule

////////////////////////////////////////////////////////////////////////////

module bs_tx_cell
(
    input   SI,		    // Boundary scan chain input
    output  SO,		    // Boundary scan chain output
    input   CAPTURE,	    // = ~dr_shift && dec_bst;
    input   TCK,	    // = dec_bst && (dr_shift || dr_capture);
    input   UPDATE,	    // = dr_update  && dec_bst;

    input   TEST_DATA_SEL,  // 0 - boundary scan, 1 - scan-atpg. (default 1)
    input   TEST_DATA,	    // scan-in(TSTO) or TSTE.
    input   FUNC_DATA,	    // functional data
    input   MODE,	    // 1 - select test data, 0 - select func data. (default 0)
    output  DATA_OUT	    // Test Output Data
);

reg capture_ff_pos, capture_ff_neg;
reg update_ff;

always @(posedge TCK) begin
   capture_ff_pos <= (CAPTURE) ? FUNC_DATA : SI;
end

always @(negedge TCK) begin
   capture_ff_neg <= capture_ff_pos;
end

always @(posedge UPDATE) begin  // could be latches
  update_ff <= capture_ff_pos;
end

assign SO = capture_ff_neg;

wire scan_data	= (TEST_DATA_SEL) ? TEST_DATA	: update_ff;  
assign DATA_OUT	= (MODE)	  ? scan_data	: FUNC_DATA;

endmodule

////////////////////////////////////////////////////////////////////////////

module bs_rx_cell
(
    input   SI,		    // Boundary scan chain input
    output  SO,		    // Boundary scan chain output
    input   CAPTURE,	    // = ~dr_shift && dec_bst;
    input   TCK,	    // = dec_bst && (dr_shift || dr_capture);
    input   UPDATE,	    // = dr_update  && dec_bst;

    input   IDATA,	    // input data 
    input   MODE_INTEST,    // 1 - select test data, 0 - select func data.
    output  SCAN_IN_TO_CORE	    // Test Output Data
);

reg capture_ff_pos, capture_ff_neg;
reg update_ff;

always @(posedge TCK) begin
   capture_ff_pos <= (CAPTURE) ? IDATA : SI;
end

always @(negedge TCK) begin
   capture_ff_neg <= capture_ff_pos;
end

always @(posedge UPDATE) begin  // could be latches
  update_ff <= capture_ff_pos;
end

assign SO = capture_ff_neg;

assign SCAN_IN_TO_CORE	= (MODE_INTEST)	  ? update_ff	: IDATA;

endmodule
////////////////////////////////////////////////////////////////////////////
