//////////////////////////////////////////////////////////////////////////////////////////////////////////
//Description: Local clock gating module for the Shader complex. It is a simple clock gater with iclk as one
//             input and four clock enables. Four clock enables are: (1) OVERRIDE (2) ENABLE GATE (3) BUSY
//             (4) SPARE. If a design requires more than 4 inputs then we can parameterize this module
//////////////////////////////////////////////////////////////////////////////////////////////////////////
// following defines are added to instance the clock buffers and mark them for scan grouping
`define SCAN_GRP0 0
`define SCAN_GRP1 1
`define SCAN_GRP2 2
`define SCAN_GRP3 3
`define SCAN_GRP4 4
`define SCAN_GRP5 5
`define SCAN_GRP6 6
`define SCAN_GRP7 7
module cgts_local(
   // Outputs
   dyn_clk ,
   // Inputs
   `ifdef ABV InputChkDis, `endif
   iclk, override, en1, en2, en3
   );
//////////////////////////
//in/outs
//////////////////////////
 input       iclk;
 output      dyn_clk;
 input       override;
 input       en1;
 input       en2;
 input       en3;
 `ifdef ABV
 input       InputChkDis;
 `endif
//////////////////////////
//
//////////////////////////
  wire override_s;
wire core_busy ;
assign core_busy = en1 | en2 | en3 ;
// sync usync_override( .D(override), .CK(iclk), .Q(override_s) );
techind_sync_icd #(.depth           (3)
       )
u_techind_sync(
               .SB              (1'b1       ),
               .RB              (1'b1       ),
               .FLUSH           (1'b0       ),
               .CKsrc           (1'b0       ),
               .CKdst           (iclk       ),
               .EN              (1'b1       ),
               .Dsrc            (override   ),
               .Qsrc            (),
               .Qdst            (override_s ),
               `ifdef ABV
               .InputChkDis     (InputChkDis),
               `endif
               .SC1             (1'b0       ),
               .SC2             (1'b0       ),
               .SE              (1'b0       ),
               .SI              (1'b0       ),
               .SO              ()
       ) ;

wire active = override_s | core_busy;
wire dyn_clk_int;
wire dyn_clk_gate;
`ifdef SIMULATION_WITH_x

assign dyn_clk_int = active ? dyn_clk_gate : 1'bx;
`else
assign dyn_clk_int = dyn_clk_gate;
`endif
  wire dft_te_wire;
ati_TE_driver u_TE_driver
  (
   .TEST_CLK_EN(1'b0),
   .DFT_TE(dft_te_wire)
  );
ati_clock_gate u_dyn_clk(
    .clk_out(dyn_clk_gate),
    .clk_src(iclk),
    .clk_en(active),
    .TE(dft_te_wire)
    );
// following code is added to instance the clock buffers and mark them for scan grouping
wire [`SCAN_GRP7:0] clks;
parameter scan_grp_dyn_clk = `SCAN_GRP0; //defaulted
aticlk_buf scan_grp0_clk(.Y(clks[`SCAN_GRP0]), .A(dyn_clk_int));
aticlk_buf scan_grp1_clk(.Y(clks[`SCAN_GRP1]), .A(dyn_clk_int));
aticlk_buf scan_grp2_clk(.Y(clks[`SCAN_GRP2]), .A(dyn_clk_int));
aticlk_buf scan_grp3_clk(.Y(clks[`SCAN_GRP3]), .A(dyn_clk_int));
aticlk_buf scan_grp4_clk(.Y(clks[`SCAN_GRP4]), .A(dyn_clk_int));
aticlk_buf scan_grp5_clk(.Y(clks[`SCAN_GRP5]), .A(dyn_clk_int));
aticlk_buf scan_grp6_clk(.Y(clks[`SCAN_GRP6]), .A(dyn_clk_int));
aticlk_buf scan_grp7_clk(.Y(clks[`SCAN_GRP7]), .A(dyn_clk_int));
assign dyn_clk = clks[scan_grp_dyn_clk];
`ifdef SIMULATION
cgts_local_cov ucgts_local_cov(
    .override (override_s),
    .en1      (en1),
    .en2      (en2),
    .en3      (en3),
    .iclk     (iclk),
    .dyn_clk  (dyn_clk)
);
`endif
endmodule
//////////////////////////////////////////////////////////////////////////////////////////////////////////
