`include "msgio_defines.vh"
module cgts_local_cov(
    // Inputs to cgts_local
    override, en1, en2, en3, iclk,
    // Outputs of cgts_local
    dyn_clk
);
//////////////////////////
input           override;
input           en1;
input           en2;
input           en3;
input           iclk;
input           dyn_clk;
//////////////////////////
//VCS coverage off
`ifdef VCS
reg enable;
initial begin
  if($test$plusargs("cgt_assert_disable"))
    enable = 1'b0;
  else
    enable = 1'b1;
end
reg cgt_en_assert_enable;
initial begin
  if($test$plusargs("cgt_en_assert_disable"))
    cgt_en_assert_enable = 1'b0;
  else
    cgt_en_assert_enable = 1'b1;
end
 `ifdef CGTS_COV
  wire seq = override | en1 | en2 | en3;
core_busy_active_dyn_clk_inactive : assert property (@(negedge iclk) disable iff(~enable) (~seq ##1  seq [*4] |->  dyn_clk));
core_busy_inactive_dyn_clk_active : assert property (@(negedge iclk) disable iff(~enable) ( seq ##1 ~seq [*4] |-> (dyn_clk!==1'b1)));
//input_unknown                     : assert property (@(posedge iclk) (##20 ~($isunknown(override) | $isunknown(en1) | $isunknown(en2) | $isunknown(en3))));
 `endif //  `ifdef CGTS_COV
reg override_disabled;
initial override_disabled = 1'b0;
always @(posedge iclk) if(override == 1'b0) override_disabled <= 1'b1;
`ifdef CG_W2E
final if((override_disabled == 1'b0) && enable && cgt_en_assert_enable) `MSGIO_DISPLAY("%m ERROR: Override never deactivated");
`else
final if((override_disabled == 1'b0) && enable && cgt_en_assert_enable) `MSGIO_DISPLAY("%m WARNING: Override never deactivated");
`endif
// en1 check
reg en1_deasserted;
initial en1_deasserted = 1'b0;
always @(posedge iclk) if (en1 == 1'b0) en1_deasserted <= 1'b1;
final if (en1_deasserted == 1'b0 && enable && cgt_en_assert_enable)
`ifdef CG_VLD_W2E
	`MSGIO_DISPLAY("%m ERROR:CG_VLD checker: en1 never deactivated");
`else
	`MSGIO_DISPLAY("%m WARNING:CG_VLD checker:  en1 never deactivated");
`endif
// en2 check
reg en2_deasserted;
initial en2_deasserted = 1'b0;
always @(posedge iclk) if (en2 == 1'b0) en2_deasserted <= 1'b1;
final if (en2_deasserted == 1'b0 && enable && cgt_en_assert_enable)
`ifdef CG_VLD_W2E
	`MSGIO_DISPLAY("%m ERROR:CG_VLD checker: en2 never deactivated");
`else
	`MSGIO_DISPLAY("%m WARNING:CG_VLD checker:  en2 never deactivated");
`endif
// en3 check
reg en3_deasserted;
initial en3_deasserted = 1'b0;
always @(posedge iclk) if (en3 == 1'b0) en3_deasserted <= 1'b1;
final if (en3_deasserted == 1'b0 && enable && cgt_en_assert_enable)
`ifdef CG_VLD_W2E
	`MSGIO_DISPLAY("%m ERROR:CG_VLD checker:  en3 never deactivated");
`else
	`MSGIO_DISPLAY("%m WARNING:CG_VLD checker:  en3 never deactivated");
`endif
`ifdef CGT_FCOV
covergroup cgts_local_cg_t @(negedge iclk);
    dyn_clk_cp : coverpoint dyn_clk
    {
        bins num_clocks_dyn_clk_inactive = {0};
        bins num_clocks_dyn_clk_active   = {1};
    }
endgroup //cgts_local_cg_t
cgts_local_cg_t cgts_local_cg = new;
`endif // CGT_FCOV
`else
initial begin
  $display("%m at %0t: ld/src/misc/cgts_local_cov.sv: SVA not yet supported in NCSIM in wekiva scripts", $realtime);
end
`endif // VCS
//VCS coverage on
// clock efficiency monitor
`ifdef CLK_EFF_MONITOR
        reg[31:0] clocks_counter;
        initial
            clocks_counter = 0;
        always @(posedge dyn_clk)
                clocks_counter <= clocks_counter + 1;
        final `MSGIO_DISPLAY("%m: clock gating efficiency check : clock counter = %d",clocks_counter);
`endif
`ifdef CG_SIGNALS_DUMP_EN
initial
begin
  #1;
  $fmonitor(tb.cg_file_handle, "CG CGTS_OV:(%t) %m %b",$time,override);
end
`endif
endmodule
