//////////////////////////////////////////////////////////////////////////////////////////////////////////
//Description: Local (aka medium-grain) clock gating module. It generates three type of clocks: static_oclk, dyn_oclki*, reg_oclk.
//             static_oclk is same as incoming (iclk) clock & is expected to drive your (flow control) interface logic.
//         dyn_oclk* is gated based on your 'busy' signals. reg_oclk is to drive your xRBM registers. reg_oclk is enabled
//         by enable signal coming from xRBM block.
//
//         If you find any issue or have any feedback, please email to sagheer.ahmad@amd.com
//////////////////////////////////////////////////////////////////////////////////////////////////////////
`timescale 1ps/1ps
module cgtt_local_2r1d(
   // Outputs
   dyn_oclk,
   dyn_oclk_vld,
   static_oclk, reg_oclk1, reg_oclk_vld1, reg_oclk0, reg_oclk_vld0,
   // Inputs
   resetb, iclk, cg_override,
   core_busy, reg_busy1, reg_busy0,
   CGTT_BLK_CTRL_REG
   );
//////////////////////////
//in/outs
//////////////////////////
 input       resetb;
 input       iclk;
 input       cg_override; 
        
 output      dyn_oclk;
 output      dyn_oclk_vld;
 input       core_busy;
 output      static_oclk;
 output      reg_oclk1, reg_oclk0;
 output      reg_oclk_vld1, reg_oclk_vld0;
 input       reg_busy1, reg_busy0;
 input [31:0] CGTT_BLK_CTRL_REG;
/////////////////////////////////////////////////////
//instantiate base module
/////////////////////////////////////////////////////
aticlk_buf u_static_clk(.Y(static_oclk), .A(iclk));
  wire override_s;
// sync usync_override( .D(cg_override), .CK(static_oclk), .Q(override_s) );
techind_sync_icd #(.depth           (3)
       )
u_techind_sync(
               .SB              (1'b1       ),
               .RB              (1'b1       ),
               .FLUSH           (1'b0       ),
               .CKsrc           (1'b0       ),
               .CKdst           (static_oclk),
               .EN              (1'b1       ),
               .Dsrc            (cg_override),
               .Qsrc            (),
               .Qdst            (override_s ),
               `ifdef ABV
               .InputChkDis     (!resetb    ),
               `endif
               .SC1             (1'b0       ),
               .SC2             (1'b0       ),
               .SE              (1'b0       ),
               .SI              (1'b0       ),
               .SO              ()
       ) ;
parameter scan_grp_dyn_oclk = 0; //defaulted
cgtt_local_dcg #(
          .scan_grp_dyn_oclk (scan_grp_dyn_oclk)
      )
      u_cgtt_dclk
  (.dyn_oclk      (dyn_oclk),
   .dyn_oclk_vld      (dyn_oclk_vld),
   .core_busy     (core_busy),
   .soft_on_delay     (CGTT_BLK_CTRL_REG[3:0]),
   .soft_off_delay    (CGTT_BLK_CTRL_REG[11:4]),
   .soft_override     (CGTT_BLK_CTRL_REG[29]),
   
   // Inputs
   .resetb      (resetb),
   .iclk        (iclk),
   .cg_override     (override_s),
   .static_oclk     (static_oclk));
parameter scan_grp_reg_oclk1 = 0; //defaulted
cgtt_local_rcg #(
          .scan_grp_reg_oclk (scan_grp_reg_oclk1)
   )
   u_cgtt_rclk1
  (.soft_override   (CGTT_BLK_CTRL_REG[30]),
   
   // Outputs
   .reg_oclk      (reg_oclk1),
   .reg_oclk_vld      (reg_oclk_vld1),
   // Inputs
   .resetb      (resetb),
   .iclk        (iclk),
   .cg_override     (override_s),
   .reg_busy      (reg_busy1),
   .static_oclk     (static_oclk));
parameter scan_grp_reg_oclk0 = 0; //defaulted
cgtt_local_rcg #(
          .scan_grp_reg_oclk (scan_grp_reg_oclk0)
   )
   u_cgtt_rclk0
  (.soft_override   (CGTT_BLK_CTRL_REG[31]),
   
   // Outputs
   .reg_oclk      (reg_oclk0),
   .reg_oclk_vld      (reg_oclk_vld0),
   // Inputs
   .resetb      (resetb),
   .iclk        (iclk),
   .cg_override     (override_s),
   .reg_busy      (reg_busy0),
   .static_oclk     (static_oclk));
endmodule
//////////////////////////////////////////////////////////////////////////////////////////////////////////
