`include "msgio_defines.vh"
module cgtt_local_dcg_cov(
   // Outputs of cggt_local_dcg
   dyn_oclk, dyn_oclk_vld,
   // Inputs to cgtt_local_dcg
   resetb, iclk, cg_override, core_busy, soft_on_delay,
   soft_off_delay, soft_override, static_oclk,
   // Internal signals of cgtt_local_dcg
   core_busy_filtered
   );
//////////////////////////
 input          resetb;
 input          iclk;
 input          cg_override;
 input          core_busy;
 input  [3:0]   soft_on_delay;
 input [11:4]   soft_off_delay;
 input          soft_override;
 input          static_oclk;
 input          dyn_oclk;
 input          dyn_oclk_vld;
 input          core_busy_filtered;
//////////////////////////
`ifdef VCS
//VCS coverage off
reg enable;
initial begin
  if($test$plusargs("cgt_assert_disable"))
    enable = 1'b0;
  else
    enable = 1'b1;
end
reg hier_enable;
reg [8*256-1:0] Hier;
initial begin
  hier_enable = 1'b1;
  $sformat(Hier, "%m");
  if($test$plusargs({Hier,"__cgt_assert_hier_disable"}))
  begin
    hier_enable = 1'b0;
    `MSGIO_DISPLAY("%m:cg assertion is waived via hierarchy");
  end
end
reg cgt_en_assert_enable;
initial begin
  if($test$plusargs("cgt_en_assert_disable"))
    cgt_en_assert_enable = 1'b0;
  else
    cgt_en_assert_enable = 1'b1;
end
  wire seq0 = ~core_busy_filtered & ~soft_override;
  wire seq1 = seq0 & ~cg_override;
static_oclk_inactive                            : assert property (@(negedge iclk) disable iff(~enable) (##10 (static_oclk || iclk === 1'bx)));
core_busy_active_dyn_oclk_vld_inactive          : assert property (@(negedge iclk) disable iff(~resetb | ~enable) (~core_busy_filtered ##1 core_busy_filtered [*4] |-> dyn_oclk_vld));
core_busy_override_inactive_dyn_oclk_vld_active : assert property (@(negedge iclk) disable iff(~resetb | ~enable) ( core_busy_filtered | soft_override ##1 seq0 [*4] |-> (dyn_oclk_vld!==1'b1)));
soft_override_active_dyn_oclk_vld_inactive      : assert property (@(negedge iclk) disable iff(~resetb | ~enable) (~soft_override ##1 soft_override [*5] |-> dyn_oclk_vld));
`ifdef CGTS_COV
core_busy_active_dyn_oclk_inactive              : assert property (@(negedge iclk) disable iff(~resetb | ~enable) (~core_busy_filtered ##1 core_busy_filtered [*4] |-> dyn_oclk));
core_busy_override_inactive_dyn_oclk_active     : assert property (@(negedge iclk) disable iff(~resetb | ~enable) ( core_busy_filtered | cg_override | soft_override ##1 seq1 [*5] |-> (dyn_oclk!==1'b1)));
cg_override_active_dyn_oclk_inactive            : assert property (@(negedge iclk) disable iff(~resetb | ~enable) ( cg_override ##1 cg_override |-> dyn_oclk));
soft_override_active_dyn_oclk_inactive          : assert property (@(negedge iclk) disable iff(~resetb | ~enable) (~soft_override ##1 soft_override [*4] |-> dyn_oclk));
`endif
reg reset_deactivated;
initial reset_deactivated = 0;
always @(posedge iclk) if(resetb) reset_deactivated <= 1;
`ifdef CG_W2E
final if((reset_deactivated == 0) && enable && hier_enable) `MSGIO_DISPLAY("%m ERROR: CLKGATE Reset never deactivated");
`else
final if((reset_deactivated == 0) && enable && hier_enable ) `MSGIO_DISPLAY("%m WARNING: CLKGATE Reset never deactivated");
`endif
reg override_disabled;
initial override_disabled = 1'b0;
always @(negedge cg_override or posedge iclk) if(cg_override == 1'b0 ) override_disabled <= 1'b1;
`ifdef CG_W2E
final if((override_disabled == 1'b0) && enable && cgt_en_assert_enable) `MSGIO_DISPLAY("%m ERROR: CLKGATE Master Override never deactivated");
`else
final if((override_disabled == 1'b0) && enable && cgt_en_assert_enable) `MSGIO_DISPLAY("%m WARNING: CLKGATE Master Override never deactivated");
`endif
reg soft_override_disabled;
initial soft_override_disabled = 1'b0;
always @(posedge iclk) if(soft_override == 1'b0) soft_override_disabled <= 1'b1;
`ifdef CG_W2E
final if((soft_override_disabled == 1'b0) && enable && hier_enable) `MSGIO_DISPLAY("%m ERROR: CLKGATE Soft Override never deactivated");
`else
final if((soft_override_disabled == 1'b0) && enable && hier_enable) `MSGIO_DISPLAY("%m WARNING: CLKGATE Soft Override never deactivated");
`endif
reg vld_deasserted;
initial vld_deasserted = 1'b0;
always @(posedge iclk) if (dyn_oclk_vld == 1'b0) vld_deasserted <= 1'b1;
final if (vld_deasserted == 1'b0 && enable && cgt_en_assert_enable)
`ifdef CG_VLD_W2E
	`MSGIO_DISPLAY("%m ERROR:CG_VLD checker:clk never turn off, check clock gating setting");
`else
	`MSGIO_DISPLAY("%m WARNING:CG_VLD checker: clk never turn off, check clock gating setting");
`endif
// master override is set after reset until it is cleared by chip_config or
// ucode
always @ (posedge resetb)
begin
  if (enable && hier_enable)
  begin
    repeat(3) @(posedge iclk);
    if (cg_override == 0 && soft_override==0)
`ifdef CG_W4E
      `MSGIO_DISPLAY("%m ERROR: CLKGATE Master/Soft Override should be set after reset");
`else
      `MSGIO_DISPLAY("%m WARNING: CLKGATE Master/Soft Override should be set after reset");
`endif
  end
end
// soft override should be cleared before the end of test
`ifdef CG_W2E
final if((soft_override == 1'b1) && enable && hier_enable && cgt_en_assert_enable) `MSGIO_DISPLAY("%m ERROR: CLKGATE Soft Override is set at the end of test");
`else
final if((soft_override == 1'b1) && enable && hier_enable && cgt_en_assert_enable) `MSGIO_DISPLAY("%m WARNING: CLKGATE Soft Override is set at the end of test");
`endif
`ifdef CGT_FCOV
covergroup cgtt_local_dcg_cg_t @(negedge iclk);
    dyn_oclk_cp : coverpoint dyn_oclk iff(resetb)
    {
        bins num_clocks_dyn_oclk_inactive = {0};
        bins num_clocks_dyn_oclk_active   = {1};
    }
endgroup //cgtt_dcg_local_cg_t
cgtt_local_dcg_cg_t cgtt_local_dcg_cg = new;
`endif // CGT_FCOV
`ifdef CG_SIGNALS_DUMP_EN
initial
begin
  #1;
  $fmonitor(tb.cg_file_handle, "CG CGTT_DCG_OV:(%t) %m %b",$time,cg_override);
  $fmonitor(tb.cg_file_handle, "CG CGTT_DCG_SOFT_OV:(%t) %m %b",$time,soft_override);
  $fmonitor(tb.cg_file_handle, "CG CGTT_DCG_OFF_DLY:(%t) %m h%x",$time,soft_off_delay);
  $fmonitor(tb.cg_file_handle, "CG CGTT_DCG_ON_DLY:(%t) %m h%x",$time,soft_on_delay);
  $fmonitor(tb.cg_file_handle, "CG CGTT_DCG_RST:(%t) %m h%x",$time,resetb);
end
`endif
`else
initial begin
  $display("%m at %0t: ld/src/misc/cgtt_local_dcg_cov.sv: SVA not yet supported in NCSIM in wekiva scripts", $realtime);
end
`endif // VCS
//VCS coverage on
// clock efficiency monitor
`ifdef CLK_EFF_MONITOR
    reg[31:0] clocks_counter;
        initial
                clocks_counter = 0;
    always @(posedge dyn_oclk)
        clocks_counter <= clocks_counter + 1;
    final `MSGIO_DISPLAY("%m: clock gating efficiency check : clock counter = %d",clocks_counter);
`endif
endmodule
