//////////////////////////////////////////////////////////////////////////////////////////////////////////
//Description: Local clock gater dynamic clock on generation.  The logic in this module is largely identical
//             to cgtt_local_dcg module, without the clock gate. 
//////////////////////////////////////////////////////////////////////////////////////////////////////////
`ifdef SIMULATION
`timescale 1ps/1ps
`endif 
module cgtt_local_dclkon(
   // Outputs
   dyn_oclk_vld, 
   // Inputs
   resetb, core_busy, soft_on_delay, 
   soft_off_delay, soft_override, static_oclk
   );
//////////////////////////
//in/outs
//////////////////////////
 input 	     resetb;
 input       core_busy;
 input  [3:0] soft_on_delay;
 input [11:4] soft_off_delay;
 input        soft_override;
 input       static_oclk;
 output      dyn_oclk_vld;
//////////////////////////
reg  core_busy_d1;
reg  core_busy_d2;
//reg  [3:0] core_busy_filtered;  //4 copies of this flop for PD purpose.
reg  core_busy_filtered;
reg  core_busy_filtered_d1;
reg  core_busy_filtered_d2;
reg  oclk_vld_dly;
reg  [3:0] ON_DELAY;
reg [11:4] OFF_DELAY;
reg        SOFT_CORE_OVERRIDE;
reg  [3:0] on_counter;
reg [11:4] off_counter;
wire oclk_vld; 
//
//leaf (always on) clock domain logic
//
wire [3:0] on_counter_nxt = (~core_busy_d2 && core_busy_d1 ) ?  ON_DELAY : //rising edge of busy. to turn on quickly, using busy w/o registering.
			    ( on_counter == 4'd0 )           ?  on_counter : (on_counter - 4'd1);   
wire [7:0] off_counter_nxt= (core_busy_d2 && ~core_busy_d1 ) ?  OFF_DELAY : //falling edge of busy
			    ( off_counter == 8'd0 )          ?  off_counter : (off_counter - 8'd1);   
always @(posedge static_oclk) 
begin
	if(~resetb) begin
		ON_DELAY     <= 4'd0;
		OFF_DELAY    <= 8'd0;
		SOFT_CORE_OVERRIDE <= 1'b1;
		on_counter   <= 4'd0;
		off_counter  <= 8'd0;
		core_busy_d1 <= 1'b1;
		core_busy_d2 <= 1'b1;
		core_busy_filtered   <= 1'b1;
		core_busy_filtered_d1<= 1'b1;
		core_busy_filtered_d2<= 1'b1;
	end else begin
		ON_DELAY     <= soft_on_delay;
		OFF_DELAY    <= soft_off_delay;
		SOFT_CORE_OVERRIDE <= soft_override;
		core_busy_d1 <= core_busy;
		core_busy_d2 <= core_busy_d1;
		on_counter   <= on_counter_nxt;
		off_counter  <= off_counter_nxt;
        if (core_busy && ~core_busy_d1 && oclk_vld) begin
            core_busy_filtered <= 1'b1;
        end
		else begin
            if((on_counter_nxt == 4'd0) && (off_counter_nxt <= 8'd0)) begin
			    core_busy_filtered <= core_busy_d1 || SOFT_CORE_OVERRIDE;
		    end	
        end
		core_busy_filtered_d1 <= core_busy_filtered;
		core_busy_filtered_d2 <= core_busy_filtered_d1;
	end
end
//
//clock gates
//
assign oclk_vld = core_busy_filtered_d2 && core_busy_filtered_d1;
`ifdef SIMULATION
//always @(oclk_vld) 
//begin
//	oclk_vld_dly = #2 oclk_vld;
//end
assign #2 dyn_oclk_vld = oclk_vld;
`else
assign dyn_oclk_vld = oclk_vld;
`endif
endmodule
