`include "msgio_defines.vh"
module cgtt_local_rcg_cov(
   // Outputs of cgtt_local_rcg
   reg_oclk, reg_oclk_vld,
   // Inputs to cgtt_local_rcg
   resetb, iclk, cg_override, reg_busy, soft_override, static_oclk
   );
//////////////////////////
 input          resetb;
 input          iclk;
 input          cg_override;
 input          reg_busy;
 input          soft_override;
 input          static_oclk;
 input          reg_oclk;
 input          reg_oclk_vld;
//////////////////////////
`ifdef VCS
//VCS coverage off
reg enable;
initial begin
  if($test$plusargs("cgt_assert_disable"))
    enable = 1'b0;
  else
    enable = 1'b1;
end
reg hier_enable;
reg [8*256-1:0] Hier;
initial begin
  hier_enable = 1'b1;
  $sformat(Hier, "%m");
  if($test$plusargs({Hier,"__cgt_assert_hier_disable"}))
  begin
    hier_enable = 1'b0;
    `MSGIO_DISPLAY("%m:cg assertion is waived via hierarchy");
  end
end
reg cgt_en_assert_enable;
initial begin
  if($test$plusargs("cgt_en_assert_disable"))
    cgt_en_assert_enable = 1'b0;
  else
    cgt_en_assert_enable = 1'b1;
end
  wire seq0 = ~reg_busy & ~soft_override;
  wire seq1 = seq0 & ~cg_override;
static_oclk_inactive                           : assert property (@(negedge iclk) disable iff(~enable) (##10 (static_oclk || iclk === 1'bx)));
reg_busy_active_reg_oclk_vld_inactive          : assert property (@(negedge iclk) disable iff(~resetb | ~enable) (~reg_busy ##1 reg_busy [*5] |-> reg_oclk_vld));
soft_override_active_reg_oclk_vld_inactive     : assert property (@(negedge iclk) disable iff(~resetb | ~enable) (~soft_override ##1 soft_override [*5] |-> reg_oclk_vld));
reg_busy_override_inactive_reg_oclk_vld_active : assert property (@(negedge iclk) disable iff(~resetb | ~enable) ( reg_busy | soft_override ##1 seq0 [*4] |-> (reg_oclk_vld!==1'b1)));
 `ifdef CGTS_COV
reg_busy_active_reg_oclk_inactive              : assert property (@(negedge iclk) disable iff(~resetb | ~enable) (~reg_busy ##1 reg_busy [*4] |-> reg_oclk));
reg_busy_override_inactive_reg_oclk_active     : assert property (@(negedge iclk) disable iff(~resetb | ~enable) ( reg_busy | cg_override | soft_override ##1 seq1 [*5]  |-> (reg_oclk!==1'b1)));
cg_override_active_reg_oclk_inactive           : assert property (@(negedge iclk) disable iff(~resetb | ~enable) ( cg_override ##1 cg_override |-> reg_oclk));
soft_override_active_reg_oclk_inactive         : assert property (@(negedge iclk) disable iff(~resetb | ~enable) (~soft_override ##1 soft_override [*4] |-> reg_oclk));
 `endif
reg reset_deactivated;
initial reset_deactivated = 0;
always @(posedge iclk) if(resetb) reset_deactivated <= 1;
`ifdef CG_W2E
final if((reset_deactivated == 0) && enable && hier_enable) `MSGIO_DISPLAY("%m ERROR: Reset never deactivated");
`else
final if((reset_deactivated == 0) && enable && hier_enable) `MSGIO_DISPLAY("%m WARNING: Reset never deactivated");
`endif
reg override_disabled;
initial override_disabled = 1'b0;
always @(negedge cg_override or posedge iclk) if(cg_override == 1'b0 ) override_disabled <= 1'b1;
`ifdef CG_W2E
final if((override_disabled == 1'b0) && enable && cgt_en_assert_enable) `MSGIO_DISPLAY("%m ERROR: Master Override never deactivated");
`else
final if((override_disabled == 1'b0) && enable && cgt_en_assert_enable) `MSGIO_DISPLAY("%m WARNING: Master  Override never deactivated");
`endif
reg soft_override_disabled;
initial soft_override_disabled = 1'b0;
always @(posedge iclk) if(soft_override == 1'b0) soft_override_disabled <= 1'b1;
`ifdef CG_W2E
final if((soft_override_disabled == 1'b0) && enable && hier_enable) `MSGIO_DISPLAY("%m ERROR: Soft Override never deactivated");
`else
final if((soft_override_disabled == 1'b0) && enable && hier_enable) `MSGIO_DISPLAY("%m WARNING: Soft Override never deactivated");
`endif
reg vld_deasserted;
initial vld_deasserted = 1'b0;
always @(posedge iclk) if (reg_oclk_vld == 1'b0) vld_deasserted <= 1'b1;
final if (vld_deasserted == 1'b0 && enable && cgt_en_assert_enable)
`ifdef CG_VLD_W2E
	`MSGIO_DISPLAY("%m ERROR:CG_VLD checker: clk never turn off, check clock gating setting");
`else
	`MSGIO_DISPLAY("%m WARNING:CG_VLD checker: clk never turn off, check clock gating setting");
`endif
// master override is set after reset until it is cleared by chip_config or
// rlc ucode
always @ (posedge resetb)
begin
  if (enable && hier_enable)
  begin
    repeat(3) @(posedge iclk);
    if (cg_override == 0 && soft_override==0)
`ifdef CG_W4E
      `MSGIO_DISPLAY("%m ERROR: Master/Soft Override should be set after reset");
`else
      `MSGIO_DISPLAY("%m WARNING: Master/Soft Override should be set after reset");
`endif
  end
end
// soft override should be cleared before the end of test
`ifdef CG_W2E
final if((soft_override == 1'b1) && enable && hier_enable && cgt_en_assert_enable) `MSGIO_DISPLAY("%m ERROR: Soft Override is set at the end of test");
`else
final if((soft_override == 1'b1) && enable && hier_enable && cgt_en_assert_enable) `MSGIO_DISPLAY("%m WARNING: Soft Override is set at the end of test");
`endif
`ifdef CGT_FCOV
covergroup cgtt_local_rcg_cg_t @(negedge iclk);
    reg_oclk_cp : coverpoint reg_oclk iff(resetb)
    {
        bins num_clocks_reg_oclk_inactive = {0};
        bins num_clocks_reg_oclk_active   = {1};
    }
endgroup //cgtt_rcg_local_cg_t
cgtt_local_rcg_cg_t cgtt_local_rcg_cg = new;
`endif // CGT_FCOV
`else
initial begin
  $display("%m at %0t: ld/src/misc/cgtt_local_rcg_cov.sv: SVA not yet supported in NCSIM in wekiva scripts", $realtime);
end
`endif // VCS
//VCS coverage on
// clock efficiency monitor
`ifdef CLK_EFF_MONITOR
    reg[31:0] clocks_counter;
        initial
            clocks_counter = 0;
    always @(posedge reg_oclk)
        clocks_counter <= clocks_counter + 1;
    final `MSGIO_DISPLAY("%m: clock gating efficiency check : clock counter = %d",clocks_counter);
`endif
`ifdef CG_SIGNALS_DUMP_EN
initial
begin
  #1;
  $fmonitor(tb.cg_file_handle, "CG CGTT_RCG_OV:(%t) %m %b",$time,cg_override);
  $fmonitor(tb.cg_file_handle, "CG CGTT_RCG_SOFT_OV:(%t) %m %b",$time,soft_override);
  $fmonitor(tb.cg_file_handle, "CG CGTT_RCG_RST:(%t) %m %b",$time,resetb);
end
`endif
endmodule
