module clk_divide 
# (
parameter width = 4
) 
(
input   wire	clk_in,
output  wire	clk_out,
input   wire	en,
input   wire	byp_dfs,
input   wire [width-1:0] clk_divider
);


wire clk_div_0_or_1;
wire clkEnA_muxed;

reg [width-1:0]   cnt;
wire [width-1:0]  div_2;
reg clkEnA_ff;
reg clkEnA;
reg clkEnB;

 //Div/1
 assign clk_div_0_or_1 =  (clk_divider[width-1:0] == {(width){1'b0}} || clk_divider[width-1:0] == {{(width-1){1'b0}}, 1'b1} );

 assign div_2[width-1:0]   = (clk_divider[width-1:0] >> 1);

// Note: en and clk_divider must be in clk_in domain.
always @(posedge clk_in)
begin
 if(~en) 
  begin
    cnt <= {(width){1'b0}};
    clkEnA      <= 1'b0;
    clkEnB      <= 1'b0;
  end
 else if(clk_div_0_or_1)    //DIV-0/1, Keep one gater open all the time.
  begin
    cnt <= {(width){1'b0}};
    clkEnA      <= 1'b1;
    clkEnB      <= 1'b0;
  end
 else if(cnt == (clk_divider - 1))  //Open gater
  begin
    cnt <= {(width){1'b0}};
    clkEnA      <= 1'b1;
    //clkEnB      <= 1'b1;
    clkEnB      <= ~byp_dfs;    //Keep clk_gaterB close all the time when DFS mode is not used. 
  end
 else if( (cnt == (div_2 - 1)) | byp_dfs )    //Close gater
  begin
    cnt <= cnt + 1'b1;
    clkEnA      <= 1'b0;
    clkEnB      <= 1'b0;
  end
 else
  begin
    cnt <= cnt + 1'b1;
    clkEnA      <= clkEnA; 
    clkEnB      <= clkEnB;
  end
end

always @(posedge clk_in)
 begin
  clkEnA_ff <= clkEnA;
 end

//50% duty cycle.
//bit-0 = 1, odd divider
//bit-0 = 0, even divider
//odd divider, extend clkAEn for one cycle
assign clkEnA_muxed = (clk_divider[0] & ~byp_dfs) ? (clkEnA_ff|clkEnA) : clkEnA;
//assign clkEnA_muxed = clkEnA;

//Note: Carefull PD P&R needed on below instance. No extra cells must be inserted between any elements of below instance.
   dfx_ckgate dfx_ckgate 
   (
    .clkIn(clk_in),
    .clkEn_AR(clkEnA_muxed),
    .clkEn_BR(clkEnB),
    .clkOut(clk_out)
    );

endmodule
