//                              -*- Mode: Verilog -*-

////////////////////////////////////////////////////////////////////////////
/*
 *
 *  Trade secret of ATI Technologies, Inc.
 *  Copyright 2005, ATI Technologies, Inc., (unpublished)
 *
 *  All rights reserved.  This notice is intended as a precaution against
 *  inadvertent publication and does not imply publication or any waiver
 *  of confidentiality.  The year included in the foregoing notice is the
 *  year of creation of the work.
 *
 */
////////////////////////////////////////////////////////////////////////////
//
// Filename    : clk_observe.v
// Description : clock observe
//
////////////////////////////////////////////////////////////////////////////

// MAX allowed SEL_WIDTH = 3.
module clk_observe_gc10 
# (
parameter SEL_WIDTH = 3,
parameter BYPASS_DIVIDER = 0
)
(
//Broadcast to All TDRs
 input wire Tdr_Tck,
 input wire Tdr_Trstb,
 input wire Tdr_Shift,
 input wire Tdr_Capture,
 input wire Tdr_Update,
 
 //Tdr
 input wire Tdr_clk_obs_Tdi,
 input wire Tdr_clk_obs_En,
 output wire Tdr_clk_obs_Tdo,

input wire Clk_0,
input wire Clk_1,
input wire Clk_2,
input wire Clk_3,
input wire Clk_4,
input wire Clk_5,
input wire Clk_6,
input wire Clk_7,
input wire Clk_8,
input wire Clk_9,
input wire Clk_10,
input wire Clk_11,
input wire Clk_12,
input wire Clk_13,
input wire Clk_14,
input wire Clk_15,

input wire Clk_Daisy_0_In,
input wire Clk_Daisy_1_In,
output wire Clk_Daisy_0_Out,
output wire Clk_Daisy_1_Out,
output wire Clk_Out

);

///////////////////////////////////////////////////////////////////
wire [5-1:0] divider;
wire [1-1:0] byp_dfs;
wire [3-1:0] sel_clk_0;
wire [3-1:0] sel_clk_1;
wire [1-1:0] sel_clk_out;
wire [1-1:0] sel_daisy_clk;
clk_obs_tdr_gc10 clk_obs_tdr ( 

 .Tdr_Tck               (Tdr_Tck) ,
 .Tdr_Trstb             (Tdr_Trstb),
 .Tdr_Tdi               (Tdr_clk_obs_Tdi),
 .Tdr_Shift             (Tdr_Shift),
 .Tdr_Capture           (Tdr_Capture),
 .Tdr_Update            (Tdr_Update)


//TDR --> CLK_OBS
 ,.divider               (divider[5-1:0])
 ,.byp_dfs               (byp_dfs[1-1:0])
 ,.sel_clk_0             (sel_clk_0[3-1:0])
 ,.sel_clk_1             (sel_clk_1[3-1:0])
 ,.sel_clk_out           (sel_clk_out[1-1:0])
 ,.sel_daisy_clk                 (sel_daisy_clk[1-1:0])
 ,.Tdr_clk_obs_En               (Tdr_clk_obs_En)
 ,.Tdr_clk_obs_Tdo              (Tdr_clk_obs_Tdo)
); 

///////////////////////////////////////////////////////////////////

dft_clk_obs_gc10  
#(.SEL_WIDTH(SEL_WIDTH), .BYPASS_DIVIDER(BYPASS_DIVIDER) )
dft_clk_obs_0
            (
            .Tdr_Tck            (Tdr_Tck),
            .Tdr_Trstb          (Tdr_Trstb),
            .Tdr_Update         (Tdr_Update),
            .Tdr_clk_obs_En     (Tdr_clk_obs_En),
            .Clk_0              (Clk_0),
            .Clk_1              (Clk_1),
            .Clk_2              (Clk_2),
            .Clk_3              (Clk_3),
            .Clk_4              (Clk_4),
            .Clk_5              (Clk_5),
            .Clk_6              (Clk_6),
            .Clk_7              (Clk_7),
            .divider            (divider[4:0]),
            .byp_dfs            (byp_dfs[1-1:0]),
            .sel_clk            (sel_clk_0[2:0]), 
            .sel_daisy_clk      (sel_daisy_clk),
            .Clk_Daisy_In       (Clk_Daisy_0_In),
            .Clk_Daisy_Out      (Clk_Daisy_0_Out)
            );

dft_clk_obs_gc10 
#(.SEL_WIDTH(SEL_WIDTH), .BYPASS_DIVIDER(BYPASS_DIVIDER) )
dft_clk_obs_1
            (
            .Tdr_Tck            (Tdr_Tck),
            .Tdr_Trstb          (Tdr_Trstb),
            .Tdr_Update         (Tdr_Update),
            .Tdr_clk_obs_En     (Tdr_clk_obs_En),
            .Clk_0              (Clk_8),
            .Clk_1              (Clk_9),
            .Clk_2              (Clk_10),
            .Clk_3              (Clk_11),
            .Clk_4              (Clk_12),
            .Clk_5              (Clk_13),
            .Clk_6              (Clk_14),
            .Clk_7              (Clk_15),
            .divider            (divider[4:0]),
            .byp_dfs            (byp_dfs[1-1:0]),
            .sel_clk            (sel_clk_1[2:0]), 
            .sel_daisy_clk      (sel_daisy_clk),
            .Clk_Daisy_In       (Clk_Daisy_1_In),
            .Clk_Daisy_Out      (Clk_Daisy_1_Out)
            );

            
            aticlk_mux2 mrk_clk_out (
                                .clk_o(Clk_Out ),
                                .clk_i({
                                        Clk_Daisy_1_Out,
                                        Clk_Daisy_0_Out
                                        }),
                                .sel(sel_clk_out[0])
                                );
            

endmodule
