// Included in this file, clkds_40clkpicker_leafcells.v, are behavoral models for
// custom leaf cells used in clkds_40clkpicker.v

// Not included in this file are those cells from standard cell libraries, such as

// wiremodel_mac from amd_simlib
// amd_simlib /proj/dk/technology/GF/C14nm/14LPP/11M_3Mx_4Cx_2Kx_2Gx_LB/production/amdPDK/amd_simlib

// all other logic gates, latches, flip-flops from following libraries
// hdcdccell /proj/csmba/users/snps_libs/STDCELL_GF14LPE_pPDK/HDCDC/v02.02_p1.0.2.0/hdcdccell
// hdckcell /proj/csmba/users/snps_libs/STDCELL_GF14LPE_pPDK/HDCK/v02.00_p1.0.2.0_PPDK_20140828/hdckcell
// hdcmbcell /proj/csmba/users/snps_libs/STDCELL_GF14LPE_pPDK/HDCMB/v02.02_p1.0.2.0/hdcmbcell
// hdcoremvcell /proj/csmba/users/snps_libs/STDCELL_GF14LPE_pPDK/HDCOREMV/v02.02_p1.0.2.0/hdcoremvcell
// hdecocell /proj/csmba/users/snps_libs/STDCELL_GF14LPE_pPDK/HDECO/v02.02_p1.0.2.0/hdecocell
// hdiohlmvcell /proj/csmba/users/snps_libs/STDCELL_GF14LPE_pPDK/HDIOHLMV/v02.02_p1.0.2.0/hdiohlmvcell
// hdiolhmvcell /proj/csmba/users/snps_libs/STDCELL_GF14LPE_pPDK/HDIOLHMV/v02.02_p1.0.2.0/hdiolhmvcell
// hdpdffillcell /proj/csmba/users/snps_libs/STDCELL_GF14LPE_pPDK/HDPDFFILL/v02.02_p1.0.2.0/hdpdffillcell
// hdreghlmvcell /proj/csmba/users/snps_libs/STDCELL_GF14LPE_pPDK/HDREGHLMV/v02.02_p1.0.2.0/hdreghlmvcell
// hdreglhmvcell /proj/csmba/users/snps_libs/STDCELL_GF14LPE_pPDK/HDREGLHMV/v02.02_p1.0.2.0/hdreglhmvcell
// hdseqcell /proj/csmba/users/snps_libs/STDCELL_GF14LPE_pPDK/HDSEQ/v02.02_p1.0.2.0/hdseqcell

// Please include verilog models for above standard cell libraries separately at simulation compilation time.


// ========================================================
// clkds_40clkpicker custom leaf cell module declaration starts here
// ========================================================

//Verilog HDL for "gf14lpe_clkds", "clkds_gcknand2_gpu" "verilog_leaf"
`timescale 1ps/1ps
module clkds_gcknand2_gpu ( A, B, VDD0P9, VSS, Y );

  input A;
  output Y;
  input VSS;
  input VDD0P9;
  input B;

wire pg_good = VDD0P9 & ~VSS;

//Transport delay
reg temp;
always @ *
 temp <= #6 pg_good ? ~(A & B) : 1'bx;

assign Y = temp;

endmodule // clkds_gcknand2_gpu

// clkds_skew_detect
`timescale 1ps/1ps
module clkds_skew_detect (
    inp,
    inn,
    enable,
    msg,
    skew_detected
    );

input inp;
input inn;
input enable;
input msg;
output skew_detected;

parameter skew_limit = 10;
time negedge_time;
wire xor_inp_inn;
reg reg_skew_detected;
reg msg_displayed;

assign skew_detected = enable ? reg_skew_detected : 1'b0;

assign xor_inp_inn = inp ^ inn;
initial begin
    negedge_time = 0;
    reg_skew_detected = 1'b0;
    msg_displayed = 1'b0;
end
always @(negedge xor_inp_inn) begin
    negedge_time = $time;
end
always begin
    wait (~xor_inp_inn);
    @(posedge xor_inp_inn) begin
      if (xor_inp_inn !== 1'bx && enable===1'b1) begin
        if (($time - negedge_time) > skew_limit) begin
            reg_skew_detected <= 1'b1;
            if (msg && ~msg_displayed) begin
                $display("ERROR(%m): TIME(%0t) Skew detected between inp and inn\n", $time);
                msg_displayed <= 1'b1;
            end
        end
        else
            reg_skew_detected <= 1'b0;
      end //if (xor_inp_inn !== 1'bx && enable===1'b1) begin
    end
end

endmodule // clkds_skew_detect

//Verilog HDL for "gf14lpe_clkds", "clkds_clkpickerDFFR" "verilog_leaf"
`timescale 1ps/1ps
module clkds_clkpickerDFFR ( D, VSS, rstB, VDD, qB, q, ck, ckB );

  output q;
  output qB;
  input ckB;
  input VDD;
  input rstB;
  input VSS;
  input D;
  input ck;

wire pg_good = VDD & ~VSS;

reg q;
assign qB = ~q;

always @(posedge ck or negedge rstB or negedge pg_good) begin
    if (~pg_good)
        q <= 1'bx;
    else if (~rstB)
    #6  q <= 1'b0;
    else
    #12 q <= D;
end

`ifndef HWEMUL
// leda off
always @ (posedge VDD) begin
   force q = 1'b0;
   #1;
   release q;
end
// leda on
`endif //`ifndef HWEMUL

`ifdef CLKDS_SYNTHESIS
wire skew_detected = 1'b0;
`else
`define CLKDS_CHK_SKEW_ERROR 1'b1
wire skew_detected;
clkds_skew_detect u_skew_detect (
    .inp(ck),
    .inn(ckB),
    .enable(rstB),
    .msg(`CLKDS_CHK_SKEW_ERROR),
    .skew_detected(skew_detected)
    );
`endif

endmodule // clkds_clkpickerDFFR

//Verilog HDL for "gf14lpe_clkds", "clkds_ckmux2" "verilog_leaf"
`timescale 1ps/1ps
module clkds_ckmux2 ( ock, ck0, ck1, sel, selB, VDD, VSS );

  input ck0;
  input selB;
  input ck1;
  output ock;
  input VDD;
  input sel;
  input VSS;

reg reg_ock;
wire pg_good = VDD & ~VSS;

assign #2 ock = pg_good ? reg_ock : 1'bx;

always @(*) begin
    if (sel & ~selB)
        reg_ock = ck1;
    else if (~sel & selB)
        reg_ock = ck0;
    else
        reg_ock = 1'bx;
end

endmodule // clkds_ckmux2

//Verilog HDL for "gf14lpe_clkds", "clkds_mux3to1_1ur" "verilog_leaf"
`timescale 1ps/1ps
module clkds_mux3to1_1ur ( D00, D01, D1X, S0, S0B, S1, S1B, VDD, VSS, Z );

  input S1;
  input S0B;
  input VDD;
  input S1B;
  input S0;
  input D00;
  output Z;
  input D01;
  input VSS;
  input D1X;

wire pg_good = VDD & ~VSS;

wire preZ = (S1  & ~S1B) ? D1X :       // 1x
            (S0B & S1B)  ? D00 :       // 00
            (S0  & S1B)  ? D01 :       // 01
                           1'bx;

assign #4 Z = pg_good ? preZ : 1'bx;

endmodule // clkds_mux3to1_1ur

//Verilog HDL for "gf14lpe_clkds", "clkds_mux3to1_1ul" "verilog_leaf"
`timescale 1ps/1ps
module clkds_mux3to1_1ul ( D00, D01, D1X, S0, S0B, S1, S1B, VDD, VSS, Z );

  input S1;
  input S0B;
  input VDD;
  input S1B;
  input S0;
  input D00;
  output Z;
  input D01;
  input VSS;
  input D1X;

wire pg_good = VDD & ~VSS;

wire preZ = (S1  & ~S1B) ? D1X :       // 1x
            (S0B & S1B)  ? D00 :       // 00
            (S0  & S1B)  ? D01 :       // 01
                           1'bx;

assign #8 Z = pg_good ? preZ : 1'bx;

endmodule // clkds_mux3to1_1ul

//Verilog HDL for "gf14lpe_clkds", "clkds_mux3to1_2ul" "verilog_leaf"
`timescale 1ps/1ps
module clkds_mux3to1_2ul ( D00, D01, D1X, S0, S0B, S1, S1B, VDD, VSS, Z );

  input S1;
  input S0B;
  input VDD;
  input S1B;
  input S0;
  input D00;
  output Z;
  input D01;
  input VSS;
  input D1X;

wire pg_good = VDD & ~VSS;

wire preZ = (S1  & ~S1B) ? D1X :       // 1x
            (S0B & S1B)  ? D00 :       // 00
            (S0  & S1B)  ? D01 :       // 01
                           1'bx;

assign #6 Z = pg_good ? preZ : 1'bx;

endmodule // clkds_mux3to1_2ul

//Verilog HDL for "gf14lpe_clkds", "clkds_mux2to1" "verilog_leaf"
`timescale 1ps/1ps
module clkds_mux2to1 ( D0, D1, S0, S0B, VDD, VSS, Z );

  input D1;
  input D0;
  input S0B;
  input VDD;
  input S0;
  output Z;
  input VSS;

wire pg_good = VDD & ~VSS;

wire preZ = (S0 & ~S0B) ? D1 :
            (~S0 & S0B) ? D0 :
            1'bx;

assign #6 Z = pg_good ? preZ : 1'bx;

endmodule // clkds_mux2to1

//Verilog HDL for "gf14lpe_clkds", "clkds_mux2x4ul" "verilog_leaf"
`timescale 1ps/1ps
module clkds_mux2x4ul ( D0, D1, S0, S0B, VDD, VSS, Z );

  input D1;
  input D0;
  input S0B;
  input VDD;
  input S0;
  output Z;
  input VSS;

wire pg_good = VDD & ~VSS;

wire preZ = (S0 & ~S0B) ? D1 :
            (~S0 & S0B) ? D0 :
            1'bx;

assign #2 Z = pg_good ? preZ : 1'bx;

endmodule // clkds_mux2x4ul


//Verilog HDL for "gf14lpe_clkds", "clkds_cknand2_gpu" "verilog_leaf"
`timescale 1ps/1ps
module clkds_cknand2_gpu ( A, B, VDD0P9, VSS, Y );

  input A;
  output Y;
  input VSS;
  input VDD0P9;
  input B;

wire pg_good = VDD0P9 & ~VSS;

//Transport delay
reg temp;
always @ *
 temp <= #8 pg_good ? ~(A & B) : 1'bx;

assign Y = temp;

endmodule // clkds_cknand2_gpu

//Verilog HDL for "gf14lpe_clkds", "clkds_cknor2" "verilog_leaf"
`timescale 1ps/1ps
module clkds_cknor2 ( A, B, VDD0P9, VSS, Y );

  input A;
  output Y;
  input VSS;
  input VDD0P9;
  input B;

wire pg_good = VDD0P9 & ~VSS;

//Transport delay
reg temp;
always @ *
 temp <= #8 pg_good ? ~(A | B) : 1'bx;

assign Y = temp;

endmodule // clkds_cknor2
