`timescale 1ps/1ps

module clkds_ldo ( ldo_en18, VDDPLL,VDD, VSS, cntl, LDO_OFF, comp_lo, comp_hi, comp_en18, ldo_bias_sel, VDDREG);

  output comp_hi;
  input  [3:0] cntl;
  input ldo_en18;
  output comp_lo;
  input VDDPLL;
  input VDD;
  input comp_en18;
  input ldo_bias_sel;
  input LDO_OFF;
  input VSS;
  output VDDREG;

`define CLKDS_LDO_RAMP_UP_TIME 50000
`define CLKDS_LDO_STEP_TIME 10000

wire VDDREG_del;
wire LDO_READY_del;
wire LDO_READY;
reg reg_comp_hi;
reg reg_comp_lo;
wire pbias ;
//wire [3:0] thres_hi = (cntl_buf>=4'he) ? 4'hf  : cntl_buf + 4'h2;
//wire [3:0] thres_lo = (cntl_buf<=4'h1) ? 4'h0  : cntl_buf - 4'h2;

wire [3:0] cntl_buf;
wire [4:0] cntl_buf_offset = {1'b0,cntl_buf} + 5'd4;

wire [4:0] thres_hi = cntl_buf_offset + 5'd2; // thres_hi = cntl[3:0] + 6
wire [4:0] thres_lo = cntl_buf_offset - 5'd2; // thres_lo = cntl[3:0] + 2

reg [4:0] VDDREG_val;
initial VDDREG_val <= 0;

`ifdef CLKDS_LDO_OVERSHOOT_1
`else
always @(*) begin
    if (VDDPLL & ldo_en18) begin
        //if (VDDREG_val > cntl_buf)
        if (VDDREG_val > cntl_buf_offset)
            #`CLKDS_LDO_STEP_TIME VDDREG_val <= VDDREG_val - 1;
        //else if (VDDREG_val < cntl_buf)
        else if (VDDREG_val < cntl_buf_offset)
            #`CLKDS_LDO_STEP_TIME VDDREG_val <= VDDREG_val + 1;
        else
            #`CLKDS_LDO_STEP_TIME VDDREG_val <= VDDREG_val;
    end
    else begin
        VDDREG_val <= 0;
    end
end
`endif

always @(*) begin
    if (VDDREG_val>thres_lo && VDDREG_val<thres_hi) begin
    //if (VDDREG_val>=thres_lo && VDDREG_val<=thres_hi) begin
        reg_comp_hi <= 0;
        reg_comp_lo <= 1;
    end
    else if (VDDREG_val <= thres_lo) begin
    //else if (VDDREG_val < thres_lo) begin
        reg_comp_hi <= 0;
        reg_comp_lo <= 0;
    end
    else if (VDDREG_val >= thres_hi) begin
    //else if (VDDREG_val > thres_hi) begin
        reg_comp_hi <= 1;
        reg_comp_lo <= 1;
    end
    else begin
        reg_comp_hi <= 1'bx;
        reg_comp_lo <= 1'bx;
    end
end

reg VDDREG_temp;
assign pbias = ~VDDREG_temp | (VDDREG===1'bz);

always @(*) begin
  if (VDDREG_temp !== 1'b1) 
    //Initially when VDDREG ramps up, this equation holds true
    VDDREG_temp = (VDDREG_val<thres_hi) & (VDDREG_val>thres_lo) & (^cntl_buf[3:0]!==1'bx);
  else
    //Once VDDREG has ramped up, changes in cntl[3:0] shouldn't drive 
    //VDDREG to 0, but only the voltage levels change 
    //VDDREG=0 only when staticEnable=ldo_en18=0 or VDDPLL=0
    VDDREG_temp = ((VDDREG_val<thres_hi) & (VDDREG_val>thres_lo) & (^cntl_buf[3:0]!==1'bx)) || (VDDREG_val != 5'b0) ;
end


assign comp_hi = comp_en18 & reg_comp_hi & (^cntl_buf[3:0]!==1'bx);
assign comp_lo = comp_en18 & reg_comp_lo & (^cntl_buf[3:0]!==1'bx);


mbclkds_CDCBUF I_cntl_0 (
           `ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
           .DSTCLK(1'b0),
           `endif
           .A(cntl[0]), 
           .Z(cntl_buf[0]) );

mbclkds_CDCBUF I_cntl_1 (
           `ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
           .DSTCLK(1'b0),
           `endif
           .A(cntl[1]), 
           .Z(cntl_buf[1]) );

mbclkds_CDCBUF I_cntl_2 (
           `ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
           .DSTCLK(1'b0),
           `endif
           .A(cntl[2]), 
           .Z(cntl_buf[2]) );

mbclkds_CDCBUF I_cntl_3 (
           `ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
           .DSTCLK(1'b0),
           `endif
           .A(cntl[3]), 
           .Z(cntl_buf[3]) );


assign VDDREG  =    LDO_OFF                      ? VDD    :
                  (~LDO_OFF & ~pbias & ldo_en18) ? VDDPLL :
                  1'b0;

endmodule

