//                              -*- Mode: Verilog -*-

////////////////////////////////////////////////////////////////////////////
/*
 *
 *  Trade secret of ATI Technologies, Inc.
 *  Copyright 2005, ATI Technologies, Inc., (unpublished)
 *
 *  All rights reserved.  This notice is intended as a precaution against
 *  inadvertent publication and does not imply publication or any waiver
 *  of confidentiality.  The year included in the foregoing notice is the
 *  year of creation of the work.
 *
 */
////////////////////////////////////////////////////////////////////////////
//`include "dft_clk_cntl_features.vh"


module clone_for_mpcts_gc10 
  (
   input wire Func_Clk,
   output wire Func_Clk_Muxed,
   input wire Tile_Shift_Clk,
   input wire Stuck_At_Mode_Cg_En,
   input wire At_Speed_Mode_Cg_En,
   input wire mux_sel,
   input wire SE_Sel,
   input wire mode_sel_ssb_inv
   
   );
   
   wire   Func_Clk_Gated;
   wire   Tile_Shift_Clk_Muxed;
   wire   Tile_Shift_Clk_Inv; 
   reg 	  gater_en_ff;
   reg 	  mux_sel_ff;
   
   
   always @ (posedge Func_Clk)
     begin
	// mission mode- 0
	gater_en_ff <= At_Speed_Mode_Cg_En;    
     end
   
   assign Cg_En = gater_en_ff | Stuck_At_Mode_Cg_En;
   
   ati_clock_gate tile_root_DfxCLOCKGATER
     (.clk_out   (Func_Clk_Gated),
      .clk_src   (Func_Clk),
      .clk_en    (Cg_En),   
      .TE        (1'b0));
   
   always @ (negedge Tile_Shift_Clk)
     begin
	  mux_sel_ff <= mux_sel;
     end
   
   ati_mux2 mux_clk_sel  
     (
      .i0(mux_sel),
      .i1(mux_sel_ff | mux_sel), // To set the mux to 1 asycn when Se=1, de-assertion control through flop  
      .s(SE_Sel),
      .z(func_shift_clk_mux_sel));
   
   
   ati_clk_inv d0nt_shift_clk_inv ( .clk_out(Tile_Shift_Clk_Inv), .clk_in(Tile_Shift_Clk));
   
   aticlk_mux2 d0nt_ssb_clkmux 
     (
      .clk_o(Tile_Shift_Clk_Muxed),
      .clk_i({Tile_Shift_Clk_Inv, Tile_Shift_Clk}),
      .sel(mode_sel_ssb_inv)
      );
   
   aticlk_mux2 d0nt_scan_clkmux 
     (
      .clk_o(Func_Clk_Muxed),
      .clk_i({Tile_Shift_Clk_Muxed, Func_Clk_Gated}),
      .sel(func_shift_clk_mux_sel)
      );
   ////////////////////////////////////////////////////////////////////////////
     
endmodule // clone_for_mpcts


