module counter_divide_gc10 
# (
parameter hi_cnt_width = 20,
parameter mid_cnt_width = 20,
parameter low_cnt_width = 20
) (
input  wire clk,
input  wire rstb,	// active low, should have to be sync on clk
output wire rstb_sync,
input  wire load,	// load the counter value
input  wire enable,	// Enable the counter
input  wire stop,	// stop the counter
input  wire [hi_cnt_width-1:0] hi_cnt_val,
input  wire [mid_cnt_width-1:0] mid_cnt_val,
input  wire [low_cnt_width-1:0] low_cnt_val,
output wire [hi_cnt_width-1:0] hi_cnt_out,
output wire [mid_cnt_width-1:0] mid_cnt_out,
output wire [low_cnt_width-1:0] low_cnt_out,
output wire count_done
);

wire load_sync;
reg  load_sync_p1;
wire load_pulse;
reg  load_pulse_p1;
wire enable_sync;
reg  enable_sync_p1;
wire stop_sync;
reg  stop_sync_p1;

reg [hi_cnt_width-1:0] hi_cnt;
reg [mid_cnt_width-1:0] mid_cnt;
reg [low_cnt_width-1:0] low_cnt;

reg hi_sat;
reg mid_sat;
reg low_sat;
reg hi_enable;
reg mid_enable;
///////////////////////////// rstb /////////////////////////////////
    dfx_async_rst_sync3_gc10 rst_sync_rstb (
      .clk      (clk),
      .rstinB   (rstb),
      .rstoutB  (rstb_sync)
     );

///////////////////////////// load /////////////////////////////////
// load_pulse

   dfx_sync3_gc10 
   sync_load
     (.CK(clk),
      .D(load),
      .Q(load_sync));

   always @ (posedge clk)
     if (~rstb_sync)
       load_sync_p1 <= 1'b0;
     else
       load_sync_p1 <= load_sync;

   // edge detect of load
   assign        load_pulse = ~load_sync_p1 && load_sync;

   always @ (posedge clk)
     if (~rstb_sync)
       load_pulse_p1 <= 1'b0;
     else
       load_pulse_p1 <= load_pulse;

///////////////////////////// enable /////////////////////////////////

     dfx_sync3_gc10 
     sync_enable
     (.CK(clk),
      .D(enable),
      .Q(enable_sync));

   always @ (posedge clk)
     if (~rstb_sync)
       enable_sync_p1 <= 1'b0;
     else
       enable_sync_p1 <= enable_sync;


///////////////////////////// stop /////////////////////////////////

    dfx_sync3_gc10 
    sync_stop
     (.CK(clk),
      .D(stop),
      .Q(stop_sync));


   always @ (posedge clk)
     if (~rstb_sync)
       stop_sync_p1 <= 1'b0;
     else
    stop_sync_p1 <= stop_sync;


///////////////////////////// counter /////////////////////////////////

assign hi_cnt_out[hi_cnt_width-1:0]    = hi_cnt[hi_cnt_width-1:0];
assign mid_cnt_out[mid_cnt_width-1:0]   = mid_cnt[mid_cnt_width-1:0];
assign low_cnt_out[low_cnt_width-1:0]  = low_cnt[low_cnt_width-1:0];

   // hi counter
   always @ (posedge clk)
     if (load_pulse | ~rstb_sync) begin
       hi_cnt[hi_cnt_width-1:0] <= hi_cnt_val[hi_cnt_width-1:0];
       hi_sat       <= 1'b0;
     end else if (stop_sync_p1) begin
       hi_cnt[hi_cnt_width-1:0] <= hi_cnt[hi_cnt_width-1:0];
       hi_sat       <= hi_sat;
     end else if (hi_cnt[hi_cnt_width-1:0] == {(hi_cnt_width){1'b0}}) begin
       hi_cnt[hi_cnt_width-1:0] <= hi_cnt[hi_cnt_width-1:0];
       hi_sat       <= 1'b1;
     end else if (hi_enable) begin
       hi_cnt[hi_cnt_width-1:0] <= hi_cnt[hi_cnt_width-1:0] - 1'b1;
       hi_sat       <= 1'b0;
     end else begin
       hi_cnt[hi_cnt_width-1:0] <= hi_cnt[hi_cnt_width-1:0];
       hi_sat       <= 1'b0;
     end


  // mid counter
   always @ (posedge clk)
     if (load_pulse | ~rstb_sync) begin
       mid_cnt[mid_cnt_width-1:0]  <= mid_cnt_val[mid_cnt_width-1:0];
       hi_enable      <= 1'b0;
       mid_sat        <= 1'b0;
     end else if (stop_sync_p1) begin
       mid_cnt[mid_cnt_width-1:0]  <= mid_cnt[mid_cnt_width-1:0];
       mid_sat        <= mid_sat;
       hi_enable      <= hi_enable;
     end else if (mid_cnt[mid_cnt_width-1:0] == {(mid_cnt_width){1'b0}}) begin
       if (hi_sat == 1'b1) begin
         mid_cnt[mid_cnt_width-1:0]  <= mid_cnt[mid_cnt_width-1:0];
         mid_sat        <= 1'b1;
         hi_enable      <= 1'b0;
       end else begin
         mid_cnt[mid_cnt_width-1:0]  <= mid_enable ? (mid_cnt[mid_cnt_width-1:0] - 1'b1)
						   : mid_cnt[mid_cnt_width-1:0];
         mid_sat        <= 1'b0;
         hi_enable      <= mid_enable;
       end
     end else if (mid_enable) begin
       mid_cnt[mid_cnt_width-1:0]  <= mid_cnt[mid_cnt_width-1:0] - 1'b1;
       mid_sat        <= 1'b0;
       hi_enable      <= 1'b0;
     end else begin
       mid_cnt[mid_cnt_width-1:0]  <= mid_cnt[mid_cnt_width-1:0];
       mid_sat        <= 1'b0;
       hi_enable      <= 1'b0;
     end


   // low counter
   always @ (posedge clk)
     if (load_pulse | ~rstb_sync ) begin
       low_cnt[low_cnt_width-1:0]  <= low_cnt_val[low_cnt_width-1:0];
       mid_enable     <= 1'b0;
       low_sat        <= 1'b0;
     end else if (stop_sync_p1) begin
       low_cnt[low_cnt_width-1:0]  <= low_cnt[low_cnt_width-1:0];
       low_sat        <= low_sat;
       mid_enable     <= mid_enable;
     end else if ( (low_cnt[low_cnt_width-1:0] == {(low_cnt_width){1'b0}}) & enable_sync_p1 ) begin

       if ((mid_sat == 1'b1) && (hi_sat == 1'b1)) begin	  //mid_sat and hi_sat set to 0 on rstb. when rstb de-asserted, if mid/hi counter 0 then mid/hi_sat = 1

       //All 3 counter saturate 
         low_cnt[low_cnt_width-1:0]  <= low_cnt[low_cnt_width-1:0];
         mid_enable     <= 1'b0;
         low_sat        <= 1'b1;    // all 3 counter saturate. Count_done.
       end else begin
         //low_cnt[low_cnt_width-1:0]  <= enable_sync ? (low_cnt[low_cnt_width-1:0] - 1'b1)
		//				    : low_cnt[low_cnt_width-1:0];	    // Start low count when enable_sync = 1
         low_cnt[low_cnt_width-1:0]  <=  low_cnt[low_cnt_width-1:0] - 1'b1;// Start low count when enable_sync = 1
         mid_enable     <= ~load_pulse_p1;  // Enable mid_cnt after load is complete
         low_sat        <= 1'b0;
       end

     end else if (enable_sync_p1) begin
       low_cnt[low_cnt_width-1:0]  <= low_cnt[low_cnt_width-1:0] - 1'b1;
       mid_enable     <= 1'b0;
       low_sat        <= 1'b0;
     end else begin // When enable de-asserted, counter keep the value and restart from that point when enable assert again.
       low_cnt[low_cnt_width-1:0]  <= low_cnt[low_cnt_width-1:0];
       low_sat        <= low_sat;
       mid_enable     <= mid_enable;
     end

assign count_done = low_sat;

endmodule
