// ==============================================================================================
// PLEASE DONOT CHANGES THIS FILE
// THIS FILE IS AUTO_GENERATED
// Source: import/address/src/rtl/gen_one/db_addr_calc.v
// ===============================================================================================
// Filename              : db_10xx_addr_calc.v
// Perforce changelist   : $Change$
//
// Copyright             : Trade secret of AMD, Inc.
//                         Copyright 2008, AMD, Inc., (unpublished)
//                         All rights reserved.  This notice is intended as a precaution against
//                         inadvertent publication and does not imply publication or any waiver
//                         of confidentiality.  The year included in the foregoing notice is the
//                         year of creation of the work.
//
// RTL author: kevin.gu
//
// Description           : Combine the tile index and the tile offset into a final address while respecting tiling. 
//                         2-pipeline: no register-in, no register-out, 2 middle register

`include "gc__db_reg.v"
`include "addrlib_defines_gc.vh"

module    db_addr_calc_gc(
                              //input
                         sclk,
                         iRTS,
                         oRTR,
                         oRTS,
                         iRTR,
                         pass_thru_data_in,
                         pass_thru_data_out,
                         srst,
                         tile_index_h0,          //input from db_dtile_index_calc module       
                         tile_offset_h0,         //size drived from lagest 32bpe tile
                         pipe_interleave_h0,     //0: 256B, 1:512B
                         baseAddr256B_h0,        //base addr 256B align
                         log2_pixelsize_h0,      //0: 8bpp, 1:16bpp, 2:32bpp, 3:64bpp, 4:128bpp
                         log2_samples_h0,        //the number of samples
                         slice_tile_max_h0,      //in tiles, slice_tiles -1
                         slice_tile_max_log2_h0, //log2(slice_tiles)
                         array_mode_h0,          //as defines
                         tile_split_h0,          //0: 64B split, 1: 128B split, 2: 256B ...
                         num_banks_h0,           //0: 2banks,1:4banks, 2:8banks, 3:16banks
                         pipe_config_h0,         //10xx new added
                            //output
                         address                    //32B-align
                        ,busy
                        ,input_en
                        ,output_en
                      );


input                     sclk;

//input per memory request
input        [ 25:0]      tile_index_h0;
input        [ 10:0]      tile_offset_h0;

//global tiling register settings
input        [  2:0]      pipe_interleave_h0;
input        [  4:0]      pipe_config_h0;     

//surface parameter;
input        [ 31:0]      baseAddr256B_h0;
input        [  2:0]      log2_pixelsize_h0;
input        [  2:0]      log2_samples_h0;
input        [ 21:0]      slice_tile_max_h0;
input        [  4:0]      slice_tile_max_log2_h0;
input        [  3:0]      array_mode_h0;
input        [  2:0]      tile_split_h0;
input        [  1:0]      num_banks_h0;

//supertiling, not needed in this module
output       [ 34:0]      address;                                 //32-byte aligned
output        busy;

input input_en;
output output_en;

// Add this just below the last port
// =-=-=-=-=- RTR/RTS interface support
parameter PASSTHRU_WIDTH 		 = 1;	// Bus Width of the pass through data
parameter COLLAPSABLE_PIPE_STAGE = 1;   // COLLAPSABLE/NONCOLLAPSABLE, 1/0
parameter RI = 1;
parameter RO = 1;
localparam CALC_STAGES = `BC_DB_CALC_STAGES; //number of "core" stages to calculate "outputs"
parameter PAD_STAGES = (`ADDR_FULL_STAGES - CALC_STAGES);

input 						iRTS;
output  					oRTR;
input  [PASSTHRU_WIDTH-1:0]	pass_thru_data_in; //pass through data bus, pipelined PIPE_STAGES and enabled by (<pipe_stage_enable> & ~stall)

output 						oRTS;
input  						iRTR;
output [PASSTHRU_WIDTH-1:0]	pass_thru_data_out;

input                       srst;

localparam TOTAL_PAD_STAGES = RO+PAD_STAGES;
localparam PIPE_STAGES = RI + CALC_STAGES + TOTAL_PAD_STAGES;

wire [PIPE_STAGES-1:0] en_stage;
wire [1:0] load_en;

assign load_en[0] = en_stage[RI];
assign load_en[1] = en_stage[RI+1];

addrlib_px_stage_gc #(.COLLAPSIBLE(COLLAPSABLE_PIPE_STAGE), .WIDTH(PASSTHRU_WIDTH), .DEPTH(PIPE_STAGES))
px_stage(
			  // Outputs
			  .oRTR			(oRTR),
			  .oRTS			(oRTS),
			  .oDATA		(pass_thru_data_out[PASSTHRU_WIDTH-1:0]),
			  .pipe_stage_en (en_stage[PIPE_STAGES-1:0]),
			  .busy 		(busy),
			  // Inputs
			  .iCLK			(sclk),
			  .iRESET		(srst),
			  .iRTS			(iRTS),
			  .iDATA		(pass_thru_data_in[PASSTHRU_WIDTH-1:0]),
			  .iRTR			(iRTR)
              ,.i_en        (input_en)
              ,.o_en        (output_en)
        );
// =-=-=-=-=- End if RTR/RTS interface support

logic         [ 34:0]      address_out, address_out_stages[TOTAL_PAD_STAGES:0], address; 
logic       [-1+40:0]      ByteAddr;

logic [ 25:0]  tile_index, tile_index_reg;
logic [ 10:0]  tile_offset, tile_offset_reg;
logic [  2:0]  pipe_interleave, pipe_interleave_reg;
logic [ 31:0]  baseAddr256B, baseAddr256B_reg;
logic [  2:0]  log2_pixelsize, log2_pixelsize_reg;
logic [  2:0]  log2_samples, log2_samples_reg;
logic [ 21:0]  slice_tile_max, slice_tile_max_reg;
logic [  4:0]  slice_tile_max_log2, slice_tile_max_log2_reg;
logic [  3:0]  array_mode, array_mode_reg;
logic [  2:0]  tile_split, tile_split_reg;
logic [  1:0]  num_banks, num_banks_reg;
logic [  4:0]  pipe_config, pipe_config_reg;
       
always @(posedge sclk)
begin
     if(en_stage[0])
        begin
            tile_index_reg          <=  tile_index_h0;
            tile_offset_reg         <=  tile_offset_h0;
            pipe_interleave_reg     <=  pipe_interleave_h0;
            baseAddr256B_reg        <=  baseAddr256B_h0;
            log2_pixelsize_reg      <=  log2_pixelsize_h0;
            log2_samples_reg        <=  log2_samples_h0;
            slice_tile_max_reg      <=  slice_tile_max_h0;
            slice_tile_max_log2_reg <=  slice_tile_max_log2_h0;
            array_mode_reg          <=  array_mode_h0;
            tile_split_reg          <=  tile_split_h0;
            num_banks_reg           <=  num_banks_h0;
            pipe_config_reg         <=  pipe_config_h0;
        end    
end

//wire instead of reg
assign  tile_index           = (RI)? tile_index_reg          : tile_index_h0;
assign  tile_offset          = (RI)? tile_offset_reg         : tile_offset_h0;
assign  pipe_interleave      = (RI)? pipe_interleave_reg     : pipe_interleave_h0;
assign  baseAddr256B         = (RI)? baseAddr256B_reg        : baseAddr256B_h0;
assign  log2_pixelsize       = (RI)? log2_pixelsize_reg      : log2_pixelsize_h0;
assign  log2_samples         = (RI)? log2_samples_reg        : log2_samples_h0;
assign  slice_tile_max       = (RI)? slice_tile_max_reg      : slice_tile_max_h0;
assign  slice_tile_max_log2  = (RI)? slice_tile_max_log2_reg : slice_tile_max_log2_h0;
assign  array_mode           = (RI)? array_mode_reg          : array_mode_h0;
assign  tile_split           = (RI)? tile_split_reg          : tile_split_h0;
assign  num_banks            = (RI)? num_banks_reg           : num_banks_h0;
assign  pipe_config          = (RI)? pipe_config_reg         : pipe_config_h0;

//rename
wire     [ 1:0]  bank_tiling = num_banks[1:0];
wire     [10:0]  Element_Offset = tile_offset;
wire     [ 0:0]  pipe_interleave_size = pipe_interleave[0];

/////10xx new logic to get pipes//////

   reg    [ 2 :0]   log2pipes;
   always@(*) begin
     case(pipe_config[4:0])
     5'b00000: log2pipes = 3'b001; //2 pipes
     5'b00100: log2pipes = 3'b010; //4 pipes
     5'b00101: log2pipes = 3'b010; //4 pipes
     5'b00110: log2pipes = 3'b010; //4 pipes
     5'b00111: log2pipes = 3'b010; //4 pipes
     5'b01000: log2pipes = 3'b011; //8 pipes
     5'b01001: log2pipes = 3'b011; //8 pipes
     5'b01010: log2pipes = 3'b011; //8 pipes
     5'b01011: log2pipes = 3'b011; //8 pipes
     5'b01100: log2pipes = 3'b011; //8 pipes
     5'b01101: log2pipes = 3'b011; //8 pipes
     5'b01110: log2pipes = 3'b011; //8 pipes
  	 5'b10010: log2pipes = 3'b100; //16 pipes
     //VCS coverage off
     default:  log2pipes = 3'b011; //default to be 8 pipes
     //VCS coverage on
     endcase
   end

   wire [-1+3:0] T21_Log2TileSliceBitsMinus9_B; // [1...7]
   assign T21_Log2TileSliceBitsMinus9_B = log2_samples + log2_pixelsize;

    
   wire [0:0] addr_1D    ;
   // r11xx FIXing: 08/16 assign addr_1D     = (array_mode == `ARRAY_1D_TILED_THIN1) ;
   assign addr_1D     = (array_mode == `ARRAY_1D_TILED_THIN1) | (array_mode == `ARRAY_1D_TILED_THICK);


  //get z from tile_index
  reg     [21:0]    new_tile_index;
  always@(*)
    begin
      case(slice_tile_max_log2)   
	5'd22: new_tile_index = tile_index[21:0];      
        5'd21: new_tile_index = tile_index[20:0];
        5'd20: new_tile_index = tile_index[19:0];
        5'd19: new_tile_index = tile_index[18:0];
        5'd18: new_tile_index = tile_index[17:0];
        5'd17: new_tile_index = tile_index[16:0];
        5'd16: new_tile_index = tile_index[15:0];
        default: new_tile_index = tile_index[14:0];//26-bit - max(11-bit) = 15-bit at least
      endcase	      
    end	    

    wire   [25:0]   index_clear = tile_index ^ {4'h0, new_tile_index};    
    wire   z_0  = index_clear[25];
    wire   z_1  = index_clear[24];
    wire   z_2  = index_clear[23];
    wire   z_3  = index_clear[22];
    wire   z_4  = index_clear[21];
    wire   z_5  = index_clear[20];
    wire   z_6  = index_clear[19];
    wire   z_7  = index_clear[18];
    wire   z_8  = index_clear[17];
    wire   z_9  = index_clear[16];
    wire   z_10 = index_clear[15];

    wire   [10:0]  z = {z_10, z_9, z_8, z_7, z_6, z_5, z_4, z_3, z_2, z_1, z_0};



    wire   [39:0]     total_accum;

////////////////1D mode, thin////////////////////////    
    wire   [39:0]     T_mem_address_t = total_accum;

//////////////////2D mode, thin///////////////////////////
reg     [ 3:0]    pipe_number_0;
reg     [ 3:0]    bank_number_0;
always@(*)
    begin
      case({log2pipes[2:0]}) 
	//not support pipe=1
	//VCS coverage off
        3'b000:
                  pipe_number_0 = {4'h0};
	//VCS coverage on
        3'b001:
                  pipe_number_0 = {3'h0, new_tile_index[0]};
        3'b010:
                  pipe_number_0 = {2'h0, new_tile_index[1:0]};
        3'b011:
                  pipe_number_0 = {1'h0, new_tile_index[2:0]};
        3'b100:
                  pipe_number_0 = {      new_tile_index[3:0]};
      default :
                  pipe_number_0 = {1'h0, new_tile_index[2:0]};
      endcase	      
    end	    
    
always@(*)
    begin
      case({log2pipes[2:0], bank_tiling[1:0]})
	//not support pipe=1
	//VCS coverage off
        5'b000_00://1-pipe, 2-bank
                  bank_number_0 = {3'b0, new_tile_index[0]};
        5'b000_01://1-pipe, 4-bank
                  bank_number_0 = {2'b0, new_tile_index[1:0]};
        5'b000_10://1-pipe, 8-bank
                  bank_number_0 = {1'b0, new_tile_index[2:0]};
        5'b000_11://1-pipe, 16-bank
                  bank_number_0 = {      new_tile_index[3:0]};
	//VCS coverage on
        5'b001_00://2-pipe, 2-bank
                  bank_number_0 = {3'b0, new_tile_index[1]};
        5'b001_01://2-pipe, 4-bank
                  bank_number_0 = {2'b0, new_tile_index[2:1]};
        5'b001_10://2-pipe, 8-bank
                  bank_number_0 = {1'b0, new_tile_index[3:1]};
        5'b001_11://2-pipe, 16-bank
                  bank_number_0 = {      new_tile_index[4:1]};
        5'b010_00://4-pipe, 2-bank
                  bank_number_0 = {3'b0, new_tile_index[2]};
        5'b010_01://4-pipe, 4-bank
                  bank_number_0 = {2'b0, new_tile_index[3:2]};
        5'b010_10://4-pipe, 8-bank
                  bank_number_0 = {1'b0, new_tile_index[4:2]};
        5'b010_11://4-pipe, 16-bank
                  bank_number_0 = {      new_tile_index[5:2]};
        5'b011_00://8-pipe, 2-bank
                  bank_number_0 = {3'b0, new_tile_index[3]};
        5'b011_01://8-pipe, 4-bank
                  bank_number_0 = {2'b0, new_tile_index[4:3]};
        5'b011_10://8-pipe, 8-bank
                  bank_number_0 = {1'b0, new_tile_index[5:3]};
        5'b011_11://8-pipe, 16-bank
                  bank_number_0 = {      new_tile_index[6:3]};
        5'b100_00://16-pipe, 2-bank
                  bank_number_0 = {3'b0, new_tile_index[4]};
        5'b100_01://16-pipe, 4-bank
                  bank_number_0 = {2'b0, new_tile_index[5:4]};
        5'b100_10://16-pipe, 8-bank
                  bank_number_0 = {1'b0, new_tile_index[6:4]};
        5'b100_11://16-pipe, 16-bank
                  bank_number_0 = {      new_tile_index[7:4]};
        default  :
                  bank_number_0 = {      new_tile_index[6:3]};
      endcase	      
    end	    


reg  [20:0]    extract_tile_index;
always@(*)
    begin
      case({log2pipes[2:0], bank_tiling[1:0]}) //256B align
    	//not support pipe=1
    	//VCS coverage off
        5'b000_00://1-pipe, 2-bank
                  extract_tile_index = {      new_tile_index[21:1]};
        5'b000_01://1-pipe, 4-bank
                  extract_tile_index = {1'b0, new_tile_index[21:2]};
        5'b000_10://1-pipe, 8-bank
                  extract_tile_index = {2'b0, new_tile_index[21:3]};
        5'b000_11://1-pipe, 16-bank
                  extract_tile_index = {3'b0, new_tile_index[21:4]};
	//VCS coverage on
        5'b001_00://2-pipe, 2-bank
                  extract_tile_index = {1'b0, new_tile_index[21:2]};
        5'b001_01://2-pipe, 4-bank
                  extract_tile_index = {2'b0, new_tile_index[21:3]};
        5'b001_10://2-pipe, 8-bank
                  extract_tile_index = {3'b0, new_tile_index[21:4]};
        5'b001_11://2-pipe, 16-bank
                  extract_tile_index = {4'b0, new_tile_index[21:5]};
        5'b010_00://4-pipe, 2-bank
                  extract_tile_index = {2'b0, new_tile_index[21:3]};
        5'b010_01://4-pipe, 4-bank
                  extract_tile_index = {3'b0, new_tile_index[21:4]};
        5'b010_10://4-pipe, 8-bank
                  extract_tile_index = {4'b0, new_tile_index[21:5]};
        5'b010_11://4-pipe, 16-bank
                  extract_tile_index = {5'b0, new_tile_index[21:6]};
        5'b011_00://8-pipe, 2-bank
                  extract_tile_index = {3'b0, new_tile_index[21:4]};
        5'b011_01://8-pipe, 4-bank
                  extract_tile_index = {4'b0, new_tile_index[21:5]};
        5'b011_10://8-pipe, 8-bank
                  extract_tile_index = {5'b0, new_tile_index[21:6]};
        5'b011_11://8-pipe, 16-bank
                  extract_tile_index = {6'b0, new_tile_index[21:7]};
        5'b100_00://16-pipe, 2-bank
                  extract_tile_index = {4'b0, new_tile_index[21:5]};
        5'b100_01://16-pipe, 4-bank
                  extract_tile_index = {5'b0, new_tile_index[21:6]};
        5'b100_10://16-pipe, 8-bank
                  extract_tile_index = {6'b0, new_tile_index[21:7]};
        5'b100_11://16-pipe, 16-bank
                  extract_tile_index = {7'b0, new_tile_index[21:8]};
      default    :
                  extract_tile_index = {6'b0, new_tile_index[21:7]};
      endcase	      
    end	    


/////////////////////////////extract baseaddr256b/////////////////////////////////////
   
   wire   [ 0: 0]          shift_size_1_h1;
   wire   [ 2: 0]          shift_size_2_h1;
   wire   [ 2: 0]          shift_size_3_h1;
   wire   [ 3: 0]          shift_size_4_minus_1_h1;
   wire   [ 0: 0]          shift_size_5_h1;
   assign   shift_size_1_h1 = pipe_interleave_size;//0...1
   assign   shift_size_2_h1 = pipe_interleave_size + log2pipes[2:0];//0+0...1+3
   assign   shift_size_3_h1 = pipe_interleave_size + log2pipes[2:0];// + bank_interleave_size[1:0];//0+0+0...1+3+3
   assign   shift_size_4_minus_1_h1 = pipe_interleave_size + log2pipes[2:0]+ bank_tiling[1:0];// + bank_interleave_size[1:0]+bank_tiling[1:0];//0+0+0+0...1+3+3+3
   assign   shift_size_5_h1 = pipe_interleave_size; // + bank_interleave_size[1:0];//0+0...1+3


   wire   [-1+32:0]    base_shift_1 = baseAddr256B >> shift_size_1_h1;
   wire   [-1+32:0]    base_shift_3 = baseAddr256B >> shift_size_3_h1;
   wire   [-1+32:0]    base_shift_2 = baseAddr256B >> shift_size_2_h1;//(pipe_interleave_size+log2pipes[1:0]);
   wire   [-1+32:0]    base_shift_4 = baseAddr256B[31:1] >> shift_size_4_minus_1_h1;

   reg     base_1_part;
   always@(*)
     begin
       case(pipe_interleave_size[0])
       1'b0: base_1_part = 1'b0;
       1'b1: base_1_part = baseAddr256B[0];	       
       endcase	       
     end	     

   wire [-1+32:0]  base_1_part_t;
   assign   base_1_part_t = {31'h0, base_1_part};


   wire [-1+32:0]  base_shift_4_t;//offset
   assign   base_shift_4_t = base_shift_4 << shift_size_5_h1;//(pipe_interleave_size + bank_interleave_size_t[2:0]);

   wire   [32-1: 0]   T21_extract_bitoffset_baseaddr = base_1_part_t | base_shift_4_t;


//////////////////////////tile split///////////////////////////
    wire [-1+4:0]      T21_Log2TileSlices_signed;  //  0+0+1-7 ... 3+4+1-1 => [-6...7]
    assign T21_Log2TileSlices_signed = {2'd0,log2_samples} + log2_pixelsize - tile_split;//-6 ... 7

    wire [-1+3:0]      T21_Log2TileSlices;  //0 ... 3+4+1-1
    assign T21_Log2TileSlices = T21_Log2TileSlices_signed[3] ? 3'd0 : T21_Log2TileSlices_signed[2:0];
    
    wire [-1+4:0] T21_Log2TileSliceBitsMinus9_max_select;  
    assign T21_Log2TileSliceBitsMinus9_max_select = {2'd0,log2_samples} + log2_pixelsize - tile_split;  //[-6...+7]

    wire [-1+3:0] T21_Log2TileSliceBitsMinus9_A; // [1...7]
    assign T21_Log2TileSliceBitsMinus9_A = tile_split;//tile_split: 0 to 6

   
    wire [-1+3:0] T21_Log2TileSliceBitsMinus9; // [0...7]
    assign T21_Log2TileSliceBitsMinus9 = ~T21_Log2TileSliceBitsMinus9_max_select[3] ? T21_Log2TileSliceBitsMinus9_A : 
                                          T21_Log2TileSliceBitsMinus9_B;

    wire [-1+5:0]  sampleSlice_shift;
    assign   sampleSlice_shift = Element_Offset[10:6] >> T21_Log2TileSliceBitsMinus9[2:0];
    wire [-1+5:0]  sampleSlice;
    assign   sampleSlice = ((T21_Log2TileSliceBitsMinus9_max_select[3]==0))?sampleSlice_shift: 5'h0;
    
    //sampleSlice * (slice_tile_max+1)/pipes*banks
    wire [-1+22-1:0]     T21_SliceMacros;
    wire [3:0] log2_macrotile;
    assign  log2_macrotile = log2pipes + bank_tiling +1;//1...7
    assign  T21_SliceMacros = slice_tile_max >> log2_macrotile;
    
    reg [-1+11:0]     new_elem_Offset;//element_offset after tile split
    always @(*)
    begin
       case(T21_Log2TileSliceBitsMinus9)
       3'b100: new_elem_Offset = { 1'h0, Element_Offset[ 9:0]};
       3'b011: new_elem_Offset = { 2'h0, Element_Offset[ 8:0]};
       3'b010: new_elem_Offset = { 3'h0, Element_Offset[ 7:0]};
       3'b001: new_elem_Offset = { 4'h0, Element_Offset[ 6:0]};
       3'b000: new_elem_Offset = { 5'h0, Element_Offset[ 5:0]};
       default:new_elem_Offset = {       Element_Offset[10:0]};
       endcase
    end


 wire  [39:0]   T21_totalOffset = total_accum;

///////////////////shared alu///////////////////////
//add input register for share alu///

 wire  [10:0]      mula1 = z;
 wire  [10:0]      mulb1 = (addr_1D)? slice_tile_max[10:0] : T21_SliceMacros[10:0];
 wire  [10:0]      mulc1 = (addr_1D)? slice_tile_max[21:11] : T21_SliceMacros[20:11];

 reg   [10:0]      mula1_reg_s0;
 reg   [10:0]      mulb1_reg_s0;
 reg   [10:0]      mulc1_reg_s0;
 reg   [-1+3:0]    T21_Log2TileSliceBitsMinus9_B_reg_s0;

//stage 0
 always@(posedge sclk) begin
   if(load_en[0]) begin
     mula1_reg_s0 <= mula1;
     mulb1_reg_s0 <= mulb1;
     mulc1_reg_s0 <= mulc1;
     T21_Log2TileSliceBitsMinus9_B_reg_s0 <= T21_Log2TileSliceBitsMinus9_B;
   end
 end

 wire  [28:0]      mula1b1_accum = ((mula1_reg_s0 * mulb1_reg_s0)<<6);
 wire  [35:0]      mula1b1_accum_shift = mula1b1_accum << T21_Log2TileSliceBitsMinus9_B_reg_s0;
 wire  [23:0]      mula1_shift = {mula1_reg_s0, 6'h0} << T21_Log2TileSliceBitsMinus9_B_reg_s0;

 wire  [ 4:0]      mula2 = (addr_1D)?5'd0 : sampleSlice;
 wire  [20:0]      mulb2 = (addr_1D)?21'd0: T21_SliceMacros;

 reg   [ 4:0]      mula2_reg_s0;
 reg   [20:0]      mulb2_reg_s0;
 reg   [-1+3:0]    T21_Log2TileSliceBitsMinus9_reg_s0;   

//stage 0
 always@(posedge sclk) begin
   if(load_en[0]) begin
     mula2_reg_s0 <= mula2;
     mulb2_reg_s0 <= mulb2;
     T21_Log2TileSliceBitsMinus9_reg_s0 <= T21_Log2TileSliceBitsMinus9;   
   end
 end

 wire  [32:0]      mula2b2_accum = ((mula2_reg_s0 * mulb2_reg_s0)<<6);
 wire  [39:0]      mula2b2_accum_shift = mula2b2_accum << T21_Log2TileSliceBitsMinus9_reg_s0;
 wire  [18:0]      mula2_shift = {mula2_reg_s0, 6'h0} << T21_Log2TileSliceBitsMinus9_reg_s0;

 wire  [39:0]      adden_2d_baseaddr = {T21_extract_bitoffset_baseaddr, 8'h0};
 wire  [39:0]      adden_1d_baseaddr = {baseAddr256B, 8'h0};
 wire  [39:0]      adden_c1 = (addr_1D)?adden_1d_baseaddr : adden_2d_baseaddr;
 reg   [39:0]      adden_c1_reg_s0;
//stage 0
 always@(posedge sclk) begin
   if(load_en[0]) begin
     adden_c1_reg_s0 <= adden_c1;
   end
 end

 wire  [39:0]      adden_2d_tile_offset = {extract_tile_index, 6'h0} << T21_Log2TileSliceBitsMinus9;
 wire  [39:0]      adden_1d_tile_offset = {new_tile_index, 6'h0} << T21_Log2TileSliceBitsMinus9_B;
 wire  [39:0]      adden_d1 = (addr_1D)?adden_1d_tile_offset : adden_2d_tile_offset; 
 reg   [39:0]      adden_d1_reg_s0;
//stage 0
 always@(posedge sclk) begin
   if(load_en[0]) begin
     adden_d1_reg_s0 <= adden_d1;
   end
 end

 wire  [10:0]      adden_e1 = (addr_1D)? Element_Offset : new_elem_Offset;
 reg   [10:0]      adden_e1_reg_s0;
//stage 0
 always@(posedge sclk) begin
   if(load_en[0]) begin
     adden_e1_reg_s0 <= adden_e1;
   end
 end
 
 reg   [35:0]      mula1b1_accum_shift_reg_s1;
 reg   [23:0]      mula1_shift_reg_s1;
 reg   [39:0]      mula2b2_accum_shift_reg_s1;
 reg   [18:0]      mula2_shift_reg_s1;
 reg   [39:0]      adden_c1_reg_s1;
 reg   [39:0]      adden_d1_reg_s1;
 reg   [10:0]      adden_e1_reg_s1;
//stage 1
 always @(posedge sclk)
    begin
        if(load_en[1]) begin
     	    mula1b1_accum_shift_reg_s1 <= mula1b1_accum_shift;
	    mula1_shift_reg_s1 <= mula1_shift;
	    mula2b2_accum_shift_reg_s1 <= mula2b2_accum_shift;
	    mula2_shift_reg_s1 <= mula2_shift;
	    adden_c1_reg_s1 <= adden_c1_reg_s0;
            adden_d1_reg_s1 <= adden_d1_reg_s0;
	    adden_e1_reg_s1 <= adden_e1_reg_s0;
       end
    end

  wire   [39:0]   mula1c1 = (mula1_reg_s0 * mulc1_reg_s0) << 17;
  wire   [39:0]   mula1c1_shift = mula1c1 << T21_Log2TileSliceBitsMinus9_B_reg_s0;
  reg    [39:0]   mula1c1_shift_reg_s1;
//stage 1
  always @(posedge sclk)
    begin
        if(load_en[1]) begin
          mula1c1_shift_reg_s1 <= mula1c1_shift;
       end
    end

  wire  [39:0]  part_sum =  mula1b1_accum_shift_reg_s1 + mula1_shift_reg_s1 + mula2b2_accum_shift_reg_s1 + mula2_shift_reg_s1
                           + adden_c1_reg_s1 + adden_d1_reg_s1 + adden_e1_reg_s1 + mula1c1_shift_reg_s1;
  assign total_accum = part_sum;
    
   //pipe swizzle and rotation:
   reg   [ 3: 0]    pipeswizzle;
    always@(*)
     begin
       case(log2pipes[2:0])
    	 //not support pipe=1
    	 //VCS coverage off
	 3'b000: pipeswizzle = 3'b0;
    	 //VCS coverage on
	 3'b001: pipeswizzle = {3'b0, base_shift_1[0]};
         3'b010: pipeswizzle = {2'b0, base_shift_1[1:0]};
         3'b011: pipeswizzle = {1'b0, base_shift_1[2:0]}; 
         3'b100: pipeswizzle = {      base_shift_1[3:0]}; 
        default: pipeswizzle = {1'b0, base_shift_1[2:0]};
       endcase	       
     end

    wire    [10:0]    T21_slice_z = z;

   wire [7:0]  	T21_slice_z_mul7;    // 16-pipes
   assign  T21_slice_z_mul7 = {T21_slice_z[5:0],2'h0} + {T21_slice_z[6:0],1'h0} + T21_slice_z[7:0];  // 16-pipes
    wire  [6:0]     T21_slice_z_mul3;
    wire  [3:0]     T21_slice_z_mul1;
    wire  [3:0]     T21_slice_z_mul_rotation;
    wire  [3:0]     Slicerotation;
    assign T21_slice_z_mul3 = {T21_slice_z[5:0],1'b0}+T21_slice_z[6:0];
    assign T21_slice_z_mul1 = T21_slice_z[3:0];
    assign T21_slice_z_mul_rotation = (log2pipes == 3'b100) ? T21_slice_z_mul7 :      // 16-pipes: refer to 5.23: max(1, (num_pipes(16)/2)-1)=7
                                     (log2pipes == 3'b011) ? T21_slice_z_mul3[2:0] : 
                                                             T21_slice_z_mul1[2:0];  
// r11xx: PRT tile mode
// Spec 5.3:
//      ARRAY_PRT_TILED_THIN1/ARRAY_PRT_TILED_THICK don't rotate bank/pipe for tile slice split or slice splitting.
   assign Slicerotation = (
                          (array_mode == `ARRAY_PRT_3D_TILED_THIN1) |   // r11xx
                          (array_mode == `ARRAY_PRT_3D_TILED_THICK) |   // r11xx
                          (array_mode == `ARRAY_3D_TILED_THIN1) |
                          (array_mode == `ARRAY_3D_TILED_THICK) |
                          (array_mode == `ARRAY_3D_TILED_XTHICK)                   
                          ) ?T21_slice_z_mul_rotation : 4'h0;

// end r11xx: no bank/pipe rotation
 
    wire  [3:0]   pipe_number_1;			  
    assign pipe_number_1 = Slicerotation + pipeswizzle;
    
    wire  [3:0]   pipe_number_t;
    assign pipe_number_t = pipe_number_0 ^ pipe_number_1;
      
    reg   [3:0]   pipe_number_h1;
    always@(*)
     begin
       case(log2pipes)
    	 //VCS coverage off
	 3'b000: pipe_number_h1 = 4'b0;
    	 //VCS coverage on
	 3'b001: pipe_number_h1 = {3'b0, pipe_number_t[0]};
         3'b010: pipe_number_h1 = {2'b0, pipe_number_t[1:0]};
         3'b011: pipe_number_h1 = {1'b0, pipe_number_t[2:0]};		  
         3'b100: pipe_number_h1 = {      pipe_number_t[3:0]};		  
       default : pipe_number_h1 = {1'b0, pipe_number_t[2:0]};
       endcase	       
     end	     

    //bank swizzle and rotation
   reg   [3:0]   bank_swizzle;
   always@(*)
     begin
       case(bank_tiling[1:0])
	 2'b00: bank_swizzle = {3'b0, base_shift_3[0]};
	 2'b01: bank_swizzle = {2'b0, base_shift_3[1:0]};
         2'b10: bank_swizzle = {1'b0, base_shift_3[2:0]};
         2'b11:	bank_swizzle =        base_shift_3[3:0]; 
       endcase	       
     end

   //2D				                        
   wire  [3:0]    SliceRotation_2D_bank;
   assign  SliceRotation_2D_bank = (bank_tiling==2'b11)?T21_slice_z_mul7:
	                           (bank_tiling==2'b10)?T21_slice_z_mul3:
                                   (bank_tiling==2'b01)?T21_slice_z_mul1:
				                                    4'h0;
   //3D
   reg  [3:0]    SliceRotation_3D_bank;
   always@(*)
     begin
case(log2pipes[2:0])  // 16chan ??
    	 //not support pipe=1
    	 //VCS coverage off
	 3'b000: SliceRotation_3D_bank = T21_slice_z[3:0];
    	 //VCS coverage on
	 3'b001: SliceRotation_3D_bank = T21_slice_z[4:1];
         3'b010: SliceRotation_3D_bank = T21_slice_z[5:2];
         3'b011: SliceRotation_3D_bank = T21_slice_z_mul3[6:3]; 
         3'b100: SliceRotation_3D_bank = T21_slice_z_mul7[7:4]; 
        default: SliceRotation_3D_bank = T21_slice_z_mul3[6:3];
       endcase	       
     end	     

// r11xx: PRT tile mode
// Spec 5.3:
//      ARRAY_PRT_TILED_THIN1/ARRAY_PRT_TILED_THICK don't rotate bank/pipe for tile slice split or slice splitting.
   wire  [3:0]    bank_rotation;
   assign   bank_rotation= ((array_mode == `ARRAY_PRT_3D_TILED_THIN1) |   // r11xx
                            (array_mode == `ARRAY_PRT_3D_TILED_THICK) |   // r11xx
                            (array_mode == `ARRAY_3D_TILED_THIN1) |
                            (array_mode == `ARRAY_3D_TILED_THICK) |
                            (array_mode == `ARRAY_3D_TILED_XTHICK)) ? SliceRotation_3D_bank : 
                           ((array_mode == `ARRAY_PRT_TILED_THIN1) |                                 // r11xx
                            (array_mode == `ARRAY_PRT_TILED_THICK)) ? 4'h0 : SliceRotation_2D_bank;  // r11xx


    //slice rotation
   reg  [3:0]    sampleslice_rotation;
   always@(*)
     if ((array_mode == `ARRAY_PRT_TILED_THIN1) || (array_mode == `ARRAY_PRT_TILED_THICK))         // r11xx
         sampleslice_rotation = 4'h0;                                                              // r11xx
     else 
       case(bank_tiling)
	 2'b00: sampleslice_rotation = {sampleSlice[2:0],1'd0};
	 2'b01: sampleslice_rotation = {sampleSlice[2:0],1'd0} + sampleSlice[3:0];
         2'b10:	sampleslice_rotation = {sampleSlice[1:0],2'd0} + sampleSlice[3:0]; 
         2'b11: sampleslice_rotation = {sampleSlice[0:0],3'd0} + sampleSlice[3:0];
       endcase	     
      
// end r11xx: PRT tile mode


   wire [3:0]   bank_number_1;
   assign  bank_number_1 = bank_rotation + bank_swizzle;

   wire [3:0]   bank_number_h1_t;
   assign  bank_number_h1_t = bank_number_0 ^ bank_number_1 ^ sampleslice_rotation;  
   reg  [3:0]   bank_number_h1;
   always@(*)
     begin
       case(bank_tiling)
	 2'b00: bank_number_h1 = {3'b0, bank_number_h1_t[0]};
	 2'b01: bank_number_h1 = {2'b0, bank_number_h1_t[1:0]};
         2'b10: bank_number_h1 = {1'b0, bank_number_h1_t[2:0]};
         2'b11:	bank_number_h1 = bank_number_h1_t; 
       endcase	       
     end	     
 
    reg   [3:0]  pipe_number_reg_s0;
    reg   [3:0]  bank_number_reg_s0;
    reg   [3:0]  pipe_number_reg_s1;
    reg   [3:0]  bank_number_reg_s1;
    
/************pipe stage 0 ***************/
    always @(posedge sclk)
    begin
        if(load_en[0]) begin
            pipe_number_reg_s0 <= pipe_number_h1;
            bank_number_reg_s0 <= bank_number_h1;
       end
    end
/****************************************/

/************pipe stage 1 ***************/
    always @(posedge sclk)
    begin
        if(load_en[1]) begin
            pipe_number_reg_s1 <= pipe_number_reg_s0;
            bank_number_reg_s1 <= bank_number_reg_s0;
       end
    end
/****************************************/

//rename
wire [3:0]  pipe_number = pipe_number_reg_s1;
wire [3:0]  bank_number = bank_number_reg_s1;


/////////////////////////combine address//////////////////////////////////
//stage 0   
   reg    [ 0: 0]          shift_size_1_reg_s0;
   reg    [ 2: 0]          shift_size_2_reg_s0;
   reg    [ 2: 0]          shift_size_3_reg_s0;
   reg    [ 3: 0]          shift_size_4_minus_1_reg_s0;
   reg    [ 0: 0]          shift_size_5_reg_s0;
   reg    [ 0: 0]          pipe_interleave_size_reg_s0;
   always @(posedge sclk)
    begin
        if(load_en[0]) begin
            shift_size_1_reg_s0 <= shift_size_1_h1;
            shift_size_2_reg_s0 <= shift_size_2_h1;
            shift_size_3_reg_s0 <= shift_size_3_h1;
            shift_size_4_minus_1_reg_s0 <= shift_size_4_minus_1_h1;
            shift_size_5_reg_s0 <= shift_size_5_h1;
            pipe_interleave_size_reg_s0 <= pipe_interleave_size;
       end
    end

//stage 1
   reg    [ 0: 0]          shift_size_1_reg_s1;
   reg    [ 2: 0]          shift_size_2_reg_s1;
   reg    [ 2: 0]          shift_size_3_reg_s1;
   reg    [ 3: 0]          shift_size_4_minus_1_reg_s1;
   reg    [ 0: 0]          shift_size_5_reg_s1;
   reg    [ 0: 0]          pipe_interleave_size_reg_s1;
   always @(posedge sclk)
    begin
        if(load_en[1]) begin
            shift_size_1_reg_s1 <= shift_size_1_reg_s0;
            shift_size_2_reg_s1 <= shift_size_2_reg_s0;
            shift_size_3_reg_s1 <= shift_size_3_reg_s0;
            shift_size_4_minus_1_reg_s1 <= shift_size_4_minus_1_reg_s0;
            shift_size_5_reg_s1 <= shift_size_5_reg_s0;
            pipe_interleave_size_reg_s1 <= pipe_interleave_size_reg_s0;
       end
    end

//rename 
   wire [ 0: 0]   shift_size_1 = shift_size_1_reg_s1;
   wire [ 2: 0]   shift_size_2 = shift_size_2_reg_s1;
   wire [ 2: 0]   shift_size_3 = shift_size_3_reg_s1;
   wire [ 3: 0]   shift_size_4_minus_1 = shift_size_4_minus_1_reg_s1;
   wire [ 0: 0]   shift_size_5 = shift_size_5_reg_s1;
   wire [ 0: 0]   pipe_interleave_size_reg = pipe_interleave_size_reg_s1;

   wire  [39-8:0] offset;
   assign  offset = T21_totalOffset[39:8] >> shift_size_5;

   wire  [39-8:0]  T21_totalOffset_shift_1;
   assign  T21_totalOffset_shift_1 = T21_totalOffset[39:8] >> shift_size_1;

   reg   [-1+1+8:0]  pipe_interleave_offset;
   always@(*)
   begin
     case(pipe_interleave_size_reg[0])
     1'b0: pipe_interleave_offset = {1'b0, T21_totalOffset[7:0]};
     1'b1: pipe_interleave_offset = {      T21_totalOffset[8:0]};	     
     endcase	     
   end	   

   wire  [3+1:0]  pipe_number_t_256B;
   assign   pipe_number_t_256B = {1'b0, pipe_number}<< shift_size_1;//pipe_interleave_size;//0...1

   wire  [3+7:0]  bank_number_t_256B;
   assign bank_number_t_256B = {7'h0, bank_number}<< shift_size_3;//bank_shift_size;//max:7

   wire  [31:0]  offset_t_256B;
   assign   offset_t_256B = {offset, 1'h0}<<shift_size_4_minus_1;//offset_size;//0+0+0+0...4+3+6+3

   wire  [31:0]  pipe_interleave_offset_t_256B;
   assign  pipe_interleave_offset_t_256B = {31'h0, pipe_interleave_offset[8]};

   wire  [31:0]  T21_mem_address_t_256B;
   assign   T21_mem_address_t_256B = (pipe_interleave_offset_t_256B) 
                                 | ({27'h0, pipe_number_t_256B}) 
				 | ({21'h0, bank_number_t_256B})
				 | (offset_t_256B); 

   wire  [39:0]  T21_mem_address_t;
   assign    T21_mem_address_t = {T21_mem_address_t_256B, pipe_interleave_offset[ 7: 0]};


    reg     addr_1D_reg_s0;
    reg     addr_1D_reg_s1;
//stage 0
    always @(posedge sclk)
    begin
        if(load_en[0]) begin
            addr_1D_reg_s0 <= addr_1D;
        end
    end
//stage 1    
    always @(posedge sclk)
    begin
        if(load_en[1]) begin
            addr_1D_reg_s1 <= addr_1D_reg_s0;
        end
    end


   assign  ByteAddr = addr_1D_reg_s1 ? T_mem_address_t : T21_mem_address_t;
   assign  address_out = ByteAddr[39:5];


int i;
always @(posedge sclk)
begin 
      for(i=0;i<TOTAL_PAD_STAGES;i=i+1) begin
        if(en_stage[RI+CALC_STAGES+i]) 
            if(i==0)    address_out_stages[i+1] <= address_out;
            else        address_out_stages[i+1] <= address_out_stages[i];
      end
end
localparam INSERT_PAD_STAGES = (TOTAL_PAD_STAGES > 'd0);

assign address = (INSERT_PAD_STAGES)? address_out_stages[TOTAL_PAD_STAGES] : address_out;

`include "db_addr_calc_fcov_gc.svh"

endmodule

