// ==============================================================================================
// PLEASE DONOT CHANGES THIS FILE
// THIS FILE IS AUTO_GENERATED
// Source: import/address/src/rtl/gen_one/db_dtile_index_calc.v
// ===============================================================================================
// Filename              : db_10xx_dtile_index_calc.v
// Perforce changelist   : $Change$
//
// Copyright             : Trade secret of AMD, Inc.
//                         Copyright 2008, AMD, Inc., (unpublished)
//                         All rights reserved.  This notice is intended as a precaution against
//                         inadvertent publication and does not imply publication or any waiver
//                         of confidentiality.  The year included in the foregoing notice is the
//                         year of creation of the work.
//
// RTL author:             kevin.gu
//
// Description           : Create a unique tile index by performing all the tile level swizzling 
//                         before adding in micro-tiling and address swizzling.
//                         The tile index can later be combined with a tile offset and address swizzling 
//                         to create a final address.
//                         1-pipeline: no register-in, no register-out, 1-middle

`include  "gc__db_reg.v"



module   db_dtile_index_calc_gc(
	                    //output
                            tile_index_h2,
			    
			    //input	    
	                    sclk,
			    load_en,
                            x_h0,                              //Tile part of the X position  
                            y_h0,                              //Tile part of the Y position
                            z_h0,                              //Z/slice/RTIndex
                            pitch_tile_max_h0,                 //in tiles, pitch_element/8 -1
                            array_mode_h0,                     //as defines
                            num_banks_h0,                      //0: 2banks,1:4banks, 2:8banks, 3:16banks
                            bank_width_h0,                     //0: 1tile, 1:2tiles, 2:4tiles, 3:8tiles
                            bank_height_h0,                    //0: 1tile, 1:2tiles, 2:4tiles, 3:8tiles
                            macro_tile_aspect_h0,              //0: 1tile, 1:2tiles, 2:4tiles, 3:8tiles
                            pipe_config_h0                     //10xx new added, specify the num of pipes and how they interleaved
                            );
			   
parameter LOG2_MAX_X = 14;
parameter LOG2_MAX_Y = 14;
parameter LOG2_MAX_Z = 11;
parameter PITCH_TILE_MAX_SIZE = 11;
parameter SLICE_TILE_MAX_SIZE = 22;

input                            sclk;
input                            load_en;

//pixel parameter
input      [LOG2_MAX_X-1:3]      x_h0;
input      [LOG2_MAX_Y-1:3]      y_h0;
input      [LOG2_MAX_Z-1:0]      z_h0;

//surface parameters
input      [ PITCH_TILE_MAX_SIZE-1:0]     pitch_tile_max_h0;

input      [  3:0]     array_mode_h0;
input      [  1:0]     num_banks_h0;
input      [  1:0]     bank_width_h0;
input      [  1:0]     bank_height_h0;
input      [  1:0]     macro_tile_aspect_h0;

input      [ 4:0]      pipe_config_h0;

output     [ 25:0]     tile_index_h2;
wire       [ 25:0]     tile_index_h2;

//wire instead of reg
wire     [LOG2_MAX_X-1:3]      x = x_h0;
wire     [LOG2_MAX_Y-1:3]      y = y_h0;
wire     [LOG2_MAX_Z-1:0]      z = z_h0;
wire     [ PITCH_TILE_MAX_SIZE-1:0]     pitch_tile_max = pitch_tile_max_h0;
wire     [  3:0]     array_mode = array_mode_h0;
wire     [  1:0]     num_banks = num_banks_h0;
wire     [  1:0]     bank_width = bank_width_h0;
wire     [  1:0]     bank_height = bank_height_h0;
wire     [  1:0]     macro_tile_aspect = macro_tile_aspect_h0;
wire     [ 4:0]      pipe_config = pipe_config_h0;

//**************Combinational Logics*********///  
//rename
wire     [ 1:0]  bank_tiling = num_banks[1:0];

//get num of pipes
   reg    [ 2 :0]   log2pipes;
   always@(*) begin
     case(pipe_config[4:0])
     5'b00000: log2pipes = 3'b001; //2 pipes
     5'b00100: log2pipes = 3'b010; //4 pipes
     5'b00101: log2pipes = 3'b010; //4 pipes
     5'b00110: log2pipes = 3'b010; //4 pipes
     5'b00111: log2pipes = 3'b010; //4 pipes
     5'b01000: log2pipes = 3'b011; //8 pipes
     5'b01001: log2pipes = 3'b011; //8 pipes
     5'b01010: log2pipes = 3'b011; //8 pipes
     5'b01011: log2pipes = 3'b011; //8 pipes
     5'b01100: log2pipes = 3'b011; //8 pipes
     5'b01101: log2pipes = 3'b011; //8 pipes
     5'b01110: log2pipes = 3'b011; //8 pipes
     5'b10010: log2pipes = 3'b100; //16 pipes
     //VCS coverage off
     default:  log2pipes = 3'b011; //default to be 8 pipes
     //VCS coverage on
     endcase
   end



// Spec:        macro_tile_width  = tile_width  * bank_width  * num_pipes * macro_tile_aspect
// Spec:        macro_tile_height = tile_height * bank_height * num_banks / macro_tile_aspect
wire [3:0] macro_tile_width_log2  = 2'h3        + bank_width  + log2pipes         + macro_tile_aspect; // max: "3+3+3+3=12" min: "3+0+0+0=3"
wire [3:0] macro_tile_height_log2 = 2'h3        + bank_height + (bank_tiling+1'b1) - macro_tile_aspect; // max: "3+3+(3+1)-0=10 min: "3+0+(0+1)-3=1"

// Spec: if (ARRAY_MODE == ARRAY_PRT_TILING_<THIN, THICK>)
// Spec: Xh = X % macro_tile_width
// Spec: Yh = Y % macro_tile_height
wire [LOG2_MAX_X-1:0] Xh = {x, 3'h0} & (~(14'b11_1111_1111_1111 << macro_tile_width_log2));
wire [LOG2_MAX_Y-1:0] Yh = {y, 3'h0} & (~(14'b11_1111_1111_1111 << macro_tile_height_log2));

wire [LOG2_MAX_X-1:0] sx_prt = ((array_mode == `ARRAY_PRT_TILED_THIN1) || (array_mode == `ARRAY_PRT_TILED_THICK)) ? Xh : {x, 3'h0};
wire [LOG2_MAX_Y-1:0] sy_prt = ((array_mode == `ARRAY_PRT_TILED_THIN1) || (array_mode == `ARRAY_PRT_TILED_THICK)) ? Yh : {y, 3'h0};


wire     [-1+LOG2_MAX_X:3]    sx;
wire     [-1+LOG2_MAX_Y:3]    sy;

  assign  sx = x;
  assign  sy = y;

//////////////////////////////1D tiling mode, thin mode only/////////////////////////////////
//log2_pixelsize = 0
//log2_samples = 0  
   wire [-1+LOG2_MAX_X+3:0] T_mulc1;
   assign T_mulc1 = {sx[LOG2_MAX_X-1:3], 6'd0}; 
  
   wire [   35:0]     accum;
   wire [-1+36:0]     T_mul_accum;
   assign T_mul_accum = accum[35:0];

   reg   [10: 0]   z_reg;
   always @(posedge sclk)
      begin
        if(load_en) begin
            z_reg <= z;
 	end
      end
   wire        new_z_10_1D = T_mul_accum[31] | z_reg[0];
   wire        new_z_9_1D  = T_mul_accum[30] | z_reg[1];
   wire        new_z_8_1D  = T_mul_accum[29] | z_reg[2];
   wire        new_z_7_1D  = T_mul_accum[28] | z_reg[3];
   wire        new_z_6_1D  = T_mul_accum[27] | z_reg[4];
   wire        new_z_5_1D  = T_mul_accum[26] | z_reg[5];
   wire        new_z_4_1D  = T_mul_accum[25] | z_reg[6];
   wire        new_z_3_1D  = T_mul_accum[24] | z_reg[7];
   wire        new_z_2_1D  = T_mul_accum[23] | z_reg[8];
   wire        new_z_1_1D  = T_mul_accum[22] | z_reg[9];
   wire        new_z_0_1D  = T_mul_accum[21] | z_reg[10];

   wire [-1+26:0]     T_mem_address = {new_z_10_1D, new_z_9_1D, new_z_8_1D, new_z_7_1D, new_z_6_1D,
	                               new_z_5_1D, new_z_4_1D, new_z_3_1D, new_z_2_1D, new_z_1_1D, new_z_0_1D, T_mul_accum[20:6]};
   
/////////////////////////////2D and 3D tiling mode, thin /////////////////////////

    wire   [2:0]   log2_macrotileheight;//warning: make sure bank_height + bank_tiling +1 -macro_tile_aspect >=0
    assign   log2_macrotileheight = bank_height + bank_tiling + 1 - macro_tile_aspect;//min:0, max:7
    wire   [-1+LOG2_MAX_Y-3:0]  T21_yMacros;
    assign  T21_yMacros = sy[-1+LOG2_MAX_Y:3] >> log2_macrotileheight;

    wire   [3:0]   log2_macrotilewidth;
    assign   log2_macrotilewidth = bank_width + log2pipes + macro_tile_aspect;//0...9
    wire [-1+PITCH_TILE_MAX_SIZE:0]     T21_PitchMacros;
    assign   T21_PitchMacros = pitch_tile_max[PITCH_TILE_MAX_SIZE-1:0] >> log2_macrotilewidth;

    wire [-1+LOG2_MAX_X-3:0] T21_xMacros;
    assign   T21_xMacros = sx[-1+LOG2_MAX_X:3] >> log2_macrotilewidth; 
    
    wire  [2:0]  newmacrosize;
    assign   newmacrosize = bank_width + bank_height;//0+0...3+3

    //new: (y[:3]%bankheight)*bank_width + (x[:3]/numpipes)%bank_width
    //first calc: (y[:3]%bankheight)
     reg   [2:0]   y_mod_bankheight;
     always@(*)
       begin
	 case(bank_height)
	   2'b00: y_mod_bankheight = 3'b0;//bankheight is 1
	   2'b01: y_mod_bankheight = {2'b0, sy[3]};//bankheight is 2
           2'b10: y_mod_bankheight = {1'b0, sy[4:3]};//bankheight is 4
           2'b11: y_mod_bankheight = sy[5:3];//bankheight is 8	   
	 endcase  	 
       end	       
    //second calc: x[:3]/numpipes
    wire [-1+LOG2_MAX_X-3:0]   x_shift_pipes;
    assign   x_shift_pipes = sx[-1+LOG2_MAX_X:3] >> log2pipes;
    //third calc: y_mod_bankheight*bankwidth + x_shift_pipes%bank_width
    reg   [5:0]  xy_additional_part;
    always@(*)
      begin
	case(bank_width)
	  2'b00: xy_additional_part = {3'b0, y_mod_bankheight};// 1
	  2'b01: xy_additional_part = {2'b0, y_mod_bankheight, x_shift_pipes[0]};// 2
          2'b10: xy_additional_part = {1'b0, y_mod_bankheight, x_shift_pipes[1:0]};// 4
          2'b11: xy_additional_part = {      y_mod_bankheight, x_shift_pipes[2:0]};// 8	  
	endcase  	
      end

      wire   [-1+13+6:0]     tile_and_elem_offset = {7'h0, xy_additional_part, 6'h0};

    wire   [-1+13+6:0]    T21_mule1;   
    assign  T21_mule1 = tile_and_elem_offset >> newmacrosize;

    wire [35:0]     T21_macroNumber;
    assign T21_macroNumber = accum[35:0];
     
    reg    [2:0]     newmacrosize_h1;
    always @(posedge sclk)
      begin
        if(load_en) begin
            newmacrosize_h1 <= newmacrosize;
 	end
      end

    wire  [-1+36:0]     T21_totalOffset_t;
    assign  T21_totalOffset_t = (newmacrosize_h1 == 4'h6)?{T21_macroNumber[29: 0], 6'b0}: 
                                (newmacrosize_h1 == 4'h5)?{T21_macroNumber[30: 0], 5'b0}: 
                                (newmacrosize_h1 == 4'h4)?{T21_macroNumber[31: 0], 4'b0}: 
                                (newmacrosize_h1 == 4'h3)?{T21_macroNumber[32: 0], 3'b0}: 
                                (newmacrosize_h1 == 4'h2)?{T21_macroNumber[33: 0], 2'b0}: 
                                (newmacrosize_h1 == 4'h1)?{T21_macroNumber[34: 0], 1'b0}: 
                                                          {T21_macroNumber[35: 0]};
   
   wire   [35:0]   T21_totalOffset = T21_totalOffset_t;

///////////////////////////bank pipe logics///////////////////////////////
// For this new setting (ADDR_SURF_P16), the pipe select P is calculated in the addrlib as.
// P[0] = x[3] ^ y[3] ^ x[4] ->   tx[0] ^ ty[0] ^ tx[1]
// P[1] = x[4] ^ y[4]			  tx[1] ^ ty[1]
// P[2] = x[5] ^ y[5]			  tx[2] ^ ty[2]
// P[3] = x[6] ^ y[5]			  tx[3] ^ ty[2]
// If GFX_BACK_COMPAT.ENABLE == 1, then PIPE_CONFIG==18 will revert to P8_32x32_16x16.
    reg   [ 3: 0]  pipe_number_0;
    always@(*) begin
      case(pipe_config[4:0])
      5'b00000: pipe_number_0 = {3'h0, sx_prt[3]^sy_prt[3]};
      5'b00100: pipe_number_0 = {2'h0, sx_prt[3]^sy_prt[4], sx_prt[4]^sy_prt[3]};
      5'b00101: pipe_number_0 = {2'h0, sx_prt[4]^sy_prt[4], sx_prt[3]^sy_prt[3]^sx_prt[4]};
      5'b00110: pipe_number_0 = {2'h0, sx_prt[4]^sy_prt[5], sx_prt[3]^sy_prt[3]^sx_prt[4]};
      5'b00111: pipe_number_0 = {2'h0, sx_prt[5]^sy_prt[5], sx_prt[3]^sy_prt[3]^sx_prt[5]};
      5'b01000: pipe_number_0 = {1'h0, sx_prt[4]^sy_prt[4], sx_prt[3]^sy_prt[5], sx_prt[4]^sy_prt[3]^sx_prt[5]};
      5'b01001: pipe_number_0 = {1'h0, sx_prt[4]^sy_prt[5], sx_prt[3]^sy_prt[4], sx_prt[4]^sy_prt[3]^sx_prt[5]};
      5'b01010: pipe_number_0 = {1'h0, sx_prt[5]^sy_prt[5], sx_prt[3]^sy_prt[4], sx_prt[4]^sy_prt[3]^sx_prt[5]};
      5'b01011: pipe_number_0 = {1'h0, sx_prt[4]^sy_prt[5], sx_prt[5]^sy_prt[4], sx_prt[3]^sy_prt[3]^sx_prt[4]};
      5'b01100: pipe_number_0 = {1'h0, sx_prt[5]^sy_prt[5], sx_prt[4]^sy_prt[4], sx_prt[3]^sy_prt[3]^sx_prt[4]};
      5'b01101: pipe_number_0 = {1'h0, sx_prt[5]^sy_prt[5], sx_prt[4]^sy_prt[6], sx_prt[3]^sy_prt[3]^sx_prt[4]};
      5'b01110: pipe_number_0 = {1'h0, sx_prt[5]^sy_prt[6], sx_prt[6]^sy_prt[5], sx_prt[3]^sy_prt[3]^sx_prt[5]};
      5'b10010: pipe_number_0 = {sx_prt[6]^sy_prt[5], sx_prt[5]^sy_prt[5], sx_prt[4]^sy_prt[4], sx_prt[3]^sy_prt[3]^sx_prt[4]};
      //VCS coverage off
      default:  pipe_number_0 = 4'h0;
      //VCS coverage on    
      endcase
    end


reg      [3:0]     bank_number_0_0;

reg   x3_bank_bit;
reg   x4_bank_bit;
reg   x5_bank_bit;
reg   x6_bank_bit;

reg   y3_bank_bit;
reg   y4_bank_bit;
reg   y5_bank_bit;
reg   y6_bank_bit;
wire  [2:0]   bank_width_plus_pipes;//0 -6
assign   bank_width_plus_pipes = bank_width + log2pipes[2:0];  //max is 7


always@(*)
begin
  case(bank_width_plus_pipes)
//log2pipes=0 is removed from axx   3'b000: x3_bank_bit = sx_prt[3];
    3'b001: x3_bank_bit = sx_prt[4];
    3'b010: x3_bank_bit = sx_prt[5];
    3'b011: x3_bank_bit = sx_prt[6];
    3'b100: x3_bank_bit = sx_prt[7];
    3'b101: x3_bank_bit = sx_prt[8];
    3'b110: x3_bank_bit = sx_prt[9];
    3'b111: x3_bank_bit = sx_prt[10];
    //VCS coverage off
    default:x3_bank_bit = sx_prt[3];
    //VCS coverage on
  endcase	
end	

always@(*)
begin
  case(bank_width_plus_pipes)
//log2pipes=0 is removed from axx   3'b000: x4_bank_bit = sx_prt[4];
    3'b001: x4_bank_bit = sx_prt[5];
    3'b010: x4_bank_bit = sx_prt[6];
    3'b011: x4_bank_bit = sx_prt[7];
    3'b100: x4_bank_bit = sx_prt[8];
    3'b101: x4_bank_bit = sx_prt[9];
    3'b110: x4_bank_bit = sx_prt[10];
    3'b111: x4_bank_bit = sx_prt[11];
    //VCS coverage off
    default:x4_bank_bit = sx_prt[4];
    //VCS coverage on
  endcase	
end	

always@(*)
begin
  case(bank_width_plus_pipes)
//log2pipes=0 is removed from axx   3'b000: x5_bank_bit = sx_prt[5];
    3'b001: x5_bank_bit = sx_prt[6];
    3'b010: x5_bank_bit = sx_prt[7];
    3'b011: x5_bank_bit = sx_prt[8];
    3'b100: x5_bank_bit = sx_prt[9];
    3'b101: x5_bank_bit = sx_prt[10];
    3'b110: x5_bank_bit = sx_prt[11];
    3'b111: x5_bank_bit = sx_prt[12];
    //VCS coverage off
    default:x5_bank_bit = sx_prt[5];
    //VCS coverage on
  endcase	
end

always@(*)
begin
  case(bank_width_plus_pipes)
//log2pipes=0 is removed from axx   3'b000: x6_bank_bit = sx_prt[6];
    3'b001: x6_bank_bit = sx_prt[7];
    3'b010: x6_bank_bit = sx_prt[8];
    3'b011: x6_bank_bit = sx_prt[9];
    3'b100: x6_bank_bit = sx_prt[10];
    3'b101: x6_bank_bit = sx_prt[11];
    3'b110: x6_bank_bit = sx_prt[12];
    3'b111: x6_bank_bit = sx_prt[13];
    //VCS coverage off
    default:x6_bank_bit = sx_prt[6];
    //VCS coverage on
  endcase	
end	

always@(*)
begin
  case(bank_height)
    2'b00://bank_height = 1
       y3_bank_bit = sy_prt[3];
    2'b01://bank_height = 2
       y3_bank_bit = sy_prt[4];
    2'b10://bank_height = 4
       y3_bank_bit = sy_prt[5];
    2'b11://bank_height = 8
       y3_bank_bit = sy_prt[6];
  endcase	
end	

always@(*)
begin
  case(bank_height)
    2'b00://bank_height = 1
       y4_bank_bit = sy_prt[4];
    2'b01://bank_height = 2
       y4_bank_bit = sy_prt[5];
    2'b10://bank_height = 4
       y4_bank_bit = sy_prt[6];
    2'b11://bank_height = 8
       y4_bank_bit = sy_prt[7];
  endcase	
end	

always@(*)
begin
  case(bank_height)
    2'b00://bank_height = 1
       y5_bank_bit = sy_prt[5];
    2'b01://bank_height = 2
       y5_bank_bit = sy_prt[6];
    2'b10://bank_height = 4
       y5_bank_bit = sy_prt[7];
    2'b11://bank_height = 8
       y5_bank_bit = sy_prt[8];
  endcase	
end	

always@(*)
begin
  case(bank_height)
    2'b00://bank_height = 1
       y6_bank_bit = sy_prt[6];
    2'b01://bank_height = 2
       y6_bank_bit = sy_prt[7];
    2'b10://bank_height = 4
       y6_bank_bit = sy_prt[8];
    2'b11://bank_height = 8
       y6_bank_bit = sy_prt[9];
  endcase	
end	

always@(*)
begin
  case(bank_tiling)
    2'b00: //2 banks
          bank_number_0_0 = {3'b0, x3_bank_bit^y3_bank_bit};
    2'b01: //4 banks
          bank_number_0_0 = {2'b0, x4_bank_bit^y3_bank_bit, x3_bank_bit^y4_bank_bit};
    2'b10: //8 banks
          bank_number_0_0 = {1'b0, x5_bank_bit^y3_bank_bit, x4_bank_bit^y4_bank_bit^y5_bank_bit, x3_bank_bit^y5_bank_bit};
    2'b11: //16 banks
	  bank_number_0_0 = {x6_bank_bit^y3_bank_bit, x5_bank_bit^y4_bank_bit, x4_bank_bit^y5_bank_bit^y6_bank_bit, x3_bank_bit^y6_bank_bit};
    //default: bank_number_0 = 0;
  endcase		
end	

//10xx new added logic
//The pipe configurations P4_32x32 and P8_32x64_32x32 repeat a pipe on the horizontal access before iterating through all the pipes.  For bank_width == 1, this results in hitting the same bank in the same pipe consecutively.  We want a modification to the LSB of the calculated bank id above when we hit this case
//if (((PIPE_CONFIG == P4_32x32) || (PIPE_CONFIG == P8_32x64_32x32)) && bank_width == 1))
//    bank[0] = bank[0] ^ xh[4] ^ xh[5]
//
wire [3:0] bank_number_0 = ((pipe_config == 5'b00111 || pipe_config == 5'b01110)&&(bank_width == 0))? {bank_number_0_0[3:1], bank_number_0_0[0]^sx_prt[4]^sx_prt[5]}: bank_number_0_0;


   reg  [3:0]   pipe_number;
   always @(posedge sclk)
    begin
        if(load_en) begin
             pipe_number <= pipe_number_0;       
        end
    end


   reg  [3:0]   bank_number;
   always @(posedge sclk)
    begin
        if(load_en) begin
             bank_number <= bank_number_0;       
        end
    end

   reg  [2:0]   log2pipes_h1;
   always @(posedge sclk)
    begin
        if(load_en) begin
             log2pipes_h1 <= log2pipes;       
        end
    end

    reg  [1:0]  bank_tiling_h1;
    always @(posedge sclk)
    begin
        if(load_en) begin
             bank_tiling_h1 <= bank_tiling;       
        end
    end

/////////////////////////combine address//////////////////////////////////
 
    reg  [35:0]  T21_mem_address_t;
    always@(*)
    begin
      case({log2pipes_h1, bank_tiling_h1}) //256B align
        5'b000_00://1-pipe, 2-bank
                  T21_mem_address_t = {T21_totalOffset[34: 6], bank_number[0],   T21_totalOffset[ 5: 0]};
        5'b000_01://1-pipe, 4-bank
		  T21_mem_address_t = {T21_totalOffset[33: 6], bank_number[1:0], T21_totalOffset[ 5: 0]};
        5'b000_10://1-pipe, 8-bank
	          T21_mem_address_t = {T21_totalOffset[32: 6], bank_number[2:0], T21_totalOffset[ 5: 0]};
        5'b000_11://1-pipe, 16-bank
	          T21_mem_address_t = {T21_totalOffset[31: 6], bank_number[3:0], T21_totalOffset[ 5: 0]};
        5'b001_00://2-pipe, 2-bank
		  T21_mem_address_t = {T21_totalOffset[33: 6], bank_number[  0], pipe_number[ 0],   T21_totalOffset[ 5: 0]};
        5'b001_01://2-pipe, 4-bank
	          T21_mem_address_t = {T21_totalOffset[32: 6], bank_number[1:0], pipe_number[ 0],   T21_totalOffset[ 5: 0]};
        5'b001_10://2-pipe, 8-bank
	          T21_mem_address_t = {T21_totalOffset[31: 6], bank_number[2:0], pipe_number[ 0],   T21_totalOffset[ 5: 0]};
        5'b001_11://2-pipe, 16-bank
	          T21_mem_address_t = {T21_totalOffset[30: 6], bank_number[3:0], pipe_number[ 0],   T21_totalOffset[ 5: 0]};
        5'b010_00://4-pipe, 2-bank
                  T21_mem_address_t = {T21_totalOffset[32: 6], bank_number[  0], pipe_number[1:0],   T21_totalOffset[ 5: 0]};
        5'b010_01://4-pipe, 4-bank
	          T21_mem_address_t = {T21_totalOffset[31: 6], bank_number[1:0],  pipe_number[1:0],   T21_totalOffset[ 5: 0]};
        5'b010_10://4-pipe, 8-bank
	          T21_mem_address_t = {T21_totalOffset[30: 6], bank_number[2:0],  pipe_number[1:0],   T21_totalOffset[ 5: 0]};
        5'b010_11://4-pipe, 16-bank
	          T21_mem_address_t = {T21_totalOffset[29: 6], bank_number[3:0],  pipe_number[1:0],   T21_totalOffset[ 5: 0]};
        5'b011_00://8-pipe, 2-bank
	          T21_mem_address_t = {T21_totalOffset[31: 6], bank_number[  0],  pipe_number[2:0],   T21_totalOffset[ 5: 0]};
        5'b011_01://8-pipe, 4-bank
	          T21_mem_address_t = {T21_totalOffset[30: 6], bank_number[1:0],  pipe_number[2:0],   T21_totalOffset[ 5: 0]};
        5'b011_10://8-pipe, 8-bank
	          T21_mem_address_t = {T21_totalOffset[29: 6], bank_number[2:0],  pipe_number[2:0],   T21_totalOffset[ 5: 0]};
        5'b011_11://8-pipe, 16-bank
                  T21_mem_address_t = {T21_totalOffset[28: 6], bank_number[3:0],  pipe_number[2:0],   T21_totalOffset[ 5: 0]};	
        5'b100_00://16-pipe, 2-bank
	          T21_mem_address_t = {T21_totalOffset[30: 6], bank_number[  0],  pipe_number[3:0],   T21_totalOffset[ 5: 0]};
        5'b100_01://16-pipe, 4-bank
	          T21_mem_address_t = {T21_totalOffset[29: 6], bank_number[1:0],  pipe_number[3:0],   T21_totalOffset[ 5: 0]};
        5'b100_10://16-pipe, 8-bank
	          T21_mem_address_t = {T21_totalOffset[28: 6], bank_number[2:0],  pipe_number[3:0],   T21_totalOffset[ 5: 0]};
        5'b100_11://16-pipe, 16-bank
                  T21_mem_address_t = {T21_totalOffset[27: 6], bank_number[3:0],  pipe_number[3:0],   T21_totalOffset[ 5: 0]};	
         default : 
                  T21_mem_address_t = {T21_totalOffset[28: 6], bank_number[3:0],  pipe_number[2:0],   T21_totalOffset[ 5: 0]};
      endcase	      
    end	    



   wire [0:0] addr_1D    ;
   assign addr_1D     = (array_mode == `ARRAY_1D_TILED_THIN1) ;

   wire    [-1+LOG2_MAX_Y:0] mula2;    //width LOG2_MAX_Y
   assign  mula2 = (array_mode == `ARRAY_1D_TILED_THIN1)  ? {sy[-1+LOG2_MAX_Y:3], 3'd0} :   //1D_THIN
                                                            {T21_yMacros, 3'd0};

   wire    [-1+PITCH_TILE_MAX_SIZE:0]  mulb2;  //width PITCH_TILE_MAX_SIZE
   assign  mulb2 = (addr_1D) ? pitch_tile_max :
                               T21_PitchMacros;

   wire    [-1+LOG2_MAX_Y+PITCH_TILE_MAX_SIZE:0] mula2b2;  //width LOG2_MAX_Y+PITCH_TILE_MAX_SIZE
  
   wire    [-1+LOG2_MAX_X+3:0] mulc1;  //width LOG2_MAX_X+6
   assign  mulc1 = addr_1D ? {T_mulc1}: {T21_xMacros, 6'h0};

   wire   [-1+12+7 :0]  mule1;
   assign    mule1 =  (addr_1D ? 0  : T21_mule1);

   assign  mula2b2 = mula2*mulb2;

   reg    [-1+LOG2_MAX_Y+PITCH_TILE_MAX_SIZE:0] mula2b2_h1;  //width LOG2_MAX_Y+PITCH_TILE_MAX_SIZE  
   reg    [-1+LOG2_MAX_Y:0] mula2_h1;
   reg    [-1+LOG2_MAX_X+3:0] mulc1_h1;
   reg    [-1+12+7 :0]  mule1_h1;
   /************pipe stage 1 ***************/
     always @(posedge sclk)
      begin
        if(load_en) begin
            mula2b2_h1 <= mula2b2;
            mula2_h1 <= mula2;
            mulc1_h1 <= mulc1;
            mule1_h1 <= mule1;
        end
      end
  /****************************************/

   assign accum = (mula2b2_h1 << 3) + (mula2_h1 << 3) +    //y*(tile_pitch_max+1)
                  mulc1_h1 + mule1_h1; 


   wire     new_z_10_2D = T21_mem_address_t[31] | z_reg[0];
   wire     new_z_9_2D  = T21_mem_address_t[30] | z_reg[1];
   wire     new_z_8_2D  = T21_mem_address_t[29] | z_reg[2];
   wire     new_z_7_2D  = T21_mem_address_t[28] | z_reg[3];
   wire     new_z_6_2D  = T21_mem_address_t[27] | z_reg[4];
   wire     new_z_5_2D  = T21_mem_address_t[26] | z_reg[5];
   wire     new_z_4_2D  = T21_mem_address_t[25] | z_reg[6];
   wire     new_z_3_2D  = T21_mem_address_t[24] | z_reg[7];
   wire     new_z_2_2D  = T21_mem_address_t[23] | z_reg[8];
   wire     new_z_1_2D  = T21_mem_address_t[22] | z_reg[9];
   wire     new_z_0_2D  = T21_mem_address_t[21] | z_reg[10];

   wire [-1+26:0]  T21_mem_address = {new_z_10_2D, new_z_9_2D, new_z_8_2D, new_z_7_2D, new_z_6_2D,
	                              new_z_5_2D, new_z_4_2D, new_z_3_2D, new_z_2_2D, new_z_1_2D, new_z_0_2D, T21_mem_address_t[20:6]};


   wire [-1+26:0] ByteAddr;

   reg   addr_1D_h1;
   always @(posedge sclk)
    begin
        if(load_en) begin
            addr_1D_h1 <= addr_1D;
        end
    end

   assign   ByteAddr = addr_1D_h1 ? T_mem_address : T21_mem_address;

   assign tile_index_h2 = ByteAddr;

endmodule

