//                              -*- Mode: Verilog -*-
// (c) 2015, 2016 Advanced Micro Devices, Inc. All rights reserved.

////////////////////////////////////////////////////////////////////////////
//
// Filename    : dbg_client_64to1_lane_mux.v.dpl
// Description : Mux for 1 lane of debug client. This is combo logic only.
//               Used for both Synchronous and Source-synchronous Client
//
// NOTE: Thought is to place the byte mux with 64:1 data mux may allow
//       synthesys to better optimize
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
// Print out some input parameters to dperl --- begin
// MKEY          = 
// synchronicity = 
// DBG_WIDTH     = 
// DBG_IN_CNT    = 
// Print out some input parameters to dperl ----- end
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////

module dbg_client_64to1_lane_mux
#(parameter LANE_COUNT = 4
,           LANE_WIDTH = 8
,           LANE_INDEX = 0
)

// ==========================================================================
// interface with dbg_client_ssb__steer
// Begin --------------------------------------------------------------------
( input       [8:0]            dbg_Mux_SelLane      // LANE_COUNT * ([8] Lane Enable; [7:6] Byte Select; [5:0] Channel Select;)
, output      [LANE_WIDTH-1:0] dbg_L2DataMuxLane    // Debug-bus data outputs after muxing for this lane
// End ----------------------------------------------------------------------

// ==========================================================================
// Debug-bus data inputs from functional logic
// Begin --------------------------------------------------------------------
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug63
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug62
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug61
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug60
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug59
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug58
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug57
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug56
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug55
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug54
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug53
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug52
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug51
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug50
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug49
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug48
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug47
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug46
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug45
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug44
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug43
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug42
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug41
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug40
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug39
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug38
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug37
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug36
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug35
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug34
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug33
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug32
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug31
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug30
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug29
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug28
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug27
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug26
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug25
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug24
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug23
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug22
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug21
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug20
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug19
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug18
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug17
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug16
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug15
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug14
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug13
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug12
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug11
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug10
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug9
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug8
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug7
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug6
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug5
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug4
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug3
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug2
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug1
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug0
// End ----------------------------------------------------------------------
);

  reg [LANE_COUNT*LANE_WIDTH-1:0]       dbg_L1DataMux;
  wire[LANE_WIDTH-1:0]           dbg_L1DataMuxDefault;
  reg [LANE_WIDTH-1:0]           reg_L2DataMuxLane;

  assign dbg_L2DataMuxLane[LANE_WIDTH-1:0] = reg_L2DataMuxLane[LANE_WIDTH-1:0];

//////////////////////////////////////////////////////////////////////////|
//                       <Debug-Bus Mux - Level 2>                      //|
//                   (Level 1 is outside of dbg_common)                 //|
//////////////////////////////////////////////////////////////////////////|
// Muxing for byte selection for each of the 3 24bit debug-bus inputs   //|
// This muxing is active only if Multi-Block mode is enabled.           //|
// In multiblock mode from each of the 3 input buses one byte (segment) //|
// is selected. This selection is independant for each input bus.       //|
// In SingleBlock mode (MultiBlock is not enables) all 3 bytes will be  //|
// selected from first input bus.                                       //|
// ----                                                                 //|
// to overload 2 bit Byte Select for Dbug-bus test purposes,
// to work with dbg_client_ssb__debug, in 3 lane case we set the default//|
// so if byte select is set to 2'b11 we pass the lane output            //|
//////////////////////////////////////////////////////////////////////////|
generate
if (LANE_COUNT>3) begin
   assign dbg_L1DataMuxDefault[LANE_WIDTH-1:0]= dbg_L1DataMux[(4*LANE_WIDTH)-1:(3*LANE_WIDTH)];
end else begin
   assign dbg_L1DataMuxDefault[LANE_WIDTH-1:0]= dbg_L1DataMux[((LANE_INDEX+1)*LANE_WIDTH)-1:(LANE_INDEX*LANE_WIDTH)];
end
endgenerate

always @(*) begin
   if (!dbg_Mux_SelLane[8])
      reg_L2DataMuxLane[LANE_WIDTH-1:0] = {LANE_WIDTH {1'b0}};
   else begin
      case (dbg_Mux_SelLane[7:6])
         2'b00  : reg_L2DataMuxLane[LANE_WIDTH-1:0] = dbg_L1DataMux[LANE_WIDTH-1:0];
         2'b01  : reg_L2DataMuxLane[LANE_WIDTH-1:0] = dbg_L1DataMux[(2*LANE_WIDTH)-1:LANE_WIDTH];
         2'b10  : reg_L2DataMuxLane[LANE_WIDTH-1:0] = dbg_L1DataMux[(3*LANE_WIDTH)-1:(2*LANE_WIDTH)];
         2'b11  : reg_L2DataMuxLane[LANE_WIDTH-1:0] = dbg_L1DataMuxDefault[LANE_WIDTH-1:0];
         default: reg_L2DataMuxLane[LANE_WIDTH-1:0] = dbg_L1DataMuxDefault[LANE_WIDTH-1:0];
      endcase // case(dbg_Mux_SelLane[7:6])
   end // else if (!dbg_Mux_SelLane[8])

//////////////////////////////////////////////////////////////////////////|
//                       <Debug-Bus Mux - Level 1>                      //|
//                      Debug-bus data muxing logic                     //|
//////////////////////////////////////////////////////////////////////////|
   case (dbg_Mux_SelLane[5:0])
     6'b000000 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug0[LANE_COUNT*LANE_WIDTH-1:0];
     6'b000001 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug1[LANE_COUNT*LANE_WIDTH-1:0];
     6'b000010 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug2[LANE_COUNT*LANE_WIDTH-1:0];
     6'b000011 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug3[LANE_COUNT*LANE_WIDTH-1:0];
     6'b000100 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug4[LANE_COUNT*LANE_WIDTH-1:0];
     6'b000101 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug5[LANE_COUNT*LANE_WIDTH-1:0];
     6'b000110 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug6[LANE_COUNT*LANE_WIDTH-1:0];
     6'b000111 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug7[LANE_COUNT*LANE_WIDTH-1:0];
     6'b001000 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug8[LANE_COUNT*LANE_WIDTH-1:0];
     6'b001001 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug9[LANE_COUNT*LANE_WIDTH-1:0];
     6'b001010 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug10[LANE_COUNT*LANE_WIDTH-1:0];
     6'b001011 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug11[LANE_COUNT*LANE_WIDTH-1:0];
     6'b001100 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug12[LANE_COUNT*LANE_WIDTH-1:0];
     6'b001101 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug13[LANE_COUNT*LANE_WIDTH-1:0];
     6'b001110 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug14[LANE_COUNT*LANE_WIDTH-1:0];
     6'b001111 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug15[LANE_COUNT*LANE_WIDTH-1:0];
     6'b010000 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug16[LANE_COUNT*LANE_WIDTH-1:0];
     6'b010001 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug17[LANE_COUNT*LANE_WIDTH-1:0];
     6'b010010 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug18[LANE_COUNT*LANE_WIDTH-1:0];
     6'b010011 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug19[LANE_COUNT*LANE_WIDTH-1:0];
     6'b010100 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug20[LANE_COUNT*LANE_WIDTH-1:0];
     6'b010101 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug21[LANE_COUNT*LANE_WIDTH-1:0];
     6'b010110 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug22[LANE_COUNT*LANE_WIDTH-1:0];
     6'b010111 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug23[LANE_COUNT*LANE_WIDTH-1:0];
     6'b011000 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug24[LANE_COUNT*LANE_WIDTH-1:0];
     6'b011001 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug25[LANE_COUNT*LANE_WIDTH-1:0];
     6'b011010 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug26[LANE_COUNT*LANE_WIDTH-1:0];
     6'b011011 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug27[LANE_COUNT*LANE_WIDTH-1:0];
     6'b011100 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug28[LANE_COUNT*LANE_WIDTH-1:0];
     6'b011101 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug29[LANE_COUNT*LANE_WIDTH-1:0];
     6'b011110 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug30[LANE_COUNT*LANE_WIDTH-1:0];
     6'b011111 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug31[LANE_COUNT*LANE_WIDTH-1:0];
     6'b100000 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug32[LANE_COUNT*LANE_WIDTH-1:0];
     6'b100001 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug33[LANE_COUNT*LANE_WIDTH-1:0];
     6'b100010 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug34[LANE_COUNT*LANE_WIDTH-1:0];
     6'b100011 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug35[LANE_COUNT*LANE_WIDTH-1:0];
     6'b100100 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug36[LANE_COUNT*LANE_WIDTH-1:0];
     6'b100101 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug37[LANE_COUNT*LANE_WIDTH-1:0];
     6'b100110 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug38[LANE_COUNT*LANE_WIDTH-1:0];
     6'b100111 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug39[LANE_COUNT*LANE_WIDTH-1:0];
     6'b101000 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug40[LANE_COUNT*LANE_WIDTH-1:0];
     6'b101001 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug41[LANE_COUNT*LANE_WIDTH-1:0];
     6'b101010 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug42[LANE_COUNT*LANE_WIDTH-1:0];
     6'b101011 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug43[LANE_COUNT*LANE_WIDTH-1:0];
     6'b101100 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug44[LANE_COUNT*LANE_WIDTH-1:0];
     6'b101101 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug45[LANE_COUNT*LANE_WIDTH-1:0];
     6'b101110 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug46[LANE_COUNT*LANE_WIDTH-1:0];
     6'b101111 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug47[LANE_COUNT*LANE_WIDTH-1:0];
     6'b110000 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug48[LANE_COUNT*LANE_WIDTH-1:0];
     6'b110001 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug49[LANE_COUNT*LANE_WIDTH-1:0];
     6'b110010 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug50[LANE_COUNT*LANE_WIDTH-1:0];
     6'b110011 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug51[LANE_COUNT*LANE_WIDTH-1:0];
     6'b110100 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug52[LANE_COUNT*LANE_WIDTH-1:0];
     6'b110101 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug53[LANE_COUNT*LANE_WIDTH-1:0];
     6'b110110 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug54[LANE_COUNT*LANE_WIDTH-1:0];
     6'b110111 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug55[LANE_COUNT*LANE_WIDTH-1:0];
     6'b111000 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug56[LANE_COUNT*LANE_WIDTH-1:0];
     6'b111001 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug57[LANE_COUNT*LANE_WIDTH-1:0];
     6'b111010 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug58[LANE_COUNT*LANE_WIDTH-1:0];
     6'b111011 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug59[LANE_COUNT*LANE_WIDTH-1:0];
     6'b111100 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug60[LANE_COUNT*LANE_WIDTH-1:0];
     6'b111101 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug61[LANE_COUNT*LANE_WIDTH-1:0];
     6'b111110 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug62[LANE_COUNT*LANE_WIDTH-1:0];
     6'b111111 : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = idebug63[LANE_COUNT*LANE_WIDTH-1:0];
     default   : dbg_L1DataMux[LANE_COUNT*LANE_WIDTH-1:0]  = {LANE_COUNT*LANE_WIDTH {1'b0}};
   endcase // case(dbg_Mux_SelLane[5:0])

end // always @(*)
endmodule // dbg_client_64to1_lane_mux
