//                              -*- Mode: Verilog -*-
// (c) 2015, 2016 Advanced Micro Devices, Inc. All rights reserved.

////////////////////////////////////////////////////////////////////////////
//
// Filename    : dbg_client_ssb__aon_muxf.v.dpl
// Description : Mux for synchronous debug bus client.
//
// NOTE : Multi-clock case should use Source-synchronous debug bus client
//
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
// Print out some input parameters to dperl --- begin
// MKEY          = 
// MUX_COUNT     = 
// Print out some input parameters to dperl ----- end
////////////////////////////////////////////////////////////////////////////

module dbg_client_ssb__aon_muxf
#(parameter LANE_WIDTH = 8
,           MUX_USED   = 6'b000000
)

// ==========================================================================
// Debug bus Daisy-chain inputs and outputs
// Data flows towards DSM. ctl flows away from DSM
// Begin --------------------------------------------------------------------
( input                   DBUS_TARG_daisychain_dclk  // Daisy-chain input,  source-synchronous clock for data
, input  [LANE_WIDTH-1:0] DBUS_TARG_daisychain_data  // Daisy-chain input
, output                  TARG_DBUS_daisychain_dclk  // Daisy-chain output, source-synchronous clock for data
, output [LANE_WIDTH-1:0] TARG_DBUS_daisychain_data  // Daisy-chain output
// End ----------------------------------------------------------------------

// ==========================================================================
// interface with first dbg_client_ssb__mux instance
// Begin --------------------------------------------------------------------
, input             [2:0] LaneMuxId                  // Mux Id of this lane
, input             [5:0] dbg_Mux_Clk                // Debug-bus clock output  from  mux

, input  [LANE_WIDTH-1:0] dbg_Data0Muxed     // Debug-bus data outputs after muxing
, input  [LANE_WIDTH-1:0] dbg_Data1Muxed     // Debug-bus data outputs after muxing
, input  [LANE_WIDTH-1:0] dbg_Data2Muxed     // Debug-bus data outputs after muxing
, input  [LANE_WIDTH-1:0] dbg_Data3Muxed     // Debug-bus data outputs after muxing
, input  [LANE_WIDTH-1:0] dbg_Data4Muxed     // Debug-bus data outputs after muxing
, input  [LANE_WIDTH-1:0] dbg_Data5Muxed     // Debug-bus data outputs after muxing
// End ----------------------------------------------------------------------

// ==========================================================================
// Misc
// Begin --------------------------------------------------------------------
, output                  ClkEn_Raw                  // Raw Clock enable based on selected clock from mux
, input                   ClkEnSel                   // control program : Clock enable select 1'b1 -> ClkEnIn
, output                  ClkEnOut                   // Clock enable Out
, input                   ClkEnIn                    // Clock enable In
, input                   ClkEnBypass                // used in async mode, to bypass the clkgating logic. brings the async data over directly.

, input                   ClkOn_async                // 1'b0 -> MuxClk at 1'b0; 1'b1 -> output selected clock
, input                   cclk                       // Control Programming Clock to use for reset data pipe
, input                   dbg_rst                    // Reset and clear data pipe
, input                   testClk                    // clock to use if TEST_CLK_EN == 1'b1
, input                   TEST_CLK_EN                // override for test mode
// End ----------------------------------------------------------------------
);

  localparam  LOCAL_MUX_USED = {2'b11, MUX_USED[5:0]} ;
  // ==========================================================================
  // last stage clock mux  - not glitch free
  // BOZO: Need assertion to check
  //     - ClkOn_async == 1'b0 when MuxSelLaneEn change
  // Begin -------------------------------------------------------------------
  wire            Clk_Raw;
  wire            gclk;
  wire            Clk_bypass;
  wire            pipeClk;
  wire            opipeClk;
  wire            opipeClkRst;

  wire            laneEn;
  reg  [2:0]      clkSel;
  wire [2:0]      ctlClkSel;

  // 3'b111 - lane is not selected
  // 3'b110 - Use test clock
  // bug 556183 - This case should not be reached since mux 3'b110 = 6 is not a legale pprogram select.
  //              but it's possible for scan to reach this condition, so add case to eliminate this issue
  always @(*) begin
     casez ({TEST_CLK_EN, LaneMuxId[2:0]})
       4'b1??? : clkSel[2:0] = 3'b110;
       4'b0110 : clkSel[2:0] = 3'b111; // bug 556183
       default : clkSel[2:0] = LaneMuxId[2:0];
     endcase // casez ({TEST_CLK_EN, LaneMuxId[2:0]})
  end
  assign          laneEn      = ~(&clkSel[2:0]) & ~dbg_rst;

  dbg_client_ssb__clkmux #(.MUX_USED(LOCAL_MUX_USED))
    I_dbg_client_ssb__clkmux (.clk_o(Clk_Raw),
                              .clk_i({DBUS_TARG_daisychain_dclk, testClk, dbg_Mux_Clk[5:0]}),
                              .sel(clkSel[2:0])
                              );
  // pipeClk is for clocking the input flop of data pipe
  aticlk_mux2 d0nt_clkmux (.clk_o(pipeClk),
                           .clk_i({testClk, DBUS_TARG_daisychain_dclk}),
                           .sel(TEST_CLK_EN));
  wire mrkFirstClientClk;
  wire mrkCreateGeneratedClock;
  wire mrkStopPropagation;

  //----------------------------------------------------
  // 3 marker buffers for clocking SDC
  // opipeClk is for clocking the output flop of data pipe
  // NOTE: Marker point to do 'create_generated_clock' with '-add' in timing sdc
  aticlk_buf  d0nt_dbgcMRK_GeneratedClk (.A(mrkCreateGeneratedClock), .Y(opipeClk                 ));
  // NOTE: can swap in below to change to internal 1/2 cycle path.
//  ati_clk_inv  d0nt_dbgcMRK_GeneratedClk (.clk_in(mrkCreateGeneratedClock), .clk_out(opipeClk));
  // MOTE: Marker point to do 'create_clock' !!! ONLY FOR THE 1st debug client !!!
  aticlk_buf  d0nt_dbgc1MRK             (.A(cclk                   ), .Y(mrkFirstClientClk        ));
  // NOTE: Marker point to do 'set_clock_sense -stop_propagation ...' in timing sdc
  aticlk_buf  d0nt_dbgcMRK_StopProp     (.A(mrkStopPropagation     ), .Y(TARG_DBUS_daisychain_dclk));
  //---------

  //----------------------------------------------------
  // d0nt_oclkmux - this mux is required so we do not need to change ClkEn to allow DBUS_TARG_daisychain_dclk to pass through
  // Prioritize clock to favor gclk path
  // ctlClkSel[2:0] == 3'b1?? -> gclk
  // ctlClkSel[2:0] == 3'b010 -> mrkFirstClientClk -- this is cclk
  // ctlClkSel[2:0] == 3'b011 -> Clk_Raw -- Special async mode
  assign ctlClkSel[2] = laneEn & ~(|ctlClkSel[1:0]);
  assign ctlClkSel[1] = dbg_rst | ctlClkSel[0];
  assign ctlClkSel[0] = ClkEnBypass & ClkEnSel;
  aticlk_mux2 d0nt_oclkmux     (.clk_o(mrkCreateGeneratedClock),
                                .clk_i({gclk, opipeClkRst}),
                                .sel(ctlClkSel[2]));
  aticlk_mux2 d0nt_opipeClkRst (.clk_o(opipeClkRst),
                                .clk_i({Clk_bypass, DBUS_TARG_daisychain_dclk}),
                                .sel(ctlClkSel[1]));
  aticlk_mux2 d0nt_Clk_bypass  (.clk_o(Clk_bypass),
                                .clk_i({Clk_Raw, mrkFirstClientClk}),
                                .sel(ctlClkSel[0]));

  ati_clk_inv d0nt_inv_TARG_DBUS_daisychain_dclk (.clk_out (mrkStopPropagation), .clk_in  (opipeClk));
  //----------------------------------------------------
  // End ----------------------------------------------------------------------

  // ==========================================================================
  //
  // Begin --------------------------------------------------------------------
  // Assume 1 synchronizer goodenough for EGAS comp.
 
  techind_sync_icd #(.size(1), .depth(3), .TransportDelay(0), .TransportCycle(2), .asyncReset(1))
    I_ClkOn_async_sync ( .SB(1'b1), .FLUSH(1'b0), .CKsrc(1'b0), .EN(1'b1)
                       , .RB( ClkOn_async )
                       , .CKdst( Clk_Raw )
                       , .Dsrc( ClkOn_async )
                       , .Qsrc(), .Qdst( ClkEn_Raw )
                       , .SC1(1'b0), .SC2(1'b0), .SE(1'b0), .SI(1'b0), .SO()
  `ifdef ABV
                       , .InputChkDis(1'b1)
  `endif
                       );

  wire       ClkEnOutPre;
  ati_mux2 I_ClkEnOut (.s(ClkEnSel), .i0(ClkEn_Raw), .i1(ClkEnIn), .z(ClkEnOutPre));

  // Need Clk_Raw here to properly control cdcbug x-inject
  // Bug : http://ontrack-internal.amd.com/browse/DECDFXDFD-574
  //  -- .DSTCLK needs to toggle for techind_cdcfpm to clear x-inject (reset)
 wire Clk_Raw_cdcbuf;
`ifdef ABV
  assign Clk_Raw_cdcbuf = dbg_rst ? cclk : Clk_Raw;
`else // ABV
  assign Clk_Raw_cdcbuf = Clk_Raw;
`endif // else ABV

  wire      cdc_ClkOn_async;
  techind_cdcfpm #(.size ( 1),.applyParams(1),  .DestClkCnt(3), .CdcDestSetup(0))
                 mrk_cdc_ClkOn_async(.A(ClkOn_async), .DSTCLK(Clk_Raw_cdcbuf), .Z(cdc_ClkOn_async));

  assign     ClkEnOut = ClkEnOutPre & cdc_ClkOn_async;
  // Clock gater (/import/rtllib/src/rtl/ati_clock_gate.v)
  ati_clock_gate I_gclk (.clk_out(gclk), .clk_src(Clk_Raw), .clk_en(ClkEnOut), .TE(TEST_CLK_EN));
  // End ----------------------------------------------------------------------

  // ==========================================================================
  // Begin --------------------------------------------------------------------
  reg [LANE_WIDTH-1:0]       daisychain_data_d1;
  reg [LANE_WIDTH-1:0]       daisychain_data_d2;
  reg [LANE_WIDTH-1:0]       daisychain_data_d2I;

  assign          TARG_DBUS_daisychain_data[LANE_WIDTH-1:0] = daisychain_data_d2[LANE_WIDTH-1:0];

  // NOTE: need to code async reset with if statement due to dc_elab sensitivity
  always @(posedge pipeClk or posedge dbg_rst)
    if (dbg_rst == 1'b1)
      daisychain_data_d1[LANE_WIDTH-1:0] <= {LANE_WIDTH {1'b0}};
    else
      daisychain_data_d1[LANE_WIDTH-1:0] <= DBUS_TARG_daisychain_data[LANE_WIDTH-1:0];

  always @(posedge opipeClk) daisychain_data_d2[LANE_WIDTH-1:0] <= daisychain_data_d2I[LANE_WIDTH-1:0];


//  always @(*)
//    case (LaneMuxId[2:0])
//       3'b000  : daisychain_data_d2I[LANE_WIDTH-1:0] = dbg_Data0Muxed[LANE_WIDTH-1:0];
//       3'b001  : daisychain_data_d2I[LANE_WIDTH-1:0] = dbg_Data1Muxed[LANE_WIDTH-1:0];
//       3'b010  : daisychain_data_d2I[LANE_WIDTH-1:0] = dbg_Data2Muxed[LANE_WIDTH-1:0];
//       3'b011  : daisychain_data_d2I[LANE_WIDTH-1:0] = dbg_Data3Muxed[LANE_WIDTH-1:0];
//       3'b100  : daisychain_data_d2I[LANE_WIDTH-1:0] = dbg_Data4Muxed[LANE_WIDTH-1:0];
//       3'b101  : daisychain_data_d2I[LANE_WIDTH-1:0] = dbg_Data5Muxed[LANE_WIDTH-1:0];
//       default : daisychain_data_d2I[LANE_WIDTH-1:0] = daisychain_data_d1[LANE_WIDTH-1:0];
//    endcase // case(LaneMuxId[2:0])

  dbg_client_ssb__datmux #(.MUX_USED(LOCAL_MUX_USED), .WIDTH(LANE_WIDTH))
    I_dbg_client_ssb__datmux ( .dat_i0(dbg_Data0Muxed)
                             , .dat_i1(dbg_Data1Muxed)
                             , .dat_i2(dbg_Data2Muxed)
                             , .dat_i3(dbg_Data3Muxed)
                             , .dat_i4(dbg_Data4Muxed)
                             , .dat_i5(dbg_Data5Muxed)
                             , .dat_i6(daisychain_data_d1)
                             , .dat_i7(daisychain_data_d1)
                             , .sel   (LaneMuxId[2:0])
                             , .dat_o (daisychain_data_d2I)
                             );

  // End ----------------------------------------------------------------------

endmodule // dbg_client_ssb__aon_muxf
