//                              -*- Mode: Verilog -*-
// (c) 2015, 2016 Advanced Micro Devices, Inc. All rights reserved.

////////////////////////////////////////////////////////////////////////////
//
// Filename    : dbg_client_ssb__aon_prog.v.dpl
// Description :
//
// NOTE : Multi-clock case should use Source-synchronous debug bus client
//
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
// Print out some input parameters to dperl --- begin
// MKEY          = 
// DBG_WIDTH     = 
// MUX_COUNT     = 
// Print out some input parameters to dperl ----- end
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////

module dbg_client_ssb__aon_prog
#( parameter LANE_COUNT = 4
,            LANE_WIDTH = 8
)

// ==========================================================================
// Debug bus Daisy-chain inputs and outputs
// Data flows towards DSM. ctl flows away from DSM
// Begin --------------------------------------------------------------------
( output             DBUS_TARG_daisychain_cclk  // Daisy-chain program output, source-synchronous clock for config
, output      [ 1:0] DBUS_TARG_daisychain_ctl   // Daisy-chain program output, reverse of data flow
 
, input              TARG_DBUS_daisychain_cclk  // Daisy-chain program input,  source-synchronous clock for config
, input       [ 1:0] TARG_DBUS_daisychain_ctl   // Daisy-chain program input,  reverse of data flow
// End ----------------------------------------------------------------------

// ==========================================================================
// interface with first dbg_client_ssb__clkmux instance
// Begin --------------------------------------------------------------------
, output             ctlProg_ClkOn              // Clock on / off;

, output      [(LANE_COUNT*9-1):0]Mux5SelLane                // LANE_COUNT * ([8] Lane Enable; [7:6] Byte Select; [5:0] Chennel Select;)
, output      [(LANE_COUNT*9-1):0]Mux4SelLane                // LANE_COUNT * ([8] Lane Enable; [7:6] Byte Select; [5:0] Chennel Select;)
, output      [(LANE_COUNT*9-1):0]Mux3SelLane                // LANE_COUNT * ([8] Lane Enable; [7:6] Byte Select; [5:0] Chennel Select;)
, output      [(LANE_COUNT*9-1):0]Mux2SelLane                // LANE_COUNT * ([8] Lane Enable; [7:6] Byte Select; [5:0] Chennel Select;)
, output      [(LANE_COUNT*9-1):0]Mux1SelLane                // LANE_COUNT * ([8] Lane Enable; [7:6] Byte Select; [5:0] Chennel Select;)
, output      [(LANE_COUNT*9-1):0]Mux0SelLane                // LANE_COUNT * ([8] Lane Enable; [7:6] Byte Select; [5:0] Chennel Select;)
// End ----------------------------------------------------------------------

// ==========================================================================
// Misc
// Begin --------------------------------------------------------------------
, output      [(LANE_COUNT*3-1):0]LaneMuxId    // Mux Id selection for each lane
, output      [(LANE_COUNT-1):0]  ClkEnSel     // control program : Clock enable select
, input       [7:0]               DbgBlkId     // Debug client ID - hard tie
, input                           AEB_DBUS     // Security lock down. 1'b0-> hold program to reset == data forward
, input                           TEST_CLK_EN  // Gater override for test mode. In test mode drive all logic in TARG_DBUS_daisychain_cclk
, output                          dbg_rst      // Reset and clear data
// Block-ID for current debug-bus block. Should be hirdwired to vias during integration
, output                    [7:0] dbg_BlkId    // Debug bus ID used for debug the debug bus client
// End ----------------------------------------------------------------------
);

  wire                      R_ctlProg_ClkOn   ;
  wire [(LANE_COUNT*9-1):0] R_Mux0SelLane     ; // LANE_COUNT * ([8] Lane Enable; [7:6] Byte Select; [5:0] Chennel Select;)
  wire [(LANE_COUNT*9-1):0] R_Mux1SelLane     ; // LANE_COUNT * ([8] Lane Enable; [7:6] Byte Select; [5:0] Chennel Select;)
  wire [(LANE_COUNT*9-1):0] R_Mux2SelLane     ; // LANE_COUNT * ([8] Lane Enable; [7:6] Byte Select; [5:0] Chennel Select;)
  wire [(LANE_COUNT*9-1):0] R_Mux3SelLane     ; // LANE_COUNT * ([8] Lane Enable; [7:6] Byte Select; [5:0] Chennel Select;)
  wire [(LANE_COUNT*9-1):0] R_Mux4SelLane     ; // LANE_COUNT * ([8] Lane Enable; [7:6] Byte Select; [5:0] Chennel Select;)
  wire [(LANE_COUNT*9-1):0] R_Mux5SelLane     ; // LANE_COUNT * ([8] Lane Enable; [7:6] Byte Select; [5:0] Chennel Select;)
  wire                      mrk_aeb_dbus;
  // Marker bug so this will stay in gate
  ati_buffer1 mrk_dbg_BlkId[7:0] (.A(DbgBlkId[7:0]), .Y(dbg_BlkId[7:0]));
  ati_buffer1 d0nt_mrk_aeb_dbus  (.A(AEB_DBUS     ), .Y(mrk_aeb_dbus  )); // Add marker buffer on AEB to make it easier to set timing constraint

  // ==========================================================================
  // Control Path
  // Begin --------------------------------------------------------------------
  wire             gcclk             ; // control logic clock
  wire             ctlProgClrDat     ; // Clear the data Pipe
  wire             ctlProgClrCtl     ; // Clear the Control logic data
  wire             ctlProgVal        ; // control program : control valid
  wire             ctlProg_ClkEnSel  ; // control program : Clock enable select
  wire [1:0]       ctlProg_ByteSel   ; // control program : bit Byte select // removed [2]Multi-block mode
  wire [2:0]       ctlProg_MuxId     ; // control program : Mux ID of the current program target
  wire [1:0]       ctlProg_LaneSel   ; // control program : Lane select of mux
  wire [5:0]       ctlProg_ChannelSel; // control program : Channel select of mux

  assign           dbg_rst    = ctlProgClrDat;

  dbg_client_ssb__aon_s2p
    I_dbg_client_ssb__aon_s2p (
                              .conf_clk                 (gcclk                         ) , // control logic clock
                              .TARG_DBUS_daisychain_cclk(TARG_DBUS_daisychain_cclk     ) , // Daisy-chain program input,  source-synchronous clock for config
                              .DBUS_TARG_daisychain_cclk(DBUS_TARG_daisychain_cclk     ) , // Daisy-chain program output, source-synchronous clock for config
                              .DbgBlkId                 (DbgBlkId                 [7:0]) , // Debug client ID - hard tie
                              .AEB_DBUS                 (mrk_aeb_dbus                  ) , // Security lock down. 1'b0-> hold program to reset == data forward

                              .DBUS_TARG_daisychain_ctl (DBUS_TARG_daisychain_ctl [1:0]) , // Daisy-chain program output, reverse of data flow
                              .TARG_DBUS_daisychain_ctl (TARG_DBUS_daisychain_ctl [1:0]) , // Daisy-chain program input,  reverse of data flow

                              .ctlProgClrDat            (ctlProgClrDat                 ) , // Clear the data pipe
                              .ctlProgClrCtl            (ctlProgClrCtl                 ) , // Clear the Control logic data
                              .ctlProgVal               (ctlProgVal                    ) , // control program : control valid
                              .ctlProg_ClkOn            (R_ctlProg_ClkOn               ) , // control program : Clock on / off;
                              .ctlProg_ClkEnSel         (ctlProg_ClkEnSel              ) , // control program : Clock enable select
                              .ctlProg_ByteSel          (ctlProg_ByteSel          [1:0]) , // control program : bit Byte select // removed [2]Multi-block mode
                              .ctlProg_MuxId            (ctlProg_MuxId            [2:0]) , // control program : Mux ID of the current program target
                              .ctlProg_LaneSel          (ctlProg_LaneSel          [1:0]) , // control program : Lane select of mux
                              .ctlProg_ChannelSel       (ctlProg_ChannelSel       [5:0])   // control program : Channel select of mux
                              );

  reg [(LANE_COUNT-1):0]       regClkEnSel;
  // Store Mux select and clock enable select info per Lane
  // [7:6] Byte Select; [5:0] Chennel Select;
  reg [7:0]                    MuxSelLane0;
  reg [7:0]                    MuxSelLane1;
  reg [7:0]                    MuxSelLane2;

  always @(posedge gcclk or posedge ctlProgClrCtl)
     if (ctlProgClrCtl == 1'b1) begin
        MuxSelLane0[7:0] <= {8 {1'b0}};
        MuxSelLane1[7:0] <= {8 {1'b0}};
        MuxSelLane2[7:0] <= {8 {1'b0}};
     end else
       case ({ctlProgVal,ctlProg_LaneSel[1:0]})
         3'b100 : begin
            MuxSelLane0[7:0] <= {ctlProg_ByteSel[1:0], ctlProg_ChannelSel[5:0]};
            MuxSelLane1[7:0] <= MuxSelLane1[7:0];
            MuxSelLane2[7:0] <= MuxSelLane2[7:0];
         end
         3'b101 : begin
            MuxSelLane0[7:0] <= MuxSelLane0[7:0];
            MuxSelLane1[7:0] <= {ctlProg_ByteSel[1:0], ctlProg_ChannelSel[5:0]};
            MuxSelLane2[7:0] <= MuxSelLane2[7:0];
         end
         3'b110 : begin
            MuxSelLane0[7:0] <= MuxSelLane0[7:0];
            MuxSelLane1[7:0] <= MuxSelLane1[7:0];
            MuxSelLane2[7:0] <= {ctlProg_ByteSel[1:0], ctlProg_ChannelSel[5:0]};
         end
         default : begin
            MuxSelLane0[7:0] <= MuxSelLane0[7:0];
            MuxSelLane1[7:0] <= MuxSelLane1[7:0];
            MuxSelLane2[7:0] <= MuxSelLane2[7:0];
         end
       endcase // case ({ctlProgVal,ctlProg_LaneSel[1:0]})

  always @(posedge gcclk or posedge ctlProgClrCtl)
     if (ctlProgClrCtl == 1'b1)
        regClkEnSel[2:0] <= {3 {1'b0}};
     else
       case ({ctlProgVal,ctlProg_LaneSel[1:0]})
         3'b100  : regClkEnSel[2:0] <= {regClkEnSel[2:1], ctlProg_ClkEnSel};
         3'b101  : regClkEnSel[2:0] <= {regClkEnSel[2],   ctlProg_ClkEnSel, regClkEnSel[0]};
         3'b110  : regClkEnSel[2:0] <= {ctlProg_ClkEnSel, regClkEnSel[1:0]};
         default : regClkEnSel[2:0] <= regClkEnSel[2:0];
       endcase // case ({ctlProgVal,ctlProg_LaneSel[1:0]})

generate
if (LANE_COUNT>3) begin
  reg [7:0]                    MuxSelLane3;
  always @(posedge gcclk or posedge ctlProgClrCtl)
     if (ctlProgClrCtl == 1'b1) begin
        MuxSelLane3[7:0] <= {8 {1'b0}};
        regClkEnSel[3]   <=     1'b0;
     end else
       case ({ctlProgVal,ctlProg_LaneSel[1:0]})
         3'b111  : begin
            MuxSelLane3[7:0] <= {ctlProg_ByteSel[1:0], ctlProg_ChannelSel[5:0]};
            regClkEnSel[3]   <=  ctlProg_ClkEnSel;
         end
         default : begin
            MuxSelLane3[7:0] <= MuxSelLane3[7:0];
            regClkEnSel[3]   <= regClkEnSel[3];
         end
       endcase // case ({ctlProgVal,ctlProg_LaneSel[1:0]})

  // Re-map nets
  assign R_Mux0SelLane[34:27] = MuxSelLane3[7:0];
  assign R_Mux1SelLane[34:27] = MuxSelLane3[7:0];
  assign R_Mux2SelLane[34:27] = MuxSelLane3[7:0];
  assign R_Mux3SelLane[34:27] = MuxSelLane3[7:0];
  assign R_Mux4SelLane[34:27] = MuxSelLane3[7:0];
  assign R_Mux5SelLane[34:27] = MuxSelLane3[7:0];
end
endgenerate

  assign R_Mux0SelLane[25:18] = MuxSelLane2[7:0];
  assign R_Mux1SelLane[25:18] = MuxSelLane2[7:0];
  assign R_Mux2SelLane[25:18] = MuxSelLane2[7:0];
  assign R_Mux3SelLane[25:18] = MuxSelLane2[7:0];
  assign R_Mux4SelLane[25:18] = MuxSelLane2[7:0];
  assign R_Mux5SelLane[25:18] = MuxSelLane2[7:0];
  assign R_Mux0SelLane[16: 9] = MuxSelLane1[7:0];
  assign R_Mux1SelLane[16: 9] = MuxSelLane1[7:0];
  assign R_Mux2SelLane[16: 9] = MuxSelLane1[7:0];
  assign R_Mux3SelLane[16: 9] = MuxSelLane1[7:0];
  assign R_Mux4SelLane[16: 9] = MuxSelLane1[7:0];
  assign R_Mux5SelLane[16: 9] = MuxSelLane1[7:0];
  assign R_Mux0SelLane[ 7: 0] = MuxSelLane0[7:0];
  assign R_Mux1SelLane[ 7: 0] = MuxSelLane0[7:0];
  assign R_Mux2SelLane[ 7: 0] = MuxSelLane0[7:0];
  assign R_Mux3SelLane[ 7: 0] = MuxSelLane0[7:0];
  assign R_Mux4SelLane[ 7: 0] = MuxSelLane0[7:0];
  assign R_Mux5SelLane[ 7: 0] = MuxSelLane0[7:0];

  // Store 3 bit Mux address per Lane and decode   (for 3 or more mux)
  // Reset Mux address to 3'b111, since that is not a valid MuxId
  reg [2:0]                    regLaneMuxId[LANE_COUNT-1:0];

generate
   genvar i;
   for(i=0; i < LANE_COUNT; i = i+1) begin : lanemuxid_i
      assign LaneMuxId[i*3 +: 3] = regLaneMuxId[i][2:0];
   end
endgenerate

  always @(posedge gcclk or posedge ctlProgClrCtl)
     if (ctlProgClrCtl == 1'b1) begin
        regLaneMuxId[0][2:0] <= {3 {1'b1}};
        regLaneMuxId[1][2:0] <= {3 {1'b1}};
        regLaneMuxId[2][2:0] <= {3 {1'b1}};
     end else
       case ({ctlProgVal,ctlProg_LaneSel[1:0]})
         3'b100 : begin
            regLaneMuxId[0][2:0] <= ctlProg_MuxId[2:0];
            regLaneMuxId[1][2:0] <= regLaneMuxId[1][2:0];
            regLaneMuxId[2][2:0] <= regLaneMuxId[2][2:0];
         end
         3'b101 : begin
            regLaneMuxId[0][2:0] <= regLaneMuxId[0][2:0];
            regLaneMuxId[1][2:0] <= ctlProg_MuxId[2:0];
            regLaneMuxId[2][2:0] <= regLaneMuxId[2][2:0];
         end
         3'b110 : begin
            regLaneMuxId[0][2:0] <= regLaneMuxId[0][2:0];
            regLaneMuxId[1][2:0] <= regLaneMuxId[1][2:0];
            regLaneMuxId[2][2:0] <= ctlProg_MuxId[2:0];
         end
         default : begin
            regLaneMuxId[0][2:0] <= regLaneMuxId[0][2:0];
            regLaneMuxId[1][2:0] <= regLaneMuxId[1][2:0];
            regLaneMuxId[2][2:0] <= regLaneMuxId[2][2:0];
         end
       endcase // case ({ctlProgVal,ctlProg_LaneSel[1:0]})

// Decode the MuxId for each lane
// If timing needs it, we can multi-cycle this path
  assign R_Mux0SelLane[ 8] = (regLaneMuxId[0][2:0] == 3'b000);
  assign R_Mux0SelLane[17] = (regLaneMuxId[1][2:0] == 3'b000);
  assign R_Mux0SelLane[26] = (regLaneMuxId[2][2:0] == 3'b000);
  assign R_Mux1SelLane[ 8] = (regLaneMuxId[0][2:0] == 3'b001);
  assign R_Mux1SelLane[17] = (regLaneMuxId[1][2:0] == 3'b001);
  assign R_Mux1SelLane[26] = (regLaneMuxId[2][2:0] == 3'b001);
  assign R_Mux2SelLane[ 8] = (regLaneMuxId[0][2:0] == 3'b010);
  assign R_Mux2SelLane[17] = (regLaneMuxId[1][2:0] == 3'b010);
  assign R_Mux2SelLane[26] = (regLaneMuxId[2][2:0] == 3'b010);
  assign R_Mux3SelLane[ 8] = (regLaneMuxId[0][2:0] == 3'b011);
  assign R_Mux3SelLane[17] = (regLaneMuxId[1][2:0] == 3'b011);
  assign R_Mux3SelLane[26] = (regLaneMuxId[2][2:0] == 3'b011);
  assign R_Mux4SelLane[ 8] = (regLaneMuxId[0][2:0] == 3'b100);
  assign R_Mux4SelLane[17] = (regLaneMuxId[1][2:0] == 3'b100);
  assign R_Mux4SelLane[26] = (regLaneMuxId[2][2:0] == 3'b100);
  assign R_Mux5SelLane[ 8] = (regLaneMuxId[0][2:0] == 3'b101);
  assign R_Mux5SelLane[17] = (regLaneMuxId[1][2:0] == 3'b101);
  assign R_Mux5SelLane[26] = (regLaneMuxId[2][2:0] == 3'b101);
generate
if (LANE_COUNT>3) begin
  always @(posedge gcclk or posedge ctlProgClrCtl)
     if (ctlProgClrCtl == 1'b1) begin
        regLaneMuxId[3][2:0] <= {3 {1'b1}};
     end else
       case ({ctlProgVal,ctlProg_LaneSel[1:0]})
         3'b111  : regLaneMuxId[3][2:0] <= ctlProg_MuxId[2:0];
         default : regLaneMuxId[3][2:0] <= regLaneMuxId[3][2:0];
       endcase // case ({ctlProgVal,ctlProg_LaneSel[1:0]})

// Decode the MuxId for each lane
// If timing needs it, we can multi-cycle this path
   assign R_Mux0SelLane[35] = (regLaneMuxId[3][2:0] == 3'b000);
   assign R_Mux1SelLane[35] = (regLaneMuxId[3][2:0] == 3'b001);
   assign R_Mux2SelLane[35] = (regLaneMuxId[3][2:0] == 3'b010);
   assign R_Mux3SelLane[35] = (regLaneMuxId[3][2:0] == 3'b011);
   assign R_Mux4SelLane[35] = (regLaneMuxId[3][2:0] == 3'b100);
   assign R_Mux5SelLane[35] = (regLaneMuxId[3][2:0] == 3'b101);
end // if (LANE_COUNT>3)
endgenerate
  // End ----------------------------------------------------------------------

// Using source clock because dest clock could be off
// Using AEB_DBUS with cdce to clean up x on output
  techind_cdcfpm  #(.size (LANE_COUNT*9),.applyParams(1),  .DestClkCnt(0), .CdcDestSetup(11000)) mrk_Mux0SelLane (.A(R_Mux0SelLane[(LANE_COUNT*9-1):0]), .DSTCLK(gcclk), .Z(Mux0SelLane[(LANE_COUNT*9-1):0]));
  techind_cdcfpm  #(.size (LANE_COUNT*9),.applyParams(1),  .DestClkCnt(0), .CdcDestSetup(11000)) mrk_Mux1SelLane (.A(R_Mux1SelLane[(LANE_COUNT*9-1):0]), .DSTCLK(gcclk), .Z(Mux1SelLane[(LANE_COUNT*9-1):0]));
  techind_cdcfpm  #(.size (LANE_COUNT*9),.applyParams(1),  .DestClkCnt(0), .CdcDestSetup(11000)) mrk_Mux2SelLane (.A(R_Mux2SelLane[(LANE_COUNT*9-1):0]), .DSTCLK(gcclk), .Z(Mux2SelLane[(LANE_COUNT*9-1):0]));
  techind_cdcfpm  #(.size (LANE_COUNT*9),.applyParams(1),  .DestClkCnt(0), .CdcDestSetup(11000)) mrk_Mux3SelLane (.A(R_Mux3SelLane[(LANE_COUNT*9-1):0]), .DSTCLK(gcclk), .Z(Mux3SelLane[(LANE_COUNT*9-1):0]));
  techind_cdcfpm  #(.size (LANE_COUNT*9),.applyParams(1),  .DestClkCnt(0), .CdcDestSetup(11000)) mrk_Mux4SelLane (.A(R_Mux4SelLane[(LANE_COUNT*9-1):0]), .DSTCLK(gcclk), .Z(Mux4SelLane[(LANE_COUNT*9-1):0]));
  techind_cdcfpm  #(.size (LANE_COUNT*9),.applyParams(1),  .DestClkCnt(0), .CdcDestSetup(11000)) mrk_Mux5SelLane (.A(R_Mux5SelLane[(LANE_COUNT*9-1):0]), .DSTCLK(gcclk), .Z(Mux5SelLane[(LANE_COUNT*9-1):0]));

  techind_cdcfpm   #(.size (LANE_COUNT),.applyParams(1),  .DestClkCnt(0), .CdcDestSetup(11000)) mrk_ClkEnSel (.A(regClkEnSel[(LANE_COUNT-1):0]), .DSTCLK(gcclk), .Z(ClkEnSel[(LANE_COUNT-1):0]));

// couyang - Can not cdc bug this because of synchronizer
//wire    cdc_ctlProg_ClkOn;
//techind_cdcfpm   #(.size ( 1),.applyParams(1),  .DestClkCnt(3), .CdcDestSetup(0)) mrk_cdc_ctlProg_ClkOn(.A(R_ctlProg_ClkOn    ), .DSTCLK(gcclk), .Z(cdc_ctlProg_ClkOn));
//dbg_client_mrk_or mrk_ctlProg_ClkOn (.A(cdc_ctlProg_ClkOn), .B(TEST_CLK_EN), .Y(ctlProg_ClkOn)); // Force ClkOn during scan shift
  dbg_client_mrk_or mrk_ctlProg_ClkOn (.A(  R_ctlProg_ClkOn), .B(TEST_CLK_EN), .Y(ctlProg_ClkOn)); // Force ClkOn during scan shift
endmodule // dbg_client_ssb__aon_prog
