//                              -*- Mode: Verilog -*-
// (c) 2015, 2016 Advanced Micro Devices, Inc. All rights reserved.


////////////////////////////////////////////////////////////////////////////
//
// Filename    : dbg_client_ssb__aon_steer.v.dpl
// Description : Mux for synchronous debug bus client.
//
// NOTE : Multi-clock case should use Source-synchronous debug bus client
//
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
// Print out some input parameters to dperl --- begin
// MKEY          = 
// DBG_WIDTH     = 
// MUX_COUNT     = 
// Print out some input parameters to dperl ----- end
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////

module dbg_client_ssb__aon_steer
#(parameter LANE_COUNT  = 4
,           LANE_WIDTH  = 8
,           MUX_USED    = 6'b000000
,           MUX5_USECLK = 3'b101     // Used to avoid clock re-convergence issues in physical design
,           MUX4_USECLK = 3'b100     // Used to avoid clock re-convergence issues in physical design
,           MUX3_USECLK = 3'b011     // Used to avoid clock re-convergence issues in physical design
,           MUX2_USECLK = 3'b010     // Used to avoid clock re-convergence issues in physical design
,           MUX1_USECLK = 3'b001     // Used to avoid clock re-convergence issues in physical design
,           MUX0_USECLK = 3'b000     // Used to avoid clock re-convergence issues in physical design
)

// ==========================================================================
// Debug bus Daisy-chain inputs and outputs
// Data flows towards DSM. ctl flows away from DSM
// Begin --------------------------------------------------------------------
( output                            [1:0] DBUS_TARG_daisychain_ctl  // Daisy-chain program output, reverse of data flow
, input                             [1:0] TARG_DBUS_daisychain_ctl  // Daisy-chain program input,  reverse of data flow
                                         
, output                                  DBUS_TARG_daisychain_cclk // Daisy-chain program output, source-synchronous clock for config
, input                                   TARG_DBUS_daisychain_cclk // Daisy-chain program input,  source-synchronous clock for config
  
, input       [LANE_COUNT*LANE_WIDTH-1:0] DBUS_TARG_daisychain_data // Daisy-chain input
, output      [LANE_COUNT*LANE_WIDTH-1:0] TARG_DBUS_daisychain_data // Daisy-chain output
  
, input       [LANE_COUNT-1           :0] DBUS_TARG_daisychain_dclk // Daisy-chain input, source-synchronous clock for data
, output      [LANE_COUNT-1           :0] TARG_DBUS_daisychain_dclk // Daisy-chain output source-synchronous clock for data

// End ----------------------------------------------------------------------

// ==========================================================================
// interface to all dbg_client_ssb__clkmux instance
// Begin --------------------------------------------------------------------
, output                                  ctlProg_ClkOn              // Clock on / off; Used to as async resetb in dbg_client_ssb__mux
// End ----------------------------------------------------------------------

// ==========================================================================
// interface to each dbg_client_ssb__mux instance
// Begin --------------------------------------------------------------------
, output      [(LANE_COUNT*9-1)       :0] dbg_Mux5_SelLane  // LANE_COUNT Lanes * ([8] Lane Enable; [7:6] Byte Select; [5:0] Chennel Select;)
, input       [LANE_COUNT*LANE_WIDTH-1:0] dbg_Mux5_Data     // Debug-bus data outputs after muxing
, input       [LANE_COUNT-1           :0] dbg_Mux5_Clk      // Selected Clock from each dbg_client_ssb__clkmux
, output      [(LANE_COUNT*9-1)       :0] dbg_Mux4_SelLane  // LANE_COUNT Lanes * ([8] Lane Enable; [7:6] Byte Select; [5:0] Chennel Select;)
, input       [LANE_COUNT*LANE_WIDTH-1:0] dbg_Mux4_Data     // Debug-bus data outputs after muxing
, input       [LANE_COUNT-1           :0] dbg_Mux4_Clk      // Selected Clock from each dbg_client_ssb__clkmux
, output      [(LANE_COUNT*9-1)       :0] dbg_Mux3_SelLane  // LANE_COUNT Lanes * ([8] Lane Enable; [7:6] Byte Select; [5:0] Chennel Select;)
, input       [LANE_COUNT*LANE_WIDTH-1:0] dbg_Mux3_Data     // Debug-bus data outputs after muxing
, input       [LANE_COUNT-1           :0] dbg_Mux3_Clk      // Selected Clock from each dbg_client_ssb__clkmux
, output      [(LANE_COUNT*9-1)       :0] dbg_Mux2_SelLane  // LANE_COUNT Lanes * ([8] Lane Enable; [7:6] Byte Select; [5:0] Chennel Select;)
, input       [LANE_COUNT*LANE_WIDTH-1:0] dbg_Mux2_Data     // Debug-bus data outputs after muxing
, input       [LANE_COUNT-1           :0] dbg_Mux2_Clk      // Selected Clock from each dbg_client_ssb__clkmux
, output      [(LANE_COUNT*9-1)       :0] dbg_Mux1_SelLane  // LANE_COUNT Lanes * ([8] Lane Enable; [7:6] Byte Select; [5:0] Chennel Select;)
, input       [LANE_COUNT*LANE_WIDTH-1:0] dbg_Mux1_Data     // Debug-bus data outputs after muxing
, input       [LANE_COUNT-1           :0] dbg_Mux1_Clk      // Selected Clock from each dbg_client_ssb__clkmux
, output      [(LANE_COUNT*9-1)       :0] dbg_Mux0_SelLane  // LANE_COUNT Lanes * ([8] Lane Enable; [7:6] Byte Select; [5:0] Chennel Select;)
, input       [LANE_COUNT*LANE_WIDTH-1:0] dbg_Mux0_Data     // Debug-bus data outputs after muxing
, input       [LANE_COUNT-1           :0] dbg_Mux0_Clk      // Selected Clock from each dbg_client_ssb__clkmux
// End ----------------------------------------------------------------------

// ==========================================================================
// Misc
// Begin --------------------------------------------------------------------
//, input                        ClkEnEx                   // External clock enable trigger
//, output      [LANE_COUNT-1:0] ClkEnOut                  // Raw clock enable trigger for external use
, input       [7:0]            DbgBlkId                  // Debug client ID - hard tie
, input                        AEB_DBUS                  // Security lock down. 1'b0-> hold program to reset == data forward
, input                        TEST_CLK_EN               // Gater override for test mode. In test mode drive all logic in TARG_DBUS_daisychain_cclk
, output      [7:0]            dbg_BlkId                  // Debug bus ID used for debug the debug bus client (same as DbgBlkId)
// End ----------------------------------------------------------------------
);


  // Plan to be non-scan so tie off TEST_CLK_EN connections and leave port unused
  //! unused TEST_CLK_EN

  wire [(LANE_COUNT-1):0] ClkEnOut;
  wire [(LANE_COUNT-1):0] ClkEnSel;
  wire [ LANE_COUNT   :0] ClkEn;

  assign                  ClkEn[0] = 1'b1; //ClkEnEx;
  wire [(LANE_COUNT*3-1):0] LaneMuxId;
  // ==========================================================================
  // Control Path
  // Begin --------------------------------------------------------------------

  dbg_client_ssb__aon_prog #(.LANE_COUNT(LANE_COUNT))
    I_dbg_client_ssb__aon_prog 
    ( .DBUS_TARG_daisychain_cclk (DBUS_TARG_daisychain_cclk           )  // Daisy-chain program output, source-synchronous clock for config
    , .DBUS_TARG_daisychain_ctl  (DBUS_TARG_daisychain_ctl      [ 1:0])  // Daisy-chain program output, reverse of data flow

    , .TARG_DBUS_daisychain_cclk (TARG_DBUS_daisychain_cclk           )  // Daisy-chain program input,  source-synchronous clock for config
    , .TARG_DBUS_daisychain_ctl  (TARG_DBUS_daisychain_ctl      [ 1:0])  // Daisy-chain program input,  reverse of data flow

    , .ctlProg_ClkOn             (ctlProg_ClkOn                       )  // Clock on / off;

    , .Mux5SelLane               (dbg_Mux5_SelLane[(LANE_COUNT*9-1):0])  // MuxId 5: LANE_COUNT * ([8] Lane Enable; [7:6] Byte Select; [5:0] Chennel Select;)
    , .Mux4SelLane               (dbg_Mux4_SelLane[(LANE_COUNT*9-1):0])  // MuxId 4: LANE_COUNT * ([8] Lane Enable; [7:6] Byte Select; [5:0] Chennel Select;)
    , .Mux3SelLane               (dbg_Mux3_SelLane[(LANE_COUNT*9-1):0])  // MuxId 3: LANE_COUNT * ([8] Lane Enable; [7:6] Byte Select; [5:0] Chennel Select;)
    , .Mux2SelLane               (dbg_Mux2_SelLane[(LANE_COUNT*9-1):0])  // MuxId 2: LANE_COUNT * ([8] Lane Enable; [7:6] Byte Select; [5:0] Chennel Select;)
    , .Mux1SelLane               (dbg_Mux1_SelLane[(LANE_COUNT*9-1):0])  // MuxId 1: LANE_COUNT * ([8] Lane Enable; [7:6] Byte Select; [5:0] Chennel Select;)
    , .Mux0SelLane               (dbg_Mux0_SelLane[(LANE_COUNT*9-1):0])  // MuxId 0: LANE_COUNT * ([8] Lane Enable; [7:6] Byte Select; [5:0] Chennel Select;)
    , .LaneMuxId                 (LaneMuxId       [(LANE_COUNT*3-1):0])  // Mux Id selection for each lane
    , .ClkEnSel                  (ClkEnSel          [(LANE_COUNT-1):0])  // control program : Clock enable select
    , .DbgBlkId                  (DbgBlkId                       [7:0])  // Debug client ID - hard tie
    , .AEB_DBUS                  (AEB_DBUS                            )  // Security lock down. 1'b0-> hold program to reset == data forward
    , .TEST_CLK_EN               (1'b0                                )  // TEST_CLK_EN - Gater override for test mode. In test mode drive all logic in TARG_DBUS_daisychain_cclk
    , .dbg_rst                   (dbg_rst                             )  // Reset and clear data pipe
    , .dbg_BlkId                 (dbg_BlkId                      [7:0])  // Debug bus ID used for debug the debug bus client (same as DbgBlkId)
    );
  // End ----------------------------------------------------------------------

  // ==========================================================================
  // Data Pipe
  // Each lane is it's own clock domaine.
  // However, for physical design it should be controlled to be 1 clock domain
  // because all lane can be driven by same clock
  // Begin --------------------------------------------------------------------
  wire [5:0]            map_Mux_Clk  [(LANE_COUNT-1):0];
  wire [2:0]            map_LaneMuxId[(LANE_COUNT-1):0];

generate
   genvar    i;
   for(i=0; i < LANE_COUNT; i = i+1) begin :mux_clk_map
      assign map_Mux_Clk[i][5:0]   = { dbg_Mux5_Clk[i], dbg_Mux4_Clk[i], dbg_Mux3_Clk[i], dbg_Mux2_Clk[i], dbg_Mux1_Clk[i], dbg_Mux0_Clk[i]};
      assign map_LaneMuxId[i][2:0] = LaneMuxId[(i+1)*3-1 : i*3];
   end
endgenerate

  wire [LANE_WIDTH-1:0] map_in__data [(LANE_COUNT-1):0];
  wire [LANE_WIDTH-1:0] map_out_data [(LANE_COUNT-1):0];
  wire [LANE_WIDTH-1:0] map_Mux0_Data[(LANE_COUNT-1):0];
  wire [LANE_WIDTH-1:0] map_Mux1_Data[(LANE_COUNT-1):0];
  wire [LANE_WIDTH-1:0] map_Mux2_Data[(LANE_COUNT-1):0];
  wire [LANE_WIDTH-1:0] map_Mux3_Data[(LANE_COUNT-1):0];
  wire [LANE_WIDTH-1:0] map_Mux4_Data[(LANE_COUNT-1):0];
  wire [LANE_WIDTH-1:0] map_Mux5_Data[(LANE_COUNT-1):0];

generate
   genvar    j;
   for(j=0; j < LANE_COUNT; j = j+1) begin :map_bus
      assign map_in__data[j][LANE_WIDTH-1:0] = DBUS_TARG_daisychain_data[(j+1)*LANE_WIDTH-1 : j*LANE_WIDTH];
      assign TARG_DBUS_daisychain_data[(j+1)*LANE_WIDTH-1 : j*LANE_WIDTH] = map_out_data[j][LANE_WIDTH-1:0];
      assign map_Mux0_Data[j][LANE_WIDTH-1:0] = dbg_Mux0_Data[(j+1)*LANE_WIDTH-1 : j*LANE_WIDTH];
      assign map_Mux1_Data[j][LANE_WIDTH-1:0] = dbg_Mux1_Data[(j+1)*LANE_WIDTH-1 : j*LANE_WIDTH];
      assign map_Mux2_Data[j][LANE_WIDTH-1:0] = dbg_Mux2_Data[(j+1)*LANE_WIDTH-1 : j*LANE_WIDTH];
      assign map_Mux3_Data[j][LANE_WIDTH-1:0] = dbg_Mux3_Data[(j+1)*LANE_WIDTH-1 : j*LANE_WIDTH];
      assign map_Mux4_Data[j][LANE_WIDTH-1:0] = dbg_Mux4_Data[(j+1)*LANE_WIDTH-1 : j*LANE_WIDTH];
      assign map_Mux5_Data[j][LANE_WIDTH-1:0] = dbg_Mux5_Data[(j+1)*LANE_WIDTH-1 : j*LANE_WIDTH];
   end // block: map_bus
endgenerate

generate
   if (LANE_COUNT>3) begin
      dbg_client_ssb__aon_muxf #(.MUX_USED(MUX_USED), .LANE_WIDTH(LANE_WIDTH))
        I_dbg_client_ssb__aon_muxf3
        ( .DBUS_TARG_daisychain_dclk (DBUS_TARG_daisychain_dclk     [3])  // Daisy-chain input,  source-synchronous clock for data
        , .DBUS_TARG_daisychain_data (map_in__data[3][LANE_WIDTH-1 : 0])  // Daisy-chain input
        , .TARG_DBUS_daisychain_dclk (TARG_DBUS_daisychain_dclk     [3])  // Daisy-chain output, source-synchronous clock for data
        , .TARG_DBUS_daisychain_data (map_out_data[3][LANE_WIDTH-1 : 0])  // Daisy-chain output
        , .LaneMuxId                 (map_LaneMuxId            [3][2:0])  // Mux Id selection for each lane
        , .dbg_Mux_Clk               (map_Mux_Clk              [3][5:0])  // Mux Id selection for each lane

        , .dbg_Data0Muxed            (map_Mux0_Data [3][LANE_WIDTH-1:0])  // Debug-bus data  outputs after muxing
        , .dbg_Data1Muxed            (map_Mux1_Data [3][LANE_WIDTH-1:0])  // Debug-bus data  outputs after muxing
        , .dbg_Data2Muxed            (map_Mux2_Data [3][LANE_WIDTH-1:0])  // Debug-bus data  outputs after muxing
        , .dbg_Data3Muxed            (map_Mux3_Data [3][LANE_WIDTH-1:0])  // Debug-bus data  outputs after muxing
        , .dbg_Data4Muxed            (map_Mux4_Data [3][LANE_WIDTH-1:0])  // Debug-bus data  outputs after muxing
        , .dbg_Data5Muxed            (map_Mux5_Data [3][LANE_WIDTH-1:0])  // Debug-bus data  outputs after muxing

        , .ClkEn_Raw                 (ClkEnOut                      [3])  // Raw clock enable trigger for external use
        , .ClkEnSel                  (ClkEnSel                      [3])  // control program : Clock enable select
        , .ClkEnBypass               (ClkEnSel[0]) 
        , .ClkEnOut                  (ClkEn                       [3+1])  // Clock enable Out
        , .ClkEnIn                   (ClkEn                         [3])  // Clock enable In
        , .ClkOn_async               (ctlProg_ClkOn                    )  // Clock on / off;
        , .cclk                      (TARG_DBUS_daisychain_cclk        )  // Control Programming Clock to use for reset data pipe
        , .dbg_rst                   (dbg_rst                          )  // Reset and clear data pipe
        , .testClk                   (1'b0                             )  // clock to use if TEST_CLK_EN == 1'b1
        , .TEST_CLK_EN               (1'b0                             )  // TEST_CLK_EN - Gater override for test mode
        );
   end

   if (LANE_COUNT>2) begin
      dbg_client_ssb__aon_muxf #(.MUX_USED(MUX_USED), .LANE_WIDTH(LANE_WIDTH))
        I_dbg_client_ssb__aon_muxf2
        ( .DBUS_TARG_daisychain_dclk (DBUS_TARG_daisychain_dclk     [2])  // Daisy-chain input,  source-synchronous clock for data
        , .DBUS_TARG_daisychain_data (map_in__data[2][LANE_WIDTH-1 : 0])  // Daisy-chain input
        , .TARG_DBUS_daisychain_dclk (TARG_DBUS_daisychain_dclk     [2])  // Daisy-chain output, source-synchronous clock for data
        , .TARG_DBUS_daisychain_data (map_out_data[2][LANE_WIDTH-1 : 0])  // Daisy-chain output
        , .LaneMuxId                 (map_LaneMuxId            [2][2:0])  // Mux Id selection for each lane
        , .dbg_Mux_Clk               (map_Mux_Clk              [2][5:0])  // Mux Id selection for each lane

        , .dbg_Data0Muxed            (map_Mux0_Data [2][LANE_WIDTH-1:0])  // Debug-bus data  outputs after muxing
        , .dbg_Data1Muxed            (map_Mux1_Data [2][LANE_WIDTH-1:0])  // Debug-bus data  outputs after muxing
        , .dbg_Data2Muxed            (map_Mux2_Data [2][LANE_WIDTH-1:0])  // Debug-bus data  outputs after muxing
        , .dbg_Data3Muxed            (map_Mux3_Data [2][LANE_WIDTH-1:0])  // Debug-bus data  outputs after muxing
        , .dbg_Data4Muxed            (map_Mux4_Data [2][LANE_WIDTH-1:0])  // Debug-bus data  outputs after muxing
        , .dbg_Data5Muxed            (map_Mux5_Data [2][LANE_WIDTH-1:0])  // Debug-bus data  outputs after muxing

        , .ClkEn_Raw                 (ClkEnOut                      [2])  // Raw clock enable trigger for external use
        , .ClkEnSel                  (ClkEnSel                      [2])  // control program : Clock enable select
        , .ClkEnBypass               (ClkEnSel[0]) 
        , .ClkEnOut                  (ClkEn                       [2+1])  // Clock enable Out
        , .ClkEnIn                   (ClkEn                         [2])  // Clock enable In
        , .ClkOn_async               (ctlProg_ClkOn                    )  // Clock on / off;
        , .cclk                      (TARG_DBUS_daisychain_cclk        )  // Control Programming Clock to use for reset data pipe
        , .dbg_rst                   (dbg_rst                          )  // Reset and clear data pipe
        , .testClk                   (1'b0                             )  // clock to use if TEST_CLK_EN == 1'b1
        , .TEST_CLK_EN               (1'b0                             )  // TEST_CLK_EN - Gater override for test mode
        );
   end

   if (LANE_COUNT>1) begin
      dbg_client_ssb__aon_muxf #(.MUX_USED(MUX_USED), .LANE_WIDTH(LANE_WIDTH))
        I_dbg_client_ssb__aon_muxf1
        ( .DBUS_TARG_daisychain_dclk (DBUS_TARG_daisychain_dclk     [1])  // Daisy-chain input,  source-synchronous clock for data
        , .DBUS_TARG_daisychain_data (map_in__data[1][LANE_WIDTH-1 : 0])  // Daisy-chain input
        , .TARG_DBUS_daisychain_dclk (TARG_DBUS_daisychain_dclk     [1])  // Daisy-chain output, source-synchronous clock for data
        , .TARG_DBUS_daisychain_data (map_out_data[1][LANE_WIDTH-1 : 0])  // Daisy-chain output
        , .LaneMuxId                 (map_LaneMuxId            [1][2:0])  // Mux Id selection for each lane
        , .dbg_Mux_Clk               (map_Mux_Clk              [1][5:0])  // Mux Id selection for each lane

        , .dbg_Data0Muxed            (map_Mux0_Data [1][LANE_WIDTH-1:0])  // Debug-bus data  outputs after muxing
        , .dbg_Data1Muxed            (map_Mux1_Data [1][LANE_WIDTH-1:0])  // Debug-bus data  outputs after muxing
        , .dbg_Data2Muxed            (map_Mux2_Data [1][LANE_WIDTH-1:0])  // Debug-bus data  outputs after muxing
        , .dbg_Data3Muxed            (map_Mux3_Data [1][LANE_WIDTH-1:0])  // Debug-bus data  outputs after muxing
        , .dbg_Data4Muxed            (map_Mux4_Data [1][LANE_WIDTH-1:0])  // Debug-bus data  outputs after muxing
        , .dbg_Data5Muxed            (map_Mux5_Data [1][LANE_WIDTH-1:0])  // Debug-bus data  outputs after muxing

        , .ClkEn_Raw                 (ClkEnOut                      [1])  // Raw clock enable trigger for external use
        , .ClkEnSel                  (ClkEnSel                      [1])  // control program : Clock enable select
        , .ClkEnBypass               (ClkEnSel[0]) 
        , .ClkEnOut                  (ClkEn                       [1+1])  // Clock enable Out
        , .ClkEnIn                   (ClkEn                         [1])  // Clock enable In
        , .ClkOn_async               (ctlProg_ClkOn                    )  // Clock on / off;
        , .cclk                      (TARG_DBUS_daisychain_cclk        )  // Control Programming Clock to use for reset data pipe
        , .dbg_rst                   (dbg_rst                          )  // Reset and clear data pipe
        , .testClk                   (1'b0                             )  // clock to use if TEST_CLK_EN == 1'b1
        , .TEST_CLK_EN               (1'b0                             )  // TEST_CLK_EN - Gater override for test mode
        );
   end

   if (LANE_COUNT>0) begin
      dbg_client_ssb__aon_muxf #(.MUX_USED(MUX_USED), .LANE_WIDTH(LANE_WIDTH))
        I_dbg_client_ssb__aon_muxf0
        ( .DBUS_TARG_daisychain_dclk (DBUS_TARG_daisychain_dclk     [0])  // Daisy-chain input,  source-synchronous clock for data
        , .DBUS_TARG_daisychain_data (map_in__data[0][LANE_WIDTH-1 : 0])  // Daisy-chain input
        , .TARG_DBUS_daisychain_dclk (TARG_DBUS_daisychain_dclk     [0])  // Daisy-chain output, source-synchronous clock for data
        , .TARG_DBUS_daisychain_data (map_out_data[0][LANE_WIDTH-1 : 0])  // Daisy-chain output
        , .LaneMuxId                 (map_LaneMuxId            [0][2:0])  // Mux Id selection for each lane
        , .dbg_Mux_Clk               (map_Mux_Clk              [0][5:0])  // Mux Id selection for each lane

        , .dbg_Data0Muxed            (map_Mux0_Data [0][LANE_WIDTH-1:0])  // Debug-bus data  outputs after muxing
        , .dbg_Data1Muxed            (map_Mux1_Data [0][LANE_WIDTH-1:0])  // Debug-bus data  outputs after muxing
        , .dbg_Data2Muxed            (map_Mux2_Data [0][LANE_WIDTH-1:0])  // Debug-bus data  outputs after muxing
        , .dbg_Data3Muxed            (map_Mux3_Data [0][LANE_WIDTH-1:0])  // Debug-bus data  outputs after muxing
        , .dbg_Data4Muxed            (map_Mux4_Data [0][LANE_WIDTH-1:0])  // Debug-bus data  outputs after muxing
        , .dbg_Data5Muxed            (map_Mux5_Data [0][LANE_WIDTH-1:0])  // Debug-bus data  outputs after muxing

        , .ClkEn_Raw                 (ClkEnOut                      [0])  // Raw clock enable trigger for external use
        , .ClkEnSel                  (ClkEnSel                      [0])  // control program : Clock enable select
        , .ClkEnBypass               (ClkEnSel[0]) 
        , .ClkEnOut                  (ClkEn                       [0+1])  // Clock enable Out
        , .ClkEnIn                   (ClkEn                         [0])  // Clock enable In
        , .ClkOn_async               (ctlProg_ClkOn                    )  // Clock on / off;
        , .cclk                      (TARG_DBUS_daisychain_cclk        )  // Control Programming Clock to use for reset data pipe
        , .dbg_rst                   (dbg_rst                          )  // Reset and clear data pipe
        , .testClk                   (1'b0                             )  // clock to use if TEST_CLK_EN == 1'b1
        , .TEST_CLK_EN               (1'b0                             )  // TEST_CLK_EN - Gater override for test mode
        );
   end

endgenerate
  //! unused ClkEn[LANE_COUNT]
  //! unused ClkEnOut
  // End ----------------------------------------------------------------------

endmodule // dbg_client_ssb__aon_steer
