//                              -*- Mode: Verilog -*-
// (c) 2015, 2016 Advanced Micro Devices, Inc. All rights reserved.

////////////////////////////////////////////////////////////////////////////
//
// Filename    : dbg_client_ssb__datmux.v
//
//
////////////////////////////////////////////////////////////////////////////
// Hard code this to only support 8:1
////////////////////////////////////////////////////////////////////////////
module dbg_client_ssb__datmux
#( parameter MUX_USED = 8'b11000000
 , parameter WIDTH    = 8
)
( input  [WIDTH-1:0] dat_i0
, input  [WIDTH-1:0] dat_i1
, input  [WIDTH-1:0] dat_i2
, input  [WIDTH-1:0] dat_i3
, input  [WIDTH-1:0] dat_i4
, input  [WIDTH-1:0] dat_i5
, input  [WIDTH-1:0] dat_i6
, input  [WIDTH-1:0] dat_i7
, input  [2:0]       sel
, output [WIDTH-1:0] dat_o
);

  localparam  MUX_L1 = {|MUX_USED[7:6], |MUX_USED[5:4], |MUX_USED[3:2], |MUX_USED[1:0]} ;
  localparam  MUX_L2 = {|MUX_USED[7:4], |MUX_USED[3:0]} ;

  wire [WIDTH-1:0]  dat_mux_l0_0;
  wire [WIDTH-1:0]  dat_mux_l0_1;
  wire [WIDTH-1:0]  dat_mux_l0_2;
  wire [WIDTH-1:0]  dat_mux_l0_3;
  wire [WIDTH-1:0]  dat_mux_l1_0;
  wire [WIDTH-1:0]  dat_mux_l1_1;

  // Level Zero Muxing 8:4
  dbg_client_datmux_2to1 #( .MUX_USED(MUX_USED[1:0]), .WIDTH(WIDTH))
    I_mux_l0_0 ( .dat_i0(dat_i0[WIDTH-1:0])
               , .dat_i1(dat_i1[WIDTH-1:0])
               , .sel(sel[0])
               , .dat_o(dat_mux_l0_0[WIDTH-1:0])
               );
  dbg_client_datmux_2to1 #( .MUX_USED(MUX_USED[3:2]) , .WIDTH(WIDTH))
    I_mux_l0_1 ( .dat_i0(dat_i2[WIDTH-1:0])
               , .dat_i1(dat_i3[WIDTH-1:0])
               , .sel(sel[0])
               , .dat_o(dat_mux_l0_1[WIDTH-1:0])
               );
  dbg_client_datmux_2to1 #( .MUX_USED(MUX_USED[5:4]) , .WIDTH(WIDTH))
    I_mux_l0_2 ( .dat_i0(dat_i4[WIDTH-1:0])
               , .dat_i1(dat_i5[WIDTH-1:0])
               , .sel(sel[0])
               , .dat_o(dat_mux_l0_2[WIDTH-1:0])
               );
  dbg_client_datmux_2to1 #( .MUX_USED(MUX_USED[7:6]) , .WIDTH(WIDTH))
    I_mux_l0_3 ( .dat_i0(dat_i6[WIDTH-1:0])
               , .dat_i1(dat_i7[WIDTH-1:0])
               , .sel(sel[0])
               , .dat_o(dat_mux_l0_3[WIDTH-1:0])
               );

  // Level One Muxing 4:2
  dbg_client_datmux_2to1 #( .MUX_USED(MUX_L1[1:0]) , .WIDTH(WIDTH))
    I_mux_l1_0 ( .dat_i0(dat_mux_l0_0[WIDTH-1:0])
               , .dat_i1(dat_mux_l0_1[WIDTH-1:0])
               , .sel(sel[1])
               , .dat_o(dat_mux_l1_0[WIDTH-1:0])
               );
  dbg_client_datmux_2to1 #( .MUX_USED(MUX_L1[3:2]) , .WIDTH(WIDTH))
    I_mux_l1_1 ( .dat_i0(dat_mux_l0_2[WIDTH-1:0])
               , .dat_i1(dat_mux_l0_3[WIDTH-1:0])
               , .sel(sel[1])
               , .dat_o(dat_mux_l1_1[WIDTH-1:0])
               );

  // Final Muxing 2:1
  dbg_client_datmux_2to1 #( .MUX_USED(MUX_L2[1:0]) , .WIDTH(WIDTH))
    I_mux_l2_0 ( .dat_i0(dat_mux_l1_0[WIDTH-1:0])
               , .dat_i1(dat_mux_l1_1[WIDTH-1:0])
               , .sel(sel[2])
               , .dat_o(dat_o[WIDTH-1:0])
               );

endmodule // dbg_client_ssb__datmux
