//                              -*- Mode: Verilog -*-
// (c) 2015, 2016 Advanced Micro Devices, Inc. All rights reserved.

////////////////////////////////////////////////////////////////////////////
//
// Filename    : dbg_client_ssb__mux.v.dpl
// Description : Mux for synchronous debug bus client.
//
// NOTE : Multi-clock case should use Source-synchronous debug bus client
//
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
// Print out some input parameters to dperl --- begin
// MKEY          = 
// synchronicity = 
// DBG_WIDTH     = 
// DBG_IN_CNT    = 
// Print out some input parameters to dperl ----- end
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////

module dbg_client_ssb__mux
#(parameter LANE_COUNT     = 4
,           LANE_WIDTH     = 8
)

// ==========================================================================
// interface with dbg_client_ssb__steer
// Begin --------------------------------------------------------------------
( input       [7:0]                       dbg_BlkId          // Debug bus ID used for debug the debug bus client (same as DbgBlkId)
, input       [LANE_COUNT*9         -1:0] dbg_Mux_SelLane     // LANE_COUNT * ([8] Lane Enable; [7:6] Byte Select; [5:0] Channel Select;)
, output      [LANE_COUNT*LANE_WIDTH-1:0] dbg_Mux_Data        // Debug-bus data outputs after muxing
, output      [LANE_COUNT           -1:0] dbg_Mux_Clk         // Clock output for each lane
// End ----------------------------------------------------------------------

// ==========================================================================
// Debug-bus data inputs from functional logic
// Begin --------------------------------------------------------------------
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug62 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug61 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug60 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug59 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug58 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug57 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug56 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug55 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug54 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug53 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug52 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug51 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug50 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug49 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug48 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug47 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug46 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug45 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug44 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug43 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug42 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug41 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug40 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug39 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug38 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug37 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug36 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug35 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug34 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug33 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug32 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug31 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug30 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug29 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug28 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug27 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug26 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug25 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug24 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug23 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug22 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug21 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug20 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug19 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug18 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug17 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug16 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug15 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug14 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug13 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug12 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug11 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug10 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug9 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug8 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug7 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug6 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug5 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug4 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug3 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug2 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug1 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug0 
// End ----------------------------------------------------------------------

, input                            clk            // Clocking of input data
, input                            TEST_CLK_EN    // Gater override for test mode
, input                            ctlProg_ClkOn  // Used to as asyn resetb (async)

);

localparam BUS_WIDTH      = LANE_COUNT*LANE_WIDTH;

wire [LANE_WIDTH-1:0]     dbg_L2DataMuxLane[LANE_COUNT-1:0];
reg  [LANE_WIDTH-1:0]     reg_MuxData[LANE_COUNT-1:0];
wire [LANE_COUNT-1:0]     gclk;
wire [BUS_WIDTH-1:0] debugOut;

//--------------------------------------- Begin
// avoid 2d array duee  to leda complication
wire [BUS_WIDTH-1:0] bdebug63;
wire [BUS_WIDTH-1:0] bdebug62;
wire [BUS_WIDTH-1:0] bdebug61;
wire [BUS_WIDTH-1:0] bdebug60;
wire [BUS_WIDTH-1:0] bdebug59;
wire [BUS_WIDTH-1:0] bdebug58;
wire [BUS_WIDTH-1:0] bdebug57;
wire [BUS_WIDTH-1:0] bdebug56;
wire [BUS_WIDTH-1:0] bdebug55;
wire [BUS_WIDTH-1:0] bdebug54;
wire [BUS_WIDTH-1:0] bdebug53;
wire [BUS_WIDTH-1:0] bdebug52;
wire [BUS_WIDTH-1:0] bdebug51;
wire [BUS_WIDTH-1:0] bdebug50;
wire [BUS_WIDTH-1:0] bdebug49;
wire [BUS_WIDTH-1:0] bdebug48;
wire [BUS_WIDTH-1:0] bdebug47;
wire [BUS_WIDTH-1:0] bdebug46;
wire [BUS_WIDTH-1:0] bdebug45;
wire [BUS_WIDTH-1:0] bdebug44;
wire [BUS_WIDTH-1:0] bdebug43;
wire [BUS_WIDTH-1:0] bdebug42;
wire [BUS_WIDTH-1:0] bdebug41;
wire [BUS_WIDTH-1:0] bdebug40;
wire [BUS_WIDTH-1:0] bdebug39;
wire [BUS_WIDTH-1:0] bdebug38;
wire [BUS_WIDTH-1:0] bdebug37;
wire [BUS_WIDTH-1:0] bdebug36;
wire [BUS_WIDTH-1:0] bdebug35;
wire [BUS_WIDTH-1:0] bdebug34;
wire [BUS_WIDTH-1:0] bdebug33;
wire [BUS_WIDTH-1:0] bdebug32;
wire [BUS_WIDTH-1:0] bdebug31;
wire [BUS_WIDTH-1:0] bdebug30;
wire [BUS_WIDTH-1:0] bdebug29;
wire [BUS_WIDTH-1:0] bdebug28;
wire [BUS_WIDTH-1:0] bdebug27;
wire [BUS_WIDTH-1:0] bdebug26;
wire [BUS_WIDTH-1:0] bdebug25;
wire [BUS_WIDTH-1:0] bdebug24;
wire [BUS_WIDTH-1:0] bdebug23;
wire [BUS_WIDTH-1:0] bdebug22;
wire [BUS_WIDTH-1:0] bdebug21;
wire [BUS_WIDTH-1:0] bdebug20;
wire [BUS_WIDTH-1:0] bdebug19;
wire [BUS_WIDTH-1:0] bdebug18;
wire [BUS_WIDTH-1:0] bdebug17;
wire [BUS_WIDTH-1:0] bdebug16;
wire [BUS_WIDTH-1:0] bdebug15;
wire [BUS_WIDTH-1:0] bdebug14;
wire [BUS_WIDTH-1:0] bdebug13;
wire [BUS_WIDTH-1:0] bdebug12;
wire [BUS_WIDTH-1:0] bdebug11;
wire [BUS_WIDTH-1:0] bdebug10;
wire [BUS_WIDTH-1:0] bdebug9;
wire [BUS_WIDTH-1:0] bdebug8;
wire [BUS_WIDTH-1:0] bdebug7;
wire [BUS_WIDTH-1:0] bdebug6;
wire [BUS_WIDTH-1:0] bdebug5;
wire [BUS_WIDTH-1:0] bdebug4;
wire [BUS_WIDTH-1:0] bdebug3;
wire [BUS_WIDTH-1:0] bdebug2;
wire [BUS_WIDTH-1:0] bdebug1;
wire [BUS_WIDTH-1:0] bdebug0;
//----------------------------------------- End


wire [(LANE_COUNT-1):0]  laneEn;
generate
genvar i;
for(i=0; i < LANE_COUNT; i = i+1) begin : lane_i
   dbg_client_ssb__mux_laneEn I_dbg_client_ssb__mux_laneEn
                              ( .dbg_Mux_SelLane_LaneEnable    (dbg_Mux_SelLane[i*9+8])
                              , .TEST_CLK_EN                   (TEST_CLK_EN)
                              , .laneEn                        (laneEn[i])
                              );
   // Marker clock mux for dasychain clocking
   ati_clk_and d0nt_dbg_Mux_Clk (.Z(dbg_Mux_Clk[i]), .A2(laneEn[i]), .A1(clk));  // BOZO: May be redundant (see next section)

//////////////////////////////////////////////////////////////////////////|
//                        Debug-Bus PipeLine                            //|
//////////////////////////////////////////////////////////////////////////|
   // Clock gater (/import/rtllib/src/rtl/ati_clk_and.v)
   // NOTE: Lane Enable (dbg_Mux_SelLane[8]) is cdcbuffed, and is not of the same clock domain.
   //       Clock will be clipped, but metastble data will be filtered in the Steering Logic
   // BUG 614478: need to used ati_clock_gate so reg_MuxData is scanable
   //ati_clk_and I_gclk (.Z(gclk[i]), .A1(clk), .A2(laneEn[i]));
   ati_clock_gate I_gclk (.clk_out(gclk[i]), .clk_src(clk), .clk_en(laneEn[i]), .TE(TEST_CLK_EN));

   always @(posedge gclk[i]) reg_MuxData[i][LANE_WIDTH-1:0] <= laneEn[i] ? dbg_L2DataMuxLane[i][LANE_WIDTH-1:0] : {LANE_WIDTH {1'b0}};

   assign dbg_Mux_Data[((i+1)*LANE_WIDTH)-1:i*LANE_WIDTH] = reg_MuxData[i][LANE_WIDTH-1:0] ;

//////////////////////////////////////////////////////////////////////////|
//                   <Debug-Bus Mux - all combo logic>                  //|
//////////////////////////////////////////////////////////////////////////|
// This module is in common with synchronous version                    //|
//////////////////////////////////////////////////////////////////////////|
  dbg_client_64to1_lane_mux #( .LANE_COUNT(LANE_COUNT)
                             , .LANE_WIDTH(LANE_WIDTH)
                             , .LANE_INDEX(i)
                             )
          I_dbg_client_64to1_lane_mux
            ( .dbg_Mux_SelLane(dbg_Mux_SelLane[i*9+8:i*9])
            , .idebug63(bdebug63[BUS_WIDTH-1:0])
            , .idebug62(bdebug62[BUS_WIDTH-1:0])
            , .idebug61(bdebug61[BUS_WIDTH-1:0])
            , .idebug60(bdebug60[BUS_WIDTH-1:0])
            , .idebug59(bdebug59[BUS_WIDTH-1:0])
            , .idebug58(bdebug58[BUS_WIDTH-1:0])
            , .idebug57(bdebug57[BUS_WIDTH-1:0])
            , .idebug56(bdebug56[BUS_WIDTH-1:0])
            , .idebug55(bdebug55[BUS_WIDTH-1:0])
            , .idebug54(bdebug54[BUS_WIDTH-1:0])
            , .idebug53(bdebug53[BUS_WIDTH-1:0])
            , .idebug52(bdebug52[BUS_WIDTH-1:0])
            , .idebug51(bdebug51[BUS_WIDTH-1:0])
            , .idebug50(bdebug50[BUS_WIDTH-1:0])
            , .idebug49(bdebug49[BUS_WIDTH-1:0])
            , .idebug48(bdebug48[BUS_WIDTH-1:0])
            , .idebug47(bdebug47[BUS_WIDTH-1:0])
            , .idebug46(bdebug46[BUS_WIDTH-1:0])
            , .idebug45(bdebug45[BUS_WIDTH-1:0])
            , .idebug44(bdebug44[BUS_WIDTH-1:0])
            , .idebug43(bdebug43[BUS_WIDTH-1:0])
            , .idebug42(bdebug42[BUS_WIDTH-1:0])
            , .idebug41(bdebug41[BUS_WIDTH-1:0])
            , .idebug40(bdebug40[BUS_WIDTH-1:0])
            , .idebug39(bdebug39[BUS_WIDTH-1:0])
            , .idebug38(bdebug38[BUS_WIDTH-1:0])
            , .idebug37(bdebug37[BUS_WIDTH-1:0])
            , .idebug36(bdebug36[BUS_WIDTH-1:0])
            , .idebug35(bdebug35[BUS_WIDTH-1:0])
            , .idebug34(bdebug34[BUS_WIDTH-1:0])
            , .idebug33(bdebug33[BUS_WIDTH-1:0])
            , .idebug32(bdebug32[BUS_WIDTH-1:0])
            , .idebug31(bdebug31[BUS_WIDTH-1:0])
            , .idebug30(bdebug30[BUS_WIDTH-1:0])
            , .idebug29(bdebug29[BUS_WIDTH-1:0])
            , .idebug28(bdebug28[BUS_WIDTH-1:0])
            , .idebug27(bdebug27[BUS_WIDTH-1:0])
            , .idebug26(bdebug26[BUS_WIDTH-1:0])
            , .idebug25(bdebug25[BUS_WIDTH-1:0])
            , .idebug24(bdebug24[BUS_WIDTH-1:0])
            , .idebug23(bdebug23[BUS_WIDTH-1:0])
            , .idebug22(bdebug22[BUS_WIDTH-1:0])
            , .idebug21(bdebug21[BUS_WIDTH-1:0])
            , .idebug20(bdebug20[BUS_WIDTH-1:0])
            , .idebug19(bdebug19[BUS_WIDTH-1:0])
            , .idebug18(bdebug18[BUS_WIDTH-1:0])
            , .idebug17(bdebug17[BUS_WIDTH-1:0])
            , .idebug16(bdebug16[BUS_WIDTH-1:0])
            , .idebug15(bdebug15[BUS_WIDTH-1:0])
            , .idebug14(bdebug14[BUS_WIDTH-1:0])
            , .idebug13(bdebug13[BUS_WIDTH-1:0])
            , .idebug12(bdebug12[BUS_WIDTH-1:0])
            , .idebug11(bdebug11[BUS_WIDTH-1:0])
            , .idebug10(bdebug10[BUS_WIDTH-1:0])
            , .idebug9(bdebug9[BUS_WIDTH-1:0])
            , .idebug8(bdebug8[BUS_WIDTH-1:0])
            , .idebug7(bdebug7[BUS_WIDTH-1:0])
            , .idebug6(bdebug6[BUS_WIDTH-1:0])
            , .idebug5(bdebug5[BUS_WIDTH-1:0])
            , .idebug4(bdebug4[BUS_WIDTH-1:0])
            , .idebug3(bdebug3[BUS_WIDTH-1:0])
            , .idebug2(bdebug2[BUS_WIDTH-1:0])
            , .idebug1(bdebug1[BUS_WIDTH-1:0])
            , .idebug0(bdebug0[BUS_WIDTH-1:0])
            , .dbg_L2DataMuxLane(dbg_L2DataMuxLane[i][LANE_WIDTH-1:0])
            );
end
endgenerate

//////////////////////////////////////////////////////////////////////////|
//                                                                      //|
//////////////////////////////////////////////////////////////////////////|
   //! unused MuxClkEn
      assign bdebug62[BUS_WIDTH-1:0] = idebug62[BUS_WIDTH-1:0];
      assign bdebug61[BUS_WIDTH-1:0] = idebug61[BUS_WIDTH-1:0];
      assign bdebug60[BUS_WIDTH-1:0] = idebug60[BUS_WIDTH-1:0];
      assign bdebug59[BUS_WIDTH-1:0] = idebug59[BUS_WIDTH-1:0];
      assign bdebug58[BUS_WIDTH-1:0] = idebug58[BUS_WIDTH-1:0];
      assign bdebug57[BUS_WIDTH-1:0] = idebug57[BUS_WIDTH-1:0];
      assign bdebug56[BUS_WIDTH-1:0] = idebug56[BUS_WIDTH-1:0];
      assign bdebug55[BUS_WIDTH-1:0] = idebug55[BUS_WIDTH-1:0];
      assign bdebug54[BUS_WIDTH-1:0] = idebug54[BUS_WIDTH-1:0];
      assign bdebug53[BUS_WIDTH-1:0] = idebug53[BUS_WIDTH-1:0];
      assign bdebug52[BUS_WIDTH-1:0] = idebug52[BUS_WIDTH-1:0];
      assign bdebug51[BUS_WIDTH-1:0] = idebug51[BUS_WIDTH-1:0];
      assign bdebug50[BUS_WIDTH-1:0] = idebug50[BUS_WIDTH-1:0];
      assign bdebug49[BUS_WIDTH-1:0] = idebug49[BUS_WIDTH-1:0];
      assign bdebug48[BUS_WIDTH-1:0] = idebug48[BUS_WIDTH-1:0];
      assign bdebug47[BUS_WIDTH-1:0] = idebug47[BUS_WIDTH-1:0];
      assign bdebug46[BUS_WIDTH-1:0] = idebug46[BUS_WIDTH-1:0];
      assign bdebug45[BUS_WIDTH-1:0] = idebug45[BUS_WIDTH-1:0];
      assign bdebug44[BUS_WIDTH-1:0] = idebug44[BUS_WIDTH-1:0];
      assign bdebug43[BUS_WIDTH-1:0] = idebug43[BUS_WIDTH-1:0];
      assign bdebug42[BUS_WIDTH-1:0] = idebug42[BUS_WIDTH-1:0];
      assign bdebug41[BUS_WIDTH-1:0] = idebug41[BUS_WIDTH-1:0];
      assign bdebug40[BUS_WIDTH-1:0] = idebug40[BUS_WIDTH-1:0];
      assign bdebug39[BUS_WIDTH-1:0] = idebug39[BUS_WIDTH-1:0];
      assign bdebug38[BUS_WIDTH-1:0] = idebug38[BUS_WIDTH-1:0];
      assign bdebug37[BUS_WIDTH-1:0] = idebug37[BUS_WIDTH-1:0];
      assign bdebug36[BUS_WIDTH-1:0] = idebug36[BUS_WIDTH-1:0];
      assign bdebug35[BUS_WIDTH-1:0] = idebug35[BUS_WIDTH-1:0];
      assign bdebug34[BUS_WIDTH-1:0] = idebug34[BUS_WIDTH-1:0];
      assign bdebug33[BUS_WIDTH-1:0] = idebug33[BUS_WIDTH-1:0];
      assign bdebug32[BUS_WIDTH-1:0] = idebug32[BUS_WIDTH-1:0];
      assign bdebug31[BUS_WIDTH-1:0] = idebug31[BUS_WIDTH-1:0];
      assign bdebug30[BUS_WIDTH-1:0] = idebug30[BUS_WIDTH-1:0];
      assign bdebug29[BUS_WIDTH-1:0] = idebug29[BUS_WIDTH-1:0];
      assign bdebug28[BUS_WIDTH-1:0] = idebug28[BUS_WIDTH-1:0];
      assign bdebug27[BUS_WIDTH-1:0] = idebug27[BUS_WIDTH-1:0];
      assign bdebug26[BUS_WIDTH-1:0] = idebug26[BUS_WIDTH-1:0];
      assign bdebug25[BUS_WIDTH-1:0] = idebug25[BUS_WIDTH-1:0];
      assign bdebug24[BUS_WIDTH-1:0] = idebug24[BUS_WIDTH-1:0];
      assign bdebug23[BUS_WIDTH-1:0] = idebug23[BUS_WIDTH-1:0];
      assign bdebug22[BUS_WIDTH-1:0] = idebug22[BUS_WIDTH-1:0];
      assign bdebug21[BUS_WIDTH-1:0] = idebug21[BUS_WIDTH-1:0];
      assign bdebug20[BUS_WIDTH-1:0] = idebug20[BUS_WIDTH-1:0];
      assign bdebug19[BUS_WIDTH-1:0] = idebug19[BUS_WIDTH-1:0];
      assign bdebug18[BUS_WIDTH-1:0] = idebug18[BUS_WIDTH-1:0];
      assign bdebug17[BUS_WIDTH-1:0] = idebug17[BUS_WIDTH-1:0];
      assign bdebug16[BUS_WIDTH-1:0] = idebug16[BUS_WIDTH-1:0];
      assign bdebug15[BUS_WIDTH-1:0] = idebug15[BUS_WIDTH-1:0];
      assign bdebug14[BUS_WIDTH-1:0] = idebug14[BUS_WIDTH-1:0];
      assign bdebug13[BUS_WIDTH-1:0] = idebug13[BUS_WIDTH-1:0];
      assign bdebug12[BUS_WIDTH-1:0] = idebug12[BUS_WIDTH-1:0];
      assign bdebug11[BUS_WIDTH-1:0] = idebug11[BUS_WIDTH-1:0];
      assign bdebug10[BUS_WIDTH-1:0] = idebug10[BUS_WIDTH-1:0];
      assign bdebug9[BUS_WIDTH-1:0] = idebug9[BUS_WIDTH-1:0];
      assign bdebug8[BUS_WIDTH-1:0] = idebug8[BUS_WIDTH-1:0];
      assign bdebug7[BUS_WIDTH-1:0] = idebug7[BUS_WIDTH-1:0];
      assign bdebug6[BUS_WIDTH-1:0] = idebug6[BUS_WIDTH-1:0];
      assign bdebug5[BUS_WIDTH-1:0] = idebug5[BUS_WIDTH-1:0];
      assign bdebug4[BUS_WIDTH-1:0] = idebug4[BUS_WIDTH-1:0];
      assign bdebug3[BUS_WIDTH-1:0] = idebug3[BUS_WIDTH-1:0];
      assign bdebug2[BUS_WIDTH-1:0] = idebug2[BUS_WIDTH-1:0];
      assign bdebug1[BUS_WIDTH-1:0] = idebug1[BUS_WIDTH-1:0];
      assign bdebug0[BUS_WIDTH-1:0] = idebug0[BUS_WIDTH-1:0];

//////////////////////////////////////////////////////////////////////////|
//                      Debug Debug bus                                 //|
//////////////////////////////////////////////////////////////////////////|
  dbg_client_ssb__debug #(.LANE_COUNT(LANE_COUNT), .LANE_WIDTH(LANE_WIDTH))
    I_dbg_client_ssb__debug ( .clk            (clk                                )
                            , .dbg_BlkId      (dbg_BlkId[7:0]                     )
                            , .dbg_Mux_SelLane(dbg_Mux_SelLane[(LANE_COUNT*9-1):0])
                            , .idebug         (debugOut[BUS_WIDTH-1:0]       )
                            , .ctlProg_ClkOn  (ctlProg_ClkOn                      )
                            );
  assign bdebug63[BUS_WIDTH-1:0] = debugOut[BUS_WIDTH-1:0];

endmodule // dbg_client_ssb__mux
