//                              -*- Mode: Verilog -*-
// (c) 2015, 2016 Advanced Micro Devices, Inc. All rights reserved.

////////////////////////////////////////////////////////////////////////////
//
// Filename    : dbg_client_ssb__mux_8x1.v.dpl
// Description : Mux for synchronous debug bus client.
//
// NOTE : Multi-clock case should use Source-synchronous debug bus client
//
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
// Print out some input parameters to dperl --- begin
// MKEY          = 
// synchronicity = 
// DBG_WIDTH     = 
// DBG_IN_CNT    = 
// Print out some input parameters to dperl ----- end
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////

module dbg_client_ssb__mux_8x1
#(parameter LANE_COUNT     = 4
,           LANE_WIDTH     = 8
)

// ==========================================================================
// interface with dbg_client_ssb__steer
// Begin --------------------------------------------------------------------
( input       [7:0]                       dbg_BlkId          // Debug bus ID used for debug the debug bus client (same as DbgBlkId)
, input       [LANE_COUNT*9         -1:0] dbg_Mux_SelLane     // LANE_COUNT * ([8] Lane Enable; [7:6] Byte Select; [5:0] Channel Select;)
, output      [LANE_COUNT*LANE_WIDTH-1:0] dbg_Mux_Data        // Debug-bus data outputs after muxing
, output      [LANE_COUNT           -1:0] dbg_Mux_Clk         // Clock output for each lane
// End ----------------------------------------------------------------------

// ==========================================================================
// Debug-bus data inputs from functional logic
// Begin --------------------------------------------------------------------
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug7 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug6 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug5 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug4 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug3 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug2 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug1 
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug0 
// End ----------------------------------------------------------------------

, input                            clk            // Clocking of input data
, input                            TEST_CLK_EN    // Gater override for test mode
, input                            ctlProg_ClkOn  // Used to as asyn resetb (async)

);

localparam BUS_WIDTH = LANE_COUNT*LANE_WIDTH;

reg  [LANE_WIDTH-1:0]  reg_MuxData[LANE_COUNT-1:0];
wire [LANE_WIDTH-1:0]  dbg_L2DataMuxLane[LANE_COUNT-1:0];
wire [LANE_COUNT-1:0]  gclk;
wire [LANE_COUNT-1:0]  laneEn;
wire [BUS_WIDTH -1:0]  debugOut;
wire [BUS_WIDTH -1:0]  idebug63; // For Debug

generate
genvar i;
for(i=0; i < LANE_COUNT; i = i+1) begin : lane_i
   dbg_client_ssb__mux_laneEn I_dbg_client_ssb__mux_laneEn
                              ( .dbg_Mux_SelLane_LaneEnable    (dbg_Mux_SelLane[i*9+8])
                              , .TEST_CLK_EN                   (TEST_CLK_EN)
                              , .laneEn                        (laneEn[i])
                              );

   // Marker clock mux for dasychain clocking
   ati_clk_and d0nt_dbg_Mux_Clk (.Z(dbg_Mux_Clk[i]), .A2(laneEn[i]), .A1(clk));  // BOZO: May be redundant (see next section)

//////////////////////////////////////////////////////////////////////////|
//                        Debug-Bus PipeLine                            //|
//////////////////////////////////////////////////////////////////////////|
   // Clock gater (/import/rtllib/src/rtl/ati_clk_and.v)
   // NOTE: Lane Enable (dbg_Mux_SelLane[8]) is cdcbuffed, and is not of the same clock domain.
   //       Clock will be clipped, but metastble data will be filtered in the Steering Logic
   // BUG 614478: need to used ati_clock_gate so reg_MuxData is scanable
   //ati_clk_and I_gclk (.Z(gclk[i]), .A1(clk), .A2(laneEn[i]));
   ati_clock_gate I_gclk (.clk_out(gclk[i]), .clk_src(clk), .clk_en(laneEn[i]), .TE(TEST_CLK_EN));

   always @(posedge gclk[i])
      reg_MuxData[i][LANE_WIDTH-1:0] <= laneEn[i] ? dbg_L2DataMuxLane[i][LANE_WIDTH-1:0] : {LANE_WIDTH {1'b0}};

   assign dbg_Mux_Data[((i+1)*LANE_WIDTH)-1:i*LANE_WIDTH] = reg_MuxData[i][LANE_WIDTH-1:0] ;

//////////////////////////////////////////////////////////////////////////|
//                   <Debug-Bus Mux - all combo logic>                  //|
//////////////////////////////////////////////////////////////////////////|
// This module is in common with synchronous version                    //|
//////////////////////////////////////////////////////////////////////////|
  dbg_client_9to1_lane_mux #( .LANE_COUNT(LANE_COUNT)
                             , .LANE_WIDTH(LANE_WIDTH)
                             , .LANE_INDEX(i)
                             )
    I_dbg_client_9to1_lane_mux ( .dbg_Mux_SelLane(dbg_Mux_SelLane[i*9+8:i*9])
                               , .idebug63(idebug63[BUS_WIDTH-1:0])
                               , .idebug7(idebug7[BUS_WIDTH-1:0])
                               , .idebug6(idebug6[BUS_WIDTH-1:0])
                               , .idebug5(idebug5[BUS_WIDTH-1:0])
                               , .idebug4(idebug4[BUS_WIDTH-1:0])
                               , .idebug3(idebug3[BUS_WIDTH-1:0])
                               , .idebug2(idebug2[BUS_WIDTH-1:0])
                               , .idebug1(idebug1[BUS_WIDTH-1:0])
                               , .idebug0(idebug0[BUS_WIDTH-1:0])
                               , .dbg_L2DataMuxLane(dbg_L2DataMuxLane[i][LANE_WIDTH-1:0])
                               );
end
endgenerate

//////////////////////////////////////////////////////////////////////////|
//                      Debug Debug bus                                 //|
//////////////////////////////////////////////////////////////////////////|
  dbg_client_ssb__debug #(.LANE_COUNT(LANE_COUNT), .LANE_WIDTH(LANE_WIDTH))
    I_dbg_client_ssb__debug ( .clk            (clk                                )
                            , .dbg_BlkId      (dbg_BlkId[7:0]                     )
                            , .dbg_Mux_SelLane(dbg_Mux_SelLane[(LANE_COUNT*9-1):0])
                            , .idebug         (debugOut[BUS_WIDTH-1:0]       )
                            , .ctlProg_ClkOn  (ctlProg_ClkOn                      )
                            );
  assign idebug63[BUS_WIDTH-1:0] = debugOut[BUS_WIDTH-1:0];

endmodule // dbg_client_ssb__mux_8x1
