//                              -*- Mode: Verilog -*-
// (c) 2015, 2016 Advanced Micro Devices, Inc. All rights reserved.

////////////////////////////////////////////////////////////////////////////
//
// Perforce File         : : //depot/dfd_dfv/dbus/trunk/src/rtl/dbg_client_ssb__mux_async.v.dpl n// Description : Mux for clock only, used for async signals to overload the source-sync clock
//               This is an expanded version of dbg_client_ssb__mux_clk
//               This module will take in 63 channels, each channel width is set by parameter LANE_COUNT
//               Only configuraion of LANE_COUNT == 3 and LANE_COUNT == 4 is tested
//
// NOTE : Multi-clock case should use Source-synchronous debug bus client
//
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
// Print out some input parameters to dperl --- begin
// MKEY          = 
// synchronicity = 
// DBG_WIDTH     = 
// DBG_IN_CNT    = 
// Print out some input parameters to dperl ----- end
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////

module dbg_client_ssb__mux_async
#(parameter LANE_COUNT = 4
,           LANE_WIDTH = 8
)

// ==========================================================================
// interface with dbg_client_ssb__steer
// Begin --------------------------------------------------------------------
( input       [(LANE_COUNT*9-1)         :0] dbg_Mux_SelLane     // LANE_COUNT * ([8] Lane Enable; [7:6] Byte Select; [5:0] Channel Select;)
, output wire [(LANE_COUNT*LANE_WIDTH-1):0] dbg_Mux_Data        // Debug-bus data outputs after muxing
, output wire [(LANE_COUNT-1)           :0] dbg_Mux_Clk         // Clock output for each lane
// End ----------------------------------------------------------------------

// ==========================================================================
// Debug-bus data inputs from functional logic
// Begin --------------------------------------------------------------------
, input       [LANE_COUNT-1:0] idebug62 
, input       [LANE_COUNT-1:0] idebug61 
, input       [LANE_COUNT-1:0] idebug60 
, input       [LANE_COUNT-1:0] idebug59 
, input       [LANE_COUNT-1:0] idebug58 
, input       [LANE_COUNT-1:0] idebug57 
, input       [LANE_COUNT-1:0] idebug56 
, input       [LANE_COUNT-1:0] idebug55 
, input       [LANE_COUNT-1:0] idebug54 
, input       [LANE_COUNT-1:0] idebug53 
, input       [LANE_COUNT-1:0] idebug52 
, input       [LANE_COUNT-1:0] idebug51 
, input       [LANE_COUNT-1:0] idebug50 
, input       [LANE_COUNT-1:0] idebug49 
, input       [LANE_COUNT-1:0] idebug48 
, input       [LANE_COUNT-1:0] idebug47 
, input       [LANE_COUNT-1:0] idebug46 
, input       [LANE_COUNT-1:0] idebug45 
, input       [LANE_COUNT-1:0] idebug44 
, input       [LANE_COUNT-1:0] idebug43 
, input       [LANE_COUNT-1:0] idebug42 
, input       [LANE_COUNT-1:0] idebug41 
, input       [LANE_COUNT-1:0] idebug40 
, input       [LANE_COUNT-1:0] idebug39 
, input       [LANE_COUNT-1:0] idebug38 
, input       [LANE_COUNT-1:0] idebug37 
, input       [LANE_COUNT-1:0] idebug36 
, input       [LANE_COUNT-1:0] idebug35 
, input       [LANE_COUNT-1:0] idebug34 
, input       [LANE_COUNT-1:0] idebug33 
, input       [LANE_COUNT-1:0] idebug32 
, input       [LANE_COUNT-1:0] idebug31 
, input       [LANE_COUNT-1:0] idebug30 
, input       [LANE_COUNT-1:0] idebug29 
, input       [LANE_COUNT-1:0] idebug28 
, input       [LANE_COUNT-1:0] idebug27 
, input       [LANE_COUNT-1:0] idebug26 
, input       [LANE_COUNT-1:0] idebug25 
, input       [LANE_COUNT-1:0] idebug24 
, input       [LANE_COUNT-1:0] idebug23 
, input       [LANE_COUNT-1:0] idebug22 
, input       [LANE_COUNT-1:0] idebug21 
, input       [LANE_COUNT-1:0] idebug20 
, input       [LANE_COUNT-1:0] idebug19 
, input       [LANE_COUNT-1:0] idebug18 
, input       [LANE_COUNT-1:0] idebug17 
, input       [LANE_COUNT-1:0] idebug16 
, input       [LANE_COUNT-1:0] idebug15 
, input       [LANE_COUNT-1:0] idebug14 
, input       [LANE_COUNT-1:0] idebug13 
, input       [LANE_COUNT-1:0] idebug12 
, input       [LANE_COUNT-1:0] idebug11 
, input       [LANE_COUNT-1:0] idebug10 
, input       [LANE_COUNT-1:0] idebug9 
, input       [LANE_COUNT-1:0] idebug8 
, input       [LANE_COUNT-1:0] idebug7 
, input       [LANE_COUNT-1:0] idebug6 
, input       [LANE_COUNT-1:0] idebug5 
, input       [LANE_COUNT-1:0] idebug4 
, input       [LANE_COUNT-1:0] idebug3 
, input       [LANE_COUNT-1:0] idebug2 
, input       [LANE_COUNT-1:0] idebug1 
, input       [LANE_COUNT-1:0] idebug0 
// End ----------------------------------------------------------------------

, input                                     TEST_CLK_EN         // Gater override for test mode
, input                                     ctlProg_ClkOn       // Used to as asyn resetb (async)
);

wire [LANE_COUNT-1:0]     gclk;

//--------------------------------------- Begin
wire [LANE_COUNT-1:0] bdebug63;
//----------------------------------------- End

//////////////////////////////////////////////////////////////////////////|
//         Tie data out to 0's (No more DaisyChaining)                  //|
//////////////////////////////////////////////////////////////////////////|
  assign dbg_Mux_Data[LANE_COUNT*LANE_WIDTH-1:0] = {(LANE_COUNT*LANE_WIDTH){1'b0}};

wire [(LANE_COUNT-1):0]  laneEn;
generate
genvar i;
for(i=0; i < LANE_COUNT; i = i+1) begin : lane_i
   dbg_client_ssb__mux_laneEn
     I_dbg_client_ssb__mux_laneEn ( .dbg_Mux_SelLane_LaneEnable (dbg_Mux_SelLane[i*9+8])
                                  , .TEST_CLK_EN                (TEST_CLK_EN)
                                  , .laneEn                     (laneEn[i])
                                  );
   // (WAS) Marker clock mux for dasychain clocking
   ati_clk_and d0nt_dbg_Mux_Clk (.Z(dbg_Mux_Clk[i]), .A2(laneEn[i]), .A1(gclk[i]));

  //////////////////////////////////////////////////////////////////////////|
  //                   <Debug-Bus Mux - all combo logic>                  //|
  //////////////////////////////////////////////////////////////////////////|
  // This module is in common with synchronous version                    //|
  //////////////////////////////////////////////////////////////////////////|
  dbg_client_64to1_lane_mux #( .LANE_COUNT(LANE_COUNT)
                             , .LANE_WIDTH(1)
                             , .LANE_INDEX(i))
    I_dbg_client_64to1_lane_mux ( .dbg_Mux_SelLane(dbg_Mux_SelLane[i*9+8:i*9])
                                , .idebug63(bdebug63[LANE_COUNT-1:0])
                                , .idebug62(idebug62[LANE_COUNT-1:0])
                                , .idebug61(idebug61[LANE_COUNT-1:0])
                                , .idebug60(idebug60[LANE_COUNT-1:0])
                                , .idebug59(idebug59[LANE_COUNT-1:0])
                                , .idebug58(idebug58[LANE_COUNT-1:0])
                                , .idebug57(idebug57[LANE_COUNT-1:0])
                                , .idebug56(idebug56[LANE_COUNT-1:0])
                                , .idebug55(idebug55[LANE_COUNT-1:0])
                                , .idebug54(idebug54[LANE_COUNT-1:0])
                                , .idebug53(idebug53[LANE_COUNT-1:0])
                                , .idebug52(idebug52[LANE_COUNT-1:0])
                                , .idebug51(idebug51[LANE_COUNT-1:0])
                                , .idebug50(idebug50[LANE_COUNT-1:0])
                                , .idebug49(idebug49[LANE_COUNT-1:0])
                                , .idebug48(idebug48[LANE_COUNT-1:0])
                                , .idebug47(idebug47[LANE_COUNT-1:0])
                                , .idebug46(idebug46[LANE_COUNT-1:0])
                                , .idebug45(idebug45[LANE_COUNT-1:0])
                                , .idebug44(idebug44[LANE_COUNT-1:0])
                                , .idebug43(idebug43[LANE_COUNT-1:0])
                                , .idebug42(idebug42[LANE_COUNT-1:0])
                                , .idebug41(idebug41[LANE_COUNT-1:0])
                                , .idebug40(idebug40[LANE_COUNT-1:0])
                                , .idebug39(idebug39[LANE_COUNT-1:0])
                                , .idebug38(idebug38[LANE_COUNT-1:0])
                                , .idebug37(idebug37[LANE_COUNT-1:0])
                                , .idebug36(idebug36[LANE_COUNT-1:0])
                                , .idebug35(idebug35[LANE_COUNT-1:0])
                                , .idebug34(idebug34[LANE_COUNT-1:0])
                                , .idebug33(idebug33[LANE_COUNT-1:0])
                                , .idebug32(idebug32[LANE_COUNT-1:0])
                                , .idebug31(idebug31[LANE_COUNT-1:0])
                                , .idebug30(idebug30[LANE_COUNT-1:0])
                                , .idebug29(idebug29[LANE_COUNT-1:0])
                                , .idebug28(idebug28[LANE_COUNT-1:0])
                                , .idebug27(idebug27[LANE_COUNT-1:0])
                                , .idebug26(idebug26[LANE_COUNT-1:0])
                                , .idebug25(idebug25[LANE_COUNT-1:0])
                                , .idebug24(idebug24[LANE_COUNT-1:0])
                                , .idebug23(idebug23[LANE_COUNT-1:0])
                                , .idebug22(idebug22[LANE_COUNT-1:0])
                                , .idebug21(idebug21[LANE_COUNT-1:0])
                                , .idebug20(idebug20[LANE_COUNT-1:0])
                                , .idebug19(idebug19[LANE_COUNT-1:0])
                                , .idebug18(idebug18[LANE_COUNT-1:0])
                                , .idebug17(idebug17[LANE_COUNT-1:0])
                                , .idebug16(idebug16[LANE_COUNT-1:0])
                                , .idebug15(idebug15[LANE_COUNT-1:0])
                                , .idebug14(idebug14[LANE_COUNT-1:0])
                                , .idebug13(idebug13[LANE_COUNT-1:0])
                                , .idebug12(idebug12[LANE_COUNT-1:0])
                                , .idebug11(idebug11[LANE_COUNT-1:0])
                                , .idebug10(idebug10[LANE_COUNT-1:0])
                                , .idebug9(idebug9[LANE_COUNT-1:0])
                                , .idebug8(idebug8[LANE_COUNT-1:0])
                                , .idebug7(idebug7[LANE_COUNT-1:0])
                                , .idebug6(idebug6[LANE_COUNT-1:0])
                                , .idebug5(idebug5[LANE_COUNT-1:0])
                                , .idebug4(idebug4[LANE_COUNT-1:0])
                                , .idebug3(idebug3[LANE_COUNT-1:0])
                                , .idebug2(idebug2[LANE_COUNT-1:0])
                                , .idebug1(idebug1[LANE_COUNT-1:0])
                                , .idebug0(idebug0[LANE_COUNT-1:0])
                                , .dbg_L2DataMuxLane(gclk[i])
                                );
end // block: lane_i
endgenerate

//////////////////////////////////////////////////////////////////////////|
//                      Debug Debug bus                                 //|
//////////////////////////////////////////////////////////////////////////|
   assign bdebug63[LANE_COUNT-1:0] = {LANE_COUNT {1'b0}};

endmodule // dbg_client_ssb__mux_async
