//                              -*- Mode: Verilog -*-
// (c) 2015, 2016 Advanced Micro Devices, Inc. All rights reserved.

////////////////////////////////////////////////////////////////////////////
//
// Perforce File         : $File: //depot/dfd_dfv/dbus/trunk/src/rtl/dbg_client_ssb__mux_clk.v $
// Description : Mux for clock only, used for async signals to overload the source-sync clock
//
// NOTE : Multi-clock case should use Source-synchronous debug bus client
//
////////////////////////////////////////////////////////////////////////////
module dbg_client_ssb__mux_clk
#(parameter LANE_COUNT = 4
,           LANE_WIDTH = 8
)

// ==========================================================================
// interface with dbg_client_ssb__steer
// Begin --------------------------------------------------------------------
( input       [(LANE_COUNT*9-1)         :0] dbg_Mux_SelLane   // LANE_COUNT * ([8] Lane Enable; [7:6] Byte Select; [5:0] Channel Select;)
, output wire [(LANE_COUNT*LANE_WIDTH-1):0] dbg_Mux_Data      // Debug-bus data tied to 0's
, output wire [(LANE_COUNT-1)           :0] dbg_Mux_Clk       // Clock output for each lane
// End ----------------------------------------------------------------------

, input                                     clk               // Clocking of input data
, input                                     TEST_CLK_EN       // Gater override for test mode
, input                                     ctlProg_ClkOn     // Used to as asyn resetb (async)
);

localparam MUXSELLENGTH   = 9;
localparam MUXSELEN_IDX   = 8;

//////////////////////////////////////////////////////////////////////////|
//                        Match Channel Select with INPUT_USED          //|
//////////////////////////////////////////////////////////////////////////|
wire [(LANE_COUNT-1):0]  laneEn;

// Pass through data from next dbg_client_ssb__mux
assign  dbg_Mux_Data[(LANE_COUNT*LANE_WIDTH-1):0] = {(LANE_COUNT*LANE_WIDTH){1'b0}};

generate
genvar i;
for(i=0; i < LANE_COUNT; i = i+1) begin : lane_i
   dbg_client_ssb__mux_laneEn
     I_dbg_client_ssb__mux_laneEn ( .dbg_Mux_SelLane_LaneEnable    (dbg_Mux_SelLane[i*9+8    ])
                                  , .TEST_CLK_EN                   (TEST_CLK_EN               )
                                  , .laneEn                        (laneEn[i]                 )
                                  );
   // Marker clock mux for dasychain clocking
   ati_clk_and d0nt_dbg_Mux_Clk (.Z(dbg_Mux_Clk[i]), .A2(laneEn[i]), .A1(clk));

end // for(i=0; i < LANE_COUNT; i = i+1)
endgenerate

endmodule // dbg_client_ssb__mux
