//                              -*- Mode: Verilog -*-
// (c) 2015, 2016 Advanced Micro Devices, Inc. All rights reserved.

////////////////////////////////////////////////////////////////////////////
//
// Filename    : dbg_client_sync_gclk.v.dpl
// Description : Mux for synchronous debug bus client.
//
// NOTE : Multi-clock case should use Source-synchronous debug bus client
//
////////////////////////////////////////////////////////////////////////////
module dbg_client_sync_aon_gclk (

input                     ipclk                    , // system clock
output wire               dbg_active               , // Clock enable for data path

output wire [1:0]         daisychain_ctl_d1        , // Daisy-chain program output, reverse of data flow
input       [1:0]         TARG_DBUS_daisychain_ctl   // Daisy-chain program input,  reverse of data flow
);

  reg [1:0]               rdaisychainctl_d1;
  reg                     gate_clk;
  assign                  dbg_active             = gate_clk;
  assign                  daisychain_ctl_d1[1:0] = rdaisychainctl_d1[1:0];

  always @(posedge ipclk) begin
     rdaisychainctl_d1[1:0] <= TARG_DBUS_daisychain_ctl[1:0];

     case ({rdaisychainctl_d1[0], TARG_DBUS_daisychain_ctl[0]})
       2'b01   : gate_clk <= TARG_DBUS_daisychain_ctl[1]; // Rising  edge, sample input
       2'b10   : gate_clk <= rdaisychainctl_d1[1]       ; // Falling edge, sample d1 data
       default : gate_clk <= gate_clk                   ; // This is don't care, but assign a value
     endcase // case({TARG_DBUS_daisychain_ctl[0], rdaisychainctl_d1[0]})
  end

endmodule // dbg_client_sync_aon_gclk
