//                              -*- Mode: Verilog -*-
// (c) 2015, 2016 Advanced Micro Devices, Inc. All rights reserved.

////////////////////////////////////////////////////////////////////////////
//
// Filename    : dbg_client_sync_aon_muxf.v.dpl
// Description : Mux for synchronous debug bus client.
//
// NOTE : Multi-clock case should use Source-synchronous debug bus client
//
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
// Print out some input parameters to dperl --- begin
// MKEY          = 
// DBG_WIDTH     = 
// MUX_COUNT     = 
// Print out some input parameters to dperl ----- end
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////

module dbg_client_sync_aon_muxf #(parameter
LANE_COUNT = 4,
LANE_WIDTH = 8
) (

// ==========================================================================
// Debug bus Daisy-chain inputs and outputs
// Data flows towards DSM. ctl flows away from DSM
// Begin --------------------------------------------------------------------
input       [(LANE_COUNT*LANE_WIDTH-1):0] DBUS_TARG_daisychain_data       , // Daisy-chain input
output wire [(LANE_COUNT*LANE_WIDTH-1):0] TARG_DBUS_daisychain_data       , // Daisy-chain output
// End ----------------------------------------------------------------------

// ==========================================================================
// interface with first dbg_client_sync_mux instance
// Begin --------------------------------------------------------------------
input       [(LANE_COUNT*6-1):0] MuxSelLaneEn                   , // All Lane Enable;
input       [(LANE_COUNT*LANE_WIDTH-1):0] dbg_DataMuxed5                 , // Debug-bus data from mux(s) output
input       [(LANE_COUNT*LANE_WIDTH-1):0] dbg_DataMuxed4                 , // Debug-bus data from mux(s) output
input       [(LANE_COUNT*LANE_WIDTH-1):0] dbg_DataMuxed3                 , // Debug-bus data from mux(s) output
input       [(LANE_COUNT*LANE_WIDTH-1):0] dbg_DataMuxed2                 , // Debug-bus data from mux(s) output
input       [(LANE_COUNT*LANE_WIDTH-1):0] dbg_DataMuxed1                 , // Debug-bus data from mux(s) output
input       [(LANE_COUNT*LANE_WIDTH-1):0] dbg_DataMuxed0                 , // Debug-bus data from mux(s) output
// End ----------------------------------------------------------------------

// ==========================================================================
// Misc
// Begin --------------------------------------------------------------------
input                          ipclk                           , // system clock
input                          dbg_rst                         , // Reset and clear data pipe
input                          TEST_CLK_EN                     , // Gater override for test mode
input                          dbg_active                        // Clock gater term for data pipe
// End ----------------------------------------------------------------------

);
  // ==========================================================================
  // Data Pipe
  // Begin --------------------------------------------------------------------
  wire gclk; // gated clock
  wire dbg_clkOn;
  assign dbg_clkOn = dbg_active | dbg_rst;
  // Clock gater (/import/rtllib/src/rtl/ati_clock_gate.v)
  ati_clock_gate I_gclk (.clk_out(gclk), .clk_src(ipclk), .clk_en(dbg_clkOn), .TE(TEST_CLK_EN));

  reg [(LANE_COUNT*LANE_WIDTH-1):0] daisychain_data_d1;
  reg [(LANE_COUNT*LANE_WIDTH-1):0] daisychain_data_d2;
  assign                 TARG_DBUS_daisychain_data[(LANE_COUNT*LANE_WIDTH-1):0] = daisychain_data_d2[(LANE_COUNT*LANE_WIDTH-1):0];

  reg  [7:0]             lane0;
  reg  [7:0]             lane1;
  reg  [7:0]             lane2;

  always @(posedge gclk)
    if (dbg_rst)
      daisychain_data_d1[(LANE_COUNT*LANE_WIDTH-1):0] <= {(LANE_COUNT*LANE_WIDTH) {1'b0}};
    else
      daisychain_data_d1[(LANE_COUNT*LANE_WIDTH-1):0] <= DBUS_TARG_daisychain_data[(LANE_COUNT*LANE_WIDTH-1):0];

  always @(posedge gclk) daisychain_data_d2[23:0] <= { lane2[7:0],lane1[7:0],lane0[7:0] };

  // Priority mux, favoring lower index selct
  always @(*) begin
     casez (MuxSelLaneEn[ 5: 0])
       6'b1????? : lane0[7:0] = dbg_DataMuxed0[LANE_WIDTH*1-1:LANE_WIDTH*0];
       6'b01???? : lane0[7:0] = dbg_DataMuxed1[LANE_WIDTH*1-1:LANE_WIDTH*0];
       6'b001??? : lane0[7:0] = dbg_DataMuxed2[LANE_WIDTH*1-1:LANE_WIDTH*0];
       6'b0001?? : lane0[7:0] = dbg_DataMuxed3[LANE_WIDTH*1-1:LANE_WIDTH*0];
       6'b00001? : lane0[7:0] = dbg_DataMuxed4[LANE_WIDTH*1-1:LANE_WIDTH*0];
       6'b000001 : lane0[7:0] = dbg_DataMuxed5[LANE_WIDTH*1-1:LANE_WIDTH*0];
       default   : lane0[7:0] = daisychain_data_d1[LANE_WIDTH*1-1:LANE_WIDTH*0];
     endcase
     casez (MuxSelLaneEn[11: 6])
       6'b1????? : lane1[7:0] = dbg_DataMuxed0[LANE_WIDTH*2-1:LANE_WIDTH*1];
       6'b01???? : lane1[7:0] = dbg_DataMuxed1[LANE_WIDTH*2-1:LANE_WIDTH*1];
       6'b001??? : lane1[7:0] = dbg_DataMuxed2[LANE_WIDTH*2-1:LANE_WIDTH*1];
       6'b0001?? : lane1[7:0] = dbg_DataMuxed3[LANE_WIDTH*2-1:LANE_WIDTH*1];
       6'b00001? : lane1[7:0] = dbg_DataMuxed4[LANE_WIDTH*2-1:LANE_WIDTH*1];
       6'b000001 : lane1[7:0] = dbg_DataMuxed5[LANE_WIDTH*2-1:LANE_WIDTH*1];
       default   : lane1[7:0] = daisychain_data_d1[LANE_WIDTH*2-1:LANE_WIDTH*1];
     endcase
     casez (MuxSelLaneEn[17:12])
       6'b1????? : lane2[7:0] = dbg_DataMuxed0[LANE_WIDTH*3-1:LANE_WIDTH*2];
       6'b01???? : lane2[7:0] = dbg_DataMuxed1[LANE_WIDTH*3-1:LANE_WIDTH*2];
       6'b001??? : lane2[7:0] = dbg_DataMuxed2[LANE_WIDTH*3-1:LANE_WIDTH*2];
       6'b0001?? : lane2[7:0] = dbg_DataMuxed3[LANE_WIDTH*3-1:LANE_WIDTH*2];
       6'b00001? : lane2[7:0] = dbg_DataMuxed4[LANE_WIDTH*3-1:LANE_WIDTH*2];
       6'b000001 : lane2[7:0] = dbg_DataMuxed5[LANE_WIDTH*3-1:LANE_WIDTH*2];
       default   : lane2[7:0] = daisychain_data_d1[LANE_WIDTH*3-1:LANE_WIDTH*2];
     endcase
  end

generate
if (LANE_COUNT>3) begin
  reg  [7:0]             lane3;

  always @(posedge gclk) daisychain_data_d2[31:24] <= lane3[7:0];

  // Priority mux, favoring lower index selct
  always @(*)
    casez (MuxSelLaneEn[23:18])
      6'b1????? : lane3[7:0] = dbg_DataMuxed0[LANE_WIDTH*4-1:LANE_WIDTH*3];
      6'b01???? : lane3[7:0] = dbg_DataMuxed1[LANE_WIDTH*4-1:LANE_WIDTH*3];
      6'b001??? : lane3[7:0] = dbg_DataMuxed2[LANE_WIDTH*4-1:LANE_WIDTH*3];
      6'b0001?? : lane3[7:0] = dbg_DataMuxed3[LANE_WIDTH*4-1:LANE_WIDTH*3];
      6'b00001? : lane3[7:0] = dbg_DataMuxed4[LANE_WIDTH*4-1:LANE_WIDTH*3];
      6'b000001 : lane3[7:0] = dbg_DataMuxed5[LANE_WIDTH*4-1:LANE_WIDTH*3];
      default   : lane3[7:0] = daisychain_data_d1[LANE_WIDTH*4-1:LANE_WIDTH*3];
    endcase
end // if (LANE_COUNT>3)
endgenerate

  // End ----------------------------------------------------------------------

endmodule // dbg_client_sync_aon_muxf
