//                              -*- Mode: Verilog -*-
// (c) 2015, 2016 Advanced Micro Devices, Inc. All rights reserved.

////////////////////////////////////////////////////////////////////////////
//
// Filename    : dbg_client_sync_aon_prog.v.dpl
// Description : Mux for synchronous debug bus client.
//
// NOTE : This source code is 4-LANE synchronous debug bus client
//        Multi-clock case should use Source-synchronous debug bus client
//
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
// Print out some input parameters to dperl --- begin
// MKEY          = 
// DBG_WIDTH     = 
// MUX_COUNT     = 
// Print out some input parameters to dperl ----- end
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////

module dbg_client_sync_aon_prog #(parameter
LANE_COUNT = 4
) (

// ==========================================================================
// Debug bus Daisy-chain inputs and outputs
// Data flows towards DSM. ctl flows away from DSM
// Begin --------------------------------------------------------------------
output wire [ 1:0] DBUS_TARG_daisychain_ctl , // Daisy-chain program output, reverse of data flow
input       [ 1:0] TARG_DBUS_daisychain_ctl , // Daisy-chain program input,  reverse of data flow
// End ----------------------------------------------------------------------

// ==========================================================================
// interface with first dbg_client_sync_mux instance
// Begin --------------------------------------------------------------------
output wire [(LANE_COUNT*9-1) :0] Mux5SelLane, // LANE_COUNT * ([8] Lane Enable; [7:6] Byte Select; [5:0] Chennel Select;)
output wire [(LANE_COUNT*9-1) :0] Mux4SelLane, // LANE_COUNT * ([8] Lane Enable; [7:6] Byte Select; [5:0] Chennel Select;)
output wire [(LANE_COUNT*9-1) :0] Mux3SelLane, // LANE_COUNT * ([8] Lane Enable; [7:6] Byte Select; [5:0] Chennel Select;)
output wire [(LANE_COUNT*9-1) :0] Mux2SelLane, // LANE_COUNT * ([8] Lane Enable; [7:6] Byte Select; [5:0] Chennel Select;)
output wire [(LANE_COUNT*9-1) :0] Mux1SelLane, // LANE_COUNT * ([8] Lane Enable; [7:6] Byte Select; [5:0] Chennel Select;)
output wire [(LANE_COUNT*9-1) :0] Mux0SelLane, // LANE_COUNT * ([8] Lane Enable; [7:6] Byte Select; [5:0] Chennel Select;)

// End ----------------------------------------------------------------------

// ==========================================================================
// Misc
// Begin --------------------------------------------------------------------
input              ipclk                    , // system clock
input       [7:0]  DbgBlkId                 , // Debug client ID - hard tie
input              cold_resetb              , // From rsmu, Assert for Cold, Cool reset
input              AEB_DBUS                 , // Security lock down. 1'b0-> hold program to reset == data forward
input              TEST_CLK_EN              , // Gater override for test mode. In test mode drive all logic in TARG_DBUS_daisychain_cclk
output wire        dbg_active               , // Clock gater term for data pipe
output wire        dbg_rst                  , // Reset and clear data
output wire [ 7:0] dbg_BlkId                  // Debug bus ID used for debug the debug bus client

// End ----------------------------------------------------------------------

);

  // Map net
  wire  [8:0]                  Mux0SelLaneL[(LANE_COUNT-1) :0]; // [8] Lane Enable; [7:6] Byte Select; [5:0] Chennel Select;
  wire  [8:0]                  Mux1SelLaneL[(LANE_COUNT-1) :0]; // [8] Lane Enable; [7:6] Byte Select; [5:0] Chennel Select;
  wire  [8:0]                  Mux2SelLaneL[(LANE_COUNT-1) :0]; // [8] Lane Enable; [7:6] Byte Select; [5:0] Chennel Select;
  wire  [8:0]                  Mux3SelLaneL[(LANE_COUNT-1) :0]; // [8] Lane Enable; [7:6] Byte Select; [5:0] Chennel Select;
  wire  [8:0]                  Mux4SelLaneL[(LANE_COUNT-1) :0]; // [8] Lane Enable; [7:6] Byte Select; [5:0] Chennel Select;
  wire  [8:0]                  Mux5SelLaneL[(LANE_COUNT-1) :0]; // [8] Lane Enable; [7:6] Byte Select; [5:0] Chennel Select;

generate
case (LANE_COUNT)
3: begin
  assign                       Mux0SelLane[26 :0] = { Mux0SelLaneL[2][8:0], Mux0SelLaneL[1][8:0], Mux0SelLaneL[0][8:0]};
  assign                       Mux1SelLane[26 :0] = { Mux1SelLaneL[2][8:0], Mux1SelLaneL[1][8:0], Mux1SelLaneL[0][8:0]};
  assign                       Mux2SelLane[26 :0] = { Mux2SelLaneL[2][8:0], Mux2SelLaneL[1][8:0], Mux2SelLaneL[0][8:0]};
  assign                       Mux3SelLane[26 :0] = { Mux3SelLaneL[2][8:0], Mux3SelLaneL[1][8:0], Mux3SelLaneL[0][8:0]};
  assign                       Mux4SelLane[26 :0] = { Mux4SelLaneL[2][8:0], Mux4SelLaneL[1][8:0], Mux4SelLaneL[0][8:0]};
  assign                       Mux5SelLane[26 :0] = { Mux5SelLaneL[2][8:0], Mux5SelLaneL[1][8:0], Mux5SelLaneL[0][8:0]};
end // 3
4: begin
  assign                       Mux0SelLane[35 :0] = { Mux0SelLaneL[3][8:0], Mux0SelLaneL[2][8:0], Mux0SelLaneL[1][8:0], Mux0SelLaneL[0][8:0]};
  assign                       Mux1SelLane[35 :0] = { Mux1SelLaneL[3][8:0], Mux1SelLaneL[2][8:0], Mux1SelLaneL[1][8:0], Mux1SelLaneL[0][8:0]};
  assign                       Mux2SelLane[35 :0] = { Mux2SelLaneL[3][8:0], Mux2SelLaneL[2][8:0], Mux2SelLaneL[1][8:0], Mux2SelLaneL[0][8:0]};
  assign                       Mux3SelLane[35 :0] = { Mux3SelLaneL[3][8:0], Mux3SelLaneL[2][8:0], Mux3SelLaneL[1][8:0], Mux3SelLaneL[0][8:0]};
  assign                       Mux4SelLane[35 :0] = { Mux4SelLaneL[3][8:0], Mux4SelLaneL[2][8:0], Mux4SelLaneL[1][8:0], Mux4SelLaneL[0][8:0]};
  assign                       Mux5SelLane[35 :0] = { Mux5SelLaneL[3][8:0], Mux5SelLaneL[2][8:0], Mux5SelLaneL[1][8:0], Mux5SelLaneL[0][8:0]};
end // 4
endcase // case (LANE_COUNT)
endgenerate

  // Marker bug so this will stay in gate
  ati_buffer1 mrk_dbg_BlkId[7:0] (.A(DbgBlkId[7:0]), .Y(dbg_BlkId[7:0]));

  // ==========================================================================
  // Control Path
  // Begin --------------------------------------------------------------------
  wire                         gcclk             ; // gated system clock (ipclk) for control logic
  wire                         ctlProgClrDat     ; // Clear the data Pipe
  wire                         ctlProgClrCtl     ; // Clear the Control logic data
  wire                         ctlProgVal        ; // control program : control valid
  wire [1:0]                   ctlProg_ByteSel   ; // control program : bit Byte select // removed [2]Multi-block mode
  wire [2:0]                   ctlProg_MuxId     ; // control program : Mux ID of the current program target
  wire [1:0]                   ctlProg_LaneSel   ; // control program : Lane select of mux
  wire [5:0]                   ctlProg_ChannelSel; // control program : Channel select of mux

  assign                       dbg_rst = ctlProgClrDat;
  dbg_client_sync_aon_s2p
    I_dbg_client_sync_aon_s2p (
                              .ipclk                    (ipclk                         ) , // system clock
                              .DbgBlkId                 (DbgBlkId                 [7:0]), // Debug client ID - hard tie
                              .cold_resetb              (cold_resetb                   ) , // From rsmu, Assert for Cold, Cool reset
                              .AEB_DBUS                 (AEB_DBUS                      ) , // Security lock down. 1'b0-> hold program to reset == data forward
                              .TEST_CLK_EN              (TEST_CLK_EN                   ) , // Gater override for test mode
                              .dbg_active               (dbg_active                    ) , // output the gater term for data pipe (xxx_xxx_daisychain_data)

                              .DBUS_TARG_daisychain_ctl (DBUS_TARG_daisychain_ctl [1:0]) , // Daisy-chain program output, reverse of data flow
                              .TARG_DBUS_daisychain_ctl (TARG_DBUS_daisychain_ctl [1:0]) , // Daisy-chain program input,  reverse of data flow

                              .gcclk                    (gcclk                         ) , // gated system clock (ipclk) for control logic
                              .ctlProgClrDat            (ctlProgClrDat                 ) , // Clear the data pipe
                              .ctlProgClrCtl            (ctlProgClrCtl                 ) , // Clear the Control logic data
                              .ctlProgVal               (ctlProgVal                    ) , // control program : control valid
                              .ctlProg_ByteSel          (ctlProg_ByteSel          [1:0]) , // control program : bit Byte select // removed [2]Multi-block mode
                              .ctlProg_MuxId            (ctlProg_MuxId            [2:0]) , // control program : Mux ID of the current program target
                              .ctlProg_LaneSel          (ctlProg_LaneSel          [1:0]) , // control program : Lane select of mux
                              .ctlProg_ChannelSel       (ctlProg_ChannelSel       [5:0])   // control program : Channel select of mux
                              );

  // Store Mux select info per Lane
  reg [7:0]                    MuxSelLane0;
  reg [7:0]                    MuxSelLane1;
  reg [7:0]                    MuxSelLane2;
  always @(posedge gcclk)
     if (ctlProgClrCtl) begin
        MuxSelLane0[7:0] <= {8 {1'b0}};
        MuxSelLane1[7:0] <= {8 {1'b0}};
        MuxSelLane2[7:0] <= {8 {1'b0}};
     end else
       case ({ctlProgVal,ctlProg_LaneSel[1:0]})
         3'b100 : begin
            MuxSelLane0[7:0] <= {ctlProg_ByteSel[1:0], ctlProg_ChannelSel[5:0]};
            MuxSelLane1[7:0] <= MuxSelLane1[7:0];
            MuxSelLane2[7:0] <= MuxSelLane2[7:0];
         end
         3'b101 : begin
            MuxSelLane0[7:0] <= MuxSelLane0[7:0];
            MuxSelLane1[7:0] <= {ctlProg_ByteSel[1:0], ctlProg_ChannelSel[5:0]};
            MuxSelLane2[7:0] <= MuxSelLane2[7:0];
         end
         3'b110 : begin
            MuxSelLane0[7:0] <= MuxSelLane0[7:0];
            MuxSelLane1[7:0] <= MuxSelLane1[7:0];
            MuxSelLane2[7:0] <= {ctlProg_ByteSel[1:0], ctlProg_ChannelSel[5:0]};
         end
         default : begin
            MuxSelLane0[7:0] <= MuxSelLane0[7:0];
            MuxSelLane1[7:0] <= MuxSelLane1[7:0];
            MuxSelLane2[7:0] <= MuxSelLane2[7:0];
         end
       endcase // case ({ctlProgVal,ctlProg_LaneSel[1:0]})

  // Forward the Mux select info per Lane to output
   assign Mux0SelLaneL[0][7:0] = MuxSelLane0[7:0];
   assign Mux0SelLaneL[1][7:0] = MuxSelLane1[7:0];
   assign Mux0SelLaneL[2][7:0] = MuxSelLane2[7:0];
   assign Mux1SelLaneL[0][7:0] = MuxSelLane0[7:0];
   assign Mux1SelLaneL[1][7:0] = MuxSelLane1[7:0];
   assign Mux1SelLaneL[2][7:0] = MuxSelLane2[7:0];
   assign Mux2SelLaneL[0][7:0] = MuxSelLane0[7:0];
   assign Mux2SelLaneL[1][7:0] = MuxSelLane1[7:0];
   assign Mux2SelLaneL[2][7:0] = MuxSelLane2[7:0];
   assign Mux3SelLaneL[0][7:0] = MuxSelLane0[7:0];
   assign Mux3SelLaneL[1][7:0] = MuxSelLane1[7:0];
   assign Mux3SelLaneL[2][7:0] = MuxSelLane2[7:0];
   assign Mux4SelLaneL[0][7:0] = MuxSelLane0[7:0];
   assign Mux4SelLaneL[1][7:0] = MuxSelLane1[7:0];
   assign Mux4SelLaneL[2][7:0] = MuxSelLane2[7:0];
   assign Mux5SelLaneL[0][7:0] = MuxSelLane0[7:0];
   assign Mux5SelLaneL[1][7:0] = MuxSelLane1[7:0];
   assign Mux5SelLaneL[2][7:0] = MuxSelLane2[7:0];

generate
  reg [7:0]                    MuxSelLane3;
  always @(posedge gcclk)
     if (ctlProgClrCtl) begin
        MuxSelLane3[7:0] <= {8 {1'b0}};
     end else
       case ({ctlProgVal,ctlProg_LaneSel[1:0]})
         3'b111  : MuxSelLane3[7:0] <= {ctlProg_ByteSel[1:0], ctlProg_ChannelSel[5:0]};
         default : MuxSelLane3[7:0] <= MuxSelLane3[7:0];
       endcase // case ({ctlProgVal,ctlProg_LaneSel[1:0]})
  // Forward the Mux select info per Lane to output
   assign Mux0SelLaneL[3][7:0] = MuxSelLane3[7:0];
   assign Mux1SelLaneL[3][7:0] = MuxSelLane3[7:0];
   assign Mux2SelLaneL[3][7:0] = MuxSelLane3[7:0];
   assign Mux3SelLaneL[3][7:0] = MuxSelLane3[7:0];
   assign Mux4SelLaneL[3][7:0] = MuxSelLane3[7:0];
   assign Mux5SelLaneL[3][7:0] = MuxSelLane3[7:0];
endgenerate

  // Store 3 bit Mux address per Lane and decode
  // Reset Mux address to 3'b111, since that is not a valid MuxId
  reg [2:0]                    regLaneMuxId0;
  reg [2:0]                    regLaneMuxId1;
  reg [2:0]                    regLaneMuxId2;
  always @(posedge gcclk)
     if (ctlProgClrCtl) begin
        regLaneMuxId0[2:0] <= {3 {1'b1}};
        regLaneMuxId1[2:0] <= {3 {1'b1}};
        regLaneMuxId2[2:0] <= {3 {1'b1}};
     end else
       case ({ctlProgVal,ctlProg_LaneSel[1:0]})
         3'b100 : begin
            regLaneMuxId0[2:0] <= ctlProg_MuxId[2:0];
            regLaneMuxId1[2:0] <= regLaneMuxId1[2:0];
            regLaneMuxId2[2:0] <= regLaneMuxId2[2:0];
         end
         3'b101 : begin
            regLaneMuxId0[2:0] <= regLaneMuxId0[2:0];
            regLaneMuxId1[2:0] <= ctlProg_MuxId[2:0];
            regLaneMuxId2[2:0] <= regLaneMuxId2[2:0];
         end
         3'b110 : begin
            regLaneMuxId0[2:0] <= regLaneMuxId0[2:0];
            regLaneMuxId1[2:0] <= regLaneMuxId1[2:0];
            regLaneMuxId2[2:0] <= ctlProg_MuxId[2:0];
         end
         default : begin
            regLaneMuxId0[2:0] <= regLaneMuxId0[2:0];
            regLaneMuxId1[2:0] <= regLaneMuxId1[2:0];
            regLaneMuxId2[2:0] <= regLaneMuxId2[2:0];
         end
       endcase // case ({ctlProgVal,ctlProg_LaneSel[1:0]})

// Decode the MuxId for each lane
// If timing needs it, we can multi-cycle this path
   assign Mux0SelLaneL[0][8] = (regLaneMuxId0[2:0] == 3'b000);
   assign Mux0SelLaneL[1][8] = (regLaneMuxId1[2:0] == 3'b000);
   assign Mux0SelLaneL[2][8] = (regLaneMuxId2[2:0] == 3'b000);
   assign Mux1SelLaneL[0][8] = (regLaneMuxId0[2:0] == 3'b001);
   assign Mux1SelLaneL[1][8] = (regLaneMuxId1[2:0] == 3'b001);
   assign Mux1SelLaneL[2][8] = (regLaneMuxId2[2:0] == 3'b001);
   assign Mux2SelLaneL[0][8] = (regLaneMuxId0[2:0] == 3'b010);
   assign Mux2SelLaneL[1][8] = (regLaneMuxId1[2:0] == 3'b010);
   assign Mux2SelLaneL[2][8] = (regLaneMuxId2[2:0] == 3'b010);
   assign Mux3SelLaneL[0][8] = (regLaneMuxId0[2:0] == 3'b011);
   assign Mux3SelLaneL[1][8] = (regLaneMuxId1[2:0] == 3'b011);
   assign Mux3SelLaneL[2][8] = (regLaneMuxId2[2:0] == 3'b011);
   assign Mux4SelLaneL[0][8] = (regLaneMuxId0[2:0] == 3'b100);
   assign Mux4SelLaneL[1][8] = (regLaneMuxId1[2:0] == 3'b100);
   assign Mux4SelLaneL[2][8] = (regLaneMuxId2[2:0] == 3'b100);
   assign Mux5SelLaneL[0][8] = (regLaneMuxId0[2:0] == 3'b101);
   assign Mux5SelLaneL[1][8] = (regLaneMuxId1[2:0] == 3'b101);
   assign Mux5SelLaneL[2][8] = (regLaneMuxId2[2:0] == 3'b101);
generate
  reg [2:0]                    regLaneMuxId3;
  always @(posedge gcclk)
     if (ctlProgClrCtl) begin
        regLaneMuxId3[2:0] <= {3 {1'b1}};
     end else
       case ({ctlProgVal,ctlProg_LaneSel[1:0]})
         3'b111  : regLaneMuxId3[2:0] <= ctlProg_MuxId[2:0];
         default : regLaneMuxId3[2:0] <= regLaneMuxId3[2:0];
       endcase // case ({ctlProgVal,ctlProg_LaneSel[1:0]})

// Decode the MuxId for each lane
// If timing needs it, we can multi-cycle this path
   assign Mux0SelLaneL[3][8] = (regLaneMuxId3[2:0] == 3'b000);
   assign Mux1SelLaneL[3][8] = (regLaneMuxId3[2:0] == 3'b001);
   assign Mux2SelLaneL[3][8] = (regLaneMuxId3[2:0] == 3'b010);
   assign Mux3SelLaneL[3][8] = (regLaneMuxId3[2:0] == 3'b011);
   assign Mux4SelLaneL[3][8] = (regLaneMuxId3[2:0] == 3'b100);
   assign Mux5SelLaneL[3][8] = (regLaneMuxId3[2:0] == 3'b101);
endgenerate
  // End ----------------------------------------------------------------------

endmodule // dbg_client_sync_aon_prog
