//                              -*- Mode: Verilog -*-
// (c) 2015, 2016 Advanced Micro Devices, Inc. All rights reserved.

////////////////////////////////////////////////////////////////////////////
//
// Filename    : dbg_client_sync_aon_retimer.v
// Description : Mux for synchronous debug bus client.
//
// NOTE : Multi-clock case should use Source-synchronous debug bus client
//
////////////////////////////////////////////////////////////////////////////

module dbg_client_sync_aon_retimer #(parameter WIDTH = 32) (

input                   ipclk                    , // system clock
input                   TEST_CLK_EN              , // Gater override for test mode

input       [WIDTH-1:0] DBUS_TARG_daisychain_data, // Daisy-chain input
output wire [1:0]       DBUS_TARG_daisychain_ctl , // Daisy-chain program output, reverse of data flow

output wire [WIDTH-1:0] TARG_DBUS_daisychain_data, // Daisy-chain output
input       [1:0]       TARG_DBUS_daisychain_ctl   // Daisy-chain program input,  reverse of data flow
);

  reg [WIDTH-1:0]       daisychain_data_d1;
  assign                TARG_DBUS_daisychain_data[WIDTH-1:0] = daisychain_data_d1[WIDTH-1:0];

  wire                  gclk;
  wire                  dbg_active;
  dbg_client_sync_aon_gclk I_dbg_active (
                                        .ipclk                    (ipclk                        ),
                                        .dbg_active               (dbg_active                   ),
                                        .daisychain_ctl_d1        (DBUS_TARG_daisychain_ctl[1:0]),
                                        .TARG_DBUS_daisychain_ctl (TARG_DBUS_daisychain_ctl[1:0])
                                        );

  // Clock gater ($STEM/import/rtllib/src/rtl/ati_clock_gate.v)
  ati_clock_gate I_gclk (.clk_out(gclk), .clk_src(ipclk), .clk_en(dbg_active), .TE(TEST_CLK_EN));

  always @(posedge gclk) daisychain_data_d1[WIDTH-1:0] <= DBUS_TARG_daisychain_data[WIDTH-1:0];

endmodule // dbg_client_sync_aon_retimer
