//                              -*- Mode: Verilog -*-
// (c) 2015, 2016 Advanced Micro Devices, Inc. All rights reserved.

////////////////////////////////////////////////////////////////////////////
//
// Filename    : dbg_client_sync_aon_steer.v.dpl
// Description : Mux for synchronous debug bus client.
//
// NOTE : Multi-clock case should use Source-synchronous debug bus client
//        BOZO: need to add assertion to check for DbgBlkId match when idebug63 is selected
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
// Print out some input parameters to dperl --- begin
// MKEY          = 
// DBG_WIDTH     = 
// MUX_COUNT     = 
// Print out some input parameters to dperl ----- end
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////

module dbg_client_sync_aon_steer #(parameter
LANE_COUNT = 4,
LANE_WIDTH = 8
) (

// ==========================================================================
// Debug bus Daisy-chain inputs and outputs
// Data flows towards DSM. ctl flows away from DSM
// Begin --------------------------------------------------------------------
output wire [1:0]  DBUS_TARG_daisychain_ctl , // Daisy-chain program output, reverse of data flow
input       [1:0]  TARG_DBUS_daisychain_ctl , // Daisy-chain program input,  reverse of data flow

input       [(LANE_COUNT*LANE_WIDTH-1):0] DBUS_TARG_daisychain_data, // Daisy-chain input
output wire [(LANE_COUNT*LANE_WIDTH-1):0] TARG_DBUS_daisychain_data, // Daisy-chain output
// End ----------------------------------------------------------------------

// ==========================================================================
// interface with first dbg_client_sync_mux instance
// Begin --------------------------------------------------------------------
output wire [(LANE_COUNT*9-1):0] dbg_Mux5_SelLane , // MuxId 5: LANE_COUNT * ([8] Lane Enable; [7:6] Byte Select; [5:0] Chennel Select;)
input       [(LANE_COUNT*LANE_WIDTH-1):0] dbg_Mux5_Data    , // MuxId 5: Debug-bus data outputs after muxing
output wire [(LANE_COUNT*9-1):0] dbg_Mux4_SelLane , // MuxId 4: LANE_COUNT * ([8] Lane Enable; [7:6] Byte Select; [5:0] Chennel Select;)
input       [(LANE_COUNT*LANE_WIDTH-1):0] dbg_Mux4_Data    , // MuxId 4: Debug-bus data outputs after muxing
output wire [(LANE_COUNT*9-1):0] dbg_Mux3_SelLane , // MuxId 3: LANE_COUNT * ([8] Lane Enable; [7:6] Byte Select; [5:0] Chennel Select;)
input       [(LANE_COUNT*LANE_WIDTH-1):0] dbg_Mux3_Data    , // MuxId 3: Debug-bus data outputs after muxing
output wire [(LANE_COUNT*9-1):0] dbg_Mux2_SelLane , // MuxId 2: LANE_COUNT * ([8] Lane Enable; [7:6] Byte Select; [5:0] Chennel Select;)
input       [(LANE_COUNT*LANE_WIDTH-1):0] dbg_Mux2_Data    , // MuxId 2: Debug-bus data outputs after muxing
output wire [(LANE_COUNT*9-1):0] dbg_Mux1_SelLane , // MuxId 1: LANE_COUNT * ([8] Lane Enable; [7:6] Byte Select; [5:0] Chennel Select;)
input       [(LANE_COUNT*LANE_WIDTH-1):0] dbg_Mux1_Data    , // MuxId 1: Debug-bus data outputs after muxing
output wire [(LANE_COUNT*9-1):0] dbg_Mux0_SelLane , // MuxId 0: LANE_COUNT * ([8] Lane Enable; [7:6] Byte Select; [5:0] Chennel Select;)
input       [(LANE_COUNT*LANE_WIDTH-1):0] dbg_Mux0_Data    , // MuxId 0: Debug-bus data outputs after muxing
// End ----------------------------------------------------------------------

// ==========================================================================
// Misc
// Begin --------------------------------------------------------------------
input              ipclk                    , // system clock
input       [7:0]  DbgBlkId                 , // Debug client ID - hard tie
input              cold_resetb              , // From rsmu, Assert for Cold, Cool reset
input              AEB_DBUS                 , // Security lock down. 1'b0-> hold program to reset == data forward
input              TEST_CLK_EN              , // Gater override for test mode
output wire        dbg_active               , // Clock gater term for data pipe
output wire        dbg_rst                  , // Reset and clear data
output wire [7:0]  dbg_BlkId                  // Debug bus ID used for debug the debug bus client (same as DbgBlkId)
// End ----------------------------------------------------------------------

);

  // ==========================================================================
  // Control Path
  // Begin --------------------------------------------------------------------
  dbg_client_sync_aon_prog #(.LANE_COUNT(LANE_COUNT))
    I_dbg_client_sync_aon_prog (
                                .DBUS_TARG_daisychain_ctl (DBUS_TARG_daisychain_ctl               [1:0]), // Daisy-chain program output, reverse of data flow
                                .TARG_DBUS_daisychain_ctl (TARG_DBUS_daisychain_ctl               [1:0]), // Daisy-chain program input,  reverse of data flow
                                .Mux5SelLane              (dbg_Mux5_SelLane        [(LANE_COUNT*9-1):0]), // MuxId 5: LANE_COUNT * ([8] Lane Enable; [7:6] Byte Select; [5:0] Chennel Select;)
                                .Mux4SelLane              (dbg_Mux4_SelLane        [(LANE_COUNT*9-1):0]), // MuxId 4: LANE_COUNT * ([8] Lane Enable; [7:6] Byte Select; [5:0] Chennel Select;)
                                .Mux3SelLane              (dbg_Mux3_SelLane        [(LANE_COUNT*9-1):0]), // MuxId 3: LANE_COUNT * ([8] Lane Enable; [7:6] Byte Select; [5:0] Chennel Select;)
                                .Mux2SelLane              (dbg_Mux2_SelLane        [(LANE_COUNT*9-1):0]), // MuxId 2: LANE_COUNT * ([8] Lane Enable; [7:6] Byte Select; [5:0] Chennel Select;)
                                .Mux1SelLane              (dbg_Mux1_SelLane        [(LANE_COUNT*9-1):0]), // MuxId 1: LANE_COUNT * ([8] Lane Enable; [7:6] Byte Select; [5:0] Chennel Select;)
                                .Mux0SelLane              (dbg_Mux0_SelLane        [(LANE_COUNT*9-1):0]), // MuxId 0: LANE_COUNT * ([8] Lane Enable; [7:6] Byte Select; [5:0] Chennel Select;)
                                .ipclk                    (ipclk                                       ), // system clock
                                .DbgBlkId                 (DbgBlkId                               [7:0]), // Debug client ID - hard tie
                                .cold_resetb              (cold_resetb                                 ), // From rsmu, Assert for Cold, Cool reset
                                .AEB_DBUS                 (AEB_DBUS                                    ), // Security lock down. 1'b0-> hold program to reset == data forward
                                .TEST_CLK_EN              (TEST_CLK_EN                                 ), // Gater override for test mode. In test mode drive all logic in TARG_DBUS_daisychain_cclk
                                .dbg_active               (dbg_active                                  ), // Clock gater term for data pipe
                                .dbg_rst                  (dbg_rst                                     ), // Reset and clear data pipe
                                .dbg_BlkId                (dbg_BlkId                              [7:0])  // Debug bus ID used for debug the debug bus client (same as DbgBlkId)
                                );

  // End ----------------------------------------------------------------------

  // ==========================================================================
  // Data Pipe
  // Begin --------------------------------------------------------------------
  wire [(LANE_COUNT*6-1) :0] MuxSelLaneEn;

generate
case (LANE_COUNT)
3:
  assign MuxSelLaneEn[17 : 0] = {
                                  dbg_Mux0_SelLane[26], dbg_Mux1_SelLane[26], dbg_Mux2_SelLane[26], dbg_Mux3_SelLane[26], dbg_Mux4_SelLane[26], dbg_Mux5_SelLane[26],
                                  dbg_Mux0_SelLane[17], dbg_Mux1_SelLane[17], dbg_Mux2_SelLane[17], dbg_Mux3_SelLane[17], dbg_Mux4_SelLane[17], dbg_Mux5_SelLane[17],
                                  dbg_Mux0_SelLane[ 8], dbg_Mux1_SelLane[ 8], dbg_Mux2_SelLane[ 8], dbg_Mux3_SelLane[ 8], dbg_Mux4_SelLane[ 8], dbg_Mux5_SelLane[ 8]
                                 };
4:
  assign MuxSelLaneEn[23 : 0] = {
                                  dbg_Mux0_SelLane[35], dbg_Mux1_SelLane[35], dbg_Mux2_SelLane[35], dbg_Mux3_SelLane[35], dbg_Mux4_SelLane[35], dbg_Mux5_SelLane[35],
                                  dbg_Mux0_SelLane[26], dbg_Mux1_SelLane[26], dbg_Mux2_SelLane[26], dbg_Mux3_SelLane[26], dbg_Mux4_SelLane[26], dbg_Mux5_SelLane[26],
                                  dbg_Mux0_SelLane[17], dbg_Mux1_SelLane[17], dbg_Mux2_SelLane[17], dbg_Mux3_SelLane[17], dbg_Mux4_SelLane[17], dbg_Mux5_SelLane[17],
                                  dbg_Mux0_SelLane[ 8], dbg_Mux1_SelLane[ 8], dbg_Mux2_SelLane[ 8], dbg_Mux3_SelLane[ 8], dbg_Mux4_SelLane[ 8], dbg_Mux5_SelLane[ 8]
                                 };

endcase // case (LANE_COUNT)
endgenerate

  dbg_client_sync_aon_muxf #(.LANE_COUNT(LANE_COUNT), .LANE_WIDTH(LANE_WIDTH))
    I_dbg_client_sync_aon_muxf (
                                .DBUS_TARG_daisychain_data (DBUS_TARG_daisychain_data [(LANE_COUNT*LANE_WIDTH-1):0]), // Daisy-chain input
                                .TARG_DBUS_daisychain_data (TARG_DBUS_daisychain_data [(LANE_COUNT*LANE_WIDTH-1):0]), // Daisy-chain output
                                .MuxSelLaneEn              (MuxSelLaneEn              [(LANE_COUNT*6-1):0]),
                                .dbg_DataMuxed5            (dbg_Mux5_Data             [(LANE_COUNT*LANE_WIDTH-1):0]), // Debug-bus data from mux(s) output
                                .dbg_DataMuxed4            (dbg_Mux4_Data             [(LANE_COUNT*LANE_WIDTH-1):0]), // Debug-bus data from mux(s) output
                                .dbg_DataMuxed3            (dbg_Mux3_Data             [(LANE_COUNT*LANE_WIDTH-1):0]), // Debug-bus data from mux(s) output
                                .dbg_DataMuxed2            (dbg_Mux2_Data             [(LANE_COUNT*LANE_WIDTH-1):0]), // Debug-bus data from mux(s) output
                                .dbg_DataMuxed1            (dbg_Mux1_Data             [(LANE_COUNT*LANE_WIDTH-1):0]), // Debug-bus data from mux(s) output
                                .dbg_DataMuxed0            (dbg_Mux0_Data             [(LANE_COUNT*LANE_WIDTH-1):0]), // Debug-bus data from mux(s) output
                                .ipclk                     (ipclk                                         ), // system clock
                                .dbg_rst                   (dbg_rst                                       ), // Reset and clear data pipe
                                .TEST_CLK_EN               (TEST_CLK_EN                                   ), // Gater override for test mode
                                .dbg_active                (dbg_active                                    )  // Clock gater term for data pipe
                                );
  // End ----------------------------------------------------------------------

endmodule // dbg_client_sync_aon_steer
