//                              -*- Mode: Verilog -*-
// (c) 2015, 2016 Advanced Micro Devices, Inc. All rights reserved.

////////////////////////////////////////////////////////////////////////////
//
// Filename    : dbg_client_sync_mux_flop.v
// Description : Flop stage to help timing for input to dbg_client_sync_mux
//               User not required to used this module, but served as an example of usage
//
////////////////////////////////////////////////////////////////////////////
module dbg_client_sync_mux_flop (
idebug  , // Debug-bus data output to dbg_client_sync_mux
debugIn , // Debug-bus data inputs from functional logic

gclk    , // Gated clock for potential pipe stage
dbg_rst   // Combined global reset with command decoded reset - This is optional make the data pip depth easier to track

);
parameter LANE_COUNT     = 4            ;
parameter LANE_WIDTH     = 8            ;

output wire [(LANE_COUNT*LANE_WIDTH)-1:0] idebug  ; // Debug-bus data output to dbg_client_sync_mux
input       [(LANE_COUNT*LANE_WIDTH)-1:0] debugIn ; // Debug-bus data inputs from functional logic

input                                     gclk    ; // Gated clock for potential pipe stage
input                                     dbg_rst ; // Combined global reset with command decoded reset - This is optional make the data pip depth easier to track

  reg [(LANE_COUNT*LANE_WIDTH)-1:0]       reg_debug;
  assign                                  idebug[(LANE_COUNT*LANE_WIDTH)-1:0] = reg_debug[(LANE_COUNT*LANE_WIDTH)-1:0];

  always @(posedge gclk)
    if (dbg_rst)
      reg_debug[(LANE_COUNT*LANE_WIDTH)-1:0] <= {(LANE_COUNT*LANE_WIDTH) {1'b0}};
    else
      reg_debug[(LANE_COUNT*LANE_WIDTH)-1:0] <= debugIn[(LANE_COUNT*LANE_WIDTH)-1:0];

endmodule // dbg_client_sync_mux_flop
