
//
// $Id$
//
// $Change$
//
//
// Notes:       Debug bus - Common Module
//              24 bits of debug bus, bytes selectable with one clock per byte.
//
// Copyright:   Trade secret of ATI Technologies, Inc.
//              Copyright 2005 ATI Technologies, Inc., (unpublished)
//
//              All rights reserved.  This notice is intended as a precaution against
//              inadvertent publication and does not imply publication or any waiver
//              of confidentiality.  The year included in the foregoing notice is the
//              year of creation of the work.
//
//          
//--------------------------------------------------------------------------


module dbg_common (
		  
		   DEBUG_bus_out,
		   DEBUG_ctrl_out,

                   DEBUG_bus_in,
                   DEBUG_ctrl_in,
		   
		   Block_ID,
		   local_data1,
		   local_data0,

                   DEBUG_group_sel0,
                   DEBUG_group_sel1,
                   
		   global_sclk
               );


   // Debug Bus output signals
   output [23:0] DEBUG_bus_out;
   output        DEBUG_ctrl_out;
  
   // Debug Bus input signals
   input [23:0]  DEBUG_bus_in;
   input         DEBUG_ctrl_in;

   // Local Block signals
   output [4:0]  DEBUG_group_sel0;
   output [4:0]  DEBUG_group_sel1;
   
   input [7:0]	 Block_ID;
   input [23:0]	 local_data1;
   input [23:0]	 local_data0;
 
   input 	 global_sclk;

   

   // Locally registered control signals
   reg [7:0] 	 DEBUG_block_sel0;
   reg [7:0] 	 DEBUG_block_sel1;

   reg [4:0]     DEBUG_group_sel0;
   reg [4:0]     DEBUG_group_sel1;

   reg [2:0]     DEBUG_out_mux_sel0;
   reg [2:0]     DEBUG_out_mux_sel1;

   reg           DEBUG_mux_sel0;
   reg           DEBUG_mux_sel1;
  
   wire 	 dbg_data1_selected;
   wire 	 dbg_data0_selected;
  
 
   wire [23:0] 	 dbg_bus_out;
   wire          dbg_ctrl_out;          
 
  
   reg [23:0] 	 DEBUG_bus_out;
   reg           DEBUG_ctrl_out;
   
   reg [23:0] 	 dbg_bus_in;
   reg           dbg_ctrl_in;           
   reg           dbg_chan_sel_in;           
  
   reg [11:0] 	 local_data1_reg;
   reg [11:0] 	 local_data0_reg;
 

    //==============================
   // Register input data and control before using it
   // using the global sclk
   //==============================			   
   
   always @(posedge global_sclk)
     dbg_bus_in <= DEBUG_bus_in[23:0];
 
   always @(posedge global_sclk)
     dbg_ctrl_in <= DEBUG_ctrl_in;
 
   always @(*)
     dbg_chan_sel_in = dbg_bus_in[23];
 
    
   
   //==============================
   // Register the control information on dbg_ctrl_in
   //==============================
   always @(posedge global_sclk)
     begin
        if (~dbg_ctrl_in)
          begin
            if (~dbg_chan_sel_in)
	      begin
                DEBUG_block_sel0   <= dbg_bus_in[7:0];
                DEBUG_group_sel0   <= dbg_bus_in[16:12];
                DEBUG_out_mux_sel0 <= dbg_bus_in[21:19];
                DEBUG_mux_sel0     <= dbg_bus_in[22:22];
              end
	    else
	      begin
                DEBUG_block_sel1   <= dbg_bus_in[7:0];
                DEBUG_group_sel1   <= dbg_bus_in[16:12];
                DEBUG_out_mux_sel1 <= dbg_bus_in[21:19];
                DEBUG_mux_sel1     <= dbg_bus_in[22:22];
              end
	  end
     end
   
   
   //==============================
   // Register local data if selected
   //==============================
				   
   always @(posedge global_sclk)
     begin
        if (dbg_data0_selected)
	  begin
             local_data0_reg <= DEBUG_mux_sel0 ? local_data0[23:12] : local_data0[11:0];
	  end
    
        if (dbg_data1_selected)
	  begin
             local_data1_reg <= DEBUG_mux_sel1 ? local_data1[23:12] : local_data1[11:0];
	  end
        
     end // always @ (posedge global_sclk)
   
  
  
   //=============================
   //=== DBG block selection 
   //=============================
   assign 	 dbg_data1_selected = (DEBUG_block_sel1 == Block_ID);
   assign 	 dbg_data0_selected = (DEBUG_block_sel0 == Block_ID);
   
   //===============================================================
   // Muxing between the input data, regs, constants, or local data
   //===============================================================

 function [11:0] out_mux_func;
      input        ctrl;
      input        sel;
      input  [2:0] out_mux_sel;
      input [11:0] local_bus;
      input [11:0] dbg_bus;
      input  [7:0] Block_ID;
      input  [7:0] block_sel;
      begin
	// If program mode or this block not selected out <= in
	if (~ctrl | ~sel)
	  begin
		out_mux_func[11:0] = dbg_bus[11:0];
	  end
	else // it is a function of out_mux_sel only
	  begin
		case(out_mux_sel[2:0])
		3'h0: out_mux_func[11:0] = local_bus[11:0];
		3'h1: out_mux_func[11:0] =  {4'h0, block_sel[7:0]};
		3'h2: out_mux_func[11:0] = ~{4'h0, block_sel[7:0]};
		3'h3: out_mux_func[11:0] =  {4'h0, Block_ID[7:0]};
		3'h4: out_mux_func[11:0] = 12'h555;
		3'h5: out_mux_func[11:0] = 12'haaa;
		default: 
		      out_mux_func[11:0] = 12'h3fc;
		endcase
	  end // else
      end 
   endfunction // out_mux_func


   //=============================================
   // Assign Output Buses based on muxing function
   //=============================================

   assign dbg_ctrl_out       = dbg_ctrl_in; 
   assign dbg_bus_out[23:12] = out_mux_func(
					  dbg_ctrl_in, 
					  dbg_data1_selected, 
					  DEBUG_out_mux_sel1[2:0],
					  local_data1_reg[11:0], 
					  dbg_bus_in[23:12], 
					  Block_ID[7:0],
					  DEBUG_block_sel1[7:0]
				         );

    assign dbg_bus_out[11:0] = out_mux_func(
					  dbg_ctrl_in, 
					  dbg_data0_selected, 
					  DEBUG_out_mux_sel0[2:0],
					  local_data0_reg[11:0], 
					  dbg_bus_in[11:0], 
					  Block_ID[7:0],
					  DEBUG_block_sel0[7:0]
				         );
   
    
   //==============================
   // Register output data and control 
   //==============================
   
   always @(posedge global_sclk)
     DEBUG_bus_out[23:0] <= dbg_bus_out;

   always @(posedge global_sclk)
     DEBUG_ctrl_out <= dbg_ctrl_out;

endmodule
