//                              -*- Mode: Verilog -*-

////////////////////////////////////////////////////////////////////////////
/*
 *
 *  Trade secret of ATI Technologies, Inc.
 *  Copyright 2005, ATI Technologies, Inc., (unpublished)
 *
 *  All rights reserved.  This notice is intended as a precaution against
 *  inadvertent publication and does not imply publication or any waiver
 *  of confidentiality.  The year included in the foregoing notice is the
 *  year of creation of the work.
 *
 */
////////////////////////////////////////////////////////////////////////////
//
// Filename    : dft_clk_cntl.v
// Description : DFT clock controller.
//
////////////////////////////////////////////////////////////////////////////

//`include "dft_clk_cntl_features.vh"

//;use comp_config;



module dft_clk_cntl_gc10
#(
  parameter DFT_CLK_CNTL_PULSE_CNT = 8,
  parameter DFT_CLK_CNTL_LATENCY_CNT = 10,
  parameter DFT_CLK_CNTL_NO_SHIFT_CLK = 0
  )
(
 
 
 //Broadcast to All TDRs
 input wire Tdr_Tck,
 input wire Tdr_Trstb,
 input wire Tdr_Shift,
 input wire Tdr_Capture,
 input wire Tdr_Update,
 
 //From Tdr_Broadcast
 input wire Tdr_clk_cntl_Tdi,
 input wire Tdr_latency_cnt_Tdi,
 input wire Tdr_pulse_cnt_Tdi,
 
 //Tdr En/So
 input  wire Tdr_clk_cntl_En,
 output wire Tdr_clk_cntl_Tdo,
 
 input  wire Tdr_latency_cnt_En,
 output wire Tdr_latency_cnt_Tdo,
 
 input  wire Tdr_pulse_cnt_En,
 output wire Tdr_pulse_cnt_Tdo,
 
 input  wire Tile_ScanEn,  // flopped scanEn from scan_cntl
 output wire Tile_CapEn,  
 input  wire Edt_Update,
 input  wire Tile_GaterEn,
 output wire ScanEn_IP,
 output wire ScanEn_OP,
 output wire ScanEn_CP,
 input  wire Gater_Flop_ScanIn,
 output wire Gater_Flop_ScanOut,
 
 input  wire LOS_Sel,   //Unused
 input  wire SE_Sel,
 input  wire Scan_Mode,
 input  wire Gater_Table_Mode,
 input  wire Stuck_At_Mode,
 input  wire Extest_Mode,
 input  wire ScanEn_IP_Override,
 input  wire ScanEn_OP_Override,
 input  wire ScanEn_CP_Override,
 
 input  wire Ext_Clk_Stop,
 output wire Clk_Stop_Tdr,

 output wire  Dft_Clk_Gater_Aon,
 output wire  Dft_Clk_Gater_En,
 

 input  wire Tile_Shift_Clk,
 input  wire Func_Clk,
 output  wire Out_Clk
 );

   //Mode_Sel
   wire  mode_sel_default;
   wire  mode_sel_shift_clk;
   wire  mode_sel_ip_clk;
   wire  mode_sel_ssb_inv;
   wire  mode_gater_open;
   wire  mode_sel_ip_clk_gater_always_open;
   wire  mode_sel_shift_ip_clk_gater_always_open;
   wire  mode_sel_gater_always_open;
   wire  mode_sel_shift_ip_clk_gater_always_open_close;
   wire  mode_3, mode_4, mode_6, mode_7;

   wire  Func_Clk_Gated;
   
   //TDR
   wire [DFT_CLK_CNTL_PULSE_CNT-1:0] pulse_cnt;
   wire [DFT_CLK_CNTL_LATENCY_CNT-1:0] latency_cnt;
   wire [1-1:0] 		       clk_stop;
   wire [1-1:0] 		       ext_clk_stop_en;
   wire [3-1:0] 		       mode_sel;
   
   wire 			       load_pul_cnt;
   wire [DFT_CLK_CNTL_PULSE_CNT-1:0]   gater_table_ff;
   wire 			       pulse_cnt_en;
   wire 			       clk_enable;
   wire 			       Stuck_At_Mode_Cg_En, At_Speed_Mode_Cg_En;
   wire 			       clk_stop_ored;
   wire 			       ext_clk_stop_anded;
   wire 			       mux_sel_pre;
   wire 			       mux_sel;
   wire 			       se_cp_override;
   wire 			       se_ip_override;
   wire 			       se_op_override;
// wire                    trstb_sync_func_clk;
   wire                    scan_mode_sync_func_clk;
   wire                    pulse_cnt_muxed_ored;

   
   ///////////////////////////////////////////////////////////////////
   pulse_cnt_tdr_gc10 
# (
.DFT_CLK_CNTL_PULSE_CNT (DFT_CLK_CNTL_PULSE_CNT)
  ) 
 pulse_cnt_tdr 
(	
	.Tdr_Tck               (Tdr_Tck) ,
	.Tdr_Trstb             (Tdr_Trstb),
	.Tdr_Tdi               (Tdr_pulse_cnt_Tdi),
	.Tdr_Shift             (Tdr_Shift),
	.Tdr_Capture           (Tdr_Capture),
	.Tdr_Update            (Tdr_Update)
	
	
	//TDR --> PULSE_CNT
	,.pulse_cnt                    (pulse_cnt[DFT_CLK_CNTL_PULSE_CNT-1:0])
	,.Tdr_pulse_cnt_En             (Tdr_pulse_cnt_En)
	,.Tdr_pulse_cnt_Tdo            (Tdr_pulse_cnt_Tdo)
	);
   ///////////////////////////////////////////////////////////////////
  latency_cnt_tdr_gc10 
# (
.DFT_CLK_CNTL_LATENCY_CNT (DFT_CLK_CNTL_LATENCY_CNT)
  ) 
latency_cnt_tdr 
(
 
 .Tdr_Tck               (Tdr_Tck) ,
 .Tdr_Trstb             (Tdr_Trstb),
 .Tdr_Tdi               (Tdr_latency_cnt_Tdi),
 .Tdr_Shift             (Tdr_Shift),
 .Tdr_Capture           (Tdr_Capture),
 .Tdr_Update            (Tdr_Update)
 

 //TDR --> LATENCY_CNT
 ,.latency_cnt                  (latency_cnt[DFT_CLK_CNTL_LATENCY_CNT-1:0])
 ,.Tdr_latency_cnt_En           (Tdr_latency_cnt_En)
 ,.Tdr_latency_cnt_Tdo          (Tdr_latency_cnt_Tdo)
 );
   
   
   ///////////////////////////////////////////////////////////////////
  clk_cntl_tdr_gc10 clk_cntl_tdr 
( 
  
  .Tdr_Tck               (Tdr_Tck) ,
  .Tdr_Trstb             (Tdr_Trstb),
  .Tdr_Tdi               (Tdr_clk_cntl_Tdi),
  .Tdr_Shift             (Tdr_Shift),
  .Tdr_Capture           (Tdr_Capture),
  .Tdr_Update            (Tdr_Update)
  
  
  //TDR --> CLK_CNTL
  ,.clk_stop              (clk_stop[1-1:0])
  ,.ext_clk_stop_en               (ext_clk_stop_en[1-1:0])
  ,.mode_sel              (mode_sel[3-1:0])
  ,.Tdr_clk_cntl_En              (Tdr_clk_cntl_En)
  ,.Tdr_clk_cntl_Tdo             (Tdr_clk_cntl_Tdo)
  ); 
   
   ///////////////////////////////////////////////////////////////////
     
   dfx_async_rst_sync3_gc10 rst_sync_trstb 
    (
     .clk      (Func_Clk),
     .rstinB   (Tdr_Trstb),
     .rstoutB  (trstb_sync_func_clk)
     );

// mission mode- Tdr_Trstb=0, gate Func_clk always
// ATE mode- Tdr_Tdrb=1 always befroe scan_mode become-1
//           Hence, clock ungated when scan_mode=0. this would reset synchronous logic properly.

   ati_clock_gate Func_Clk_DfxCLOCKGATER
     (.clk_out   (Func_Clk_Gated),
      .clk_src   (Func_Clk),
      .clk_en    (trstb_sync_func_clk),
      .TE        (1'b0));
   

   dfx_async_rst_sync3_gc10 rst_sync_scan_mode 
    (
     .clk      (Func_Clk_Gated),
     .rstinB   (Scan_Mode),
     .rstoutB  (scan_mode_sync_func_clk)
     );
   
   

latency_cnt_gc10 
# (
.DFT_CLK_CNTL_LATENCY_CNT (DFT_CLK_CNTL_LATENCY_CNT)
) lat_cnt 
(
 .scan_mode_sync_func_clk  (scan_mode_sync_func_clk)
 ,.SE_Sel              (SE_Sel)
 ,.Scan_Mode           (Scan_Mode)
 ,.Func_Clk            (Func_Clk_Gated)
 ,.Tile_Shift_Clk      (Tile_Shift_Clk)
 ,.Tile_ScanEn         (Tile_ScanEn)
 ,.latency_cnt         (latency_cnt[DFT_CLK_CNTL_LATENCY_CNT-1:0])
 ,.scan_en_ff_muxed    (scan_en_ff_muxed)
 ,.load_pul_cnt        (load_pul_cnt)
 );
   
   
   
gater_table_gc10 
# (
.DFT_CLK_CNTL_PULSE_CNT (DFT_CLK_CNTL_PULSE_CNT)
) gater_table 
(
 .Scan_Mode            (Scan_Mode)
 ,.Tile_ScanEn         (Tile_ScanEn)
 ,.Tile_Shift_Clk      (Tile_Shift_Clk)
 ,.Gater_Flop_ScanIn   (Gater_Flop_ScanIn)
 ,.Gater_Flop_ScanOut  (Gater_Flop_ScanOut)
 ,.gater_table_ff      (gater_table_ff[DFT_CLK_CNTL_PULSE_CNT-1:0])
 );

dfx_mrk_and_gc10 mrk_ext_clk_stop 
  ( 
    .A(ext_clk_stop_en), 
    .B(Ext_Clk_Stop),
    .Y(ext_clk_stop_anded)
    );

   assign clk_stop_ored = clk_stop | ext_clk_stop_anded;
   assign Clk_Stop_Tdr = clk_stop_ored;
   
dft_pulse_cnt_gc10 
# (
.DFT_CLK_CNTL_PULSE_CNT (DFT_CLK_CNTL_PULSE_CNT)
) dft_pulse_cnt 
  (
   .scan_mode_sync_func_clk  (scan_mode_sync_func_clk)
   ,.Func_Clk            (Func_Clk_Gated)
   ,.Gater_Table_Mode    (Gater_Table_Mode)
   ,.Stuck_At_Mode       (Stuck_At_Mode)
   ,.Scan_Mode           (Scan_Mode) 
   ,.clk_stop            (clk_stop_ored)
   ,.load_pul_cnt        (load_pul_cnt)
   ,.pulse_cnt           (pulse_cnt[DFT_CLK_CNTL_PULSE_CNT-1:0])
   ,.mode_gater_open     (mode_gater_open)
   ,.mode_sel_gater_always_open     (mode_sel_gater_always_open)
   ,.gater_table_ff      (gater_table_ff[DFT_CLK_CNTL_PULSE_CNT-1:0])
   ,.pulse_cnt_muxed_ored (pulse_cnt_muxed_ored)
   ,.Stuck_At_Mode_Cg_En (Stuck_At_Mode_Cg_En)
   ,.At_Speed_Mode_Cg_En (At_Speed_Mode_Cg_En)
   ,.clk_enable (clk_enable)
   ,.pulse_cnt_en (pulse_cnt_en)
   );
   
   ///////////////////////////////////////////////////////////////////

   ati_buffer1 mrk_d0nt_mode_sel_default   (.A( (mode_sel[2:0] == 3'b000) ), .Y(mode_sel_default));    //MODE_0
   ati_buffer1 mrk_d0nt_mode_sel_shift_clk (.A( (mode_sel[2:0] == 3'b001) ), .Y(mode_sel_shift_clk));  //MODE_1
   ati_buffer1 mrk_d0nt_mode_sel_ip_clk    (.A( (mode_sel[2:0] == 3'b010) ), .Y(mode_sel_ip_clk));     //MODE_2
   ati_buffer1 mrk_d0nt_mode_sel_3         (.A( (mode_sel[2:0] == 3'b011) ), .Y(mode_3));        //MODE_3
   ati_buffer1 mrk_d0nt_mode_sel_4         (.A( (mode_sel[2:0] == 3'b100) ), .Y(mode_4));    //MODE_4
   ati_buffer1 mrk_d0nt_mode_sel_ip_clk_gater_always_open    (.A( (mode_sel[2:0] == 3'b101) ), .Y(mode_sel_ip_clk_gater_always_open)); //MODE_5
   ati_buffer1 mrk_d0nt_mode_sel_shift_ip_clk_gater_always_open    (.A( (mode_sel[2:0] == 3'b110) ), .Y(mode_6)); //MODE_6
   ati_buffer1 mrk_d0nt_mode_sel_shift_ip_clk_gater_always_open_close    (.A( (mode_sel[2:0] == 3'b111) ), .Y(mode_7)); //MODE_7
  
  // Gater open during capture mode after sync + latency_cnt saturate
   ati_buffer1 mrk_d0nt_mode_gater_open    (.A( mode_sel_ip_clk), .Y(mode_gater_open));
   

   assign mode_sel_shift_ip_clk_gater_always_open = mode_3 | mode_6;
   assign mode_sel_shift_ip_clk_gater_always_open_close = mode_4 | mode_7;
   assign mode_sel_ssb_inv = mode_3 | mode_4;


  // Gater always open regardless of SE, latency_cnt.  mode-5 |  mode-3,6 | mode-4,7&pulse_cnt_muxed_ored
   ati_buffer1 mrk_d0nt_mode_sel_gater_always_open    (.A( ( mode_sel_ip_clk_gater_always_open |
                                                             mode_sel_shift_ip_clk_gater_always_open |
                                                            (mode_sel_shift_ip_clk_gater_always_open_close & pulse_cnt_muxed_ored)
                                                             ) ), 
                                                    .Y(mode_sel_gater_always_open)); 

    // Extest mode: 
    //mode_sel_shift_ip_clk_gater_always_open_close & pulse_cnt_muxed_ored - become 1 if any bit of gater table (pulse_cnt TDR) is 1..
    //Gater table mode enable same TDR setting for both targated and non-targated clock domains.

   
   ///////////////////////////////////////////////////////////////////
   
   //Scan_Mode keep leg to 0 in functional mode.
   //When clk_mux_sel = 1 & pulse_cnt != {width{1'b1}}, mux_sel= 1, source Shift_Clk, regardless of ScanEn.
   //When pulse_cnt = {(width){1'b1}}, mux_sel = 0, source IP_Clk, regardless of ScanEn.
   assign mux_sel_pre = (Tile_ScanEn | mode_sel_shift_clk) & ~(mode_sel_ip_clk | mode_sel_ip_clk_gater_always_open);
   dfx_mrk_and_gc10  mrk_mux_sel (.A(Scan_Mode), .B(mux_sel_pre), .Y(mux_sel));
   
   
   // Needed OR gate to set SE=1 asynchronosuly
   // MCP for timing.
   assign se_cp_override = Tile_ScanEn | ScanEn_CP_Override;
   assign se_ip_override = Tile_ScanEn | ScanEn_IP_Override;
   assign se_op_override = Tile_ScanEn | ScanEn_OP_Override;

generate
if(DFT_CLK_CNTL_NO_SHIFT_CLK == 0 ) begin
   clone_for_mpcts_gc10 clone_for_mpcts 
     (
      .Func_Clk               (Func_Clk)
      ,.Func_Clk_Muxed        (Out_Clk)
      ,.Tile_Shift_Clk        (Tile_Shift_Clk)
      ,.Stuck_At_Mode_Cg_En   (Stuck_At_Mode_Cg_En)
      ,.At_Speed_Mode_Cg_En   (At_Speed_Mode_Cg_En)
      ,.mux_sel               (mux_sel)
      ,.SE_Sel                (SE_Sel)
      ,.mode_sel_ssb_inv      (mode_sel_ssb_inv)
      );
end else begin
   clone_dfx_cg_gc10 clone_dfx_cg 
    (
      .Func_Clk               (Func_Clk)
      ,.Func_Clk_Gated        (Out_Clk)
      ,.Dft_Clk_Gater_Aon     (Dft_Clk_Gater_Aon)
      ,.Dft_Clk_Gater_En      (Dft_Clk_Gater_En)
      ,.Func_Gater_En         (1'b1)
      ,.Tile_GaterEn          (Tile_GaterEn)
      ,.Edt_Update            (Edt_Update)
    );
end
endgenerate


clone_SE_gc10 clone_SE_CP
     (
      .Func_Clk    (Out_Clk)
      ,.ScanEn      (Tile_ScanEn)
      ,.SE_Sel     (SE_Sel)
      ,.ScanEn_Override    (se_cp_override)
      ,.ScanEn_Out          (ScanEn_CP)
      );

clone_SE_gc10 clone_SE_IP
  (
   .Func_Clk    (Out_Clk)
   ,.ScanEn      (Tile_ScanEn)
   ,.SE_Sel     (SE_Sel)
   ,.ScanEn_Override    (se_ip_override)
   ,.ScanEn_Out          (ScanEn_IP)
   );

clone_SE_gc10 clone_SE_OP
  (
   .Func_Clk    (Out_Clk)
   ,.ScanEn      (Tile_ScanEn)
   ,.SE_Sel     (SE_Sel)
   ,.ScanEn_Override    (se_op_override)
   ,.ScanEn_Out          (ScanEn_OP)
   );


   ///////////////////////////////////////////////////////////////////
   assign Tile_CapEn = ~scan_en_ff_muxed & ~load_pul_cnt & Scan_Mode;
   ///////////////////////////////////////////////////////////////////

  reg dft_gater_aon, dft_gater_en;

   always @ (posedge Func_Clk_Gated or negedge trstb_sync_func_clk)
     begin
        if (~trstb_sync_func_clk) begin
           dft_gater_aon <= 1'b0;   //default, mission mode 0   (mode_6, mode_5)
           dft_gater_en  <= 1'b1;    //default, mission mode 1 (mode0=1 (default), mode_2)
        end else if (~clk_enable) begin
           dft_gater_aon <= 1'b0;
           dft_gater_en  <= 1'b0;
        end else if (scan_mode_sync_func_clk) begin
           dft_gater_aon <= ((pulse_cnt_en & mode_6) | mode_sel_ip_clk_gater_always_open) ; //CDC waive
           //Keeping mode_0(default=1) here to keep the gater_en unchanged (1'b1) when scan_mode is set to 1.
           dft_gater_en  <= ((pulse_cnt_en & mode_sel_default) | mode_gater_open);    //CDC waive. 
        end else begin
           dft_gater_aon <= dft_gater_aon;
           dft_gater_en  <= dft_gater_en;
        end
     end

  //~scan_en_ff_muxed  keep signal 0 during scan shift to flush out pipeline flops with 0.
  // Added AND below to keep Aon/En to 0 async when SE=1 to disable the gater during edt_update. 
  // TBD: How to async reset the pipeline flop before CGCG gater for edt_update?
  assign Dft_Clk_Gater_Aon = dft_gater_aon & ~scan_en_ff_muxed; // CDC waive. SE 0->1  is async event.
  assign Dft_Clk_Gater_En  = ~(dft_gater_en  & ~scan_en_ff_muxed); // CDC waive
   
   ///////////////////////////////////////////////////////////////////

   `include "dft_clk_cntl_assertions.vh"
 
endmodule // dft_clk_cntl

