//                              -*- Mode: Verilog -*-

////////////////////////////////////////////////////////////////////////////
/*
 *
 *  Trade secret of ATI Technologies, Inc.
 *  Copyright 2005, ATI Technologies, Inc., (unpublished)
 *
 *  All rights reserved.  This notice is intended as a precaution against
 *  inadvertent publication and does not imply publication or any waiver
 *  of confidentiality.  The year included in the foregoing notice is the
 *  year of creation of the work.
 *
 */
////////////////////////////////////////////////////////////////////////////
//
// Filename    : dft_clk_obs.v
// Description : clock observe
//
////////////////////////////////////////////////////////////////////////////

// MAX allowed SEL_WIDTH = 3.
module dft_clk_obs_gc10
# (
parameter SEL_WIDTH = 3,
parameter BYPASS_DIVIDER = 0
)
(
input wire Tdr_Tck,
input wire Tdr_Trstb,
input wire Tdr_Update,
input wire Tdr_clk_obs_En,

input wire Clk_0,
input wire Clk_1,
input wire Clk_2,
input wire Clk_3,
input wire Clk_4,
input wire Clk_5,
input wire Clk_6,
input wire Clk_7,
input wire [4:0] divider,
input wire byp_dfs,
input wire [2:0] sel_clk,
input wire  sel_daisy_clk,

input wire Clk_Daisy_In,
output wire Clk_Daisy_Out

);

localparam TOTAL_WIDTH = 2 ** SEL_WIDTH;    //8(3) 4(2) 2(1)

wire    [7:0] max_clk_observe_in;
wire    [TOTAL_WIDTH-1:0] clk_observe_in;   //8, 4, 2 

wire    [SEL_WIDTH-1:0] clk_observe_sel;    //3,2,1
wire clk_out_muxed, clk_out_muxed_pre;


///////////////////////////////////////////////////////////////////
assign clk_observe_sel[SEL_WIDTH-1:0] = sel_clk[SEL_WIDTH-1:0]; // 2:0, 1:0, 0:0
assign max_clk_observe_in[7:0] = 
{
Clk_7,
Clk_6,
Clk_5,
Clk_4,
Clk_3,
Clk_2,
Clk_1,
Clk_0
};

//8,4,2
assign clk_observe_in[TOTAL_WIDTH-1:0] = max_clk_observe_in[TOTAL_WIDTH-1:0];

    // 3 clkmux in each clock path.
    dfx_clkmux_gc10 #(SEL_WIDTH) mux_clkobs (
         .clk_i({
                clk_observe_in[TOTAL_WIDTH-1:0]
                 }),      
        .sel(clk_observe_sel[SEL_WIDTH-1:0] ),
       .clk_o(clk_out_muxed_pre));

    aticlk_mux2 mux_clkobs_daisy (
         .clk_i({
                Clk_Daisy_In,
                clk_out_muxed_pre
                 }),      
        .sel(sel_daisy_clk ),
       .clk_o(clk_out_muxed));

///////////////////////////////////////////////////////////////////

   
generate
 if(BYPASS_DIVIDER == 1 ) begin
  assign Clk_Daisy_Out = clk_out_muxed;
 end
 else begin
    
    
    //TDR-CDC
    wire [4:0] divider_cdc;
    wire       byp_dfs_cdc;
    reg        div_en;
    wire       div_en_sync;
    always @(posedge Tdr_Tck)
      begin
	 if(~Tdr_Trstb) 
	   begin
	      div_en <= 1'b0;
	   end
	 else if(Tdr_clk_obs_En & Tdr_Update) //Enable after clk_mux select the clock
	   begin
	      div_en <= 1'b1;
	   end
	 else 
	   begin
	      div_en <= div_en;    // Once Instruction is set, divider is enabled. Only way to disable it is to assert Trstb.
	   end
      end // always @ (posedge Tdr_Tck)
   
   
   dfx_sync3_gc10 #(
      .TransportDelay(0),
      .TransportCycle(1)
      ) 
   sync_div_en
     (
      .CK(clk_out_muxed),
      .D(div_en),
      .Q(div_en_sync));
   
   
   tdr_cdc_gc10 #(.width(6)) tdr_cdc 
     (
      .clk        (Tdr_Tck),
      .rstb       (Tdr_Trstb),
      .update     (Tdr_Update),
      .en         (Tdr_clk_obs_En),
      .d_rst      (6'b0), // Setting default to 0.
      .q          ({byp_dfs,divider[4:0]}),
      
      .dst_clk    (clk_out_muxed),
      .dst_rstb   (div_en_sync),
      .dst_q      ({byp_dfs_cdc,divider_cdc[4:0]})
      );
   
   
   clk_divide_gc10  #(.width(5)) clk_divide 
     (
      .clk_in     (clk_out_muxed),
      .clk_out    (Clk_Daisy_Out),
      .en         (div_en_sync),
      .byp_dfs    (byp_dfs_cdc),
      .clk_divider(divider_cdc[4:0])
      );
   
   
   
end
endgenerate
   
   

endmodule
