////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////
//
// Filename    : dft_iso_override.v
// Description : Override logic for isolation cell control in always-on blocks.
//
////////////////////////////////////////////////////////////////////////////
module dft_iso_override (
   // Outputs
   dft_clamp, dft_ana_clamp, dft_vddc_clamp, override_active,
   ssf_pfh_tclk,
   // Inputs
   resetb, clock, clamp_select, clamp_override, iso_dis, ana_iso_dis,
   vddc_iso_dis, TST_TARG_miscpfh_tclk, TST_TARG_miscpfh_tclk_func
   );
input resetb; 		//Asynchronous reset driven by non-isolated signal (active low).
input clock; 		//Test clock used to set sticky flop.
input clamp_select; 	//Indicates the clamp control is being driven by the test controller.
input clamp_override; 	//Indicates isolation should be forced on.
input iso_dis; 		//Functional isolation control.
input ana_iso_dis;     	//Functional isolation control for analog macros.
input vddc_iso_dis;     //Functional isolation control for some VDDC's analog macros.
input TST_TARG_miscpfh_tclk;       //DFT isolated p1500 tclk.  This is used during functional and ATPG modes.
input TST_TARG_miscpfh_tclk_func;  //Functionally isolated p1500 tclk.  This is only used during iso observe mode.
output dft_clamp; 	//Isolation control to be used for all DFT inputs to the bf_t.
output dft_ana_clamp; 	//Isolation control to be used for all analog macro isolation control.
output dft_vddc_clamp; 	//Isolation control to be used for VDDC analog macro isolation control.
output override_active; //Force isolation off on functional signals when dft signal isolation override is active.
output ssf_pfh_tclk;    //Muxed p1500 tclk to be used by the ssf_tst block.
wire clamp_sticky;
wire dft_clamp_mux_select;
wire dft_clamp_pre;
wire clamp_sticky_pre;
wire NO_CONNECT;
wire NO_CONNECT_2;
wire NO_CONNECT_3;
wire muxed_pfh_tclk;
wire ana_clamp;
wire clamp_select_reg;
wire clamp_override_reg;
//Adding structural instance to avoid async reset flops in tile synthesis.
//always @(posedge clock or negedge resetb) begin
//  if (~resetb)
//    clamp_sticky <= 1'b0;
//  else
//    clamp_sticky <= clamp_sticky | (clamp_select & clamp_override);
//end
//assign clamp_sticky_pre = clamp_sticky | (clamp_select_reg & clamp_override_reg);
assign clamp_sticky_pre = clamp_sticky | (clamp_select & clamp_override);
ati_async_ff uclamp_sticky_reg (.D(clamp_sticky_pre), .CP(clock), .CDN(resetb), .Q(clamp_sticky), .QN(NO_CONNECT));
//Adding flops on select and override signals to break timing loops
ati_async_ff uclamp_select_reg (.D(clamp_select), .CP(clock), .CDN(resetb), .Q(clamp_select_reg), .QN(NO_CONNECT_2));
ati_async_ff uclamp_override_reg (.D(clamp_override), .CP(clock), .CDN(resetb), .Q(clamp_override_reg), .QN(NO_CONNECT_3));
//assign dft_clamp_mux_select = clamp_sticky | clamp_select_reg;
assign dft_clamp_mux_select = clamp_sticky | clamp_select;
assign dft_clamp_pre = ~resetb | (dft_clamp_mux_select ? ~clamp_sticky : iso_dis);
//assign ana_clamp = ana_iso_dis | clamp_select_reg;
assign ana_clamp = ana_iso_dis | clamp_select;
assign vddc_clamp = vddc_iso_dis | clamp_select;
ati_buffer1 mrk_dft_clamp (.A(dft_clamp_pre), .Y(dft_clamp));
ati_buffer1 mrk_dft_ana_clamp (.A(ana_clamp), .Y(dft_ana_clamp));
ati_buffer1 mrk_dft_vddc_clamp (.A(vddc_clamp), .Y(dft_vddc_clamp));
ati_buffer1 mrk_override_active (.A(clamp_sticky), .Y(override_active));
aticlk_mux2 mrk_pfh_tclk_mux (.clk_o(muxed_pfh_tclk),
				.clk_i({TST_TARG_miscpfh_tclk_func, TST_TARG_miscpfh_tclk}),
				.sel(clamp_sticky));
aticlk_buf mrk_ssf_pfh_tclk (.A(muxed_pfh_tclk), .Y(ssf_pfh_tclk));
endmodule
