//                              -*- Mode: Verilog -*-

////////////////////////////////////////////////////////////////////////////
/*
 *
 *  Trade secret of ATI Technologies, Inc.
 *  Copyright 2005, ATI Technologies, Inc., (unpublished)
 *
 *  All rights reserved.  This notice is intended as a precaution against
 *  inadvertent publication and does not imply publication or any waiver
 *  of confidentiality.  The year included in the foregoing notice is the
 *  year of creation of the work.
 *
 */
////////////////////////////////////////////////////////////////////////////
//
// Filename    : dft_lanes_clk_cntl.v
// Description : DFT clock controller for lane clocks.
//               Use this for uncontrolable clock which require scan capture clock from controllable clock domain
//               The uncontrollable clock must be stopped outside this block before scan_mode is set
//               
////////////////////////////////////////////////////////////////////////////

//`include "dft_clk_cntl_features.vh"

//;use comp_config;




module dft_lanes_clk_cntl_gc10
#(
  parameter DFT_CLK_CNTL_PULSE_CNT = 8,
  parameter DFT_CLK_CNTL_LATENCY_CNT = 10,
  parameter DFT_CLK_CNTL_NO_SHIFT_CLK = 0,
  parameter WITH_CLK_STOP_GATER = 19'b0
  )
(
 
 
 //Broadcast to All TDRs
 input wire Tdr_Tck,
 input wire Tdr_Trstb,
 input wire Tdr_Shift,
 input wire Tdr_Capture,
 input wire Tdr_Update,
 
 //From Tdr_Broadcast
 input wire Tdr_clk_cntl_Tdi,
 input wire Tdr_latency_cnt_Tdi,
 input wire Tdr_pulse_cnt_Tdi,
 
 //Tdr En/So
 input  wire Tdr_clk_cntl_En,
 output wire Tdr_clk_cntl_Tdo,
 
 input  wire Tdr_latency_cnt_En,
 output wire Tdr_latency_cnt_Tdo,
 
 input  wire Tdr_pulse_cnt_En,
 output wire Tdr_pulse_cnt_Tdo,
 
 input  wire Tile_ScanEn,  // flopped scanEn from scan_cntl
 output  wire Tile_CapEn, 
 input  wire Edt_Update, 
 input  wire Tile_GaterEn, 
// output wire ScanEn_IP,
// output wire ScanEn_OP,
// output wire ScanEn_CP,
 input  wire Gater_Flop_ScanIn,
 output wire Gater_Flop_ScanOut,
 
 input  wire LOS_Sel,
 input  wire SE_Sel,
 input  wire Scan_Mode,
 input  wire Gater_Table_Mode,
 input  wire Stuck_At_Mode,
 input  wire Extest_Mode,
 input  wire ScanEn_IP_Override,
 input  wire ScanEn_OP_Override,
 input  wire ScanEn_CP_Override,
 
 input  wire Ext_Clk_Stop,

 output wire Dft_Clk_Gater_Aon,
 output wire Dft_Clk_Gater_En,

 
 input  wire Lane_Clk_0,
 output wire Out_Clk_0,
 output wire ScanEn_IP_0,
 output wire ScanEn_OP_0,
 output wire ScanEn_CP_0,
 input  wire Lane_Clk_1,
 output wire Out_Clk_1,
 output wire ScanEn_IP_1,
 output wire ScanEn_OP_1,
 output wire ScanEn_CP_1,
 input  wire Lane_Clk_2,
 output wire Out_Clk_2,
 output wire ScanEn_IP_2,
 output wire ScanEn_OP_2,
 output wire ScanEn_CP_2,
 input  wire Lane_Clk_3,
 output wire Out_Clk_3,
 output wire ScanEn_IP_3,
 output wire ScanEn_OP_3,
 output wire ScanEn_CP_3,
 input  wire Lane_Clk_4,
 output wire Out_Clk_4,
 output wire ScanEn_IP_4,
 output wire ScanEn_OP_4,
 output wire ScanEn_CP_4,
 input  wire Lane_Clk_5,
 output wire Out_Clk_5,
 output wire ScanEn_IP_5,
 output wire ScanEn_OP_5,
 output wire ScanEn_CP_5,
 input  wire Lane_Clk_6,
 output wire Out_Clk_6,
 output wire ScanEn_IP_6,
 output wire ScanEn_OP_6,
 output wire ScanEn_CP_6,
 input  wire Lane_Clk_7,
 output wire Out_Clk_7,
 output wire ScanEn_IP_7,
 output wire ScanEn_OP_7,
 output wire ScanEn_CP_7,
 input  wire Lane_Clk_8,
 output wire Out_Clk_8,
 output wire ScanEn_IP_8,
 output wire ScanEn_OP_8,
 output wire ScanEn_CP_8,
 input  wire Lane_Clk_9,
 output wire Out_Clk_9,
 output wire ScanEn_IP_9,
 output wire ScanEn_OP_9,
 output wire ScanEn_CP_9,
 input  wire Lane_Clk_10,
 output wire Out_Clk_10,
 output wire ScanEn_IP_10,
 output wire ScanEn_OP_10,
 output wire ScanEn_CP_10,
 input  wire Lane_Clk_11,
 output wire Out_Clk_11,
 output wire ScanEn_IP_11,
 output wire ScanEn_OP_11,
 output wire ScanEn_CP_11,
 input  wire Lane_Clk_12,
 output wire Out_Clk_12,
 output wire ScanEn_IP_12,
 output wire ScanEn_OP_12,
 output wire ScanEn_CP_12,
 input  wire Lane_Clk_13,
 output wire Out_Clk_13,
 output wire ScanEn_IP_13,
 output wire ScanEn_OP_13,
 output wire ScanEn_CP_13,
 input  wire Lane_Clk_14,
 output wire Out_Clk_14,
 output wire ScanEn_IP_14,
 output wire ScanEn_OP_14,
 output wire ScanEn_CP_14,
 input  wire Lane_Clk_15,
 output wire Out_Clk_15,
 output wire ScanEn_IP_15,
 output wire ScanEn_OP_15,
 output wire ScanEn_CP_15,
 input  wire Lane_Clk_16,
 output wire Out_Clk_16,
 output wire ScanEn_IP_16,
 output wire ScanEn_OP_16,
 output wire ScanEn_CP_16,
 input  wire Lane_Clk_17,
 output wire Out_Clk_17,
 output wire ScanEn_IP_17,
 output wire ScanEn_OP_17,
 output wire ScanEn_CP_17,
 input  wire Lane_Clk_18,
 output wire Out_Clk_18,
 output wire ScanEn_IP_18,
 output wire ScanEn_OP_18,
 output wire ScanEn_CP_18,

 input  wire Tile_Shift_Clk,
 input  wire Func_Clk

 );

wire Out_Clk;
wire ScanEn_IP;
wire ScanEn_OP;
wire ScanEn_CP;
wire [19-1:0]   gater_table_ff;
wire Gater_Flop_ScanIn_Lanes_Clk;


  dft_clk_cntl_gc10 
# (
.DFT_CLK_CNTL_PULSE_CNT (DFT_CLK_CNTL_PULSE_CNT),
.DFT_CLK_CNTL_LATENCY_CNT (DFT_CLK_CNTL_LATENCY_CNT),
.DFT_CLK_CNTL_NO_SHIFT_CLK (DFT_CLK_CNTL_NO_SHIFT_CLK)
  ) 
dft_clk_cntl 
(
 
 .Tdr_Tck               (Tdr_Tck) ,
 .Tdr_Trstb             (Tdr_Trstb),
 .Tdr_Shift             (Tdr_Shift),
 .Tdr_Capture           (Tdr_Capture),
 .Tdr_Update            (Tdr_Update),
 .Func_Clk(Func_Clk)
 ,.Out_Clk(Out_Clk)
 ,.ScanEn_IP(ScanEn_IP)
 ,.ScanEn_OP(ScanEn_OP)
 ,.ScanEn_CP(ScanEn_CP)
 ,.Ext_Clk_Stop(Ext_Clk_Stop)
 ,.Clk_Stop_Tdr(Clk_Stop_Tdr)
 ,.Dft_Clk_Gater_Aon(Dft_Clk_Gater_Aon)
 ,.Dft_Clk_Gater_En(Dft_Clk_Gater_En)
 ,.Gater_Flop_ScanIn(Gater_Flop_ScanIn)
 ,.Tdr_clk_cntl_Tdi(Tdr_clk_cntl_Tdi)
 ,.Tdr_latency_cnt_Tdi(Tdr_latency_cnt_Tdi)
 ,.Tdr_pulse_cnt_Tdi(Tdr_pulse_cnt_Tdi)
 ,.Tdr_clk_cntl_En(Tdr_clk_cntl_En)
 ,.Tdr_clk_cntl_Tdo(Tdr_clk_cntl_Tdo)
 ,.Tdr_latency_cnt_En(Tdr_latency_cnt_En)
 ,.Tdr_latency_cnt_Tdo(Tdr_latency_cnt_Tdo)
 ,.Tdr_pulse_cnt_En(Tdr_pulse_cnt_En)
 ,.Tdr_pulse_cnt_Tdo(Tdr_pulse_cnt_Tdo)
 ,.Tile_ScanEn(Tile_ScanEn)
 ,.Tile_CapEn(Tile_CapEn)
 ,.Edt_Update(Edt_Update)
 ,.Tile_GaterEn(Tile_GaterEn)
 ,.Gater_Flop_ScanOut(Gater_Flop_ScanIn_Lanes_Clk)
 ,.LOS_Sel(LOS_Sel)
 ,.SE_Sel(SE_Sel)
 ,.Scan_Mode(Scan_Mode)
 ,.Gater_Table_Mode(Gater_Table_Mode)
 ,.Stuck_At_Mode(Stuck_At_Mode)
 ,.Extest_Mode(Extest_Mode)
 ,.ScanEn_CP_Override(ScanEn_CP_Override)
 ,.ScanEn_IP_Override(ScanEn_IP_Override)
 ,.ScanEn_OP_Override(ScanEn_OP_Override)
 ,.Tile_Shift_Clk(Tile_Shift_Clk)

 );

gater_table_gc10 
# (
.DFT_CLK_CNTL_PULSE_CNT (19)
) gater_table_lanes_clk
(
 .Scan_Mode            (Scan_Mode)
 ,.Tile_ScanEn         (Tile_ScanEn)
 ,.Tile_Shift_Clk      (Tile_Shift_Clk)
 ,.Gater_Flop_ScanIn   (Gater_Flop_ScanIn_Lanes_Clk)
 ,.Gater_Flop_ScanOut  (Gater_Flop_ScanOut)
 ,.gater_table_ff      (gater_table_ff[19-1:0])
 );

   
   
   ///////////////////////////////////////////////////////////////////

   wire gater_en_0;
   wire Lane_Clk_Gated_0;
   assign gater_en_0  = Gater_Table_Mode ? (gater_table_ff[0] | Tile_ScanEn) : 1'b1 ;

   ati_clock_gate Out_Clk_DfxCLOCKGATER_0
     (.clk_out   (Out_Clk_Gated_0),
      .clk_src   (Out_Clk),
      .clk_en    (gater_en_0),
      .TE        (1'b0));

generate
if(WITH_CLK_STOP_GATER[0] == 1 ) begin

//Active low set
   dfx_async_set_sync3_gc10 rst_sync_clk_enable_0 
     (
      .clk(Lane_Clk_0),
      .rstinB(Clk_Stop_Tdr),           //mission mode 0.
      .rstout(clk_enable_0)
      ); 

   ati_clock_gate Lane_Clk_DfxCLOCKGATER_0
     (.clk_out   (Lane_Clk_Gated_0),
      .clk_src   (Lane_Clk_0),
      .clk_en    (clk_enable_0),
      .TE        (1'b0));


end else begin
 assign Lane_Clk_Gated_0 = Lane_Clk_0;

end
endgenerate
   


   aticlk_mux2 mrk_scan_clkmux_0 ( .clk_o(Out_Clk_0),
                                  .clk_i({Out_Clk_Gated_0, Lane_Clk_Gated_0}),
                                  .sel(Scan_Mode) );

   ati_buffer1 mrk_d0nt_ScanEn_IP_0 (.A(ScanEn_IP), .Y(ScanEn_IP_0)); 
   ati_buffer1 mrk_d0nt_ScanEn_OP_0 (.A(ScanEn_OP), .Y(ScanEn_OP_0)); 
   ati_buffer1 mrk_d0nt_ScanEn_CP_0 (.A(ScanEn_CP), .Y(ScanEn_CP_0)); 


   wire gater_en_1;
   wire Lane_Clk_Gated_1;
   assign gater_en_1  = Gater_Table_Mode ? (gater_table_ff[1] | Tile_ScanEn) : 1'b1 ;

   ati_clock_gate Out_Clk_DfxCLOCKGATER_1
     (.clk_out   (Out_Clk_Gated_1),
      .clk_src   (Out_Clk),
      .clk_en    (gater_en_1),
      .TE        (1'b0));

generate
if(WITH_CLK_STOP_GATER[1] == 1 ) begin

//Active low set
   dfx_async_set_sync3_gc10 rst_sync_clk_enable_1 
     (
      .clk(Lane_Clk_1),
      .rstinB(Clk_Stop_Tdr),           //mission mode 0.
      .rstout(clk_enable_1)
      ); 

   ati_clock_gate Lane_Clk_DfxCLOCKGATER_1
     (.clk_out   (Lane_Clk_Gated_1),
      .clk_src   (Lane_Clk_1),
      .clk_en    (clk_enable_1),
      .TE        (1'b0));


end else begin
 assign Lane_Clk_Gated_1 = Lane_Clk_1;

end
endgenerate
   


   aticlk_mux2 mrk_scan_clkmux_1 ( .clk_o(Out_Clk_1),
                                  .clk_i({Out_Clk_Gated_1, Lane_Clk_Gated_1}),
                                  .sel(Scan_Mode) );

   ati_buffer1 mrk_d0nt_ScanEn_IP_1 (.A(ScanEn_IP), .Y(ScanEn_IP_1)); 
   ati_buffer1 mrk_d0nt_ScanEn_OP_1 (.A(ScanEn_OP), .Y(ScanEn_OP_1)); 
   ati_buffer1 mrk_d0nt_ScanEn_CP_1 (.A(ScanEn_CP), .Y(ScanEn_CP_1)); 


   wire gater_en_2;
   wire Lane_Clk_Gated_2;
   assign gater_en_2  = Gater_Table_Mode ? (gater_table_ff[2] | Tile_ScanEn) : 1'b1 ;

   ati_clock_gate Out_Clk_DfxCLOCKGATER_2
     (.clk_out   (Out_Clk_Gated_2),
      .clk_src   (Out_Clk),
      .clk_en    (gater_en_2),
      .TE        (1'b0));

generate
if(WITH_CLK_STOP_GATER[2] == 1 ) begin

//Active low set
   dfx_async_set_sync3_gc10 rst_sync_clk_enable_2 
     (
      .clk(Lane_Clk_2),
      .rstinB(Clk_Stop_Tdr),           //mission mode 0.
      .rstout(clk_enable_2)
      ); 

   ati_clock_gate Lane_Clk_DfxCLOCKGATER_2
     (.clk_out   (Lane_Clk_Gated_2),
      .clk_src   (Lane_Clk_2),
      .clk_en    (clk_enable_2),
      .TE        (1'b0));


end else begin
 assign Lane_Clk_Gated_2 = Lane_Clk_2;

end
endgenerate
   


   aticlk_mux2 mrk_scan_clkmux_2 ( .clk_o(Out_Clk_2),
                                  .clk_i({Out_Clk_Gated_2, Lane_Clk_Gated_2}),
                                  .sel(Scan_Mode) );

   ati_buffer1 mrk_d0nt_ScanEn_IP_2 (.A(ScanEn_IP), .Y(ScanEn_IP_2)); 
   ati_buffer1 mrk_d0nt_ScanEn_OP_2 (.A(ScanEn_OP), .Y(ScanEn_OP_2)); 
   ati_buffer1 mrk_d0nt_ScanEn_CP_2 (.A(ScanEn_CP), .Y(ScanEn_CP_2)); 


   wire gater_en_3;
   wire Lane_Clk_Gated_3;
   assign gater_en_3  = Gater_Table_Mode ? (gater_table_ff[3] | Tile_ScanEn) : 1'b1 ;

   ati_clock_gate Out_Clk_DfxCLOCKGATER_3
     (.clk_out   (Out_Clk_Gated_3),
      .clk_src   (Out_Clk),
      .clk_en    (gater_en_3),
      .TE        (1'b0));

generate
if(WITH_CLK_STOP_GATER[3] == 1 ) begin

//Active low set
   dfx_async_set_sync3_gc10 rst_sync_clk_enable_3 
     (
      .clk(Lane_Clk_3),
      .rstinB(Clk_Stop_Tdr),           //mission mode 0.
      .rstout(clk_enable_3)
      ); 

   ati_clock_gate Lane_Clk_DfxCLOCKGATER_3
     (.clk_out   (Lane_Clk_Gated_3),
      .clk_src   (Lane_Clk_3),
      .clk_en    (clk_enable_3),
      .TE        (1'b0));


end else begin
 assign Lane_Clk_Gated_3 = Lane_Clk_3;

end
endgenerate
   


   aticlk_mux2 mrk_scan_clkmux_3 ( .clk_o(Out_Clk_3),
                                  .clk_i({Out_Clk_Gated_3, Lane_Clk_Gated_3}),
                                  .sel(Scan_Mode) );

   ati_buffer1 mrk_d0nt_ScanEn_IP_3 (.A(ScanEn_IP), .Y(ScanEn_IP_3)); 
   ati_buffer1 mrk_d0nt_ScanEn_OP_3 (.A(ScanEn_OP), .Y(ScanEn_OP_3)); 
   ati_buffer1 mrk_d0nt_ScanEn_CP_3 (.A(ScanEn_CP), .Y(ScanEn_CP_3)); 


   wire gater_en_4;
   wire Lane_Clk_Gated_4;
   assign gater_en_4  = Gater_Table_Mode ? (gater_table_ff[4] | Tile_ScanEn) : 1'b1 ;

   ati_clock_gate Out_Clk_DfxCLOCKGATER_4
     (.clk_out   (Out_Clk_Gated_4),
      .clk_src   (Out_Clk),
      .clk_en    (gater_en_4),
      .TE        (1'b0));

generate
if(WITH_CLK_STOP_GATER[4] == 1 ) begin

//Active low set
   dfx_async_set_sync3_gc10 rst_sync_clk_enable_4 
     (
      .clk(Lane_Clk_4),
      .rstinB(Clk_Stop_Tdr),           //mission mode 0.
      .rstout(clk_enable_4)
      ); 

   ati_clock_gate Lane_Clk_DfxCLOCKGATER_4
     (.clk_out   (Lane_Clk_Gated_4),
      .clk_src   (Lane_Clk_4),
      .clk_en    (clk_enable_4),
      .TE        (1'b0));


end else begin
 assign Lane_Clk_Gated_4 = Lane_Clk_4;

end
endgenerate
   


   aticlk_mux2 mrk_scan_clkmux_4 ( .clk_o(Out_Clk_4),
                                  .clk_i({Out_Clk_Gated_4, Lane_Clk_Gated_4}),
                                  .sel(Scan_Mode) );

   ati_buffer1 mrk_d0nt_ScanEn_IP_4 (.A(ScanEn_IP), .Y(ScanEn_IP_4)); 
   ati_buffer1 mrk_d0nt_ScanEn_OP_4 (.A(ScanEn_OP), .Y(ScanEn_OP_4)); 
   ati_buffer1 mrk_d0nt_ScanEn_CP_4 (.A(ScanEn_CP), .Y(ScanEn_CP_4)); 


   wire gater_en_5;
   wire Lane_Clk_Gated_5;
   assign gater_en_5  = Gater_Table_Mode ? (gater_table_ff[5] | Tile_ScanEn) : 1'b1 ;

   ati_clock_gate Out_Clk_DfxCLOCKGATER_5
     (.clk_out   (Out_Clk_Gated_5),
      .clk_src   (Out_Clk),
      .clk_en    (gater_en_5),
      .TE        (1'b0));

generate
if(WITH_CLK_STOP_GATER[5] == 1 ) begin

//Active low set
   dfx_async_set_sync3_gc10 rst_sync_clk_enable_5 
     (
      .clk(Lane_Clk_5),
      .rstinB(Clk_Stop_Tdr),           //mission mode 0.
      .rstout(clk_enable_5)
      ); 

   ati_clock_gate Lane_Clk_DfxCLOCKGATER_5
     (.clk_out   (Lane_Clk_Gated_5),
      .clk_src   (Lane_Clk_5),
      .clk_en    (clk_enable_5),
      .TE        (1'b0));


end else begin
 assign Lane_Clk_Gated_5 = Lane_Clk_5;

end
endgenerate
   


   aticlk_mux2 mrk_scan_clkmux_5 ( .clk_o(Out_Clk_5),
                                  .clk_i({Out_Clk_Gated_5, Lane_Clk_Gated_5}),
                                  .sel(Scan_Mode) );

   ati_buffer1 mrk_d0nt_ScanEn_IP_5 (.A(ScanEn_IP), .Y(ScanEn_IP_5)); 
   ati_buffer1 mrk_d0nt_ScanEn_OP_5 (.A(ScanEn_OP), .Y(ScanEn_OP_5)); 
   ati_buffer1 mrk_d0nt_ScanEn_CP_5 (.A(ScanEn_CP), .Y(ScanEn_CP_5)); 


   wire gater_en_6;
   wire Lane_Clk_Gated_6;
   assign gater_en_6  = Gater_Table_Mode ? (gater_table_ff[6] | Tile_ScanEn) : 1'b1 ;

   ati_clock_gate Out_Clk_DfxCLOCKGATER_6
     (.clk_out   (Out_Clk_Gated_6),
      .clk_src   (Out_Clk),
      .clk_en    (gater_en_6),
      .TE        (1'b0));

generate
if(WITH_CLK_STOP_GATER[6] == 1 ) begin

//Active low set
   dfx_async_set_sync3_gc10 rst_sync_clk_enable_6 
     (
      .clk(Lane_Clk_6),
      .rstinB(Clk_Stop_Tdr),           //mission mode 0.
      .rstout(clk_enable_6)
      ); 

   ati_clock_gate Lane_Clk_DfxCLOCKGATER_6
     (.clk_out   (Lane_Clk_Gated_6),
      .clk_src   (Lane_Clk_6),
      .clk_en    (clk_enable_6),
      .TE        (1'b0));


end else begin
 assign Lane_Clk_Gated_6 = Lane_Clk_6;

end
endgenerate
   


   aticlk_mux2 mrk_scan_clkmux_6 ( .clk_o(Out_Clk_6),
                                  .clk_i({Out_Clk_Gated_6, Lane_Clk_Gated_6}),
                                  .sel(Scan_Mode) );

   ati_buffer1 mrk_d0nt_ScanEn_IP_6 (.A(ScanEn_IP), .Y(ScanEn_IP_6)); 
   ati_buffer1 mrk_d0nt_ScanEn_OP_6 (.A(ScanEn_OP), .Y(ScanEn_OP_6)); 
   ati_buffer1 mrk_d0nt_ScanEn_CP_6 (.A(ScanEn_CP), .Y(ScanEn_CP_6)); 


   wire gater_en_7;
   wire Lane_Clk_Gated_7;
   assign gater_en_7  = Gater_Table_Mode ? (gater_table_ff[7] | Tile_ScanEn) : 1'b1 ;

   ati_clock_gate Out_Clk_DfxCLOCKGATER_7
     (.clk_out   (Out_Clk_Gated_7),
      .clk_src   (Out_Clk),
      .clk_en    (gater_en_7),
      .TE        (1'b0));

generate
if(WITH_CLK_STOP_GATER[7] == 1 ) begin

//Active low set
   dfx_async_set_sync3_gc10 rst_sync_clk_enable_7 
     (
      .clk(Lane_Clk_7),
      .rstinB(Clk_Stop_Tdr),           //mission mode 0.
      .rstout(clk_enable_7)
      ); 

   ati_clock_gate Lane_Clk_DfxCLOCKGATER_7
     (.clk_out   (Lane_Clk_Gated_7),
      .clk_src   (Lane_Clk_7),
      .clk_en    (clk_enable_7),
      .TE        (1'b0));


end else begin
 assign Lane_Clk_Gated_7 = Lane_Clk_7;

end
endgenerate
   


   aticlk_mux2 mrk_scan_clkmux_7 ( .clk_o(Out_Clk_7),
                                  .clk_i({Out_Clk_Gated_7, Lane_Clk_Gated_7}),
                                  .sel(Scan_Mode) );

   ati_buffer1 mrk_d0nt_ScanEn_IP_7 (.A(ScanEn_IP), .Y(ScanEn_IP_7)); 
   ati_buffer1 mrk_d0nt_ScanEn_OP_7 (.A(ScanEn_OP), .Y(ScanEn_OP_7)); 
   ati_buffer1 mrk_d0nt_ScanEn_CP_7 (.A(ScanEn_CP), .Y(ScanEn_CP_7)); 


   wire gater_en_8;
   wire Lane_Clk_Gated_8;
   assign gater_en_8  = Gater_Table_Mode ? (gater_table_ff[8] | Tile_ScanEn) : 1'b1 ;

   ati_clock_gate Out_Clk_DfxCLOCKGATER_8
     (.clk_out   (Out_Clk_Gated_8),
      .clk_src   (Out_Clk),
      .clk_en    (gater_en_8),
      .TE        (1'b0));

generate
if(WITH_CLK_STOP_GATER[8] == 1 ) begin

//Active low set
   dfx_async_set_sync3_gc10 rst_sync_clk_enable_8 
     (
      .clk(Lane_Clk_8),
      .rstinB(Clk_Stop_Tdr),           //mission mode 0.
      .rstout(clk_enable_8)
      ); 

   ati_clock_gate Lane_Clk_DfxCLOCKGATER_8
     (.clk_out   (Lane_Clk_Gated_8),
      .clk_src   (Lane_Clk_8),
      .clk_en    (clk_enable_8),
      .TE        (1'b0));


end else begin
 assign Lane_Clk_Gated_8 = Lane_Clk_8;

end
endgenerate
   


   aticlk_mux2 mrk_scan_clkmux_8 ( .clk_o(Out_Clk_8),
                                  .clk_i({Out_Clk_Gated_8, Lane_Clk_Gated_8}),
                                  .sel(Scan_Mode) );

   ati_buffer1 mrk_d0nt_ScanEn_IP_8 (.A(ScanEn_IP), .Y(ScanEn_IP_8)); 
   ati_buffer1 mrk_d0nt_ScanEn_OP_8 (.A(ScanEn_OP), .Y(ScanEn_OP_8)); 
   ati_buffer1 mrk_d0nt_ScanEn_CP_8 (.A(ScanEn_CP), .Y(ScanEn_CP_8)); 


   wire gater_en_9;
   wire Lane_Clk_Gated_9;
   assign gater_en_9  = Gater_Table_Mode ? (gater_table_ff[9] | Tile_ScanEn) : 1'b1 ;

   ati_clock_gate Out_Clk_DfxCLOCKGATER_9
     (.clk_out   (Out_Clk_Gated_9),
      .clk_src   (Out_Clk),
      .clk_en    (gater_en_9),
      .TE        (1'b0));

generate
if(WITH_CLK_STOP_GATER[9] == 1 ) begin

//Active low set
   dfx_async_set_sync3_gc10 rst_sync_clk_enable_9 
     (
      .clk(Lane_Clk_9),
      .rstinB(Clk_Stop_Tdr),           //mission mode 0.
      .rstout(clk_enable_9)
      ); 

   ati_clock_gate Lane_Clk_DfxCLOCKGATER_9
     (.clk_out   (Lane_Clk_Gated_9),
      .clk_src   (Lane_Clk_9),
      .clk_en    (clk_enable_9),
      .TE        (1'b0));


end else begin
 assign Lane_Clk_Gated_9 = Lane_Clk_9;

end
endgenerate
   


   aticlk_mux2 mrk_scan_clkmux_9 ( .clk_o(Out_Clk_9),
                                  .clk_i({Out_Clk_Gated_9, Lane_Clk_Gated_9}),
                                  .sel(Scan_Mode) );

   ati_buffer1 mrk_d0nt_ScanEn_IP_9 (.A(ScanEn_IP), .Y(ScanEn_IP_9)); 
   ati_buffer1 mrk_d0nt_ScanEn_OP_9 (.A(ScanEn_OP), .Y(ScanEn_OP_9)); 
   ati_buffer1 mrk_d0nt_ScanEn_CP_9 (.A(ScanEn_CP), .Y(ScanEn_CP_9)); 


   wire gater_en_10;
   wire Lane_Clk_Gated_10;
   assign gater_en_10  = Gater_Table_Mode ? (gater_table_ff[10] | Tile_ScanEn) : 1'b1 ;

   ati_clock_gate Out_Clk_DfxCLOCKGATER_10
     (.clk_out   (Out_Clk_Gated_10),
      .clk_src   (Out_Clk),
      .clk_en    (gater_en_10),
      .TE        (1'b0));

generate
if(WITH_CLK_STOP_GATER[10] == 1 ) begin

//Active low set
   dfx_async_set_sync3_gc10 rst_sync_clk_enable_10 
     (
      .clk(Lane_Clk_10),
      .rstinB(Clk_Stop_Tdr),           //mission mode 0.
      .rstout(clk_enable_10)
      ); 

   ati_clock_gate Lane_Clk_DfxCLOCKGATER_10
     (.clk_out   (Lane_Clk_Gated_10),
      .clk_src   (Lane_Clk_10),
      .clk_en    (clk_enable_10),
      .TE        (1'b0));


end else begin
 assign Lane_Clk_Gated_10 = Lane_Clk_10;

end
endgenerate
   


   aticlk_mux2 mrk_scan_clkmux_10 ( .clk_o(Out_Clk_10),
                                  .clk_i({Out_Clk_Gated_10, Lane_Clk_Gated_10}),
                                  .sel(Scan_Mode) );

   ati_buffer1 mrk_d0nt_ScanEn_IP_10 (.A(ScanEn_IP), .Y(ScanEn_IP_10)); 
   ati_buffer1 mrk_d0nt_ScanEn_OP_10 (.A(ScanEn_OP), .Y(ScanEn_OP_10)); 
   ati_buffer1 mrk_d0nt_ScanEn_CP_10 (.A(ScanEn_CP), .Y(ScanEn_CP_10)); 


   wire gater_en_11;
   wire Lane_Clk_Gated_11;
   assign gater_en_11  = Gater_Table_Mode ? (gater_table_ff[11] | Tile_ScanEn) : 1'b1 ;

   ati_clock_gate Out_Clk_DfxCLOCKGATER_11
     (.clk_out   (Out_Clk_Gated_11),
      .clk_src   (Out_Clk),
      .clk_en    (gater_en_11),
      .TE        (1'b0));

generate
if(WITH_CLK_STOP_GATER[11] == 1 ) begin

//Active low set
   dfx_async_set_sync3_gc10 rst_sync_clk_enable_11 
     (
      .clk(Lane_Clk_11),
      .rstinB(Clk_Stop_Tdr),           //mission mode 0.
      .rstout(clk_enable_11)
      ); 

   ati_clock_gate Lane_Clk_DfxCLOCKGATER_11
     (.clk_out   (Lane_Clk_Gated_11),
      .clk_src   (Lane_Clk_11),
      .clk_en    (clk_enable_11),
      .TE        (1'b0));


end else begin
 assign Lane_Clk_Gated_11 = Lane_Clk_11;

end
endgenerate
   


   aticlk_mux2 mrk_scan_clkmux_11 ( .clk_o(Out_Clk_11),
                                  .clk_i({Out_Clk_Gated_11, Lane_Clk_Gated_11}),
                                  .sel(Scan_Mode) );

   ati_buffer1 mrk_d0nt_ScanEn_IP_11 (.A(ScanEn_IP), .Y(ScanEn_IP_11)); 
   ati_buffer1 mrk_d0nt_ScanEn_OP_11 (.A(ScanEn_OP), .Y(ScanEn_OP_11)); 
   ati_buffer1 mrk_d0nt_ScanEn_CP_11 (.A(ScanEn_CP), .Y(ScanEn_CP_11)); 


   wire gater_en_12;
   wire Lane_Clk_Gated_12;
   assign gater_en_12  = Gater_Table_Mode ? (gater_table_ff[12] | Tile_ScanEn) : 1'b1 ;

   ati_clock_gate Out_Clk_DfxCLOCKGATER_12
     (.clk_out   (Out_Clk_Gated_12),
      .clk_src   (Out_Clk),
      .clk_en    (gater_en_12),
      .TE        (1'b0));

generate
if(WITH_CLK_STOP_GATER[12] == 1 ) begin

//Active low set
   dfx_async_set_sync3_gc10 rst_sync_clk_enable_12 
     (
      .clk(Lane_Clk_12),
      .rstinB(Clk_Stop_Tdr),           //mission mode 0.
      .rstout(clk_enable_12)
      ); 

   ati_clock_gate Lane_Clk_DfxCLOCKGATER_12
     (.clk_out   (Lane_Clk_Gated_12),
      .clk_src   (Lane_Clk_12),
      .clk_en    (clk_enable_12),
      .TE        (1'b0));


end else begin
 assign Lane_Clk_Gated_12 = Lane_Clk_12;

end
endgenerate
   


   aticlk_mux2 mrk_scan_clkmux_12 ( .clk_o(Out_Clk_12),
                                  .clk_i({Out_Clk_Gated_12, Lane_Clk_Gated_12}),
                                  .sel(Scan_Mode) );

   ati_buffer1 mrk_d0nt_ScanEn_IP_12 (.A(ScanEn_IP), .Y(ScanEn_IP_12)); 
   ati_buffer1 mrk_d0nt_ScanEn_OP_12 (.A(ScanEn_OP), .Y(ScanEn_OP_12)); 
   ati_buffer1 mrk_d0nt_ScanEn_CP_12 (.A(ScanEn_CP), .Y(ScanEn_CP_12)); 


   wire gater_en_13;
   wire Lane_Clk_Gated_13;
   assign gater_en_13  = Gater_Table_Mode ? (gater_table_ff[13] | Tile_ScanEn) : 1'b1 ;

   ati_clock_gate Out_Clk_DfxCLOCKGATER_13
     (.clk_out   (Out_Clk_Gated_13),
      .clk_src   (Out_Clk),
      .clk_en    (gater_en_13),
      .TE        (1'b0));

generate
if(WITH_CLK_STOP_GATER[13] == 1 ) begin

//Active low set
   dfx_async_set_sync3_gc10 rst_sync_clk_enable_13 
     (
      .clk(Lane_Clk_13),
      .rstinB(Clk_Stop_Tdr),           //mission mode 0.
      .rstout(clk_enable_13)
      ); 

   ati_clock_gate Lane_Clk_DfxCLOCKGATER_13
     (.clk_out   (Lane_Clk_Gated_13),
      .clk_src   (Lane_Clk_13),
      .clk_en    (clk_enable_13),
      .TE        (1'b0));


end else begin
 assign Lane_Clk_Gated_13 = Lane_Clk_13;

end
endgenerate
   


   aticlk_mux2 mrk_scan_clkmux_13 ( .clk_o(Out_Clk_13),
                                  .clk_i({Out_Clk_Gated_13, Lane_Clk_Gated_13}),
                                  .sel(Scan_Mode) );

   ati_buffer1 mrk_d0nt_ScanEn_IP_13 (.A(ScanEn_IP), .Y(ScanEn_IP_13)); 
   ati_buffer1 mrk_d0nt_ScanEn_OP_13 (.A(ScanEn_OP), .Y(ScanEn_OP_13)); 
   ati_buffer1 mrk_d0nt_ScanEn_CP_13 (.A(ScanEn_CP), .Y(ScanEn_CP_13)); 


   wire gater_en_14;
   wire Lane_Clk_Gated_14;
   assign gater_en_14  = Gater_Table_Mode ? (gater_table_ff[14] | Tile_ScanEn) : 1'b1 ;

   ati_clock_gate Out_Clk_DfxCLOCKGATER_14
     (.clk_out   (Out_Clk_Gated_14),
      .clk_src   (Out_Clk),
      .clk_en    (gater_en_14),
      .TE        (1'b0));

generate
if(WITH_CLK_STOP_GATER[14] == 1 ) begin

//Active low set
   dfx_async_set_sync3_gc10 rst_sync_clk_enable_14 
     (
      .clk(Lane_Clk_14),
      .rstinB(Clk_Stop_Tdr),           //mission mode 0.
      .rstout(clk_enable_14)
      ); 

   ati_clock_gate Lane_Clk_DfxCLOCKGATER_14
     (.clk_out   (Lane_Clk_Gated_14),
      .clk_src   (Lane_Clk_14),
      .clk_en    (clk_enable_14),
      .TE        (1'b0));


end else begin
 assign Lane_Clk_Gated_14 = Lane_Clk_14;

end
endgenerate
   


   aticlk_mux2 mrk_scan_clkmux_14 ( .clk_o(Out_Clk_14),
                                  .clk_i({Out_Clk_Gated_14, Lane_Clk_Gated_14}),
                                  .sel(Scan_Mode) );

   ati_buffer1 mrk_d0nt_ScanEn_IP_14 (.A(ScanEn_IP), .Y(ScanEn_IP_14)); 
   ati_buffer1 mrk_d0nt_ScanEn_OP_14 (.A(ScanEn_OP), .Y(ScanEn_OP_14)); 
   ati_buffer1 mrk_d0nt_ScanEn_CP_14 (.A(ScanEn_CP), .Y(ScanEn_CP_14)); 


   wire gater_en_15;
   wire Lane_Clk_Gated_15;
   assign gater_en_15  = Gater_Table_Mode ? (gater_table_ff[15] | Tile_ScanEn) : 1'b1 ;

   ati_clock_gate Out_Clk_DfxCLOCKGATER_15
     (.clk_out   (Out_Clk_Gated_15),
      .clk_src   (Out_Clk),
      .clk_en    (gater_en_15),
      .TE        (1'b0));

generate
if(WITH_CLK_STOP_GATER[15] == 1 ) begin

//Active low set
   dfx_async_set_sync3_gc10 rst_sync_clk_enable_15 
     (
      .clk(Lane_Clk_15),
      .rstinB(Clk_Stop_Tdr),           //mission mode 0.
      .rstout(clk_enable_15)
      ); 

   ati_clock_gate Lane_Clk_DfxCLOCKGATER_15
     (.clk_out   (Lane_Clk_Gated_15),
      .clk_src   (Lane_Clk_15),
      .clk_en    (clk_enable_15),
      .TE        (1'b0));


end else begin
 assign Lane_Clk_Gated_15 = Lane_Clk_15;

end
endgenerate
   


   aticlk_mux2 mrk_scan_clkmux_15 ( .clk_o(Out_Clk_15),
                                  .clk_i({Out_Clk_Gated_15, Lane_Clk_Gated_15}),
                                  .sel(Scan_Mode) );

   ati_buffer1 mrk_d0nt_ScanEn_IP_15 (.A(ScanEn_IP), .Y(ScanEn_IP_15)); 
   ati_buffer1 mrk_d0nt_ScanEn_OP_15 (.A(ScanEn_OP), .Y(ScanEn_OP_15)); 
   ati_buffer1 mrk_d0nt_ScanEn_CP_15 (.A(ScanEn_CP), .Y(ScanEn_CP_15)); 


   wire gater_en_16;
   wire Lane_Clk_Gated_16;
   assign gater_en_16  = Gater_Table_Mode ? (gater_table_ff[16] | Tile_ScanEn) : 1'b1 ;

   ati_clock_gate Out_Clk_DfxCLOCKGATER_16
     (.clk_out   (Out_Clk_Gated_16),
      .clk_src   (Out_Clk),
      .clk_en    (gater_en_16),
      .TE        (1'b0));

generate
if(WITH_CLK_STOP_GATER[16] == 1 ) begin

//Active low set
   dfx_async_set_sync3_gc10 rst_sync_clk_enable_16 
     (
      .clk(Lane_Clk_16),
      .rstinB(Clk_Stop_Tdr),           //mission mode 0.
      .rstout(clk_enable_16)
      ); 

   ati_clock_gate Lane_Clk_DfxCLOCKGATER_16
     (.clk_out   (Lane_Clk_Gated_16),
      .clk_src   (Lane_Clk_16),
      .clk_en    (clk_enable_16),
      .TE        (1'b0));


end else begin
 assign Lane_Clk_Gated_16 = Lane_Clk_16;

end
endgenerate
   


   aticlk_mux2 mrk_scan_clkmux_16 ( .clk_o(Out_Clk_16),
                                  .clk_i({Out_Clk_Gated_16, Lane_Clk_Gated_16}),
                                  .sel(Scan_Mode) );

   ati_buffer1 mrk_d0nt_ScanEn_IP_16 (.A(ScanEn_IP), .Y(ScanEn_IP_16)); 
   ati_buffer1 mrk_d0nt_ScanEn_OP_16 (.A(ScanEn_OP), .Y(ScanEn_OP_16)); 
   ati_buffer1 mrk_d0nt_ScanEn_CP_16 (.A(ScanEn_CP), .Y(ScanEn_CP_16)); 


   wire gater_en_17;
   wire Lane_Clk_Gated_17;
   assign gater_en_17  = Gater_Table_Mode ? (gater_table_ff[17] | Tile_ScanEn) : 1'b1 ;

   ati_clock_gate Out_Clk_DfxCLOCKGATER_17
     (.clk_out   (Out_Clk_Gated_17),
      .clk_src   (Out_Clk),
      .clk_en    (gater_en_17),
      .TE        (1'b0));

generate
if(WITH_CLK_STOP_GATER[17] == 1 ) begin

//Active low set
   dfx_async_set_sync3_gc10 rst_sync_clk_enable_17 
     (
      .clk(Lane_Clk_17),
      .rstinB(Clk_Stop_Tdr),           //mission mode 0.
      .rstout(clk_enable_17)
      ); 

   ati_clock_gate Lane_Clk_DfxCLOCKGATER_17
     (.clk_out   (Lane_Clk_Gated_17),
      .clk_src   (Lane_Clk_17),
      .clk_en    (clk_enable_17),
      .TE        (1'b0));


end else begin
 assign Lane_Clk_Gated_17 = Lane_Clk_17;

end
endgenerate
   


   aticlk_mux2 mrk_scan_clkmux_17 ( .clk_o(Out_Clk_17),
                                  .clk_i({Out_Clk_Gated_17, Lane_Clk_Gated_17}),
                                  .sel(Scan_Mode) );

   ati_buffer1 mrk_d0nt_ScanEn_IP_17 (.A(ScanEn_IP), .Y(ScanEn_IP_17)); 
   ati_buffer1 mrk_d0nt_ScanEn_OP_17 (.A(ScanEn_OP), .Y(ScanEn_OP_17)); 
   ati_buffer1 mrk_d0nt_ScanEn_CP_17 (.A(ScanEn_CP), .Y(ScanEn_CP_17)); 


   wire gater_en_18;
   wire Lane_Clk_Gated_18;
   assign gater_en_18  = Gater_Table_Mode ? (gater_table_ff[18] | Tile_ScanEn) : 1'b1 ;

   ati_clock_gate Out_Clk_DfxCLOCKGATER_18
     (.clk_out   (Out_Clk_Gated_18),
      .clk_src   (Out_Clk),
      .clk_en    (gater_en_18),
      .TE        (1'b0));

generate
if(WITH_CLK_STOP_GATER[18] == 1 ) begin

//Active low set
   dfx_async_set_sync3_gc10 rst_sync_clk_enable_18 
     (
      .clk(Lane_Clk_18),
      .rstinB(Clk_Stop_Tdr),           //mission mode 0.
      .rstout(clk_enable_18)
      ); 

   ati_clock_gate Lane_Clk_DfxCLOCKGATER_18
     (.clk_out   (Lane_Clk_Gated_18),
      .clk_src   (Lane_Clk_18),
      .clk_en    (clk_enable_18),
      .TE        (1'b0));


end else begin
 assign Lane_Clk_Gated_18 = Lane_Clk_18;

end
endgenerate
   


   aticlk_mux2 mrk_scan_clkmux_18 ( .clk_o(Out_Clk_18),
                                  .clk_i({Out_Clk_Gated_18, Lane_Clk_Gated_18}),
                                  .sel(Scan_Mode) );

   ati_buffer1 mrk_d0nt_ScanEn_IP_18 (.A(ScanEn_IP), .Y(ScanEn_IP_18)); 
   ati_buffer1 mrk_d0nt_ScanEn_OP_18 (.A(ScanEn_OP), .Y(ScanEn_OP_18)); 
   ati_buffer1 mrk_d0nt_ScanEn_CP_18 (.A(ScanEn_CP), .Y(ScanEn_CP_18)); 

   ///////////////////////////////////////////////////////////////////

    `include "dft_lanes_clk_cntl_chk_gc10.vh"
   
endmodule // dft_lanes_clk_cntl

