//                              -*- Mode: Verilog -*-

////////////////////////////////////////////////////////////////////////////
/*
 *
 *  Trade secret of ATI Technologies, Inc.
 *  Copyright 2005, ATI Technologies, Inc., (unpublished)
 *
 *  All rights reserved.  This notice is intended as a precaution against
 *  inadvertent publication and does not imply publication or any waiver
 *  of confidentiality.  The year included in the foregoing notice is the
 *  year of creation of the work.
 *
 */
////////////////////////////////////////////////////////////////////////////
//`include "dft_clk_cntl_features.vh"


module dft_pulse_cnt_gc10 
# (
   parameter DFT_CLK_CNTL_PULSE_CNT = 5
   )
(
 input wire scan_mode_sync_func_clk,
 input wire Func_Clk,
 input wire Gater_Table_Mode,
 input wire Stuck_At_Mode,
 input wire Scan_Mode,
 input wire clk_stop,
 input wire load_pul_cnt,
 input wire [DFT_CLK_CNTL_PULSE_CNT-1:0] pulse_cnt,
 input wire mode_gater_open,
 input wire mode_sel_gater_always_open,
 input wire [DFT_CLK_CNTL_PULSE_CNT-1:0] gater_table_ff,
 output wire pulse_cnt_muxed_ored,
 output wire Stuck_At_Mode_Cg_En,
 output wire At_Speed_Mode_Cg_En,
 output wire clk_enable,
 output reg  pulse_cnt_en
 
 );
   
   ///////////////// Wire/Reg /////////////////////
   reg [DFT_CLK_CNTL_PULSE_CNT-1:0] pul_cnt;
   wire 			    SA_mode_clk_en;
   wire 			    clk_enable_tdr;
   wire 			    clk_enable_sync;
   wire [DFT_CLK_CNTL_PULSE_CNT-1:0] pulse_cnt_muxed;
   
   
   //assign clk_enable_tdr = ~clk_stop;

//Active low set
   dfx_async_set_sync3_gc10 rst_sync_clk_enable 
     (
      .clk(Func_Clk),
      .rstinB(clk_stop),           //mission mode 0.
      .rstout(clk_enable)
      ); 
   
   
   generate
      genvar i;
      for (i=0; i<DFT_CLK_CNTL_PULSE_CNT; i=i+1) begin : mux_pulse_cnt_loop
	 ati_mux2 mux_pulse_cnt  
	   (
	    .i0(pulse_cnt[i]),
	    .i1(gater_table_ff[i]),      //0 when SE=1.
	    .s(Gater_Table_Mode),
	    .z(pulse_cnt_muxed[i]));
      end
   endgenerate
  
   assign pulse_cnt_muxed_ored = (|pulse_cnt_muxed);
   
   // Reason for CDC waiver.
   // Func mode, below flow is d0nt care. It won't contorl the clock gater.
   // in clock stop mode(debug mode), clk_enable is synchronously set to 0, load_pul_cnt is held to 0 by Scan_Mode.
   // Hence rest of the inputs is d0nt care.
   // In MCM AC mode, load_pul_cnt will reset the values.load_pul_cnt is synchronos and timeand timed
   // In SCM mode, this is d0n't care.
   
   always @ (posedge Func_Clk or negedge scan_mode_sync_func_clk)
     begin
        if (~scan_mode_sync_func_clk) begin   //mission mode-0
           pul_cnt <= {(DFT_CLK_CNTL_PULSE_CNT){1'b0}};
           pulse_cnt_en <= 1'b1;  // TBD- verification. Keep gater open. Clock stop use this path.
        end else if (load_pul_cnt) begin 
	   // load when SE=1. when latency_cnt saturate, counter starts. 
	   // For gater table, this is loaded after last shift cycle.
           //mission mode, load_pul_cnt = 0,
           pul_cnt <= pulse_cnt_muxed;  //CDC waive..
           pulse_cnt_en <= 1'b0;
        end else if (mode_gater_open) begin //CDC waive
           //For root_dft_clk_cntl in INTEST mode to keep top level clock freerunning all the time
           //For SSB mode.
	   //mission mode mode_gater_open = 0
           pul_cnt <= pul_cnt;
           pulse_cnt_en <= 1'b1;
        end else if (pul_cnt == {(DFT_CLK_CNTL_PULSE_CNT){1'b0}}) begin
           //If gater flop all bits programmed to 0, clock is stopped during capture.
	   //this would put pulse_cnt_en = 0 in mission mode.
           pul_cnt <= pul_cnt;
           pulse_cnt_en <= 1'b0;
        end else begin
           if(Gater_Table_Mode) begin //CDC waiver
              pul_cnt <= {1'b0, pul_cnt[DFT_CLK_CNTL_PULSE_CNT-1:1]};
              pulse_cnt_en <= pul_cnt[0];
           end else begin
              pul_cnt <= pul_cnt - {{(DFT_CLK_CNTL_PULSE_CNT-1){1'd0}}, 1'b1};
              pulse_cnt_en <= 1'b1;
           end   
        end // else: !if(pul_cnt == {(DFT_CLK_CNTL_PULSE_CNT){1'b0}})
     end // always @ (posedge Func_Clk)
   
  
  //Mission mode, Stuck_At_Mode=0, pulse_cnt_en = 1;
  //Clk_Stop mode use At_Speed_Mode_Cg_En path to stop the gater.  
   assign At_Speed_Mode_Cg_En  = (clk_enable & ~Stuck_At_Mode & pulse_cnt_en) | mode_sel_gater_always_open;
   assign SA_mode_clk_en       = (clk_enable &  Stuck_At_Mode & pulse_cnt_en) | mode_sel_gater_always_open;   
   
   ati_mux2 mux_scan_mode_clk_en  
     (
      .i0(clk_enable),         //Keep gater open during mission mode.
      .i1(SA_mode_clk_en), 
      .s(Scan_Mode),
      .z(Stuck_At_Mode_Cg_En)); //CDC waive


   
endmodule // dft_pulse_cnt


