//                              -*- Mode: Verilog -*-
////////////////////////////////////////////////////////////////////////////
/*
 *
 *  Trade secret of ATI Technologies, Inc.
 *  Copyright 2005, ATI Technologies, Inc., (unpublished)
 *
 *  All rights reserved.  This notice is intended as a precaution against
 *  inadvertent publication and does not imply publication or any waiver
 *  of confidentiality.  The year included in the foregoing notice is the
 *  year of creation of the work.
 *
 */
////////////////////////////////////////////////////////////////////////////
//
// Filename    : dfx_ckgate.v
// Description : This block is the clock gater portion of the "2 phase DFS"
//    input  clkEn_AR; // Taks effect 2 rising  clkIn edge later
//    input  clkEn_BR; // Taks effect 2 falling clkIn edge later
////////////////////////////////////////////////////////////////////////////
// Perforce revision     : $Revision: #2 $
// Perforce changelist   : $Change: 2885168 $
// Perforce date         : $Date: 2017/06/27 $
// Perforce Last edit by : $Author: couyang $
////////////////////////////////////////////////////////////////////////////
//
module dfx_ckgate
  ( clkIn, clkEn_AR, clkEn_BR, clkOut );
   input  clkIn   ; // Input clock
   input  clkEn_AR; // Clock gating term of A-phase portion of pulse, 1'b1 -> corresponding output high
   input  clkEn_BR; // Clock gating term of B-phase portion of pulse, 1'b1 -> corresponding output high
   output clkOut  ; // Output clock

  wire    clkInA;
  wire    clkInB;
  aticlk_buf  d0nt_clkInStart(.A     (clkIn ), .Y      (clkInA));
  ati_clk_inv d0nt_clkInB    (.clk_in(clkInA), .clk_out(clkInB));

  reg     clkEnA;
  reg     clkEnB;
  reg     clkEnBB;

  always @(posedge clkInA) begin
     clkEnA  <= clkEn_AR;
     clkEnBB <= clkEn_BR;
  end

  // Use the b-phase to avoid hols time issues with clock gater latch
  always @(posedge clkInB) begin
     clkEnB <= clkEnBB;
  end

  wire clkOutA;
  wire clkOutA_Inv;
  wire clkOutB;
  wire clkOutB_Inv;
  wire clkOutBuf;

  ati_clock_gate d0nt_clkOutA_DfxCLOCKGATER (.clk_src(clkInA), .clk_en(clkEnA), .TE(1'b0), .clk_out(clkOutA)); // Half cycle to this gater
  ati_clock_gate d0nt_clkOutB_DfxCLOCKGATER (.clk_src(clkInB), .clk_en(clkEnB), .TE(1'b0), .clk_out(clkOutB)); // Half cycle to this gater

  // There are no clock OR in rtllib, so compose one
  ati_clk_inv  d0nt_clkOutA_Inv (.clk_in(clkOutA    ), .clk_out(clkOutA_Inv));
  ati_clk_inv  d0nt_clkOutB_Inv (.clk_in(clkOutB    ), .clk_out(clkOutB_Inv));
  ati_clk_nand d0nt_clkOutNand  (.A1    (clkOutA_Inv), .A2     (clkOutB_Inv), .ZN(clkOutBuf));

  aticlk_buf   d0nt_clkOut      (.A     (clkOutBuf  ), .Y      (clkOut     ));

endmodule // dfx_ckgate
