/////////////////////////////////////////////////////////////////////////////
// File Name: dfx_clkmux
// Description: parametrized binary decoding clock mux
/////////////////////////////////////////////////////////////////////////////
  module dfx_clkmux_gc10(
		      clk_o, clk_i, sel
		      );
   
   parameter MUX_SEL_WIDTH = 2;
   //parameter MUX_WIDTH = 4;
   
   localparam MUX_WIDTH  = 2 ** MUX_SEL_WIDTH;
   localparam MUX_WIDTH_BY_2  = MUX_WIDTH/2;
   
   output    clk_o;
   input [MUX_WIDTH - 1:0] clk_i;
   input [MUX_SEL_WIDTH - 1:0] sel;
   
   wire 		       clk_o;
   wire [MUX_WIDTH - 1:0]      clk_i;
   wire [MUX_SEL_WIDTH - 1:0]  sel;
   wire [MUX_SEL_WIDTH - 1:0]  clk_sel;
   
   /////////////////////////////////////////////////////////////////////////////
   
  // use a buffer so it's easy to make constraints
  ati_buffer0 clk_sel_buf [MUX_SEL_WIDTH-1:0] (.A(sel[MUX_SEL_WIDTH-1:0]), .Y(clk_sel[MUX_SEL_WIDTH-1:0]) );
   
   
   //wire [ MUX_SEL_WIDTH - 1 :0] out [MUX_WIDTH_BY_2 -1 :0] ;
   //wire [ 3 :0] out [7 :0] ;
   //wire [ 7 :0] out [3 :0] ;
   wire 		       out [MUX_SEL_WIDTH-1:0][MUX_WIDTH_BY_2:0] ;
   
   
   
   // river translate_off
   generate
   genvar i;
   genvar j;
   
   for(i=0; i < MUX_SEL_WIDTH; i = i+1) begin : mux_i
      for(j=0; j < ( 2 ** (MUX_SEL_WIDTH - i - 1) ); j = j+1) begin : mux_j
	 
	 if(i == 0) begin
	    
	    aticlk_mux2 d0nt_clkmux (
				.clk_o( out[i][j] ),
				.clk_i({
					clk_i[j*2+ 1],
					clk_i[j*2]
					}),
				.sel(clk_sel[i])
				);
	    
	    //initial
	    //$display("i=%d, j=%d,  leg_0 = %d leg_1 = %d out = %d\n", i, j, mux_leg_0, mux_leg_1, mux_out);
	    
	 end else  begin
	    
	    aticlk_mux2 d0nt_clkmux (
				.clk_o( out[i][j] ),
				.clk_i({
					out[i-1][j*2 +1],
					out[i-1][j*2]
					}),
				.sel(clk_sel[i])
				);
	    
	 end
	 
	 
      end
   end
   endgenerate
   // river translate_on
   
   aticlk_buf d0nt_clk_buf (.A(out[MUX_SEL_WIDTH-1][0]), .Y(clk_o));
   
   /////////////////////////////////////////////////////////////////////////////
endmodule
