//                              -*- Mode: Verilog -*-

////////////////////////////////////////////////////////////////////////////
/*
 *
 *  Trade secret of ATI Technologies, Inc.
 *  Copyright 2005, ATI Technologies, Inc., (unpublished)
 *
 *  All rights reserved.  This notice is intended as a precaution against
 *  inadvertent publication and does not imply publication or any waiver
 *  of confidentiality.  The year included in the foregoing notice is the
 *  year of creation of the work.
 *
 */
////////////////////////////////////////////////////////////////////////////
//
// Filename    : dfx_counter.v
// Description : Counter
//
////////////////////////////////////////////////////////////////////////////

//COUNTER_WIDTH minimum must be 3
module dfx_counter_gc10 
# (
parameter COUNTER_WIDTH = 3
)
(
//Broadcast to All TDRs
 input wire Tdr_Tck,
 input wire Tdr_Trstb,
 input wire Tdr_Shift,
 input wire Tdr_Capture,
 input wire Tdr_Update,
 
 //Tdr
 input wire Tdr_dfx_counter_Tdi,
 input wire Tdr_dfx_counter_En,
 output wire Tdr_dfx_counter_Tdo,

 input  clk,
// input  rstb; // active low, should have to be sync on clk
// input  load;    // load the counter value
 input  wire enable,  // Enable the counter
 input  wire stop,   // stop the counter
 output wire count_done


);
wire count_done_pre;

///////////////////////////////////////////////////////////////////
wire [COUNTER_WIDTH-1:0] counter_value;
wire [1-1:0] count_done_mask;

dfx_counter_tdr_gc10 
#(
   .DFX_COUNTER_WIDTH (COUNTER_WIDTH)
 ) dfx_counter_tdr ( 

 .Tdr_Tck               (Tdr_Tck) ,
 .Tdr_Trstb             (Tdr_Trstb),
 .Tdr_Tdi               (Tdr_dfx_counter_Tdi),
 .Tdr_Shift             (Tdr_Shift),
 .Tdr_Capture           (Tdr_Capture),
 .Tdr_Update            (Tdr_Update)

//TDR --> DFX_COUNTER
 ,.count_done_mask               (count_done_mask[1-1:0])
 ,.counter_value        (counter_value[COUNTER_WIDTH-1:0])
 ,.Tdr_dfx_counter_En   (Tdr_dfx_counter_En)
 ,.Tdr_dfx_counter_Tdo  (Tdr_dfx_counter_Tdo)
); 

//Bug-DEGNRTL-850 
assign count_done = count_done_pre & count_done_mask;
///////////////////////////////////////////////////////////////////

localparam LOW_CNT_WIDTH    = COUNTER_WIDTH/3;
localparam MID_CNT_WIDTH    = COUNTER_WIDTH/3;
localparam HI_CNT_WIDTH     = COUNTER_WIDTH-LOW_CNT_WIDTH-MID_CNT_WIDTH;

wire [HI_CNT_WIDTH-1:0]  hi_cnt_val     = counter_value[COUNTER_WIDTH-1:LOW_CNT_WIDTH+MID_CNT_WIDTH];
wire [MID_CNT_WIDTH-1:0] mid_cnt_val    = counter_value[LOW_CNT_WIDTH+MID_CNT_WIDTH-1:LOW_CNT_WIDTH];
wire [LOW_CNT_WIDTH-1:0] low_cnt_val    = counter_value[LOW_CNT_WIDTH-1:0];

reg counter_load;

// The first dr_shift after instruction is active would load the counter
// The subsequent dr_shift will be ignored.
// To reload counter to new value, the instruction need to be exited first, then re-enable.
always @(posedge Tdr_Tck, negedge Tdr_Trstb)
 begin
 if(~Tdr_Trstb)
  counter_load <= 1'b0;
 else if(~Tdr_dfx_counter_En)
  counter_load <= 1'b0;
 else if(Tdr_Update)
  counter_load <= 1'b1; // The REFCLK might not be toggling when load asserted. Hence, keeping it asserted to 1 all the time until instruction is active.
 else
  counter_load <= counter_load;
 end

counter_divide_gc10  
#(.hi_cnt_width(HI_CNT_WIDTH), .mid_cnt_width(MID_CNT_WIDTH), .low_cnt_width(LOW_CNT_WIDTH) )
counter_divide
            (
            .clk                (clk),
            .rstb               (Tdr_Trstb),
            .rstb_sync          (),
            .load               (counter_load),
            .enable             (enable),
            .stop               (stop),
            .hi_cnt_val         (hi_cnt_val),   //CDC waiver. One TCK + sync in clk domain give few cycles before TDR data get loaded in counter.
            .mid_cnt_val        (mid_cnt_val),
            .low_cnt_val        (low_cnt_val),
            .hi_cnt_out         (),
            .mid_cnt_out        (),
            .low_cnt_out        (),
            .count_done         (count_done_pre)
            );


endmodule
