//                              -*- Mode: Verilog -*-
//////////////////////////////////////////////////////////////////////////////////////////
//
// Perforce File         : $File: //sysip/dfx/dfx-4/4.1/branches/dfxip4_gfxip_gfx10/dftmisc/src/rtl/dfx_sync_async_rst.v.dpl $
// Perforce revision     : $Revision: #2 $
// Perforce changelist   : $Change: 2885168 $
// Perforce date         : $Date: 2017/06/27 $
// Perforce Author       : $Author: couyang $
//
// Copyright (c) 2015 by Advanced Micro Devices, Inc.
//
// This file is protected by Federal Copyright Law, with all rights
// reserved. No part of this file may be reproduced, stored in a
// retrieval system, translated, transcribed, or transmitted, in any
// form, or by any means manual, electric, electronic, mechanical,
// electro-magnetic, chemical, optical, or otherwise, without prior
// explicit written permission from Advanced Micro Devices, Inc.
//
//////////////////////////////////////////////////////////////////////////////////////////
`include "rtllib_features.vh"
// Description:
//  - Asynchronous resetable synchronizer
module dfx_sync_async_rst ( CK, D, rstB, SE, Q );
  parameter      WIDTH          =  1;
  parameter      depth          =  3; // This should only be 3 or 4
  parameter      ASYNCSET       =  0; // ASYNCSET == 1 => reset value is 1, ASYNCSET == 0 => reset value is 0 (default)
  parameter      TransportDelay =  0;
  parameter      TransportCycle =  1;

input              CK  ;
input  [WIDTH-1:0] D   ;
input              rstB;
input              SE  ;
output [WIDTH-1:0] Q   ;

    wire ZERO=1'b0;
    wire ONE=1'b1 ;
    wire [WIDTH-1:0] Dint  ;
    wire [WIDTH-1:0] Qint  ;

generate
   case ( ASYNCSET )
     1: begin : ASYNCSET_1
        assign Dint[WIDTH-1:0] = ~(D[WIDTH-1:0]);
        assign Q   [WIDTH-1:0] = ~(Qint[WIDTH-1:0]);
     end
     default: begin : ASYNCSET_0
        assign Dint[WIDTH-1:0] =  (D[WIDTH-1:0]);
        assign Q   [WIDTH-1:0] =  (Qint[WIDTH-1:0]);
     end
   endcase // case ( ASYNCSET )
endgenerate

techind_sync_icd  
# (
   .size           (WIDTH         ),
   .depth          (depth         ),
   .sourceFlop     (             0),
   .asyncSet       (             0),
   .asyncReset     (             1),
   .asyncFlush     (             0),
   .clockEnable    (             0),
   .scan_muxD      (             1),
   .scan_LSSD      (             0),
   .MultiVolt      (             0),
   .TransportDelay (TransportDelay),
   .TransportCycle (TransportCycle),
   .InputCheck     (             1),
   .MetaWindowMode (             0)

) techind_sync_icd (
    .SB   (           1'b0),
    .RB   (rstB           ),
    .FLUSH(           1'b0),
    .CKsrc(           1'b0),
    .CKdst(CK             ),
    .EN   (           1'b1),
    .Dsrc (Dint[WIDTH-1:0]),
    .Qsrc (               ),
    .Qdst (Qint[WIDTH-1:0]),
`ifdef ABV
`ifdef DFXSIP_EnableIPWC
        .InputChkDis(ZERO),
`else
        .InputChkDis(ONE),
`endif
`endif
    .SC1(1'b0),
    .SC2(1'b0),
    .SE (SE  ),
    .SI (1'b0),
    .SO (    )
);

endmodule // dfx_sync_async_rst
