//                              -*- Mode: Verilog -*-
//
// Perforce File         : : //sysip/dfx/dfx-4/4.1/branches/dfxip4_gfxip_gfx10/dftmisc/src/rtl/dfx_tdr_to_ipclk.v.dpl n// Perforce revision     : : #3 n// Perforce changelist   : : 2972984 n// Perforce date         : : 2017/08/18 n// Perforce Author       : : couyang n//
// Copyright             : Trade secret of ATI Technologies, Inc.
//                         Copyright 2001-2002, ATI Technologies, Inc., (unpublished)
//                         All rights reserved.  This notice is intended as a precaution against
//                         inadvertent publication and does not imply publication or any waiver
//                         of confidentiality.  The year included in the foregoing notice is the
//                         year of creation of the work.
//
// Description           : Module to synchronize TDR (tck domain) into ipclk clock domain
//
// ======================================================================================================
module dfx_tdr_to_ipclk_gc10 ( /*AUTOARG*/
   // Outputs
   Tdr_Reg_ipclk, trstb_raw_ipclk, jtag_active_ipclk,
   // Inputs
   Tdr_Tck, Tdr_Update, Tdr_Trstb, Tdr_En, Tdr_Reg_tck, ipclk,
   ipclk_gated, resetb_ipclk
   );
   //============================
   // PARAMETERS
   //============================
   parameter TDR_WIDTH = 1;

   // JTAG control signals
   input                  Tdr_Tck    	   ;
   input                  Tdr_Update 	   ;
   input                  Tdr_Trstb  	   ;
   input                  Tdr_En     	   ; // TDR instruction decode
   input  [TDR_WIDTH-1:0] Tdr_Reg_tck	   ; // TDR register output in tck

   input                  ipclk            ;

   // Gated version of ipclk, gater must use jtag_active_ipclk term, 1 cycle to turn on clock
   // If not availabe, connect to same clock as above port
   input                  ipclk_gated      ;

   input                  resetb_ipclk     ; // functional resetb
   output [TDR_WIDTH-1:0] Tdr_Reg_ipclk    ; // Synchronized output
   output                 trstb_raw_ipclk  ; // Synchronized Tdr_Trstb
   output                 jtag_active_ipclk; // Indicate JTAG is actuve. For clock gating

   // wire & Reg without '_tck' postfix are in ipclk ------- begin
   // delayed version of UpdateDR for sync data into ipclk
   wire                 dec_update_d1_tck ;
   wire                 dec_updateEn_tck  ;

   reg   [3:1]          sync_updateEn_d   ;
   wire                 sync_updateEn     ;
   reg   [2:1]          trstb_ipclk_d     ;
   wire                 cdc_Tdr_En        ;
   wire                 updateEn_setup    ;
   reg  [TDR_WIDTH-1:0] Tdr_Reg           ;
   wire [TDR_WIDTH-1:0] cdce_Tdr_Reg      ;
   wire                 trstb_fe_ipclk    ;
   wire                 updateEn_re_ipclk ;
   wire                 any_resetb        ;
   // wire & Reg without '_tck' postfix are in ipclk ------- end

   assign jtag_active_ipclk = trstb_fe_ipclk | updateEn_re_ipclk;
   dfx_mrk_and_gc10 I_any_resetb (.A(trstb_raw_ipclk), .B(resetb_ipclk), .Y(any_resetb));
   // http://ontrack-internal.amd.com/browse/DECDFXIP40-505
   //  Toggle dec_update_d1_tck at Tdr_Update to relax clock ratio requirement
   //  so ipclk to tck can be releaxed from 4:1 to 1:1 (ipclk is the faster clock)
   dfx_mrk_and_gc10 I_dec_updateEn_tck (.A( Tdr_Trstb),
								   .B((Tdr_En & Tdr_Update) ? ~dec_update_d1_tck : dec_update_d1_tck),
								   .Y( dec_updateEn_tck));
   dfx_sync_1b_gc10 I_sync_updateEn (.DBL_FLOP_EN(1'b1  ), .RST  (~Tdr_Trstb       ), // These 2 ports are not used in actual function
                                                                .D(dec_updateEn_tck),                            // input in SRC_CLK
                                                                .SRC_CLK(Tdr_Tck   ), .Q_SRC(dec_update_d1_tck),
                                                                .DST_CLK(ipclk     ), .Q    (sync_updateEn    ));

   dfx_async_rst_sync3_gc10 I_trstb_ipclk (.clk(ipclk), .rstinB(Tdr_Trstb), .rstoutB(trstb_raw_ipclk)); // async_rst_sync
   assign trstb_fe_ipclk = (~trstb_raw_ipclk & trstb_ipclk_d[1]) | (~trstb_ipclk_d[1] & trstb_ipclk_d[2]);
   always @(posedge ipclk)
     if (~resetb_ipclk)
       trstb_ipclk_d[2:1] <= 2'b01;
     else
       trstb_ipclk_d[2:1] <= {trstb_ipclk_d[1], trstb_raw_ipclk};
   
   assign      updateEn_re_ipclk = (sync_updateEn_d[1] ^ sync_updateEn_d[2]) | (sync_updateEn_d[2] ^sync_updateEn_d[3]);
   techind_cdcfpm  #(.size (1), .applyParams (1), .DestClkCnt(3), .CdcDestSetup(0)) mrk_cdc_Tdr_En (.A(Tdr_En ), .DSTCLK(ipclk), .Z(cdc_Tdr_En ));
   dfx_mrk_and_gc10 I_updateEn_setup (.A(updateEn_re_ipclk), .B(cdc_Tdr_En), .Y(updateEn_setup));

   // Using this set of flops to filter the x from the synchronizer (I_trstb_ipclk)
   always @(posedge ipclk)
     if (~any_resetb) begin
        sync_updateEn_d[3:1] <= { 3 {1'b0}};
     end else begin
        sync_updateEn_d[3:1] <= { sync_updateEn_d[2:1], sync_updateEn};
     end

   assign Tdr_Reg_ipclk[TDR_WIDTH-1:0] = Tdr_Reg[TDR_WIDTH-1:0];
   techind_cdcefpm #(.size (TDR_WIDTH), .applyParams(1), .DestClkCnt(3), .CdcDestSetup(0))  I_cdce_Tdr_Reg (.A(Tdr_Reg_tck[TDR_WIDTH-1:0]), .EN(updateEn_setup), .DSTCLK(ipclk), .Z(cdce_Tdr_Reg[TDR_WIDTH-1:0]));
   always @(posedge ipclk_gated)
     if (~any_resetb | updateEn_setup) begin
        Tdr_Reg[TDR_WIDTH-1:0] <= cdce_Tdr_Reg[TDR_WIDTH-1:0];
     end else begin
        Tdr_Reg[TDR_WIDTH-1:0] <= Tdr_Reg[TDR_WIDTH-1:0];
     end

endmodule // dfx_tdr_to_ipclk_gc10
