`include "global_features.vh"

`define DIDT_CTRL_REG0_WIDTH 32
`define DIDT_CTRL_REG1_WIDTH 32
`define PSM_CURPSM_LENGTH 14

module didt_droop_detection (
  // outputs
  didt_throttle_level,
  didt_dbus,
  psm_didt_dd_stat,
  // inputs
  Cpl_GFXCLK,
  resetb,
  psm_didt_dd_cntl0,
  psm_didt_dd_cntl1,
  cgcg_en,
  psm_counter,
  dft_clk_gate_ctrl,
  dft_shift_clk
);

  // outputs
  output [2:0]                        didt_throttle_level;
  output [31:0]                       didt_dbus;
  output [31:0]                       psm_didt_dd_stat;

  // inputs
  input                               Cpl_GFXCLK;
  input                               resetb;
  input                               cgcg_en;
  input  [1:0]                        dft_clk_gate_ctrl;
  input                               dft_shift_clk;
  input  [`DIDT_CTRL_REG0_WIDTH-1:0]  psm_didt_dd_cntl0;
  input  [`DIDT_CTRL_REG1_WIDTH-1:0]  psm_didt_dd_cntl1;
  input  [`PSM_CURPSM_LENGTH-1:0]     psm_counter;


//***************************************************************************//
//                          DIDT droop_level detection                       //
//***************************************************************************//


  wire		sw_rst;
  wire		psm_didt_dd_en;
  wire		cg_override;
  wire		legacy_didt_mode;
  wire[3:0]	psm_avgdiv;
  wire[7:0]	psm_counter_dyn_threshold;
  wire[13:0]	psm_counter_static_threshold;
  wire		throttle_mode;
  wire		dbg_en;

  assign	sw_rst					        = psm_didt_dd_cntl0[0];
  assign	psm_didt_dd_en			        = psm_didt_dd_cntl0[1];
  assign	cg_override				        = psm_didt_dd_cntl0[2];
  assign	legacy_didt_mode		        = psm_didt_dd_cntl0[3];
  assign	psm_avgdiv                      = psm_didt_dd_cntl0[7:4];
  assign	psm_counter_dyn_threshold 	    = psm_didt_dd_cntl0[15:8];
  assign	psm_counter_static_threshold	= psm_didt_dd_cntl0[29:16];
  assign	throttle_mode			        = psm_didt_dd_cntl0[30];
  assign	dbg_en					        = psm_didt_dd_cntl0[31];

  wire[3:0] static_droop_level_index;
  wire[1:0] dyn_droop_level_index;
  wire[1:0] psm_clkdiv;

  wire      didt_droop_level_enable     = psm_didt_dd_en;
  assign	static_droop_level_index 	= psm_didt_dd_cntl1[3:0];
  assign	dyn_droop_level_index 		= psm_didt_dd_cntl1[5:4];
  assign	psm_clkdiv                  = psm_didt_dd_cntl1[7:6];

  wire	didt_rstn = ~sw_rst & resetb;

   reg [13:0] psm_counter_q;

   wire didt_droop_clk_gated;
   wire didt_droop_clk_en= ~didt_rstn | psm_didt_dd_en | cg_override;
   wire pipe_en_d;
   reg  pipe_en_q;

   gfx_cgtt_local_dft didt_droop_clkgater (
      .Cpl_GFXCLK  			(Cpl_GFXCLK            ),
      .core_busy   			(didt_droop_clk_en     ),
      .oclk  				(didt_droop_clk_gated  ),
      .dft_clk_gate_ctrl	(dft_clk_gate_ctrl),
      .dft_shift_clk		(dft_shift_clk)
   );

   reg [2:0] free_counter;
   wire [2:0] psm_clkdiv_value = (psm_clkdiv == 2'b00)? 3'b000 : 
                                 (psm_clkdiv == 2'b01)? 3'b001 : 
                                 (psm_clkdiv == 2'b10)? 3'b011 : 
                                 (psm_clkdiv == 2'b11)? 3'b111 : 3'b111 ; 

   always @(posedge didt_droop_clk_gated) begin
     if(~didt_rstn)
       psm_counter_q <= 'd0;
     else if(pipe_en_d)
       psm_counter_q <= psm_counter;
   end
   assign pipe_en_d = (psm_counter != psm_counter_q) || (free_counter == psm_clkdiv_value);

   always @(posedge didt_droop_clk_gated) begin
     if(~didt_rstn)
       free_counter <= 3'b000;
     else if(pipe_en_d)
       free_counter <= 3'b000; 
     else
       free_counter <= free_counter + 1;
   end
 
   always @(posedge didt_droop_clk_gated) begin
     if(~didt_rstn)
        pipe_en_q <= 1'b0;
     else
        pipe_en_q <= pipe_en_d;
   end

// dyn droop check
   wire [29:0] acc_psm_d;
   reg [29:0] acc_psm_q;

   always @(posedge didt_droop_clk_gated) begin
     if(~didt_rstn)
       acc_psm_q <= 'd0;
     else if(pipe_en_q)
       acc_psm_q <= acc_psm_d;
   end

   wire [13:0] avg_psm = acc_psm_q >> psm_avgdiv;
   assign acc_psm_d = acc_psm_q - avg_psm + psm_counter_q;
   wire [7:0] dyn_droop = (avg_psm > psm_counter_q)? (avg_psm - psm_counter_q) : 8'd0;
   wire [7:0] didt_dyn_droop = (dyn_droop > psm_counter_dyn_threshold)? (dyn_droop - psm_counter_dyn_threshold) : 8'd0;

   reg dyn_droop_level_overflow;
   integer m;
   always @*
    begin
        dyn_droop_level_overflow = 1'b0;
        for (m = 0; m < 7; m=m+1) begin
          if(m > dyn_droop_level_index + 2)
           dyn_droop_level_overflow = (dyn_droop_level_overflow | didt_dyn_droop[m]) & didt_droop_level_enable;
         end
     end

    wire[2:0] didt_dyn_droop_level;
   assign didt_dyn_droop_level[0] = didt_droop_level_enable ? (dyn_droop_level_overflow ? 1'b1 : didt_dyn_droop[dyn_droop_level_index]) : 1'b0;
   assign didt_dyn_droop_level[1] = didt_droop_level_enable ? (dyn_droop_level_overflow ? 1'b1 : didt_dyn_droop[dyn_droop_level_index+1]) : 1'b0;
   assign didt_dyn_droop_level[2] = didt_droop_level_enable ? (dyn_droop_level_overflow ? 1'b1 : didt_dyn_droop[dyn_droop_level_index+2]) : 1'b0;

// static droop check
   wire signed [14:0] s_didt_psm_counter = didt_droop_level_enable ? {1'b0,psm_counter_q} : 'd0;
   wire signed [14:0] s_didt_static_droop_threshold = {1'b0,psm_counter_static_threshold};
   wire signed [14:0] s_didt_static_droop = s_didt_static_droop_threshold - s_didt_psm_counter;
   wire [13:0]  didt_static_droop = s_didt_static_droop[14] ? 'd0 : s_didt_static_droop[13:0];

   reg static_droop_level_overflow;
   integer n;
   always @*
    begin
        static_droop_level_overflow = 1'b0;
        for (n = 0; n < 14; n=n+1) begin
          if(n > static_droop_level_index + 2)
           static_droop_level_overflow = (static_droop_level_overflow | didt_static_droop[n]) & didt_droop_level_enable;
         end
     end

	wire[2:0] didt_static_droop_level;
   assign didt_static_droop_level[0] = didt_droop_level_enable ? (static_droop_level_overflow ? 1'b1 : didt_static_droop[static_droop_level_index]) : 1'b0;
   assign didt_static_droop_level[1] = didt_droop_level_enable ? (static_droop_level_overflow ? 1'b1 : didt_static_droop[static_droop_level_index+1]) : 1'b0;
   assign didt_static_droop_level[2] = didt_droop_level_enable ? (static_droop_level_overflow ? 1'b1 : didt_static_droop[static_droop_level_index+2]) : 1'b0;

  wire [2:0] didt_throttle_level_d;
  wire [2:0] final_droop_level = (didt_static_droop_level != 0)? 3'b111 : didt_dyn_droop_level;
  wire [2:0] final_stall_level;
  wire static_throttle = (didt_static_droop_level != 0);
  wire dyn_throttle = (didt_dyn_droop_level != 0);
  assign final_stall_level[0] = static_throttle | dyn_throttle;   //stall signal
  assign final_stall_level[1] = static_throttle; // reuse MSA_HI
  assign final_stall_level[2] = dyn_throttle;    // reuse MSA_LO

  assign didt_throttle_level_d =  legacy_didt_mode ? didt_static_droop_level : (throttle_mode ? final_droop_level :  final_stall_level);
  reg[2:0] didt_throttle_level_q;
  always @(posedge didt_droop_clk_gated) begin
  	if(~didt_rstn)
  		didt_throttle_level_q <= 'd0;
  	else
  		didt_throttle_level_q <= didt_throttle_level_d;
  end
  assign didt_throttle_level = didt_throttle_level_q;
  
  assign psm_didt_dd_stat[13:0] 	= psm_counter_q;
  assign psm_didt_dd_stat[14]		= static_throttle;
  assign psm_didt_dd_stat[15]		= dyn_throttle;
  assign psm_didt_dd_stat[18:16]	= didt_throttle_level_q;
  assign psm_didt_dd_stat[31:19]	= 'd0;

  wire [19:0] dbus = {dyn_throttle,static_throttle,didt_throttle_level_q,dyn_throttle,static_throttle,psm_counter_q};
  assign  didt_dbus = {12'b0,dbus};

endmodule
