// Filename              : dma_ce_cmd.v
// Perforce changelist   : $Change$
//
// Copyright             : Trade secret of AMD, Inc.
//                         Copyright 2008, AMD, Inc., (unpublished)
//                         All rights reserved.  This notice is intended as a precaution against
//                         inadvertent publication and does not imply publication or any waiver
//                         of confidentiality.  The year included in the foregoing notice is the
//                         year of creation of the work.
//
// Name:                 : interface with F32 controller
//
// Description           : cmd receive and decode, send to other units 
//                         

`include "dma_common.vh"
`include "sdma_reg.v"
`include "f32_dma_defines.v"

module dma_ce_cmd_bc(

    input clk, 
    input resetb, 

   input oStMemLocal,
   input [40-1:0] oStMemAdrs,
   input [32-1:0] oStMemData,
   input oStMemRts,

   input [`SDMA0_GFX_RB_CNTL_RB_VMID_SIZE-1:0] ex_vmid,
   input ex_priv,
   input use_ib,
   input dma_ib_swap_enable,
   input dma_rb_swap_enable,

    input dma_data_swap_enable,
    input next_drm_ctx_restore_start,
    input [31:0] dma_drm_ivload0_rd,
    input [`SDMA0_GFX_CONTEXT_CNTL_SESSION_SEL_SIZE-1:0] session_sel,

    input cmd_info_full,
    input cmd_info1_full,
    input rreq_rtr,
    input wreq_rtr,
    input dma_back_compat_enable,
    output broadcast,
    output reg cmd_ie_enable,
    output [`CMD_PARAM_RANGE] ex_param ,
    output bc_mode, 
    output reg ex_start
	      );


   /********************************** Decoded Signals ****************************************/

reg [31:0] copy_header, copy_count, copy_base_lo, copy_base_hi, copy_base1_lo, copy_base1_hi, copy_tslice, copy_xy, copy_z,
        copy_xy1, copy_z1, copy_tslice1, copy_addr_lo, copy_addr_hi, copy_dxdy, copy_dz, copy_sindex,
	copy_dw6, copy_dw8, copy_dw11, copy_dw12, copy_dw14;
reg [`SDMA0_GFX_RB_CNTL_RB_VMID_SIZE-1:0] copy_vmid;
reg copy_priv;
wire skip_cmd, skip_aes;
//deta x/y/z increment
wire [3:0] inc_xy;
wire inc_z = 1'b1;
wire [3:0] count_inc;

always @(posedge clk) begin
  if(~resetb) begin
     {copy_header, copy_count, copy_addr_lo, copy_addr_hi, copy_base_lo, copy_base_hi, copy_base1_lo, copy_base1_hi, copy_dw6, copy_tslice, copy_tslice1, copy_dw8, copy_dw11, copy_dw12, copy_dw14, copy_xy, copy_z, copy_xy1, copy_z1, copy_sindex, copy_dxdy, copy_dz, ex_start} <= 0;
     copy_vmid <= 0;
     copy_priv <= 0;
     cmd_ie_enable <= 1;
  end
  else if(oStMemRts & oStMemLocal) begin
    case(oStMemAdrs)
     `SDMA_PKT_HEADER_HEADER_OFFSET: begin copy_header <= oStMemData;   cmd_ie_enable <= 1'b1; end
     `COPY_ENGINE_PKT_COUNT_OFFSET, `COPY_ENGINE_PKT_INDEX_COUNT_OFFSET: copy_count <= oStMemData + count_inc;
     `COPY_ENGINE_PKT_LINEAR_ADDR_LO_OFFSET: copy_addr_lo <= oStMemData;
     `COPY_ENGINE_PKT_LINEAR_ADDR_HI_OFFSET: copy_addr_hi <= oStMemData;
     `COPY_ENGINE_PKT_TILE_ADDR_LO_OFFSET: copy_base_lo <= oStMemData;
     `COPY_ENGINE_PKT_TILE_ADDR_HI_OFFSET: copy_base_hi <= oStMemData;
     `COPY_ENGINE_PKT_DW_6_OFFSET: copy_dw6 <= oStMemData;
     `COPY_ENGINE_PKT_DW_7_OFFSET: copy_tslice <= oStMemData;
     `COPY_ENGINE_PKT_DW_8_OFFSET: copy_dw8 <= oStMemData;
     `COPY_ENGINE_PKT_DW_9_OFFSET: copy_xy <= oStMemData;
     `COPY_ENGINE_PKT_DW_10_OFFSET: copy_z <= oStMemData;
     `COPY_ENGINE_PKT_DW_11_OFFSET: copy_dw11 <= oStMemData;
     `COPY_ENGINE_PKT_DW_12_OFFSET: copy_dw12 <= oStMemData;
     `COPY_ENGINE_PKT_DW_13_OFFSET: copy_tslice1 <= oStMemData;
     `COPY_ENGINE_PKT_DW_14_OFFSET: copy_dw14 <= oStMemData;
     `COPY_ENGINE_PKT_DW_15_OFFSET: copy_xy1 <= oStMemData;
     `COPY_ENGINE_PKT_DW_16_OFFSET: copy_z1 <= oStMemData;
     `COPY_ENGINE_PKT_DW_17_OFFSET: copy_dxdy <= oStMemData + {inc_xy, 12'b0, inc_xy};
     `COPY_ENGINE_PKT_DW_18_OFFSET: copy_dz <= oStMemData + inc_z;
     `COPY_ENGINE_PKT_START_INDEX_OFFSET: copy_sindex <= oStMemData;
     `COPY_ENGINE_PKT_DST2_ADDR_LO_OFFSET: copy_base1_lo <= oStMemData;
     `COPY_ENGINE_PKT_DST2_ADDR_HI_OFFSET: copy_base1_hi <= oStMemData;
     `StDisableIE: cmd_ie_enable <= 0;
    endcase
    if(oStMemAdrs == `COPY_START && !ex_start && !skip_cmd && !skip_aes) begin
       ex_start <= 1;
       copy_vmid <= ex_vmid;
       copy_priv <= ex_priv;
    end
    else if(rreq_rtr && wreq_rtr && !cmd_info_full && !cmd_info1_full) ex_start <= 0;
    end
  else if(rreq_rtr && wreq_rtr && !cmd_info_full && !cmd_info1_full) begin
    ex_start <= 0;
  end
end


   reg [`SDMA0_GFX_CONTEXT_CNTL_SESSION_SEL_SIZE-1:0] kc_session_sel;
   always @(posedge clk) begin
      if(~resetb) begin
	 kc_session_sel <= 0;
      end
      else begin
	 kc_session_sel <= oStMemRts && oStMemLocal && oStMemAdrs==`DRM_KEY_PKT_HEADER ? 
                oStMemData[`SDMA_PKT_HEADER_UNWRAP_RANGE] ? session_sel : 3 :
                next_drm_ctx_restore_start ?
                        dma_drm_ivload0_rd[`DRM_CLIENTPORT_PKT_HEADER_UNWRAP_KEY_RANGE] ? session_sel : 3 :
                kc_session_sel;
      end
   end


   wire [3:0] cmd_op = copy_header[`SDMA_PKT_HEADER_OP_RANGE];  //only 4 bits of op used 
   wire [7:0] copy_sub_code = copy_header[`SDMA_PKT_HEADER_SUB_OP_RANGE];  //only 4 bits of sub op used 
   wire video = (copy_sub_code == `SUBOP_COPY_TILED || copy_sub_code == `SUBOP_COPY_TILED_BC) && copy_header[`SDMA_PKT_HEADER_VIDEOCOPY_RANGE];
   assign broadcast = (copy_sub_code == `SUBOP_COPY_LINEAR || copy_sub_code == `SUBOP_COPY_LINEAR_BC || copy_sub_code == `SUBOP_COPY_TILED || copy_sub_code == `SUBOP_COPY_TILED_BC) && 
		copy_header[`SDMA_PKT_HEADER_BROADCAST_RANGE];
   wire structt = copy_sub_code == `SUBOP_COPY_SOA;
   wire t2t = copy_sub_code == `SUBOP_COPY_T2T_SUB_WIND || copy_sub_code == `SUBOP_COPY_T2T_SUB_WIND_BC;
   wire subwin = copy_sub_code == `SUBOP_COPY_LINEAR_SUB_WIND || copy_sub_code == `SUBOP_COPY_LINEAR_SUB_WIND_BC || copy_sub_code == `SUBOP_COPY_TILED_SUB_WIND || copy_sub_code == `SUBOP_COPY_TILED_SUB_WIND_BC || t2t;

   wire crc = 0;  //No CRC copy in CI
   wire aes = cmd_op==`OP_COPY ? !subwin && copy_header[`SDMA_PKT_HEADER_AES_RANGE] : copy_header[`SDMA_PKT_WRITE_UNTILED_AES_RANGE];
   wire aes_cbc = aes && (cmd_op==`OP_COPY ? copy_header[`SDMA_PKT_HEADER_ALG_RANGE]==`ALG_AES_CBC : 
		copy_header[`SDMA_PKT_WRITE_UNTILED_ALG_RANGE]==`ALG_AES_CBC);
   wire trns = cmd_op==`OP_COPY && aes && copy_header[`SDMA_PKT_HEADER_ALG_RANGE]==3;
   assign skip_aes = trns && kc_session_sel == 3 || aes && !trns && kc_session_sel != 3;

   wire linear = copy_sub_code == `SUBOP_COPY_LINEAR || copy_sub_code == `SUBOP_COPY_LINEAR_BC || copy_sub_code == `SUBOP_COPY_LINEAR_SUB_WIND || copy_sub_code == `SUBOP_COPY_LINEAR_SUB_WIND_BC;
   //wire [`CMD_COUNT_SIZE-1+1:0] ssize = linear ? copy_count[`CMD_COUNT_SIZE-1:0] : {copy_count[`CMD_COUNT_SIZE-3:0],2'b0}; //unit: byte
   wire [`CMD_COUNT_SIZE-1+1:0] ssize = copy_count[`CMD_COUNT_SIZE-1+1:0]; //unit: byte
   wire [`CMD_ADDR_SIZE-1:0] taddr =  t2t ? {copy_addr_hi, copy_addr_lo} : {copy_base_hi, copy_base_lo}; //t2t: src. tile: base. linear: dst
   wire [`CMD_ADDR_SIZE-1:0] taddr1 = t2t ? {copy_base_hi, copy_base_lo} : {copy_base1_hi, copy_base1_lo}; //t2t: dst. broadcast: 2nd base
   wire [`CMD_ADDR_SIZE-1:0] laddr = {copy_addr_hi, copy_addr_lo};     
   wire [`SDMA_PKT_COPY_TILED_BC_MIT_MODE_SIZE-1:0] nondisp = structt ? 0 :  
                                                        copy_dw8[`SDMA_PKT_COPY_TILED_BC_MIT_MODE_RANGE]==2 ? 1 : //mt_mode=1,2 -> nondisp=1
							copy_dw8[`SDMA_PKT_COPY_TILED_BC_MIT_MODE_RANGE];
   wire [`SDMA_PKT_COPY_TILED_BC_MIT_MODE_SIZE-1:0] nondisp1 = //Used in Rotate tile T2T
                                                        copy_dw14[`SDMA_PKT_COPY_TILED_BC_MIT_MODE_RANGE]==2 ? 1 : //mt_mode=1,2 -> nondisp=1
							copy_dw14[`SDMA_PKT_COPY_TILED_BC_MIT_MODE_RANGE];
   wire  t2l = cmd_op==`OP_COPY && !linear & copy_header[`SDMA_PKT_HEADER_DETILE_RANGE];
   wire [1:0] dswap= dma_data_swap_enable ? (t2l || linear || t2t ? copy_dw11[`COPY_ENGINE_PKT_LINEAR_SW_RANGE] :
		copy_dw11[`COPY_ENGINE_PKT_TILE_SW_RANGE]) : 0;
   wire [1:0] sswap= cmd_op==`OP_WRITE && (use_ib ? dma_ib_swap_enable : dma_rb_swap_enable) ? `DMA_MC_SWAP_8IN32 :
		dma_data_swap_enable ? (~t2l && ~linear && ~t2t ? copy_dw11[`COPY_ENGINE_PKT_LINEAR_SW_RANGE] :
                copy_dw11[`COPY_ENGINE_PKT_TILE_SW_RANGE]) : 0;
   wire dhash = t2l || linear ? copy_dw11[`COPY_ENGINE_PKT_LINEAR_HA_RANGE] : copy_dw11[`COPY_ENGINE_PKT_TILE_HA_RANGE];
   wire shash = ~t2l && ~linear ? copy_dw11[`COPY_ENGINE_PKT_LINEAR_HA_RANGE] : copy_dw11[`COPY_ENGINE_PKT_TILE_HA_RANGE];
   wire [1:0] swap1 = dma_data_swap_enable ? copy_dw11[`COPY_ENGINE_PKT_DST2_SW_RANGE] : 0;
   wire hash1 = copy_dw11[`COPY_ENGINE_PKT_DST2_HA_RANGE];
   assign bc_mode = copy_sub_code[4];// || (linear && (dhash || shash || hash1));
   //tci interface not applicable in CI
/*   wire dtci = t2l ? copy_dw11[`COPY_ENGINE_PKT_LINEAR_TCI_RANGE] : copy_dw11[`COPY_ENGINE_PKT_TILE_TCI_RANGE];
   wire stci = ~t2l ? copy_dw11[`COPY_ENGINE_PKT_LINEAR_TCI_RANGE] : copy_dw11[`COPY_ENGINE_PKT_TILE_TCI_RANGE];
   wire dvol = t2l ? copy_dw11[`COPY_ENGINE_PKT_LINEAR_VOL_RANGE] : copy_dw11[`COPY_ENGINE_PKT_TILE_VOL_RANGE];
   wire svol = ~t2l ? copy_dw11[`COPY_ENGINE_PKT_LINEAR_VOL_RANGE] : copy_dw11[`COPY_ENGINE_PKT_TILE_VOL_RANGE];
   wire dpol = t2l ? copy_dw11[`COPY_ENGINE_PKT_LINEAR_POL_RANGE] : copy_dw11[`COPY_ENGINE_PKT_TILE_POL_RANGE];
   wire spol = ~t2l ? copy_dw11[`COPY_ENGINE_PKT_LINEAR_POL_RANGE] : copy_dw11[`COPY_ENGINE_PKT_TILE_POL_RANGE];
   wire tci1 = copy_dw11[`COPY_ENGINE_PKT_DST2_TCI_RANGE];
   wire vol1 = copy_dw11[`COPY_ENGINE_PKT_DST2_VOL_RANGE];
   wire pol1 = copy_dw11[`COPY_ENGINE_PKT_DST2_POL_RANGE];*/
   assign count_inc = structt ? 4'd0 : cmd_op==`OP_WRITE || copy_sub_code == `SUBOP_COPY_TILED_BC ? 4'd4 : 4'd1;
   assign inc_xy = t2t ? 4'd8 : 4'd1;
   wire dtci=0, stci=0, dvol=0, svol=0, dpol=0, spol=0, tci1=0, vol1=0, pol1=0;

   wire [`SDMA_PKT_COPY_TILED_BC_ELEMENT_SIZE_SIZE-1:0] psize = linear && subwin ? copy_header[`SDMA_PKT_HEADER_ELEMENTSIZE_RANGE] :
							copy_dw8[`SDMA_PKT_COPY_TILED_BC_ELEMENT_SIZE_RANGE];
   wire [`SDMA_PKT_COPY_TILED_BC_BANK_W_SIZE-1:0]  bwidth = copy_dw8[`SDMA_PKT_COPY_TILED_BC_BANK_W_RANGE];
   wire [`SDMA_PKT_COPY_TILED_BC_BANK_H_SIZE-1:0] bheight = copy_dw8[`SDMA_PKT_COPY_TILED_BC_BANK_H_RANGE];
   wire [`SDMA_PKT_COPY_TILED_BC_MAT_ASPT_SIZE-1:0]   aspect = copy_dw8[`SDMA_PKT_COPY_TILED_BC_MAT_ASPT_RANGE];
   wire [`SDMA_PKT_COPY_TILED_BC_NUM_BANK_SIZE-1:0]    numbank = copy_dw8[`SDMA_PKT_COPY_TILED_BC_NUM_BANK_RANGE];
   wire [`SDMA_PKT_COPY_TILED_BC_PIPE_CONFIG_SIZE-1:0] pipecfg = bc_mode & dma_back_compat_enable & (copy_dw8[`SDMA_PKT_COPY_TILED_BC_PIPE_CONFIG_RANGE] == 18) ? 12 : copy_dw8[`SDMA_PKT_COPY_TILED_BC_PIPE_CONFIG_RANGE];
   wire [`SDMA_PKT_COPY_TILED_BC_ARRAY_MODE_SIZE-1:0]       tmode = copy_dw8[`SDMA_PKT_COPY_TILED_BC_ARRAY_MODE_RANGE];
   wire [`SDMA_PKT_COPY_TILED_BC_TILESPLIT_SIZE_SIZE-1:0]     tsplit = copy_dw8[`SDMA_PKT_COPY_TILED_BC_TILESPLIT_SIZE_RANGE];
   wire [`SDMA_PKT_COPY_TILED_BC_BANK_W_SIZE-1:0]  bwidth1 = copy_dw14[`SDMA_PKT_COPY_TILED_BC_BANK_W_RANGE];
   wire [`SDMA_PKT_COPY_TILED_BC_BANK_H_SIZE-1:0] bheight1 = copy_dw14[`SDMA_PKT_COPY_TILED_BC_BANK_H_RANGE];
   wire [`SDMA_PKT_COPY_TILED_BC_MAT_ASPT_SIZE-1:0]   aspect1 = copy_dw14[`SDMA_PKT_COPY_TILED_BC_MAT_ASPT_RANGE];
   wire [`SDMA_PKT_COPY_TILED_BC_NUM_BANK_SIZE-1:0]    numbank1 = copy_dw14[`SDMA_PKT_COPY_TILED_BC_NUM_BANK_RANGE];
   wire [`SDMA_PKT_COPY_TILED_BC_PIPE_CONFIG_SIZE-1:0] pipecfg1 = bc_mode & dma_back_compat_enable & (copy_dw14[`SDMA_PKT_COPY_TILED_BC_PIPE_CONFIG_RANGE] == 18) ? 12 : copy_dw14[`SDMA_PKT_COPY_TILED_BC_PIPE_CONFIG_RANGE];
   wire [`SDMA_PKT_COPY_TILED_BC_ARRAY_MODE_SIZE-1:0]       tmode1 = copy_dw14[`SDMA_PKT_COPY_TILED_BC_ARRAY_MODE_RANGE];
   wire [`SDMA_PKT_COPY_TILED_BC_TILESPLIT_SIZE_SIZE-1:0]     tsplit1 = copy_dw14[`SDMA_PKT_COPY_TILED_BC_TILESPLIT_SIZE_RANGE];
   wire [`SDMA_PKT_COPY_TILED_BC_HEIGHT_SIZE-1:0] height = copy_tslice[13:0];
   wire [`SDMA_PKT_COPY_TILED_BC_PITCH_IN_TILE_SIZE-1:0] tpitch = copy_dw6[13:3];
   wire [`SDMA_PKT_COPY_TILED_BC_HEIGHT_SIZE-1:0] height1 = copy_tslice1[13:0];
   wire [`SDMA_PKT_COPY_TILED_BC_PITCH_IN_TILE_SIZE-1:0] tpitch1 = copy_dw12[13:3];
   wire [`SDMA_PKT_COPY_TILED_BC_SLICE_PITCH_SIZE-1:0] tslice;
   //lpitch changed to byte unit.
   wire [`SDMA_PKT_COPY_LINEAR_SUBWIN_BC_SRC_PITCH_SIZE+4:0] lpitch = structt || subwin ? linear ? ((copy_z[`SDMA_PKT_COPY_LINEAR_SUBWIN_BC_SRC_PITCH_RANGE]+1'b1)<<psize) : 
											 ((copy_z1[`SDMA_PKT_COPY_TILED_SUBWIN_BC_LINEAR_PITCH_RANGE]+1'b1)<<psize) :
                                                                                ((tpitch+1'b1)<< (psize+3));	//non-subwin share tpitch
   wire [`SDMA_PKT_COPY_LINEAR_SUBWIN_BC_DST_PITCH_SIZE+4:0] lpitch1 = (copy_z1[`SDMA_PKT_COPY_LINEAR_SUBWIN_BC_DST_PITCH_RANGE]+1'b1)<<psize;
   wire [`SDMA_PKT_COPY_TILED_BC_SLICE_PITCH_SIZE-1:0] tslice1;
   wire [`SDMA_PKT_COPY_TILED_BC_X_SIZE-1:0]             tx = copy_xy[`SDMA_PKT_COPY_TILED_BC_X_RANGE];
   wire [`SDMA_PKT_COPY_TILED_BC_Y_SIZE-1:0]             ty = copy_xy[`SDMA_PKT_COPY_TILED_BC_Y_RANGE];
   wire [`SDMA_PKT_COPY_TILED_BC_Z_SIZE-1:0]             tz = copy_z[`SDMA_PKT_COPY_TILED_BC_Z_RANGE];
   wire [`SDMA_PKT_COPY_TILED_BC_X_SIZE-1:0]             tx1 = copy_xy1[`SDMA_PKT_COPY_TILED_BC_X_RANGE];
   wire [`SDMA_PKT_COPY_TILED_BC_Y_SIZE-1:0]             ty1 = copy_xy1[`SDMA_PKT_COPY_TILED_BC_Y_RANGE];
   wire [`SDMA_PKT_COPY_TILED_BC_Z_SIZE-1:0]             tz1 = copy_z1[`SDMA_PKT_COPY_TILED_BC_Z_RANGE];
   wire [`SDMA_PKT_COPY_TILED_BC_X_SIZE-1+1:0]             dx = copy_dxdy[1+`SDMA_PKT_COPY_TILED_SUBWIN_BC_RECT_X_RANGE];
   wire [`SDMA_PKT_COPY_TILED_BC_Y_SIZE-1+1:0]             dy = copy_dxdy[1+`SDMA_PKT_COPY_TILED_SUBWIN_BC_RECT_Y_RANGE];
   wire [`SDMA_PKT_COPY_TILED_BC_Z_SIZE-1+1:0]             dz = copy_dz[1+`SDMA_PKT_COPY_TILED_SUBWIN_BC_RECT_Z_RANGE];

   wire [`SDMA_PKT_COPY_STRUCT_STRIDE_SIZE-1:0]    stride = copy_z[`SDMA_PKT_COPY_STRUCT_STRIDE_RANGE];
   wire [`SDMA_PKT_COPY_STRUCT_START_INDEX_SIZE-1:0] sindex = copy_sindex[`SDMA_PKT_COPY_STRUCT_START_INDEX_RANGE];
   wire [`SDMA_PKT_COPY_STRUCT_COUNT_SIZE-1:0] cindex = copy_count[`SDMA_PKT_COPY_STRUCT_COUNT_RANGE];

   //lslice changed to byte unit.
   reg [`SDMA_PKT_COPY_TILED_SUBWIN_LINEAR_SLICE_PITCH_SIZE+4:0] lslice;
   reg [`SDMA_PKT_COPY_TILED_SUBWIN_LINEAR_SLICE_PITCH_SIZE+4:0] lslice1;
   always @(posedge clk) begin
     if (bc_mode) begin
     lslice <= linear ?
                ((copy_tslice[`SDMA_PKT_COPY_TILED_SUBWIN_LINEAR_SLICE_PITCH_RANGE]+1'b1)<<psize) :
                ((copy_tslice1[`SDMA_PKT_COPY_TILED_SUBWIN_LINEAR_SLICE_PITCH_RANGE]+1'b1)<<psize);
     lslice1 <= (copy_tslice1[`SDMA_PKT_COPY_TILED_SUBWIN_LINEAR_SLICE_PITCH_RANGE]+1'b1)<<psize;
     end
   end

   reg [`SDMA_PKT_COPY_TILED_BC_PITCH_IN_TILE_SIZE:0] tpitch_p1;
   reg [`SDMA_PKT_COPY_TILED_BC_PITCH_IN_TILE_SIZE:0] tpitch1_p1;

   reg [`SDMA_PKT_COPY_TILED_BC_HEIGHT_SIZE-3:0] height_p1;
   reg [`SDMA_PKT_COPY_TILED_BC_HEIGHT_SIZE-3:0] height1_p1;
   reg [`SDMA_PKT_COPY_TILED_BC_SLICE_PITCH_SIZE+1:0] tslice_p1;
   reg [`SDMA_PKT_COPY_TILED_BC_SLICE_PITCH_SIZE+1:0] tslice1_p1;

   always @(posedge clk) begin
      if(~resetb) begin
	 tpitch_p1 <= 0;
	 tpitch1_p1 <= 0;
	 height_p1 <= 0;
	 height1_p1 <= 0;
	 tslice_p1 <= 0;
	 tslice1_p1 <= 0;
      end
      else begin
	 tpitch_p1 <= tpitch + 1;
	 tpitch1_p1 <= tpitch1 + 1;
	 height_p1 <= height[13:3] + 1;
	 height1_p1 <= height1[13:3] + 1;
	 tslice_p1 <= tpitch_p1 * height_p1;
	 tslice1_p1 <= tpitch1_p1 * height1_p1;
      end
   end
   assign tslice1 = tslice1_p1 - 1;
   assign tslice = tslice_p1 - 1;

   //skip zero count copy
   assign skip_cmd = !structt && !subwin && ssize==0;


   assign ex_param = {video, broadcast, dhash, shash, dswap, sswap, dtci, stci, dvol, svol, dpol, spol, hash1, swap1, tci1, vol1, pol1, 
	copy_vmid, copy_priv, structt, stride, sindex, cindex, crc, trns, aes_cbc, aes, linear, subwin, ssize, 
	taddr, taddr1, laddr, nondisp, nondisp1, t2t, t2l, psize, bwidth, bheight, aspect, numbank, pipecfg, tmode, height, tpitch, tslice,
	lpitch, lslice, tsplit, bwidth1, bheight1, aspect1, numbank1, pipecfg1, tmode1, tsplit1, height1, tpitch1, tslice1, lpitch1, lslice1,
	tx, ty, tz, tx1, ty1, tz1, dx, dy, dz};

endmodule 

