`include "dma_common.vh"


module dma_ce_in_bc(

    input clk,
    input resetb,

    input bc_mode,
		      // dma_mc_rd interface
    input 			       mc_rd_ce_vld,
    input [`MC_DMA_RDRET_DATA_SIZE-1:0] mc_rd_ce_data,
    input [`MC_DMA_RDRET_TAG_SIZE-1:0] 	mc_rd_ce_tag,
    input [`MC_DMA_RDRET_TID_SIZE-1:0] 	mc_rd_ce_tid,
    input [`MC_DMA_RDRET_TID_SIZE-1:0] 	MC_DMA_rdret_tid_d1,
    input [1:0] 			mc_rd_ce_nack,
    input [`MC_DMA_RDRET_TAG_SIZE-1:0] 	MC_DMA_rdret_tag,
    input [`MC_DMA_RDRET_TAG_SIZE-1:0] 	MC_DMA_rdret_tag_d1,
    input 				MC_DMA_rdret_vld_d1,
    input 				MC_DMA_rdret_vld,
    input                               mc_rdret_ecc,

    input [`DMA_DATA_LUT_WIDTH-1:0]     ram_data_lut_dout,
    output [`DMA_DATA_LUT_DEPTH-1:0]    ram_data_lut_raddr,
    output                              ram_data_lut_re,

`ifdef SDMA__UTC_L1__1
    input				dma_utc_l1_enable,
    input			   	l1_alloc_vld,
    input [`DMA_MC_RDREQ_CE_ALLOC_WIDTH-1+14:0]	l1_alloc_data,
    //input [`DMA_RD_DATA_BUF_DEPTH:0] 	saved_alloc_wptr,
`endif
		      // dma_ce_rreq interface
    output [`DMA_RD_DATA_BUF_DEPTH:0] 	data_rreq_alloc_free,
    output [`DMA_RD_DATA_BUF_DEPTH:0] 	data_rreq_alloc_used,
    output [`DMA_RD_DATA_BUF_DEPTH:0] 	data_rreq_alloc_wptr,
    output [3:0]                                       data_rreq_alloc_wptr_inc ,
    output reg[2:0] 			next_quarter_entry,
    input 				ce_rd_vld,
    input [`DMA_MC_RDREQ_TAG_SIZE-1:0]  ce_rd_tag,
    input [`DMA_MC_RDREQ_DWORD_SIZE-1:0] ce_rd_size,
    input 				 ce_rd_t2l_last,

    input ram_rd_data_rptr_inc,
    output reg dma_ce_in_idle,

    input [`SDMA0_CHICKEN_BITS_CE_DFIFO_WATERMARK_SIZE-1:0] 	dfifo_watermark,

    output reg [`DMA_RD_DATA_BUF_ENTRIES-1:0] fifo_entry_vld,
    output reg [`DMA_RD_DATA_BUF_ENTRIES-1:0] fifo_entry_ecc,

    output reg[`DMA_RD_DATA_DATA_SIZE-1:0]        ram_mbank_data_din,
    output reg[8*`DMA_RD_DATA_BUF_DEPTH-1:0]                             ram_mbank_data_waddr,
    output reg [7:0] ram_mbank_data_wmask,
    output [7:0]                                 ram_mbank_data_we
		      );   


   ///////////////////
   // dma_ex interface
   ///////////////////
   wire [`DMA_RD_DATA_BUF_DEPTH:0] 		    alloc_free, alloc_used;
   reg [`DMA_RD_DATA_BUF_DEPTH:0] 		    next_alloc_wptr;
   wire [3:0]                               alloc_wptr_inc;
   assign data_rreq_alloc_free = alloc_free; 
   assign data_rreq_alloc_used = alloc_used;
   assign data_rreq_alloc_wptr = next_alloc_wptr;
   assign data_rreq_alloc_wptr_inc = alloc_wptr_inc;



   ////////////////////////////
   // Data buffer read pointer
   ////////////////////////////
   reg [`DMA_RD_DATA_BUF_DEPTH:0]     alloc_wptr, rptr; 
   reg [`DMA_RD_DATA_BUF_ENTRIES-1:0] fifo_entry_vld2, fifo_entry_vld1, fifo_entry_vld0;
   
   //Buffer rptr/wptr pointers refer to address of each MC-width 32-byte entry
   //next_rptr is the read address of buffer
   wire [`DMA_RD_DATA_BUF_DEPTH:0]	   next_rptr = ram_rd_data_rptr_inc ? rptr[`DMA_RD_DATA_BUF_DEPTH-1:0] == `DMA_RD_DATA_BUF_ENTRIES-1 ? 
							{rptr[`DMA_RD_DATA_BUF_DEPTH]+1,{`DMA_RD_DATA_BUF_DEPTH{1'b0}}} : rptr + 1 : rptr; 
   wire 				   empty = (alloc_wptr == rptr);

   wire 						   next_we;
   reg [`DMA_RD_DATA_BUF_DEPTH-1:0] 			   next_wptr;

   reg 			       mc_rd_ce_vld_d, mc_rd_ce_vld_d2;
   reg [`MC_DMA_RDRET_DATA_SIZE-1:0] mc_rd_ce_data_d, mc_rd_ce_data_d2;
   reg [`MC_DMA_RDRET_TAG_SIZE-1:0] mc_rd_ce_tag_d;
   reg [`MC_DMA_RDRET_TID_SIZE-1:0] mc_rd_ce_tid_d, mc_rd_ce_tid_d2;
   always @(posedge clk) 
     if (~resetb) begin
       mc_rd_ce_vld_d <= 0;
       mc_rd_ce_vld_d2 <= 0;
       mc_rd_ce_data_d<= 0; 
       mc_rd_ce_data_d2 <= 0; 
       mc_rd_ce_tag_d <= 0;
       mc_rd_ce_tid_d <= 0;
       mc_rd_ce_tid_d2 <= 0;
     end
     else begin
       mc_rd_ce_vld_d <= mc_rd_ce_vld;
       mc_rd_ce_vld_d2 <= mc_rd_ce_vld_d;
       if (mc_rd_ce_vld)
       mc_rd_ce_data_d <= mc_rd_ce_data;
       if (mc_rd_ce_vld_d)
       mc_rd_ce_data_d2 <= mc_rd_ce_data_d;
       mc_rd_ce_tag_d <= mc_rd_ce_tag;
       mc_rd_ce_tid_d <= mc_rd_ce_tid;
       mc_rd_ce_tid_d2 <= mc_rd_ce_tid_d;
     end 

   reg[`MC_DMA_RDRET_DATA_TAG_ID_RANGE] last_tag_id;   // 10-bit ID   
   reg[2:0] start_tid, start_tid_d;
   wire is_start_tid;
   reg new_tid, is_start_tid_d, is_start_tid_d2;

`ifdef SDMA__UTC_L1__1
   wire ce_rd_atc_vld = dma_utc_l1_enable  ? l1_alloc_vld & l1_alloc_data[36] : ce_rd_vld;
   wire [`DMA_MC_RDREQ_TAG_SIZE-1:0]  ce_rd_atc_tag = dma_utc_l1_enable ? l1_alloc_data[50:37] : ce_rd_tag;
`endif

   always @(posedge clk) 
      if (~resetb) begin
        last_tag_id <= 0;
	start_tid <= 0;
        start_tid_d <= 0;
	new_tid <= 1;
        is_start_tid_d <= 0;
        is_start_tid_d2 <= 0;
      end
      else begin
        if(is_start_tid && MC_DMA_rdret_tag_d1[`MC_DMA_RDRET_RB_TAG_TYPE_RANGE]==`TAG_TYPE_DATA) begin
	  last_tag_id <= {MC_DMA_rdret_tag_d1[`MC_DMA_RDRET_DATA_TAG_ID_RANGE]};
	  start_tid <= MC_DMA_rdret_tid_d1;
	  new_tid <= 0;
        end
`ifdef SDMA__UTC_L1__1
        else if(dma_ce_in_idle || ce_rd_atc_vld && ce_rd_atc_tag[`MC_DMA_RDRET_DATA_TAG_ID_RANGE] == last_tag_id)
`else
        else if(dma_ce_in_idle || ce_rd_vld && ce_rd_tag[`MC_DMA_RDRET_DATA_TAG_ID_RANGE] == last_tag_id)
`endif
        //else if(dma_ce_in_idle)
	  new_tid <= 1;
        is_start_tid_d <= is_start_tid;
        is_start_tid_d2 <= is_start_tid_d;
        start_tid_d <= start_tid;
      end

   //MC returns are in order and tags are never interleaved; must change this implementation if MC team changes its mind
   assign is_start_tid = (MC_DMA_rdret_tag_d1[`MC_DMA_RDRET_DATA_TAG_ID_RANGE] != last_tag_id || new_tid) && MC_DMA_rdret_vld_d1;
   
   //#################################### data buffer LUT #########

   assign ram_data_lut_raddr = MC_DMA_rdret_tag[`MC_DMA_RDRET_DATA_TAG_ID_RANGE];
   assign ram_data_lut_re = MC_DMA_rdret_vld && MC_DMA_rdret_tag[`MC_DMA_RDRET_RB_TAG_TYPE_RANGE]==`TAG_TYPE_DATA;

   reg ram_data_lut_re_d, ram_data_lut_re_d1;

   always @(posedge clk) begin
     ram_data_lut_re_d <= ram_data_lut_re;
     ram_data_lut_re_d1 <= ram_data_lut_re_d;
   end

   reg [`DMA_DATA_LUT_WIDTH-1:0]     ram_data_lut_dout_d;
   wire [`DMA_RD_DATA_BUF_DEPTH-1:0] write_entry; 
   wire [1:0] granu;
   wire ie, t2l, t2t;
   wire [1:0] micro_tile_mode;
   wire[`SDMA_PKT_COPY_TILED_BC_MIT_MODE_SIZE-1:0] nondisp; 
   wire[2:0] psize, nr_dwords, unalign;
   wire[3:0] offset_ini;
   reg [3:0] offset, offset_d;
   wire[4:0] psize_y;
   wire burst_start;
   wire       structt = 1'b0;
   reg 	      struct_d;
   wire [1:0] y_lo;
   wire       z_lo;
   assign {burst_start, unalign, offset_ini, z_lo, y_lo, ie, t2l, micro_tile_mode, psize, last, granu, write_entry} = ram_data_lut_re_d1 ? ram_data_lut_dout : 0;
   assign nr_dwords = unalign;  //unalign used in ie. nr_dwords used in normal
   wire [2:0] detile_quarter = &granu ? 0 : offset_ini[2:0];  //offset used in ie. detile_quarter used in normal
   assign psize_y = {psize[1:0], z_lo, y_lo[1:0]};
   //assign t2t = nondisp==5;  //to save lut bits, use special value
   assign t2t = micro_tile_mode == 2;
   assign nondisp = micro_tile_mode == 3 ? 4 : micro_tile_mode;

   //when burst return, generate offset for each 256b
   always @(posedge clk) begin
        if(is_start_tid_d && (t2l || t2t))
	  offset_d <= offset_ini + 1;
        else if(mc_rd_ce_vld && bc_mode)
	  offset_d <= offset_d + 1;
        offset <= ~(t2l || t2t) || is_start_tid_d && (t2l || t2t) ? offset_ini : offset_d;
   end

   //#################################### LUT output registered (for timing) #########
   
   wire [`DMA_RD_DATA_BUF_DEPTH-1:0] write_entry_d; 
   wire [1:0] granu_d;
   wire ie_d, t2l_d, t2t_d;
   wire [1:0] micro_tile_mode_d;
   wire[`SDMA_PKT_COPY_TILED_BC_MIT_MODE_SIZE-1:0] nondisp_d; 
   wire[2:0] psize_d, nr_dwords_d, unalign_d;
   wire[3:0] offset_ini_d;
   wire[4:0] psize_y_d;
   wire burst_start_d;
   wire       structt_d;
   wire [1:0] y_lo_d;
   wire       z_lo_d;
   assign {burst_start_d, unalign_d, offset_ini_d, z_lo_d, y_lo_d, ie_d, t2l_d, micro_tile_mode_d, psize_d, last_d, granu_d, write_entry_d} = ram_data_lut_dout_d;
   assign nr_dwords_d = unalign_d;  //unalign used in ie. nr_dwords used in normal
   wire [2:0] detile_quarter_d = &granu_d ? 0 : offset_ini_d[2:0];  //offset used in ie. detile_quarter used in normal
   assign t2t_d = micro_tile_mode_d == 2;//nondisp_d==5;  //to save lut bits, use special value
   assign nondisp_d = micro_tile_mode_d == 3 ? 4 : micro_tile_mode_d;
   always @(posedge clk) begin
     if (ram_data_lut_re_d1) begin
     ram_data_lut_dout_d <= ram_data_lut_dout;
     end
     struct_d <= structt;
   end

   //========================================================================
   // Data Buffer wptr, allocation
   //========================================================================

   reg [2:0] 			quarter_entry;

   always @(posedge clk) begin
      if (~resetb) begin
         alloc_wptr <= 0;
         rptr <= 0;
	 quarter_entry <= 0;
      end
      else begin
         alloc_wptr <= next_alloc_wptr;
         rptr <= next_rptr;
	 quarter_entry <= next_quarter_entry;
      end
   end
   
   //wptr in a burst, generated from write_entry
   always @(posedge clk) 
      if (~resetb) begin
         next_wptr <= 0;
      end
      else begin
         next_wptr <= !is_start_tid_d && write_entry + mc_rd_ce_tid_d - start_tid_d >= `DMA_RD_DATA_BUF_ENTRIES ? 
		write_entry + mc_rd_ce_tid_d - start_tid_d - `DMA_RD_DATA_BUF_ENTRIES :
		write_entry + (is_start_tid_d ? 0 : mc_rd_ce_tid_d - start_tid_d);
      end

   assign next_we = bc_mode && mc_rd_ce_vld_d2; 

`ifdef SDMA__UTC_L1__1
   reg [`DMA_RD_DATA_BUF_DEPTH:0] 	            l1_alloc_wptr;
   wire [`DMA_RD_DATA_BUF_DEPTH:0] 	            l1_alloc_wptr_merge;
   wire [`DMA_RD_DATA_BUF_DEPTH:0] saved_alloc_wptr_temp = l1_alloc_data[`DMA_RD_DATA_BUF_DEPTH-1:0] + l1_alloc_data[35:32];
   reg [`DMA_RD_DATA_BUF_DEPTH:0] saved_alloc_wptr;

   always @* begin
        saved_alloc_wptr[`DMA_RD_DATA_BUF_DEPTH:`DMA_RD_DATA_BUF_DEPTH] = l1_alloc_wptr[`DMA_RD_DATA_BUF_DEPTH:`DMA_RD_DATA_BUF_DEPTH];
        saved_alloc_wptr[`DMA_RD_DATA_BUF_DEPTH-1:0] = saved_alloc_wptr_temp[`DMA_RD_DATA_BUF_DEPTH-1:0]; 
     if(&saved_alloc_wptr_temp[`DMA_RD_DATA_BUF_DEPTH-1:`DMA_RD_DATA_BUF_DEPTH-2]) begin
        saved_alloc_wptr[`DMA_RD_DATA_BUF_DEPTH:`DMA_RD_DATA_BUF_DEPTH] = ~l1_alloc_wptr[`DMA_RD_DATA_BUF_DEPTH:`DMA_RD_DATA_BUF_DEPTH];
        saved_alloc_wptr[`DMA_RD_DATA_BUF_DEPTH-1:0] = {2'b0, saved_alloc_wptr_temp[`DMA_RD_DATA_BUF_DEPTH-3:0]};
     end
   end

   assign l1_alloc_wptr_merge = l1_alloc_vld && l1_alloc_data[36] ? saved_alloc_wptr : l1_alloc_wptr;
   always @(posedge clk) 
      if (~resetb) begin
         l1_alloc_wptr <= 0;
      end
      else begin
         if (l1_alloc_vld && l1_alloc_data[36])
            l1_alloc_wptr <= saved_alloc_wptr; 
      end
`endif

   wire [3:0] ce_rd_size_quarter = ce_rd_size[2:0] + quarter_entry;
   assign alloc_wptr_inc = ce_rd_vld ? ce_rd_t2l_last ? 1 : ce_rd_size[`DMA_MC_RDREQ_DWORD_SIZE-1:3] + ce_rd_size_quarter[3] : 0;

   // Allocate fifo entries for MC read return data
   always @* begin
     {next_alloc_wptr, next_quarter_entry} = 
		 (ce_rd_t2l_last && ce_rd_vld) ? {alloc_wptr + 1'b1, 3'b0} : 
		 ({alloc_wptr, quarter_entry} + (ce_rd_vld ? ce_rd_size[`DMA_MC_RDREQ_DWORD_SIZE-1:0] : 0)); 
		// aligned copy, convert dword req_size to MC/fifo size
     if(next_alloc_wptr[`DMA_RD_DATA_BUF_DEPTH-1:0] >= `DMA_RD_DATA_BUF_ENTRIES)
	next_alloc_wptr = next_alloc_wptr + ({1'b1, {`DMA_RD_DATA_BUF_DEPTH{1'b0}}} - `DMA_RD_DATA_BUF_ENTRIES);
   end

`ifdef SDMA__UTC_L1__1
   wire [`DMA_RD_DATA_BUF_DEPTH:0] 		    utcl1_alloc_wptr = dma_utc_l1_enable ? l1_alloc_wptr_merge : next_alloc_wptr;

   assign alloc_free = (utcl1_alloc_wptr[`DMA_RD_DATA_BUF_DEPTH] == rptr[`DMA_RD_DATA_BUF_DEPTH]) ?
		       ((`DMA_RD_DATA_BUF_ENTRIES >> dfifo_watermark) - utcl1_alloc_wptr[`DMA_RD_DATA_BUF_DEPTH-1:0] + rptr[`DMA_RD_DATA_BUF_DEPTH-1:0]) :
		       ((`DMA_RD_DATA_BUF_ENTRIES >> dfifo_watermark) - `DMA_RD_DATA_BUF_ENTRIES + rptr[`DMA_RD_DATA_BUF_DEPTH-1:0] - utcl1_alloc_wptr[`DMA_RD_DATA_BUF_DEPTH-1:0]);
   assign alloc_used = (utcl1_alloc_wptr[`DMA_RD_DATA_BUF_DEPTH] == rptr[`DMA_RD_DATA_BUF_DEPTH]) ?
		       (utcl1_alloc_wptr[`DMA_RD_DATA_BUF_DEPTH-1:0] - rptr[`DMA_RD_DATA_BUF_DEPTH-1:0]) :
		       (`DMA_RD_DATA_BUF_ENTRIES - rptr[`DMA_RD_DATA_BUF_DEPTH-1:0] + utcl1_alloc_wptr[`DMA_RD_DATA_BUF_DEPTH-1:0]);
`else
   assign alloc_free = (next_alloc_wptr[`DMA_RD_DATA_BUF_DEPTH] == rptr[`DMA_RD_DATA_BUF_DEPTH]) ?
		       (`DMA_RD_DATA_BUF_ENTRIES - next_alloc_wptr[`DMA_RD_DATA_BUF_DEPTH-1:0] + rptr[`DMA_RD_DATA_BUF_DEPTH-1:0]) :
		       (rptr[`DMA_RD_DATA_BUF_DEPTH-1:0] - next_alloc_wptr[`DMA_RD_DATA_BUF_DEPTH-1:0]);
   assign alloc_used = (next_alloc_wptr[`DMA_RD_DATA_BUF_DEPTH] == rptr[`DMA_RD_DATA_BUF_DEPTH]) ?
		       (next_alloc_wptr[`DMA_RD_DATA_BUF_DEPTH-1:0] - rptr[`DMA_RD_DATA_BUF_DEPTH-1:0]) :
		       (`DMA_RD_DATA_BUF_ENTRIES - rptr[`DMA_RD_DATA_BUF_DEPTH-1:0] + next_alloc_wptr[`DMA_RD_DATA_BUF_DEPTH-1:0]);
`endif
   

   /////////////////////
   // Data Buffer memory
   /////////////////////

   wire[127:0] src_t2l_data;
   wire [`DMA_RD_DATA_DATA_RANGE] next_wdata = ((t2l_d && 
			 (granu_d == `DMA_MC_wr_granularity_min_128b ||
			  granu_d == `DMA_MC_wr_granularity_min_32b ||
			  granu_d == `DMA_MC_wr_granularity_min_64b) ) ?
			src_t2l_data : mc_rd_ce_data_d2) << {detile_quarter_d,5'b0};

   reg[31:0] d7,d6,d5,d4,d3,d2,d1,d0;        //registered for timing
   reg[31:0] _d7,_d6,_d5,_d4,_d3,_d2,_d1,_d0;    //comb logic

   always @* begin
     {_d7,_d6,_d5,_d4,_d3,_d2,_d1,_d0} = mc_rd_ce_data_d;
     if(t2t && ie)   //Rotate tile T2T
       case(psize)
`ifdef SDMA__DISP_TO_ROTATE__1
         0: {_d7,_d6,_d5,_d4,_d3,_d2,_d1,_d0} = {_d7[31:24],_d3[31:24],_d5[31:24],_d1[31:24],_d7[15:8],_d3[15:8],_d5[15:8],_d1[15:8],
                                         _d7[23:16],_d3[23:16],_d5[23:16],_d1[23:16],_d7[7:0],_d3[7:0],_d5[7:0],_d1[7:0],
                                         _d6[31:24],_d2[31:24],_d4[31:24],_d0[31:24],_d6[15:8],_d2[15:8],_d4[15:8],_d0[15:8],
                                         _d6[23:16],_d2[23:16],_d4[23:16],_d0[23:16],_d6[7:0],_d2[7:0],_d4[7:0],_d0[7:0]};
         1: {_d7,_d6,_d5,_d4,_d3,_d2,_d1,_d0} = {_d7[31:16],_d3[31:16],_d7[15:0],_d3[15:0], _d6[31:16],_d2[31:16],_d6[15:0],_d2[15:0],
                                         _d5[31:16],_d1[31:16],_d5[15:0],_d1[15:0], _d4[31:16],_d0[31:16],_d4[15:0],_d0[15:0]};
         2: {_d7,_d6,_d5,_d4,_d3,_d2,_d1,_d0} = {_d7,_d3,_d6,_d2,_d5,_d1,_d4,_d0};
         3: {_d7,_d6,_d5,_d4,_d3,_d2,_d1,_d0} = {_d7,_d6,_d3,_d2,_d5,_d4,_d1,_d0};
`endif
          //VCS coverage off
         default: {_d7,_d6,_d5,_d4,_d3,_d2,_d1,_d0} = {_d7,_d6,_d5,_d4,_d3,_d2,_d1,_d0};
          //VCS coverage on
       endcase
     else if(t2l && ~structt && ie && |nondisp)    //Nondisp Pack: pack scattered 32/64/128 bit nondisp source data together
     begin
      if (nondisp == 4 && psize == 0) begin   //8bpp thick tile
       case({z_lo, offset_ini[0]})            //offset[0] set high/low 128b, z_lo set even/odd z surface
         0: {_d6,_d4,_d2,_d0} = {_d3[31:16],_d2[31:16],_d3[15:0],_d2[15:0],_d1[31:16],_d0[31:16],_d1[15:0],_d0[15:0]};
         1: {_d7,_d5,_d3,_d1} = {_d3[31:16],_d2[31:16],_d3[15:0],_d2[15:0],_d1[31:16],_d0[31:16],_d1[15:0],_d0[15:0]};
         2: {_d6,_d4,_d2,_d0} = {_d7[31:16],_d6[31:16],_d7[15:0],_d6[15:0],_d5[31:16],_d4[31:16],_d5[15:0],_d4[15:0]};
         3: {_d7,_d5,_d3,_d1} = {_d7[31:16],_d6[31:16],_d7[15:0],_d6[15:0],_d5[31:16],_d4[31:16],_d5[15:0],_d4[15:0]};
       endcase
      end
      else
       case(psize)
         0: {_d7,_d6,_d5,_d4,_d3,_d2,_d1,_d0} = {_d7[31:16],_d6[31:16],_d3[31:16],_d2[31:16],_d7[15:0],_d6[15:0],_d3[15:0],_d2[15:0],
                                         _d5[31:16],_d4[31:16],_d1[31:16],_d0[31:16],_d5[15:0],_d4[15:0],_d1[15:0],_d0[15:0]};
         1: {_d7,_d6,_d5,_d4,_d3,_d2,_d1,_d0} = {_d7,_d5,_d6,_d4,_d3,_d1,_d2,_d0};
         2: {_d7,_d6,_d5,_d4,_d3,_d2,_d1,_d0} = {_d7,_d6,_d3,_d2,_d5,_d4,_d1,_d0};
       endcase
     end
     else if(t2l && ~structt && ie && (nondisp==0) && (psize==0))       //for t2l display 8bpp
            {_d7,_d6,_d5,_d4,_d3,_d2,_d1,_d0} = {_d7,_d6,_d3,_d2,_d5,_d4,_d1,_d0};
     else if(~t2l && ie)          //Linear DW Align
       case(unalign)
         1: {_d7,_d6,_d5,_d4,_d3,_d2,_d1,_d0} = {_d0,_d7,_d6,_d5,_d4,_d3,_d2,_d1};
         2: {_d7,_d6,_d5,_d4,_d3,_d2,_d1,_d0} = {_d1,_d0,_d7,_d6,_d5,_d4,_d3,_d2};
         3: {_d7,_d6,_d5,_d4,_d3,_d2,_d1,_d0} = {_d2,_d1,_d0,_d7,_d6,_d5,_d4,_d3};
         4: {_d7,_d6,_d5,_d4,_d3,_d2,_d1,_d0} = {_d3,_d2,_d1,_d0,_d7,_d6,_d5,_d4};
         5: {_d7,_d6,_d5,_d4,_d3,_d2,_d1,_d0} = {_d4,_d3,_d2,_d1,_d0,_d7,_d6,_d5};
         6: {_d7,_d6,_d5,_d4,_d3,_d2,_d1,_d0} = {_d5,_d4,_d3,_d2,_d1,_d0,_d7,_d6};
         7: {_d7,_d6,_d5,_d4,_d3,_d2,_d1,_d0} = {_d6,_d5,_d4,_d3,_d2,_d1,_d0,_d7};
       endcase
   end

   always @(posedge clk)
     if (mc_rd_ce_vld_d)
     {d7,d6,d5,d4,d3,d2,d1,d0} <= {_d7,_d6,_d5,_d4,_d3,_d2,_d1,_d0};

   reg first_unalign_d;
   always @(posedge clk) begin
     first_unalign_d <= is_start_tid_d && ie && !t2l && burst_start;
   end
   wire [`DMA_RD_DATA_BUF_DEPTH-1:0] prev_wptr = next_wptr==0 ? `DMA_RD_DATA_BUF_ENTRIES - 1 : next_wptr - 1;

 always @* begin
   if(ie_d) begin                  //Interleave: for R/W from up to 8 bank in one cycle.
     ram_mbank_data_din = {d7,d6,d5,d4,d3,d2,d1,d0};
`ifdef SDMA__DISP_TO_ROTATE__1
     if(t2t_d)     //Rotate tile T2T
      case(psize_d)
       3'b000: 
        case(offset[0]) 
        0: ram_mbank_data_din = {d7,d6,d5,d4,d3,d2,d1,d0}; 
        1: ram_mbank_data_din = {d3,d2,d1,d0,d7,d6,d5,d4}; 
        endcase
       3'b001: 
        case(offset[1:0]) 
        0: ram_mbank_data_din = {d7,d6,d5,d4,d3,d2,d1,d0}; 
        1: ram_mbank_data_din = {d1,d0,d7,d6,d5,d4,d3,d2}; 
        2: ram_mbank_data_din = {d3,d2,d1,d0,d7,d6,d5,d4}; 
        3: ram_mbank_data_din = {d5,d4,d3,d2,d1,d0,d7,d6}; 
        endcase
       3'b010: 
        case(offset[1]) 
        0: ram_mbank_data_din = {d7,d6,d5,d4,d3,d2,d1,d0}; 
        1: ram_mbank_data_din = {d3,d2,d1,d0,d7,d6,d5,d4}; 
        endcase
       3'b011: 
           ram_mbank_data_din = {d7,d6,d5,d4,d3,d2,d1,d0}; 
       endcase
     else if(struct_d) begin
`else
     if(struct_d) begin
`endif
      if (t2l_d)  //s2l
	case(offset[3:1])
        0: ram_mbank_data_din = {d7,d6,d5,d4,d3,d2,d1,d0};
        1: ram_mbank_data_din = {d0,d7,d6,d5,d4,d3,d2,d1};
        2: ram_mbank_data_din = {d1,d0,d7,d6,d5,d4,d3,d2};
        3: ram_mbank_data_din = {d2,d1,d0,d7,d6,d5,d4,d3};
        4: ram_mbank_data_din = {d3,d2,d1,d0,d7,d6,d5,d4};
        5: ram_mbank_data_din = {d4,d3,d2,d1,d0,d7,d6,d5};
        6: ram_mbank_data_din = {d5,d4,d3,d2,d1,d0,d7,d6};
        7: ram_mbank_data_din = {d6,d5,d4,d3,d2,d1,d0,d7};
        endcase
      else        //l2s
        case(offset[2:0])
        0: ram_mbank_data_din = {d7,d6,d5,d4,d3,d2,d1,d0};
        1: ram_mbank_data_din = {d0,d7,d6,d5,d4,d3,d2,d1};
        2: ram_mbank_data_din = {d1,d0,d7,d6,d5,d4,d3,d2};
        3: ram_mbank_data_din = {d2,d1,d0,d7,d6,d5,d4,d3};
        4: ram_mbank_data_din = {d3,d2,d1,d0,d7,d6,d5,d4};
        5: ram_mbank_data_din = {d4,d3,d2,d1,d0,d7,d6,d5};
        6: ram_mbank_data_din = {d5,d4,d3,d2,d1,d0,d7,d6};
        7: ram_mbank_data_din = {d6,d5,d4,d3,d2,d1,d0,d7};
        endcase
     end
     else case({t2l_d,|nondisp_d,psize_d})
      5'b00_000,
      5'b01_000,
      5'b01_001:
        case(offset[1:0])
        0: ram_mbank_data_din = {d7,d6,d5,d4,d3,d2,d1,d0}; 
        1: ram_mbank_data_din = {d1,d0,d7,d6,d5,d4,d3,d2}; 
        2: ram_mbank_data_din = {d3,d2,d1,d0,d7,d6,d5,d4}; 
        3: ram_mbank_data_din = {d5,d4,d3,d2,d1,d0,d7,d6}; 
        endcase
      5'b01_010,
      5'b01_011,
      5'b00_001,
      5'b00_010,
      5'b00_011:
        case(offset[0])
        0: ram_mbank_data_din = {d7,d6,d5,d4,d3,d2,d1,d0}; 
        1: ram_mbank_data_din = {d3,d2,d1,d0,d7,d6,d5,d4}; 
        endcase
	//t2l
      5'b10_000,
      5'b11_000:
       if (nondisp_d == 3'b100)	//8bpp thick tile
        case(offset[3:2])
        0: ram_mbank_data_din = {d7,d6,d5,d4,d3,d2,d1,d0};
        1: ram_mbank_data_din = {d1,d0,d7,d6,d5,d4,d3,d2};
        2: ram_mbank_data_din = {d3,d2,d1,d0,d7,d6,d5,d4};
        3: ram_mbank_data_din = {d5,d4,d3,d2,d1,d0,d7,d6};
        endcase
       else
        case(offset[2:1])
        0: ram_mbank_data_din = {d7,d6,d5,d4,d3,d2,d1,d0};
        1: ram_mbank_data_din = {d1,d0,d7,d6,d5,d4,d3,d2};
        2: ram_mbank_data_din = {d3,d2,d1,d0,d7,d6,d5,d4};
        3: ram_mbank_data_din = {d5,d4,d3,d2,d1,d0,d7,d6};
        endcase
      5'b11_001:
        case({offset[2], offset[0]})
        0: ram_mbank_data_din = {d7,d6,d5,d4,d3,d2,d1,d0};
        1: ram_mbank_data_din = {d1,d0,d7,d6,d5,d4,d3,d2};
        2: ram_mbank_data_din = {d3,d2,d1,d0,d7,d6,d5,d4};
        3: ram_mbank_data_din = {d5,d4,d3,d2,d1,d0,d7,d6};
        endcase
      5'b10_001:
        case(offset[2])
        0: ram_mbank_data_din = {d7,d6,d5,d4,d3,d2,d1,d0};
        1: ram_mbank_data_din = {d3,d2,d1,d0,d7,d6,d5,d4};
        endcase
      5'b11_010:
        case(offset[1])
        0: ram_mbank_data_din = {d7,d6,d5,d4,d3,d2,d1,d0};
        1: ram_mbank_data_din = {d3,d2,d1,d0,d7,d6,d5,d4};
        endcase
      5'b10_010,
      5'b10_011,
      5'b11_011:
        case(offset[0])
        0: ram_mbank_data_din = {d7,d6,d5,d4,d3,d2,d1,d0};
        1: ram_mbank_data_din = {d3,d2,d1,d0,d7,d6,d5,d4};
        endcase
      endcase

      if(t2l_d)	begin
        ram_mbank_data_waddr = {8{next_wptr}};
      end
      else begin               //Linear DW Align
        case(unalign_d)
         0: ram_mbank_data_waddr = {next_wptr, next_wptr, next_wptr, next_wptr, next_wptr, next_wptr, next_wptr, next_wptr}; 
         1: ram_mbank_data_waddr = {prev_wptr, next_wptr, next_wptr, next_wptr, next_wptr, next_wptr, next_wptr, next_wptr}; 
         2: ram_mbank_data_waddr = {prev_wptr, prev_wptr, next_wptr, next_wptr, next_wptr, next_wptr, next_wptr, next_wptr}; 
         3: ram_mbank_data_waddr = {prev_wptr, prev_wptr, prev_wptr, next_wptr, next_wptr, next_wptr, next_wptr, next_wptr}; 
         4: ram_mbank_data_waddr = {prev_wptr, prev_wptr, prev_wptr, prev_wptr, next_wptr, next_wptr, next_wptr, next_wptr}; 
         5: ram_mbank_data_waddr = {prev_wptr, prev_wptr, prev_wptr, prev_wptr, prev_wptr, next_wptr, next_wptr, next_wptr}; 
         6: ram_mbank_data_waddr = {prev_wptr, prev_wptr, prev_wptr, prev_wptr, prev_wptr, prev_wptr, next_wptr, next_wptr}; 
         7: ram_mbank_data_waddr = {prev_wptr, prev_wptr, prev_wptr, prev_wptr, prev_wptr, prev_wptr, prev_wptr, next_wptr}; 
        endcase
      end

	//Interleave: for R/W from up to 8 bank in one cycle.
       //if(struct_d) begin
       // if (!t2l_d)  //l2s
       //   case(offset[2:0])
       //   0: ram_mbank_data_waddr = ram_mbank_data_waddr;
       //   1: ram_mbank_data_waddr = {ram_mbank_data_waddr[8:0], ram_mbank_data_waddr[71:9]};
       //   2: ram_mbank_data_waddr = {ram_mbank_data_waddr[17:0], ram_mbank_data_waddr[71:18]};
       //   3: ram_mbank_data_waddr = {ram_mbank_data_waddr[26:0], ram_mbank_data_waddr[71:27]};
       //   4: ram_mbank_data_waddr = {ram_mbank_data_waddr[35:0], ram_mbank_data_waddr[71:36]};
       //   5: ram_mbank_data_waddr = {ram_mbank_data_waddr[44:0], ram_mbank_data_waddr[71:45]};
       //   6: ram_mbank_data_waddr = {ram_mbank_data_waddr[53:0], ram_mbank_data_waddr[71:54]};
       //   7: ram_mbank_data_waddr = {ram_mbank_data_waddr[62:0], ram_mbank_data_waddr[71:63]};
       //   endcase
	//s//2l wr_addr have no interleave
       //end
       // else case({t2l_d,|nondisp_d,psize_d})
       case({t2l_d,|nondisp_d,psize_d})
         5'b00_000,
         5'b01_000,
         5'b01_001:
          case(offset[1:0])
               0: ram_mbank_data_waddr = ram_mbank_data_waddr; 
               1: ram_mbank_data_waddr = {ram_mbank_data_waddr[2*`DMA_RD_DATA_BUF_DEPTH-1:0], ram_mbank_data_waddr[8*`DMA_RD_DATA_BUF_DEPTH-1:2*`DMA_RD_DATA_BUF_DEPTH]}; 
               2: ram_mbank_data_waddr = {ram_mbank_data_waddr[4*`DMA_RD_DATA_BUF_DEPTH-1:0], ram_mbank_data_waddr[8*`DMA_RD_DATA_BUF_DEPTH-1:4*`DMA_RD_DATA_BUF_DEPTH]}; 
               3: ram_mbank_data_waddr = {ram_mbank_data_waddr[6*`DMA_RD_DATA_BUF_DEPTH-1:0], ram_mbank_data_waddr[8*`DMA_RD_DATA_BUF_DEPTH-1:6*`DMA_RD_DATA_BUF_DEPTH]}; 
          endcase
         5'b01_010,
         5'b01_011,
         5'b00_001,
         5'b00_010,
         5'b00_011:
          case(offset[0])
               0: ram_mbank_data_waddr = ram_mbank_data_waddr; 
               1: ram_mbank_data_waddr = {ram_mbank_data_waddr[4*`DMA_RD_DATA_BUF_DEPTH-1:0], ram_mbank_data_waddr[8*`DMA_RD_DATA_BUF_DEPTH-1:4*`DMA_RD_DATA_BUF_DEPTH]}; 
          endcase
	//t2l wr_addr have no interleave
        endcase

      //Linear DW Align
      ram_mbank_data_wmask = first_unalign_d ? (8'hff >> unalign_d) :
			     (t2l_d && nondisp_d == 4 && psize_d == 0) ? 	//t2l 8bpp thick tile
                                                offset[0] ? 8'b10101010  : 8'b01010101 : 8'hff; 
	//Interleave: for R/W from up to 8 bank in one cycle.
       //if(struct_d) begin
       // if (t2l_d)  //s2l
       //   case(offset[3:1])
       //   0: ram_mbank_data_wmask = ram_mbank_data_wmask;
       //   1: ram_mbank_data_wmask = {ram_mbank_data_wmask[31:0], ram_mbank_data_wmask[255:32]};
       //   2: ram_mbank_data_wmask = {ram_mbank_data_wmask[63:0], ram_mbank_data_wmask[255:64]};
       //   3: ram_mbank_data_wmask = {ram_mbank_data_wmask[95:0], ram_mbank_data_wmask[255:96]};
       //   4: ram_mbank_data_wmask = {ram_mbank_data_wmask[127:0], ram_mbank_data_wmask[255:128]};
       //   5: ram_mbank_data_wmask = {ram_mbank_data_wmask[159:0], ram_mbank_data_wmask[255:160]};
       //   6: ram_mbank_data_wmask = {ram_mbank_data_wmask[191:0], ram_mbank_data_wmask[255:192]};
       //   7: ram_mbank_data_wmask = {ram_mbank_data_wmask[223:0], ram_mbank_data_wmask[255:224]};
       //   endcase
       // else	  //l2s
       //   case(offset[2:0])
       //   0: ram_mbank_data_wmask = ram_mbank_data_wmask; 
       //   1: ram_mbank_data_wmask = {ram_mbank_data_wmask[31:0], ram_mbank_data_wmask[255:32]}; 
       //   2: ram_mbank_data_wmask = {ram_mbank_data_wmask[63:0], ram_mbank_data_wmask[255:64]}; 
       //   3: ram_mbank_data_wmask = {ram_mbank_data_wmask[95:0], ram_mbank_data_wmask[255:96]}; 
       //   4: ram_mbank_data_wmask = {ram_mbank_data_wmask[127:0], ram_mbank_data_wmask[255:128]}; 
       //   5: ram_mbank_data_wmask = {ram_mbank_data_wmask[159:0], ram_mbank_data_wmask[255:160]}; 
       //   6: ram_mbank_data_wmask = {ram_mbank_data_wmask[191:0], ram_mbank_data_wmask[255:192]}; 
       //   7: ram_mbank_data_wmask = {ram_mbank_data_wmask[223:0], ram_mbank_data_wmask[255:224]}; 
       //   endcase
       //end
       // else case({t2l_d,|nondisp_d,psize_d})
       case({t2l_d,|nondisp_d,psize_d})
         5'b00_000,
         5'b01_000,
         5'b01_001:
          case(offset[1:0])
             0: ram_mbank_data_wmask = ram_mbank_data_wmask; 
             1: ram_mbank_data_wmask = {ram_mbank_data_wmask[1:0], ram_mbank_data_wmask[7:2]}; 
             2: ram_mbank_data_wmask = {ram_mbank_data_wmask[3:0], ram_mbank_data_wmask[7:4]}; 
             3: ram_mbank_data_wmask = {ram_mbank_data_wmask[5:0], ram_mbank_data_wmask[7:6]}; 
          endcase
         5'b01_010,
         5'b01_011,
         5'b00_001,
         5'b00_010,
         5'b00_011:
          case(offset[0])
             0: ram_mbank_data_wmask = ram_mbank_data_wmask; 
             1: ram_mbank_data_wmask = {ram_mbank_data_wmask[3:0], ram_mbank_data_wmask[7:4]}; 
          endcase
	//t2l
         5'b10_000,
         5'b11_000:
         if (nondisp_d == 3'b100)   //8bpp thick tile
          case(offset[3:2])
               0: ram_mbank_data_wmask = ram_mbank_data_wmask;
               1: ram_mbank_data_wmask = {ram_mbank_data_wmask[1:0], ram_mbank_data_wmask[7:2]};
               2: ram_mbank_data_wmask = {ram_mbank_data_wmask[3:0], ram_mbank_data_wmask[7:4]};
               3: ram_mbank_data_wmask = {ram_mbank_data_wmask[5:0], ram_mbank_data_wmask[7:6]};
          endcase
         else
          case(offset[2:1])
               0: ram_mbank_data_wmask = ram_mbank_data_wmask;
               1: ram_mbank_data_wmask = {ram_mbank_data_wmask[1:0], ram_mbank_data_wmask[7:2]};
               2: ram_mbank_data_wmask = {ram_mbank_data_wmask[3:0], ram_mbank_data_wmask[7:4]};
               3: ram_mbank_data_wmask = {ram_mbank_data_wmask[5:0], ram_mbank_data_wmask[7:6]};
          endcase
         5'b11_001:
          case({offset[2], offset[0]})
               0: ram_mbank_data_wmask = ram_mbank_data_wmask;
               1: ram_mbank_data_wmask = {ram_mbank_data_wmask[1:0], ram_mbank_data_wmask[7:2]};
               2: ram_mbank_data_wmask = {ram_mbank_data_wmask[3:0], ram_mbank_data_wmask[7:4]};
               3: ram_mbank_data_wmask = {ram_mbank_data_wmask[5:0], ram_mbank_data_wmask[7:6]};
          endcase
         5'b10_001:
          case(offset[2])
             0: ram_mbank_data_wmask = ram_mbank_data_wmask; 
             1: ram_mbank_data_wmask = {ram_mbank_data_wmask[3:0], ram_mbank_data_wmask[7:4]}; 
          endcase
         5'b11_010:
          case(offset[1])
             0: ram_mbank_data_wmask = ram_mbank_data_wmask; 
             1: ram_mbank_data_wmask = {ram_mbank_data_wmask[3:0], ram_mbank_data_wmask[7:4]}; 
          endcase
         5'b10_010,
         5'b10_011,
         5'b11_011:
          case(offset[0])
             0: ram_mbank_data_wmask = ram_mbank_data_wmask; 
             1: ram_mbank_data_wmask = {ram_mbank_data_wmask[3:0], ram_mbank_data_wmask[7:4]}; 
          endcase
        endcase
      //ram_mbank_data_we = {8{next_we}};
     end
     else begin
         ram_mbank_data_din = next_wdata;
         ram_mbank_data_wmask = (t2l_d &&
                       granu_d == `DMA_MC_wr_granularity_min_32b) ?
                      (1'b1 << detile_quarter_d) :
                      (t2l_d && granu_d == `DMA_MC_wr_granularity_min_64b) ?
                      (2'b11 << detile_quarter_d) :
                      (t2l_d && granu_d == `DMA_MC_wr_granularity_min_128b) ?
                      (4'hf << detile_quarter_d) : 8'hff;
         ram_mbank_data_waddr = {8{next_wptr}};
	 //ram_mbank_data_we = {8{next_we}};
     end
   end

   assign ram_mbank_data_we = ram_mbank_data_wmask & {8{next_we}};
   reg [31:0] rd_data_src_mask;  //mask of valid data in a return 256b tiled data. for t2l normal copy.
   always @* begin
`ifdef THICK_TILE_EN
      if(nondisp == 2'b01 || nondisp == 2'b10)
`else
      if(|nondisp)
`endif
	casez (psize_y)
          //8bpp
          5'b00_?00: rd_data_src_mask = 32'b0000_0000_0011_0011_0000_0000_0011_0011;
          5'b00_?01: rd_data_src_mask = 32'b0000_0000_1100_1100_0000_0000_1100_1100;
          5'b00_?10: rd_data_src_mask = 32'b0011_0011_0000_0000_0011_0011_0000_0000;
          5'b00_?11: rd_data_src_mask = 32'b1100_1100_0000_0000_1100_1100_0000_0000;
          //16bpp
          5'b01_?00: rd_data_src_mask = 32'b0000_0000_0000_0000_0000_1111_0000_1111;
          5'b01_?01: rd_data_src_mask = 32'b0000_0000_0000_0000_1111_0000_1111_0000;
          5'b01_?10: rd_data_src_mask = 32'b0000_1111_0000_1111_0000_0000_0000_0000;
          5'b01_?11: rd_data_src_mask = 32'b1111_0000_1111_0000_0000_0000_0000_0000;
          //32bpp
          5'b10_??0: rd_data_src_mask = 32'b0000_0000_1111_1111_0000_0000_1111_1111;
          5'b10_??1: rd_data_src_mask = 32'b1111_1111_0000_0000_1111_1111_0000_0000;
	  //VCS coverage off
          //64bpp
          5'b11_??0: rd_data_src_mask = 32'b0000_0000_0000_0000_1111_1111_1111_1111;
          5'b11_??1: rd_data_src_mask = 32'b1111_1111_1111_1111_0000_0000_0000_0000;
          //128bpp   256b granularity not need rd_data_src_mask
          //6'b100_xxx: rd_data_src_mask = 32'b1111_1111_1111_1111_1111_1111_1111_1111;
	  //VCS coverage on
          default: rd_data_src_mask = 0;
	endcase
   //Thick tile
`ifdef THICK_TILE_EN
     else if (nondisp == 3'b100)
       casez (psize_y)
       //8bpp
       6'b000_000: rd_data_src_mask = 32'b0000_0000_0000_0000_0000_0000_0011_0011;
       6'b000_001: rd_data_src_mask = 32'b0000_0000_0000_0000_0000_0000_1100_1100;
       6'b000_010: rd_data_src_mask = 32'b0000_0000_0000_0000_0011_0011_0000_0000;
       6'b000_011: rd_data_src_mask = 32'b0000_0000_0000_0000_1100_1100_0000_0000;
       6'b000_100: rd_data_src_mask = 32'b0000_0000_0011_0011_0000_0000_0000_0000;
       6'b000_101: rd_data_src_mask = 32'b0000_0000_1100_1100_0000_0000_0000_0000;
       6'b000_110: rd_data_src_mask = 32'b0011_0011_0000_0000_0000_0000_0000_0000;
       6'b000_111: rd_data_src_mask = 32'b1100_1100_0000_0000_0000_0000_0000_0000;
       //16bpp
       6'b001_?00: rd_data_src_mask = 32'b0000_0000_0000_0000_0000_1111_0000_1111;
       6'b001_?01: rd_data_src_mask = 32'b0000_0000_0000_0000_1111_0000_1111_0000;
       6'b001_?10: rd_data_src_mask = 32'b0000_1111_0000_1111_0000_0000_0000_0000;
       6'b001_?11: rd_data_src_mask = 32'b1111_0000_1111_0000_0000_0000_0000_0000;
       //32bpp
       6'b010_??0: rd_data_src_mask = 32'b0000_0000_1111_1111_0000_0000_1111_1111;
       6'b010_??1: rd_data_src_mask = 32'b1111_1111_0000_0000_1111_1111_0000_0000;
       //64bpp
       6'b011_??0: rd_data_src_mask = 32'b0000_0000_0000_0000_1111_1111_1111_1111;
       6'b011_??1: rd_data_src_mask = 32'b1111_1111_1111_1111_0000_0000_0000_0000;
       //128bpp
       //6'b100_???: rd_data_src_mask = 32'b1111_1111_1111_1111_1111_1111_1111_1111;
       default: rd_data_src_mask = 0;
       endcase
`endif       
      else  //displayable
	if(granu==2)	//128 granu 
	  case (psize_y[0])
	    1'b0: rd_data_src_mask = 32'h0000ffff;
	    1'b1: rd_data_src_mask = 32'hffff0000;
	  endcase
	else if(granu==1)  //64 granu
	  case (psize_y[1:0])
	    2'b00: rd_data_src_mask = 32'h000000ff;
	    2'b01: rd_data_src_mask = 32'h00ff0000;
	    2'b10: rd_data_src_mask = 32'h0000ff00;
	    2'b11: rd_data_src_mask = 32'hff000000;
	  endcase
	else if(granu==0)  //32 granu
	  rd_data_src_mask = 32'hf << {psize_y[2:0], 2'b0};
	else
	  rd_data_src_mask = 0;
   end

   
   wire [15:0] rotate_data_word[15:0];

   //Pack valid data of t2l source 256b into 128b 64b or 32b, according to rd_data_src_mask.
   integer 			     i,j,k,m;
   genvar n;
   generate for(n=0; n<=15; n=n+1) begin:gen2
      assign rotate_data_word[n] = mc_rd_ce_data_d2[n*16+15:n*16];
   end
   endgenerate

   reg[3:0] vld_word_sum[15:0];
   always @* begin
    vld_word_sum[0] = rd_data_src_mask[0];
    for(i=1; i<=15; i=i+1) 
      vld_word_sum[i] = vld_word_sum[i-1] + rd_data_src_mask[2*i];
   end

   reg [3:0] vld_word_t2l[7:0];     //registered for timing
   reg [3:0] next_vld_word_t2l[7:0];     //comb logic

   always @*
     for(k=0; k<=7; k=k+1) begin
        next_vld_word_t2l[k] = 0;
        for(j=0; j<=15; j=j+1) begin
           if(vld_word_sum[j] == k+1 && rd_data_src_mask[2*j])
             next_vld_word_t2l[k] = j;
        end
     end

   always @(posedge clk)
     for(m=0; m<=7; m=m+1)
        vld_word_t2l[m] <= next_vld_word_t2l[m];

   generate for(n=0; n<=7; n=n+1) begin:gen3
      assign src_t2l_data[16*n+15:16*n] = rotate_data_word[vld_word_t2l[n]];
   end
   endgenerate



   /////////////////////
     // Entry Valid Control
   /////////////////////

   // Marking buffer entry as valid when MC read data returns, invalid as data is being read out of the buffer
   wire [3:0] fifo_entry_vld_inc = (ie_d && t2l_d && !struct_d && nondisp_d==4 && psize_d==0) ? 4'b0100 :
	   (granu_d == `DMA_MC_wr_granularity_min_128b || granu_d == `DMA_MC_wr_granularity_min_64b || granu_d == `DMA_MC_wr_granularity_min_32b) && t2l_d && !ie_d ? (1 << granu_d) + (last_d && |nr_dwords_d ? 8-nr_dwords_d : 0) :
											        4'b1000; 
   always @(posedge clk) begin
      if (~resetb) begin
	 fifo_entry_vld <= `DMA_RD_DATA_BUF_ENTRIES'd0;
	 fifo_entry_vld0<= `DMA_RD_DATA_BUF_ENTRIES'd0;
	 fifo_entry_vld1 <= `DMA_RD_DATA_BUF_ENTRIES'd0;
	 fifo_entry_vld2 <= `DMA_RD_DATA_BUF_ENTRIES'd0;
      end
      else begin
	 if (next_we) begin
	      {fifo_entry_vld[next_wptr[`DMA_RD_DATA_BUF_DEPTH-1:0]],
	       fifo_entry_vld2[next_wptr[`DMA_RD_DATA_BUF_DEPTH-1:0]],
	       fifo_entry_vld1[next_wptr[`DMA_RD_DATA_BUF_DEPTH-1:0]],
	       fifo_entry_vld0[next_wptr[`DMA_RD_DATA_BUF_DEPTH-1:0]]} <= //4-bit adder for each entry. for t2l multiple src -> one entry.
			  {fifo_entry_vld[next_wptr[`DMA_RD_DATA_BUF_DEPTH-1:0]],
			   fifo_entry_vld2[next_wptr[`DMA_RD_DATA_BUF_DEPTH-1:0]],
			   fifo_entry_vld1[next_wptr[`DMA_RD_DATA_BUF_DEPTH-1:0]],
			   fifo_entry_vld0[next_wptr[`DMA_RD_DATA_BUF_DEPTH-1:0]]} + fifo_entry_vld_inc;
	 end
	 if (ram_rd_data_rptr_inc) begin
	    fifo_entry_vld[rptr[`DMA_RD_DATA_BUF_DEPTH-1:0]] <= 1'b0;
	    fifo_entry_vld2[rptr[`DMA_RD_DATA_BUF_DEPTH-1:0]] <= 1'b0;
	    fifo_entry_vld1[rptr[`DMA_RD_DATA_BUF_DEPTH-1:0]] <= 1'b0;
	    fifo_entry_vld0[rptr[`DMA_RD_DATA_BUF_DEPTH-1:0]] <= 1'b0;
	 end
      end
   end 

   always @(posedge clk) begin
      if (~resetb) begin
         fifo_entry_ecc <= `DMA_RD_DATA_BUF_ENTRIES'd0;
      end
`ifdef GPU__CHIP__EDCMEM1__1
      else begin
         if (next_we) begin
            fifo_entry_ecc[next_wptr[`DMA_RD_DATA_BUF_DEPTH-1:0]] <= mc_rdret_ecc | fifo_entry_ecc[next_wptr[`DMA_RD_DATA_BUF_DEPTH-1:0]];
         end
         if (ram_rd_data_rptr_inc) begin
            fifo_entry_ecc[rptr[`DMA_RD_DATA_BUF_DEPTH-1:0]] <= 1'b0;
         end
      end
`endif
   end

   /////////////////////
   // dma_idle interface
   /////////////////////
   always @(posedge clk) begin
      if (~resetb) begin
	 dma_ce_in_idle <= 1'b1;
      end
      else begin
	 dma_ce_in_idle <= empty && (alloc_free == `DMA_RD_DATA_BUF_ENTRIES >> dfifo_watermark) 
	   && (fifo_entry_vld2 == `DMA_RD_DATA_BUF_ENTRIES'd0)
	     && (fifo_entry_vld1 == `DMA_RD_DATA_BUF_ENTRIES'd0) && (fifo_entry_vld0 == `DMA_RD_DATA_BUF_ENTRIES'd0)
	       && (fifo_entry_vld == `DMA_RD_DATA_BUF_ENTRIES'd0) && fifo_entry_ecc == `DMA_RD_DATA_BUF_ENTRIES'd0;
      end			        
   end

endmodule 
