
`include "dma_common.vh"


module dma_ce_out_bc(

    input clk,
    input resetb,

    input bc_mode,
    input [`DMA_RD_DATA_BUF_ENTRIES-1:0] fifo_entry_vld,
    input [`DMA_RD_DATA_BUF_ENTRIES-1:0] fifo_entry_ecc,
    output ram_rd_data_rptr_inc,
    output [1:0] out_granu,
    output [6:0] out_chunk_size,
    output out_ie,
    output out_ecc,
		      // memory interface
    input [`DMA_RD_DATA_DATA_SIZE-1:0]	    	    ram_mbank_data_dout,
    output reg[8*`DMA_RD_DATA_BUF_DEPTH-1:0] 	    		    ram_mbank_data_raddr,
    output reg [7:0]                    	    ram_mbank_data_re,

    input [`DMA_RD_INFO_BUF_WIDTH-1:0]              ram_cmd_info_dout,
    output [`DMA_RD_INFO_BUF_DEPTH-1:0]             ram_cmd_info_waddr,
    output [`DMA_RD_INFO_BUF_DEPTH-1:0]             ram_cmd_info_raddr,
    output                                          ram_cmd_info_re,
    input                                           ram_cmd_info_we,
    output reg                                      cmd_info_full,

    output reg dma_ce_out_idle,
    output skid_empty,
    input data_skid_re,
    output [`DMA_RD_DATA_DATA_SIZE-1:0] data_skid_dout
);

   ////////////////////////////
   // Data buffer read pointer
   ////////////////////////////
   reg                                			   input_valid_gen_d, input_valid, output_valid;
   
   wire 						   fifo_rd_rdy, fifo_rd_rdy_l2t;

   reg [`DMA_RD_DATA_DATA_RANGE] 			   mem_rdata;

   wire 						   cmd_info_empty;

   //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   // Cmd Info FIFO
   //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

   reg[`DMA_RD_INFO_BUF_WIDTH-1:0] mem_cmd_info_skid;
   wire [`DMA_RD_INFO_BUF_WIDTH-1:0] mem_cmd_info;
   reg                               cmd_info_output_valid, cmd_info_skid_valid;
   assign mem_cmd_info = cmd_info_skid_valid ? mem_cmd_info_skid : ram_cmd_info_dout;
   reg cmd_info_rd_rdy;
   wire                              cmd_info_input_valid;
   reg cmd_info_rd_vld;
   reg cmd_info_re;

   always @(*) begin
      case ({cmd_info_input_valid, cmd_info_output_valid, cmd_info_skid_valid})
        3'b000,
          3'b001,
          3'b010: cmd_info_rd_rdy = 1'b1;
        3'b011: cmd_info_rd_rdy = cmd_info_rd_vld;
        3'b100: cmd_info_rd_rdy = 1'b1;
        3'b101,
          3'b110: cmd_info_rd_rdy = cmd_info_rd_vld;
        // VCS coverage off
        3'b111: cmd_info_rd_rdy = 1'b0; // assert here
        // VCS coverage on
      endcase // case({input_valid, output_valid, skid_valid})
   end

   reg[`DMA_RD_INFO_BUF_DEPTH-1:0] cmd_info_rptr;
   reg [`DMA_RD_INFO_BUF_DEPTH-1:0] cmd_info_wptr;
   reg [`DMA_RD_INFO_BUF_DEPTH-1:0] cmd_info_wptr_d;
   wire [`DMA_RD_INFO_BUF_DEPTH-1:0] next_cmd_info_rptr = cmd_info_re ? (cmd_info_rptr + 1) : cmd_info_rptr;
   wire [`DMA_RD_INFO_BUF_DEPTH-1:0] cmd_info_wptr_plus2 = cmd_info_wptr + 2;

   assign cmd_info_empty = cmd_info_wptr_d == next_cmd_info_rptr; // && !cmd_info_output_valid && !cmd_info_skid_valid;

   assign cmd_info_input_valid = cmd_info_re; // input flop inside memory is valid
   wire                              next_cmd_info_output_valid = cmd_info_input_valid || (cmd_info_output_valid && cmd_info_skid_valid);
   wire                              next_cmd_info_skid_valid = !cmd_info_rd_vld && (cmd_info_output_valid || cmd_info_skid_valid);

   wire cmd_info_full_next = cmd_info_full && cmd_info_wptr_plus2 < cmd_info_rptr ? 0 : cmd_info_wptr_plus2 == cmd_info_rptr;

   always @(posedge clk) begin
      if (~resetb) begin
         cmd_info_re <= 1'b0;
         cmd_info_rptr <= 7'd0;
         cmd_info_wptr <= 7'd0;
         cmd_info_wptr_d <= 7'd0;
         cmd_info_output_valid <= 1'b0;
         cmd_info_skid_valid <= 1'b0;
         mem_cmd_info_skid <= 0;
         cmd_info_full <= 0;
      end
      else begin
         cmd_info_re <= cmd_info_rd_rdy & ~cmd_info_empty;
         cmd_info_rptr <= next_cmd_info_rptr;
         cmd_info_wptr <= cmd_info_wptr + ram_cmd_info_we;
         cmd_info_wptr_d <= cmd_info_wptr;
         cmd_info_output_valid <= next_cmd_info_output_valid;
         cmd_info_skid_valid <= next_cmd_info_skid_valid;
         mem_cmd_info_skid <= mem_cmd_info;
         cmd_info_full <= cmd_info_full_next;
      end // else: !if(~resetb)
   end

   wire [5:0] tilenum;
   wire [1:0] granu;
   wire [`SDMA_PKT_COPY_TILED_BC_MIT_MODE_SIZE-1:0] nondisp;
   wire [2:0] psize;
   wire t2l, structt, t2t;
   wire [`CMD_COUNT_SIZE-1+1:0] ssize;
   wire [4:0] src_start;
   wire [6:0] chunk_size;
   reg t2t_d;
   reg t2l_d;
   reg [2:0] psize_d;
   reg ie_d;
   reg [6:0] chunk_size_d;
   reg [`SDMA_PKT_COPY_TILED_BC_MIT_MODE_SIZE-1:0] nondisp_d;

   assign  { t2t, structt, tilenum, granu, ie, t2l, nondisp, psize, ssize, src_start, chunk_size } = cmd_info_output_valid || cmd_info_skid_valid ? mem_cmd_info : 0;

    always @(posedge clk) begin
       if (~resetb) begin
   	  t2t_d <=0;
   	  t2l_d <=0;
   	  psize_d <=0;
   	  ie_d <=0;
          chunk_size_d <= 0;
   	  nondisp_d <=0;
	end
       else begin 
   	  t2t_d <= t2t;
   	  t2l_d <= t2l;
   	  psize_d <= psize;
   	  ie_d <= ie;
          chunk_size_d <= chunk_size;
   	  nondisp_d <= nondisp;
	end
    end

      // Interface for cmd info FIFO
   assign ram_cmd_info_waddr = cmd_info_wptr[`DMA_RD_INFO_BUF_DEPTH-1:0];
   assign ram_cmd_info_raddr = next_cmd_info_rptr[`DMA_RD_INFO_BUF_DEPTH-1:0];
   assign ram_cmd_info_re = cmd_info_rd_rdy && !cmd_info_empty ;
   
   ////////////////////////////
   // Data buffer control
   ////////////////////////////

   wire skid_almost_full;

   // output pipeline register inside memory is valid 
   wire next_output_valid = input_valid;
   wire next_input_valid;

   reg[`DMA_RD_DATA_BUF_DEPTH-1:0] chunk_base;  //base address of a chunk, or rptr of normal sub copy
   parameter[1:0] state_start = 0, state_ie = 1, state_normal = 2;

   reg[1:0] out_state;
   reg[`CMD_COUNT_SIZE-1+1:0] out_size;
   reg[7:0] ie_cnt, ie_cnt_input, ie_cnt_input_d, ie_cnt_output;
   reg[3:0] line_cnt, line_cnt_input, line_cnt_input_d, line_cnt_output;
   reg[2:0] row_cnt, row_cnt_input, row_cnt_input_d, row_cnt_output;
//   wire [79:0] chunk_entry_vld = {fifo_entry_vld[79:0],fifo_entry_vld} >> chunk_base; //up to 80 (64 if aligned) valid bits for a chunk
   wire [15:0] chunk_entry_vld_rot = {fifo_entry_vld[15:0],fifo_entry_vld} >> chunk_base; //rotate tile T2T
   reg chunk_vld, chunk_vld_rot;
   wire thick_8bpp_ie = ie && !structt && (nondisp == 4) && (psize == 0);	//for 8bpp thick tile IE
   assign chunk_read_done = ~skid_almost_full && (ie_cnt+1) == (thick_8bpp_ie && ~t2l ? {chunk_size[6:0], 1'b0} : chunk_size);  
   wire thick_8bpp_ie_timing = ie_d && (nondisp_d == 4) && (psize_d == 0);	//for 8bpp thick tile IE
   wire chunk_read_done_timing = ~skid_almost_full && (ie_cnt+1) == (thick_8bpp_ie_timing && ~t2l_d ? {chunk_size[6:0], 1'b0} : chunk_size);  
   wire entry_rdy = fifo_entry_vld[chunk_base];   //entry ready of normal sub copy
   assign out_chunk_size = chunk_size;

   //for timing
   reg [79:0] chunk_entry_vld;
   reg [`DMA_RD_DATA_BUF_DEPTH-1:0] chunk_base_ie_timing;
   wire[`DMA_RD_DATA_BUF_DEPTH-1:0] next_chunk_base = out_state==state_ie && chunk_read_done ? chunk_base_ie_timing : chunk_base;
   always@(posedge clk) begin
     chunk_base_ie_timing <= chunk_base + chunk_size >= `DMA_RD_DATA_BUF_ENTRIES ? chunk_base + chunk_size - `DMA_RD_DATA_BUF_ENTRIES :
                        chunk_base + chunk_size;
     //chunk_entry_vld <= out_state==state_normal ? 0 : ({fifo_entry_vld[79:0],fifo_entry_vld} >> next_chunk_base); //up to 80 (64 if aligned) valid bits for a chunk
   end

    always @(posedge clk) begin
       if (~resetb) begin
           chunk_entry_vld <= 0;
       end
       else if (out_state==state_start || out_state==state_ie) begin
           chunk_entry_vld <= ({fifo_entry_vld[79:0],fifo_entry_vld} >> chunk_base); //up to 80 (64 if aligned) valid bits for a chunk
       end
    end

   always @* 
     case(chunk_size[6:3])
       0: chunk_vld = chunk_size[2:0]==2 ? &chunk_entry_vld[1:0] : &chunk_entry_vld[3:0];  //For Rotate tile T2T
       1: chunk_vld = &chunk_entry_vld[7:0];
       2: chunk_vld = &chunk_entry_vld[15:0];
       3: chunk_vld = &chunk_entry_vld[23:0];
       4: chunk_vld = &chunk_entry_vld[31:0];
       5: chunk_vld = &chunk_entry_vld[39:0];
       6: chunk_vld = &chunk_entry_vld[47:0];
       7: chunk_vld = &chunk_entry_vld[55:0];
       8: chunk_vld = &chunk_entry_vld[63:0];
       9: chunk_vld = &chunk_entry_vld[71:0];
       10: chunk_vld = &chunk_entry_vld[79:0];
       default: chunk_vld = &chunk_entry_vld[63:0];
      endcase

   always @* 
     case(chunk_size[4:1])
       1: chunk_vld_rot = &chunk_entry_vld_rot[1:0];
       2: chunk_vld_rot = &chunk_entry_vld_rot[3:0];
       4: chunk_vld_rot = &chunk_entry_vld_rot[7:0];
       8: chunk_vld_rot = &chunk_entry_vld_rot[15:0];
       default: chunk_vld_rot = &chunk_entry_vld_rot[15:0];
      endcase
 
   wire [79:0] chunk_entry_ecc = {fifo_entry_ecc[79:0],fifo_entry_ecc} >> chunk_base; //up to 80 (64 if aligned) valid bits for a chunk
   reg chunk_ecc;
   always@(posedge clk)
     if(~resetb) 
       chunk_ecc <= 0;
     else if(chunk_vld)
      case(chunk_size[6:3])
       0: chunk_ecc <= chunk_size[2:0]==2 ? |chunk_entry_ecc[1:0] : |chunk_entry_ecc[3:0];
       1: chunk_ecc <= |chunk_entry_ecc[7:0];
       2: chunk_ecc <= |chunk_entry_ecc[15:0];
       3: chunk_ecc <= |chunk_entry_ecc[23:0];
       4: chunk_ecc <= |chunk_entry_ecc[31:0];
       5: chunk_ecc <= |chunk_entry_ecc[39:0]; 
       6: chunk_ecc <= |chunk_entry_ecc[47:0];
       7: chunk_ecc <= |chunk_entry_ecc[55:0]; 
       8: chunk_ecc <= |chunk_entry_ecc[63:0];
       9: chunk_ecc <= |chunk_entry_ecc[71:0];
       10: chunk_ecc <= |chunk_entry_ecc[79:0];
       default: chunk_ecc <= |chunk_entry_ecc[63:0];
      endcase

   always@(posedge clk)
     if(~resetb) begin
       out_state <= 0;
       out_size  <= 0;
     end
     else case(out_state)
       state_start: begin
          out_size <= ssize + src_start; // create one more cycle in state_normal if src_start not aligned to 256b
          if((cmd_info_output_valid || cmd_info_skid_valid) && (ie && chunk_vld || !ie && entry_rdy))  //
             out_state <= ie ? state_ie : /*ssize + src_start > 32 ?*/ state_normal;// : state_start;
       end
       state_ie: begin
          out_size <= !skid_almost_full && (t2t ? (|ie_cnt || chunk_vld) : 1) ? out_size > 32 ? out_size - 32 : 0 : out_size;  //for rotate t2t
          out_state <= (t2t ? !skid_almost_full && out_size <= 32 : chunk_read_done) ? state_start : state_ie;
       end
       state_normal: begin
          out_size <= !skid_almost_full && entry_rdy ? out_size > 32 ? out_size - 32 : 0 : out_size;
          out_state <= !skid_almost_full && entry_rdy && out_size <= 32 ? state_start : state_normal;
       end
       default: out_state <= out_state;
     endcase

   always @*  //move to next entry of cmd_info fifo 
     case(out_state)
       state_start: cmd_info_rd_vld = 0;//!cmd_info_empty && !ie && entry_rdy && !skid_almost_full && ssize + src_start < 32;
       state_ie: cmd_info_rd_vld = t2t_d ? !skid_almost_full && out_size <= 32 : chunk_read_done_timing;
       state_normal: cmd_info_rd_vld = !skid_almost_full && entry_rdy && out_size <= 32;
        // VCS coverage off
       default: cmd_info_rd_vld = 0; 
        // VCS coverage on
     endcase

   assign out_granu = t2l && !ie ? granu : t2l && thick_8bpp_ie ? 2 : 3;

   always @(posedge clk)
     begin
       if (~resetb) begin
          ie_cnt <= 0;
          row_cnt <= 0;
          line_cnt <= 0;
          chunk_base <= 0;
       end
       else begin
         if(out_state==state_ie && chunk_read_done) begin
           chunk_base <= chunk_base + chunk_size >= `DMA_RD_DATA_BUF_ENTRIES ? chunk_base + chunk_size - `DMA_RD_DATA_BUF_ENTRIES :
			chunk_base + chunk_size;
           ie_cnt <= 0;
           row_cnt <= 0;
           line_cnt <= 0;
         end
         else if(out_state==state_ie && ~skid_almost_full && (|ie_cnt || chunk_vld)) begin
	   ie_cnt <= ie_cnt + 1;
           line_cnt <= row_cnt==(structt ? tilenum[5:3]-1 : ((tilenum<<(psize+1))>>3)-1) ? line_cnt + 1 : line_cnt;
	   row_cnt <= row_cnt==(structt ? tilenum[5:3]-1 : ((tilenum<<(psize+1))>>3)-1) ? 0 : row_cnt+1;
         end
	 else if(out_state==state_normal && ~skid_almost_full && entry_rdy /*|| out_state==state_start && cmd_info_re*/)
	   chunk_base <= chunk_base + 1 == `DMA_RD_DATA_BUF_ENTRIES ? 0 : chunk_base + 1;
       end
     end

   wire rptr_inc = out_state==state_normal && ~skid_almost_full && entry_rdy && (cmd_info_output_valid || cmd_info_skid_valid); 
   assign next_input_valid = ie ? (~skid_almost_full && out_state==state_ie && (|ie_cnt || chunk_vld)) : rptr_inc;
   wire next_ecc = out_state==state_ie ? chunk_ecc : fifo_entry_ecc[chunk_base];
   assign out_ie = out_state==state_ie;

   //========================================================================
   // Tile Copy with Improved Efficiecy
   //========================================================================


   
   /////////////////////
   // Data Buffer memory
   /////////////////////

   reg[6:0] offset0, offset1, offset2, offset3, offset4, offset5, offset6, offset7;
   reg[`DMA_RD_DATA_BUF_DEPTH-1:0] ie_addr0, ie_addr1, ie_addr2, ie_addr3, ie_addr4, ie_addr5, ie_addr6, ie_addr7;
   reg ie_input, ie_input_d, ie_output;
   reg t2l_input, t2l_input_d, t2l_output;
   reg t2t_input, t2t_input_d, t2t_output;
   reg structt_input, structt_input_d, structt_output;
   reg[`SDMA_PKT_COPY_TILED_BC_MIT_MODE_SIZE-1:0] nondisp_input, nondisp_input_d, nondisp_output;
   reg[2:0] psize_input, psize_input_d, psize_output;
   reg[1:0] granu_input, granu_input_d, granu_output;
   reg ecc_input, ecc_input_d, ecc_output;

   reg [8*`DMA_RD_DATA_BUF_DEPTH-1:0] ram_mbank_data_raddr_normal;

   always @(posedge clk)
     begin
       if (~resetb) begin
         ram_mbank_data_re <= 0;
	 ram_mbank_data_raddr <= 0;
	end
       else begin
         ram_mbank_data_re <= out_state == state_start ? 8'h0 : 8'hff;
     	 if(bc_mode) begin
	 ram_mbank_data_raddr <=  ram_mbank_data_raddr_normal;
     	 end
	end
     end



   always @* begin
     offset0=0; offset1=0; offset2=0; offset3=0; offset4=0; offset5=0; offset6=0; offset7=0;
     if(t2l_d) begin
      //if (structt)	//s2l 
      // case(line_cnt[2:0])
      //       0: begin offset0={row_cnt,3'd0,line_cnt[3]};
      //                offset1={row_cnt,3'd7,line_cnt[3]};
      //                offset2={row_cnt,3'd6,line_cnt[3]};
      //                offset3={row_cnt,3'd5,line_cnt[3]};
      //                offset4={row_cnt,3'd4,line_cnt[3]};
      //                offset5={row_cnt,3'd3,line_cnt[3]};
      //                offset6={row_cnt,3'd2,line_cnt[3]};
      //                offset7={row_cnt,3'd1,line_cnt[3]}; end
      //       1: begin offset0={row_cnt,3'd1,line_cnt[3]};
      //  	      offset1={row_cnt,3'd0,line_cnt[3]};
      //                offset2={row_cnt,3'd7,line_cnt[3]};
      //                offset3={row_cnt,3'd6,line_cnt[3]};
      //                offset4={row_cnt,3'd5,line_cnt[3]};
      //                offset5={row_cnt,3'd4,line_cnt[3]};
      //                offset6={row_cnt,3'd3,line_cnt[3]};
      //                offset7={row_cnt,3'd2,line_cnt[3]}; end
      //       2: begin offset0={row_cnt,3'd2,line_cnt[3]};
      //                offset1={row_cnt,3'd1,line_cnt[3]};
      //                offset2={row_cnt,3'd0,line_cnt[3]};
      //                offset3={row_cnt,3'd7,line_cnt[3]};
      //                offset4={row_cnt,3'd6,line_cnt[3]};
      //                offset5={row_cnt,3'd5,line_cnt[3]};
      //                offset6={row_cnt,3'd4,line_cnt[3]};
      //                offset7={row_cnt,3'd3,line_cnt[3]}; end
      //       3: begin offset0={row_cnt,3'd3,line_cnt[3]};
      //  	      offset1={row_cnt,3'd2,line_cnt[3]};
      //                offset2={row_cnt,3'd1,line_cnt[3]};
      //                offset3={row_cnt,3'd0,line_cnt[3]};
      //                offset4={row_cnt,3'd7,line_cnt[3]};
      //                offset5={row_cnt,3'd6,line_cnt[3]};
      //                offset6={row_cnt,3'd5,line_cnt[3]};
      //                offset7={row_cnt,3'd4,line_cnt[3]}; end
      //       4: begin offset0={row_cnt,3'd4,line_cnt[3]};
      //                offset1={row_cnt,3'd3,line_cnt[3]};
      //                offset2={row_cnt,3'd2,line_cnt[3]};
      //                offset3={row_cnt,3'd1,line_cnt[3]};
      //                offset4={row_cnt,3'd0,line_cnt[3]};
      //                offset5={row_cnt,3'd7,line_cnt[3]};
      //                offset6={row_cnt,3'd6,line_cnt[3]};
      //                offset7={row_cnt,3'd5,line_cnt[3]}; end
      //       5: begin offset0={row_cnt,3'd5,line_cnt[3]};
      //  	      offset1={row_cnt,3'd4,line_cnt[3]};
      //                offset2={row_cnt,3'd3,line_cnt[3]};
      //                offset3={row_cnt,3'd2,line_cnt[3]};
      //                offset4={row_cnt,3'd1,line_cnt[3]};
      //                offset5={row_cnt,3'd0,line_cnt[3]};
      //                offset6={row_cnt,3'd7,line_cnt[3]};
      //                offset7={row_cnt,3'd6,line_cnt[3]}; end
      //       6: begin offset0={row_cnt,3'd6,line_cnt[3]};
      //                offset1={row_cnt,3'd5,line_cnt[3]};
      //                offset2={row_cnt,3'd4,line_cnt[3]};
      //                offset3={row_cnt,3'd3,line_cnt[3]};
      //                offset4={row_cnt,3'd2,line_cnt[3]};
      //                offset5={row_cnt,3'd1,line_cnt[3]};
      //                offset6={row_cnt,3'd0,line_cnt[3]};
      //                offset7={row_cnt,3'd7,line_cnt[3]}; end
      //       7: begin offset0={row_cnt,3'd7,line_cnt[3]};
      //  	      offset1={row_cnt,3'd6,line_cnt[3]};
      //                offset2={row_cnt,3'd5,line_cnt[3]};
      //                offset3={row_cnt,3'd4,line_cnt[3]};
      //                offset4={row_cnt,3'd3,line_cnt[3]};
      //                offset5={row_cnt,3'd2,line_cnt[3]};
      //                offset6={row_cnt,3'd1,line_cnt[3]};
      //                offset7={row_cnt,3'd0,line_cnt[3]}; end
      // endcase
      //else 
       casez({|nondisp_d, psize_d})  //t2l. determine offset by row_cnt and line_cnt
        5'b0?_000: case(line_cnt[1:0])
             0: begin offset0={row_cnt,2'h0,line_cnt[2]};  offset1=offset0;
                      offset2={row_cnt,2'h3,line_cnt[2]};  offset3=offset2;
                      offset4={row_cnt,2'h2,line_cnt[2]};  offset5=offset4;
                      offset6={row_cnt,2'h1,line_cnt[2]};  offset7=offset6; end
             1: begin offset0={row_cnt,2'h1,line_cnt[2]};  offset1=offset0;
                      offset2={row_cnt,2'h0,line_cnt[2]};  offset3=offset2;
                      offset4={row_cnt,2'h3,line_cnt[2]};  offset5=offset4;
                      offset6={row_cnt,2'h2,line_cnt[2]};  offset7=offset6; end
             2: begin offset0={row_cnt,2'h2,line_cnt[2]};  offset1=offset0;
                      offset2={row_cnt,2'h1,line_cnt[2]};  offset3=offset2;
                      offset4={row_cnt,2'h0,line_cnt[2]};  offset5=offset4;
                      offset6={row_cnt,2'h3,line_cnt[2]};  offset7=offset6; end
             3: begin offset0={row_cnt,2'h3,line_cnt[2]};  offset1=offset0;
                      offset2={row_cnt,2'h2,line_cnt[2]};  offset3=offset2;
                      offset4={row_cnt,2'h1,line_cnt[2]};  offset5=offset4;
                      offset6={row_cnt,2'h0,line_cnt[2]};  offset7=offset6; end
            endcase
         5'b01_001: case(line_cnt[1:0])
	     0: begin offset0={row_cnt,1'b0,line_cnt[2],1'b0};  offset1=offset0; 
                      offset2={row_cnt,1'h1,line_cnt[2],1'h1};  offset3=offset2; 
                      offset4={row_cnt,1'h1,line_cnt[2],1'b0};  offset5=offset4; 
                      offset6={row_cnt,1'h0,line_cnt[2],1'h1};  offset7=offset6; end
	     1: begin offset0={row_cnt,1'b0,line_cnt[2],1'b1};  offset1=offset0; 
                      offset2={row_cnt,1'h0,line_cnt[2],1'h0};  offset3=offset2; 
                      offset4={row_cnt,1'h1,line_cnt[2],1'b1};  offset5=offset4; 
                      offset6={row_cnt,1'h1,line_cnt[2],1'h0};  offset7=offset6; end
	     2: begin offset0={row_cnt,1'b1,line_cnt[2],1'b0};  offset1=offset0; 
                      offset2={row_cnt,1'h0,line_cnt[2],1'h1};  offset3=offset2; 
                      offset4={row_cnt,1'h0,line_cnt[2],1'b0};  offset5=offset4; 
                      offset6={row_cnt,1'h1,line_cnt[2],1'h1};  offset7=offset6; end
	     3: begin offset0={row_cnt,1'b1,line_cnt[2],1'b1};  offset1=offset0; 
                      offset2={row_cnt,1'h1,line_cnt[2],1'h0};  offset3=offset2; 
                      offset4={row_cnt,1'h0,line_cnt[2],1'b1};  offset5=offset4; 
                      offset6={row_cnt,1'h0,line_cnt[2],1'h0};  offset7=offset6; end
            endcase
         5'b01_010: 
	     begin offset0={row_cnt,line_cnt[2],line_cnt[0],line_cnt[1]};  {offset1,offset2,offset3}={3{offset0}}; 
		   offset4={row_cnt,line_cnt[2],~line_cnt[0],line_cnt[1]};  {offset5,offset6,offset7}={3{offset4}}; end
         5'b01_011: 
	     begin offset0={row_cnt[2:1],line_cnt[2],row_cnt[0],line_cnt[1], line_cnt[0]};  {offset1,offset2,offset3}={3{offset0}}; 
		   offset4={row_cnt[2:1],line_cnt[2],row_cnt[0],line_cnt[1], ~line_cnt[0]};  {offset5,offset6,offset7}={3{offset4}}; end
         5'b00_001: 
	     begin offset0={row_cnt,line_cnt[0], line_cnt[2:1]};  {offset1,offset2,offset3}={3{offset0}}; 
		   offset4={row_cnt,~line_cnt[0], line_cnt[2:1]};  {offset5,offset6,offset7}={3{offset4}}; end
         5'b00_010: 
	     begin offset0={row_cnt,line_cnt[2:1],line_cnt[0]};  {offset1,offset2,offset3}={3{offset0}}; 
		   offset4={row_cnt,line_cnt[2:1],~line_cnt[0]};  {offset5,offset6,offset7}={3{offset4}}; end
         5'b00_011:
	     begin offset0={row_cnt[2:1],line_cnt[2],line_cnt[1],row_cnt[0], line_cnt[0]};  {offset1,offset2,offset3}={3{offset0}}; 
		   offset4={row_cnt[2:1],line_cnt[2],line_cnt[1],row_cnt[0], ~line_cnt[0]};  {offset5,offset6,offset7}={3{offset4}}; end
       endcase
     end	//end of t2l
     else 
     begin
     if(t2t_d)    //Rotate tile T2T
       case(psize_d)
`ifdef SDMA__DISP_TO_ROTATE__1
         3'b000:
	       begin offset0=ie_cnt[0];  {offset1,offset2,offset3}={3{offset0}};
		     offset4={6'b0, ~ie_cnt[0]}; {offset5,offset6,offset7}={3{offset4}}; end
         3'b001: case(ie_cnt[1:0])
	       0: begin offset0=0; offset1=0; offset2=3; offset3=3; offset4=2; offset5=2; offset6=1; offset7=1; end
	       1: begin offset0=1; offset1=1; offset2=0; offset3=0; offset4=3; offset5=3; offset6=2; offset7=2; end
	       2: begin offset0=2; offset1=2; offset2=1; offset3=1; offset4=0; offset5=0; offset6=3; offset7=3; end
	       3: begin offset0=3; offset1=3; offset2=2; offset3=2; offset4=1; offset5=1; offset6=0; offset7=0; end
                 endcase
         3'b010: begin
                 case(ie_cnt[2:0])
	       0: begin offset0=0; offset4=2; end
	       1: begin offset0=4; offset4=6; end
	       2: begin offset0=2; offset4=0; end
	       3: begin offset0=6; offset4=4; end
	       4: begin offset0=1; offset4=3; end
	       5: begin offset0=5; offset4=7; end
	       6: begin offset0=3; offset4=1; end
	       7: begin offset0=7; offset4=5; end
                 endcase
		 {offset1,offset2,offset3}={3{offset0}}; {offset5,offset6,offset7}={3{offset4}};
                 end
         3'b011:
                 begin
                 case(ie_cnt[3:0])
	       0: begin offset0=0; end
	       1: begin offset0=4; end
	       2: begin offset0=1; end
	       3: begin offset0=5; end
	       4: begin offset0=2; end
	       5: begin offset0=6; end
	       6: begin offset0=3; end
	       7: begin offset0=7; end
	       8: begin offset0=8; end
	       9: begin offset0=12; end
	       10: begin offset0=9; end
	       11: begin offset0=13; end
	       12: begin offset0=10; end
	       13: begin offset0=14; end
	       14: begin offset0=11; end
	       15: begin offset0=15; end
                 endcase
	         {offset1,offset2,offset3,offset4,offset5,offset6,offset7}={7{offset0}};
		 end
`endif
        default: begin offset0=0; offset1=0; offset2=0; offset3=0; offset4=0; offset5=0; offset6=0; offset7=0; end
       endcase
     //else if (structt)	//l2s 
     //  case(ie_cnt[3:1])
     //       0: begin offset0={ie_cnt[0],3'd0}*chunk_size[6:4]+ie_cnt[5:4];
     //       	     offset1={ie_cnt[0],3'd7}*chunk_size[6:4]+ie_cnt[5:4];
     //       	     offset2={ie_cnt[0],3'd6}*chunk_size[6:4]+ie_cnt[5:4];
     //       	     offset3={ie_cnt[0],3'd5}*chunk_size[6:4]+ie_cnt[5:4];
     //       	     offset4={ie_cnt[0],3'd4}*chunk_size[6:4]+ie_cnt[5:4];
     //       	     offset5={ie_cnt[0],3'd3}*chunk_size[6:4]+ie_cnt[5:4];
     //       	     offset6={ie_cnt[0],3'd2}*chunk_size[6:4]+ie_cnt[5:4];
     //       	     offset7={ie_cnt[0],3'd1}*chunk_size[6:4]+ie_cnt[5:4]; end
     //       1: begin offset0={ie_cnt[0],3'd1}*chunk_size[6:4]+ie_cnt[5:4];
     //                offset1={ie_cnt[0],3'd0}*chunk_size[6:4]+ie_cnt[5:4];
     //                offset2={ie_cnt[0],3'd7}*chunk_size[6:4]+ie_cnt[5:4];
     //                offset3={ie_cnt[0],3'd6}*chunk_size[6:4]+ie_cnt[5:4];
     //                offset4={ie_cnt[0],3'd5}*chunk_size[6:4]+ie_cnt[5:4];
     //                offset5={ie_cnt[0],3'd4}*chunk_size[6:4]+ie_cnt[5:4];
     //                offset6={ie_cnt[0],3'd3}*chunk_size[6:4]+ie_cnt[5:4];
     //                offset7={ie_cnt[0],3'd2}*chunk_size[6:4]+ie_cnt[5:4]; end
     //       2: begin offset0={ie_cnt[0],3'd2}*chunk_size[6:4]+ie_cnt[5:4];
     //                offset1={ie_cnt[0],3'd1}*chunk_size[6:4]+ie_cnt[5:4];
     //                offset2={ie_cnt[0],3'd0}*chunk_size[6:4]+ie_cnt[5:4];
     //                offset3={ie_cnt[0],3'd7}*chunk_size[6:4]+ie_cnt[5:4];
     //                offset4={ie_cnt[0],3'd6}*chunk_size[6:4]+ie_cnt[5:4];
     //                offset5={ie_cnt[0],3'd5}*chunk_size[6:4]+ie_cnt[5:4];
     //                offset6={ie_cnt[0],3'd4}*chunk_size[6:4]+ie_cnt[5:4];
     //                offset7={ie_cnt[0],3'd3}*chunk_size[6:4]+ie_cnt[5:4]; end
     //       3: begin offset0={ie_cnt[0],3'd3}*chunk_size[6:4]+ie_cnt[5:4];
     //                offset1={ie_cnt[0],3'd2}*chunk_size[6:4]+ie_cnt[5:4];
     //                offset2={ie_cnt[0],3'd1}*chunk_size[6:4]+ie_cnt[5:4];
     //                offset3={ie_cnt[0],3'd0}*chunk_size[6:4]+ie_cnt[5:4];
     //                offset4={ie_cnt[0],3'd7}*chunk_size[6:4]+ie_cnt[5:4];
     //                offset5={ie_cnt[0],3'd6}*chunk_size[6:4]+ie_cnt[5:4];
     //                offset6={ie_cnt[0],3'd5}*chunk_size[6:4]+ie_cnt[5:4];
     //                offset7={ie_cnt[0],3'd4}*chunk_size[6:4]+ie_cnt[5:4]; end
     //       4: begin offset0={ie_cnt[0],3'd4}*chunk_size[6:4]+ie_cnt[5:4];
     //                offset1={ie_cnt[0],3'd3}*chunk_size[6:4]+ie_cnt[5:4];
     //                offset2={ie_cnt[0],3'd2}*chunk_size[6:4]+ie_cnt[5:4];
     //                offset3={ie_cnt[0],3'd1}*chunk_size[6:4]+ie_cnt[5:4];
     //                offset4={ie_cnt[0],3'd0}*chunk_size[6:4]+ie_cnt[5:4];
     //                offset5={ie_cnt[0],3'd7}*chunk_size[6:4]+ie_cnt[5:4];
     //                offset6={ie_cnt[0],3'd6}*chunk_size[6:4]+ie_cnt[5:4];
     //                offset7={ie_cnt[0],3'd5}*chunk_size[6:4]+ie_cnt[5:4]; end
     //       5: begin offset0={ie_cnt[0],3'd5}*chunk_size[6:4]+ie_cnt[5:4];
     //                offset1={ie_cnt[0],3'd4}*chunk_size[6:4]+ie_cnt[5:4];
     //                offset2={ie_cnt[0],3'd3}*chunk_size[6:4]+ie_cnt[5:4];
     //                offset3={ie_cnt[0],3'd2}*chunk_size[6:4]+ie_cnt[5:4];
     //                offset4={ie_cnt[0],3'd1}*chunk_size[6:4]+ie_cnt[5:4];
     //                offset5={ie_cnt[0],3'd0}*chunk_size[6:4]+ie_cnt[5:4];
     //                offset6={ie_cnt[0],3'd7}*chunk_size[6:4]+ie_cnt[5:4];
     //                offset7={ie_cnt[0],3'd6}*chunk_size[6:4]+ie_cnt[5:4]; end
     //       6: begin offset0={ie_cnt[0],3'd6}*chunk_size[6:4]+ie_cnt[5:4];
     //                offset1={ie_cnt[0],3'd5}*chunk_size[6:4]+ie_cnt[5:4];
     //                offset2={ie_cnt[0],3'd4}*chunk_size[6:4]+ie_cnt[5:4];
     //                offset3={ie_cnt[0],3'd3}*chunk_size[6:4]+ie_cnt[5:4];
     //                offset4={ie_cnt[0],3'd2}*chunk_size[6:4]+ie_cnt[5:4];
     //                offset5={ie_cnt[0],3'd1}*chunk_size[6:4]+ie_cnt[5:4];
     //                offset6={ie_cnt[0],3'd0}*chunk_size[6:4]+ie_cnt[5:4];
     //                offset7={ie_cnt[0],3'd7}*chunk_size[6:4]+ie_cnt[5:4]; end
     //       7: begin offset0={ie_cnt[0],3'd7}*chunk_size[6:4]+ie_cnt[5:4];
     //                offset1={ie_cnt[0],3'd6}*chunk_size[6:4]+ie_cnt[5:4];
     //                offset2={ie_cnt[0],3'd5}*chunk_size[6:4]+ie_cnt[5:4];
     //                offset3={ie_cnt[0],3'd4}*chunk_size[6:4]+ie_cnt[5:4];
     //                offset4={ie_cnt[0],3'd3}*chunk_size[6:4]+ie_cnt[5:4];
     //                offset5={ie_cnt[0],3'd2}*chunk_size[6:4]+ie_cnt[5:4];
     //                offset6={ie_cnt[0],3'd1}*chunk_size[6:4]+ie_cnt[5:4];
     //                offset7={ie_cnt[0],3'd0}*chunk_size[6:4]+ie_cnt[5:4]; end
     //       endcase
     else
       casez({|nondisp_d, psize_d}) //l2t. determine offset by ie_cnt
         5'b0?_000: 
	   if (nondisp_d == 4)	//thick tile 8bpp
            case({ie_cnt[3:2]})
	    0: begin offset0={ie_cnt[1],2'b00}*chunk_size_d[6:3]+ie_cnt[6:4];  offset1=offset0;
		     offset2={ie_cnt[1],2'b11}*chunk_size_d[6:3]+ie_cnt[6:4];  offset3=offset2; 
		     offset4={ie_cnt[1],2'b10}*chunk_size_d[6:3]+ie_cnt[6:4];  offset5=offset4; 
		     offset6={ie_cnt[1],2'b01}*chunk_size_d[6:3]+ie_cnt[6:4];  offset7=offset6; end
	    1: begin offset0={ie_cnt[1],2'b01}*chunk_size_d[6:3]+ie_cnt[6:4];  offset1=offset0;
		     offset2={ie_cnt[1],2'b00}*chunk_size_d[6:3]+ie_cnt[6:4];  offset3=offset2; 
		     offset4={ie_cnt[1],2'b11}*chunk_size_d[6:3]+ie_cnt[6:4];  offset5=offset4; 
		     offset6={ie_cnt[1],2'b10}*chunk_size_d[6:3]+ie_cnt[6:4];  offset7=offset6; end
	    2: begin offset0={ie_cnt[1],2'b10}*chunk_size_d[6:3]+ie_cnt[6:4];  offset1=offset0;
		     offset2={ie_cnt[1],2'b01}*chunk_size_d[6:3]+ie_cnt[6:4];  offset3=offset2; 
		     offset4={ie_cnt[1],2'b00}*chunk_size_d[6:3]+ie_cnt[6:4];  offset5=offset4; 
		     offset6={ie_cnt[1],2'b11}*chunk_size_d[6:3]+ie_cnt[6:4];  offset7=offset6; end
	    3: begin offset0={ie_cnt[1],2'b11}*chunk_size_d[6:3]+ie_cnt[6:4];  offset1=offset0;
		     offset2={ie_cnt[1],2'b10}*chunk_size_d[6:3]+ie_cnt[6:4];  offset3=offset2; 
		     offset4={ie_cnt[1],2'b01}*chunk_size_d[6:3]+ie_cnt[6:4];  offset5=offset4; 
		     offset6={ie_cnt[1],2'b00}*chunk_size_d[6:3]+ie_cnt[6:4];  offset7=offset6; end
            endcase
	   else
            case({ie_cnt[2:1]})
	    0: begin offset0={ie_cnt[0],2'b00}*chunk_size_d[6:3]+ie_cnt[5:3];  offset1=offset0;
		     offset2={ie_cnt[0],2'b11}*chunk_size_d[6:3]+ie_cnt[5:3];  offset3=offset2; 
		     offset4={ie_cnt[0],2'b10}*chunk_size_d[6:3]+ie_cnt[5:3];  offset5=offset4; 
		     offset6={ie_cnt[0],2'b01}*chunk_size_d[6:3]+ie_cnt[5:3];  offset7=offset6; end
	    1: begin offset0={ie_cnt[0],2'b01}*chunk_size_d[6:3]+ie_cnt[5:3];  offset1=offset0;
		     offset2={ie_cnt[0],2'b00}*chunk_size_d[6:3]+ie_cnt[5:3];  offset3=offset2; 
		     offset4={ie_cnt[0],2'b11}*chunk_size_d[6:3]+ie_cnt[5:3];  offset5=offset4; 
		     offset6={ie_cnt[0],2'b10}*chunk_size_d[6:3]+ie_cnt[5:3];  offset7=offset6; end
	    2: begin offset0={ie_cnt[0],2'b10}*chunk_size_d[6:3]+ie_cnt[5:3];  offset1=offset0;
		     offset2={ie_cnt[0],2'b01}*chunk_size_d[6:3]+ie_cnt[5:3];  offset3=offset2; 
		     offset4={ie_cnt[0],2'b00}*chunk_size_d[6:3]+ie_cnt[5:3];  offset5=offset4; 
		     offset6={ie_cnt[0],2'b11}*chunk_size_d[6:3]+ie_cnt[5:3];  offset7=offset6; end
	    3: begin offset0={ie_cnt[0],2'b11}*chunk_size_d[6:3]+ie_cnt[5:3];  offset1=offset0;
		     offset2={ie_cnt[0],2'b10}*chunk_size_d[6:3]+ie_cnt[5:3];  offset3=offset2; 
		     offset4={ie_cnt[0],2'b01}*chunk_size_d[6:3]+ie_cnt[5:3];  offset5=offset4; 
		     offset6={ie_cnt[0],2'b00}*chunk_size_d[6:3]+ie_cnt[5:3];  offset7=offset6; end
            endcase
         5'b01_001: case({ie_cnt[2],ie_cnt[0]})
	    0: begin offset0={ie_cnt[1],2'b00}*chunk_size_d[6:3]+ie_cnt[5:3];  offset1=offset0;
		     offset2={ie_cnt[1],2'b11}*chunk_size_d[6:3]+ie_cnt[5:3];  offset3=offset2; 
		     offset4={ie_cnt[1],2'b10}*chunk_size_d[6:3]+ie_cnt[5:3];  offset5=offset4; 
		     offset6={ie_cnt[1],2'b01}*chunk_size_d[6:3]+ie_cnt[5:3];  offset7=offset6; end
	    1: begin offset0={ie_cnt[1],2'b01}*chunk_size_d[6:3]+ie_cnt[5:3];  offset1=offset0;
		     offset2={ie_cnt[1],2'b00}*chunk_size_d[6:3]+ie_cnt[5:3];  offset3=offset2; 
		     offset4={ie_cnt[1],2'b11}*chunk_size_d[6:3]+ie_cnt[5:3];  offset5=offset4; 
		     offset6={ie_cnt[1],2'b10}*chunk_size_d[6:3]+ie_cnt[5:3];  offset7=offset6; end
	    2: begin offset0={ie_cnt[1],2'b10}*chunk_size_d[6:3]+ie_cnt[5:3];  offset1=offset0;
		     offset2={ie_cnt[1],2'b01}*chunk_size_d[6:3]+ie_cnt[5:3];  offset3=offset2; 
		     offset4={ie_cnt[1],2'b00}*chunk_size_d[6:3]+ie_cnt[5:3];  offset5=offset4; 
		     offset6={ie_cnt[1],2'b11}*chunk_size_d[6:3]+ie_cnt[5:3];  offset7=offset6; end
	    3: begin offset0={ie_cnt[1],2'b11}*chunk_size_d[6:3]+ie_cnt[5:3];  offset1=offset0;
		     offset2={ie_cnt[1],2'b10}*chunk_size_d[6:3]+ie_cnt[5:3];  offset3=offset2; 
		     offset4={ie_cnt[1],2'b01}*chunk_size_d[6:3]+ie_cnt[5:3];  offset5=offset4; 
		     offset6={ie_cnt[1],2'b00}*chunk_size_d[6:3]+ie_cnt[5:3];  offset7=offset6; end
            endcase
         5'b01_010:
	       begin offset0={ie_cnt[2],ie_cnt[0],ie_cnt[1]}*chunk_size_d[6:3]+ie_cnt[5:3];  {offset1,offset2,offset3}={3{offset0}};
		     offset4={ie_cnt[2],ie_cnt[0],~ie_cnt[1]}*chunk_size_d[6:3]+ie_cnt[5:3];  {offset5,offset6,offset7}={3{offset4}}; end
         5'b01_011:
	       begin offset0={ie_cnt[3],ie_cnt[1],ie_cnt[0]}*chunk_size_d[6:3]+{ie_cnt[5:4],ie_cnt[2]};  {offset1,offset2,offset3}={3{offset0}};
		     offset4={ie_cnt[3],ie_cnt[1],~ie_cnt[0]}*chunk_size_d[6:3]+{ie_cnt[5:4],ie_cnt[2]};  {offset5,offset6,offset7}={3{offset4}}; end
         5'b00_001:
	       begin offset0={ie_cnt[1:0],ie_cnt[2]}*chunk_size_d[6:3]+ie_cnt[5:3];  {offset1,offset2,offset3}={3{offset0}};
		     offset4={ie_cnt[1:0],~ie_cnt[2]}*chunk_size_d[6:3]+ie_cnt[5:3];  {offset5,offset6,offset7}={3{offset4}}; end
         5'b00_010:
	       begin offset0={ie_cnt[2:1],ie_cnt[0]}*chunk_size_d[6:3]+ie_cnt[5:3];  {offset1,offset2,offset3}={3{offset0}};
		     offset4={ie_cnt[2:1],~ie_cnt[0]}*chunk_size_d[6:3]+ie_cnt[5:3];  {offset5,offset6,offset7}={3{offset4}}; end
         5'b00_011:
	       begin offset0={ie_cnt[3],ie_cnt[2],ie_cnt[0]}*chunk_size_d[6:3]+{ie_cnt[5:4],ie_cnt[1]};  {offset1,offset2,offset3}={3{offset0}};
		     offset4={ie_cnt[3],ie_cnt[2],~ie_cnt[0]}*chunk_size_d[6:3]+{ie_cnt[5:4],ie_cnt[1]};  {offset5,offset6,offset7}={3{offset4}}; end
       endcase
     end
     ie_addr0 = chunk_base + offset0;
     ie_addr1 = chunk_base + offset1;
     ie_addr2 = chunk_base + offset2;
     ie_addr3 = chunk_base + offset3;
     ie_addr4 = chunk_base + offset4;
     ie_addr5 = chunk_base + offset5;
     ie_addr6 = chunk_base + offset6;
     ie_addr7 = chunk_base + offset7;
     ie_addr0 = &ie_addr0[`DMA_RD_DATA_BUF_DEPTH-1:`DMA_RD_DATA_BUF_DEPTH-2] ? {2'b00, ie_addr0[`DMA_RD_DATA_BUF_DEPTH-3:0]} : ie_addr0;
     ie_addr1 = &ie_addr1[`DMA_RD_DATA_BUF_DEPTH-1:`DMA_RD_DATA_BUF_DEPTH-2] ? {2'b00, ie_addr1[`DMA_RD_DATA_BUF_DEPTH-3:0]} : ie_addr1;
     ie_addr2 = &ie_addr2[`DMA_RD_DATA_BUF_DEPTH-1:`DMA_RD_DATA_BUF_DEPTH-2] ? {2'b00, ie_addr2[`DMA_RD_DATA_BUF_DEPTH-3:0]} : ie_addr2;
     ie_addr3 = &ie_addr3[`DMA_RD_DATA_BUF_DEPTH-1:`DMA_RD_DATA_BUF_DEPTH-2] ? {2'b00, ie_addr3[`DMA_RD_DATA_BUF_DEPTH-3:0]} : ie_addr3;
     ie_addr4 = &ie_addr4[`DMA_RD_DATA_BUF_DEPTH-1:`DMA_RD_DATA_BUF_DEPTH-2] ? {2'b00, ie_addr4[`DMA_RD_DATA_BUF_DEPTH-3:0]} : ie_addr4;
     ie_addr5 = &ie_addr5[`DMA_RD_DATA_BUF_DEPTH-1:`DMA_RD_DATA_BUF_DEPTH-2] ? {2'b00, ie_addr5[`DMA_RD_DATA_BUF_DEPTH-3:0]} : ie_addr5;
     ie_addr6 = &ie_addr6[`DMA_RD_DATA_BUF_DEPTH-1:`DMA_RD_DATA_BUF_DEPTH-2] ? {2'b00, ie_addr6[`DMA_RD_DATA_BUF_DEPTH-3:0]} : ie_addr6;
     ie_addr7 = &ie_addr7[`DMA_RD_DATA_BUF_DEPTH-1:`DMA_RD_DATA_BUF_DEPTH-2] ? {2'b00, ie_addr7[`DMA_RD_DATA_BUF_DEPTH-3:0]} : ie_addr7;
     if(ie_d) begin
	ram_mbank_data_raddr_normal = {ie_addr7, ie_addr6, ie_addr5, ie_addr4, ie_addr3, ie_addr2, ie_addr1, ie_addr0};
     end
     else begin
         ram_mbank_data_raddr_normal = {8{chunk_base}};
     end
   end

   always @(posedge clk) begin
      if (~resetb) begin
         input_valid_gen_d <= 1'b0;
	 input_valid <= 1'b0;
         output_valid <= 1'b0;
      end
      else begin
	 // input flop inside memory is valid
	 input_valid_gen_d <= next_input_valid && !((structt ? chunk_size[6:4]==ie_cnt[6:4]+1 : chunk_size[6:3]==(thick_8bpp_ie ? ie_cnt[7:4] : ie_cnt[6:3])+1) && ie && ~t2l & |src_start[4:2]);
         input_valid <= input_valid_gen_d;
         output_valid <= next_output_valid;
      end // else: !if(~resetb)
   end 
   
   assign ram_rd_data_rptr_inc = next_input_valid & (thick_8bpp_ie && ~t2l ? ie_cnt[0] : 1'b1);

   //2-level info/counter pipeline, match memory read
   always @(posedge clk) begin
      if (~resetb) begin
      end 
      else begin
	 if(next_input_valid) begin
            ie_input <= ie;
            t2l_input <= t2l;
            t2t_input <= t2t;
	    structt_input <= structt;
	    granu_input <= granu;
            nondisp_input <= nondisp;
            psize_input <= psize;
            ie_cnt_input <= ie_cnt;
            row_cnt_input <= row_cnt;
            line_cnt_input <= line_cnt;
	    ecc_input <= next_ecc;
	 end
         if(input_valid_gen_d) begin
            ie_input_d <= ie_input;
            t2l_input_d <= t2l_input;
            t2t_input_d <= t2t_input;
            structt_input_d <= structt_input;
            granu_input_d <= granu_input;
            nondisp_input_d <= nondisp_input;
            psize_input_d <= psize_input;
            ie_cnt_input_d <= ie_cnt_input;
            row_cnt_input_d <= row_cnt_input;
            line_cnt_input_d <= line_cnt_input;
            ecc_input_d <= ecc_input;
         end
         if(input_valid) begin
            ie_output <= ie_input_d;
            t2l_output <= t2l_input_d;
            t2t_output <= t2t_input_d;
	    structt_output <= structt_input_d;
	    granu_output <= granu_input_d;
            nondisp_output <= nondisp_input_d;
            psize_output <= psize_input_d;
            ie_cnt_output <= ie_cnt_input_d;
            row_cnt_output <= row_cnt_input_d;
            line_cnt_output <= line_cnt_input_d;
	    ecc_output <= ecc_input_d;
         end
      end
   end

   wire[31:0] d7,d6,d5,d4,d3,d2,d1,d0;
   assign {d7,d6,d5,d4,d3,d2,d1,d0} = ram_mbank_data_dout;  

   wire[15:0] d7h,d6h,d5h,d4h,d3h,d2h,d1h,d0h, d7l,d6l,d5l,d4l,d3l,d2l,d1l,d0l;
   assign {d7h,d7l,d6h,d6l,d5h,d5l,d4h,d4l,d3h,d3l,d2h,d2l,d1h,d1l,d0h,d0l} = ram_mbank_data_dout;

   always @* begin
     if(ie_output) begin
       mem_rdata = ram_mbank_data_dout;
       if(t2t_output) begin   //Rotate tile T2T
`ifdef SDMA__DISP_TO_ROTATE__1
	 case(psize_output)
           3'b000:
           case(ie_cnt_output[0])  //num
           0:  mem_rdata = {d7,d3,d6,d2,d5,d1,d4,d0} ;
           1:  mem_rdata = {d3,d7,d2,d6,d1,d5,d0,d4} ;
           endcase
           3'b001:
           case(ie_cnt_output[1:0])  //num
           0:  mem_rdata = {d3,d5,d7,d1,d2,d4,d6,d0} ;
           1:  mem_rdata = {d5,d7,d1,d3,d4,d6,d0,d2} ;
           2:  mem_rdata = {d7,d1,d3,d5,d6,d0,d2,d4} ;
           3:  mem_rdata = {d1,d3,d5,d7,d0,d2,d4,d6} ;
           endcase
           3'b010:
           case(ie_cnt_output[1])  //num
           0:  mem_rdata = {d7,d6,d3,d2,d5,d4,d1,d0} ;
           1:  mem_rdata = {d3,d2,d7,d6,d1,d0,d5,d4} ;
           endcase
           3'b011:
               mem_rdata = {d7,d6,d5,d4,d3,d2,d1,d0} ;
         endcase
`endif
       end
       else if(t2l_output) begin
         if(structt_output)
           case(line_cnt_output[2:0])  //num
           0:  mem_rdata = {d1,d2,d3,d4,d5,d6,d7,d0};
           1:  mem_rdata = {d2,d3,d4,d5,d6,d7,d0,d1};
           2:  mem_rdata = {d3,d4,d5,d6,d7,d0,d1,d2};
           3:  mem_rdata = {d4,d5,d6,d7,d0,d1,d2,d3};
           4:  mem_rdata = {d5,d6,d7,d0,d1,d2,d3,d4};
           5:  mem_rdata = {d6,d7,d0,d1,d2,d3,d4,d5};
           6:  mem_rdata = {d7,d0,d1,d2,d3,d4,d5,d6};
           7:  mem_rdata = {d0,d1,d2,d3,d4,d5,d6,d7};
           endcase
	//thick tile 8bpp case interleave operation like 64b granu
         else if(granu_output == `DMA_MC_wr_granularity_min_64b || granu_output == `DMA_MC_wr_granularity_min_32b)
           case(line_cnt_output[1:0])  //num
           0:  mem_rdata = {d3,d2,d5,d4,d7,d6,d1,d0};
           1:  mem_rdata = {d5,d4,d7,d6,d1,d0,d3,d2};
           2:  mem_rdata = {d7,d6,d1,d0,d3,d2,d5,d4};
           3:  mem_rdata = {d1,d0,d3,d2,d5,d4,d7,d6};
           endcase
         else// if(granu_output == `DMA_MC_wr_granularity_min_128b)
           case(line_cnt_output[0])  //num
           0:  mem_rdata = {d7,d6,d5,d4,d3,d2,d1,d0} ;
           1:  mem_rdata = {d3,d2,d1,d0,d7,d6,d5,d4} ;
           endcase
       end

       else begin
	 if (structt_output) //l2s
           case(ie_cnt_output[3:1])  //num
           0:  mem_rdata = {d1,d2,d3,d4,d5,d6,d7,d0} ;
           1:  mem_rdata = {d2,d3,d4,d5,d6,d7,d0,d1} ;
           2:  mem_rdata = {d3,d4,d5,d6,d7,d0,d1,d2} ;
           3:  mem_rdata = {d4,d5,d6,d7,d0,d1,d2,d3} ;
           4:  mem_rdata = {d5,d6,d7,d0,d1,d2,d3,d4} ;
           5:  mem_rdata = {d6,d7,d0,d1,d2,d3,d4,d5} ;
           6:  mem_rdata = {d7,d0,d1,d2,d3,d4,d5,d6} ;
           7:  mem_rdata = {d0,d1,d2,d3,d4,d5,d6,d7} ;
           endcase
         else case({|nondisp_output,psize_output})
          5'b00_000:
           case(ie_cnt_output[2:1])  //num
           0:  mem_rdata = {d3,d2,d7,d6,d5,d4,d1,d0};
           1:  mem_rdata = {d5,d4,d1,d0,d7,d6,d3,d2};
           2:  mem_rdata = {d7,d6,d3,d2,d1,d0,d5,d4};
           3:  mem_rdata = {d1,d0,d5,d4,d3,d2,d7,d6};
           endcase
          5'b01_000:
	  if (nondisp_output==4 && ie_cnt_output[0])     //thick tile 8bpp, change high/low 128b
           case(ie_cnt_output[3:2])  	     
           0:  mem_rdata = {d2h,d4h,d2l,d4l, d6h,d0h,d6l,d0l, d3h,d5h,d3l,d5l, d7h,d1h,d7l,d1l};
           1:  mem_rdata = {d4h,d6h,d4l,d6l, d0h,d2h,d0l,d2l, d5h,d7h,d5l,d7l, d1h,d3h,d1l,d3l};
           2:  mem_rdata = {d6h,d0h,d6l,d0l, d2h,d4h,d2l,d4l, d7h,d1h,d7l,d1l, d3h,d5h,d3l,d5l};
           3:  mem_rdata = {d0h,d2h,d0l,d2l, d4h,d6h,d4l,d6l, d1h,d3h,d1l,d3l, d5h,d7h,d5l,d7l};
           endcase
	  else if (nondisp_output==4 && !ie_cnt_output[0])     //thick tile 8bpp, keep high/low 128b
           case(ie_cnt_output[3:2]) 	   	    
           0:  mem_rdata = {d3h,d5h,d3l,d5l, d7h,d1h,d7l,d1l, d2h,d4h,d2l,d4l, d6h,d0h,d6l,d0l};
           1:  mem_rdata = {d5h,d7h,d5l,d7l, d1h,d3h,d1l,d3l, d4h,d6h,d4l,d6l, d0h,d2h,d0l,d2l};
           2:  mem_rdata = {d7h,d1h,d7l,d1l, d3h,d5h,d3l,d5l, d6h,d0h,d6l,d0l, d2h,d4h,d2l,d4l};
           3:  mem_rdata = {d1h,d3h,d1l,d3l, d5h,d7h,d5l,d7l, d0h,d2h,d0l,d2l, d4h,d6h,d4l,d6l};
           endcase
	  else
           case(ie_cnt_output[2:1])  //num
           0:  mem_rdata = {d3h,d5h,d3l,d5l, d7h,d1h,d7l,d1l, d2h,d4h,d2l,d4l, d6h,d0h,d6l,d0l};
           1:  mem_rdata = {d5h,d7h,d5l,d7l, d1h,d3h,d1l,d3l, d4h,d6h,d4l,d6l, d0h,d2h,d0l,d2l};
           2:  mem_rdata = {d7h,d1h,d7l,d1l, d3h,d5h,d3l,d5l, d6h,d0h,d6l,d0l, d2h,d4h,d2l,d4l};
           3:  mem_rdata = {d1h,d3h,d1l,d3l, d5h,d7h,d5l,d7l, d0h,d2h,d0l,d2l, d4h,d6h,d4l,d6l};
           endcase
          5'b01_001:
           case({ie_cnt_output[2],ie_cnt_output[0]})  //num
           0:  mem_rdata = {d3,d5,d2,d4,d7,d1,d6,d0};
           1:  mem_rdata = {d5,d7,d4,d6,d1,d3,d0,d2};
           2:  mem_rdata = {d7,d1,d6,d0,d3,d5,d2,d4};
           3:  mem_rdata = {d1,d3,d0,d2,d5,d7,d4,d6};
           endcase
          5'b01_010:
           case(ie_cnt_output[1])  //num
           0:  mem_rdata = {d7,d6,d3,d2,d5,d4,d1,d0} ;
           1:  mem_rdata = {d3,d2,d7,d6,d1,d0,d5,d4} ;
           endcase
          5'b00_001:
           case(ie_cnt_output[2])  //num
           0:  mem_rdata = {d7,d6,d5,d4,d3,d2,d1,d0} ;
           1:  mem_rdata = {d3,d2,d1,d0,d7,d6,d5,d4} ;
           endcase
          5'b00_010,
          5'b00_011,
          5'b01_011:
           case(ie_cnt_output[0])  //num
           0:  mem_rdata = {d7,d6,d5,d4,d3,d2,d1,d0} ;
           1:  mem_rdata = {d3,d2,d1,d0,d7,d6,d5,d4} ;
           endcase
         endcase
       end
     end
     else 
       mem_rdata = ram_mbank_data_dout;
   end

   //========================================================================
   //Skid fifo at the output of the read data fifo
   //========================================================================

   wire skid_we;

   assign skid_we = output_valid;

   dma_ati_fifo #(257,3,5,3,"skid_data_fifo",0) skid_data_fifo(
				   .we(skid_we),
				   .re(data_skid_re),
				   .full(skid_almost_full),
				   .empty(skid_empty),
				   .write_data({ecc_output, mem_rdata}),
				   .read_data({out_ecc, data_skid_dout}),
				   .clk(clk),
				   .reset(~resetb)
				   );

   /////////////////////
   // dma_idle interface
   /////////////////////
   always @(posedge clk) begin
      if (~resetb) begin
         dma_ce_out_idle <= 1'b1;
      end
      else begin
         dma_ce_out_idle <= skid_empty && cmd_info_empty && ie_cnt==0
                && !input_valid && !output_valid && out_state==state_start;
      end
   end

endmodule 
