`include "dma_common.vh"
`include "sdma_reg.v"

module dma_ce_rreq_bc (

   input clk ,
   input resetb ,

   input split_rts,
   input [`CMD_PARAM_RANGE] split_param,

   input bc_mode, 

   //rd_ret I/F
   input                               MC_DMA_rdret_vld,
   input [`MC_DMA_RDRET_TAG_SIZE-1:0]  MC_DMA_rdret_tag,
   input                               MC_DMA_rdret_last,

   //data buffer interface
   input [`DMA_RD_DATA_BUF_DEPTH:0]                  data_rreq_alloc_free ,
   input [`DMA_RD_DATA_BUF_DEPTH:0]                  data_rreq_alloc_used ,
   input [`DMA_RD_DATA_BUF_DEPTH:0]                  data_rreq_alloc_wptr ,
   input [3:0]                                       data_rreq_alloc_wptr_inc ,
   input [2:0]                                       next_quarter_entry ,
   //use to calculate alloc_wptr
   output reg[`DMA_MC_RDREQ_DWORD_SIZE-1:0]          ce_rd_dw_size ,
   output                                            next_ce_rd_rts ,
   output reg                                        ce_rd_t2l_last ,

   input [`SDMA0_GB_ADDR_CONFIG_PIPE_INTERLEAVE_SIZE_SIZE-1:0] dma_addr_conf_pipe_interleave_size ,

   //memory interface
   input  mc_rd_rtr , //indicate mc_rd have credit to send out read requests
   input  next_mc_rd_rtr , //use to generate rreq_rts signal
   output rreq_rtr,

   output                                              ce_rd_vld ,
   output reg[`DMA_MC_RDREQ_ADDR_SIZE-1+4:0]           ce_rd_addr,   //extending for hash.  MSB = i+j*dma_bank_bits = 7+7*4=35
   output reg[1:0]                                     ce_rd_size,
   output reg[`DMA_MC_RDREQ_SWAP_SIZE-1:0]             ce_rd_swap,
   output reg[`DMA_MC_RDREQ_MASK_SIZE-1:0]             ce_rd_mask,
   output reg[`DMA_MC_RDREQ_TAG_SIZE-1:0]              ce_rd_tag,
   output reg                                          ce_rd_stall,
   output reg                                          ce_rd_priv,
   output reg[`SDMA0_GFX_RB_CNTL_RB_VMID_SIZE-1:0]     ce_rd_vmid,
   output reg[`DMA_MC_RDNFO_URG_SIZE-1:0]              ce_rd_urg,
   output reg                                          ce_rd_tci,
   output reg                                          ce_rd_vol,
   output reg                                          ce_rd_pol,

`ifdef SDMA__UTC_L1__1
   input					       dma_utc_l1_enable,
   input					       l1_alloc_vld,
   input [`DMA_MC_RDREQ_CE_ALLOC_WIDTH -1:0]	       l1_alloc_data,	
   output [`DMA_MC_RDREQ_CE_ALLOC_WIDTH -1:0]          ce_rd_lut_info,
   output                                              ce_rd_stall_atc,
`endif

   //LUT write data
   output [`DMA_DATA_LUT_WIDTH-1:0]  ram_data_lut_din,
   output reg[`DMA_DATA_LUT_DEPTH-1:0]  ram_data_lut_waddr,
   output                            ram_data_lut_we,
   input                             ram_rd_data_rptr_inc,
   input [1:0]                       out_granu,
   input [6:0]                       out_chunk_size,
   input                             out_ie,

   input [`SDMA0_HASH_CHANNEL_BITS_SIZE-1:0]      dma_channel_bits,
   input [`SDMA0_HASH_BANK_BITS_SIZE-1:0]         dma_bank_bits,
   input [`SDMA0_HASH_CHANNEL_XOR_COUNT_SIZE-1:0] dma_channel_xor,
   input [`SDMA0_HASH_BANK_XOR_COUNT_SIZE-1:0]    dma_bank_xor,
   input [`SDMA0_RD_BURST_CNTL_RD_BURST_SIZE-1:0] rd_burst,
   input [`SDMA0_CHICKEN_BITS_CE_LFIFO_WATERMARK_SIZE-1:0] 	lfifo_watermark,
 
   output reg [`DMA_RD_DATA_BUF_DEPTH:0] rd_data_dbg_alloc_free,
   output reg dma_ce_rreq_idle
);

   reg video, broadcast;
   reg[2:0] voidd;
   reg[1:0] swap;
   reg tci, vol, pol;
   reg crc, hash, priv;
   reg [`SDMA0_GFX_RB_CNTL_RB_VMID_SIZE-1:0] vmid;
   reg trns;
   reg aes_cbc;
   reg aes;

   reg subwin;
   reg linear;
   reg [1:0] granu;
   reg [5:0] tilenum;
   reg [`CMD_COUNT_SIZE-1+1:0] ssize;
   reg [`CMD_ADDR_SIZE-1:0]            taddr, taddr1;
   reg [`CMD_ADDR_SIZE-1:0]            laddr;
   reg [`SDMA_PKT_COPY_TILED_BC_MIT_MODE_SIZE-1:0]  nondisp;
   reg                                           t2t;
   reg                                           t2l;
   reg [`SDMA_PKT_COPY_TILED_BC_ELEMENT_SIZE_SIZE-1:0]       psize;
   reg [`SDMA_PKT_COPY_TILED_BC_BANK_W_SIZE-1:0]  bwidth;
   reg [`SDMA_PKT_COPY_TILED_BC_BANK_H_SIZE-1:0] bheight;
   reg [`SDMA_PKT_COPY_TILED_BC_MAT_ASPT_SIZE-1:0]   aspect;
   reg [`SDMA_PKT_COPY_TILED_BC_NUM_BANK_SIZE-1:0]    numbank;
   reg [`SDMA_PKT_COPY_TILED_BC_PIPE_CONFIG_SIZE-1:0] pipecfg;
   reg [`SDMA_PKT_COPY_TILED_BC_ARRAY_MODE_SIZE-1:0]       tmode;
   reg [`SDMA_PKT_COPY_TILED_BC_HEIGHT_SIZE-1:0] height;
   reg [`SDMA_PKT_COPY_TILED_BC_PITCH_IN_TILE_SIZE-1:0] tpitch;
   reg [`SDMA_PKT_COPY_TILED_BC_SLICE_PITCH_SIZE-1:0] tslice;
   reg [`SDMA_PKT_COPY_TILED_SUBWIN_LINEAR_PITCH_SIZE+4:0] lpitch;
   reg [`SDMA_PKT_COPY_TILED_SUBWIN_LINEAR_SLICE_PITCH_SIZE+4:0] lslice;
   reg [`SDMA_PKT_COPY_TILED_BC_TILESPLIT_SIZE_SIZE-1:0]     tsplit;
   reg [`SDMA_PKT_COPY_TILED_BC_X_SIZE-1:0]             tx;
   reg [`SDMA_PKT_COPY_TILED_BC_Y_SIZE-1:0]             ty;
   reg [`SDMA_PKT_COPY_TILED_BC_Z_SIZE-1:0]             tz;
   reg [`SDMA_PKT_COPY_TILED_BC_X_SIZE-1+1:0]             dx;
   reg [`SDMA_PKT_COPY_TILED_BC_Y_SIZE-1+1:0]             dy;
   reg [`SDMA_PKT_COPY_TILED_BC_Z_SIZE-1+1:0]             dz;
   reg ie;
   reg structt;
   reg [`SDMA_PKT_COPY_STRUCT_STRIDE_SIZE-1:0]      stride;

  wire rtr;
  reg split_vld;
  always @ (posedge clk)
   if(!resetb) begin
     split_vld <= 0;
     {video, broadcast, hash, swap, tci, vol, pol, vmid, priv, structt, stride, voidd, crc, trns, aes_cbc, aes, subwin, linear, 
	granu, tilenum, ssize, taddr, taddr1, laddr, nondisp, t2t, t2l, psize, bwidth, bheight, aspect, numbank, pipecfg, tmode, 
	height, tpitch, tslice, lpitch, lslice, tsplit, tx, ty, tz, dx, dy, dz, ie} <= 0;
   end
   else begin
     if(split_rts)
        {video, broadcast, hash, swap, tci, vol, pol, vmid, priv, structt, stride, voidd, crc, trns, aes_cbc, aes, subwin, linear, 
	granu, tilenum, ssize, taddr, taddr1, laddr, nondisp, t2t, t2l, psize, bwidth, bheight, aspect, numbank, pipecfg, tmode, 
	height, tpitch, tslice, lpitch, lslice, tsplit, tx, ty, tz, dx, dy, dz, ie} <= split_param;
     split_vld <= split_rts ? 1 : split_vld && rtr ? 0 : split_vld;
   end

   reg[3:0] rreq_state, rreq_state_d, rreq_state_reg;
   parameter[3:0] rreq_start = 0, rreq_linear = 1, rreq_tile_64 = 2, rreq_tile_128 = 3, rreq_ie_linear = 4, rreq_ie_tile = 5, 
	rreq_tile_32 = 6, rreq_struct = 7, rreq_tile_256 = 8;
   reg [`CMD_COUNT_SIZE-1+1:0] rreq_size, alloc_size, alloc_size_d, rreq_count;
   wire [`CMD_COUNT_SIZE-1+1:0] rreq_size_unalignfix;

   wire thick_8bpp_ie;
   reg [3:0] offset_inc;
   reg rreq_vld, rreq_last, tile_en, rreq_ie;
   // reuqest number
   wire [7:0]                                                      req_cnt; //most 32x8 request for one xthick tile 128bpp
   reg [7:0]                                                       req_cnt_q;
   wire [8:0]                                                      req_cnt_max;
   reg [`DMA_TILE_PARAM_TX_SIZE-4+1:0]                             tile_x_max;
   reg [`DMA_TILE_PARAM_TX_SIZE-4:0]                               tile_x;
   wire[4:0] tile_burst;

  reg burst_state;
  wire burst_rtr;
  assign rtr = burst_state==0 && burst_rtr;

  reg[`CMD_ADDR_SIZE-1:0] rreq_addr;
  wire ie_linear_unalign = structt ? |stride[2:0] : |lpitch[4:2];
  always @ (posedge clk)
     if(!resetb) begin
       rreq_state <= rreq_start;
       rreq_size  <= 0;
       rreq_count <= 0;
       rreq_addr  <= 0;
     end
     else case(rreq_state)
       rreq_start: begin
	 rreq_state <= rtr && split_vld ? 
	       structt ? ie ? t2l ? rreq_linear : rreq_ie_linear : t2l ? rreq_struct : rreq_linear :
	       t2t ? rreq_ie_tile : 
	       ie ? t2l ? rreq_ie_tile : rreq_ie_linear :
	       t2l ? granu == `DMA_MC_wr_granularity_min_32b ? rreq_tile_32:
		     granu == `DMA_MC_wr_granularity_min_64b ? rreq_tile_64: 
		     granu == `DMA_MC_wr_granularity_min_128b ? rreq_tile_128 : rreq_tile_256 :
		       rreq_linear : rreq_start;
	 rreq_count <= ie ? structt ? 16 : /*t2l ? (tilenum<<(psize+1)) :*/ 8 : ssize;
         rreq_size <= ie && !t2t ? structt ? t2l ? {ssize,4'b0} : {tilenum,2'b0} : t2l ? (thick_8bpp_ie ? 5'b10000 : {tile_burst,5'b0}) : (tilenum<<(psize+3)) : 
		t2t ? {tile_burst,5'b0} : 
		t2l ? granu == `DMA_MC_wr_granularity_min_32b ? 4:
                     granu == `DMA_MC_wr_granularity_min_64b ? 8:
                     granu == `DMA_MC_wr_granularity_min_128b ? 16 : 32 :
		ssize;
         rreq_addr <= t2l ? taddr : laddr;
         //rreq_addr <= structt && t2l ? ie ? {taddr[`CMD_ADDR_SIZE-1:5],5'b0}/*TB to mdf*/ : taddr : ie ? laddr : laddr;
       end
       rreq_linear: begin
         rreq_addr <= rreq_addr;
         rreq_count <= rreq_count;
         rreq_size <= rreq_size;
         rreq_state <= rtr ? rreq_start : rreq_linear;
       end
       rreq_struct: begin
         rreq_addr <= rtr ? rreq_addr + 64  : rreq_addr;
         rreq_count <= rtr ? rreq_count - 4 : rreq_count;
         rreq_size <= rreq_count < 4 && rtr ? rreq_count : 4;
         rreq_state <= rreq_count > 4 || !rtr ? rreq_struct : rreq_start;
       end
       rreq_tile_32: begin
         rreq_count <= rtr ? rreq_count - 4 : rreq_count;
         rreq_size <= rreq_count < 4 && rtr ? rreq_count : 4;
         rreq_state <= rreq_count > 4 || !rtr ? rreq_tile_32 : rreq_start;
       end
       rreq_tile_64: begin
         rreq_count <= rtr ? rreq_count - 8 : rreq_count;
         rreq_size <= rreq_count < 8 && rtr ? rreq_count : 8;
         rreq_state <= rreq_count > 8 || !rtr ? rreq_tile_64 : rreq_start;
       end
       rreq_tile_128: begin
         rreq_count <= rtr ? rreq_count - 16 : rreq_count;
         rreq_size <= rreq_count < 16 && rtr ? rreq_count : 16;
	 rreq_state <= rreq_count > 16 || !rtr ? rreq_tile_128 : rreq_start;
       end
       rreq_tile_256: begin
         rreq_count <= rtr ? rreq_count - 32 : rreq_count;
         rreq_size <= rreq_count < 32 && rtr ? rreq_count : 32;
         rreq_state <= rreq_count > 32 || !rtr ? rreq_tile_256 : rreq_start;
       end
       rreq_ie_linear: begin
         rreq_addr <= rtr ? structt ? rreq_addr + {stride,2'b0} : rreq_addr + lpitch : rreq_addr;
         rreq_count <= rtr ? rreq_count - 1 : rreq_count;
         rreq_state <= rreq_count == 1 && rtr ? rreq_start : rreq_ie_linear;
       end
       rreq_ie_tile: begin
         rreq_state <= req_cnt_q + tile_burst == req_cnt_max && tile_x == tile_x_max - 1 && rtr ? rreq_start : rreq_ie_tile;
       end
       default: rreq_state <= rreq_state;
     endcase


  always @*
     case(rreq_state)
       rreq_start: begin
         rreq_vld = 0;//split_vld ? (ie && (~t2l || structt) || !ie && (linear || ~t2l && ~t2t)) : 0;
         tile_en = 0;
         rreq_ie = ie;
         offset_inc = 0;
         rreq_last = 0;
       end
       rreq_linear: begin
         rreq_vld = 1;
         tile_en = 0;
         rreq_ie = ie;
         offset_inc = 0;
         rreq_last = 0;
       end
       rreq_struct: begin
         rreq_vld = 1;
         tile_en = 1;
         rreq_ie = 0;
         offset_inc = 1;
         rreq_last = rreq_count <= 4;
       end
       rreq_tile_32: begin
         rreq_vld = 1;
         tile_en = 1;
         rreq_ie = 0;
         offset_inc = 1;
         rreq_last = rreq_count <= 4;
       end
       rreq_tile_64: begin
         rreq_vld = 1;
         tile_en = 1;
         rreq_ie = 0;
         offset_inc = 2;
         rreq_last = rreq_count <= 8;
       end
       rreq_tile_128: begin
         rreq_vld = 1;
         tile_en = 1;
         rreq_ie = 0;
         offset_inc = 4;
         rreq_last = rreq_count <= 16;
       end
       rreq_tile_256: begin
         rreq_vld = 1;
         tile_en = 1;
         rreq_ie = 0;
         offset_inc = 8;
         rreq_last = rreq_count <= 32;
       end
       rreq_ie_linear: begin
         rreq_vld = 1;
         rreq_ie = 1;
         tile_en = 0;
         offset_inc = 0;
         rreq_last = 0;
       end
       rreq_ie_tile: begin
         rreq_vld = 1;
         rreq_ie = ie;
         tile_en = 1;
         offset_inc = 0;
         rreq_last = 0;
       end
          //VCS coverage off
       default: begin
         rreq_vld = 0;
         rreq_ie = ie;
         tile_en = 0;
         offset_inc = 0;
         rreq_last = 0;
       end
          //VCS coverage on
     endcase


  assign rreq_size_unalignfix = rreq_state == rreq_ie_linear && ie_linear_unalign && ~|rreq_addr[4:0] ? rreq_size + 32 : rreq_size;

  assign rd_ex_data_ack = 1;

   reg [`DMA_TILE_PARAM_TX_SIZE-1:0]                               x;
   reg [`DMA_TILE_PARAM_TY_SIZE-1:0]                               y;
   reg [`DMA_TILE_PARAM_TZ_SIZE-1:0]                               z;

   wire [`DMA_TILE_PARAM_TX_SIZE-1:0]                              inc_x;
   wire                                                            inc_y;
   wire                                                            inc_z;
   wire [`DMA_TILE_PARAM_TX_SIZE-1:0]                              next_x;
   wire [`DMA_TILE_PARAM_TY_SIZE-1:0]                              next_y;
   wire [`DMA_TILE_PARAM_TZ_SIZE-1:0]                              next_z;

   wire [`DMA_TILE_PARAM_TPITCH_SIZE+3:0]                           tile_pitch_max;
   wire [`DMA_TILE_PARAM_HEIGHT_SIZE-1:0]                           tile_height_max;
   assign           tile_pitch_max = {tpitch + 1, 3'd0} - 1;
   assign         tile_height_max = height;


   assign inc_x = {offset_inc, 2'd0} >> psize;
   wire [`DMA_TILE_PARAM_TX_SIZE:0]                                x_plus_inc_x; //extend 1 bit
   assign x_plus_inc_x = x + inc_x;
   assign inc_y = (!subwin) & (x_plus_inc_x > tile_pitch_max);
   wire [`DMA_TILE_PARAM_TY_SIZE:0]                                y_plus_inc_y; //extend 1 bit
   assign y_plus_inc_y = y + inc_y;
   assign inc_z = (!subwin) & (y_plus_inc_y > tile_height_max);

   assign next_x = |offset_inc ? (inc_y ? `DMA_TILE_PARAM_TX_SIZE'd0 : (x + inc_x)) : x;
   assign next_y = |offset_inc ? (inc_z ? `DMA_TILE_PARAM_TY_SIZE'd0 : (y + inc_y)) : y;
   assign next_z = |offset_inc ? (z + inc_z) : z;


   reg [2:0]                                                       next_x_in_tile;
   reg [2:0]                                                       next_y_in_tile;
   reg [2:0]                                                       next_z_in_tile;
   reg [2:0]                                                       x_in_tile;
   reg [2:0]                                                       y_in_tile;
   reg [2:0]                                                       z_in_tile;
   reg [`DMA_TILE_PARAM_TY_SIZE-4:0]                               tile_y;
   reg [`DMA_TILE_PARAM_TZ_SIZE-1:0]                               tile_z;

   wire xthick_tile = (tmode == 8 || tmode == 14);
   assign thick_8bpp_ie = ie && !structt && (nondisp == 4) && (psize == 0);

   assign req_cnt_max = 
`ifdef THICK_TILE_EN
                       nondisp == 4 && t2t ? xthick_tile ? (6'd2 << (psize+3)) : (6'd2 << (psize+2)) : //t2t handle 8x8x8/8x8x4 Thick tile
		       thick_8bpp_ie ? 4'd4 :  //thick tile 8bpp t2l IE copy only read 128b data each req
`endif
                        (6'd2 << psize); //2 for 8bpp, 4 for 16bpp, 8 for 32bpp, 16 for 64bpp, 32 for 128bpp

   assign tile_burst = nondisp == 4 && !t2t ? 1 : //thick tile IE copy only handle one z surface
                       tsplit < 2 ? req_cnt_max > (2 << tsplit) ? (2 << tsplit) : req_cnt_max : 
                                    req_cnt_max > 8 ? 8 : req_cnt_max;

   assign req_cnt = (rreq_state==rreq_ie_tile && rtr && (req_cnt_q + tile_burst == req_cnt_max)) ? 5'd0 :
                    (rreq_state==rreq_ie_tile && rtr) ? (req_cnt_q + tile_burst) :
                    req_cnt_q;


   always @ (posedge clk)
     begin
        if(~resetb) begin
           req_cnt_q <= 5'd0;
           rreq_state_d <= 0;
           rreq_state_reg <= 0;
        end
        else begin
           req_cnt_q <= req_cnt;
           rreq_state_d <= rtr ? rreq_state : rreq_state_d;
           rreq_state_reg <= rreq_state;
        end
     end

   //tile num in chunk
   wire [`DMA_TILE_PARAM_TX_SIZE-4:0] tile_idx = tile_x - tx[`DMA_TILE_PARAM_TX_SIZE-1:3];

   wire[3:0] reorder_offset = structt ? t2l ? 0 :  16 - rreq_count : 	//s2l update offset after pipe
		   t2l || (t2t && ie) ? psize == 0 ? nondisp == 4 ? {tile_idx[1:0], req_cnt_q[1:0]} : {tile_idx[1:0], req_cnt_q[0]} :	//add rotate tile
                                    psize == 1 ? {tile_idx[0], req_cnt_q[1:0]} : req_cnt_q[2:0] : 
                                    8 - rreq_count;   //mdf  


   // coord in tile normal T2T
   always @ * begin
      next_z_in_tile = 3'b000;
      case({nondisp[2:0], psize})
        //disp
        6'b000_000: begin next_x_in_tile = 3'b000;                            next_y_in_tile = {req_cnt[0],2'b00};                end
        6'b000_001: begin next_x_in_tile = 3'b000;                            next_y_in_tile = {req_cnt[1],req_cnt[0],1'b0};      end
        6'b000_010: begin next_x_in_tile = {req_cnt[0],2'b00};                next_y_in_tile = {req_cnt[2],req_cnt[1],1'b0};      end
        6'b000_011: begin next_x_in_tile = {req_cnt[1],req_cnt[0],1'b0};      next_y_in_tile = {req_cnt[3],req_cnt[2],1'b0};      end
        6'b000_100: begin next_x_in_tile = {req_cnt[2],req_cnt[1],req_cnt[0]};next_y_in_tile = {req_cnt[4],req_cnt[3],1'b0};      end
        //non-disp
        6'b001_000,6'b010_000: begin next_x_in_tile = 3'b000;                  next_y_in_tile = {req_cnt[0],2'b00};                end
        6'b001_001,6'b010_001: begin next_x_in_tile = {req_cnt[0],2'b00};      next_y_in_tile = {req_cnt[1],2'b00};                end
        6'b001_010,6'b010_010: begin next_x_in_tile = {req_cnt[1],2'b00};      next_y_in_tile = {req_cnt[2],req_cnt[0],1'b0};      end
        6'b001_011,6'b010_011: begin next_x_in_tile = {req_cnt[2],req_cnt[0],1'b0};  next_y_in_tile = {req_cnt[3],req_cnt[1],1'b0};      end
        6'b001_100,6'b010_100: begin next_x_in_tile = {req_cnt[3],req_cnt[1],1'b0};  next_y_in_tile = {req_cnt[4],req_cnt[2],req_cnt[0]};end
`ifdef THICK_TILE_EN	 //thick tile IE only handlw one z surface
        6'b100_000: if (t2t) {next_z_in_tile[2], next_y_in_tile[2], next_x_in_tile[2], next_z_in_tile[1], next_z_in_tile[0], next_y_in_tile[1], next_x_in_tile[1], next_y_in_tile[0], next_x_in_tile[0]} = {req_cnt[3:0], 5'b0};
        	    else     {next_y_in_tile[2], next_x_in_tile[2], next_y_in_tile[1], next_x_in_tile[1], next_y_in_tile[0], next_x_in_tile[0]} = {req_cnt[1:0], 4'b0};
        6'b100_001: if (t2t) {next_z_in_tile[2], next_y_in_tile[2], next_x_in_tile[2], next_z_in_tile[1], next_z_in_tile[0], next_y_in_tile[1], next_x_in_tile[1], next_y_in_tile[0], next_x_in_tile[0]} = {req_cnt[4:0], 4'b0};
        	    else     {next_y_in_tile[2], next_x_in_tile[2], next_y_in_tile[1], next_x_in_tile[1], next_y_in_tile[0], next_x_in_tile[0]} = {req_cnt[1:0], 4'b0};
        6'b100_010: if (t2t) {next_z_in_tile[2], next_y_in_tile[2], next_x_in_tile[2], next_z_in_tile[1], next_y_in_tile[1], next_z_in_tile[0], next_x_in_tile[1], next_y_in_tile[0], next_x_in_tile[0]} = {req_cnt[5:0], 3'b0};
        	    else     {next_y_in_tile[2], next_x_in_tile[2], next_y_in_tile[1], next_x_in_tile[1], next_y_in_tile[0], next_x_in_tile[0]} = {req_cnt[2:0], 3'b0};
        6'b100_011: if (t2t) {next_z_in_tile[2], next_y_in_tile[2], next_x_in_tile[2], next_z_in_tile[1], next_y_in_tile[1], next_x_in_tile[1], next_z_in_tile[0], next_y_in_tile[0], next_x_in_tile[0]} = {req_cnt[6:0], 2'b0};
                    else     {next_y_in_tile[2], next_x_in_tile[2], next_y_in_tile[1], next_x_in_tile[1], next_y_in_tile[0], next_x_in_tile[0]} = {req_cnt[3:0], 2'b0};
        6'b100_100: if (t2t) {next_z_in_tile[2], next_y_in_tile[2], next_x_in_tile[2], next_z_in_tile[1], next_y_in_tile[1], next_x_in_tile[1], next_z_in_tile[0], next_y_in_tile[0], next_x_in_tile[0]} = {req_cnt[7:0], 1'b0};
        	    else     {next_y_in_tile[2], next_x_in_tile[2], next_y_in_tile[1], next_x_in_tile[1], next_y_in_tile[0], next_x_in_tile[0]} = {req_cnt[4:0], 1'b0};
`endif
        //rotate tile
        5'b11_000:  {next_x_in_tile[2], next_x_in_tile[0], next_x_in_tile[1],
                          next_y_in_tile[2], next_y_in_tile[1], next_y_in_tile[0]} = {req_cnt, 5'b0};
        5'b11_001:  {next_x_in_tile[2], next_x_in_tile[1], next_x_in_tile[0],
                          next_y_in_tile[2], next_y_in_tile[1], next_y_in_tile[0]} = {req_cnt, 4'b0};
        5'b11_010:  {next_x_in_tile[2], next_x_in_tile[1], next_y_in_tile[2],
                          next_x_in_tile[0], next_y_in_tile[1], next_y_in_tile[0]} = {req_cnt, 3'b0};
        5'b11_011:  {next_y_in_tile[2], next_x_in_tile[2], next_x_in_tile[1],
                          next_y_in_tile[1], next_x_in_tile[0], next_y_in_tile[0]} = {req_cnt, 2'b0};
        default:  begin next_x_in_tile = 3'b000;  next_y_in_tile = 3'b000;  next_z_in_tile = 3'b000; end
      endcase
   end
   wire   inc_tile_x;
   assign   inc_tile_x  = (req_cnt_q + tile_burst == req_cnt_max) ? 1 : 0;


   always @ (posedge clk)
     begin
        if(~resetb) begin
	 x <= 0; y <= 0; z <= 0;
           x_in_tile <= 3'b0;
           y_in_tile <= 3'b0;
           z_in_tile <= 3'b0;
           tile_x_max<= 0;
           tile_x <= {(`DMA_TILE_PARAM_TX_SIZE-3){1'b0}};
           tile_y <= {(`DMA_TILE_PARAM_TY_SIZE-3){1'b0}};
           tile_z <= {`DMA_TILE_PARAM_TZ_SIZE{1'b0}};
        end
        else if (split_vld) begin
         x <= tx;
         y <= ty;
         z <= tz;
           x_in_tile <= 0;
           y_in_tile <= 0;
           z_in_tile <= 0;
           tile_x_max    <= t2t ? tx[`DMA_TILE_PARAM_TX_SIZE-1:3] + dx[`DMA_TILE_PARAM_TX_SIZE-1+1:3] :
				tx[`DMA_TILE_PARAM_TX_SIZE-1:3] + tilenum; 
           tile_x    <= tx[`SDMA_PKT_COPY_TILED_BC_X_SIZE-1:3];
           tile_y    <= ty[`SDMA_PKT_COPY_TILED_BC_Y_SIZE-1:3];
           tile_z    <= tz[`SDMA_PKT_COPY_TILED_BC_Z_SIZE-1:3];
        end
        else if (rtr) begin
         x <= next_x;
         y <= next_y;
         z <= next_z;
           x_in_tile <= next_x_in_tile;
           y_in_tile <= next_y_in_tile;
           z_in_tile <= next_z_in_tile;
	   //if(rreq_vld && ie && t2l)
	   if(rreq_state==rreq_ie_tile)
              tile_x    <= tile_x + inc_tile_x;
           tile_y    <= tile_y;
           tile_z    <= tile_z;
        end
     end


   wire[`DMA_TILE_TADDR_SIZE+8-1:0] tile_taddr_out;
   wire [`DMA_TILE_PARAM_TX_SIZE-1:0] addr_calc_x = rreq_state==rreq_ie_tile ? {tile_x, x_in_tile} : x;
   wire [`DMA_TILE_PARAM_TY_SIZE-1:0] addr_calc_y = rreq_state==rreq_ie_tile ? {tile_y, y_in_tile} : y;
   wire [`DMA_TILE_PARAM_TZ_SIZE-1:0] addr_calc_z = rreq_state==rreq_ie_tile && nondisp == 4 && t2t ? 
						    (xthick_tile ? {tile_z, z_in_tile[2:0]} : {tile_z, z[2], z_in_tile[1:0]}) : z;

   wire [`DMA_TILE_TADDR_SIZE-1:0] baseAddr256B=taddr[`CMD_ADDR_SIZE-1:8];
   wire                           h0_reg_en = rtr;// && tile_vld;
   wire                           h1_reg_en = rtr;// && tile_vld_d1;
   wire                           h2_reg_en = rtr;// && tile_vld_d2;
   wire                           h3_reg_en = rtr;// && tile_vld_d3;
   wire tile_vld = rtr & tile_en & !structt;

   generic_addr_calc2_sdma_gc #(.RI(1), .RO(1), .PAD_STAGES(0), .COLLAPSABLE_PIPE_STAGE(0))
        dma_rd_addr_calc(
                                                      //outputs
              .address(tile_taddr_out), //byte address: 40bit
              .busy (),
              //inputs
              .x(addr_calc_x),                         //coordinate x
              .y(addr_calc_y),                         //coordinate y
              .z(addr_calc_z[10:0]),                         //coordinate z
              .s(3'd0),                      //sample number
              .baseAddr256B(baseAddr256B),      //base addr 256B align
              .log2_pixelsize(psize),            //0: 8bpp, 1:16bpp, 2:32bpp, 3:64bpp, 4:128bpp
              .log2_samples(2'd0),                   //the number of samples
              .pitch_tile_max(tpitch),            //in tiles, pitch_element/8 -1
              .slice_tile_max(tslice),            //in tiles, slice_tiles -1
              .micro_tile_mode(nondisp),     //3:thick tile, 2:depth, 1:non-disp, 0:displayable
              .pipe_config(pipecfg),
              .array_mode(tmode),                //as defines
              .tile_split(tsplit),                //0: 64B split, 1: 128B split, 2: 256B ...
              .bank_width(bwidth),                //0: 1tile, 1:2tiles, 2:4tiles, 3:8tiles
              .bank_height(bheight),               //0: 1tile, 1:2tiles, 2:4tiles, 3:8tiles
              .macro_tile_aspect(aspect),         //0: 1tile, 1:2tiles, 2:4tiles, 3:8tiles
              .num_banks(numbank),               //0: 2banks,1:4banks, 2:8banks, 3:16banks
              .pipe_interleave_size(dma_addr_conf_pipe_interleave_size),      //0: 256B, 1:512B, 2:1024B, 3:2048B, 4:4096B
              .sclk(clk),
	          .srst (~resetb),                      
	          .iRTS (tile_vld),                      
	          .oRTR (),                      
	          .oRTS (),                      
	          .iRTR (rtr),                      
              .input_en (tile_vld),
              .output_en(),
	          .pass_thru_data_in(),
	          .pass_thru_data_out()       
              );

   //pipeline down register

   reg[255:0] pipe0, pipe1, pipe2, pipe3;   
   reg pipe0_vld, pipe1_vld, pipe2_vld, pipe3_vld, pipe_vld;
   reg[1:0] pipe_granu;
   wire[3:0] struct_y = {structt, z[0], y[1:0]};

   always@(posedge clk)
     if(!resetb) begin 
       pipe0 <= 0; pipe1 <= 0; pipe2 <= 0; pipe3 <= 0;
       {pipe0_vld, pipe1_vld, pipe2_vld, pipe3_vld} <= 0;
     end
     else if(rtr) begin
       pipe0 <= {thick_8bpp_ie, reorder_offset, structt, rreq_ie, tile_en, rreq_last, struct_y, granu, psize, t2l, t2t, nondisp, 
		swap, tci, vol, pol, vmid, priv, hash, rreq_size_unalignfix, rreq_addr};
       pipe1 <= pipe0;
       pipe2 <= pipe1;
       pipe3 <= pipe2;
       {pipe0_vld, pipe1_vld, pipe2_vld, pipe3_vld} <= {rreq_vld, pipe0_vld, pipe1_vld, pipe2_vld};
     end


   reg pipe_priv, pipe_hash, pipe_t2l, pipe_t2t;
   reg pipe_tci, pipe_vol, pipe_pol;
   reg pipe_tile, pipe_last, pipe_ie, pipe_struct;
   reg [`SDMA0_GFX_RB_CNTL_RB_VMID_SIZE-1:0] pipe_vmid;
   reg[1:0] pipe_swap;
   reg[`SDMA_PKT_COPY_TILED_BC_MIT_MODE_SIZE-1:0] pipe_nondisp;
   reg[`CMD_ADDR_SIZE-1:0] burst_addr, next_burst_addr, burst_end;
   reg[`CMD_ADDR_SIZE-1:0] pipe_addr;
   reg[8:0] burst_mask;
   reg[`CMD_COUNT_SIZE-1+1:0] pipe_size;
   reg[3:0] pipe_struct_y;
   reg[2:0] pipe_psize;
   reg[3:0] pipe_reorder_offset;
   reg pipe_thick_8bpp_ie;

   always @* begin
     {pipe_thick_8bpp_ie, pipe_reorder_offset, pipe_struct, pipe_ie, pipe_tile, pipe_last, pipe_struct_y, pipe_granu, pipe_psize, pipe_t2l, pipe_t2t, 
	pipe_nondisp, pipe_swap, pipe_tci, pipe_vol, pipe_pol, pipe_vmid, pipe_priv, pipe_hash, pipe_size, pipe_addr} = pipe3;
     if(pipe_tile && !pipe_struct) 
	pipe_addr[`DMA_TILE_TADDR_SIZE+8-1:0] = tile_taddr_out;
     pipe_vld = pipe3_vld && rtr;
   end

   //num of 256b in 256byte burst
   reg[3:0] burst_id_ini;

   always @ (posedge clk)
     begin
        if(~resetb)
           burst_id_ini <= 0;
        else
           burst_id_ini <= pipe_ie && pipe_struct && pipe_t2l ? pipe_vld ? alloc_size[9:5] : next_ce_rd_rts ? burst_id_ini + alloc_size[9:5] : burst_id_ini : burst_id_ini;
     end

   /**********************************************************/

  reg next_burst_state;
  parameter burst_start = 0, burst_body = 1;
  reg [`CMD_COUNT_SIZE-1+1:0] burst_size, next_burst_size;
  reg[3:0] next_burst_reorder_offset, burst_reorder_offset;

  reg [6:0] chunk_alloc_timing;
   always @ (posedge clk)
     begin
	chunk_alloc_timing <= out_chunk_size + 40;
     end

  reg[`DMA_DATA_LUT_DEPTH-1:0] lut_used;
`ifdef SDMA__UTC_L1__1
  wire next_ce_rd_stall = !(data_rreq_alloc_free >= (out_ie ? chunk_alloc_timing : 44) && (lut_used<(`DMA_DATA_LUT_ENTRIES >> lfifo_watermark)-6) );
`else
  wire next_ce_rd_stall = !(data_rreq_alloc_free >= (out_ie ? chunk_alloc_timing : 8) && (lut_used<(`DMA_DATA_LUT_ENTRIES >> lfifo_watermark)-16) );
`endif
 // wire next_ce_rd_stall = !(data_rreq_alloc_free >= (out_ie ? chunk_alloc_timing : 8) &&
 //       (out_ie ? lut_used<`DMA_DATA_LUT_ENTRIES-out_chunk_size : lut_used<`DMA_DATA_LUT_ENTRIES-8) );

  reg ce_rd_fifo_stall;
   always@(posedge clk)
     if(!resetb) begin
	ce_rd_fifo_stall <= 1'b0;
     end
     else begin
	ce_rd_fifo_stall <= next_ce_rd_stall;
     end

`ifdef SDMA__UTC_L1__1
  assign burst_rtr = mc_rd_rtr && (dma_utc_l1_enable || !ce_rd_fifo_stall);
`else
  assign burst_rtr = next_mc_rd_rtr && !ce_rd_fifo_stall;
`endif
  //assign burst_rtr = next_mc_rd_rtr && !next_ce_rd_stall;

   wire tile_burst_off = (pipe_tile && !pipe_ie && !pipe_t2t) || (pipe_thick_8bpp_ie && pipe_t2l);

   wire skip_before = 0;
   wire skip_after = 0;
   wire skip_before_body = 0;
   wire skip_after_body = 0;

//MDF linear burst gen, ie t2l fixing
   always@(posedge clk)
     if(!resetb) begin
       burst_state <= burst_start;
       burst_size <= 0;
       burst_addr <= 0;
       burst_reorder_offset <= 0;
       alloc_size_d <= 0;
     end
     else if(burst_rtr) begin
       burst_state <= next_burst_state;
       burst_size <= next_burst_size;
       burst_addr <= next_burst_addr;
       burst_reorder_offset <= next_burst_reorder_offset;
       alloc_size_d <= alloc_size;
     end

   always@*
     case(burst_state)
       burst_start: begin
         burst_end = pipe_size + pipe_addr - 1;
         burst_mask = 0;
         alloc_size = pipe_addr[4:0] + pipe_size;
         alloc_size = pipe_vld ? tile_burst_off ? pipe_size : 
				    skip_before ? 32 :
                                   rd_burst == 2'd2 ? pipe_addr[5] ? 32 : pipe_addr[6:0] + pipe_size > 128 ? 128 - {pipe_addr[6:5],5'b0} : 
						(pipe_addr[6:5] == 0) & (pipe_addr[6:0] + pipe_size) > 64  & (pipe_addr[6:0] + pipe_size) <= 96 ? 64 : {alloc_size[21:5] + |alloc_size[4:0],5'b0} :
                                   rd_burst == 2'd1 ? pipe_addr[5:0] + pipe_size >  64 ?  64 - {pipe_addr[5]  ,5'b0} : {alloc_size[21:5] + |alloc_size[4:0],5'b0} :
                                                      32 : 0;
         
         next_burst_addr = skip_before ? {pipe_addr[`CMD_ADDR_SIZE-1:5] + 1'b1, 5'b0} : {pipe_addr[`CMD_ADDR_SIZE-1:5], 5'b0};
         next_burst_size = pipe_vld ? (pipe_size + pipe_addr[4:0] >  (tile_burst_off ? 32 : alloc_size)) ? pipe_size + pipe_addr[4:0] - (tile_burst_off ? 32 : alloc_size) : 0 : 0;
         next_burst_state = pipe_vld ? (pipe_size + pipe_addr[4:0] >  (tile_burst_off ? 32 : alloc_size)) || skip_after ? burst_body : burst_state : burst_state;
         next_burst_reorder_offset = pipe_reorder_offset;
       end
       burst_body: begin
         burst_end = burst_size + next_burst_addr - 1;
         burst_mask = 0;
         alloc_size = burst_size;
         alloc_size = burst_rtr ? (skip_after_body && burst_size == 0) ? 32 :
                                  rd_burst == 2'd2 ? next_burst_addr[5] ? 32 : next_burst_addr[6:0] + burst_size > 128 ? 128 - {next_burst_addr[6:5],5'b0} :
                                                     next_burst_addr[6:5] == 0 && (burst_size > 64 & burst_size <= 96) ? 64 : {alloc_size[21:5] + |alloc_size[4:0],5'b0} :
                                  rd_burst == 2'd1 ? next_burst_addr[5:0] + burst_size >  64 ?  64 - {next_burst_addr[5]  ,5'b0} : {alloc_size[21:5] + |alloc_size[4:0],5'b0} :
                                                     32 : alloc_size;

         next_burst_addr = skip_before_body || (skip_after_body && burst_size == 0) ? burst_addr : burst_addr + alloc_size_d;
         next_burst_size = burst_size < alloc_size ? 0 : burst_size - alloc_size;
         next_burst_state = burst_size > alloc_size || (skip_after_body && burst_size != 0) ? burst_body : burst_start; 
         next_burst_reorder_offset = (pipe_t2l | pipe_t2t) ? (rd_burst == 2'd0) ? burst_reorder_offset + 4'd1 :
                                                             (rd_burst == 2'd1) ? burst_reorder_offset + 4'd2 :
                                                             (rd_burst == 2'd2) ? burst_reorder_offset + 4'd4 :
                                                                                  burst_reorder_offset 
                                                           : burst_reorder_offset;
       end
     endcase


   ////////////////////////////////////
   ////////LUT data ///////////////////
   ////////////////////////////////////
   wire [`DMA_MC_RDREQ_TAG_SIZE-1:0]  next_ce_rd_tag;
   reg [2:0] unalign_d;
   wire[2:0] unalign;
   reg [2:0] copy_nr_dword_d;
   reg [`DMA_RD_DATA_BUF_DEPTH:0] data_rreq_alloc_wptr_d;
   wire[2:0] copy_nr_dword = data_rreq_alloc_wptr[`DMA_RD_DATA_BUF_DEPTH-1:0]==data_rreq_alloc_wptr_d[`DMA_RD_DATA_BUF_DEPTH-1:0] ? 
                        alloc_size[4:2]+copy_nr_dword_d : alloc_size[4:2] ;
   wire lut_last = /*next_burst_size == 0 &&*/ pipe_last;
   wire[2:0] quarter = next_quarter_entry;
   wire[1:0] lut_granu = pipe_ie ? `DMA_MC_wr_granularity_min_256b : pipe_granu;
   wire [3:0] struct_y_final = pipe_struct ? {pipe_struct, pipe_addr[4:2]} : pipe_struct_y;
   wire [1:0] micro_tile_mode = pipe_nondisp == 4 ? 2'd3 : pipe_nondisp[1:0];

`ifdef SDMA__UTC_L1__1
wire [31:0] saved_ram_data_lut_din = l1_alloc_data[31:0];
wire [31:0] lut_info = pipe_ie ? {~burst_state, unalign, (pipe_t2l && pipe_struct ? burst_id_ini : next_burst_reorder_offset), struct_y_final[2:0], pipe_ie, pipe_t2l, (pipe_t2t ? 2'd2 : micro_tile_mode), pipe_psize, lut_last, lut_granu, data_rreq_alloc_wptr[`DMA_RD_DATA_BUF_DEPTH-1:0]}:
	{~burst_state, copy_nr_dword, 1'b0, quarter, struct_y_final[2:0], pipe_ie, pipe_t2l, micro_tile_mode, pipe_psize, lut_last, lut_granu, data_rreq_alloc_wptr[`DMA_RD_DATA_BUF_DEPTH-1:0]};
   assign ram_data_lut_din = dma_utc_l1_enable ? saved_ram_data_lut_din : lut_info;
`else
   assign ram_data_lut_din = pipe_ie ? {~burst_state, unalign, (pipe_t2l && pipe_struct ? burst_id_ini : next_burst_reorder_offset), 
	struct_y_final, pipe_ie, pipe_t2l, (pipe_t2t? 3'd5 : pipe_nondisp), pipe_psize, lut_last, lut_granu, data_rreq_alloc_wptr[`DMA_RD_DATA_BUF_DEPTH-1:0]}:
	{pipe_t2t, ~burst_state, copy_nr_dword, 1'b0, quarter,
        struct_y_final, pipe_ie, pipe_t2l, pipe_nondisp, pipe_psize, lut_last, lut_granu, data_rreq_alloc_wptr[`DMA_RD_DATA_BUF_DEPTH-1:0]};
`endif

   assign unalign = pipe_vld ? pipe_addr[4:2] : unalign_d;
   always @(posedge clk)
     begin
        if (~resetb)
           unalign_d <= 0;
        else if(pipe_vld)
           unalign_d <= pipe_addr[4:2];
     end

   always @(posedge clk)
     begin
        if (~resetb) begin
           copy_nr_dword_d <= 0;
	   data_rreq_alloc_wptr_d <= 0;
        end
        else if(next_ce_rd_rts) begin
           copy_nr_dword_d <= copy_nr_dword;
	   data_rreq_alloc_wptr_d <= data_rreq_alloc_wptr;
        end
     end

   ////////////////////////////////////
   /////////memory interface //////////
   ////////////////////////////////////

   wire[`DMA_DATA_LUT_DEPTH-1:0] next_data_lut_waddr = ram_data_lut_waddr == `DMA_DATA_LUT_ENTRIES-1 ? 0 : ram_data_lut_waddr + 1;
   always @(posedge clk)
     if (~resetb)
       ram_data_lut_waddr <= 0;
     else if(ram_data_lut_we)
       ram_data_lut_waddr <= next_data_lut_waddr;
`ifdef SDMA__UTC_L1__1
   assign ram_data_lut_we = dma_utc_l1_enable ? l1_alloc_vld & l1_alloc_data[36] : next_ce_rd_rts;
`else
   assign ram_data_lut_we = next_ce_rd_rts;
`endif
   //assign ram_data_lut_we = next_ce_rd_rts;

   wire[3:0] burst_mask_sum =burst_mask[0]+burst_mask[1]+burst_mask[2]+burst_mask[3]+burst_mask[4]+burst_mask[5]+burst_mask[6]+burst_mask[7];

`ifdef SDMA__UTC_L1__1
wire [3:0] lut_entry_inc = (~pipe_ie || pipe_thick_8bpp_ie) && pipe_t2l? pipe_last ? ((8-quarter)>>lut_granu) : 1 : burst_mask_sum == 0 ? 1 : burst_mask_sum;
wire [3:0] saved_lut_entry_inc = l1_alloc_data[35:32];
`endif

//LUT re-order
   reg [`DMA_DATA_LUT_ENTRIES-1:0] lut_entry_vld;
   reg [`DMA_DATA_LUT_DEPTH-1:0] lut_rptr; 
   wire [`DMA_DATA_LUT_DEPTH-1:0] next_lut_rptr;

   wire [`DMA_DATA_LUT_DEPTH-1:0] ram_data_lut_raddr = MC_DMA_rdret_tag[`MC_DMA_RDRET_DATA_TAG_ID_RANGE];
   wire lut_rptr_inc = lut_entry_vld[lut_rptr[`DMA_DATA_LUT_DEPTH-1:0]];
   assign next_lut_rptr = lut_rptr_inc ? lut_rptr[`DMA_DATA_LUT_DEPTH-1:0] == `DMA_DATA_LUT_ENTRIES-1'b1 ? 0 : lut_rptr + 1'b1 : lut_rptr;

   always @(posedge clk) begin
      if (~resetb) begin
         lut_rptr <= `DMA_DATA_LUT_DEPTH'd0;
	 lut_entry_vld <= `DMA_DATA_LUT_ENTRIES'd0;
      end
      else begin
         lut_rptr <= next_lut_rptr;

	 if (bc_mode && MC_DMA_rdret_vld && MC_DMA_rdret_tag[`MC_DMA_RDRET_RB_TAG_TYPE_RANGE]==`TAG_TYPE_DATA && MC_DMA_rdret_last)
	    lut_entry_vld[ram_data_lut_raddr[`DMA_DATA_LUT_DEPTH-1:0]] <= 1'b1;

	 if (lut_rptr_inc)
	    lut_entry_vld[lut_rptr[`DMA_DATA_LUT_DEPTH-1:0]] <= 1'b0;
      end
   end

   always @(posedge clk)
     if (~resetb) begin
       lut_used <= 0;
     end
     else
       lut_used <= lut_used + ram_data_lut_we - lut_rptr_inc;         
//       lut_used <= lut_used  
//	+ (ram_data_lut_we ? (~pipe_ie || pipe_thick_8bpp_ie) && pipe_t2l? pipe_last ? ((8-quarter)>>lut_granu) : 1 : burst_mask_sum : 0)
//	- (ram_rd_data_rptr_inc ? out_granu==3 ? 1 : (4'h8 >> out_granu) : 0);

`ifdef SDMA__UTC_L1__1
   reg [`MC_DMA_RDRET_DATA_TAG_ID_RANGE]  ce_rreq_tag;
   wire [`MC_DMA_RDRET_DATA_TAG_ID_RANGE] next_ce_rreq_tag = ce_rreq_tag == `DMA_DATA_LUT_ENTRIES-1 ? 0 : ce_rreq_tag + 1;

   always @(posedge clk)
     if (~resetb)
       ce_rreq_tag <= 0;
     else if(next_ce_rd_rts)
       ce_rreq_tag <= next_ce_rreq_tag;
`endif


   //tag generation
`ifdef SDMA__UTC_L1__1
   assign next_ce_rd_tag[`MC_DMA_RDRET_DATA_TAG_ID_RANGE] =  dma_utc_l1_enable ? ce_rreq_tag : ram_data_lut_waddr;//data_rreq_alloc_wptr;
`else
   assign next_ce_rd_tag[`MC_DMA_RDRET_DATA_TAG_ID_RANGE] =  ram_data_lut_waddr;//data_rreq_alloc_wptr;
`endif
   //assign next_ce_rd_tag[`MC_DMA_RDRET_DATA_TAG_ID_RANGE] =  ram_data_lut_waddr;//data_rreq_alloc_wptr;
   //assign next_ce_rd_tag[`MC_DMA_RDRET_DATA_TAG_NULL_RANGE] = 0;
   assign next_ce_rd_tag[`MC_DMA_RDRET_DATA_TAG_TYPE_RANGE] = `TAG_TYPE_DATA;
   assign next_ce_rd_tag[`MC_DMA_RDRET_DATA_REQ_TYPE_RANGE] = `REQ_TYPE_READ;

   wire [`DMA_RD_DATA_BUF_DEPTH:0]    next_data_rreq_alloc_free = data_rreq_alloc_free ; //- next_data_mc_rd_size;   simplified for timing
   wire [`DMA_RD_DATA_BUF_DEPTH:0]    next_data_rreq_alloc_used = data_rreq_alloc_used + 8; //+ next_data_mc_rd_size;
   wire [`DMA_MC_RDNFO_URG_SIZE-1:0]  next_ce_rd_urg;
   assign next_ce_rd_urg  = next_data_rreq_alloc_free ?
                                   next_data_rreq_alloc_used[`DMA_RD_DATA_BUF_DEPTH:`DMA_RD_DATA_BUF_DEPTH-3] : 4'hf;

   ////////////////////////////////////////////////////////////////////
   reg                                ce_rd_rts;
   reg                                ce_rd_hash;
   reg [`DMA_MC_RDREQ_ADDR_SIZE-1:0]  ce_rd_addr_tmp;

`ifdef SDMA__UTC_L1__1
   reg [31:0] lut_alloc_info;
   wire [31:0] next_lut_alloc_info = lut_info[31:0];
   assign ce_rd_lut_info = {ce_rd_rts, data_rreq_alloc_wptr_inc[3:0], lut_alloc_info};
   assign ce_rd_stall_atc = ce_rd_fifo_stall;
`endif

   always @(posedge clk)
     begin
        if (~resetb) begin
           ce_rd_rts  <= 1'b0;
           ce_rd_mask <= `DMA_MC_RDREQ_MASK_SIZE'd0;
           ce_rd_tag  <= `DMA_MC_RDREQ_TAG_SIZE'd0;
           ce_rd_addr  <= 0;
           ce_rd_swap  <= 0;   
           ce_rd_priv  <= 0;
           ce_rd_vmid  <= 0;
           ce_rd_hash  <= 0;
           ce_rd_tci   <= 0;
           ce_rd_vol   <= 0;
           ce_rd_pol   <= 0;
           ce_rd_stall <= 0;
           ce_rd_urg   <= 0;
           ce_rd_size  <= 0;
           ce_rd_dw_size <= 0;
           ce_rd_t2l_last <= 0;
`ifdef SDMA__UTC_L1__1
	   lut_alloc_info <= 0;
`endif
        end
        else begin
           ce_rd_rts   <= next_ce_rd_rts;
	if (next_ce_rd_rts) 
           ce_rd_addr  <= {ce_rd_addr_tmp, next_burst_addr[7:5]};
           ce_rd_mask  <= burst_mask;
           ce_rd_swap  <= burst_state == burst_body ? ce_rd_swap : pipe_swap;
           ce_rd_priv  <= burst_state == burst_body ? ce_rd_priv : pipe_priv;
           ce_rd_vmid  <= burst_state == burst_body ? ce_rd_vmid : pipe_vmid;
           ce_rd_hash  <= burst_state == burst_body ? ce_rd_hash : pipe_hash;
           ce_rd_tci   <= burst_state == burst_body ? ce_rd_tci : pipe_tci;
           ce_rd_vol   <= burst_state == burst_body ? ce_rd_vol : pipe_vol;
           ce_rd_pol   <= burst_state == burst_body ? ce_rd_pol : pipe_pol;
           ce_rd_tag   <= next_ce_rd_rts ? next_ce_rd_tag : ce_rd_tag;
           ce_rd_stall <= next_ce_rd_stall;
           ce_rd_urg   <= next_ce_rd_urg;
           ce_rd_size   <= alloc_size[7:6];
           ce_rd_dw_size   <= alloc_size[`DMA_MC_RDREQ_DWORD_SIZE-1+2:2];	//dw num
           ce_rd_t2l_last <= pipe_last;
`ifdef SDMA__UTC_L1__1
	if (bc_mode)
	   lut_alloc_info <= next_lut_alloc_info;
`endif
        end // else: !if(~resetb)
     end // always @ (posedge clk)

integer i,j;
   always@* begin
      ce_rd_addr_tmp = next_burst_addr[`CMD_ADDR_SIZE-1:8];
	   if(ce_rd_hash) 
              for(i=0; i<8; i=i+1) begin
                if(i<dma_channel_bits && |dma_channel_bits && |dma_channel_xor && dma_channel_bits<5)
                  for(j=1; j<=7; j=j+1) begin
                    if(j<=dma_channel_xor)
                      ce_rd_addr_tmp[i] = ce_rd_addr_tmp[i] ^ ce_rd_addr_tmp[i+j*dma_channel_bits]; 
	          end
                else if(i>=dma_channel_bits && i<dma_channel_bits+dma_bank_bits && 
			|dma_bank_bits && |dma_bank_xor && dma_bank_bits<5 && dma_channel_bits<5)
                  for(j=1; j<=7; j=j+1) begin
                    if(j<=dma_bank_xor)
		      ce_rd_addr_tmp[i] = ce_rd_addr_tmp[i] ^ ce_rd_addr_tmp[i+j*dma_bank_bits];
                  end
	      end
      end


assign next_ce_rd_rts = pipe_vld || burst_size != 0 && burst_rtr;
assign ce_rd_vld = ce_rd_rts; //& mc_rd_rtr; 

assign rreq_rtr = rreq_state == 0 && rtr && !split_vld;
//assign rreq_rtr = rreq_state == 0 && (split_ie ? split_vld == 0 && rreq_vld == 0 : rtr && !(split_vld && ie));


   //reg [`DMA_RD_DATA_BUF_DEPTH:0] rd_data_dbg_alloc_free;
   always @(posedge clk)
     rd_data_dbg_alloc_free <= data_rreq_alloc_free;

   always@(posedge clk)
     if(!resetb)
       dma_ce_rreq_idle <= 1;
     else
       dma_ce_rreq_idle <= rreq_state == 0 && !split_vld && burst_state == 0 && !ce_rd_rts && 
		{pipe0_vld, pipe1_vld, pipe2_vld, pipe3_vld}==0 && lut_used==0;



endmodule

