// Filename              : dma_copy_split.v
// Perforce changelist   : $Change$
//
// Copyright             : Trade secret of AMD, Inc.
//                         Copyright 2008, AMD, Inc., (unpublished)
//                         All rights reserved.  This notice is intended as a precaution against
//                         inadvertent publication and does not imply publication or any waiver
//                         of confidentiality.  The year included in the foregoing notice is the
//                         year of creation of the work.
//
// Name:                 : sdma copy split 
//
// Description           : Split the following copy cmds into sub-copies: partial, struct.
//			   Add small change to copy size for tiled copy not aligned to granularity

`include "dma_common.vh"
`include "sdma_reg.v"


module dma_ce_split_bc(

    input clk,
    input resetb,

    input ex_start,
    input bc_mode,
    input [`CMD_PARAM_RANGE] ex_param,
    output reg split_start,

    input copy_ie_en,

    input cmd_info_full,
    input cmd_info1_full,
    output ram_cmd_info_we,
    output [`DMA_RD_INFO_BUF_WIDTH-1:0] ram_cmd_info_din,
    output [`DMA_RD_INFO1_BUF_WIDTH-1:0] ram_cmd_info1_din,

    output split_rts,
    output reg last_sub_cp,
    output [`CMD_PARAM_RANGE] split_param,
    output [`CMD_PARAM_RANGE] split_param_dst,
    output [`CMD_COUNT_SIZE-1+1:0] ssize,
    output aes,
    output aes_cbc,
    output trns,

    input rreq_rtr,
    input wreq_rtr,
    output reg dma_ce_split_idle
		      );

   wire video, broadcast;
   wire [1:0] dswap, sswap, swap1;
   wire dhash, shash, hash1, crc, priv;
   wire dtci, stci, dvol, svol, dpol, spol, tci1, vol1, pol1;
   wire [`SDMA0_GFX_RB_CNTL_RB_VMID_SIZE-1:0] vmid;
   wire structt;
   wire linear;
   wire subwin;
   wire [`CMD_ADDR_SIZE-1:0]            _laddr;
   wire [`CMD_ADDR_SIZE-1:0]            _taddr;
   wire [`CMD_ADDR_SIZE-1:0]            taddr1;
   wire [`SDMA_PKT_COPY_TILED_BC_MIT_MODE_SIZE-1:0]  nondisp, nondisp1;
   wire                                           t2t;
   wire                                           t2l;
   wire [`SDMA_PKT_COPY_TILED_BC_ELEMENT_SIZE_SIZE-1:0]       psize;
   wire [`SDMA_PKT_COPY_TILED_BC_BANK_W_SIZE-1:0]  bwidth, bwidth1;
   wire [`SDMA_PKT_COPY_TILED_BC_BANK_H_SIZE-1:0] bheight, bheight1;
   wire [`SDMA_PKT_COPY_TILED_BC_MAT_ASPT_SIZE-1:0]   aspect, aspect1;
   wire [`SDMA_PKT_COPY_TILED_BC_NUM_BANK_SIZE-1:0]    numbank, numbank1;
   wire [`SDMA_PKT_COPY_TILED_BC_PIPE_CONFIG_SIZE-1:0] pipecfg, pipecfg1;
   wire [`SDMA_PKT_COPY_TILED_BC_ARRAY_MODE_SIZE-1:0]       tmode, tmode1;
   wire [`SDMA_PKT_COPY_TILED_BC_HEIGHT_SIZE-1:0] height, height1;
   wire [`SDMA_PKT_COPY_TILED_BC_PITCH_IN_TILE_SIZE-1:0] tpitch, tpitch1;
   wire [`SDMA_PKT_COPY_TILED_BC_SLICE_PITCH_SIZE-1:0] tslice, tslice1;
   wire [`SDMA_PKT_COPY_TILED_SUBWIN_LINEAR_PITCH_SIZE+4:0] lpitch, lpitch1;
   wire [`SDMA_PKT_COPY_TILED_SUBWIN_LINEAR_SLICE_PITCH_SIZE+4:0] lslice, lslice1;
   wire [`SDMA_PKT_COPY_TILED_BC_TILESPLIT_SIZE_SIZE-1:0]     tsplit, tsplit1;
   wire [`SDMA_PKT_COPY_TILED_BC_X_SIZE-1:0]             _tx;
   wire [`SDMA_PKT_COPY_TILED_BC_Y_SIZE-1:0]             _ty;
   wire [`SDMA_PKT_COPY_TILED_BC_Z_SIZE-1:0]             _tz;
   wire [`SDMA_PKT_COPY_TILED_BC_X_SIZE-1:0]             _tx1;
   wire [`SDMA_PKT_COPY_TILED_BC_Y_SIZE-1:0]             _ty1;
   wire [`SDMA_PKT_COPY_TILED_BC_Z_SIZE-1:0]             _tz1;
   wire [`SDMA_PKT_COPY_TILED_BC_X_SIZE-1+1:0]             dx;
   wire [`SDMA_PKT_COPY_TILED_BC_Y_SIZE-1+1:0]             dy;
   wire [`SDMA_PKT_COPY_TILED_BC_Z_SIZE-1+1:0]             dz;

   wire [`SDMA_PKT_COPY_STRUCT_STRIDE_SIZE-1:0]      stride;
   wire [`SDMA_PKT_COPY_STRUCT_START_INDEX_SIZE-1:0] sindex;
   wire [`SDMA_PKT_COPY_STRUCT_COUNT_SIZE-1:0] cindex;
   reg [`SDMA_PKT_COPY_STRUCT_COUNT_SIZE-1:0] tindex;
   reg [`SDMA_PKT_COPY_STRUCT_STRIDE_SIZE-1:0]      tstride;

   reg [`CMD_ADDR_SIZE-1:0]            laddr;
   reg [`CMD_ADDR_SIZE-1:0]            taddr;
   reg [`SDMA_PKT_COPY_TILED_BC_X_SIZE-1:0]            tx;
   reg [`SDMA_PKT_COPY_TILED_BC_Y_SIZE-1:0]             ty;
   reg [`SDMA_PKT_COPY_TILED_BC_Z_SIZE-1:0]             tz;
   reg [`SDMA_PKT_COPY_TILED_BC_X_SIZE-1:0]            tx1;
   reg [`SDMA_PKT_COPY_TILED_BC_Y_SIZE-1:0]             ty1;
   reg [`SDMA_PKT_COPY_TILED_BC_Z_SIZE-1:0]             tz1;
   reg [`CMD_ADDR_SIZE-1:0]            start_laddr;  //base linear addr of each z surface, for subwin
   reg [`CMD_ADDR_SIZE-1:0]            start_laddr1;
   wire [`CMD_ADDR_SIZE-1:0]            laddr_voidfix;
   wire [`SDMA_PKT_COPY_TILED_BC_X_SIZE-1:0] tx_voidfix;

   assign       {video, broadcast, dhash, shash, dswap, sswap, dtci, stci, dvol, svol, dpol, spol, hash1, swap1, tci1, vol1, pol1,
    vmid, priv, structt, stride, sindex, cindex, crc, trns, aes_cbc, aes, linear, subwin, ssize, 
    _taddr, taddr1, _laddr, nondisp, nondisp1, t2t, t2l, psize, bwidth, bheight, aspect, numbank, pipecfg, tmode, height, tpitch, tslice,
    lpitch, lslice, tsplit, bwidth1, bheight1, aspect1, numbank1, pipecfg1, tmode1, tsplit1, height1, tpitch1, tslice1, lpitch1, lslice1,
    _tx, _ty, _tz, _tx1, _ty1, _tz1, dx, dy, dz} = ex_param;


   reg [`SDMA_PKT_COPY_TILED_BC_X_SIZE-1:0] start_x;
   reg [`SDMA_PKT_COPY_TILED_BC_Y_SIZE-1:0] start_y;
   reg [`SDMA_PKT_COPY_TILED_BC_Z_SIZE-1:0] start_z;
   wire [`SDMA_PKT_COPY_TILED_BC_X_SIZE:0]   ex_endx = subwin ? start_x + dx : ((tpitch+1)*8);
   wire [`SDMA_PKT_COPY_TILED_BC_Z_SIZE-1+1:0] ex_endz = start_z + dz ;
   wire [`SDMA_PKT_COPY_TILED_BC_Y_SIZE:0]   ex_endy = start_y + dy ;

   reg [`SDMA_PKT_COPY_TILED_BC_Y_SIZE-4:0] ty_chunk;	//used as right block end flag for normal tile IE 
   reg [`CMD_COUNT_SIZE-1+1:0] size_left_ie;		//used for cont_ie judgement for normal tile IE
   wire [`CMD_COUNT_SIZE-1:0] chunk_line_size = (tpitch+1) << (3+3+psize);
   wire [`SDMA_PKT_COPY_TILED_BC_X_SIZE+3:0] x_width = ((tpitch+1)<< psize);

   reg[3:0] split_state, split_next;

   wire rotate = t2t && nondisp==0 && nondisp1==3;

   wire [4:0] right_dx = psize==0 ? ex_endx[4:0] :
			 psize==1 ? ex_endx[3:0] :
				    ex_endx[2:0];

   wire [6:0] right_block_size = right_dx << psize;

   reg [`SDMA_PKT_COPY_STRUCT_COUNT_SIZE-1:0] cont_struct_timing1;
   reg cont_struct_timing2;
   always@(posedge clk)
     begin
     if (bc_mode)
       cont_struct_timing1 <= cindex+sindex-31;
       cont_struct_timing2 <= cindex+sindex >= 31 ? 1 : 0;
     end

   reg[5:0] tilenum;  
   wire[`CMD_COUNT_SIZE-1+1:0] cmd_info_size ;
   reg [`CMD_COUNT_SIZE-1+1:0] size_left;
   wire has_left = subwin ? (psize==0 ? |_tx[4:0] : psize==1 ? |_tx[3:0] : |_tx[2:0]) : 0;
   wire has_right = psize==0 ?  |ex_endx[4:0] : psize==1 ?  |ex_endx[3:0] : |ex_endx[2:0];
   wire has_top = subwin ? |_ty[2:0] : (|_tx || |_ty[2:0]);
   wire has_bottom = subwin ? |ex_endy[2:0] : size_left_ie > chunk_line_size;
   //tiled copy can run with improved efficiency
   //128bpp donnot support IE copy
   wire has_ie = copy_ie_en && !(linear || video || aes) && (psize!=4) && (subwin ? (|_ty[2:0] ? dy+_ty[2:0]>=16 : dy>=8) && (
		psize==0 ? (|_tx[4:0] ? dx+_tx[4:0]>=64 : dx>=32) :
		psize==1 ? (|_tx[3:0] ? dx+_tx[3:0]>=32 : dx>=16) :
		(|_tx[2:0] ? dx+_tx[2:0]>=16 : dx>=8) ) :
		(x_width >= 32) && (has_top ? ssize>= (chunk_line_size<<1) : ssize>= (chunk_line_size)));  //at least one tile line for count based copy
   reg [`SDMA_PKT_COPY_TILED_BC_Y_SIZE-1:0] right_start_y; 
   wire[`SDMA_PKT_COPY_TILED_BC_X_SIZE-1+1:0] right_start_x = psize==0 ? {ex_endx[`SDMA_PKT_COPY_TILED_BC_X_SIZE-1+1:5],5'b0}  :
                        psize==1 ? {ex_endx[`SDMA_PKT_COPY_TILED_BC_X_SIZE-1+1:4],4'b0} : {ex_endx[`SDMA_PKT_COPY_TILED_BC_X_SIZE-1+1:3],3'b0};
   //X of first pixel of the chunk area
   wire[`SDMA_PKT_COPY_TILED_BC_X_SIZE-1:0] ie_start_x = psize==0 ? {start_x[`SDMA_PKT_COPY_TILED_X_SIZE-1:5]+|start_x[4:0],5'b0} :
		psize==1 ? {start_x[`SDMA_PKT_COPY_TILED_BC_X_SIZE-1:4]+|start_x[3:0],4'b0} : 
			   {start_x[`SDMA_PKT_COPY_TILED_BC_X_SIZE-1:3]+|start_x[2:0],3'b0};
   wire next_ie = tx + {tilenum,3'b0} < right_start_x;   //there are more chunks to the right
   //cont_ie: no more chunks to the right. more chunks below.
   wire cont_ie = (subwin ? (ty+16<=ex_endy) : (size_left_ie >= (chunk_line_size<<1))) && !next_ie;
   wire [`SDMA_PKT_COPY_STRUCT_STRIDE_SIZE-1:0]      stride_32B = {stride[`SDMA_PKT_COPY_STRUCT_STRIDE_SIZE-1:3],3'b0};
   wire next_struct = tstride+tilenum<stride_32B;  			  //more chunks to the right. for struct ie
   //wire cont_struct = (tindex+31<=cindex+sindex) && !next_struct; //more chunks below  and no more chunk to the right
   wire cont_struct = tindex<=cont_struct_timing1 && cont_struct_timing2 && !next_struct; //more chunks below  and no more chunk to the right

   //line_addr: laddr or taddr of first pixel of a line
   reg [`CMD_ADDR_SIZE-1:0] line_laddr, line_taddr;
   //first_ie_line_addr: laddr or taddr of first pixel of first line of right area (= chunk area).
   reg [`CMD_ADDR_SIZE-1:0] first_ie_line_laddr, first_ie_line_taddr;
   reg first_ie; //flag signal for recording first_ie_line_addr

   parameter[3:0] state_start = 0, state_line = 1, state_ie = 2, state_t2t = 3, state_struct = 4, state_pending = 5, state_top = 6,
	state_bottom = 7, state_left = 8, state_right = 9, state_incz = 10, state_stride = 11, state_struct_right = 12, state_end = 13, 
        state_incz_end = 14;
   always@(posedge clk)
     if(~resetb) begin
       split_state <= 0;
       split_next <= 0;
       {tindex, taddr, laddr, tx, ty, tz, tx1, ty1, tz1} <= 0;
       {start_laddr1, start_laddr, start_y, start_z} <= 0;
       line_laddr <= 0;
       line_taddr <= 0;
       start_x <= 0;
       tilenum <= 0;
       tstride <= 0;
       size_left <= 0;
       size_left_ie <= 0;
       first_ie <= 0;
       ty_chunk <= 0;
       right_start_y <= 0;
     end
     else case(split_state)
       state_start: begin
          {tindex, taddr, laddr, tx, ty, tz, tx1, ty1, tz1} <= {sindex, _taddr, _laddr, _tx, _ty, _tz, _tx1, _ty1, _tz1} ; 
          {start_laddr1, start_laddr, start_y, start_z} <= {_taddr, _laddr, _ty, _tz};  
          line_laddr <= _laddr;
          line_taddr <= _taddr;
	  start_x <= subwin ? _tx : 0;
	  tilenum <= structt ? stride_32B-tstride<32 ? stride_32B-tstride : 32 : //tile number of a struct chunk. 1024b at most
	     		 t2t ? dx[`SDMA_PKT_COPY_TILED_BC_X_SIZE-1+1:3] :	//tile number of a tile line
             		 subwin ? (dx<<psize)>256 ? (32>>psize) : psize==0 ? {dx[`SDMA_PKT_COPY_TILED_BC_X_SIZE-1+1:5],2'b0} : //tile number of a chunk. 2048b at most
	     							  psize==1 ? {dx[`SDMA_PKT_COPY_TILED_BC_X_SIZE-1+1:4],1'b0} : dx[`SDMA_PKT_COPY_TILED_X_SIZE-1:3] :
				  ((ex_endx-tx)<<psize)>256 ? (32>>psize) : right_start_x[`SDMA_PKT_COPY_TILED_BC_X_SIZE-1+1:3] - tx[`SDMA_PKT_COPY_TILED_X_SIZE-1:3];
	  tstride <= 0;
          size_left <= ssize;
          size_left_ie <= ssize;
          first_ie <= 1;
          ty_chunk <= _ty[`SDMA_PKT_COPY_TILED_BC_Y_SIZE-1:3];
	  right_start_y <= has_top ? {_ty[`SDMA_PKT_COPY_TILED_BC_Y_SIZE-1:3]+1'b1,3'b0}  : {_ty[`SDMA_PKT_COPY_TILED_Y_SIZE-1:3] ,3'b0};
          if(ex_start && bc_mode && rreq_rtr && wreq_rtr && !cmd_info_full && !cmd_info1_full)
	     split_state <= //structt ? copy_ie_en && stride_32B>=8 && sindex==0 && cindex>=15 ? state_struct : state_stride :
		subwin ? t2t ? state_t2t : has_ie ? has_top ? state_top : has_left ? state_left : state_ie : state_line : 
		has_ie ? has_top ? state_top : state_ie : 
		state_start;
       end
       state_ie: begin
	  tx <= subwin ? next_ie ? tx + {tilenum,3'b0} : cont_ie ? ie_start_x : has_right ? right_start_x : has_bottom ? start_x : tx :
                        ty+8 > height ? next_ie ? tx + {tilenum,3'b0} : has_right ? right_start_x : 0 :
                                         next_ie ? tx + {tilenum,3'b0} : cont_ie ? ie_start_x : has_right ? right_start_x : 0; 
          ty <= subwin ? next_ie ? ty : cont_ie ? ty + 8 : has_right ? right_start_y : has_bottom ? ty+8 : ty :
                         ty+8 > height ? next_ie ? ty : has_right ? right_start_y : 0 :
                                         next_ie ? ty : cont_ie ? ty + 8 : has_right ? right_start_y : has_bottom ? ty + 8 : ty;
	  line_laddr <= subwin ? next_ie ? line_laddr : cont_ie ? line_laddr + {lpitch,3'b0} : 
                                 			has_right ? first_ie ? line_laddr : first_ie_line_laddr : has_bottom ? line_laddr + {lpitch,3'b0} : line_laddr :
				 ty+8 > height ? next_ie ? line_laddr : has_right ? (first_ie ? line_laddr : first_ie_line_laddr) : line_laddr + {lpitch,3'b0} :
                                                 next_ie ? line_laddr : cont_ie ? line_laddr + {lpitch,3'b0} : has_right ? (first_ie ? line_laddr : first_ie_line_laddr) : line_laddr + {lpitch,3'b0};
          if(first_ie) begin first_ie_line_laddr <= line_laddr;   first_ie <= 0; end
	  split_state <= state_pending;
	  split_next <= subwin ? (cont_ie || next_ie) ? state_ie : has_right ? state_right : has_bottom ? state_bottom : tz==ex_endz-1 ? state_start : state_incz :
				 ty+8 > height ? next_ie ? state_ie : has_right ? state_right : cont_ie ? state_incz : has_bottom ? state_incz_end : state_start :
                                                 (cont_ie || next_ie) ? state_ie : has_right ? state_right : has_bottom ? state_end : state_start;
		
	  size_left <= size_left - (tilenum<<(psize+6));
	  if (!subwin) ty_chunk <= ty[`SDMA_PKT_COPY_TILED_BC_Y_SIZE-1:3];
	  if (!subwin && tx + {tilenum,3'b0} == right_start_x) size_left_ie <= size_left_ie - chunk_line_size;
       end
       state_line: begin  //for l2l subwin, tiled subwin without ie
	  split_state <= state_pending;
          split_next <= ty==ex_endy-1 ? tz==ex_endz-1 ? state_start : state_incz : state_line;
          ty <= ty + 1;
          ty1 <= ty1 + 1;
	  line_laddr <= line_laddr + lpitch;
	  taddr <= linear ?  taddr + lpitch1 : taddr;
       end
       state_top: begin
          split_state <= state_pending;
          split_next <= subwin ? ty[2:0]==7 ? has_left ? state_left : state_ie : state_top :
				 ty[2:0]==7 ? ty == height ? state_incz : state_ie : state_top;
	  size_left <= size_left - ((ex_endx-tx)<<psize);
	  if (!subwin) size_left_ie <= size_left_ie - ((ex_endx-tx)<<psize);
          tx <= !subwin ? 0 : tx;
          ty <= !subwin && (ty==height) ? 0 : ty + 1;
	  line_laddr <= line_laddr + (subwin ? lpitch : ((ex_endx-tx)<<psize));
       end
       state_bottom: begin	//NOT for normal tile IE
          split_state <= state_pending;
          split_next <= ty==ex_endy-1 ? tz==ex_endz-1 ? state_start : state_incz : state_bottom;
          ty <= ty + 1;
	  line_laddr <= line_laddr + lpitch;
       end
       state_left: begin
	  split_state <= state_pending;
          split_next <= ty=={ex_endy[`SDMA_PKT_COPY_TILED_BC_Y_SIZE-1+1:3],3'b0}-1 ? state_ie : state_left;
          tx <= ty=={ex_endy[`SDMA_PKT_COPY_TILED_BC_Y_SIZE-1+1:3],3'b0}-1 ? ie_start_x : tx;
          ty <= ty=={ex_endy[`SDMA_PKT_COPY_TILED_BC_Y_SIZE-1+1:3],3'b0}-1 ? right_start_y : ty + 1;
          line_laddr <= ty=={ex_endy[`SDMA_PKT_COPY_TILED_BC_Y_SIZE-1+1:3],3'b0}-1 ? first_ie_line_laddr : line_laddr + lpitch;
          if(first_ie) begin first_ie_line_laddr <= laddr;   first_ie <= 0; end
       end
       state_right: begin
          split_state <= state_pending;
          split_next <= subwin ? ty=={ex_endy[`SDMA_PKT_COPY_TILED_BC_Y_SIZE-1+1:3],3'b0}-1 ? has_bottom ? state_bottom : tz==ex_endz-1 ? state_start : state_incz : state_right :
                                 ty=={ty_chunk, 3'h7} ? size_left == ((ex_endx-tx)<<psize) ? state_start : cont_ie ? state_incz : ty==height ? state_incz_end : state_end : state_right;
          tx <= ty==(subwin ? {ex_endy[`SDMA_PKT_COPY_TILED_BC_Y_SIZE-1+1:3],3'b0}-1 : {ty_chunk, 3'h7}) ? start_x : tx;
          ty <= ty + 1;
	  line_laddr <= line_laddr + lpitch;
	  size_left <= size_left - ((ex_endx-tx)<<psize);
       end
       state_end: begin		//only for normal IE bottom block
	  split_state <= state_pending;
          split_next <= state_start;
       end
       state_pending: begin  //fsm goes to this state after each sub copy cmd, waiting for rtr.
	  tilenum <= structt ? stride_32B-tstride<32 ? stride_32B-tstride : 32 :
		t2t ? dx[`SDMA_PKT_COPY_TILED_BC_X_SIZE-1+1:3] :
		((ex_endx-tx)<<psize)>256 ? (32>>psize) : right_start_x[`SDMA_PKT_COPY_TILED_BC_X_SIZE-1+1:3]
		- tx[`SDMA_PKT_COPY_TILED_BC_X_SIZE-1:3];
          laddr <= structt ? line_laddr + {tstride,2'b0} : line_laddr + ((tx-start_x)<<psize); 
          taddr <= structt ? line_taddr + {tstride,6'b0} : taddr;
	  split_state <= rreq_rtr && wreq_rtr && !cmd_info_full && !cmd_info1_full ? split_next : state_pending;
       end
       state_incz: begin //change to next z plane 
          {laddr, tx} <= subwin ? {start_laddr+lslice, _tx} : {laddr, `SDMA_PKT_COPY_TILED_BC_X_SIZE'h0};   
          {start_y, ty1, ty} <= subwin ? {_ty, _ty1, _ty} : 0;
          right_start_y <= subwin ? right_start_y : 0;
          {start_laddr, start_x} <= subwin ? {start_laddr+lslice, _tx} : {laddr, `SDMA_PKT_COPY_TILED_BC_X_SIZE'h0};  
          start_laddr1 <= start_laddr1+lslice1;
          line_laddr <= subwin ? start_laddr + lslice : line_laddr;
          first_ie_line_laddr <= subwin ? first_ie_line_laddr + lslice : line_laddr;	//next z-surface first ie line_laddr
	  taddr <= linear ? start_laddr1 + lslice1 : taddr;
          tz <= tz +   (t2t && nondisp == 4 ? tmode == 8 || tmode == 14 ? 8 : 4 : 1);
          tz1 <= tz1 + (t2t && nondisp == 4 ? tmode == 8 || tmode == 14 ? 8 : 4 : 1);
          tilenum <= t2t ? dx[`SDMA_PKT_COPY_TILED_BC_X_SIZE-1+1:3] :
                subwin ? (dx<<psize)>256 ? (32>>psize) : psize==0 ? {dx[`SDMA_PKT_COPY_TILED_BC_X_SIZE-1+1:5],2'b0} :
                                                         psize==1 ? {dx[`SDMA_PKT_COPY_TILED_BC_X_SIZE-1+1:4],1'b0} : dx[`SDMA_PKT_COPY_TILED_X_SIZE-1+1:3] :
                         ((ex_endx-tx)<<psize)>256 ? (32>>psize) : right_start_x[`SDMA_PKT_COPY_TILED_BC_X_SIZE-1+1:3] - tx[`SDMA_PKT_COPY_TILED_X_SIZE-1:3];
	  split_state <= subwin ? t2t ? state_t2t : has_ie ? has_top ? state_top : has_left ? state_left : state_ie : state_line : 
				  state_ie;
	  ty_chunk <= 0;
       end
       state_incz_end: begin  //count based copy. no ie after changed to next z plane.
          {laddr, tx} <= {laddr, `SDMA_PKT_COPY_TILED_BC_X_SIZE'h0};   
          {start_y, ty1, ty} <= 0;
          right_start_y <= 0;
          {start_laddr, start_x} <= {laddr, `SDMA_PKT_COPY_TILED_BC_X_SIZE'h0};  
          tz <= tz + 1;
	  split_state <= state_end;
	  ty_chunk <= 0;
       end
       state_t2t: begin  //also for Rotate tile T2T, treat as IE for Rotate
	  split_state <= state_pending;
          split_next <= ty==ex_endy-8 ? tz==ex_endz - (nondisp == 4 ? tmode == 8 || tmode == 14 ? 8 : 4 : 1) ? state_start : state_incz : state_t2t;
          ty <= ty + 8;
          ty1 <= ty1 + 8;
       end
       //state_struct: begin  //struct chunk
       //   tstride <= cont_struct ? 0 : next_struct ? tstride+tilenum : |stride[2:0] ? tstride+tilenum : 0; 
       //   tindex <= cont_struct ? tindex+16 : next_struct ? tindex : |stride[2:0] ? {(sindex[`SDMA_PKT_COPY_STRUCT_START_INDEX_SIZE-1:4] + |sindex[3:0]), 4'b0} : tindex+16;
       //   line_laddr <= cont_struct ? line_laddr + {stride,6'b0} : next_struct ? line_laddr : 
       //         |stride[2:0] ? first_ie ? line_laddr : first_ie_line_laddr : line_laddr + {stride,6'b0};
       //   line_taddr <= cont_struct ? line_taddr + {stride,6'b0} : next_struct ? line_taddr :
       // 	|stride[2:0] ? first_ie ? line_taddr : first_ie_line_taddr : line_taddr + {stride,6'b0}; 
       //   if(first_ie) begin first_ie_line_laddr <= line_laddr; first_ie_line_taddr <= line_taddr;  first_ie <= 0; end
       //   split_state <= state_pending;
       //   split_next <= cont_struct || next_struct ? state_struct : |stride[2:0] ? state_struct_right : tindex+15==cindex+sindex ? state_start : state_stride;
       //end
       //state_stride: begin  //struct stride. For top, bottom or no ie.
       //   split_state <= state_pending;
       //   split_next <= tindex==cindex+sindex ? state_start : 
       // 	copy_ie_en && stride_32B>=8 && tindex[3:0]==15 && tindex+16<=cindex+sindex ? state_struct : state_stride;
       //   tindex <= tindex + 1;
       //   line_laddr <= line_laddr + {stride,2'b0};
       //   line_taddr <= line_taddr + (tindex[3:0]==15 ? {stride,6'b0}-15*4 : 4); 
       //end
       //state_struct_right: begin
       //   split_state <= state_pending;
       //   tstride <= tindex[3:0]==15 && tindex+16>cindex+sindex ? 0 : tstride; 
       //   tindex <= tindex + 1;
       //   line_laddr <= line_laddr + {stride,2'b0};
       //   line_taddr <= line_taddr + (tindex[3:0]==15 ? {stride,6'b0}-15*4 : 4); 
       //   split_next <= tindex==cindex+sindex ? state_start : tindex[3:0]==15 && tindex+16>cindex+sindex ? state_stride : state_struct_right;
       //end
       default: split_state <= split_state;
     endcase

   always @* 
     case(split_state)
       state_start:
          last_sub_cp = split_start && !structt && !subwin && !has_ie;
       state_ie:
	  last_sub_cp = !next_ie && !cont_ie && !has_right && !has_bottom && (subwin ? tz==ex_endz-1 : 1'b1);
       state_line,
       state_bottom:
	  last_sub_cp = video ? size_left <= ((ex_endx-tx)<<psize) : ty==ex_endy-1 && tz==ex_endz-1; 
       state_right:
	  last_sub_cp = subwin ? ty=={ex_endy[`SDMA_PKT_COPY_TILED_Y_SIZE-1+1:3],3'b0}-1 && !has_bottom && tz==ex_endz-1 :
			         ty=={ty_chunk, 3'h7} && size_left == ((ex_endx-tx)<<psize);
       state_end:
	  last_sub_cp = 1'b1;
       state_t2t:
	  last_sub_cp = ty==ex_endy-8 && tz==ex_endz - (nondisp == 4 ? tmode == 8 || tmode == 14 ? 8 : 4 : 1);
       state_struct:
	  last_sub_cp = !cont_struct && !next_struct && !(|stride[2:0]) && tindex+15==cindex+sindex;
       state_stride,
       state_struct_right:
	  last_sub_cp = tindex==cindex+sindex;
       default: last_sub_cp = 1'b0;
     endcase

   //reg split_start;
   always@(posedge clk)
     if(~resetb) begin
       split_start <= 0;
     end
     else
       split_start <= ex_start && bc_mode && rreq_rtr && wreq_rtr && !cmd_info_full && !cmd_info1_full;

   reg[1:0] granu;
   always @* 
     begin
        if(/*split_state == state_ie ||*/ split_state == state_t2t || linear)
          granu = `DMA_MC_wr_granularity_min_256b;
        else if(split_state == state_stride || split_state == state_struct_right)
          granu = `DMA_MC_wr_granularity_min_32b;
        else casez ({
                     nondisp[2:0],
		     psize[2:0] // 0 = 8bpp, 1 = 16bpp, 2 = 32bpp, 3 = 64bpp, 4 = 128bpp
		     })
`ifdef THICK_TILE_EN
	       // Thick Tile
                6'b100000: granu = `DMA_MC_wr_granularity_min_32b;
		6'b100001: granu = `DMA_MC_wr_granularity_min_64b;
		6'b100010: granu = `DMA_MC_wr_granularity_min_128b;
		6'b100011: granu = `DMA_MC_wr_granularity_min_128b;
		6'b100100: granu = `DMA_MC_wr_granularity_min_256b;
`endif	
               // Non-display.
	       5'b0100?,5'b1000?: granu = `DMA_MC_wr_granularity_min_64b;
               5'b01010,5'b10010: granu = `DMA_MC_wr_granularity_min_128b;
               5'b01011,5'b10011: granu = `DMA_MC_wr_granularity_min_128b;
               5'b01100,5'b10100: granu = `DMA_MC_wr_granularity_min_256b;

               5'b00000: granu = `DMA_MC_wr_granularity_min_64b; 
               5'b00001: granu = `DMA_MC_wr_granularity_min_128b;
               5'b00010: granu = `DMA_MC_wr_granularity_min_128b;
               5'b00011: granu = `DMA_MC_wr_granularity_min_128b;
               5'b00100: granu = `DMA_MC_wr_granularity_min_128b;
               // VCS coverage off
               default: granu = `DMA_MC_wr_granularity_min_256b; 
               // VCS coverage on
             endcase
     end

wire[2:0] voidd;
wire split_ie;
assign split_param = {video, broadcast, shash, sswap, stci, svol, spol, vmid, priv, structt, stride, voidd, crc, trns, aes_cbc, aes, subwin, linear, granu, tilenum, cmd_info_size, 
	taddr, taddr1, laddr_voidfix, nondisp, t2t, t2l, psize, bwidth, bheight, aspect, numbank, pipecfg, tmode, height, tpitch, tslice,
        lpitch, lslice, tsplit, tx_voidfix, ty, tz, dx, dy, dz, split_ie};

assign split_param_dst = t2t ? {video, broadcast, dhash, dswap, dtci, dvol, dpol, hash1, swap1, tci1, vol1, pol1, vmid, priv, structt, stride, voidd, crc, trns, aes_cbc, aes, subwin, linear, granu, tilenum, 
	cmd_info_size, taddr, taddr1, laddr_voidfix, (rotate? nondisp1 : nondisp), t2t, t2l, psize, bwidth1, bheight1, aspect1, numbank1, pipecfg1, tmode1, 
	height1, tpitch1, tslice1, lpitch, lslice, tsplit1, tx1, ty1, tz1, dx, dy, dz, split_ie} :
	{video, broadcast, dhash, dswap, dtci, dvol, dpol, hash1, swap1, tci1, vol1, pol1, vmid, priv, structt, stride, voidd, crc, trns, aes_cbc, aes, subwin, linear, granu, tilenum, cmd_info_size, 
	taddr, taddr1, laddr_voidfix, nondisp, t2t, t2l, psize, bwidth, bheight, aspect, numbank, pipecfg, tmode, height, tpitch, tslice,
        lpitch, lslice, tsplit, tx_voidfix, ty, tz, dx, dy, dz, split_ie};

assign split_rts = split_state == state_ie || split_state == state_line || split_state == state_top || split_state == state_bottom ||
	split_state == state_left || split_state == state_right || split_state == state_start && split_start ||
	split_state == state_t2t || split_state == state_struct || split_state == state_stride || split_state == state_struct_right ||
	split_state == state_end;

   ////////////////////////////////////
   ////////  Cmd Info FIFO Wr /////////
   ////////////////////////////////////

//Void: in normal tiled copy, X should be aligned to DW at least. When start x is not aligned to granu, copy the whole granu, and abondon the beginning void dwords when output
  assign voidd = 	linear || structt ? 0 :
			psize==0 && nondisp!=4 ? tx[2] : 
		        psize==1 && nondisp!=0 ? tx[1] : 
			psize==1 && nondisp==0 ? tx[2:1] : 
`ifdef THICK_TILE_EN
			psize==2 && nondisp==4 ? tx[1:0] :
`endif
		        psize==2 ? tx[1:0] : 
			psize==3 ? {tx[0],1'b0} :
			psize==4 && nondisp!=0 ? {tx[0],2'b0} : 0;

   assign laddr_voidfix = laddr - {voidd, 2'b0};

   assign tx_voidfix =    linear ? tx :
			  psize==0 && nondisp!=4 ? {tx[`SDMA_PKT_COPY_TILED_BC_X_SIZE-1:3], 3'h0} :
 			  psize==1 && nondisp!=0 ? {tx[`SDMA_PKT_COPY_TILED_BC_X_SIZE-1:2], 2'h0} :
                          psize==1 && nondisp==0 ? {tx[`SDMA_PKT_COPY_TILED_BC_X_SIZE-1:3], 3'h0} :
`ifdef THICK_TILE_EN
                          psize==2 && nondisp==4 ? {tx[`SDMA_PKT_COPY_TILED_BC_X_SIZE-1:2], 2'h0} :
`endif
                          psize==2 ? {tx[`SDMA_PKT_COPY_TILED_BC_X_SIZE-1:2],2'h0} :
                          psize==3 ? {tx[`SDMA_PKT_COPY_TILED_BC_X_SIZE-1:1],1'h0} :
                          psize==4 && nondisp!=0 ? {tx[`SDMA_PKT_COPY_TILED_BC_X_SIZE-1:1], 1'h0} : tx;

  assign ram_cmd_info_we = split_rts;
  assign split_ie = split_state == state_ie || split_next == state_ie || split_state == state_struct || split_next == state_struct ||
	split_state == state_t2t && rotate;
  assign cmd_info_size = split_state == state_ie || split_next == state_ie ? t2l ? (tilenum<<(psize+3)) : (tilenum<<(psize+6)) : 
		split_state == state_line || split_state == state_top || split_state == state_bottom ? ((ex_endx-tx_voidfix)<<psize) : 
		split_state == state_left ? ((ie_start_x-tx_voidfix)<<psize) : 
		split_state == state_right ? right_block_size : 
		split_state == state_end ? size_left : 
		split_state == state_t2t ? (nondisp == 4 ? tmode == 8 || tmode == 14 ? (dx << (psize+6)) : (dx << (psize+5)) : (dx << (psize+3))) : 
		split_state == state_stride ? {stride,2'b0} :
		split_state == state_struct ? t2l ? {tilenum,2'b0} : {tilenum,6'b0} :
		split_state == state_struct_right ? {stride[2:0],2'b0} :
		linear ? ssize : ssize+{voidd,2'b0}; 

  wire[4:0] ie_l2t_src_start = split_ie && |(structt ? stride[2:0] : lpitch[4:2]) && ~|laddr[4:2] ? 4 : laddr_voidfix[4:0];

  //how many 256b data in a chunk. +8 for unaligned l2t. +16 for unaligned l2s.
  wire[6:0] chunk_size = rotate ? (2<<psize) : structt ? {tilenum,1'b0}+{(~t2l & |ie_l2t_src_start[4:2]),4'b0} : (tilenum<<(psize+1))+{(~t2l & |ie_l2t_src_start[4:2]),3'b0};

   assign ram_cmd_info_din =  
	t2t ?	 {t2t, structt, tilenum, granu, rotate,   t2l, nondisp, psize, cmd_info_size, 5'b0, chunk_size} :
	linear ? {1'b0, structt, tilenum, granu, split_ie, t2l, nondisp, psize, cmd_info_size, laddr[4:0], chunk_size} :
	t2l ? {1'b0, structt, tilenum, granu, split_ie, t2l, nondisp, psize, cmd_info_size, 5'b0, chunk_size} : 
		 {1'b0, structt, tilenum, granu, split_ie, t2l, nondisp, psize, cmd_info_size, ie_l2t_src_start, chunk_size};

   assign ram_cmd_info1_din =  
	t2t ?	 {/*1'b0,*/ 1'b0, 1'b0,   3'b0, structt, tilenum, granu, 1'b0,   t2l, nondisp, psize, cmd_info_size, 5'b0,  5'b0} :
	linear ? {/*crc,*/ trns, aes_cbc, 3'b0, structt, tilenum, granu, split_ie, t2l, nondisp, psize, cmd_info_size, laddr[4:0], taddr[4:0]} :
	t2l ? {(structt ? stride[2:0] : lpitch[4:2]), /*crc,*/ trns, aes_cbc, voidd, structt, tilenum, granu, split_ie, t2l, nondisp, psize, cmd_info_size, 5'b0, laddr_voidfix[4:0]} : 
		 {/*crc,*/ trns, aes_cbc, voidd, structt, tilenum, granu, split_ie, t2l, nondisp, psize, cmd_info_size, ie_l2t_src_start, 5'b0};


   always@(posedge clk)
     if(!resetb)
       dma_ce_split_idle <= 1;
     else
       dma_ce_split_idle <= split_state == 0 && !ex_start && !split_start;

endmodule


