`include "dma_common.vh"
`include "sdma_reg.v"

module dma_ce_wreq_bc (

   input clk ,
   input resetb ,

   input split_rts,
   input [`CMD_PARAM_RANGE] split_param_dst,
   input last_sub_cp,
   input bc_mode,

   input mc_wr_wreq_rtr_next,
`ifdef SDMA__UTC_L1__1
   input dma_utc_l1_enable,
   input utcl1_wr_addr_rtr,
   output wreq_mc_wr_atc_rts ,
   output reg  wreq_mc_wr_atc_eoc ,
`endif
   output wreq_rtr,
   output reg dma_ce_wreq_idle,
   output reg wreq_mc_wr_rts ,
   output reg[`CMD_ADDR_SIZE-5-1:0]  wreq_mc_wr_addr ,
   output reg[`CMD_ADDR_SIZE-5-1:0]  wreq_delta ,
   output reg wreq_mc_wr_broadcast ,
   output reg[1:0]  wreq_mc_wr_swap ,
   output reg  wreq_mc_wr_aes ,
   output reg  wreq_mc_wr_crc ,
   output reg  wreq_mc_wr_priv ,
   output reg [`SDMA0_GFX_RB_CNTL_RB_VMID_SIZE-1:0] wreq_mc_wr_vmid ,
   output reg[15:0]  wreq_mc_wr_mask ,
   output reg  wreq_mc_wr_tci ,
   output reg  wreq_mc_wr_vol ,
   output reg  wreq_mc_wr_pol ,
   output reg[5:0] wreq_dst2_info ,
   output reg  wreq_mc_wr_eoc ,

   input [`SDMA0_HASH_CHANNEL_BITS_SIZE-1:0]      dma_channel_bits,
   input [`SDMA0_HASH_BANK_BITS_SIZE-1:0]         dma_bank_bits,
   input [`SDMA0_HASH_CHANNEL_XOR_COUNT_SIZE-1:0] dma_channel_xor,
   input [`SDMA0_HASH_BANK_XOR_COUNT_SIZE-1:0]    dma_bank_xor,
   input [`SDMA0_GB_ADDR_CONFIG_PIPE_INTERLEAVE_SIZE_SIZE-1:0] dma_addr_conf_pipe_interleave_size 
);

   reg video, broadcast;
   reg[2:0] voidd;
   reg[1:0] swap, swap1;
   reg priv;
   reg [`SDMA0_GFX_RB_CNTL_RB_VMID_SIZE-1:0] vmid;
   reg crc, hash, hash1;
   reg tci, vol, pol, tci1, vol1, pol1;
   reg trns;
   reg aes_cbc;
   reg aes;
   reg subwin;
   reg linear;
   reg [1:0] granu;
   reg [5:0] tilenum;
   reg [`CMD_COUNT_SIZE-1+1:0] ssize;
   reg [`CMD_ADDR_SIZE-1:0]            taddr,taddr1;
   reg [`CMD_ADDR_SIZE-1:0]            laddr;
   reg [`SDMA_PKT_COPY_TILED_BC_MIT_MODE_SIZE-1:0]  nondisp;
   reg                                           t2t;
   reg                                           t2l;
   reg [`SDMA_PKT_COPY_TILED_BC_ELEMENT_SIZE_SIZE-1:0]       psize;
   reg [`SDMA_PKT_COPY_TILED_BC_BANK_W_SIZE-1:0]  bwidth;
   reg [`SDMA_PKT_COPY_TILED_BC_BANK_H_SIZE-1:0] bheight;
   reg [`SDMA_PKT_COPY_TILED_BC_MAT_ASPT_SIZE-1:0]   aspect;
   reg [`SDMA_PKT_COPY_TILED_BC_NUM_BANK_SIZE-1:0]    numbank;
   reg [`SDMA_PKT_COPY_TILED_BC_PIPE_CONFIG_SIZE-1:0] pipecfg;
   reg [`SDMA_PKT_COPY_TILED_BC_ARRAY_MODE_SIZE-1:0]       tmode;
   reg [`SDMA_PKT_COPY_TILED_BC_HEIGHT_SIZE-1:0] height;
   reg [`SDMA_PKT_COPY_TILED_BC_PITCH_IN_TILE_SIZE-1:0] tpitch;
   reg [`SDMA_PKT_COPY_TILED_BC_SLICE_PITCH_SIZE-1:0] tslice;
   reg [`SDMA_PKT_COPY_TILED_SUBWIN_LINEAR_PITCH_SIZE+4:0] lpitch;
   reg [`SDMA_PKT_COPY_TILED_SUBWIN_LINEAR_SLICE_PITCH_SIZE+4:0] lslice;
   reg [`SDMA_PKT_COPY_TILED_BC_TILESPLIT_SIZE_SIZE-1:0]     tsplit;
   reg [`SDMA_PKT_COPY_TILED_BC_X_SIZE-1:0]             tx;
   reg [`SDMA_PKT_COPY_TILED_BC_Y_SIZE-1:0]             ty;
   reg [`SDMA_PKT_COPY_TILED_BC_Z_SIZE-1:0]             tz;
   reg [`SDMA_PKT_COPY_TILED_BC_X_SIZE-1+1:0]             dx;
   reg [`SDMA_PKT_COPY_TILED_BC_Y_SIZE-1+1:0]             dy;
   reg [`SDMA_PKT_COPY_TILED_BC_Z_SIZE-1+1:0]             dz;
   reg ie;
   reg structt;
   reg [`SDMA_PKT_COPY_STRUCT_STRIDE_SIZE-1:0]      stride;

  wire rtr;
  reg split_vld;
  reg wreq_vld;
  reg wreq_vld2;
  reg last_sub_cp_d1;
  reg last_sub_cp_vld;
  reg last_data;
  reg last_data2;
  reg broadcast_done;
  always @ (posedge clk)
   if(!resetb) begin
     split_vld <= 0;
     last_sub_cp_d1 <= 0;
     {video, broadcast, hash, swap, tci, vol, pol, hash1, swap1, tci1, vol1, pol1, vmid, priv, structt, stride, 
	voidd, crc, trns, aes_cbc, aes, subwin, linear, granu, tilenum, ssize, 
	taddr, taddr1, laddr, nondisp, t2t, t2l, psize, bwidth, bheight, aspect, numbank, pipecfg, tmode, height, tpitch, tslice,
        lpitch, lslice, tsplit, tx, ty, tz, dx, dy, dz, ie} <= 0;

   end
   else begin
     if(split_rts)
       {video, broadcast, hash, swap, tci, vol, pol, hash1, swap1, tci1, vol1, pol1, vmid, priv, structt, stride, 
	voidd, crc, trns, aes_cbc, aes, subwin, linear, granu, tilenum, ssize, 
	taddr, taddr1, laddr, nondisp, t2t, t2l, psize, bwidth, bheight, aspect, numbank, pipecfg, tmode, height, tpitch, tslice,
        lpitch, lslice, tsplit, tx, ty, tz, dx, dy, dz, ie} <= split_param_dst;
`ifdef SDMA__UTC_L1__1
     else if((dma_utc_l1_enable & broadcast & !t2l ? broadcast_done & wreq_vld2 : wreq_vld) && rtr)
`else
     else if(wreq_vld && rtr)
`endif
     //else if(wreq_vld && rtr)
       voidd <= 0;
     split_vld <= split_rts ? 1 : split_vld && rtr ? 0 : split_vld;
     last_sub_cp_d1 <= last_sub_cp ? 1 : last_sub_cp_d1 && rtr ? 0 : last_sub_cp_d1;
   end

  always @ (posedge clk)
   if(!resetb)
     	last_sub_cp_vld <= 1'b0;
   else if(split_vld && rtr)
     	last_sub_cp_vld <= last_sub_cp_d1;

   reg[`CMD_ADDR_SIZE-1:0] wreq_addr, wreq_addr1;

   reg[3:0] wreq_state, wreq_state_d;
   parameter[3:0] wreq_start = 0, wreq_tile_32 = 1, wreq_tile_64 = 2, wreq_tile_128 = 3, wreq_ie_linear = 4, wreq_ie_tile = 5, 
	wreq_linear = 6, wreq_struct = 7, wreq_tile_256 = 8;
   reg [`CMD_COUNT_SIZE-1+1:0] wreq_size, wreq_count;
   reg [3:0] offset_inc;
`ifdef SDMA__UTC_L1__1
   assign rtr = mc_wr_wreq_rtr_next && (dma_utc_l1_enable ? utcl1_wr_addr_rtr : 1'b1);
`else
   assign rtr = mc_wr_wreq_rtr_next;
`endif
   //assign rtr = mc_wr_wreq_rtr_next;
   reg [3:0] wreq_ie_cnt;
   reg tile_en;
   wire [7:0]                                                      req_cnt; //most 32x8 request for one xthick tile 128bpp
   reg [7:0]                                                       req_cnt_q;
   wire [8:0]                                                      req_cnt_max;
   wire [7:0]   next_req_cnt;
   reg [`DMA_TILE_PARAM_TX_SIZE-4+1:0]                             tile_x_max;
   reg [`DMA_TILE_PARAM_TX_SIZE-4:0]                               tile_x;

   //lines in chunk updated new line_addr
   wire [6:0]	laddr_offset = (structt ? stride[2:0] : lpitch[4:2]) * (wreq_ie_cnt + 1) + laddr[4:2];

   always@(posedge clk)
     if(~resetb) begin
       wreq_state <= 0;
       wreq_addr  <= 0;
       wreq_size  <= 0;
       wreq_count <= 0;
       broadcast_done <= 0;
       wreq_ie_cnt<= 0;
     end
     else case(wreq_state)
       wreq_start: begin
         wreq_addr <= t2t ? taddr1 : linear || !t2l ? taddr : laddr;
         //wreq_addr <= linear && !ie || structt && !t2l ? taddr : laddr;
         wreq_ie_cnt <= 0;
         wreq_size <= ie && t2l ? structt ? ssize[`CMD_COUNT_SIZE-1+1:5] + |laddr[4:0] : ((tilenum<<psize) >>2) + |laddr[4:0] : 
		ssize + (linear && !ie || structt && ie ? taddr[4:0] : laddr[4:0]);
         wreq_count <= /*ie ? t2l ? (tilenum<<(psize+1)) : 8 :*/ ssize;
         wreq_state <= rtr && split_vld ?
	       structt ? ie ? t2l ? wreq_ie_linear : wreq_linear : t2l ? wreq_linear : wreq_struct :
               t2t ? wreq_ie_tile :
               ie ? t2l ? wreq_ie_linear : wreq_ie_tile :
               t2l || linear ? wreq_linear : granu==`DMA_MC_wr_granularity_min_32b ? wreq_tile_32 :
                                granu==`DMA_MC_wr_granularity_min_64b ? wreq_tile_64 : 
				granu==`DMA_MC_wr_granularity_min_128b ? wreq_tile_128 : wreq_tile_256 :
                         wreq_start;
         broadcast_done <= 0;
       end
`ifdef SDMA__UTC_L1__1
       wreq_linear: begin
	 broadcast_done <= rtr ? ~broadcast_done : broadcast_done;
         wreq_addr <= rtr && (!broadcast || !dma_utc_l1_enable || broadcast_done) ? wreq_addr+32 : wreq_addr;
         wreq_addr1 <= rtr && (!broadcast || !dma_utc_l1_enable || broadcast_done) ? wreq_addr1+32 : wreq_addr1;
         wreq_size <= rtr && (!broadcast || !dma_utc_l1_enable || broadcast_done) ? wreq_size - 32 : wreq_size;
	 wreq_state <= wreq_size <= 32 && rtr && (!broadcast || !dma_utc_l1_enable || broadcast_done) ? wreq_start : wreq_linear;
       end
       wreq_tile_32: begin
	 broadcast_done <= rtr ? ~broadcast_done : broadcast_done;
         wreq_count <= rtr && (!broadcast || !dma_utc_l1_enable || broadcast_done) ? wreq_count - 4 : wreq_count;
         wreq_size <= 4;
         wreq_state <= wreq_count <= 4 && rtr && (!broadcast || !dma_utc_l1_enable || broadcast_done) ? wreq_start : wreq_tile_32;
       end
       wreq_tile_64: begin
	 broadcast_done <= rtr ? ~broadcast_done : broadcast_done;
         wreq_count <= rtr && (!broadcast || !dma_utc_l1_enable || broadcast_done) ? wreq_count - 8 : wreq_count;
         wreq_size <= 8;
         wreq_state <= wreq_count <= 8 && rtr && (!broadcast || !dma_utc_l1_enable || broadcast_done) ? wreq_start : wreq_tile_64;
       end
       wreq_tile_128: begin
	 broadcast_done <= rtr ? ~broadcast_done : broadcast_done;
         wreq_count <= rtr && (!broadcast || !dma_utc_l1_enable || broadcast_done) ? wreq_count - 16 : wreq_count;
         wreq_size <= 16;
         wreq_state <= wreq_count <= 16 && rtr && (!broadcast || !dma_utc_l1_enable || broadcast_done) ? wreq_start : wreq_tile_128;
       end
       wreq_tile_256: begin
	 broadcast_done <= rtr ? ~broadcast_done : broadcast_done;
         wreq_count <= rtr && (!broadcast || !dma_utc_l1_enable || broadcast_done) ? wreq_count - 32 : wreq_count;
         wreq_size <= 32;
         wreq_state <= wreq_count <= 32 && rtr && (!broadcast || !dma_utc_l1_enable || broadcast_done) ? wreq_start : wreq_tile_256;
       end
       wreq_ie_tile: begin
	 broadcast_done <= rtr ? ~broadcast_done : broadcast_done;
         wreq_state <= req_cnt_q == req_cnt_max - 1 && tile_x == tile_x_max - 1 && rtr && (!broadcast || !dma_utc_l1_enable || broadcast_done) ? wreq_start : wreq_ie_tile;
       end
`else
       wreq_linear: begin
         wreq_addr <= rtr ? wreq_addr+32 : wreq_addr;
         wreq_size <= rtr ? wreq_size - 32 : wreq_size;
	 wreq_state <= wreq_size > 32 || !rtr ? wreq_linear : wreq_start;
       end
       wreq_tile_32: begin
         wreq_count <= rtr ? wreq_count - 4 : wreq_count;
         wreq_size <= 4;
         wreq_state <= wreq_count > 4 || !rtr ? wreq_tile_32 : wreq_start;
       end
       wreq_tile_64: begin
         wreq_count <= rtr ? wreq_count - 8 : wreq_count;
         wreq_size <= 8;
         wreq_state <= wreq_count > 8 || !rtr ? wreq_tile_64 : wreq_start;
       end
       wreq_tile_128: begin
         wreq_count <= rtr ? wreq_count - 16 : wreq_count;
         wreq_size <= 16;
         wreq_state <= wreq_count > 16 || !rtr ? wreq_tile_128 : wreq_start;
       end
       wreq_tile_256: begin
         wreq_count <= rtr ? wreq_count - 32 : wreq_count;
         wreq_size <= 32;
         wreq_state <= wreq_count > 32 || !rtr ? wreq_tile_256 : wreq_start;
       end
       wreq_ie_tile: begin
         wreq_state <= req_cnt_q == req_cnt_max - 1 && tile_x == tile_x_max - 1 && rtr ? wreq_start : wreq_ie_tile;
       end
`endif
       wreq_struct: begin
         wreq_count <= rtr ? wreq_count - 4 : wreq_count;
         wreq_size <= 4;
         wreq_addr <= rtr ? wreq_addr+64 : wreq_addr;
         wreq_state <= wreq_count > 4 || !rtr ? wreq_struct : wreq_start;
       end
       wreq_ie_linear: begin
         wreq_size <= rtr ? wreq_size == 1 ? structt ? ssize[`CMD_COUNT_SIZE-1+1:5] + |laddr_offset[2:0] : ((tilenum<<psize) >>2) + |laddr_offset[2:0] : wreq_size - 1 : wreq_size;
         wreq_ie_cnt <= rtr && wreq_size == 1 ? wreq_ie_cnt + 1 : wreq_ie_cnt;
         wreq_addr <= rtr ? structt ? wreq_size == 1 ? laddr + {stride,2'b0} * (wreq_ie_cnt+1) : wreq_addr + 32 : 
			wreq_size == 1 ? laddr + lpitch * (wreq_ie_cnt+1) : wreq_addr + 32 : wreq_addr;
         wreq_state <= rtr && wreq_size == 1 && (structt ? wreq_ie_cnt == 15 : wreq_ie_cnt == 7) ? wreq_start : wreq_ie_linear;
       end
       default: wreq_state <= wreq_state;
     endcase

   always@*
     case(wreq_state)
       wreq_start: begin
         wreq_vld = 0;//split_vld && !ie && (linear || t2l || structt && !t2l);
         wreq_vld2 = 0;
         tile_en = 0;//rtr ? !ie && structt && !t2l : tile_en;//!ie && !t2l && !linear && (granu==1 || granu==2) || t2t;
         offset_inc = 0;
	 last_data = 0;
	 last_data2 = 0;
       end
`ifdef SDMA__UTC_L1__1
       wreq_linear: begin
         wreq_vld = (!broadcast || !dma_utc_l1_enable || !broadcast_done);
         wreq_vld2 = 1;
         tile_en = 0;
         offset_inc = 0;
	 last_data = wreq_size <= 32 && (!broadcast || !dma_utc_l1_enable || !broadcast_done);
	 last_data2 = wreq_size <= 32 && (!broadcast || !dma_utc_l1_enable || broadcast_done);
       end
       wreq_tile_32: begin
         wreq_vld = (!broadcast || !dma_utc_l1_enable || !broadcast_done);
         wreq_vld2 = 1;
         tile_en = 1;
         offset_inc = (!broadcast || !dma_utc_l1_enable || broadcast_done);
	 last_data = wreq_count <= 4 && (!broadcast || !dma_utc_l1_enable || !broadcast_done);
	 last_data2 = wreq_count <= 4 && (!broadcast || !dma_utc_l1_enable || broadcast_done);
       end
       wreq_tile_64: begin
         wreq_vld = (!broadcast || !dma_utc_l1_enable || !broadcast_done);
         wreq_vld2 = 1;
         tile_en = 1;
         offset_inc = {(!broadcast || !dma_utc_l1_enable || broadcast_done),1'b0};
	 last_data = wreq_count <= 8 && (!broadcast || !dma_utc_l1_enable || !broadcast_done);
	 last_data2 = wreq_count <= 8 && (!broadcast || !dma_utc_l1_enable || broadcast_done);
       end
       wreq_tile_128: begin
         wreq_vld = (!broadcast || !dma_utc_l1_enable || !broadcast_done);
         wreq_vld2 = 1;
         tile_en = 1;
         offset_inc = {(!broadcast || !dma_utc_l1_enable || broadcast_done),2'h0};
	 last_data = wreq_count <= 16 && (!broadcast || !dma_utc_l1_enable || !broadcast_done);
	 last_data2 = wreq_count <= 16 && (!broadcast || !dma_utc_l1_enable || broadcast_done);
       end
       wreq_tile_256: begin
         wreq_vld = (!broadcast || !dma_utc_l1_enable || !broadcast_done);
         wreq_vld2 = 1;
         tile_en = 1;
         offset_inc = {(!broadcast || !dma_utc_l1_enable || broadcast_done),3'h0};
	 last_data = wreq_count <= 32 && (!broadcast || !dma_utc_l1_enable || !broadcast_done);
	 last_data2 = wreq_count <= 32 && (!broadcast || !dma_utc_l1_enable || broadcast_done);
       end
       wreq_ie_tile: begin
         wreq_vld = (!broadcast || !dma_utc_l1_enable || !broadcast_done);
         wreq_vld2 = 1;
         tile_en = 1;
         offset_inc = 0;
         last_data = req_cnt_q == req_cnt_max - 1 && tile_x == tile_x_max - 1 &&  (!broadcast || !dma_utc_l1_enable || !broadcast_done);
         last_data2 = req_cnt_q == req_cnt_max - 1 && tile_x == tile_x_max - 1 &&  (!broadcast || !dma_utc_l1_enable || broadcast_done);
       end
`else
       wreq_linear: begin
         wreq_vld = 1;
         tile_en = 0;
         offset_inc = 0;
       end
       wreq_tile_32: begin
         wreq_vld = 1;
         tile_en = 1;
         offset_inc = 1;
       end
       wreq_tile_64: begin
         wreq_vld = 1;
         tile_en = 1;
         offset_inc = 2;
       end
       wreq_tile_128: begin
         wreq_vld = 1;
         tile_en = 1;
         offset_inc = 4;
       end
       wreq_tile_256: begin
         wreq_vld = 1;
         tile_en = 1;
         offset_inc = 8;
       end
       wreq_ie_tile: begin
         wreq_vld = 1;
         tile_en = 1;
         offset_inc = 0;
       end
`endif
       wreq_struct: begin
         wreq_vld = 1;
         wreq_vld2 = 1;
         tile_en = 1;
         offset_inc = 0;
	 last_data = wreq_count <= 4;
	 last_data2 = wreq_count <= 4;
       end
       wreq_ie_linear: begin
         wreq_vld = 1;//wreq_size != 0 || wreq_ie_cnt < 8;
         wreq_vld2 = 1;
         tile_en = 0;
         offset_inc = 0;
	 last_data = wreq_size == 1 && (structt ? wreq_ie_cnt == 15 : wreq_ie_cnt == 7);
	 last_data2 = wreq_size == 1 && (structt ? wreq_ie_cnt == 15 : wreq_ie_cnt == 7);
       end
       default: begin
         wreq_vld = 0;
         wreq_vld2 = 0;
         tile_en = 0;
         offset_inc = 0;
	 last_data = 0;
	 last_data2 = 0;
       end
     endcase

   reg [`DMA_TILE_PARAM_TX_SIZE-1:0]                               x;
   reg [`DMA_TILE_PARAM_TY_SIZE-1:0]                               y;
   reg [`DMA_TILE_PARAM_TZ_SIZE-1:0]                               z;

   wire [`DMA_TILE_PARAM_TX_SIZE-1:0]                              inc_x;
   wire                                                            inc_y;
   wire                                                            inc_z;
   wire [`DMA_TILE_PARAM_TX_SIZE-1:0]                              next_x;
   wire [`DMA_TILE_PARAM_TY_SIZE-1:0]                              next_y;
   wire [`DMA_TILE_PARAM_TZ_SIZE-1:0]                              next_z;

   wire [`DMA_TILE_PARAM_TPITCH_SIZE+3:0]                           tile_pitch_max;
   wire [`DMA_TILE_PARAM_HEIGHT_SIZE-1:0]                           tile_height_max;
   assign           tile_pitch_max = {tpitch + 1, 3'd0} - 1;
   assign         tile_height_max = height;


   assign inc_x = {offset_inc, 2'd0} >> psize;
   wire [`DMA_TILE_PARAM_TX_SIZE:0]                                x_plus_inc_x; //extend 1 bit
   assign x_plus_inc_x = x + inc_x;
   assign inc_y = (!subwin) & (x_plus_inc_x > tile_pitch_max);
   wire [`DMA_TILE_PARAM_TY_SIZE:0]                                y_plus_inc_y; //extend 1 bit
   assign y_plus_inc_y = y + inc_y;
   assign inc_z = (!subwin) & (y_plus_inc_y > tile_height_max);

   assign next_x = |offset_inc ? (inc_y ? `DMA_TILE_PARAM_TX_SIZE'd0 : (x + inc_x)) : x;
   assign next_y = |offset_inc ? (inc_z ? `DMA_TILE_PARAM_TY_SIZE'd0 : (y + inc_y)) : y;
   assign next_z = |offset_inc ? (z + inc_z) : z;


   reg [2:0]                                                       next_x_in_tile;
   reg [2:0]                                                       next_y_in_tile;
   reg [2:0]                                                       next_z_in_tile;
   reg [2:0]                                                       x_in_tile;
   reg [2:0]                                                       y_in_tile;
   reg [2:0]                                                       z_in_tile;
   reg [`DMA_TILE_PARAM_TY_SIZE-4:0]                               tile_y;
   reg [`DMA_TILE_PARAM_TZ_SIZE-1:0]                               tile_z;

   wire xthick_tile = (tmode == 8 || tmode == 14);
   wire thick_8bpp_ie = ie && !structt && (nondisp == 4) && (psize == 0); 

   assign req_cnt_max = 
`ifdef THICK_TILE_EN
                        nondisp == 4 && t2t ? xthick_tile ? (6'd2 << (psize+3)) : (6'd2 << (psize+2)) : // 8x8x8/8x8x4 Thick tile
			thick_8bpp_ie ? 4'd4 :	//thick tile 8bpp only handle 128b data each z-surface
`endif
                        (6'd2 << psize); //2 for 8bpp, 4 for 16bpp, 8 for 32bpp, 16 for 64bpp, 32 for 128bpp

   assign next_req_cnt = req_cnt_q + 1;

`ifdef SDMA__UTC_L1__1
   assign req_cnt = (wreq_state==wreq_ie_tile && rtr && (next_req_cnt == req_cnt_max) && (!broadcast || !dma_utc_l1_enable || broadcast_done)) ? 0 :
                    (wreq_state==wreq_ie_tile && rtr && (!broadcast || !dma_utc_l1_enable || broadcast_done)) ? (next_req_cnt) :
                    req_cnt_q;
`else //SDMA__UTC_L1__1
   assign req_cnt = (wreq_state==wreq_ie_tile && rtr && (req_cnt_q == req_cnt_max-1)) ? 5'd0 :
                    (wreq_state==wreq_ie_tile && rtr) ? (req_cnt_q + 1) :
                    req_cnt_q;
`endif


   always @ (posedge clk)
     begin
        if(~resetb) begin
           req_cnt_q <= 5'd0;
           wreq_state_d <= 0;
           wreq_delta   <= 0;
           wreq_dst2_info<= 0;
        end
        else begin
           req_cnt_q <= req_cnt;
           wreq_state_d <= wreq_state;
	if (bc_mode)
           wreq_delta <= taddr1[`CMD_ADDR_SIZE-1:5] - taddr[`CMD_ADDR_SIZE-1:5]; 
           wreq_dst2_info <= {tci1,vol1,pol1,swap1,hash1};
        end
     end

   // coord in tile normal T2T
   always @ * begin
      next_z_in_tile = 3'b000;
      case({nondisp[2:0], psize})
        //disp
        6'b000_000: begin next_x_in_tile = 3'b000;                            next_y_in_tile = {req_cnt[0],2'b00};                end
        6'b000_001: begin next_x_in_tile = 3'b000;                            next_y_in_tile = {req_cnt[1],req_cnt[0],1'b0};      end
        6'b000_010: begin next_x_in_tile = {req_cnt[0],2'b00};                next_y_in_tile = {req_cnt[2],req_cnt[1],1'b0};      end
        6'b000_011: begin next_x_in_tile = {req_cnt[1],req_cnt[0],1'b0};      next_y_in_tile = {req_cnt[3],req_cnt[2],1'b0};      end
        6'b000_100: begin next_x_in_tile = {req_cnt[2],req_cnt[1],req_cnt[0]};next_y_in_tile = {req_cnt[4],req_cnt[3],1'b0};      end
        //non-disp
        6'b001_000,6'b010_000: begin next_x_in_tile = 3'b000;                  next_y_in_tile = {req_cnt[0],2'b00};                end
        6'b001_001,6'b010_001: begin next_x_in_tile = {req_cnt[0],2'b00};      next_y_in_tile = {req_cnt[1],2'b00};                end
        6'b001_010,6'b010_010: begin next_x_in_tile = {req_cnt[1],2'b00};      next_y_in_tile = {req_cnt[2],req_cnt[0],1'b0};      end
        6'b001_011,6'b010_011: begin next_x_in_tile = {req_cnt[2],req_cnt[0],1'b0};  next_y_in_tile = {req_cnt[3],req_cnt[1],1'b0};      end
        6'b001_100,6'b010_100: begin next_x_in_tile = {req_cnt[3],req_cnt[1],1'b0};  next_y_in_tile = {req_cnt[4],req_cnt[2],req_cnt[0]};end
`ifdef THICK_TILE_EN	//thick tile IE only handlw one z surface 
        6'b100_000: if (t2t) {next_z_in_tile[2], next_y_in_tile[2], next_x_in_tile[2], next_z_in_tile[1], next_z_in_tile[0], next_y_in_tile[1], next_x_in_tile[1], next_y_in_tile[0], next_x_in_tile[0]} = {req_cnt[3:0], 5'b0};
        	    else     {next_y_in_tile[2], next_x_in_tile[2], next_y_in_tile[1], next_x_in_tile[1], next_y_in_tile[0], next_x_in_tile[0]} = {req_cnt[1:0], 4'b0};	//req_cnt menas 128b
        6'b100_001: if (t2t) {next_z_in_tile[2], next_y_in_tile[2], next_x_in_tile[2], next_z_in_tile[1], next_z_in_tile[0], next_y_in_tile[1], next_x_in_tile[1], next_y_in_tile[0], next_x_in_tile[0]} = {req_cnt[4:0], 4'b0};
        	    else     {next_y_in_tile[2], next_x_in_tile[2], next_y_in_tile[1], next_x_in_tile[1], next_y_in_tile[0], next_x_in_tile[0]} = {req_cnt[1:0], 4'b0};
        6'b100_010: if (t2t) {next_z_in_tile[2], next_y_in_tile[2], next_x_in_tile[2], next_z_in_tile[1], next_y_in_tile[1], next_z_in_tile[0], next_x_in_tile[1], next_y_in_tile[0], next_x_in_tile[0]} = {req_cnt[5:0], 3'b0};
        	    else     {next_y_in_tile[2], next_x_in_tile[2], next_y_in_tile[1], next_x_in_tile[1], next_y_in_tile[0], next_x_in_tile[0]} = {req_cnt[2:0], 3'b0};
        6'b100_011: if (t2t) {next_z_in_tile[2], next_y_in_tile[2], next_x_in_tile[2], next_z_in_tile[1], next_y_in_tile[1], next_x_in_tile[1], next_z_in_tile[0], next_y_in_tile[0], next_x_in_tile[0]} = {req_cnt[6:0], 2'b0};
                    else     {next_y_in_tile[2], next_x_in_tile[2], next_y_in_tile[1], next_x_in_tile[1], next_y_in_tile[0], next_x_in_tile[0]} = {req_cnt[3:0], 2'b0};
        6'b100_100: if (t2t) {next_z_in_tile[2], next_y_in_tile[2], next_x_in_tile[2], next_z_in_tile[1], next_y_in_tile[1], next_x_in_tile[1], next_z_in_tile[0], next_y_in_tile[0], next_x_in_tile[0]} = {req_cnt[7:0], 1'b0};
        	    else     {next_y_in_tile[2], next_x_in_tile[2], next_y_in_tile[1], next_x_in_tile[1], next_y_in_tile[0], next_x_in_tile[0]} = {req_cnt[4:0], 1'b0};
`endif
	//rotate tile
        5'b11_000:  {next_x_in_tile[2], next_x_in_tile[0], next_x_in_tile[1],
                          next_y_in_tile[2], next_y_in_tile[1], next_y_in_tile[0]} = {req_cnt, 5'b0};
        5'b11_001:  {next_x_in_tile[2], next_x_in_tile[1], next_x_in_tile[0],
                          next_y_in_tile[2], next_y_in_tile[1], next_y_in_tile[0]} = {req_cnt, 4'b0};
        5'b11_010:  {next_x_in_tile[2], next_x_in_tile[1], next_y_in_tile[2],
                          next_x_in_tile[0], next_y_in_tile[1], next_y_in_tile[0]} = {req_cnt, 3'b0};
        5'b11_011:  {next_y_in_tile[2], next_x_in_tile[2], next_x_in_tile[1],
                          next_y_in_tile[1], next_x_in_tile[0], next_y_in_tile[0]} = {req_cnt, 2'b0};
        default:  begin next_x_in_tile = 3'b000;  next_y_in_tile = 3'b000;  next_z_in_tile = 3'b000; end
      endcase
   end

   wire   inc_tile_x;
`ifdef SDMA__UTC_L1__1
   assign inc_tile_x = (next_req_cnt == req_cnt_max) && (!broadcast || !dma_utc_l1_enable || broadcast_done);
`else
   assign   inc_tile_x  = (req_cnt_q == req_cnt_max-1) ? 1 : 0;
`endif

   always @ (posedge clk)
     begin
        if(~resetb) begin
         x <= 0; y <= 0; z <= 0;
           x_in_tile <= 3'b0;
           y_in_tile <= 3'b0;
           z_in_tile <= 3'b0;
           tile_x_max<= 0;
           tile_x <= {(`DMA_TILE_PARAM_TX_SIZE-3){1'b0}};
           tile_y <= {(`DMA_TILE_PARAM_TY_SIZE-3){1'b0}};
           tile_z <= {`DMA_TILE_PARAM_TZ_SIZE{1'b0}};
        end
        else if (split_vld) begin
         x <= tx;
         y <= ty;
         z <= tz;
           x_in_tile <= 0;
           y_in_tile <= 0;
           z_in_tile <= 0;
           tile_x_max    <= t2t ? tx[`DMA_TILE_PARAM_TX_SIZE-1:3] + dx[`DMA_TILE_PARAM_TX_SIZE-1+1:3] :
				tx[`DMA_TILE_PARAM_TX_SIZE-1:3] + tilenum; 
           tile_x    <= tx[`SDMA_PKT_COPY_TILED_BC_X_SIZE-1:3];
           tile_y    <= ty[`SDMA_PKT_COPY_TILED_BC_Y_SIZE-1:3];
           tile_z    <= tz[`SDMA_PKT_COPY_TILED_BC_Z_SIZE-1:3];
        end
        else if (rtr) begin
         x <= next_x;
         y <= next_y;
         z <= next_z;
           x_in_tile <= next_x_in_tile;
           y_in_tile <= next_y_in_tile;
           z_in_tile <= next_z_in_tile;
           tile_x    <= tile_x + inc_tile_x;
           tile_y    <= tile_y;
           tile_z    <= tile_z;
        end
     end

   wire[`DMA_TILE_TADDR_SIZE+8-1:0] tile_taddr_out;
   wire [`DMA_TILE_PARAM_TX_SIZE-1:0] addr_calc_x = wreq_state==wreq_ie_tile ? {tile_x, x_in_tile} : x;
   wire [`DMA_TILE_PARAM_TY_SIZE-1:0] addr_calc_y = wreq_state==wreq_ie_tile ? {tile_y, y_in_tile} : y;
   wire [`DMA_TILE_PARAM_TZ_SIZE-1:0] addr_calc_z = wreq_state==wreq_ie_tile && nondisp == 4 && t2t ? 
                                                    (xthick_tile ? {tile_z, z_in_tile[2:0]} : {tile_z, z[2], z_in_tile[1:0]}) : z;
`ifdef SDMA__UTC_L1__1
   wire dst2 = video && y[0] || broadcast && dma_utc_l1_enable && broadcast_done;
`else
   wire dst2 = video && y[0];
`endif
   //wire dst2 = video && y[0];
   wire [`DMA_TILE_TADDR_SIZE-1:0] baseAddr256B = dst2 || t2t ? taddr1[`CMD_ADDR_SIZE-1:8] : taddr[`CMD_ADDR_SIZE-1:8];
   wire                           h0_reg_en = rtr;// && tile_vld;
   wire                           h1_reg_en = rtr;// && tile_vld_d1;
   wire                           h2_reg_en = rtr;// && tile_vld_d2;
   wire                           h3_reg_en = rtr;// && tile_vld_d3;
   wire tile_vld = rtr & tile_en & !structt;

   generic_addr_calc2_sdma_gc #(.RI(1), .RO(1), .PAD_STAGES(0), .COLLAPSABLE_PIPE_STAGE(0))
        dma_wr_addr_calc(
                                                      //outputs
              .address(tile_taddr_out), //byte address: 40bit
              .busy (),
              //inputs
              .x(addr_calc_x),                         //coordinate x
              .y(addr_calc_y),                         //coordinate y
              .z(addr_calc_z[10:0]),                         //coordinate z
              .s(3'd0),                      //sample number
              .baseAddr256B(baseAddr256B),      //base addr 256B align
              .log2_pixelsize(psize),            //0: 8bpp, 1:16bpp, 2:32bpp, 3:64bpp, 4:128bpp
              .log2_samples(2'd0),                   //the number of samples
              .pitch_tile_max(tpitch),            //in tiles, pitch_element/8 -1
              .slice_tile_max(tslice),            //in tiles, slice_tiles -1
              .micro_tile_mode(nondisp),     //3:thick tile, 2:depth, 1:non-disp, 0:displayable
              .pipe_config(pipecfg),
              .array_mode(tmode),                //as defines
              .tile_split(tsplit),                //0: 64B split, 1: 128B split, 2: 256B ...
              .bank_width(bwidth),                //0: 1tile, 1:2tiles, 2:4tiles, 3:8tiles
              .bank_height(bheight),               //0: 1tile, 1:2tiles, 2:4tiles, 3:8tiles
              .macro_tile_aspect(aspect),         //0: 1tile, 1:2tiles, 2:4tiles, 3:8tiles
              .num_banks(numbank),               //0: 2banks,1:4banks, 2:8banks, 3:16banks
              .pipe_interleave_size(dma_addr_conf_pipe_interleave_size),      //0: 256B, 1:512B, 2:1024B, 3:2048B, 4:4096B
              .sclk(clk),
	          .srst (~resetb),                      
	          .iRTS (tile_vld),                      
	          .oRTR (),                      
	          .oRTS (),                      
	          .iRTR (rtr),                      
              .input_en (tile_vld),
              .output_en(),
	          .pass_thru_data_in(),
	          .pass_thru_data_out()       
              );
   

   //pipeline down register

   wire end_of_cmd = last_data && last_sub_cp_vld;
   wire end_of_cmd2 = last_data2 && last_sub_cp_vld;
   reg[255:0] pipe0, pipe1, pipe2, pipe3;
   reg pipe0_vld, pipe1_vld, pipe2_vld, pipe3_vld, pipe_vld;
   reg pipe0_vld2, pipe1_vld2, pipe2_vld2, pipe3_vld2, pipe_vld2;

   wire[4:0] psize_y = structt ? taddr[4:2] : {psize[1:0], z[0], y[1:0]};
   reg [31:0] tile_mask;

   always@(posedge clk)
     if(!resetb) begin
       pipe0 <= 0; pipe1 <= 0; pipe2 <= 0; pipe3 <= 0;
       {pipe0_vld, pipe1_vld, pipe2_vld, pipe3_vld} <= 0;
       {pipe0_vld2, pipe1_vld2, pipe2_vld2, pipe3_vld2} <= 0;
     end
     else if(rtr) begin
       pipe0 <= {wreq_addr, (dst2? {tci1,vol1,pol1,swap1,hash1}:{tci,vol,pol,swap,hash}), broadcast, structt, ie, voidd, //wreq_addr must be first
		tile_en, crc, aes, granu, psize_y, nondisp, t2l, t2t, vmid, priv, tile_mask, thick_8bpp_ie, end_of_cmd2, end_of_cmd};
       pipe1 <= pipe0;
       pipe2 <= pipe1;
       pipe3 <= pipe2;
       {pipe0_vld, pipe1_vld, pipe2_vld, pipe3_vld} <= {wreq_vld, pipe0_vld, pipe1_vld, pipe2_vld};
       {pipe0_vld2, pipe1_vld2, pipe2_vld2, pipe3_vld2} <= {wreq_vld2, pipe0_vld2, pipe1_vld2, pipe2_vld2};
     end

   reg pipe_end_of_cmd;
   reg pipe_end_of_cmd2;
   reg pipe_ie, pipe_priv, pipe_hash, pipe_aes, pipe_crc, pipe_tile, pipe_t2l, pipe_t2t, pipe_broadcast, pipe_struct, pipe_thick_8bpp_ie;
   reg pipe_tci, pipe_vol, pipe_pol;
   reg[`SDMA0_GFX_RB_CNTL_RB_VMID_SIZE-1:0] pipe_vmid;
   reg[2:0] pipe_void;
   reg[1:0] pipe_swap, pipe_granu;
   reg[`SDMA_PKT_COPY_TILED_BC_MIT_MODE_SIZE-1:0] pipe_nondisp;
   reg[`CMD_ADDR_SIZE-1+4:0] pipe_addr;   //extend 4 bits for hash
   reg[31:0] pipe_tile_mask;
   wire[15:0] pipe_mask;
   reg[4:0] pipe_psize_y;
   wire[15:0] tile_mask_2B;
   reg [15:0] tile_mask_voidfix;
   integer i,j;
   wire pipe_z_lo = pipe_psize_y[2];

   always @* begin
    pipe_vld = pipe3_vld;
    {pipe_addr, pipe_tci, pipe_vol, pipe_pol, pipe_swap, pipe_hash, pipe_broadcast, pipe_struct, pipe_ie, pipe_void, pipe_tile, //pipe_addr 4b+
	pipe_crc, pipe_aes, pipe_granu, pipe_psize_y, pipe_nondisp, pipe_t2l, pipe_t2t,  
	pipe_vmid, pipe_priv, pipe_tile_mask, pipe_thick_8bpp_ie, pipe_end_of_cmd2, pipe_end_of_cmd} = pipe3;
    if(pipe_tile && !pipe_struct) begin
	pipe_addr[`DMA_TILE_TADDR_SIZE+8-1:0] = tile_taddr_out;
    end
    else if(pipe_hash) 
              for(i=8; i<8+8; i=i+1) begin
                if(i<8+dma_channel_bits && |dma_channel_bits && |dma_channel_xor && dma_channel_bits<5)
                  for(j=1; j<=7; j=j+1) begin
                    if(j<=dma_channel_xor)
                      pipe_addr[i] = pipe_addr[i] ^ pipe_addr[i+j*dma_channel_bits];
                  end
                else if(i>=8+dma_channel_bits && i<8+dma_channel_bits+dma_bank_bits &&
                        |dma_bank_bits && |dma_bank_xor && dma_bank_bits<5 && dma_channel_bits<5)
                  for(j=1; j<=7; j=j+1) begin
                    if(j<=dma_bank_xor)
                      pipe_addr[i] = pipe_addr[i] ^ pipe_addr[i+j*dma_bank_bits];
                  end
              end
   end

   assign pipe_mask = !pipe_ie && pipe_tile && !pipe_t2t ? tile_mask_voidfix : 
                                                           pipe_thick_8bpp_ie && !pipe_t2l? (pipe_z_lo ? 16'hff00 : 16'h00ff) : 16'hffff;	//thick 8bpp handle 128b

   wire[1:0] last_size = |wreq_count[`CMD_COUNT_SIZE-1+1:4] || granu==1 && wreq_count[3] ? 0 : wreq_count[3:2];

   always @* begin
      if(structt && granu==0)  //32 granu
          tile_mask = 32'hf << {psize_y[2:0], 2'b0};
      else
`ifdef THICK_TILE_EN
      if(nondisp == 2'b01)
`else
      if(|nondisp)
`endif
        casez (psize_y)
          //8bpp
          5'b00_?00: case(last_size[0])
			0: tile_mask = 32'b0000_0000_0011_0011_0000_0000_0011_0011;
			1: tile_mask = 32'b0000_0000_0000_0000_0000_0000_0011_0011;
			endcase
          5'b00_?01: case(last_size[0])
                        0: tile_mask = 32'b0000_0000_1100_1100_0000_0000_1100_1100;
                        1: tile_mask = 32'b0000_0000_0000_0000_0000_0000_1100_1100;
			endcase
          5'b00_?10: case(last_size[0])
                        0: tile_mask = 32'b0011_0011_0000_0000_0011_0011_0000_0000;
                        1: tile_mask = 32'b0000_0000_0000_0000_0011_0011_0000_0000;
			endcase
          5'b00_?11: case(last_size[0])
                        0: tile_mask = 32'b1100_1100_0000_0000_1100_1100_0000_0000;
                        1: tile_mask = 32'b0000_0000_0000_0000_1100_1100_0000_0000;
			endcase
          //16bpp
          5'b01_?00: case(last_size[0])
                        0: tile_mask = 32'b0000_0000_0000_0000_0000_1111_0000_1111;
                        1: tile_mask = 32'b0000_0000_0000_0000_0000_0000_0000_1111;
			endcase
          5'b01_?01: case(last_size[0])
                        0: tile_mask = 32'b0000_0000_0000_0000_1111_0000_1111_0000;
                        1: tile_mask = 32'b0000_0000_0000_0000_0000_0000_1111_0000;
			endcase
          5'b01_?10: case(last_size[0])
                        0: tile_mask = 32'b0000_1111_0000_1111_0000_0000_0000_0000;
                        1: tile_mask = 32'b0000_0000_0000_1111_0000_0000_0000_0000;
			endcase
          5'b01_?11: case(last_size[0])
                        0: tile_mask = 32'b1111_0000_1111_0000_0000_0000_0000_0000;
                        1: tile_mask = 32'b0000_0000_1111_0000_0000_0000_0000_0000;
			endcase
          //32bpp
          5'b10_??0: case(last_size)
                        0: tile_mask = 32'b0000_0000_1111_1111_0000_0000_1111_1111;
                        1: tile_mask = 32'b0000_0000_0000_0000_0000_0000_0000_1111;
                        2: tile_mask = 32'b0000_0000_0000_0000_0000_0000_1111_1111;
                        3: tile_mask = 32'b0000_0000_0000_1111_0000_0000_1111_1111;
			endcase
          5'b10_??1: case(last_size)
                        0: tile_mask = 32'b1111_1111_0000_0000_1111_1111_0000_0000;
                        1: tile_mask = 32'b0000_0000_0000_0000_0000_1111_0000_0000;
                        2: tile_mask = 32'b0000_0000_0000_0000_1111_1111_0000_0000;
                        3: tile_mask = 32'b0000_1111_0000_0000_1111_1111_0000_0000;
			endcase
          //64bpp
          5'b11_??0: case(last_size)
                        0: tile_mask = 32'b0000_0000_0000_0000_1111_1111_1111_1111;
                        1: tile_mask = 32'b0000_0000_0000_0000_0000_0000_0000_1111;
                        2: tile_mask = 32'b0000_0000_0000_0000_0000_0000_1111_1111;
                        3: tile_mask = 32'b0000_0000_0000_0000_0000_1111_1111_1111;
			endcase
          5'b11_??1: case(last_size)
                        0: tile_mask = 32'b1111_1111_1111_1111_0000_0000_0000_0000;
                        1: tile_mask = 32'b0000_0000_0000_1111_0000_0000_0000_0000;
                        2: tile_mask = 32'b0000_0000_1111_1111_0000_0000_0000_0000;
                        3: tile_mask = 32'b0000_1111_1111_1111_0000_0000_0000_0000;
			endcase
          //128bpp   256b granularity not need rd_data_src_mask
          //6'b100_xxx: tile_mask = 32'b1111_1111_1111_1111_1111_1111_1111_1111;
          default: tile_mask = 0;
        endcase
   //Thick tile
`ifdef THICK_TILE_EN
     else if (nondisp == 3'b100)
       casez (psize_y)
       //8bpp
       6'b000_000: tile_mask = 32'b0000_0000_0000_0000_0000_0000_0011_0011;
       6'b000_001: tile_mask = 32'b0000_0000_0000_0000_0000_0000_1100_1100;
       6'b000_010: tile_mask = 32'b0000_0000_0000_0000_0011_0011_0000_0000;
       6'b000_011: tile_mask = 32'b0000_0000_0000_0000_1100_1100_0000_0000;
       6'b000_100: tile_mask = 32'b0000_0000_0011_0011_0000_0000_0000_0000;
       6'b000_101: tile_mask = 32'b0000_0000_1100_1100_0000_0000_0000_0000;
       6'b000_110: tile_mask = 32'b0011_0011_0000_0000_0000_0000_0000_0000;
       6'b000_111: tile_mask = 32'b1100_1100_0000_0000_0000_0000_0000_0000;
       //16bpp3
       6'b001_?00: case(last_size[0]) 
		0: tile_mask = 32'b0000_0000_0000_0000_0000_1111_0000_1111;
		1: tile_mask = 32'b0000_0000_0000_0000_0000_0000_0000_1111;
		   endcase
       6'b001_?01: case(last_size[0]) 
                0: tile_mask = 32'b0000_0000_0000_0000_1111_0000_1111_0000;
                1: tile_mask = 32'b0000_0000_0000_0000_0000_0000_1111_0000;
		   endcase
       6'b001_?10: case(last_size[0]) 
                0: tile_mask = 32'b0000_1111_0000_1111_0000_0000_0000_0000;
                1: tile_mask = 32'b0000_0000_0000_1111_0000_0000_0000_0000;
		   endcase
       6'b001_?11: case(last_size[0]) 
                0: tile_mask = 32'b1111_0000_1111_0000_0000_0000_0000_0000;
                1: tile_mask = 32'b0000_0000_1111_0000_0000_0000_0000_0000;
		   endcase
       //32bpp
       6'b010_??0: case(last_size)
		0: tile_mask = 32'b0000_0000_1111_1111_0000_0000_1111_1111;
		1: tile_mask = 32'b0000_0000_0000_0000_0000_0000_0000_1111;
		2: tile_mask = 32'b0000_0000_0000_0000_0000_0000_1111_1111;
		3: tile_mask = 32'b0000_0000_0000_1111_0000_0000_1111_1111;
		   endcase
       6'b010_??1: case(last_size)
                0: tile_mask = 32'b1111_1111_0000_0000_1111_1111_0000_0000;
                1: tile_mask = 32'b0000_0000_0000_0000_0000_1111_0000_0000;
                2: tile_mask = 32'b0000_0000_0000_0000_1111_1111_0000_0000;
                3: tile_mask = 32'b0000_1111_0000_0000_1111_1111_0000_0000;
 		   endcase
       //64bpp
       6'b011_??0: case(last_size)
                     0: tile_mask = 32'b0000_0000_0000_0000_1111_1111_1111_1111;
                     1: tile_mask = 32'b0000_0000_0000_0000_0000_0000_0000_1111;
                     2: tile_mask = 32'b0000_0000_0000_0000_0000_0000_1111_1111;
                     3: tile_mask = 32'b0000_0000_0000_0000_0000_1111_1111_1111;
     		   endcase
       6'b011_??1: case(last_size)
                     0: tile_mask = 32'b1111_1111_1111_1111_0000_0000_0000_0000;
                     1: tile_mask = 32'b0000_0000_0000_1111_0000_0000_0000_0000;
                     2: tile_mask = 32'b0000_0000_1111_1111_0000_0000_0000_0000;
                     3: tile_mask = 32'b0000_1111_1111_1111_0000_0000_0000_0000;
     		   endcase
       //128bpp
       //6'b100_???: tile_mask = 32'b1111_1111_1111_1111_1111_1111_1111_1111;
       default: tile_mask = 0;
       endcase
`endif
      else  //displayable
        if(granu==2)       //128 granu
          case (psize_y[0])
            1'b0: case(last_size)
                        0: tile_mask = 32'h0000ffff;
                        1: tile_mask = 32'h0000000f;
                        2: tile_mask = 32'h000000ff;
                        3: tile_mask = 32'h00000fff;
                  endcase
            1'b1: case(last_size)
                        0: tile_mask = 32'hffff0000;
                        1: tile_mask = 32'h000f0000;
                        2: tile_mask = 32'h00ff0000;
                        3: tile_mask = 32'h0fff0000;
                  endcase
          endcase
        else if(granu==1)  //64 granu
          case (psize_y[1:0])
            2'b00: tile_mask = last_size[0] ? 32'h0000000f : 32'h000000ff;
            2'b01: tile_mask = last_size[0] ? 32'h000f0000 : 32'h00ff0000;
            2'b10: tile_mask = last_size[0] ? 32'h00000f00 : 32'h0000ff00;
            2'b11: tile_mask = last_size[0] ? 32'h0f000000 : 32'hff000000;
          endcase
        else if(granu==0)  //32 granu
          tile_mask = 32'hf << {psize_y[2:0], 2'b0};
        else
          tile_mask = 32'hffffffff;
      if(psize==4 && !structt) tile_mask = 32'hffffffff;
   end

   assign tile_mask_2B = {pipe_tile_mask[30], pipe_tile_mask[28], pipe_tile_mask[26], pipe_tile_mask[24], pipe_tile_mask[22], pipe_tile_mask[20], pipe_tile_mask[18],
        pipe_tile_mask[16], pipe_tile_mask[14], pipe_tile_mask[12], pipe_tile_mask[10], pipe_tile_mask[8], pipe_tile_mask[6], pipe_tile_mask[4], pipe_tile_mask[2], pipe_tile_mask[0] };

   reg[5:0] void_cnt1, void_cnt;

   integer    i1,n1;
   always @* begin
      void_cnt1 = 0;
      void_cnt = 0;
	for(i1=0; i1<=15; i1=i1+1) begin
	   void_cnt1 = void_cnt1 + tile_mask_2B[i1];
	   if(void_cnt1 == pipe_void*2)
	     void_cnt = i1;
	end
   end

   always @* begin
      tile_mask_voidfix = tile_mask_2B;
      if(|pipe_void)
	for(n1=0; n1<=15; n1=n1+1) begin
	   if(n1 <= void_cnt)
	     tile_mask_voidfix[n1] = 0;
	end
   end


   ////////////////////////////////////////////////////////////////////
`ifdef SDMA__UTC_L1__1
reg           wreq_mc_wr_rts2;

assign wreq_mc_wr_atc_rts = wreq_mc_wr_rts2 /*&& rtr*/;
`endif	

   always @(posedge clk)
     begin
        if (~resetb) begin
`ifdef SDMA__UTC_L1__1
           //wreq_mc_wr_atc_rts  <= 1'b0;
           wreq_mc_wr_rts2  <= 1'b0;
	   wreq_mc_wr_atc_eoc <= 1'b0;
`endif
           wreq_mc_wr_rts  <= 1'b0;
           wreq_mc_wr_mask <= `DMA_MC_RDREQ_MASK_SIZE'd0;
           wreq_mc_wr_addr <= 0;
           wreq_mc_wr_swap <= 0; 
           wreq_mc_wr_priv <= 0;
           wreq_mc_wr_vmid <= 0;
	   wreq_mc_wr_aes  <= 0;
	   wreq_mc_wr_crc  <= 0;
	   wreq_mc_wr_tci  <= 0;
	   wreq_mc_wr_vol  <= 0;
	   wreq_mc_wr_pol  <= 0;
	   wreq_mc_wr_broadcast  <= 0;
	   wreq_mc_wr_eoc <= 0;
        end
        else begin
           wreq_mc_wr_rts   <= pipe_vld && rtr;
`ifdef SDMA__UTC_L1__1
           wreq_mc_wr_rts2	<= pipe3_vld2 && rtr;
        if (dma_utc_l1_enable ? rtr : 1'b1) begin
           //wreq_mc_wr_atc_rts   <= pipe3_vld2;
	   wreq_mc_wr_atc_eoc <= pipe_end_of_cmd2;
`endif
           //wreq_mc_wr_rts   <= pipe_vld;
           wreq_mc_wr_addr  <= pipe_addr[`CMD_ADDR_SIZE-1:5];
           wreq_mc_wr_mask  <= pipe_mask;
           wreq_mc_wr_swap  <= pipe_swap;
           wreq_mc_wr_priv  <= pipe_priv;
           wreq_mc_wr_vmid  <= pipe_vmid;
	   wreq_mc_wr_aes  <= pipe_aes;
	   wreq_mc_wr_crc  <= pipe_crc;
	   wreq_mc_wr_tci  <= pipe_tci;
	   wreq_mc_wr_vol  <= pipe_vol;
	   wreq_mc_wr_pol  <= pipe_pol;
	   wreq_mc_wr_broadcast  <= pipe_broadcast;
	   wreq_mc_wr_eoc <= pipe_end_of_cmd;
`ifdef SDMA__UTC_L1__1
	   end
`endif
        end // else: !if(~resetb)
     end // always @ (posedge clk)

assign wreq_rtr = wreq_state == 0 && rtr && !split_vld;
//assign wreq_rtr = wreq_state == 0 && (split_ie ? split_vld == 0 && wreq_vld == 0 : rtr && !(split_vld && ie));


   always@(posedge clk)
     if(!resetb)
       dma_ce_wreq_idle <= 1;
     else
       dma_ce_wreq_idle <= wreq_state == 0 && !split_vld && !wreq_mc_wr_rts && {pipe0_vld, pipe1_vld, pipe2_vld, pipe3_vld}==0;


endmodule

