//--------------------------------------------------------------------------
//
// $Id: //depot/xenos/devel/parts_lib/src/common/dum_mem_p2.v#1 $
//
// $Change: 112877 $
//
//
// Notes:	DFF library including plain dff, dff w/reset, dff w/clk enable,
//  		and dff w/reset and clk enable.  Reset causes rval to be loaded
//		into the flop
// Copyright:	Trade secret of ATI Technologies, Inc.
//		Copyright 2001-2002, ATI Technologies, Inc., (unpublished) 
//
//		All rights reserved.  This notice is intended as a precaution against
//		inadvertent publication and does not imply publication or any waiver
//		of confidentiality.  The year included in the foregoing notice is the
//		year of creation of the work.
//
//--------------------------------------------------------------------------
`ifdef SIMULATION
`include "msgio_defines.vh"
`endif

module dum_mem_mask 
( 
  iWEN, 
  iMEW, 
  iWADR,
  iD, 
  iWMSK,
  iWCLK, 

  iMER, 
  iRADR, 
  oQ,
  iRCLK 
);

  parameter ADDR_WIDTH = 7;        
  parameter DATA_WIDTH = 103;
  parameter WORDS = 72;
  parameter DEBUG = 1;
   
  input iRCLK, iWCLK, iMER, iMEW, iWEN;
  input [ADDR_WIDTH - 1:0] iWADR, iRADR;
  input [DATA_WIDTH - 1:0] iD;
  input [DATA_WIDTH - 1:0] iWMSK;
   
  output [DATA_WIDTH - 1:0] oQ;

`ifdef SYN
   parameter WORDS2 = 2;
   parameter ADDR_WIDTH2 = 1;        
   wire [ADDR_WIDTH2 - 1:0] iWADR2 = ^iWADR;
   wire [ADDR_WIDTH2 - 1:0] iRADR2 = ^iRADR;
`else
   parameter WORDS2 = WORDS;
   parameter ADDR_WIDTH2 = ADDR_WIDTH;        
   wire [ADDR_WIDTH2 - 1:0] iWADR2 = iWADR;
   wire [ADDR_WIDTH2 - 1:0] iRADR2 = iRADR;
`endif
   
  reg    [DATA_WIDTH - 1:0] oQ;
   
  reg    [DATA_WIDTH - 1:0] bram [WORDS2 - 1:0];
  
  // write   
  always @( posedge iWCLK ) begin
    if ( iWEN & iMEW ) begin
       bram[ iWADR2 ] <= ( iD & iWMSK ) | ( bram[ iWADR2 ] & ~iWMSK );
`ifdef SIMULATION
       if (DEBUG && (iWADR2 >= WORDS2))
         `MSGIO_DISPLAY("ERROR: Writing address that does not exist (iWADR = %x) in %m at %t", iWADR2, $time);
`endif
    end
  end
   
  // read

   // If a read/write addr collision happens, bits being written (in iWMSK) are read as X.  Bits not being written are returned. 
   integer i;
   wire [DATA_WIDTH-1:0] Q_read = bram[iRADR2] ;
   reg [DATA_WIDTH-1:0]  Q_collision;
   always @(/*AUTOSENSE*/Q_read or iWMSK) begin
      for (i=0; i<DATA_WIDTH; i=i+1) begin
         Q_collision[i] = iWMSK[i] ? 1'bX : Q_read[i];
      end
   end
   
   always @(posedge iRCLK) begin
  	if (iMER)
	  if ( (iRADR2 == iWADR2) & (iMEW & iWEN) ) begin
             oQ <= Q_collision;
`ifdef SIMULATION
	     if (DEBUG && (iRADR2!=0)) begin
                if (~iWMSK ==0)
                  `MSGIO_DISPLAY("ERROR: reading and writing same location with full write mask (ADR = %x, WMSK = %x) in %m at %t", iRADR2, iWMSK, $time);
                else
                  `MSGIO_DISPLAY("WARNING: reading and writing same location (ADR = %x, WMSK = %x) in %m at %t", iRADR2, iWMSK, $time);
             end
	     if (DEBUG && (iRADR2 >= WORDS2)) begin
                `MSGIO_DISPLAY("ERROR: Reading address that does not exist (iRADR = %x) in %m at %t", iRADR2, $time);
             end
`endif
	  end 
          else
            oQ <= Q_read;
   end // always @ (posedge iRCLK)

   // for debug:
`ifdef SIMULATION
   wire [DATA_WIDTH-1:0] ram_data0 = bram['h0];
   wire [DATA_WIDTH-1:0] ram_data1 = bram['h1];
   wire [DATA_WIDTH-1:0] ram_data2 = bram['h2];
   wire [DATA_WIDTH-1:0] ram_data3 = bram['h3];
   wire [DATA_WIDTH-1:0] ram_data4 = bram['h4];
   wire [DATA_WIDTH-1:0] ram_data5 = bram['h5];
   wire [DATA_WIDTH-1:0] ram_data6 = bram['h6];
   wire [DATA_WIDTH-1:0] ram_data7 = bram['h7];
   wire [DATA_WIDTH-1:0] ram_data8 = bram['h8];
   wire [DATA_WIDTH-1:0] ram_data9 = bram['h9];
   wire [DATA_WIDTH-1:0] ram_dataa = bram['ha];
   wire [DATA_WIDTH-1:0] ram_datab = bram['hb];
   wire [DATA_WIDTH-1:0] ram_datac = bram['hc];
   wire [DATA_WIDTH-1:0] ram_datad = bram['hd];
   wire [DATA_WIDTH-1:0] ram_datae = bram['he];
   wire [DATA_WIDTH-1:0] ram_dataf = bram['hf];
   wire [DATA_WIDTH-1:0] ram_data10 = bram['h10];
   wire [DATA_WIDTH-1:0] ram_data11 = bram['h11];
   wire [DATA_WIDTH-1:0] ram_data12 = bram['h12];
   wire [DATA_WIDTH-1:0] ram_data13 = bram['h13];
   wire [DATA_WIDTH-1:0] ram_data14 = bram['h14];
   wire [DATA_WIDTH-1:0] ram_data15 = bram['h15];
   wire [DATA_WIDTH-1:0] ram_data16 = bram['h16];
   wire [DATA_WIDTH-1:0] ram_data17 = bram['h17];
   wire [DATA_WIDTH-1:0] ram_data18 = bram['h18];
   wire [DATA_WIDTH-1:0] ram_data19 = bram['h19];
   wire [DATA_WIDTH-1:0] ram_data1a = bram['h1a];
   wire [DATA_WIDTH-1:0] ram_data1b = bram['h1b];
   wire [DATA_WIDTH-1:0] ram_data1c = bram['h1c];
   wire [DATA_WIDTH-1:0] ram_data1d = bram['h1d];
   wire [DATA_WIDTH-1:0] ram_data1e = bram['h1e];
   wire [DATA_WIDTH-1:0] ram_data1f = bram['h1f];
`endif
   
endmodule // dum_mem_mask














