//--------------------------------------------------------------------------
//
// $Id$
//
// $Change$
//
//
// Notes:	DFF library including plain dff, dff w/reset, dff w/clk enable,
//  		and dff w/reset and clk enable.  Reset causes rval to be loaded
//		into the flop
// Copyright:	Trade secret of ATI Technologies, Inc.
//		Copyright 2001-2002, ATI Technologies, Inc., (unpublished) 
//
//		All rights reserved.  This notice is intended as a precaution against
//		inadvertent publication and does not imply publication or any waiver
//		of confidentiality.  The year included in the foregoing notice is the
//		year of creation of the work.
//
//--------------------------------------------------------------------------
`ifdef SIMULATION
`include "msgio_defines.vh"
`endif

module dum_mem_p2 
( 
   iWEN, 
   iMEW, 
   iWADR,
   iD, 
   iWCLK, 

   iMER, 
   iRADR, 
   oQ,
   iRCLK 
);

   parameter ADDR_WIDTH = 2;        
   parameter DATA_WIDTH = 2;
   parameter WORDS = 4;
   parameter DEBUG = 1;
   
   input iRCLK, iWCLK, iMER, iMEW, iWEN;
   input [ADDR_WIDTH - 1:0] iWADR, iRADR;
   input [DATA_WIDTH - 1:0] iD;
   
   output [DATA_WIDTH - 1:0] oQ;

`ifdef SYN
   parameter WORDS2 = 2;
   parameter ADDR_WIDTH2 = 1;        
   wire [ADDR_WIDTH2 - 1:0] iWADR2 = ^iWADR;
   wire [ADDR_WIDTH2 - 1:0] iRADR2 = ^iRADR;
`else
   parameter WORDS2 = WORDS;
   parameter ADDR_WIDTH2 = ADDR_WIDTH;        
   wire [ADDR_WIDTH2 - 1:0] iWADR2 = iWADR;
   wire [ADDR_WIDTH2 - 1:0] iRADR2 = iRADR;
`endif
   
   reg    [DATA_WIDTH - 1:0] oQ;
   
   reg    [DATA_WIDTH - 1:0] bram [WORDS2 - 1:0];

`ifdef SIMULATION
   reg mem_written;
   initial mem_written = 1'b0;
`endif

   // write
   integer i;
   always @(posedge iWCLK) begin
      if (iWADR2 < WORDS2 && iWEN && iMEW)
	bram[iWADR2] <= iD;
// VCS coverage off
`ifdef SIMULATION
      if (iWEN && iMEW && DEBUG && (iWADR2 >= WORDS2))
        `MSGIO_DISPLAY("ERROR: Writing address that does not exist (iWADR = %x) in %m at %t", iWADR2, $time);

      if ((iWEN == 1'b1) && (iMEW == 1'b1))
        mem_written = 1'b1;
`endif
// VCS coverage on
   end
   
   
   // read 
   always @(posedge iRCLK) begin
      if (iMER)
// VCS coverage off
	if ( (iRADR2 == iWADR2) & (iMEW & iWEN) ) begin
	   oQ <= {DATA_WIDTH{1'bx}};
`ifdef SIMULATION
	   if (DEBUG && mem_written)
             `MSGIO_DISPLAY("WARNING: reading and writing same location (ADR = %x) in %m at %t", iRADR2, $time);
	   if (DEBUG && (iRADR2 >= WORDS2))
              `MSGIO_DISPLAY("ERROR: Reading address that does not exist (iRADR = %x) in %m at %t", iRADR2, $time);
`endif
// VCS coverage on
	end else
          oQ <= bram[iRADR2]; 
   end
endmodule 






