`include "global_features.vh"

`define EDC_CTRL_REG_WIDTH 32
`define PSM_CURPSM_LENGTH 14

module edc_droop_detection (
  // outputs
  edc_throttle_level,
  edc_dbus,
  psm_edc_dd_stat,
  // inputs
  Cpl_GFXCLK,
  resetb,
  psm_edc_dd_cntl,
  cgcg_en,
  psm_counter,
  dft_clk_gate_ctrl,
  dft_shift_clk
);

  // outputs
  output  [2:0]                  	edc_throttle_level;
  output  [31:0]                    edc_dbus;
  output  [31:0]                    psm_edc_dd_stat;
  // inputs
  input                             Cpl_GFXCLK;
  input                             resetb;
  input                             cgcg_en;
  input                             dft_shift_clk;
  input   [`EDC_CTRL_REG_WIDTH-1:0] psm_edc_dd_cntl;
  input   [`PSM_CURPSM_LENGTH-1:0]  psm_counter;
  input   [1:0]                     dft_clk_gate_ctrl;


//***************************************************************************//
//                          EDC droop_level detection                        //
//***************************************************************************//

  reg 		psm_edc_dd_en;
  reg 		sw_rst;
  reg		cg_override;
  reg[13:0] edc_droop_threshold;
  reg[4:0]	edc_droop_level_index;
  reg		avg_psm_sel;
  reg		dbg_en;

  gfx_cgtt_local_dft ucgtt_local_dft(
    .Cpl_GFXCLK(Cpl_GFXCLK),
    .core_busy(1'b1),
    .oclk(oCpl_GFXCLK),
    .dft_clk_gate_ctrl(dft_clk_gate_ctrl),
    .dft_shift_clk(dft_shift_clk)
  );

  always @(posedge oCpl_GFXCLK) begin
  	if(~resetb)
		begin
			sw_rst <= 'd0;
			psm_edc_dd_en <= 'd0;
			cg_override <= 'd0;
			edc_droop_threshold <= 'd0;
			edc_droop_level_index <= 'd0;
			avg_psm_sel	<= 'd0;
			dbg_en <= 'd0;
		end
	else
		begin
			sw_rst 				<= psm_edc_dd_cntl[0];
			psm_edc_dd_en			<= psm_edc_dd_cntl[1];
			cg_override				<= psm_edc_dd_cntl[2];
			edc_droop_threshold		<= psm_edc_dd_cntl[16:3];
			edc_droop_level_index	<= psm_edc_dd_cntl[21:17];
			avg_psm_sel				<= psm_edc_dd_cntl[22];
			dbg_en					<= psm_edc_dd_cntl[23];
		end
  end


   reg [14+8:0] edc_rollingdroopdelta_q;
   reg psm_edc_dd_en_d1;

   wire edc_droop_clk_gated;
   wire edc_rst = sw_rst | ~resetb;

   wire edc_droop_clk_en;
   assign edc_droop_clk_en = edc_rst | psm_edc_dd_en | cg_override;

   gfx_cgtt_local_dft edc_droop_clkgater (
      .Cpl_GFXCLK  			(Cpl_GFXCLK              ),
      .core_busy   			(edc_droop_clk_en     ),
      .oclk  				(edc_droop_clk_gated  ),
      .dft_clk_gate_ctrl	(dft_clk_gate_ctrl),
      .dft_shift_clk		(dft_shift_clk)
   );

	reg[13:0]	psm_counter_q;
	always @(posedge edc_droop_clk_gated) begin
		if(edc_rst) begin
			psm_counter_q <= 'd0;
		end else begin
			psm_counter_q <= psm_counter;
		end
	end

   wire edc_droop_level_enable = psm_edc_dd_en & psm_edc_dd_en_d1;
   wire signed [14:0] s_edc_droop_threshold = {1'b0,edc_droop_threshold};
   wire signed [14:0] s_edc_psm_counter = edc_droop_level_enable ? {1'b0,psm_counter_q} : 'd0;
   wire signed [14:0] s_edc_droop_delta = edc_droop_level_enable ? (s_edc_droop_threshold - s_edc_psm_counter) : 'd0;
   wire signed [14+9:0] s_edc_rollingdroopdelta_q = avg_psm_sel ? 24'd0 : {1'b0,edc_rollingdroopdelta_q};
   wire signed [14+10:0] s_edc_rollingdroopdelta = {{10{s_edc_droop_delta[14]}},s_edc_droop_delta} + {1'b0,s_edc_rollingdroopdelta_q};
   wire [14+8:0] edc_rollingdroopdelta_d = (~s_edc_rollingdroopdelta[24] & s_edc_rollingdroopdelta[23]) ? 23'h7f_ffff : //ovefflow clamp 
                                                    s_edc_rollingdroopdelta[24] ? 23'h00_0000 : //< 0 clamp
                                                    s_edc_rollingdroopdelta[14+8:0];
                                                      
   always @(posedge oCpl_GFXCLK) begin
     if(edc_rst)
       psm_edc_dd_en_d1 <= 1'b0;
     else
       psm_edc_dd_en_d1 <= psm_edc_dd_en;
   end
                                                      
   always @(posedge edc_droop_clk_gated) begin
     if(edc_rst)
       edc_rollingdroopdelta_q <= 'd0;
     else
       edc_rollingdroopdelta_q <= edc_rollingdroopdelta_d;
   end


   reg edc_droop_level_overflow;
   integer k;
   always @*
    begin
        edc_droop_level_overflow = 1'b0;
        for (k = 0; k < 23; k=k+1) begin
          if(k > edc_droop_level_index + 2)
           edc_droop_level_overflow = (edc_droop_level_overflow | edc_rollingdroopdelta_q[k]) & edc_droop_level_enable;
         end
     end

	wire[2:0] edc_droop_level;
   assign edc_droop_level[0] = edc_droop_level_enable ? (edc_droop_level_overflow ? 1'b1 : edc_rollingdroopdelta_q[edc_droop_level_index]) : 1'b0;
   assign edc_droop_level[1] = edc_droop_level_enable ? (edc_droop_level_overflow ? 1'b1 : edc_rollingdroopdelta_q[edc_droop_level_index+1]) : 1'b0;
   assign edc_droop_level[2] = edc_droop_level_enable ? (edc_droop_level_overflow ? 1'b1 : edc_rollingdroopdelta_q[edc_droop_level_index+2]) : 1'b0;


reg[2:0] edc_throttle_level_q;
always @(posedge edc_droop_clk_gated) begin
	if(edc_rst)
		edc_throttle_level_q <= 'd0;
	else
		edc_throttle_level_q <= edc_droop_level;
end
assign edc_throttle_level = edc_throttle_level_q;

assign psm_edc_dd_stat[2:0] 	= edc_throttle_level_q;
assign psm_edc_dd_stat[3]		= edc_droop_level_overflow;
assign psm_edc_dd_stat[17:4]	= psm_counter_q;
assign psm_edc_dd_stat[31:18]	= edc_rollingdroopdelta_q[13:0];

assign edc_dbus[2:0] 	= dbg_en ? edc_throttle_level_q : 3'b0;
assign edc_dbus[3]		= dbg_en ? edc_droop_level_overflow : 1'b0;
assign edc_dbus[15:4]	= dbg_en ? psm_counter_q[11:0] : 12'b0;
assign edc_dbus[31:16]	= dbg_en ? edc_rollingdroopdelta_q[15:0] : 16'b0;



endmodule
