//                              -*- Mode: Verilog -*-
////////////////////////////////////////////////////////////////////////////
/*
 *
 *  Trade secret of ATI Technologies, Inc.
 *  Copyright 2005, ATI Technologies, Inc., (unpublished)
 *
 *  All rights reserved.  This notice is intended as a precaution against
 *  inadvertent publication and does not imply publication or any waiver
 *  of confidentiality.  The year included in the foregoing notice is the
 *  year of creation of the work.
 *
 */
////////////////////////////////////////////////////////////////////////////
//
// Filename    : fuse_if_drclk.v
//
////////////////////////////////////////////////////////////////////////////
// Perforce revision     : : #2 n// Perforce changelist   : : 2885168 n// Perforce date         : : 2017/06/27 n// Perforce Last edit by : : couyang n////////////////////////////////////////////////////////////////////////////
module fuse_if_drclk_gc10 (
ick       , // input clock
resetb    , // input synchronous reset
targClkDiv, // input target clock divide value (50% of cycle)
drClkEn   , // input enable output clock (drClk)
drClk     , // output clock
drClkR    , // output marker in ick, marking drClk rising  edge
drClkF	    // output marker in ick, marking drClk falling edge
);

   input       ick       ; // input clock
   input       resetb    ; // input synchronous reset
   input [4:0] targClkDiv; // input target clock divide value (50% of cycle)
   input       drClkEn   ; // input enable output clock (drClk)
   output      drClk     ; // output clock
   output      drClkR    ; // output marker in ick, marking drClk rising  edge
   output      drClkF    ; // output marker in ick, marking drClk falling edge

   reg  [4:0]  ickCnt    ;
   wire [4:0]  ickCntMax ; // This counter describes the 50% of cycle
   wire        ickCntClr ;
   reg  [2:1]  rising_d  ; // Delay marker because latency through dfx_ckgate 3; marker set 1 cycle before edge
   reg  [2:1]  fallingd  ; // Delay marker because latency through dfx_ckgate 3; marker set 1 cycle before edge
   reg         drClkReg  ;
   wire        drClkNxt  ;
   wire        drClkEnInt; // need to ensure the clock always stop low

   assign      drClkEnInt = drClkEn | drClkReg;
   assign      drClkNxt   = drClkEnInt & ((ickCnt[4:0] == ickCntMax[4:0]) ? ~drClkReg : drClkReg);

// assign              ickCntMax[4:0] = targClkDiv[4:0];
   assign              ickCntMax[4:0] = targClkDiv[4:0] ^ 5'b00011 ; // flip the LSB so default fuse value of {5 {1'b0}} is div by 8 ((3+1)*2)

   assign              ickCntClr      = (ickCnt[4:0] == ickCntMax[4:0]) | ~drClkEnInt;
   assign              drClkR         =  rising_d[2];
   assign              drClkF         =  fallingd[2];

  always @(posedge ick)
    if (~resetb) begin
       ickCnt     [4:0]  <= { 5 {1'b0}};
       drClkReg          <=      1'b0  ;
       rising_d   [2:1]  <= { 2 {1'b0}};
       fallingd   [2:1]  <= { 2 {1'b0}};
    end else begin
       ickCnt[4:0]   <= (ickCnt[4:0] + 5'b00001) & {5 {~ickCntClr}};
       rising_d[2:1] <= {rising_d[1], {~drClkReg &  drClkNxt}}; // detect rising  edge
       fallingd[2:1] <= {fallingd[1], { drClkReg & ~drClkNxt}}; // detect falling edge
       drClkReg      <=  drClkNxt;
    end // else: !if(~resetb)

   // Delay the BR control by 1 cycle because we want edge on posedge
   dfx_ckgate_gc10 I_drClk (.clkIn(ick), .clkEn_AR(drClkReg), .clkEn_BR(drClkReg), .clkOut(drClk));

endmodule // fuse_if_drclk_gc10

