//                              -*- Mode: Verilog -*-
////////////////////////////////////////////////////////////////////////////
/*
 *
 *  Trade secret of ATI Technologies, Inc.
 *  Copyright 2005, ATI Technologies, Inc., (unpublished)
 *
 *  All rights reserved.  This notice is intended as a precaution against
 *  inadvertent publication and does not imply publication or any waiver
 *  of confidentiality.  The year included in the foregoing notice is the
 *  year of creation of the work.
 *
 */
////////////////////////////////////////////////////////////////////////////
//
// Filename    : fuse_if_drclkf.v
//
////////////////////////////////////////////////////////////////////////////
// Perforce revision     : : #2 n// Perforce changelist   : : 2885168 n// Perforce date         : : 2017/06/27 n// Perforce Last edit by : : couyang n////////////////////////////////////////////////////////////////////////////
module fuse_if_drclkf_gc10 (
ick       , // input clock
resetb    , // input synchronous reset
targClkDiv, // input target clock divide value (50% of cycle)
drClkEn   , // input enable output clock (drClk)
drClk     , // output clock
drClkR    , // output marker in ick, marking drClk rising  edge
drClkF	    // output marker in ick, marking drClk falling edge
);

   input       ick       ; // input clock
   input       resetb    ; // input synchronous reset
   input [4:0] targClkDiv; // input target clock divide value (50% of cycle)
   input       drClkEn   ; // input enable output clock (drClk)
   output      drClk     ; // output clock
   output      drClkR    ; // output marker in ick, marking drClk rising  edge
   output      drClkF    ; // output marker in ick, marking drClk falling edge

   reg  [4:0]  ickCnt    ;
   wire [4:0]  ickCntMax ; // This counter describes the 50% of cycle
   wire        ickCntClr ;
   reg         drClkReg  ;
   wire        drClkNxt  ;
   wire        drClkEnInt; // need to ensure the clock always stop low

   assign      drClkEnInt = drClkEn | drClkReg;
   assign      drClkNxt   = drClkEnInt & ((ickCnt[4:0] == ickCntMax[4:0]) ? ~drClkReg : drClkReg);

   assign      ickCntMax[4:0] = targClkDiv[4:0] ^ 5'b00011 ; // flip the LSB so default fuse value of {5 {1'b0}} is div by 8 ((3+1)*2)

   assign      ickCntClr      = (ickCnt[4:0] == ickCntMax[4:0]) | ~drClkEnInt;
   assign      drClkR         = {~drClkReg &  drClkNxt}; // detect rising  edge
   assign      drClkF         = { drClkReg & ~drClkNxt}; // detect falling edge

  always @(posedge ick)
    if (~resetb) begin
       ickCnt     [4:0]  <= { 5 {1'b0}};
       drClkReg          <=      1'b0  ;
    end else begin
       ickCnt[4:0]   <= (ickCnt[4:0] + 5'b00001) & {5 {~ickCntClr}};
       drClkReg      <=  drClkNxt;
    end // else: !if(~resetb)

   aticlk_buf  d0nt_cclk (.Y(drClk), .A(drClkReg));

endmodule // fuse_if_drclkf_gc10

