//                              -*- Mode: Verilog -*-
////////////////////////////////////////////////////////////////////////////
/*
 *
 *  Trade secret of ATI Technologies, Inc.
 *  Copyright 2005, ATI Technologies, Inc., (unpublished)
 *
 *  All rights reserved.  This notice is intended as a precaution against
 *  inadvertent publication and does not imply publication or any waiver
 *  of confidentiality.  The year included in the foregoing notice is the
 *  year of creation of the work.
 *
 */
////////////////////////////////////////////////////////////////////////////
//
// Filename    : fuse_if_drsmsg.v
//
////////////////////////////////////////////////////////////////////////////
// Perforce revision     : : #2 n// Perforce changelist   : : 2885168 n// Perforce date         : : 2017/06/27 n// Perforce Last edit by : : couyang n////////////////////////////////////////////////////////////////////////////
`include "fuse_if_gc10.vh"
module fuse_if_drsmsg_gc10 (
ipclk ,
ick   ,
wrckLeaf,
fclkEn,
drClkF,
resetb,
RSMU_RDFT_sms_fuse_grp_data,
Async_Rstb_Scan_Override           , // JIRA SMS-61 (http://jira.amd.com/browse/sms-61)
Async_Rstb_Scan_Override_Sel       , // JIRA DECDFXIP41-70
// ==========================================================================
// SMSg interface
// Begin --------------------------------------------------------------------
smsg_ef_req                        , // (Per SMSg) - Every toggle is a new request
smsg_ef_addr                       , // (Per SMSg) - 7 bit byte address
smsg_ef_ready                      , // (Per SMSg) - every toggle is ack for request
smsg_ef_byte                       , // (Per SMSg) - 8 bit efuse data, valid at smsg_ack toggle

smsg_sfp_ready                     , // (Per SMSg) - SMSg ready to accept command
smsg_sfp_fail                      , // (Per SMSg) - SMSg BIST or BISR fail status
smsg_sfp_request                   , // (Per SMSg) - Trigger signal for bihr, bist, and bisr
smsg_sfp_resetb                    , // (Per SMSg) - Signal to reset SFP of SMSg (does not clear fuse)

smsg_bihr                          , // launch Built In Hard Repair
smsg_bist                          , // launch Built In Self Test
smsg_bisr                          , // launch Built In Soft Repair
smsg_fclken                        , // Clock gater term for functional clock, synchroonizer in smsg
// End ----------------------------------------------------------------------

// ==========================================================================
// interface with fseCtl Main controller
// Begin --------------------------------------------------------------------
fseCtl                             , //               Main statemachine in fuse_if
fseCtlNxt                          , // Next state of Main statemachine in fuse_if
smsg_ef_req_toggle                 , // Toggie to indicate SMSg is ready to receive more fuse
smsg_ef_ready_toggle               , // Toggle to indicate SMSg is ready to accept more commands
sfp_ready                          , // ick version of smsg_sfp_ready
sfp_request                        , // ick version of smsg_sfp_request
// End ----------------------------------------------------------------------

// ==========================================================================
// interface with fuse_if_status_tdr
// Begin --------------------------------------------------------------------
fuse_if_sms_bihr_done              , // SMSg BIHR done to fuse_if_status_tdr
fuse_if_sms_bist_done              , // SMSg BIST done to fuse_if_status_tdr
fuse_if_sms_bist_fail                // SMSg BIST fail to fuse_if_status_tdr
// End ----------------------------------------------------------------------
                       );
input          ipclk ;
input          ick   ;
input          wrckLeaf;
input          fclkEn;
input          drClkF;
input          resetb;
input   [31:0] RSMU_RDFT_sms_fuse_grp_data;
input          Async_Rstb_Scan_Override           ; // JIRA SMS-61 (http://jira.amd.com/browse/sms-61
input          Async_Rstb_Scan_Override_Sel       ; // JIRA DECDFXIP41-70
// ==========================================================================
// SMSg interface
// Begin --------------------------------------------------------------------
input          smsg_ef_req                        ; // (Per SMSg) - Every toggle is a new request
input   [ 6:0] smsg_ef_addr                       ; // (Per SMSg) - 7 bit byte address
output         smsg_ef_ready                      ; // (Per SMSg) - every toggle is ack for request
output  [ 7:0] smsg_ef_byte                       ; // (Per SMSg) - 8 bit efuse data, valid at smsg_ack toggle

input          smsg_sfp_ready                     ; // (Per SMSg) - SMSg ready to accept command
input          smsg_sfp_fail                      ; // (Per SMSg) - SMSg BIST or BISR fail status
output         smsg_sfp_request                   ; // (Per SMSg) - Trigger signal for bihr, bist, and bisr
output         smsg_sfp_resetb                    ; // (Per SMSg) - Signal to reset SFP of SMSg (does not clear fuse)

output         smsg_bihr                          ; // launch Built In Hard Repair
output         smsg_bist                          ; // launch Built In Self Test
output         smsg_bisr                          ; // launch Built In Soft Repair
output         smsg_fclken                        ; // Clock gater term for functional clock, synchroonizer in smsg
// End ----------------------------------------------------------------------

// ==========================================================================
// interface with fseCtl Main controller
// Begin --------------------------------------------------------------------
input   [ 4:0] fseCtl                             ; //               Main statemachine in fuse_if
input   [ 4:0] fseCtlNxt                          ; // Next state of Main statemachine in fuse_if
output         smsg_ef_req_toggle                 ; // Toggie to indicate SMSg is ready to receive more fuse
output         smsg_ef_ready_toggle               ; // Toggle to indicate SMSg is ready to accept more commands
output         sfp_ready                          ; // ick version of smsg_sfp_ready
output         sfp_request                        ; // ick version of smsg_sfp_request
// End ----------------------------------------------------------------------

// ==========================================================================
// interface with fuse_if_status_tdr
// Begin --------------------------------------------------------------------
output         fuse_if_sms_bihr_done              ; // SMSg BIHR done to fuse_if_status_tdr
output         fuse_if_sms_bist_done              ; // SMSg BIST done to fuse_if_status_tdr
output         fuse_if_sms_bist_fail              ; // SMSg BIST fail to fuse_if_status_tdr
// End ----------------------------------------------------------------------

   wire      cdc_smsg_ef_req   ;
   wire      cdc_smsg_sfp_ready;
   wire      cdc_smsg_sfp_fail ;

   reg       sfp_resetb      ;
   reg       sfp_resetbNxt   ;
   reg       sfp_fail        ;
   reg       sfp_requestReg  ;
   reg       sfp_bihr_done   ;
   reg       sfp_bist_done   ;
   reg [1:0] ef_byteIdx      ;
   reg       ef_ready        ;
   reg       ef_ready_d1     ;
   reg       ef_req          ;
   reg       ef_req_d1       ;
   reg       sfp_bihr        ;
   reg       sfp_bist        ;
   reg       sfp_readyReg    ;
   reg       sfp_readyReg_d1 ;
   wire      sfp_readyRegR   ; // Detect rising edge of sfp_readyReg

   reg       sfp_requestNxt;
   reg [1:0] ef_byteIdxNxt ;
   reg       ef_readyNxt   ;
   reg       sfp_bihrNxt   ;
   reg       sfp_bistNxt   ;
   wire      sfp_resetb_gated  ;
   //dfx_mrk_or_gc10 I_sfp_resetb_gated (.Y(sfp_resetb_gated), .A(sfp_resetb), .B(Async_Rstb_Scan_Override));
   ati_mux2 I_sfp_resetb_muxed (.s(Async_Rstb_Scan_Override_Sel), .z(sfp_resetb_gated), .i0(sfp_resetb), .i1(Async_Rstb_Scan_Override));

   // ==========================================================================
   // SMSg interface
   // Begin --------------------------------------------------------------------
   dfx_sync_async_rst_gc10  I_cdc_smsg_ef_req    (.CK(ick     ), .rstB( sfp_resetb_gated), .D(smsg_ef_req   ), .Q(cdc_smsg_ef_req), .SE(1'b0));
   //! unused smsg_ef_addr

   //---------------
   wire        ef_ready_wrck       ;
   reg         ef_ready_wrck_d1    ;
   wire        ef_ready_wrck_toggle;
   reg  [ 7:0] ef_byte_wrck        ;
   reg  [ 7:0] ef_byte_wrckNxt     ;
   wire [ 1:0] cdc_ef_byteIdx      ;
   wire [31:0] cdc_rsmu_data       ;
   dfx_sync_async_rst_gc10  I_smsg_ef_ready      (.CK(wrckLeaf), .rstB( sfp_resetb_gated), .D(ef_ready      ), .Q(ef_ready_wrck  ), .SE(1'b0));
   techind_cdcefpm #(.size (2),  .applyParams(1), .DestClkCnt(3), .CdcDestSetup(0)) mrk_ef_byteIdx (.A(ef_byteIdx[1:0]                  ), .EN(ef_ready_wrck_toggle), .DSTCLK(wrckLeaf), .Z(cdc_ef_byteIdx[1:0]));
   techind_cdcefpm #(.size (32), .applyParams(1), .DestClkCnt(3), .CdcDestSetup(0)) mrk_rsmu_data  (.A(RSMU_RDFT_sms_fuse_grp_data[31:0]), .EN(ef_ready_wrck_toggle), .DSTCLK(wrckLeaf), .Z(cdc_rsmu_data[31:0]));
   assign smsg_ef_ready = ef_ready_wrck_d1;
   assign ef_ready_wrck_toggle = ef_ready_wrck_d1 ^ ef_ready_wrck;
   assign smsg_ef_byte[7:0]    = ef_byte_wrck[7:0];
  always @(posedge wrckLeaf or negedge sfp_resetb_gated)
     if (~sfp_resetb_gated) begin
       ef_ready_wrck_d1  <= 1'b0;
        ef_byte_wrck[7:0] <= {8{1'b0}};
     end else begin
        ef_ready_wrck_d1  <= ef_ready_wrck;
        ef_byte_wrck[7:0] <= ef_ready_wrck_toggle ? ef_byte_wrckNxt[7:0] : ef_byte_wrck[7:0];
     end
   always @(*)
     case (cdc_ef_byteIdx[1:0])
       2'b00   : ef_byte_wrckNxt[7:0] = cdc_rsmu_data[ 7: 0];
       2'b01   : ef_byte_wrckNxt[7:0] = cdc_rsmu_data[15: 8];
       2'b10   : ef_byte_wrckNxt[7:0] = cdc_rsmu_data[23:16];
       2'b11   : ef_byte_wrckNxt[7:0] = cdc_rsmu_data[31:24];
       default : ef_byte_wrckNxt[7:0] = cdc_rsmu_data[ 7: 0];
     endcase // case (cdc_ef_byteIdx[1:0])
   //---------------

   dfx_sync_async_rst_gc10  I_cdc_smsg_sfp_ready (.CK(ick     ), .rstB( sfp_resetb_gated), .D(smsg_sfp_ready), .Q(cdc_smsg_sfp_ready), .SE(1'b0));
   dfx_sync_async_rst_gc10  I_smsg_sfp_requestReg(.CK(wrckLeaf), .rstB( sfp_resetb_gated), .D(sfp_requestReg), .Q(smsg_sfp_request  ), .SE(1'b0));
   dfx_sync_async_rst_gc10  I_smsg_fclken        (.CK(wrckLeaf), .rstB( sfp_resetb_gated), .D(fclkEn        ), .Q(smsg_fclken       ), .SE(1'b0));
   ati_buffer1 mrk_sfp_resetb (.A(sfp_resetb_gated), .Y(smsg_sfp_resetb));
   techind_cdcefpm #(.size (1), .applyParams(1), .DestClkCnt(3), .CdcDestSetup(0)) mrk_cdc_smsg_sfp_fail  (.A(smsg_sfp_fail ), .EN(sfp_readyRegR), .DSTCLK(ipclk), .Z(cdc_smsg_sfp_fail ));
   techind_cdcefpm #(.size (1), .applyParams(1), .DestClkCnt(3), .CdcDestSetup(0)) mrk_sfp_bihr (.A(sfp_bihr), .EN(smsg_sfp_request), .DSTCLK(wrckLeaf), .Z(smsg_bihr));
   techind_cdcefpm #(.size (1), .applyParams(1), .DestClkCnt(3), .CdcDestSetup(0)) mrk_sfp_bist (.A(sfp_bist), .EN(smsg_sfp_request), .DSTCLK(wrckLeaf), .Z(smsg_bist));

   assign smsg_bisr             = 1'b0        ; // BISR is not suported
   // End ----------------------------------------------------------------------

   assign sfp_request           = sfp_requestReg;
   assign smsg_ef_req_toggle    = (ef_req_d1   ^ ef_req  ) & resetb;
   assign smsg_ef_ready_toggle  = (ef_ready_d1 ^ ef_ready) & resetb;
   assign fuse_if_sms_bihr_done = sfp_bihr_done;
   assign fuse_if_sms_bist_done = sfp_bist_done;
   assign fuse_if_sms_bist_fail = sfp_fail;
   assign sfp_ready             = cdc_smsg_sfp_ready;
   assign sfp_readyRegR         = sfp_readyReg & ~sfp_readyReg_d1; // Detect rising edge of sfp_readyReg

   always @(*) begin
      case (fseCtlNxt[4:0])// DECDFXIP40-540 -- Bug - this should have been using fseCtlNxt[4:0]
        `FSECTL_BIHR_F0T_GC10 : ef_byteIdxNxt[1:0] = 2'b00;
        `FSECTL_BIHR_F1T_GC10 : ef_byteIdxNxt[1:0] = 2'b01;
        `FSECTL_BIHR_F2T_GC10 : ef_byteIdxNxt[1:0] = 2'b10;
        `FSECTL_BIHR_F3T_GC10 : ef_byteIdxNxt[1:0] = 2'b11;
        default          : ef_byteIdxNxt[1:0] = ef_byteIdx[1:0];
      endcase // case (fseCtlNxt[4:0])
      case (fseCtlNxt[4:0])
        `FSECTL_RST__LOW_GC10 : begin sfp_resetbNxt = 1'b0; ef_readyNxt =  ef_ready; sfp_requestNxt =  1'b0          ; sfp_bihrNxt = 1'b0; sfp_bistNxt = 1'b0; end
        `FSECTL_RST__DLY_GC10 : begin sfp_resetbNxt = 1'b1; ef_readyNxt =  ef_ready; sfp_requestNxt =  1'b0          ; sfp_bihrNxt = 1'b0; sfp_bistNxt = 1'b0; end
        `FSECTL_CONF_RDY_GC10 : begin sfp_resetbNxt = 1'b1; ef_readyNxt =  ef_ready; sfp_requestNxt =  1'b0          ; sfp_bihrNxt = 1'b0; sfp_bistNxt = 1'b0; end
        `FSECTL_CONF_LD0_GC10 : begin sfp_resetbNxt = 1'b1; ef_readyNxt =  ef_ready; sfp_requestNxt =  1'b0          ; sfp_bihrNxt = 1'b0; sfp_bistNxt = 1'b0; end
        `FSECTL_CONF_LD1_GC10 : begin sfp_resetbNxt = 1'b1; ef_readyNxt =  ef_ready; sfp_requestNxt =  1'b0          ; sfp_bihrNxt = 1'b0; sfp_bistNxt = 1'b0; end

        `FSECTL_BIHR_DL0_GC10 : begin sfp_resetbNxt = 1'b1; ef_readyNxt =  ef_ready; sfp_requestNxt =  1'b0          ; sfp_bihrNxt = 1'b1; sfp_bistNxt = 1'b0; end
        `FSECTL_BIHR_REQ_GC10 : begin sfp_resetbNxt = 1'b1; ef_readyNxt =  ef_ready; sfp_requestNxt =  1'b1          ; sfp_bihrNxt = 1'b1; sfp_bistNxt = 1'b0; end
        `FSECTL_BIHR_F0W_GC10 : begin sfp_resetbNxt = 1'b1; ef_readyNxt =  ef_ready; sfp_requestNxt =  1'b1          ; sfp_bihrNxt = 1'b1; sfp_bistNxt = 1'b0; end
        `FSECTL_BIHR_RRQ_GC10 : begin sfp_resetbNxt = 1'b1; ef_readyNxt =  ef_ready; sfp_requestNxt =  1'b1          ; sfp_bihrNxt = 1'b1; sfp_bistNxt = 1'b0; end
        `FSECTL_BIHR_RWF_GC10 : begin sfp_resetbNxt = 1'b1; ef_readyNxt =  ef_ready; sfp_requestNxt =  1'b1          ; sfp_bihrNxt = 1'b1; sfp_bistNxt = 1'b0; end
        `FSECTL_BIHR_F0T_GC10 : begin sfp_resetbNxt = 1'b1; ef_readyNxt = ~ef_ready; sfp_requestNxt =  1'b1          ; sfp_bihrNxt = 1'b1; sfp_bistNxt = 1'b0; end
        `FSECTL_BIHR_F1W_GC10 : begin sfp_resetbNxt = 1'b1; ef_readyNxt =  ef_ready; sfp_requestNxt =  1'b1          ; sfp_bihrNxt = 1'b1; sfp_bistNxt = 1'b0; end
        `FSECTL_BIHR_F1T_GC10 : begin sfp_resetbNxt = 1'b1; ef_readyNxt = ~ef_ready; sfp_requestNxt =  1'b1          ; sfp_bihrNxt = 1'b1; sfp_bistNxt = 1'b0; end
        `FSECTL_BIHR_F2W_GC10 : begin sfp_resetbNxt = 1'b1; ef_readyNxt =  ef_ready; sfp_requestNxt =  1'b1          ; sfp_bihrNxt = 1'b1; sfp_bistNxt = 1'b0; end
        `FSECTL_BIHR_F2T_GC10 : begin sfp_resetbNxt = 1'b1; ef_readyNxt = ~ef_ready; sfp_requestNxt =  1'b1          ; sfp_bihrNxt = 1'b1; sfp_bistNxt = 1'b0; end
        `FSECTL_BIHR_F3W_GC10 : begin sfp_resetbNxt = 1'b1; ef_readyNxt =  ef_ready; sfp_requestNxt =  1'b1          ; sfp_bihrNxt = 1'b1; sfp_bistNxt = 1'b0; end
        `FSECTL_BIHR_F3T_GC10 : begin sfp_resetbNxt = 1'b1; ef_readyNxt = ~ef_ready; sfp_requestNxt =  1'b1          ; sfp_bihrNxt = 1'b1; sfp_bistNxt = 1'b0; end
        `FSECTL_BIHR_DL1_GC10 : begin sfp_resetbNxt = 1'b1; ef_readyNxt =  ef_ready; sfp_requestNxt =  1'b0          ; sfp_bihrNxt = 1'b1; sfp_bistNxt = 1'b0; end
        `FSECTL_BIHRDONE_GC10 : begin sfp_resetbNxt = 1'b1; ef_readyNxt =  ef_ready; sfp_requestNxt =  1'b0          ; sfp_bihrNxt = 1'b1; sfp_bistNxt = 1'b0; end

        `FSECTL____PAUSE_GC10 : begin sfp_resetbNxt = 1'b1; ef_readyNxt =  ef_ready; sfp_requestNxt =  1'b0          ; sfp_bihrNxt = 1'b0; sfp_bistNxt = 1'b0; end

        `FSECTL_BIST_DL0_GC10 : begin sfp_resetbNxt = 1'b1; ef_readyNxt =  ef_ready; sfp_requestNxt =  1'b0          ; sfp_bihrNxt = 1'b0; sfp_bistNxt = 1'b1; end
        `FSECTL_BIST_REQ_GC10 : begin sfp_resetbNxt = 1'b1; ef_readyNxt =  ef_ready; sfp_requestNxt = ~sfp_requestReg; sfp_bihrNxt = 1'b0; sfp_bistNxt = 1'b1; end
        `FSECTL_BIST_ACK_GC10 : begin sfp_resetbNxt = 1'b1; ef_readyNxt =  ef_ready; sfp_requestNxt =  sfp_requestReg; sfp_bihrNxt = 1'b0; sfp_bistNxt = 1'b1; end
        `FSECTL_BIST_RUN_GC10 : begin sfp_resetbNxt = 1'b1; ef_readyNxt =  ef_ready; sfp_requestNxt =  sfp_requestReg; sfp_bihrNxt = 1'b0; sfp_bistNxt = 1'b1; end
        `FSECTL_BISTBURN_GC10 : begin sfp_resetbNxt = 1'b1; ef_readyNxt =  ef_ready; sfp_requestNxt =  1'b0          ; sfp_bihrNxt = 1'b0; sfp_bistNxt = 1'b1; end
        `FSECTL_BIST_DL1_GC10 : begin sfp_resetbNxt = 1'b1; ef_readyNxt =  ef_ready; sfp_requestNxt =  1'b0          ; sfp_bihrNxt = 1'b0; sfp_bistNxt = 1'b1; end
        `FSECTL_BISTDONE_GC10 : begin sfp_resetbNxt = 1'b1; ef_readyNxt =  ef_ready; sfp_requestNxt =  1'b0          ; sfp_bihrNxt = 1'b0; sfp_bistNxt = 1'b1; end

        `FSECTL_____IDEL_GC10 : begin sfp_resetbNxt = 1'b1; ef_readyNxt =  ef_ready; sfp_requestNxt =  1'b0          ; sfp_bihrNxt = 1'b0; sfp_bistNxt = 1'b0; end
        default          : begin sfp_resetbNxt = 1'b1; ef_readyNxt =  ef_ready; sfp_requestNxt =  1'b0          ; sfp_bihrNxt = 1'b0; sfp_bistNxt = 1'b0; end
      endcase // case (fseCtlNxt[4:0])
   end // always @ begin
   always @(posedge ick)
     if (~resetb) begin
        sfp_resetb      <=    1'b0  ;
        ef_req_d1       <=    1'b0  ;

        ef_ready        <=    1'b0  ;
        ef_ready_d1     <=    1'b0  ;
        ef_byteIdx[1:0] <= {2{1'b0}};
        sfp_requestReg  <=    1'b0  ;

        ef_req          <=    1'b0  ;
        sfp_readyReg    <=    1'b0  ;
        sfp_readyReg_d1 <=    1'b0  ;
        sfp_fail        <=    1'b0  ;
        sfp_bihr        <=    1'b0  ;
        sfp_bist        <=    1'b0  ;
        sfp_bihr_done   <=    1'b0  ;
        sfp_bist_done   <=    1'b0  ;
     end else begin // if (~resetb)
        sfp_resetb      <= sfp_resetbNxt;
        ef_req_d1       <= ef_req       ;
        ef_ready_d1     <= ef_ready     ;
        sfp_bihr_done   <=   (fseCtl[4:0] == `FSECTL_BIHRDONE_GC10)                  | sfp_bihr_done;
        sfp_bist_done   <= (((fseCtl[4:0] == `FSECTL_BIST_RUN_GC10) & sfp_readyRegR) | sfp_bist_done) &  // sticky, detect sms completed BIST
                             (fseCtl[4:0] != `FSECTL_BIST_REQ_GC10);                                     // Clear for Burn-in mode
        sfp_readyReg_d1 <= sfp_readyReg;
        sfp_fail        <= sfp_readyRegR ? cdc_smsg_sfp_fail : sfp_fail;
        // Only update output at posedge of drClk
        // Only sample input at negedge of drClk
        if (drClkF) begin
           // output related
           ef_ready        <= ef_readyNxt    ;
           ef_byteIdx[1:0] <= ef_byteIdxNxt[1:0];
           sfp_requestReg  <= sfp_requestNxt ;
           // Input related
           ef_req          <= cdc_smsg_ef_req   ;
           sfp_readyReg    <= cdc_smsg_sfp_ready;
           sfp_bihr        <= sfp_bihrNxt       ;
           sfp_bist        <= sfp_bistNxt       ;
        end else begin
           // output related
           ef_ready        <= ef_ready    ;
           ef_byteIdx[1:0] <= ef_byteIdx[1:0];
           sfp_requestReg  <= sfp_requestReg ;
           // Input related
           ef_req          <= ef_req      ;
           sfp_readyReg    <= sfp_readyReg;
           sfp_bihr        <= sfp_bihr    ;
           sfp_bist        <= sfp_bist    ;
        end
     end // else: !if(~resetb)

endmodule // fuse_if_drsmsg_gc10
