//                              -*- Mode: Verilog -*-
////////////////////////////////////////////////////////////////////////////
/*
 *
 *  Trade secret of ATI Technologies, Inc.
 *  Copyright 2005, ATI Technologies, Inc., (unpublished)
 *
 *  All rights reserved.  This notice is intended as a precaution against
 *  inadvertent publication and does not imply publication or any waiver
 *  of confidentiality.  The year included in the foregoing notice is the
 *  year of creation of the work.
 *
 */
////////////////////////////////////////////////////////////////////////////
//
// Filename    : fuse_if.v
// Description : Fuse interface module to connect between RSMU and/or latch array
//
////////////////////////////////////////////////////////////////////////////
// Perforce revision     : : #2 n// Perforce changelist   : : 2885168 n// Perforce date         : : 2017/06/27 n// Perforce Last edit by : : couyang n////////////////////////////////////////////////////////////////////////////
`include "fuse_if_gc10_features.vh"
`include "fuse_if_gc10.vh"



module fuse_if_gc10 (
ipclk                       , // local clock within IP
Scan_Mode                   , // Signal from scan_cntl to indicate we are in test mode
TEST_CLK_EN                 , // Gater override for test mode
Async_Rstb_Scan_Override    , // JIRA SMS-61 (http://jira.amd.com/browse/sms-61)
Async_Rstb_Scan_Override_Sel , // JIRA DECDFXIP41-70 
Async_sms_resetb            , // JIRA DECDFXIP41-60 (http://ontrack-internal.amd.com/browse/DECDFXIP41-60)
Async_do_not_bihr           , // JIRA DECDFXIP41-60 (http://ontrack-internal.amd.com/browse/DECDFXIP41-60)
// ==========================================================================
// Interface with RSMU
// --------------------------------------------------------------------------
// RSMU Fuse interface
// Begin --------------------------------------------------------------------
RSMU_RDFT_sms_fuse_grp_resetb      , //
RSMU_RDFT_sms_fuse_grp_run_MBIST   , //
RSMU_RDFT_sms_fuse_grp_run_BIHR    , //
RSMU_RDFT_sms_fuse_grp_valid       , //
RSMU_RDFT_sms_fuse_grp_data        , //
RDFT_RSMU_sms_fuse_grp_repair_busy , //
RDFT_RSMU_sms_fuse_grp_repair_done , //
RDFT_RSMU_sms_fuse_grp_bist_fail   , //
// End ----------------------------------------------------------------------

// ==========================================================================
// SMSg interface
BC1500_Wrck                        , // output Tck clock to SMSg & larr
wrckLeaf                           , // input  Tck clock at leaf level, with BC1500_Wrck as the root
// fuse interface with SMSg
// Signals are synchronous to BC1500_Wrck clock
// Begin --------------------------------------------------------------------
smsg_ef_req                        , // (Per SMSg) - Every toggle is a new request
smsg_ef_addr                       , // (Per SMSg) - 7 bit byte address
smsg_ef_ready                      , // (Per SMSg) - every toggle is ack for request
smsg_ef_byte                       , // (Per SMSg) - 8 bit efuse data, valid at smsg_ack toggle

smsg_sfp_ready                     , // (Per SMSg) - SMSg ready to accept command
smsg_sfp_fail                      , // (Per SMSg) - SMSg BIST or BISR fail status
smsg_sfp_request                   , // (Per SMSg) - Trigger signal for bihr, bist, and bisr
smsg_sfp_resetb                    , // (Per SMSg) - Signal to reset SFP of SMSg (does not clear fuse)

smsg_bihr                          , // launch Built In Hard Repair
smsg_bist                          , // launch Built In Self Test
smsg_bisr                          , // launch Built In Soft Repair
smsg_fclken                        , // Clock gater term for functional clock, synchroonizer in smsg
// End ----------------------------------------------------------------------

// ==========================================================================
// BIST interface with larr
// Begin --------------------------------------------------------------------
LARR_BIST_FAIL_Daisy_Out_0         , // Latch arrays BIST fail status (Valis only when DONE == 1'b1)
LARR_BIST_DONE_Daisy_Out_0         , // Latch arrays BIST done
LARR_BIST_smartmode_en             , // Trigger signal to lanunch latch arrays BIST
LARR_BIST_smartmode_rstb           , // Reset output to latch arrays
// End ----------------------------------------------------------------------

// TDR interface
// Begin --------------------------------------------------------------------
Tdr_Tck                  ,
Tdr_Tdi                  ,
Tdr_Shift                ,
Tdr_Capture              ,
Tdr_Update               ,
Tdr_Trstb                ,

Tdr_fuse_if_ate_mode_En  ,
Tdr_fuse_if_ate_mode_Tdo ,
Tdr_fuse_if_status_En    ,
Tdr_fuse_if_status_Tdo   ,
Tdr_fuse_if_setup_En     ,
Tdr_fuse_if_setup_Tdo
// End ----------------------------------------------------------------------

);
// ==========================================================================
// Begin --------------------------------------------------------------------
// NO_SMSG_LARR == 0 : There are SMSg and LARR    connected to fuse_if
// NO_SMSG_LARR == 2 : Only               LARR is connected to fuse_if; no SMSg
// NO_SMSG_LARR == 1 : Only      SMSg          is connected to fuse_if; no LARR
parameter NO_SMSG_LARR = 0;
// End ----------------------------------------------------------------------

input          ipclk                       ; // local clock within IP
input          Scan_Mode                   ; // Signal from scan_cntl to indicate we are in test mode
input          TEST_CLK_EN                 ; // Gater override for test mode
input          Async_Rstb_Scan_Override    ; // JIRA SMS-61 (http://jira.amd.com/browse/sms-61)
input          Async_Rstb_Scan_Override_Sel; // JIRA DECDFXIP41-70
input          Async_sms_resetb            ; // JIRA DECDFXIP41-60 (http://ontrack-internal.amd.com/browse/DECDFXIP41-60)
input          Async_do_not_bihr           ; // JIRA DECDFXIP41-60 (http://ontrack-internal.amd.com/browse/DECDFXIP41-60)
// ==========================================================================
// Interface with RSMU
// --------------------------------------------------------------------------
// RSMU Fuse interface
// Begin --------------------------------------------------------------------
input          RSMU_RDFT_sms_fuse_grp_resetb      ; //
input          RSMU_RDFT_sms_fuse_grp_run_MBIST   ; //
input          RSMU_RDFT_sms_fuse_grp_run_BIHR    ; //
input          RSMU_RDFT_sms_fuse_grp_valid       ; //
input   [31:0] RSMU_RDFT_sms_fuse_grp_data        ; //
output         RDFT_RSMU_sms_fuse_grp_repair_busy ; //
output         RDFT_RSMU_sms_fuse_grp_repair_done ; //
output         RDFT_RSMU_sms_fuse_grp_bist_fail   ; //
// End ----------------------------------------------------------------------

// ==========================================================================
// SMSg interface
output         BC1500_Wrck                        ; // output Tck clock to SMSg & larr
input          wrckLeaf                           ; // input  Tck clock at leaf level, with BC1500_Wrck as the root
// fuse interface with SMSg
// Signals are synchronous to BC1500_Wrck clock
// Begin --------------------------------------------------------------------
input          smsg_ef_req                        ; // (Per SMSg) - Every toggle is a new request
input   [ 6:0] smsg_ef_addr                       ; // (Per SMSg) - 7 bit byte address
output         smsg_ef_ready                      ; // (Per SMSg) - every toggle is ack for request
output  [ 7:0] smsg_ef_byte                       ; // (Per SMSg) - 8 bit efuse data, valid at smsg_ack toggle

input          smsg_sfp_ready                     ; // (Per SMSg) - SMSg ready to accept command
input          smsg_sfp_fail                      ; // (Per SMSg) - SMSg BIST or BISR fail status
output         smsg_sfp_request                   ; // (Per SMSg) - Trigger signal for bihr, bist, and bisr
output         smsg_sfp_resetb                    ; // (Per SMSg) - Signal to reset SFP of SMSg (does not clear fuse)

output         smsg_bihr                          ; // launch Built In Hard Repair
output         smsg_bist                          ; // launch Built In Self Test
output         smsg_bisr                          ; // launch Built In Soft Repair
output         smsg_fclken                        ; // Clock gater term for functional clock, synchroonizer in smsg
// End ----------------------------------------------------------------------

// ==========================================================================
// BIST interface with larr
// Begin --------------------------------------------------------------------
input          LARR_BIST_FAIL_Daisy_Out_0        ; // Latch arrays BIST fail status (Valis only when DONE == 1'b1)
input          LARR_BIST_DONE_Daisy_Out_0        ; // Latch arrays BIST done
output         LARR_BIST_smartmode_en            ; // Trigger signal to lanunch latch arrays BIST
output         LARR_BIST_smartmode_rstb          ; // Reset output to latch arrays
// End ----------------------------------------------------------------------

// ==========================================================================
// TDR interface
// Begin --------------------------------------------------------------------
input          Tdr_Tck                  ;
input          Tdr_Tdi                  ;
input          Tdr_Shift                ;
input          Tdr_Capture              ;
input          Tdr_Update               ;
input          Tdr_Trstb                ;

input          Tdr_fuse_if_ate_mode_En  ;
output         Tdr_fuse_if_ate_mode_Tdo ;
input          Tdr_fuse_if_status_En    ;
output         Tdr_fuse_if_status_Tdo   ;
input          Tdr_fuse_if_setup_En     ;
output         Tdr_fuse_if_setup_Tdo    ;
// End ----------------------------------------------------------------------
   //===============================================================================
   wire        trstb_fe_ipclk; // Tdr_Trstb synched to ipclk
   wire        trstb_ipclk_raw  ;
   wire        resetb    ;
   reg         fuseIf_En ;
   wire        ick       ;
   wire        drClk     ;
   wire        drClkR    ; // rising  edge
   wire        drClkF    ; // falling edge
   wire        cntDone   ;
   wire        cntGE8    ; // cnt >=8
   wire        bist_fail ;
   wire        all__done ;
   reg         busy      ;

   reg  [ 4:0] fseCtl    ; // Main controller FSM
   reg  [ 4:0] fseCtlNxt ;

   // smsg
   wire        sfp_ready           ;

   // For TDR : fuse_if_setup
   reg         fuse_if_sync_burn_in_mode;
   reg         fuse_if_sync_read_config ;
   reg         fuse_if_sync_config_force;
   reg         fuse_if_sync_run_mbist   ;
   reg         fuse_if_sync_reset_mbist ;
   reg  [`FSECONF_USED__RNG_GC10] fuse_if_sync_config_value;

   // For TDR : fuse_if_status
   wire        fuse_if_read_config_mode;
   wire [31:0] fuse_if_info           ;
   wire [31:0] fuse_if_info2          ;

   wire [31:0] fuse_if_config         ;
   wire        fuse_if_lar_bist_done  ;
   wire        fuse_if_lar_bist_fail  ;
   wire        fuse_if_sms_bihr_done  ;
   wire        fuse_if_sms_bist_done  ;
   wire        fuse_if_sms_bist_fail  ;
   wire        fuse_if_rsmu_run_bist  ;
   wire        fuse_if_rsmu_run_bihr  ;
   wire        ate_mode               ;
   wire        fuse_if_is_burn_in_mode;
   reg  [7:0]  fuse_if_counter        ;
   wire [1:0]  fuse_if_no_smsg_larr   ;
   wire        fuse_if_bist_done      ;
   wire        fuse_if_bist_fail      ;

   // For JTAG triggered MBIST
   reg  [ 2:0] fseJtag   ; // FSM for JTAG triggered MBIST
   reg  [ 2:0] fseJtagNxt;
   wire        sync_updateEn_re       ;
   wire        jtag_trigger_resetb    ;
   wire        jtag_trigger_mbist_active;
   wire        jtag_trigger_mbist     ;

   // For enhancement bug: DECDFXIP40-537 - Item 1  Add 2 fields to "Configuration" to ...
   // http://ontrack-internal.amd.com/browse/DECDFXIP40-537
   wire      run_MBIST_filtered       ;
   wire      run_BIHR_filtered        ;
   //===============================================================================

   // <Misc> ==================================================== begin
   // JIRA DECDFXIP41-60 (http://ontrack-internal.amd.com/browse/DECDFXIP41-60)
   // Ass async reset port for power partition
   wire sync_sms_restb;
   wire comb_restb;
   reg  is_restore;
   reg  sync_sms_restb_d1;
   dfx_async_rst_sync3_gc10  I_sync_sms_rstb (.clk(ipclk), .rstinB (Async_sms_resetb), .rstoutB(sync_sms_restb));
   assign comb_restb = sync_sms_restb & RSMU_RDFT_sms_fuse_grp_resetb;
   ati_mux2 d0nt_resetb (.s(jtag_trigger_mbist_active), .z(resetb), .i0(comb_restb), .i1(jtag_trigger_resetb));

   always @(posedge ipclk)
     if (~resetb)
       fuseIf_En <= 1'b1;
     else
       fuseIf_En <= (fseCtl[4:0] != `FSECTL_____IDEL_GC10) | sync_updateEn_re | fuse_if_sync_config_force | trstb_fe_ipclk; // If JTAG force is on, keep clock enabled

   always @(posedge ipclk)
     if (~sync_sms_restb)
       sync_sms_restb_d1 <= 1'b0;
     else
       sync_sms_restb_d1 <= sync_sms_restb;
   always @(posedge ipclk)
     if (~resetb)
       is_restore <= 1'b0;
     else
       is_restore <= (sync_sms_restb & ~sync_sms_restb_d1) ? RSMU_RDFT_sms_fuse_grp_resetb : is_restore;

   // Clock gater (/import/rtllib/src/rtl/ati_clock_gate.v)
   ati_clock_gate I_iclk (.clk_out(ick), .clk_src(ipclk), .clk_en(fuseIf_En), .TE(TEST_CLK_EN));

   wire runBist;
   wire runBihr;

   assign fuse_if_rsmu_run_bist = runBist;
   assign fuse_if_rsmu_run_bihr = runBihr;

generate
   case ( NO_SMSG_LARR )
     2: begin : BISTBISR_NO_SMSG_LARR__2
        reg  runBistReg;
        assign runBihr = 1'b0;
        assign runBist = runBistReg;
        always @(posedge ick) begin
           if (~resetb) begin
              runBistReg  <= 1'b0;
           end else begin
              runBistReg  <= (fseCtl[4:0] == `FSECTL_CONF_LD0_GC10) ? run_MBIST_filtered : runBist;
           end
        end
     end // block: BISTBISR_NO_SMSG_LARR__2
     3: begin : BISTBISR_NO_SMSG_LARR__3
        assign runBist = 1'b0;
        assign runBihr = 1'b0;
     end // block: BISTBISR_NO_SMSG_LARR__3
     default: begin : BISTBISR_NO_SMSG_LARR__01
        reg  runBihrReg;
        reg  runBistReg;
        assign runBihr = runBihrReg;
        assign runBist = runBistReg;
        always @(posedge ick) begin
           if (~resetb) begin
              {runBihrReg,runBistReg} <= {2 {1'b0}};
           end else begin
              runBihrReg <= (fseCtl[4:0] == `FSECTL_CONF_LD0_GC10) ? run_BIHR_filtered  : runBihr;
              runBistReg <= (fseCtl[4:0] == `FSECTL_CONF_LD0_GC10) ? run_MBIST_filtered : runBist;
           end
        end
     end // block: BISTBISR_NO_SMSG_LARR__01
   endcase // case ( NO_SMSG_LARR )
   case ( NO_SMSG_LARR )
     0 : begin : TDR_NO_SMSG_LARR__0
        assign fuse_if_no_smsg_larr[1:0] = 2'b00;
     end
     1 : begin : TDR_NO_SMSG_LARR__1
        assign fuse_if_no_smsg_larr[1:0] = 2'b01;
     end
     2 : begin : TDR_NO_SMSG_LARR__2
        assign fuse_if_no_smsg_larr[1:0] = 2'b10;
     end
     3 : begin : TDR_NO_SMSG_LARR__3
        assign fuse_if_no_smsg_larr[1:0] = 2'b11;
     end
   endcase // case ( NO_SMSG_LARR )
endgenerate
   // <Misc> ==================================================== end

   // <Drive RSMU interface> ==================================== begin
   reg       skipBihr_mon;
   reg       config_15bit;
   reg       do_not_mbist;
   reg       do_not_bihr;
   reg [4:0] bist_clk_div;
   reg [4:0] bihr_clk_div;
   reg       to_rsmu_done;

   assign run_MBIST_filtered = RSMU_RDFT_sms_fuse_grp_run_MBIST & ~do_not_mbist;                 // For enhancement bug: DECDFXIP40-537 - Item 1
   assign run_BIHR_filtered  = RSMU_RDFT_sms_fuse_grp_run_BIHR  & ~(do_not_bihr | skipBihr_mon); // For enhancement bug: DECDFXIP40-537 - Item 1 & DECDFXIP41-60
   assign RDFT_RSMU_sms_fuse_grp_repair_busy = busy;
   assign RDFT_RSMU_sms_fuse_grp_repair_done = to_rsmu_done; // fseCtl[4:0] == `FSECTL_____IDEL_GC10;
   assign RDFT_RSMU_sms_fuse_grp_bist_fail   = bist_fail & (fseCtl[4:0] == `FSECTL_____IDEL_GC10);
   assign fuse_if_config[`FSECONF_UNUSED__RNG_GC10           ] = {19 {1'b0}};
   assign fuse_if_config[`FSECONF_CONFIG_15BIT__RNG_GC10     ] = config_15bit;
   assign fuse_if_config[`FSECONF_DO_NOT_MBIST__RNG_GC10     ] = do_not_mbist;
   assign fuse_if_config[`FSECONF_DO_NOT_BIHR__RNG_GC10      ] = do_not_bihr;
   assign fuse_if_config[`FSECONF_SMSG_BIST_CLK_DIV__RNG_GC10] = bist_clk_div[4:0];
   assign fuse_if_config[`FSECONF_SMSG_BIHR_CLK_DIV__RNG_GC10] = bihr_clk_div[4:0];
   assign fuse_if_info2[31:0] = {fuse_if_sync_run_mbist      ,   // [31:31]
                                 fuse_if_sync_reset_mbist    ,   // [30:30]
                                 fuse_if_sync_config_force   ,   // [29:29]
                                 fseJtag                [2:0],   // [28:26]
                                 fseCtl                 [4:0],   // [25:21]
                                 fuse_if_no_smsg_larr   [1:0],   // [20:19]
                                 fuse_if_lar_bist_done       ,   // [18:18]
                                 fuse_if_lar_bist_fail       ,   // [17:17]
                                 fuse_if_sms_bihr_done       ,   // [16:16]
                                 fuse_if_sms_bist_done       ,   // [15:15]
                                 fuse_if_sms_bist_fail       ,   // [14:14]
                                 fuse_if_rsmu_run_bist       ,   // [13:13]
                                 fuse_if_rsmu_run_bihr       ,   // [12:12]
                                 ate_mode                    ,   // [11:11]
                                 fuse_if_is_burn_in_mode     ,   // [10:10]
                                 fuse_if_counter        [7:0],   // [ 9: 2]
                                 fuse_if_bist_done           ,   // [ 1: 1]
                                 fuse_if_bist_fail            }; // [ 0: 0]

   assign fuse_if_info[31:0] = fuse_if_read_config_mode ? fuse_if_config[31:0] : fuse_if_info2[31:0];

   wire cdc_skipBihr_mon;
   wire is_FSECTL_CONF_LD0;
   techind_cdcefpm #(.size (1), .applyParams(1), .DestClkCnt(3), .CdcDestSetup(0)) mrk_cdc_skipBihr_mon (.A(Async_do_not_bihr), .EN(is_FSECTL_CONF_LD0), .DSTCLK(ipclk), .Z(cdc_skipBihr_mon));
   assign is_FSECTL_CONF_LD0 = (fseCtlNxt[4:0] == `FSECTL_CONF_LD0_GC10);

   always @(posedge ick)
     if (~resetb) begin
         skipBihr_mon <= 1'b0;
         config_15bit <= 1'b0;
         do_not_mbist <= 1'b0;
         do_not_bihr  <= 1'b0;
        {bist_clk_div[4:0], bihr_clk_div[4:0]} <= {10 {1'b0}};
     end else begin
	skipBihr_mon <= is_FSECTL_CONF_LD0 ? cdc_skipBihr_mon : skipBihr_mon;
        casez ({(fseCtlNxt[4:0] == `FSECTL_CONF_LD0_GC10), fuse_if_sync_config_force})
          2'b10   : {config_15bit, do_not_mbist, do_not_bihr, bist_clk_div[4:0], bihr_clk_div[4:0]} <= RSMU_RDFT_sms_fuse_grp_data[`FSECONF_USED__RNG_GC10];
          2'b?1   : {config_15bit, do_not_mbist, do_not_bihr, bist_clk_div[4:0], bihr_clk_div[4:0]} <= fuse_if_sync_config_value[`FSECONF_USED__RNG_GC10];
          default : {config_15bit, do_not_mbist, do_not_bihr, bist_clk_div[4:0], bihr_clk_div[4:0]} <= {config_15bit, do_not_mbist, do_not_bihr, bist_clk_div[4:0], bihr_clk_div[4:0]};
        endcase // casez({(fseCtlNxt[4:0] == `FSECTL_CONF_LD0_GC10), fuse_if_sync_config_force})
     end // else: !if(~resetb)

   // <Drive RSMU interface> ==================================== end

   // <fseJtag Main controller> ================================= begin

   assign jtag_trigger_mbist_active = ~((fseJtag[2:0] == `FSEJTAG__IDEL_GC10) | (fseJtag[2:0] == `FSEJTAG__DONE_GC10)) & trstb_ipclk_raw;
   assign jtag_trigger_mbist        = fuse_if_sync_config_force & fuse_if_sync_run_mbist;
   assign jtag_trigger_resetb       = 1'b1;

   always @(*) begin
      case (fseJtag[2:0])
        `FSEJTAG__IDEL_GC10 : fseJtagNxt[2:0] = jtag_trigger_mbist                                ? `FSEJTAG_START_GC10 : `FSEJTAG__IDEL_GC10;
        `FSEJTAG_START_GC10 : fseJtagNxt[2:0] = fuse_if_sync_reset_mbist                          ? `FSEJTAG_RSTB0_GC10 : `FSEJTAG_RSTB1_GC10;
        `FSEJTAG_RSTB0_GC10 : fseJtagNxt[2:0] = (fseCtlNxt[4:0] == `FSECTL_RST__DLY_GC10) ? `FSEJTAG_RSTB1_GC10 : `FSEJTAG_RSTB0_GC10;
        `FSEJTAG_RSTB1_GC10 : fseJtagNxt[2:0] = (fseCtlNxt[4:0] == `FSECTL____PAUSE_GC10) ? `FSEJTAG_MBIST_GC10 : `FSEJTAG_RSTB1_GC10;
        `FSEJTAG_MBIST_GC10 : fseJtagNxt[2:0] = (fseCtl[4:0] == `FSECTL_____IDEL_GC10)    ? `FSEJTAG__DONE_GC10 : `FSEJTAG_MBIST_GC10;
        `FSEJTAG__DONE_GC10 : fseJtagNxt[2:0] = `FSEJTAG__DONE_GC10;
        default        : fseJtagNxt[2:0] = fseJtag[2:0];
      endcase // case (fseJtag[2:0])
   end // always @ begin

   always @(posedge ick) begin
      if (~trstb_ipclk_raw) begin
         fseJtag[2:0] <= `FSEJTAG__IDEL_GC10;
      end else begin
         fseJtag[2:0] <= fseJtagNxt[2:0];
      end
   end
   // <fseJtag Main controller> ================================= end

   // <fseCtl Main controller> ================================== begin
   wire      smsg_ef_req_toggle;
   wire      smsg_ef_ready_toggle;
   wire      sfp_request;

   always @(posedge ick) begin
      if (~resetb) begin
         fseCtl[4:0] <= `FSECTL_RST__LOW_GC10;
      end else begin
         fseCtl[4:0] <= fseCtlNxt[4:0];
      end
   end

   always @(*) begin
      case (fseJtag[2:0])
        `FSEJTAG_RSTB0_GC10       : fseCtlNxt[4:0] = cntDone                      ? `FSECTL_RST__DLY_GC10 : `FSECTL_RST__LOW_GC10;
        `FSEJTAG_RSTB1_GC10       : fseCtlNxt[4:0] = cntDone                      ? `FSECTL____PAUSE_GC10 : `FSECTL_RST__DLY_GC10;
        default                           :
          // JIRA DECDFXIP41-60 (http://ontrack-internal.amd.com/browse/DECDFXIP41-60)
	  // If reset is from sync_sms_restb, need to release the reset & go to done state
          case (fseCtl[4:0])
            `FSECTL_RST__LOW_GC10 : fseCtlNxt[4:0] = cntDone                      ? `FSECTL_RST__DLY_GC10 : `FSECTL_RST__LOW_GC10;
            `FSECTL_RST__DLY_GC10 : fseCtlNxt[4:0] = cntDone                      ? `FSECTL_CONF_RDY_GC10 : `FSECTL_RST__DLY_GC10;
            `FSECTL_CONF_RDY_GC10 : fseCtlNxt[4:0] = RSMU_RDFT_sms_fuse_grp_valid ? `FSECTL_CONF_LD0_GC10 : `FSECTL_CONF_RDY_GC10;
            `FSECTL_CONF_LD0_GC10 : fseCtlNxt[4:0] = `FSECTL_CONF_LD1_GC10;

            `FSECTL_CONF_LD1_GC10 :
              casez ({runBihr, runBist})
                2'b1?        : fseCtlNxt[4:0] = `FSECTL_BIHR_DL0_GC10;
                2'b01        : fseCtlNxt[4:0] = `FSECTL_BIST_DL0_GC10;
                default      : fseCtlNxt[4:0] = `FSECTL_____IDEL_GC10;
              endcase // casez ({runBihr, runBist})
            `FSECTL_BIHR_DL0_GC10 : fseCtlNxt[4:0] = cntDone                      ? `FSECTL_BIHR_REQ_GC10 : `FSECTL_BIHR_DL0_GC10;
            `FSECTL_BIHR_REQ_GC10 : //  fseCtlNxt[4:0] = sfp_ready                ? `FSECTL_BIHR_REQ_GC10 : `FSECTL_BIHR_F2W_GC10; // : `FSECTL_BIHR_F0W_GC10; // `FSECTL_BIHR_F2W_GC10
              casez ({sfp_ready, config_15bit})
                2'b1?        : fseCtlNxt[4:0] = `FSECTL_BIHR_REQ_GC10;
                2'b00        : fseCtlNxt[4:0] = `FSECTL_BIHR_F0W_GC10; // don't use bits [31:16] of config packet as repair fuse
                2'b01        : fseCtlNxt[4:0] = `FSECTL_BIHR_F2W_GC10; //       use bits [31:16] of config packet as repair fuse
                default      : fseCtlNxt[4:0] = `FSECTL_BIHR_REQ_GC10;
              endcase // casez ({sfp_ready, config_15bit})
            `FSECTL_BIHR_F0W_GC10 :
              casez({sfp_ready, smsg_ef_req_toggle})
                2'b1?        : fseCtlNxt[4:0] = `FSECTL_BIHR_DL1_GC10; // SMSg does not have BIHR or do not need more fuse
                2'b01        : fseCtlNxt[4:0] = `FSECTL_BIHR_RRQ_GC10; // SMSg does     have BIHR and       need more fuse
                default      : fseCtlNxt[4:0] = `FSECTL_BIHR_F0W_GC10;
              endcase // casez ({sfp_ready, smsg_ef_req_toggle})
            `FSECTL_BIHR_RRQ_GC10 : fseCtlNxt[4:0] = RSMU_RDFT_sms_fuse_grp_valid ? `FSECTL_BIHR_RRQ_GC10 : `FSECTL_BIHR_RWF_GC10;
            `FSECTL_BIHR_RWF_GC10 : fseCtlNxt[4:0] = RSMU_RDFT_sms_fuse_grp_valid ? `FSECTL_BIHR_F0T_GC10 : `FSECTL_BIHR_RWF_GC10;
            `FSECTL_BIHR_F0T_GC10 : fseCtlNxt[4:0] = smsg_ef_ready_toggle         ? `FSECTL_BIHR_F1W_GC10 : `FSECTL_BIHR_F0T_GC10;

            `FSECTL_BIHR_F1W_GC10 :
              casez({sfp_ready, smsg_ef_req_toggle})
                2'b1?        : fseCtlNxt[4:0] = `FSECTL_BIHR_DL1_GC10; // SMSg do not need more fuse
                2'b01        : fseCtlNxt[4:0] = `FSECTL_BIHR_F1T_GC10; // SMSg        need more fuse
                default      : fseCtlNxt[4:0] = `FSECTL_BIHR_F1W_GC10;
              endcase // casez ({sfp_ready, smsg_ef_req_toggle})
            `FSECTL_BIHR_F1T_GC10: fseCtlNxt[4:0] = smsg_ef_ready_toggle         ? `FSECTL_BIHR_F2W_GC10 : `FSECTL_BIHR_F1T_GC10;

            `FSECTL_BIHR_F2W_GC10 :
              casez({sfp_ready, smsg_ef_req_toggle})
                2'b1?        : fseCtlNxt[4:0] = `FSECTL_BIHR_DL1_GC10; // SMSg do not need more fuse
                2'b01        : fseCtlNxt[4:0] = `FSECTL_BIHR_F2T_GC10; // SMSg        need more fuse
                default      : fseCtlNxt[4:0] = `FSECTL_BIHR_F2W_GC10;
              endcase // casez ({sfp_ready, smsg_ef_req_toggle})
            `FSECTL_BIHR_F2T_GC10 : fseCtlNxt[4:0] = smsg_ef_ready_toggle         ? `FSECTL_BIHR_F3W_GC10 : `FSECTL_BIHR_F2T_GC10;
            `FSECTL_BIHR_F3W_GC10 :
              casez({sfp_ready, smsg_ef_req_toggle})
                2'b1?        : fseCtlNxt[4:0] = `FSECTL_BIHR_DL1_GC10; // SMSg do not need more fuse
                2'b01        : fseCtlNxt[4:0] = `FSECTL_BIHR_F3T_GC10; // SMSg        need more fuse
                default      : fseCtlNxt[4:0] = `FSECTL_BIHR_F3W_GC10;
              endcase // casez ({sfp_ready, smsg_ef_req_toggle})
            `FSECTL_BIHR_F3T_GC10 : fseCtlNxt[4:0] = smsg_ef_ready_toggle         ? `FSECTL_BIHR_F0W_GC10 : `FSECTL_BIHR_F3T_GC10;
            `FSECTL_BIHR_DL1_GC10 : fseCtlNxt[4:0] = cntDone                      ? `FSECTL_BIHRDONE_GC10 : `FSECTL_BIHR_DL1_GC10;
            `FSECTL_BIHRDONE_GC10 :
              casez ({cntDone, runBist})
                2'b0?        : fseCtlNxt[4:0] = `FSECTL_BIHRDONE_GC10;
                2'b10        : fseCtlNxt[4:0] = `FSECTL_____IDEL_GC10;
                2'b11        : fseCtlNxt[4:0] = `FSECTL____PAUSE_GC10;
                default      : fseCtlNxt[4:0] = `FSECTL_BIHRDONE_GC10;
              endcase // casez ({cntDone, runBist})
            `FSECTL____PAUSE_GC10 : fseCtlNxt[4:0] = `FSECTL_BIST_DL0_GC10;
            `FSECTL_BIST_DL0_GC10 : fseCtlNxt[4:0] = cntDone                      ? `FSECTL_BIST_REQ_GC10 : `FSECTL_BIST_DL0_GC10;
            `FSECTL_BIST_REQ_GC10 : fseCtlNxt[4:0] = sfp_request                  ? `FSECTL_BIST_ACK_GC10 : `FSECTL_BIST_REQ_GC10;
            `FSECTL_BIST_ACK_GC10 : fseCtlNxt[4:0] = ~sfp_ready                   ? `FSECTL_BIST_RUN_GC10 : `FSECTL_BIST_ACK_GC10;
            `FSECTL_BIST_RUN_GC10 : fseCtlNxt[4:0] = all__done                    ? `FSECTL_BISTBURN_GC10 : `FSECTL_BIST_RUN_GC10;
            `FSECTL_BISTBURN_GC10 :
//            casez ({fuse_if_sync_burn_in_mode, drClkF}) // bug 627576 - sfp_request duration too short for synchronizer
              casez ({fuse_if_sync_burn_in_mode, cntDone})// bug 627576
                2'b0?   : fseCtlNxt[4:0] = `FSECTL_BIST_DL1_GC10;
                2'b11   : fseCtlNxt[4:0] = `FSECTL_BIST_REQ_GC10;
                default : fseCtlNxt[4:0] = `FSECTL_BISTBURN_GC10;
              endcase // casez ({fuse_if_sync_burn_in_mode, drClkF})
            `FSECTL_BIST_DL1_GC10 : fseCtlNxt[4:0] = cntDone                      ? `FSECTL_BISTDONE_GC10 : `FSECTL_BIST_DL1_GC10;
            `FSECTL_BISTDONE_GC10 : fseCtlNxt[4:0] = cntDone                      ? `FSECTL_____IDEL_GC10 : `FSECTL_BISTDONE_GC10;
            `FSECTL_____IDEL_GC10 : fseCtlNxt[4:0] = `FSECTL_____IDEL_GC10;
            default          : fseCtlNxt[4:0] = `FSECTL_RST__LOW_GC10;
          endcase // case (fseCtl[4:0])
      endcase // case (fseJtag[2:0])

   end // always @ begin
   // <fseCtl Main controller> ================================== end

   // <drClk Gen & Clocking Control> ============================ begin
   wire wrckEn;
   wire fclkEn;
generate
   case ( NO_SMSG_LARR )
     3 : // NO_SMSG_LARR == 3: no SMSg and no LARR
       begin : DRCLK_NO_SMSG_NO_LARR
          assign wrckEn = 1'b0;
          assign fclkEn = 1'b0;
          assign drClk  = 1'b0;
          assign drClkR = 1'b0;
          assign drClkF = 1'b0;
       end
     default : // We need to generate clock if either SMSg or LARR exist
       begin : DRCLK_DEFAULT

          reg  [ 4:0] targClkDiv  ;
          reg         fclkEnReg   ;
          reg         fclkEnRegNxt;
          reg         wrckEnReg   ;
          reg         wrckEnRegNxt;
          assign      wrckEn       = wrckEnReg;
          assign      fclkEn       = fclkEnReg;

          fuse_if_drclkf_gc10 I_fuse_if_drclk (.ick       (ick            ),
                                          .resetb    (resetb         ),
                                          .targClkDiv(targClkDiv[4:0]),
                                          .drClkEn   (wrckEn         ),
                                          .drClk     (drClk          ),
                                          .drClkR    (drClkR         ),
                                          .drClkF    (drClkF         ) );

          always @(*) begin
             case (fseCtl[4:0])
               `FSECTL_BIHR_DL0_GC10, `FSECTL_BIHR_REQ_GC10,
               `FSECTL_BIHR_F0W_GC10, `FSECTL_BIHR_RRQ_GC10,
               `FSECTL_BIHR_RWF_GC10, `FSECTL_BIHR_F0T_GC10,
               `FSECTL_BIHR_F1W_GC10, `FSECTL_BIHR_F1T_GC10,
               `FSECTL_BIHR_F2W_GC10, `FSECTL_BIHR_F2T_GC10,
               `FSECTL_BIHR_F3W_GC10, `FSECTL_BIHR_F3T_GC10,
               `FSECTL_BIHR_DL1_GC10, `FSECTL_BIHRDONE_GC10 : targClkDiv[4:0] = bihr_clk_div[4:0];
               default                            : targClkDiv[4:0] = bist_clk_div[4:0];
             endcase // case (fseCtlNxt[4:0])
             case (fseCtlNxt[4:0])
               `FSECTL_BIHR_DL0_GC10, `FSECTL_BIHR_REQ_GC10,
               `FSECTL_BIHR_F0W_GC10, `FSECTL_BIHR_RRQ_GC10,
               `FSECTL_BIHR_RWF_GC10, `FSECTL_BIHR_F0T_GC10,
               `FSECTL_BIHR_F1W_GC10, `FSECTL_BIHR_F1T_GC10,
               `FSECTL_BIHR_F2W_GC10, `FSECTL_BIHR_F2T_GC10,
               `FSECTL_BIHR_F3W_GC10, `FSECTL_BIHR_F3T_GC10,

               `FSECTL_BIST_DL0_GC10, `FSECTL_BIST_REQ_GC10,
               `FSECTL_BIST_ACK_GC10, `FSECTL_BIST_RUN_GC10,
               `FSECTL_BISTBURN_GC10 :
                 begin
                    wrckEnRegNxt = 1'b1;
                    fclkEnRegNxt = 1'b1;
                 end // case: `FSECTL_BIHR_DL0, `FSECTL_BIHR_REQ,...
               `FSECTL_BIHR_DL1_GC10, `FSECTL_BIST_DL1_GC10 :
                 begin
                    wrckEnRegNxt = 1'b1;
                    fclkEnRegNxt = cntGE8; // deassert 8 clock early for 3 deep sync
                 end
               default :
                 begin
                    wrckEnRegNxt = 1'b0;
                    fclkEnRegNxt = 1'b0;
                 end
             endcase // case (fseCtlNxt[4:0])
          end // always @ begin

          always @(posedge ick)
            if (~resetb) begin
               wrckEnReg <= 1'b0;
               fclkEnReg <= 1'b0;
            end else begin
               wrckEnReg <= wrckEnRegNxt;
               fclkEnReg <= fclkEnRegNxt;
            end
       end // block: DRCLK_DEFAULT
   endcase // case ( NO_SMSG_LARR )
endgenerate
   // <drClk Gen & Clocking Control> ============================ end

   // <Delay counter> =========================================== begin
   wire [5:0] cntStart;
   reg  [5:0] cnt     ;
   reg        cntEn   ;
   reg        cntAdv  ;

   assign cntStart[5:0] = 6'h20; // == 32 clock edge
   assign cntDone = (cnt[5:0] == {6 {1'b0}}) & resetb;
   assign cntGE8  = (cnt[5:0] >=  6'b001000) & resetb;

   always @(*) begin
      case (fseCtlNxt[4:0])
        `FSECTL_RST__LOW_GC10 : begin cntEn = 1'b1; cntAdv = cntEn         ; end
        `FSECTL_RST__DLY_GC10 : begin cntEn = 1'b1; cntAdv = cntEn         ; end
        `FSECTL_CONF_RDY_GC10 : begin cntEn = 1'b0; cntAdv = cntEn         ; end
        `FSECTL_CONF_LD0_GC10 : begin cntEn = 1'b0; cntAdv = cntEn         ; end
        `FSECTL_CONF_LD1_GC10 : begin cntEn = 1'b0; cntAdv = cntEn         ; end

        `FSECTL_BIHR_DL0_GC10 : begin cntEn = 1'b1; cntAdv = cntEn & drClkR; end
        `FSECTL_BIHR_REQ_GC10 : begin cntEn = 1'b0; cntAdv = cntEn & drClkR; end
        `FSECTL_BIHR_F0W_GC10 : begin cntEn = 1'b0; cntAdv = cntEn & drClkR; end
        `FSECTL_BIHR_RRQ_GC10 : begin cntEn = 1'b0; cntAdv = cntEn & drClkR; end
        `FSECTL_BIHR_RWF_GC10 : begin cntEn = 1'b0; cntAdv = cntEn & drClkR; end
        `FSECTL_BIHR_F0T_GC10 : begin cntEn = 1'b0; cntAdv = cntEn & drClkR; end
        `FSECTL_BIHR_F1W_GC10 : begin cntEn = 1'b0; cntAdv = cntEn & drClkR; end
        `FSECTL_BIHR_F1T_GC10 : begin cntEn = 1'b0; cntAdv = cntEn & drClkR; end
        `FSECTL_BIHR_F2W_GC10 : begin cntEn = 1'b0; cntAdv = cntEn & drClkR; end
        `FSECTL_BIHR_F2T_GC10 : begin cntEn = 1'b0; cntAdv = cntEn & drClkR; end
        `FSECTL_BIHR_F3W_GC10 : begin cntEn = 1'b0; cntAdv = cntEn & drClkR; end
        `FSECTL_BIHR_F3T_GC10 : begin cntEn = 1'b0; cntAdv = cntEn & drClkR; end
        `FSECTL_BIHR_DL1_GC10 : begin cntEn = 1'b1; cntAdv = cntEn & drClkR; end
        `FSECTL_BIHRDONE_GC10 : begin cntEn = 1'b1; cntAdv = cntEn         ; end

        `FSECTL____PAUSE_GC10 : begin cntEn = 1'b0; cntAdv = cntEn         ; end

        `FSECTL_BIST_DL0_GC10 : begin cntEn = 1'b1; cntAdv = cntEn & drClkR; end
        `FSECTL_BIST_REQ_GC10 : begin cntEn = 1'b0; cntAdv = cntEn & drClkR; end
        `FSECTL_BIST_ACK_GC10 : begin cntEn = 1'b0; cntAdv = cntEn & drClkR; end
        `FSECTL_BIST_RUN_GC10 : begin cntEn = 1'b0; cntAdv = cntEn & drClkR; end
//      `FSECTL_BISTBURN_GC10 : begin cntEn = 1'b0; cntAdv = cntEn & drClkR; end // bug 627576 - sfp_request duration too short for synchronizer
        `FSECTL_BISTBURN_GC10 : begin cntEn = 1'b1; cntAdv = cntEn & drClkR; end // bug 627576
        `FSECTL_BIST_DL1_GC10 : begin cntEn = 1'b1; cntAdv = cntEn & drClkR; end
        `FSECTL_BISTDONE_GC10 : begin cntEn = 1'b1; cntAdv = cntEn         ; end
        `FSECTL_____IDEL_GC10 : begin cntEn = 1'b0; cntAdv = cntEn         ; end
        default                       : begin cntEn = 1'b0; cntAdv = cntEn         ; end
      endcase // case (fseCtlNxt[4:0])
   end

   always @(posedge ick)
     if (~resetb | ~cntEn)
       cnt[5:0] <= cntStart[5:0];
     else
       cnt[5:0] <= cntAdv ? cnt[5:0] - 6'h01 : cnt[5:0];

   // <Delay counter> =========================================== end

   // <Drive SMSg interface> ==================================== begin

generate
   case ( NO_SMSG_LARR )
     3, 2: // NO_SMSG_LARR == (2 | 3) : no SMSg
       begin : SMSG_NO_SMSG
          assign smsg_ef_ready         =     1'b0  ;
          assign smsg_ef_byte[7:0]     = {8 {1'b0}};
          assign smsg_sfp_request      =     1'b0  ;
          assign smsg_sfp_resetb       =     1'b0  ;
          assign smsg_bihr             =     1'b0  ;
          assign smsg_bist             =     1'b0  ;
          assign smsg_bisr             =     1'b0  ;
          assign smsg_fclken           =     1'b0  ;

          assign smsg_ef_req_toggle    =     1'b0  ;
          assign smsg_ef_ready_toggle  =     1'b0  ;
          assign fuse_if_sms_bihr_done =     1'b1  ;
          assign fuse_if_sms_bist_done =     1'b1  ;
          assign fuse_if_sms_bist_fail =     1'b0  ;
          assign sfp_ready             =     1'b0  ;
          assign sfp_request           =     1'b1  ; // Fake request
          // input unused
          //! unused smsg_ef_req
          //! unused smsg_sfp_ready
          //! unused smsg_sfp_fail
          //! unused smsg_ef_addr
       end // SMSG_NO_SMSG
     default:
       begin : SMSG_WT_SMSG
`ifdef USE_DFX_FAKE_SMSG
 `ifdef ABV
          //! unused smsg_ef_addr
          //! unused smsg_ef_req
          //! unused smsg_sfp_ready
          //! unused smsg_sfp_fail

          wire [6:0] fk_smsg_addr ;
          wire       fk_smsg_req  ;
          wire       fk_smsg_ready;
          wire       fk_smsg_fail ;

          fuse_if_fk_smsg_gc10 I_fuse_if_fk_smsg (/*AUTOINST*/
                                             // Outputs
                                             .smsg_ef_addr     (fk_smsg_addr ),
                                             .smsg_ef_req      (fk_smsg_req  ),
                                             .smsg_sfp_ready   (fk_smsg_ready),
                                             .smsg_sfp_fail    (fk_smsg_fail ),
                                             // Inputs
                                             .wrckLeaf         (wrckLeaf         ),
                                             .smsg_ef_ready    (smsg_ef_ready    ),
                                             .smsg_ef_byte     (smsg_ef_byte[7:0]),
                                             .smsg_sfp_request (smsg_sfp_request ),
                                             .smsg_sfp_resetb  (smsg_sfp_resetb  ),
                                             .smsg_bihr        (smsg_bihr        ),
                                             .smsg_bist        (smsg_bist        ),
                                             .smsg_bisr        (smsg_bisr        ),
                                             .smsg_fclken      (smsg_fclken      )
                                             );
          fuse_if_drsmsg_gc10 I_fuse_if_drsmsg (
                                           .Async_Rstb_Scan_Override(Async_Rstb_Scan_Override), // JIRA SMS-61 (http://jira.amd.com/browse/sms-61)
                                           .Async_Rstb_Scan_Override_Sel(Async_Rstb_Scan_Override_Sel),
                                           /*AUTOINST*/
                                           // Outputs
                                           .smsg_ef_ready       (smsg_ef_ready),
                                           .smsg_ef_byte        (smsg_ef_byte[7:0]),
                                           .smsg_sfp_request    (smsg_sfp_request),
                                           .smsg_sfp_resetb     (smsg_sfp_resetb),
                                           .smsg_bihr           (smsg_bihr),
                                           .smsg_bist           (smsg_bist),
                                           .smsg_bisr           (smsg_bisr),
                                           .smsg_fclken         (smsg_fclken),
                                           .smsg_ef_req_toggle  (smsg_ef_req_toggle),
                                           .smsg_ef_ready_toggle(smsg_ef_ready_toggle),
                                           .sfp_ready           (sfp_ready),
                                           .sfp_request         (sfp_request),
                                           .fuse_if_sms_bihr_done(fuse_if_sms_bihr_done),
                                           .fuse_if_sms_bist_done(fuse_if_sms_bist_done),
                                           .fuse_if_sms_bist_fail(fuse_if_sms_bist_fail),
                                           // Inputs
                                           .ipclk               (ipclk),
                                           .ick                 (ick),
                                           .wrckLeaf            (wrckLeaf),
                                           .fclkEn              (fclkEn),
                                           .drClkF              (drClkF),
                                           .resetb              (resetb),
                                           .RSMU_RDFT_sms_fuse_grp_data(RSMU_RDFT_sms_fuse_grp_data[31:0]),
                                           .smsg_ef_req         (fk_smsg_req),
                                           .smsg_ef_addr        (fk_smsg_addr[6:0]),
                                           .smsg_sfp_ready      (fk_smsg_ready),
                                           .smsg_sfp_fail       (fk_smsg_fail),
                                           .fseCtl              (fseCtl[4:0]),
                                           .fseCtlNxt           (fseCtlNxt[4:0]));
 `endif // ifdef ABV
`else
          fuse_if_drsmsg_gc10 I_fuse_if_drsmsg (
                                           .Async_Rstb_Scan_Override(Async_Rstb_Scan_Override), // JIRA SMS-61 (http://jira.amd.com/browse/sms-61)
                                           .Async_Rstb_Scan_Override_Sel(Async_Rstb_Scan_Override_Sel),
                                           /*AUTOINST*/
                                           // Outputs
                                           .smsg_ef_ready       (smsg_ef_ready),
                                           .smsg_ef_byte        (smsg_ef_byte[7:0]),
                                           .smsg_sfp_request    (smsg_sfp_request),
                                           .smsg_sfp_resetb     (smsg_sfp_resetb),
                                           .smsg_bihr           (smsg_bihr),
                                           .smsg_bist           (smsg_bist),
                                           .smsg_bisr           (smsg_bisr),
                                           .smsg_fclken         (smsg_fclken),
                                           .smsg_ef_req_toggle  (smsg_ef_req_toggle),
                                           .smsg_ef_ready_toggle(smsg_ef_ready_toggle),
                                           .sfp_ready           (sfp_ready),
                                           .sfp_request         (sfp_request),
                                           .fuse_if_sms_bihr_done(fuse_if_sms_bihr_done),
                                           .fuse_if_sms_bist_done(fuse_if_sms_bist_done),
                                           .fuse_if_sms_bist_fail(fuse_if_sms_bist_fail),
                                           // Inputs
                                           .ipclk               (ipclk),
                                           .ick                 (ick),
                                           .wrckLeaf            (wrckLeaf),
                                           .fclkEn              (fclkEn),
                                           .drClkF              (drClkF),
                                           .resetb              (resetb),
                                           .RSMU_RDFT_sms_fuse_grp_data(RSMU_RDFT_sms_fuse_grp_data[31:0]),
                                           .smsg_ef_req         (smsg_ef_req),
                                           .smsg_ef_addr        (smsg_ef_addr[6:0]),
                                           .smsg_sfp_ready      (smsg_sfp_ready),
                                           .smsg_sfp_fail       (smsg_sfp_fail),
                                           .fseCtl              (fseCtl[4:0]),
                                           .fseCtlNxt           (fseCtlNxt[4:0]));
`endif // ifdef USE_DFX_FAKE_SMSG
       end // SMSG_WT_SMSG
   endcase // case ( NO_SMSG_LARR )
endgenerate
   // <Drive SMSg interface> ==================================== end

   // <Drive LARR interface> ==================================== begin
generate
   case ( NO_SMSG_LARR )
     3, 1:  // NO_SMSG_LARR == 1 : Only      SMSg          is connected to fuse_if; no LARR
       begin : LARR_NO_LARR
          assign LARR_BIST_smartmode_en     = 1'b0; // Trigger signal to lanunch latch arrays BIST
          assign LARR_BIST_smartmode_rstb   = 1'b0; // Reset output to latch arrays
          assign fuse_if_lar_bist_done      = 1'b1;
          assign fuse_if_lar_bist_fail      = 1'b0;
       end // LARR_NO_LARR
     default :
       begin : LARR_WITH_LARR
          // runBist

          wire      larr_failVal ;
          reg       larr_fail    ; // cdcenbuf
          wire      cdc_larr_fail;

          wire      larr_syncOff_muxed;
          reg       larr_syncOff     ;
          wire      sync_larr_doneRaw;
          wire      sync_larr_done   ;
          reg       larr_done_1shot  ;

          reg       larr_rstb    ;
          reg       larr_en      ;

          reg       larrr_rstbNxt;
          reg       larr_enNxt   ;

          techind_cdcefpm #(.size (1), .applyParams(1), .DestClkCnt(3), .CdcDestSetup(0)) mrk_cdc_larr_fail (.A(LARR_BIST_FAIL_Daisy_Out_0), .EN(larr_failVal), .DSTCLK(ipclk), .Z(cdc_larr_fail));
          ati_mux2 I_larr_syncOff_muxed (.s(Async_Rstb_Scan_Override_Sel), .z(larr_syncOff_muxed), .i0(~larr_syncOff), .i1(Async_Rstb_Scan_Override));
          dfx_sync_async_rst_gc10  I_sync_larr_doneRaw (.CK   (ick                       ),
                                                             .D    (LARR_BIST_DONE_Daisy_Out_0),
                                                             .rstB (larr_syncOff_muxed        ),
                                                             .Q    (sync_larr_doneRaw         ),
                                                             .SE   (                      1'b0));

          dfx_mrk_and_gc10 I_sync_larr_done (.A(sync_larr_doneRaw), .B(~larr_syncOff), .Y(sync_larr_done));

          assign LARR_BIST_smartmode_en   = larr_en                          ; // Trigger signal to lanunch latch arrays BIST
          assign LARR_BIST_smartmode_rstb = larr_rstb                        ; // Reset output to latch arrays
          assign fuse_if_lar_bist_done    = larr_done_1shot                  ;
          assign fuse_if_lar_bist_fail    = larr_fail                        ;
          assign larr_failVal             = sync_larr_done & ~larr_done_1shot;

          always @(*) begin
             case (fseCtlNxt[4:0])
               `FSECTL_RST__LOW_GC10 : begin larrr_rstbNxt = 1'b0; larr_enNxt = 1'b0; larr_syncOff = 1'b1; end
               `FSECTL_RST__DLY_GC10 : begin larrr_rstbNxt = 1'b0; larr_enNxt = 1'b0; larr_syncOff = 1'b1; end
               `FSECTL_CONF_RDY_GC10 : begin larrr_rstbNxt = 1'b1; larr_enNxt = 1'b0; larr_syncOff = 1'b1; end
               `FSECTL_CONF_LD0_GC10 : begin larrr_rstbNxt = 1'b1; larr_enNxt = 1'b0; larr_syncOff = 1'b1; end
               `FSECTL_CONF_LD1_GC10 : begin larrr_rstbNxt = 1'b1; larr_enNxt = 1'b0; larr_syncOff = 1'b1; end
               `FSECTL_BIST_DL0_GC10 : begin larrr_rstbNxt = 1'b1; larr_enNxt = 1'b0; larr_syncOff = 1'b1; end
               `FSECTL_BIST_REQ_GC10 : begin larrr_rstbNxt = 1'b1; larr_enNxt = 1'b1; larr_syncOff = 1'b1; end
               `FSECTL_BIST_ACK_GC10 : begin larrr_rstbNxt = 1'b1; larr_enNxt = 1'b1; larr_syncOff = 1'b1; end
               `FSECTL_BIST_RUN_GC10 : begin larrr_rstbNxt = 1'b1; larr_enNxt = 1'b1; larr_syncOff = 1'b0; end
               `FSECTL_BISTBURN_GC10 : begin larrr_rstbNxt = 1'b1; larr_enNxt = 1'b0; larr_syncOff = 1'b0; end
               `FSECTL_BISTDONE_GC10 : begin larrr_rstbNxt = 1'b1; larr_enNxt = 1'b0; larr_syncOff = 1'b0; end
               `FSECTL_____IDEL_GC10 : begin larrr_rstbNxt = 1'b1; larr_enNxt = 1'b0; larr_syncOff = 1'b1; end
               default                       : begin larrr_rstbNxt = 1'b1; larr_enNxt = 1'b0; larr_syncOff = 1'b1; end
             endcase // case (fseCtl[4:0])
          end // always @ begin

          always @(posedge ick)
            if (~resetb) begin
               larr_rstb       <= 1'b0;
               larr_en         <= 1'b0;
               larr_fail       <= 1'b0;
               larr_done_1shot <= 1'b0;
            end else begin
               larr_rstb       <= larrr_rstbNxt;
               larr_en         <= larr_enNxt   ;
               larr_done_1shot <= (sync_larr_done | larr_done_1shot) & // sticky, detect LARR completed BIST
                                  (fseCtl[4:0] != `FSECTL_BIST_REQ_GC10);    // Clear for Burn-in mode
               larr_fail       <= larr_failVal ? cdc_larr_fail : larr_fail;
            end // else: !if(~resetb)
       end // LARR_WITH_LARR
   endcase // case ( NO_SMSG_LARR )
endgenerate
   // <Drive LARR interface> ==================================== end

   // <Common signals> ========================================== begin
   assign fuse_if_bist_fail = bist_fail;
   assign fuse_if_bist_done = all__done;
generate
   case ( NO_SMSG_LARR )
     0: begin : DONE_NO_SMSG_LARR__0 // NO_SMSG_LARR == 0 : There are SMSg and LARR    connected to fuse_if
        reg bistFailReg;
        reg allDoneReg;
        assign bist_fail = bistFailReg;
        assign all__done = allDoneReg ;
        always @(posedge ick)
          if (~resetb) begin
             bistFailReg <= 1'b0;
             allDoneReg  <= 1'b0;
          end else begin
             bistFailReg <= fuse_if_lar_bist_fail | fuse_if_sms_bist_fail;
             allDoneReg  <= fuse_if_lar_bist_done & fuse_if_sms_bist_done;
          end
     end // block: DONE_NO_SMSG_LARR__0
     1: begin : DONE_NO_SMSG_LARR__1 // NO_SMSG_LARR == 1 : Only      SMSg          is connected to fuse_if; no LARR
        reg bistFailReg;
        reg allDoneReg;
        assign bist_fail = bistFailReg;
        assign all__done = allDoneReg ;
        always @(posedge ick)
          if (~resetb) begin
             bistFailReg <= 1'b0;
             allDoneReg  <= 1'b0;
          end else begin
             bistFailReg <= fuse_if_sms_bist_fail;
             allDoneReg  <= fuse_if_sms_bist_done;
          end
     end // block: DONE_NO_SMSG_LARR__1
     2: begin : DONE_NO_SMSG_LARR__2 // NO_SMSG_LARR == 2 : Only               LARR is connected to fuse_if; no SMSg
        reg bistFailReg;
        reg allDoneReg;
        assign bist_fail = bistFailReg;
        assign all__done = allDoneReg ;
        always @(posedge ick)
          if (~resetb) begin
             bistFailReg <= 1'b0;
             allDoneReg  <= 1'b0;
          end else begin
             bistFailReg <= fuse_if_lar_bist_fail;
             allDoneReg  <= fuse_if_lar_bist_done;
          end
     end // block: DONE_NO_SMSG_LARR__2
     default :
       begin : DONE_NO_SMSG_LARR__DEFAULT
          assign bist_fail = 1'b0;
          assign all__done = 1'b0;
       end // block: DONE_NO_SMSG_LARR__DEFAULT
   endcase // case ( NO_SMSG_LARR )
endgenerate

   always @(posedge ick) begin
      if (~trstb_ipclk_raw)
        to_rsmu_done <= fseCtlNxt[4:0] == `FSECTL_____IDEL_GC10;
      else
        to_rsmu_done <= jtag_trigger_mbist_active ? to_rsmu_done : fseCtlNxt[4:0] == `FSECTL_____IDEL_GC10;

      if (~resetb)
        busy <= 1'b1;
      else
        case (fseJtag[2:0])
          `FSEJTAG_START_GC10, `FSEJTAG_RSTB0_GC10, `FSEJTAG_RSTB1_GC10, `FSEJTAG_MBIST_GC10 : busy <= busy;
          default :
            case (fseCtl[4:0])
              `FSECTL_RST__LOW_GC10 : busy <= 1'b1;
              `FSECTL_RST__DLY_GC10 : busy <= 1'b1;
              `FSECTL_CONF_RDY_GC10 : busy <= cntDone;
              `FSECTL_CONF_LD0_GC10 : busy <= 1'b1;
              `FSECTL_CONF_LD1_GC10 : busy <= runBihr | runBist;
              `FSECTL_BIHR_DL0_GC10 : busy <= 1'b1;
              `FSECTL_BIHR_REQ_GC10 : busy <= 1'b1;
              `FSECTL_BIHR_F0W_GC10 : busy <= 1'b1;
              `FSECTL_BIHR_RRQ_GC10 : busy <= 1'b0;
              `FSECTL_BIHR_RWF_GC10 : busy <= RSMU_RDFT_sms_fuse_grp_valid;
              `FSECTL_BIHR_F0T_GC10 : busy <= 1'b1;
              `FSECTL_BIHR_F1W_GC10 : busy <= 1'b1;
              `FSECTL_BIHR_F1T_GC10 : busy <= 1'b1;
              `FSECTL_BIHR_F2W_GC10 : busy <= 1'b1;
              `FSECTL_BIHR_F2T_GC10 : busy <= 1'b1;
              `FSECTL_BIHR_F3W_GC10 : busy <= 1'b1;
              `FSECTL_BIHR_F3T_GC10 : busy <= 1'b1;
              `FSECTL_BIHR_DL1_GC10 : busy <= 1'b1;
              `FSECTL_BIHRDONE_GC10 : busy <= ~cntDone | runBist;
              `FSECTL____PAUSE_GC10 : busy <= 1'b1;
              `FSECTL_BIST_DL0_GC10 : busy <= 1'b1;
              `FSECTL_BIST_REQ_GC10 : busy <= 1'b1;
              `FSECTL_BIST_ACK_GC10 : busy <= 1'b1;
              `FSECTL_BIST_RUN_GC10 : busy <= 1'b1;
              `FSECTL_BISTBURN_GC10 : busy <= 1'b1;
              `FSECTL_BIST_DL1_GC10 : busy <= 1'b1;
              `FSECTL_BISTDONE_GC10 : busy <= ~cntDone;
              `FSECTL_____IDEL_GC10 : busy <= 1'b0;
              default                       : busy <= 1'b0;
            endcase // case (fseCtlNxt[4:0])
        endcase // case (fseJtag[2:0])
   end // always @ (posedge ick)
   // <Common signals> ========================================== end

   // <1500 TDR Code> =========================================== begin

   // Begin --------------------------------------------------------------------
   assign fuse_if_is_burn_in_mode = fuse_if_sync_burn_in_mode;
   wire fuse_if_counter_inc;
   assign fuse_if_counter_inc     = (all__done & fuse_if_sync_burn_in_mode & (fseCtl[4:0] == `FSECTL_BISTBURN_GC10) & cntDone);
   always @(posedge ick)
     if (~resetb)
       fuse_if_counter[7:0] <= {8 {1'b0}};
     else
       fuse_if_counter[7:0] <= fuse_if_counter_inc ? fuse_if_counter[7:0] + {{7 {1'b0}}, 1'b1} :  fuse_if_counter[7:0];

   // End ----------------------------------------------------------------------

  // delayed version of UpdateDR for sync data into ipclk
   reg   [2:1] trstb_ipclk_d    ;
   wire        any_Tdr_En       ;
   wire        sync_updateEn    ;
   reg   [3:1] sync_updateEn_d  ;
   wire        dec_update_d1    ;
   wire        dec_updateEn     ;
   assign      any_Tdr_En      =  Tdr_fuse_if_ate_mode_En | Tdr_fuse_if_setup_En;

   // http://ontrack-internal.amd.com/browse/DECDFXIP40-505
   //  Toggle dec_update_d1 at Tdr_Update to relax clock ratio requirement
   //  so ipclk to tck can be releaxed from 4:1 to 1:1 (ipclk is the faster clock)
   assign      dec_updateEn    =  Tdr_Trstb & ( (any_Tdr_En & Tdr_Update) ? ~dec_update_d1 : dec_update_d1);

   // in (/src/rtl/sync3_wrapper3_1b.v)
   dfx_sync_1b_gc10       I_sync_updateEn (.DBL_FLOP_EN(1'b1), .RST(~Tdr_Trstb),    // These 2 ports are not used in actual function
                                      .D(dec_updateEn),                        // input in SRC_CLK
                                      .SRC_CLK(Tdr_Tck), .Q_SRC(dec_update_d1),
                                      .DST_CLK(ipclk  ), .Q    (sync_updateEn));

   // NOTE: this is new bug fix bug 617451 ------------------------------------------ begin
   dfx_async_rst_sync3_gc10 I_trstb_ipclk (.clk(ipclk), .rstinB(Tdr_Trstb), .rstoutB(trstb_ipclk_raw)); // async_rst_sync
   assign trstb_fe_ipclk = (~trstb_ipclk_raw & trstb_ipclk_d[1]) | (~trstb_ipclk_d[1] & trstb_ipclk_d[2]);
   always @(posedge ipclk)
     if (~resetb)
       trstb_ipclk_d[2:1] <= 2'b01;
     else
       trstb_ipclk_d[2:1] <= {trstb_ipclk_d[1], trstb_ipclk_raw};

   wire                 cdc_any_Tdr_En  ;
   techind_cdcfpm  #(.size (1), .applyParams (1), .DestClkCnt(3), .CdcDestSetup(0)) mrk_cdc_any_Tdr_En (.A(any_Tdr_En), .DSTCLK(ipclk), .Z(cdc_any_Tdr_En ));
   // NOTE: this is new bug fix bug 617451 ------------------------------------------ end

   assign      sync_updateEn_re = (sync_updateEn_d[1] ^ sync_updateEn_d[2]) | (sync_updateEn_d[2] ^sync_updateEn_d[3]);
   wire        cdc_Tdr_fuse_if_setup_En  ;
   wire        updateEn_setup   ;
   techind_cdcfpm  #(.size (1), .applyParams (1), .DestClkCnt(3), .CdcDestSetup(0)) mrk_cdc_Tdr_fuse_if_setup_En (.A(Tdr_fuse_if_setup_En ), .DSTCLK(ipclk), .Z(cdc_Tdr_fuse_if_setup_En ));
   dfx_mrk_and_gc10 I_updateEn_setup (.A(sync_updateEn_re), .B(cdc_Tdr_fuse_if_setup_En), .Y(updateEn_setup));

   // NOTE: this is not needed ------------------------------------------ begin
   wire        updateEn_ate_mode;
   assign      updateEn_ate_mode = 1'b0;
   // NOTE: this is not needed ------------------------------------------ end

  // Using this set of flops to filter the x from the synchronizer (I_trstb_ipclk)
  always @(posedge ipclk)
     if (~trstb_ipclk_raw) begin
        sync_updateEn_d[3:1] <= { 3 {1'b0}};
     end else begin
        sync_updateEn_d[3:1] <= { sync_updateEn_d[2:1], sync_updateEn};
     end

   wire        fuse_if_ate_mode_tck;
   wire        fuse_if_burn_in_mode;
   wire        fuse_if_read_config ;
   wire        fuse_if_config_force;
   wire        fuse_if_run_mbist   ;
   wire        fuse_if_reset_mbist ;
   wire        fuse_if_reserve     ;
   wire [28:0] fuse_if_config_value;

   assign fuse_if_read_config_mode      = fuse_if_sync_read_config;

   always @(posedge ick)
     if (~trstb_ipclk_raw | updateEn_setup) begin
        fuse_if_sync_burn_in_mode                                  <= fuse_if_burn_in_mode                    ;
        fuse_if_sync_read_config                                   <= fuse_if_read_config                     ;
        fuse_if_sync_config_force                                  <= fuse_if_config_force                    ;
        fuse_if_sync_run_mbist                                     <= fuse_if_run_mbist                       ;
        fuse_if_sync_reset_mbist                                   <= fuse_if_reset_mbist                     ;
        fuse_if_sync_config_value[`FSECONF_USED__RNG_GC10] <= fuse_if_config_value[`FSECONF_USED__RNG_GC10];
     end else begin
        fuse_if_sync_burn_in_mode                                  <= fuse_if_sync_burn_in_mode                    ;
        fuse_if_sync_read_config                                   <= fuse_if_sync_read_config                     ;
        fuse_if_sync_config_force                                  <= fuse_if_sync_config_force                    ;
        fuse_if_sync_run_mbist                                     <= fuse_if_sync_run_mbist                       ;
        fuse_if_sync_reset_mbist                                   <= fuse_if_sync_reset_mbist                     ;
        fuse_if_sync_config_value[`FSECONF_USED__RNG_GC10] <= fuse_if_sync_config_value[`FSECONF_USED__RNG_GC10];
     end

  fuse_if_ate_mode_tdr_gc10 I_fuse_if_ate_mode_tdr    (
                                            .cdce                (updateEn_ate_mode), // this is not needed (hard tied)
                                            .fuse_if_ate_mode_tck(fuse_if_ate_mode_tck),
                                            /*AUTOINST*/
                                            // Outputs
                                            .Tdr_fuse_if_ate_mode_Tdo(Tdr_fuse_if_ate_mode_Tdo),
                                            // Inputs
                                            .ipclk                  (ipclk),
                                            .Tdr_Tck                (Tdr_Tck),
                                            .Tdr_Trstb              (Tdr_Trstb),
                                            .Tdr_Tdi                (Tdr_Tdi),
                                            .Tdr_Shift              (Tdr_Shift),
                                            .Tdr_Update             (Tdr_Update),
                                            .Tdr_Capture            (Tdr_Capture),
                                            .Tdr_fuse_if_ate_mode_En(Tdr_fuse_if_ate_mode_En));
  fuse_if_setup_tdr_gc10 I_fuse_if_setup_tdr    (
                                            .cdce               (updateEn_setup),
                                            /*AUTOINST*/
                                            // Outputs
                                            .fuse_if_config_force(fuse_if_config_force),
                                            .fuse_if_run_mbist   (fuse_if_run_mbist),
                                            .fuse_if_reset_mbist (fuse_if_reset_mbist),
                                            .fuse_if_reserve     (fuse_if_reserve),
                                            .fuse_if_config_value(fuse_if_config_value[28:0]),
                                            .fuse_if_read_config(fuse_if_read_config),
                                            .fuse_if_burn_in_mode(fuse_if_burn_in_mode),
                                            .Tdr_fuse_if_setup_Tdo(Tdr_fuse_if_setup_Tdo),
                                            // Inputs
                                            .ipclk              (ipclk),
                                            .Tdr_Tck            (Tdr_Tck),
                                            .Tdr_Trstb          (Tdr_Trstb),
                                            .Tdr_Tdi            (Tdr_Tdi),
                                            .Tdr_Shift          (Tdr_Shift),
                                            .Tdr_Update         (Tdr_Update),
                                            .Tdr_Capture        (Tdr_Capture),
                                            .Tdr_fuse_if_setup_En(Tdr_fuse_if_setup_En));
   //! unused fuse_if_reserve
   fuse_if_status_tdr_gc10 I_fuse_if_status_tdr (/*AUTOINST*/
                                            // Outputs
                                            .Tdr_fuse_if_status_Tdo(Tdr_fuse_if_status_Tdo),
                                            // Inputs
                                            .Tdr_Tck            (Tdr_Tck),
                                            .Tdr_Tdi            (Tdr_Tdi),
                                            .Tdr_Shift          (Tdr_Shift),
                                            .Tdr_Capture        (Tdr_Capture),
                                            .fuse_if_read_config_mode(fuse_if_read_config_mode),
                                            .fuse_if_info       (fuse_if_info[31:0]),
                                            .Tdr_fuse_if_status_En(Tdr_fuse_if_status_En));
  // <1500 TDR Code> =========================================== end

  // Add marker buffer for timing
  wire  prebuff_Wrck;
  aticlk_buf d0nt_BC1500_Wrck (.A(prebuff_Wrck), .Y(BC1500_Wrck));

  // clock mux
  assign  ate_mode =  fuse_if_ate_mode_tck;
  // DECDFXIP40-532 - Switching clock mux to Tdr_Tck needs to be glitch free, Add clock gater and delay to make timing easier.
  //   Make the clk_en window, at least 6 cycles narrower than mux sel. Giving timing 3 or more tck cycle margin on both rise and fall
  //   clk_en : __________|-----------|_______
  //   sel    : _______|-----------------|____
  wire      ate_mode_clk_en ;
  reg       ate_mode_mux_sel;
  reg [3:1] ate_mode_dly;
  wire      g_Tdr_Tck ;
  always @(posedge Tdr_Tck or negedge Tdr_Trstb)
    if (~Tdr_Trstb) begin
      ate_mode_dly[3:1] <= {3 {1'b0}};
      ate_mode_mux_sel <= 1'b0 ;
    end else begin
      casez ({ate_mode, &ate_mode_dly[3:1], |ate_mode_dly[3:1]})
         3'b000  : ate_mode_dly[3:1] <= 3'b000;
         3'b0?1  : ate_mode_dly[3:1] <= ate_mode_dly[3:1] - 3'b001; // decrement
         3'b10?  : ate_mode_dly[3:1] <= ate_mode_dly[3:1] + 3'b001; // increment
         3'b111  : ate_mode_dly[3:1] <= 3'b111;
         default : ate_mode_dly[3:1] <= ate_mode_dly[3:1];
      endcase // casez ({ate_mode, &ate_mode_dly[3:1], |ate_mode_dly[3:1]})

      case ({ate_mode_dly[3:1], ate_mode})
         4'b0001 : ate_mode_mux_sel <= 1'b1;
         4'b0000 : ate_mode_mux_sel <= 1'b0;
         default : ate_mode_mux_sel <= ate_mode_mux_sel;
      endcase // case ({ate_mode_dly[3:1], ate_mode})
    end
  assign ate_mode_clk_en = &{ate_mode_dly[3:1], ate_mode};
  ati_clock_gate I_g_Tdr_Tck (.clk_out(g_Tdr_Tck), .clk_src(Tdr_Tck), .clk_en(ate_mode_clk_en), .TE(1'b0));
  aticlk_mux2 I_BC1500_Wrck (.clk_o(prebuff_Wrck), .clk_i({g_Tdr_Tck, drClk}), .sel(ate_mode_mux_sel));

`ifdef ABV
 `include "fuse_if_assert_control.vh"
   //Verify: Free running wrck clock should not be required
   ovl_win_unchange #(
                      .severity_level (`OVL_ERROR),
                      .width (1),
                      .property_type (`OVL_ASSERT),
                      .msg ("Error: wrck should not be free running. Checked during bist operation."),
                      .coverage_level (`OVL_COVER_DEFAULT),
                      .clock_edge (`OVL_POSEDGE),
                      .reset_polarity (`OVL_ACTIVE_LOW),
                      .gating_type (`OVL_GATE_CLOCK) )
   assert_wrck_not_freerunning (
                                .clock (ipclk),
                                .reset (),
                                .enable (`FUSE_IF_ABV_PWROK_EN & 1'b0), // Disable until reviewed
                                .start_event (smsg_bist),
                                .test_expr (BC1500_Wrck),
                                .end_event (fuse_if_bist_done),
                                .fire () );

   //Verify: clk_sms > 10x wrck (except during FLOAD where wrck sped up for simulation)
   // SV-style:    assert_clk_sms_wrck_ratio_h:
   // SV-style:      assert property ( @(posedge ipclk) disable iff (!(`FUSE_IF_ABV_PWROK_EN & smsg_bist === 1'b1))
   // SV-style:                        $rose(BC1500_Wrck) |=> ##1 BC1500_Wrck[*4:$] ##1 !BC1500_Wrck);
   // SV-style:    assert_clk_sms_wrck_ratio_l:
   // SV-style:      assert property ( @(posedge ipclk) disable iff (!(`FUSE_IF_ABV_PWROK_EN & smsg_bist === 1'b1))
   // SV-style:                        $fell(BC1500_Wrck) |=> ##1 !BC1500_Wrck[*4:$] ##1 BC1500_Wrck);
   ovl_cycle_sequence #(
                        .severity_level(`OVL_ERROR),
                        .num_cks(5),
                        .necessary_condition(`OVL_TRIGGER_ON_MOST_PIPE),
                        .property_type(`OVL_ASSERT),
                        .msg("Error: clk_sms should be > 10x BC1500_Wrck"),
                        .coverage_level(`OVL_COVER_DEFAULT),
                        .clock_edge(`OVL_POSEDGE),
                        .reset_polarity(`OVL_ACTIVE_LOW),
                        .gating_type(`OVL_GATE_CLOCK ) )
                        assert_clk_sms_wrck_ratio_h (
                                              .clock(ipclk),
                                              .reset(resetb),
                                              .enable(`FUSE_IF_ABV_PWROK_EN & smsg_bist === 1'b1 & 1'b0), // Disable until reviewed
                                              .event_sequence({BC1500_Wrck
                                                               ,BC1500_Wrck
                                                               ,BC1500_Wrck
                                                               ,BC1500_Wrck
                                                               ,BC1500_Wrck}),
                                              .fire() );

   //Verify: Fuse busy and fuse done handshake check between rsmu and fuse_if
   //One or more cycles of fuse_grp_valid is followed by one or more cycles of repair_busy
  assert_repair_busy:
    assert property ( @(posedge ipclk) disable iff (!RSMU_RDFT_sms_fuse_grp_resetb | 1'b1)  // Disable until reviewed
                      !RSMU_RDFT_sms_fuse_grp_valid ##1 RSMU_RDFT_sms_fuse_grp_valid[*1:$]
                      |-> !RDFT_RSMU_sms_fuse_grp_repair_busy[*1:$] ##1 RDFT_RSMU_sms_fuse_grp_repair_busy[*1:$]);

`endif

endmodule // fuse_if_gc10
// =========================================================================================================
// setup for emacs
//Local Variables:
//verilog-library-directories:(".")
//verilog-library-flags:("")
//verilog-library-files:("fuse_if_tdr.v")
//End:
// =========================================================================================================
