//                              -*- Mode: Verilog -*-

////////////////////////////////////////////////////////////////////////////
/*
 *
 *  Trade secret of ATI Technologies, Inc.
 *  Copyright 2005, ATI Technologies, Inc., (unpublished)
 *
 *  All rights reserved.  This notice is intended as a precaution against
 *  inadvertent publication and does not imply publication or any waiver
 *  of confidentiality.  The year included in the foregoing notice is the
 *  year of creation of the work.
 *
 */
////////////////////////////////////////////////////////////////////////////
//`include "dft_clk_cntl_features.vh"


module gater_table_gc10 
# (
parameter DFT_CLK_CNTL_PULSE_CNT = 5
)
(
 input wire Scan_Mode,
 input wire Tile_Shift_Clk,
 input wire Gater_Flop_ScanIn,
 output wire Gater_Flop_ScanOut,
 input wire Tile_ScanEn,
 output wire [DFT_CLK_CNTL_PULSE_CNT-1:0] gater_table_ff
 
 );


   // TBD- LOS last shift cycle won't get the clock.
   reg [DFT_CLK_CNTL_PULSE_CNT-1:0] sdff_mrk;
   
   // All gater flops on shift clock. No need for neg-edge flop
   //reg sdff_neg_mrk;
   
   always @ (posedge Tile_Shift_Clk or negedge Scan_Mode)
     begin
	if (~Scan_Mode) begin // CDC waiver
	   sdff_mrk[DFT_CLK_CNTL_PULSE_CNT-1:0] <= {(DFT_CLK_CNTL_PULSE_CNT-1){1'h0}};
	end else if(Tile_ScanEn) begin
	   sdff_mrk[DFT_CLK_CNTL_PULSE_CNT-1:0] <= {Gater_Flop_ScanIn, sdff_mrk[DFT_CLK_CNTL_PULSE_CNT-1:1]};
	end else begin
	   sdff_mrk[DFT_CLK_CNTL_PULSE_CNT-1:0] <= sdff_mrk[DFT_CLK_CNTL_PULSE_CNT-1:0];
	end
     end
   
   //assign Gater_Flop_ScanOut = sdff_neg_mrk;
   assign Gater_Flop_ScanOut = sdff_mrk[0];
   
   //only load gater table to pulse counter after last shift cycle
   // False path from gater_table_ff to pulse_cnt.
   generate
      genvar i;
      for (i=0; i<DFT_CLK_CNTL_PULSE_CNT; i=i+1) begin : mux_gater_table_loop
	 
	 ati_mux2 mux_gater_table  
	   (
	    .i0(sdff_mrk[i]),      
	    .i1(1'b0),
	    .s(Tile_ScanEn),
	    .z(gater_table_ff[i]));
      end
   endgenerate
   
endmodule // gater_table


