//------------------------------------------------------------------------
//
// didt_edc_cntl_core.v  -- Di/Dt and EDC Controller Core
//
//
// Copyright:   Trade secret of AMD, Inc.
//              Copyright 2010, AMD, Inc., (unpublished)
//------------------------------------------------------------------------

module gc_didt_edc_cntl_core
  #(parameter UNITS=4,
    NUM_SA = 2,
    CLK_GATE_EN = 1,
    PHASE_SIZE= 2, 
    POWER_SIZE = 16,
	EDC_TRRESHOLD_SIZE = 16,
    WEIGHT_SIZE = 8,
    WEIGHT_COUNT = 12,
    MAX_POWER_DELTA_SIZE = 14,
    SHORT_TERM_INTERVAL_SIZE = 10,
    LONG_TERM_INTERVAL_RATIO_SIZE = 4,
	//New DIDT parameter
	DIDT_STALL_PATTERN_BIT_NUMS_SIZE = 8,
	DIDT_STALL_INST_NUMS = 8,
    //EDC Paramter
    STALL_PATTERN_BIT_NUMS_SIZE = 8,
    MAX_DELAY_NUMS = 64,
    EDC_STALL_INST_NUMS = 8,
    EDC_STALL_DELAY_SIZE = 6,
    EDC_TIMER_WIDTH = 14)
  (
   input Cpl_GFXCLK,
   input oCpl_GFXCLK,
   input rst,

   input dft_shift_clk,
   input Dft_Clk_Gater_Aon,
   input Dft_Clk_Gater_En,

   input [UNITS*NUM_SA-1:0] credit_consumed,
   output wire stall,
   output wire power_is_hi,
   input [POWER_SIZE-1:0] hi_power_threshold,
   input wire auto_mpd_enable,
   input [2:0] RLC_DIDT_force_stall,
   input rlc_force_stall_en,
   input rlc_stall_level_sel,

   input [POWER_SIZE-1:0] min_power,
   input [POWER_SIZE-1:0] max_power,
   input [PHASE_SIZE-1:0] phase_offset,
   input [MAX_POWER_DELTA_SIZE-1:0] max_power_delta,
   input [SHORT_TERM_INTERVAL_SIZE-1:0] short_term_interval_size,
   input [LONG_TERM_INTERVAL_RATIO_SIZE-1:0] long_term_interval_ratio,
   input [3:0] mpd_ratio_scale_level1,
   input [3:0] mpd_ratio_scale_level2,
   input [3:0] mpd_ratio_scale_level3,
   input [3:0] mpd_ratio_scale_level4,
   input [3:0] mpd_scale_level0,
   input [3:0] mpd_scale_level1,
   input [3:0] mpd_scale_level2,
   input [3:0] mpd_scale_level3,

   input [WEIGHT_SIZE-1:0] weight_0,
   input [WEIGHT_SIZE-1:0] weight_1,
   input [WEIGHT_SIZE-1:0] weight_2,
   input [WEIGHT_SIZE-1:0] weight_3,
   input [WEIGHT_SIZE-1:0] weight_4,
   input [WEIGHT_SIZE-1:0] weight_5,
   input [WEIGHT_SIZE-1:0] weight_6,
   input [WEIGHT_SIZE-1:0] weight_7,
   input [WEIGHT_SIZE-1:0] weight_8,
   input [WEIGHT_SIZE-1:0] weight_9,
   input [WEIGHT_SIZE-1:0] weight_10,
   input [WEIGHT_SIZE-1:0] weight_11,

   //New DIDT
   output[3:0] power_level,
   input[2:0] stall_level,
   input[4:0] didt_trigger_throttle_lowbit,
   input[4:0] didt_power_level_lowbit,
   input[1:0] throttle_policy,
   input gc_didt_enable,
   input gc_didt_clk_en_override,
   input [DIDT_STALL_PATTERN_BIT_NUMS_SIZE-1:0] didt_stall_pattern_bit_nums,
   input[14:0]	didt_stall_pattern1,
   input[14:0]	didt_stall_pattern2,
   input[14:0]	didt_stall_pattern3,
   input[14:0]	didt_stall_pattern4,
   input[14:0]	didt_stall_pattern5,
   input[14:0]	didt_stall_pattern6,
   input[14:0]	didt_stall_pattern7,

   input stall_event_enable,
   input stall_event_counter_clear,
   output reg [31:0] stall_event_counter,


   input didt_en,
   input didt_clk_en_override,
   input TEST_CLK_EN,
   input didt_soft_rst,
   input didt_force_stall,
   input didt_stall_delay_en,
   output didt_dbg_stall_out,
   output [2:0] didt_throttle_level,
   output       didt_fsm_state,
   input	didt_throttle_mode,

   //DIDT throttle level from BPM DDD
   input[2:0]	PSM_didt_throttle_level,

   // EDC Controller Interface
   input edc_en,
   input edc_sw_rst,
   input edc_clk_en_override,
   input edc_force_stall,
   input [31:0]	edc_threshold,
   input [4:0] edc_trigger_throttle_lowbit,
   input [STALL_PATTERN_BIT_NUMS_SIZE-1:0] edc_stall_pattern_bit_nums, 
   input [14:0] edc_stall_pattern1,
   input [14:0] edc_stall_pattern2,
   input [14:0] edc_stall_pattern3,
   input [14:0] edc_stall_pattern4,
   input [14:0] edc_stall_pattern5,
   input [14:0] edc_stall_pattern6,
   input [14:0] edc_stall_pattern7,
   input [EDC_STALL_INST_NUMS*EDC_STALL_DELAY_SIZE-1:0] edc_stall_delay,
   input edc_allow_write_pwrdelta,
   input edc_grbm_wr_pwrdelta_en,
   input [31:0] edc_grbm_wr_pwrdelta_data,
   input        edc_grbm_clear_ov_counter,
   input		edc_grbm_clear_rolling_power_delta_ov,

   output       edc_fsm_state,
   output [2:0] edc_throttle_level,
   output reg   edc_rolling_power_delta_ov,
   output [31:0] edc_rolling_power_delta,
   output reg [15:0] edc_throttle_level_ov_counter,

   output [EDC_STALL_INST_NUMS-1:0]  edc_stall,
   output							edc_dbg_stall_out,

   input		gc_edc_en,
   input [2:0]	gc_edc_stall_level,
   input [1:0]	gc_edc_stall_policy,

   input 		gc_didt_level_comb_en,
   input 		se_didt_level_comb_en,
   input 		didt_qualify_stall_en,
   input[1:0] 	didt_stall_sel,

   input gc_edc_level_comb_en,
   input se_edc_level_comb_en,
   input edc_level_mode_sel,
   input edc_level_comb_adapt_mode_en,
   input gc_edc_stall_en,
   input pcc_stall_en,
   input pwrbrk_stall_en,
   input gc_se_stall,

   output[31:0]	edc_pcc_perf_counter,
   input		rd_edc_pcc_perf_counter,
   input		gc_edc_only_mode,

   input [EDC_TIMER_WIDTH-1:0] edc_timer_period

   );
   
   localparam ACT_WIDTH = UNITS*NUM_SA;
   localparam ACT_WIDTH_LOG2 = ACT_WIDTH > 32 ? 6 : ACT_WIDTH > 16 ? 5 : ACT_WIDTH > 8 ? 4 : ACT_WIDTH > 4 ? 3 : ACT_WIDTH > 2 ? 2 : 1;
   localparam LT_POWER_SIZE = POWER_SIZE+LONG_TERM_INTERVAL_RATIO_SIZE;
   localparam CREDIT_SIZE = ACT_WIDTH_LOG2+WEIGHT_SIZE;
   


   reg       stall_gated;

   wire       core_clk_gated;
   wire [WEIGHT_COUNT*WEIGHT_SIZE-1:0] weights;

   reg  combined_reset;

   // current time
   reg  [POWER_SIZE-1:0] 		current_time_in_short_term_interval;


   // flop inputs
   reg [ACT_WIDTH_LOG2+WEIGHT_SIZE-1:0] credit_incr, credit_incr_q;

   // keep a copy of the long term ratio, it wants the previous short term interval's value when doing its updates
   reg [LONG_TERM_INTERVAL_RATIO_SIZE-1:0] long_term_interval_ratio_prev;

   // power consumed in the current short term interval, not counting the current clock
   reg [POWER_SIZE-1:0] short_term_power_consumed;

   // power consumed in the current short term interval, including the current clock
	wire [POWER_SIZE:0] next_short_term_power_consumed = short_term_power_consumed + credit_incr_q;

   // max power allowed to be used for current short term interval
   reg [POWER_SIZE-1:0] short_term_power_allocated;

   // approximate power consumed over the last "LONG_TERM_INTERVAL_RATIO" short term intervals
   reg [LT_POWER_SIZE-1:0] long_term_power_average;

   // update long term average based on last short term interval activity
   wire [LT_POWER_SIZE-1:0] next_long_term_power_average;

   wire [POWER_SIZE:0] next_short_term_power_allocated_no_clamp ;
   wire [LT_POWER_SIZE-1:0] next_short_term_power_allocated_no_clamp_expanded ;
   wire [POWER_SIZE-1:0] next_short_term_power_allocated_clamped ;


   assign weights[WEIGHT_COUNT*WEIGHT_SIZE-1:0] = {weight_11,weight_10,weight_9,weight_8,weight_7,weight_6,weight_5,weight_4,weight_3,weight_2,weight_1,weight_0};

   // combined reset
   always @(posedge oCpl_GFXCLK) begin
        combined_reset <= rst | didt_soft_rst ;
   end


   	reg[2:0] auto_mpd_ratio;

   //STI counter
   always @(posedge oCpl_GFXCLK) begin
     if (combined_reset) begin
       current_time_in_short_term_interval <= 'd0;
	 end else if (~didt_en) begin
       current_time_in_short_term_interval <= 'd0;
	 end else if (current_time_in_short_term_interval >= short_term_interval_size-1) begin
       		current_time_in_short_term_interval <= 'd0;
     end else begin
       		current_time_in_short_term_interval <= current_time_in_short_term_interval + 1;
		end
	end

   //Assert STI_done_pulse for 1 STI_clk cycle at the configured phase offset
   reg STI_done_pulse;
   always @(posedge oCpl_GFXCLK) begin
     if (combined_reset) begin
       STI_done_pulse <= 1'b0;
	 end else if(~didt_en) begin
       STI_done_pulse <= 1'b0;
     end else if (current_time_in_short_term_interval == ({{(POWER_SIZE-SHORT_TERM_INTERVAL_SIZE){1'b0}},short_term_interval_size} >> PHASE_SIZE) * {{(POWER_SIZE-PHASE_SIZE){1'b0}},((phase_offset) % (1 << PHASE_SIZE))}) begin 
       STI_done_pulse <= 1'b1;
     end else begin
       STI_done_pulse <= 1'b0; 
     end
   end

   //CLK_GATE_EN
   wire clk_en;
   if (CLK_GATE_EN == 1) begin
      assign clk_en = combined_reset | didt_en | didt_clk_en_override;
   end else begin
      assign clk_en = 1'b1;      
   end

   gfx_cgtt_local_dft didt_clkgater (
     .Cpl_GFXCLK  		(Cpl_GFXCLK),
     .core_busy	   		(clk_en),
     .oclk  				(core_clk_gated),
		 .dft_clk_gate_ctrl	({Dft_Clk_Gater_En, Dft_Clk_Gater_Aon}),
		 .dft_shift_clk		(dft_shift_clk)
   );

   // sanity check inputs
   // psl credit_consumed_isunknown: assert (never (isunknown(^credit_consumed) && combined_reset===0)) @(posedge oCpl_GFXCLK);

   // calculate total credidts consumed on the current SCLK cycle
   integer i, p;
   always @*
     begin
        credit_incr = 'd0;
        for (p = 0; p < NUM_SA; p=p+1) begin
          for (i = 0; i < UNITS; i=i+1) begin
			if(credit_consumed[i+p*UNITS])
			credit_incr = credit_incr + (credit_consumed[i+p*UNITS] ? weights[8*i +: 8] : 8'd0);
			else
			credit_incr = credit_incr;
          end
	end
     end
   
	always @(posedge oCpl_GFXCLK) begin
		if (combined_reset) begin
			credit_incr_q <= {(CREDIT_SIZE){1'b0}};
		end
		else if(didt_en || edc_en || edc_clk_en_override)begin
			credit_incr_q <= credit_incr;
		end
		else begin
			credit_incr_q <= {(CREDIT_SIZE){1'b0}};
		end
	end

   reg power_level_overflow;
   integer m;
   always @*
    begin
        power_level_overflow = 1'b0;
        for (m = 0; m < CREDIT_SIZE; m=m+1) begin
          if(m > didt_power_level_lowbit+3)
           power_level_overflow = power_level_overflow | credit_incr_q[m];
         end
     end

	assign power_level[0] = power_level_overflow ? 1'b1 : credit_incr_q[didt_power_level_lowbit];
	assign power_level[1] = power_level_overflow ? 1'b1 : credit_incr_q[didt_power_level_lowbit+1];
	assign power_level[2] = power_level_overflow ? 1'b1 : credit_incr_q[didt_power_level_lowbit+2];
	assign power_level[3] = power_level_overflow ? 1'b1 : credit_incr_q[didt_power_level_lowbit+3];

   //always @(posedge oCpl_GFXCLK) begin
   //   credit_incr_q <= (combined_reset || ~(didt_en || edc_en || edc_clk_en_override)) ? {(CREDIT_SIZE+1){1'b0}} : credit_incr;
   //end

   assign next_long_term_power_average  = long_term_power_average - (long_term_power_average >> long_term_interval_ratio_prev) + (next_short_term_power_consumed > short_term_power_allocated ? {{(LT_POWER_SIZE-POWER_SIZE){1'b0}},short_term_power_allocated} : {{(LT_POWER_SIZE-POWER_SIZE){1'b0}},next_short_term_power_consumed});

   // psl long_term_power_overflow : assert never {next_long_term_power_average[LT_POWER_SIZE-1]} @(posedge core_clk_gated);

   // figure out allocation for next short term interval, use long_term_power_average instead of next_long_term_power_average for timing reasons.  This means power will be allocated based on the long-term average from one short term interval ago.
   // for max power ramp, the sequence of power allocation values will be the same as using the "current" version just 1 short term interval delayed

   //debug feature, dynamic max_power_delta setting based on last STI power consumption. 3-bits accuracy
   reg [MAX_POWER_DELTA_SIZE-1:0] max_power_delta_dynamic;
   wire[MAX_POWER_DELTA_SIZE-1:0] max_power_delta_dynamic_t;

   //max_power_delta_dynamic should never be 0
   assign max_power_delta_dynamic_t = (auto_mpd_ratio >= 3'h4) ? max_power_delta :
   									  (auto_mpd_ratio == 3'h3) ? ((max_power_delta >> 4)*mpd_scale_level3) :
   									  (auto_mpd_ratio == 3'h2) ? ((max_power_delta >> 4)*mpd_scale_level2) :
   									  (auto_mpd_ratio == 3'h1) ? ((max_power_delta >> 4)*mpd_scale_level1) :
   									  (auto_mpd_ratio == 3'h0) ? ((max_power_delta >> 4)*mpd_scale_level0) :
									  max_power_delta;

   //Fix timing violation
   //assign max_power_delta_dynamic = (max_power_delta_dynamic_t == 0) ? 1 : max_power_delta_dynamic_t;
   always @(posedge core_clk_gated) begin
     if (combined_reset) begin
       max_power_delta_dynamic <= 'd0;
     end else if (max_power_delta_dynamic_t == 0) begin
       max_power_delta_dynamic <= 'd1;
     end else begin
       max_power_delta_dynamic <= max_power_delta_dynamic_t;
     end
   end

   assign next_short_term_power_allocated_no_clamp_expanded = (long_term_power_average >> long_term_interval_ratio) + {{(LT_POWER_SIZE-MAX_POWER_DELTA_SIZE){1'b0}},(auto_mpd_enable ? max_power_delta_dynamic : max_power_delta)};
   assign next_short_term_power_allocated_no_clamp = next_short_term_power_allocated_no_clamp_expanded[POWER_SIZE:0];

   assign next_short_term_power_allocated_clamped  = (next_short_term_power_allocated_no_clamp > max_power) ? max_power : (next_short_term_power_allocated_no_clamp < min_power ? min_power : next_short_term_power_allocated_no_clamp);

always @(posedge core_clk_gated) begin
	if (combined_reset) begin
        short_term_power_consumed <= 'd0;
        short_term_power_allocated <= min_power;
        long_term_power_average <= 'd0;
        long_term_interval_ratio_prev <= long_term_interval_ratio;
		auto_mpd_ratio <= 3'b111;
		//next_max_power_delta_dynamic <= {MAX_POWER_DELTA_SIZE{1'b1}};

	 end else if (STI_done_pulse) begin //this is the end of the STI

 	     if (next_short_term_power_consumed > short_term_power_allocated) begin
          	// assign any of the power overage in this short term interval to the next interval
          	short_term_power_consumed <= (next_short_term_power_consumed - short_term_power_allocated) > {POWER_SIZE{1'b1}} ? {POWER_SIZE{1'b1}} : (next_short_term_power_consumed - short_term_power_allocated);  //saturate if next_short_term_power_consumed is too big and short_term_power_allocated is too small.
		  	auto_mpd_ratio <= 	(next_short_term_power_consumed - short_term_power_allocated) > short_term_power_allocated ? 3'h0 :
		  						(next_short_term_power_consumed - short_term_power_allocated) > ((short_term_power_allocated >> 4)*mpd_ratio_scale_level1) ? 3'h1 :
		  						(next_short_term_power_consumed - short_term_power_allocated) > ((short_term_power_allocated >> 4)*mpd_ratio_scale_level2) ? 3'h2 :
		  						(next_short_term_power_consumed - short_term_power_allocated) > ((short_term_power_allocated >> 4)*mpd_ratio_scale_level3) ? 3'h3 :
		  						(next_short_term_power_consumed - short_term_power_allocated) > ((short_term_power_allocated >> 4)*mpd_ratio_scale_level4) ? 3'h4 :
								3'b111;
        end else begin
          short_term_power_consumed <= 'd0;
		  auto_mpd_ratio <= 3'b111;
        end

        // ST_power_alloc = MIN(max_power, MAX(min_power, (LT_avg/LT _ratio + max_power_delta)))
        short_term_power_allocated <= next_short_term_power_allocated_clamped;

        // LT_power_avg =  LT_power_avg * (LT_ratio-1 / LT _ratio) + ST_power_consumed
        long_term_power_average <= next_long_term_power_average[LT_POWER_SIZE-1:0];

        // Update LT interval ratio in case it was updated
        long_term_interval_ratio_prev <= long_term_interval_ratio;
      
   end else begin //to this on every SCLK cycle of the STI except on the last cycle
        	// saturate power consumed
       		if (next_short_term_power_consumed > {POWER_SIZE {1'b1}}) begin
          		short_term_power_consumed <= {POWER_SIZE {1'b1}};
       		// accumulate the power consumed
        	end else begin
          		short_term_power_consumed <= next_short_term_power_consumed;
        	end
   	   end
end

   assign power_is_hi = ((long_term_power_average >> long_term_interval_ratio) >= hi_power_threshold) ? 1 : 0;
   
   wire next_stall = (next_short_term_power_consumed >= short_term_power_allocated) & didt_en;
   wire next_stall_rlc_sync = rlc_force_stall_en ? ((next_stall | (|RLC_DIDT_force_stall)) & didt_en) : next_stall;

   always @(posedge core_clk_gated) begin
      stall_gated <= combined_reset ? 1'b0 : next_stall_rlc_sync;
   end

   wire legacy_stall_qualified = didt_qualify_stall_en ? stall_gated & (|stall_level) : stall_gated;

   wire new_stall;
   wire new_stall_qualified = didt_qualify_stall_en ? new_stall & (|stall_level) : new_stall;

   reg stall_qualified;
   always @ (*) begin
		case (didt_stall_sel)
			2'b00	:	stall_qualified = legacy_stall_qualified;
			2'b01	:	stall_qualified = legacy_stall_qualified | new_stall;
			2'b10	:	stall_qualified = new_stall_qualified;
			default	:	stall_qualified = legacy_stall_qualified;
		endcase
	end

   wire didt_stall = didt_en ? (didt_force_stall ? new_stall : (gc_didt_enable ? stall_qualified : stall_gated)) : 1'b0;
   assign didt_dbg_stall_out = didt_stall;

   // didt_stall_delay_en != 1'b0, share EDC delay logic, need clamp stall to '0'
   //assign stall = didt_en ? (didt_stall & (~didt_stall_delay_en)) : 0;
	assign stall = didt_en ? ((didt_throttle_mode == 0)? PSM_didt_throttle_level[0] : (didt_stall & (~didt_stall_delay_en)) ) : 0;

   // psl power_throttle_stall: cover {stall && combined_reset ===0} @(posedge oCpl_GFXCLK);


//***************************************************************************//
//                          New DIDT stall generation                         //
//***************************************************************************//
   localparam DIDT_IDLE = 1'b0;
   localparam DIDT_STALL = 1'b1;

      //CLK_GATE_EN
   wire didt_clk_en_2;
   if (CLK_GATE_EN == 1) begin
      assign didt_clk_en_2 = combined_reset || (didt_en && (gc_didt_enable || didt_force_stall)) || gc_didt_clk_en_override;    
   end else begin
      assign didt_clk_en_2 = 1'b1;      
   end

	wire core_clk_gated_2;

  gfx_cgtt_local_dft didt_clkgater_2 (
    	.Cpl_GFXCLK  		(Cpl_GFXCLK),
    	.core_busy   		(didt_clk_en_2),
    	.oclk  				(core_clk_gated_2),
    	.dft_clk_gate_ctrl	({Dft_Clk_Gater_En, Dft_Clk_Gater_Aon}),
    	.dft_shift_clk		(dft_shift_clk)
  );

	
	wire [POWER_SIZE:0] power_delta;
	assign power_delta = (next_short_term_power_consumed > short_term_power_allocated)? (next_short_term_power_consumed - short_term_power_allocated) : {(POWER_SIZE+1){1'b0}};

   reg didt_throttle_level_overflow;
   integer n;
   always @*
    begin
        didt_throttle_level_overflow = 1'b0;
        for (n = 0; n < POWER_SIZE+1; n=n+1) begin
          if(n > didt_trigger_throttle_lowbit+2)
           didt_throttle_level_overflow = didt_throttle_level_overflow | power_delta[n];
         end
     end

	wire [2:0] didt_throttle_level_pre;
	assign didt_throttle_level_pre [0] = se_didt_level_comb_en ? (didt_throttle_level_overflow ? 1'b1 : power_delta[didt_trigger_throttle_lowbit]) : 1'b0;
	assign didt_throttle_level_pre [1] = se_didt_level_comb_en ? (didt_throttle_level_overflow ? 1'b1 : power_delta[didt_trigger_throttle_lowbit+1]) : 1'b0;
	assign didt_throttle_level_pre [2] = se_didt_level_comb_en ? (didt_throttle_level_overflow ? 1'b1 : power_delta[didt_trigger_throttle_lowbit+2]) : 1'b0;

	wire [3:0] didt_throttle_level_comb; //Extra MSB for overflow
	assign didt_throttle_level_comb = gc_didt_enable ? (gc_didt_level_comb_en ? (didt_throttle_level_pre + ((|stall_level) ? (stall_level >> (3- throttle_policy)) : 3'd0)) : didt_throttle_level_pre) : 4'd0;

	wire [2:0] didt_throttle_level_comb_clamped_pre = (didt_throttle_level_comb[3]) ? 3'b111 : didt_throttle_level_comb[2:0];
	wire [2:0] didt_throttle_level_mux = (RLC_DIDT_force_stall >= didt_throttle_level_comb_clamped_pre) ? RLC_DIDT_force_stall : didt_throttle_level_comb_clamped_pre;
	wire [2:0] didt_throttle_level_comb_clamped = (gc_didt_enable && rlc_force_stall_en) ? (rlc_stall_level_sel ? didt_throttle_level_mux : RLC_DIDT_force_stall) : didt_throttle_level_comb_clamped_pre;

	reg[14:0]	didt_stall_pattern;
	//didt stall pattern
	always @*
	begin
		case (didt_throttle_level_comb_clamped)
			3'b000: didt_stall_pattern = 15'b0;
			3'b001:	didt_stall_pattern = didt_stall_pattern1; 
			3'b010:	didt_stall_pattern = didt_stall_pattern2;
			3'b011:	didt_stall_pattern = didt_stall_pattern3;
			3'b100:	didt_stall_pattern = didt_stall_pattern4;
			3'b101:	didt_stall_pattern = didt_stall_pattern5;
			3'b110:	didt_stall_pattern = didt_stall_pattern6;
			3'b111:	didt_stall_pattern = didt_stall_pattern7;
		endcase
	end

	reg next_didt_fsm_state, curr_didt_fsm_state;
	reg [DIDT_STALL_PATTERN_BIT_NUMS_SIZE-1:0] didt_stall_counter_d, didt_stall_counter_q;
	reg[14:0] didt_stall_pattern_d, didt_stall_pattern_q; 
	reg	didt_stall_q;
	reg didt_stall_running;
	reg [2:0] didt_throttle_level_q, didt_throttle_level_d;
	
	//FSM
	always @*
	begin
		next_didt_fsm_state = curr_didt_fsm_state;
		didt_stall_running = 1'b0;
		didt_stall_counter_d = didt_stall_counter_q;
		didt_stall_pattern_d = didt_stall_pattern_q;
		didt_throttle_level_d = didt_throttle_level_q;

		case(curr_didt_fsm_state)
			DIDT_IDLE:
				begin 
                  //didt force stall
                  if(didt_force_stall) begin
                        next_didt_fsm_state = DIDT_STALL;
                        didt_stall_running = 1'b1;
                        didt_stall_counter_d = didt_stall_pattern_bit_nums; //load counter max value 
                        didt_stall_pattern_d = didt_stall_pattern1; //force stall pattern reuse stall pattern1
                        didt_throttle_level_d = 3'b001; //reuse stall pattern1, so throttle_level = 1
                  end 
                  //didt normal stall                
                  else if(didt_throttle_level_comb_clamped != 3'b0) begin
						next_didt_fsm_state = DIDT_STALL;
						didt_stall_running = 1'b1;
						didt_stall_counter_d = didt_stall_pattern_bit_nums;
						didt_stall_pattern_d = didt_stall_pattern;
						didt_throttle_level_d = didt_throttle_level_comb_clamped;
				  end
				end

			DIDT_STALL:
				begin
				   didt_stall_running = 1'b1;

                   //didt force stall
                   if(didt_force_stall) begin
                     didt_throttle_level_d = 3'b001; //reuse stall pattern1, so throttle_level = 1

                     if(didt_stall_counter_q == {(STALL_PATTERN_BIT_NUMS_SIZE){1'b0}}) begin
                       didt_stall_counter_d = didt_stall_pattern_bit_nums; //load counter max value 
                       didt_stall_pattern_d = didt_stall_pattern1; //force stall pattern reuse stall pattern1
                     end
                     else begin
                       didt_stall_counter_d = didt_stall_counter_q - 1'b1;
                       didt_stall_pattern_d = {1'b0,didt_stall_pattern_q[14:1]}; // shift stall pattern to serially out stall signal
                     end
                   end
                   // normal stall
                   else begin
					 if(didt_stall_counter_q == {(STALL_PATTERN_BIT_NUMS_SIZE){1'b0}}) begin
						if(didt_throttle_level_comb_clamped == 3'b0) begin
							next_didt_fsm_state = DIDT_IDLE;
							didt_stall_pattern_d = 15'b0;
						end
						else begin
							next_didt_fsm_state = DIDT_STALL;
							didt_stall_counter_d = didt_stall_pattern_bit_nums;
							didt_stall_pattern_d = didt_stall_pattern;
						end
						didt_throttle_level_d = didt_throttle_level_comb_clamped;
					 end
				 	 else begin
						next_didt_fsm_state = DIDT_STALL;
						didt_stall_counter_d = didt_stall_counter_q - 1'b1;
						didt_stall_pattern_d = {1'b0, didt_stall_pattern_q[14:1]};
					 end
                   end
				end
		endcase
	end

	always @(posedge core_clk_gated_2) begin
		if(combined_reset)
			curr_didt_fsm_state <= DIDT_IDLE;
		else if(gc_didt_enable || didt_force_stall) begin
				curr_didt_fsm_state <= next_didt_fsm_state;
			end
	end

	assign didt_fsm_state = curr_didt_fsm_state;

	always @(posedge core_clk_gated_2) begin
		if(combined_reset) begin
			didt_stall_counter_q <= 'd0;
			didt_stall_pattern_q <= 'd0;
		end
		else if(didt_stall_running) begin
			didt_stall_counter_q <= didt_stall_counter_d;
			didt_stall_pattern_q <= didt_stall_pattern_d;
		end
	end

    assign didt_throttle_level = didt_throttle_level_q;

	always @(posedge core_clk_gated_2) begin
		if(combined_reset) begin
			didt_stall_q <= 'd0;
			didt_throttle_level_q <= 'd0;
		end
		else if (gc_didt_enable || didt_force_stall)begin
			didt_stall_q <= didt_stall_pattern_d[0];
			didt_throttle_level_q <= didt_throttle_level_d;
		end
	end

	assign new_stall = (gc_didt_enable || didt_force_stall) ? didt_stall_q : 0;


   //stall event counter, will count how many cycles stall is asserted
   always @(posedge core_clk_gated) begin
   	if(combined_reset) begin
		stall_event_counter <= 'd0;
	end
	else if(stall_event_counter_clear) begin
		stall_event_counter <= 'd0;
	end
	else if(stall_event_enable && stall)begin
		stall_event_counter <= (stall_event_counter == 32'hffffffff) ? 32'hffffffff : (stall_event_counter + 1'b1);
	end
   end


//***************************************************************************//
//                          EDC controller core                              //
//***************************************************************************//

   localparam POWERDELTA_SIZE = CREDIT_SIZE + 6;
   localparam EDC_IDLE = 1'b0;
   localparam EDC_STALL = 1'b1;

   wire	[CREDIT_SIZE-1:0] edc_threshold_clamped;
   assign edc_threshold_clamped = edc_threshold[CREDIT_SIZE-1:0];

   wire  [CREDIT_SIZE-1 : 0]     percyclepowerproxy;
   wire  [POWERDELTA_SIZE : 0]   rollingpowerdelta;
   wire  [POWERDELTA_SIZE : 0]   rollingpowersum;
   wire  [POWERDELTA_SIZE-1 : 0] rollingpowerdelta_d; 
   reg   [POWERDELTA_SIZE-1 : 0] rollingpowerdelta_q; 

   reg        next_fsm_state,curr_fsm_state;
   reg [STALL_PATTERN_BIT_NUMS_SIZE-1:0]  stall_counter_d,stall_counter_q;
   reg [14:0] stall_pattern_d,stall_pattern_q,stall_pattern;
   reg        edc_stall_q;
   reg [EDC_STALL_INST_NUMS-1:0]  edc_pipe_stall_d,edc_pipe_stall_q;
   reg [EDC_STALL_DELAY_SIZE-1:0] edc_delay; 
   // MAX_DELAY_NUMS means max delay cycle nums, 
   //   For Example: 2 SE, 1 SH/SE, 8 CU/SH, total CU nums is 16, 
   //                to get a average 4 cycles interval between any 2 CU, total need max 16*4 = 64 cycles delay
   reg [MAX_DELAY_NUMS-1-1:0] edc_delayed_stall_q;
   reg        edc_stall_running;
   wire [2:0] throttle_level;
   reg        throttle_level_overflow;
   reg [2:0]  edc_throttle_level_q,edc_throttle_level_d;
   //reg [15:0] edc_throttle_level_ov_counter;


   wire       edc_gated_clk;

   // high valid reset
   wire edc_rst = rst || edc_sw_rst ;

   wire edc_clk_en;
   if (CLK_GATE_EN == 1) begin
      assign edc_clk_en = edc_rst | edc_en | edc_clk_en_override | (edc_allow_write_pwrdelta && edc_grbm_wr_pwrdelta_en) | edc_grbm_clear_ov_counter;
   end else begin
      assign edc_clk_en = 1'b1;
   end

   //CLK_GATE_EN
   gfx_cgtt_local_dft edc_clkgater (
      .Cpl_GFXCLK  		(Cpl_GFXCLK),
      .core_busy   		(edc_clk_en),
      .oclk  				(edc_gated_clk),
      .dft_clk_gate_ctrl	({Dft_Clk_Gater_En, Dft_Clk_Gater_Aon}),
      .dft_shift_clk		(dft_shift_clk)
   );

   // operation isolation
   assign percyclepowerproxy = (edc_en || edc_clk_en_override) ? credit_incr_q : {CREDIT_SIZE{1'b0}}; 

   assign rollingpowersum = rollingpowerdelta_q + percyclepowerproxy;

   // underflow clamp 
   assign rollingpowerdelta = (rollingpowersum < {{(POWERDELTA_SIZE-CREDIT_SIZE+1){1'b0}},edc_threshold_clamped}) ? {(POWERDELTA_SIZE+1){1'b0}} :  rollingpowersum - edc_threshold_clamped;

   // overflow clamp 
   assign rollingpowerdelta_d = rollingpowerdelta[POWERDELTA_SIZE] ? {POWERDELTA_SIZE{1'b1}} : rollingpowerdelta[POWERDELTA_SIZE-1 : 0];
   //assign edc_rolling_power_delta_ov = rollingpowerdelta[POWERDELTA_SIZE];

   always @(posedge edc_gated_clk) begin
   	if(edc_rst) begin
		edc_rolling_power_delta_ov <= 'd0;
	end
	else if(edc_grbm_clear_rolling_power_delta_ov) begin
		edc_rolling_power_delta_ov <= 'd0;
	end 
	else if(rollingpowerdelta[POWERDELTA_SIZE]) begin
		edc_rolling_power_delta_ov <= 1'b1;
	end
   end

   // 
   always @(posedge edc_gated_clk) begin
      if(edc_rst)
        rollingpowerdelta_q <= {POWERDELTA_SIZE{1'b0}};
      else if(edc_allow_write_pwrdelta && edc_grbm_wr_pwrdelta_en)
        rollingpowerdelta_q <= edc_grbm_wr_pwrdelta_data[POWERDELTA_SIZE-1:0]; 
      else
        rollingpowerdelta_q <= rollingpowerdelta_d; 
   end
   assign edc_rolling_power_delta = {{(32-POWERDELTA_SIZE){1'b0}},rollingpowerdelta_q}; //TODO: be careful, suppose POWERDELTA_SIZE < 32

   integer j;
   always @*
    begin
        throttle_level_overflow = 1'b0;
        for (j = 0; j < POWERDELTA_SIZE; j=j+1) begin
          if(j > edc_trigger_throttle_lowbit+2)
           throttle_level_overflow = throttle_level_overflow | rollingpowerdelta_q[j];
         end
     end


   // throttle level
   assign throttle_level[0] = se_edc_level_comb_en ? ( throttle_level_overflow ? 1'b1 : rollingpowerdelta_q[edc_trigger_throttle_lowbit]) : 1'b0;
   assign throttle_level[1] = se_edc_level_comb_en ? ( throttle_level_overflow ? 1'b1 : rollingpowerdelta_q[edc_trigger_throttle_lowbit+1]) : 1'b0;
   assign throttle_level[2] = se_edc_level_comb_en ? ( throttle_level_overflow ? 1'b1 : rollingpowerdelta_q[edc_trigger_throttle_lowbit+2]) : 1'b0;

   //EDC MODE 2+: dynamically update right shift bits of gc_edc_stall_level
    wire [3:0] throttle_level_comb;
    reg [EDC_TIMER_WIDTH-1:0] edc_timer_counter;
   
    always @(posedge edc_gated_clk) begin
      if(edc_rst) begin
        edc_timer_counter <= {EDC_TIMER_WIDTH{1'b0}};
      end else begin
        if (edc_en && edc_level_comb_adapt_mode_en && (throttle_level_comb !=0)) begin
          if (edc_timer_counter >= edc_timer_period)
            edc_timer_counter <= {EDC_TIMER_WIDTH{1'b0}};
          else
            edc_timer_counter <= edc_timer_counter + 1'b1;
        end else begin
          edc_timer_counter <= {EDC_TIMER_WIDTH{1'b0}};
        end
      end
    end

    wire count_done = (edc_en && edc_level_comb_adapt_mode_en) ? (edc_timer_counter >= edc_timer_period) : 1'b0;

    reg [1:0] gc_edc_stall_policy_update;
    always @(posedge edc_gated_clk) begin
      if(edc_rst) begin
        gc_edc_stall_policy_update <= 2'b0;
      end else begin
        if (edc_en && edc_level_comb_adapt_mode_en && (throttle_level_comb !=0)) begin
          if ((count_done) && (gc_edc_stall_policy_update < 2'b11)) begin
            gc_edc_stall_policy_update <= gc_edc_stall_policy_update + 1'b1;
          end
        end else begin
          gc_edc_stall_policy_update <= gc_edc_stall_policy;
        end
      end
    end

    wire [1:0] gc_edc_stall_policy_selected = edc_level_comb_adapt_mode_en ? gc_edc_stall_policy_update : gc_edc_stall_policy;
   //End of EDC MODE 2+ for gc_edc_stall_policy update

   //wire [3:0] throttle_level_comb = {{1'b0},throttle_level} + {{1'b0},((edc_en && gc_edc_en && gc_edc_level_comb_en) ? (gc_edc_stall_level >> (3 - gc_edc_stall_policy)) : 3'b0)};
   assign throttle_level_comb =  ((edc_level_mode_sel == 1) && (gc_edc_stall_level == 3'b0)) ? 4'b0 : {{1'b0},throttle_level} + {{1'b0},((edc_en && gc_edc_en && gc_edc_level_comb_en) ? (gc_edc_stall_level >> (3 - gc_edc_stall_policy_selected)) : 3'b0)};

   wire [2:0] throttle_level_comb_clamp = throttle_level_comb[3] ? 3'b111 : throttle_level_comb[2:0];

   // throttle_level overflow counter
   reg rollingpowerdelta_ov;
   always @(posedge edc_gated_clk) begin
      if(edc_rst)
        rollingpowerdelta_ov <= 1'b0;
      else
        rollingpowerdelta_ov <= rollingpowerdelta[POWERDELTA_SIZE];
   end

   always @(posedge edc_gated_clk) begin
      if(edc_rst)
        edc_throttle_level_ov_counter <= 16'b0;
      else if (edc_grbm_clear_ov_counter && (throttle_level_overflow || rollingpowerdelta_ov)) 
        edc_throttle_level_ov_counter <= 16'b1;
      else if(edc_grbm_clear_ov_counter)
        edc_throttle_level_ov_counter <= 16'b0;
      else if((edc_throttle_level_ov_counter != 16'hffff) && (throttle_level_overflow || rollingpowerdelta_ov))
        edc_throttle_level_ov_counter <= edc_throttle_level_ov_counter + 1'b1;
   end

   // stall pattern mux
   always @*
   begin
     //case(throttle_level)
     case(throttle_level_comb_clamp)
      3'b000: stall_pattern = 15'b0; 
      3'b001: stall_pattern = edc_stall_pattern1; 
      3'b010: stall_pattern = edc_stall_pattern2; 
      3'b011: stall_pattern = edc_stall_pattern3; 
      3'b100: stall_pattern = edc_stall_pattern4; 
      3'b101: stall_pattern = edc_stall_pattern5; 
      3'b110: stall_pattern = edc_stall_pattern6; 
      3'b111: stall_pattern = edc_stall_pattern7; 
     endcase
   end

   // FSM 
   always @*
   begin
     
     next_fsm_state = curr_fsm_state;
     edc_stall_running = 1'b0;
     stall_counter_d = stall_counter_q;
     stall_pattern_d = stall_pattern_q;
     edc_throttle_level_d = edc_throttle_level_q;
     
     case(curr_fsm_state)
       EDC_IDLE:
                begin
                  // force stall
                  if(edc_force_stall) begin
                    next_fsm_state = EDC_STALL;
                    edc_stall_running = 1'b1;
                    stall_counter_d = edc_stall_pattern_bit_nums; //load counter max value 
                    stall_pattern_d = edc_stall_pattern1; //force stall pattern reuse stall pattern1
                    edc_throttle_level_d = 3'b001; //reuse stall pattern1, so throttle_level = 1
                  end 
                  // normal stall
                  //else if(throttle_level != 3'b0) begin
                  else if(throttle_level_comb_clamp != 3'b0) begin
                    next_fsm_state = EDC_STALL;
                    edc_stall_running = 1'b1;
                    stall_counter_d = edc_stall_pattern_bit_nums; //load counter max value 
                    stall_pattern_d = stall_pattern; //load stall pattern
                    //edc_throttle_level_d = throttle_level;
                    edc_throttle_level_d = throttle_level_comb_clamp;
                  end
                end

       EDC_STALL:
                 begin
                 
                   edc_stall_running = 1'b1;

                   // force stall
                   if(edc_force_stall) begin
                     edc_throttle_level_d = 3'b001; //reuse stall pattern1, so throttle_level = 1

                     if(stall_counter_q == {(STALL_PATTERN_BIT_NUMS_SIZE){1'b0}}) begin
                       stall_counter_d = edc_stall_pattern_bit_nums; //load counter max value 
                       stall_pattern_d = edc_stall_pattern1; //force stall pattern reuse stall pattern1
                     end
                     else begin
                       stall_counter_d = stall_counter_q - 1'b1;
                       stall_pattern_d = {1'b0,stall_pattern_q[14:1]}; // shift stall pattern to serially out stall signal
                     end
                   end
                   // normal stall
                   else begin
                     if(stall_counter_q == {(STALL_PATTERN_BIT_NUMS_SIZE){1'b0}}) begin
                       //if(throttle_level == 3'b0) begin
                       if(throttle_level_comb_clamp == 3'b0) begin
                         next_fsm_state = EDC_IDLE;
                         stall_pattern_d = 15'b0; //load stall pattern0
                       end
                       else begin
                         next_fsm_state = EDC_STALL;
                         stall_counter_d = edc_stall_pattern_bit_nums; //load counter max value 
                         stall_pattern_d = stall_pattern; //load stall pattern
                       end

                       //edc_throttle_level_d = throttle_level;
                       edc_throttle_level_d = throttle_level_comb_clamp;
                     end

                     else begin
                       next_fsm_state = EDC_STALL;
                       stall_counter_d = stall_counter_q - 1'b1;
                       stall_pattern_d = {1'b0,stall_pattern_q[14:1]}; // shift stall pattern to serially out stall signal
                     end
                   end
                 end

     endcase
   end

   // 
   always @(posedge edc_gated_clk) begin
      if(edc_rst)
        curr_fsm_state <= EDC_IDLE;
      else
        curr_fsm_state <= next_fsm_state;
   end

   assign edc_fsm_state = curr_fsm_state;

   always @(posedge edc_gated_clk) begin
      if(edc_rst) begin
        stall_counter_q      <= {(STALL_PATTERN_BIT_NUMS_SIZE){1'b0}};
        stall_pattern_q      <= 15'b0;
      end
      else if(edc_stall_running) begin
        stall_counter_q      <= stall_counter_d;
        stall_pattern_q      <= stall_pattern_d;
      end
   end

   assign edc_throttle_level = edc_throttle_level_q;

   always @(posedge edc_gated_clk) begin
      if(edc_rst) begin
        edc_stall_q <= 1'b0;
        edc_throttle_level_q <= 3'b0;
      end
      else begin
        edc_stall_q <= stall_pattern_d[0];
        edc_throttle_level_q <= edc_throttle_level_d;
      end
   end

   assign edc_dbg_stall_out = edc_stall_q  & (edc_en | edc_clk_en_override);

   // didt_stall_delay_en = 1, share EDC delay logic
   wire edc_didt_comb_stall = (edc_en & edc_stall_q) | (didt_stall & didt_stall_delay_en);

   // delayed stall 
   always @(posedge oCpl_GFXCLK) begin
      if(edc_rst)
        edc_delayed_stall_q <= {(MAX_DELAY_NUMS-1){1'b0}};
      else
	  	if (edc_clk_en | didt_stall_delay_en)
        	edc_delayed_stall_q <= {edc_delayed_stall_q[MAX_DELAY_NUMS-1-1-1:0],edc_didt_comb_stall};
   end

   wire [MAX_DELAY_NUMS-1:0] edc_delayed_stall = {edc_delayed_stall_q,edc_didt_comb_stall};

   integer k;
   always @*
     begin
        for (k = 0; k < EDC_STALL_INST_NUMS; k=k+1) begin
            edc_delay = edc_stall_delay[EDC_STALL_DELAY_SIZE*k +: EDC_STALL_DELAY_SIZE];
            edc_pipe_stall_d[k] = edc_delayed_stall[edc_delay];
        end
     end

//   // pipe stall
//   always @(posedge edc_gated_clk) begin
//      if(edc_rst)
//        edc_pipe_stall_q <= {(EDC_STALL_INST_NUMS){1'b0}};
//      else
//        edc_pipe_stall_q <= edc_pipe_stall_d;
//   end

   // didt_stall_delay_en = 1, share EDC delay logic
   wire[EDC_STALL_INST_NUMS-1:0] edc_stall_pre;
   assign edc_stall_pre = (edc_en | didt_stall_delay_en) ? edc_pipe_stall_d : {(EDC_STALL_INST_NUMS){1'b0}};
	
	//override by PWRBRK and PCC stall
	assign edc_stall = (pwrbrk_stall_en && (gc_edc_stall_level[2:1] == 2'b11)) ? {(EDC_STALL_INST_NUMS){(gc_edc_stall_level[0] & gc_se_stall)}} :
							(pcc_stall_en && (gc_edc_stall_level[2:1] == 2'b10)) ? {(EDC_STALL_INST_NUMS){(gc_edc_stall_level[0] & gc_se_stall)}} :
							 	(gc_edc_stall_en && (gc_edc_stall_level[2:1] == 2'b01)) ? {(EDC_STALL_INST_NUMS){(gc_edc_stall_level[0] & gc_se_stall)}} : edc_stall_pre;


reg[32:0] edc_pcc_perf_counter_d;
wire edc_pcc_perf_counter_overflow = edc_pcc_perf_counter_d[32];

always @(posedge edc_gated_clk) begin
	if(edc_rst) begin
		edc_pcc_perf_counter_d <= 'd0;
	end
	else if(rd_edc_pcc_perf_counter) begin
		edc_pcc_perf_counter_d <= 'd0;
	end
	else if(~gc_edc_only_mode) begin
		if(edc_pcc_perf_counter_overflow) begin
			edc_pcc_perf_counter_d <= 33'h1_FFFF_FFFF;
		end
		else if (!(gc_se_stall && gc_edc_stall_level[2:1] == 2'b11) && |edc_stall) begin
			edc_pcc_perf_counter_d <= edc_pcc_perf_counter_d + 1'b1;
		end
	end
end
assign edc_pcc_perf_counter = edc_pcc_perf_counter_d[31:0];

endmodule
