`include "grbm_defs.vh"
`include "gc_register_addr.v"


module gc_didt_stall_cntl (
	//input
	stall_in,
	stall_ctrl_enable,
	stall_delay,
	max_stalls_allowed,
	stall_auto_release_enable,
	stall_auto_release_time,

	clk,
	resetb,
	TEST_CLK_EN, //reserved

	//output
	stall_out

);

	parameter NUM_INSTANCE = 16; //will not be 1 or 0. If 1, don't instance this block in se_cac.v
	
	input 		stall_in;
	input 	    stall_ctrl_enable;
	input reg [5:0]		stall_delay;
	input [5:0]     	max_stalls_allowed;
	input 	    		stall_auto_release_enable;
	input reg [23:0]	stall_auto_release_time;

	input 		TEST_CLK_EN;
	input		clk;
	input		resetb;

	output reg [NUM_INSTANCE-1:0] stall_out;


	reg stall_in_d;
	wire stall_raise;
	wire stall_fall;
	always @(posedge clk) begin
		if(~resetb) begin
			stall_in_d <= 1'b0;
		end else begin
			stall_in_d <= stall_in;
		end
	end
	assign stall_raise = stall_in && ~stall_in_d;
	assign stall_fall = ~stall_in && stall_in_d;


wire auto_release;
wire auto_release_raise;
gc_didt_stall_auto_release_cntl u_stall_auto_release (
	.stall_auto_release_enable	(stall_auto_release_enable),
	.stall_auto_release_time	(stall_auto_release_time),
	.clk						(clk),
	.resetb						(resetb),
	.stall_auto_release			(auto_release),
	.stall_auto_release_raise	(auto_release_raise)
);

	reg [5:0] max_stalls_allowed_clamped;
	always @(posedge clk) begin
		if(~resetb) begin
			max_stalls_allowed_clamped <= 6'h0;
		end
		else begin
			max_stalls_allowed_clamped <= (max_stalls_allowed > NUM_INSTANCE) ? NUM_INSTANCE : max_stalls_allowed;
		end
	end


		reg[5:0] cntr;
		wire  count_done;
		reg[4:0] stall_index;

		reg [NUM_INSTANCE-1:0] stall_on_mask;
		reg unstall_pending; 
		reg unstall_pending_d; 
		wire unstall_pending_done; 

		always @(posedge clk) begin
			if(~resetb) begin
				unstall_pending <= 1'b0;
			end
			else if (stall_fall || auto_release_raise) begin
				unstall_pending <= 1'b1;
			end
			else if(stall_in && ~(|stall_on_mask)) begin
				unstall_pending <= 1'b0;
			end
		end

		always @(posedge clk) begin
			if(~resetb) begin
				unstall_pending_d <= 1'b0;
			end
			else begin
				unstall_pending_d <= unstall_pending;
			end
		end
		assign unstall_pending_done = ~unstall_pending && unstall_pending_d;


		always @(posedge clk) begin
			if(~resetb) begin
				cntr <= 6'h0;
			end
			else if (stall_raise || unstall_pending_done) begin
				cntr <= 6'h0;
			end else if (stall_ctrl_enable) begin
				cntr <= count_done ? 6'h0 : cntr + 1'b1;
			end 
		end

		assign count_done = ~stall_ctrl_enable ? 0 : stall_raise ? 1 : (cntr >= (stall_delay)) ? 1 : 0 ;
		always @(posedge clk) begin
			if(~resetb) begin
				stall_index <= 5'h1;
			end
			else if(stall_raise || unstall_pending_done || auto_release) begin
				stall_index <= 5'h1;
			end 
			else if (stall_in && count_done && (stall_index <= max_stalls_allowed_clamped)) begin
				stall_index <= stall_index + 1'b1;
			end 
		end


always @(posedge clk) begin
//FIXME if only 1 instances, combine stall_out with auto_release
//;if($num_instances == 1) {
//		if(~resetb) begin
//  			stall_on_mask <= {NUM_INSTANCE{1'b0}};
//		end
//		else begin
//  			stall_on_mask <= stall_in & ~auto_release;
//		end
//;}
		if(~resetb) begin
  			stall_on_mask <= 'd0;
		end
		else if(unstall_pending_done ) begin
  			stall_on_mask <= 'd0;
		end
		else if(unstall_pending && count_done) begin
  		stall_on_mask <= {stall_on_mask[NUM_INSTANCE-2:0],1'b0};
		end
		else if(stall_in && count_done && ~unstall_pending && ~auto_release) begin
			stall_on_mask <= (stall_index <= max_stalls_allowed_clamped) ? ((2 ** stall_index) - 1) : {stall_on_mask[NUM_INSTANCE-2:0],stall_on_mask[NUM_INSTANCE-1]};
	end
end


	assign stall_out = (stall_auto_release_enable && auto_release) ? (stall_ctrl_enable ? stall_on_mask : {NUM_INSTANCE{1'b0}} ) : (stall_ctrl_enable ? stall_on_mask : {NUM_INSTANCE{stall_in}});


endmodule
