//                              -*- Mode: Verilog -*-

////////////////////////////////////////////////////////////////////////////
/*
 *
 *  Trade secret of ATI Technologies, Inc.
 *  Copyright 2005, ATI Technologies, Inc., (unpublished)
 *
 *  All rights reserved.  This notice is intended as a precaution against
 *  inadvertent publication and does not imply publication or any waiver
 *  of confidentiality.  The year included in the foregoing notice is the
 *  year of creation of the work.
 *
 */
////////////////////////////////////////////////////////////////////////////
//
// Filename    : gen_edt_update.v
// Description : Edt update generator
//
////////////////////////////////////////////////////////////////////////////

module gen_edt_update_gc10 (
input wire ScanEn,
input wire Shift_Clk,
input wire IO_Edt_Update,
input wire edt_update_sel,
output wire Edt_Update,
input wire scan_mode,
input wire scm_mode

);

////////////////////////////////////////////////////////////////////////////////////////////////////////////
     // edt_update generator

   reg        edt_update_pos_ff;
   reg        edt_update_neg_ff;

   //AC scan - SE sync in dft_t tile. Need to check timing from Sync in dft_t tile to below flop.
   //DC scan - SE assert de-assert when clock is not toggling.

 // Async reset flop when SE is 0. This is needed to clear below flops to generate pulse for next pattern. 
   always @ (posedge Shift_Clk, negedge ScanEn)
     begin
       if (~ScanEn)     // Path from Sync to this flop is real.
        begin
         edt_update_pos_ff <= 1'b0;
        end
       else
        begin
         edt_update_pos_ff <= 1'b1;
        end
     end

   always @ (negedge Shift_Clk, negedge ScanEn)
     begin
      if (~ScanEn)
       begin
        edt_update_neg_ff <= 1'b0;
       end
      else
       begin
        edt_update_neg_ff <= edt_update_pos_ff;  //half cycle path.
       end
     end

   wire int_edt_update_neg =  ~edt_update_neg_ff & ScanEn;

  wire edt_update_muxed;
   ati_mux2 mux_edt_update  (
   .i0(IO_Edt_Update),
   .i1(int_edt_update_neg),
   .s(edt_update_sel),
   .z(edt_update_muxed));


  dfx_mrk_and_gc10 d0nt_edt_update (
                                  .A(edt_update_muxed),
                                  .B(scan_mode & ~scm_mode), // Set to 0 in SCM
                                  .Y(Edt_Update) );



endmodule
