//                              -*- Mode: Verilog -*-

////////////////////////////////////////////////////////////////////////////
/*
 *
 *  Trade secret of ATI Technologies, Inc.
 *  Copyright 2005, ATI Technologies, Inc., (unpublished)
 *
 *  All rights reserved.  This notice is intended as a precaution against
 *  inadvertent publication and does not imply publication or any waiver
 *  of confidentiality.  The year included in the foregoing notice is the
 *  year of creation of the work.
 *
 */
////////////////////////////////////////////////////////////////////////////
//
// Filename    : gen_scan_en.v
// Description : Edt update generator
//
////////////////////////////////////////////////////////////////////////////


//`include "scan_cntl_features.vh"



module gen_scan_en_gc10 
(
 input wire ScanEn,
 input wire [1:0] cap_disable,
 input wire rstb,
 input wire Shift_Clk,
 input wire scan_mode,
 input wire SE_Sel,
 input [16-1:0] shift_cnt_tdr,
 output Tile_ScanEn,
 output Tile_ScanEn_Neg
 
 
 );
   
   wire load_shift_cnt_pre; //SE asserted while back before test_clock starts toggling. Else, this could be half cycle pulse.
   wire shift_cnt_en;
   wire cap_dis_cnt_en;
   wire SCAN_EN_ff_ored;
   wire SCAN_EN_ff_ored_1;
   reg 	SCAN_EN_p1, SCAN_EN_p2;
   reg [16-1:0] shift_cnt;
   reg [1:0] cap_dis_cnt;
   reg 				 SCAN_EN_ff;
   reg 				 SCAN_EN_ff_neg;
   ////////////////////////////////////////////////////////////////////////////////////////////////////////////
   
   
   
   always @ (negedge Shift_Clk)
     begin
        if(~rstb)
          begin
             SCAN_EN_p1 <= 1'b0;
          end
        else
          begin
             SCAN_EN_p1 <= ScanEn;
          end
     end // always @ (negedge Shift_Clk)
   
   
   always @ (posedge Shift_Clk)
     begin
        if(~rstb)
          begin
             SCAN_EN_p2 <= 1'b0;
          end
        else
          begin
             SCAN_EN_p2 <= SCAN_EN_p1; //half cycle path
          end
     end // always @ (posedge Shift_Clk)
   
   
   
//SE asserted while back before test_clock starts toggling. Else, this could be half cycle pulse.
   dfx_mrk_and_gc10 d0nt_load_shift_cnt_pulse 
     (
      .A(ScanEn),   //MCP
      .B(~SCAN_EN_p2), 
      .Y(load_shift_cnt_pre)
      );
   
   
   
   ati_buffer1 mrk_load_shift_cnt
     (
      .A(load_shift_cnt_pre),
      .Y(load_shift_cnt)
      );
   
   
   
   
   //No need for async set flops as load_se is 1 as soon as SE=1 from I/O.
   // First test_sclk pulse is controlled from tester. Give enought time to propagete SE from I/O to this flop.
   // Then toggle Shift_Clk.
   always @ (negedge Shift_Clk)
     begin
        if(~rstb)
          begin
             shift_cnt <= {(16){1'b0}};
          end
        else if (load_shift_cnt)              //CDC??
          begin
             shift_cnt <= shift_cnt_tdr;  //CDC WAIVE
          end
        else if (shift_cnt == {(16){1'b0}})
          begin
             shift_cnt   <= shift_cnt;
          end
        else
          begin
             shift_cnt <= shift_cnt - {{(16-1){1'd0}}, 1'd1};
          end
     end // always @ (negedge Shift_Clk)
  
   assign shift_cnt_en = (|shift_cnt); 

   always @ (negedge Shift_Clk or negedge rstb)
     begin
        if(~rstb)
          begin
             cap_dis_cnt <= cap_disable[1:0];   //TDR
          end
        else if (cap_dis_cnt == {(2){1'b0}})
          begin
             cap_dis_cnt   <= cap_dis_cnt;
          end
        else if (load_shift_cnt)              //CDC??
          begin
             cap_dis_cnt <= cap_dis_cnt - {{(1){1'd0}}, 1'd1};
          end
        else
          begin
             cap_dis_cnt <= cap_dis_cnt;
          end
     end // always @ (negedge Shift_Clk)
   
   
   
   assign cap_dis_cnt_en = (|cap_dis_cnt); 
   
   
   ////////////////////////////////////////////////////////////////////////////////////////////////////
   //SE out

   // Async Set when SCAN_EN asserts.
   // shift mode- assert SCAN_EN for enough time before clock start toggling
   // async set register below, so SE will propagate without any clock.
   // during de-assertion, the SE will deassert while back before entering the capture mode
   // last shift clock negedge will de-assert the SCAN_EN in the tile.
   
   //NOTE: The path from below flop to all the SE pins needs to be timed for HOLD.
   



    //Async not needed due to mux and OR gate in dft_clk_cntl.
   //always @(posedge Shift_Clk, negedge rstb, posedge load_shift_cnt)
   always @(posedge Shift_Clk, negedge rstb)
     begin
        if(~rstb)
          begin
             SCAN_EN_ff <= 1'b0;
          end
        else
          begin
             SCAN_EN_ff <= shift_cnt_en;
          end
     end // always @ (posedge Shift_Clk, negedge rstb)

   always @(negedge Shift_Clk, negedge rstb)
     begin
        if(~rstb)
          begin
             SCAN_EN_ff_neg <= 1'b0;
          end
        else
          begin
             SCAN_EN_ff_neg <= SCAN_EN_ff;
          end
     end // always @ (posedge Shift_Clk, negedge rstb)
   
   

dfx_mrk_or_gc10 mrk_SE_or_ff 
  (    
       .A(SCAN_EN_ff), 
       .B(ScanEn),             //MCP
       .Y(SCAN_EN_ff_ored_1) );

dfx_mrk_or_gc10  mrk_SE_or_cap_dis_cnt 
  (    
       .A(SCAN_EN_ff_ored_1), 
       .B(cap_dis_cnt_en),
       .Y(SCAN_EN_ff_ored) );
   
   
   ati_mux2 mux_se_sel  
     (
      .i0(ScanEn),
      .i1(SCAN_EN_ff_ored),
      .s(SE_Sel),
      .z(Tile_ScanEn)
      );   

   ati_mux2 mux_neg_se_sel  
     (
      .i0(ScanEn),
      .i1(SCAN_EN_ff_neg | ScanEn | cap_dis_cnt_en),
      .s(SE_Sel),
      .z(Tile_ScanEn_Neg)
      );   
   
   
endmodule // gen_scan_en

