/////////////////////////////////////////////////////////////////////////
//
// $Id: $
//
// $Change: 1704 $
//
//                            Copyright 2001
//                            ATI Research, Inc.
//                            All Rights Reserved
//
//  Filename: ati_rd_rtn_1to32.v
//
//  Filetype   : Verilog
//  Description: Converts a Serialized 33bit data stream into a 32bit output with 
//  a valid. The transfer size is 33 bits because of a 1 bit header with a 32 bit payload.
//                          
//  Heiarchy:
//
//  Additional Files
//
/////////////////////////////////////////////////////////////////////////
//                        Modification History
//
//   Date       Author         Revision    Comments
//   -------    -----------    --------    -----------------------------
//              Alexander Ashkar
/////////////////////////////////////////////////////////////////////////

module  gfx_ati_rd_rtn_1to32 (

        iSERREAD_DATA,
        oDATA,
        oVALID,
        iSRST,
        iSCLK
        );

    input           iSERREAD_DATA; // Serialized input data stream

    output  [31:0]  oDATA;   // 32 bit output data

    reg     [31:0]  DATA;

    output          oVALID;  // Valid for output data
    reg             oVALID;
    wire            dVALID;

    input           iSRST;     //Synchronized Reset
    input           iSCLK;     //Clock

    reg             XFER_START;

    reg     [4:0]   COUNT;
    wire    [5:0]   tmp_COUNT;

	assign oDATA = (oVALID) ? DATA : 32'b0;

    assign dVALID = COUNT == 0 & XFER_START;

    always @(posedge iSCLK)
    begin
      if (iSRST)
      begin
        oVALID <= 1'b0;
        DATA  <= 32'b0;
      end
      else
      begin
      oVALID <= dVALID;
      
      if (XFER_START)
        DATA[COUNT] <= iSERREAD_DATA;
      end
    end

    always @(posedge iSCLK)
    begin
      if (iSRST || dVALID)
        XFER_START <= 1'b0;
      else
      if (iSERREAD_DATA && (COUNT == 5'd31)) // Found the Header, switch to the Data Phase
        XFER_START <= 1'b1;
    end

    // Down counter.
    assign tmp_COUNT = COUNT + 5'h1f;

    always @(posedge iSCLK)
    begin
        if (iSRST)
           COUNT <= 5'd31;
        else
        if (XFER_START)
           COUNT <= tmp_COUNT[4:0];
    end

endmodule
