

// fixed-delay ring fifo using memory(latches/flops), need use "zg" type to align cycle sequence
// (lower-power data pipeline)
module gfx_ati_ring_fifo_ctrl(
  //Outputs
  dout,
 
  //Memory interface
  mem_we,
  mem_wraddr,
  mem_wrdata,
  mem_re,
  mem_rdaddr,
  mem_rddata,

  // Inputs
  CLK, STALL_N, EN, resetb, din
);

  parameter DATA_WIDTH = 32;
  parameter PIPE_DEPTH = 8;

  // max depth = 4096
  parameter ADDR_WIDTH = ((PIPE_DEPTH <= 4) ? 2 :
                          (PIPE_DEPTH <= 8) ? 3 :
                          (PIPE_DEPTH <= 16) ? 4 :
                          (PIPE_DEPTH <= 32) ? 5 :
                          (PIPE_DEPTH <= 64) ? 6 :
                          (PIPE_DEPTH <= 128) ? 7 :
                          (PIPE_DEPTH <= 256) ? 8 :
                          (PIPE_DEPTH <= 512) ? 9 : 
                          (PIPE_DEPTH <= 1024) ? 10 :
                          (PIPE_DEPTH <= 2048) ? 11 : 12);

  input CLK;
  input [PIPE_DEPTH-1:0] EN;
  input STALL_N;
  input resetb;

  input [DATA_WIDTH-1:0] din;
  output [DATA_WIDTH-1:0] dout;

  output mem_we;
  output [ADDR_WIDTH-1:0] mem_wraddr;
  output [DATA_WIDTH-1:0] mem_wrdata;
  output mem_re;
  output [ADDR_WIDTH-1:0] mem_rdaddr;
  input  [DATA_WIDTH-1:0] mem_rddata;

  wire [ADDR_WIDTH-1:0] mem_wraddr;
  wire [DATA_WIDTH-1:0] mem_wrdata;
  wire [ADDR_WIDTH-1:0] mem_rdaddr;
  wire [DATA_WIDTH-1:0] mem_rddata;

  reg [ADDR_WIDTH-1:0] wr_ptr;
  reg [ADDR_WIDTH-1:0] rd_ptr;

  wire [ADDR_WIDTH-1:0] max_ptr = (PIPE_DEPTH-1);
  wire [ADDR_WIDTH-1:0] zero_ptr = {ADDR_WIDTH {1'b0}};

  assign mem_we = STALL_N & EN[0];
  assign mem_wraddr = wr_ptr;
  assign mem_wrdata = din;
  assign mem_re = 1'b1;
  assign mem_rdaddr = rd_ptr;
  assign dout = mem_rddata;
 
  always @(posedge CLK) begin
    if (!resetb) begin
      rd_ptr <= max_ptr;
      wr_ptr <= zero_ptr;
    end else begin
      if (STALL_N) begin
        if (EN[0]) begin
          wr_ptr <= (wr_ptr == max_ptr) ? zero_ptr : (wr_ptr + 1'b1);
        end

        // increment rd_ptr when something is next in line
        // (reduce toggling and keep output values stable)
        if (EN[PIPE_DEPTH-1]) begin
          rd_ptr <= (rd_ptr == max_ptr) ? zero_ptr : (rd_ptr + 1'b1);
        end
      end
    end
  end  

endmodule

