//*************************************************************************************************
//
// FILE_OWNER: stephen.kosonocky@amd.com
//
//  Copyright  2013 Advanced Micro Devices, Inc.
//  All rights reserved.
//
//  This file is protected by Federal Copyright Law, with all rights
//  reserved. No part of this file may be reproduced, stored in a
//  retrieval system, translated, transcribed, or transmitted, in any
//  form, or by any means manual, electric, electronic, mechanical,
//  electro-magnetic, chemical, optical, or otherwise, without prior
//  explicit written permission from Advanced Micro Devices, Inc.
//
//*************************************************************************************************
// **************************************************************************************************
//                       Zen RTL design file
// **************************************************************************************************
//                  AMD PROPRIETARY AND CONFIDENTIAL
//               (c) 2013 Advanced Micro Devices, Inc.
// **************************************************************************************************
// Module Description:
//
// This file is an AVFS library file. The AVFS wrapper does not directly instantiate stdcells, but
// instead instantiates stdcell wrappers, to facilitate sharing the wrapper across different non-ZAK 
// SOCs.Each SOC provides its own stdcell wrapper library file to map the stdcell wrappers to its
// own stdcell library. This file is the Zen/Aspen/K12 (ZAK) version.
//
//
// **************************************************************************************************
//! disable one_module_per_file
// Disable the Leda one-per-file check
// leda E_001 off

//module dff_noscan2
// #(parameter WIDTH=1               // default WIDTH of output
//   ) 
//  (
//   output     [WIDTH-1:0]   q,     // data out
//   input      [WIDTH-1:0]   clk,   // clock
//   input      [WIDTH-1:0]   d      // data in
//   );
//
//  genvar index;
//  generate
//    for(index=0;index<WIDTH; index=index+1)
//    begin:flops
//      hdmsfnsxss1us I_dff_noscan ( .CLK(clk[index]), .D(d[index]), .Q(q[index]), .QB() );
//    end  
//  endgenerate
//
//endmodule
//
//module AVFS3_CPO_FLOP_GATED_RIPPLE #(parameter
//  size = 1
//)(
//  input [size-1:0] C,
//  input [size-1:0] D,
//  input G,
//  input R,
//  output reg [size-1:0] Q
//);
//
//  wire [size-1:0] d;
//  
//  // connect reset -> SSE on fine gater to turn on clk during reset
//  // vscan off cellprop
//  //cg I_cg_noscan (.gclk(RCLK_CgenEn_AR), .clk(C), .en(G), .SSE(R));
//  // vscan on cellprop
//
//  assign d[size-1:0] = D[size-1:0] & ~{size{R}};
//  //dff_noscan # (.WIDTH(size)) I_dff_noscan (.clk(RCLK_CgenEn_AR), .d(d[size-1:0]), .q(Q[size-1:0]));
//  dff_noscan2 # (.WIDTH(size)) I_dff_noscan (.clk(C[size-1:0]), .d(d[size-1:0]), .q(Q[size-1:0]));
//  //module hdmsfqbnsxss1us ( CLK, D, QB );
//  //hdmsfqnsxss1us # (.WIDTH(size)) I_dff_noscan ( .CLK(C[size-1:0]), .D(d[size-1:0), .Q(Q(size-1:0]), .QB() );
//
//
//endmodule // AVFS3_CPO_FLOP_GATED

`timescale 1ps/1ps

module gfx_AVFS3_CPO_FLOP_GATED #(parameter
  size = 1
)(
  input C,
  input [size-1:0] D,
  input G,
  input R,
  output reg [size-1:0] Q
);

  wire [size-1:0] d;
  
  // connect reset -> SSE on fine gater to turn on clk during reset
  // vscan off cellprop
  gfx_cg I_cg_noscan (.gclk(RCLK_CgenEn_AR), .clk(C), .en(G), .SSE(R));
  // vscan on cellprop

  assign d[size-1:0] = D[size-1:0] & ~{size{R}};
  gfx_dff_noscan #(.WIDTH(size)) I_dff_noscan (.clk(RCLK_CgenEn_AR), .d(d[size-1:0]), .q(Q[size-1:0]));

endmodule // AVFS3_CPO_FLOP_GATED

module gfx_AVFS3_CPO_FLOP #(parameter
  size = 1
)(
  input C,
  input [size-1:0] D,
  input R,
  output reg [size-1:0] Q
);

  wire [size-1:0] d;
  
  assign d[size-1:0] = D[size-1:0] & ~{size{R}};
  gfx_dff_noscan #(.WIDTH(size)) I_dff_noscan (.clk(C), .d(d[size-1:0]), .q(Q[size-1:0]));

endmodule // AVFS3_CPO_FLOP

module gfx_AVFS3_CPO_RESET (
  input Clk,
  input ResetIn,
  output ResetOut
);
  wire n0;
  
  gfx_srdff I_reset_ff1 (.q(n0),.d(1'b1),.clk(Clk),.rst(ResetIn),.set(1'b0));
  gfx_srdff I_reset_ff2 (.q(ResetOut),.d(n0),.clk(Clk),.rst(ResetIn),.set(1'b0));

endmodule // AVFS3_CPO_RESET

// combined THERMAL macro wrapper

module gfx_AVFS3_THERMALRO (
//  input                      slvt_ro_en,
//  input                      lvt_ro_en,
//  input                      rvt_ro_en,
//  output wire                slvt_ro_out,
//  output wire                lvt_ro_out,
//  output wire                rvt_ro_out
  input [6:0] en,
  output wire [6:0] out
);

// Note the delay increments below are multiple of 300ps clock period
// 750*2 = 1500 (5 times the 300ps CCLK period used in the testbench)
// 900*2 = 1800 (6 times the 300ps CCLK period used in the testbench)
// 1050*2 = 2100 (7 times the 300ps CCLK period used in the testbench)
`ifdef AVFS_BEHAV // for simulation output RO clock
//   assign #750 slvt_ro_out = slvt_ro_en? ~slvt_ro_out : slvt_ro_en; // osc count = "sample size" / 5
//   assign #900 lvt_ro_out = lvt_ro_en? ~lvt_ro_out : lvt_ro_en;     // osc count = "sample size" / 6
//   assign #1050 rvt_ro_out = rvt_ro_en? ~rvt_ro_out : rvt_ro_en;    // osc count = "sample size" / 7
   assign #750 out[0] = en[0] ? ~out[0] : en[0]; // osc count = "sample size" / 5
   assign #900 out[1] = en[1] ? ~out[1] : en[1]; // osc count = "sample size" / 6
   assign #1050 out[2] = en[2] ? ~out[2] : en[2]; // osc count = "sample size" / 7
   assign #1150 out[3] = en[3] ? ~out[3] : en[3]; // osc count = "sample size" / ?
   assign #1250 out[4] = en[4] ? ~out[4] : en[4]; // osc count = "sample size" / ?
   assign #1350 out[5] = en[5] ? ~out[5] : en[5]; // osc count = "sample size" / ?
   assign #1450 out[6] = en[6] ? ~out[6] : en[6]; // osc count = "sample size" / ?
`else
      // Expectation is that AVFS_BEHAV is NOT defined in synthesis step.
      // Real macro publish is available now - Jan 2015.
      // Only verification flow defines AVFS_BEHAV

      // Instantiate real thermal RO macro
      mathermalro I_mathermalro (
      `ifndef SYNTHESIS
        .VDD(),
        .VSS(),
      `endif
//        .en({rvt_ro_en,lvt_ro_en,slvt_ro_en}),
//        .out({rvt_ro_out,lvt_ro_out,slvt_ro_out})
        .en(en),
        .out(out)
      );
`endif // !`ifdef AVFS_BEHAV


endmodule // AVFS3_THERMALRO_SLVT


//==================================================================================================
//! enable one_module_per_file
// leda E_001 on
//==================================================================================================
