//*************************************************************************************************
//
// FILE_OWNER: kelvin.goveas@amd.com
//
//  Copyright  2013 Advanced Micro Devices, Inc.
//  All rights reserved.
//
//  This file is protected by Federal Copyright Law, with all rights
//  reserved. No part of this file may be reproduced, stored in a
//  retrieval system, translated, transcribed, or transmitted, in any
//  form, or by any means manual, electric, electronic, mechanical,
//  electro-magnetic, chemical, optical, or otherwise, without prior
//  explicit written permission from Advanced Micro Devices, Inc.
//
//*************************************************************************************************
// *************************************************************************************************
//
//  $RCSfile:  $
//  $Revision: $
//  $Date:  $
//  $Author: $
//  $State:  $
//  $Locker:  $
//
//  Author: S.Toh
//
//  Description: 
//
//  This rtl contains a top-level AVFS wrapper module, which integrates the following components:
//     AVFS/PSM Register Interface Block
//     AVFS Critical Path Oscillator (CPO) - New in AVFS 3.0
//     AVFS Gate Pack
//     AVFS Wire Pack
//     AVFS Therm Pack (New in AVFS 3.0)
//     Power Supply Monitor (PSM)
//  The AVFS/PSM register interface block, 'AvfsWrapJtgInterface', supports SPMI, JTAG (IEEE-1500)
//  & M&M access to the internal AVFS/PSM registers. The PSM rtl is in CORE_PSM_IP_NO_JTAG and
//  contains 'CorePsmIpCore' which instantiates the PSM macro. CorePsmIpCore needs to be fast
//  enough to support the highest frequency under worst case droop. 
//
//  AvfsWrapJtgInterface -- Comes from AVFS2.0 code - ${CPLKIT_AVFS}/rtl/avfs_psm.v 
//
// *************************************************************************************************
//==================================================================================================
// <Module Description>
//
// This is a template for creating verilog shells.  Guidelines and a how-to guide to
// creating HLD verilog shells can be found at:
// http://mpdwww.amd.com/twiki/bin/view/BDRTL/K10VerilogShells
// All questions and comments should be directed to Gene Shen (genes@amd.com)
//
// -> Outputs are declared using V2K syntax - i.e. using combined port and data type
//    declarations.  In Verilog-1995, inputs and outputs of a module were declared in two
//    separate statements (the direction of the port and the data type).  In Verilog-2000, a
//    simpler syntax combines the declarations into one statement.
// -> Please provide a short description of the signal.
// -> Signal names should be of the form:
//        <SourceIdentifier>_<SignalName>_<Suffix>
//    where <SourceIdentifier> is a unique prefix and SignalName is a short descriptive
//    signal name.  For now, suffixes should not be used - mostly because we have not decided
//    on our suffixes.
//
//
//==================================================================================================
//==================================================================================================
//  Non-scanable registers use the following flop wrappers:
//
//    CORE_PSM_IP_FLOP
//    CORE_PSM_IP_FLOP_GATED
//
//==================================================================================================
//==================================================================================================
//! disable one_module_per_file
// Disable the Leda one-per-file check
// leda E_001 off
`include "gfx_core_psm_ip_cfg.d"
`include "gfx_avfs3_cpo_lib.d"

`ifdef AVFS_ZAK
  // conditional include of M&M defines
  `include "mm_global.d"
`endif
`include "gfx_avfs_lib.d"
`include "gfx_avfs_psm_lib.d"

// AVFS (Adaptive Voltage and Frequency Scaling) Library file


//================================================================================================================================
// <AVFS3_CPO> - changed from AVFS_CPA (Critical path Accumulator) in AVFS 3.0
//================================================================================================================================
// AVFS (Adaptive Voltage and Frequency Scaling) CPO (Critical Path Oscillator) 
// The CPO module controls and gathers data from local GATEPACK, WIREPACK and THERMPACK
// blocks' ripple counters
// It has a 64-bit SRB interface that can be interfaced with SPMI/JTAG/p1500

// todo:
//   parameterize by input width 

// New wrapper module that has data/control registers and AVFS3_CPO instantiation

module gfx_AVFS3_CPO_WRAP #(parameter 
CDC_TST_PROP_DLY=0, // needs to be non-zero, top level overrides this value with 5ns AVFS3_TOP
CPO_TO_CCLK_SYNC_TC_CYCLES=1, 
CDC_RESET_TO_CPOCLK_DLY = 5000,
CCLK_TO_CPO_SYNC_TD_DELAY=5000
) 
(
//=========================================================================
//  IO Declarations
//=========================================================================

input                  ScanShiftEn,    // Scan shift Enable
input                  CGater01_CG0,   // Coarse Gater
input                  CCLK,
input                  HostReg_CCLK,
input                  Reset,          // reset signal to reset flops for SOCs which don't use scan-shift-reset

input                  AvfsGo,         // Pulse to start the state machine
 
output wire            CgenAvfsEn,     // gate AvfsGoPulse flops
output wire            AvfsFuncDbgEn,  // Debug Enable (Func Mode)
input                  LBistEn,        // LBistEn to be used to control multicycle protection for ATPG.
input                  SkipPhase,
output wire            AvfsTestData,   // TestData to be used in SFFGSPD
output wire [1:0]      AvfsTestMode,   // TestMode to be used in SFFGSPD
output wire [63:0]     AvfsSrbRdDat,   // SRB read data 

output wire [23:0] AvfsCpoGPCnt0,
output wire [23:0] AvfsCpoGPCnt1,
output wire [23:0] AvfsCpoWPCnt,
output wire [23:0] AvfsCpoTPCnt,
output wire [23:0] AvfsCpoSPCnt,

input                  SelAvfsCtl1,    // SRB register decodes used for both read and write 
input                  SelAvfsCtl2,
input                  SelAvfsBucket0,
input                  SelAvfsBucket1,

input [63:0]           AvfsSrbWrDat_4, // SRB write data
input                  SrbLclWrEn_4    // SRB write enable 

);


   wire [1:0] AvfsTestMode_pre;
   wire       ScanWrapCtlMC4;
   wire       AtpgMC4Protect;

   //wire AvfsStart;
//   wire AvfsGoSync;   // removing unused/dead-end flops; see bug 636868
//   wire AvfsGoSyncIn; // removing unused/dead-end flops; see bug 636868
   wire AvfsEn;
   //wire AvfsEn_D1;

   // AvfsTestMode & AvfsTestData are scan-only signals that can be used to configure the delay elements in the
   // gate & wire packs.
//   wire [`CLKBUS_SIZE-1:0] clkbus_scan_only;

   //wire [`CLKBUS_SIZE-1:0] clkbus_CgenAvfsEn;

   //wire ResetSyncDeassert;
   // adding synchronous deassert on Reset, avfs_psm modules already do this (according to James)
   //AVFS3_CPO_RESET I_CPO_Reset (.Clk(CCLK), .ResetIn(Reset), .ResetOut(ResetSyncDeassert) );

   // Removing below code as it's not needed in AVFS3.0, no delay elements in replica CPs, They are now CPOs.
   // Ravi Rachala 4-40-2015
   //
   // Removing CORE_CG I_cg_scan_only (.clkbus(clkbus_scan_only[`CLKBUS_SIZE-1:0]), .C(CCLK), .G(1'b0), .R(1'b0), .SSE(ScanShiftEn));

   // Removing CORE_FLOP_DGATE #(.size(2)) I_AvfsTestMode_pre (.clkbus(clkbus_scan_only[`CLKBUS_SIZE-1:0]), .DG(1'b1), .D(2'b0), .Q(AvfsTestMode_pre[1:0]));
   // Removing CORE_FLOP_DGATE             I_AvfsTestData     (.clkbus(clkbus_scan_only[`CLKBUS_SIZE-1:0]), .DG(1'b1), .D(1'b0), .Q(AvfsTestData));
   // Removing CORE_FLOP_DGATE             I_ScanWrapCtlMC4   (.clkbus(clkbus_scan_only[`CLKBUS_SIZE-1:0]), .DG(1'b1), .D(1'b0), .Q(ScanWrapCtlMC4));

   // Removing wire       LBistEn_CDC;

   // Removing // CDCBUF for LBistEn (ATPG test JTAG -> CCLK)
   // Removing CORE_CDCBUF #(.size(1), .Cycles(`CDC_JTAG_X_DLY_CYCLES))
   // Removing   I_CDC_LBistEn (.A(LBistEn),
   // Removing                  .Z(LBistEn_CDC),
   // Removing                  .DSTCLK(CCLK) );   
   // Removing 
   // Removing assign AvfsTestMode[1:0] = AvfsTestMode_pre[1:0] | {2{LBistEn_CDC}};
   // Removing 
   // Removing assign AtpgMC4Protect = ScanWrapCtlMC4 | LBistEn_CDC;
   assign AvfsTestMode[1:0] = 2'b00;
   assign AvfsTestData = 1'b0;


// AvfsGo is a pulse that may extend for multiple CCLKs
// convert to an edge and sync to PsmClk
//   CORE_FLOP_DGATE  #(.size(1)) I_AvfsGo_d1  (.clkbus(clkbus_CgenAvfsEn[`CLKBUS_SIZE-1:0]), .DG(1'b1), .D(AvfsGo), .Q(AvfsGo_D1)); // removing unused/dead-end flops; see bug 636868
//   assign AvfsGoEdge = AvfsGo & ~AvfsGo_D1; // removing unused/dead-end flops; see bug 636868
//   assign AvfsGoSyncIn = AvfsGoEdge | AvfsGoSync ;   // set on AvfsGo // removing unused/dead-end flops; see bug 636868
//   CORE_FLOP_DGATE  #(.size(1)) I_AvfsGoSync (.clkbus(clkbus_CgenAvfsEn[`CLKBUS_SIZE-1:0]), .DG(1'b1), .D(AvfsGoSyncIn), .Q(AvfsGoSync)); // removing unused/dead-end flops; see bug 636868

   // Not using AvfsStart signal now, AvfsGo oulse and AvfsEn are passed to AVFS3_CPO to kick start
   // AVFS loop
   // Commenting out next 2 lines : Ref Bug - http://bugs.amd.com/ubts/prod/issue/view?id=597035
   //assign AvfsStart = AvfsGoSync & // start testing paths when SPMI input bit is set 
   //                   AvfsEn;      // and this CPO is enabled

// Global enable 

   // Fix to : http://bugs.amd.com/ubts/prod/issue/view?id=636868 
   // rt.avfs_psm_0.I_AVFS_CPO_WRAP.I_AvfsEn_D1.I_dff.dff_q (unused flop reported in CDC tool report)

   //CORE_FLOP_DGATE #(.size(1)) I_AvfsEn_D1 (.clkbus(clkbus_CgenAvfsEn[`CLKBUS_SIZE-1:0]), .DG(1'b1), .D(AvfsEn), .Q(AvfsEn_D1));  // Delay to create leading edge
   //assign CgenAvfsEn = AvfsEn | AvfsEn_D1;   // Global Clock enable for any ungated flops

   //CORE_CG I_cg_CgenAvfsEn (.clkbus(clkbus_CgenAvfsEn[`CLKBUS_SIZE-1:0]), .C(CCLK), .G(CgenAvfsEn), .R(Reset), .SSE(ScanShiftEn));
   assign CgenAvfsEn = 1'b0 ;


//================================================================================================================================
// <AVFS3_CPO> - replacing AVFS_CPA
//================================================================================================================================
// AVFS (Adaptive Voltage and Frequency Scaling) CPO (Critical Path Oscillator) 
// The CPO module controls and gathers data from local oscillators in GATEPACK, WIREPACK and THERMPACK modules
// It has a 64-bit SRB interface that can be interfaced with SPMI/JTAG/p1500

wire [3:0] AvfsGatePackSel0;
wire [3:0] AvfsGatePackSel1;
wire [3:0] AvfsWirePackSel;
wire [2:0] AvfsThermPackSel;
wire [15:0] AvfsGatePackState0;
wire [15:0] AvfsGatePackState1;
wire [15:0] AvfsWirePackState;
wire [23:0] AvfsSmpStart;
wire [23:0] AvfsSmpMax;
wire [63:0] AvfsCtl1;
wire [63:0] AvfsCtl2;

  gfx_AVFS3_CPO #(
.CDC_TST_PROP_DLY(CDC_TST_PROP_DLY), 
.CPO_TO_CCLK_SYNC_TC_CYCLES(CPO_TO_CCLK_SYNC_TC_CYCLES), 
.CDC_RESET_TO_CPOCLK_DLY(CDC_RESET_TO_CPOCLK_DLY),
.CCLK_TO_CPO_SYNC_TD_DELAY(CCLK_TO_CPO_SYNC_TD_DELAY) 
) I_AVFS_CPO (
.LBistEn(LBistEn),
.SkipPhase(SkipPhase),
.AvfsSmpStart(AvfsSmpStart[`AVFS3_RCNT_WIDTH-1:0]),       // input  [23:0]  AvfsSmpStart,     // oscillator enable start count
.AvfsSmpMax(AvfsSmpMax[`AVFS3_RCNT_WIDTH-1:0]),           // input  [23:0]  AvfsSmpMax,       // oscillator stop count
// Changing "AvfsStart" to "AvfsGo" pulse : http://bugs.amd.com/ubts/prod/issue/view?id=597035
.AvfsGo(AvfsGo),                              // input          AvfsGo,           // avfs go pulse to start the FSM sequence
.AvfsEn(AvfsEn),                                 // input          AvfsEn,           // AVFS FSM Enable signal
.AvfsRepeat(AvfsRepeat),                         // input          AvfsRepeat,       // if set, run AVFS continuously following AvfsGo pulse
.AvfsGateEn0(AvfsGateEn0),                       // input          AvfsGateEn0,       // Gatepack Enable
.AvfsGateEn1(AvfsGateEn1),                       // input          AvfsGateEn1,       // Gatepack Enable
.AvfsWireEn(AvfsWireEn),                         // input          AvfsWireEn,       // Wirepack Enable
.AvfsThermEn(AvfsThermEn),                       // input          AvfsThermEn,      // Thermpack Enable
.AvfsGatePackSel0(AvfsGatePackSel0[3:0]),        // input [3:0]    AvfsGatePackSel0,  // 1/32 Selection of Gatepack path
.AvfsGatePackSel1(AvfsGatePackSel1[3:0]),        // input [3:0]    AvfsGatePackSel1,  // 1/32 Selection of Gatepack path
.AvfsWirePackSel(AvfsWirePackSel[3:0]),          // input [3:0]    AvfsWirePackSel,  // 1/16 Selection of Wirepack path
.AvfsThermPackSel(AvfsThermPackSel[2:0]),        // input [2:0]    AvfsThermPackSel, // 1/7 Selection of Thermal Oscillator
.AvfsGatePackState0(AvfsGatePackState0[15:0]),   // input [31:0]   AvfsGatePackState,// Rest state of the AVFS Gate paths
.AvfsGatePackState1(AvfsGatePackState1[15:0]),   // input [31:0]   AvfsGatePackState,// Rest state of the AVFS Gate paths
.AvfsWirePackState(AvfsWirePackState[15:0]),     // input [15:0]   AvfsWirePackState,// Rest state of the AVFS Wire paths
.AvfsStateWrite(AvfsStateWrite),                 // input          AvfsStateWrite,   // Enables writing override value for AVFS replica path state (normally 1'b1)
.CCLK(CCLK),                                     // input          CCLK,             // core clock
.Reset(Reset),                                   // input          Reset,            // reset signal to reset flops on power-up
.ScanShiftEn(ScanShiftEn),                       
.GatePackCnt0(AvfsCpoGPCnt0[23:0]),            // output [23:0]  GatePackCnt,      // Flopped version of Gatepack Cnt
.GatePackCnt1(AvfsCpoGPCnt1[23:0]),            // output [23:0]  GatePackCnt,      // Flopped version of Gatepack Cnt
.WirePackCnt(AvfsCpoWPCnt[23:0]),              // output [23:0]  WirePackCnt,      // Flopped version of Wirepack Cnt   
.ThermPackCnt(AvfsCpoTPCnt[23:0]),              // output [23:0]  ThermPackCnt      // Flopped version of Thermpack Cnt
.SkipPhaseCnt(AvfsCpoSPCnt[23:0])              // output [23:0]  ThermPackCnt      // Flopped version of Thermpack Cnt
);

// Register Interfaces 
// SRB based
// AvfsCtl1
// The SRB drives data in two 32b chunks. We need to capture both to get the 50b mask register. 

// AVFS_CTL1 [56:0]
// AvfsThermPackSel[2:0]
// AvfsEn
// AvfsGateEn0
// AvfsGateEn1
// AvfsWireEn
// AvfsThermEn
// AvfsRepeat
// AvfsSmpStart[23:0]
// AvfsSmpMax[23:0]
// 
// AVFS_CTL2 [63:0]
// AvfsStateWrite
// AvfsGatePackSel0[3:0]
// AvfsGatePackSel1[3:0]
// AvfsWirePackSel[3:0]
// Rsvd[1:0]
// AvfsGatePackState0[15:0]
// AvfsGatePackState1[15:0]
// AvfsWirePackState[15:0]
// 
// AVFS_BUCKET0 [47:0]
// AvfsCpoGPCnt0[23:0]
// AvfsCpoGPCnt1[23:0]
// AVFS_BUCKET1 [47:0]
// AvfsCpoTPCnt[23:0]
// AvfsCpoWPCnt[23:0]

   wire [`CLKBUS_SIZE-1:0] clkbus_CgenAvfsCtl1;
   assign CgenSrbAvfsCtl1WrEn = SelAvfsCtl1 & SrbLclWrEn_4;

//  AvfsCtl1: 
  gfx_CORE_CG I_cg_CgenAvfsCtl1 (.clkbus(clkbus_CgenAvfsCtl1[`CLKBUS_SIZE-1:0]), .C(HostReg_CCLK), .G(CgenSrbAvfsCtl1WrEn), .R(Reset), .SSE(ScanShiftEn));

  gfx_CORE_FLOP_DGATE_RESET  #(.size(64),.rv(64'h0000000000ffffff)) I_AvfsCtl1 (.clkbus(clkbus_CgenAvfsCtl1[`CLKBUS_SIZE-1:0]), .DG(1'b1), .D(AvfsSrbWrDat_4[63:0]), .Q(AvfsCtl1[63:0]));

   assign AvfsThermPackSel[2:0] = AvfsCtl1[56:54];
   assign AvfsEn = AvfsCtl1[53];        // global CPO enable
   assign AvfsGateEn0 = AvfsCtl1[52];     
   assign AvfsGateEn1 = AvfsCtl1[51];     
   assign AvfsWireEn = AvfsCtl1[50];     
   assign AvfsThermEn = AvfsCtl1[49];     
   assign AvfsRepeat = AvfsCtl1[48];     
   assign AvfsSmpStart[`AVFS3_RCNT_WIDTH-1:0] = AvfsCtl1[47:24];
   assign AvfsSmpMax[`AVFS3_RCNT_WIDTH-1:0] = AvfsCtl1[23:0];

//! unused AvfsCtl2[63:54]

//  AvfsCtl2: 
   wire [`CLKBUS_SIZE-1:0] clkbus_CgenAvfsCtl2;
   assign CgenSrbAvfsCtl2WrEn = SelAvfsCtl2 & SrbLclWrEn_4;

  gfx_CORE_CG I_cg_CgenAvfsCtl2 (.clkbus(clkbus_CgenAvfsCtl2[`CLKBUS_SIZE-1:0]), .C(HostReg_CCLK), .G(CgenSrbAvfsCtl2WrEn), .R(Reset), .SSE(ScanShiftEn));

  gfx_CORE_FLOP_DGATE_RESET  #(.size(64),.rv(64'h0)) I_AvfsCtl2 (.clkbus(clkbus_CgenAvfsCtl2[`CLKBUS_SIZE-1:0]), .DG(1'b1), .D(AvfsSrbWrDat_4[63:0]), .Q(AvfsCtl2[63:0]));
   assign AvfsFuncDbgEn = AvfsCtl2[63]; // http://bugs.amd.com/ubts/prod/issue/view?id=610136
   assign AvfsStateWrite = AvfsCtl2[62];
//   assign AvfsGatePackSel0[3:0] = {4{~LBistEn}} & AvfsCtl2[61:58]; // Using ~LBistEn to force all 0s on AvfsOscSel to solve combinational loop issue for DFT
//   assign AvfsGatePackSel1[3:0] = {4{~LBistEn}} & AvfsCtl2[57:54];
//   assign AvfsWirePackSel[3:0] = {4{~LBistEn}} & AvfsCtl2[53:50];
   assign AvfsGatePackSel0[3:0] = AvfsCtl2[61:58]; 
   assign AvfsGatePackSel1[3:0] = AvfsCtl2[57:54];
   assign AvfsWirePackSel[3:0] = AvfsCtl2[53:50];
//   assign AvfsThermPackSel[1:0] = AvfsCtl2[49:48];                 // Don't need ~LBistEn here, loops are inside the custom macro, mathermalro 
   assign AvfsGatePackState0[15:0] = AvfsCtl2[47:32];
   assign AvfsGatePackState1[15:0] = AvfsCtl2[31:16];
   assign AvfsWirePackState[15:0] = AvfsCtl2[15:0];

wire [63:0] AvfsBucket0;
wire [63:0] AvfsBucket1;

assign AvfsBucket0 = {AvfsCpoSPCnt[15:0],AvfsCpoGPCnt0[23:0],AvfsCpoGPCnt1[23:0]};
assign AvfsBucket1 = {8'b0,AvfsCpoSPCnt[23:16],AvfsCpoWPCnt[23:0],AvfsCpoTPCnt[23:0]};

// SRB Read interface
// return zeroes for reserved bits
   assign AvfsSrbRdDat[63:0] = ({64{SelAvfsCtl1}}    & {AvfsCtl1[63:0]}) |
                               ({64{SelAvfsCtl2}}    & {AvfsCtl2[63:0]}) |
                               ({64{SelAvfsBucket0}} & {AvfsBucket0[63:0]}) |
                               ({64{SelAvfsBucket1}} & {AvfsBucket1[63:0]});


endmodule
                                
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// 
// 
//
// Top Level AVFS and PSM wrapper with muxed SPMI/p1500 interface and M&M Bus
//
// 
// 
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

// AVFS_PSM_WRAP old module name (in AVFS2.0)
module gfx_AVFS3_TOP #(parameter 
  CPO_TO_CCLK_SYNC_TC_CYCLES=1, 
  CCLK_TO_CPO_SYNC_TD_DELAY=5000,
  CDC_RESET_TO_CPOCLK_DLY = 5000,
  SYNC_TC_CYCLES       = 3,    // Overriding as 3 for Zen, Aspen and Zepplin at AVFS3_TOP, making the parameter programmable at AVFS3_TOP
  CDC_TST_PROP_DLY     = 5000,
  DBG_LENGTH           = 16,
  DBG_CFG_LENGTH       = 3,
  MCP_CYCLES_AVGPSMREG = 3,
  MCP_CYCLES_MAPSMRO   = 2,
  PSM_CURPSM_LENGTH    = 14,
  NEG_JTAG_TDO         = 0     // Enables TDO/WRO on negedge of JTAG clock
)(
//Scan options
//1. LSSD Scan with scan reset (Might not be needed - no product)
//2. MUXD with scan based reset (CM, ZAK)
//3. MUXD with async reset (KK)
  input                      PowerOk_VDD2, // New PowerOk port to drive PSM macro pwrOK pin
  input                      ScanShiftEn,  // Scan shift Enable
  input                      CGater01_CG0, // Coarse Gater
  input                      Reset,        // reset signal to reset flops for SOCs which don't use scan-shift-reset
  input                      PwrMonEnX,    // mux between SPMI (0) and p1500 path(1)
`ifndef AVFS_ZAK
  // ZAK does not support Spmi
  input                      SpmiClk,      // SPMI
  input                      SpmiTdi,      // SPMI
  input                      SpmiShift,    // SPMI
  input                      SpmiUpdate,   // SPMI
  input                      SpmiCapEn,    // SPMI
  input                      SpmiPathEn,   // SPMI
  input                      SpmiTrst,     // SPMI
  output wire                SpmiTdo,      // SPMI
  input                      SpmiMeas,     // SPMI
`endif
  input                      WRCK,         // p1500
  input                      WSI,          // p1500
  input                      ShiftWR,      // p1500
  input                      UpdateWR,     // p1500
  input                      CaptureWR,    // p1500
  input                      WDREn,        // p1500
  input                      WRSTN,        // p1500
  input                      MeasureWR,    // p1500 (does not exist in JTAG protocol - can use Idle signal)
  output wire                WRO,          // p1500
`ifndef AVFS_ZAK
  input                      Host_Clk,     // gated clk to AVFS3_CPO/GATEPACK/WIREPACK/THERMPACK
  input                      PsmClk,       // free running clk to PSM for AC scan test
`else
  input                      CCLK,             // ZAK uses mesh clock
  input                      TstCcgEn,         // ZAK test coarse gater enable (non-scannable flops would use 1'b1)
  input                      Cgen_UnitActive,  // ZAK coarse gater enable
  input                      RCLK_Free_AR,     // ZAK uses free-running clock for coarse gater flops
`endif
`ifdef GFX_DROOP_DETECT
  output wire [PSM_CURPSM_LENGTH-1:0] PsmOut, // adding new port to direct PSM count out through AVFS3 for GNL purposes
`endif
  input                      LBistEn,
  input                      PsmEn,
  input                      PsmAvfs_Gater,
  input                      Avfs_ScanMode,
  output wire                Psm_Alarm_Flag,
  input                      SkipPhase,    // from clock stretcher module - needs to be conditionally gated at the PSM when IP is updated
  input                      Isolate,      // indicates if tile containing AVFS module is being power-gated
  input [DBG_CFG_LENGTH-1:0] DbgBusSel,
  output wire [DBG_LENGTH-1:0] DbgBus

`ifdef AVFS_ZAK                            // M&M interface
  ,input       [`MM_DATA]    MmBusIn_M1,   //   8-bit M&M bus input contains local reg addr in M2 and write data in M4
  input                      MmLclRdEn_M2, //   local register read indication
  input                      MmLclWrEn_M2, //   local register write indication
  input       [1:0]          MmLclSz_M2,   //  local register size indication
  input       [`MM_STATE]    MmState,      //   M&M state for read/write byte select
  output wire [`MM_DATA]     MmDatOut_M4   //   8-bit data back to M&M bus for read
`endif

);
  wire             JTAG_ClkEn_ASYNC;
  wire             JTAG_ClkEn_CDC;
  wire             JTAG_ClkEn_CCLK;
  wire             MmActiveEn;

  wire             TrstSyncd;
  wire             CgenAvfsEn;
  wire             PsmDivClkGater;

  wire [DBG_LENGTH-1:0] DbgBus_int;
  wire [PSM_CURPSM_LENGTH-1:0] dbgCurPsm;
  wire [PSM_CURPSM_LENGTH-1:0] dbgAvgPsm;
  wire [PSM_CURPSM_LENGTH-1:0] dbgThreshold;
  wire [PSM_CURPSM_LENGTH-1:0] dbgMinMax;
  wire dbgAlarm;
   
  wire             AvfsGo;
  wire             AvfsTestData;
  wire [1:0]       AvfsTestMode;
  wire             dbgEn;
  wire [63:0]      SrbLclAvfsRdDat;
  wire             SrbLclSelAvfsCtl1;
  wire             SrbLclSelAvfsCtl2;
  wire             SrbLclSelAvfsBucket0;
  wire             SrbLclSelAvfsBucket1;
   wire [63:0]     PSM_RdDat;
   wire            MM_Psm_Enable;
   wire            SelPsmParam;
   wire            SelPsmData;
   wire            JTAG_Psm_Enable;
   wire            JTAG_Psm_AvgEn;
   wire            JTAG_Psm_Rst;
   wire            JTAG_Psm_Rst_Reset;
   wire            JTAG_Psm_OneShot;
   wire            Psm_Rst;
   
  // Register access decoder and signals
  wire RegisterRWLo;
  wire RegisterRWHi;
  wire RegisterReadEnLo;
  wire RegisterReadEnHi;
  wire RegisterWriteEn;
  wire [3:0] RegisterAddress;
  wire [63:0] RegisterWrDat;
  wire GlobalSelPsm;
  wire GlobalSelCPO;

  assign RegisterRWLo = RegisterReadEnLo | RegisterWriteEn;
  assign RegisterRWHi = RegisterReadEnHi | RegisterWriteEn;

  //decoder
  assign SelPsmParam = (RegisterAddress[3:0] == `AVFS_WRAP_PSM_PARAM_ADDR) & RegisterRWLo;
  assign SelPsmData = (RegisterAddress[3:0] == `AVFS_WRAP_PSM_DATA_ADDR) & RegisterRWLo;
  assign SrbLclSelAvfsCtl1 = (RegisterAddress[3:0] == `AVFS_WRAP_CPA_CTL1) & RegisterRWLo;
  assign SrbLclSelAvfsCtl2 = (RegisterAddress[3:0] == `AVFS_WRAP_CPA_CTL2) & RegisterRWLo;
  // unused assign SrbLclSelAvfsValid = (RegisterAddress[3:0] == `AVFS_WRAP_CPA_VALID) & RegisterRWLo;
  // unused assign SrbLclSelAvfsNorm0 = (RegisterAddress[3:0] == `AVFS_WRAP_CPA_NORM0) & RegisterRWLo;
  // unused assign SrbLclSelAvfsNorm1 = (RegisterAddress[3:0] == `AVFS_WRAP_CPA_NORM1) & RegisterRWLo;
  // unused assign SrbLclSelAvfsNorm2 = (RegisterAddress[3:0] == `AVFS_WRAP_CPA_NORM2) & RegisterRWLo;

  // Support 16-byte read of both AvfsBucket0 & AvfsBucket1 with one M&M transaction
  //   SrbLclSelAvfsBucket0 asserted for lower 8 bytes
  //   SrbLclSelAvfsBucket1 asserted for upper 8 bytes
  assign SrbLclSelAvfsBucket0 = (RegisterAddress[3:0] == `AVFS_WRAP_CPA_BUCKET0) & RegisterRWLo;
  assign SrbLclSelAvfsBucket1 = ((RegisterAddress[3:0] == `AVFS_WRAP_CPA_BUCKET1) & RegisterRWLo) |
                                ((RegisterAddress[3:0] == `AVFS_WRAP_CPA_BUCKET0) & RegisterRWHi);

  assign GlobalSelPsm = (RegisterAddress[3:0] == `AVFS_WRAP_PSM_PARAM_ADDR) |
                        (RegisterAddress[3:0] == `AVFS_WRAP_PSM_DATA_ADDR);
  assign GlobalSelCPO = (RegisterAddress[3:0] == `AVFS_WRAP_CPA_CTL1) |
                        (RegisterAddress[3:0] == `AVFS_WRAP_CPA_CTL2) |
                        (RegisterAddress[3:0] == `AVFS_WRAP_CPA_BUCKET0) |
                        (RegisterAddress[3:0] == `AVFS_WRAP_CPA_BUCKET1);

`ifdef AVFS_ZAK
// Force Spmi inputs to 1'b0 and declare output as unused
  wire           SpmiClk;
  wire           SpmiTdi;
  wire           SpmiShift;
  wire           SpmiUpdate;
  wire           SpmiCapEn;
  wire           SpmiPathEn;
  wire           SpmiTrst;
  wire           SpmiMeas;

  assign         SpmiClk = 1'b0;
  assign         SpmiTdi = 1'b0;
  assign         SpmiShift = 1'b0;
  assign         SpmiUpdate = 1'b0;
  assign         SpmiCapEn = 1'b0;
  assign         SpmiPathEn = 1'b0;
  assign         SpmiTrst = 1'b0;
  assign         SpmiMeas = 1'b0;

  wire           PsmClk;
  wire           Host_Clk;
  wire           Psm_ClkEn;
  wire           HostJtag_ClkEn;

  wire Reset_d1;
  wire [`CLKBUS_SIZE-1:0] clkbus_free;

  // // Adding a flop on Reset to block Xs at ZAK level simulations as the core methodology
  // // drives Xs initially before clocks are toggling. Flop Reset here drive Reset_d1 downstream
  gfx_CORE_CGU I_cgu_clkbus_free (.clkbus(clkbus_free[`CLKBUS_SIZE-1:0]), .C(Host_CDC_Clk), .R(Reset), .SSE(ScanShiftEn));
  gfx_CORE_FLOP_DGATE_RESET #(.size(1),.rv(1'b1)) I_Reset_d1 (.D(Reset), .DG(1'b1), .Q(Reset_d1), .clkbus(clkbus_free[`CLKBUS_SIZE-1:0]));

   //CORE_CDCENBUF #(.size(1), .Cycles(0), .Setup(CDC_TST_PROP_DLY))
   //  I_CDC_LBistEn (.A(LBistEn),
   //                 .Z(LBistEn_CDC),
   //                 .EN(1'b1),
   //                 .DSTCLK(CCLK) );   
   //
   // Fix for http://bugs.amd.com/ubts/prod/issue/view?id=643502
   techind_cdcefpm_icd #(.DestClkCnt(0), .CdcDestSetup(CDC_TST_PROP_DLY), .size(1)) I_LBistEn_CDC (
     .A(LBistEn), 
     .DSTCLK(CCLK), 
     .EN(1'b1), 
     .Z(LBistEn_CDC)
     `ifdef ABV
       ,.InputChkDis((WRSTN!==1'b1)) 
     `endif
     );

  wire Avfs_ScanMode_CDC;

  //assign         HostJtag_CDC_ClkEn = JTAG_ClkEn_CDC | JTAG_ClkEn_CCLK;  // async assertion and synchronous de-assertion
  assign         HostJtag_ClkEn = JTAG_ClkEn_CCLK;  // synchronous-only assertion and de-assertion
  assign         HostReg_ClkEn  = JTAG_ClkEn_CCLK | (MmState[`MM_STATE]==`MM_STATE_M2) | MmActiveEn;  // shallow enable term to feed CCG_NF (i.e. fast wakeup for MM transactions)
//assign         HostCpo_ClkEn  = Psm_Rst | MM_Psm_Enable | JTAG_Psm_Enable | LBistEn;
//assign         Psm_ClkEn      = Psm_Rst | MM_Psm_Enable | JTAG_Psm_Enable | JTAG_Psm_Rst_Reset;
  assign         HostCpo_ClkEn  = Psm_Rst | MM_Psm_Enable | JTAG_Psm_Enable | LBistEn_CDC | Avfs_ScanMode_CDC; //Term HostReg_ClkEn is safe to be removed, as CPO wakes up with Psm_Enable; added Avfs_ScanMode_CDC for DECPLCPLK-2355
  assign         Psm_ClkEn      = Psm_Rst | MM_Psm_Enable | JTAG_Psm_Enable | Avfs_ScanMode_CDC; //Terms HostReg_ClkEn and JTAG_Psm_Rst_Reset are safe to be removed, as PSM wakes up with Psm_Enable; added Avfs_ScanMode_CDC for DECPLCPLK-2355


  // vscan off cellprop
  gfx_ccgu  I_KCLK_Free_AR         (.gclk(Host_CDC_Clk), .clk(CCLK), .TstCcgEn(TstCcgEn)); // Removing gating from Host_CDC_Clk, this really should be called Free clock, runs just one synchronizer in AvfsWrapJtgInterface
  //ccg    I_KCLK_HostJtag_CDC_Clk_AR (.gclk(Host_CDC_Clk),     .clk(CCLK), .fclk(RCLK_Free_AR), .en(HostJtag_CDC_ClkEn), .TstCcgEn(TstCcgEn), .SSE(ScanShiftEn));
  gfx_ccg    I_KCLK_HostJtag_Clk_AR (.gclk(Host_Clk),     .clk(CCLK), .fclk(RCLK_Free_AR), .en(HostJtag_ClkEn), .TstCcgEn(TstCcgEn), .SSE(ScanShiftEn));
  gfx_ccg_nf I_KCLK_HostReg_Clk_AR  (.gclk(HostReg_Clk),  .clk(CCLK),                      .en(HostReg_ClkEn),  .TstCcgEn(TstCcgEn), .SSE(ScanShiftEn));
  gfx_ccg    I_KCLK_Host_Clk_AR     (.gclk(HostCpo_Clk),  .clk(CCLK), .fclk(RCLK_Free_AR), .en(HostCpo_ClkEn),  .TstCcgEn(TstCcgEn), .SSE(ScanShiftEn));
  gfx_ccg    I_KCLK_PsmClk_AR       (.gclk(PsmClk),       .clk(CCLK), .fclk(RCLK_Free_AR), .en(Psm_ClkEn),      .TstCcgEn(TstCcgEn), .SSE(ScanShiftEn));
  // vscan on cellprop

  assign PsmReg_Clk   = HostReg_Clk;  // TODO:  will this cause ATPG problems?  PSM is non-scan, but "HostReg_Clk" also drives scannable flops
`else
  assign HostCpo_Clk  = Host_Clk;  // needed to allow non-AVFS_ZAK compile
  assign PsmReg_Clk   = PsmClk;    // needed to allow non-AVFS_ZAK compile
  assign MmActiveEn   = 1'b0;      // needed to allow non-AVFS_ZAK compile
//!unused JTAG_ClkEn_ASYNC
//!unused JTAG_ClkEn_CCLK
`endif

  gfx_AvfsWrapJtgInterface #(.SYNC_TC_CYCLES(SYNC_TC_CYCLES), .NEG_JTAG_TDO(NEG_JTAG_TDO)) I_AvfsWrapJtgInterface (
                                            .RegisterOutput(RegisterWrDat[63:0]),
                                            .TrstSyncd_CCLK(TrstSyncd),
                                            .JTAG_ClkEn_ASYNC(JTAG_ClkEn_ASYNC),
                                            .JTAG_ClkEn_CCLK(JTAG_ClkEn_CCLK),
                                            .RegisterAddress(RegisterAddress[3:0]),
                                            .RegisterWriteEn(RegisterWriteEn),
                                            .RegisterReadEnLo(RegisterReadEnLo),
                                            .RegisterReadEnHi(RegisterReadEnHi),
                                            .JTAG_Psm_Enable(JTAG_Psm_Enable),
                                            .JTAG_Psm_AvgEn(JTAG_Psm_AvgEn),
                                            .JTAG_Psm_Rst(JTAG_Psm_Rst),
                                            .JTAG_Psm_OneShot(JTAG_Psm_OneShot),
                                            .AvfsGo_OneShot(AvfsGo),
                                            .JTAG_DbgEn(dbgEn),
                                            .RegisterInput(SrbLclAvfsRdDat[63:0] | PSM_RdDat[63:0]),
                                            .PwrMonEnX(PwrMonEnX),
                                            .SpmiClk(SpmiClk),
                                            .SpmiTdi(SpmiTdi),
                                            .SpmiShift(SpmiShift),
                                            .SpmiUpdate(SpmiUpdate),
                                            .SpmiCapEn(SpmiCapEn),
                                            .SpmiTrst(SpmiTrst),
                                            .SpmiTdo(SpmiTdo),
                                            .SpmiMeas(SpmiMeas),
                                            .SpmiPathEn(SpmiPathEn),
                                            .WRCK(WRCK),
                                            .WSI(WSI),
                                            .ShiftWR(ShiftWR),
                                            .UpdateWR(UpdateWR),
                                            .CaptureWR(CaptureWR),
                                            .WDREn(WDREn),
                                            .WRSTN(WRSTN),
                                            .MeasureWR(MeasureWR),
                                            .WRO(WRO),
                                            .PsmAvfs_Gater(PsmAvfs_Gater),
                                            .Isolate(Isolate),
                                            .SkipPhase(SkipPhase),
                                            .Reset(Reset_d1),
                                            .CCLK(Host_Clk),
                                            .CCLK_CDC(Host_CDC_Clk),
                                            .HostReg_CCLK(HostReg_Clk)
                                          `ifdef AVFS_ZAK                         // M&M bus interface
                                          , .MmBusIn_M1(MmBusIn_M1[7:0]),         //   8-bit M&M bus input contains local reg addr in M2 and write data in M4
                                            .MmLclRdEn_M2(MmLclRdEn_M2),          //   local register read indication
                                            .MmLclWrEn_M2(MmLclWrEn_M2),          //   local register write indication
                                            .MmLclSz_M2(MmLclSz_M2[1:0]),         //   local register size indication
                                            .MmState(MmState[`MM_STATE]),         //   M&M state for read/write byte select
                                            .MmDatOut_M4(MmDatOut_M4[`MM_DATA]),  //   8-bit data back to M&M bus for read
                                            .ScanShiftEn(ScanShiftEn),            //   SSE needed for M&M write staging flops
                                            .LBistEn(LBistEn_CDC),                //   SSE needed for M&M write staging flops
                                            .MmActiveEn(MmActiveEn)               // feedback to top-level CCG to allow register access (i.e. wake up clocks)
                                          `endif
                                            );

`ifdef AVFS_ZAK
// SpmiTdo is not an output for ZAK
//! unused SpmiTdo
`endif

// Default ck-gater output from AVFS_CPA but not used because there are no flops at top level
//! unused CgenAvfsEn
                     
  wire [23:0] AvfsCpoGPCnt0;
  wire [23:0] AvfsCpoGPCnt1;
  wire [23:0] AvfsCpoWPCnt;
  wire [23:0] AvfsCpoTPCnt;
  wire [23:0] AvfsCpoSPCnt;
  wire AvfsFuncDbgEn;

           
  gfx_AVFS3_CPO_WRAP #(
.CDC_TST_PROP_DLY(CDC_TST_PROP_DLY),
.CCLK_TO_CPO_SYNC_TD_DELAY(CCLK_TO_CPO_SYNC_TD_DELAY),
.CPO_TO_CCLK_SYNC_TC_CYCLES(CPO_TO_CCLK_SYNC_TC_CYCLES),
.CDC_RESET_TO_CPOCLK_DLY(CDC_RESET_TO_CPOCLK_DLY)
) I_AVFS_CPO_WRAP (
.ScanShiftEn(ScanShiftEn),
.CGater01_CG0(CGater01_CG0),
.Reset(Reset_d1),
.CCLK(HostCpo_Clk),
.HostReg_CCLK(HostReg_Clk),
.CgenAvfsEn(CgenAvfsEn),
.AvfsFuncDbgEn(AvfsFuncDbgEn),
.AvfsGo(AvfsGo),
.LBistEn(LBistEn_CDC),
.SkipPhase(SkipPhase),
.AvfsTestData(AvfsTestData),
.AvfsTestMode(AvfsTestMode[1:0]),
.AvfsSrbRdDat(SrbLclAvfsRdDat[63:0]),
.SelAvfsCtl1(SrbLclSelAvfsCtl1),
.SelAvfsCtl2(SrbLclSelAvfsCtl2),
.SelAvfsBucket0(SrbLclSelAvfsBucket0),
.SelAvfsBucket1(SrbLclSelAvfsBucket1),
.AvfsSrbWrDat_4(RegisterWrDat[63:0]),
.SrbLclWrEn_4(GlobalSelCPO & RegisterWriteEn),
.AvfsCpoGPCnt0(AvfsCpoGPCnt0[23:0]),
.AvfsCpoGPCnt1(AvfsCpoGPCnt1[23:0]),
.AvfsCpoWPCnt(AvfsCpoWPCnt[23:0]),
.AvfsCpoTPCnt(AvfsCpoTPCnt[23:0]),
.AvfsCpoSPCnt(AvfsCpoSPCnt[23:0])
);  


   // Added a Host Clock Reset signal to JTAG_Psm_Rst_Reset for initialization at start-up.
   assign     JTAG_Psm_Rst_Reset = JTAG_Psm_Rst | Reset_d1 | ScanShiftEn ;
   assign     Psm_Rst = Reset_d1 | ScanShiftEn ;

   // PSM Oscillator connections
   wire [1:0]  clkDivider;
   wire        oscEn;
   wire [31:0] psmOscLsb;
   wire [7:0]  psmOscMsb0;
   wire [7:0]  psmOscMsb1;
   wire        psmOscGater;
   wire        psmOscGater_1;
   
   wire [13:0]      PsmDeltaOut;
   wire [31:0] psmOscLsb_mcp;
   wire [7:0] psmOscMsb0_mcp;
   wire [7:0] psmOscMsb1_mcp;

  gfx_CORE_PSM_IP_NO_JTAG #(
                         .PSM_CURPSM_LENGTH(PSM_CURPSM_LENGTH),
                         .MCP_CYCLES_MAPSMRO(MCP_CYCLES_MAPSMRO),
                         .MCP_CYCLES_AVGPSMREG(MCP_CYCLES_AVGPSMREG)
                         ) I_Psm (
       .ScanShiftEn(ScanShiftEn),
       .Psm_Rst(Psm_Rst),
       .Psm_ScanMode(Avfs_ScanMode_CDC),
       .Psm_Gater(PsmAvfs_Gater),  
       .JTAG_Psm_Enable(JTAG_Psm_Enable),
       .JTAG_Psm_AvgEn(JTAG_Psm_AvgEn),
       .JTAG_Psm_Rst(JTAG_Psm_Rst_Reset),
       .JTAG_Psm_OneShot(JTAG_Psm_OneShot),               
       .PsmEn(PsmEn),
       .Host_Clk(PsmClk),
       .HostReg_Clk(PsmReg_Clk),
       .nullNxtAvg(SkipPhase),
       .div_clk_gater(PsmDivClkGater), // unused
`ifdef GFX_DROOP_DETECT
       .psmCountOut(PsmOut[PSM_CURPSM_LENGTH-1:0]), // directing PSM count out through AVFS3 for GNL purposes
`endif
       .deltaPsmRegOut(PsmDeltaOut[PSM_CURPSM_LENGTH-1:0]), // unused
       .Alarm_Flag(Psm_Alarm_Flag),
       .WrDat(RegisterWrDat[63:0]),
       .RdDat(PSM_RdDat[63:0]),
       .MM_Psm_Enable(MM_Psm_Enable),
       .SelPsmParam(SelPsmParam),
       .SelPsmData(SelPsmData),
       .WrEn(GlobalSelPsm & RegisterWriteEn),
       .clkDivider(clkDivider),
       .oscEn(oscEn),
       .psmOscLsb(psmOscLsb_mcp),
       .psmOscMsb0(psmOscMsb0_mcp),
       .psmOscMsb1(psmOscMsb1_mcp),
       .psmOscGater(psmOscGater_1),            
       .dbgCurPsm(dbgCurPsm[PSM_CURPSM_LENGTH-1:0]),
       .dbgAvgPsm(dbgAvgPsm[PSM_CURPSM_LENGTH-1:0]),
       .dbgThreshold(dbgThreshold[PSM_CURPSM_LENGTH-1:0]),
       .dbgMinMax(dbgMinMax[PSM_CURPSM_LENGTH-1:0]),
       .dbgAlarm(dbgAlarm)
        );

  // PSM Oscillator Macro Instantiation
  gfx_CORE_PSM_IP_OSC #(
    .JTAG_CLK_DIV_W (2),
    .MCP_CYCLES_MAPSMRO(MCP_CYCLES_MAPSMRO),
    .CURPSM_W       (14)
  )
  I_CORE_PSM_IP_OSC (
    .clk         (CCLK), // Fixing for http://bugs.amd.com/ubts/prod/issue/view?id=590907
    .div         (clkDivider[1:0]),
    .pwrOk       (PowerOk_VDD2), // Adding new pwrok port on AVFS3_TOP
    .oscEn       (oscEn),
    .psmOscLsb   (psmOscLsb[31:0]),
    .psmOscMsb0  (psmOscMsb0[7:0]),
    .psmOscMsb1  (psmOscMsb1[7:0]),
    .psmOscGater (psmOscGater)
  );

  // Move the MCBUF here on mapsmro outputs as Reset is not available inside CORE_PSM_IP_OSC wrapper 

`ifdef AVFS3_MCENBUF_ON
  wire mcbufEn;
  assign mcbufEn = oscEn & (~Psm_Rst) & psmOscGater ; 
  gfx_CORE_MCENBUF #(.size(32), .Cycles(MCP_CYCLES_MAPSMRO)) I_psmOscLsb_mcp (.A(psmOscLsb[31:0]), 
                                                                          .EN(mcbufEn),
                                                                          .DSTCLK(CCLK),
                                                                          .Z(psmOscLsb_mcp[31:0]));
  gfx_CORE_MCENBUF #(.size(8), .Cycles(MCP_CYCLES_MAPSMRO)) I_psmOscMsb0_mcp (.A(psmOscMsb0[7:0]), 
                                                                          .EN(mcbufEn),
                                                                          .DSTCLK(CCLK),
                                                                          .Z(psmOscMsb0_mcp[7:0]));
  gfx_CORE_MCENBUF #(.size(8), .Cycles(MCP_CYCLES_MAPSMRO)) I_psmOscMsb1_mcp (.A(psmOscMsb1[7:0]), 
                                                                          .EN(mcbufEn),
                                                                          .DSTCLK(CCLK),
                                                                          .Z(psmOscMsb1_mcp[7:0]));
`else
  gfx_CORE_MCBUF #(.size(32), .Cycles(MCP_CYCLES_MAPSMRO)) I_psmOscLsb_mcp (.A(psmOscLsb[31:0]), 
                                                                          .DSTCLK(CCLK),
                                                                          .Z(psmOscLsb_mcp[31:0]));
  gfx_CORE_MCBUF #(.size(8), .Cycles(MCP_CYCLES_MAPSMRO)) I_psmOscMsb0_mcp (.A(psmOscMsb0[7:0]), 
                                                                          .DSTCLK(CCLK),
                                                                          .Z(psmOscMsb0_mcp[7:0]));
  gfx_CORE_MCBUF #(.size(8), .Cycles(MCP_CYCLES_MAPSMRO)) I_psmOscMsb1_mcp (.A(psmOscMsb1[7:0]), 
                                                                          .DSTCLK(CCLK),
                                                                          .Z(psmOscMsb1_mcp[7:0]));
`endif

  assign psmOscGater_1 = LBistEn_CDC ? 1'b1 : psmOscGater ; // Forcing psmOscGater to 1'b1 in LBistEn mode

// debug interface
  reg  [DBG_LENGTH-1:0] DbgBusNxt;
  wire dbgEn_ScanMode;
   
                                                        // Review Zijian's stimuli, likely a test case issue
  // CDCBUF for Avfs_ScanMode (ATPG test JTAG -> Host_clk)
  //CORE_CDCBUF #(.size(1), .Cycles(`CDC_JTAG_X_DLY_CYCLES))
  //CORE_CDCENBUF #(.size(1), .Cycles(0), .Setup(CDC_TST_PROP_DLY)) // add a top level param to configure X prop delay
  //   I_CDC_Avfs_ScanMode (.A(Avfs_ScanMode),
  //                  .Z(Avfs_ScanMode_CDC),
  //                  .EN(1'b1),
  //                  .DSTCLK(Host_Clk) );   
   // Fix for http://bugs.amd.com/ubts/prod/issue/view?id=643502
  techind_cdcefpm_icd #(.DestClkCnt(0), .CdcDestSetup(CDC_TST_PROP_DLY), .size(1)) I_Avfs_ScanMode_CDC (
     .A(Avfs_ScanMode), 
     .DSTCLK(Host_Clk), 
     .EN(1'b1), 
     .Z(Avfs_ScanMode_CDC)
     `ifdef ABV
       ,.InputChkDis((WRSTN!==1'b1)) 
     `endif
     );

  // adding debug counter for DECPLCPLK-2833
  logic [3:0] DbgCntrNxt;
  logic [3:0] DbgCntr;
  assign  DbgCntrNxt = DbgCntr + 1;
  gfx_CORE_FLOP_GATED_RESET  #(.size(4),.rv({4'b0}))  I_DbgCntr (.D(DbgCntrNxt[3:0]), .Q(DbgCntr[3:0]), .G(dbgEn_ScanMode), .GC(1'b0), .R(TrstSyncd|Reset_d1), .SSE(ScanShiftEn), .C(Host_Clk)); // NOTE: ".GC(1'b0)" is unused

  assign dbgEn_ScanMode = AvfsFuncDbgEn | dbgEn | Avfs_ScanMode_CDC; // http://bugs.amd.com/ubts/prod/issue/view?id=610136
  always @* begin
    casez({dbgEn_ScanMode, DbgBusSel[DBG_CFG_LENGTH-1:0]})
      4'b0_???: DbgBusNxt[DBG_LENGTH-1:0] = {DBG_LENGTH{1'b0}};
      4'b1_000: DbgBusNxt[DBG_LENGTH-1:0] = {1'b0, dbgCurPsm[PSM_CURPSM_LENGTH-1:0],dbgAlarm}; //DbgBusPSM
      4'b1_001: DbgBusNxt[DBG_LENGTH-1:0] = {1'b0, dbgAvgPsm[PSM_CURPSM_LENGTH-1:0],dbgAlarm}; //DbgBusPSM
      //4'b1_010: DbgBusNxt[DBG_LENGTH-1:0] = {2'b0, dbgThreshold[PSM_CURPSM_LENGTH-1:0]}; //DbgBusPSM
      4'b1_010: DbgBusNxt[DBG_LENGTH-1:0] = {{12'hAFE}, DbgCntr[3:0]}; //Adding AVFS signature (12'hAFE), counter value
      4'b1_011: DbgBusNxt[DBG_LENGTH-1:0] = {1'b0, dbgMinMax[PSM_CURPSM_LENGTH-1:0], dbgAlarm}; //DbgBusPSM
      4'b1_100: DbgBusNxt[DBG_LENGTH-1:0] = {AvfsCpoGPCnt0[15:0]}; //DbgBus, capturing only 16 LSB bits
      4'b1_101: DbgBusNxt[DBG_LENGTH-1:0] = {AvfsCpoGPCnt1[15:0]}; //DbgBus, capturing only 16 LSB bits
      4'b1_110: DbgBusNxt[DBG_LENGTH-1:0] = {AvfsCpoWPCnt[15:0]}; //DbgBus, capturing only 16 LSB bits
      4'b1_111: DbgBusNxt[DBG_LENGTH-1:0] = {AvfsCpoTPCnt[15:0]}; //DbgBus, capturing only 16 LSB bits
      // VCS coverage off
      default: DbgBusNxt[DBG_LENGTH-1:0]      = {DBG_LENGTH{1'bx}};
      // VCS coverage on
    endcase
  end

  gfx_CORE_FLOP_GATED_RESET  #(.size(DBG_LENGTH))  I_DbgBus (.D(DbgBusNxt[DBG_LENGTH-1:0]), .Q(DbgBus_int[DBG_LENGTH-1:0]), .G(dbgEn_ScanMode | TrstSyncd), .GC(1'b0), .R(TrstSyncd|Reset_d1), .SSE(ScanShiftEn), .C(Host_Clk)); // NOTE: ".GC(1'b0)" is unused

  // Avfs_ScanMode==1 will force DbgBus and Alarm_Flag to 0

  assign DbgBus[DBG_LENGTH-1:0] = DbgBus_int[DBG_LENGTH-1:0] & {DBG_LENGTH{~Avfs_ScanMode_CDC}};
  

                 
endmodule // AVFS3_TOP
//==================================================================================================
//! enable one_module_per_file
// leda E_001 on
//==================================================================================================
