//*************************************************************************************************
//
// FILE_OWNER: kelvin.goveas@amd.com
//
//  Copyright  2013 Advanced Micro Devices, Inc.
//  All rights reserved.
//
//  This file is protected by Federal Copyright Law, with all rights
//  reserved. No part of this file may be reproduced, stored in a
//  retrieval system, translated, transcribed, or transmitted, in any
//  form, or by any means manual, electric, electronic, mechanical,
//  electro-magnetic, chemical, optical, or otherwise, without prior
//  explicit written permission from Advanced Micro Devices, Inc.
//
//*************************************************************************************************
//==================================================================================================
//==================================================================================================
//  Non-scanable registers use the following flop wrappers:
//
//    CORE_PSM_IP_FLOP
//    CORE_PSM_IP_FLOP_GATED
//
//==================================================================================================
//==================================================================================================
//! disable one_module_per_file
// Disable the Leda one-per-file check
// leda E_001 off

`define CLKBUS_SIZE 3
`define C_PIN       2
`define R_PIN       1
`define SSE_PIN     0


//CORE_NOSCAN_FLOP_DGATE
// ===============
module gfx_CORE_NOSCAN_FLOP_DGATE #(parameter
  size = 1,
  rv = {size{1'b0}}
)(
  input [size-1:0] D,
  input DG,  // 1=D flows into flop, 0=flop holds value
  input [`CLKBUS_SIZE-1:0] clkbus,
  output reg [size-1:0] Q
);

  wire C;
  wire R;
  wire [size-1:0] dgate_d;

  assign C=clkbus[`C_PIN];
  assign R=clkbus[`R_PIN];

  always @ ( posedge C ) begin
    if( R ) begin
      Q[size-1:0] <= rv[size-1:0];
//      Q[size-1:0] <= {size{1'b0}};
    end
    else if( DG ) begin                                       // implement re-circulating data-gating DMUX
      Q[size-1:0] <= D[size-1:0];
    end
  end

//!unused clkbus[`SSE_PIN]

endmodule // CORE_NOSCAN_FLOP_DGATE

// CORE_FLOP_DGATE
// ===============
module gfx_CORE_FLOP_DGATE #(parameter
  size = 1,
  rv = {size{1'b0}},
  dro = 0,
  et = 0
)(
  input [size-1:0] D,
  input DG,  // 1=D flows into flop, 0=flop holds value
  input [`CLKBUS_SIZE-1:0] clkbus,
  output reg [size-1:0] Q
);

  wire C;
  wire R;
  wire SSE;

  assign C=clkbus[`C_PIN];
  assign R=clkbus[`R_PIN];
  assign SSE=clkbus[`SSE_PIN];

  always @ ( posedge C ) begin
    if( SSE ) begin
      Q[size-1:0] <= rv[size-1:0];
    end
    else if( R ) begin
      Q[size-1:0] <= rv[size-1:0];
    end
    else if( DG ) begin                                       // implement re-circulating data-gating DMUX
      Q[size-1:0] <= D[size-1:0];
    end
  end


endmodule // CORE_FLOP_DGATE

module gfx_CORE_FLOP_DGATE_RESET #(parameter
  size = 1,
  rv = {size{1'b0}},
  dro = 0,
  et = 0
)(
  input [size-1:0] D,
  input DG,  // 1=D flows into flop, 0=flop holds value
  input [`CLKBUS_SIZE-1:0] clkbus,
  output reg [size-1:0] Q
);

  wire C;
  wire R;
  wire SSE;

  assign C=clkbus[`C_PIN];
  assign R=clkbus[`R_PIN];
  assign SSE=clkbus[`SSE_PIN];

  always @ ( posedge C ) begin
    if( SSE ) begin
      Q[size-1:0] <= rv[size-1:0];
    end
    else if( R ) begin
      Q[size-1:0] <= rv[size-1:0];
    end
    else if( DG ) begin                                       // implement re-circulating data-gating DMUX
      Q[size-1:0] <= D[size-1:0];
    end
  end


endmodule // CORE_FLOP_DGATE_RESET

// CORE_CG
// ===============
module gfx_CORE_CG
(
  input C,
  input G,
  input R,
  input SSE,
  output [`CLKBUS_SIZE-1:0] clkbus
);

 wire KCLK_CgenEn_AR;
 wire RCLK_CgenEn_AR;
 assign KCLK_CgenEn_AR = C;
  
  // RRR: OR-ing "Reset" with "Gater Enable". This is to address potential rest/init issues
  // like the one highlighted in http://bugs.amd.com/ubts/prod/issue/view?id=523957. Although in
  // ZAK like designs scan-shift-reset should ensure that we get clocks in ColdReset state
  // This is mainly needed in dGPU/SOC designs, but duplicating it here for proper verif coverage
  ati_clock_gate d0nt_avfs_clock_gate_FuncMGCG (
    .clk_src    (KCLK_CgenEn_AR),
    .clk_out    (RCLK_CgenEn_AR),
    .clk_en     (G | SSE | R ), // RRR: OR-ing Reset 
    .TE         (1'b0)          
  );

  assign clkbus[`C_PIN] = RCLK_CgenEn_AR;
  assign clkbus[`R_PIN] = R;
  assign clkbus[`SSE_PIN] = SSE;   


endmodule // CORE_CG

// CORE_CGU
// ========
module gfx_CORE_CGU
(
  input C,
  input R,
  input SSE,
  output [`CLKBUS_SIZE-1:0] clkbus
);



  assign clkbus[`C_PIN] = C;
  assign clkbus[`R_PIN] = R;
  assign clkbus[`SSE_PIN] = SSE;

endmodule // CORE_CGU


// CORE_PSM_IP_FLOP 
// ===============
module gfx_CORE_PSM_IP_FLOP #(parameter
  size = 1
)(
  input C,
  input [size-1:0] D,
  input R,
  output reg [size-1:0] Q
);

//  wire [size-1:0] d0;
//  assign d0[size-1:0] = D[size-1:0] & ~{size{R}};
//  etl1noscan #(.upper(size-1),.lower(0))  I_dff_noscan (.clk(C), .d(d0[size-1:0]), .q(Q[size-1:0]));


  always @ ( posedge C ) begin
    if( R ) begin
      Q <= {size{1'b0}};
    end
    else begin
      Q <= D;
    end
  end

endmodule // CORE_PSM_IP_FLOP


// CORE_PSM_IP_FLOP_GATED 
// ================
module gfx_CORE_PSM_IP_FLOP_GATED #(parameter
  size = 1
)(
  input C,
  input [size-1:0] D,
  input G,
  input R,
  output reg [size-1:0] Q
);

//  wire [size-1:0] d0;
//  wire    RCLK_gated;
   
  
//  com1  I_cg_noscan (.cond_clk(RCLK_gated), .clk(C), .condition(G), .SSE(1'b0));

//  assign d0[size-1:0] = D[size-1:0] & ~{size{R}};
//etl1noscan #(.upper(size-1),.lower(0))  I_dff_noscan (.clk(RCLK_gated), .d(d0[size-1:0]), .q(Q[size-1:0]));

  reg gate_1;
  wire clk_gated;

  ati_clock_gate d0nt_psm_clock_gate_FuncMGCG (
    .clk_src    (C),
    .clk_out    (clk_gated),
    .clk_en     (G),
    .TE         (1'b0)
  );

  always @ ( posedge clk_gated ) begin
    if( R ) begin
      Q <= {size{1'b0}};
    end
    else begin
      Q <= D;
    end
  end

endmodule // CORE_PSM_IP_FLOP_GATED


// Synchronizer with an asynchronous reset
module gfx_AVFS_SYNC_NOCG_AR #(parameter
  size = 1,
  TC=0, 
  TD=0,
  IC=1,
  rv = {size{1'b0}}                   //data-based reset isn't ideal, but X from SSE_CDCBUF causes synchronizer to X when using asyncFlush mode
)(
  input SampleClk,
  input [size-1:0] DataIn,
  input Reset,
  input SSE,
  output wire [size-1:0] DataOut
);

wire             TECHIND_TIELO       ;
wire             TECHIND_TIEHI       ;
wire [size-1:0]  TECHIND_UNUSED_QSRC ;
wire [size-1:0]  TECHIND_UNUSED_SO   ;

assign  TECHIND_TIELO  =  1'b0;
assign  TECHIND_TIEHI  =  1'b1;

wire [size-1:0] d;
wire [size-1:0] q;

//wire [size-1:0] D_reset;
//assign D_reset = Reset ? rv : DataIn; //data-based reset isn't ideal, but X from SSE_CDCBUF causes synchronizer to X when using asyncFlush mode (see bug 637452)
//
//assign d[size-1:0] = D_reset;         //data-based reset isn't ideal, but X from SSE_CDCBUF causes synchronizer to X when using asyncFlush mode (see bug 637452)
assign d[size-1:0] = DataIn;        //data-based reset isn't ideal, but X from SSE_CDCBUF causes synchronizer to X when using asyncFlush mode (see bug 637452)
assign DataOut = q[size-1:0];

techind_sync #(
  .size          (1),  // default 1
  .depth         (4),  // default 2
  .sourceFlop    (0),  // default 0
  .halfStage     (0),  // default 0
  .asyncSet      (0),  // default 0
  .asyncReset    (1),  // default 0
  .asyncFlush    (0),  // default 0   
  .clockEnable   (0),  // default 0
  .scan_muxD     (1),  // default 0
  .scan_LSSD     (0),  // default 0
  .MultiVolt     (0),  // default 0
  .TransportDelay(TD), // default 0
  .TransportCycle(TC), // default 0
  .InputCheck    (IC), // default 1
  .MetaWindowMode(0)   // default 0
)
  techind_sync__scan[size-1:0] (
  .SB       ( TECHIND_TIEHI                ),
  .RB       ( ~Reset                       ),
  .FLUSH    ( TECHIND_TIELO                ), 
  .CKsrc    ( TECHIND_TIELO                ),
  .CKdst    ( SampleClk                    ),
  .EN       ( TECHIND_TIEHI                ),
  .Dsrc     ( d[size-1:0]                  ),
  .Qsrc     ( TECHIND_UNUSED_QSRC[size-1:0]),
  .Qdst     ( q[size-1:0]                  ),
  .SC1      ( TECHIND_TIELO                ),
  .SC2      ( TECHIND_TIELO                ),
  .SE       ( TECHIND_TIELO                ), // Changing to 1'b0 to allow SOC/dGPU flows to hook-up the sync flops to the scan chain correctly
  .SI       ( {size{TECHIND_TIELO}}        ),
  .SO       ( TECHIND_UNUSED_SO[size-1:0]  ) 
);

//! unused TECHIND_UNUSED_QSRC[size-1:0]
//! unused TECHIND_UNUSED_SO[size-1:0]

endmodule // AVFS_SYNC_NOCG_AR 

module gfx_AVFS_SYNC_NOCG #(parameter
  size = 1,
  TC=0, 
  TD=0,
  IC=1, 
  rv = {size{1'b0}}                   //data-based reset isn't ideal, but X from SSE_CDCBUF causes synchronizer to X when using asyncFlush mode
)(
  input SampleClk,
  input [size-1:0] DataIn,
  input Reset,
  input SSE,
  output wire [size-1:0] DataOut
);

wire             TECHIND_TIELO       ;
wire             TECHIND_TIEHI       ;
wire [size-1:0]  TECHIND_UNUSED_QSRC ;
wire [size-1:0]  TECHIND_UNUSED_SO   ;

assign  TECHIND_TIELO  =  1'b0;
assign  TECHIND_TIEHI  =  1'b1;

wire [size-1:0] d;
wire [size-1:0] q;

//wire [size-1:0] D_reset;
//assign D_reset = Reset ? rv : DataIn; //data-based reset isn't ideal, but X from SSE_CDCBUF causes synchronizer to X when using asyncFlush mode (see bug 637452)

//assign d[size-1:0] = D_reset;         //data-based reset isn't ideal, but X from SSE_CDCBUF causes synchronizer to X when using asyncFlush mode (see bug 637452)
assign d[size-1:0] = DataIn;        //data-based reset isn't ideal, but X from SSE_CDCBUF causes synchronizer to X when using asyncFlush mode (see bug 637452)
assign DataOut = q[size-1:0];

techind_sync #(
  .size          (1),  // default 1
  .depth         (4),  // default 2
  .sourceFlop    (0),  // default 0
  .halfStage     (0),  // default 0
  .asyncSet      (0),  // default 0
  .asyncReset    (0),  // default 0
  .asyncFlush    (0),  // default 0   // removing asyncFlush, reduce risk of metastability on reset deasserting, JTAG signals need to be stable in Reset
  .clockEnable   (0),  // default 0
  .scan_muxD     (1),  // default 0
  .scan_LSSD     (0),  // default 0
  .MultiVolt     (0),  // default 0
  .TransportDelay(TD), // default 0
  .TransportCycle(TC), // default 0
  .InputCheck    (IC), // default 1
  .MetaWindowMode(0)   // default 0
)
  techind_sync__scan[size-1:0] (
  .SB       ( TECHIND_TIEHI                ),
  .RB       ( TECHIND_TIEHI                ),
  .FLUSH    ( TECHIND_TIELO                ), // Removing Reset as asyncFlush
  .CKsrc    ( TECHIND_TIELO                ),
  .CKdst    ( SampleClk                    ),
  .EN       ( TECHIND_TIEHI                ),
  .Dsrc     ( d[size-1:0]                  ),
  .Qsrc     ( TECHIND_UNUSED_QSRC[size-1:0]),
  .Qdst     ( q[size-1:0]                  ),
  .SC1      ( TECHIND_TIELO                ),
  .SC2      ( TECHIND_TIELO                ),
  .SE       ( TECHIND_TIELO                ), // Changing to 1'b0 to allow SOC/dGPU flows to hook-up the sync flops to the scan chain correctly
  .SI       ( {size{TECHIND_TIELO}}        ),
  .SO       ( TECHIND_UNUSED_SO[size-1:0]  ) 
);

//! unused TECHIND_UNUSED_QSRC[size-1:0]
//! unused TECHIND_UNUSED_SO[size-1:0]

endmodule // AVFS_SYNC_NOCG


module gfx_AVFS_SYNC_NOSCAN_NOCG #(parameter
  size = 1,
  TC=0, 
  TD=0,
  IC=1, 
  rv = {size{1'b0}}                   //data-based reset isn't ideal, but X from SSE_CDCBUF causes synchronizer to X when using asyncFlush mode
)(
  input SampleClk,
  input [size-1:0] DataIn,
  input Reset,
  output wire [size-1:0] DataOut
);

wire [size-1:0] d;
wire [size-1:0] q;

wire [size-1:0] D_reset;
assign D_reset = Reset ? rv : DataIn; //data-based reset isn't ideal, but X from SSE_CDCBUF causes synchronizer to X when using asyncFlush mode

assign d[size-1:0] = D_reset;         //data-based reset isn't ideal, but X from SSE_CDCBUF causes synchronizer to X when using asyncFlush mode
//assign d[size-1:0] = DataIn;        //data-based reset isn't ideal, but X from SSE_CDCBUF causes synchronizer to X when using asyncFlush mode
assign DataOut = q[size-1:0];

wire             TECHIND_TIELO       ;
wire             TECHIND_TIEHI       ;
wire [size-1:0]  TECHIND_UNUSED_QSRC ;
wire [size-1:0]  TECHIND_UNUSED_SO   ;

assign  TECHIND_TIELO  =  1'b0;
assign  TECHIND_TIEHI  =  1'b1;

techind_sync #(
  .size          (1),  // default 1
  .depth         (4),  // default 2
  .sourceFlop    (0),  // default 0
  .halfStage     (0),  // default 0
  .asyncSet      (0),  // default 0
  .asyncReset    (0),  // default 0
  .asyncFlush    (0),  // default 0   //data-based reset isn't ideal, but X from SSE_CDCBUF causes synchronizer to X when using asyncFlush mode
//.asyncFlush    (1),  // default 0   //data-based reset isn't ideal, but X from SSE_CDCBUF causes synchronizer to X when using asyncFlush mode
  .clockEnable   (0),  // default 0
  .scan_muxD     (1),  // default 0
  .scan_LSSD     (0),  // default 0
  .MultiVolt     (0),  // default 0
  .TransportDelay(TD), // default 0
  .TransportCycle(TC), // default 0
  .InputCheck    (IC), // default 1
  .MetaWindowMode(0)   // default 0
)
  techind_sync__noscan[size-1:0] (
  .SB       ( TECHIND_TIEHI                ),
  .RB       ( TECHIND_TIEHI                ),
  .FLUSH    ( TECHIND_TIELO                ), //data-based reset isn't ideal, but X from SSE_CDCBUF causes synchronizer to X when using asyncFlush mode
//.FLUSH    ( Reset                        ), //data-based reset isn't ideal, but X from SSE_CDCBUF causes synchronizer to X when using asyncFlush mode
  .CKsrc    ( TECHIND_TIELO                ),
  .CKdst    ( SampleClk                          ),
  .EN       ( TECHIND_TIEHI                ),
  .Dsrc     ( d[size-1:0]                  ),
  .Qsrc     ( TECHIND_UNUSED_QSRC[size-1:0]),
  .Qdst     ( q[size-1:0]                  ),
  .SC1      ( TECHIND_TIELO                ),
  .SC2      ( TECHIND_TIELO                ),
  .SE       ( TECHIND_TIELO                ),
  .SI       ( {size{TECHIND_TIELO}}        ),
  .SO       ( TECHIND_UNUSED_SO[size-1:0]  ) 
);

//! unused TECHIND_UNUSED_QSRC[size-1:0]
//! unused TECHIND_UNUSED_SO[size-1:0]

endmodule // AVFS_SYNC_NOSCAN_NOCG


module gfx_AVFS_SYNC_NOSCAN #(parameter
  size = 1,
  TC=0, 
  TD=0,
  IC=1, 
  rv = {size{1'b0}}                   //data-based reset isn't ideal, but X from SSE_CDCBUF causes synchronizer to X when using asyncFlush mode
)(
  input SampleClk,
  input SampleClkEn,
  input Reset,
  input [size-1:0] DataIn,
  output wire [size-1:0] DataOut
);

wire gatedClk;

  gfx_cg I_cg (.gclk(gatedClk), .clk(SampleClk), .en(SampleClkEn), .SSE(1'b0));
wire [size-1:0] D_reset;
assign D_reset = Reset ? rv : DataIn; //data-based reset isn't ideal, but X from SSE_CDCBUF causes synchronizer to X when using asyncFlush mode

wire [size-1:0] d;
wire clk;
wire [size-1:0] q;

assign d[size-1:0] = D_reset;         //data-based reset isn't ideal, but X from SSE_CDCBUF causes synchronizer to X when using asyncFlush mode
//assign d[size-1:0] = DataIn;        //data-based reset isn't ideal, but X from SSE_CDCBUF causes synchronizer to X when using asyncFlush mode
assign clk = gatedClk;
assign DataOut = q[size-1:0];

wire             TECHIND_TIELO       ;
wire             TECHIND_TIEHI       ;
wire [size-1:0]  TECHIND_UNUSED_QSRC ;
wire [size-1:0]  TECHIND_UNUSED_SO   ;

assign  TECHIND_TIELO  =  1'b0;
assign  TECHIND_TIEHI  =  1'b1;

techind_sync #(
  .size          (1),  // default 1
  .depth         (4),  // default 2
  .sourceFlop    (0),  // default 0
  .halfStage     (0),  // default 0
  .asyncSet      (0),  // default 0
  .asyncReset    (0),  // default 0
  .asyncFlush    (0),  // default 0   //Can remove these now - data-based reset isn't ideal, but X from SSE_CDCBUF causes synchronizer to X when using asyncFlush mode
//.asyncFlush    (1),  // default 0   //data-based reset isn't ideal, but X from SSE_CDCBUF causes synchronizer to X when using asyncFlush mode
  .clockEnable   (0),  // default 0
  .scan_muxD     (1),  // default 0
  .scan_LSSD     (0),  // default 0
  .MultiVolt     (0),  // default 0
  .TransportDelay(TD), // default 0
  .TransportCycle(TC), // default 0
  .InputCheck    (IC), // default 1
  .MetaWindowMode(0)   // default 0
)
  techind_sync__noscan[size-1:0] (
  .SB       ( TECHIND_TIEHI                ),
  .RB       ( TECHIND_TIEHI                ),
  .FLUSH    ( TECHIND_TIELO                ), //data-based reset isn't ideal, but X from SSE_CDCBUF causes synchronizer to X when using asyncFlush mode
//.FLUSH    ( Reset                        ), //data-based reset isn't ideal, but X from SSE_CDCBUF causes synchronizer to X when using asyncFlush mode
  .CKsrc    ( TECHIND_TIELO                ),
  .CKdst    ( clk                          ),
  .EN       ( TECHIND_TIEHI                ),
  .Dsrc     ( d[size-1:0]                  ),
  .Qsrc     ( TECHIND_UNUSED_QSRC[size-1:0]),
  .Qdst     ( q[size-1:0]                  ),
  .SC1      ( TECHIND_TIELO                ),
  .SC2      ( TECHIND_TIELO                ),
  .SE       ( TECHIND_TIELO                ),
  .SI       ( {size{TECHIND_TIELO}}        ),
  .SO       ( TECHIND_UNUSED_SO[size-1:0]  ) 
);

//! unused TECHIND_UNUSED_QSRC[size-1:0]
//! unused TECHIND_UNUSED_SO[size-1:0]

endmodule // AVFS_SYNC_NOSCAN


// CORE_PSM_IP_JTAGFLOP 
// ====================
module gfx_CORE_PSM_IP_JTAGFLOP #(parameter
  size = 1
)(
  input ASYNCCLK,
  input [size-1:0] ASYNCVAL,
  input CLK,
  input [size-1:0] D,
  output reg [size-1:0] Q
);

  // map to asynchronous set/reset rising edge flop
  //   use vector array instead of passing in 'WIDTH' parameter because stdcell
  //   only has 1-bit set & reset input ports, whereas ASYNCVAL is a vector
  //JTAGSRFLOP  I_srdff[size-1:0] (.Q(Q[size-1:0]), .D(D[size-1:0]), .CLK(CLK), .ASYNCCLK(ASYNCCLK), .ASYNCVAL(ASYNCVAL));


  wire  [size-1:0] set;
  wire  [size-1:0] reset;

  assign set[size-1:0] = {size{ASYNCCLK}} & ASYNCVAL[size-1:0];
  assign reset[size-1:0] = {size{ASYNCCLK}} & ~ASYNCVAL[size-1:0];

  gfx_srdff I_srdff[size-1:0] (.q(Q), .d(D), .clk(CLK), .set(set), .rst(reset));

endmodule

module gfx_CORE_PSM_IP_JTAGFLOP_B #(parameter
  size = 1
)(
  input ASYNCCLK,
  input [size-1:0] ASYNCVAL,
  input CLK,
  input [size-1:0] D,
  output [size-1:0] Q // I have modified this, it was an output reg before
);

  wire  [size-1:0] set;
  wire  [size-1:0] reset;

  assign set[size-1:0] = {size{ASYNCCLK}} & ASYNCVAL[size-1:0];
  assign reset[size-1:0] = {size{ASYNCCLK}} & ~ASYNCVAL[size-1:0];

  // Use vector array instead of passing in 'WIDTH' parameter because stdcell
  // only has 1-bit set & reset input ports, whereas ASYNCVAL is a vector

  gfx_srdff2 I_srdff2[size-1:0] (.q(Q), .d(D), .clk(CLK), .set(set), .rst(reset)); // RRR : negedge is in srdff2

endmodule // CORE_PSM_IP_JTAGFLOP_B



// CORE_SCAN_ONLY_FLOP
// ===================
module gfx_CORE_SCAN_ONLY_FLOP #(parameter
  size = 1
)(
  input C,
  input SSE,
  output reg [size-1:0] Q
);

//   wire [size-1:0] sdo;
   
// map scan-only flop to regular flop with D-input tied to 1'b0
//etl1 #(.upper(size-1),.lower(0))  I_dff (.clk(C), .d({size{1'b0}}), .q(Q[size-1:0]), .SSE(SSE), .SDI({size{1'b0}}), .SDO(sdo));

//! unused sdo[0]

  always @ ( posedge C ) begin
      Q <= {size{1'b0}};
    end

endmodule // CORE_SCAN_ONLY_FLOP

//CORE_FLOP_GATED
//===============
module gfx_CORE_FLOP_GATED #(parameter
  size = 1,
  rv = {size{1'b0}},
  dro = 0,
  et = 0
)(
  input [size-1:0] D,
  input C,
  input G,
  input GC,
  input R,
  input SSE,
  output reg [size-1:0] Q
);

  reg gate_1;
  wire clk_gated;

  ati_clock_gate d0nt_psm_clock_gate_FuncMGCG (
    .clk_src    (C),
    .clk_out    (clk_gated),
    .clk_en     (G | SSE | R ),
    .TE         (1'b0)
  );

  always @ ( posedge clk_gated ) begin
    if( R ) begin
      Q <= rv[size-1:0];
    end
    else begin
      Q <= D;
    end
  end

endmodule // CORE_FLOP_GATED

module gfx_CORE_FLOP_GATED_RESET #(parameter
  size = 1,
  rv = {size{1'b0}},
  dro = 0,
  et = 0
)(
  input [size-1:0] D,
  input C,
  input G,
  input GC,
  input R,
  input SSE,
  output reg [size-1:0] Q
);

  reg gate_1;
  wire clk_gated;

  ati_clock_gate d0nt_psm_clock_gate_FuncMGCG (
    .clk_src    (C),
    .clk_out    (clk_gated),
    .clk_en     (G | SSE | R),
    .TE         (1'b0)
  );

  always @ ( posedge clk_gated ) begin
    if( R ) begin
      Q <= rv[size-1:0];
    end
    else begin
      Q <= D;
    end
  end

endmodule // CORE_FLOP_GATED_RESET

module gfx_CORE_CDCBUF #(parameter
  CdcXRawDly = 1,
  Cycles = 1,
  Setup = 0 ,
  BufMetaWaiveMode = 0,
  size = 1
)(
  input [size-1:0] A,
  output wire [size-1:0] Z,
  input  DSTCLK
);

//   cc_cdcbufclk #(.upper(size-1),.lower(0),.Cycles(Cycles), .Setup(0))
//     I_cc_cdcbufclk (.A(A[size-1:0]), .Z(Z[size-1:0]), .DSTCLK(DSTCLK));

techind_cdcfpm #(.size(size), .applyParams(1), .DestClkCnt(Cycles), .CdcDestSetup(Setup), .BufMetaWaiveMode(BufMetaWaiveMode))
  I_techind_cdcfpm (.A(A[size-1:0]), .Z(Z[size-1:0]), .DSTCLK(DSTCLK));

endmodule // CORE_CDCBUF

module gfx_CORE_CDCENBUF #(parameter
  CdcXRawDly = 1,
  Cycles = 1,
  Setup = 0,
  size = 1
)(
  input       [size-1:0] A,
  input                  EN,
  output wire [size-1:0] Z,
  input  DSTCLK
);

 techind_cdcefpm #(.size(size), .applyParams(1), .DestClkCnt(Cycles), .CdcDestSetup(Setup))
   I_techind_cdcefpm (.A(A[size-1:0]), .Z(Z[size-1:0]), .EN(EN), .DSTCLK(DSTCLK));

endmodule // CORE_CDCENBUF

module gfx_CORE_MCBUF #(parameter
  Cycles = 1,
  size = 1
)(
  input [size-1:0] A,
  output wire [size-1:0] Z,
  input  DSTCLK
);

  //cc_mcbuf #(.upper(size-1),.lower(0),.Cycles(Cycles), .Setup(0))
  //I_cc_mcbuf (.A(A[size-1:0]), .Z(Z[size-1:0]), .DSTCLK(DSTCLK));
//  cc_cdcbufclk #(.upper(size-1),.lower(0),.Cycles(Cycles), .Setup(0))
//  I_cc_cdcbufclk (.A(A[size-1:0]), .Z(Z[size-1:0]), .DSTCLK(DSTCLK));

techind_cdcfpm #(.size(size), .applyParams(1), .DestClkCnt(Cycles), .CdcDestSetup(0))
  I_techind_cdcfpm (.A(A[size-1:0]), .Z(Z[size-1:0]), .DSTCLK(DSTCLK));

endmodule // CORE_MCBUF


module gfx_CORE_MCENBUF #(parameter
  Cycles = 1,
  size = 1
)(
  input       [size-1:0] A,
  input                  EN,
  output wire [size-1:0] Z,
  input  DSTCLK
);

 techind_emcpm #(.size(size), .applyParams(1), .DestClkCnt(Cycles), .CdcDestSetup(0))
  I_techind_emcpm (.A(A[size-1:0]), .EN(EN), .Z(Z[size-1:0]), .DSTCLK(DSTCLK));

endmodule // CORE_MCENBUF

module gfx_CORE_ENC3(
input       [7:0] A,
output wire [2:0] Z);

//! unused A[0]
assign Z[2] = |A[7:4];
assign Z[1] = |A[3:2]
            | (|A[7:6]);
assign Z[0] = A[7]
            | A[5]
            | A[3]
            | A[1];

endmodule // ENC3
     
module gfx_CORE_ENC4(
input      [15:0] A,
output wire [3:0] Z);

//! unused A[0]
assign Z[3] =  |A[15:8];
assign Z[2] = (|A[15:12])
            | (|A[7:4]);
assign Z[1] = (|A[15:14])
            | (|A[11:10])
            | (|A[7:6])
            | (|A[3:2]);
assign Z[0] = A[15]
            | A[13]
            | A[11]
            | A[9]
            | A[7]
            | A[5]
            | A[3]
            | A[1];

endmodule // ENC4

module gfx_CORE_PSM_IP_OSC #(parameter
  HIGH_VOLT = 0,
  JTAG_CLK_DIV_W = 2,
  MCP_CYCLES_MAPSMRO = 0,
  CURPSM_W = 14
) (
  input                      clk,
  input [JTAG_CLK_DIV_W-1:0] div,
  input                      pwrOk,
  input                      oscEn,
  output wire [31:0]         psmOscLsb,
  output wire [7:0]          psmOscMsb0,
  output wire [7:0]          psmOscMsb1,
  output wire                psmOscGater
);

// KK/CM PD flow does not want explicit VDD/VSS port connections to mapsmro macro
//   KK PD flow sets CC_SYNTHESIS_KK in order to leave VDD/VSS ports unconnected 
//   CM PD flow does not set CC_SYNTHESIS_KK -> will manually remove VDD/VSS port connections
//   CM RTL compile uses RPL and needs VDD/VSS ports to be connected
//   KK RTL compile does not use RPL and VDD/VSS ports can be left unconnected, but will also
//     see VDD/VSS connections.
//  mapsmro I_mapsm (
//`ifdef      CC_SYNTHESIS_KK
//`else
//    .VDD          (1'b1),   
//    .VSS          (1'b0),   
//`endif
//    .CLK1         (clk),
//    .CLK2         (clk),
//    .oscEn1       (oscEn),
//    .oscEn2       (oscEn),
//    .div          (div[1:0]),
//    .invalid      (),
//    .overFlow     (),
//    .psmOscLsb   (psmOscLsb[31:0]),
//    .psmOscMsb0  (psmOscMsb0[7:0]),
//    .psmOscMsb1  (psmOscMsb1[7:0]),
//    .psmOscGater (psmOscGater)
//  );

generate
 if (HIGH_VOLT == 1) begin : instantiate_mapsmro_pg
    mapsmro_pg I_psm (
       .VDD1        ( ),
       .VDD2        ( ),
       .VSS         ( ),
       .CLK1        (clk),
       .CLK2        (clk),
       .pwrOK       (pwrOk),
       .oscEn1      (oscEn),
       .oscEn2      (oscEn),
       .div         (div[1:0]),
       .invalid      (),
       .overFlow     (),
       .psmOscGater (psmOscGater),
       .psmOscLsb   (psmOscLsb[31:0]),
       .psmOscMsb0  (psmOscMsb0[7:0]),
       .psmOscMsb1  (psmOscMsb1[7:0])
     );
     end
  else begin : instantiate_mapsmro
    mapsmro I_psm (
       .VDD1        ( ),
       .VDD2        ( ),
       .VSS         ( ),
       .CLK1        (clk),
       .CLK2        (clk),
       .pwrOK       (pwrOk),
       .oscEn1      (oscEn),
       .oscEn2      (oscEn),
       .div         (div[1:0]),
       .invalid      (),
       .overFlow     (),
       .psmOscGater (psmOscGater),
       .psmOscLsb   (psmOscLsb[31:0]),
       .psmOscMsb0  (psmOscMsb0[7:0]),
       .psmOscMsb1  (psmOscMsb1[7:0])
     );
     end
endgenerate

endmodule // CORE_PSM_IP_OSC


// Behavioral model of the delay cell
// this should be replaced by the actual library gate
// 
module gfx_avfsPdly ( A, CBYPASS, CSEL0, CSEL1, CSEL2, Z, avfsTestData, muxSel );

  input  A ;
  input  CBYPASS ;
  input  CSEL0 ;
  input  CSEL1 ;
  input  CSEL2 ;
  input  avfsTestData ;
  input  muxSel ;
  output Z ;

   wire  net1;
   wire  net2;
   wire  net3;
   wire  net4;
   wire  net5;
   wire  net6;

   assign net1 = ~avfsTestData & ~(A & CSEL2);
   assign net2 = net1 & ~(A & CSEL1);
   assign net3 = net2 & ~(A & CSEL0);
   assign net4 = ~(A & CBYPASS);
   assign net5 = ~(net4 & net3);
   assign net6 = ~(net4 & net3);
   assign Z = muxSel ? net5 : net6;
   
endmodule   // avfsPdly

// This version of AVFS_FLOP is based on structure of SFFGSPDT used in XV
//
// S.Kosonocky: Added a 2nd normalizer delay in series to double the total delay per setting
//              This can be used to guarantee AVFS CP paths can fail under worst case margins
//
module gfx_AVFS_FLOP #( parameter
                  size = 1,
                  rv = {size{1'b0}},
                  dro = 0,
                  et = 0
) (
  input                       C,
  input            [size-1:0] D,
  input                       G,
  input                       GC,
  input                       R,
  input                       SSE,
  input   [size*3-1:0]        NDLY,    // need a copy for each speed flop bit
  input   [2:0]               SDLY,   // used to select TST input, can be shared 
  input   [1:0]               TM_X,
  input                       TST,
  output wire   [size-1:0]    Q,
  output wire   [size-1:0]    NM
);
   
   // Wire definition
   wire [size-1:0]            Q_AVFS;
   wire [size-1:0]            FLOP_IN;

   // AVFS Flop Associated Wires
   wire [size-1:0]            FLOP_IN_AVFS;
   wire [size-1:0]            NDLY_OUT_AVFS;
   wire [size-1:0]            NDLY_OUT_AVFS_BUF;

   integer                    m;
   
//   reg [2:0]                  ndlysel[size-1:0];
   reg [size-1:0]	      ndlysel[2:0];
   reg [size-1:0]             ndlymsel;
   reg [size-1:0]             ndlydecsel[3:0];
   reg [size-1:0]             ndlydecseltest[3:0];
   reg [size-1:0]             sdlysel[2:0];
   reg [size-1:0]             sdlymsel;
   reg [size-1:0]             sdlydecsel[3:0];
   reg [size-1:0]             sdlydecseltest[3:0];

   wire [size*3-1:0] NDLY_MC;
   wire [size*3-1:0] SDLY_MC;

   // Delay controls will be setup from CPA module at least 2 cycles before use (bug 423581)
   //   add MCBUF to SDLY[2:0] and NDLY[size*3-1:0] path
   //   insert MCBUF in SDLY[2:0] path before fanout to each speed flop to reduce #MCBUFs
   //   NDLY[size*3-1:0] bits are unique so need to insert MCBUF for each speed flop
  gfx_CORE_MCBUF #(.size(size*3), .Cycles(2)) I_NDLY_MC (.A(NDLY[size*3-1:0]),
                                                      .DSTCLK(C),
                                                      .Z(NDLY_MC[size*3-1:0]));
  gfx_CORE_MCBUF #(.size(3), .Cycles(2))      I_SDLY_MC (.A(SDLY[2:0]),
                                                      .DSTCLK(C),
                                                      .Z(SDLY_MC[2:0]));
   
   // Setup delay inputs by bit
   always @* begin
      for (m=0; m<size; m=m+1) begin
         ndlysel[2][m] = NDLY_MC[m*3+2];
         ndlysel[1][m] = NDLY_MC[m*3+1];
         ndlysel[0][m] = NDLY_MC[m*3+0];
         ndlymsel[m] = ndlysel[0][m] & TM_X[1];
         ndlydecsel[0][m] = ({ndlysel[2][m],ndlysel[1][m]} == 2'b00);
         ndlydecsel[1][m] = ({ndlysel[2][m],ndlysel[1][m]} == 2'b01);
         ndlydecsel[2][m] = ({ndlysel[2][m],ndlysel[1][m]} == 2'b10);
         ndlydecsel[3][m] = ({ndlysel[2][m],ndlysel[1][m]} == 2'b11);
         ndlydecseltest[0][m] = ndlydecsel[0][m] & TM_X[0];
         ndlydecseltest[1][m] = ndlydecsel[1][m] & TM_X[1];
         ndlydecseltest[2][m] = ndlydecsel[2][m] & TM_X[0];
         ndlydecseltest[3][m] = ndlydecsel[3][m] & TM_X[1];
         sdlysel[2][m] = SDLY_MC[2];
         sdlysel[1][m] = SDLY_MC[1];
         sdlysel[0][m] = SDLY_MC[0];
         sdlymsel[m] = sdlysel[0][m] & TM_X[1];
         sdlydecsel[0][m] = ({sdlysel[2][m],sdlysel[1][m]} == 2'b00);
         sdlydecsel[1][m] = ({sdlysel[2][m],sdlysel[1][m]} == 2'b01);
         sdlydecsel[2][m] = ({sdlysel[2][m],sdlysel[1][m]} == 2'b10);
         sdlydecsel[3][m] = ({sdlysel[2][m],sdlysel[1][m]} == 2'b11);
         sdlydecseltest[0][m] = sdlydecsel[0][m] & TM_X[0];
         sdlydecseltest[1][m] = sdlydecsel[1][m] & TM_X[1];
         sdlydecseltest[2][m] = sdlydecsel[2][m] & TM_X[0];
         sdlydecseltest[3][m] = sdlydecsel[3][m] & TM_X[1];
      end
   end
   
   //vrq translate_off
   
   assign FLOP_IN[size-1:0] = D[size-1:0];

   // AVFS Flop
  gfx_CORE_FLOP_GATED #(.size(size),.rv(rv),.dro(dro),.et(et)) 
   I_flop (.C(C),
           .G(G),
           .GC(GC),
           .R(R),
           .SSE(SSE),
           .D(FLOP_IN[size-1:0]),
           .Q(Q[size-1:0])
           );
   

   // Delay Cells
`ifdef FJ_SYNTHESIS_DM
   //! disable parser everything   
   pdlych3c0b2gxss2ur avfs_ndly_d0nt[size-1:0] (
                                             .Z(NDLY_OUT_AVFS[size-1:0]),
                                             .A(D[size-1:0]),
                                             .avfsTestData(TST),
                                             .muxSel(ndlymsel[size-1:0]),
                                             .CSEL2(ndlydecseltest[3][size-1:0]),
                                             .CSEL1(ndlydecseltest[2][size-1:0]),
                                             .CSEL0(ndlydecseltest[1][size-1:0]),
                                             .CBYPASS(ndlydecseltest[0][size-1:0]));
`elsif GNL_SYNTHESIS_DM
   //! disable parser everything   
   hdpdlych3c0b2gxss2ur avfs_sdly_d0nt[size-1:0] (
                                             .Z(FLOP_IN_AVFS[size-1:0]),
                                             .A(NDLY_OUT_AVFS_BUF[size-1:0]),
                                             .avfsTestData(TST),
                                             .muxSel(sdlymsel[size-1:0]),
                                             .CSEL2(sdlydecseltest[3][size-1:0]),
                                             .CSEL1(sdlydecseltest[2][size-1:0]),
                                             .CSEL0(sdlydecseltest[1][size-1:0]),
                                             .CBYPASS(sdlydecseltest[0][size-1:0]));						  
   //! enable parser everything
`else // !`ifdef FJ_SYNTHESIS_DM
  gfx_avfsPdly  avfs_ndly[size-1:0] (
                                             .Z(NDLY_OUT_AVFS[size-1:0]),
                                             .A(D[size-1:0]),
                                             .avfsTestData(TST),
                                             .muxSel(ndlymsel[size-1:0]),
                                             .CSEL2(ndlydecseltest[3][size-1:0]),
                                             .CSEL1(ndlydecseltest[2][size-1:0]),
                                             .CSEL0(ndlydecseltest[1][size-1:0]),
                                             .CBYPASS(ndlydecseltest[0][size-1:0]));
`endif // !`ifdef FJ_SYNTHESIS_DM
   
   // The an explicit buffer here
   assign NDLY_OUT_AVFS_BUF[size-1:0] = NDLY_OUT_AVFS[size-1:0];

   
`ifdef FJ_SYNTHESIS_DM
   //! disable parser everything   
   pdlych3c0b2gxss2ur avfs_sdly_d0nt[size-1:0] (
                                             .Z(FLOP_IN_AVFS[size-1:0]),
                                             .A(NDLY_OUT_AVFS_BUF[size-1:0]),
                                             .avfsTestData(TST),
                                             .muxSel(sdlymsel[size-1:0]),
                                             .CSEL2(sdlydecseltest[3][size-1:0]),
                                             .CSEL1(sdlydecseltest[2][size-1:0]),
                                             .CSEL0(sdlydecseltest[1][size-1:0]),
                                             .CBYPASS(sdlydecseltest[0][size-1:0]));
`elsif GNL_SYNTHESIS_DM
   //! disable parser everything   
   hdpdlych3c0b2gxss2ur avfs_sdly_d0nt[size-1:0] (
                                             .Z(FLOP_IN_AVFS[size-1:0]),
                                             .A(NDLY_OUT_AVFS_BUF[size-1:0]),
                                             .avfsTestData(TST),
                                             .muxSel(sdlymsel[size-1:0]),
                                             .CSEL2(sdlydecseltest[3][size-1:0]),
                                             .CSEL1(sdlydecseltest[2][size-1:0]),
                                             .CSEL0(sdlydecseltest[1][size-1:0]),
                                             .CBYPASS(sdlydecseltest[0][size-1:0]));					     
   //! enable parser everything
`else // !`ifdef FJ_SYNTHESIS_DM
  gfx_avfsPdly  avfs_sdly[size-1:0] (
                                             .Z(FLOP_IN_AVFS[size-1:0]),
                                             .A(NDLY_OUT_AVFS_BUF[size-1:0]),
                                             .avfsTestData(TST),
                                             .muxSel(sdlymsel[size-1:0]),
                                             .CSEL2(sdlydecseltest[3][size-1:0]),
                                             .CSEL1(sdlydecseltest[2][size-1:0]),
                                             .CSEL0(sdlydecseltest[1][size-1:0]),
                                             .CBYPASS(sdlydecseltest[0][size-1:0]));
`endif // !`ifdef FJ_SYNTHESIS_DM
   
   // Flop
  gfx_CORE_FLOP_GATED #(.size(size),.rv(rv),.dro(dro),.et(et)) 
   I_avfs (.C(C),
           .G(G),
           .GC(GC),
           .R(R),
           .SSE(SSE),
           .D(FLOP_IN_AVFS[size-1:0]),
           .Q(Q_AVFS[size-1:0])
           );
   
   // AVFS Post Process
`ifdef FJ_SYNTHESIS_DM
   wire [size-1:0] NM_pre;
   //! disable parser everything   
   xr2xss2ur I_Q_AVFS_d0nt[size-1:0] (.A(Q_AVFS[size-1:0]), .B(Q[size-1:0]), .Z(NM_pre[size-1:0]));
   //! enable parser everything
   assign 	   NM[size-1:0] = NM_pre[size-1:0];
`elsif GNL_SYNTHESIS_DM
   wire [size-1:0] NM_pre;
   //! disable parser everything   
   hdxr2xss2ur I_Q_AVFS_d0nt[size-1:0] (.A(Q_AVFS[size-1:0]), .B(Q[size-1:0]), .Z(NM_pre[size-1:0]));
   //! enable parser everything
   assign 	   NM[size-1:0] = NM_pre[size-1:0];
`else
   assign 	   NM[size-1:0] = Q_AVFS[size-1:0] ^ Q[size-1:0];
`endif

   
   //vrq translate_on
   
endmodule // AVFS_FLOP 



module gfx_cg (
  input clk,
  input en,
  input SSE,
  output gclk
);

  ati_clock_gate d0nt_avfs_clock_gate_FuncMGCG (
    .clk_src    (clk),
    .clk_out    (gclk),
    .clk_en     (en | SSE),
    .TE         (1'b0)
  );

endmodule // cg

module gfx_dff (
  clk,
  SSE,
  d,
  q
);

  parameter   WIDTH = 1;

  input  clk;
  input  SSE;
  input  [WIDTH-1:0] d;
  output [WIDTH-1:0] q;
  reg    [WIDTH-1:0] q;

  always @ ( posedge clk ) begin
    if (SSE) begin
      q <= {WIDTH{1'b0}};
    end 
    else begin
      q <= d;
    end
  end

endmodule // dff

//==================================================================================================
//! enable one_module_per_file
// leda E_001 on
//==================================================================================================


module gfx_ccg
 #(parameter WIDTH=1,                 // default WIDTH of output
             SSE_RESET=1'b1,  // enable scan-toggle reset when SSE is inserted
             RVAL = {WIDTH {1'b0}}  // reset state for the embedded enable flop
   ) 
  (
   output     [WIDTH-1:0]   gclk,  // gated/conditional clock output
   input                    clk,   // coarse gater clock (CCLK)
   input                    fclk,  // flop clock (from a fine gater)
   input      [WIDTH-1:0]   en,    // active high gater enable, valid previous cycle
   input                    TstCcgEn, // active low. Driven by DFT during test modes, asynchronous with respect to CCLK
   input                    SSE    // scan shift enable 
   );

reg  [WIDTH-1:0] ccg_q;
wire [WIDTH-1:0] ccgtestenX_q;
wire [WIDTH-1:0] iq;     // phi2 latch
wire [WIDTH-1:0] gclk_X;
wire [WIDTH-1:0] en_d1;

   
always @(posedge fclk)
    ccg_q <= (en ^ RVAL) & ~{WIDTH{SSE}};

assign en_d1 = ccg_q ^ RVAL;

 ati_clock_gate d0nt_avfs_clock_gate_FuncCGCG (
    .clk_src    (clk),
    .clk_out    (gclk),
    .clk_en     (en_d1 | SSE), // SSE is tied to ~resetn at SOC level
    .TE         (TstCcgEn)
  );

endmodule

module gfx_ccg_nf
 #(parameter WIDTH=1,              // default WIDTH of output
             AN=0                  // use a ckan2 stdcell in the tech view instead of a cknd2+ckinv
   ) 
  (
   output     [WIDTH-1:0]   gclk,  // gated/conditional clock output
   input                    clk,   // coarse gater clock (CCLK)
   input      [WIDTH-1:0]   en,    // active high gater enable, valid previous cycle
   input                    TstCcgEn, // active low. Driven by DFT during test modes, asynchronous with respect to CCLK
   input                    SSE    // scan shift enable 
   );


// Coarse clock gater with test enable and without the flop on enable

   ati_clock_gate clkgater_internal_FuncCGCG (
                .clk_src(clk),
                .clk_en(en | SSE), // SSE is tied to ~resetn at SOC level. 
                .TE(TstCcgEn),
                .clk_out(gclk)
            );

endmodule


module gfx_srdff
 #(parameter size=1               // default WIDTH of output
   ) 
  (
   output   reg  [size-1:0]   q,   // data out
   input                    clk,   // clock
   input         [size-1:0]   d,   // data in
   input                    set,   // async set
   input                    rst    // async reset or clear
   );

// Turn off rule that caused this failure:
// NTL_STR56: Do not use sequential registers with both asynchronous set and asynchronous reset
// leda NTL_STR56 off

  always @ (posedge clk or posedge set or posedge rst) begin
    if( rst ) begin
      q <= {size{1'b0}};
    end
    else if (set) begin
      q <= {size{1'b1}};
    end
    else begin
      q <= d;
    end
  end

// leda NTL_STR56 on
  
endmodule

// adding srdff2 from cores_rtl_lib/leaf/behavioral for B-phase JTAG flop
module gfx_srdff2
 #(parameter WIDTH=1               // default WIDTH of output
   ) 
  (
   output     [WIDTH-1:0]   q,     // data out
   input                    clk,   // clock
   input      [WIDTH-1:0]   d,     // data in
   input                    set,   // async set
   input                    rst    // async reset or clear
   );

reg [WIDTH-1:0] srdff2_q;

// Turn off rule that caused this failure:
// NTL_STR56: Do not use sequential registers with both asynchronous set and asynchronous reset
// leda NTL_STR56 off

always @(negedge clk or posedge set or posedge rst)
    if (set)
        srdff2_q <= {WIDTH{1'b1}};
    else if (rst)
        srdff2_q <= {WIDTH{1'b0}};
    else
        srdff2_q <= d;

assign q = srdff2_q;

// leda NTL_STR56 on

endmodule

module gfx_dff_noscan
 #(parameter WIDTH=1               // default WIDTH of output
   ) 
  (
   output     reg [WIDTH-1:0]   q,     // data out
   input                    clk,   // clock
   input      [WIDTH-1:0]   d      // data in
   );


 always @ ( posedge clk ) begin
      q <= d;
    end


endmodule

module gfx_cgu #(parameter SSR=0, WIDTH=1)
  (
   output     [WIDTH-1:0]   gclk,  // unconditional clock output
   input                    clk    // fine gater input clock
   );

// `if ("" ne "")
// `else
    ati_clock_gate clkgater_internal_FuncMGCG (
                .clk_src(clk),
                .clk_en(1'b1),
                .TE(1'b0),
                .clk_out(gclk)
            );
// `endif



endmodule

module gfx_ccgu #(parameter SSR=0, WIDTH=1)
  (
   output     [WIDTH-1:0]   gclk,    // unconditional clock output
   input                    clk,     // coarse gater clock 
   input                    TstCcgEn // High in normal operation, Driven low by DFT during test modes 
   );

// `if ("" ne "")
// `else
    ati_clock_gate clkgater_internal_FuncCGCG (
                .clk_src(clk),
                .clk_en(1'b1),
                .TE(TstCcgEn),
                .clk_out(gclk)
            );
// `endif



endmodule

/*
module gfx_lsbuf
 #(parameter WIDTH        = 1,                // WIDTH of input/output
             LSVAL        = {WIDTH{1'b0}},    // Value to drive when PWROK=0
             PWROKCHECK   = 1,                // Check that A input is equal to LSVAL when pwrok rises
             BUFFER       = 1,                // BUFFER=1 is the slower level shifter. Lower input cap, ISO=low, high
                                              // BUFFER=0 is the faster level shifter. Higher input cap, ISO=low
             NOPWROK      = 0,                // Do not use PwrOk input or LSVAL parameter
                                              //   Uses level shifter cell without pwrok
             SNIFFER      = 0,                // Use sniffer level shifter 
                                              //   'a' input usually PwrOk signal, pwrok input to module is unused
                                              //   LSVAL is unused, physical cell outputs 0 if VDDSRC is low
             SRC          = 0                 // Use src version, otherwise use dst
   )
  (
   output logic    [WIDTH-1:0]   z,     // buffer output (to the destination clock domain)
   input           [WIDTH-1:0]   a,     // buffer input  (from the source clock domain)
   input                         pwrok  // buffer enable
   );

genvar i;

// river translate_off
generate 
for (i=0; i<WIDTH; i=i+1)
    if (SRC) begin
	if (NOPWROK) begin
         
	    hdlsbf2g600_1250srceqxss4bl lsbuf (.Z(z[i]), .A(a[i]));
	end else if (SNIFFER) begin
            // There is no source flavor sniffer cell in GF14LPE and ZAK should not use this option.
            // In gf14lpe, the sniffer level shifter cell is instantiated with the pslsbuf module
	end else begin
            if (LSVAL[i]) begin
               if(BUFFER)
		   hdlsclamphin2g600_1250srceqxss4bl lsbuf (.Z(z[i]), .A(a[i]), .ISON(pwrok));
            end else begin
               if(BUFFER)
        	   hdlsclamplon2g600_1250srceqxss4bl lsbuf (.Z(z[i]), .A(a[i]), .ISON(pwrok));
              
        	   //there is no faster level shifter in current library
            end
	end
    end
    else begin
	if (NOPWROK) begin        
	    hdlsbf2g600_1250dsteqxss4bl lsbuf (.Z(z[i]), .A(a[i]));
	end else if (SNIFFER) begin
            // Level shifter with isolation control driven by internal sniffer
            // In gf14lpe, the sniffer level shifter cell is instantiated with the pslsbuf module
            // hppsls15g600_1250dstne0xss1br lsbuf (.Z(z[i]), .A(a[i]), .SANE(pwrok));
	end else begin
            if (LSVAL[i]) begin
               if(BUFFER)
        	   hdlsclamphin2g600_1250dsteqxss4bl lsbuf (.Z(z[i]), .A(a[i]), .ISON(pwrok));
            end else begin
               if(BUFFER)
        	   hdlsclamplon2g600_1250dsteqxss4bl lsbuf (.Z(z[i]), .A(a[i]), .ISON(pwrok));
                    	  
		   //there is no faster level shifter in current library
            end
	end
    end
endgenerate
// river translate_on
endmodule
*/

// Adding B-Latch module - used in AVFS3_RippleCnt to latch CPO clock gating enable
module gfx_lat2
 #(parameter WIDTH=1             // default WIDTH of output
   ) 
  (
   output     [WIDTH-1:0] q,     // data out
   input                  clk,   // clock
   input      [WIDTH-1:0] d      // data in
   );

reg [WIDTH-1:0] lat2_q;

// Turn off the rule that checks for latches
// leda NTL_STR47 off
// leda W410 off
// leda XPROP_IF off
always @*
    if (~clk)
        lat2_q <= d;
// leda NTL_STR47 on
// leda W410 on
// leda XPROP_IF on

assign q = lat2_q;

endmodule


module gfx_AVFS_clkbuf(output Y, input A);
`ifdef SYNTHESIS
  ati_clkbuf1 clkbuf (.A(A),.Y(Y));
`else
  assign Y=A;
`endif
endmodule // AVFS_clkbuf

module gfx_AVFS_mux2(output Z, input S, input D1, input D0);
`ifdef SYNTHESIS
  ati_mux2 mux2 (.z(Z), .s(S), .i1(D1), .i0(D0));
`else
  assign Z = S ? D1 : D0;
`endif
endmodule // AVFS_mux2


