//////////////////////////////////////////////////////////////////////////////////////////////////////////
//Description: Local clock gating module for the Shader complex. It is a simple clock gater with iclk as one
//             input and four clock enables. Four clock enables are: (1) OVERRIDE (2) ENABLE GATE (3) BUSY
//             (4) SPARE. If a design requires more than 4 inputs then we can parameterize this module
//////////////////////////////////////////////////////////////////////////////////////////////////////////

// following defines are added to instance the clock buffers and mark them for scan grouping

`define SCAN_GRP0 0
`define SCAN_GRP1 1
`define SCAN_GRP2 2
`define SCAN_GRP3 3
`define SCAN_GRP4 4
`define SCAN_GRP5 5
`define SCAN_GRP6 6
`define SCAN_GRP7 7

`include "gc_features.vh"

module gfx_cgts_local(/*AUTOARG*/
   // Outputs
   dyn_clk, dyn_clk_vld,
   // Inputs
   Cpl_GFXCLK, dft_clk_gate_ctrl, dft_shift_clk,
   iclk, static_iclk, cgcg_en, override, en1, en2, en3
   );

//////////////////////////
//in/outs
//////////////////////////
 input       iclk;
 input       static_iclk;
 output      dyn_clk;
 output      dyn_clk_vld;

 input       cgcg_en;
 input       override;
 input       en1;
 input       en2;
 input       en3;
 input       Cpl_GFXCLK;
 input [1:0] dft_clk_gate_ctrl;
 input       dft_shift_clk;

//////////////////////////
//
//////////////////////////
parameter switch_code = 0;
parameter enable_dftcgmg_gate_merge = 0;
parameter first_dynclk_edge_dirty = 0;

reg  [1:0]  n0scan_dft_clk_gate_ctrl_d1;
wire [1:0]  wdft_clk_gate_ctrl;

wire sclk;
reg         n0scan_cgcg_en_d1;
generate
  if(enable_dftcgmg_gate_merge == 1) begin
    always @(posedge Cpl_GFXCLK) begin
      n0scan_dft_clk_gate_ctrl_d1 <= dft_clk_gate_ctrl[1:0];
    end
    assign sclk = Cpl_GFXCLK;
    assign wdft_clk_gate_ctrl = n0scan_dft_clk_gate_ctrl_d1;
  end
  else begin
    assign sclk = iclk;
    assign wdft_clk_gate_ctrl = 2'b00;
  end
endgenerate

always @(posedge sclk) begin
  n0scan_cgcg_en_d1 <= cgcg_en;
end

reg override_s;

wire core_busy ;

assign core_busy = en1 | en2 | en3 ;

always @(posedge static_iclk) begin
  override_s <= override;
end

reg  n0scan_active_d1;
reg  active_d2, active_d3;
reg  dyn_clk_vld_pre;
wire dyn_clk_vld;

`ifdef GC__CGCG_MGCG_MERGE__1
wire active = n0scan_cgcg_en_d1 && (override_s | core_busy);
generate
  if (first_dynclk_edge_dirty == 1) begin
    always @(posedge static_iclk) begin
      active_d2 <= n0scan_active_d1;
      active_d3 <= active_d2;
    end
    assign dyn_clk_vld = active_d2 & active_d3;
  end
  else begin
   always @(posedge static_iclk) begin
     dyn_clk_vld_pre <= n0scan_active_d1;
   end
   assign dyn_clk_vld = dyn_clk_vld_pre;
  end
endgenerate
`else
wire active = override_s | core_busy;
`endif

generate
  if(enable_dftcgmg_gate_merge == 1)
always @(posedge sclk) begin
  n0scan_active_d1 <= (active & ~wdft_clk_gate_ctrl[1]) | wdft_clk_gate_ctrl[0];
end
  else
always @(posedge sclk) begin
  n0scan_active_d1 <= active;
end
endgenerate

wire dyn_clk_int;
wire dyn_clk_gate;

`ifdef SIMULATION_WITH_x
/* -----\/----- EXCLUDED -----\/-----
reg  dyn_clk_gatex;
always @(posedge iclk)
begin
   if (active)
      dyn_clk_gatex <= 1'b1;
   else
      dyn_clk_gatex <= 1'b0;
end
assign dyn_clk_int = dyn_clk_gatex ? dyn_clk_gate : 1'bx;
 -----/\----- EXCLUDED -----/\----- */
`ifdef GC__PIPE_MGCG_GATER__1
assign dyn_clk_int = n0scan_active_d1 ? dyn_clk_gate : 1'bx;
`else
assign dyn_clk_int = active ? dyn_clk_gate : 1'bx;
`endif
`else
assign dyn_clk_int = dyn_clk_gate;
`endif

  wire dft_te_wire;
ati_TE_driver u_TE_driver
  (
   .TEST_CLK_EN(dft_shift_clk),
   .DFT_TE(dft_te_wire)
  );

ati_clock_gate u_dyn_clk(
    .clk_out(dyn_clk_gate),
    .clk_src(sclk),
    .clk_en(n0scan_active_d1),
    .TE(dft_te_wire)
    );

// following code is added to instance the clock buffers and mark them for scan grouping

wire [`SCAN_GRP7:0] clks;

parameter scan_grp_dyn_clk = `SCAN_GRP0; //defaulted

generate
  if(scan_grp_dyn_clk == `SCAN_GRP0)
    begin
      aticlk_buf scan_grp0_clk(.Y(clks[`SCAN_GRP0]), .A(dyn_clk_int));
    end
  else if (scan_grp_dyn_clk == `SCAN_GRP1)
    begin
      aticlk_buf scan_grp1_clk(.Y(clks[`SCAN_GRP1]), .A(dyn_clk_int));
    end
  else if (scan_grp_dyn_clk == `SCAN_GRP2)
    begin
      aticlk_buf scan_grp2_clk(.Y(clks[`SCAN_GRP2]), .A(dyn_clk_int));
    end
  else if (scan_grp_dyn_clk == `SCAN_GRP3)
    begin
      aticlk_buf scan_grp3_clk(.Y(clks[`SCAN_GRP3]), .A(dyn_clk_int));
    end
  else if (scan_grp_dyn_clk == `SCAN_GRP4)
    begin
      aticlk_buf scan_grp4_clk(.Y(clks[`SCAN_GRP4]), .A(dyn_clk_int));
    end
  else if (scan_grp_dyn_clk == `SCAN_GRP5)
    begin
      aticlk_buf scan_grp5_clk(.Y(clks[`SCAN_GRP5]), .A(dyn_clk_int));
    end
  else if (scan_grp_dyn_clk == `SCAN_GRP6)
    begin
      aticlk_buf scan_grp6_clk(.Y(clks[`SCAN_GRP6]), .A(dyn_clk_int));
    end
  else if (scan_grp_dyn_clk == `SCAN_GRP7)
    begin
      aticlk_buf scan_grp7_clk(.Y(clks[`SCAN_GRP7]), .A(dyn_clk_int));
    end
endgenerate

assign dyn_clk = clks[scan_grp_dyn_clk];

`ifndef GFX_CG_MONITOR_DISABLE
`ifdef SIMULATION
gfx_cgts_local_cov ugfx_cgts_local_cov(
    .override (override_s),
    .cgcg_en  (n0scan_cgcg_en_d1),
    .active   (n0scan_active_d1),
    .en1      (en1),
    .en2      (en2),
    .en3      (en3),
    .iclk     (Cpl_GFXCLK),
	.static_iclk	(static_iclk),
    .dyn_clk  (dyn_clk)
);
`endif
`endif //  `ifdef GFX_CG_MONITOR_DISABLE

endmodule
//////////////////////////////////////////////////////////////////////////////////////////////////////////

