//////////////////////////////////////////////////////////////////////////////////////////////////////////
//Description: Local clock gating control module for the Shader complex.
//////////////////////////////////////////////////////////////////////////////////////////////////////////

`include "gc_features.vh"
`include "mem_features.vh"
`define CLK_2_LS_DLY 7

module gfx_cgts_local_cntl(/*AUTOARG*/
   // Outputs
   cgts_cntl_sout_override ,
   cgts_cntl_sout ,
   cgts_clken_sout ,
   cgts_cntl_dout_override ,
   cgts_cntl_dout ,
   cgts_clken_dout ,
   cgts_cntl_sout_ls_override ,
   cgts_cntl_dout_ls_override ,

   gate_en0 ,
   gate_en1 ,
   gate_en2 ,
   gate_en3 ,
   gate_en4 ,
   gate_en5 ,
   mem_power_ctrl0 ,
   mem_power_ctrl1 ,
   mem_power_ctrl2 ,
   mem_power_ctrl3 ,
   mem_power_ctrl4 ,
   mem_power_ctrl5 ,

   alu_busy_out,
   fetch_busy_out,
   simd_busy_out,
`ifdef GPU__GC__CGTS_BYPASS__1
      oCGTS_core_clk_en,
      oCGTS_hcore_clk_en,
      oCGTS_vbus_clk_en,
`endif
   // Inputs
`ifdef GPU__GC__CGTS_BYPASS__1
      iCGTS_core_clk_en,
      iCGTS_vbus_clk_en,
`endif
   cgcg_en,
   iclk, resetb,
   free_iclk, 
   dft_clk_gate_ctrl,
   dft_shift_clk,

   cgts_cntl_in_ls_override,
   cgts_cntl_in_override,
   cgts_cntl_in,
   cgts_clken_in,
   mem_power_ctrl_in,

   alu_busy_in,
   fetch_busy_in,
   fetch_busy_sel_in,
   simd_busy_in
   );

//////////////////////////
//in/outs
//////////////////////////
`ifdef GPU__GC__CGTS_BYPASS__1
 output     oCGTS_core_clk_en;
 output     oCGTS_hcore_clk_en;
 output     oCGTS_vbus_clk_en;
`endif
   // Inputs
`ifdef GPU__GC__CGTS_BYPASS__1
 input     iCGTS_core_clk_en;
 input     iCGTS_vbus_clk_en;
`endif
 input       cgcg_en;
 input       iclk;
 input       resetb;
 input       free_iclk;
 input       [1:0] dft_clk_gate_ctrl;
 input       dft_shift_clk;

 input       cgts_cntl_in_ls_override;
 input       cgts_cntl_in_override;
 input [3:0] cgts_cntl_in;
 input       cgts_clken_in;
 input [`MEM__MEM_POWER_CTRL-1:0] mem_power_ctrl_in;

 input       alu_busy_in;
 input       fetch_busy_in;
 input       fetch_busy_sel_in;
 input       simd_busy_in;


 output       cgts_cntl_sout_override;
 output [3:0] cgts_cntl_sout;
 output       cgts_clken_sout;
 output       cgts_cntl_dout_override;
 output [3:0] cgts_cntl_dout;
 output       cgts_clken_dout;
 output       cgts_cntl_sout_ls_override;
 output       cgts_cntl_dout_ls_override;

 output      gate_en0;
 output      gate_en1;
 output      gate_en2;
 output      gate_en3;
 output      gate_en4;
 output      gate_en5;
 output [`MEM__MEM_POWER_CTRL-1:0] mem_power_ctrl0;
 output [`MEM__MEM_POWER_CTRL-1:0] mem_power_ctrl1;
 output [`MEM__MEM_POWER_CTRL-1:0] mem_power_ctrl2;
 output [`MEM__MEM_POWER_CTRL-1:0] mem_power_ctrl3;
 output [`MEM__MEM_POWER_CTRL-1:0] mem_power_ctrl4;
 output [`MEM__MEM_POWER_CTRL-1:0] mem_power_ctrl5;
 output      alu_busy_out;
 output      fetch_busy_out;
 output      simd_busy_out;


integer i;
integer i_dly;
// added for CGTS BYPASS.

`ifdef GPU__GC__CGTS_BYPASS__1
reg    dCGTS_core_clk_en;
reg    dCGTS_vbus_clk_en;
assign oCGTS_core_clk_en = dCGTS_core_clk_en;
assign oCGTS_hcore_clk_en = dCGTS_core_clk_en;
assign oCGTS_vbus_clk_en = dCGTS_vbus_clk_en;
`endif

always @(posedge iclk)
begin
`ifdef GPU__GC__CGTS_BYPASS__1
  dCGTS_core_clk_en <= iCGTS_core_clk_en;
  dCGTS_vbus_clk_en <= iCGTS_vbus_clk_en;
`endif
end

// Added for cgts_clken. it is for cgts_local_cntl's MGCG.

reg        cgts_clken_dly;
wire       cgts_clken_resetb =  ~resetb | cgts_clken_in;
always @(posedge iclk)
begin
    cgts_clken_dly <= cgts_clken_in;
end
wire       cgts_clken_sout = cgts_clken_dly;
wire       cgts_clken_dout = cgts_clken_dly;


wire       gated_iclk;

   gfx_cgts_local #(.scan_grp_dyn_clk(0), .enable_dftcgmg_gate_merge(1)) gfx_cgts_local_inst0
     (// Inputs
      .Cpl_GFXCLK  (free_iclk),
      .static_iclk (iclk),
      .dft_clk_gate_ctrl             (dft_clk_gate_ctrl),
      .dft_shift_clk                 (dft_shift_clk),
      .cgcg_en     (cgcg_en),
      .override    (1'b0),
      .en1         (cgts_clken_resetb),
      .en2         (1'b0),
      .en3         (1'b0),
      // Outputs
      .dyn_clk     (gated_iclk),
      .dyn_clk_vld ()
      );

reg         gate_en0;
reg         gate_en1;
reg         gate_en2;
reg         gate_en3;
reg         gate_en4;
reg         gate_en5;
reg         alu_busy_out;
reg         fetch_busy_out;
reg         simd_busy_out;

wire        ls_override_in;
wire        override_in;
wire        program_in;
wire        state_in;
wire        valid_in;
wire        data_in;
//reg   [1:0] override_dly;
reg        ls_override_dly;
reg        override_dly;
wire        mem_power_ctrl_dly;
reg   [1:0] program_dly;
reg   [1:0] state_dly;
reg   [1:0] valid_dly;
reg  [16:0] data_dly;
reg         off_cmd;
reg         rd_valid_cmd;
reg         on_cmd;
reg         clear_all_locks;
reg         program_cmd;
reg         blk_lock_cmd;
reg         row_lock_cmd;
reg         read_cmd;
reg         read_cmd_d1;
//reg         read_cmd_d2;
reg         read_mode;
reg         read_mode_d1;
reg         rd_clear_blk_lock;
reg         rd_clear_blk_lock_d1;
reg         rd_clear_row_lock;
reg         rd_set_all_locks;
//reg         rd_set_all_locks_d1;
reg         unlock_cmd;
wire         reset_cmd;
//reg         reset_cmd_d1;
wire         always_pass_the_cmd;
reg         always_pass_the_cmd_d1;
wire         shift_reg_en;
reg         blk_locked_en;
reg         pre_blk_locked;
reg         blk_locked;
reg         blk_locked_d1;
//reg  [12:0] blk_locked_dly;
reg         row_locked_en;
reg         pre_row_locked;
reg         row_locked;
reg         row_locked_d1;
reg         pre_sout_program;
reg         pre_sout_state;
reg         pre_sout_valid;
reg         pre_sout_data;
wire   [3:0] pre_cgts_cntl_sout;
reg   [3:0] pre_cgts_cntl_sout_d1;
reg         pre_dout_program;
reg         pre_dout_state;
reg         pre_dout_valid;
reg         pre_dout_data;
wire   [3:0] pre_cgts_cntl_dout;
reg   [3:0] pre_cgts_cntl_dout_d1;
wire        on_cmd_pulse;
wire        off_cmd_pulse;
reg         pre_on_flag;
reg         pre_off_flag;
reg         on_off_flag_en;
reg  [10:0] delay_mult_shifted;
reg  [11:0] pre_delay_mult_cnt;
reg         delay_mult_cnt_ld;
reg         delay_mult_cnt_eq0;
wire        middle_of_off_but_turn_on;
reg         turn_on_ge;
reg         turn_on_ge_d1;
reg         turn_on_ge_d2;
reg         turn_on_ge_d3;
reg         turn_on_ge_d4;
reg         turn_on_ge_d5;
reg         turn_on_ge_d6;
reg         turn_on_ge_d7;
reg         turn_on_ge_dly;

reg[`CLK_2_LS_DLY-1:0]	turn_on_g1_dly;
reg[`CLK_2_LS_DLY-1:0]	turn_on_g2_dly;
reg[`CLK_2_LS_DLY-1:0]	turn_on_g5_dly;
reg			pre_turn_on_g1_dly;
reg			pre_turn_on_g2_dly;
reg			pre_turn_on_g5_dly;

wire        pre_turn_on_ge_dly;
reg         turn_off_ls;
reg         turn_off_ls_g1;
reg         turn_off_ls_g2;
reg         turn_off_ls_g5;
reg         simdbusy_soft_override;
reg         ls_soft_override;
reg         cg_soft_override;
reg   [1:0] busy_override;
reg         ge5;
reg         ge4;
reg         ge3;
reg         ge2;
reg         ge1;
reg         ge0;
reg         base;
reg   [3:0] delay_mult_new;
wire  [6:0] delay_mult = {3'b0,delay_mult_new};
reg         on_cmd_d1;
reg         off_cmd_d1;
reg         on_flag;
reg         off_flag;
reg  [10:0] delay_mult_cnt;
reg   [4:0] pre_rd_data_mux_sel;
reg   [4:0] rd_data_mux_sel;
reg         rd_data_mux_sel_en;
reg         rd_data;
reg         rd_data_d1;
//wire        soft_override;
//reg         soft_override_d1;
wire        busy_mux;
reg         busy_mux_d1;
reg         simd_busy_d1;
wire        ls_en_flag;


assign ls_override_in = cgts_cntl_in_ls_override;
assign override_in = cgts_cntl_in_override;
assign program_in  = cgts_cntl_in[3];
assign state_in    = cgts_cntl_in[2];
assign valid_in    = cgts_cntl_in[1];
assign data_in     = cgts_cntl_in[0];

always @(posedge iclk) begin
  if(cgts_clken_resetb) 
  begin
  ls_override_dly <= ls_override_in;
  override_dly    <= override_in;
  end
end

`ifdef MEM__MEM__AMD32
  assign mem_power_ctrl_dly = 1'b0;  //no lightsleep in SOI memory
`else
  assign mem_power_ctrl_dly = mem_power_ctrl_in[0];
`endif

always @(posedge iclk)
begin
  //ls_override_dly <= ls_override_in;
  //override_dly    <= override_in;
  //mem_power_ctrl_dly <= mem_power_ctrl_in;
  if(cgts_clken_resetb) 
  begin
  program_dly[0]  <= program_in;
  state_dly[0]    <= state_in;
  valid_dly[0]    <= valid_in;
  data_dly[0]     <= data_in;
  end
end

always @(posedge gated_iclk)
begin
  program_dly[1]  <= program_dly[0];
  state_dly[1]    <= state_dly[0];
  valid_dly[1]    <= valid_dly[0];
  for(i=0; i<16; i=i+1)
  begin
    data_dly[i+1] <= data_dly[i];
  end
end

always @(*)
begin
  // Decode the commands
  // The rd_valid_cmd is needed to pass valids down without having the other cgts_local_cntls
  //   downstream interpret it as a command.
  off_cmd           = ~override_dly & ~program_dly[0] & ~state_dly[0] & ~valid_dly[0];
  rd_valid_cmd      = ~override_dly & ~program_dly[0] & ~state_dly[0] &  valid_dly[0];
  on_cmd            = ~override_dly & ~program_dly[0] &  state_dly[0] & ~valid_dly[0];
  clear_all_locks   = ~override_dly & ~program_dly[0] &  state_dly[0] &  valid_dly[0];
  program_cmd       = ~override_dly &  program_dly[0] & ~state_dly[0] & ~valid_dly[0];
  blk_lock_cmd      = ~override_dly &  program_dly[0] & ~state_dly[0] &  valid_dly[0] & ~data_dly[0];
  row_lock_cmd      = ~override_dly &  program_dly[0] & ~state_dly[0] &  valid_dly[0] &  data_dly[0];
  rd_clear_blk_lock = ~override_dly &  program_dly[0] &  state_dly[0] & ~valid_dly[0] & ~data_dly[0];
  rd_clear_row_lock = ~override_dly &  program_dly[0] &  state_dly[0] & ~valid_dly[0] &  data_dly[0];
  read_cmd          = ~override_dly &  program_dly[0] &  state_dly[0] &  valid_dly[0] & ~data_dly[0];
  rd_set_all_locks  = ~override_dly &  program_dly[0] &  state_dly[0] &  valid_dly[0] &  data_dly[0];
  unlock_cmd        =  override_dly;

  read_mode = ~override_dly & program_dly[0] & state_dly[0];

end

assign reset_cmd    = ~resetb | unlock_cmd;
assign always_pass_the_cmd = off_cmd | rd_valid_cmd | on_cmd | program_cmd |
                             clear_all_locks | rd_set_all_locks | reset_cmd;

assign shift_reg_en = blk_lock_cmd & ~blk_locked;


always @(*)
begin
  // PROGRAMMING --
  //   Set the blk_locked flag on a block lock command.  The blk_locked flag indicates that
  //   the serial data has been stored into the shift registers.  If blk_locked is not
  //   set, lock commands are not sent to the cgts_cntl_sout[4:0] path.  If set, lock commands are
  //   allowed to be sent.
  // READING --
  //   Clear the blk_locked flag on a clear block lock command.  The blk_locked flag indicates that
  //   the serial data has been read from the shift registers.  If blk_locked is
  //   set, clear commands are not sent to the cgts_cntl_sout[4:0] path.  If cleared, clear commands are
  //   allowed to be sent.
  pre_blk_locked = ~reset_cmd & (blk_lock_cmd | rd_set_all_locks);
  blk_locked_en = reset_cmd | (blk_lock_cmd & ~blk_locked) | rd_set_all_locks |
                  clear_all_locks | rd_clear_blk_lock;

  // PROGRAMMING --
  //   Set the row_locked_flag when a row lock command is initiated.  If row is not locked,
  //   do not let valids or lock commands go to the cgts_cntl_dout[4:0] path.  Only allow
  //   them once the row is locked (ie. all blocks in the row have their shift registers loaded.
  // READING --
  //   Clear the row_locked_flag when a clear_row_lock command is initiated.  If row is locked,
  //   do not let valids or lock commands go to the cgts_cntl_dout[4:0] path.  Only allow
  //   them once the row is unlocked (ie. all blocks in the row have their shift registers read.
  pre_row_locked = ~reset_cmd & (row_lock_cmd | rd_set_all_locks);
  row_locked_en = reset_cmd | (row_lock_cmd & ~row_locked) | rd_set_all_locks |
                  clear_all_locks | rd_clear_row_lock;


  pre_sout_program   = (blk_locked_d1 & rd_clear_blk_lock_d1) ? 1'b0 : program_dly[1];

  pre_sout_state     = (always_pass_the_cmd_d1 & state_dly[1]) |
                       (read_mode_d1 ? ((~blk_locked & state_dly[1]) & ~(blk_locked_d1 & rd_clear_blk_lock_d1)) :
                                        (state_dly[1] & blk_locked_d1));

  pre_sout_valid     = (always_pass_the_cmd_d1 & valid_dly[1]) |
                       (read_mode_d1 ? ((~blk_locked & valid_dly[1]) | (blk_locked_d1 & rd_clear_blk_lock_d1)) :
                                       (valid_dly[1] & blk_locked_d1));

  pre_sout_data      = (read_cmd_d1 & blk_locked_d1) ? rd_data_d1 : data_dly[1];

end
assign pre_cgts_cntl_sout = {
                             pre_sout_program,
                             pre_sout_state,
                             pre_sout_valid,
                             pre_sout_data
                            };


always @(*)
begin

  pre_dout_program   = (always_pass_the_cmd_d1 & pre_sout_program) |
                       (read_mode_d1 ? 1'b1 : pre_sout_program);

  pre_dout_state     = (always_pass_the_cmd_d1 & pre_sout_state) |
                       (read_mode_d1 ? (~row_locked_d1 & pre_sout_state) : (pre_sout_state & row_locked_d1));

  pre_dout_valid     = (always_pass_the_cmd_d1 & pre_sout_valid) |
                       (read_mode_d1 ? (~row_locked_d1 & pre_sout_valid) : (pre_sout_valid & row_locked_d1));

  pre_dout_data      = pre_sout_data;

end

assign pre_cgts_cntl_dout = {
                             pre_dout_program,
                             pre_dout_state,
                             pre_dout_valid,
                             pre_sout_data
                            };

always @(posedge gated_iclk) begin
  if(delay_mult_cnt_ld) begin
    delay_mult_cnt_eq0 <= (pre_delay_mult_cnt == 0);
  end
end

always @(*)
begin
    // on_flag is set when ON command is initiated & cleared when the OFF command arrives.
    // off_flag is set when OFF command is initiated & cleared when the ON command arrives.
  pre_on_flag     = reset_cmd | on_cmd_pulse;
  pre_off_flag    = ~reset_cmd & off_cmd_pulse;
  on_off_flag_en  = reset_cmd | on_cmd_pulse | off_cmd_pulse;

    // When either an ON or OFF command is received, multiply the delay multiplier by 8 or
    // 16 & then load it into the count.  Decrement the count until is gets to zero.  The
    // delay_mult_cnt is the number of delays before the clocks are turned on/off.
  case (base)
    1'b1:  delay_mult_shifted = {delay_mult, 4'b0};       // mulitply by 16 (ie. left-shift of 4)
    1'b0:  delay_mult_shifted = {1'b0, delay_mult, 3'b0}; // mulitply by 8 (ie. left-shift of 3)
  endcase

    // If in the middle of counting down to turn off and get ON command, clear the
    // delay_mult_cnt right away because we're already ON.
  case ({(reset_cmd | middle_of_off_but_turn_on),
         (reset_cmd | on_cmd_pulse | off_cmd_pulse)})
    2'b11:  pre_delay_mult_cnt = 12'b0;
    2'b10:  pre_delay_mult_cnt = 12'b0;
    2'b01:  pre_delay_mult_cnt = {1'b0, delay_mult_shifted};
    2'b00:  pre_delay_mult_cnt = delay_mult_cnt - 1'b1;
  endcase

  //delay_mult_cnt_eq0 = (delay_mult_cnt == 11'h0);
    // Ignore ON command pulse if ON command flag is already set and
    // ignore OFF command pulse if OFF command flag is already set.
    // Off command pulses can occur after reading of a row.
  delay_mult_cnt_ld = reset_cmd | (on_cmd_pulse & ~on_flag) | (off_cmd_pulse & ~off_flag) | ~delay_mult_cnt_eq0;

  turn_on_ge = (on_flag & delay_mult_cnt_eq0) | (off_flag & ~delay_mult_cnt_eq0)
`ifdef GPU__GC__CGTS_BYPASS__1
            | dCGTS_core_clk_en
`endif
               ;
    // Turn off light-sleep as soon as possible (ie. turn mems on quickly) and turn on
    // light-sleep slowly (ie. turn mems off slowly).
  turn_off_ls = on_flag  | (off_flag & ~delay_mult_cnt_eq0) | turn_on_ge_dly
`ifdef GPU__GC__CGTS_BYPASS__1
                | dCGTS_core_clk_en
`endif
                ;

  turn_off_ls_g1 = gate_en1 | pre_turn_on_g1_dly;
  turn_off_ls_g2 = gate_en2 | pre_turn_on_g2_dly;
  turn_off_ls_g5 = gate_en5 | pre_turn_on_g5_dly;

  gate_en0 = (turn_on_ge & ge0) | cg_soft_override;
  gate_en1 = ((turn_on_ge & (busy_override[0] | busy_mux_d1)) & ge1) | cg_soft_override;
  gate_en2 = ((turn_on_ge & (busy_override[1] | busy_mux_d1)) & ge2) | cg_soft_override;
  gate_en3 = (turn_on_ge & ge3) | cg_soft_override;
  //gate_en3 = ge3 | cg_soft_override;
  gate_en4 = (turn_on_ge & 
`ifdef GPU__GC__CGTS_BYPASS__1
            dCGTS_vbus_clk_en & 
`endif
             ge4) | cg_soft_override;

  gate_en5 = (turn_on_ge & ge5 & (simd_busy_d1 | simdbusy_soft_override)) | cg_soft_override;
end

// Create a 4 clock delay of turn_on_ge
assign pre_turn_on_ge_dly = turn_on_ge | turn_on_ge_d1 | turn_on_ge_d2 | turn_on_ge_d3 |
                         turn_on_ge_d4 | turn_on_ge_d5 | turn_on_ge_d6 | turn_on_ge_d7;

assign ls_en_flag = ~(ls_soft_override | ls_override_dly | override_dly);

reg pre_mem_power_ctrl0_ls;
reg pre_mem_power_ctrl1_ls;
reg pre_mem_power_ctrl2_ls;
reg pre_mem_power_ctrl3_ls;
reg pre_mem_power_ctrl4_ls;
reg pre_mem_power_ctrl5_ls;
always @(posedge iclk)
begin
  pre_mem_power_ctrl0_ls <= ~(turn_off_ls & ge0) & ls_en_flag;
  pre_mem_power_ctrl1_ls <= ~turn_off_ls_g1 & ls_en_flag;
  pre_mem_power_ctrl2_ls <= ~turn_off_ls_g2 & ls_en_flag;
  pre_mem_power_ctrl3_ls <= ~(turn_off_ls & ge3) & ls_en_flag;
  pre_mem_power_ctrl4_ls <= ~(turn_off_ls & ge4) & ls_en_flag;
  pre_mem_power_ctrl5_ls <= ~turn_off_ls_g5 & ls_en_flag;
end 
assign mem_power_ctrl0[0] = pre_mem_power_ctrl0_ls | mem_power_ctrl_dly;
assign mem_power_ctrl1[0] = pre_mem_power_ctrl1_ls | mem_power_ctrl_dly; //turn_off_ls_g1 contain ge1
assign mem_power_ctrl2[0] = pre_mem_power_ctrl2_ls | mem_power_ctrl_dly;//turn_off_ls_g2 contain ge2
assign mem_power_ctrl3[0] = pre_mem_power_ctrl3_ls | mem_power_ctrl_dly;
assign mem_power_ctrl4[0] = pre_mem_power_ctrl4_ls | mem_power_ctrl_dly;
assign mem_power_ctrl5[0] = pre_mem_power_ctrl5_ls | mem_power_ctrl_dly;//turn_off_ls_g5 contain ge5

//Rashad Oreifej: Pass the rest of the mem_power_ctrl wires as is (this is basiaclly for DS and SD in perSIMDPG
`ifndef MEM__MEM_POWER_CTRL__1 //this means bus width is > 1
assign mem_power_ctrl0[`MEM__MEM_POWER_CTRL-1:1] = mem_power_ctrl_in[`MEM__MEM_POWER_CTRL-1:1];
assign mem_power_ctrl1[`MEM__MEM_POWER_CTRL-1:1] = mem_power_ctrl_in[`MEM__MEM_POWER_CTRL-1:1];
assign mem_power_ctrl2[`MEM__MEM_POWER_CTRL-1:1] = mem_power_ctrl_in[`MEM__MEM_POWER_CTRL-1:1];
assign mem_power_ctrl3[`MEM__MEM_POWER_CTRL-1:1] = mem_power_ctrl_in[`MEM__MEM_POWER_CTRL-1:1];
assign mem_power_ctrl4[`MEM__MEM_POWER_CTRL-1:1] = mem_power_ctrl_in[`MEM__MEM_POWER_CTRL-1:1];
assign mem_power_ctrl5[`MEM__MEM_POWER_CTRL-1:1] = mem_power_ctrl_in[`MEM__MEM_POWER_CTRL-1:1];
`endif

 // Create 1-clk pulses of on/off commands
assign on_cmd_pulse  = on_cmd & ~on_cmd_d1;
 // Ignore off_cmd_pulse if get OFF command in the middle of counting down to turn ON.
assign off_cmd_pulse = (off_cmd & on_flag & ~delay_mult_cnt_eq0) ? 1'b0 : (off_cmd & ~off_cmd_d1);

assign middle_of_off_but_turn_on = (off_flag & ~delay_mult_cnt_eq0 & on_cmd_pulse);

always @(posedge gated_iclk)
begin
  //reset_cmd_d1 <= reset_cmd;

  read_cmd_d1 <= read_cmd;
  //read_cmd_d2 <= read_cmd_d1;

  rd_clear_blk_lock_d1 <= rd_clear_blk_lock;
  //rd_set_all_locks_d1  <= rd_set_all_locks;

  always_pass_the_cmd_d1 <= always_pass_the_cmd;

    // shift registers -- msb to lsb
  if (reset_cmd)
  begin
    simdbusy_soft_override <= 1'b1;
    ls_soft_override <= 1'b1;
    cg_soft_override <= 1'b1;
    busy_override[1:0] <= 2'h3;
    ge5 <= 1'b1;
    ge4 <= 1'b1;
    ge3 <= 1'b1;
    ge2 <= 1'b1;
    ge1 <= 1'b1;
    ge0 <= 1'b1;
    base <= 1'b0;
    delay_mult_new[3:0] <= 4'b0;
  end
  else if (shift_reg_en)
  begin
    simdbusy_soft_override <= data_dly[1];
    ls_soft_override <= data_dly[2];
    cg_soft_override <= data_dly[3];
    busy_override[1] <= data_dly[4];
    busy_override[0] <= data_dly[5];
    ge5 <= data_dly[6];
    ge4 <= data_dly[7];
    ge3 <= data_dly[8];
    ge2 <= data_dly[9];
    ge1 <= data_dly[10];
    ge0 <= data_dly[11];
    base <= data_dly[12];
   // delay_mult[6] <= data_dly[13];
    //delay_mult[5] <= data_dly[14];
   // delay_mult[4] <= data_dly[15];
    delay_mult_new[3] <= data_dly[13];
    delay_mult_new[2] <= data_dly[14];
    delay_mult_new[1] <= data_dly[15];
    delay_mult_new[0] <= data_dly[16];
  end

    // blk_locked flag
  if (blk_locked_en)
    blk_locked <= pre_blk_locked;


    // row_locked flag
  if (row_locked_en)
    row_locked <= pre_row_locked;

  blk_locked_d1    <= blk_locked;
  row_locked_d1    <= row_locked;
  on_cmd_d1        <= on_cmd;
  off_cmd_d1       <= off_cmd;
  read_mode_d1     <= read_mode;

  if (on_off_flag_en)
  begin
    on_flag  <= pre_on_flag;
    off_flag <= pre_off_flag;
  end

  if (delay_mult_cnt_ld)
    delay_mult_cnt <= pre_delay_mult_cnt[10:0];

    // Down control outputs
  pre_cgts_cntl_dout_d1 <= pre_cgts_cntl_dout;

    // Side control outputs
  pre_cgts_cntl_sout_d1 <= pre_cgts_cntl_sout;
//  soft_override_d1 <= soft_override;
end

always @(posedge iclk)
begin
  busy_mux_d1      <= busy_mux;
  alu_busy_out     <= alu_busy_in;
  fetch_busy_out   <= fetch_busy_in;
  simd_busy_out    <= simd_busy_in;
  simd_busy_d1     <= simd_busy_in;
end

always @(posedge iclk)
begin
  turn_on_ge_d1  <= turn_on_ge;
  turn_on_ge_d2  <= turn_on_ge_d1;
  turn_on_ge_d3  <= turn_on_ge_d2;
  turn_on_ge_d4  <= turn_on_ge_d3;
  turn_on_ge_d5  <= turn_on_ge_d4;
  turn_on_ge_d6  <= turn_on_ge_d5;
  turn_on_ge_d7  <= turn_on_ge_d6;
  turn_on_ge_dly <= pre_turn_on_ge_dly;
end  // always

wire g1_neg_pulse = ~gate_en1 &  pre_turn_on_g1_dly; // g1 1->0; use pre_turn_on_g*_dly for make serial reg all change to 0.
wire g1_pos_pulse =  gate_en1 & ~turn_on_g1_dly[0]; // g1 0->1; only one pulse to transfer the 1 to _dly[0],  pre_turn_on_ to be 1.
wire g2_neg_pulse = ~gate_en2 &  pre_turn_on_g2_dly; // g2 1->0;
wire g2_pos_pulse =  gate_en2 & ~turn_on_g2_dly[0]; // g2 0->1;
wire g5_neg_pulse = ~gate_en5 &  pre_turn_on_g5_dly; // g5 1->0;
wire g5_pos_pulse =  gate_en5 & ~turn_on_g5_dly[0]; // g5 0->1;
wire g1_cg_enable = ~resetb | g1_neg_pulse | g1_pos_pulse;
wire g2_cg_enable = ~resetb | g2_neg_pulse | g2_pos_pulse;
wire g5_cg_enable = ~resetb | g5_neg_pulse | g5_pos_pulse;
always @(posedge iclk)
begin
  if(g1_cg_enable) 
  begin
  turn_on_g1_dly[0] <= ~resetb? 0 : gate_en1;
  for (i_dly =1; i_dly < `CLK_2_LS_DLY;i_dly = i_dly +1)
  begin
	turn_on_g1_dly[i_dly] <= ~resetb? 0 : turn_on_g1_dly[i_dly-1];
  end
  end
end  // always
always @(posedge iclk)
begin
  if(g2_cg_enable) 
  begin
  turn_on_g2_dly[0] <= ~resetb? 0 : gate_en2;
  for (i_dly =1; i_dly < `CLK_2_LS_DLY;i_dly = i_dly +1)
  begin
	turn_on_g2_dly[i_dly] <= ~resetb? 0 : turn_on_g2_dly[i_dly-1];
  end
  end
end  // always
always @(posedge iclk)
begin
  if(g5_cg_enable) 
  begin
  turn_on_g5_dly[0] <= ~resetb? 0 : gate_en5;
  for (i_dly =1; i_dly < `CLK_2_LS_DLY;i_dly = i_dly +1)
  begin
	turn_on_g5_dly[i_dly] <= ~resetb? 0 : turn_on_g5_dly[i_dly-1];
  end
  end
end  // always
always @*
begin
	pre_turn_on_g1_dly = 0;
	pre_turn_on_g2_dly = 0;
	pre_turn_on_g5_dly = 0;
	for (i_dly =0; i_dly < `CLK_2_LS_DLY;i_dly = i_dly +1)
	begin
		pre_turn_on_g1_dly = pre_turn_on_g1_dly | turn_on_g1_dly[i_dly];
		pre_turn_on_g2_dly = pre_turn_on_g2_dly | turn_on_g2_dly[i_dly];
		pre_turn_on_g5_dly = pre_turn_on_g5_dly | turn_on_g5_dly[i_dly];
	end

end


  // soft_override is true if all programmed bits are set.  Force all clocks on.
//assign soft_override = (&busy_override[3:0]) & ge5 & ge4 & ge3 & ge2 & ge1 & ge0 & base & (&delay_mult[6:0]);

assign cgts_cntl_dout = pre_cgts_cntl_dout_d1;
ati_buffer0 dout_ls_override_buff(.A(ls_override_in), .Y(cgts_cntl_dout_ls_override));
ati_buffer0    dout_override_buff(.A(override_in),    .Y(cgts_cntl_dout_override));
assign cgts_cntl_sout = pre_cgts_cntl_sout_d1;
ati_buffer0 sout_ls_override_buff(.A(ls_override_in), .Y(cgts_cntl_sout_ls_override));
ati_buffer0    sout_override_buff(.A(override_in),    .Y(cgts_cntl_sout_override));


  // mux busy signals and register
assign busy_mux = fetch_busy_sel_in ? fetch_busy_in : alu_busy_in;

always @(*)
begin
    // Create the selects to choose the shift data bit to be read out
  pre_rd_data_mux_sel = (reset_cmd | rd_set_all_locks) ? 5'h0 : (rd_data_mux_sel + 1'b1);
  rd_data_mux_sel_en  = (reset_cmd | rd_set_all_locks) | read_cmd;

    // Mux the read data to output
  case (rd_data_mux_sel)
    5'h0f:  rd_data = simdbusy_soft_override;
    5'h0e:  rd_data = ls_soft_override;
    5'h0d:  rd_data = cg_soft_override;
    5'h0c:  rd_data = busy_override[1];
    5'h0b:  rd_data = busy_override[0];
    5'h0a:  rd_data = ge5;
    5'h09:  rd_data = ge4;
    5'h08:  rd_data = ge3;
    5'h07:  rd_data = ge2;
    5'h06:  rd_data = ge1;
    5'h05:  rd_data = ge0;
    5'h04:  rd_data = base;
//    5'h06:  rd_data = delay_mult[6];
//    5'h05:  rd_data = delay_mult[5];
//    5'h04:  rd_data = delay_mult[4];
    5'h03:  rd_data = delay_mult[3];
    5'h02:  rd_data = delay_mult[2];
    5'h01:  rd_data = delay_mult[1];
    5'h00:  rd_data = delay_mult[0];
    default:  rd_data = 1'b0;
  endcase

end


always @(posedge gated_iclk)
begin
  if (rd_data_mux_sel_en)
    rd_data_mux_sel <= pre_rd_data_mux_sel;

  rd_data_d1 <= rd_data;
end


//-----------------------------------
// Assertions
//-----------------------------------
`ifdef SIMULATION
wire     reset;
reg      reset_d1;
wire     reset_pulse;
//reg      toggle_flag;
wire     program_dly_bit1;
wire     state_dly_bit1;
reg      fetch_busy_sel_d1;
//reg      cg_soft_override_d1;
//wire     cg_override_flag;

assign reset = ~resetb;
assign reset_pulse = ~reset & reset_d1;
assign program_dly_bit1 = program_dly[1];
assign state_dly_bit1 = state_dly[1];
//assign cg_override_flag = cg_soft_override | cg_soft_override_d1;

always @(posedge iclk)
begin
  reset_d1 <= reset;
  fetch_busy_sel_d1 <= fetch_busy_sel_in;
  //cg_soft_override_d1 <= cg_soft_override;

  //if (reset)
  //  toggle_flag <= 1'b0;
  //else
  //  toggle_flag <= ~toggle_flag;
end

`endif
//// Both on_flag and off_flag are active
//// psl Both_onoff_flags_active: assert
//// never (resetb && on_flag && off_flag) @(posedge iclk);
//
//// Turning on in middle of off but gate_en0 still off
//// psl On_in_midoff_gate_en0_off: assert
//// never {middle_of_off_but_turn_on; ~gate_en0} @(posedge iclk);
//
//// Gate_en0 did not turn off
//// psl Did_not_turn_off: assert
//// never (resetb && ~cg_soft_override && off_flag && delay_mult_cnt_eq0 && gate_en0) @(posedge iclk);
//
//// Gate_en0 did not turn on
//// psl Did_not_turn_on: assert
//// never (resetb && ~cg_soft_override && on_flag && delay_mult_cnt_eq0 && ~gate_en0) @(posedge iclk);
//
//// Gate_en0 turned on too soon
//// psl Turned_on_too_soon: assert
//// never (resetb && ~cg_soft_override && on_flag && ~delay_mult_cnt_eq0 && gate_en0) @(posedge iclk);
//
//// Gate_en0 turned off too soon
//// psl Turned_off_too_soon: assert
//// never (resetb && ~cg_soft_override && off_flag && ~delay_mult_cnt_eq0 && ~gate_en0) @(posedge iclk);
//
//// Gate_en0 never turned on
//// psl gate_en0_never_turned_on: assert
//// never {reset_pulse; ~gate_en0} @(posedge iclk);
//
//// ON_flag did not stay on
//// psl Did_not_stay_on: assert
//// never {(resetb && ~cg_soft_override && on_flag && ~delay_mult_cnt_eq0 && off_cmd && ~off_cmd_d1); ~on_flag} @(posedge iclk);
//
//// ge0 not set after an override
//// psl ge0_not_set_after_override: assert
//// never {(resetb && override_dly); ~ge0} @(posedge iclk);
//
//// ge1 not set after an override
//// psl ge1_not_set_after_override: assert
//// never {(resetb && override_dly); ~ge1} @(posedge iclk);
//
//// ge2 not set after an override
//// psl ge2_not_set_after_override: assert
//// never {(resetb && override_dly); ~ge2} @(posedge iclk);
//
//// ge3 not set after an override
//// psl ge3_not_set_after_override: assert
//// never {(resetb && override_dly); ~ge3} @(posedge iclk);
//
//// ge4 not set after an override
//// psl ge4_not_set_after_override: assert
//// never {(resetb && override_dly); ~ge4} @(posedge iclk);
//
//// ge5 not set after an override
//// psl ge5_not_set_after_override: assert
//// never {(resetb && override_dly); ~ge5} @(posedge iclk);
//
//// gate_en0 not set after a cg_soft_override
//// psl gate_en0_not_set_after_cg_soft_override: assert
//// never {(resetb && cg_soft_override && ~shift_reg_en); ~gate_en0} @(posedge iclk);
//
//// gate_en1 not set after a cg_soft_override
//// psl gate_en1_not_set_after_cg_soft_override: assert
//// never {(resetb && cg_soft_override && ~shift_reg_en); ~gate_en1} @(posedge iclk);
//
//// gate_en2 not set after a cg_soft_override
//// psl gate_en2_not_set_after_cg_soft_override: assert
//// never {(resetb && cg_soft_override && ~shift_reg_en); ~gate_en2} @(posedge iclk);
//
//// gate_en3 not set after a cg_soft_override
//// psl gate_en3_not_set_after_cg_soft_override: assert
//// never {(resetb && cg_soft_override && ~shift_reg_en); ~gate_en3} @(posedge iclk);
//
//// gate_en4 not set after a cg_soft_override
//// psl gate_en4_not_set_after_cg_soft_override: assert
//// never {(resetb && cg_soft_override && ~shift_reg_en); ~gate_en4} @(posedge iclk);
//
//// gate_en5 not set after a cg_soft_override
//// psl gate_en5_not_set_after_cg_soft_override: assert
//// never {(resetb && cg_soft_override && ~shift_reg_en); ~gate_en5} @(posedge iclk);
//
//// During programming, row_locked set before blk_locked
//// psl rowlock_set_before_blklock_during_pgm: assert
//// never (resetb && program_dly_bit1 && ~state_dly_bit1) && (~blk_locked && row_locked) @(posedge iclk);
//
//// During reading, row_locked cleared before blk_locked
//// psl rowlock_reset_before_blklock_during_read: assert
//// never (resetb && program_dly_bit1 && state_dly_bit1) && (blk_locked && ~row_locked) @(posedge iclk);
//
//// Locks not all cleared
//// psl Locks_not_all_cleared: assert
//// never {clear_all_locks; ~(~blk_locked && ~row_locked)} @(posedge iclk);
//
//// Locks not all set
//// psl Locks_not_all_set: assert
//// never {rd_set_all_locks; ~(blk_locked && row_locked)} @(posedge iclk);
//
//
//// gate_en1 not set during an alu_busy
//// psl gate_en1_not_set_after_alu_busy: assert
//// never {resetb && ~fetch_busy_sel_d1 && alu_busy_out && turn_on_ge && ge1 && ~shift_reg_en && ~gate_en1} @(posedge iclk);
//
//// gate_en1 not set during a fetch_busy
//// psl gate_en1_not_set_during_fetch_busy: assert
//// never {resetb && fetch_busy_sel_d1 && fetch_busy_out && turn_on_ge && ge1 && ~shift_reg_en && ~gate_en1} @(posedge iclk);
//
//// gate_en1 not set during its busy_override
//// psl gate_en1_not_set_during_its_busy_override: assert
//// never {resetb && busy_override[0] && turn_on_ge && ge1 && ~shift_reg_en && ~gate_en1} @(posedge iclk);
//
//// gate_en2 not set during an alu_busy
//// psl gate_en2_not_set_after_alu_busy: assert
//// never {resetb && ~fetch_busy_sel_d1 && alu_busy_out && turn_on_ge && ge2 && ~shift_reg_en && ~gate_en2} @(posedge iclk);
//
//// gate_en2 not set during a fetch_busy
//// psl gate_en2_not_set_during_fetch_busy: assert
//// never {resetb && fetch_busy_sel_d1 && fetch_busy_out && turn_on_ge && ge2 && ~shift_reg_en && ~gate_en2} @(posedge iclk);
//
//// gate_en2 not set during its busy_override
//// psl gate_en2_not_set_during_its_busy_override: assert
//// never {resetb && busy_override[1] && turn_on_ge && ge2 && ~shift_reg_en && ~gate_en2} @(posedge iclk);
//
//
//// mem_power_ctrl0 not set when mem_power_ctrl_in is set
//// psl mem_power_ctrl0_not_set_during_mem_power_ctrl_dly: assert
//// never {resetb && mem_power_ctrl_dly && ~mem_power_ctrl0[0]} @(posedge iclk);
//
//// mem_power_ctrl1 not set when mem_power_ctrl_in is set
//// psl mem_power_ctrl1_not_set_during_mem_power_ctrl_dly: assert
//// never {resetb && mem_power_ctrl_dly && ~mem_power_ctrl1[0]} @(posedge iclk);
//
//// mem_power_ctrl2 not set when mem_power_ctrl_in is set
//// psl mem_power_ctrl2_not_set_during_mem_power_ctrl_dly: assert
//// never {resetb && mem_power_ctrl_dly && ~mem_power_ctrl2[0]} @(posedge iclk);
//
//// mem_power_ctrl3 not set when mem_power_ctrl_in is set
//// psl mem_power_ctrl3_not_set_during_mem_power_ctrl_dly: assert
//// never {resetb && mem_power_ctrl_dly && ~mem_power_ctrl3[0]} @(posedge iclk);
//
//// mem_power_ctrl4 not set when mem_power_ctrl_in is set
//// psl mem_power_ctrl4_not_set_during_mem_power_ctrl_dly: assert
//// never {resetb && mem_power_ctrl_dly && ~mem_power_ctrl4[0]} @(posedge iclk);
//
//// mem_power_ctrl5 not set when mem_power_ctrl_in is set
//// psl mem_power_ctrl5_not_set_during_mem_power_ctrl_dly: assert
//// never {resetb && mem_power_ctrl_dly && ~mem_power_ctrl5[0]} @(posedge iclk);
//
//
//// mem_power_ctrl0 not cleared when override is set
//// psl mem_power_ctrl0_not_cleared_during_override: assert
//// never {resetb && (ls_soft_override || ls_override_dly || override_dly) && ~mem_power_ctrl_dly && mem_power_ctrl0[0]} @(posedge iclk);
//
//// mem_power_ctrl1 not cleared when override is set
//// psl mem_power_ctrl1_not_cleared_during_override: assert
//// never {resetb && (ls_soft_override || ls_override_dly || override_dly) && ~mem_power_ctrl_dly && mem_power_ctrl1[0]} @(posedge iclk);
//
//// mem_power_ctrl2 not cleared when override is set
//// psl mem_power_ctrl2_not_cleared_during_override: assert
//// never {resetb && (ls_soft_override || ls_override_dly || override_dly) && ~mem_power_ctrl_dly && mem_power_ctrl2[0]} @(posedge iclk);
//
//// mem_power_ctrl3 not cleared when override is set
//// psl mem_power_ctrl3_not_cleared_during_override: assert
//// never {resetb && (ls_soft_override || ls_override_dly || override_dly) && ~mem_power_ctrl_dly && mem_power_ctrl3[0]} @(posedge iclk);
//
//// mem_power_ctrl4 not cleared when override is set
//// psl mem_power_ctrl4_not_cleared_during_override: assert
//// never {resetb && (ls_soft_override || ls_override_dly || override_dly) && ~mem_power_ctrl_dly && mem_power_ctrl4[0]} @(posedge iclk);
//
//// mem_power_ctrl5 not cleared when override is set
//// psl mem_power_ctrl5_not_cleared_during_override: assert
//// never {resetb && (ls_soft_override || ls_override_dly || override_dly) && ~mem_power_ctrl_dly && mem_power_ctrl5[0]} @(posedge iclk);
//
//
//// turn_off_ls is not set when turn_on_ge is set
//// psl turn_off_ls_not_set_during_turn_on_ge: assert
//// never {resetb && turn_on_ge && ~turn_off_ls} @(posedge iclk);
//
//
//// LS is set too soon from gate_en*
//
//// psl mem_power_ctrl0_set_too_soon_from_gate_en0:assert
//// always ( fell(gate_en0 & ~cg_soft_override)   -> (!mem_power_ctrl0[0])[*3] ) @(posedge iclk);
//
//
//// psl mem_power_ctrl1_set_too_soon_from_gate_en1:assert
//// always ( fell(gate_en1 & ~cg_soft_override & ~busy_override[0]) & ge1  -> (!mem_power_ctrl1[0])[*3] ) @(posedge iclk);
//
//// psl mem_power_ctrl2_set_too_soon_from_gate_en2:assert
//// always ( fell(gate_en2 & ~cg_soft_override & ~busy_override[1]) & ge2   -> (!mem_power_ctrl2[0])[*3] ) @(posedge iclk);
//
//// psl mem_power_ctrl3_set_too_soon_from_gate_en3:assert
//// always ( fell(gate_en3 & ~cg_soft_override)   -> (!mem_power_ctrl3[0])[*3] ) @(posedge iclk);
//
//// psl mem_power_ctrl4_set_too_soon_from_gate_en4:assert
//// always ( fell(gate_en4 & ~cg_soft_override)   -> (!mem_power_ctrl4[0])[*3] ) @(posedge iclk);
////
//// psl mem_power_ctrl5_set_too_soon_from_gate_en5:assert
//// always ( fell(gate_en5 & ~cg_soft_override & ~simdbusy_soft_override) & ge5  -> (!mem_power_ctrl5[0])[*3] ) @(posedge iclk);
//
//// TODO
//// gate_en* is set too soon from LS
//
//
//// ************************************************************************
//// FIXME: mem_power_ctrl*[0] are now delayed by one cycle to meet timing.
////        Hack gate_en0/gate_en3/gate_en4 checkers to avoid assertion cry.
////        Need to monitor if mem_power_ctrl0(3,4) is used in the future,
////        or we may miss the memory ON_DELAY issue.
//// ************************************************************************
//// mem_power_ctrl0 and gate_en0 are both set when they shouldn't
//// psl mem_power_ctrl0_and_gate_en0_set: assert
//// never{resetb && ~mem_power_ctrl_dly && ~cg_soft_override && gate_en0 && (~(turn_off_ls & ge0) & ls_en_flag)} @(posedge iclk);
////// never{resetb && ~mem_power_ctrl_dly && ~cg_soft_override && gate_en0 && mem_power_ctrl0[0]} @(posedge iclk);
//
//// mem_power_ctrl1 and gate_en1 are both set when they shouldn't
//// psl mem_power_ctrl1_and_gate_en1_set: assert
//// never{resetb && ~mem_power_ctrl_dly && ~cg_soft_override && gate_en1 && (~turn_off_ls_g1 & ls_en_flag)} @(posedge iclk);
////// never{resetb && ~mem_power_ctrl_dly && ~cg_soft_override && gate_en1 && mem_power_ctrl1[0]} @(posedge iclk);
//
//// mem_power_ctrl2 and gate_en2 are both set when they shouldn't
//// psl mem_power_ctrl2_and_gate_en2_set: assert
//// never{resetb && ~mem_power_ctrl_dly && ~cg_soft_override && gate_en2 && (~turn_off_ls_g2 & ls_en_flag)} @(posedge iclk);
////// never{resetb && ~mem_power_ctrl_dly && ~cg_soft_override && gate_en2 && mem_power_ctrl2[0]} @(posedge iclk);
//
//// mem_power_ctrl3 and gate_en3 are both set when they shouldn't
//// psl mem_power_ctrl3_and_gate_en3_set: assert
//// never{resetb && ~mem_power_ctrl_dly && ~cg_soft_override && gate_en3 && (~(turn_off_ls & ge3) & ls_en_flag)} @(posedge iclk);
////// never{resetb && ~mem_power_ctrl_dly && ~cg_soft_override && gate_en3 && mem_power_ctrl3[0]} @(posedge iclk);
//
//// mem_power_ctrl4 and gate_en4 are both set when they shouldn't
//// psl mem_power_ctrl4_and_gate_en4_set: assert
//// never{resetb && ~mem_power_ctrl_dly && ~cg_soft_override && gate_en4 && (~(turn_off_ls & ge4) & ls_en_flag)} @(posedge iclk);
////// never{resetb && ~mem_power_ctrl_dly && ~cg_soft_override && gate_en4 && mem_power_ctrl4[0]} @(posedge iclk);
//
//// mem_power_ctrl5 and gate_en5 are both set when they shouldn't
//// psl mem_power_ctrl5_and_gate_en5_set: assert
//// never{resetb && ~mem_power_ctrl_dly && ~cg_soft_override && gate_en5 && (~turn_off_ls_g5 & ls_en_flag)} @(posedge iclk);
////// never{resetb && ~mem_power_ctrl_dly && ~cg_soft_override && gate_en5 && mem_power_ctrl5[0]} @(posedge iclk);

`ifndef GFX_CG_MONITOR_DISABLE
`ifdef SIMULATION
gfx_cgts_local_cntl_cov ugfx_cgts_local_cntl_cov(
`ifdef GPU__GC__CGTS_BYPASS__1
    .iCGTS_core_clk_en(iCGTS_core_clk_en),
    .iCGTS_vbus_clk_en(iCGTS_vbus_clk_en),
    .cgts_cntl_in(cgts_cntl_in),
`endif
    .simdbusy_soft_override (simdbusy_soft_override),
    .ls_soft_override       (ls_soft_override),
    .cg_soft_override       (cg_soft_override),
    .busy_override          (busy_override),
	.ls_override			(ls_override_dly),
	.override				(override_dly),
	.ls_override_in			(ls_override_in),
	.override_in			(override_in),
    .gated_iclk             (gated_iclk),
    .iclk                   (iclk),
    .resetb                   (resetb)
);
`endif
`endif //  `ifdef GFX_CG_MONITOR_DISABLE

`ifdef DEBUG_CLKGATE
  wire        out_sbo;
  wire        out_lso;
  wire        out_cgo;
  wire  [1:0] out_bo;
  wire  [5:0] out_ge;
  wire        out_base;
  wire  [6:0] out_dm;
  wire [23:0] out_all;

  assign out_sbo  = simdbusy_soft_override;
  assign out_lso  = ls_soft_override;
  assign out_cgo  = cg_soft_override;
  assign out_bo   = busy_override;
  assign out_ge   = {ge5, ge4, ge3, ge2, ge1, ge0};
  assign out_base = base;
  assign out_dm   = delay_mult;
  assign out_all  = {out_sbo, out_lso, out_cgo, out_bo, out_ge, out_base, out_dm};
`endif

endmodule
