module gfx_cgts_local_cov(
    // Inputs to cgts_local
    override, cgcg_en, active, en1, en2, en3, iclk, static_iclk,
    // Outputs of cgts_local
    dyn_clk
);
//////////////////////////
input           override;
input		cgcg_en;
input 	        active;
input           en1;
input           en2;
input           en3;
input           iclk;
input			static_iclk;

input           dyn_clk;
//////////////////////////

//VCS coverage off
`ifdef VCS

reg enable;

initial begin
  if($test$plusargs("cgt_assert_disable"))
    enable = 1'b0;
  else
    enable = 1'b1;
end

reg hier_enable;
string Hier;
reg acv_hier_enable;

initial begin
  hier_enable = 1'b1;
  acv_hier_enable = 1'b0;

  $sformat(Hier, "%m");

for(integer i=0;i<Hier.len();i++) 
  begin 
    if((Hier.substr(i,i+18)=="gfx_cgts_local_cntl") || (Hier.substr(i,i+17)=="cgts_sp_local_cntl") || (Hier.substr(i,i+2)=="acv") || (Hier.substr(i,i+2)=="sqc") || (Hier.substr(i,i+2)=="spi") || (Hier.substr(i,i+2)=="sqg") || (Hier.substr(i,i+6)=="sq_comm") )      
	begin 
           hier_enable = 1'b0; 
           break; 
    end 
  end

  for(integer i=0;i<Hier.len();i++) 
  begin 
    if(Hier.substr(i,i+2)=="acv")      
	begin 
           acv_hier_enable = 1'b1; 
           break; 
    end 
  end

end

wire seq = cgcg_en && (override | en1 | en2 | en3);
`ifdef GC__PIPE_MGCG_GATER__1
//core_busy_active_dyn_clk_inactive : assert property (@(negedge iclk) disable iff(~enable) (seq ##1 seq |=> dyn_clk));
//core_busy_inactive_dyn_clk_active : assert property (@(negedge iclk) disable iff(~enable) (seq ##1 ~seq  |-> ##3 (dyn_clk!==1'b1)));
//cgcg_clk_active					  : assert property (@(negedge iclk) disable iff(~enable) (~cgcg_en |-> ##2 (dyn_clk!==1'b1)));
`else
core_busy_active_dyn_clk_inactive : assert property (@(negedge iclk) disable iff(~enable) (seq ##1 seq |-> dyn_clk));
core_busy_inactive_dyn_clk_active : assert property (@(negedge iclk) disable iff(~enable) (~seq  |=> (dyn_clk!==1'b1)));
cgcg_clk_active					  : assert property (@(negedge iclk) disable iff(~enable) (~cgcg_en |=> (dyn_clk!==1'b1)));
`endif

//reg	signal_checked;
//initial signal_checked		= 1'b0;
//wire a = $isunknown(cgcg_en);
//wire b = $isunknown(override);
//wire c = $isunknown(en1);
//wire d = $isunknown(en2);
//wire e = $isunknown(en3);
//wire sall = a | b | c | d | e;
//always @(posedge iclk) if (~sall) signal_checked <= 1'b1;
//final if((signal_checked == 1'b0) && enable && cgt_en_assert_enable) `MSGIO_DISPLAY("%m ERROR:CG_VLD checker: the connection of CGTS signal check is failed");

// busy check
reg busy_deasserted;
reg	busy_asserted;
reg override_disabled;
reg override_enabled;

wire busy_en = en1 | en2 | en3;


initial busy_deasserted = 1'b0;
initial busy_asserted   = 1'b0;
initial override_disabled = 1'b0;
initial override_enabled = 1'b0;

always @(posedge iclk) if (busy_en == 1'b0) busy_deasserted <= 1'b1;
always @(posedge iclk) if (busy_en == 1'b1) busy_asserted	<= 1'b1;
always @(posedge iclk) if (override == 1'b0) override_disabled <= 1'b1;
always @(posedge iclk) if (override == 1'b1) override_enabled <= 1'b1;

wire busy_changed = busy_deasserted && busy_asserted;
wire override_changed = override_disabled && override_enabled;

wire tied = (~busy_changed) && (~override_changed);

final if(tied) `MSGIO_DISPLAY("%m WARNING:CG tied checker: core busy and override were been tied");


`ifdef GFX_CG_W2E
reg cgt_en_assert_enable;
initial begin
  if($test$plusargs("cgt_en_assert_disable"))
    cgt_en_assert_enable = 1'b0;
  else
    cgt_en_assert_enable = 1'b1;
end
reg MGCG_Coverage_enable;
initial begin
  if($test$plusargs("MGCG_Coverage_assert_enable"))
    MGCG_Coverage_enable = 1'b1;
  else
    MGCG_Coverage_enable = 1'b0;
end
//FIXME cause of SPI CU BUSY NOT READY
//final if ((busy_changed == 1'b0) && enable && cgt_en_assert_enable && ~tied)
//	`MSGIO_DISPLAY("%m ERROR:CG_VLD checker: core busy never deactivated or activated");
final if((override_changed == 1'b0) && enable && cgt_en_assert_enable && ~tied && hier_enable)
	`MSGIO_DISPLAY("%m ERROR: Override never deactivated");
final if((override == 1'b0) && enable && MGCG_Coverage_enable && hier_enable) `MSGIO_DISPLAY("%m ERROR: Override never activated");

reg cgcg_en_deactivated;
initial cgcg_en_deactivated = 0;
always @(posedge iclk) if(cgcg_en == 1'b0) cgcg_en_deactivated <= 1;
final if((cgcg_en_deactivated == 0) && enable && cgt_en_assert_enable && (~acv_hier_enable)) `MSGIO_DISPLAY("%m ERROR: CLKGATE cgcg_en never deactivated");

//added dis acv assertion
reg resetb = 1'b1;
`include "gfx_cgtt_local_acv_cov.vh"
`endif

`else
initial begin
  $display("%m at %0t: ld/src/misc/cgts_local_cov.sv: SVA not yet supported in NCSIM in wekiva scripts", $realtime);
end
`endif // VCS
//VCS coverage on

// clock efficiency monitor
`ifdef CLK_EFF_MONITOR
        reg[31:0] clocks_counter;
        initial
            clocks_counter = 0;
        always @(posedge dyn_clk)
                clocks_counter <= clocks_counter + 1;
        final `MSGIO_DISPLAY("%m: clock gating efficiency check : clock counter = %d",clocks_counter);
`endif

`ifdef GFX_CG_SIGNALS_DUMP_EN
initial
begin
  #1;
  `ifdef GC__SOC_ENV__1 
    `ifdef GC__FLAVOR__APU 
      $fmonitor(sys.cg_file_handle, "CG CGTS_OV:(%t) %m %b",$time,override);
    `else
      $fmonitor(tb.cg_file_handle, "CG CGTS_OV:(%t) %m %b",$time,override);
     `endif
    `else
      $fmonitor(tb.cg_file_handle, "CG CGTS_OV:(%t) %m %b",$time,override);
    `endif
end
`endif

endmodule



