//////////////////////////////////////////////////////////////////////////////////////////////////////////
//Description: Local (aka medium-grain) clock gating module. It generates three type of clocks: static_oclk, dyn_oclki*, reg_oclk.
//             static_oclk is same as incoming (iclk) clock & is expected to drive your (flow control) interface logic.
//         dyn_oclk* is gated based on your 'busy' signals. reg_oclk is to drive your xRBM registers. reg_oclk is enabled
//         by enable signal coming from xRBM block.
//////////////////////////////////////////////////////////////////////////////////////////////////////////
`timescale 1ps/1ps

`include "gc_features.vh"
`include "mem_features.vh"

module gfx_cgtt_local_1r18d(/*AUTOARG*/
   // Outputs
   dyn_oclk17, dyn_oclk16, dyn_oclk15, dyn_oclk14, dyn_oclk13, dyn_oclk12,
   dyn_oclk11, dyn_oclk10, dyn_oclk9, dyn_oclk8, dyn_oclk7, dyn_oclk6,
   dyn_oclk5, dyn_oclk4, dyn_oclk3, dyn_oclk2, dyn_oclk1, dyn_oclk0, 

   dyn_oclk_vld17, dyn_oclk_vld16, dyn_oclk_vld15, dyn_oclk_vld14, dyn_oclk_vld13, dyn_oclk_vld12,
   dyn_oclk_vld11, dyn_oclk_vld10, dyn_oclk_vld9, dyn_oclk_vld8, dyn_oclk_vld7, dyn_oclk_vld6,
   dyn_oclk_vld5, dyn_oclk_vld4, dyn_oclk_vld3, dyn_oclk_vld2, dyn_oclk_vld1, dyn_oclk_vld0,

   static_oclk, reg_oclk, reg_oclk_vld,

   p1_mem_power_ctrl17, p1_mem_power_ctrl16, p1_mem_power_ctrl15, p1_mem_power_ctrl14, p1_mem_power_ctrl13, p1_mem_power_ctrl12,
   p1_mem_power_ctrl11, p1_mem_power_ctrl10, p1_mem_power_ctrl9, p1_mem_power_ctrl8, p1_mem_power_ctrl7, p1_mem_power_ctrl6,
   p1_mem_power_ctrl5, p1_mem_power_ctrl4, p1_mem_power_ctrl3, p1_mem_power_ctrl2, p1_mem_power_ctrl1, p1_mem_power_ctrl0,

   // Inputs
   Cpl_GFXCLK, dft_clk_gate_ctrl, dft_shift_clk,
   resetb, iclk, cgcg_en, cg_override, 
   mgls_override, mgls_current_frequency_code,

   core_busy17, core_busy16, core_busy15, core_busy14, core_busy13, core_busy12,
   core_busy11, core_busy10, core_busy9, core_busy8, core_busy7, core_busy6,
   core_busy5, core_busy4, core_busy3, core_busy2, core_busy1, core_busy0,

   reg_busy, stall,
   TTOP_P1_mem_power_ctrl,
   CGTT_BLK_CTRL_REG
   );

//////////////////////////
//in/outs
//////////////////////////
 input       resetb;
 input       iclk;
 input       cgcg_en;
 input       cg_override; /*functional or testmode override. testmode override is incorporated in cg*/
 input        Cpl_GFXCLK;
 input  [1:0] dft_clk_gate_ctrl;
 input        dft_shift_clk;
        /*it is an async signal*/

 output      dyn_oclk17, dyn_oclk16, dyn_oclk15, dyn_oclk14, dyn_oclk13, dyn_oclk12;
 output      dyn_oclk11, dyn_oclk10, dyn_oclk9, dyn_oclk8, dyn_oclk7, dyn_oclk6;
 output      dyn_oclk5, dyn_oclk4, dyn_oclk3, dyn_oclk2, dyn_oclk1, dyn_oclk0;

 output      dyn_oclk_vld17, dyn_oclk_vld16, dyn_oclk_vld15, dyn_oclk_vld14, dyn_oclk_vld13, dyn_oclk_vld12;
 output      dyn_oclk_vld11, dyn_oclk_vld10, dyn_oclk_vld9, dyn_oclk_vld8, dyn_oclk_vld7, dyn_oclk_vld6;
 output      dyn_oclk_vld5, dyn_oclk_vld4, dyn_oclk_vld3, dyn_oclk_vld2, dyn_oclk_vld1, dyn_oclk_vld0;

 input       core_busy17, core_busy16, core_busy15, core_busy14, core_busy13, core_busy12;
 input       core_busy11, core_busy10, core_busy9, core_busy8, core_busy7, core_busy6;
 input       core_busy5, core_busy4, core_busy3, core_busy2, core_busy1, core_busy0;

 input       stall;

 output [`MEM__MEM_POWER_CTRL -1 : 0] p1_mem_power_ctrl17, p1_mem_power_ctrl16, p1_mem_power_ctrl15, p1_mem_power_ctrl14, p1_mem_power_ctrl13, p1_mem_power_ctrl12;
 output [`MEM__MEM_POWER_CTRL -1 : 0] p1_mem_power_ctrl11, p1_mem_power_ctrl10, p1_mem_power_ctrl9, p1_mem_power_ctrl8, p1_mem_power_ctrl7, p1_mem_power_ctrl6;
 output [`MEM__MEM_POWER_CTRL -1 : 0] p1_mem_power_ctrl5, p1_mem_power_ctrl4, p1_mem_power_ctrl3, p1_mem_power_ctrl2, p1_mem_power_ctrl1, p1_mem_power_ctrl0;

 output      static_oclk;
 output      reg_oclk;
 output      reg_oclk_vld;
 input       reg_busy;

 input        mgls_override;
 input [3:0]  mgls_current_frequency_code;
 input [31:0] CGTT_BLK_CTRL_REG;
 input [`MEM__MEM_POWER_CTRL -1 : 0] TTOP_P1_mem_power_ctrl;

parameter switch_code = 0;
parameter enable_dftcgmg_gate_merge = 0;

/////////////////////////////////////////////////////
// Medium Grain Light Sleep control
/////////////////////////////////////////////////////
reg ls_override;
reg override_s;

always @(posedge static_oclk) begin
  if(~resetb) begin
    ls_override <= 1'b1;
    override_s  <= 1'b1;
  end else begin
    ls_override <= mgls_override || (mgls_current_frequency_code > CGTT_BLK_CTRL_REG[3:0]); // if current frequency code is greater than ON_DELAY, the disable medium grain Light Sleep
    override_s  <= cg_override;
  end
end


wire mgls17, mgls16, mgls15, mgls14, mgls13, mgls12;
wire mgls11, mgls10, mgls9, mgls8, mgls7, mgls6;
wire mgls5, mgls4, mgls3, mgls2, mgls1, mgls0;

reg [`MEM__MEM_POWER_CTRL -1 : 0] p1_mem_power_ctrl17, p1_mem_power_ctrl16, p1_mem_power_ctrl15, p1_mem_power_ctrl14, p1_mem_power_ctrl13, p1_mem_power_ctrl12;
reg [`MEM__MEM_POWER_CTRL -1 : 0] p1_mem_power_ctrl11, p1_mem_power_ctrl10, p1_mem_power_ctrl9, p1_mem_power_ctrl8, p1_mem_power_ctrl7, p1_mem_power_ctrl6;
reg [`MEM__MEM_POWER_CTRL -1 : 0] p1_mem_power_ctrl5, p1_mem_power_ctrl4, p1_mem_power_ctrl3, p1_mem_power_ctrl2, p1_mem_power_ctrl1, p1_mem_power_ctrl0;

always @(*) begin
   p1_mem_power_ctrl0 = TTOP_P1_mem_power_ctrl;
   p1_mem_power_ctrl1 = TTOP_P1_mem_power_ctrl;
   p1_mem_power_ctrl2 = TTOP_P1_mem_power_ctrl;
   p1_mem_power_ctrl3 = TTOP_P1_mem_power_ctrl;
   p1_mem_power_ctrl4 = TTOP_P1_mem_power_ctrl;
   p1_mem_power_ctrl5 = TTOP_P1_mem_power_ctrl;
   p1_mem_power_ctrl6 = TTOP_P1_mem_power_ctrl;
   p1_mem_power_ctrl7 = TTOP_P1_mem_power_ctrl;
   p1_mem_power_ctrl8 = TTOP_P1_mem_power_ctrl;
   p1_mem_power_ctrl9 = TTOP_P1_mem_power_ctrl;
   p1_mem_power_ctrl10 = TTOP_P1_mem_power_ctrl;
   p1_mem_power_ctrl11 = TTOP_P1_mem_power_ctrl;
   p1_mem_power_ctrl12 = TTOP_P1_mem_power_ctrl;
   p1_mem_power_ctrl13 = TTOP_P1_mem_power_ctrl;
   p1_mem_power_ctrl14 = TTOP_P1_mem_power_ctrl;
   p1_mem_power_ctrl15 = TTOP_P1_mem_power_ctrl;
   p1_mem_power_ctrl16 = TTOP_P1_mem_power_ctrl;
   p1_mem_power_ctrl17 = TTOP_P1_mem_power_ctrl;

   p1_mem_power_ctrl0[`MEM__MEM_POWER_CTRL_LS] = mgls0 | TTOP_P1_mem_power_ctrl[`MEM__MEM_POWER_CTRL_LS];
   p1_mem_power_ctrl1[`MEM__MEM_POWER_CTRL_LS] = mgls1 | TTOP_P1_mem_power_ctrl[`MEM__MEM_POWER_CTRL_LS];
   p1_mem_power_ctrl2[`MEM__MEM_POWER_CTRL_LS] = mgls2 | TTOP_P1_mem_power_ctrl[`MEM__MEM_POWER_CTRL_LS];
   p1_mem_power_ctrl3[`MEM__MEM_POWER_CTRL_LS] = mgls3 | TTOP_P1_mem_power_ctrl[`MEM__MEM_POWER_CTRL_LS];
   p1_mem_power_ctrl4[`MEM__MEM_POWER_CTRL_LS] = mgls4 | TTOP_P1_mem_power_ctrl[`MEM__MEM_POWER_CTRL_LS];
   p1_mem_power_ctrl5[`MEM__MEM_POWER_CTRL_LS] = mgls5 | TTOP_P1_mem_power_ctrl[`MEM__MEM_POWER_CTRL_LS];
   p1_mem_power_ctrl6[`MEM__MEM_POWER_CTRL_LS] = mgls6 | TTOP_P1_mem_power_ctrl[`MEM__MEM_POWER_CTRL_LS];
   p1_mem_power_ctrl7[`MEM__MEM_POWER_CTRL_LS] = mgls7 | TTOP_P1_mem_power_ctrl[`MEM__MEM_POWER_CTRL_LS];
   p1_mem_power_ctrl8[`MEM__MEM_POWER_CTRL_LS] = mgls8 | TTOP_P1_mem_power_ctrl[`MEM__MEM_POWER_CTRL_LS];
   p1_mem_power_ctrl9[`MEM__MEM_POWER_CTRL_LS] = mgls9 | TTOP_P1_mem_power_ctrl[`MEM__MEM_POWER_CTRL_LS];
   p1_mem_power_ctrl10[`MEM__MEM_POWER_CTRL_LS] = mgls10 | TTOP_P1_mem_power_ctrl[`MEM__MEM_POWER_CTRL_LS];
   p1_mem_power_ctrl11[`MEM__MEM_POWER_CTRL_LS] = mgls11 | TTOP_P1_mem_power_ctrl[`MEM__MEM_POWER_CTRL_LS];
   p1_mem_power_ctrl12[`MEM__MEM_POWER_CTRL_LS] = mgls12 | TTOP_P1_mem_power_ctrl[`MEM__MEM_POWER_CTRL_LS];
   p1_mem_power_ctrl13[`MEM__MEM_POWER_CTRL_LS] = mgls13 | TTOP_P1_mem_power_ctrl[`MEM__MEM_POWER_CTRL_LS];
   p1_mem_power_ctrl14[`MEM__MEM_POWER_CTRL_LS] = mgls14 | TTOP_P1_mem_power_ctrl[`MEM__MEM_POWER_CTRL_LS];
   p1_mem_power_ctrl15[`MEM__MEM_POWER_CTRL_LS] = mgls15 | TTOP_P1_mem_power_ctrl[`MEM__MEM_POWER_CTRL_LS];
   p1_mem_power_ctrl16[`MEM__MEM_POWER_CTRL_LS] = mgls16 | TTOP_P1_mem_power_ctrl[`MEM__MEM_POWER_CTRL_LS];
   p1_mem_power_ctrl17[`MEM__MEM_POWER_CTRL_LS] = mgls17 | TTOP_P1_mem_power_ctrl[`MEM__MEM_POWER_CTRL_LS];
end


/////////////////////////////////////////////////////
//instantiate base module
/////////////////////////////////////////////////////

wire        sclk;
reg         n0scan_static_oclk_en;

`ifdef GC__CGCG_MGCG_MERGE__1
  wire dft_te_wire;
ati_TE_driver u_TE_driver
  (
   .TEST_CLK_EN(dft_shift_clk),
   .DFT_TE(dft_te_wire)
  );

reg                     n0scan_cgcg_en_d1;
always @(posedge sclk) begin
  n0scan_cgcg_en_d1 <= cgcg_en;
end

generate
  if(enable_dftcgmg_gate_merge == 1)
ati_clock_gate u_static_oclk(
    .clk_out (static_oclk),
    .clk_src (sclk),
    .clk_en  (n0scan_static_oclk_en),
    .TE      (dft_te_wire)
   );
  else
ati_clock_gate u_static_oclk(
    .clk_out (static_oclk),
    .clk_src (sclk),
    .clk_en  (n0scan_cgcg_en_d1),
    .TE      (dft_te_wire)
   );
endgenerate

`else
aticlk_buf u_static_clk(.Y(static_oclk), .A(iclk));
`endif

reg  [1:0]   n0scan_dft_clk_gate_ctrl_d1;
wire [1:0]   wdft_clk_gate_ctrl;

generate
  if(enable_dftcgmg_gate_merge == 1) begin
    always @(posedge Cpl_GFXCLK) begin
      n0scan_dft_clk_gate_ctrl_d1 <= dft_clk_gate_ctrl[1:0];
      n0scan_static_oclk_en       <= (n0scan_cgcg_en_d1 & ~n0scan_dft_clk_gate_ctrl_d1[1]) | n0scan_dft_clk_gate_ctrl_d1[0];
    end
    assign sclk = Cpl_GFXCLK;
    assign wdft_clk_gate_ctrl= n0scan_dft_clk_gate_ctrl_d1;
  end
  else begin
    assign sclk = iclk;
    assign wdft_clk_gate_ctrl= 2'b00;
  end
endgenerate

parameter scan_grp_dyn_oclk17 = 0; //defaulted

gfx_cgtt_local_dcg #(
          .scan_grp_dyn_oclk (scan_grp_dyn_oclk17))

        u_cgtt_dclk17
  (.dyn_oclk      (dyn_oclk17),
   .dyn_oclk_vld      (dyn_oclk_vld17),
   .core_busy     (core_busy17),
   .soft_on_delay     (CGTT_BLK_CTRL_REG[3:0]),
   .soft_off_delay    (CGTT_BLK_CTRL_REG[11:4]),
   .soft_override     (CGTT_BLK_CTRL_REG[12]),
   .soft_stall_override     (CGTT_BLK_CTRL_REG[22]),
   .stall             (stall),
   /*AUTOINST*/
   // Inputs
   .resetb      (resetb),
   .iclk        (sclk),
   .cgcg_en         (cgcg_en),
   .dft_clk_gate_ctrl (wdft_clk_gate_ctrl),
   .dft_shift_clk     (dft_shift_clk),
   .cg_override     (override_s),
   .ls_override     (ls_override),
   .mgls            (mgls17),
   .static_oclk     (static_oclk));

parameter scan_grp_dyn_oclk16 = 0; //defaulted

gfx_cgtt_local_dcg #(
          .scan_grp_dyn_oclk (scan_grp_dyn_oclk16))

        u_cgtt_dclk16
  (.dyn_oclk      (dyn_oclk16),
   .dyn_oclk_vld      (dyn_oclk_vld16),
   .core_busy     (core_busy16),
   .soft_on_delay     (CGTT_BLK_CTRL_REG[3:0]),
   .soft_off_delay    (CGTT_BLK_CTRL_REG[11:4]),
   .soft_override     (CGTT_BLK_CTRL_REG[13]),
   .soft_stall_override     (CGTT_BLK_CTRL_REG[22]),
   .stall             (stall),
   /*AUTOINST*/
   // Inputs
   .resetb      (resetb),
   .iclk        (sclk),
   .cgcg_en         (cgcg_en),
   .dft_clk_gate_ctrl (wdft_clk_gate_ctrl),
   .dft_shift_clk     (dft_shift_clk),
   .cg_override     (override_s),
   .ls_override     (ls_override),
   .mgls            (mgls16),
   .static_oclk     (static_oclk));

parameter scan_grp_dyn_oclk15 = 0; //defaulted

gfx_cgtt_local_dcg #(
          .scan_grp_dyn_oclk (scan_grp_dyn_oclk15))

        u_cgtt_dclk15
  (.dyn_oclk      (dyn_oclk15),
   .dyn_oclk_vld      (dyn_oclk_vld15),
   .core_busy     (core_busy15),
   .soft_on_delay     (CGTT_BLK_CTRL_REG[3:0]),
   .soft_off_delay    (CGTT_BLK_CTRL_REG[11:4]),
   .soft_override     (CGTT_BLK_CTRL_REG[14]),
   .soft_stall_override     (CGTT_BLK_CTRL_REG[22]),
   .stall             (stall),
   /*AUTOINST*/
   // Inputs
   .resetb      (resetb),
   .iclk        (sclk),
   .cgcg_en         (cgcg_en),
   .dft_clk_gate_ctrl (wdft_clk_gate_ctrl),
   .dft_shift_clk     (dft_shift_clk),
   .cg_override     (override_s),
   .ls_override     (ls_override),
   .mgls            (mgls15),
   .static_oclk     (static_oclk));

parameter scan_grp_dyn_oclk14 = 0; //defaulted

gfx_cgtt_local_dcg #(
          .scan_grp_dyn_oclk (scan_grp_dyn_oclk14))

        u_cgtt_dclk14
  (.dyn_oclk      (dyn_oclk14),
   .dyn_oclk_vld      (dyn_oclk_vld14),
   .core_busy     (core_busy14),
   .soft_on_delay     (CGTT_BLK_CTRL_REG[3:0]),
   .soft_off_delay    (CGTT_BLK_CTRL_REG[11:4]),
   .soft_override     (CGTT_BLK_CTRL_REG[15]),
   .soft_stall_override     (CGTT_BLK_CTRL_REG[22]),
   .stall             (stall),
   /*AUTOINST*/
   // Inputs
   .resetb      (resetb),
   .iclk        (sclk),
   .cgcg_en         (cgcg_en),
   .dft_clk_gate_ctrl (wdft_clk_gate_ctrl),
   .dft_shift_clk     (dft_shift_clk),
   .cg_override     (override_s),
   .ls_override     (ls_override),
   .mgls            (mgls14),
   .static_oclk     (static_oclk));

parameter scan_grp_dyn_oclk13 = 0; //defaulted

gfx_cgtt_local_dcg #(
          .scan_grp_dyn_oclk (scan_grp_dyn_oclk13))

        u_cgtt_dclk13
  (.dyn_oclk      (dyn_oclk13),
   .dyn_oclk_vld      (dyn_oclk_vld13),
   .core_busy     (core_busy13),
   .soft_on_delay     (CGTT_BLK_CTRL_REG[3:0]),
   .soft_off_delay    (CGTT_BLK_CTRL_REG[11:4]),
   .soft_override     (CGTT_BLK_CTRL_REG[16]),
   .soft_stall_override     (CGTT_BLK_CTRL_REG[22]),
   .stall             (stall),
   /*AUTOINST*/
   // Inputs
   .resetb      (resetb),
   .iclk        (sclk),
   .cgcg_en         (cgcg_en),
   .dft_clk_gate_ctrl (wdft_clk_gate_ctrl),
   .dft_shift_clk     (dft_shift_clk),
   .cg_override     (override_s),
   .ls_override     (ls_override),
   .mgls            (mgls13),
   .static_oclk     (static_oclk));

parameter scan_grp_dyn_oclk12 = 0; //defaulted

gfx_cgtt_local_dcg #(
          .scan_grp_dyn_oclk (scan_grp_dyn_oclk12))

        u_cgtt_dclk12
  (.dyn_oclk      (dyn_oclk12),
   .dyn_oclk_vld      (dyn_oclk_vld12),
   .core_busy     (core_busy12),
   .soft_on_delay     (CGTT_BLK_CTRL_REG[3:0]),
   .soft_off_delay    (CGTT_BLK_CTRL_REG[11:4]),
   .soft_override     (CGTT_BLK_CTRL_REG[17]),
   .soft_stall_override     (CGTT_BLK_CTRL_REG[22]),
   .stall             (stall),
   /*AUTOINST*/
   // Inputs
   .resetb      (resetb),
   .iclk        (sclk),
   .cgcg_en         (cgcg_en),
   .dft_clk_gate_ctrl (wdft_clk_gate_ctrl),
   .dft_shift_clk     (dft_shift_clk),
   .cg_override     (override_s),
   .ls_override     (ls_override),
   .mgls            (mgls12),
   .static_oclk     (static_oclk));

parameter scan_grp_dyn_oclk11 = 0; //defaulted

gfx_cgtt_local_dcg #(
          .scan_grp_dyn_oclk (scan_grp_dyn_oclk11))

        u_cgtt_dclk11
  (.dyn_oclk      (dyn_oclk11),
   .dyn_oclk_vld      (dyn_oclk_vld11),
   .core_busy     (core_busy11),
   .soft_on_delay     (CGTT_BLK_CTRL_REG[3:0]),
   .soft_off_delay    (CGTT_BLK_CTRL_REG[11:4]),
   .soft_override     (CGTT_BLK_CTRL_REG[18]),
   .soft_stall_override     (CGTT_BLK_CTRL_REG[22]),
   .stall             (stall),
   /*AUTOINST*/
   // Inputs
   .resetb      (resetb),
   .iclk        (sclk),
   .cgcg_en         (cgcg_en),
   .dft_clk_gate_ctrl (wdft_clk_gate_ctrl),
   .dft_shift_clk     (dft_shift_clk),
   .cg_override     (override_s),
   .ls_override     (ls_override),
   .mgls            (mgls11),
   .static_oclk     (static_oclk));

parameter scan_grp_dyn_oclk10 = 0; //defaulted

gfx_cgtt_local_dcg #(
          .scan_grp_dyn_oclk (scan_grp_dyn_oclk10))

        u_cgtt_dclk10
  (.dyn_oclk      (dyn_oclk10),
   .dyn_oclk_vld      (dyn_oclk_vld10),
   .core_busy     (core_busy10),
   .soft_on_delay     (CGTT_BLK_CTRL_REG[3:0]),
   .soft_off_delay    (CGTT_BLK_CTRL_REG[11:4]),
   .soft_override     (CGTT_BLK_CTRL_REG[19]),
   .soft_stall_override     (CGTT_BLK_CTRL_REG[22]),
   .stall             (stall),
   /*AUTOINST*/
   // Inputs
   .resetb      (resetb),
   .iclk        (sclk),
   .cgcg_en         (cgcg_en),
   .dft_clk_gate_ctrl (wdft_clk_gate_ctrl),
   .dft_shift_clk     (dft_shift_clk),
   .cg_override     (override_s),
   .ls_override     (ls_override),
   .mgls            (mgls10),
   .static_oclk     (static_oclk));

parameter scan_grp_dyn_oclk9 = 0; //defaulted

gfx_cgtt_local_dcg #(
          .scan_grp_dyn_oclk (scan_grp_dyn_oclk9))

        u_cgtt_dclk9
  (.dyn_oclk      (dyn_oclk9),
   .dyn_oclk_vld      (dyn_oclk_vld9),
   .core_busy     (core_busy9),
   .soft_on_delay     (CGTT_BLK_CTRL_REG[3:0]),
   .soft_off_delay    (CGTT_BLK_CTRL_REG[11:4]),
   .soft_override     (CGTT_BLK_CTRL_REG[20]),
   .soft_stall_override     (CGTT_BLK_CTRL_REG[22]),
   .stall             (stall),
   /*AUTOINST*/
   // Inputs
   .resetb      (resetb),
   .iclk        (sclk),
   .cgcg_en         (cgcg_en),
   .dft_clk_gate_ctrl (wdft_clk_gate_ctrl),
   .dft_shift_clk     (dft_shift_clk),
   .cg_override     (override_s),
   .ls_override     (ls_override),
   .mgls            (mgls9),
   .static_oclk     (static_oclk));

parameter scan_grp_dyn_oclk8 = 0; //defaulted

gfx_cgtt_local_dcg #(
          .scan_grp_dyn_oclk (scan_grp_dyn_oclk8))

        u_cgtt_dclk8
  (.dyn_oclk      (dyn_oclk8),
   .dyn_oclk_vld      (dyn_oclk_vld8),
   .core_busy     (core_busy8),
   .soft_on_delay     (CGTT_BLK_CTRL_REG[3:0]),
   .soft_off_delay    (CGTT_BLK_CTRL_REG[11:4]),
   .soft_override     (CGTT_BLK_CTRL_REG[21]),
   .soft_stall_override     (CGTT_BLK_CTRL_REG[22]),
   .stall             (stall),
   /*AUTOINST*/
   // Inputs
   .resetb      (resetb),
   .iclk        (sclk),
   .cgcg_en         (cgcg_en),
   .dft_clk_gate_ctrl (wdft_clk_gate_ctrl),
   .dft_shift_clk     (dft_shift_clk),
   .cg_override     (override_s),
   .ls_override     (ls_override),
   .mgls            (mgls8),
   .static_oclk     (static_oclk));

parameter scan_grp_dyn_oclk7 = 0; //defaulted

gfx_cgtt_local_dcg #(
          .scan_grp_dyn_oclk (scan_grp_dyn_oclk7))

          u_cgtt_dclk7
  (.dyn_oclk      (dyn_oclk7),
   .dyn_oclk_vld      (dyn_oclk_vld7),
   .core_busy     (core_busy7),
   .soft_on_delay     (CGTT_BLK_CTRL_REG[3:0]),
   .soft_off_delay    (CGTT_BLK_CTRL_REG[11:4]),
   .soft_override     (CGTT_BLK_CTRL_REG[23]),
   .soft_stall_override     (CGTT_BLK_CTRL_REG[22]),
   .stall             (stall),
   /*AUTOINST*/
   // Inputs
   .resetb      (resetb),
   .iclk        (sclk),
   .cgcg_en         (cgcg_en),
   .dft_clk_gate_ctrl (wdft_clk_gate_ctrl),
   .dft_shift_clk     (dft_shift_clk),
   .cg_override     (override_s),
   .ls_override     (ls_override),
   .mgls            (mgls7),
   .static_oclk     (static_oclk));

parameter scan_grp_dyn_oclk6 = 0; //defaulted

gfx_cgtt_local_dcg #(
          .scan_grp_dyn_oclk (scan_grp_dyn_oclk6))

        u_cgtt_dclk6
  (.dyn_oclk      (dyn_oclk6),
   .dyn_oclk_vld      (dyn_oclk_vld6),
   .core_busy     (core_busy6),
   .soft_on_delay     (CGTT_BLK_CTRL_REG[3:0]),
   .soft_off_delay    (CGTT_BLK_CTRL_REG[11:4]),
   .soft_override     (CGTT_BLK_CTRL_REG[24]),
   .soft_stall_override     (CGTT_BLK_CTRL_REG[22]),
   .stall             (stall),
   /*AUTOINST*/
   // Inputs
   .resetb      (resetb),
   .iclk        (sclk),
   .cgcg_en         (cgcg_en),
   .dft_clk_gate_ctrl (wdft_clk_gate_ctrl),
   .dft_shift_clk     (dft_shift_clk),
   .cg_override     (override_s),
   .ls_override     (ls_override),
   .mgls            (mgls6),
   .static_oclk     (static_oclk));

parameter scan_grp_dyn_oclk5 = 0; //defaulted

gfx_cgtt_local_dcg #(
          .scan_grp_dyn_oclk (scan_grp_dyn_oclk5))

        u_cgtt_dclk5
  (.dyn_oclk      (dyn_oclk5),
   .dyn_oclk_vld      (dyn_oclk_vld5),
   .core_busy     (core_busy5),
   .soft_on_delay     (CGTT_BLK_CTRL_REG[3:0]),
   .soft_off_delay    (CGTT_BLK_CTRL_REG[11:4]),
   .soft_override     (CGTT_BLK_CTRL_REG[25]),
   .soft_stall_override     (CGTT_BLK_CTRL_REG[22]),
   .stall             (stall),
   /*AUTOINST*/
   // Inputs
   .resetb      (resetb),
   .iclk        (sclk),
   .cgcg_en         (cgcg_en),
   .dft_clk_gate_ctrl (wdft_clk_gate_ctrl),
   .dft_shift_clk     (dft_shift_clk),
   .cg_override     (override_s),
   .ls_override     (ls_override),
   .mgls            (mgls5),
   .static_oclk     (static_oclk));

parameter scan_grp_dyn_oclk4 = 0; //defaulted

gfx_cgtt_local_dcg #(
          .scan_grp_dyn_oclk (scan_grp_dyn_oclk4))

        u_cgtt_dclk4
  (.dyn_oclk      (dyn_oclk4),
   .dyn_oclk_vld      (dyn_oclk_vld4),
   .core_busy     (core_busy4),
   .soft_on_delay     (CGTT_BLK_CTRL_REG[3:0]),
   .soft_off_delay    (CGTT_BLK_CTRL_REG[11:4]),
   .soft_override     (CGTT_BLK_CTRL_REG[26]),
   .soft_stall_override     (CGTT_BLK_CTRL_REG[22]),
   .stall             (stall),
   /*AUTOINST*/
   // Inputs
   .resetb      (resetb),
   .iclk        (sclk),
   .cgcg_en         (cgcg_en),
   .dft_clk_gate_ctrl (wdft_clk_gate_ctrl),
   .dft_shift_clk     (dft_shift_clk),
   .cg_override     (override_s),
   .ls_override     (ls_override),
   .mgls            (mgls4),
   .static_oclk     (static_oclk));

parameter scan_grp_dyn_oclk3 = 0; //defaulted

gfx_cgtt_local_dcg #(
          .scan_grp_dyn_oclk (scan_grp_dyn_oclk3))

        u_cgtt_dclk3
  (.dyn_oclk      (dyn_oclk3),
   .dyn_oclk_vld      (dyn_oclk_vld3),
   .core_busy     (core_busy3),
   .soft_on_delay     (CGTT_BLK_CTRL_REG[3:0]),
   .soft_off_delay    (CGTT_BLK_CTRL_REG[11:4]),
   .soft_override     (CGTT_BLK_CTRL_REG[27]),
   .soft_stall_override     (CGTT_BLK_CTRL_REG[22]),
   .stall             (stall),
   /*AUTOINST*/
   // Inputs
   .resetb      (resetb),
   .iclk        (sclk),
   .cgcg_en         (cgcg_en),
   .dft_clk_gate_ctrl (wdft_clk_gate_ctrl),
   .dft_shift_clk     (dft_shift_clk),
   .cg_override     (override_s),
   .ls_override     (ls_override),
   .mgls            (mgls3),
   .static_oclk     (static_oclk));

parameter scan_grp_dyn_oclk2 = 0; //defaulted

gfx_cgtt_local_dcg #(
          .scan_grp_dyn_oclk (scan_grp_dyn_oclk2))

        u_cgtt_dclk2
  (.dyn_oclk      (dyn_oclk2),
   .dyn_oclk_vld      (dyn_oclk_vld2),
   .core_busy     (core_busy2),
   .soft_on_delay     (CGTT_BLK_CTRL_REG[3:0]),
   .soft_off_delay    (CGTT_BLK_CTRL_REG[11:4]),
   .soft_override     (CGTT_BLK_CTRL_REG[28]),
   .soft_stall_override     (CGTT_BLK_CTRL_REG[22]),
   .stall             (stall),
   /*AUTOINST*/
   // Inputs
   .resetb      (resetb),
   .iclk        (sclk),
   .cgcg_en         (cgcg_en),
   .dft_clk_gate_ctrl (wdft_clk_gate_ctrl),
   .dft_shift_clk     (dft_shift_clk),
   .cg_override     (override_s),
   .ls_override     (ls_override),
   .mgls            (mgls2),
   .static_oclk     (static_oclk));

parameter scan_grp_dyn_oclk1 = 0; //defaulted

gfx_cgtt_local_dcg #(
          .scan_grp_dyn_oclk (scan_grp_dyn_oclk1))

        u_cgtt_dclk1
  (.dyn_oclk      (dyn_oclk1),
   .dyn_oclk_vld      (dyn_oclk_vld1),
   .core_busy     (core_busy1),
   .soft_on_delay     (CGTT_BLK_CTRL_REG[3:0]),
   .soft_off_delay    (CGTT_BLK_CTRL_REG[11:4]),
   .soft_override     (CGTT_BLK_CTRL_REG[29]),
   .soft_stall_override     (CGTT_BLK_CTRL_REG[22]),
   .stall             (stall),
   /*AUTOINST*/
   // Inputs
   .resetb      (resetb),
   .iclk        (sclk),
   .cgcg_en         (cgcg_en),
   .dft_clk_gate_ctrl (wdft_clk_gate_ctrl),
   .dft_shift_clk     (dft_shift_clk),
   .cg_override     (override_s),
   .ls_override     (ls_override),
   .mgls            (mgls1),
   .static_oclk     (static_oclk));

parameter scan_grp_dyn_oclk0 = 0; //defaulted

gfx_cgtt_local_dcg #(
          .scan_grp_dyn_oclk (scan_grp_dyn_oclk0))

        u_cgtt_dclk0
  (.dyn_oclk      (dyn_oclk0),
   .dyn_oclk_vld      (dyn_oclk_vld0),
   .core_busy     (core_busy0),
   .soft_on_delay     (CGTT_BLK_CTRL_REG[3:0]),
   .soft_off_delay    (CGTT_BLK_CTRL_REG[11:4]),
   .soft_override     (CGTT_BLK_CTRL_REG[30]),
   .soft_stall_override     (CGTT_BLK_CTRL_REG[22]),
   .stall             (stall),
   /*AUTOINST*/
   // Inputs
   .resetb      (resetb),
   .iclk        (sclk),
   .cgcg_en         (cgcg_en),
   .dft_clk_gate_ctrl (wdft_clk_gate_ctrl),
   .dft_shift_clk     (dft_shift_clk),
   .cg_override     (override_s),
   .ls_override     (ls_override),
   .mgls            (mgls0),
   .static_oclk     (static_oclk));

parameter scan_grp_reg_oclk = 0; //defaulted

parameter enable_rcg = 1; // defaulted

generate
if(enable_rcg == 1)
begin : reg_oclk_gen
gfx_cgtt_local_rcg #(
          .scan_grp_reg_oclk (scan_grp_reg_oclk))

        u_cgtt_rclk
  (.soft_override   (CGTT_BLK_CTRL_REG[31]),
   /*AUTOINST*/
   // Outputs
   .reg_oclk      (reg_oclk),
   .reg_oclk_vld      (reg_oclk_vld),
   // Inputs
   .resetb      (resetb),
   .iclk        (sclk),
   .cgcg_en         (cgcg_en),
   .dft_clk_gate_ctrl (wdft_clk_gate_ctrl),
   .dft_shift_clk     (dft_shift_clk),
   .cg_override     (override_s),
   .reg_busy      (reg_busy),
   .static_oclk     (static_oclk));
end
else
begin
  assign reg_oclk     = 1'b0;
  assign reg_oclk_vld = 1'b0;
end
endgenerate

endmodule
//////////////////////////////////////////////////////////////////////////////////////////////////////////
