`include "msgio_defines.vh"
`include "gc_features.vh"
`include "env_features.vh"

module gfx_cgtt_local_dcg_cov(

   dyn_oclk, dyn_oclk_vld,
   resetb, iclk, cgcg_en, cg_override, core_busy,
   soft_override, soft_stall_override, static_oclk,soft_off_delay,soft_on_delay,
   mgls
   );
// vrq translate_off

//////////////////////////
 input          resetb;
 input          iclk;
 input			cgcg_en;
 input          cg_override;
 input          core_busy;
 input          soft_override;
 input          soft_stall_override;
 input          static_oclk;
 input [11:4]	soft_off_delay;
 input  [3:0]	soft_on_delay;

 input          dyn_oclk;
 input          dyn_oclk_vld;

 input          mgls;  // medium grain light sleep
//////////////////////////

`ifdef VCS
//VCS coverage off


//initial begin
//  `MSGIO_DISPLAY("%m at %0t: src/rtl/shared/gfx_cgtt_local_dcg_cov.sv: initial with display", $realtime);
//  `MSGIO_DISPLAY("%m src/rtl/shared/gfx_cgtt_local_dcg_cov.sv: initial with MSGIO_DISPLAY");
//end

reg enable;

initial begin
  if($test$plusargs("cgt_assert_disable"))
    enable = 1'b0;
  else
    enable = 1'b1;
end

reg  mgls_checker_disable;
initial begin
  if($test$plusargs("mgls_checker_disable"))
    mgls_checker_disable = 1'b1;
  else
    mgls_checker_disable = 1'b0;
end

reg     mgls_cgcgchecker_disable;
reg     sq_mgls_checker_disable;
string  BlkHier;
initial begin
  mgls_cgcgchecker_disable = 1'b0;
  sq_mgls_checker_disable = 1'b0;
  $sformat(BlkHier, "%m");
  for(integer i=0;i<BlkHier.len();i++) begin 
     if ((BlkHier.substr(i,i+2)=="sct") || (BlkHier.substr(i,i+4)=="atcl2") || (BlkHier.substr(i,i+2)=="cha") || (BlkHier.substr(i,i+3)=="sdma")) begin
       mgls_cgcgchecker_disable = 1'b1;
       break;
     end
     else if(BlkHier.substr(i,i+6)=="sq_comm") begin
       sq_mgls_checker_disable = 1'b1;
       break;
     end
  end
end

reg   cgcg_en_d1;
always @(posedge iclk) begin
    cgcg_en_d1 <= cgcg_en;
  end

// CGCG clock on, and not in ramp up/down stage
wire cgcg_on = cgcg_en_d1 && cgcg_en;
reg   cgcg_on_d1;
reg   cgcg_on_d2;
reg   cgcg_on_d3;
always @(posedge iclk) begin
    cgcg_on_d1 <= cgcg_on;
    cgcg_on_d2 <= cgcg_on_d1;
    cgcg_on_d3 <= cgcg_on_d2;
  end

wire MGCG_AS_DISABLE = (~resetb | ~enable) || (resetb === 1'bx) ;


iclk_inactive									: assert property (@(negedge iclk) disable iff(MGCG_AS_DISABLE) (iclk !== 1'bx));

// kshao begin: disable this assertion till dft/cg/mg gater merge feature is done.
//static_oclk_inactive                            : assert property (@(negedge iclk) disable iff(MGCG_AS_DISABLE) (cgcg_en |=> (static_oclk && (static_oclk !== 1'bx))));
//static_oclk_active								: assert property (@(negedge iclk) disable iff(MGCG_AS_DISABLE) (~cgcg_en |=> (static_oclk!==1'b1)));
// end

dyn_oclk_vld_active								: assert property (@(negedge iclk) disable iff(MGCG_AS_DISABLE) (~cgcg_en  |=> (dyn_oclk_vld!==1'b1)));
// Updated due to logic change before MGCG gater
dyn_oclk_active									: assert property (@(negedge iclk) disable iff(MGCG_AS_DISABLE) (~cgcg_en  [*4] |-> (dyn_oclk!==1'b1)));

mgls_dyn_oclk_active1							: assert property (@(negedge iclk) disable iff(MGCG_AS_DISABLE | sq_mgls_checker_disable | mgls_checker_disable) (mgls  |-> (dyn_oclk!==1'b1)));
mgls_dyn_oclk_active2							: assert property (@(negedge iclk) disable iff(MGCG_AS_DISABLE | sq_mgls_checker_disable | mgls_checker_disable) (mgls  ##1 ~mgls |-> (dyn_oclk!==1'b1)));

// mgls must be dis-asserted at least 2 cycles before dyn_oclk enabled
mgls_must_inactive_2cycles_before_dyn_oclk_active  : assert property (@(negedge iclk) disable iff(MGCG_AS_DISABLE | (mgls_cgcgchecker_disable && ~cgcg_on_d3) | sq_mgls_checker_disable | mgls_checker_disable) (mgls  ##1 ~mgls |=> (dyn_oclk!==1'b1)));

cg_override_dyn_oclk_vld_active		: assert property (@(negedge iclk) disable iff(MGCG_AS_DISABLE) ((cg_override && cgcg_en) |=> dyn_oclk_vld));

`ifdef GFX_CG_W2E
reg cgt_en_assert_enable;
reg MGCG_Coverage_enable;
reg CGTT_Coverage_enable;
reg CGTT_ONOFF_Coverage_enable;
reg CGTT_Stall_Coverage_enable;
initial begin
  if($test$plusargs("cgt_en_assert_disable"))
    cgt_en_assert_enable = 1'b0;
  else
    cgt_en_assert_enable = 1'b1;

  if($test$plusargs("MGCG_Coverage_assert_enable"))
    MGCG_Coverage_enable = 1'b1;
  else
    MGCG_Coverage_enable = 1'b0;

  if($test$plusargs("CGTT_Coverage_assert_enable"))
    CGTT_Coverage_enable = 1'b1;
  else
    CGTT_Coverage_enable = 1'b0;

  if($test$plusargs("CGTT_ONOFF_Coverage_assert_enable"))
    CGTT_ONOFF_Coverage_enable = 1'b1;
  else
    CGTT_ONOFF_Coverage_enable = 1'b0;

  if($test$plusargs("CGTT_Stall_Coverage_assert_enable"))
    CGTT_Stall_Coverage_enable = 1'b1;
  else
    CGTT_Stall_Coverage_enable = 1'b0;
	
end

reg hier_enable;
reg stall_hier_enable;
reg cgcgchecker_enable;
reg dump_enable;
string Hier;
initial begin
  hier_enable = 1'b1;
  stall_hier_enable = 1'b0;
  cgcgchecker_enable = 1'b1;
  dump_enable = 1'b1;

  $sformat(Hier, "%m");

  if($test$plusargs({Hier,"__cgt_assert_hier_disable"}))
  begin
    hier_enable = 1'b0;
    `MSGIO_DISPLAY("%m:cg assertion is waived via hierarchy");
  end

  if($test$plusargs({Hier,"__stall_assert_hier_enable"}))
  begin
    stall_hier_enable = 1'b1;
    `MSGIO_DISPLAY("%m: stall coverage check is enabled via hierarchy");
  end

for(integer i=0;i<Hier.len();i++) 
    if( (Hier.substr(i,i+2)=="cpf") || (Hier.substr(i,i+2)=="rlc") || (Hier.substr(i,i+3)=="grbm") || (Hier.substr(i,i+2)=="tpi") )      
	begin 
           cgcgchecker_enable=1'b0; 
           break; 
    end
for(integer i=0;i<Hier.len();i++) 
    `ifdef CORE_TILE
	if( (Hier.substr(i,i+2)=="acv") || (Hier.substr(i,i+4)=="gdfll") || (Hier.substr(i,i+4)=="gc_ge") || (Hier.substr(i,i+8)=="gl1r_cgtt") || (Hier.substr(i,i+4)=="mmhub") || (Hier.substr(i,i+2)=="tpi") || (Hier.substr(i,i+10)=="sqc.p1.cgtt") || (Hier.substr(i,i+19)=="uspi_clkgate_if_cgtt") || (Hier.substr(i,i+8)=="gl1i_cgtt") || (Hier.substr(i,i+4)=="utcl1") || (Hier.substr(i,i+2)=="cpf") || (Hier.substr(i,i+4)=="spi_s") || (Hier.substr(i,i+2)=="spi") || (Hier.substr(i,i+2)=="sqg") || (Hier.substr(i,i+6)=="sq_comm") )      
    `else
	if( (Hier.substr(i,i+2)=="acv") || (Hier.substr(i,i+4)=="gdfll") || (Hier.substr(i,i+4)=="gc.ge") || (Hier.substr(i,i+8)=="gl1r_cgtt") || (Hier.substr(i,i+4)=="mmhub") || (Hier.substr(i,i+8)=="gl1i_cgtt") || (Hier.substr(i,i+4)=="utcl1") || (Hier.substr(i,i+2)=="cpf") || (Hier.substr(i,i+4)=="spi_s") || (Hier.substr(i,i+2)=="spi") || (Hier.substr(i,i+2)=="sqg") || (Hier.substr(i,i+6)=="sq_comm"))      
    `endif
	begin 
           hier_enable=1'b0; 
           break; 
    end
for(integer i=0;i<Hier.len();i++) 
    if( (Hier.substr(i,i+3)=="rspm") )      
	begin 
           hier_enable=1'b0; 
	   cgcgchecker_enable=1'b0; 
           break; 
    end
end

reg[64]    core_busy_count;
reg[64]    dyn_oclk_count;

always @(posedge static_oclk)
begin
  if(~resetb) 
  begin
	core_busy_count <= 0;
  end 
  else
  begin
	if(core_busy)		core_busy_count <= core_busy_count + 1;
  end
end

always @(posedge dyn_oclk)
begin
  if(~resetb) 
  begin
    dyn_oclk_count  <= 0;
  end 
  else
  begin
	dyn_oclk_count  <= dyn_oclk_count + 1;
  end
end
final if((dyn_oclk_count < core_busy_count) && enable && hier_enable) `MSGIO_DISPLAY("%m ERROR: dyn_oclk_count less than core_busy_count");

reg reset_deactivated;
initial reset_deactivated = 0;
always @(posedge iclk) if(resetb) reset_deactivated <= 1;
final if((reset_deactivated == 0) && enable && hier_enable) `MSGIO_DISPLAY("%m ERROR: CLKGATE Reset never deactivated");

reg cgcg_en_deactivated;
initial cgcg_en_deactivated = 0;
always @(posedge iclk) if(cgcg_en == 1'b0) cgcg_en_deactivated <= 1;
final if((cgcg_en_deactivated == 0) && enable && hier_enable && cgcgchecker_enable) `MSGIO_DISPLAY("%m ERROR: CLKGATE cgcg_en never deactivated");

reg override_disabled;
initial override_disabled = 1'b0;
always @(posedge iclk) if(cg_override == 1'b0 ) override_disabled <= 1'b1;
final if((override_disabled == 1'b0) && enable && cgt_en_assert_enable && hier_enable) `MSGIO_DISPLAY("%m ERROR: CLKGATE Master Override never deactivated");

final if((cg_override == 1'b0) && enable && MGCG_Coverage_enable) `MSGIO_DISPLAY("%m ERROR: CLKGATE Master Override never activated");

reg soft_override_disabled;
initial soft_override_disabled = 1'b0;
always @(posedge iclk) if(soft_override == 1'b0) soft_override_disabled <= 1'b1;
final if((soft_override_disabled == 1'b0) && enable && hier_enable) `MSGIO_DISPLAY("%m ERROR: CLKGATE Soft Override never deactivated");

final if((soft_override == 1'b0) && CGTT_Coverage_enable && hier_enable) `MSGIO_DISPLAY("%m ERROR: CLKGATE CGTT Coverage: Soft Override never deactivated");
final if(((soft_on_delay != 4'b0) || (soft_off_delay != 8'h18)) && CGTT_ONOFF_Coverage_enable && hier_enable) `MSGIO_DISPLAY("%m ERROR: CLKGATE CGTT ONOFF Coverage");
final if((soft_stall_override == 1'b0) && CGTT_Stall_Coverage_enable && stall_hier_enable && hier_enable) `MSGIO_DISPLAY("%m ERROR: CLKGATE CGTT Coverage: Stall Soft Override never deactivated");


reg soft_override_d1;
always @(posedge iclk)
begin
soft_override_d1<=soft_override;
end
wire pos_soft_override = ~soft_override_d1 & soft_override;
wire neg_soft_override = ~soft_override & soft_override_d1;
reg [3:0] edge_counter;
always @(posedge iclk) begin
  if(~resetb) begin
    edge_counter <= 'h0;
  end
  else begin
    if(pos_soft_override)
      edge_counter<=edge_counter+1;
    else if(neg_soft_override)
      edge_counter<=edge_counter-1;
  end
end
final if((edge_counter != 1'b0) && enable && hier_enable) `MSGIO_DISPLAY("%m ERROR: CLKGATE Soft Override was activated in the final");

reg vld_deasserted;
initial vld_deasserted = 1'b0;
always @(posedge iclk) if (dyn_oclk_vld == 1'b0) vld_deasserted <= 1'b1;
final if (vld_deasserted == 1'b0 && enable && cgt_en_assert_enable && hier_enable)
	`MSGIO_DISPLAY("%m ERROR:CG_VLD checker:clk never turn off, check clock gating setting");

`ifdef MGCG_CHECKER
reg busy_deasserted;
initial busy_deasserted = 1'b0;
always @(posedge iclk) if (core_busy == 1'b0) busy_deasserted <= 1'b1;
final if (busy_deasserted == 1'b0 && enable && cgt_en_assert_enable && hier_enable)
	`MSGIO_DISPLAY("%m ERROR:MGCG checker: core_busy never deactivated");

reg busy_asserted;
initial busy_asserted = 1'b0;
always @(posedge iclk) if (core_busy == 1'b1) busy_asserted <= 1'b1;
final if (busy_asserted == 1'b0 && enable && cgt_en_assert_enable && hier_enable)
	`MSGIO_DISPLAY("%m ERROR:MGCG checker: core_busy never activated");
`endif  //`ifdef MGCG_CHECKER

//added dis acv assertion
`include "gfx_cgtt_local_acv_cov.vh"
`endif  //`ifdef GFX_CG_W2E

// soft override should be cleared before the end of test
//`ifdef GFX_CG_W2E
//final if((soft_override == 1'b1) && enable && hier_enable && cgt_en_assert_enable) `MSGIO_DISPLAY("%m ERROR: CLKGATE Soft Override is set at the end of test");
//`else
//final if((soft_override == 1'b1) && enable && hier_enable && cgt_en_assert_enable) `MSGIO_DISPLAY("%m WARNING: CLKGATE Soft Override is set at the end of test");
//`endif


`ifdef CGT_FCOV

covergroup cgtt_local_dcg_cg_t @(negedge iclk);

    dyn_oclk_cp : coverpoint dyn_oclk iff(resetb)
    {
        bins num_clocks_dyn_oclk_inactive = {0};
        bins num_clocks_dyn_oclk_active   = {1};
    }

endgroup //cgtt_dcg_local_cg_t

cgtt_local_dcg_cg_t cgtt_local_dcg_cg = new;

`endif // CGT_FCOV

`ifdef GFX_CG_SIGNALS_DUMP_EN
reg resetconfirm;
always @(posedge static_oclk)
begin
	resetconfirm <= resetb;
end
initial
begin
  #1;
`ifdef GC__SOC_ENV__1 
 `ifdef GC__FLAVOR__APU 
  if(dump_enable) begin
    $fmonitor(sys.cg_file_handle, "CG CGTT_DCG_RST:(%t) %m h%x",$time,resetconfirm);
  end
  `else
  if(dump_enable) begin
    $fmonitor(tb.cg_file_handle, "CG CGTT_DCG_RST:(%t) %m h%x",$time,resetconfirm);
  end
  `endif
`else
  if(dump_enable) begin
    $fmonitor(tb.cg_file_handle, "CG CGTT_DCG_RST:(%t) %m h%x",$time,resetconfirm);
  end
`endif
end
`endif

`ifdef CLK_EFF_MONITOR
/////////////////////////////////////////////////////////////////////////
//To provide information about how often a block is turning its clock off
/////////////////////////////////////////////////////////////////////////
real    dyn_oclk_off_count;
real    static_oclk_count;
real    mgls_count;
integer mgls_entries;
reg     last_mgls;

initial
begin
  dyn_oclk_off_count = 0;
  static_oclk_count  = 0;
  mgls_count = 0;
  last_mgls = 0;
  mgls_entries = 0;
end

always @(posedge static_oclk)
begin
  last_mgls <= mgls;
end

always @(posedge static_oclk)
begin
  if(resetb && ~dyn_oclk_vld) begin
    dyn_oclk_off_count = dyn_oclk_off_count + 1;  //don't count when resetb is active
  end
  if(resetb && mgls) begin
    mgls_count = mgls_count + 1;  //don't count when resetb is active
  end
  if(~last_mgls && mgls) begin
    mgls_entries = mgls_entries + 1;
  end
  static_oclk_count = static_oclk_count + 1;
end

final
begin
    `MSGIO_DISPLAY("INFO:  LCG: final dyn-clk off percentage = %4.1f, total clocks = %f; %m", dyn_oclk_off_count / static_oclk_count * 100.0, static_oclk_count);
    `MSGIO_DISPLAY("INFO: MGLS: final light sleep percentage = %4.1f, sleep entries = %0d; %m", mgls_count / static_oclk_count * 100.0, mgls_entries);
end
`endif

`else
initial begin
  $display("%m at %0t: ld/src/misc/cgtt_local_dcg_cov.sv: SVA not yet supported in NCSIM in wekiva scripts", $realtime);
end

//VCS coverage on

`endif


// vrq translate_on

endmodule




