`include "msgio_defines.vh"
`include "gc_features.vh"

module gfx_cgtt_local_rcg_cov(
   // Outputs of cgtt_local_rcg
   reg_oclk, reg_oclk_vld,
   // Inputs to cgtt_local_rcg
   resetb, iclk, cgcg_en ,cg_override, reg_busy, reg_busy_filtered, soft_override, static_oclk
   );
// vrq translate_off

//////////////////////////
 input          resetb;
 input          iclk;
 input			cgcg_en;
 input          cg_override;
 input          reg_busy;
 input			reg_busy_filtered;
 input          soft_override;
 input          static_oclk;

 input          reg_oclk;
 input          reg_oclk_vld;
//////////////////////////

`ifdef VCS
//VCS coverage off

reg enable;

initial begin
  if($test$plusargs("cgt_assert_disable"))
    enable = 1'b0;
  else
    enable = 1'b1;
end

wire MGCG_AS_DISABLE = (~resetb | ~enable) || (resetb === 1'bx) ;

wire seq = ~reg_busy & ~soft_override & ~cg_override;

wire	reg_busyWithCGCG			= reg_busy & cgcg_en;
wire	soft_overrideWithCGCG		= soft_override & cgcg_en;
wire	cg_overrideWithCGCG			= cg_override & cgcg_en;

reg   cgcg_en_d1;
always @(posedge iclk) begin
  cgcg_en_d1 <= cgcg_en;
end

// CGCG clock on, and not in ramp up/down stage
wire cgcg_on = cgcg_en_d1 && cgcg_en;

iclk_inactive									: assert property (@(negedge iclk) disable iff(MGCG_AS_DISABLE) (iclk !== 1'bx));

// kshao begin: disable this assertion till dft/cg/mg gater merge feature is done.
//static_oclk_inactive                            : assert property (@(negedge iclk) disable iff(MGCG_AS_DISABLE) (cgcg_en |=> (static_oclk && (static_oclk !== 1'bx))));
//static_oclk_active								: assert property (@(negedge iclk) disable iff(MGCG_AS_DISABLE) (~cgcg_en |=> (static_oclk!==1'b1)));
// end

reg_oclk_vld_active								: assert property (@(negedge iclk) disable iff(MGCG_AS_DISABLE) (~cgcg_en  |=> (reg_oclk_vld!==1'b1)));
reg_oclk_active									: assert property (@(negedge iclk) disable iff(MGCG_AS_DISABLE) (~cgcg_en [*3] |-> (reg_oclk!==1'b1)));

//kshao: disable these assertions for 3DCG temporarily
//reg_busy_override_inactive_reg_oclk_vld_active	: assert property (@(negedge iclk) disable iff(MGCG_AS_DISABLE) (seq |=> (reg_oclk_vld!==1'b1)));
//reg_busy_override_inactive_reg_oclk_active		: assert property (@(negedge iclk) disable iff(MGCG_AS_DISABLE) (seq ##1 seq |=> (reg_oclk!==1'b1)));

// disable this check when CGCG ramp up/down
reg_busy_active_reg_oclk_vld_inactive			: assert property (@(negedge iclk) disable iff(MGCG_AS_DISABLE || ~cgcg_on) (reg_busyWithCGCG [*5] |-> reg_oclk_vld));
reg_busy_active_reg_oclk_inactive				: assert property (@(negedge iclk) disable iff(MGCG_AS_DISABLE || ~cgcg_on) (reg_busyWithCGCG [*4] |-> reg_oclk));

soft_override_active_reg_oclk_vld_inactive		: assert property (@(negedge iclk) disable iff(MGCG_AS_DISABLE) (soft_overrideWithCGCG [*5] |-> ##1 reg_oclk_vld));
soft_override_active_reg_oclk_inactive			: assert property (@(negedge iclk) disable iff(MGCG_AS_DISABLE) (soft_overrideWithCGCG [*4] |-> reg_oclk));

cg_override_active_reg_oclk_vld_inactive		: assert property (@(negedge iclk) disable iff(MGCG_AS_DISABLE) (cg_overrideWithCGCG [*5] |-> ##1 reg_oclk_vld));
cg_override_active_reg_oclk_inactive			: assert property (@(negedge iclk) disable iff(MGCG_AS_DISABLE) (cg_overrideWithCGCG [*4] |-> reg_oclk));


`ifdef GFX_CG_W2E
reg hier_enable;
reg cgcgchecker_enable;
reg dump_enable;
string Hier;
initial begin
  hier_enable = 1'b1;
  cgcgchecker_enable = 1'b1;
  dump_enable = 1'b1;

  $sformat(Hier, "%m");

  if($test$plusargs({Hier,"__cgt_assert_hier_disable"}))
  begin
    hier_enable = 1'b0;
    `MSGIO_DISPLAY("%m:cg assertion is waived via hierarchy");
  end

for(integer i=0;i<Hier.len();i++) 
	if( (Hier.substr(i,i+2)=="cpf") || (Hier.substr(i,i+2)=="rlc") || (Hier.substr(i,i+3)=="grbm") || (Hier.substr(i,i+2)=="tpi") )
	begin 
           cgcgchecker_enable=1'b0; 
           break; 
    end 
for(integer i=0;i<Hier.len();i++) 
    `ifdef CORE_TILE
    if((Hier.substr(i,i+4)=="mmhub")|| (Hier.substr(i,i+6)=="hubs_ea") || (Hier.substr(i,i+2)=="acv") || (Hier.substr(i,i+4)=="gdfll") || (Hier.substr(i,i+4)=="gc_ge") || (Hier.substr(i,i+4)=="utcl1") || (Hier.substr(i,i+2)=="cpf") || (Hier.substr(i,i+2)=="spi") || (Hier.substr(i,i+2)=="sqg") || (Hier.substr(i,i+6)=="sq_comm"))      
    `else
    if((Hier.substr(i,i+4)=="mmhub")|| (Hier.substr(i,i+6)=="hubs_ea") || (Hier.substr(i,i+2)=="acv") || (Hier.substr(i,i+4)=="gdfll") || (Hier.substr(i,i+4)=="gc.ge") || (Hier.substr(i,i+4)=="utcl1") || (Hier.substr(i,i+2)=="cpf") || (Hier.substr(i,i+2)=="spi") || (Hier.substr(i,i+2)=="sqg") || (Hier.substr(i,i+6)=="sq_comm"))      
    `endif
	begin 
           hier_enable=1'b0; 
           dump_enable=1'b0; 
           break; 
    end

for(integer i=0;i<Hier.len();i++) 
    if( (Hier.substr(i,i+3)=="rspm") )      
	begin 
           hier_enable=1'b0; 
	   cgcgchecker_enable=1'b0; 
           break; 
    end
for(integer i=0;i<Hier.len();i++) 
    if(Hier.substr(i,i+4)=="gc_ea")      
	begin 
           dump_enable=1'b0; 
           break; 
    end
end

reg cgt_en_assert_enable;
reg MGCG_Coverage_enable;
reg CGTT_Coverage_enable;
initial begin
  if($test$plusargs("cgt_en_assert_disable"))
    cgt_en_assert_enable = 1'b0;
  else
    cgt_en_assert_enable = 1'b1;

  if($test$plusargs("MGCG_Coverage_assert_enable"))
    MGCG_Coverage_enable = 1'b1;
  else
    MGCG_Coverage_enable = 1'b0;

  if($test$plusargs("CGTT_Coverage_assert_enable"))
    CGTT_Coverage_enable = 1'b1;
  else
    CGTT_Coverage_enable = 1'b0;
end

reg reset_deactivated;
initial reset_deactivated = 0;
always @(posedge iclk) if(resetb) reset_deactivated <= 1;
final if((reset_deactivated == 0) && enable && hier_enable) `MSGIO_DISPLAY("%m ERROR: Reset never deactivated");

reg cgcg_en_deactivated;
initial cgcg_en_deactivated = 0;
always @(posedge iclk) if(cgcg_en == 1'b0) cgcg_en_deactivated <= 1;
final if((cgcg_en_deactivated == 0) && enable && hier_enable && cgcgchecker_enable) `MSGIO_DISPLAY("%m ERROR: CLKGATE cgcg_en never deactivated");

reg override_disabled;
initial override_disabled = 1'b0;
always @(posedge iclk) if(cg_override == 1'b0 ) override_disabled <= 1'b1;
final if((override_disabled == 1'b0) && enable && cgt_en_assert_enable && hier_enable) `MSGIO_DISPLAY("%m ERROR: Master Override never deactivated");

final if((cg_override == 1'b0) && enable && MGCG_Coverage_enable) `MSGIO_DISPLAY("%m ERROR: CLKGATE Master Override never activated");

final if((soft_override == 1'b0) && CGTT_Coverage_enable && enable) `MSGIO_DISPLAY("%m ERROR: CLKGATE CGTT Coverage: Soft Override never deactivated");

reg soft_override_disabled;
initial soft_override_disabled = 1'b0;
always @(posedge iclk) if(soft_override == 1'b0) soft_override_disabled <= 1'b1;
final if((soft_override_disabled == 1'b0) && enable && hier_enable) `MSGIO_DISPLAY("%m ERROR: Soft Override never deactivated");

reg soft_override_d1;
always @(posedge iclk)
begin
soft_override_d1<=soft_override;
end
wire pos_soft_override = ~soft_override_d1 & soft_override;
wire neg_soft_override = ~soft_override & soft_override_d1;
reg [3:0] edge_counter;
always @(posedge iclk) begin
  if(~resetb) begin
    edge_counter <= 'h0;
  end
  else begin
    if(pos_soft_override)
      edge_counter<=edge_counter+1;
    else if(neg_soft_override)
      edge_counter<=edge_counter-1;
  end
end
final if((edge_counter != 1'b0) && enable && hier_enable) `MSGIO_DISPLAY("%m ERROR: CLKGATE Soft Override was activated in the final");

reg vld_deasserted;
initial vld_deasserted = 1'b0;
always @(posedge iclk) if (reg_oclk_vld == 1'b0) vld_deasserted <= 1'b1;
final if (vld_deasserted == 1'b0 && enable && cgt_en_assert_enable && hier_enable)
	`MSGIO_DISPLAY("%m ERROR:CG_VLD checker: clk never turn off, check clock gating setting");

`ifdef MGCG_CHECKER
reg busy_deasserted;
initial busy_deasserted = 1'b0;
always @(posedge iclk) if (reg_busy == 1'b0) busy_deasserted <= 1'b1;
final if (busy_deasserted == 1'b0 && enable && cgt_en_assert_enable && hier_enable)
	`MSGIO_DISPLAY("%m ERROR:MGCG checker: reg_busy never deactivated");

reg busy_asserted;
initial busy_asserted = 1'b0;
always @(posedge iclk) if (reg_busy == 1'b1) busy_asserted <= 1'b1;
final if (busy_asserted == 1'b0 && enable && cgt_en_assert_enable && hier_enable)
	`MSGIO_DISPLAY("%m ERROR:MGCG checker: reg_busy never activated");
`endif  //`ifdef MGCG_CHECKER

//added dis acv assertion
`include "gfx_cgtt_local_acv_cov.vh"
`endif  //`ifdef GFX_CG_W2E

// soft override should be cleared before the end of test
//`ifdef GFX_CG_W2E
//final if((soft_override == 1'b1) && enable && hier_enable && cgt_en_assert_enable) `MSGIO_DISPLAY("%m ERROR: Soft Override is set at the end of test");
//`else
//final if((soft_override == 1'b1) && enable && hier_enable && cgt_en_assert_enable) `MSGIO_DISPLAY("%m WARNING: Soft Override is set at the end of test");
//`endif

`ifdef CGT_FCOV

covergroup cgtt_local_rcg_cg_t @(negedge iclk);

    reg_oclk_cp : coverpoint reg_oclk iff(resetb)
    {
        bins num_clocks_reg_oclk_inactive = {0};
        bins num_clocks_reg_oclk_active   = {1};
    }

endgroup //cgtt_rcg_local_cg_t
cgtt_local_rcg_cg_t cgtt_local_rcg_cg = new;

`endif // CGT_FCOV

`ifdef GFX_CG_SIGNALS_DUMP_EN
reg resetconfirm;
always @(posedge static_oclk)
begin
	resetconfirm <= resetb;
end
initial
begin
  #1;
`ifdef GC__SOC_ENV__1 
  `ifdef GC__FLAVOR__APU 
  if(dump_enable) begin
    $fmonitor(sys.cg_file_handle, "CG CGTT_RCG_OV:(%t) %m %b",$time,cg_override);
    $fmonitor(sys.cg_file_handle, "CG CGTT_RCG_SOFT_OV:(%t) %m %b",$time,soft_override);
    $fmonitor(sys.cg_file_handle, "CG CGTT_RCG_RST:(%t) %m %b",$time,resetconfirm);
  end
  `else
  if(dump_enable) begin
    $fmonitor(tb.cg_file_handle, "CG CGTT_RCG_OV:(%t) %m %b",$time,cg_override);
    $fmonitor(tb.cg_file_handle, "CG CGTT_RCG_SOFT_OV:(%t) %m %b",$time,soft_override);
    $fmonitor(tb.cg_file_handle, "CG CGTT_RCG_RST:(%t) %m %b",$time,resetconfirm);
  end
  `endif
 `else
  if(dump_enable) begin
    $fmonitor(tb.cg_file_handle, "CG CGTT_RCG_OV:(%t) %m %b",$time,cg_override);
    $fmonitor(tb.cg_file_handle, "CG CGTT_RCG_SOFT_OV:(%t) %m %b",$time,soft_override);
    $fmonitor(tb.cg_file_handle, "CG CGTT_RCG_RST:(%t) %m %b",$time,resetconfirm);
  end
 `endif
end
`endif


`ifdef CLK_EFF_MONITOR
/////////////////////////////////////////////////////////////////////////
//To provide information about how often a block is turning its clock off
/////////////////////////////////////////////////////////////////////////
real    static_oclk_count;
real    reg_oclk_off_count;

initial
begin
  reg_oclk_off_count = 0;
  static_oclk_count  = 0;
end

always @(posedge static_oclk)
begin
  if(resetb && ~reg_oclk_vld) begin
    reg_oclk_off_count = reg_oclk_off_count + 1;  //don't count when resetb is active
  end
  static_oclk_count = static_oclk_count + 1;
end

final
begin
    `MSGIO_DISPLAY("INFO:  LCG: final reg-clk off percentage = %4.1f, total clocks = %f;", reg_oclk_off_count / static_oclk_count * 100.0, static_oclk_count);
end
`endif

`else
initial begin
  $display("%m at %0t: ld/src/misc/cgtt_local_rcg_cov.sv: SVA not yet supported in NCSIM in wekiva scripts", $realtime);
end

//VCS coverage on

`endif // VCS


// vrq translate_on

endmodule



