// This file generated by convertRsrcSamplerXlsxToCode.pl from //gfxip/gfx10/doc/architecture/ResourceSampler.xlsx#94.

// 
//  Trade Secret of Advanced Micro Devices, Inc.
//  Copyright 2016, Advanced Micro Devices, Inc., (unpublished)
//  
//  All rights reserved.  This notice is intended as a precausion against
//  inadvertant publication and does not imply publication or any waiver
//  of confidentiality.  The year included is the foregoing notice is the
//  year of creation of the work.
//  

// This will convert from the MTBUF, MUBUF, or MIMG resource format of native gfx10 resources
// and gfx7 resources into the gfx10 internal data format and number format values.

`include "gc_features.vh"
`include "gc__gb_reg.v"

module gfx_conv_rsrc_dfmt (
   input logic [6:0] inputRsrcBufferFormat,  // For a gfx10 resource this will be the combined format view that
   input logic [9:0] inputRsrcImageFormat,   //   SW uses (eg IMG_FMT_8_UNORM or BUF_FMT_8_UNORM) in the LSBs of
                                             //   this bus directly from the image or buffer resource.
                                             // For a gfx7 resource this will be the data format and number format
                                             //   concatenated together (eg {UNORM,IMG_FMT_8} or {BUF_FMT_8,UNORM})
                                             //   directly from the image or buffer resource.
   input logic [6:0] inputInstBufferFormat,  // For a gfx10 resource this will be the combined format view that
                                             //   SW uses directly from the instruction.
                                             // For a gfx7 resource this will be data_format and number_format
                                             //   concatenated together (eg {UNORM,BUF_FMT_8}) directly from the instruction.
   input logic resourceLevelBuffer,          // ResourceLevel field directly from buffer resource
   input logic resourceLevelImage,           // ResourceLevel field directly from image resource
                                             //   0: Legacy Resource, 1: Native Resource
   input logic resourceOverride,             // ResourceOverride field from Compat Config Register
                                             //   0: Use register CompatLevel input, 1: Resource may override CompatLevel to a lower level
   input logic [1:0] compatLevel,            // CompatLevel field from Compat Config Register
                                             //   0: GEN_ZERO (legacy), 1: GEN_ONE (legacy), 2: GEN_TWO (native)
   input logic isImage,                      // Is this from an image resource?  Should be the MSB of RsrcType from the resource
   input logic useInstFormat,                // Use instruction format?  Only valid if isImage==0

   output logic [6:0] dataFormat,            // Separated data format (eg IMG_DATA_FORMAT_8 or {0,BUF_DATA_FORMAT_8})
   output logic [3:0] numFormat              // Separated number type (eg IMG_NUM_FORMAT_UNORM or {0,BUF_NUM_FORMAT_UNORM})
   );

   logic isLegacyUnboundBuffer;
   logic isNativeUnboundBuffer;

   logic isNativePipe;
   logic resourceLevel;
   logic isLegacyResource;
   logic isUnboundBuffer;
   logic isUnboundImage;
   logic isUnboundResource;
   logic [6:0] nativeSelectedBufferFormat;
   logic [8:0] nativeSelectedImageFormat;
   logic [8:0] nativeSelectedFormat;
   logic decodeAsLegacyFormat;

   // Detect unbound resource format in both native and back compat resources
   assign isLegacyUnboundBuffer = (inputRsrcBufferFormat[6:3] == `BUF_DATA_FORMAT_INVALID);
   assign isNativeUnboundBuffer = (inputRsrcBufferFormat[6:0] == `BUF_FMT_INVALID);
   assign isLegacyUnboundImage  = (inputRsrcImageFormat[5:0] == `IMG_DATA_FORMAT_INVALID);
   assign isNativeUnboundImage  = (inputRsrcImageFormat[8:0] == `IMG_FMT_INVALID);

   // Determine pipeline and resource compat levels
   assign isNativePipe = (compatLevel == 2'h2);
   assign resourceLevel = (isImage) ? resourceLevelImage : resourceLevelBuffer;
   assign isLegacyResource = ((compatLevel == 2'h2) && ((resourceOverride==1'h0) || (resourceLevel==1'h1))) ? 1'h0 : 1'h1;

   // Determine unbound resource
   assign isUnboundBuffer = (!isImage && ((isLegacyResource && isLegacyUnboundBuffer) || (!isLegacyResource && isNativeUnboundBuffer))) ? 1'b1 : 1'b0;
   assign isUnboundImage  = (isImage && ((isLegacyResource && isLegacyUnboundImage) || (!isLegacyResource && isNativeUnboundImage))) ? 1'b1 : 1'b0;
   assign isUnboundResource = isUnboundBuffer || isUnboundImage;

   // Select the native format
   assign nativeSelectedBufferFormat = (isUnboundBuffer) ? `BUF_FMT_INVALID : ((useInstFormat) ? inputInstBufferFormat : inputRsrcBufferFormat);  // Native MTBUF vs MUBUF, Unbound check handles native or legacy resource
   assign nativeSelectedImageFormat  = (isUnboundImage) ? `IMG_FMT_INVALID : inputRsrcImageFormat;  // Unbound check handles native or legacy resource
   assign nativeSelectedFormat = (isImage) ? nativeSelectedImageFormat[8:0] : {2'h0,nativeSelectedBufferFormat[6:0]};

   // Parsing a legacy format if we have a legacy resource, unless it is a native pipe with an unbound resource or a MTBUF op.
   assign decodeAsLegacyFormat = (isLegacyResource && !(isNativePipe && (isUnboundResource || (!isImage && useInstFormat))));

   always @(*) begin
      if ( decodeAsLegacyFormat ) begin
         if ( isImage ) begin                                         // MIMG resource

            dataFormat = {1'h0,inputRsrcImageFormat[5:0]};
            numFormat  = inputRsrcImageFormat[9:6];

         end
	 // Skip legacy unbound buffer check because it was missing in gfx7 (see Jira DEGGIGXX0-2851)
         // else if ( useInstFormat && (!isLegacyUnboundBuffer) ) begin  // MTBUF resource with Legacy unbound check
         else if ( useInstFormat ) begin                              // MTBUF resource without Legacy unbound check

            dataFormat = {3'h0,inputInstBufferFormat[3:0]};
            numFormat  = {1'h0,inputInstBufferFormat[6:4]};

         end
         else begin                                                   // MUBUF resource

            dataFormat = {3'h0,inputRsrcBufferFormat[6:3]};
            numFormat  = {1'h0,inputRsrcBufferFormat[2:0]};

         end
      end
      else begin

         // Buffer defines are guaranteed to be a strict subset of Image defines with equal numeric values
         // No need for separate case statements

         case ( nativeSelectedFormat )
            `IMG_FMT_INVALID                    : begin dataFormat = `IMG_DATA_FORMAT_INVALID             ; numFormat = `IMG_NUM_FORMAT_UINT               ; end    // 0x000 => (0x00, 0x4)
            `IMG_FMT_8_UNORM                    : begin dataFormat = `IMG_DATA_FORMAT_8                   ; numFormat = `IMG_NUM_FORMAT_UNORM              ; end    // 0x001 => (0x01, 0x0)
            `IMG_FMT_8_SNORM                    : begin dataFormat = `IMG_DATA_FORMAT_8                   ; numFormat = `IMG_NUM_FORMAT_SNORM              ; end    // 0x002 => (0x01, 0x1)
            `IMG_FMT_8_USCALED                  : begin dataFormat = `IMG_DATA_FORMAT_8                   ; numFormat = `IMG_NUM_FORMAT_USCALED            ; end    // 0x003 => (0x01, 0x2)
            `IMG_FMT_8_SSCALED                  : begin dataFormat = `IMG_DATA_FORMAT_8                   ; numFormat = `IMG_NUM_FORMAT_SSCALED            ; end    // 0x004 => (0x01, 0x3)
            `IMG_FMT_8_UINT                     : begin dataFormat = `IMG_DATA_FORMAT_8                   ; numFormat = `IMG_NUM_FORMAT_UINT               ; end    // 0x005 => (0x01, 0x4)
            `IMG_FMT_8_SINT                     : begin dataFormat = `IMG_DATA_FORMAT_8                   ; numFormat = `IMG_NUM_FORMAT_SINT               ; end    // 0x006 => (0x01, 0x5)
            `IMG_FMT_16_UNORM                   : begin dataFormat = `IMG_DATA_FORMAT_16                  ; numFormat = `IMG_NUM_FORMAT_UNORM              ; end    // 0x007 => (0x02, 0x0)
            `IMG_FMT_16_SNORM                   : begin dataFormat = `IMG_DATA_FORMAT_16                  ; numFormat = `IMG_NUM_FORMAT_SNORM              ; end    // 0x008 => (0x02, 0x1)
            `IMG_FMT_16_USCALED                 : begin dataFormat = `IMG_DATA_FORMAT_16                  ; numFormat = `IMG_NUM_FORMAT_USCALED            ; end    // 0x009 => (0x02, 0x2)
            `IMG_FMT_16_SSCALED                 : begin dataFormat = `IMG_DATA_FORMAT_16                  ; numFormat = `IMG_NUM_FORMAT_SSCALED            ; end    // 0x00A => (0x02, 0x3)
            `IMG_FMT_16_UINT                    : begin dataFormat = `IMG_DATA_FORMAT_16                  ; numFormat = `IMG_NUM_FORMAT_UINT               ; end    // 0x00B => (0x02, 0x4)
            `IMG_FMT_16_SINT                    : begin dataFormat = `IMG_DATA_FORMAT_16                  ; numFormat = `IMG_NUM_FORMAT_SINT               ; end    // 0x00C => (0x02, 0x5)
            `IMG_FMT_16_FLOAT                   : begin dataFormat = `IMG_DATA_FORMAT_16                  ; numFormat = `IMG_NUM_FORMAT_FLOAT              ; end    // 0x00D => (0x02, 0x7)
            `IMG_FMT_8_8_UNORM                  : begin dataFormat = `IMG_DATA_FORMAT_8_8                 ; numFormat = `IMG_NUM_FORMAT_UNORM              ; end    // 0x00E => (0x03, 0x0)
            `IMG_FMT_8_8_SNORM                  : begin dataFormat = `IMG_DATA_FORMAT_8_8                 ; numFormat = `IMG_NUM_FORMAT_SNORM              ; end    // 0x00F => (0x03, 0x1)
            `IMG_FMT_8_8_USCALED                : begin dataFormat = `IMG_DATA_FORMAT_8_8                 ; numFormat = `IMG_NUM_FORMAT_USCALED            ; end    // 0x010 => (0x03, 0x2)
            `IMG_FMT_8_8_SSCALED                : begin dataFormat = `IMG_DATA_FORMAT_8_8                 ; numFormat = `IMG_NUM_FORMAT_SSCALED            ; end    // 0x011 => (0x03, 0x3)
            `IMG_FMT_8_8_UINT                   : begin dataFormat = `IMG_DATA_FORMAT_8_8                 ; numFormat = `IMG_NUM_FORMAT_UINT               ; end    // 0x012 => (0x03, 0x4)
            `IMG_FMT_8_8_SINT                   : begin dataFormat = `IMG_DATA_FORMAT_8_8                 ; numFormat = `IMG_NUM_FORMAT_SINT               ; end    // 0x013 => (0x03, 0x5)
            `IMG_FMT_32_UINT                    : begin dataFormat = `IMG_DATA_FORMAT_32                  ; numFormat = `IMG_NUM_FORMAT_UINT               ; end    // 0x014 => (0x04, 0x4)
            `IMG_FMT_32_SINT                    : begin dataFormat = `IMG_DATA_FORMAT_32                  ; numFormat = `IMG_NUM_FORMAT_SINT               ; end    // 0x015 => (0x04, 0x5)
            `IMG_FMT_32_FLOAT                   : begin dataFormat = `IMG_DATA_FORMAT_32                  ; numFormat = `IMG_NUM_FORMAT_FLOAT              ; end    // 0x016 => (0x04, 0x7)
            `IMG_FMT_16_16_UNORM                : begin dataFormat = `IMG_DATA_FORMAT_16_16               ; numFormat = `IMG_NUM_FORMAT_UNORM              ; end    // 0x017 => (0x05, 0x0)
            `IMG_FMT_16_16_SNORM                : begin dataFormat = `IMG_DATA_FORMAT_16_16               ; numFormat = `IMG_NUM_FORMAT_SNORM              ; end    // 0x018 => (0x05, 0x1)
            `IMG_FMT_16_16_USCALED              : begin dataFormat = `IMG_DATA_FORMAT_16_16               ; numFormat = `IMG_NUM_FORMAT_USCALED            ; end    // 0x019 => (0x05, 0x2)
            `IMG_FMT_16_16_SSCALED              : begin dataFormat = `IMG_DATA_FORMAT_16_16               ; numFormat = `IMG_NUM_FORMAT_SSCALED            ; end    // 0x01A => (0x05, 0x3)
            `IMG_FMT_16_16_UINT                 : begin dataFormat = `IMG_DATA_FORMAT_16_16               ; numFormat = `IMG_NUM_FORMAT_UINT               ; end    // 0x01B => (0x05, 0x4)
            `IMG_FMT_16_16_SINT                 : begin dataFormat = `IMG_DATA_FORMAT_16_16               ; numFormat = `IMG_NUM_FORMAT_SINT               ; end    // 0x01C => (0x05, 0x5)
            `IMG_FMT_16_16_FLOAT                : begin dataFormat = `IMG_DATA_FORMAT_16_16               ; numFormat = `IMG_NUM_FORMAT_FLOAT              ; end    // 0x01D => (0x05, 0x7)
            `IMG_FMT_10_11_11_UNORM             : begin dataFormat = `IMG_DATA_FORMAT_10_11_11            ; numFormat = `IMG_NUM_FORMAT_UNORM              ; end    // 0x01E => (0x06, 0x0)
            `IMG_FMT_10_11_11_SNORM             : begin dataFormat = `IMG_DATA_FORMAT_10_11_11            ; numFormat = `IMG_NUM_FORMAT_SNORM              ; end    // 0x01F => (0x06, 0x1)
            `IMG_FMT_10_11_11_USCALED           : begin dataFormat = `IMG_DATA_FORMAT_10_11_11            ; numFormat = `IMG_NUM_FORMAT_USCALED            ; end    // 0x020 => (0x06, 0x2)
            `IMG_FMT_10_11_11_SSCALED           : begin dataFormat = `IMG_DATA_FORMAT_10_11_11            ; numFormat = `IMG_NUM_FORMAT_SSCALED            ; end    // 0x021 => (0x06, 0x3)
            `IMG_FMT_10_11_11_UINT              : begin dataFormat = `IMG_DATA_FORMAT_10_11_11            ; numFormat = `IMG_NUM_FORMAT_UINT               ; end    // 0x022 => (0x06, 0x4)
            `IMG_FMT_10_11_11_SINT              : begin dataFormat = `IMG_DATA_FORMAT_10_11_11            ; numFormat = `IMG_NUM_FORMAT_SINT               ; end    // 0x023 => (0x06, 0x5)
            `IMG_FMT_10_11_11_FLOAT             : begin dataFormat = `IMG_DATA_FORMAT_10_11_11            ; numFormat = `IMG_NUM_FORMAT_FLOAT              ; end    // 0x024 => (0x06, 0x7)
            `IMG_FMT_11_11_10_UNORM             : begin dataFormat = `IMG_DATA_FORMAT_11_11_10            ; numFormat = `IMG_NUM_FORMAT_UNORM              ; end    // 0x025 => (0x07, 0x0)
            `IMG_FMT_11_11_10_SNORM             : begin dataFormat = `IMG_DATA_FORMAT_11_11_10            ; numFormat = `IMG_NUM_FORMAT_SNORM              ; end    // 0x026 => (0x07, 0x1)
            `IMG_FMT_11_11_10_USCALED           : begin dataFormat = `IMG_DATA_FORMAT_11_11_10            ; numFormat = `IMG_NUM_FORMAT_USCALED            ; end    // 0x027 => (0x07, 0x2)
            `IMG_FMT_11_11_10_SSCALED           : begin dataFormat = `IMG_DATA_FORMAT_11_11_10            ; numFormat = `IMG_NUM_FORMAT_SSCALED            ; end    // 0x028 => (0x07, 0x3)
            `IMG_FMT_11_11_10_UINT              : begin dataFormat = `IMG_DATA_FORMAT_11_11_10            ; numFormat = `IMG_NUM_FORMAT_UINT               ; end    // 0x029 => (0x07, 0x4)
            `IMG_FMT_11_11_10_SINT              : begin dataFormat = `IMG_DATA_FORMAT_11_11_10            ; numFormat = `IMG_NUM_FORMAT_SINT               ; end    // 0x02A => (0x07, 0x5)
            `IMG_FMT_11_11_10_FLOAT             : begin dataFormat = `IMG_DATA_FORMAT_11_11_10            ; numFormat = `IMG_NUM_FORMAT_FLOAT              ; end    // 0x02B => (0x07, 0x7)
            `IMG_FMT_10_10_10_2_UNORM           : begin dataFormat = `IMG_DATA_FORMAT_10_10_10_2          ; numFormat = `IMG_NUM_FORMAT_UNORM              ; end    // 0x02C => (0x08, 0x0)
            `IMG_FMT_10_10_10_2_SNORM           : begin dataFormat = `IMG_DATA_FORMAT_10_10_10_2          ; numFormat = `IMG_NUM_FORMAT_SNORM              ; end    // 0x02D => (0x08, 0x1)
            `IMG_FMT_10_10_10_2_USCALED         : begin dataFormat = `IMG_DATA_FORMAT_10_10_10_2          ; numFormat = `IMG_NUM_FORMAT_USCALED            ; end    // 0x02E => (0x08, 0x2)
            `IMG_FMT_10_10_10_2_SSCALED         : begin dataFormat = `IMG_DATA_FORMAT_10_10_10_2          ; numFormat = `IMG_NUM_FORMAT_SSCALED            ; end    // 0x02F => (0x08, 0x3)
            `IMG_FMT_10_10_10_2_UINT            : begin dataFormat = `IMG_DATA_FORMAT_10_10_10_2          ; numFormat = `IMG_NUM_FORMAT_UINT               ; end    // 0x030 => (0x08, 0x4)
            `IMG_FMT_10_10_10_2_SINT            : begin dataFormat = `IMG_DATA_FORMAT_10_10_10_2          ; numFormat = `IMG_NUM_FORMAT_SINT               ; end    // 0x031 => (0x08, 0x5)
            `IMG_FMT_2_10_10_10_UNORM           : begin dataFormat = `IMG_DATA_FORMAT_2_10_10_10          ; numFormat = `IMG_NUM_FORMAT_UNORM              ; end    // 0x032 => (0x09, 0x0)
            `IMG_FMT_2_10_10_10_SNORM           : begin dataFormat = `IMG_DATA_FORMAT_2_10_10_10          ; numFormat = `IMG_NUM_FORMAT_SNORM              ; end    // 0x033 => (0x09, 0x1)
            `IMG_FMT_2_10_10_10_USCALED         : begin dataFormat = `IMG_DATA_FORMAT_2_10_10_10          ; numFormat = `IMG_NUM_FORMAT_USCALED            ; end    // 0x034 => (0x09, 0x2)
            `IMG_FMT_2_10_10_10_SSCALED         : begin dataFormat = `IMG_DATA_FORMAT_2_10_10_10          ; numFormat = `IMG_NUM_FORMAT_SSCALED            ; end    // 0x035 => (0x09, 0x3)
            `IMG_FMT_2_10_10_10_UINT            : begin dataFormat = `IMG_DATA_FORMAT_2_10_10_10          ; numFormat = `IMG_NUM_FORMAT_UINT               ; end    // 0x036 => (0x09, 0x4)
            `IMG_FMT_2_10_10_10_SINT            : begin dataFormat = `IMG_DATA_FORMAT_2_10_10_10          ; numFormat = `IMG_NUM_FORMAT_SINT               ; end    // 0x037 => (0x09, 0x5)
            `IMG_FMT_8_8_8_8_UNORM              : begin dataFormat = `IMG_DATA_FORMAT_8_8_8_8             ; numFormat = `IMG_NUM_FORMAT_UNORM              ; end    // 0x038 => (0x0A, 0x0)
            `IMG_FMT_8_8_8_8_SNORM              : begin dataFormat = `IMG_DATA_FORMAT_8_8_8_8             ; numFormat = `IMG_NUM_FORMAT_SNORM              ; end    // 0x039 => (0x0A, 0x1)
            `IMG_FMT_8_8_8_8_USCALED            : begin dataFormat = `IMG_DATA_FORMAT_8_8_8_8             ; numFormat = `IMG_NUM_FORMAT_USCALED            ; end    // 0x03A => (0x0A, 0x2)
            `IMG_FMT_8_8_8_8_SSCALED            : begin dataFormat = `IMG_DATA_FORMAT_8_8_8_8             ; numFormat = `IMG_NUM_FORMAT_SSCALED            ; end    // 0x03B => (0x0A, 0x3)
            `IMG_FMT_8_8_8_8_UINT               : begin dataFormat = `IMG_DATA_FORMAT_8_8_8_8             ; numFormat = `IMG_NUM_FORMAT_UINT               ; end    // 0x03C => (0x0A, 0x4)
            `IMG_FMT_8_8_8_8_SINT               : begin dataFormat = `IMG_DATA_FORMAT_8_8_8_8             ; numFormat = `IMG_NUM_FORMAT_SINT               ; end    // 0x03D => (0x0A, 0x5)
            `IMG_FMT_32_32_UINT                 : begin dataFormat = `IMG_DATA_FORMAT_32_32               ; numFormat = `IMG_NUM_FORMAT_UINT               ; end    // 0x03E => (0x0B, 0x4)
            `IMG_FMT_32_32_SINT                 : begin dataFormat = `IMG_DATA_FORMAT_32_32               ; numFormat = `IMG_NUM_FORMAT_SINT               ; end    // 0x03F => (0x0B, 0x5)
            `IMG_FMT_32_32_FLOAT                : begin dataFormat = `IMG_DATA_FORMAT_32_32               ; numFormat = `IMG_NUM_FORMAT_FLOAT              ; end    // 0x040 => (0x0B, 0x7)
            `IMG_FMT_16_16_16_16_UNORM          : begin dataFormat = `IMG_DATA_FORMAT_16_16_16_16         ; numFormat = `IMG_NUM_FORMAT_UNORM              ; end    // 0x041 => (0x0C, 0x0)
            `IMG_FMT_16_16_16_16_SNORM          : begin dataFormat = `IMG_DATA_FORMAT_16_16_16_16         ; numFormat = `IMG_NUM_FORMAT_SNORM              ; end    // 0x042 => (0x0C, 0x1)
            `IMG_FMT_16_16_16_16_USCALED        : begin dataFormat = `IMG_DATA_FORMAT_16_16_16_16         ; numFormat = `IMG_NUM_FORMAT_USCALED            ; end    // 0x043 => (0x0C, 0x2)
            `IMG_FMT_16_16_16_16_SSCALED        : begin dataFormat = `IMG_DATA_FORMAT_16_16_16_16         ; numFormat = `IMG_NUM_FORMAT_SSCALED            ; end    // 0x044 => (0x0C, 0x3)
            `IMG_FMT_16_16_16_16_UINT           : begin dataFormat = `IMG_DATA_FORMAT_16_16_16_16         ; numFormat = `IMG_NUM_FORMAT_UINT               ; end    // 0x045 => (0x0C, 0x4)
            `IMG_FMT_16_16_16_16_SINT           : begin dataFormat = `IMG_DATA_FORMAT_16_16_16_16         ; numFormat = `IMG_NUM_FORMAT_SINT               ; end    // 0x046 => (0x0C, 0x5)
            `IMG_FMT_16_16_16_16_FLOAT          : begin dataFormat = `IMG_DATA_FORMAT_16_16_16_16         ; numFormat = `IMG_NUM_FORMAT_FLOAT              ; end    // 0x047 => (0x0C, 0x7)
            `IMG_FMT_32_32_32_UINT              : begin dataFormat = `IMG_DATA_FORMAT_32_32_32            ; numFormat = `IMG_NUM_FORMAT_UINT               ; end    // 0x048 => (0x0D, 0x4)
            `IMG_FMT_32_32_32_SINT              : begin dataFormat = `IMG_DATA_FORMAT_32_32_32            ; numFormat = `IMG_NUM_FORMAT_SINT               ; end    // 0x049 => (0x0D, 0x5)
            `IMG_FMT_32_32_32_FLOAT             : begin dataFormat = `IMG_DATA_FORMAT_32_32_32            ; numFormat = `IMG_NUM_FORMAT_FLOAT              ; end    // 0x04A => (0x0D, 0x7)
            `IMG_FMT_32_32_32_32_UINT           : begin dataFormat = `IMG_DATA_FORMAT_32_32_32_32         ; numFormat = `IMG_NUM_FORMAT_UINT               ; end    // 0x04B => (0x0E, 0x4)
            `IMG_FMT_32_32_32_32_SINT           : begin dataFormat = `IMG_DATA_FORMAT_32_32_32_32         ; numFormat = `IMG_NUM_FORMAT_SINT               ; end    // 0x04C => (0x0E, 0x5)
            `IMG_FMT_32_32_32_32_FLOAT          : begin dataFormat = `IMG_DATA_FORMAT_32_32_32_32         ; numFormat = `IMG_NUM_FORMAT_FLOAT              ; end    // 0x04D => (0x0E, 0x7)
            `IMG_FMT_8_SRGB                     : begin dataFormat = `IMG_DATA_FORMAT_8                   ; numFormat = `IMG_NUM_FORMAT_SRGB               ; end    // 0x080 => (0x01, 0x9)
            `IMG_FMT_8_8_SRGB                   : begin dataFormat = `IMG_DATA_FORMAT_8_8                 ; numFormat = `IMG_NUM_FORMAT_SRGB               ; end    // 0x081 => (0x03, 0x9)
            `IMG_FMT_8_8_8_8_SRGB               : begin dataFormat = `IMG_DATA_FORMAT_8_8_8_8             ; numFormat = `IMG_NUM_FORMAT_SRGB               ; end    // 0x082 => (0x0A, 0x9)
            `IMG_FMT_6E4_FLOAT                  : begin dataFormat = `IMG_DATA_FORMAT_6E4                 ; numFormat = `IMG_NUM_FORMAT_FLOAT              ; end    // 0x083 => (0x1F, 0x7)
            `IMG_FMT_5_9_9_9_FLOAT              : begin dataFormat = `IMG_DATA_FORMAT_5_9_9_9             ; numFormat = `IMG_NUM_FORMAT_FLOAT              ; end    // 0x084 => (0x22, 0x7)
            `IMG_FMT_5_6_5_UNORM                : begin dataFormat = `IMG_DATA_FORMAT_5_6_5               ; numFormat = `IMG_NUM_FORMAT_UNORM              ; end    // 0x085 => (0x10, 0x0)
            `IMG_FMT_1_5_5_5_UNORM              : begin dataFormat = `IMG_DATA_FORMAT_1_5_5_5             ; numFormat = `IMG_NUM_FORMAT_UNORM              ; end    // 0x086 => (0x11, 0x0)
            `IMG_FMT_5_5_5_1_UNORM              : begin dataFormat = `IMG_DATA_FORMAT_5_5_5_1             ; numFormat = `IMG_NUM_FORMAT_UNORM              ; end    // 0x087 => (0x12, 0x0)
            `IMG_FMT_4_4_4_4_UNORM              : begin dataFormat = `IMG_DATA_FORMAT_4_4_4_4             ; numFormat = `IMG_NUM_FORMAT_UNORM              ; end    // 0x088 => (0x13, 0x0)
            `IMG_FMT_4_4_UNORM                  : begin dataFormat = `IMG_DATA_FORMAT_4_4                 ; numFormat = `IMG_NUM_FORMAT_UNORM              ; end    // 0x089 => (0x39, 0x0)
            `IMG_FMT_1_UNORM                    : begin dataFormat = `IMG_DATA_FORMAT_1                   ; numFormat = `IMG_NUM_FORMAT_UNORM              ; end    // 0x08A => (0x3B, 0x0)
            `IMG_FMT_1_REVERSED_UNORM           : begin dataFormat = `IMG_DATA_FORMAT_1_REVERSED          ; numFormat = `IMG_NUM_FORMAT_UNORM              ; end    // 0x08B => (0x3C, 0x0)
            `IMG_FMT_32_FLOAT_CLAMP             : begin dataFormat = `IMG_DATA_FORMAT_8_24                ; numFormat = `IMG_NUM_FORMAT_FLOAT              ; end    // 0x08C => (0x14, 0x7)
            `IMG_FMT_8_24_UNORM                 : begin dataFormat = `IMG_DATA_FORMAT_8_24                ; numFormat = `IMG_NUM_FORMAT_UNORM              ; end    // 0x08D => (0x14, 0x0)
            `IMG_FMT_8_24_UINT                  : begin dataFormat = `IMG_DATA_FORMAT_8_24                ; numFormat = `IMG_NUM_FORMAT_UINT               ; end    // 0x08E => (0x14, 0x4)
            `IMG_FMT_24_8_UNORM                 : begin dataFormat = `IMG_DATA_FORMAT_24_8                ; numFormat = `IMG_NUM_FORMAT_UNORM              ; end    // 0x08F => (0x15, 0x0)
            `IMG_FMT_24_8_UINT                  : begin dataFormat = `IMG_DATA_FORMAT_24_8                ; numFormat = `IMG_NUM_FORMAT_UINT               ; end    // 0x090 => (0x15, 0x4)
            `IMG_FMT_X24_8_32_UINT              : begin dataFormat = `IMG_DATA_FORMAT_X24_8_32            ; numFormat = `IMG_NUM_FORMAT_UINT               ; end    // 0x091 => (0x16, 0x4)
            `IMG_FMT_X24_8_32_FLOAT             : begin dataFormat = `IMG_DATA_FORMAT_X24_8_32            ; numFormat = `IMG_NUM_FORMAT_FLOAT              ; end    // 0x092 => (0x16, 0x7)
            `IMG_FMT_GB_GR_UNORM                : begin dataFormat = `IMG_DATA_FORMAT_GB_GR               ; numFormat = `IMG_NUM_FORMAT_UNORM              ; end    // 0x093 => (0x20, 0x0)
            `IMG_FMT_GB_GR_SNORM                : begin dataFormat = `IMG_DATA_FORMAT_GB_GR               ; numFormat = `IMG_NUM_FORMAT_SNORM              ; end    // 0x094 => (0x20, 0x1)
            `IMG_FMT_GB_GR_UINT                 : begin dataFormat = `IMG_DATA_FORMAT_GB_GR               ; numFormat = `IMG_NUM_FORMAT_UINT               ; end    // 0x095 => (0x20, 0x4)
            `IMG_FMT_GB_GR_SRGB                 : begin dataFormat = `IMG_DATA_FORMAT_GB_GR               ; numFormat = `IMG_NUM_FORMAT_SRGB               ; end    // 0x096 => (0x20, 0x9)
            `IMG_FMT_BG_RG_UNORM                : begin dataFormat = `IMG_DATA_FORMAT_BG_RG               ; numFormat = `IMG_NUM_FORMAT_UNORM              ; end    // 0x097 => (0x21, 0x0)
            `IMG_FMT_BG_RG_SNORM                : begin dataFormat = `IMG_DATA_FORMAT_BG_RG               ; numFormat = `IMG_NUM_FORMAT_SNORM              ; end    // 0x098 => (0x21, 0x1)
            `IMG_FMT_BG_RG_UINT                 : begin dataFormat = `IMG_DATA_FORMAT_BG_RG               ; numFormat = `IMG_NUM_FORMAT_UINT               ; end    // 0x099 => (0x21, 0x4)
            `IMG_FMT_BG_RG_SRGB                 : begin dataFormat = `IMG_DATA_FORMAT_BG_RG               ; numFormat = `IMG_NUM_FORMAT_SRGB               ; end    // 0x09A => (0x21, 0x9)
            `IMG_FMT_FMASK8_S2_F1               : begin dataFormat = `IMG_DATA_FORMAT_FMASK8_S2_F1        ; numFormat = `IMG_NUM_FORMAT_UINT               ; end    // 0x09C => (0x2C, 0x4)
            `IMG_FMT_FMASK8_S4_F1               : begin dataFormat = `IMG_DATA_FORMAT_FMASK8_S4_F1        ; numFormat = `IMG_NUM_FORMAT_UINT               ; end    // 0x09D => (0x2D, 0x4)
            `IMG_FMT_FMASK8_S8_F1               : begin dataFormat = `IMG_DATA_FORMAT_FMASK8_S8_F1        ; numFormat = `IMG_NUM_FORMAT_UINT               ; end    // 0x09E => (0x2E, 0x4)
            `IMG_FMT_FMASK8_S2_F2               : begin dataFormat = `IMG_DATA_FORMAT_FMASK8_S2_F2        ; numFormat = `IMG_NUM_FORMAT_UINT               ; end    // 0x09F => (0x2F, 0x4)
            `IMG_FMT_FMASK8_S4_F2               : begin dataFormat = `IMG_DATA_FORMAT_FMASK8_S4_F2        ; numFormat = `IMG_NUM_FORMAT_UINT               ; end    // 0x0A0 => (0x30, 0x4)
            `IMG_FMT_FMASK8_S4_F4               : begin dataFormat = `IMG_DATA_FORMAT_FMASK8_S4_F4        ; numFormat = `IMG_NUM_FORMAT_UINT               ; end    // 0x0A1 => (0x31, 0x4)
            `IMG_FMT_FMASK16_S16_F1             : begin dataFormat = `IMG_DATA_FORMAT_FMASK16_S16_F1      ; numFormat = `IMG_NUM_FORMAT_UINT               ; end    // 0x0A2 => (0x32, 0x4)
            `IMG_FMT_FMASK16_S8_F2              : begin dataFormat = `IMG_DATA_FORMAT_FMASK16_S8_F2       ; numFormat = `IMG_NUM_FORMAT_UINT               ; end    // 0x0A3 => (0x33, 0x4)
            `IMG_FMT_FMASK32_S16_F2             : begin dataFormat = `IMG_DATA_FORMAT_FMASK32_S16_F2      ; numFormat = `IMG_NUM_FORMAT_UINT               ; end    // 0x0A4 => (0x34, 0x4)
            `IMG_FMT_FMASK32_S8_F4              : begin dataFormat = `IMG_DATA_FORMAT_FMASK32_S8_F4       ; numFormat = `IMG_NUM_FORMAT_UINT               ; end    // 0x0A5 => (0x35, 0x4)
            `IMG_FMT_FMASK32_S8_F8              : begin dataFormat = `IMG_DATA_FORMAT_FMASK32_S8_F8       ; numFormat = `IMG_NUM_FORMAT_UINT               ; end    // 0x0A6 => (0x36, 0x4)
            `IMG_FMT_FMASK64_S16_F4             : begin dataFormat = `IMG_DATA_FORMAT_FMASK64_S16_F4      ; numFormat = `IMG_NUM_FORMAT_UINT               ; end    // 0x0A7 => (0x37, 0x4)
            `IMG_FMT_FMASK64_S16_F8             : begin dataFormat = `IMG_DATA_FORMAT_FMASK64_S16_F8      ; numFormat = `IMG_NUM_FORMAT_UINT               ; end    // 0x0A8 => (0x38, 0x4)
            `IMG_FMT_BC1_UNORM                  : begin dataFormat = `IMG_DATA_FORMAT_BC1                 ; numFormat = `IMG_NUM_FORMAT_UNORM              ; end    // 0x0A9 => (0x23, 0x0)
            `IMG_FMT_BC1_SRGB                   : begin dataFormat = `IMG_DATA_FORMAT_BC1                 ; numFormat = `IMG_NUM_FORMAT_SRGB               ; end    // 0x0AA => (0x23, 0x9)
            `IMG_FMT_BC2_UNORM                  : begin dataFormat = `IMG_DATA_FORMAT_BC2                 ; numFormat = `IMG_NUM_FORMAT_UNORM              ; end    // 0x0AB => (0x24, 0x0)
            `IMG_FMT_BC2_SRGB                   : begin dataFormat = `IMG_DATA_FORMAT_BC2                 ; numFormat = `IMG_NUM_FORMAT_SRGB               ; end    // 0x0AC => (0x24, 0x9)
            `IMG_FMT_BC3_UNORM                  : begin dataFormat = `IMG_DATA_FORMAT_BC3                 ; numFormat = `IMG_NUM_FORMAT_UNORM              ; end    // 0x0AD => (0x25, 0x0)
            `IMG_FMT_BC3_SRGB                   : begin dataFormat = `IMG_DATA_FORMAT_BC3                 ; numFormat = `IMG_NUM_FORMAT_SRGB               ; end    // 0x0AE => (0x25, 0x9)
            `IMG_FMT_BC4_UNORM                  : begin dataFormat = `IMG_DATA_FORMAT_BC4                 ; numFormat = `IMG_NUM_FORMAT_UNORM              ; end    // 0x0AF => (0x26, 0x0)
            `IMG_FMT_BC4_SNORM                  : begin dataFormat = `IMG_DATA_FORMAT_BC4                 ; numFormat = `IMG_NUM_FORMAT_SNORM              ; end    // 0x0B0 => (0x26, 0x1)
            `IMG_FMT_BC5_UNORM                  : begin dataFormat = `IMG_DATA_FORMAT_BC5                 ; numFormat = `IMG_NUM_FORMAT_UNORM              ; end    // 0x0B1 => (0x27, 0x0)
            `IMG_FMT_BC5_SNORM                  : begin dataFormat = `IMG_DATA_FORMAT_BC5                 ; numFormat = `IMG_NUM_FORMAT_SNORM              ; end    // 0x0B2 => (0x27, 0x1)
            `IMG_FMT_BC6_UFLOAT                 : begin dataFormat = `IMG_DATA_FORMAT_BC6                 ; numFormat = `IMG_NUM_FORMAT_UNORM              ; end    // 0x0B3 => (0x28, 0x0)
            `IMG_FMT_BC6_SFLOAT                 : begin dataFormat = `IMG_DATA_FORMAT_BC6                 ; numFormat = `IMG_NUM_FORMAT_SNORM              ; end    // 0x0B4 => (0x28, 0x1)
            `IMG_FMT_BC7_UNORM                  : begin dataFormat = `IMG_DATA_FORMAT_BC7                 ; numFormat = `IMG_NUM_FORMAT_UNORM              ; end    // 0x0B5 => (0x29, 0x0)
            `IMG_FMT_BC7_SRGB                   : begin dataFormat = `IMG_DATA_FORMAT_BC7                 ; numFormat = `IMG_NUM_FORMAT_SRGB               ; end    // 0x0B6 => (0x29, 0x9)
            `IMG_FMT_ETC2_RGB_UNORM             : begin dataFormat = `IMG_DATA_FORMAT_ETC2_RGB            ; numFormat = `IMG_NUM_FORMAT_UNORM              ; end    // 0x0B7 => (0x40, 0x0)
            `IMG_FMT_ETC2_RGB_SRGB              : begin dataFormat = `IMG_DATA_FORMAT_ETC2_RGB            ; numFormat = `IMG_NUM_FORMAT_SRGB               ; end    // 0x0B8 => (0x40, 0x9)
            `IMG_FMT_ETC2_RGBA_UNORM            : begin dataFormat = `IMG_DATA_FORMAT_ETC2_RGBA           ; numFormat = `IMG_NUM_FORMAT_UNORM              ; end    // 0x0B9 => (0x41, 0x0)
            `IMG_FMT_ETC2_RGBA_SRGB             : begin dataFormat = `IMG_DATA_FORMAT_ETC2_RGBA           ; numFormat = `IMG_NUM_FORMAT_SRGB               ; end    // 0x0BA => (0x41, 0x9)
            `IMG_FMT_ETC2_R_UNORM               : begin dataFormat = `IMG_DATA_FORMAT_ETC2_R              ; numFormat = `IMG_NUM_FORMAT_UNORM              ; end    // 0x0BB => (0x42, 0x0)
            `IMG_FMT_ETC2_R_SNORM               : begin dataFormat = `IMG_DATA_FORMAT_ETC2_R              ; numFormat = `IMG_NUM_FORMAT_SNORM              ; end    // 0x0BC => (0x42, 0x1)
            `IMG_FMT_ETC2_RG_UNORM              : begin dataFormat = `IMG_DATA_FORMAT_ETC2_RG             ; numFormat = `IMG_NUM_FORMAT_UNORM              ; end    // 0x0BD => (0x43, 0x0)
            `IMG_FMT_ETC2_RG_SNORM              : begin dataFormat = `IMG_DATA_FORMAT_ETC2_RG             ; numFormat = `IMG_NUM_FORMAT_SNORM              ; end    // 0x0BE => (0x43, 0x1)
            `IMG_FMT_ETC2_RGBA1_UNORM           : begin dataFormat = `IMG_DATA_FORMAT_ETC2_RGBA1          ; numFormat = `IMG_NUM_FORMAT_UNORM              ; end    // 0x0BF => (0x44, 0x0)
            `IMG_FMT_ETC2_RGBA1_SRGB            : begin dataFormat = `IMG_DATA_FORMAT_ETC2_RGBA1          ; numFormat = `IMG_NUM_FORMAT_SRGB               ; end    // 0x0C0 => (0x44, 0x9)
            `IMG_FMT_ASTC_2D_LDR_4X4            : begin dataFormat = `IMG_DATA_FORMAT_ASTC_2D_LDR         ; numFormat = `IMG_NUM_FORMAT_ASTC_2D_4X4        ; end    // 0x0C1 => (0x45, 0x0)
            `IMG_FMT_ASTC_2D_LDR_5X4            : begin dataFormat = `IMG_DATA_FORMAT_ASTC_2D_LDR         ; numFormat = `IMG_NUM_FORMAT_ASTC_2D_5X4        ; end    // 0x0C2 => (0x45, 0x1)
            `IMG_FMT_ASTC_2D_LDR_5X5            : begin dataFormat = `IMG_DATA_FORMAT_ASTC_2D_LDR         ; numFormat = `IMG_NUM_FORMAT_ASTC_2D_5X5        ; end    // 0x0C3 => (0x45, 0x2)
            `IMG_FMT_ASTC_2D_LDR_6X5            : begin dataFormat = `IMG_DATA_FORMAT_ASTC_2D_LDR         ; numFormat = `IMG_NUM_FORMAT_ASTC_2D_6X5        ; end    // 0x0C4 => (0x45, 0x3)
            `IMG_FMT_ASTC_2D_LDR_6X6            : begin dataFormat = `IMG_DATA_FORMAT_ASTC_2D_LDR         ; numFormat = `IMG_NUM_FORMAT_ASTC_2D_6X6        ; end    // 0x0C5 => (0x45, 0x4)
            `IMG_FMT_ASTC_2D_LDR_8X5            : begin dataFormat = `IMG_DATA_FORMAT_ASTC_2D_LDR         ; numFormat = `IMG_NUM_FORMAT_ASTC_2D_8X5        ; end    // 0x0C6 => (0x45, 0x5)
            `IMG_FMT_ASTC_2D_LDR_8X6            : begin dataFormat = `IMG_DATA_FORMAT_ASTC_2D_LDR         ; numFormat = `IMG_NUM_FORMAT_ASTC_2D_8X6        ; end    // 0x0C7 => (0x45, 0x6)
            `IMG_FMT_ASTC_2D_LDR_8X8            : begin dataFormat = `IMG_DATA_FORMAT_ASTC_2D_LDR         ; numFormat = `IMG_NUM_FORMAT_ASTC_2D_8X8        ; end    // 0x0C8 => (0x45, 0x7)
            `IMG_FMT_ASTC_2D_LDR_10X5           : begin dataFormat = `IMG_DATA_FORMAT_ASTC_2D_LDR         ; numFormat = `IMG_NUM_FORMAT_ASTC_2D_10X5       ; end    // 0x0C9 => (0x45, 0x8)
            `IMG_FMT_ASTC_2D_LDR_10X6           : begin dataFormat = `IMG_DATA_FORMAT_ASTC_2D_LDR         ; numFormat = `IMG_NUM_FORMAT_ASTC_2D_10X6       ; end    // 0x0CA => (0x45, 0x9)
            `IMG_FMT_ASTC_2D_LDR_10X8           : begin dataFormat = `IMG_DATA_FORMAT_ASTC_2D_LDR         ; numFormat = `IMG_NUM_FORMAT_ASTC_2D_10X8       ; end    // 0x0CB => (0x45, 0xA)
            `IMG_FMT_ASTC_2D_LDR_10X10          : begin dataFormat = `IMG_DATA_FORMAT_ASTC_2D_LDR         ; numFormat = `IMG_NUM_FORMAT_ASTC_2D_10X10      ; end    // 0x0CC => (0x45, 0xB)
            `IMG_FMT_ASTC_2D_LDR_12X10          : begin dataFormat = `IMG_DATA_FORMAT_ASTC_2D_LDR         ; numFormat = `IMG_NUM_FORMAT_ASTC_2D_12X10      ; end    // 0x0CD => (0x45, 0xC)
            `IMG_FMT_ASTC_2D_LDR_12X12          : begin dataFormat = `IMG_DATA_FORMAT_ASTC_2D_LDR         ; numFormat = `IMG_NUM_FORMAT_ASTC_2D_12X12      ; end    // 0x0CE => (0x45, 0xD)
            `IMG_FMT_ASTC_2D_HDR_4X4            : begin dataFormat = `IMG_DATA_FORMAT_ASTC_2D_HDR         ; numFormat = `IMG_NUM_FORMAT_ASTC_2D_4X4        ; end    // 0x0CF => (0x46, 0x0)
            `IMG_FMT_ASTC_2D_HDR_5X4            : begin dataFormat = `IMG_DATA_FORMAT_ASTC_2D_HDR         ; numFormat = `IMG_NUM_FORMAT_ASTC_2D_5X4        ; end    // 0x0D0 => (0x46, 0x1)
            `IMG_FMT_ASTC_2D_HDR_5X5            : begin dataFormat = `IMG_DATA_FORMAT_ASTC_2D_HDR         ; numFormat = `IMG_NUM_FORMAT_ASTC_2D_5X5        ; end    // 0x0D1 => (0x46, 0x2)
            `IMG_FMT_ASTC_2D_HDR_6X5            : begin dataFormat = `IMG_DATA_FORMAT_ASTC_2D_HDR         ; numFormat = `IMG_NUM_FORMAT_ASTC_2D_6X5        ; end    // 0x0D2 => (0x46, 0x3)
            `IMG_FMT_ASTC_2D_HDR_6X6            : begin dataFormat = `IMG_DATA_FORMAT_ASTC_2D_HDR         ; numFormat = `IMG_NUM_FORMAT_ASTC_2D_6X6        ; end    // 0x0D3 => (0x46, 0x4)
            `IMG_FMT_ASTC_2D_HDR_8X5            : begin dataFormat = `IMG_DATA_FORMAT_ASTC_2D_HDR         ; numFormat = `IMG_NUM_FORMAT_ASTC_2D_8X5        ; end    // 0x0D4 => (0x46, 0x5)
            `IMG_FMT_ASTC_2D_HDR_8X6            : begin dataFormat = `IMG_DATA_FORMAT_ASTC_2D_HDR         ; numFormat = `IMG_NUM_FORMAT_ASTC_2D_8X6        ; end    // 0x0D5 => (0x46, 0x6)
            `IMG_FMT_ASTC_2D_HDR_8X8            : begin dataFormat = `IMG_DATA_FORMAT_ASTC_2D_HDR         ; numFormat = `IMG_NUM_FORMAT_ASTC_2D_8X8        ; end    // 0x0D6 => (0x46, 0x7)
            `IMG_FMT_ASTC_2D_HDR_10X5           : begin dataFormat = `IMG_DATA_FORMAT_ASTC_2D_HDR         ; numFormat = `IMG_NUM_FORMAT_ASTC_2D_10X5       ; end    // 0x0D7 => (0x46, 0x8)
            `IMG_FMT_ASTC_2D_HDR_10X6           : begin dataFormat = `IMG_DATA_FORMAT_ASTC_2D_HDR         ; numFormat = `IMG_NUM_FORMAT_ASTC_2D_10X6       ; end    // 0x0D8 => (0x46, 0x9)
            `IMG_FMT_ASTC_2D_HDR_10X8           : begin dataFormat = `IMG_DATA_FORMAT_ASTC_2D_HDR         ; numFormat = `IMG_NUM_FORMAT_ASTC_2D_10X8       ; end    // 0x0D9 => (0x46, 0xA)
            `IMG_FMT_ASTC_2D_HDR_10X10          : begin dataFormat = `IMG_DATA_FORMAT_ASTC_2D_HDR         ; numFormat = `IMG_NUM_FORMAT_ASTC_2D_10X10      ; end    // 0x0DA => (0x46, 0xB)
            `IMG_FMT_ASTC_2D_HDR_12X10          : begin dataFormat = `IMG_DATA_FORMAT_ASTC_2D_HDR         ; numFormat = `IMG_NUM_FORMAT_ASTC_2D_12X10      ; end    // 0x0DB => (0x46, 0xC)
            `IMG_FMT_ASTC_2D_HDR_12X12          : begin dataFormat = `IMG_DATA_FORMAT_ASTC_2D_HDR         ; numFormat = `IMG_NUM_FORMAT_ASTC_2D_12X12      ; end    // 0x0DC => (0x46, 0xD)
            `IMG_FMT_ASTC_2D_LDR_SRGB_4X4       : begin dataFormat = `IMG_DATA_FORMAT_ASTC_2D_LDR_SRGB    ; numFormat = `IMG_NUM_FORMAT_ASTC_2D_4X4        ; end    // 0x0DD => (0x47, 0x0)
            `IMG_FMT_ASTC_2D_LDR_SRGB_5X4       : begin dataFormat = `IMG_DATA_FORMAT_ASTC_2D_LDR_SRGB    ; numFormat = `IMG_NUM_FORMAT_ASTC_2D_5X4        ; end    // 0x0DE => (0x47, 0x1)
            `IMG_FMT_ASTC_2D_LDR_SRGB_5X5       : begin dataFormat = `IMG_DATA_FORMAT_ASTC_2D_LDR_SRGB    ; numFormat = `IMG_NUM_FORMAT_ASTC_2D_5X5        ; end    // 0x0DF => (0x47, 0x2)
            `IMG_FMT_ASTC_2D_LDR_SRGB_6X5       : begin dataFormat = `IMG_DATA_FORMAT_ASTC_2D_LDR_SRGB    ; numFormat = `IMG_NUM_FORMAT_ASTC_2D_6X5        ; end    // 0x0E0 => (0x47, 0x3)
            `IMG_FMT_ASTC_2D_LDR_SRGB_6X6       : begin dataFormat = `IMG_DATA_FORMAT_ASTC_2D_LDR_SRGB    ; numFormat = `IMG_NUM_FORMAT_ASTC_2D_6X6        ; end    // 0x0E1 => (0x47, 0x4)
            `IMG_FMT_ASTC_2D_LDR_SRGB_8X5       : begin dataFormat = `IMG_DATA_FORMAT_ASTC_2D_LDR_SRGB    ; numFormat = `IMG_NUM_FORMAT_ASTC_2D_8X5        ; end    // 0x0E2 => (0x47, 0x5)
            `IMG_FMT_ASTC_2D_LDR_SRGB_8X6       : begin dataFormat = `IMG_DATA_FORMAT_ASTC_2D_LDR_SRGB    ; numFormat = `IMG_NUM_FORMAT_ASTC_2D_8X6        ; end    // 0x0E3 => (0x47, 0x6)
            `IMG_FMT_ASTC_2D_LDR_SRGB_8X8       : begin dataFormat = `IMG_DATA_FORMAT_ASTC_2D_LDR_SRGB    ; numFormat = `IMG_NUM_FORMAT_ASTC_2D_8X8        ; end    // 0x0E4 => (0x47, 0x7)
            `IMG_FMT_ASTC_2D_LDR_SRGB_10X5      : begin dataFormat = `IMG_DATA_FORMAT_ASTC_2D_LDR_SRGB    ; numFormat = `IMG_NUM_FORMAT_ASTC_2D_10X5       ; end    // 0x0E5 => (0x47, 0x8)
            `IMG_FMT_ASTC_2D_LDR_SRGB_10X6      : begin dataFormat = `IMG_DATA_FORMAT_ASTC_2D_LDR_SRGB    ; numFormat = `IMG_NUM_FORMAT_ASTC_2D_10X6       ; end    // 0x0E6 => (0x47, 0x9)
            `IMG_FMT_ASTC_2D_LDR_SRGB_10X8      : begin dataFormat = `IMG_DATA_FORMAT_ASTC_2D_LDR_SRGB    ; numFormat = `IMG_NUM_FORMAT_ASTC_2D_10X8       ; end    // 0x0E7 => (0x47, 0xA)
            `IMG_FMT_ASTC_2D_LDR_SRGB_10X10     : begin dataFormat = `IMG_DATA_FORMAT_ASTC_2D_LDR_SRGB    ; numFormat = `IMG_NUM_FORMAT_ASTC_2D_10X10      ; end    // 0x0E8 => (0x47, 0xB)
            `IMG_FMT_ASTC_2D_LDR_SRGB_12X10     : begin dataFormat = `IMG_DATA_FORMAT_ASTC_2D_LDR_SRGB    ; numFormat = `IMG_NUM_FORMAT_ASTC_2D_12X10      ; end    // 0x0E9 => (0x47, 0xC)
            `IMG_FMT_ASTC_2D_LDR_SRGB_12X12     : begin dataFormat = `IMG_DATA_FORMAT_ASTC_2D_LDR_SRGB    ; numFormat = `IMG_NUM_FORMAT_ASTC_2D_12X12      ; end    // 0x0EA => (0x47, 0xD)
            `IMG_FMT_ASTC_3D_LDR_3X3X3          : begin dataFormat = `IMG_DATA_FORMAT_ASTC_3D_LDR         ; numFormat = `IMG_NUM_FORMAT_ASTC_3D_3X3X3      ; end    // 0x0EB => (0x48, 0x0)
            `IMG_FMT_ASTC_3D_LDR_4X3X3          : begin dataFormat = `IMG_DATA_FORMAT_ASTC_3D_LDR         ; numFormat = `IMG_NUM_FORMAT_ASTC_3D_4X3X3      ; end    // 0x0EC => (0x48, 0x1)
            `IMG_FMT_ASTC_3D_LDR_4X4X3          : begin dataFormat = `IMG_DATA_FORMAT_ASTC_3D_LDR         ; numFormat = `IMG_NUM_FORMAT_ASTC_3D_4X4X3      ; end    // 0x0ED => (0x48, 0x2)
            `IMG_FMT_ASTC_3D_LDR_4X4X4          : begin dataFormat = `IMG_DATA_FORMAT_ASTC_3D_LDR         ; numFormat = `IMG_NUM_FORMAT_ASTC_3D_4X4X4      ; end    // 0x0EE => (0x48, 0x3)
            `IMG_FMT_ASTC_3D_LDR_5X4X4          : begin dataFormat = `IMG_DATA_FORMAT_ASTC_3D_LDR         ; numFormat = `IMG_NUM_FORMAT_ASTC_3D_5X4X4      ; end    // 0x0EF => (0x48, 0x4)
            `IMG_FMT_ASTC_3D_LDR_5X5X4          : begin dataFormat = `IMG_DATA_FORMAT_ASTC_3D_LDR         ; numFormat = `IMG_NUM_FORMAT_ASTC_3D_5X5X4      ; end    // 0x0F0 => (0x48, 0x5)
            `IMG_FMT_ASTC_3D_LDR_5X5X5          : begin dataFormat = `IMG_DATA_FORMAT_ASTC_3D_LDR         ; numFormat = `IMG_NUM_FORMAT_ASTC_3D_5X5X5      ; end    // 0x0F1 => (0x48, 0x6)
            `IMG_FMT_ASTC_3D_LDR_6X5X5          : begin dataFormat = `IMG_DATA_FORMAT_ASTC_3D_LDR         ; numFormat = `IMG_NUM_FORMAT_ASTC_3D_6X5X5      ; end    // 0x0F2 => (0x48, 0x7)
            `IMG_FMT_ASTC_3D_LDR_6X6X5          : begin dataFormat = `IMG_DATA_FORMAT_ASTC_3D_LDR         ; numFormat = `IMG_NUM_FORMAT_ASTC_3D_6X6X5      ; end    // 0x0F3 => (0x48, 0x8)
            `IMG_FMT_ASTC_3D_LDR_6X6X6          : begin dataFormat = `IMG_DATA_FORMAT_ASTC_3D_LDR         ; numFormat = `IMG_NUM_FORMAT_ASTC_3D_6X6X6      ; end    // 0x0F4 => (0x48, 0x9)
            `IMG_FMT_ASTC_3D_HDR_3X3X3          : begin dataFormat = `IMG_DATA_FORMAT_ASTC_3D_HDR         ; numFormat = `IMG_NUM_FORMAT_ASTC_3D_3X3X3      ; end    // 0x0F5 => (0x49, 0x0)
            `IMG_FMT_ASTC_3D_HDR_4X3X3          : begin dataFormat = `IMG_DATA_FORMAT_ASTC_3D_HDR         ; numFormat = `IMG_NUM_FORMAT_ASTC_3D_4X3X3      ; end    // 0x0F6 => (0x49, 0x1)
            `IMG_FMT_ASTC_3D_HDR_4X4X3          : begin dataFormat = `IMG_DATA_FORMAT_ASTC_3D_HDR         ; numFormat = `IMG_NUM_FORMAT_ASTC_3D_4X4X3      ; end    // 0x0F7 => (0x49, 0x2)
            `IMG_FMT_ASTC_3D_HDR_4X4X4          : begin dataFormat = `IMG_DATA_FORMAT_ASTC_3D_HDR         ; numFormat = `IMG_NUM_FORMAT_ASTC_3D_4X4X4      ; end    // 0x0F8 => (0x49, 0x3)
            `IMG_FMT_ASTC_3D_HDR_5X4X4          : begin dataFormat = `IMG_DATA_FORMAT_ASTC_3D_HDR         ; numFormat = `IMG_NUM_FORMAT_ASTC_3D_5X4X4      ; end    // 0x0F9 => (0x49, 0x4)
            `IMG_FMT_ASTC_3D_HDR_5X5X4          : begin dataFormat = `IMG_DATA_FORMAT_ASTC_3D_HDR         ; numFormat = `IMG_NUM_FORMAT_ASTC_3D_5X5X4      ; end    // 0x0FA => (0x49, 0x5)
            `IMG_FMT_ASTC_3D_HDR_5X5X5          : begin dataFormat = `IMG_DATA_FORMAT_ASTC_3D_HDR         ; numFormat = `IMG_NUM_FORMAT_ASTC_3D_5X5X5      ; end    // 0x0FB => (0x49, 0x6)
            `IMG_FMT_ASTC_3D_HDR_6X5X5          : begin dataFormat = `IMG_DATA_FORMAT_ASTC_3D_HDR         ; numFormat = `IMG_NUM_FORMAT_ASTC_3D_6X5X5      ; end    // 0x0FC => (0x49, 0x7)
            `IMG_FMT_ASTC_3D_HDR_6X6X5          : begin dataFormat = `IMG_DATA_FORMAT_ASTC_3D_HDR         ; numFormat = `IMG_NUM_FORMAT_ASTC_3D_6X6X5      ; end    // 0x0FD => (0x49, 0x8)
            `IMG_FMT_ASTC_3D_HDR_6X6X6          : begin dataFormat = `IMG_DATA_FORMAT_ASTC_3D_HDR         ; numFormat = `IMG_NUM_FORMAT_ASTC_3D_6X6X6      ; end    // 0x0FE => (0x49, 0x9)
            `IMG_FMT_ASTC_3D_LDR_SRGB_3X3X3     : begin dataFormat = `IMG_DATA_FORMAT_ASTC_3D_LDR_SRGB    ; numFormat = `IMG_NUM_FORMAT_ASTC_3D_3X3X3      ; end    // 0x0FF => (0x4A, 0x0)
            `IMG_FMT_ASTC_3D_LDR_SRGB_4X3X3     : begin dataFormat = `IMG_DATA_FORMAT_ASTC_3D_LDR_SRGB    ; numFormat = `IMG_NUM_FORMAT_ASTC_3D_4X3X3      ; end    // 0x100 => (0x4A, 0x1)
            `IMG_FMT_ASTC_3D_LDR_SRGB_4X4X3     : begin dataFormat = `IMG_DATA_FORMAT_ASTC_3D_LDR_SRGB    ; numFormat = `IMG_NUM_FORMAT_ASTC_3D_4X4X3      ; end    // 0x101 => (0x4A, 0x2)
            `IMG_FMT_ASTC_3D_LDR_SRGB_4X4X4     : begin dataFormat = `IMG_DATA_FORMAT_ASTC_3D_LDR_SRGB    ; numFormat = `IMG_NUM_FORMAT_ASTC_3D_4X4X4      ; end    // 0x102 => (0x4A, 0x3)
            `IMG_FMT_ASTC_3D_LDR_SRGB_5X4X4     : begin dataFormat = `IMG_DATA_FORMAT_ASTC_3D_LDR_SRGB    ; numFormat = `IMG_NUM_FORMAT_ASTC_3D_5X4X4      ; end    // 0x103 => (0x4A, 0x4)
            `IMG_FMT_ASTC_3D_LDR_SRGB_5X5X4     : begin dataFormat = `IMG_DATA_FORMAT_ASTC_3D_LDR_SRGB    ; numFormat = `IMG_NUM_FORMAT_ASTC_3D_5X5X4      ; end    // 0x104 => (0x4A, 0x5)
            `IMG_FMT_ASTC_3D_LDR_SRGB_5X5X5     : begin dataFormat = `IMG_DATA_FORMAT_ASTC_3D_LDR_SRGB    ; numFormat = `IMG_NUM_FORMAT_ASTC_3D_5X5X5      ; end    // 0x105 => (0x4A, 0x6)
            `IMG_FMT_ASTC_3D_LDR_SRGB_6X5X5     : begin dataFormat = `IMG_DATA_FORMAT_ASTC_3D_LDR_SRGB    ; numFormat = `IMG_NUM_FORMAT_ASTC_3D_6X5X5      ; end    // 0x106 => (0x4A, 0x7)
            `IMG_FMT_ASTC_3D_LDR_SRGB_6X6X5     : begin dataFormat = `IMG_DATA_FORMAT_ASTC_3D_LDR_SRGB    ; numFormat = `IMG_NUM_FORMAT_ASTC_3D_6X6X5      ; end    // 0x107 => (0x4A, 0x8)
            `IMG_FMT_ASTC_3D_LDR_SRGB_6X6X6     : begin dataFormat = `IMG_DATA_FORMAT_ASTC_3D_LDR_SRGB    ; numFormat = `IMG_NUM_FORMAT_ASTC_3D_6X6X6      ; end    // 0x108 => (0x4A, 0x9)
            `IMG_FMT_MM_8_UNORM                 : begin dataFormat = `IMG_DATA_FORMAT_MM_8                ; numFormat = `IMG_NUM_FORMAT_UNORM              ; end    // 0x109 => (0x4C, 0x0)
            `IMG_FMT_MM_8_UINT                  : begin dataFormat = `IMG_DATA_FORMAT_MM_8                ; numFormat = `IMG_NUM_FORMAT_UINT               ; end    // 0x10A => (0x4C, 0x4)
            `IMG_FMT_MM_8_8_UNORM               : begin dataFormat = `IMG_DATA_FORMAT_MM_8_8              ; numFormat = `IMG_NUM_FORMAT_UNORM              ; end    // 0x10B => (0x4D, 0x0)
            `IMG_FMT_MM_8_8_UINT                : begin dataFormat = `IMG_DATA_FORMAT_MM_8_8              ; numFormat = `IMG_NUM_FORMAT_UINT               ; end    // 0x10C => (0x4D, 0x4)
            `IMG_FMT_MM_8_8_8_8_UNORM           : begin dataFormat = `IMG_DATA_FORMAT_MM_8_8_8_8          ; numFormat = `IMG_NUM_FORMAT_UNORM              ; end    // 0x10D => (0x4E, 0x0)
            `IMG_FMT_MM_8_8_8_8_UINT            : begin dataFormat = `IMG_DATA_FORMAT_MM_8_8_8_8          ; numFormat = `IMG_NUM_FORMAT_UINT               ; end    // 0x10E => (0x4E, 0x4)
            `IMG_FMT_MM_VYUY8_UNORM             : begin dataFormat = `IMG_DATA_FORMAT_MM_VYUY8            ; numFormat = `IMG_NUM_FORMAT_UNORM              ; end    // 0x10F => (0x4F, 0x0)
            `IMG_FMT_MM_VYUY8_UINT              : begin dataFormat = `IMG_DATA_FORMAT_MM_VYUY8            ; numFormat = `IMG_NUM_FORMAT_UINT               ; end    // 0x110 => (0x4F, 0x4)
            `IMG_FMT_MM_10_11_11_UNORM          : begin dataFormat = `IMG_DATA_FORMAT_MM_10_11_11         ; numFormat = `IMG_NUM_FORMAT_UNORM              ; end    // 0x111 => (0x50, 0x0)
            `IMG_FMT_MM_10_11_11_UINT           : begin dataFormat = `IMG_DATA_FORMAT_MM_10_11_11         ; numFormat = `IMG_NUM_FORMAT_UINT               ; end    // 0x112 => (0x50, 0x4)
            `IMG_FMT_MM_2_10_10_10_UNORM        : begin dataFormat = `IMG_DATA_FORMAT_MM_2_10_10_10       ; numFormat = `IMG_NUM_FORMAT_UNORM              ; end    // 0x113 => (0x51, 0x0)
            `IMG_FMT_MM_2_10_10_10_UINT         : begin dataFormat = `IMG_DATA_FORMAT_MM_2_10_10_10       ; numFormat = `IMG_NUM_FORMAT_UINT               ; end    // 0x114 => (0x51, 0x4)
            `IMG_FMT_MM_16_16_16_16_UNORM       : begin dataFormat = `IMG_DATA_FORMAT_MM_16_16_16_16      ; numFormat = `IMG_NUM_FORMAT_UNORM              ; end    // 0x115 => (0x52, 0x0)
            `IMG_FMT_MM_16_16_16_16_UINT        : begin dataFormat = `IMG_DATA_FORMAT_MM_16_16_16_16      ; numFormat = `IMG_NUM_FORMAT_UINT               ; end    // 0x116 => (0x52, 0x4)
            `IMG_FMT_MM_10_IN_16_UNORM          : begin dataFormat = `IMG_DATA_FORMAT_MM_10_IN_16         ; numFormat = `IMG_NUM_FORMAT_UNORM              ; end    // 0x117 => (0x53, 0x0)
            `IMG_FMT_MM_10_IN_16_UINT           : begin dataFormat = `IMG_DATA_FORMAT_MM_10_IN_16         ; numFormat = `IMG_NUM_FORMAT_UINT               ; end    // 0x118 => (0x53, 0x4)
            `IMG_FMT_MM_10_IN_16_16_UNORM       : begin dataFormat = `IMG_DATA_FORMAT_MM_10_IN_16_16      ; numFormat = `IMG_NUM_FORMAT_UNORM              ; end    // 0x119 => (0x54, 0x0)
            `IMG_FMT_MM_10_IN_16_16_UINT        : begin dataFormat = `IMG_DATA_FORMAT_MM_10_IN_16_16      ; numFormat = `IMG_NUM_FORMAT_UINT               ; end    // 0x11A => (0x54, 0x4)
            `IMG_FMT_MM_10_IN_16_16_16_16_UNORM : begin dataFormat = `IMG_DATA_FORMAT_MM_10_IN_16_16_16_16; numFormat = `IMG_NUM_FORMAT_UNORM              ; end    // 0x11B => (0x55, 0x0)
            `IMG_FMT_MM_10_IN_16_16_16_16_UINT  : begin dataFormat = `IMG_DATA_FORMAT_MM_10_IN_16_16_16_16; numFormat = `IMG_NUM_FORMAT_UINT               ; end    // 0x11C => (0x55, 0x4)
            `ifdef GC__MDR_10B_FLOAT_FORMAT__1
               `IMG_FMT_7E3_FLOAT                  : begin dataFormat = `IMG_DATA_FORMAT_7E3                 ; numFormat = `IMG_NUM_FORMAT_FLOAT              ; end    // 0x11D => (0x1E, 0x7)
            `endif
            `ifdef GC__VIDEO_YCBCR_FORMAT__1
               `IMG_FMT_YCBCR_UNORM                : begin dataFormat = `IMG_DATA_FORMAT_YCBCR               ; numFormat = `IMG_NUM_FORMAT_UNORM              ; end    // 0x11E => (0x1D, 0x0)
               `IMG_FMT_YCBCR_SNORM                : begin dataFormat = `IMG_DATA_FORMAT_YCBCR               ; numFormat = `IMG_NUM_FORMAT_SNORM              ; end    // 0x11F => (0x1D, 0x1)
               `IMG_FMT_YCBCR_USCALED              : begin dataFormat = `IMG_DATA_FORMAT_YCBCR               ; numFormat = `IMG_NUM_FORMAT_USCALED            ; end    // 0x120 => (0x1D, 0x2)
               `IMG_FMT_YCBCR_SSCALED              : begin dataFormat = `IMG_DATA_FORMAT_YCBCR               ; numFormat = `IMG_NUM_FORMAT_SSCALED            ; end    // 0x121 => (0x1D, 0x3)
               `IMG_FMT_YCBCR_UINT                 : begin dataFormat = `IMG_DATA_FORMAT_YCBCR               ; numFormat = `IMG_NUM_FORMAT_UINT               ; end    // 0x122 => (0x1D, 0x4)
               `IMG_FMT_YCBCR_SINT                 : begin dataFormat = `IMG_DATA_FORMAT_YCBCR               ; numFormat = `IMG_NUM_FORMAT_SINT               ; end    // 0x123 => (0x1D, 0x5)
               `IMG_FMT_YCBCR_SRGB                 : begin dataFormat = `IMG_DATA_FORMAT_YCBCR               ; numFormat = `IMG_NUM_FORMAT_SRGB               ; end    // 0x124 => (0x1D, 0x9)
            `endif
            //VCS coverage off
            default: begin
               dataFormat = `IMG_DATA_FORMAT_INVALID;
               numFormat = `IMG_NUM_FORMAT_UINT;

               `ifdef SIMULATION
                  `ifdef GFX_CONV_RSRC_DFMT__ASSERT_ON_RESERVED_NATIVE_INPUT
                     $error(" Error: gfx_conv_rsrc_dfmt received reserved native %s resource format 0x%01X", (isImage)?"image":"buffer", nativeSelectedFormat);
                  `endif
               `endif
            end
            //VCS coverage on
         endcase

      end
   end
endmodule
