// (c) 2016 Advanced Micro Devices, Inc.  All rights reserved.

module gfx_counter #(parameter MAX=8, //This is the max value that this counter can take and not the number of states.
`ifdef SIMULATION		     
		     SIMULATION_MAX=MAX, //This is the max value used by the assertions.
		     SIMULATION_COVER_MAX=MAX, //This value is used by the cover points.  It may be lower than the simulation_MAX since it is not always easy to determine the exact maximum value something will reach.  This is to show that testing is probably good enough.
`endif
                     BIT_WIDTH= (MAX < 2) ? 1 :
                                (MAX < 4) ? 2 :
                                (MAX < 8) ? 3 :
                                (MAX < 16) ? 4 :
                                (MAX < 32) ? 5 :
                                (MAX < 64) ? 6 :
                                (MAX < 128) ? 7 :
                                (MAX < 256) ? 8 :
                                (MAX < 512) ? 9 :
                                (MAX < 1024) ? 10 :
                                (MAX < 2048) ? 11 :
                                (MAX < 4096) ? 12 : 13,
		     RESET_VALUE={BIT_WIDTH{1'b0}}) (
    input clk,
    input reset,
                                           
    input inc,
    input dec,
                                           
    output reg [BIT_WIDTH-1:0] count
                   );

   //psl control_unknown: assert always !(isunknown({inc, dec})&& (reset===1'b0))@(posedge clk);
   
   always @(posedge clk) begin
      case({dec, inc})
	2'b01: count<=count+1'b1;
	2'b10: count<=count-1'b1;	
      endcase
      if (reset) begin
         count<=RESET_VALUE;
      end
   end
`ifdef SIMULATION
//psl underflow: assert always !(dec && !inc && ~|count && (reset===1'b0))@(posedge clk);
//psl overflow: assert always !(inc && !dec && (count>=SIMULATION_MAX) && (reset===1'b0))@(posedge clk);

//psl min_value: cover {~|count && (reset===1'b0)}@(posedge clk);
//psl max_value: cover {(count>=SIMULATION_COVER_MAX) && (reset===1'b0)}@(posedge clk);
`endif
   
endmodule


   
