//
//graphics debug mux Wrapper common module
//that instances the dbg_client_sync_mux
//
module gfx_dbg_mux #(parameter
                FLOPPED_INPUTS   = {63 {1'b0}}   )
   (
   input       ipclk         , // system clock

   // ==========================================================================
   // interface with _dbg_client_steer
   // Begin --------------------------------------------------------------------
   input                            dbg_active      , // Clock gater control
   input       [(4*9-1):0] dbg_mux_SelLane , // LANE_COUNT * ([8] Lane Enable; [7:6] Byte Select; [5:0] Channel Select;)
   output wire [(4*8-1):0] dbg_mux_Data    , // Debug-bus data outputs after muxing
   // End ----------------------------------------------------------------------

   // ==========================================================================
   // Debug-bus data inputs from functional logic
   // Begin --------------------------------------------------------------------
   input       [(4*8-1):0] idebug62 ,
   input       [(4*8-1):0] idebug61 ,
   input       [(4*8-1):0] idebug60 ,
   input       [(4*8-1):0] idebug59 ,
   input       [(4*8-1):0] idebug58 ,
   input       [(4*8-1):0] idebug57 ,
   input       [(4*8-1):0] idebug56 ,
   input       [(4*8-1):0] idebug55 ,
   input       [(4*8-1):0] idebug54 ,
   input       [(4*8-1):0] idebug53 ,
   input       [(4*8-1):0] idebug52 ,
   input       [(4*8-1):0] idebug51 ,
   input       [(4*8-1):0] idebug50 ,
   input       [(4*8-1):0] idebug49 ,
   input       [(4*8-1):0] idebug48 ,
   input       [(4*8-1):0] idebug47 ,
   input       [(4*8-1):0] idebug46 ,
   input       [(4*8-1):0] idebug45 ,
   input       [(4*8-1):0] idebug44 ,
   input       [(4*8-1):0] idebug43 ,
   input       [(4*8-1):0] idebug42 ,
   input       [(4*8-1):0] idebug41 ,
   input       [(4*8-1):0] idebug40 ,
   input       [(4*8-1):0] idebug39 ,
   input       [(4*8-1):0] idebug38 ,
   input       [(4*8-1):0] idebug37 ,
   input       [(4*8-1):0] idebug36 ,
   input       [(4*8-1):0] idebug35 ,
   input       [(4*8-1):0] idebug34 ,
   input       [(4*8-1):0] idebug33 ,
   input       [(4*8-1):0] idebug32 ,
   input       [(4*8-1):0] idebug31 ,
   input       [(4*8-1):0] idebug30 ,
   input       [(4*8-1):0] idebug29 ,
   input       [(4*8-1):0] idebug28 ,
   input       [(4*8-1):0] idebug27 ,
   input       [(4*8-1):0] idebug26 ,
   input       [(4*8-1):0] idebug25 ,
   input       [(4*8-1):0] idebug24 ,
   input       [(4*8-1):0] idebug23 ,
   input       [(4*8-1):0] idebug22 ,
   input       [(4*8-1):0] idebug21 ,
   input       [(4*8-1):0] idebug20 ,
   input       [(4*8-1):0] idebug19 ,
   input       [(4*8-1):0] idebug18 ,
   input       [(4*8-1):0] idebug17 ,
   input       [(4*8-1):0] idebug16 ,
   input       [(4*8-1):0] idebug15 ,
   input       [(4*8-1):0] idebug14 ,
   input       [(4*8-1):0] idebug13 ,
   input       [(4*8-1):0] idebug12 ,
   input       [(4*8-1):0] idebug11 ,
   input       [(4*8-1):0] idebug10 ,
   input       [(4*8-1):0] idebug9 ,
   input       [(4*8-1):0] idebug8 ,
   input       [(4*8-1):0] idebug7 ,
   input       [(4*8-1):0] idebug6 ,
   input       [(4*8-1):0] idebug5 ,
   input       [(4*8-1):0] idebug4 ,
   input       [(4*8-1):0] idebug3 ,
   input       [(4*8-1):0] idebug2 ,
   input       [(4*8-1):0] idebug1 ,
   input       [(4*8-1):0] idebug0 ,
   // End ----------------------------------------------------------------------

   output wire                      gclk          , // Gated clock for potential pipe stage
   input      [7:0]                 dbg_blkid     , // blk id for debug
   input                            dbg_rst       , // Combined global reset with command decoded reset
   input                            TEST_CLK_EN     // Gater override for test mode

   );

/////////////////////////////////////////////////////
//instantiate base module
/////////////////////////////////////////////////////

dbg_client_sync_mux #(
                            .LANE_COUNT     (4),
                            .INCL_DBG_DBG   (1),
                            .FLOPPED_INPUTS (FLOPPED_INPUTS)
                            )
                    u_dbg_client_sync_mux
           (
           //output
           .gclk                   (gclk),         //used by the block: Gated clock for potential debug bus pipe stage
           .dbg_Mux_Data           (dbg_mux_Data), //to dbg steer

           //input
           .ipclk                  (ipclk),        //connect to the local clock used by that block
           .dbg_active             (dbg_active),   //from dbg steer
           .dbg_Mux_SelLane        (dbg_mux_SelLane),  //from dbg steer
           .dbg_rst                (dbg_rst),          //from dbg steer
           .dbg_BlkId              (dbg_blkid),        //from dbg steer
           .TEST_CLK_EN            (TEST_CLK_EN),      // Hooked up by post-process

           .idebug0                (idebug0),
           .idebug1                (idebug1),
           .idebug2                (idebug2),
           .idebug3                (idebug3),
           .idebug4                (idebug4),
           .idebug5                (idebug5),
           .idebug6                (idebug6),
           .idebug7                (idebug7),
           .idebug8                (idebug8),
           .idebug9                (idebug9),
           .idebug10               (idebug10),
           .idebug11               (idebug11),
           .idebug12               (idebug12),
           .idebug13               (idebug13),
           .idebug14               (idebug14),
           .idebug15               (idebug15),
           .idebug16               (idebug16),
           .idebug17               (idebug17),
           .idebug18               (idebug18),
           .idebug19               (idebug19),
           .idebug20               (idebug20),
           .idebug21               (idebug21),
           .idebug22               (idebug22),
           .idebug23               (idebug23),
           .idebug24               (idebug24),
           .idebug25               (idebug25),
           .idebug26               (idebug26),
           .idebug27               (idebug27),
           .idebug28               (idebug28),
           .idebug29               (idebug29),
           .idebug30               (idebug30),
           .idebug31               (idebug31),
           .idebug32               (idebug32),
           .idebug33               (idebug33),
           .idebug34               (idebug34),
           .idebug35               (idebug35),
           .idebug36               (idebug36),
           .idebug37               (idebug37),
           .idebug38               (idebug38),
           .idebug39               (idebug39),
           .idebug40               (idebug40),
           .idebug41               (idebug41),
           .idebug42               (idebug42),
           .idebug43               (idebug43),
           .idebug44               (idebug44),
           .idebug45               (idebug45),
           .idebug46               (idebug46),
           .idebug47               (idebug47),
           .idebug48               (idebug48),
           .idebug49               (idebug49),
           .idebug50               (idebug50),
           .idebug51               (idebug51),
           .idebug52               (idebug52),
           .idebug53               (idebug53),
           .idebug54               (idebug54),
           .idebug55               (idebug55),
           .idebug56               (idebug56),
           .idebug57               (idebug57),
           .idebug58               (idebug58),
           .idebug59               (idebug59),
           .idebug60               (idebug60),
           .idebug61               (idebug61),
           .idebug62               (idebug62)
            );
`ifdef SIMULATION
// vrq translate_off
`ifdef VCS
//VCS coverage off
reg enable;

initial begin
  if($test$plusargs("dbg_clk_assert_enable"))
    enable = 1'b1;
  else
    enable = 1'b0;
end

reg clk;
    initial begin
    clk = 0;
    #5000;
        forever begin
            #5000 clk = 0;
            #5000 clk = 1;
        end
    end 
dbg_mux_static_clock_inactive : assert property (@(negedge clk) disable iff(~enable) (dbg_active ##1 dbg_active |-> ipclk));
//VCS coverage on
`endif
// vrq translate_on
`endif
endmodule
//////////////////////////////////////////////////////////////////////////////////////////////////////////
