//
//graphics debug Steer Wrapper common module
//that instances the dbg_client_sync_aon_steer and the dbg_client_sync_debug
//
module gfx_dbg_steer_wrapper(
   // Outputs
   DBUS_TARG_daisychain_ctl,
   TARG_DBUS_daisychain_data,
   dbg_Mux0_SelLane,
   dbg_Mux1_SelLane,
   dbg_Mux2_SelLane,
   dbg_Mux3_SelLane,
   dbg_Mux4_SelLane,
   dbg_Mux5_SelLane,
   dbg_rst,
   dbg_active,
   dbg_blkid,

   // Inputs
   TARG_DBUS_daisychain_ctl,
   DBUS_TARG_daisychain_data,
   dbg_Mux0_Data,
   dbg_Mux1_Data,
   dbg_Mux2_Data,
   dbg_Mux3_Data,
   dbg_Mux4_Data,
   dbg_Mux5_Data,

   rsmu_dbg_data,
   rsmu_dbg_sel,

   DbgBlkId_7_1,
   DbgBlkId_0,
   AEB_DBUS,
   Cpl_GFXCLK,
   Dft_Clk_Gater_Aon,
   Dft_Clk_Gater_En,
   dft_shift_clk,
   RCU_blk_hard_resetb
   );

////parameters
// 4 means for 32-bit wide debug bus (four 8-bit lanes)

//////////////////////////
//in/outs
//////////////////////////
 input        RCU_blk_hard_resetb;    //Global Synchronous reset, active low
 input        Cpl_GFXCLK;             //System Clock
 input        Dft_Clk_Gater_Aon;
 input        Dft_Clk_Gater_En;
 input        dft_shift_clk;
 input        AEB_DBUS;               //Security lock down. 1'b1-> hold program to reset == data forward

 input  [6:0] DbgBlkId_7_1;         //Debug Client ID bit 7:1, hard tie input. ID must be unique per Debug Unit
 input        DbgBlkId_0;           //Debug Client ID bit 0, hard tie input. ID must be unique per Debug Unit

 output       dbg_rst;          //reset and clear data, output to MUXF for each block
 output       dbg_active;       //Clock Gater term for data pipe, output to MUXF for each block
 output [7:0] dbg_blkid;        //Debug Client ID, to each block for test purpose

//two bits Debug Client serial programming interface. Driven from Debug Unit connected serially to 
//the first Debug Client at the beggining of the chain.
 input  [1:0] TARG_DBUS_daisychain_ctl;
 output [1:0] DBUS_TARG_daisychain_ctl;

//32-bit wide for graphics, carries data collected by the Debug Client. The Data flows from the first 
//Debug Client at the begining of the chain and ends at the Debug Unit, where the data is consumed.
 input  [(4*8-1) :0]  DBUS_TARG_daisychain_data;
 output [(4*8-1) :0]  TARG_DBUS_daisychain_data;

//this is for RSMU dbg bus only
//RSMU debug bus is 24-bit
 input         rsmu_dbg_sel;
 input  [31:0] rsmu_dbg_data;

//Debug bus data outputs after muxing for each MUXID from 0 to 4
 input  [(4*8-1) :0]  dbg_Mux0_Data;
 input  [(4*8-1) :0]  dbg_Mux1_Data;
 input  [(4*8-1) :0]  dbg_Mux2_Data;
 input  [(4*8-1) :0]  dbg_Mux3_Data;
 input  [(4*8-1) :0]  dbg_Mux4_Data;
 input  [(4*8-1) :0]  dbg_Mux5_Data;

//MUX select for each MUCID from 0 to 4
//LANE_COUNT * ([8] : Lane Enable; [7:6] Byte Select; [5:0] Channel Select);
 output [(4*9-1) :0]  dbg_Mux0_SelLane;
 output [(4*9-1) :0]  dbg_Mux1_SelLane;
 output [(4*9-1) :0]  dbg_Mux2_SelLane;
 output [(4*9-1) :0]  dbg_Mux3_SelLane;
 output [(4*9-1) :0]  dbg_Mux4_SelLane;
 output [(4*9-1) :0]  dbg_Mux5_SelLane;

/////internal wire declaration
 
 wire   [31:0]  dbg_Mux5_Data_tmp;
 wire   [31:0]  rsmu_dbg_Mux_Data;

 wire   [7:0]   dbg_blkid;
 wire           AEB_DBUS_SYNC;

 wire   [7:0]   DbgBlkId;

 wire          Cpl_GFXCLK_int;


gfx_cgtt_local_dft dbg_steer_wrapper_gfx_cgtt_local_dft (
   .oclk (Cpl_GFXCLK_int),
   .Cpl_GFXCLK (Cpl_GFXCLK), 
   .core_busy(1'b1),
   .dft_clk_gate_ctrl ({Dft_Clk_Gater_En,Dft_Clk_Gater_Aon}), 
   .dft_shift_clk (dft_shift_clk)
    );

 assign  DbgBlkId = {DbgBlkId_7_1, DbgBlkId_0};
 
/////////////////////////////////////////////////////
//instantiate base module
/////////////////////////////////////////////////////

//sync logic for AEB_DBUS
techind_sync_icd #(.depth(3)) aeb_dbus_sync (.CKdst(Cpl_GFXCLK_int), .Dsrc(AEB_DBUS), .Qdst(AEB_DBUS_SYNC),
                                .SB(1'b1), .RB(1'b1), .FLUSH(1'b0), .CKsrc(1'b0), .EN(1'b1), .Qsrc(),
                                .SC1(1'b0), .SC2(1'b0), .SE(1'b0), .SI(1'b0), .SO()
`ifdef ABV
                                ,.InputChkDis(1'b0)
`endif
);


assign  dbg_Mux5_Data_tmp = rsmu_dbg_sel ? rsmu_dbg_Mux_Data : dbg_Mux5_Data;

dbg_client_sync_mux_byte #(.LANE_COUNT(4),     // Number of Lanes, only 3 or 4 is currently supported
                           .LANE_WIDTH(8),     // Bit width of each Lane, only 8 or 16 is currently supported
                           .FLOPPED_OUTPUT(0), // flag to add flop after  the mux
                           .FLOPPED_INPUTS(0)) // flag to add flop before the mux
    rsmu_dbg_client_sync_mux_byte
       (.ipclk(1'b0),                             // system clock
        .dbg_active(1'b0),                        // Clock gater control
        .dbg_Mux_SelLane(dbg_Mux5_SelLane),       // LANE_COUNT * ([8] Lane Enable; [7:6] Byte Select; [5:0] Channel Select;)
        .dbg_Mux_Data(rsmu_dbg_Mux_Data),         // Debug-bus data outputs after muxing
        .idebug(rsmu_dbg_data),
        .dbg_rst(1'b0),                           // Combined global reset with command decoded reset
        .TEST_CLK_EN(1'b0));                      // Gater override for test mode

dbg_client_sync_aon_steer #(
                            .LANE_COUNT (4)
                            )
                          u_dbg_client_sync_aon_steer
           (
        //Outputs
            .DBUS_TARG_daisychain_ctl   (DBUS_TARG_daisychain_ctl),
            .TARG_DBUS_daisychain_data  (TARG_DBUS_daisychain_data),
            .dbg_active                 (dbg_active),
            .dbg_rst                    (dbg_rst),
            .dbg_BlkId                  (dbg_blkid),

            .dbg_Mux0_SelLane           (dbg_Mux0_SelLane),
            .dbg_Mux1_SelLane           (dbg_Mux1_SelLane),
            .dbg_Mux2_SelLane           (dbg_Mux2_SelLane),
            .dbg_Mux3_SelLane           (dbg_Mux3_SelLane),
            .dbg_Mux4_SelLane           (dbg_Mux4_SelLane),
            .dbg_Mux5_SelLane           (dbg_Mux5_SelLane),

        // Inputs
            .TARG_DBUS_daisychain_ctl   (TARG_DBUS_daisychain_ctl),
            .DBUS_TARG_daisychain_data  (DBUS_TARG_daisychain_data),

            .dbg_Mux0_Data              (dbg_Mux0_Data),
            .dbg_Mux1_Data              (dbg_Mux1_Data),
            .dbg_Mux2_Data              (dbg_Mux2_Data),
            .dbg_Mux3_Data              (dbg_Mux3_Data),
            .dbg_Mux4_Data              (dbg_Mux4_Data),
            .dbg_Mux5_Data              (dbg_Mux5_Data_tmp),

            .ipclk                      (Cpl_GFXCLK_int),        //System Clock input
            .DbgBlkId                   (DbgBlkId),
            .AEB_DBUS                   (AEB_DBUS_SYNC),
            .TEST_CLK_EN                (1'b0),
            .cold_resetb                (RCU_blk_hard_resetb)
            );

endmodule
//////////////////////////////////////////////////////////////////////////////////////////////////////////
