//
//graphics debug unit Wrapper common module
//that instances the dbgu_gfx that translates the TCIW protocol to CH protocol
//

`include "gfx_dbgu_wrapper_defs.vh"

module gfx_dbgu_wrapper(
    // I/O Ports
    input         CLK,
    input         Dft_Clk_Gater_Aon,
    input         Dft_Clk_Gater_En,
    input         dft_shift_clk,
    input         hard_resetb_resetb,
    input         Cpl_REFCLK,
    input         Cpl_PSEN,
    input         Aeb020,
    input         Aeb021,
    input         Aeb022,
    input         MiscBufferPassthrus_ZZZ_PGMEM_ZZZ,
    output        MiscBufferPassthrus_XXX_PGMEM_XXX,
    input  [`RSMU_DBGU_TIMESTAMP_GC_COUNT_RANGE] Gtsc_COUNT,
    input         Reg_send,
    input  [`GRBM_DBGU_REG_ADDR_RANGE] Reg_addr,
    input         Reg_op,
    input  [`GRBM_DBGU_REG_SSRCID_RANGE]  Reg_ssrcid,
    input  [`GRBM_DBGU_REG_WD_RANGE] Reg_wd,
    output        RegReadReturn_valid,
    output [`TARG_GRBM_READ_DATA_RANGE] RegReadReturn_data,
    input  [31:0] IpDebugBusA_data,
    output [1:0]  IpDebugBusA_ctl,
    input  [31:0] IpDebugBusB_data,
    output [1:0]  IpDebugBusB_ctl,
    input  [31:0] IpDebugBusC_data,
    output [1:0]  IpDebugBusC_ctl,
    input  [31:0] IpDebugBusD_data,
    output [1:0]  IpDebugBusD_ctl,
    input  [7:0]  CrossTrigger_SCF_DBGU_AsyncCrossTrigIn,
    output [7:0]  CrossTrigger_DBGU_SCF_CrossTrigOut,
    input  [11:0] IpTriggers_IP_DBGU_Triggers,
    output [1:0]  Actions_data,

// Port connectors for interface instance "CLIENT_CH_req" of type "CLIENT_CH_req.transmit".
    output [`CLIENT_CH_REQ_SEND_RANGE] CLIENT_CH_req_send,
    input  [`CLIENT_CH_REQ_FREE_RANGE] CLIENT_CH_req_free,
    output [`CLIENT_CH_REQ_CLKEN_RANGE] CLIENT_CH_req_clken,
    output [`CLIENT_CH_REQ_OP_RANGE] CLIENT_CH_req_op,
    output [`CLIENT_CH_REQ_ADDR_RANGE] CLIENT_CH_req_addr,
    output [`CLIENT_CH_REQ_SIZE_RANGE] CLIENT_CH_req_size,
    output [`CLIENT_CH_REQ_L2_POLICY_RANGE] CLIENT_CH_req_l2_policy,
    output [`CLIENT_CH_REQ_MTYPE_RANGE] CLIENT_CH_req_mtype,
    output [`CLIENT_CH_REQ_SNOOP_RANGE] CLIENT_CH_req_snoop,
    output [`CLIENT_CH_REQ_IO_RANGE] CLIENT_CH_req_io,
    output [`CLIENT_CH_REQ_VOL_RANGE] CLIENT_CH_req_vol,
    output [`CLIENT_CH_REQ_CID_RANGE] CLIENT_CH_req_cid,
    output [`CLIENT_CH_REQ_SPACE_RANGE] CLIENT_CH_req_space,
    output [`CLIENT_CH_REQ_NOFILL_RANGE] CLIENT_CH_req_nofill,
    output [`CLIENT_CH_REQ_PERF_CNTR_EN_RANGE] CLIENT_CH_req_perf_cntr_en,
    output [`CLIENT_CH_REQ_RD_TMZ_RANGE] CLIENT_CH_req_rd_tmz,
    output [`CLIENT_CH_REQ_WR_TMZ_RANGE] CLIENT_CH_req_wr_tmz,
    output [`CLIENT_CH_REQ_MEMLOG_RANGE] CLIENT_CH_req_memlog,
    output [`CLIENT_CH_REQ_BC_RANGE] CLIENT_CH_req_bc,
    output [`CLIENT_CH_REQ_RO_RANGE] CLIENT_CH_req_ro, // dkelling
    output [`CLIENT_CH_REQ_TAG_RANGE] CLIENT_CH_req_tag,

// Port connectors for interface instance "CLIENT_CH_src" of type "CLIENT_CH_src.transmit".
    output [`CLIENT_CH_SRC_VALID_RANGE] CLIENT_CH_src_valid,
    output [`CLIENT_CH_SRC_CLKEN_RANGE] CLIENT_CH_src_clken,
    output [`CLIENT_CH_SRC_SIZE_RANGE] CLIENT_CH_src_size,
    output [`CLIENT_CH_SRC_ADDR_LSB_RANGE] CLIENT_CH_src_addr_lsb,
    output [`CLIENT_CH_SRC_MASK_RANGE] CLIENT_CH_src_mask,
    output [`CLIENT_CH_SRC_DATA_RANGE] CLIENT_CH_src_data,
`ifdef SIMULATION
    output [`CLIENT_CH_SRC_TAG_RANGE] CLIENT_CH_src_tag,
    output [`CLIENT_CH_SRC_DBG_RANGE] CLIENT_CH_src_dbg,
    output [`CLIENT_CH_REQ_DBG_RANGE] CLIENT_CH_req_dbg,
    input  [`CH_CLIENT_32B_RET_DBG_RANGE] CH_CLIENT_ret_dbg,
`endif

// Port connectors for interface instance "CH_CLIENT_ret" of type "CH_CLIENT_32B_ret.receive".
    input  [`CH_CLIENT_32B_RET_VALID_RANGE] CH_CLIENT_ret_valid,
    input  [`CH_CLIENT_32B_RET_ACK_RANGE] CH_CLIENT_ret_ack,
    input  [`CH_CLIENT_32B_RET_SIZE_RANGE] CH_CLIENT_ret_size,
    input  [`CH_CLIENT_32B_RET_ADDR_LSBS_RANGE] CH_CLIENT_ret_addr_lsbs,
    input  [`CH_CLIENT_32B_RET_TAG_RANGE] CH_CLIENT_ret_tag,
    input  [`CH_CLIENT_32B_RET_DATA_RANGE] CH_CLIENT_ret_data

);  // End of I/O Ports

    wire          CLIENT_TCIW_req_send;
    wire          CLIENT_TCIW_req_free;
    wire  [6:0]   CLIENT_TCIW_req_op;
    wire  [3:0]   CLIENT_TCIW_req_vmid;
    wire  [43:6]  CLIENT_TCIW_req_addr;
    wire  [9:0]   CLIENT_TCIW_req_tag;
    wire          CLIENT_TCIW_req_xfer32B;
    wire          CLIENT_TCIW_req_which32B;
    wire  [31:0]  CLIENT_TCIW_req_mask;
    wire  [255:0] CLIENT_TCIW_req_data;
    wire          CLIENT_TCIW_req_clken;
    wire          CLIENT_TCIW_req_policy;
    wire  [1:0]   CLIENT_TCIW_req_mtype;
    wire          CLIENT_TCIW_req_snoop;
    wire          CLIENT_TCIW_req_fed;
    wire          CLIENT_TCIW_req_io;
    wire          CLIENT_TCIW_req_space;
    wire          CLIENT_TCIW_req_perf_cntr_en;
    wire          CLIENT_TCIW_req_rd_TMZ;
    wire          CLIENT_TCIW_req_wr_TMZ;
    wire          CLIENT_TCIW_req_memlog;
    wire          CLIENT_TCIW_req_drop;
    wire          CLIENT_TCIW_req_return_burst;
    wire  [2:0]   CLIENT_TCIW_req_cid;
    wire          TCIW_CLIENT_ret_valid;
    wire  [255:0] TCIW_CLIENT_ret_data;
    wire  [9:0]   TCIW_CLIENT_ret_tag;
    wire          TCIW_CLIENT_ret_ack;
    wire          TCIW_CLIENT_ret_fed;
    wire          TCIW_CLIENT_ret_xfer32B;
    wire          TCIW_CLIENT_ret_which32B;

    reg           CLIENT_TCIW_req_send_reg;

    wire          Cpl_GFXCLK_int;


gfx_cgtt_local_dft dbgu_wrapper_gfx_cgtt_local_dft (
   .oclk(Cpl_GFXCLK_int),
   .core_busy(1'b1),
   .Cpl_GFXCLK (CLK), 
   .dft_clk_gate_ctrl ({Dft_Clk_Gater_En,Dft_Clk_Gater_Aon}), 
   .dft_shift_clk (dft_shift_clk)
    );

    always @(posedge Cpl_GFXCLK_int)
    begin
      CLIENT_TCIW_req_send_reg <= CLIENT_TCIW_req_send;
    end
                      
    assign  CLIENT_CH_req_send    = CLIENT_TCIW_req_send & !CLIENT_TCIW_req_send_reg;
    assign  CLIENT_TCIW_req_free  = CLIENT_CH_req_free;
    assign  CLIENT_CH_req_clken   = CLIENT_TCIW_req_clken;
    assign  CLIENT_CH_req_compressed = 1'b0;
    assign  CLIENT_CH_req_op      = CLIENT_TCIW_req_op;
    assign  CLIENT_CH_req_addr    = {CLIENT_TCIW_req_addr[43:6], 1'b0};
    assign  CLIENT_CH_req_size    = 2'b01;
    assign  CLIENT_CH_req_l2_policy = 2'd0;
    assign  CLIENT_CH_req_mtype   = {1'b0, CLIENT_TCIW_req_mtype};
    assign  CLIENT_CH_req_snoop   = CLIENT_TCIW_req_snoop;
    assign  CLIENT_CH_req_io      = CLIENT_TCIW_req_io;
    assign  CLIENT_CH_req_vol     = 1'b0;
    assign  CLIENT_CH_req_cid     = {1'b0, CLIENT_TCIW_req_cid};
    assign  CLIENT_CH_req_space   = 1'b0;
    assign  CLIENT_CH_req_nofill  = 1'b0;
    assign  CLIENT_CH_req_perf_cntr_en = CLIENT_TCIW_req_perf_cntr_en;
    assign  CLIENT_CH_req_rd_tmz  = CLIENT_TCIW_req_rd_TMZ;
    assign  CLIENT_CH_req_wr_tmz  = CLIENT_TCIW_req_wr_TMZ;
    assign  CLIENT_CH_req_memlog  = CLIENT_TCIW_req_memlog;
    assign  CLIENT_CH_req_bc      = 0;
    assign  CLIENT_CH_req_ro      = 0; // dkelling
    assign  CLIENT_CH_req_tag     = {{`CLIENT_CH_REQ_TAG_WIDTH-10{1'b0}}, CLIENT_TCIW_req_tag};

    assign  CLIENT_CH_src_valid   = CLIENT_TCIW_req_send;
    assign  CLIENT_CH_src_clken   = CLIENT_TCIW_req_clken;
    assign  CLIENT_CH_src_size    = 2'b01;
    assign  CLIENT_CH_src_addr_lsb = CLIENT_CH_req_addr[5]; //FIXME
    assign  CLIENT_CH_src_mask    = CLIENT_TCIW_req_mask;
    assign  CLIENT_CH_src_data    = CLIENT_TCIW_req_data;
`ifdef SIMULAITON
    assign CLIENT_CH_src_tag = 'b0;
    assign CLIENT_CH_src_dbg = 'b0;
    assign CLIENT_CH_req_dbg = 'b0;
`endif

    assign  TCIW_CLIENT_ret_valid = CH_CLIENT_ret_valid;
    assign  TCIW_CLIENT_ret_data  = CH_CLIENT_ret_data;
    assign  TCIW_CLIENT_ret_ack   = CH_CLIENT_ret_ack;
    assign  TCIW_CLIENT_ret_tag   = CH_CLIENT_ret_tag[9:0];
    assign  TCIW_CLIENT_ret_fed   = 1'b0;
    assign  TCIW_CLIENT_ret_xfer32B  = 1'b0;
    assign  TCIW_CLIENT_ret_which32B = 1'b0;

    dbgu_gfx_wrap u_dbgu_gfx_wrap(
        .CLK(Cpl_GFXCLK_int)
        ,.hard_resetb_resetb(hard_resetb_resetb)
        ,.CLIENT_TCIW_req_send(CLIENT_TCIW_req_send)
        ,.CLIENT_TCIW_req_free(CLIENT_TCIW_req_free)
        ,.CLIENT_TCIW_req_op(CLIENT_TCIW_req_op)
        ,.CLIENT_TCIW_req_addr(CLIENT_TCIW_req_addr)
        ,.CLIENT_TCIW_req_tag(CLIENT_TCIW_req_tag)
        ,.CLIENT_TCIW_req_xfer32B(CLIENT_TCIW_req_xfer32B)
        ,.CLIENT_TCIW_req_which32B(CLIENT_TCIW_req_which32B)
        ,.CLIENT_TCIW_req_mask(CLIENT_TCIW_req_mask)
        ,.CLIENT_TCIW_req_data(CLIENT_TCIW_req_data)
        ,.CLIENT_TCIW_req_clken(CLIENT_TCIW_req_clken)
        ,.CLIENT_TCIW_req_policy(CLIENT_TCIW_req_policy)
        ,.CLIENT_TCIW_req_mtype(CLIENT_TCIW_req_mtype)
        ,.CLIENT_TCIW_req_snoop(CLIENT_TCIW_req_snoop)
        ,.CLIENT_TCIW_req_fed(CLIENT_TCIW_req_fed)
        ,.CLIENT_TCIW_req_io(CLIENT_TCIW_req_io)
        ,.CLIENT_TCIW_req_rd_TMZ(CLIENT_TCIW_req_rd_TMZ)
        ,.CLIENT_TCIW_req_wr_TMZ(CLIENT_TCIW_req_wr_TMZ)
        ,.CLIENT_TCIW_req_memlog(CLIENT_TCIW_req_memlog)
        ,.CLIENT_TCIW_req_space(CLIENT_TCIW_req_space)
        ,.CLIENT_TCIW_req_perf_cntr_en(CLIENT_TCIW_req_perf_cntr_en)
        ,.CLIENT_TCIW_req_drop(CLIENT_TCIW_req_drop)
        ,.CLIENT_TCIW_req_return_burst(CLIENT_TCIW_req_return_burst)
        ,.CLIENT_TCIW_req_cid(CLIENT_TCIW_req_cid)
        ,.TCIW_CLIENT_ret_valid(TCIW_CLIENT_ret_valid)
        ,.TCIW_CLIENT_ret_data(TCIW_CLIENT_ret_data)
        ,.TCIW_CLIENT_ret_tag(TCIW_CLIENT_ret_tag)
        ,.TCIW_CLIENT_ret_ack(TCIW_CLIENT_ret_ack)
        ,.TCIW_CLIENT_ret_fed(TCIW_CLIENT_ret_fed)
        ,.TCIW_CLIENT_ret_xfer32B(TCIW_CLIENT_ret_xfer32B)
        ,.TCIW_CLIENT_ret_which32B(TCIW_CLIENT_ret_which32B)
        ,.Gtsc_COUNT(Gtsc_COUNT)
        ,.Actions_data(Actions_data)
        ,.Reg_send(Reg_send)
        ,.Reg_addr(Reg_addr)
        ,.Reg_op(Reg_op)
        ,.Reg_ssrcid(Reg_ssrcid)
        ,.Reg_wd(Reg_wd)
        ,.IpDebugBusA_data(IpDebugBusA_data)
        ,.IpDebugBusA_ctl(IpDebugBusA_ctl)
        ,.IpDebugBusB_data(IpDebugBusB_data)
        ,.IpDebugBusB_ctl(IpDebugBusB_ctl)
        ,.IpDebugBusC_data(IpDebugBusC_data)
        ,.IpDebugBusC_ctl(IpDebugBusC_ctl)
        ,.IpDebugBusD_data(IpDebugBusD_data)
        ,.IpDebugBusD_ctl(IpDebugBusD_ctl)
        ,.RegReadReturn_valid(RegReadReturn_valid)
        ,.RegReadReturn_data(RegReadReturn_data)
        ,.IpTriggers_IP_DBGU_Triggers({20'd0, IpTriggers_IP_DBGU_Triggers})
        ,.Aeb020(Aeb020)
        ,.Aeb021(Aeb021)
        ,.Aeb022(Aeb022)
        ,.MiscBufferPassthrus_ZZZ_PGMEM_ZZZ(MiscBufferPassthrus_ZZZ_PGMEM_ZZZ)
        ,.MiscBufferPassthrus_XXX_PGMEM_XXX(MiscBufferPassthrus_XXX_PGMEM_XXX)
        ,.mem_power_ctrl(42'd0)
        ,.Cpl_REFCLK(Cpl_REFCLK)
        ,.Cpl_PSEN(Cpl_PSEN)
        ,.CrossTrigger_SCF_DBGU_AsyncCrossTrigIn(CrossTrigger_SCF_DBGU_AsyncCrossTrigIn)
        ,.CrossTrigger_DBGU_SCF_CrossTrigOut(CrossTrigger_DBGU_SCF_CrossTrigOut)
    );

endmodule
//////////////////////////////////////////////////////////////////////////////////////////////////////////
