/*
 *  Trade secret of AMD, Inc.
 *  Copyright 2012, AMD, Inc., (unpublished)
 *
 *  All rights reserved.  This notice is intended as a precaution against
 *  inadvertent publication and does not imply publication or any waiver
 *  of confidentiality.  The year included in the foregoing notice is the
 *  year of creation of the work.
 *
 *  NOTE: This *must* be instanced in the always-on power domain and attached to the static clk
 *
 *  Common module for DSM (Debug State Machine) ECC irritator bus
*/

module gfx_ecc_irritator (
    //input
    clk,
    reset,
    DSM_BLK_irritator_data,
    blk_dsm_cntl,
    blk_dsm_cntl2,
    inject_delay,
    disable_ecc,
    write_en,
    read_en,
    fed_in,
    //output
    fed_error_inject,
    single_error_inject,
    uc_error_inject
);


   input                                   clk;
   input                                   reset;
   input [1:0]                             DSM_BLK_irritator_data; 
   input [2:0]                             blk_dsm_cntl; 
   input [2:0]                             blk_dsm_cntl2;
   input [5:0]                             inject_delay;
   input                                   disable_ecc;
   input                                   write_en;
   input                                   read_en;
   input                                   fed_in;

   output                                  fed_error_inject;
   output                                  single_error_inject;
   output                                  uc_error_inject;


   reg                reset_q;
   wire               internal_reset;
   wire               not_disc_edc;

   wire               blk_irritator;
   reg                continuous_irritation;
   integer            i;
   integer            j;

   reg   [5:0]        down_cnt_1;
   wire               down_cnt_1_en;
   wire               down_cnt_1_zero;
   wire               delayed_irritation;
   wire               delayed_irritation_2;

   reg                delayed_irritation_pipe;
   wire               delayed_irritation_rise_edge;
   reg                sticky_irritation;
   wire               write_read_selected;
   wire               internal_irritation;

   reg   [2:0]        down_cnt_2;
   wire               down_cnt_2_en;
   wire               down_cnt_2_not_zero;
   wire               last7_irritation;


   always @ (posedge clk) begin
     reset_q <= reset;
   end
   //assign internal_reset = reset | disable_ecc | (blk_dsm_cntl[1:0] == 0);
   assign internal_reset = reset_q | (blk_dsm_cntl[1:0] == 0);
   assign not_disc_edc = !disable_ecc;



   //generate continuous_irritation
   assign blk_irritator = (blk_dsm_cntl[0] & DSM_BLK_irritator_data[0]) | (blk_dsm_cntl[1] & DSM_BLK_irritator_data[1]);

   always @ (posedge clk) begin
     continuous_irritation <= blk_irritator;
   end



   //generate delayed_irritation
   assign down_cnt_1_en = write_read_selected & continuous_irritation & !down_cnt_1_zero;
   assign down_cnt_1_zero = (down_cnt_1 == 0);

   always @ (posedge clk) begin
   if(internal_reset)
     down_cnt_1 <= inject_delay;
   else if(down_cnt_1_en)
     down_cnt_1 <= down_cnt_1 - 1'b1;
   end

   assign delayed_irritation = blk_dsm_cntl2[2] ? (down_cnt_1_zero & continuous_irritation) : continuous_irritation;
   assign delayed_irritation_2 = delayed_irritation & !internal_reset;


   //generate internal_irritation
   always @ (posedge clk) begin
     delayed_irritation_pipe <= delayed_irritation_2;
   end

   assign delayed_irritation_rise_edge = delayed_irritation_2 & !delayed_irritation_pipe;
   assign write_read_selected = (blk_dsm_cntl2[1:0] == 2'b00) ? write_en : read_en;

   always @ (posedge clk) begin
   if(internal_reset | !blk_dsm_cntl[2])
     sticky_irritation <= 0;
   else if(delayed_irritation_rise_edge)
     sticky_irritation <= 1;
   else if(sticky_irritation & write_read_selected)
     sticky_irritation <= 0;
   end

   assign internal_irritation = blk_dsm_cntl[2] ? sticky_irritation : delayed_irritation_2;



   //generate last7_irritation
   assign down_cnt_2_en = read_en & internal_irritation & down_cnt_2_not_zero;
   assign down_cnt_2_not_zero = !(down_cnt_2 == 0);

   always @ (posedge clk) begin
   if(internal_reset)
     down_cnt_2 <= 3'b111;
   else if(down_cnt_2_en)
     down_cnt_2 <= down_cnt_2 - 1'b1;
   end

   assign last7_irritation = internal_irritation & down_cnt_2_not_zero;


   
   //generate output signals
   assign fed_error_inject    = fed_in | ((blk_dsm_cntl2[1:0] == 2'b00) & internal_irritation);
   assign single_error_inject = (blk_dsm_cntl2[1:0] == 2'b01) & internal_irritation;
   assign uc_error_inject     = ((blk_dsm_cntl2[1:0] == 2'b10) & internal_irritation & not_disc_edc) |
                                ((blk_dsm_cntl2[1:0] == 2'b11) & last7_irritation & not_disc_edc);



endmodule
