/*
 *  Trade secret of AMD, Inc.
 *  Copyright 2012, AMD, Inc., (unpublished)
 *
 *  All rights reserved.  This notice is intended as a precaution against
 *  inadvertent publication and does not imply publication or any waiver
 *  of confidentiality.  The year included in the foregoing notice is the
 *  year of creation of the work.
 *
 *  NOTE: This *must* be instanced in the always-on power domain and attached to the static clk
 *
 *  Common module for DSM (Debug State Machine) ECC irritator bus
*/
`include "gc_defs.vh"

module gfx_ecc_irritator_wrapper (
    //input
    clk,
    reset,
    DSM_BLK_irritator_data,
    blk_dsm_cntl,
    blk_dsm_cntl2,
    inject_delay,
    disable_ecc,
    write_en,
    read_en,
    fed_in,
    //output
    BLK_TARG_irritator_data,
    fed_error_inject,
    single_error_inject,
    uc_error_inject
);

   parameter                               BLK_IRRITATORS = 1;
   parameter                               BLK_DSM_CNTL_WIDTH = BLK_IRRITATORS * 3;

   input                                   clk;
   input                                   reset;
   input [`DSM_TARG_IRRITATOR_DATA_RANGE]  DSM_BLK_irritator_data; 
   input [BLK_DSM_CNTL_WIDTH-1:0]          blk_dsm_cntl; 
   input [BLK_DSM_CNTL_WIDTH-1:0]          blk_dsm_cntl2;
   input [5:0]                             inject_delay;
   input                                   disable_ecc;
   input [BLK_IRRITATORS-1:0]              write_en;
   input [BLK_IRRITATORS-1:0]              read_en;
   input [BLK_IRRITATORS-1:0]              fed_in;

   output [`DSM_TARG_IRRITATOR_DATA_RANGE] BLK_TARG_irritator_data; // Top-level irritator chain output
   output [BLK_IRRITATORS-1:0]             fed_error_inject;
   output [BLK_IRRITATORS-1:0]             single_error_inject;
   output [BLK_IRRITATORS-1:0]             uc_error_inject;


   reg [`DSM_TARG_IRRITATOR_DATA_RANGE]  BLK_TARG_irritator_data;
   reg [`DSM_TARG_IRRITATOR_DATA_RANGE]  internal_irritator_data; 
   always @ (posedge clk) begin
      internal_irritator_data <= DSM_BLK_irritator_data;
      BLK_TARG_irritator_data <= internal_irritator_data;
   end

   generate
     genvar i;
     for (i=0; i<BLK_IRRITATORS; i=i+1)
     begin: generate_gfx_ecc_irritator
       gfx_ecc_irritator gfx_ecc_irritator_u (
       //input
       .clk                     (clk),
       .reset                   (reset),
       .DSM_BLK_irritator_data  (internal_irritator_data[1:0]),
       .blk_dsm_cntl            (blk_dsm_cntl[i*3+2:i*3]),
       .blk_dsm_cntl2           (blk_dsm_cntl2[i*3+2:i*3]),
       .inject_delay            (inject_delay),
       .disable_ecc             (disable_ecc),
       .write_en                (write_en[i]),
       .read_en                 (read_en[i]),
       .fed_in                  (fed_in[i]),
       //output
       .fed_error_inject        (fed_error_inject[i]),
       .single_error_inject     (single_error_inject[i]),
       .uc_error_inject         (uc_error_inject[i])
       );
     end
   endgenerate



endmodule
