`include "tech_features.vh"


`ifdef TECH__TECH__TSMC7N

module gfx_kc_crossbar_mux_GV #(SRC_WORDS = 32, DST_WORDS = 32, WIDTH = 32, SEL_BUF_SPLIT = 8, DATA_BUF_SPLIT = 8) (
    input [DST_WORDS-1:0][SRC_WORDS-1:0] DstSel,
    input [SRC_WORDS-1:0][WIDTH-1:0] SrcData,
    output logic [DST_WORDS-1:0][WIDTH-1:0] DstData
);


// this is a little counter-intuitive
// SrcData is buffered across destinatiion reads
// DestSel is buffered across src bits

logic [WIDTH-1:0][SRC_WORDS-1:0] SrcDataT;

gfx_kc_transpose2d #(SRC_WORDS, WIDTH) I_SrcDataT(SrcData[SRC_WORDS-1:0], SrcDataT[WIDTH-1:0]);

genvar gi;
genvar gj;

// generate all the rd trees
generate
    for (gi=0; gi<DST_WORDS; gi++) begin : words
        for (gj=0; gj<WIDTH; gj++) begin : bits
            // read across all src words at bit
            gfx_kc_rdport_GV #(SRC_WORDS, 1, 1) I_Read (
                                 DstSel[gi][SRC_WORDS-1:0],
                                 SrcDataT[gj][SRC_WORDS-1:0], 
                                 DstData[gi][gj]);
        end
    end
endgenerate

endmodule

`endif


`ifdef KC_SIMULATION

module gfx_kc_crossbar_mux_GV #(SRC_WORDS = 32, DST_WORDS = 32, WIDTH = 32, SEL_BUF_SPLIT = 8, DATA_BUF_SPLIT = 8) (
    input [DST_WORDS-1:0][SRC_WORDS-1:0] DstSel,
    input [SRC_WORDS-1:0][WIDTH-1:0] SrcData,
    output logic [DST_WORDS-1:0][WIDTH-1:0] DstData
);

genvar gi;
generate
    for (gi=0; gi<DST_WORDS; gi++) begin : words
        gfx_kc_hotmux #(WIDTH, SRC_WORDS, 0) I_Read(DstData[gi][WIDTH-1:0], DstSel[gi][SRC_WORDS-1:0], SrcData[SRC_WORDS-1:0]);
    end
endgenerate

endmodule

`endif

