
/* PGFSM block inserted in each tile that can be powergated */

`include "gfx_pgfsm_defines.h"
`include "gc_features.vh"
`include "mem_features.vh"

module gfx_pgfsm (
   PGFSM_iso_clock_1,
   PGFSM_iso_clock_2,
   PGFSM_iso_clampn_clock_1,
   PGFSM_iso_clampn_clock_2,
   PGFSM_cmd_done,

   PGFSM_SD_prev_daisy_status,
   PGFSM_SD_next_daisy_cmd,
   PGFSM_P1_mother_sleep,
   PGFSM_P1_daughter_sleep,
   PGFSM_P1_iso_clampn,
   PGFSM_P1_scan_iso_clampn,
   PGFSM_P1_resetb,
   PGFSM_MEM_P1_power_ctrl,
   PGFSM_PFH_P1_resetb,

   // Inputs
   PGFSM_SD_prev_daisy_cmd,
   PGFSM_SD_next_daisy_status,
   PGFSM_tile_ref_clk,
   PGFSM_tile_sclk,
   PGFSM_tile_hard_resetb,
   PGFSM_tile_cold_resetb,
   PGFSM_tile_local_address,
   PGFSM_in_MEM_P1_power_ctrl,

   //PGFSM-DFT interface
   PGFSM_DFT_tdr_tdi,
   PGFSM_DFT_tdr_tresetb,
   PGFSM_DFT_tdr_shift,
   PGFSM_DFT_tdr_capture,
   PGFSM_DFT_tdr_update,
   PGFSM_DFT_tdr_en_rw,
   PGFSM_DFT_tdr_en_ro,
   PGFSM_DFT_tdr_tdo_rw,
   PGFSM_DFT_tdr_tdo_ro,
   PGFSM_DFT_tdr_tck,


   TEST_CLK_EN,
   Cpl_PSEN

   ); 

   `include "gfx_pgfsm_params.vh"

   // SerDes interface signals. 
   input                                     PGFSM_SD_prev_daisy_cmd; 
   output                                    PGFSM_SD_prev_daisy_status; 
   
   output                                    PGFSM_SD_next_daisy_cmd; 
   input                                     PGFSM_SD_next_daisy_status; 
      
   input                                     PGFSM_tile_ref_clk; 
   input                                     PGFSM_tile_sclk; 
   input                                     PGFSM_tile_hard_resetb; 
   input                                     PGFSM_tile_cold_resetb; 
   
   // Address of the PGFSM 
   input [7:0]                               PGFSM_tile_local_address; 

   output                                    PGFSM_cmd_done;           //Pulses when a FSM finishes current command execution
   input                                     PGFSM_iso_clock_1;        //System tile clocks like dclk, dispclk
   input                                     PGFSM_iso_clock_2;        //System tile clocks like dclk, dispclk
   output                                    PGFSM_iso_clampn_clock_1; //Synchronized iso control for clock 1 
   output                                    PGFSM_iso_clampn_clock_2; //Synchronized iso control for clock 2 

   // P1 memory control for the light sleep 
   input [MEM__MEM_POWER_CTRL-1:0]    PGFSM_in_MEM_P1_power_ctrl; 
   
   // Powergate controls
   output                                    PGFSM_P1_mother_sleep; 
   output                                    PGFSM_P1_daughter_sleep; 
   output                                    PGFSM_P1_iso_clampn; 
   output                                    PGFSM_P1_scan_iso_clampn; 
   output                                    PGFSM_P1_resetb; 
   output [MEM__MEM_POWER_CTRL-1:0]   PGFSM_MEM_P1_power_ctrl;
   output				     PGFSM_PFH_P1_resetb; 

   input PGFSM_DFT_tdr_tdi;
   input PGFSM_DFT_tdr_tresetb;
   input PGFSM_DFT_tdr_shift;
   input PGFSM_DFT_tdr_capture;
   input PGFSM_DFT_tdr_update;
   input PGFSM_DFT_tdr_en_rw;
   input PGFSM_DFT_tdr_en_ro;
   output PGFSM_DFT_tdr_tdo_rw;
   output PGFSM_DFT_tdr_tdo_ro;
   input PGFSM_DFT_tdr_tck;


   input TEST_CLK_EN;

   // Power switch control input from analog pad
   input Cpl_PSEN;
   
   //--------------------------------------------------------------------------------

     
   //--------------------------------------------------------------------------------
   // SERDES Command Serializer
   //--------------------------------------------------------------------------------

   //resets
   wire cold_resetb_sync;
   techind_sync_icd #(.depth(3)) cold_rst_sync  (.CKdst(PGFSM_tile_sclk), .Dsrc(PGFSM_tile_cold_resetb), .Qdst(cold_resetb_sync),
                                    .SB(1'b1), .RB(1'b1), .FLUSH(1'b0), .CKsrc(1'b0), .EN(1'b1), .Qsrc(),
                                    .SC1(1'b0), .SC2(1'b0), .SE(1'b0), .SI(1'b0), .SO()
   `ifdef ABV
                                    ,.InputChkDis(1'b0)
   `endif
   );
   wire reset  = ~cold_resetb_sync;
   
   // Flopping serdes command input
   reg PGFSM_SD_prev_daisy_cmd_reg;    
   always @(posedge PGFSM_tile_sclk) begin
      if (reset) begin
         PGFSM_SD_prev_daisy_cmd_reg  <= 1'b0; 
      end else begin
         PGFSM_SD_prev_daisy_cmd_reg  <= PGFSM_SD_prev_daisy_cmd; 
      end
   end
   assign  PGFSM_SD_next_daisy_cmd = PGFSM_SD_prev_daisy_cmd_reg; 
   
   wire [31:0] SERDES_cmd; 
   wire [31:0] SERDES_wdata; 
   wire SERDES_new_cmd; 

   // Deserializer for control, wdata   
   gfx_pgfsm_deser #(64,6) deserializer  (
       .data_out({SERDES_wdata,SERDES_cmd}),
       .new_data(SERDES_new_cmd),
       .serial_in(PGFSM_SD_prev_daisy_cmd_reg),
       .iclk(PGFSM_tile_sclk),
       .reset(reset),
       .hard_resetb(1'b1)
   ); 
   
   
   //--------------------------------------------------------------------------------
   // PGFSM core
   //--------------------------------------------------------------------------------

   wire PGFSM_P1_mother_sleep_pre;
   wire PGFSM_P1_daughter_sleep_pre;
   gfx_pgfsm_core #( 
                 `include "gfx_pgfsm_params_override.vh"
               ) pgfsm_core (
               // Inputs
	       .TEST_CLK_EN			(TEST_CLK_EN),
               .PGFSM_sel 			(1'b1),					//There is only one Power Domain, so it is always selected. Setting this to '1 allow us not to require any change on other IPs when writing/reading PGFSM registers, meaning they won't have to set P1_Select for writes/reads. Only tile_top is required to do this since there is two PGFSMs instantiated in CU tiles, so they have to select the target pgfsm for the register write/read.
               .PGFSM_cmd			(SERDES_cmd),
               .PGFSM_wdata			(SERDES_wdata),
               .PGFSM_new_cmd			(SERDES_new_cmd),
	      
               // Clocks/reset
               .PGFSM_tile_ref_clk                   (PGFSM_tile_ref_clk),
               .PGFSM_tile_sclk                        (PGFSM_tile_sclk),		//Previously PGFSM_tile_iclk
               .PGFSM_tile_hard_resetb            (PGFSM_tile_hard_resetb),
               .PGFSM_tile_cold_resetb            (PGFSM_tile_cold_resetb),

               // Address for PGFSM in daisy chain
               .PGFSM_tile_local_address          (PGFSM_tile_local_address),

		//SERDES input
               .PGFSM_SD_next_daisy_status        (PGFSM_SD_next_daisy_status),

               .PGFSM_in_MEM_PD_power_ctrl        (PGFSM_in_MEM_P1_power_ctrl),
               .PGFSM_CGLS_override		  (1'b0),	//tied off for non GFX tiles

               // Outputs

               .PGFSM_SD_prev_daisy_status        (PGFSM_SD_prev_daisy_status),

               .PGFSM_cmd_done			  (PGFSM_cmd_done),
               .PGFSM_iso_clock_1                 (PGFSM_iso_clock_1),
               .PGFSM_iso_clock_2                 (PGFSM_iso_clock_2),
	       .PGFSM_iso_clampn_clock_1	  (PGFSM_iso_clampn_clock_1),
	       .PGFSM_iso_clampn_clock_2	  (PGFSM_iso_clampn_clock_2),
               //Mother, Daughter, reset, isolation outputs
               .PGFSM_PD_mother_sleep             (PGFSM_P1_mother_sleep_pre),
               .PGFSM_PD_daughter_sleep           (PGFSM_P1_daughter_sleep_pre),
               .PGFSM_PD_resetb                   (PGFSM_P1_resetb),
               .PGFSM_PD_iso_clampn               (PGFSM_P1_iso_clampn),
               .PGFSM_PD_scan_iso_clampn          (PGFSM_P1_scan_iso_clampn),
               .PGFSM_MEM_PD_power_ctrl           (PGFSM_MEM_P1_power_ctrl),
               .PGFSM_PFH_PD_resetb	          (PGFSM_PFH_P1_resetb),

	       //DFT
	       .PGFSM_DFT_tdr_tdi(PGFSM_DFT_tdr_tdi),
	       .PGFSM_DFT_tdr_tresetb(PGFSM_DFT_tdr_tresetb),
	       .PGFSM_DFT_tdr_shift(PGFSM_DFT_tdr_shift),
	       .PGFSM_DFT_tdr_capture(PGFSM_DFT_tdr_capture),
	       .PGFSM_DFT_tdr_update(PGFSM_DFT_tdr_update),
           .PGFSM_DFT_tdr_en_rw(PGFSM_DFT_tdr_en_rw),
           .PGFSM_DFT_tdr_en_ro(PGFSM_DFT_tdr_en_ro),
           .PGFSM_DFT_tdr_tdo_rw(PGFSM_DFT_tdr_tdo_rw),
           .PGFSM_DFT_tdr_tdo_ro(PGFSM_DFT_tdr_tdo_ro),
	       .PGFSM_DFT_tdr_tck(PGFSM_DFT_tdr_tck),

		//for nonGFX blocks 
	       .PGFSM_disable_strap(1'b0)
               );

   // Cpl_PSEN = 1 means power up mother/daughter switches;
   // Cpl_PSEN = 0 means keep mother/daughter switches.
   assign PGFSM_P1_mother_sleep = Cpl_PSEN ? 1'b0 : PGFSM_P1_mother_sleep_pre;
   assign PGFSM_P1_daughter_sleep = Cpl_PSEN ? 1'b0 : PGFSM_P1_daughter_sleep_pre;

endmodule





