
/* PGFSM block inserted in each tile that can be powergated */

`include "gfx_pgfsm_defines.h"
`include "gc_features.vh"
`include "mem_features.vh"
`include "tech_features.vh"

module gfx_pgfsm_core (
     TEST_CLK_EN,
     PGFSM_sel,
     PGFSM_cmd,
     PGFSM_wdata,
     PGFSM_new_cmd,
     is_bcast_addr,

     PGFSM_cmd_done,
     PGFSM_iso_clock_1,
     PGFSM_iso_clock_2,
     PGFSM_iso_clampn_clock_1,
     PGFSM_iso_clampn_clock_2,

     PGFSM_DFT_tdr_tdi,
     PGFSM_DFT_tdr_tresetb,
     PGFSM_DFT_tdr_shift,
     PGFSM_DFT_tdr_capture,
     PGFSM_DFT_tdr_update,
     PGFSM_DFT_tdr_en_rw,
     PGFSM_DFT_tdr_en_ro,
     PGFSM_DFT_tdr_tdo_rw,
     PGFSM_DFT_tdr_tdo_ro,
     PGFSM_DFT_tdr_tck,

     PGFSM_SD_prev_daisy_status,
     PGFSM_PD_mother_sleep,
     PGFSM_PD_daughter_sleep,
     PGFSM_PD_iso_clampn,
     PGFSM_PD_scan_iso_clampn,
     PGFSM_PD_resetb,
     PGFSM_MEM_PD_power_ctrl,

     PGFSM_SD_next_daisy_status,
     PGFSM_tile_ref_clk,
     PGFSM_tile_sclk,
     PGFSM_tile_hard_resetb,
     PGFSM_tile_cold_resetb,
     PGFSM_tile_local_address,
     PGFSM_in_MEM_PD_power_ctrl,

     PGFSM_CGLS_override,

     PGFSM_disable_strap,
     PGFSM_PFH_PD_resetb

   );

   parameter autotransmit_disabled = 0;
   parameter autotransmit_default_value = 1;

   `include "gfx_pgfsm_params.vh"

   input                                     TEST_CLK_EN;

   input                                     PGFSM_sel;
   input [31:0]                              PGFSM_wdata;
   input [31:0]                              PGFSM_cmd;
   input                                     PGFSM_new_cmd;
   input                                     is_bcast_addr;

   // SerDes interface signals.
   output                                    PGFSM_SD_prev_daisy_status;
   input                                     PGFSM_SD_next_daisy_status;

   input                                     PGFSM_tile_ref_clk;
   input                                     PGFSM_tile_sclk;
   input                                     PGFSM_tile_hard_resetb;
   input                                     PGFSM_tile_cold_resetb;

   // Address of the PGFSM
   input [7:0]                               PGFSM_tile_local_address;

   output                                    PGFSM_cmd_done;           //Pulses when a FSM finishes current command execution
   input                                     PGFSM_iso_clock_1;        //System tile clocks like dclk, dispclk
   input                                     PGFSM_iso_clock_2;        //System tile clocks like dclk, dispclk
   output                                    PGFSM_iso_clampn_clock_1; //Synchronized iso control for clock 1
   output                                    PGFSM_iso_clampn_clock_2; //Synchronized iso control for clock 2

   // PD memory control for the light sleep
   input [MEM__MEM_POWER_CTRL-1:0]    PGFSM_in_MEM_PD_power_ctrl;

   // CGLS override for GFX tiles
   input                                     PGFSM_CGLS_override;

   // Powergate controls
   output                                    PGFSM_PD_mother_sleep; 
   output                                    PGFSM_PD_daughter_sleep; 
   output                                    PGFSM_PD_iso_clampn; 
   output                                    PGFSM_PD_scan_iso_clampn; 
   output                                    PGFSM_PD_resetb; 

   output [MEM__MEM_POWER_CTRL -1:0]  PGFSM_MEM_PD_power_ctrl;

   output				     PGFSM_PFH_PD_resetb; 
   
//-------------------------- JTAG REG -------------------------------------------------
  input PGFSM_DFT_tdr_tdi;      //PFH_TARG_tdr_tdi
  input PGFSM_DFT_tdr_tresetb;  //PFH_TARG_tdr_tresetb
  input PGFSM_DFT_tdr_shift;    //PFH_TARG_tdr_shift
  input PGFSM_DFT_tdr_capture;  //PFH_TARG_tdr_capture
  input PGFSM_DFT_tdr_update;   //PFH_TARG_tdr_update
  input PGFSM_DFT_tdr_en_rw;       //PFH_PGFSM_tdr_en
  input PGFSM_DFT_tdr_en_ro;
  output PGFSM_DFT_tdr_tdo_rw;     //PFH_PGFSM_tdr_tdo
  output PGFSM_DFT_tdr_tdo_ro;
  input PGFSM_DFT_tdr_tck;      //TST_TARG_miscpfh_tclk

  input PGFSM_disable_strap;    //used by GFX IP to tie off outputs of pgfsm_cmd module when PGFSM_disable_strap=1; 
				//the reason to do this is because some tiles have multiple BPMs, one BPM will be master
				//and the other BPMs slaves. For the slaves, the pgfsm_cmd is not needed, so in order to reduce area
				//the strap will be used to tie off the signals so that synthesis tools can optimize logic and 
				//get rid of pgfsm_cmd for the BPM slaves
				//This doesn't affect nonGFX blocks since they use pgfsm.v which ties PGFSM_disable_strap=0 

   ////////////////////////////////////////////////////////
   // Wire/Registers
   ////////////////////////////////////////////////////////

   reg [23:0]                                pgfsm_cntl_reg;
   reg [23:0]                                pgfsm_delay0_reg;
   reg [23:0]                                pgfsm_delay1_reg;
   reg [23:0]                                pgfsm_delay2_reg;
   reg [23:0]                                pgfsm_delay3_reg;
   reg [23:0]                                pgfsm_delay4_reg;
   reg [23:0]                                pgfsm_delay5_reg;
   reg [1:0]                                 pgfsm_status_reg;
   reg [23:0]                                pgfsm_status_2_reg;
   // Remove Debug Registers
   //reg [23:0]                                pgfsm_fsm_1_reg;
   //reg [23:0]                                pgfsm_fsm_2_reg;
   //reg [23:0]                                pgfsm_fsm_3_reg;
   //reg [23:0]                                pgfsm_fsm_4_reg;
   //reg [23:0]                                pgfsm_fsm_5_reg;
   //reg [23:0]                                pgfsm_fsm_6_reg;

   wire [23:0]                               pgfsm_status_2;
   wire [1:0]                                pg_status_pd;
   wire                                      cmd_done;
   reg                                       PGFSM_cmd_done; 
   reg                                       autotransmit_request, read_request;
   reg                                       sw_cntl_d1;

   wire PGFSM_PD_mother_sleep_pre, PGFSM_PD_mother_sleep_pre2;
   wire PGFSM_PD_daughter_sleep_pre, PGFSM_PD_daughter_sleep_pre2;
   wire PGFSM_PD_scan_iso_clampn_pre, PGFSM_PD_scan_iso_clampn_pre2;
   //Add two flops to meet single cycle timing on isolation path
   reg  pd_iso_clampn_ff1, pd_iso_clampn_ff2;


   wire  [`GFX_PGFSM_TDR_WIDTH-1:0] PGFSM_TDR_reg;
   wire  PGFSM_DFT_PD_mother_sleep_status;
   wire  PGFSM_DFT_PD_daughter_sleep_status;
   wire  [(MEM__MEM_POWER_CTRL_IP_SD + 1) -1:0] PGFSM_DFT_mem_pwr_ctrl;

//   wire [`GFX_PGFSM_TDR_WIDTH-1:0] PGFSM_TDR_CapData;


   // TDR related wires
   wire   PGFSM_DFT_ScanEnable;
   wire   PGFSM_DFT_PD_mother_override_mode;
   wire   PGFSM_DFT_PD_daughter_override_mode;
   wire   PGFSM_DFT_PD_iso_override_mode;
   wire   PGFSM_DFT_PD_scan_iso_override_mode;
   wire   PGFSM_DFT_PD_resets_override_mode;
   wire   PGFSM_DFT_PD_mem_pwr_ctrl_override_mode;
   wire   PGFSM_DFT_PD_mem_pwr_ctrl_sd_override_mode;
   wire   PGFSM_DFT_PD_mother_sleep;
   wire   PGFSM_DFT_PD_daughter_sleep;
   wire   PGFSM_DFT_iso_clampn;
   wire   PGFSM_DFT_scan_iso_clampn;
   wire   PGFSM_DFT_resetb;
   wire   PGFSM_DFT_DisableIsoCG;
   wire   PGFSM_DFT_DisableRefClkCG;
   wire   PGFSM_DFT_PFH_PD_resetb;
   wire   PGFSM_DFT_PD_scandump_mode;


   wire   cold_resetb_sync, hard_resetb_old, reset;
   wire   isoclk, iclk, refclk;
 
   ////////////////////////////////////////////////////////
   // Clocks
   ////////////////////////////////////////////////////////

   assign iclk   = PGFSM_tile_sclk;
   assign refclk = PGFSM_tile_ref_clk;

   wire pgfsm_going_onoff = (pg_status_pd==`GFX_PGFSM_BLOCK_POWERING_OFF) | (pg_status_pd==`GFX_PGFSM_BLOCK_POWERING_ON);

   //ISO Flop clock gater support
   wire isoclk_en = PGFSM_DFT_DisableIsoCG | pgfsm_cntl_reg[`GFX_PGFSM_CNTL_DISABLE_ISO_CG] | reset | pgfsm_going_onoff;

   ati_clock_gate iso_clkgater (
     .clk_src  (iclk),
     .clk_en   (isoclk_en),
     .TE       (TEST_CLK_EN),
     .clk_out  (isoclk)
   );

   //REFCLK clock gater support 
   wire refclk_en = PGFSM_DFT_DisableRefClkCG | pgfsm_cntl_reg[`GFX_PGFSM_CNTL_DISABLE_REFCLK_CG] | reset | pgfsm_going_onoff;

   ////////////////////////////////////////////////////////
   // Resets
   ////////////////////////////////////////////////////////

   //Async assertion, sync deassertion
   techind_sync_icd #(.depth(3)) cold_rst_sync   (.CKdst(iclk), .Dsrc(PGFSM_tile_cold_resetb), .Qdst(cold_resetb_sync),
                                     .SB(1'b1), .RB(1'b1), .FLUSH(1'b0), .CKsrc(1'b0), .EN(1'b1), .Qsrc(),
                                     .SC1(1'b0), .SC2(1'b0), .SE(1'b0), .SI(1'b0), .SO()
   `ifdef ABV
                                     ,.InputChkDis(1'b0)
   `endif
   );

   wire resetb = PGFSM_DFT_PD_resets_override_mode ? PGFSM_tile_cold_resetb : cold_resetb_sync;
   assign reset  = ~resetb;

   techind_sync_icd #(.depth(3)) hard_rst_sync   (.CKdst(iclk), .Dsrc(PGFSM_tile_hard_resetb), .Qdst(hard_resetb_old),
                                     .SB(1'b1), .RB(1'b1), .FLUSH(1'b0), .CKsrc(1'b0), .EN(1'b1), .Qsrc(),
                                     .SC1(1'b0), .SC2(1'b0), .SE(1'b0), .SI(1'b0), .SO()
   `ifdef ABV
                                     ,.InputChkDis(1'b0)
   `endif
   );
   wire hard_resetb_sync = PGFSM_DFT_PD_resets_override_mode ? PGFSM_tile_cold_resetb : hard_resetb_old;

   ////////////////////////////////////////////////////////
   // PGFSM TDR 
   ////////////////////////////////////////////////////////

   // PGFSM TDR signals
/*
   assign PGFSM_TDR_CapData[`GFX_PGFSM_TDR_WIDTH-1:0] = 
            {
              PGFSM_TDR_reg[`GFX_PGFSM_TDR_WIDTH-1:`GFX_PGFSM_DFT_PD_daughter_sleep_status+1],
              PGFSM_DFT_PD_daughter_sleep_status,
              PGFSM_DFT_PD_mother_sleep_status,
              PGFSM_TDR_reg[`GFX_PGFSM_DFT_PD_mother_sleep_status-1:0]
            };
*/

  wire tmp_PGFSM_DFT_tdr_tdo;
//  wire [ ( (MEM__MEM_POWER_CTRL_IP_SD + 1) + `GFX_PGFSM_TDR_WIDTH-1) : 0] tmp_oTDR;
//  wire [ ( (MEM__MEM_POWER_CTRL_IP_SD + 1) + `GFX_PGFSM_TDR_WIDTH-1) : 0] tmp_iTDR;
 wire tmp_PGFSM_DFT_tdr_tdo_rw;
 wire tmp_PGFSM_DFT_tdr_tdo_ro;


//  assign tmp_iTDR = { PGFSM_DFT_mem_pwr_ctrl[(MEM__MEM_POWER_CTRL_IP_SD + 1) -1:0],
//                           PGFSM_TDR_CapData[`GFX_PGFSM_TDR_WIDTH-1:0] };

`ifdef DFT__DFXLIB_TDR__1
// start: UBTS 405552

  wire  pgfsm_tdr_shift   = PGFSM_DFT_tdr_en_rw & PGFSM_DFT_tdr_shift;
  wire  pgfsm_tdr_capture = PGFSM_DFT_tdr_en_rw & PGFSM_DFT_tdr_capture;
  wire  pgfsm_tdr_update  = PGFSM_DFT_tdr_en_rw & PGFSM_DFT_tdr_update;

  test_reg_async_rst #((MEM__MEM_POWER_CTRL_IP_SD + 1) + `GFX_PGFSM_TDR_WIDTH) PGFSM_TDR (
   .clk         (PGFSM_DFT_tdr_tck),
   .rstb        (PGFSM_DFT_tdr_tresetb),
   .en          (PGFSM_DFT_tdr_en_rw),
   .shift       (pgfsm_tdr_shift),
   .capture     (pgfsm_tdr_capture),
   .update      (pgfsm_tdr_update),
   .d_rst       ({( (MEM__MEM_POWER_CTRL_IP_SD + 1) + `GFX_PGFSM_TDR_WIDTH){1'b0}}),
   .q           (tmp_oTDR),
   .d           (tmp_iTDR),
   .si          (PGFSM_DFT_tdr_tdi),
   .so          (tmp_PGFSM_DFT_tdr_tdo),
   .q_sr        (),
   .q_sr_posedge()
  );

// end: UBTS 405552
`else

wire [1-1:0] daughter_override_mode;
wire [1-1:0] daughter_sleep_override_value;
wire [1-1:0] disableisocg;
wire [1-1:0] disablerefclkcg;
wire [1-1:0] iso_clampn_override_value;
wire [1-1:0] isolation_override_mode;
wire [1-1:0] mem_power_control_override_mode;
wire [1-1:0] mem_power_control_sd_override_mode;
wire [1-1:0] mem_power_ctrl_ls;
wire [1-1:0] mem_power_ctrl_ds;
wire [1-1:0] mem_power_ctrl_sd;
wire [1-1:0] mem_power_ctrl_ip_ds;
wire [1-1:0] mem_power_ctrl_ip_sd;
wire [1-1:0] mother_override_mode;
wire [1-1:0] mother_sleep_override_value;
wire [1-1:0] pfh_resetb_override_value;
wire [1-1:0] resetb_override_value;
wire [1-1:0] resets_override_mode;
wire [1-1:0] scan_enable;
wire [1-1:0] scan_iso_clampn_override_value;
wire [1-1:0] scan_iso_override_mode;
wire [1-1:0] scandump_mode;

gfx_pgfsm_setup_tdr gfx_pgfsm_setup_tdr (

 .Tdr_Tck 		(PGFSM_DFT_tdr_tck) ,
 .Tdr_Trstb 		(PGFSM_DFT_tdr_tresetb),
 .Tdr_Tdi 		(PGFSM_DFT_tdr_tdi),
 .Tdr_Shift 		(PGFSM_DFT_tdr_shift),
 .Tdr_Capture 		(PGFSM_DFT_tdr_capture),
 .Tdr_Update 		(PGFSM_DFT_tdr_update)

//TDR --> GFX_PGFSM_SETUP_CFG
 ,.daughter_override_mode		 (daughter_override_mode[1-1:0])
 ,.daughter_sleep_override_value		 (daughter_sleep_override_value[1-1:0])
 ,.disableisocg		 (disableisocg[1-1:0])
 ,.disablerefclkcg		 (disablerefclkcg[1-1:0])
 ,.iso_clampn_override_value		 (iso_clampn_override_value[1-1:0])
 ,.isolation_override_mode		 (isolation_override_mode[1-1:0])
 ,.mem_power_control_override_mode		 (mem_power_control_override_mode[1-1:0])
 ,.mem_power_control_sd_override_mode		 (mem_power_control_sd_override_mode[1-1:0])
 ,.mem_power_ctrl_ls		 (mem_power_ctrl_ls[1-1:0])
 ,.mem_power_ctrl_ds		 (mem_power_ctrl_ds[1-1:0])
 ,.mem_power_ctrl_sd		 (mem_power_ctrl_sd[1-1:0])
 ,.mem_power_ctrl_ip_ds	 (mem_power_ctrl_ip_ds[1-1:0])
 ,.mem_power_ctrl_ip_sd	 (mem_power_ctrl_ip_sd[1-1:0])
 ,.mother_override_mode		 (mother_override_mode[1-1:0])
 ,.mother_sleep_override_value		 (mother_sleep_override_value[1-1:0])
 ,.pfh_resetb_override_value		 (pfh_resetb_override_value[1-1:0])
 ,.resetb_override_value		 (resetb_override_value[1-1:0])
 ,.resets_override_mode		 (resets_override_mode[1-1:0])
 ,.scan_enable		 (scan_enable[1-1:0])
 ,.scan_iso_clampn_override_value		 (scan_iso_clampn_override_value[1-1:0])
 ,.scan_iso_override_mode		 (scan_iso_override_mode[1-1:0])
 ,.scandump_mode		 (scandump_mode[1-1:0])
 ,.Tdr_gfx_pgfsm_setup_cfg_En		(PGFSM_DFT_tdr_en_rw)
 ,.Tdr_gfx_pgfsm_setup_cfg_Tdo		(tmp_PGFSM_DFT_tdr_tdo_rw)
);

pgfsm_status_tdr pgfsm_status_tdr ( 

 .Tdr_Tck 		(PGFSM_DFT_tdr_tck) ,
 .Tdr_Trstb 		(PGFSM_DFT_tdr_tresetb),
 .Tdr_Tdi 		(PGFSM_DFT_tdr_tdi),
 .Tdr_Shift 		(PGFSM_DFT_tdr_shift),
 .Tdr_Capture 		(PGFSM_DFT_tdr_capture),
 .Tdr_Update 		(PGFSM_DFT_tdr_update)

//TDR --> PGFSM_STATUS
 ,.daughter_sleep_status		(PGFSM_DFT_PD_daughter_sleep_status)
 ,.mother_sleep_status		(PGFSM_DFT_PD_mother_sleep_status)
 ,.Tdr_pgfsm_status_En		(PGFSM_DFT_tdr_en_ro)
 ,.Tdr_pgfsm_status_Tdo		(tmp_PGFSM_DFT_tdr_tdo_ro)
);

/*
  TESTDATAREGISTER #(   .WIDTH(  (MEM__MEM_POWER_CTRL_IP_SD + 1) + `GFX_PGFSM_TDR_WIDTH),
             .RST_VAL({((MEM__MEM_POWER_CTRL_IP_SD + 1) + `GFX_PGFSM_TDR_WIDTH){1'b0}} )
           ) PGFSM_TDR (
   .iTCK    	(PGFSM_DFT_tdr_tck),
   .iTDI    	(PGFSM_DFT_tdr_tdi),
   .iTdrEn  	(PGFSM_DFT_tdr_en),
   .iTdrRstb 	(PGFSM_DFT_tdr_tresetb),
   .iCapData   	(tmp_iTDR),
   .iShiftDr   	(PGFSM_DFT_tdr_shift),
   .iUpdateDr  	(PGFSM_DFT_tdr_update),
   .iCaptureDr 	(PGFSM_DFT_tdr_capture),
   .oTDO 	(tmp_PGFSM_DFT_tdr_tdo),
   .oTDR	(tmp_oTDR)
);
*/

`endif


  assign PGFSM_DFT_tdr_tdo_rw = PGFSM_disable_strap ? 1'b0 : tmp_PGFSM_DFT_tdr_tdo_rw;
  assign PGFSM_DFT_tdr_tdo_ro = PGFSM_disable_strap ? 1'b0 : tmp_PGFSM_DFT_tdr_tdo_ro;
  assign PGFSM_TDR_reg[`GFX_PGFSM_TDR_WIDTH-1:0] = PGFSM_disable_strap ? {(`GFX_PGFSM_TDR_WIDTH){1'b0}} : 
//                                               tmp_oTDR [`GFX_PGFSM_TDR_WIDTH-1:0];
                                               {scandump_mode,pfh_resetb_override_value,disablerefclkcg,disableisocg,resetb_override_value,scan_iso_clampn_override_value,iso_clampn_override_value,daughter_sleep_override_value,mother_sleep_override_value,mem_power_control_sd_override_mode,mem_power_control_override_mode,resets_override_mode,scan_iso_override_mode,isolation_override_mode,daughter_override_mode,mother_override_mode,scan_enable};

  assign PGFSM_DFT_mem_pwr_ctrl[(MEM__MEM_POWER_CTRL_IP_SD + 1) -1:0] =
                                               PGFSM_disable_strap ? 
                                               {(MEM__MEM_POWER_CTRL_IP_SD + 1){1'b0}} : 
//                                               tmp_oTDR [((MEM__MEM_POWER_CTRL_IP_SD + 1) +`GFX_PGFSM_TDR_WIDTH-1):`GFX_PGFSM_TDR_WIDTH];
                                               {mem_power_ctrl_ip_sd,mem_power_ctrl_ip_ds,mem_power_ctrl_sd,mem_power_ctrl_ds,mem_power_ctrl_ls};



  assign PGFSM_DFT_ScanEnable                       = PGFSM_TDR_reg[`GFX_PGFSM_DFT_ScanEnable];
  assign PGFSM_DFT_PD_mother_override_mode          = PGFSM_TDR_reg[`GFX_PGFSM_DFT_PD_mother_override_mode];
  assign PGFSM_DFT_PD_daughter_override_mode        = PGFSM_TDR_reg[`GFX_PGFSM_DFT_PD_daughter_override_mode];
  assign PGFSM_DFT_PD_iso_override_mode             = PGFSM_TDR_reg[`GFX_PGFSM_DFT_PD_iso_override_mode];
  assign PGFSM_DFT_PD_scan_iso_override_mode        = PGFSM_TDR_reg[`GFX_PGFSM_DFT_PD_scan_iso_override_mode];
  assign PGFSM_DFT_PD_resets_override_mode          = PGFSM_TDR_reg[`GFX_PGFSM_DFT_PD_resets_override_mode];
  assign PGFSM_DFT_PD_mem_pwr_ctrl_override_mode    = PGFSM_TDR_reg[`GFX_PGFSM_DFT_PD_mem_pwr_ctrl_override_mode];
  assign PGFSM_DFT_PD_mem_pwr_ctrl_sd_override_mode = PGFSM_TDR_reg[`GFX_PGFSM_DFT_PD_mem_pwr_ctrl_sd_override_mode];
  assign PGFSM_DFT_PD_mother_sleep                  = PGFSM_TDR_reg[`GFX_PGFSM_DFT_PD_mother_sleep];
  assign PGFSM_DFT_PD_daughter_sleep                = PGFSM_TDR_reg[`GFX_PGFSM_DFT_PD_daughter_sleep];
  assign PGFSM_DFT_iso_clampn                       = PGFSM_TDR_reg[`GFX_PGFSM_DFT_iso_clampn];
  assign PGFSM_DFT_scan_iso_clampn                  = PGFSM_TDR_reg[`GFX_PGFSM_DFT_scan_iso_clampn];
  assign PGFSM_DFT_resetb                           = PGFSM_TDR_reg[`GFX_PGFSM_DFT_resetb];
  assign PGFSM_DFT_DisableIsoCG                     = PGFSM_TDR_reg[`GFX_PGFSM_DFT_DisableIsoCG];
  assign PGFSM_DFT_DisableRefClkCG                  = PGFSM_TDR_reg[`GFX_PGFSM_DFT_DisableRefClkCG];
  assign PGFSM_DFT_PFH_PD_resetb                    = PGFSM_TDR_reg[`GFX_PGFSM_PFH_PD_resetb];
  // added for UBTS 405919
  assign PGFSM_DFT_PD_scandump_mode                 = PGFSM_TDR_reg[`GFX_PGFSM_DFT_PD_scandump_mode];

//-------------------------------------------------------------------------------------   
//------------------------- Following code is powergating specific ---------------------   
//-------------------------------------------------------------------------------------   

   ////////////////////////////////////////////////////////
   // PGFSM Registers
   ////////////////////////////////////////////////////////

   wire pgfsm_addr_sel = is_bcast_addr | (PGFSM_cmd[`GFX_PGFSM_CMD_ADDRESS] == PGFSM_tile_local_address);

   wire PGFSM_new_cmd_filtered = PGFSM_new_cmd & hard_resetb_sync;
   wire pgfsm_sel_local      = PGFSM_sel & PGFSM_new_cmd_filtered & pgfsm_addr_sel;
   wire pgfsm_rd_cmd   = pgfsm_sel_local & PGFSM_cmd[`GFX_PGFSM_CMD_READ];
   wire pgfsm_wr_cmd   = pgfsm_sel_local & PGFSM_cmd[`GFX_PGFSM_CMD_WRITE]; 


   always @(posedge iclk) begin
      if (reset) begin 

         pgfsm_cntl_reg          <= autotransmit_default_value ? `GFX_PGFSM_CNTL_REG_RSTVAL1 : `GFX_PGFSM_CNTL_REG_RSTVAL2 ;
         pgfsm_delay0_reg        <= `GFX_PGFSM_DELAY0_REG_RSTVAL; 
         pgfsm_delay1_reg        <= `GFX_PGFSM_DELAY1_REG_RSTVAL;
         pgfsm_delay2_reg        <= `GFX_PGFSM_DELAY2_REG_RSTVAL;
         pgfsm_delay3_reg        <= `GFX_PGFSM_DELAY3_REG_RSTVAL;
         pgfsm_delay4_reg        <= `GFX_PGFSM_DELAY4_REG_RSTVAL;
         pgfsm_delay5_reg        <= `GFX_PGFSM_DELAY5_REG_RSTVAL;
         //pgfsm_fsm_1_reg         <= `GFX_PGFSM_FSM_1_REG_RSTVAL;
         //pgfsm_fsm_2_reg         <= `GFX_PGFSM_FSM_2_REG_RSTVAL;
         //pgfsm_fsm_3_reg         <= `GFX_PGFSM_FSM_3_REG_RSTVAL;
         //pgfsm_fsm_4_reg         <= `GFX_PGFSM_FSM_4_REG_RSTVAL;
         //pgfsm_fsm_5_reg         <= `GFX_PGFSM_FSM_5_REG_RSTVAL;
         //pgfsm_fsm_6_reg         <= `GFX_PGFSM_FSM_6_REG_RSTVAL;
      end else if (pgfsm_wr_cmd)  begin
         case (PGFSM_cmd[`GFX_PGFSM_CMD_REG_ADDR])
           `GFX_PGFSM_DELAY0_REG_ADDR :   begin
                                        pgfsm_delay0_reg[`GFX_PGFSM_RESET_DELAY_FLD]            <= PGFSM_wdata[`GFX_PGFSM_RESET_DELAY_FLD];
                                        pgfsm_delay0_reg[`GFX_PGFSM_ISOLATION_DELAY_FLD]        <= PGFSM_wdata[`GFX_PGFSM_ISOLATION_DELAY_FLD];
                                        pgfsm_delay0_reg[`GFX_PGFSM_MEM_SD_DAUGHTER_DELAY_FLD]  <= PGFSM_wdata[`GFX_PGFSM_MEM_SD_DAUGHTER_DELAY_FLD];
                                      end

           `GFX_PGFSM_DELAY1_REG_ADDR :   begin
                                        pgfsm_delay1_reg[`GFX_PGFSM_MOTHER_DELAY_FLD]           <= PGFSM_wdata[`GFX_PGFSM_MOTHER_DELAY_FLD];
                                        pgfsm_delay1_reg[`GFX_PGFSM_DAUGHTER_DELAY_FLD]         <= PGFSM_wdata[`GFX_PGFSM_DAUGHTER_DELAY_FLD];
                                      end

           `GFX_PGFSM_DELAY2_REG_ADDR :   begin
                                        pgfsm_delay2_reg[`GFX_PGFSM_RESET_DELAY_FLD]            <= PGFSM_wdata[`GFX_PGFSM_RESET_DELAY_FLD];
                                        pgfsm_delay2_reg[`GFX_PGFSM_ISOLATION_DELAY_FLD]        <= PGFSM_wdata[`GFX_PGFSM_ISOLATION_DELAY_FLD];
                                        pgfsm_delay2_reg[`GFX_PGFSM_MEM_SD_DAUGHTER_DELAY_FLD]  <= PGFSM_wdata[`GFX_PGFSM_MEM_SD_DAUGHTER_DELAY_FLD];
                                      end

           `GFX_PGFSM_DELAY3_REG_ADDR :   begin
                                        pgfsm_delay3_reg[`GFX_PGFSM_MOTHER_DELAY_FLD]           <= PGFSM_wdata[`GFX_PGFSM_MOTHER_DELAY_FLD];
                                        pgfsm_delay3_reg[`GFX_PGFSM_DAUGHTER_DELAY_FLD]         <= PGFSM_wdata[`GFX_PGFSM_DAUGHTER_DELAY_FLD];
                                      end

           `GFX_PGFSM_DELAY4_REG_ADDR :   begin
                                        pgfsm_delay4_reg[`GFX_PGFSM_MEM_SD_M_PWRDN_DELAY_FLD]   <= PGFSM_wdata[`GFX_PGFSM_MEM_SD_M_PWRDN_DELAY_FLD];
                                        pgfsm_delay4_reg[`GFX_PGFSM_MEM_SD_M_PWRUP_DELAY_FLD]   <= PGFSM_wdata[`GFX_PGFSM_MEM_SD_M_PWRUP_DELAY_FLD];
                                      end

           `GFX_PGFSM_DELAY5_REG_ADDR :   pgfsm_delay5_reg[`GFX_PWRUP_DELAY_FLD]  <= PGFSM_wdata[`GFX_PWRUP_DELAY_FLD];

           `GFX_PGFSM_CNTL_REG_ADDR   :   pgfsm_cntl_reg    <= PGFSM_wdata[23:0]; 
           //`GFX_PGFSM_FSM_1_REG_ADDR  :   pgfsm_fsm_1_reg   <= PGFSM_wdata[23:0]; 
           //`GFX_PGFSM_FSM_2_REG_ADDR  :   pgfsm_fsm_2_reg   <= PGFSM_wdata[23:0]; 
           //`GFX_PGFSM_FSM_3_REG_ADDR  :   pgfsm_fsm_3_reg   <= PGFSM_wdata[23:0]; 
           //`GFX_PGFSM_FSM_4_REG_ADDR  :   pgfsm_fsm_4_reg   <= PGFSM_wdata[23:0]; 
           //`GFX_PGFSM_FSM_5_REG_ADDR  :   pgfsm_fsm_5_reg   <= PGFSM_wdata[23:0]; 
           //`GFX_PGFSM_FSM_6_REG_ADDR  :   pgfsm_fsm_6_reg   <= PGFSM_wdata[23:0];
           default: begin end
         endcase
      end
   end

   always @(posedge iclk) begin
      if (reset) begin
         pgfsm_status_reg            <= `GFX_PGFSM_STATUS_REG_RSTVAL;
         pgfsm_status_2_reg          <= `GFX_PGFSM_STATUS_2_REG_RSTVAL;
         sw_cntl_d1                  <= 1'b0;
      end else begin
         pgfsm_status_reg[1:0]       <= pg_status_pd;
         pgfsm_status_2_reg          <= pgfsm_status_2;
         sw_cntl_d1                  <= pgfsm_cntl_reg[`GFX_PGFSM_CNTL_SW_CNTL];
      end
   end
   
   ////////////////////////////////////////////////////////
   // FSM for power down and power up.
   ////////////////////////////////////////////////////////
   wire isolation_pd;
   wire pg_reset_pd;
   wire mother_sleep_pd;
   wire daughter_sleep_pd;
   wire mem_mother_shutdown_pd;
   wire mem_daughter_shutdown_pd;

   wire [4:0] pgfsm_fsm_state;

   wire tmp_isolation_pd;
   wire tmp_pg_reset_pd;
   wire tmp_mother_sleep_pd;
   wire tmp_daughter_sleep_pd;
   wire tmp_mem_mother_shutdown_pd;
   wire tmp_mem_daughter_shutdown_pd;

   wire       fsm_cmd_done;
   reg        nonfsm_cmd_done;
   wire [4:0] fsm_state; 
   wire [1:0] tmp_pg_status_pd;

   // PGFSM_CNTL_PD_OVERRIDE is used to override PU/PD command for broadcast PU/PD commands only
   // PGFSM_CNTL_PD_OVERRIDE has no effect on non-broadcast PU/PD commands
   wire pupd_cmd_override = is_bcast_addr & pgfsm_cntl_reg[`GFX_PGFSM_CNTL_PD_OVERRIDE];
   wire pwrup_cmd         = ~pupd_cmd_override & PGFSM_cmd[`GFX_PGFSM_CMD_PWRUP];
   // PWRUP and PWRDWN command bits should never be set at same tine, but if they are PWRUP gets priority
   wire pwrdwn_cmd        = ~pupd_cmd_override & ~PGFSM_cmd[`GFX_PGFSM_CMD_PWRUP] & PGFSM_cmd[`GFX_PGFSM_CMD_PWRDN];

   gfx_pgfsm_cmd_fifo pgfsm_cmd_fifo (
                                  //outputs
                                  .isolation_pd(tmp_isolation_pd),
                                  .pg_reset_pd(tmp_pg_reset_pd),
                                  .mother_sleep_pd(tmp_mother_sleep_pd),
                                  .daughter_sleep_pd(tmp_daughter_sleep_pd),
                                  .mem_mother_shutdown_pd(tmp_mem_mother_shutdown_pd),
                                  .mem_daughter_shutdown_pd(tmp_mem_daughter_shutdown_pd),

                                  .pg_status_pd(tmp_pg_status_pd),
                                  .pgfsm_fsm_state (fsm_state),
                                  .pgfsm_cmd_done  (fsm_cmd_done),
                                  //inputs
                                  .TEST_CLK_EN(TEST_CLK_EN),
                                  .reset(reset),                           //Cold reset - active high
                                  .hard_resetb(hard_resetb_sync),          //Warm reset - active low
                                  .refclk(refclk),
				                  .refclk_en(refclk_en),
                                  .iclk(iclk),
                                  .new_cmd(PGFSM_new_cmd_filtered),
                                  .pd_sel(pgfsm_sel_local),
                                  .pwrup_cmd(pwrup_cmd),
                                  .pwrdn_cmd(pwrdwn_cmd),
				                  .use_extra_delay_regs_en(pgfsm_cntl_reg[`GFX_PGFSM_CNTL_USE_EXTRA_DELAY_REGS]),
                                  .pgfsm_delay0_reg(pgfsm_delay0_reg),
                                  .pgfsm_delay1_reg(pgfsm_delay1_reg),
                                  .pgfsm_delay2_reg(pgfsm_delay2_reg),
                                  .pgfsm_delay3_reg(pgfsm_delay3_reg),
                                  .pgfsm_delay4_reg(pgfsm_delay4_reg),
                                  .pgfsm_delay5_reg(pgfsm_delay5_reg),
                                  .pgfsm_fsm_1_reg(`GFX_PGFSM_FSM_1_REG_RSTVAL),
                                  .pgfsm_fsm_2_reg(`GFX_PGFSM_FSM_2_REG_RSTVAL),
                                  .pgfsm_fsm_3_reg(`GFX_PGFSM_FSM_3_REG_RSTVAL),
                                  .pgfsm_fsm_4_reg(`GFX_PGFSM_FSM_4_REG_RSTVAL),
                                  .pgfsm_fsm_5_reg(`GFX_PGFSM_FSM_5_REG_RSTVAL),
                                  .pgfsm_fsm_6_reg(`GFX_PGFSM_FSM_6_REG_RSTVAL)								  
                                  );

   assign isolation_pd             = PGFSM_disable_strap ? 1'b0 : tmp_isolation_pd;
   assign pg_reset_pd              = PGFSM_disable_strap ? 1'b0 : tmp_pg_reset_pd;
   assign mother_sleep_pd          = PGFSM_disable_strap ? 1'b0 : tmp_mother_sleep_pd;
   assign daughter_sleep_pd        = PGFSM_disable_strap ? 1'b0 : tmp_daughter_sleep_pd;
   assign mem_mother_shutdown_pd   = PGFSM_disable_strap ? 1'b0 : tmp_mem_mother_shutdown_pd;
   assign mem_daughter_shutdown_pd = PGFSM_disable_strap ? 1'b0 : tmp_mem_daughter_shutdown_pd;

   assign pgfsm_fsm_state[4:0]     = PGFSM_disable_strap ? 5'b0 : fsm_state; 
   assign pg_status_pd[1:0]        = PGFSM_disable_strap ? 2'b0 : tmp_pg_status_pd;

   always @(posedge iclk) begin
      if (reset) begin
         nonfsm_cmd_done   <= 1'b0;
      end else begin
         nonfsm_cmd_done   <= (PGFSM_new_cmd_filtered & pgfsm_addr_sel & ~pgfsm_cntl_reg[`GFX_PGFSM_CNTL_DISABLE_NEW_CMD_STAT]);
      end
   end // always @ (posedge iclk)

   // Combine FSM and non-FSM command done
   assign cmd_done                 = PGFSM_disable_strap ? 1'b0 : (fsm_cmd_done | nonfsm_cmd_done) ;

   always @(posedge iclk) begin
      if (reset) begin
         PGFSM_cmd_done            <= 1'b0;
      end else begin
         PGFSM_cmd_done            <= cmd_done; 
      end
   end

   //PGFSM_STATUS_2
   assign     pgfsm_status_2[0]  =  PGFSM_PD_resetb;
   assign     pgfsm_status_2[1]  =  1'b0;
   assign     pgfsm_status_2[2]  =  PGFSM_PD_mother_sleep_pre2;
   assign     pgfsm_status_2[3]  =  1'b0;
   assign     pgfsm_status_2[4]  =  PGFSM_PD_daughter_sleep_pre2;
   assign     pgfsm_status_2[5]  =  1'b0;
   assign     pgfsm_status_2[6]  =  pd_iso_clampn_ff2;
   assign     pgfsm_status_2[7]  =  PGFSM_PD_scan_iso_clampn_pre;
   //Differentiate between fusion vs discrete
   assign     pgfsm_status_2[8]  =  PGFSM_MEM_PD_power_ctrl[MEM__MEM_POWER_CTRL_IP_DS];
   assign     pgfsm_status_2[9]  =  PGFSM_MEM_PD_power_ctrl[MEM__MEM_POWER_CTRL_IP_SD];
   assign     pgfsm_status_2[18:10] =  9'b0; //Reserved
   //Differentiate between fusion vs discrete
   assign     pgfsm_status_2[23:19]=  pgfsm_fsm_state;   
   assign     PGFSM_DFT_PD_mother_sleep_status   = PGFSM_PD_mother_sleep_pre2;  
   assign     PGFSM_DFT_PD_daughter_sleep_status = PGFSM_PD_daughter_sleep_pre2;

   // **************************************************************************************** //
   // Read path control. Requires arbitration between status transmission of current pgfsm
   // and previous pgfsm. 
   // **************************************************************************************** //
    wire [31:0] fifo_read_data;
    wire        fifo_data_valid;
    reg [31:0]  fifo_transmit_data;
    reg [31:0]  local_pgfsm_data;
    reg         curr_tile_transmitting, fifo_transmitting;
    reg         curr_tile_transmitting_ff1, fifo_transmitting_ff1;
    wire        curr_tile_transmitting_pulse, fifo_transmitting_pulse;
    reg         fifo_pending, fifo_started;
    reg [5:0]   count35;

   // Generate read data output for current PGFSM/tile
   always @(posedge iclk) begin
      if (reset) begin
         read_request <= 1'b0; 
         autotransmit_request <= 1'b0;
      end else if (pgfsm_rd_cmd)  begin
         read_request <= 1'b1; 
         case (PGFSM_cmd[`GFX_PGFSM_CMD_REG_ADDR])
           `GFX_PGFSM_STATUS_REG_ADDR    : local_pgfsm_data <= {PGFSM_tile_local_address, 22'b0, pgfsm_status_reg}; //0
           `GFX_PGFSM_CNTL_REG_ADDR      : local_pgfsm_data <= {PGFSM_tile_local_address, pgfsm_cntl_reg};          //1
           `GFX_PGFSM_DELAY0_REG_ADDR    : local_pgfsm_data <= {PGFSM_tile_local_address, pgfsm_delay0_reg};        //2
           `GFX_PGFSM_DELAY1_REG_ADDR    : local_pgfsm_data <= {PGFSM_tile_local_address, pgfsm_delay1_reg};        //3
           `GFX_PGFSM_STATUS_2_REG_ADDR  : local_pgfsm_data <= {PGFSM_tile_local_address, pgfsm_status_2_reg};      //4 
           //`GFX_PGFSM_FSM_1_REG_ADDR     : local_pgfsm_data <= {PGFSM_tile_local_address, pgfsm_fsm_1_reg};         //5
           //`GFX_PGFSM_FSM_2_REG_ADDR     : local_pgfsm_data <= {PGFSM_tile_local_address, pgfsm_fsm_2_reg};         //6
           //`GFX_PGFSM_FSM_3_REG_ADDR     : local_pgfsm_data <= {PGFSM_tile_local_address, pgfsm_fsm_3_reg};         //7 
           //`GFX_PGFSM_FSM_4_REG_ADDR     : local_pgfsm_data <= {PGFSM_tile_local_address, pgfsm_fsm_4_reg};         //8
           //`GFX_PGFSM_FSM_5_REG_ADDR     : local_pgfsm_data <= {PGFSM_tile_local_address, pgfsm_fsm_5_reg};         //9
           //`GFX_PGFSM_FSM_6_REG_ADDR     : local_pgfsm_data <= {PGFSM_tile_local_address, pgfsm_fsm_6_reg};         //14
           `GFX_PGFSM_DELAY2_REG_ADDR    : local_pgfsm_data <= {PGFSM_tile_local_address, pgfsm_delay2_reg};        //10
           `GFX_PGFSM_DELAY3_REG_ADDR    : local_pgfsm_data <= {PGFSM_tile_local_address, pgfsm_delay3_reg};        //11
           `GFX_PGFSM_DELAY4_REG_ADDR    : local_pgfsm_data <= {PGFSM_tile_local_address, pgfsm_delay4_reg};        //12
           `GFX_PGFSM_DELAY5_REG_ADDR    : local_pgfsm_data <= {PGFSM_tile_local_address, pgfsm_delay5_reg};        //13
           default                   : local_pgfsm_data <= {PGFSM_tile_local_address, 22'b0, pgfsm_status_reg};
         endcase
      end else if (cmd_done & pgfsm_cntl_reg[`GFX_PGFSM_CNTL_AUTO_STATUS_READ] & (autotransmit_disabled == 0)) begin
         // Activite autotransmit only when FSM is done
         autotransmit_request    <= 1'b1; 
         local_pgfsm_data        <= {PGFSM_tile_local_address, 19'b0, 1'b1, 2'b0, pg_status_pd};
      end else if (curr_tile_transmitting) begin
         read_request <= 1'b0; 
         autotransmit_request <= 1'b0;
      end
   end

   wire tmp_PGFSM_SD_prev_daisy_status;
   wire [31:0] tmp_fifo_read_data;
   wire tmp_fifo_data_valid;   
   gfx_pgfsm_ser #(32,5) serial_read_data (
                                     // outputs
                                     .ser_busy(),
                                     .ser_data(tmp_PGFSM_SD_prev_daisy_status),
                                     // inputs
                                     .par_data( fifo_transmitting ? fifo_transmit_data : local_pgfsm_data ),
                                     .par_valid( curr_tile_transmitting_pulse | fifo_transmitting_pulse ),
                                     .short_cmd (1'b0),
                                     .clk(iclk),
                                     .reset(reset)
                                     ); 

   gfx_pgfsm_status_deser #(32,5) fifo_deser  (
                                    // outputs
                                    .data_out(tmp_fifo_read_data),
                                    .new_data(tmp_fifo_data_valid),
                                    // inputs
                                    .serial_in(PGFSM_SD_next_daisy_status),
                                    .iclk(iclk),
                                    .reset(reset),
                                    .hard_resetb(hard_resetb_sync)
                                    ); 

   assign PGFSM_SD_prev_daisy_status 	= PGFSM_disable_strap ? 1'b0  : tmp_PGFSM_SD_prev_daisy_status; 

   assign fifo_read_data 		= PGFSM_disable_strap ? 32'b0 : tmp_fifo_read_data;
   assign fifo_data_valid 		= PGFSM_disable_strap ? 1'b0  : tmp_fifo_data_valid;

   // Priority arbiter for the read path transmission. 
   always @(posedge iclk) begin
      if (reset) begin
         curr_tile_transmitting  <= 1'b0;
         fifo_transmitting       <= 1'b0; 
         count35                 <= 6'b0; 
      end else if (fifo_pending & ~fifo_transmitting & ~curr_tile_transmitting & ~(|count35)) begin
         // Highest priority is for fifo 
         fifo_transmitting       <= 1'b1; 
         count35                 <= 6'b100011;
      end else if ( (autotransmit_request | read_request) & ~fifo_transmitting & ~fifo_started 
                    & ~curr_tile_transmitting & ~(|count35) ) begin
         // Last priority for the current pgfsm's data to be sent
         curr_tile_transmitting  <= 1'b1; 
         count35                 <= 6'b100011;
      end else if (count35 == 6'b11) begin
         curr_tile_transmitting  <= 1'b0;
         fifo_transmitting       <= 1'b0; 
         count35                 <= count35 - 6'b1; 
      end else if (|count35) begin
         count35                 <= count35 - 6'b1; 
      end
   end
   
   always @(posedge iclk) begin
      curr_tile_transmitting_ff1  <= curr_tile_transmitting; 
      fifo_transmitting_ff1       <= fifo_transmitting; 
   end

   assign fifo_transmitting_pulse      = fifo_transmitting & ~fifo_transmitting_ff1;
   assign curr_tile_transmitting_pulse = curr_tile_transmitting & ~curr_tile_transmitting_ff1;

   always @(posedge iclk) begin
      if (reset) begin
         fifo_pending            <= 1'b0;
      end else if (fifo_data_valid) begin
         fifo_transmit_data      <= fifo_read_data;
         fifo_pending            <= 1'b1;
      end else if (fifo_pending & fifo_transmitting) begin
         fifo_pending            <= 1'b0;
      end
   end

   always @(posedge iclk) begin
      if (reset) fifo_started <= 1'b0;
      else if (fifo_data_valid & fifo_started) fifo_started <= 1'b0;
           else if (PGFSM_SD_next_daisy_status) fifo_started <= 1'b1;
   end

   // *************************************** //
   // SW control and JTAG override signals. 
   // *************************************** //
  
   wire       sw_cntl, inv_iso, inv_mother, inv_daughter;
   wire       toggle_ds;
   
   //Once cycle delayed version of pgfsm_cntl_reg[`GFX_PGFSM_CNTL_SW_CNTL] ; this makes sure that when output
   //MUX is switched for SW CNTL, all PGFSM_CNTL_REG bits are stable.
   assign     sw_cntl            = sw_cntl_d1;

`ifdef GFX_PGFSM_MOTHER_POLARITY_AH   
   assign     inv_mother         = 1'b0;
`else
   assign     inv_mother         = 1'b1;
`endif
   

`ifdef GFX_PGFSM_DAUGHTER_POLARITY_AH   
   assign     inv_daughter       = 1'b0;
`else
   assign     inv_daughter       = 1'b1;
`endif

`ifdef GFX_PGFSM_ISOLATION_POLARITY_AL   
   assign     inv_iso            = 1'b0;
`else
   assign     inv_iso            = 1'b1;
`endif

   assign     toggle_ds          = pgfsm_cntl_reg[`GFX_PGFSM_CNTL_MEM_PWR_CTRL_SELECT];

   wire       pd_mother_fsm;
   assign     pd_mother_fsm      = inv_mother              ? ~mother_sleep_pd : mother_sleep_pd;
   
   wire       pd_daughter_fsm;
   assign     pd_daughter_fsm    = inv_daughter            ? ~daughter_sleep_pd : daughter_sleep_pd;

   wire       pd_iso_fsm;
   assign     pd_iso_fsm         = inv_iso                 ? isolation_pd    : ~isolation_pd;

   wire [MEM__MEM_POWER_CTRL-1:0] pd_mem_in  = PGFSM_in_MEM_PD_power_ctrl;
   // TDRs for RM/RME bits and PGMEM bits are not present in PGFSM
   wire [MEM__MEM_POWER_CTRL-1:0] dft_mem_in = {
                      PGFSM_in_MEM_PD_power_ctrl [MEM__MEM_POWER_CTRL-1: MEM__MEM_POWER_CTRL_IP_SD + 1],
                          PGFSM_DFT_mem_pwr_ctrl [MEM__MEM_POWER_CTRL_IP_SD : 0]
                                                      };

   wire                                  pd_m_dft_mode = PGFSM_DFT_PD_mother_override_mode;
   wire                                  pd_d_dft_mode = PGFSM_DFT_PD_daughter_override_mode;
   wire                                  pd_s_dft_mode = PGFSM_DFT_PD_mem_pwr_ctrl_sd_override_mode;
      
   // PD domain
   // ------------------

// PD Reset control
//-------------------------------------   
   wire   PGFSM_PD_resetb_pre;
   assign PGFSM_PD_resetb_pre      =         hard_resetb_sync  & (
                                                    sw_cntl  ? pgfsm_cntl_reg[`GFX_PGFSM_CNTL_PD_RESETB] 
                                                             : ~pg_reset_pd );
   ati_mux2 d0nt_PGFSM_PD_resetb_mux ( .i0 (PGFSM_PD_resetb_pre),
                                       .i1 (PGFSM_DFT_resetb),
                                       .s  (PGFSM_DFT_PD_resets_override_mode),
                                       .z  (PGFSM_PD_resetb));

// PFH Reset control
   wire   PGFSM_PFH_PD_resetb_pre;
   assign PGFSM_PFH_PD_resetb_pre  =         sw_cntl           ? pgfsm_cntl_reg[`GFX_PGFSM_CNTL_PFH_PD_RESETB] :
                                                                 ~pg_reset_pd;
   ati_mux2 d0nt_PGFSM_PFH_PD_resetb_pre_mux ( .i0 (PGFSM_PFH_PD_resetb_pre),
                                               .i1 (PGFSM_DFT_PFH_PD_resetb),
                                               .s  (PGFSM_DFT_PD_resets_override_mode),
                                               .z  (PGFSM_PFH_PD_resetb));
// PD Mother control
//-------------------------------------   
  
`ifdef GFX_PGFSM_MOTHER_POLARITY_AH   
   wire pgfsm_mother_sleep = PGFSM_tile_cold_resetb & (resetb & (sw_cntl ? pgfsm_cntl_reg[`GFX_PGFSM_CNTL_PD_MOTHER_SLEEP] : pd_mother_fsm));
`else
   wire pgfsm_mother_sleep = ~PGFSM_tile_cold_resetb | (reset  | (sw_cntl ? pgfsm_cntl_reg[`GFX_PGFSM_CNTL_PD_MOTHER_SLEEP] : pd_mother_fsm)); 
`endif

   //assign     PGFSM_PD_mother_sleep_pre      = pd_m_dft_mode  ? PGFSM_DFT_PD_mother_sleep : pgfsm_mother_sleep;
   ati_mux2 d0nt_PGFSM_PD_mother_sleep_pre_mux ( .i0 (pgfsm_mother_sleep),
                                                 .i1 (PGFSM_DFT_PD_mother_sleep),
                                                 .s  (pd_m_dft_mode),
                                                 .z  (PGFSM_PD_mother_sleep_pre));

// PD Daughter control
//-------------------------------------   

`ifdef GFX_PGFSM_DAUGHTER_POLARITY_AH   
   wire pgfsm_daughter_sleep = PGFSM_tile_cold_resetb & (resetb & (sw_cntl ? pgfsm_cntl_reg[`GFX_PGFSM_CNTL_PD_DAUGHTER_SLEEP] : pd_daughter_fsm));
`else
   wire pgfsm_daughter_sleep = ~PGFSM_tile_cold_resetb | (reset  | (sw_cntl ? pgfsm_cntl_reg[`GFX_PGFSM_CNTL_PD_DAUGHTER_SLEEP] : pd_daughter_fsm));
`endif
  
   //assign     PGFSM_PD_daughter_sleep_pre    =   pd_d_dft_mode  ? PGFSM_DFT_PD_daughter_sleep : pgfsm_daughter_sleep;
   ati_mux2 d0nt_PGFSM_PD_daughter_sleep_pre_mux ( .i0 (pgfsm_daughter_sleep),
                                                   .i1 (PGFSM_DFT_PD_daughter_sleep),
                                                   .s  (pd_d_dft_mode),
                                                   .z  (PGFSM_PD_daughter_sleep_pre));

//------------------------------------- 
// UBTS 405919
// Sleep Controls in Scan Dump Mode 
// TDR.PGFSM_DFT_PD_scandump_mode = 1
//-------------------------------------

  wire pgfsm_mother_sleep_sd_tck;
  wire pgfsm_daughter_sleep_sd_tck;

  reg pgfsm_mother_sleep_sd_tck_ff;
  reg pgfsm_daughter_sleep_sd_tck_ff;

  // pass through sleep values when PGFSM_DFT_PD_scandump_mode = 0
  // clamp to 0   sleep values when PGFSM_DFT_PD_scandump_mode = 1
  wire pgfsm_mother_sleep_sd   = pgfsm_mother_sleep   & ~PGFSM_DFT_PD_scandump_mode;
  wire pgfsm_daughter_sleep_sd = pgfsm_daughter_sleep & ~PGFSM_DFT_PD_scandump_mode;

  // All flops are on TCK so as to exclude from scan chain and corruption during scandump mode
  techind_sync_icd #(.depth(3)) mother_sleep_tck (.CKdst(PGFSM_DFT_tdr_tck), .Dsrc(pgfsm_mother_sleep_sd), .Qdst(pgfsm_mother_sleep_sd_tck),
                                     .SB(1'b1), .RB(1'b1), .FLUSH(1'b0), .CKsrc(1'b0), .EN(1'b1), .Qsrc(),
                                     .SC1(1'b0), .SC2(1'b0), .SE(1'b0), .SI(1'b0), .SO()
  `ifdef ABV
                                     ,.InputChkDis(1'b0)
  `endif
  );
  techind_sync_icd #(.depth(3)) daughter_sleep_tck (.CKdst(PGFSM_DFT_tdr_tck), .Dsrc(pgfsm_daughter_sleep_sd), .Qdst(pgfsm_daughter_sleep_sd_tck),
                                       .SB(1'b1), .RB(1'b1), .FLUSH(1'b0), .CKsrc(1'b0), .EN(1'b1), .Qsrc(),
                                       .SC1(1'b0), .SC2(1'b0), .SE(1'b0), .SI(1'b0), .SO()
  `ifdef ABV
                                       ,.InputChkDis(1'b0)
  `endif
  );

  // value can change when PGFSM_DFT_PD_scandump_mode = 0
  // value is same    when PGFSM_DFT_PD_scandump_mode = 1
  always @(posedge PGFSM_DFT_tdr_tck) begin
    if (~PGFSM_DFT_PD_scandump_mode) begin
      pgfsm_mother_sleep_sd_tck_ff   <= pgfsm_mother_sleep_sd_tck;
      pgfsm_daughter_sleep_sd_tck_ff <= pgfsm_daughter_sleep_sd_tck;
    end
  end

  assign PGFSM_PD_mother_sleep_pre2   = PGFSM_DFT_PD_scandump_mode ? 
                                 pgfsm_mother_sleep_sd_tck_ff : PGFSM_PD_mother_sleep_pre;

  assign PGFSM_PD_daughter_sleep_pre2 = PGFSM_DFT_PD_scandump_mode ? 
                                      pgfsm_daughter_sleep_sd_tck_ff : PGFSM_PD_daughter_sleep_pre;

//-------------------------------------
// Marker Buffers to easilty identify these nets
//-------------------------------------
   ati_buffer0 d0nt_PGFSM_PD_mother_sleep   (.A(PGFSM_PD_mother_sleep_pre2),   .Y(PGFSM_PD_mother_sleep));
   ati_buffer0 d0nt_PGFSM_PD_daughter_sleep (.A(PGFSM_PD_daughter_sleep_pre2), .Y(PGFSM_PD_daughter_sleep));

//-------------------------------------   
// PD Isolation control
//-------------------------------------   

   wire  pd_iso_clampn_pre, pd_iso_clampn_pre2;
   assign  pd_iso_clampn_pre2 = sw_cntl                        ? pgfsm_cntl_reg[`GFX_PGFSM_CNTL_PD_ISO_VALUE] : pd_iso_fsm;
   ati_mux2    d0nt_pd_iso_clampn_pre_mux  ( .i0 (pd_iso_clampn_pre2),
                                             .i1 (PGFSM_DFT_iso_clampn),
                                             .s  (PGFSM_DFT_PD_iso_override_mode),
                                             .z  (pd_iso_clampn_pre));
   always @(posedge iclk) begin
      if (reset) begin
`ifdef GFX_PGFSM_ISOLATION_POLARITY_AL
         pd_iso_clampn_ff1  <= 1'b1; 
`else
         pd_iso_clampn_ff1  <= 1'b0; 
`endif
      end else begin
         pd_iso_clampn_ff1  <= pd_iso_clampn_pre;
      end // else: !if(reset)
   end
      
   always @(posedge isoclk) begin
      if (reset) begin
`ifdef GFX_PGFSM_ISOLATION_POLARITY_AL
         pd_iso_clampn_ff2  <= 1'b1;
`else
         pd_iso_clampn_ff2  <= 1'b0;
`endif
      end else begin
         pd_iso_clampn_ff2  <= pd_iso_clampn_ff1;
      end // else: !if(reset)
   end

   ati_buffer0 d0nt_PGFSM_PD_iso_clampn (.A(pd_iso_clampn_ff2),   .Y(PGFSM_PD_iso_clampn));

   //Syncronize PD isolation to other two clock domains
   gfx_pgfsm_iso_sync core_iso_sync (
      .iso_clampn            (PGFSM_PD_iso_clampn),
      .iso_clock_1           (PGFSM_iso_clock_1),
      .iso_clock_2           (PGFSM_iso_clock_2),
      .iso_clampn_clock_1    (PGFSM_iso_clampn_clock_1),
      .iso_clampn_clock_2    (PGFSM_iso_clampn_clock_2)
     );

//-------------------------------------    
// PD Scan Isolation control
//-------------------------------------   
assign PGFSM_PD_scan_iso_clampn_pre2 = reset | PGFSM_DFT_ScanEnable | (sw_cntl ? pgfsm_cntl_reg[`GFX_PGFSM_CNTL_PD_ISO_VALUE] : pd_iso_fsm);

   ati_mux2    d0nt_PGFSM_PD_scan_iso_clampn_pre_mux ( .i1 (PGFSM_DFT_scan_iso_clampn),
                                                       .i0 (PGFSM_PD_scan_iso_clampn_pre2),
                                                       .s  (PGFSM_DFT_PD_scan_iso_override_mode),
                                                       .z  (PGFSM_PD_scan_iso_clampn_pre));

   ati_buffer0 d0nt_PGFSM_PD_scan_iso_clampn (.A(PGFSM_PD_scan_iso_clampn_pre),   .Y(PGFSM_PD_scan_iso_clampn));

//-------------------------------------   
// DS_M/SD_M control logic to support new feature in gf20lpm
//-------------------------------------   
`ifdef TECH__TECH__GF20LPM
   reg  mem_mother_shutdown_pd_d1;
   reg  mem_mother_shutdown_pd_m_d1;
   reg  mem_daughter_shutdown_pd_d1;

   wire mem_daughter_shutdown_pd_fall_edge;
   wire mem_mother_shutdown_pd_rise_edge;

   always @(posedge iclk) begin
       if (reset) begin
           mem_daughter_shutdown_pd_d1   <= 1'b0;
           mem_mother_shutdown_pd_d1     <= 1'b0;
           mem_mother_shutdown_pd_m_d1   <= 1'b0;
       end else begin
           mem_daughter_shutdown_pd_d1   <= mem_daughter_shutdown_pd;
           mem_mother_shutdown_pd_m_d1     <= mem_mother_shutdown_pd;
           if (mem_daughter_shutdown_pd_fall_edge) begin
               mem_mother_shutdown_pd_d1   <= 1'b0;
           end else if (mem_mother_shutdown_pd_rise_edge) begin
               mem_mother_shutdown_pd_d1   <= 1'b1;
           end
       end
   end

   assign mem_daughter_shutdown_pd_fall_edge = ~mem_daughter_shutdown_pd & mem_daughter_shutdown_pd_d1;
   assign mem_mother_shutdown_pd_rise_edge   = mem_mother_shutdown_pd & ~mem_mother_shutdown_pd_m_d1;
`endif

//-------------------------------------   
// PD Mem power control
//-------------------------------------   
//add d0nt buffer for sdc
wire [MEM__MEM_POWER_CTRL_IP_SD:MEM__MEM_POWER_CTRL_LS] PGFSM_MEM_PD_power_ctrl_pre;

   assign     PGFSM_MEM_PD_power_ctrl_pre[MEM__MEM_POWER_CTRL_LS]   =        resetb & ( PGFSM_CGLS_override ? 1'b1 :
                                                                                   sw_cntl  ? pgfsm_cntl_reg[`GFX_PGFSM_CNTL_PD_MEM_LS] :
                                                                                   pd_mem_in[MEM__MEM_POWER_CTRL_LS]);
   ati_mux2   d0nt_PGFSM_MEM_PD_power_ctrl_LS_mux ( .i1 (dft_mem_in[MEM__MEM_POWER_CTRL_LS]),
                                                    .i0 (PGFSM_MEM_PD_power_ctrl_pre[MEM__MEM_POWER_CTRL_LS]),
                                                    .s  (PGFSM_DFT_PD_mem_pwr_ctrl_override_mode),
                                                    .z  (PGFSM_MEM_PD_power_ctrl[MEM__MEM_POWER_CTRL_LS]));
   
/* FIXME kshao: temp hack on mem_power_ctrl bus to support new bit definition
   assign     PGFSM_MEM_PD_power_ctrl_pre[MEM__MEM_POWER_CTRL_DS_D] =        resetb & ( (pgfsm_cntl_reg[`GFX_PGFSM_CNTL_DISABLE_MEM_PG]) ? 1'b0 : 
                                                                                    ( (toggle_ds & mem_daughter_shutdown_pd) | pd_mem_in[MEM__MEM_POWER_CTRL_DS_D] ) );
   ati_mux2   d0nt_PGFSM_MEM_PD_power_ctrl_DS_D_mux ( .i1 (dft_mem_in[MEM__MEM_POWER_CTRL_DS_D]),
                                                      .i0 (PGFSM_MEM_PD_power_ctrl_pre[MEM__MEM_POWER_CTRL_DS_D]),
                                                      .s  (PGFSM_DFT_PD_mem_pwr_ctrl_override_mode),
                                                      .z  (PGFSM_MEM_PD_power_ctrl[MEM__MEM_POWER_CTRL_DS_D]));

`ifdef TECH__TECH__GF20LPM
   assign     PGFSM_MEM_PD_power_ctrl_pre[MEM__MEM_POWER_CTRL_DS_M] =        resetb & ( (pgfsm_cntl_reg[`GFX_PGFSM_CNTL_DISABLE_MEM_PG]) ? 1'b0 :
                                                                                    ( (toggle_ds & mem_mother_shutdown_pd_m_d1) | pd_mem_in[MEM__MEM_POWER_CTRL_DS_M] ) );
`else
   assign     PGFSM_MEM_PD_power_ctrl_pre[MEM__MEM_POWER_CTRL_DS_M] =        resetb & ( (pgfsm_cntl_reg[`GFX_PGFSM_CNTL_DISABLE_MEM_PG]) ? 1'b0 :
                                                                                    ( (toggle_ds & mem_mother_shutdown_pd) | pd_mem_in[MEM__MEM_POWER_CTRL_DS_M] ) );
`endif
   ati_mux2   d0nt_PGFSM_MEM_PD_power_ctrl_DS_M_mux ( .i1 (dft_mem_in[MEM__MEM_POWER_CTRL_DS_M]),
                                                      .i0 (PGFSM_MEM_PD_power_ctrl_pre[MEM__MEM_POWER_CTRL_DS_M]),
                                                      .s  (PGFSM_DFT_PD_mem_pwr_ctrl_override_mode),
                                                      .z  (PGFSM_MEM_PD_power_ctrl[MEM__MEM_POWER_CTRL_DS_M]));

   assign     PGFSM_MEM_PD_power_ctrl_pre[MEM__MEM_POWER_CTRL_SD_D] =        resetb & ( (pgfsm_cntl_reg[`GFX_PGFSM_CNTL_DISABLE_MEM_PG]) ? 1'b0 :
                                                                                    ( (~toggle_ds & mem_daughter_shutdown_pd) | pd_mem_in[MEM__MEM_POWER_CTRL_SD_D] ) );
   ati_mux2   d0nt_PGFSM_MEM_PD_power_ctrl_SD_D_mux ( .i1 (dft_mem_in[MEM__MEM_POWER_CTRL_SD_D]),
                                                      .i0 (PGFSM_MEM_PD_power_ctrl_pre[MEM__MEM_POWER_CTRL_SD_D]),
                                                      .s  (PGFSM_DFT_PD_mem_pwr_ctrl_override_mode),
                                                      .z  (PGFSM_MEM_PD_power_ctrl[MEM__MEM_POWER_CTRL_SD_D]));

`ifdef TECH__TECH__GF20LPM
   assign     PGFSM_MEM_PD_power_ctrl_pre[MEM__MEM_POWER_CTRL_SD_M] =        resetb & ( (pgfsm_cntl_reg[`GFX_PGFSM_CNTL_DISABLE_MEM_PG]) ? 1'b0 :
                                                                                    ( (~toggle_ds & mem_mother_shutdown_pd_m_d1) | pd_mem_in[MEM__MEM_POWER_CTRL_SD_M] ) );
`else
   assign     PGFSM_MEM_PD_power_ctrl_pre[MEM__MEM_POWER_CTRL_SD_M] =        resetb & ( (pgfsm_cntl_reg[`GFX_PGFSM_CNTL_DISABLE_MEM_PG]) ? 1'b0 :
                                                                                    ( (~toggle_ds & mem_mother_shutdown_pd) | pd_mem_in[MEM__MEM_POWER_CTRL_SD_M] ) );
`endif
   ati_mux2   d0nt_PGFSM_MEM_PD_power_ctrl_SD_M_mux ( .i1 (dft_mem_in[MEM__MEM_POWER_CTRL_SD_M]),
                                                      .i0 (PGFSM_MEM_PD_power_ctrl_pre[MEM__MEM_POWER_CTRL_SD_M]),
                                                      .s  (PGFSM_DFT_PD_mem_pwr_ctrl_override_mode),
                                                      .z  (PGFSM_MEM_PD_power_ctrl[MEM__MEM_POWER_CTRL_SD_M]));
*/

`ifdef TECH__TECH__GF20LPM
   assign     PGFSM_MEM_PD_power_ctrl_pre[MEM__MEM_POWER_CTRL_IP_DS]   =        resetb & ( sw_cntl ? pgfsm_cntl_reg[`GFX_PGFSM_CNTL_PD_MEM_DS] :
                                                                                   (pgfsm_cntl_reg[`GFX_PGFSM_CNTL_DISABLE_MEM_PG]) ? 1'b0 :
                                                                                    (toggle_ds & mem_mother_shutdown_pd_d1) | pd_mem_in[MEM__MEM_POWER_CTRL_IP_DS]);
`else
   assign     PGFSM_MEM_PD_power_ctrl_pre[MEM__MEM_POWER_CTRL_IP_DS]   =        resetb & ( sw_cntl ? pgfsm_cntl_reg[`GFX_PGFSM_CNTL_PD_MEM_DS] :
                                                                                   (pgfsm_cntl_reg[`GFX_PGFSM_CNTL_DISABLE_MEM_PG]) ? 1'b0 :
                                                                                    (toggle_ds & mem_mother_shutdown_pd) | pd_mem_in[MEM__MEM_POWER_CTRL_IP_DS]);
`endif
   ati_mux2   d0nt_PGFSM_MEM_PD_power_ctrl_DS_mux ( .i1 (dft_mem_in[MEM__MEM_POWER_CTRL_IP_DS]),
                                                    .i0 (PGFSM_MEM_PD_power_ctrl_pre[MEM__MEM_POWER_CTRL_IP_DS]),
                                                    .s  (PGFSM_DFT_PD_mem_pwr_ctrl_override_mode),
                                                    .z  (PGFSM_MEM_PD_power_ctrl[MEM__MEM_POWER_CTRL_IP_DS]));
   
`ifdef TECH__TECH__GF20LPM
   assign     PGFSM_MEM_PD_power_ctrl_pre[MEM__MEM_POWER_CTRL_IP_SD]   =        resetb & ( sw_cntl ? pgfsm_cntl_reg[`GFX_PGFSM_CNTL_PD_MEM_SD] :
                                                                                   (pgfsm_cntl_reg[`GFX_PGFSM_CNTL_DISABLE_MEM_PG]) ? 1'b0 :
                                                                                    (~toggle_ds & mem_mother_shutdown_pd_d1) | pd_mem_in[MEM__MEM_POWER_CTRL_IP_SD]);
`else
   assign     PGFSM_MEM_PD_power_ctrl_pre[MEM__MEM_POWER_CTRL_IP_SD]   =        resetb & ( sw_cntl ? pgfsm_cntl_reg[`GFX_PGFSM_CNTL_PD_MEM_SD] :
                                                                                   (pgfsm_cntl_reg[`GFX_PGFSM_CNTL_DISABLE_MEM_PG]) ? 1'b0 :
                                                                                    (~toggle_ds & mem_mother_shutdown_pd) | pd_mem_in[MEM__MEM_POWER_CTRL_IP_SD]);
`endif
   ati_mux2   d0nt_PGFSM_MEM_PD_power_ctrl_SD_mux ( .i1 (dft_mem_in[MEM__MEM_POWER_CTRL_IP_SD]),
                                                    .i0 (PGFSM_MEM_PD_power_ctrl_pre[MEM__MEM_POWER_CTRL_IP_SD]),
                                                    .s  (pd_s_dft_mode),
                                                    .z  (PGFSM_MEM_PD_power_ctrl[MEM__MEM_POWER_CTRL_IP_SD]));


   assign     PGFSM_MEM_PD_power_ctrl_pre[MEM__MEM_POWER_CTRL_SD : MEM__MEM_POWER_CTRL_DS] = pd_mem_in[MEM__MEM_POWER_CTRL_SD : MEM__MEM_POWER_CTRL_DS];
   assign     PGFSM_MEM_PD_power_ctrl[MEM__MEM_POWER_CTRL - 1: MEM__MEM_POWER_CTRL_IP_SD + 1] = 
                           PGFSM_DFT_PD_mem_pwr_ctrl_override_mode ?
                           dft_mem_in[MEM__MEM_POWER_CTRL - 1 : MEM__MEM_POWER_CTRL_IP_SD + 1] :
                            pd_mem_in[MEM__MEM_POWER_CTRL - 1 : MEM__MEM_POWER_CTRL_IP_SD +1];

   assign     PGFSM_MEM_PD_power_ctrl[MEM__MEM_POWER_CTRL_SD : MEM__MEM_POWER_CTRL_DS] = 
                           PGFSM_DFT_PD_mem_pwr_ctrl_override_mode ?
                           dft_mem_in[MEM__MEM_POWER_CTRL_SD : MEM__MEM_POWER_CTRL_DS] :
                            PGFSM_MEM_PD_power_ctrl_pre[MEM__MEM_POWER_CTRL_SD : MEM__MEM_POWER_CTRL_DS];

`ifdef SIMULATION

`ifdef PG_ASSERT

pgfsm_cov pgfsm_cov
  (
   .clk									(iclk),
   .PGFSM_tile_local_address			(PGFSM_tile_local_address),
   .pgfsm_delay0_reg					(pgfsm_delay0_reg)
   );	


`endif // `ifdef PG_ASSERT

`endif //  `ifdef SIMULATION
//-------------------------------------------------------------------------------------   
//------------------------- Assertions  -----------------------------------------------   
//-------------------------------------------------------------------------------------   

`ifdef SIMULATION
 `ifdef ABV

  `ifdef GFX_EN_PGFSM_ASSERT_1
   reg [15:0] rd_cmd, wr_cmd, pd_cmd, pu_cmd;
   wire [3:0] pgfsm_cmd  = { PGFSM_cmd[`GFX_PGFSM_CMD_READ], PGFSM_cmd[`GFX_PGFSM_CMD_WRITE],
                             PGFSM_cmd[`GFX_PGFSM_CMD_PWRUP], PGFSM_cmd[`GFX_PGFSM_CMD_PWRDN]
                           };

   always @ (posedge iclk) begin
      if (reset) begin
         rd_cmd <= 'd0;
         wr_cmd <= 'd0;
         pu_cmd <= 'd0;
         pd_cmd <= 'd0;
      end else begin
         if(PGFSM_new_cmd_filtered & PGFSM_sel) begin
            case(pgfsm_cmd)
              4'b0001: begin
                 pd_cmd <= pd_cmd + 1;
                 `MSGIO_DISPLAY("%m at %0t --(PGFSM)-- : INFO: New power down cmd recieved. Total power down commands=%d\n", $time, pd_cmd+1);
              end
              4'b0010: begin
                 pu_cmd <= pu_cmd + 1;
                 `MSGIO_DISPLAY("%m at %0t --(PGFSM)-- : INFO: New power up cmd recieved. Total power up commands=%d\n", $time, pu_cmd+1);
              end
              4'b0100: begin
                 wr_cmd <= wr_cmd + 1;
                 //`MSGIO_DISPLAY("%m at %0t --(PGFSM)-- : INFO: New write cmd recieved. Total write commands=%d\n", $time, wr_cmd+1);
              end
              //In SW control mode, AUTO_STATUS_READ  must be 0 when read command is sent
              4'b1000: begin
                 rd_cmd <= rd_cmd + 1;
                 //`MSGIO_DISPLAY("%m at %0t --(PGFSM)-- : INFO: New read cmd recieved. Total RD commands=%d\n", $time, rd_cmd+1);
                 
                 if (pgfsm_cntl_reg[`GFX_PGFSM_CNTL_SW_CNTL] & pgfsm_cntl_reg[`GFX_PGFSM_CNTL_AUTO_STATUS_READ]) begin
                    `MSGIO_DISPLAY("%m at %0t --(PGFSM)-- : ERROR: In SW control mode, AUTO_STATUS_READ  must be 0 when read command is sent\n", $time);
                 end
              end
              //PGFSM command should be one hot
              4'b0011, 4'b0101, 4'b0110, 4'b0111, 4'b1011, 4'b1101, 4'b1110, 4'b1111: begin
                 `MSGIO_DISPLAY("%m at %0t --(PGFSM)-- : ERROR: More than one command bits are set. pgfsm_cmd=%h\n", $time, pgfsm_cmd);
              end
              default : begin
//                 `MSGIO_DISPLAY("%m at %0t --(PGFSM)-- : ERROR: Invalid command. pgfsm_cmd=%h\n", $time, pgfsm_cmd);
              end
            endcase
         end // if (PGFSM_new_cmd_filtered)
      end // else: !if(reset)
   end // always @ (posedge iclk)
  `endif //  `ifdef GFX_EN_PGFSM_ASSERT_1
   
  `ifdef GFX_EN_PGFSM_ASSERT_3
   //When mother or daughter sleep is asserted, isolation must be asserted
   always @ (posedge iclk) begin
      if (reset) begin
      end else begin
      end
   end

  `endif //  `ifdef GFX_EN_PGFSM_ASSERT_1
 `endif //  `ifdef ABV
   
   
`ifdef PGFSM_CORE_SVA_CHECKER_ENABLE
   gfx_pgfsm_assert inst_pgfsm_assert
     (/*AUTOINST*/
      // Inputs
      .iclk				(iclk),
      .reset				(reset),
      .PGFSM_DFT_tdr_en                 (PGFSM_DFT_tdr_en_rw),
      .PGFSM_disable_strap		(PGFSM_disable_strap),
      .PGFSM_SD_next_daisy_status	(PGFSM_SD_next_daisy_status),
      .PGFSM_tile_hard_resetb		(PGFSM_tile_hard_resetb),
      .PGFSM_tile_cold_resetb		(PGFSM_tile_cold_resetb),
      .PGFSM_tile_local_address		(PGFSM_tile_local_address),
      .PGFSM_in_MEM_PD_power_ctrl       (PGFSM_in_MEM_PD_power_ctrl),
      .PGFSM_DFT_tdr_tdi		(PGFSM_DFT_tdr_tdi),
      .PGFSM_DFT_tdr_tresetb		(PGFSM_DFT_tdr_tresetb),
      .PGFSM_DFT_tdr_shift		(PGFSM_DFT_tdr_shift),
      .PGFSM_DFT_tdr_capture		(PGFSM_DFT_tdr_capture),
      .PGFSM_DFT_tdr_update		(PGFSM_DFT_tdr_update),
      .TEST_CLK_EN			(TEST_CLK_EN),
      .PGFSM_CGLS_override		(PGFSM_CGLS_override),
      .PGFSM_SD_prev_daisy_status	(PGFSM_SD_prev_daisy_status),
      .PGFSM_PD_scan_iso_clampn		(PGFSM_PD_scan_iso_clampn),
      .PGFSM_MEM_PD_power_ctrl		(PGFSM_MEM_PD_power_ctrl),
      .PGFSM_DFT_tdr_tdo		(PGFSM_DFT_tdr_tdo_rw),
      .PGFSM_sel			(PGFSM_sel),
      .PGFSM_new_cmd			(PGFSM_new_cmd),
      .PGFSM_cmd			(PGFSM_cmd),
      .PGFSM_PD_resetb			(PGFSM_PD_resetb),
      .PGFSM_PD_iso_clampn		(PGFSM_PD_iso_clampn),
      .PGFSM_PD_mother_sleep		(PGFSM_PD_mother_sleep),
      .PGFSM_PD_daughter_sleep		(PGFSM_PD_daughter_sleep),
      .PGFSM_cmd_done			(PGFSM_cmd_done));

`endif // `ifdef PGFSM_CORE_SVA_CHECKER_ENABLE

//Checking the registers update to field used them.
reg 		pgfsm_wr_cmd_d;
reg [31:0]	PGFSM_cmd_d;
always @(posedge iclk) begin
      if (reset) begin
         pgfsm_wr_cmd_d            <= 1'b0;
	 PGFSM_cmd_d		   <= 32'h0; 
      end else if (pgfsm_wr_cmd)begin
         pgfsm_wr_cmd_d            <= 1'b1;
	 PGFSM_cmd_d		   <= PGFSM_cmd; 
      end else begin
	 pgfsm_wr_cmd_d            <= 1'b0;
	 PGFSM_cmd_d		   <= PGFSM_cmd_d; 
      end

   end

wire  pgfsm_wr_cmd_delay0_reg = pgfsm_wr_cmd_d &&(PGFSM_cmd_d[`GFX_PGFSM_CMD_ADDRESS] == `GFX_PGFSM_DELAY0_REG_ADDR); 
wire  pgfsm_wr_cmd_delay1_reg = pgfsm_wr_cmd_d &&(PGFSM_cmd_d[`GFX_PGFSM_CMD_ADDRESS] == `GFX_PGFSM_DELAY1_REG_ADDR); 
wire  pgfsm_wr_cmd_delay2_reg = pgfsm_wr_cmd_d &&(PGFSM_cmd_d[`GFX_PGFSM_CMD_ADDRESS] == `GFX_PGFSM_DELAY2_REG_ADDR); 
wire  pgfsm_wr_cmd_delay3_reg = pgfsm_wr_cmd_d &&(PGFSM_cmd_d[`GFX_PGFSM_CMD_ADDRESS] == `GFX_PGFSM_DELAY3_REG_ADDR); 
wire  pgfsm_wr_cmd_delay4_reg = pgfsm_wr_cmd_d &&(PGFSM_cmd_d[`GFX_PGFSM_CMD_ADDRESS] == `GFX_PGFSM_DELAY4_REG_ADDR); 
wire  pgfsm_wr_cmd_delay5_reg = pgfsm_wr_cmd_d &&(PGFSM_cmd_d[`GFX_PGFSM_CMD_ADDRESS] == `GFX_PGFSM_DELAY5_REG_ADDR); 


// psl pgfsm_delay0_reg_check : assert never (pgfsm_wr_cmd_delay0_reg && ((pgfsm_delay0_reg != pgfsm_cmd_fifo.pgfsm_delay0_reg)||(isunknown(pgfsm_cmd_fifo.pgfsm_delay0_reg)))) @(posedge iclk) report "pgfsm_delay0_reg can't be set into PGFSM ";
// psl pgfsm_delay1_reg_check : assert never (pgfsm_wr_cmd_delay1_reg && ((pgfsm_delay1_reg != pgfsm_cmd_fifo.pgfsm_delay1_reg)||(isunknown(pgfsm_cmd_fifo.pgfsm_delay1_reg)))) @(posedge iclk) report "pgfsm_delay1_reg can't be set into PGFSM ";
// psl pgfsm_delay2_reg_check : assert never (pgfsm_wr_cmd_delay2_reg && ((pgfsm_delay2_reg != pgfsm_cmd_fifo.pgfsm_delay2_reg)||(isunknown(pgfsm_cmd_fifo.pgfsm_delay2_reg)))) @(posedge iclk) report "pgfsm_delay2_reg can't be set into PGFSM ";
// psl pgfsm_delay3_reg_check : assert never (pgfsm_wr_cmd_delay3_reg && ((pgfsm_delay3_reg != pgfsm_cmd_fifo.pgfsm_delay3_reg)||(isunknown(pgfsm_cmd_fifo.pgfsm_delay3_reg)))) @(posedge iclk) report "pgfsm_delay3_reg can't be set into PGFSM ";
// psl pgfsm_delay4_reg_check : assert never (pgfsm_wr_cmd_delay4_reg && ((pgfsm_delay4_reg != pgfsm_cmd_fifo.pgfsm_delay4_reg)||(isunknown(pgfsm_cmd_fifo.pgfsm_delay4_reg)))) @(posedge iclk) report "pgfsm_delay4_reg can't be set into PGFSM ";
// psl pgfsm_delay5_reg_check : assert never (pgfsm_wr_cmd_delay5_reg && ((pgfsm_delay4_reg != pgfsm_cmd_fifo.pgfsm_delay5_reg)||(isunknown(pgfsm_cmd_fifo.pgfsm_delay5_reg)))) @(posedge iclk) report "pgfsm_delay5_reg can't be set into PGFSM ";

`endif //  `ifdef SIMULATION

endmodule


