/* Delay counter module for pgfsm that run on refclk */

`include "gfx_pgfsm_defines.h"

module gfx_pgfsm_counter( /*autoarg*/
TEST_CLK_EN,
counter_done,
irefclk, 
refclk_en, 
iclk, 
reset, 
delay_pulse,
delay
); 

input TEST_CLK_EN;
output counter_done; 
input irefclk;
input refclk_en; 
input iclk; 
input reset; 
input delay_pulse;
input [23:0] delay;


reg start_cntr_rclk, counter_done_rclk; 
wire reset_refclk; 

techind_sync_icd #(.depth(3)) refclk_reset_sync   (.CKdst(irefclk), .Dsrc(reset), .Qdst(reset_refclk),
                                      .SB(1'b1), .RB(1'b1), .FLUSH(1'b0), .CKsrc(1'b0), .EN(1'b1), .Qsrc(),
                                      .SC1(1'b0), .SC2(1'b0), .SE(1'b0), .SI(1'b0), .SO()
`ifdef ABV
                                      ,.InputChkDis(1'b0)
`endif
);

gfx_pgfsm_clk_cntl sblk_gfx_pgfsm_clk_cntl
(
    .TEST_CLK_EN(TEST_CLK_EN),
    .DST_CLK(irefclk),
    .SRC_CLK(iclk),
    .CLK_EN(refclk_en),
    .CLK_OUT(refclk)
);

ati_sync_pulse done_sync (
// output
.SYNC_OUT(counter_done),
// inputs
.ICLK(refclk),
.IRESET(reset_refclk),
.SYNC_IN(start_cntr_rclk & counter_done_rclk),
.OCLK(iclk)
); 

ati_sync_pulse rst_delay_pulse_sync (
// output
.SYNC_OUT(delay_pulse_rclk),
// inputs
.ICLK(iclk),
.IRESET(reset),
.SYNC_IN(delay_pulse),
.OCLK(refclk)
); 

wire [23:0] delay_rclk_cdc;

techind_cdcefpm #(
   .size(24),
   .applyParams(1),
   .ClockBasedDelay(1),
   .DestClkCnt(2),
   .CdcDestSetup(0),
   .AssertEnable(1)
) pgfsm_delay0_reg_refclk_CDCBUF (
   .DSTCLK(refclk),
   .EN(delay_pulse_rclk),
   .A(delay),
   .Z(delay_rclk_cdc)
);

reg [23:0] counter;

always @(posedge refclk) begin
    if (reset_refclk)                   
      counter <= 24'b0; 
    else if (delay_pulse_rclk)
      counter <= delay;
    else if (start_cntr_rclk & |counter)
      counter <= counter - 24'b1;
end // always @ (posedge refclk)
   
always @(posedge refclk) begin
    if (reset_refclk)                   
      start_cntr_rclk <= 1'b0;
    else if (delay_pulse_rclk)
       start_cntr_rclk <= 1'b1; 
    else if (counter_done_rclk)
      start_cntr_rclk <= 1'b0;
end // always @ (posedge refclk)
   
always @(posedge refclk) begin
   if (reset_refclk)                   
     counter_done_rclk <= 1'b0;
   else if (start_cntr_rclk && (counter == 24'b0)) begin
      counter_done_rclk <= 1'b1;
   end else
     counter_done_rclk <= 1'b0;
end

endmodule
