////////////////////////////////////////////////////////////////////////////
/*
 *
 *  Trade secret of AMD, Inc.
 *  Copyright 2012, AMD, Inc., (unpublished)
 *
 *  All rights reserved.  This notice is intended as a precaution against
 *  inadvertent publication and does not imply publication or any waiver
 *  of confidentiality.  The year included in the foregoing notice is the
 *  year of creation of the work.
 *
 */
////////////////////////////////////////////////////////////////////////////
// Filename    : gfx_pgfsm_iso_sync.v
// Description : This module syncs the isolation control into 2 extra clock domains
////////////////////////////////////////////////////////////////////////////


module gfx_pgfsm_iso_sync (
     iso_clampn,
     iso_clock_1,
     iso_clock_2,
     iso_clampn_clock_1,
     iso_clampn_clock_2
   );

   input    iso_clampn;                   // iso_clampn from main PGFSM clock domain
   input    iso_clock_1;                  // other clocks 1
   input    iso_clock_2;                  // other clocks 2
   output   iso_clampn_clock_1;           // iso_clamps synced to other clock domains 1
   output   iso_clampn_clock_2;           // iso_clamps synced to other clock domains 2

   // IOs
   wire   iso_clampn;
   wire   iso_clock_1;
   wire   iso_clampn_clock_1;
   wire   iso_clock_2;
   wire   iso_clampn_clock_2;
   
   // Local signals
   wire    pd_iso_clk_1;
   reg     pd_iso_clk_1_ff1;
   reg     pd_iso_clk_1_ff2;
   wire    pd_iso_clk_2;
   reg     pd_iso_clk_2_ff1;
   reg     pd_iso_clk_2_ff2;

//-------------------------------------
// PD Isolation control
//-------------------------------------

         //Syncronize PD isolation to other clock domains
         techind_sync_icd #(.depth(3)) iso_clk1   (.CKdst(iso_clock_1), .Dsrc(iso_clampn), .Qdst(pd_iso_clk_1),
                                      .SB(1'b1), .RB(1'b1), .FLUSH(1'b0), .CKsrc(1'b0), .EN(1'b1), .Qsrc(),
                                      .SC1(1'b0), .SC2(1'b0), .SE(1'b0), .SI(1'b0), .SO()
         `ifdef ABV
                                      ,.InputChkDis(1'b1)
         `endif
         );

         techind_sync_icd #(.depth(3)) iso_clk2   (.CKdst(iso_clock_2), .Dsrc(iso_clampn), .Qdst(pd_iso_clk_2),
                                      .SB(1'b1), .RB(1'b1), .FLUSH(1'b0), .CKsrc(1'b0), .EN(1'b1), .Qsrc(),
                                      .SC1(1'b0), .SC2(1'b0), .SE(1'b0), .SI(1'b0), .SO()
         `ifdef ABV
                                      ,.InputChkDis(1'b1)
         `endif
         );
   
         //Add two flops to meet single cycle timing on isolation path
         always @(posedge iso_clock_1) begin
            pd_iso_clk_1_ff1 <= pd_iso_clk_1;
            pd_iso_clk_1_ff2 <= pd_iso_clk_1_ff1;
         end

         always @(posedge iso_clock_2) begin
            pd_iso_clk_2_ff1 <= pd_iso_clk_2;
            pd_iso_clk_2_ff2 <= pd_iso_clk_2_ff1;
         end

         ati_buffer0 d0nt_PGFSM_iso_clampn_clock1 (.A(pd_iso_clk_1_ff2),   .Y(iso_clampn_clock_1));
         ati_buffer0 d0nt_PGFSM_iso_clampn_clock2 (.A(pd_iso_clk_2_ff2),   .Y(iso_clampn_clock_2));

endmodule

