
////////////////////////////////////////////////////////////////////////////
/*
 *
 *  Trade secret of AMD, Inc.
 *  Copyright 2012, AMD, Inc., (unpublished)
 *
 *  All rights reserved.  This notice is intended as a precaution against
 *  inadvertent publication and does not imply publication or any waiver
 *  of confidentiality.  The year included in the foregoing notice is the
 *  year of creation of the work.
 *
 */
////////////////////////////////////////////////////////////////////////////
// Filename    : gfx_pgfsm_ser.v
// Description : This module serializes the data 
// 				 par_valid - Parallel data is ready, pulse signal
//				 par_data  - Parallel data
//				 ser_busy  - Serializer is busy transmitting data, level signal
//				 ser_data  - 1 bit serial data (header(high pulse), data, tail(low pulse))
////////////////////////////////////////////////////////////////////////////

module gfx_pgfsm_ser (ser_busy, ser_data, par_valid, par_data, short_cmd, clk, reset);

parameter DATA_WIDTH    = 32; 
parameter COUNTER_WIDTH = 5;
parameter SHORT_COUNTER_WIDTH   = 3;

output ser_busy;
output ser_data;
input short_cmd;  // enable short command
input par_valid;
input [DATA_WIDTH -1 :0] par_data;
input clk, reset;

reg ser_data;
reg [COUNTER_WIDTH-1: 0] count; 												
reg par_valid_ff;
reg short_cmd_ff;
reg [DATA_WIDTH -1 :0] par_data_ff;

wire count_last;



always @(posedge clk) begin
    if (reset) begin
      short_cmd_ff <= 1'b0;
      par_data_ff  <= {DATA_WIDTH{1'b0}};
    end
    else if(par_valid) begin
      short_cmd_ff <= short_cmd;
      par_data_ff  <= par_data;
    end
end

always @(posedge clk) begin
    if (reset)
        par_valid_ff <= 1'b0;
    else if (par_valid) 
       par_valid_ff  <= 1'b1;
    else if (par_valid_ff & count_last)
       par_valid_ff  <= 1'b0;
end

assign count_last = ~short_cmd_ff ? 
                    &(count[COUNTER_WIDTH-1 : 0]) :
                    (~(|count[COUNTER_WIDTH-1 : SHORT_COUNTER_WIDTH]) & (&(count[SHORT_COUNTER_WIDTH-1 : 0])) );

always @(posedge clk) begin
    if (reset)                          count  <= {COUNTER_WIDTH{1'b0}};
    else if (par_valid_ff & count_last) count  <= {COUNTER_WIDTH{1'b0}};
    else if (par_valid_ff)              count  <= count + 1'b1; 
end 

always @(posedge clk) begin
    if (reset)
        ser_data <= 1'b0;
    else
        ser_data <= par_valid    ? 1'b1 :                 // start pulse
                    par_valid_ff ? par_data_ff[count] :   // serial data
                    1'b0;                                 // idle bits 
end

wire ser_busy = par_valid_ff;


endmodule

////////////////////////////////////////////////////////////////////////////

