
// De-serializer Module for PGFSM block
// Note that this block only works when the number of bits is a power of 2.  
// This is used for Read status deserialization only 

module gfx_pgfsm_status_deser ( /*autoarg*/
   // Outputs
   data_out, new_data,
   // Inputs
   serial_in, iclk, reset, hard_resetb
   ); 

parameter width = 32;                   
parameter counter_width = 5; 

output [width-1:0] data_out; 
output new_data; 
input serial_in; 

input iclk; 
input reset, hard_resetb; 

reg [width-1:0] data_out; 
reg new_data; 
reg [counter_width-1:0] counter; 
reg counter_ff1; 

always @(posedge iclk) begin
    if (reset) data_out <= {width{1'b0}};
    else if (|counter || counter_ff1) begin
        data_out[width-2:0] <= data_out[width-1:1]; 
        data_out[width-1] <= serial_in; 
    end
end

always @(posedge iclk) begin
    if (reset) counter <= {counter_width{1'b0}}; 
    else if (|counter) counter <= counter + {{counter_width-1{1'b0}},1'b1}; 
    else if (serial_in & ~counter_ff1) counter <= {{counter_width-1{1'b0}},1'b1};
end

always @(posedge iclk) counter_ff1 <= &counter; 

always @(posedge iclk) new_data <= counter_ff1 & hard_resetb; 

endmodule

