
/* PGFSM block inserted in each tile that can be powergated */

`include "gfx_pgfsm_defines.h"
`include "gc_features.vh"
`include "mem_features.vh"

module gfx_pgfsm_wrap (
   PGFSM_iso_clock_1,
   PGFSM_iso_clock_2,
   PGFSM_iso_clock_3,
   PGFSM_iso_clock_4,
   PGFSM_iso_clock_5,
   PGFSM_iso_clock_6,
   PGFSM_iso_clock_7,
   PGFSM_iso_clock_8,
   PGFSM_iso_clampn_clock_1,
   PGFSM_iso_clampn_clock_2,
   PGFSM_iso_clampn_clock_3,
   PGFSM_iso_clampn_clock_4,
   PGFSM_iso_clampn_clock_5,
   PGFSM_iso_clampn_clock_6,
   PGFSM_iso_clampn_clock_7,
   PGFSM_iso_clampn_clock_8,
   PGFSM_cmd_done,

   PGFSM_SD_prev_daisy_status,
   PGFSM_SD_next_daisy_cmd,
   PGFSM_P1_mother_sleep,
   PGFSM_P1_daughter_sleep,
   PGFSM_P1_iso_clampn,
   PGFSM_P1_scan_iso_clampn,
   PGFSM_P1_resetb,
   PGFSM_MEM_P1_power_ctrl,
   PGFSM_PFH_P1_resetb,

   // Inputs
   PGFSM_SD_prev_daisy_cmd,
   PGFSM_SD_next_daisy_status,
   PGFSM_tile_ref_clk,
   PGFSM_tile_sclk,
   PGFSM_tile_hard_resetb,
   PGFSM_tile_cold_resetb,
   PGFSM_tile_local_address,
   PGFSM_in_MEM_P1_power_ctrl,

   //PGFSM-DFT interface
   PGFSM_P1500_TARG_si,
   PGFSM_P1500_TARG_trstb,
   PGFSM_P1500_TARG_shift,
   PGFSM_P1500_TARG_capture,
   PGFSM_P1500_TARG_update,
   PGFSM_P1500_TILE_dec,
   PGFSM_P1500_TILE_so,
   PGFSM_P1500_TARG_tclk,


   TEST_CLK_EN

   ); 

   `include "gfx_pgfsm_params.vh"

   // SerDes interface signals. 
   input                                     PGFSM_SD_prev_daisy_cmd; 
   output                                    PGFSM_SD_prev_daisy_status; 
   
   output                                    PGFSM_SD_next_daisy_cmd; 
   input                                     PGFSM_SD_next_daisy_status; 
      
   input                                     PGFSM_tile_ref_clk; 
   input                                     PGFSM_tile_sclk; 
   input                                     PGFSM_tile_hard_resetb; 
   input                                     PGFSM_tile_cold_resetb; 
   
   // Address of the PGFSM 
   input [7:0]                               PGFSM_tile_local_address; 

   output                                    PGFSM_cmd_done;           //Pulses when a FSM finishes current command execution

   // Synced version of iso_clampn
   input                                     PGFSM_iso_clock_1;        //System tile clocks like dclk, dispclk
   input                                     PGFSM_iso_clock_2;        //System tile clocks like dclk, dispclk
   input                                     PGFSM_iso_clock_3;        //System tile clocks like dclk, dispclk
   input                                     PGFSM_iso_clock_4;        //System tile clocks like dclk, dispclk
   input                                     PGFSM_iso_clock_5;        //System tile clocks like dclk, dispclk
   input                                     PGFSM_iso_clock_6;        //System tile clocks like dclk, dispclk
   input                                     PGFSM_iso_clock_7;        //System tile clocks like dclk, dispclk
   input                                     PGFSM_iso_clock_8;        //System tile clocks like dclk, dispclk
   output                                    PGFSM_iso_clampn_clock_1; //Synchronized iso control for clock 1 
   output                                    PGFSM_iso_clampn_clock_2; //Synchronized iso control for clock 2 
   output                                    PGFSM_iso_clampn_clock_3; //Synchronized iso control for clock 3 
   output                                    PGFSM_iso_clampn_clock_4; //Synchronized iso control for clock 4 
   output                                    PGFSM_iso_clampn_clock_5; //Synchronized iso control for clock 5 
   output                                    PGFSM_iso_clampn_clock_6; //Synchronized iso control for clock 6 
   output                                    PGFSM_iso_clampn_clock_7; //Synchronized iso control for clock 7 
   output                                    PGFSM_iso_clampn_clock_8; //Synchronized iso control for clock 8 

   // P1 memory control for the light sleep 
   input [MEM__MEM_POWER_CTRL-1:0]    PGFSM_in_MEM_P1_power_ctrl; 
   
   // Powergate controls
   output                                    PGFSM_P1_mother_sleep; 
   output                                    PGFSM_P1_daughter_sleep; 
   output                                    PGFSM_P1_iso_clampn; 
   output                                    PGFSM_P1_scan_iso_clampn; 
   output                                    PGFSM_P1_resetb; 
   output [MEM__MEM_POWER_CTRL-1:0]   PGFSM_MEM_P1_power_ctrl;
   output				     PGFSM_PFH_P1_resetb; 

   input PGFSM_P1500_TARG_si;
   input PGFSM_P1500_TARG_trstb;
   input PGFSM_P1500_TARG_shift;
   input PGFSM_P1500_TARG_capture;
   input PGFSM_P1500_TARG_update;
   input PGFSM_P1500_TILE_dec;
   output PGFSM_P1500_TILE_so;
   input PGFSM_P1500_TARG_tclk;

   input TEST_CLK_EN;
 
   //--------------------------------------------------------------------------------
   // PGFSM module
   //--------------------------------------------------------------------------------

gfx_pgfsm #( 
         `include "gfx_pgfsm_params_override.vh" 
       ) pgfsm (
   .PGFSM_iso_clock_1           (PGFSM_iso_clock_1),
   .PGFSM_iso_clock_2           (PGFSM_iso_clock_2),
   .PGFSM_iso_clampn_clock_1    (PGFSM_iso_clampn_clock_1),
   .PGFSM_iso_clampn_clock_2    (PGFSM_iso_clampn_clock_2),
   .PGFSM_cmd_done              (PGFSM_cmd_done),

   .PGFSM_SD_prev_daisy_status  (PGFSM_SD_prev_daisy_status),
   .PGFSM_SD_next_daisy_cmd     (PGFSM_SD_next_daisy_cmd),
   .PGFSM_P1_mother_sleep       (PGFSM_P1_mother_sleep),
   .PGFSM_P1_daughter_sleep     (PGFSM_P1_daughter_sleep),
   .PGFSM_P1_iso_clampn         (PGFSM_P1_iso_clampn),
   .PGFSM_P1_scan_iso_clampn    (PGFSM_P1_scan_iso_clampn),
   .PGFSM_P1_resetb             (PGFSM_P1_pwrgood),
   .PGFSM_MEM_P1_power_ctrl     (PGFSM_MEM_P1_power_ctrl),
   .PGFSM_PFH_P1_resetb         (PGFSM_PFH_P1_resetb),

   // Inputs
   .PGFSM_SD_prev_daisy_cmd     (PGFSM_SD_prev_daisy_cmd),
   .PGFSM_SD_next_daisy_status  (PGFSM_SD_next_daisy_status),
   .PGFSM_tile_ref_clk          (PGFSM_tile_ref_clk),
   .PGFSM_tile_sclk             (PGFSM_tile_sclk),
   .PGFSM_tile_hard_resetb      (PGFSM_tile_hard_resetb),
   .PGFSM_tile_cold_resetb      (PGFSM_tile_cold_resetb),
   .PGFSM_tile_local_address    (PGFSM_tile_local_address),
   .PGFSM_in_MEM_P1_power_ctrl  (PGFSM_in_MEM_P1_power_ctrl),

   //PGFSM-DFT interface
   .PGFSM_P1500_TARG_si         (PGFSM_P1500_TARG_si),
   .PGFSM_P1500_TARG_trstb      (PGFSM_P1500_TARG_trstb),
   .PGFSM_P1500_TARG_shift      (PGFSM_P1500_TARG_shift),
   .PGFSM_P1500_TARG_capture    (PGFSM_P1500_TARG_capture),
   .PGFSM_P1500_TARG_update     (PGFSM_P1500_TARG_update),
   .PGFSM_P1500_TILE_dec        (PGFSM_P1500_TILE_dec),
   .PGFSM_P1500_TILE_so         (PGFSM_P1500_TILE_so),
   .PGFSM_P1500_TARG_tclk       (PGFSM_P1500_TARG_tclk),

   .TEST_CLK_EN                 (TEST_CLK_EN)
   );

   //--------------------------------------------------------------------------------
   // gfx_pgfsm_iso_sync module
   // Generate 6 extra sync'ed versions of isolation control
   //--------------------------------------------------------------------------------

gfx_pgfsm_iso_sync top_iso_sync78 (
   .iso_clampn            (PGFSM_P1_iso_clampn),
   .iso_clock_1           (PGFSM_iso_clock_7),
   .iso_clock_2           (PGFSM_iso_clock_8),
   .iso_clampn_clock_1    (PGFSM_iso_clampn_clock_7),
   .iso_clampn_clock_2    (PGFSM_iso_clampn_clock_8)
  );

gfx_pgfsm_iso_sync top_iso_sync56 (
   .iso_clampn            (PGFSM_P1_iso_clampn),
   .iso_clock_1           (PGFSM_iso_clock_5),
   .iso_clock_2           (PGFSM_iso_clock_6),
   .iso_clampn_clock_1    (PGFSM_iso_clampn_clock_5),
   .iso_clampn_clock_2    (PGFSM_iso_clampn_clock_6)
  );

gfx_pgfsm_iso_sync top_iso_sync34 (
   .iso_clampn            (PGFSM_P1_iso_clampn),
   .iso_clock_1           (PGFSM_iso_clock_3),
   .iso_clock_2           (PGFSM_iso_clock_4),
   .iso_clampn_clock_1    (PGFSM_iso_clampn_clock_3),
   .iso_clampn_clock_2    (PGFSM_iso_clampn_clock_4)
  );

endmodule

