//--------------------------------------------------------------------------
//
// $Id: $
//
// $Change: $
//
// Notes:   
//
// Copyright: Trade secret of ATI Technologies, Inc.
//            (C) Copyright 2001-2004, ATI Technologies, Inc., (unpublished) 
//
//        All rights reserved.  This notice is intended as a precaution against
//        inadvertent publication and does not imply publication or any waiver
//        of confidentiality.  The year included in the foregoing notice is the
//        year of creation of the work.
//
// Author: Brian Rerecich
//
//--------------------------------------------------------------------------
`ifdef SIMULATION
`include "msgio_defines.vh"
`endif
`include "gfx_random_unknown.vh"

`include "tech_features.vh"

module kc_reg_in_out_fifo (// Outputs
			    full,
			    empty,
			    read_data,
			    
			    // Inputs
			    we,
			    re,
			    write_data,
			    clk,
			    reset
			    );

   //**************
   //* Parameters *
   //**************
   parameter FIFO_WIDTH = 1;
   parameter FIFO_DEPTH = 2;
   parameter SKID_WORDS = 0;
   parameter SKID_FULL  = 0;
   parameter FIFO_NAME  = ""; // spyglass disable W175 -- declared but not used
   parameter ADDR_WIDTH = ((FIFO_DEPTH <= 2) ? 1 :
                           (FIFO_DEPTH <= 4) ? 2 :
                           (FIFO_DEPTH <= 8) ? 3 :
                           (FIFO_DEPTH <= 16) ? 4 :
                           (FIFO_DEPTH <= 32) ? 5 :
                           (FIFO_DEPTH <= 64) ? 6 :
                           (FIFO_DEPTH <= 128) ? 7 :
                           (FIFO_DEPTH <= 256) ? 8 :
                           (FIFO_DEPTH <= 512) ? 9 : 
                           (FIFO_DEPTH <= 1024) ? 10 : 
                           (FIFO_DEPTH <= 2048) ? 11 : 12);  // Bigger than 4095???  i hope not.

`ifdef SIMULATION
    parameter CYCLES_BEFORE_OVERFLOW_CHECK = 40;
    integer    cycle_count;
`endif

      //***********************
   //* Output Declarations *
   //***********************
   output [FIFO_WIDTH-1:0] read_data;     // FIFO Output Rd Data
   output 		   full;          // FIFO Full signal
   output 		   empty;         // FIFO Empty signal
   
   //**********************
   //* Input Declarations *
   //**********************
   input [FIFO_WIDTH-1:0]  write_data;    // FIFO Input Wr Data
   input 		   we;            // Write Enable
   input 		   re;            // Read Enable
   input 		   clk;            
   input 		   reset;
   
   //***************************
   //* Output REG Declarations *
   //***************************
   reg [FIFO_WIDTH-1:0]    read_data;     // FIFO read data
   reg 			   full;          // FIFO is full
   reg 			   empty;         // FIFO is empty

   //*****************************
   //* Internal REG Declarations *
   //*****************************   

   reg [FIFO_WIDTH-1:0]    d_front_rank;
   reg [FIFO_WIDTH-1:0]    q_front_rank;
   reg [FIFO_WIDTH-1:0]    d_back_rank;
   reg [FIFO_WIDTH-1:0]    q_back_rank;
   
   reg 			   int_fifo_we;
   reg 			   int_fifo_re;
   wire 		   int_fifo_full;
   wire 		   int_fifo_empty;
   wire [FIFO_WIDTH-1:0]   int_fifo_read_data;

   reg 			   set_front_rank_vld;
   reg 			   clr_front_rank_vld;
   reg 			   d_front_rank_vld;
   reg 			   q_front_rank_vld;

   reg 			   set_back_rank_vld;
   reg 			   clr_back_rank_vld;
   reg 			   d_back_rank_vld;
   reg 			   q_back_rank_vld;

   // Put in this checker fifo to help verify the performance and
   // validity of the this fifo design compared to it.
   
`ifdef SIMULATION
   // VCS coverage off
   wire [FIFO_WIDTH-1:0]   read_data_sim;
   wire 		   full_sim, empty_sim;
      
   ati_shift_fifo #(
		    FIFO_WIDTH,        
		    FIFO_DEPTH,
		    "whatever", 
		    SKID_WORDS
		    ) 
     uati_fifo2 (
		 .clk(clk),
		 .reset(reset),
		 .we(we),
		 .re(re),
		 .write_data(write_data),
		 .read_data(read_data_sim),
		 .full(full_sim),
		 .empty(empty_sim)
		 );

   assign fifo_read_data_bad = (read_data_sim != read_data);  // NOTE: not checking Xs on purpose since X assignments are different in each design
   assign fifo_full_bad      = (full_sim      != full);
   assign fifo_empty_bad     = (empty_sim     != empty);

 wire ati_random_x;
 `ifdef ATI_RANDOM_X
 assign ati_random_x = 1;
 `else
 assign ati_random_x = 0;
 `endif
   // psl fifo_bad_read_data: assert never (!ati_random_x && fifo_read_data_bad) @ (posedge clk);
   // psl fifo_bad_full:      assert never fifo_full_bad      @ (posedge clk);
   // psl fifo_bad_empty:     assert never fifo_empty_bad     @ (posedge clk);
   // VCS coverage on
`endif   
   
   //***********************
   //* Combinatorial Logic *
   //***********************   

   always @(/*AS*/int_fifo_empty or q_back_rank_vld or q_front_rank or q_front_rank_vld or re or we or write_data) begin

      d_front_rank = (q_back_rank_vld & we) ? write_data : q_front_rank;
      int_fifo_we  = we & ((q_front_rank_vld & ~re) | ~int_fifo_empty);
      int_fifo_re  = ~int_fifo_empty & re;

      set_front_rank_vld = we & (re ? q_front_rank_vld : q_back_rank_vld);
      clr_front_rank_vld = (int_fifo_we & ~we) | (re & int_fifo_empty & q_front_rank_vld);
      d_front_rank_vld   = set_front_rank_vld | (q_front_rank_vld & ~clr_front_rank_vld);
      
   end

   always @(/*AS*/int_fifo_empty or int_fifo_full or int_fifo_read_data or q_back_rank or q_back_rank_vld or q_front_rank
            or q_front_rank_vld or re or we or write_data) begin

      casez({we, re & q_back_rank_vld, int_fifo_empty, q_front_rank_vld})
	4'b00??,
	4'b100?,
	4'b1011: d_back_rank = q_back_rank;
	4'b1010: d_back_rank = q_back_rank_vld ? q_back_rank : write_data;
	4'b1110: d_back_rank = write_data;
	4'b?10?: d_back_rank = int_fifo_read_data;
	4'b?111: d_back_rank = q_front_rank;
	4'b0110: d_back_rank = `ATI_X(FIFO_WIDTH); // Going to be empty next cycle, so....
      endcase // casez({we, re, int_fifo_empty, q_front_rank_vld})

      set_back_rank_vld = we & int_fifo_empty & ~q_front_rank_vld & (re | ~q_back_rank_vld);
      clr_back_rank_vld = int_fifo_empty & ~q_front_rank_vld & re;
      d_back_rank_vld   = set_back_rank_vld | (q_back_rank_vld & ~clr_back_rank_vld);
      
      // Output Statements
      read_data = q_back_rank;
      full      = int_fifo_full;
      empty     = ~q_back_rank_vld;
      
   end // always @ (...

   generate
      case (FIFO_DEPTH)

	1,2: begin
	   assign int_fifo_full      = q_front_rank_vld & q_back_rank_vld & (~re | we);
	   assign int_fifo_empty     = 1'b1;
	   assign int_fifo_read_data = {FIFO_WIDTH{1'bx}};
	end
	default: begin
	   gfx_new_kc_fifo #(FIFO_WIDTH, FIFO_DEPTH-2, SKID_WORDS, SKID_FULL) uint_fifo
	     ( .full       (int_fifo_full),
	       .empty      (int_fifo_empty),
	       .read_data  (int_fifo_read_data),
	       .we         (int_fifo_we),
	       .re         (int_fifo_re),
	       .write_data (q_front_rank),
	       .clk        (clk),
	       .reset      (reset)
	       );
	end // case: default
      endcase // case(FIFO_DEPTH)
   endgenerate
   
   
   //******************
   //* FFs with reset *
   //******************
   always @(posedge clk) begin

      q_front_rank_vld <= reset ? 1'b0 : d_front_rank_vld;
      q_back_rank_vld  <= reset ? 1'b0 : d_back_rank_vld;

   end
   
   //*********************
   //* FFs without reset *
   //*********************
   always @(posedge clk) begin
if (reset) begin
     q_front_rank <= {FIFO_WIDTH{1'b0}};
     q_back_rank  <= {FIFO_WIDTH{1'b0}};
end
else begin
      q_front_rank <= d_front_rank;
      q_back_rank  <= d_back_rank;
end

   end
`ifdef SIMULATION
   // Adding cover and asserts
   ati_basic_fifo_cover_assert #(1, 0, 1) ati_reg_in_out_basic_cover_assert(.clk(clk), .reset(reset), .empty(empty), .full(full), .push(we), .pop(re));
`endif

endmodule // ati_reg_in_out_fifo
