//                              -*- Mode: Verilog -*-
////////////////////////////////////////////////////////////////////////////
/*
 *
 *  Trade secret of Advance Micro Devices.
 *  Copyright 2008, Advance Micro Devices., (unpublished)
 *
 *  All rights reserved.  This notice is intended as a precaution against
 *  inadvertent publication and does not imply publication or any waiver
 *  of confidentiality.  The year included in the foregoing notice is the
 *  year of creation of the work.
 *
 */
////////////////////////////////////////////////////////////////////////////
//
// Filename    : larr_bist.v
// Description : Latch array BIST controller.
//
////////////////////////////////////////////////////////////////////////////

//`include "larr_bist_features.vh"


//;use comp_config;



module larr_bist_gc10 
(
//clock
input Func_Clk_0,
input wire LARR_DAISY_bist_in_data_0,
output wire LARR_DAISY_bist_out_data_0,
input wire LARR_DAISY_bist_in_control_0,
output wire LARR_DAISY_bist_out_control_0,
output wire LARR_TARG_larr_stky_en_0,

//larr smart mode interface
input wire LARR_DAISY_test_in_select_0,
output wire LARR_DAISY_test_out_select_0,
output wire LARR_BIST_FAIL_Daisy_Out_0,
output wire LARR_BIST_DONE_Daisy_Out_0,
input wire LARR_BIST_FAIL_Daisy_In_0,
input wire LARR_BIST_DONE_Daisy_In_0,


 output wire LARR_TARG_larr_trstb,
 
 //larr smart mode interface
 input wire LARR_BIST_smartmode_en,
 input wire LARR_BIST_smartmode_rstb,
 
 //TDR interface
 input wire Tdr_larr_setup_En,
 output wire Tdr_larr_setup_Tdo,
 input wire Tdr_larr_setup_Tdi,
 
 input wire Tdr_larr_tile_En,
 output wire Tdr_larr_tile_Tdo,
 input wire Tdr_larr_tile_Tdi,

 input wire Tdr_bist_done_En,
 output wire Tdr_bist_done_Tdo,
 input wire Tdr_bist_done_Tdi,
 
 input wire Tdr_Shift,
 input wire Tdr_Capture,
 input wire Tdr_Update,
 //input        Tdr_Tdi,
 input wire Tdr_Tck,
 input wire Tdr_Trstb
 );
   parameter TOTAL_LBIST_CLOCKS = 1;


   wire      tdr_bypass;
   wire      dr_shift;
   wire      dr_update;
   wire      dec_larrmode;
   reg 	     larr_dec;
   wire      BIST_FAIL;
   wire      BIST_DONE;
   ////////////////////////////////////////////////////////////////////////////////////
	     
   wire [128+32-1-32+1-1:0] pattern;
   wire [32-1-0+1-1:0] 					  fillcmp_cntr;
   wire [1-1:0] 								  smartmode_dis;
   
   larr_setup_tdr_gc10 larr_setup_tdr 
     (
      
      .Tdr_Tck               (Tdr_Tck) ,
      .Tdr_Trstb             (Tdr_Trstb),
      .Tdr_Tdi               (Tdr_larr_setup_Tdi),
      .Tdr_Shift             (Tdr_Shift),
      .Tdr_Capture           (Tdr_Capture),
      .Tdr_Update            (Tdr_Update)
      
      
      //TDR --> LARR_SETUP
      ,.pattern               (pattern[128+32-1-32+1-1:0])
      ,.fillcmp_cntr          (fillcmp_cntr[32-1-0+1-1:0])
      ,.smartmode_dis                 (smartmode_dis[1-1:0])
      ,.Tdr_larr_setup_En            (Tdr_larr_setup_En)
      ,.Tdr_larr_setup_Tdo           (Tdr_larr_setup_Tdo)
      );
   
   

   // This flop is connected between BC1500 TDI->TDO for the case where tile only has larr_bist, but not larr_tile.
   // In such cases, this flop make sure BC1500 chain is not broken.
   
   test_reg_async_rst_gc10 #(1) reg_bypass
     (
      .q(tdr_bypass),
      .q_sr(),
      .q_sr_posedge(),
      .d(tdr_bypass),
      .en(Tdr_larr_tile_En),
      .capture(Tdr_Capture),
      .update(Tdr_Update),
      .shift(Tdr_Shift),
      .si(Tdr_larr_tile_Tdi),
      .so(Tdr_larr_tile_Tdo),
      .clk(Tdr_Tck),
      .rstb(Tdr_Trstb),
      .d_rst(1'b0)
      );
   

   
////////////////////////////////////////////////////////////////////////////////////

//Create Bus
   wire [0:0] Func_Clk;
   wire [0:0] bist_done;
   wire [0:0] smartmode_fail;
   
   wire [0:0] LARR_DAISY_bist_in_data;
   wire [0:0] LARR_DAISY_bist_out_data;
   
   wire [0:0] LARR_DAISY_bist_in_control;
   wire [0:0] LARR_DAISY_bist_out_control;
   
   wire [0:0] LARR_DAISY_test_in_select;
   wire [0:0] LARR_DAISY_test_out_select;
   
   wire [0:0] LARR_TARG_larr_stky_en;
   wire [0:0] LARR_TARG_larr_trstb_bus;
   
assign Func_Clk[0] = Func_Clk_0;

assign LARR_DAISY_bist_in_data[0] = LARR_DAISY_bist_in_data_0;
assign LARR_DAISY_bist_out_data_0 = LARR_DAISY_bist_out_data[0];

assign LARR_DAISY_bist_in_control[0]	= LARR_DAISY_bist_in_control_0;
assign LARR_DAISY_bist_out_control_0	= LARR_DAISY_bist_out_control[0];

assign LARR_TARG_larr_stky_en_0	= LARR_TARG_larr_stky_en[0];

assign LARR_DAISY_test_in_select[0] = LARR_DAISY_test_in_select_0;
assign LARR_DAISY_test_out_select_0 = LARR_DAISY_test_out_select[0];


   assign LARR_TARG_larr_trstb = LARR_TARG_larr_trstb_bus[0];
   
   
   


   assign dr_shift	= Tdr_Shift;
   assign dr_update	= Tdr_Update;
   assign dec_larrmode = Tdr_larr_tile_En;
   
   
   // flop the comb signal before cross clock domain.
   always @ (posedge Tdr_Tck)
     if(~Tdr_Trstb)
       larr_dec <= 1'b0;
     else 
       larr_dec <= Tdr_larr_setup_En | Tdr_larr_tile_En;
   
   bist_done_tdr_gc10 bist_done_tdr 
     (
      
      .Tdr_Tck               (Tdr_Tck) ,
      .Tdr_Trstb             (Tdr_Trstb),
      .Tdr_Tdi               (Tdr_bist_done_Tdi),
      .Tdr_Shift             (Tdr_Shift),
      .Tdr_Capture           (Tdr_Capture),
      .Tdr_Update            (Tdr_Update)
      
      
      //TDR --> BIST_DONE
      ,.bist_done            (bist_done[1-1:0])
      ,.smartmode_fail               (smartmode_fail[1-1:0])
      ,.Tdr_bist_done_En             (Tdr_bist_done_En)
      ,.Tdr_bist_done_Tdo            (Tdr_bist_done_Tdo)
      );
   
   
   generate
      genvar i;
      for (i=0; i<TOTAL_LBIST_CLOCKS; i=i+1) begin  : larr_cntl_inst
	 larr_cntl_gc10  larr_cntl 
	   (
	    // Outputs
	    .LARR_DAISY_bist_out_data(LARR_DAISY_bist_out_data[i]),
	    .LARR_DAISY_bist_out_control(LARR_DAISY_bist_out_control[i]),
	    .LARR_DAISY_test_out_select(LARR_DAISY_test_out_select[i]),
	    .LARR_TARG_larr_stky_en(LARR_TARG_larr_stky_en[i]),
	    .LARR_TARG_larr_trstb(LARR_TARG_larr_trstb_bus[i]),
	    .bist_done		(bist_done[i]),
	    
	    // Inputs
	    .Func_Clk		(Func_Clk[i]),
	    .Tdr_Tck		(Tdr_Tck),
	    .Tdr_Trstb		(Tdr_Trstb),
	    .Tdr_Tdi		(Tdr_larr_setup_Tdi),
	    .larr_dec		(larr_dec),
	    .dr_update		(dr_update),
	    .dr_shift		(dr_shift),
	    .dec_bistdone	(Tdr_bist_done_En),
	    .dec_larrmode	(dec_larrmode),
	    .LARR_DAISY_bist_in_data(LARR_DAISY_bist_in_data[i]),
	    .LARR_DAISY_bist_in_control(LARR_DAISY_bist_in_control[i]),
            .LARR_DAISY_test_in_select(LARR_DAISY_test_in_select[i]),
	    .pattern		(pattern[128-1:0]),
	    .fillcmp_cntr	(fillcmp_cntr[32-1:0]),
            .smartmode_dis  (smartmode_dis),
            .smartmode_en  (LARR_BIST_smartmode_en),
            .smartmode_rstb (LARR_BIST_smartmode_rstb),
            .smartmode_fail (smartmode_fail[i])
            );
	 
      end
   endgenerate
   
   assign BIST_FAIL  = smartmode_fail[0];     // first daisy input should be tied 0
   assign BIST_DONE  = bist_done[0];    // first daisy input should be tied 1 
   assign LARR_BIST_FAIL_Daisy_Out_0	= LARR_BIST_FAIL_Daisy_In_0 | BIST_FAIL;
   assign LARR_BIST_DONE_Daisy_Out_0	= LARR_BIST_DONE_Daisy_In_0 & BIST_DONE;
   
   ///////////////////////// Assertion Start ////////////////////////////////////////
   
     
     
`ifdef ABV

// Ignore X, only check z
wire check_for_z = 
    ((Func_Clk_0    === 1'bz) ? 1'bz : 0) |

//`ifndef GPU__CHIP__TILES_PRESENT__1  // for IP only.
    ((LARR_BIST_FAIL_Daisy_In_0    === 1'bz) ? 1'bz : 0) |
    ((LARR_BIST_DONE_Daisy_In_0    === 1'bz) ? 1'bz : 0) |

    ((LARR_BIST_smartmode_en    === 1'bz) ? 1'bz : 0) |
    ((LARR_BIST_smartmode_rstb  === 1'bz) ? 1'bz : 0) |
    ((Tdr_larr_setup_En === 1'bz) ? 1'bz : 0) |
    ((Tdr_larr_setup_Tdi    === 1'bz) ? 1'bz : 0) |
    ((Tdr_larr_tile_En  === 1'bz) ? 1'bz : 0) |
    ((Tdr_bist_done_En  === 1'bz) ? 1'bz : 0) |
    ((Tdr_bist_done_Tdi === 1'bz) ? 1'bz : 0) |
    ((Tdr_Shift    === 1'bz) ? 1'bz : 0) |
    ((Tdr_Capture  === 1'bz) ? 1'bz : 0) |
    ((Tdr_Update   === 1'bz) ? 1'bz : 0) |
    ((Tdr_Tck  === 1'bz) ? 1'bz : 0) |
    ((Tdr_Trstb === 1'bz) ? 1'bz : 0) ;


`include "larr_bist_assert_control.vh"
ovl_never_unknown #(
   .msg("LARR_BIST input pins are Z"))
assert_input_unknown(
   .clock     (Tdr_Tck),
   .reset     (Tdr_Trstb),
   .enable    (`LARR_BIST_ABV_ALWAYS_EN),
   .qualifier (Tdr_Trstb),
   .test_expr (check_for_z),
   .fire      ());

`endif //ABV

///////////////////////// Assertion End ////////////////////////////////////////

endmodule
