//                              -*- Mode: Verilog -*-
////////////////////////////////////////////////////////////////////////////
/*
 *
 *  Trade secret of Advance Micro Devices.
 *  Copyright 2008, Advance Micro Devices., (unpublished)
 *
 *  All rights reserved.  This notice is intended as a precaution against
 *  inadvertent publication and does not imply publication or any waiver
 *  of confidentiality.  The year included in the foregoing notice is the
 *  year of creation of the work.
 *
 */
////////////////////////////////////////////////////////////////////////////
//
// Filename    : larr_cntl.v
// Description : Latch array BIST controller per clock domain.
//
////////////////////////////////////////////////////////////////////////////

//`include "larr_bist_features.vh"
`include "larr_bist_defines.vh"


module larr_cntl_gc10 
(
 input wire Func_Clk,
 input wire Tdr_Tck,
 input wire Tdr_Trstb,
 input wire Tdr_Tdi,
 input wire dr_update,
 input wire dr_shift,
 
 input wire larr_dec,
 input wire dec_bistdone,
 input wire dec_larrmode,
 
 input wire LARR_DAISY_bist_in_data,
 input wire LARR_DAISY_bist_in_control,
 input wire LARR_DAISY_test_in_select,
 
 output reg LARR_DAISY_bist_out_data,
 output reg LARR_DAISY_bist_out_control,
 output reg LARR_DAISY_test_out_select,
 
 output wire LARR_TARG_larr_stky_en,
 output wire LARR_TARG_larr_trstb,
 
 output	reg bist_done,
 
 input wire [128-1:0] pattern,
 input wire [32-1:0] fillcmp_cntr,
 
 input wire smartmode_dis,
 input wire smartmode_en,
 input wire smartmode_rstb,
 output reg smartmode_fail
 
 );
   
   //////////////////// wire & reg //////////////////
   wire io_resetb, io_resetb_sclk, io_resetb_tclk;
   wire smartmode_en_sync_func_clk;
   wire smartmode_en_sync_tck;
   wire larr_sclk;
   wire sclk_dec_larrmode;
   wire sclk_dr_update;
   wire larr_stky_en;
   wire test_out_sel_sync;
   wire bist_data;
   wire larr_dec_sync;
   wire [128-1:0] pattern_sync;
   wire [32-1:0] fillcmp_cntr_sync;
   wire  rst_cntr;
   wire  stky_done_sync;
   wire  fsm_status_state_done_sync;
   wire  test_in_sel_sync;
   wire  LARR_DAISY_larr_bco_pre;
   wire  LARR_DAISY_larr_bdo_pre;
   wire  LARR_DAISY_larr_tsi_pre_1;
   reg  LARR_DAISY_larr_tsi_pre;
   wire  tck_stky_all_1_sync;
   wire  in_stky_tclk;
   
   reg [2:0] tck_stky;
   reg 	sclk_on;
   reg 	fsm_status_state;
   reg [128-1:0] pattern_reg;
   reg sclk_on_pre;
   reg [2:0] current_state, next_state;
   reg [1:0] current_mon, next_mon;
   reg 	start, start_reg, start_pre, larr_control_in;
   reg 	smartmode_start, smartmode_start_reg, smartmode_start_pre;
   reg [32-1:0] cntr_reg, cntr_limit;
   reg [14-1:0] tsel_cntr;
   reg fsm_status_state_done;  // seperate register is require to keep the FSM in STATUS state after stky_done become 0 
   // If we use fsm_status_state sync version, it takes 3 cycles (sync) to propate to FSM. 
   // Meanwhile FSM can already exit STATUS and reach to IDLE
   reg tck_stky_all_1;
   reg stky_done;
   reg stky_reached;
   reg bist_control;
   reg test_out_sel;
   reg in_stky;
   reg in_stky_pre;
   reg [1:0] smartmode_cnt;
   reg smartmode_bist_done;
   wire smartmode_bist_done_sync_tck;
   wire ate_bist_start_sclk;
   reg ate_bist_start;
   wire start_tck;
   reg stky_larr_dec;
   
//TBD- clock gating
   dfx_sync3_gc10  usync_smartmode_en 
     (
      .D(smartmode_en & ~smartmode_dis),
      .Q(smartmode_en_sync_func_clk),
      .CK(Func_Clk));
   
   ati_clock_gate larr_sclk_DfxCLOCKGATER
     (
      .clk_out   (larr_sclk),
      .clk_src   (Func_Clk),
      .clk_en    (sclk_on),
      .TE        (1'b0));	//TBD -- TEST_CLK_EN??
  
   
   dfx_async_rst_sync3_gc10  rst_sync_start_reg 
     (
      .clk      (Tdr_Tck),
      .rstinB   (start_reg),
      .rstoutB  (start_tck)
      );

   dfx_async_rst_sync3_gc10  rst_sync_ate_bist_start 
     (
      .clk      (larr_sclk),
      .rstinB   (ate_bist_start),
      .rstoutB  (ate_bist_start_sclk)
      );
 
   always @(posedge Tdr_Tck or negedge Tdr_Trstb) 
     begin
	if(~Tdr_Trstb)
	  begin
        ate_bist_start <= 1'b0;
	  end
	else if(start_tck)
	  begin
        ate_bist_start <= 1'b0;
	  end
	else
	  begin
        ate_bist_start <= (dec_larrmode & dr_update) | ate_bist_start;
	  end
     end // always @ (posedge Tdr_Tck or negedge Tdr_Trstb)
   
   
   //This bit indicates if the current BIST sequence has properly ended on STKY.  
   // Clear after read out.
   always @(posedge Func_Clk) 
     begin
	if (~io_resetb_sclk )
	  begin
	     smartmode_bist_done <= 1'b0;
	  end
	else if( (smartmode_cnt == 2'b10) & stky_done ) //stky_done =1 put fsm in status state. 
	  begin
	     smartmode_bist_done <= 1'b1;
	  end
	else
	  begin
	     smartmode_bist_done <= smartmode_bist_done;
	  end
     end // always @ (posedge Func_Clk)
   
   
   
   
   
   dfx_sync3_gc10  usync_smartmode_bist_done 
     (
      .D(smartmode_bist_done),
      .Q(smartmode_bist_done_sync_tck),
      .CK(Tdr_Tck));
   
   always @(posedge Tdr_Tck or negedge io_resetb_tclk) 
     begin // async rst as fuse_if block get this signal.
	if (~io_resetb_tclk )  
	  bist_done <= 1'b0;
	else if(dec_bistdone & dr_update)  //smartmode bist_done is not cleared. Need functional reset to clear this
	  bist_done <= 1'b0;
	else if( (tck_stky == 3'b011 & smartmode_dis)  | 
             (fsm_status_state_done & smartmode_bist_done_sync_tck & ~smartmode_dis)
            ) // give enough time to flush out all below signals to known value.
	  //bist_done <= ~smartmode_dis ? smartmode_bist_done_sync_tck : 1'b1;
	  bist_done <= 1'b1;
	else
	  bist_done <= bist_done;
     end // always @ (posedge Tdr_Tck or negedge io_resetb_tclk)
   
   
   
   
   //Test clock for latch arrays only toggles when we are trying to capture sticky bit.
   //tck_stky = during BFSM_STKY state
   //test_out_sel_sync & ~smartmode_dis = smart mode SETUP state.
   //fsm_status_state = smart_mode STATUS state
   assign larr_stky_en = (tck_stky == 3'b001) |  
			 (test_out_sel_sync & ~smartmode_dis) |
			 (fsm_status_state & ~smartmode_dis);
   
   //The LARR_TARG_larr_bdo bit gets assigned one of the pattern bits based on the counter value.  On compare adjust for state transition.
   assign bist_data = pattern_reg[0];
   
   
   // Synchronizing TRSTB reset de-assertion to sclk domain
   
   ati_mux2 mux_rst  
     (
      .i0(smartmode_rstb & smartmode_en),
      .i1(Tdr_Trstb),
      .s(smartmode_dis),   //TDR
      .z(io_resetb));
   
   
   dfx_async_rst_sync3_gc10  rst_sync_ioresetb_s 
     (
      .clk      (Func_Clk),
      .rstinB   (io_resetb),
      .rstoutB  (io_resetb_sclk)
      );
   
   dfx_async_rst_sync3_gc10  rst_sync_ioresetb_t 
     (
      .clk      (Tdr_Tck),
      .rstinB   (io_resetb),
      .rstoutB  (io_resetb_tclk)
      );
   
   
   ati_buffer1 mrk_larr_trstb (.A(io_resetb_tclk), .Y(LARR_TARG_larr_trstb));
   
   
   dfx_sync3_gc10  usync_larr_dec
     (
      .D(larr_dec),
      .Q(larr_dec_sync),
      .CK(Func_Clk));
   
  

   always @(posedge Func_Clk)
     begin
      if(~io_resetb_sclk) begin
       stky_larr_dec <= 1'b0;
      end else if(larr_dec_sync) begin
        stky_larr_dec <= 1'b1;
      end else  begin
        stky_larr_dec <= stky_larr_dec;
      end
     end

   //Gate off SCLK to run only when reset or a LARR BIST related instruction is active.
   always @(posedge Func_Clk)
     begin
	if (~io_resetb_sclk)
	  begin
	     sclk_on_pre <= 1'b1;
	     sclk_on <= 1'b1;
	  end
	else
	  begin
	     sclk_on_pre <= stky_larr_dec | smartmode_en_sync_func_clk  ; 
	     sclk_on <= sclk_on_pre;
	  end
     end // always @ (posedge Func_Clk)
   


always @(posedge Func_Clk) begin
   if (~io_resetb_sclk) begin
      smartmode_cnt <= 2'd0;
   end else if(smartmode_cnt == 2'd3) begin
      smartmode_cnt <= 2'd3;
   end else if(smartmode_start) begin
      smartmode_cnt <= smartmode_cnt + 2'd1;
   end
  end

//Reset state machine.  BIST is triggered by an dr_update when the LARR_TSEL instruction is loaded.
always @(posedge larr_sclk) begin
  if (~io_resetb_sclk)
    begin
      current_state <= `LARR_BIST_FSM_IDLE;
      current_mon <= `LARR_BIST_MON_FSM_IDLE;
      start_reg <= 1'b0;
      start_pre <= 1'b0;
      start <= 1'b0;
      larr_control_in <= 1'b0;

      smartmode_start_reg <= 1'b0;
      smartmode_start_pre <= 1'b0;
      smartmode_start     <= 1'b0;
    end
  else
    begin
      current_state <= next_state;
      current_mon <= next_mon;
      start_reg <= ate_bist_start_sclk;
      start_pre <= ate_bist_start_sclk & ~start_reg;
      start <= start_pre;
      larr_control_in <= LARR_DAISY_bist_in_control;

      smartmode_start_reg <= smartmode_en_sync_func_clk & (current_state != `LARR_BIST_FSM_STATUS); // run smartmode 2 times.
      smartmode_start_pre <= smartmode_en_sync_func_clk & (current_state != `LARR_BIST_FSM_STATUS) & ~smartmode_start_reg;
      smartmode_start     <= smartmode_start_pre & (smartmode_cnt != 2'b10);

    end
end





techind_cdcefpm  #(
      .size(128)
    )
  pattern_cdcbufen (
      .A(pattern),    // [size-1:0]
      .EN(start_pre | smartmode_start_pre),   // single-bit
      .DSTCLK(larr_sclk),     // single-bit
      .Z(pattern_sync)   // [size-1:0]
    );

techind_cdcefpm  #(
      .size(32)
    )
  fillcmp_cntr_cdcbufen (
      .A(fillcmp_cntr),    // [size-1:0]
      .EN(start_pre | smartmode_start_pre),   // single-bit
      .DSTCLK(larr_sclk),     // single-bit
      .Z(fillcmp_cntr_sync)   // [size-1:0]
    );


//Synchronize pattern and counter value to LARR_SCLK domain from TCLK domain.  Pattern shifts when we are not idle.

always @(posedge larr_sclk) begin
  if (~io_resetb_sclk ) begin
    cntr_limit <= 32'd0;
  end else if (start_pre | smartmode_start_pre) begin
      cntr_limit <= fillcmp_cntr_sync[32-1:0];
  end
end

always @(posedge larr_sclk) begin
   if (start_pre | smartmode_start_pre) begin
      pattern_reg <= smartmode_cnt == 2'b00 ? pattern_sync[128-1:0]: ~(pattern_sync[128-1:0]);
   end
   else begin
      if ( (current_state == `LARR_BIST_FSM_IDLE) ||  (current_state == `LARR_BIST_FSM_SETUP) )
        pattern_reg <= pattern_reg[128-1:0];
      else
        pattern_reg <= {pattern_reg[0], pattern_reg[128-1:1]};
   end
end


//Counter is reset to zero when it reaches programmed value, otherwise it increments during COMP and FILL states.
always @(posedge larr_sclk) begin
  if (~io_resetb_sclk )
    cntr_reg <= 32'd0;
  else if (rst_cntr)
    cntr_reg <= 32'd0;
  else if ( (current_state == `LARR_BIST_FSM_FILL) || (current_state == `LARR_BIST_FSM_COMP) )
    cntr_reg <= cntr_reg + 1'b1;
  else
    cntr_reg <= cntr_reg;
end



dfx_sync3_gc10 usync_stky_done (
        .D(stky_done),
        .Q(stky_done_sync),
        .CK(Tdr_Tck));



always @(posedge Tdr_Tck) begin
 if (~io_resetb_tclk ) begin
  tsel_cntr <= 14'd0;
  fsm_status_state <= 1'd0;
  fsm_status_state_done <= 1'd0;
 end else if (test_out_sel_sync) begin
  tsel_cntr <= tsel_cntr + 1'b1;    //LARR_FSM_SETUP_STATE
  fsm_status_state <= 1'd0;
  fsm_status_state_done <= 1'd0;
 end else if (tsel_cntr == 14'd0) begin
  tsel_cntr <= tsel_cntr;  
  fsm_status_state <= 1'd0;
  fsm_status_state_done <= 1'd1;  // Done when all larrs status is read.  
 end else if (fsm_status_state | stky_done_sync) begin 
  tsel_cntr <= tsel_cntr - 1'b1;    //LARR_FSM_STATUS_STATE
  fsm_status_state <= 1'd1; // after sync (3 cycles) + 2 cycles, the stky_done becomes 1 and STATE change to FSM_STATUS.
  fsm_status_state_done <= 1'd0;
 end
 end

always @(posedge Tdr_Tck) begin
 if (~io_resetb_tclk ) begin
 smartmode_fail <= 1'b0;
 end else if(fsm_status_state) begin
 smartmode_fail <= smartmode_fail | LARR_DAISY_test_in_select;
 end else begin
 smartmode_fail <= smartmode_fail;
 end
 end



dfx_sync3_gc10 usync_fsm_status_state_done (
        .D(fsm_status_state_done),
        .Q(fsm_status_state_done_sync),
        .CK(Func_Clk));



//Latch array control monitoring state machine.
always @(current_mon, larr_control_in, current_state) begin
   case (current_mon)
     `LARR_BIST_MON_FSM_IDLE:
       begin
          next_mon = (current_state == `LARR_BIST_FSM_IDLE) ? `LARR_BIST_MON_FSM_IDLE : (larr_control_in ? `LARR_BIST_MON_FSM_FILL : `LARR_BIST_MON_FSM_IDLE);
          stky_reached = 1'b0;
       end
     `LARR_BIST_MON_FSM_FILL:
       begin
          next_mon = larr_control_in ? `LARR_BIST_MON_FSM_FILL : `LARR_BIST_MON_FSM_COMP;
          stky_reached = 1'b0;
       end
     `LARR_BIST_MON_FSM_COMP:
       begin
          next_mon = larr_control_in ? `LARR_BIST_MON_FSM_COMP : `LARR_BIST_MON_FSM_STKY;
          stky_reached = 1'b0;
       end
     `LARR_BIST_MON_FSM_STKY:
       begin
          next_mon = larr_control_in ? `LARR_BIST_MON_FSM_STKY : `LARR_BIST_MON_FSM_IDLE;
          stky_reached = 1'b1;
       end

   endcase // case(current_mon)
end // always @ (current_mon, larr_control_in)



dfx_sync3_gc10 usync_test_in_sel (
        .D(LARR_DAISY_test_in_select),
        .Q(test_in_sel_sync),
        .CK(Func_Clk));


//Latch array central BIST controller state machine.
always @(stky_done, start, smartmode_start, smartmode_en_sync_func_clk, fsm_status_state_done_sync, test_in_sel_sync, rst_cntr, current_state) begin
  case (current_state)
    `LARR_BIST_FSM_IDLE:
      begin
        next_state = smartmode_start ? `LARR_BIST_FSM_SETUP :
                               start ? `LARR_BIST_FSM_FILL  : `LARR_BIST_FSM_IDLE;

        bist_control = smartmode_start ? 0 : start ;
        test_out_sel = 0;

       // next_state = start ? `LARR_BIST_FSM_FILL : `LARR_BIST_FSM_IDLE;
       // bist_control = start;
      end
    `LARR_BIST_FSM_SETUP:
      begin
        next_state = test_in_sel_sync ? `LARR_BIST_FSM_FILL : `LARR_BIST_FSM_SETUP; //Go to FILL when all TSEL is set to 1.
        bist_control = 0;
        test_out_sel = 1;   // Set LARR_DAISY_test_out_select to 1 to fill TSEL bits to 1.
        //bist_control = start;
      end
    `LARR_BIST_FSM_FILL:
      begin
        next_state = rst_cntr ? `LARR_BIST_FSM_COMP : `LARR_BIST_FSM_FILL;  //rst_cntr become 1 after cnt reach to 0.
        bist_control = !rst_cntr;
        test_out_sel = 0;   // TCK is not toggling to larr, so this is d0nt care.
      end
    `LARR_BIST_FSM_COMP:
      begin
        next_state = rst_cntr ? `LARR_BIST_FSM_STKY : `LARR_BIST_FSM_COMP;
        bist_control = !rst_cntr;
        test_out_sel = 0;   // TCK is not toggling to larr, so this is d0nt care.
      end
    `LARR_BIST_FSM_STKY:
      begin
        next_state = stky_done ? 
                     (smartmode_en_sync_func_clk ? `LARR_BIST_FSM_STATUS: `LARR_BIST_FSM_IDLE) : `LARR_BIST_FSM_STKY;

        //next_state = stky_done ? `LARR_BIST_FSM_IDLE : `LARR_BIST_FSM_STKY;
        bist_control = !stky_done;
        test_out_sel = 0;
      end
    `LARR_BIST_FSM_STATUS:
      begin
        next_state = fsm_status_state_done_sync ? `LARR_BIST_FSM_IDLE : `LARR_BIST_FSM_STATUS;

        bist_control = 0;
        test_out_sel = 0;

      end
     default:
      begin
        next_state = `LARR_BIST_FSM_IDLE;
        bist_control = 0;
        test_out_sel = 0;
      end

  endcase
end



ati_buffer1 mrk_rst_cntr (.A( (cntr_reg == cntr_limit) ), .Y(rst_cntr));
ati_buffer1 mrk_bco (.A(bist_control), .Y(LARR_DAISY_larr_bco_pre));
ati_buffer1 mrk_bdo (.A(bist_data), .Y(LARR_DAISY_larr_bdo_pre));
ati_buffer1 mrk_tsi (.A(test_out_sel), .Y(LARR_DAISY_larr_tsi_pre_1));

ati_buffer1 mrk_larr_stky_en (.A(larr_stky_en), .Y(LARR_TARG_larr_stky_en));

//Flop larr_sclk outputs
always @(posedge larr_sclk) begin
  LARR_DAISY_bist_out_control <= LARR_DAISY_larr_bco_pre;
  LARR_DAISY_bist_out_data <= LARR_DAISY_larr_bdo_pre;
end

always @(posedge larr_sclk) begin
  if (~io_resetb_sclk)
    begin
   LARR_DAISY_larr_tsi_pre <= 1'b0; 
    end
  else
    begin
   LARR_DAISY_larr_tsi_pre <= LARR_DAISY_larr_tsi_pre_1; // CDC combo-logic fix
    end
end

dfx_sync3_gc10 usync_test_out_sel (
        .D(LARR_DAISY_larr_tsi_pre),
        .Q(test_out_sel_sync),
        .CK(Tdr_Tck));

always @(posedge Tdr_Tck) begin
  LARR_DAISY_test_out_select <= test_out_sel_sync & ~smartmode_dis ;
end





//CDC fix
always @ (posedge Tdr_Tck)
 tck_stky_all_1 <= (tck_stky == 3'b111);
dfx_sync3_gc10 usync_tck_stky_all_1 (
        .D(tck_stky_all_1),
        .Q(tck_stky_all_1_sync),
        .CK(larr_sclk));


//Handshaking logic between larr_sclk and tck domain for capturing sticky bit.
always @(posedge larr_sclk) begin
  if (~io_resetb_sclk)
    begin
      stky_done <= 1'b0;
      in_stky_pre <= 1'b0;
    end
  else
    begin
      stky_done <= tck_stky_all_1_sync;
      in_stky_pre <= stky_reached;  // 1 after all LARR FSM in stky state.
    end
end



//CDC fix
dfx_sync3_gc10 usync_in_stky (
        .D(in_stky_pre),
        .Q(in_stky_tclk),
        .CK(Tdr_Tck));

always @(posedge Tdr_Tck) begin
  if (~io_resetb_tclk)
    begin
      in_stky <= 1'b0;
    end
  else
    begin
      in_stky <= in_stky_tclk;
    end
end

always @(posedge Tdr_Tck) begin
  if (~io_resetb_tclk)
    begin
      tck_stky <= 3'd0;
    end
  else
    begin
      tck_stky <= {tck_stky[1:0], in_stky};
    end
end

endmodule
