//                              -*- Mode: Verilog -*-

////////////////////////////////////////////////////////////////////////////
//
// Filename    : larr_tile.v
// Description : Latch array BIST node for core level connections.
//
////////////////////////////////////////////////////////////////////////////



module larr_tile_gc10 
  (
 
   input   wire Func_Clk,
   input   wire LARR_DAISY_bist_in_data,
   output  reg LARR_DAISY_bist_out_data,
   input   wire LARR_DAISY_bist_in_control,
   output  reg LARR_DAISY_bist_out_control,
   input   wire LARR_DAISY_test_in_select,
   output  wire LARR_DAISY_test_out_select,
   
   input   wire LARR_TARG_larr_stky_en,
   
   input   wire LARR_TILE_bist_in_data,
   output  reg LARR_TILE_bist_out_data,
   input   reg LARR_TILE_bist_in_control,
   output  reg LARR_TILE_bist_out_control,
   
   output  wire LARR_TILE_larr_tck,
   input   wire LARR_TILE_test_in_select,
   output  wire LARR_TILE_test_out_select,
   
   input   wire     Tdr_Shift,
   input   wire     Tdr_Capture,  //unused
   input   wire     Tdr_Update,   //unused
   //input wire       Tdr_Tdi,       
   input   wire     Tdr_Tck,
   input   wire     Tdr_Trstb,    //unused

   input  wire      LARR_TARG_larr_trstb,
   output  wire      LARR_TILE_rstb,
   
   //p1500
   input  wire      Tdr_larr_tile_En,
   input  wire      Tdr_larr_tile_Tdi,
   output  reg      Tdr_larr_tile_Tdo
   );
   
   reg 	   larr_tile_so;
   
   wire    larr_tck_en;
   wire    larr_tck_int;
   wire    daisy_test_in_muxed;
   
   
   
   
   ati_clkbuf1 mrk_larr_tck 
     (
      .A(larr_tck_int),
      .Y(LARR_TILE_larr_tck)    
      );
  
//IEEE standard to send negedge data on TDO.
 reg LARR_TILE_test_in_select_ff;
 always @ (negedge LARR_TILE_larr_tck or negedge LARR_TARG_larr_trstb)
  if(~LARR_TARG_larr_trstb)
   LARR_TILE_test_in_select_ff <= 1'b0;
  else
   LARR_TILE_test_in_select_ff <= LARR_TILE_test_in_select;




   ati_buffer1 mrk_tile_test_daisy_end 
     (
      .A(LARR_TILE_test_in_select_ff),
      .Y(Tdr_larr_tile_Tdo)
      );
   
   ati_buffer1 mrk_tile_test_daisy_out 
     (
      .A(LARR_TILE_test_in_select_ff),
      .Y(LARR_DAISY_test_out_select)
      );
   
   // daisy_test_in is for smart mode. 
   assign  daisy_test_in_muxed = Tdr_larr_tile_En ? Tdr_larr_tile_Tdi : LARR_DAISY_test_in_select;
   
   ati_buffer1 mrk_tile_test_daisy_start 
     (
      .A(daisy_test_in_muxed),	
      .Y(LARR_TILE_test_out_select)
      );

//Placeholder to provide rst to bcFF,bdFF flops in larr
   dfx_async_rst_sync3_gc10  rst_sync_larr_trstb 
     (
      .clk      (Func_Clk),
      .rstinB   (LARR_TARG_larr_trstb),
      .rstoutB  (LARR_TILE_rstb)
      );

 //TBD use above rst  
   always @(posedge Func_Clk) 
     begin
	LARR_TILE_bist_out_data <= LARR_DAISY_bist_in_data;
	LARR_TILE_bist_out_control <= LARR_DAISY_bist_in_control;
	LARR_DAISY_bist_out_data <= LARR_TILE_bist_in_data;
	LARR_DAISY_bist_out_control <= LARR_TILE_bist_in_control;
     end
   
   assign larr_tck_en = ( (Tdr_larr_tile_En & Tdr_Shift) || LARR_TARG_larr_stky_en);
   
   ati_clock_gate larr_tck_DfxCLOCKGATER   
     (.clk_out   (larr_tck_int),
      .clk_src   (Tdr_Tck),
      .clk_en    (larr_tck_en),
      .TE        (1'b0));
   
endmodule // larr_tile
