//                              -*- Mode: Verilog -*-

////////////////////////////////////////////////////////////////////////////
/*
 *
 *  Trade secret of ATI Technologies, Inc.
 *  Copyright 2005, ATI Technologies, Inc., (unpublished)
 *
 *  All rights reserved.  This notice is intended as a precaution against
 *  inadvertent publication and does not imply publication or any waiver
 *  of confidentiality.  The year included in the foregoing notice is the
 *  year of creation of the work.
 *
 */
////////////////////////////////////////////////////////////////////////////

//`include "dft_clk_cntl_features.vh"


module latency_cnt_gc10 
# (
parameter DFT_CLK_CNTL_LATENCY_CNT = 10
)

(
 input wire scan_mode_sync_func_clk,
 input wire SE_Sel,
 input wire Scan_Mode,
 input wire Func_Clk,
 input wire Tile_Shift_Clk,
 input wire Tile_ScanEn,
 input wire [DFT_CLK_CNTL_LATENCY_CNT-1:0] latency_cnt,
 output wire load_pul_cnt,
 output wire scan_en_ff_muxed
 
 );
   
   wire load_lat_cnt;
   reg 	scan_en_ff; // Added to match the pipeline stage.
   reg [DFT_CLK_CNTL_LATENCY_CNT-1:0] lat_cnt;
   
   always @(negedge Tile_Shift_Clk) 
     begin
	scan_en_ff <= Tile_ScanEn;
     end
   
   ati_mux2 mux_scan_en  
     (
      .i0(Tile_ScanEn),
      .i1(scan_en_ff),
      .s(SE_Sel),
      .z(scan_en_ff_muxed));
   
   
   //Require few pulses to flush out sync
   //Synchronize to pll_clk
   dfx_sync4_gc10 #(
   .TransportDelay(0),
   .TransportCycle(0)
   ) sync_scanen
     (
      .CK(Func_Clk),
      .D(scan_en_ff_muxed),
      .Q(scan_en_sync));
   
   assign load_lat_cnt = scan_en_sync;
   
   
   
   always @ (posedge Func_Clk)
     begin
        if (~scan_mode_sync_func_clk) begin    //Tdr_Trstb rst_synced
           lat_cnt <= {(DFT_CLK_CNTL_LATENCY_CNT){1'b0}};
        end else if (load_lat_cnt) begin     //load when SE=1. when se-shift counter saturate, counter starts decrementing on pll clk.
           lat_cnt <= latency_cnt;  //CDC waive
        end else if (lat_cnt == {(DFT_CLK_CNTL_LATENCY_CNT){1'b0}}) begin
           lat_cnt <= lat_cnt;
        end else begin
           lat_cnt <= lat_cnt - {{(DFT_CLK_CNTL_LATENCY_CNT-1){1'd0}}, 1'd1};
        end
     end
   
   
   assign load_pul_cnt = (|lat_cnt) | scan_en_sync; 
   
   
   
endmodule // latency_cnt


