//==================================================================================================
//
// maclkstretch.v
// 
// Copyright (c) 2002 by Advanced Micro Devices, Inc.
//
// This file is protected by Federal Copyright Law, with all rights
// reserved. No part of this file may be reproduced, stored in a
// retrieval system, translated, transcribed, or transmitted, in any
// form, or by any means manual, electric, electronic, mechanical,
// electro-magnetic, chemical, optical, or otherwise, without prior
// explicit written permission from Advanced Micro Devices, Inc.
//
//==================================================================================================
//
// INTERESTED PARTIES
//
// Circuit              Ju Tung Ng
// RTL                  Ricky Lau  
// Verification         Sean Sequeira 
//
//==================================================================================================
//
// Module Description
//
// Clock stretcher block used in discrete gpu Volcanic Islands products
//
//==================================================================================================


`timescale 1ps/1ps

module maclkstretch (

input           VDDPLL,
//input           VDDDFS,                 // New: 0.935V from DFS
input           VDD,
input           VSS,
                      
input           LDO_EN,
input           STRETCH_ENABLE,
input           masterReset,
input           dfs_clk,
input     [3:0] STRETCH_AMOUNT,         // New: added [3]
input     [3:0] STRETCH_AMOUNT_PCC,
input           pcc_detected,
input           skip_phase_bypass,
input           droopdetected,          
input     [3:0] FSM_WAIT_CYCLES,        // New: Number of cycles to wait after each delay setting change
input     [3:0] SAMPLE_SIZE,           // New: Number of samples to take to determine Lead/Lag
//input           TAKE_MULT_SAMPLES,     // New: If 1, FSM will sample lead/lag multiple times set by SAMPLE_SIZE[3:0].  If 0, FSM will only sample once.  Default = 0.
input           NO_EXTRA_COARSE_STEP, // New: If 1, FSM will increase delay by one coarse step going into FINE_DEC state.  Default = 1.  Default = 0.
//input           LDO_ON,                 // New: turns on LDO
input     [3:0] LDO_REFSEL,             // New: reference select for LDO
//input           LDO_USE_VDDDFS,         // New: Use VDDDFS instead of LDO
input           ISO_DIS_PWROK_VDDC,     // New: 0 = BACO.
input     [7:0] LDO_READY_COUNT_VAL,    // New: Number of cycles to count after LDO_READY is asserted.
input     [15:0] RESERVE,                // New: reserve pins
input     [1:0] MET_CTRL_SEL,
input           LDO_BIAS_SEL,           // Selects LDO opamp bias circuit.  Default=0
input     [1:0] FREQ_MODE,          // Selects the frequency range supported
input     [1:0] PD_SYNC_SEL,        // Selects the level of synchronization from PD output
input     [6:0] LOW_BW_CONFIG,
input     [6:0] LOW_BW_CONFIG_UP,
input     [2:0] LOW_BW_SKEW_EN,
input     [1:0] CONSERVATIVE_LOCK,
input           BYPASS_LDO,             // 1 = bypass LDO and use core VDD

input     [3:0] CHARZ_SEL_A,
input     [3:0] CHARZ_SEL_B,
input     [1:0] CHARZ_DIV_A,
input     [1:0] CHARZ_DIV_B,
input     [1:0] FSM_STEP_SIZE,
input     [1:0] FSM_CODE_MARGIN,
input           CHARZ_DEBUG_MODE,
input     [1:0] FSM_DEBUG_SEL,
input           FSM_OVERRIDE_EN,
input     [3:0] FSM_OVERRIDE_VAL,

output          charz_observ_a,
output          charz_observ_b,

// CPU specific
input      [14:0] SyncBusIn,
output     [14:0] SyncBusOut,
// end CPU specific

output wire     debug_LDO_READY,              
output wire     sclk,
output wire     cks_dll_lock,
output wire     skip_phase,
output    [3:0] debug_c0,                // New: debug
output    [3:0] debug_c19,               // New: debug
output    [3:0] debug_c29,               // New: debug
output    [3:0] debug_c39                // New: debug

);

`ifndef MACRO_HEADER_ONLY

`ifndef HWEMUL

// Delay config bits to the 20 delay cells in the dll delay chain
wire      [3:0] c0_VDD;
wire      [3:0] c1_VDD;
wire      [3:0] c2_VDD;
wire      [3:0] c3_VDD;
wire      [3:0] c4_VDD;
wire      [3:0] c5_VDD;
wire      [3:0] c6_VDD;
wire      [3:0] c7_VDD;
wire      [3:0] c8_VDD;
wire      [3:0] c9_VDD;
wire      [3:0] c10_VDD;
wire      [3:0] c11_VDD;
wire      [3:0] c12_VDD;
wire      [3:0] c13_VDD;
wire      [3:0] c14_VDD;
wire      [3:0] c15_VDD;
wire      [3:0] c16_VDD;
wire      [3:0] c17_VDD;
wire      [3:0] c18_VDD;
wire      [3:0] c19_VDD;
wire      [3:0] c20_VDD;
wire      [3:0] c21_VDD;
wire      [3:0] c22_VDD;
wire      [3:0] c23_VDD;
wire      [3:0] c24_VDD;
wire      [3:0] c25_VDD;
wire      [3:0] c26_VDD;
wire      [3:0] c27_VDD;
wire      [3:0] c28_VDD;
wire      [3:0] c29_VDD;
wire      [3:0] c30_VDD;
wire      [3:0] c31_VDD;
wire      [3:0] c32_VDD;
wire      [3:0] c33_VDD;
wire      [3:0] c34_VDD;
wire      [3:0] c35_VDD;
wire      [3:0] c36_VDD;
wire      [3:0] c37_VDD;
wire      [3:0] c38_VDD;
wire      [3:0] c39_VDD;

wire      [3:0] c0P_VDDINT;
wire      [3:0] c1P_VDDINT;
wire      [3:0] c2P_VDDINT;
wire      [3:0] c3P_VDDINT;
wire      [3:0] c4P_VDDINT;
wire      [3:0] c5P_VDDINT;
wire      [3:0] c6P_VDDINT;
wire      [3:0] c7P_VDDINT;
wire      [3:0] c8P_VDDINT;
wire      [3:0] c9P_VDDINT;
wire      [3:0] c10P_VDDINT;
wire      [3:0] c11P_VDDINT;
wire      [3:0] c12P_VDDINT;
wire      [3:0] c13P_VDDINT;
wire      [3:0] c14P_VDDINT;
wire      [3:0] c15P_VDDINT;
wire      [3:0] c16P_VDDINT;
wire      [3:0] c17P_VDDINT;
wire      [3:0] c18P_VDDINT;
wire      [3:0] c19P_VDDINT;
wire      [3:0] c20P_VDDINT;
wire      [3:0] c21P_VDDINT;
wire      [3:0] c22P_VDDINT;
wire      [3:0] c23P_VDDINT;
wire      [3:0] c24P_VDDINT;
wire      [3:0] c25P_VDDINT;
wire      [3:0] c26P_VDDINT;
wire      [3:0] c27P_VDDINT;
wire      [3:0] c28P_VDDINT;
wire      [3:0] c29P_VDDINT;
wire      [3:0] c30P_VDDINT;
wire      [3:0] c31P_VDDINT;
wire      [3:0] c32P_VDDINT;
wire      [3:0] c33P_VDDINT;
wire      [3:0] c34P_VDDINT;
wire      [3:0] c35P_VDDINT;
wire      [3:0] c36P_VDDINT;
wire      [3:0] c37P_VDDINT;
wire      [3:0] c38P_VDDINT;
wire      [3:0] c39P_VDDINT;

// === VDD to VDDINT signals ===
wire         dfs_clk_gated_VDD;
wire         dfs_clk_gated_VDDINT;
wire         stretch_clock_VDD;
wire         pcc_detected_qual_VDD;
wire         stretch_clock_VDDINT;
//wire         masterReset_VDD;
//wire         masterReset_VDDINT;
wire         clkpicker_reset_VDD;
wire         clkpicker_reset_VDDINT;
wire         STRETCH_ENABLE_VDD;
wire         STRETCH_ENABLE_VDDINT;
wire         clkpick_sel_skip2_VDD;
wire         clkpick_sel_skip2_VDDINT;
wire         clkpick_sel_skip4_VDD;
wire         clkpick_sel_skip4_VDDINT;

wire   [1:0] MET_CTRL_SEL_VDDINT;

// === VDD to VDDPLL signals ===
//wire   [3:0] LDO_REFSEL_VDD = LDO_REFSEL;
wire   [3:0] LDO_REFSEL_VDDPLL;
//wire         LDO_USE_VDDDFS_VDD = LDO_USE_VDDDFS;
//wire         LDO_USE_VDDDFS_VDDPLL;
wire         LDO_ON_VDD;
wire         LDO_ON_VDDPLL;

// === VDDINT to VDD signals ===
wire         lead_for_lock_VDDINT;
wire         lead_for_lock_VDD;
wire         ck_en0_VDDINT;
wire         ck_en0_VDD;
wire         ckout_clkpicker_VDDINT;
wire         ckout_clkpicker_VDD;

wire  [40:0] ck_phP_VDDINT;
wire  [40:0] ck_phN_VDDINT;

wire         LDO_VPB;                    // Regulator PMOS gate voltage
wire         LDO_OFF_VDD;

wire   [1:0] test_clk_sel_VDD;
wire   [1:0] test_clk_sel_VDDINT;
wire         test_clk_ref_VDD;
wire         test_clk_ref_VDDINT;
wire         test_clk_fb_VDD;
wire         test_clk_fb_VDDINT;


//====================================================================
mbstretch_LStop I_LStop (
        // VDD to VDDINT
        .c0_VDD (c0_VDD),
        .c0P_VDDINT (c0P_VDDINT),
        .c1_VDD (c1_VDD),
        .c1P_VDDINT (c1P_VDDINT),
        .c2_VDD (c2_VDD),
        .c2P_VDDINT (c2P_VDDINT),
        .c3_VDD (c3_VDD),
        .c3P_VDDINT (c3P_VDDINT),
        .c4_VDD (c4_VDD),
        .c4P_VDDINT (c4P_VDDINT),
        .c5_VDD (c5_VDD),
        .c5P_VDDINT (c5P_VDDINT),
        .c6_VDD (c6_VDD),
        .c6P_VDDINT (c6P_VDDINT),
        .c7_VDD (c7_VDD),
        .c7P_VDDINT (c7P_VDDINT),
        .c8_VDD (c8_VDD),
        .c8P_VDDINT (c8P_VDDINT),
        .c9_VDD (c9_VDD),
        .c9P_VDDINT (c9P_VDDINT),
        .c10_VDD (c10_VDD),
        .c10P_VDDINT (c10P_VDDINT),
        .c11_VDD (c11_VDD),
        .c11P_VDDINT (c11P_VDDINT),
        .c12_VDD (c12_VDD),
        .c12P_VDDINT (c12P_VDDINT),
        .c13_VDD (c13_VDD),
        .c13P_VDDINT (c13P_VDDINT),
        .c14_VDD (c14_VDD),
        .c14P_VDDINT (c14P_VDDINT),
        .c15_VDD (c15_VDD),
        .c15P_VDDINT (c15P_VDDINT),
        .c16_VDD (c16_VDD),
        .c16P_VDDINT (c16P_VDDINT),
        .c17_VDD (c17_VDD),
        .c17P_VDDINT (c17P_VDDINT),
        .c18_VDD (c18_VDD),
        .c18P_VDDINT (c18P_VDDINT),
        .c19_VDD (c19_VDD),
        .c19P_VDDINT (c19P_VDDINT),
        .c20_VDD (c20_VDD),
        .c20P_VDDINT (c20P_VDDINT),
        .c21_VDD (c21_VDD),
        .c21P_VDDINT (c21P_VDDINT),
        .c22_VDD (c22_VDD),
        .c22P_VDDINT (c22P_VDDINT),
        .c23_VDD (c23_VDD),
        .c23P_VDDINT (c23P_VDDINT),
        .c24_VDD (c24_VDD),
        .c24P_VDDINT (c24P_VDDINT),
        .c25_VDD (c25_VDD),
        .c25P_VDDINT (c25P_VDDINT),
        .c26_VDD (c26_VDD),
        .c26P_VDDINT (c26P_VDDINT),
        .c27_VDD (c27_VDD),
        .c27P_VDDINT (c27P_VDDINT),
        .c28_VDD (c28_VDD),
        .c28P_VDDINT (c28P_VDDINT),
        .c29_VDD (c29_VDD),
        .c29P_VDDINT (c29P_VDDINT),
        .c30_VDD (c30_VDD),
        .c30P_VDDINT (c30P_VDDINT),
        .c31_VDD (c31_VDD),
        .c31P_VDDINT (c31P_VDDINT),
        .c32_VDD (c32_VDD),
        .c32P_VDDINT (c32P_VDDINT),
        .c33_VDD (c33_VDD),
        .c33P_VDDINT (c33P_VDDINT),
        .c34_VDD (c34_VDD),
        .c34P_VDDINT (c34P_VDDINT),
        .c35_VDD (c35_VDD),
        .c35P_VDDINT (c35P_VDDINT),
        .c36_VDD (c36_VDD),
        .c36P_VDDINT (c36P_VDDINT),
        .c37_VDD (c37_VDD),
        .c37P_VDDINT (c37P_VDDINT),
        .c38_VDD (c38_VDD),
        .c38P_VDDINT (c38P_VDDINT),
        .c39_VDD (c39_VDD),
        .c39P_VDDINT (c39P_VDDINT),
        .ISO_DIS_PWROK_VDDC (ISO_DIS_PWROK_VDDC),
        .dfs_clk_gated_lf_VDD (dfs_clk_gated_lf_VDD),
        .dfs_clk_gated_lf_VDDINT (dfs_clk_gated_lf_VDDINT),
        .dfs_clk_gated_mf_VDD (dfs_clk_gated_mf_VDD),
        .dfs_clk_gated_mf_VDDINT (dfs_clk_gated_mf_VDDINT),
        .dfs_clk_gated_hf_VDD (dfs_clk_gated_hf_VDD),
        .dfs_clk_gated_hf_VDDINT (dfs_clk_gated_hf_VDDINT),
        .stretch_clock_VDD (stretch_clock_VDD),
        .stretch_clock_VDDINT (stretch_clock_VDDINT),
        .clkpicker_reset_VDD (clkpicker_reset_VDD),
        .clkpicker_reset_VDDINT (clkpicker_reset_VDDINT),
        .clkpicker_enable_VDD (clkpicker_enable_VDD),
        .clkpicker_enable_VDDINT (clkpicker_enable_VDDINT),
        .clkpick_sel_skip2_VDD (clkpick_sel_skip2_VDD),
        .clkpick_sel_skip2_VDDINT (clkpick_sel_skip2_VDDINT),
        .clkpick_sel_skip4_VDD (clkpick_sel_skip4_VDD),
        .clkpick_sel_skip4_VDDINT (clkpick_sel_skip4_VDDINT),
        .use_ph28_VDD (use_ph28_VDD), 
        .use_ph28_VDDINT (use_ph28_VDDINT),

        .clkpick_sel_skip2_pcc_VDD (clkpick_sel_skip2_pcc_VDD),
        .clkpick_sel_skip2_pcc_VDDINT (clkpick_sel_skip2_pcc_VDDINT),
        .clkpick_sel_skip4_pcc_VDD (clkpick_sel_skip4_pcc_VDD),
        .clkpick_sel_skip4_pcc_VDDINT (clkpick_sel_skip4_pcc_VDDINT),
        .use_ph28_pcc_VDD (use_ph28_pcc_VDD), 
        .use_ph28_pcc_VDDINT (use_ph28_pcc_VDDINT),
        
        .pcc_detected_VDD (pcc_detected_qual_VDD),
        .pcc_detected_VDDINT (pcc_detected_VDDINT),

        .MET_CTRL_SEL_1_VDD (MET_CTRL_SEL[1]),
        .MET_CTRL_SEL_1_VDDINT (MET_CTRL_SEL_VDDINT[1]),
        .MET_CTRL_SEL_0_VDD (MET_CTRL_SEL[0]),
        .MET_CTRL_SEL_0_VDDINT (MET_CTRL_SEL_VDDINT[0]),
        .test_clk_sel_1_VDD       (test_clk_sel_VDD[1]),
        .test_clk_sel_1_VDDINT    (test_clk_sel_VDDINT[1]),
        .test_clk_sel_0_VDD       (test_clk_sel_VDD[0]),
        .test_clk_sel_0_VDDINT    (test_clk_sel_VDDINT[0]),
        //.RESERVE_VDD2VDDINT_7_VDD       (RESERVE[7]),
        //.RESERVE_VDD2VDDINT_7_VDDINT    (RESERVE_7_VDDINT),
        //.RESERVE_VDD2VDDINT_6_VDD       (RESERVE[6]),
        //.RESERVE_VDD2VDDINT_6_VDDINT    (RESERVE_6_VDDINT),
        .RESERVE_VDD2VDDINT_5_VDD       (RESERVE[5]),
        .RESERVE_VDD2VDDINT_5_VDDINT    (RESERVE_5_VDDINT),
        .RESERVE_VDD2VDDINT_4_VDD       (RESERVE[4]),
        .RESERVE_VDD2VDDINT_4_VDDINT    (RESERVE_4_VDDINT),
        .RESERVE_VDD2VDDINT_3_VDD       (RESERVE[3]),
        .RESERVE_VDD2VDDINT_3_VDDINT    (RESERVE_3_VDDINT),
        .RESERVE_VDD2VDDINT_2_VDD       (RESERVE[2]),
        .RESERVE_VDD2VDDINT_2_VDDINT    (RESERVE_2_VDDINT),
        .RESERVE_VDD2VDDINT_1_VDD       (RESERVE[1]),
        .RESERVE_VDD2VDDINT_1_VDDINT    (RESERVE_1_VDDINT),
        .RESERVE_VDD2VDDINT_0_VDD       (RESERVE[0]),
        .RESERVE_VDD2VDDINT_0_VDDINT    (RESERVE_0_VDDINT),
        // VDD to VDDPLL
        .LDO_ON_VDD (LDO_ON_VDD),
        .LDO_ON_VDDPLL (LDO_ON_VDDPLL),
        .COMP_EN_VDD (COMP_EN_VDD),
        .COMP_EN_VDDPLL (COMP_EN_VDDPLL),
        .LDO_BIAS_SEL_VDD (LDO_BIAS_SEL),
        .LDO_BIAS_SEL_VDDPLL (LDO_BIAS_SEL_VDDPLL),
        .LDO_REFSEL_3_VDD (LDO_REFSEL[3]),
        .LDO_REFSEL_3_VDDPLL (LDO_REFSEL_VDDPLL[3]),
        .LDO_REFSEL_2_VDD (LDO_REFSEL[2]),
        .LDO_REFSEL_2_VDDPLL (LDO_REFSEL_VDDPLL[2]),
        .LDO_REFSEL_1_VDD (LDO_REFSEL[1]),
        .LDO_REFSEL_1_VDDPLL (LDO_REFSEL_VDDPLL[1]),
        .LDO_REFSEL_0_VDD (LDO_REFSEL[0]),
        .LDO_REFSEL_0_VDDPLL (LDO_REFSEL_VDDPLL[0]),
        .RESERVE_VDD2VDDPLL_7_VDD       (RESERVE[7]),
        .RESERVE_VDD2VDDPLL_7_VDDPLL    (RESERVE_7_VDDPLL),
        .RESERVE_VDD2VDDPLL_6_VDD       (RESERVE[6]),
        .RESERVE_VDD2VDDPLL_6_VDDPLL    (RESERVE_6_VDDPLL),
        .RESERVE_VDD2VDDPLL_5_VDD       (RESERVE[5]),
        .RESERVE_VDD2VDDPLL_5_VDDPLL    (RESERVE_5_VDDPLL),
        .RESERVE_VDD2VDDPLL_4_VDD       (RESERVE[4]),
        .RESERVE_VDD2VDDPLL_4_VDDPLL    (RESERVE_4_VDDPLL),
        .RESERVE_VDD2VDDPLL_3_VDD       (RESERVE[3]),
        .RESERVE_VDD2VDDPLL_3_VDDPLL    (RESERVE_3_VDDPLL),
        .RESERVE_VDD2VDDPLL_2_VDD       (RESERVE[2]),
        .RESERVE_VDD2VDDPLL_2_VDDPLL    (RESERVE_2_VDDPLL),
        .RESERVE_VDD2VDDPLL_1_VDD       (RESERVE[1]),
        .RESERVE_VDD2VDDPLL_1_VDDPLL    (RESERVE_1_VDDPLL),
        .RESERVE_VDD2VDDPLL_0_VDD       (RESERVE[0]),
        .RESERVE_VDD2VDDPLL_0_VDDPLL    (RESERVE_0_VDDPLL),
        // VDDPLL to VDD
        .COMP_HI_VDDPLL (COMP_HI_VDDPLL),
        .COMP_HI_VDD (COMP_HI_VDD),
        .COMP_LO_VDDPLL (COMP_LO_VDDPLL),
        .COMP_LO_VDD (COMP_LO_VDD),
        // VDDINT to VDD
        .lead_for_lock_lf_VDDINT (lead_for_lock_lf_VDDINT),
        .lead_for_lock_lf_VDD (lead_for_lock_lf_VDD),
        .lead_for_lock_mf_VDDINT (lead_for_lock_mf_VDDINT),
        .lead_for_lock_mf_VDD (lead_for_lock_mf_VDD),
        .lead_for_lock_hf_VDDINT (lead_for_lock_hf_VDDINT),
        .lead_for_lock_hf_VDD (lead_for_lock_hf_VDD),
        .ck_en0_lf_VDDINT (ck_en0_lf_VDDINT),
        .ck_en0_lf_VDD (ck_en0_lf_VDD),
        .ck_en0_mf_VDDINT (ck_en0_mf_VDDINT),
        .ck_en0_mf_VDD (ck_en0_mf_VDD),
        .ck_en0_hf_VDDINT (ck_en0_hf_VDDINT),
        .ck_en0_hf_VDD (ck_en0_hf_VDD),
        .ckout_clkpicker_lf_VDDINT (ckout_clkpicker_lf_VDDINT),
        .ckout_clkpicker_lf_VDD (ckout_clkpicker_lf_VDD),
        .ckout_clkpicker_mf_VDDINT (ckout_clkpicker_mf_VDDINT),
        .ckout_clkpicker_mf_VDD (ckout_clkpicker_mf_VDD),
        .ckout_clkpicker_hf_VDDINT (ckout_clkpicker_hf_VDDINT),
        .ckout_clkpicker_hf_VDD (ckout_clkpicker_hf_VDD),
        .test_clk_ref_VDDINT (test_clk_ref_VDDINT),
        .test_clk_ref_VDD    (test_clk_ref_VDD),
        .test_clk_fb_VDDINT  (test_clk_fb_VDDINT),
        .test_clk_fb_VDD     (test_clk_fb_VDD),
        .test_dd_sync_VDDINT (test_dd_sync_VDDINT),
        .test_dd_sync_VDD    (test_dd_sync_VDD),
        // Power and Ground
        .VDD (VDD),
        .VDDINT (VDDINT),
        .VDDPLL (VDDPLL),
        .VSS (VSS)
        );


//====================================================================
mbstretch_sapr_top I_sapr_top (
        // Inputs
        .dfs_clk (dfs_clk),
        .ckout_clkpicker_lf (ckout_clkpicker_lf_VDD),
        .ckout_clkpicker_mf (ckout_clkpicker_mf_VDD),
        .ckout_clkpicker_hf (ckout_clkpicker_hf_VDD),
        .LDO_EN (LDO_EN),
        .STRETCH_ENABLE (STRETCH_ENABLE),
        .masterReset (masterReset),
        
        .LDO_COMP_HI (COMP_HI_VDD),
        .LDO_COMP_LO (COMP_LO_VDD),
        
        .STRETCH_AMOUNT (STRETCH_AMOUNT),
        .STRETCH_AMOUNT_PCC (STRETCH_AMOUNT_PCC),
        .FSM_WAIT_CYCLES (FSM_WAIT_CYCLES),
        .lead_for_lock_lf (lead_for_lock_lf_VDD),           
        .lead_for_lock_mf (lead_for_lock_mf_VDD),           
        .lead_for_lock_hf (lead_for_lock_hf_VDD),           
        .skip_phase_bypass (skip_phase_bypass), // DFT controlled allow them unrestricted access
        .droopdetected (droopdetected),         // Allow software/firmware unrestricted access
        .pcc_detected (pcc_detected),
        .ck_en0_lf (ck_en0_lf_VDD),                   
        .ck_en0_mf (ck_en0_mf_VDD),                   
        .ck_en0_hf (ck_en0_hf_VDD),                   
        .LDO_READY_COUNT_VAL (LDO_READY_COUNT_VAL),
        .NO_EXTRA_COARSE_STEP (NO_EXTRA_COARSE_STEP),
        .SAMPLE_SIZE (SAMPLE_SIZE),
        .RESERVE (RESERVE),
        .FREQ_MODE (FREQ_MODE),
        .PD_SYNC_SEL (PD_SYNC_SEL),
        .LOW_BW_CONFIG (LOW_BW_CONFIG),
        .LOW_BW_SKEW_EN(LOW_BW_SKEW_EN),
        .LOW_BW_CONFIG_UP (LOW_BW_CONFIG_UP),
        .CONSERVATIVE_LOCK (CONSERVATIVE_LOCK),
        .BYPASS_LDO (BYPASS_LDO),
        .SyncBusIn(SyncBusIn),

        .CHARZ_SEL_A(CHARZ_SEL_A),
        .CHARZ_SEL_B(CHARZ_SEL_B),
        .CHARZ_DIV_A(CHARZ_DIV_A),
        .CHARZ_DIV_B(CHARZ_DIV_B),
        .FSM_STEP_SIZE(FSM_STEP_SIZE),
        .FSM_CODE_MARGIN(FSM_CODE_MARGIN),
        .CHARZ_DEBUG_MODE(CHARZ_DEBUG_MODE),
        .FSM_DEBUG_SEL(FSM_DEBUG_SEL),
        .FSM_OVERRIDE_EN(FSM_OVERRIDE_EN),
        .FSM_OVERRIDE_VAL(FSM_OVERRIDE_VAL),
        .test_clk_ref(test_clk_ref_VDD),
        .test_clk_fb(test_clk_fb_VDD),
        .test_dd_sync(test_dd_sync_VDD),
        
        // Outputs
        .charz_observ_a(charz_observ_a),
        .charz_observ_b(charz_observ_b),
        .test_clk_sel(test_clk_sel_VDD),
        .SyncBusOut(SyncBusOut),

        .c0 (c0_VDD),
        .c1 (c1_VDD),
        .c2 (c2_VDD),
        .c3 (c3_VDD),
        .c4 (c4_VDD),
        .c5 (c5_VDD),
        .c6 (c6_VDD),
        .c7 (c7_VDD),
        .c8 (c8_VDD),
        .c9 (c9_VDD),
        .c10 (c10_VDD),
        .c11 (c11_VDD),
        .c12 (c12_VDD),
        .c13 (c13_VDD),
        .c14 (c14_VDD),
        .c15 (c15_VDD),

        .c16 (c16_VDD),
        .c17 (c17_VDD),
        .c18 (c18_VDD),
        .c19 (c19_VDD),
        .c20 (c20_VDD),
        .c21 (c21_VDD),
        .c22 (c22_VDD),
        .c23 (c23_VDD),
        .c24 (c24_VDD),
        .c25 (c25_VDD),
        .c26 (c26_VDD),
        .c27 (c27_VDD),
        .c28 (c28_VDD),
        .c29 (c29_VDD),
        .c30 (c30_VDD),
        .c31 (c31_VDD),
        .c32 (c32_VDD),
        .c33 (c33_VDD),
        .c34 (c34_VDD),
        .c35 (c35_VDD),
        .c36 (c36_VDD),
        .c37 (c37_VDD),
        .c38 (c38_VDD),
        .c39 (c39_VDD),
       
        .clkpick_sel_skip2 (clkpick_sel_skip2_VDD),
        .clkpick_sel_skip4 (clkpick_sel_skip4_VDD),
        .use_ph28          (use_ph28_VDD),

        .clkpick_sel_skip2_pcc (clkpick_sel_skip2_pcc_VDD),
        .clkpick_sel_skip4_pcc (clkpick_sel_skip4_pcc_VDD),
        .use_ph28_pcc          (use_ph28_pcc_VDD),
        
        .LDO_COMP_EN (COMP_EN_VDD),
        .skip_phase (skip_phase),           // 1 toggle per base stretch period
        .stretch_clock (stretch_clock_VDD),
        .pcc_detected_qual (pcc_detected_qual_VDD),
        .cks_dll_lock (cks_dll_lock),
        .clkpicker_reset (clkpicker_reset_VDD),
        .clkpicker_enable (clkpicker_enable_VDD),
        .debug_c0 (debug_c0),
        .debug_c19 (debug_c19),
        .debug_c29 (debug_c29),
        .debug_c39 (debug_c39),
        .debug_LDO_READY (debug_LDO_READY),
        .dfs_clk_gated_lf (dfs_clk_gated_lf_VDD),
        .dfs_clk_gated_mf (dfs_clk_gated_mf_VDD),
        .dfs_clk_gated_hf (dfs_clk_gated_hf_VDD),
        .sclk (sclk),
        // Power/Ground 
        .VDD (VDD), 
        .VSS (VSS)
        );

//====================================================================
/*clkds_pwrswitch_control I_pwrswitch_control (
        // INPUTS
        .USE_CORE_PWR(BYPASS_LDO),
        .ISO_DIS_PWROK(ISO_DIS_PWROK_VDDC),
        .LDO_EN(LDO_EN),
        .VDD(VDD),
        .VDD18(VDDPLL),
        .VSS(VSS),
        // OUTPUTS
        .LDO_OFF(LDO_OFF_VDD),
        .LDO_ON(LDO_ON_VDD)
        );*/

//====================================================================
clkds_cs_top xtop (
        // CLKPICKER INPUTS
        .use_skip2 (clkpick_sel_skip2_VDDINT),
        .use_skip4 (clkpick_sel_skip4_VDDINT),
        .use_ph28 (use_ph28_VDDINT),

        .use_skip2_pcc (clkpick_sel_skip2_pcc_VDDINT),
        .use_skip4_pcc (clkpick_sel_skip4_pcc_VDDINT),
        .use_ph28_pcc (use_ph28_pcc_VDDINT),
        .pcc_detected (pcc_detected_VDDINT),

        .clkpicker_reset (clkpicker_reset_VDDINT),
        .clkpicker_enable (clkpicker_enable_VDDINT),
        .droopdetected (stretch_clock_VDDINT),
        .met_ctrl_sel (MET_CTRL_SEL_VDDINT),
        // DLL INPUTS
        .ckin_lf    (dfs_clk_gated_lf_VDDINT),
        .ckin_mf    (dfs_clk_gated_mf_VDDINT),
        .ckin_hf    (dfs_clk_gated_hf_VDDINT),
        //.VPB        (LDO_VPB),
        //.LDO_OFF    (LDO_OFF_VDD),
        .LDO_EN18   (LDO_ON_VDDPLL),
        .COMP_EN18  (LDO_ON_VDDPLL),
        .c0P  (c0P_VDDINT),
        .c1P  (c1P_VDDINT),
        .c2P  (c2P_VDDINT),
        .c3P  (c3P_VDDINT),
        .c4P  (c4P_VDDINT),
        .c5P  (c5P_VDDINT),
        .c6P  (c6P_VDDINT),
        .c7P  (c7P_VDDINT),
        .c8P  (c8P_VDDINT),
        .c9P  (c9P_VDDINT),
        .c10P (c10P_VDDINT),
        .c11P (c11P_VDDINT),
        .c12P (c12P_VDDINT),
        .c13P (c13P_VDDINT),
        .c14P (c14P_VDDINT),
        .c15P (c15P_VDDINT),
        .c16P (c16P_VDDINT),
        .c17P (c17P_VDDINT),
        .c18P (c18P_VDDINT),
        .c19P (c19P_VDDINT),
        .c20P (c20P_VDDINT),
        .c21P (c21P_VDDINT),
        .c22P (c22P_VDDINT),
        .c23P (c23P_VDDINT),
        .c24P (c24P_VDDINT),
        .c25P (c25P_VDDINT),
        .c26P (c26P_VDDINT),
        .c27P (c27P_VDDINT),
        .c28P (c28P_VDDINT),
        .c29P (c29P_VDDINT),
        .c30P (c30P_VDDINT),
        .c31P (c31P_VDDINT),
        .c32P (c32P_VDDINT),
        .c33P (c33P_VDDINT),
        .c34P (c34P_VDDINT),
        .c35P (c35P_VDDINT),
        .c36P (c36P_VDDINT),
        .c37P (c37P_VDDINT),
        .c38P (c38P_VDDINT),
        .c39P (c39P_VDDINT),

        .test_clk_sel (test_clk_sel_VDDINT),
        .test_clk_ref (test_clk_ref_VDDINT),
        .test_clk_fb  (test_clk_fb_VDDINT),
        .test_dd_sync (test_dd_sync_VDDINT),

        // CLKPICKER OUTPUTS
        .sclk_lf  (ckout_clkpicker_lf_VDDINT),
        .sclk_mf  (ckout_clkpicker_mf_VDDINT),
        .sclk_hf  (ckout_clkpicker_hf_VDDINT),
        .ck_en0_lf (ck_en0_lf_VDDINT),
        .ck_en0_mf (ck_en0_mf_VDDINT),
        .ck_en0_hf (ck_en0_hf_VDDINT),
        // DLL OUTPUTS
        .lead_ph2840_lf (lead_for_lock_lf_VDDINT),           
        .lead_ph2840_mf (lead_for_lock_mf_VDDINT),           
        .lead_ph2840_hf (lead_for_lock_hf_VDDINT),           
        //.phP (ck_phP_VDDINT),
        //.phN (ck_phN_VDDINT),
		   
	// PWRSNIFF INPUTS
	.BYPASS_LDO(BYPASS_LDO),
	.ISO_DIS_PWROK_VDDC(ISO_DIS_PWROK_VDDC),
	.LDO_EN(LDO_EN),
	// PWRSNIFF OUTPUTS
	.LDO_ON(LDO_ON_VDD),
	// LDO INPUTS
	.LDO_BIAS_SEL(LDO_BIAS_SEL_VDDPLL),
	.LDO_REFSEL (LDO_REFSEL_VDDPLL[3:0]),
	// LDO OUTPUTS
        .COMP_HI(COMP_HI_VDDPLL),
        .COMP_LO(COMP_LO_VDDPLL),	   
        // PWR
        .VDDINT  (VDDINT),
        .VDDPLL   (VDDPLL),
        .VDD   (VDD),
        .VSS      (VSS)
);


//====================================================================
/*CLKDS_LDO I_LDO (
        // Inputs
        .VDD18          (VDDPLL),         
        .VSS            (VSS),
        .LDO_EN18       (LDO_ON_VDDPLL),
        // coupling to LDO reference when COMP_EN de-asserts
        // fix by always enabling the comparators
        //.COMP_EN18      (COMP_EN_VDDPLL),
        .COMP_EN18      (LDO_ON_VDDPLL),
        .LDO_BIAS_SEL   (LDO_BIAS_SEL_VDDPLL),
        .VCNTL          (LDO_REFSEL_VDDPLL),
        .VDDREG         (VDDINT),
        // Outputs
        .COMP_HI        (COMP_HI_VDDPLL),
        .COMP_LO        (COMP_LO_VDDPLL),
        .VPB            (LDO_VPB)
);*/


//====================================================================
//per Chad's request for runing LEC, change from to cap_maclkstretch I_toplevel_cap <- clkds_csdecap xdecap
cap_maclkstretch I_toplevel_cap (
        .VDD            (VDD),
        .VDDPLL         (VDDPLL),
        .VDDREG         (VDDINT),
        .VSS            (VSS)
);


//==================================================================
//==================================================================
//====================================================================
// Assertions
`ifdef ABV
`include "mbstretch_assertions.v"
`endif //ABV
                          

`else

assign sclk = dfs_clk;
assign cks_dll_lock = 1'b0;
assign skip_phase = 1'b0;

assign debug_c0 = 4'd0;
assign debug_c19 = 4'd0;
assign debug_c29 = 4'd0;
assign debug_c39 = 4'd0;
assign debug_LDO_READY = 1'b1;

assign SyncBusOut = SyncBusIn;


`endif  // HWEMUL



`endif   // MACRO_HEADER_ONLY

endmodule // maclkstretch
