//==========================================================================================*/
//                                                                                          */ 
//                       (C) COPYRIGHT 2008-2016 SYNOPSYS, INC.                             */
//                                 ALL RIGHTS RESERVED                                      */
//                                                                                          */
//  This software and the associated documentation are confidential and                     */ 
//  proprietary to Synopsys, Inc.  Your use or disclosure of this                           */     
//  software is subject to the terms and conditions of a written                            */  
//  license agreement between you, or your company, and Synopsys, Inc.                      */  
//                                                                                          */
//  The entire notice above must be reproduced on all authorized copies.                    */ 
//                                                                                          */
//  Filename        : mardih_n7_ns.v                                                        */      
//  Author          : Larissa Nitchougovskaia (larissa)                                     */
//  Release version : 1.0 (Bronze)                                                          */
//  Description     : Verilog File                                                          */  
//  Special notes   : RTL verilog model                                                     */
//                                                                                          */                                                                                          */                
//  Modification History:                                                                   */
//  Date           By               Version  Change Description                             */
//==========================================================================================*/
//Oct 20 2016 (larissa)
// Power/Ground pin directon changed from inout to input: VDD18_TMON,VDD_CORE,VSS,VDD18_TMON_S
//
// 17 Jan 2015 (larissa)
// msg for all_pin_connected check ==1 only when VDD_CORE==1 && VDD18_TMON_S==1 (after CHECK_DELAY)
// 07 Jan 2014 (lnitchou)
// Default for CHECK_DELAY increased to 200000. Initial msg=1'b0
// Dec 16 2014 (larissa nitchougovskaia)
// mardih_check_all_pins : synthax of module() decl. changed: parameters defined in module() declaration
// By SOC SAPR request
//
// 13 May 2014 (lnitchou): 
// Initial (LSA) 
// Code is copied from prev node, TMON_S_CONFIG input removed
// VSS_CORE renamed to VSS

`timescale 1ps/1ps

/*--------------------------------------------------------------------------------------------------------------------*
ARDIOD behavior model
April 14, 2009: Initial version
Model description
1, TMON_S_RD_PR and TMON_S_REF_PR will generate random output pulse
2, TMON_S_SELECT is signal to enable/disable the function
3, If TMON detect "x" at any time, it means either two or more sensor are connected
4, If TMON detect "z" at any time, it means no sensor is connected
5, RANDOM_PERIOD: The output pulse length range is set by this parameter, the default value is 1000
   To change it, use this command: defparam <instance_name>.RANDOM_PERIOD = 500;
6, The remote sensor will not be operational during BACO mode, model will output 'x'

Parameter setting:
parameter	CHECK_DELAY = 100000;
parameter	RANDOM_PERIOD = 1000;
1, CHECK_DELAY means the #CHECK_DELAY picosecond later, all input pins connections checking is enabled,
   if CHECK_DELAY is 0, then this checking will never be enabled, we do not encourage such setting;
2, RANDOM_PERIOD means maximum period of output pulse
Others:
This model does not include constrains / functionalities for the setting of
{
	VDD18_TMON_S
};
*--------------------------------------------------------------------------------------------------------------------*/

module mardih_n7_ns ( TMON_S_RD_PR, TMON_S_RD_PW, TMON_S_REF_PR, TMON_S_REF_PW, TMON_S_SELECT,
               VDD18_TMON_S, VDD_CORE, VSS);

parameter	CHECK_DELAY = 200000;
parameter	RANDOM_PERIOD = 1000;

input	TMON_S_SELECT;
inout	TMON_S_RD_PW, TMON_S_REF_PW;
inout	TMON_S_RD_PR, TMON_S_REF_PR;
input	VDD18_TMON_S, VDD_CORE, VSS;

`ifndef HWEMUL
// MUST be =  {_number_of_inputs_to_consider - 1}. Now number of pins=3 => ALL_INPUT_PINS_WIDTH=2
parameter	ALL_INPUT_PINS_WIDTH = 2;
wire	[ALL_INPUT_PINS_WIDTH:0] all_input_pins;
wire	all_input_pins_connected;
wire	rd_pr, ref_pr;
reg	msg;
assign	all_input_pins = { TMON_S_RD_PW, TMON_S_REF_PW, TMON_S_SELECT};
assign	TMON_S_RD_PR =  (all_input_pins_connected && VDD_CORE && VDD18_TMON_S ) ? rd_pr  : 1'bz;
assign	TMON_S_REF_PR = (all_input_pins_connected && VDD_CORE && VDD18_TMON_S ) ? ref_pr : 1'bz;

initial
begin
	msg = 0;
end

always @(VDD_CORE or VDD18_TMON_S)
begin   
	msg = 0;
	if ((CHECK_DELAY > 0) && VDD_CORE==1'b1 && VDD18_TMON_S==1'b1) #CHECK_DELAY msg = 1;
end

mardih_n7_ns_top #(RANDOM_PERIOD) top_inst (
	.TMON_S_RD_PR(rd_pr),
	.TMON_S_RD_PW(TMON_S_RD_PW),
	.TMON_S_REF_PR(ref_pr),
	.TMON_S_REF_PW(TMON_S_REF_PW),
	.TMON_S_SELECT(TMON_S_SELECT)
);

mardih_n7_ns_check_all_pins #(ALL_INPUT_PINS_WIDTH) check_inst(
	.input_bus(all_input_pins),
	.msg(msg),
	.values_OK(all_input_pins_connected)
);

//VCS coverage off
`ifdef VCS
//enable args
reg check_en;
reg [1:0] active_type;//0:inactive; 1:V_type; 2:H_type
initial begin
    if($test$plusargs("bpm_IP_connect_assert_enable"))
        check_en = 1'b1;
    else
        check_en = 1'b0;  
    active_type = 0;
end

`ifdef GFX_CG_W2E
final if((check_en == 1'b1)&(active_type != 2)) `MSGIO_DISPLAY("%m ERROR: RDI H orietation is not active");
`else
final if((check_en == 1'b1)&(active_type != 2)) `MSGIO_DISPLAY("%m WARNING: RDI H orietation is not active");
`endif
`endif // VCS
//VCS coverage on

`endif
endmodule

`ifndef HWEMUL
module mardih_n7_ns_top ( TMON_S_RD_PR, TMON_S_RD_PW, TMON_S_REF_PR, TMON_S_REF_PW, TMON_S_SELECT);

parameter	RANDOM_PERIOD = 1000;

input	TMON_S_RD_PW, TMON_S_REF_PW, TMON_S_SELECT;
output	TMON_S_RD_PR, TMON_S_REF_PR;

wire	TMON_S_RD_PR, TMON_S_REF_PR;
reg	random_signal;

assign	TMON_S_RD_PR = (TMON_S_SELECT == 1'b0) ? 1'bz : (TMON_S_SELECT == 1'b1) ? random_signal : 1'bx;
assign	TMON_S_REF_PR = (TMON_S_SELECT == 1'b0) ? 1'bz : (TMON_S_SELECT == 1'b1) ? random_signal : 1'bx;

always @(TMON_S_SELECT)
begin
	while (TMON_S_SELECT)
	begin
		#({$random} % RANDOM_PERIOD) random_signal = 1'b0;
		#({$random} % RANDOM_PERIOD) random_signal = 1'b1;
	end
end
endmodule
`endif
`ifndef HWEMUL
module mardih_n7_ns_check_all_pins 
	#(parameter BUS_WIDTH = 130)
	(input	[BUS_WIDTH : 0] input_bus,
	input	msg,
	output	values_OK);

//parameter BUS_WIDTH = 130;

//input	[BUS_WIDTH : 0] input_bus;
//input	msg;
//output	values_OK;

assign	values_OK = (^input_bus === 1'bx) ? 1'b0 : 1'b1;

always @(msg or values_OK)
begin
	if ( msg && values_OK == 0 && $time > 0 )
	begin
		$display("\ntime=%0t; instance=%m \nERROR: Model will not work properly because atleast one pin is not connected",$realtime);
	end
end
endmodule
`endif

