
`include "csdd_assert_control.h"
`timescale 1ps/1ps

module mbstretch_clkmux_cpu (

input  VDD,
input  VSS,
input  IN0,
input  IN1,
input  SEL,

output OUT
);

wire SEL_B;
wire IN0_OUT;
wire IN1_OUT;
wire PRE_OUT;

`ifdef CLKDS_SYNTHESIS
		HDBSVT08_MUX2_CK_2 I_final_mux ( 
            //.VDD(VDD), 
            //.VSS(VSS), 
            .CLK1(IN0), 
            .Z(OUT),
    				.CLK2(IN1), 
            .CLKSEL(SEL) );

`else
wire pg_good = VDD & ~VSS;
assign OUT = pg_good ? 
                 SEL ? IN1 : IN0 
                           : 1'bx;
`endif //CLKDS_SYNTHESIS

endmodule

// adding 4 stage sync cell for the jira http://ontrack-internal.amd.com/browse/DEASPD-828

`timescale 1ps/1ps

module mbclkds_cdccell_sync4 (

input           VDD, 
input           VSS,
input           CLK,
input           DATAIN,
input 					RESET,
output reg           DATAOUT
);

`ifdef CLKDS_SYNTHESIS

wire tielo;
	 HDBSVT08_TIE0_1 I_CDC_TIELO ( .X(tielo));
        
	HDBULT08_SYNC4RMSFQ_Y4_1 I_CDC_SYNC4 (
        .Q(DATAOUT),
        .SEN(tielo),
        .SDI(tielo),
        .D(DATAIN),
				.RESETH(RESET),
        .CLK(CLK) 
        );
`else


// Depend on random output behaviors instead of InputCheck to insure legal control sequencing.

//techind_sync #(.depth(4), .InputCheck(0), .TransportCycle(1), .scan_muxD(0)) I_CDC_SYNC4
//     (.SB(1'b1),.RB(~RESET), .FLUSH(1'b0), .CKsrc(1'b0), .EN(1'b1), .Qsrc(), .SC1(1'b0), .SC2(1'b0), // Unused I/O
//      .CKdst(CLK), .SE(1'b0), .SI(1'b0), .SO(),  .Dsrc(DATAIN),  .Qdst(DATAOUT));
//

    reg O_MET;
    reg P_MET;
    reg Q_MET;

always 
`ifdef XPROP_BOTH_EDGES
    @(CLK or RESET) 
`else
    @(posedge CLK or posedge RESET) 
`endif
begin
    if(RESET) begin
        O_MET <= 1'b0;
	P_MET <= 1'b0;
	Q_MET <= 1'b0;
        DATAOUT <= 1'b0;
    end
    else
        begin
            if(~RESET) begin
                if(CLK)
                    begin
                        O_MET   <= #1 DATAIN;
                        P_MET   <= #1 O_MET;
                        Q_MET   <= #1 P_MET;
                        DATAOUT <= #1 Q_MET;
                    end
                else if(~CLK) ;
                else begin
                    DATAOUT <= 1'hx;
                    Q_MET <= 1'hx;
		    P_MET <= 1'hx;
                    O_MET <= 1'hx;
                end
            end else begin
                DATAOUT <= 1'hx;
                Q_MET <= 1'hx;
		P_MET <= 1'hx;
		O_MET <= 1'hx;
            end
        end
end


`endif //CLKDS_SYNTHESIS

endmodule
// end of 4 stage sync cell



`timescale 1ps/1ps

module mbclkds_cdccell (

input           VDD, 
input           VSS,

input           CLK,
input           DATAIN,

output           DATAOUT
);

`ifdef CLKDS_SYNTHESIS

wire tielo;
   HDBSVT08_TIE0_1 I_CDC_TIELO ( .X(tielo));
        
   HDBULT08_SYNC3MSFQ_Y4_1 I_CDC_SYNC3 (
        .Q(DATAOUT),
        .SEN(tielo),
        .SDI(tielo),
        .D(DATAIN),
        .CLK(CLK) 
        );
`else

//SYNC3OR I_CDC_SYNC3 (
//       .SampleClk      (CLK),      // Receiving Clock domain
//       .SampleClkEn    (1'b1),     // Receiving Clock Enable
//       .DataIn         (DATAIN),   // Input data
//       .AsyncOR        (1'b0),     // This provides an Asynchronous OR path for Reset-like functionality.
//    `ifdef assertions
//       .OrChkDis       (~(`DD_ABV_FUNC_EN | `CKS_ABV_FUNC_EN)),     // Disables the OVA for OR path checking
//       .InputChkDis    (1'b0),     // Disables the OVA for input pulse width checking.
//    `endif
//       .DataOut        (DATAOUT)   // Output data
//       );

// Depend on random output behaviors instead of InputCheck to insure legal control sequencing.
techind_sync #(.depth(3), .InputCheck(0), .TransportCycle(1), .scan_muxD(0)) I_CDC_SYNC3
     (.SB(1'b1),.RB(1'b1), .FLUSH(1'b0), .CKsrc(1'b0), .EN(1'b1), .Qsrc(), .SC1(1'b0), .SC2(1'b0), // Unused I/O
      .CKdst(CLK), .SE(1'b0), .SI(1'b0), .SO(),  .Dsrc(DATAIN),  .Qdst(DATAOUT));


`endif //CLKDS_SYNTHESIS

endmodule

// river translate_off
`timescale 1ps/1ps
module mbclkds_cdccell_sync2 (
          VDD, 
          VSS,
          CLK,
          DATAIN,
          RESET,
          DATAOUT
);

input           VDD; 
input           VSS;
input           CLK;
input           DATAIN;
input           RESET;
output          DATAOUT;

`ifdef CLKDS_SYNTHESIS
wire tielo;
    HDBSVT08_TIE0_1 I_CDC_TIELO ( .X(tielo));
//    HDBULT08_SYNC2RMSFQ_Y2_1 I_CDC_SYNC2OR ( , haritt updated to match 2_04 version

    HDBULT08_SYNC2RMSFQ_Y4_1 I_CDC_SYNC2OR (
            .D              (DATAIN),
            .CLK            (CLK),
            .Q              (DATAOUT),
            .SDI            (tielo),
            .SEN            (tielo),
            .RESETH         (RESET)
            );
`else
reg Q_MET;
reg DATAOUT;

//Original pre-VRQ code which fails LEDA xprop checks
//always @(posedge CLK or posedge RESET) begin
//    if (RESET) begin
//       Q_MET    <= 1'b0;
//       DATAOUT  <= 1'b0;
//    end
//    else begin
//       Q_MET    <= DATAIN;
//       DATAOUT  <= Q_MET;
//    end
//end 

//Post-VRQ code to pass LEDA xprop checks (data and clock xprop)
//Post-VRQ view of commented out pre-VRQ code found above
//This is needed because we have VRQ -passthru switch being used
//w.r.t Verilog define: CLKDS_SYNTHESIS 
//=> LEDA xprop checks fail if we don't manually x-uglify the code!
always 
`ifdef XPROP_BOTH_EDGES
    @(CLK or RESET) 
`else
    @(posedge CLK or posedge RESET) 
`endif
begin
    if(RESET) begin
        Q_MET <= 1'b0;
        DATAOUT <= 1'b0;
    end
    else
        begin
            if(~RESET) begin
                if(CLK)
                    begin
                        Q_MET <= #1 DATAIN;
                        DATAOUT <= #1 Q_MET;
                    end
                else if(~CLK) ;
                else begin
                    DATAOUT <= 1'hx;
                    Q_MET <= 1'hx;
                end
            end else begin
                DATAOUT <= 1'hx;
                Q_MET <= 1'hx;
            end
        end
end

`endif

endmodule
// river translate_on

// river translate_off
`timescale 1ps/1ps
module mbclkds_cdccell_sync1 (
          VDD, 
          VSS,
          CLK,
          DATAIN,
          RESET,
          DATAOUT
);

input           VDD; 
input           VSS;
input           CLK;
input           DATAIN;
input           RESET;
output          DATAOUT;

`ifdef CLKDS_SYNTHESIS
wire preDATAIN;
wire RESET_INV;
wire tielo;
    HDBSVT08_TIE0_1 I_CDC_TIELO ( .X(tielo));
    HDBSVT08_INV_1 I_CDC_SYNC1_INV (.A(RESET), .X(RESET_INV) );
    HDBSVT08_AN2_1 I_CDC_SYNC1_RESET (.A1(RESET_INV), .A2(DATAIN), .X(preDATAIN) );
    HDBULT08_FSDPQ_V2Y2_1 I_CDC_SYNC1 (
            .D              (preDATAIN),
            .CK            (CLK),
            .Q              (DATAOUT),
						.SI (tielo),
						.SE (tielo)
            );    


`else
reg DATAOUT;

//always @(posedge CLK)
//  if (RESET)
//    DATAOUT <= 1'b0;
//  else
//    DATAOUT <= DATAIN;
//

//Post-VRQ code to pass LEDA xprop checks (data and clock xprop)
//Post-VRQ view of commented out pre-VRQ code found above
//This is needed because we have VRQ -passthru switch being used
//w.r.t Verilog define: CLKDS_SYNTHESIS 
//=> LEDA xprop checks fail if we don't manually x-uglify the code!

    always 
    `ifdef XPROP_BOTH_EDGES
        @(CLK) 
    `else
        @(posedge CLK) 
    `endif
    begin
        if(CLK) begin
            if(RESET) begin
                DATAOUT <= 1'b0;
            end else begin
                if(~RESET) begin
                    DATAOUT <= DATAIN;
                end
                
                //vrq begin
                // VCS coverage off
                else begin
                    DATAOUT <= 1'hx;
                end
                
                // VCS coverage on
                //vrq end
            end
        end else if(~CLK) ;
        else begin
            DATAOUT <= 1'hx;
        end
    end

`endif

endmodule
// river translate_on


`timescale 1ps/1ps

module mbclkds_clkgater (

input VDD,
input VSS,
input C,
input EN,

output CG
);

`ifdef CLKDS_SYNTHESIS

   HDBSVT08_CKGTPLTN_V7Y2_4 I_CLKGATER ( 
            .Z(CG), 
            .CLK(C),
    		    .EN(EN) );		

`else
CLKGATER I_CLKGATER (
                          .C (C),
                          .D (EN),  
                          .CG (CG)
                          );

`endif // CLKDS_SYNTHESIS

endmodule

`timescale 1ps/1ps

module wiremodel_mac (
 
input T1,
output T2,
input VSS
);

assign T2 = T1;

endmodule

`timescale 1ps/1ps

module mbclkds_srlatch (
// Reset has priority over Set
input S,
input R,  
output reg Q
);

//nand s_nand (Q_int, ~S, Q_bar);
//nand r_nand (Q_bar, ~R, Q_int);

//hdnd2xss4ur I_SET_NAND   (.A(Q_bar), .B(S_bar), .Z(Q_int));
//hdnd2xss4ur I_RESET_NAND (.A(Q_int), .B(R_bar), .Z(Q_bar));
//
//hdinxss4ur I_QBAR_INV (.A(Q_bar), .Z(Q));

`ifdef CLKDS_SYNTHESIS
// Since hdsrlatchxss3ur gives priority to Set, and we need to swap
// the S and R inputs and take the inverted output in order to give
// priority to R
		HDBSVT08_SRLD_Y2_3 I_srlatch (
        .R(S),
        .S(R),
        .Q(),
        .QN(Q)
				);
`else
// synopsys async_set_reset "S,R"
//always @(S or R) 
//    begin : infer 
//       if (R == 1) 
//        Q <= 1'b0 ; 
//       else if (S == 1) 
//        Q <= 1'b1 ; 
//    end 

    //Post-VRQ code to pass LEDA xprop checks (data and clock xprop)
    //Post-VRQ view of commented out pre-VRQ code found above
    //This is needed because we have VRQ -passthru switch being used
    //w.r.t Verilog define: CLKDS_SYNTHESIS 
    //=> LEDA xprop checks fail if we don't manually x-uglify the code!
    always @(S or R) begin : infer
        if(R==1) begin
            Q <= 1'b0;
        end else begin
            if(~(R==1)) begin
                if(S==1) begin
                    Q <= 1'b1;
                end

                // VCS coverage off
                else if(~(S==1)) ;
                else begin
                    Q <= 1'hx;
                end

                // VCS coverage on
            end

            // VCS coverage off
            else begin
                Q <= 1'hx;
            end

            // VCS coverage on
        end
    end

`endif // CLKDS_SYNTHESIS

endmodule

`timescale 1ps/1ps

module mbclkds_CDCBUF (
               `ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
               input DSTCLK,
               `endif
               input A, 
               output Z);

//Instantiate CDCBUFCLK as marker cell to indicate false paths, but DO NOT inject X
//on output whenever input toggles during simulations
CDCBUFCLK #(.Setup(0),.Cycles(0)) I_CDCBUF (
                                            `ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
                                            .DSTCLK(DSTCLK),
                                            `endif
                                            .A(A), 
                                            .Z(Z) );

endmodule

`timescale 1ps/1ps

module inv_lvt4NL (
    input vnsub,
    input pwr,
    input gnd,
    input vpsub,
    input A,
    output Y
);

wire pg_good = pwr & vpsub & ~gnd & ~vnsub;

assign Y = pg_good ? ~A : 1'bx;

endmodule

`timescale 1ps/1ps

module inv_rvt4NL (
    input vnsub,
    input pwr,
    input gnd,
    input vpsub,
    input A,
    output Y
);

wire pg_good = pwr & vpsub & ~gnd & ~vnsub;

assign Y = pg_good ? ~A : 1'bx;

endmodule

//`timescale 1ps/1ps
//module clkds_skew_detect (
//    inp,
//    inn,
//    enable,
//    msg,
//    skew_detected
//    );
//
//input inp;
//input inn;
//input enable;
//input msg;
//output skew_detected;
//
//`ifdef CLKDS_SYNTHESIS
//`else
//
//parameter skew_limit = 10;
//time negedge_time;
//wire xor_inp_inn;
//reg reg_skew_detected;
//reg msg_displayed;
//
////assign skew_detected = enable ? reg_skew_detected : 1'b0;
////
////assign xor_inp_inn = inp ^ inn;
////initial begin
////    negedge_time = 0;
////    reg_skew_detected = 1'b0;
////    msg_displayed = 1'b0;
////end
////always @(negedge xor_inp_inn) begin
////    negedge_time = $time;
////end
////always begin
////    wait (~xor_inp_inn);
////    @(posedge xor_inp_inn) begin
////      if (xor_inp_inn !== 1'bx && enable===1'b1) begin   
////        if (($time - negedge_time) > skew_limit) begin
////            reg_skew_detected <= 1'b1;
////            if (msg && ~msg_displayed) begin
////                $display("ERROR(%m): TIME(%0t) Skew detected between inp and inn\n", $time);
////                msg_displayed <= 1'b1;
////            end
////        end
////        else
////            reg_skew_detected <= 1'b0;
////      end //if (xor_inp_inn !== 1'bx && enable===1'b1) begin
////    end
////end
//
////Manually copying over the post-VRQ code to get past LEDA xprop errors
//
//    assign   skew_detected=enable ? reg_skew_detected : 
//        ~enable ? 1'b0 : 1'hx;
//    assign   xor_inp_inn=inp^inn;
//    initial begin
//        negedge_time = 0;
//        reg_skew_detected = 1'b0;
//        msg_displayed = 1'b0;
//    end
//    always 
//    `ifdef XPROP_BOTH_EDGES
//        @(xor_inp_inn) 
//    `else
//        @(negedge xor_inp_inn) 
//    `endif
//    begin
//        //synopsys translate_off
//        if(~xor_inp_inn) //synopsys translate_on
//            begin
//                negedge_time = $time;
//            end
//            //synopsys translate_off
//        else if(xor_inp_inn) ;
//        else begin
//            negedge_time = 64'hxxxxxxxxxxxxxxxx;
//        end
//        //synopsys translate_on
//    end
//    always begin
//        wait(~xor_inp_inn) ;
//        @(posedge xor_inp_inn) begin
//            if(xor_inp_inn!==1'bx&&enable===1'b1) begin
//                if($time-negedge_time>skew_limit) begin
//                    reg_skew_detected <= 1'b1;
//                    if(msg&&~msg_displayed) begin
//                        $display("ERROR(%m): TIME(%0t) Skew detected between inp and inn\n",$time);
//                        msg_displayed <= 1'b1;
//                    end
//                    
//                    //vrq begin
//                    // VCS coverage off
//                    else if(~(msg&&~msg_displayed)) ;
//                    else begin
//                        msg_displayed <= 1'hx;
//                    end
//                    
//                    // VCS coverage on
//                    //vrq end
//                end
//                else begin
//                    if(~($time-negedge_time>skew_limit)) begin
//                        reg_skew_detected <= 1'b0;
//                    end
//                    
//                    //vrq begin
//                    // VCS coverage off
//                    else begin
//                        msg_displayed <= 1'hx;
//                        reg_skew_detected <= 1'hx;
//                    end
//                    
//                    // VCS coverage on
//                    //vrq end
//                end
//            end
//            
//            //vrq begin
//            // VCS coverage off
//            else if(~(xor_inp_inn!==1'bx&&enable===1'b1)) ;
//            else begin
//                msg_displayed <= 1'hx;
//                reg_skew_detected <= 1'hx;
//            end
//            
//            // VCS coverage on
//            //vrq end
//            //if (xor_inp_inn !== 1'bx && enable===1'b1) begin
//        end
//    end
//
//`endif // CLKDS_SYNTHESIS
//
//endmodule


`timescale 1ps/1ps
module ParCap (
input MINUS,
input PLUS
);
endmodule




