

module mbdroop_droop_dll_control (

input         VDD, 
input         VSS,

input         dfs_clk,
input         RESET,
input  [17:0] DLL_LEAD,               // phase detector output
input  [2:0]  STRETCH_THRESH,
input  [4:0]  UNSTRETCH_THRESH,
input  [1:0]  CONSERVATIVE_LOCK,

input         DFSSTRETCH,
input  [3:0]  DFSSTRETCH_AMOUNT,
input         dll_lock,
output  reg   PD_d0,
output        droopdetected
);


// Droop detection
reg    droopdetected_dfs_stretch ;
reg    droopdetected_dfs_unstretch ;
reg    dfsstretch_d1 ;
reg    dfsstretch_d2 ;

// ECO: DECTSCK-266
reg    dfsstretch_d3;
reg    dfsstretch_d4;

wire   dfsstretch_d2d3 = dfsstretch_d2 | dfsstretch_d3;
wire   dfsstretch_d3d4 = dfsstretch_d3 | dfsstretch_d4;
// end of ECO

wire   [3:0] dfsstretch_amount_local ;
always @(*)
  // STRETCH_THRESH functionality when dfs clock is not stretched
  case (STRETCH_THRESH[2:0])
    3'b000:  droopdetected_dfs_stretch = ~DLL_LEAD[7] ; //  0.00%
    3'b001:  droopdetected_dfs_stretch = ~DLL_LEAD[6] ; //  2.50%
    3'b010:  droopdetected_dfs_stretch = ~DLL_LEAD[5] ; //  5.00%
    3'b011:  droopdetected_dfs_stretch = ~DLL_LEAD[4] ; //  7.50%
    3'b100:  droopdetected_dfs_stretch = ~DLL_LEAD[3] ; // 10.00%
    3'b101:  droopdetected_dfs_stretch = ~DLL_LEAD[2] ; // 12.50%
    3'b110:  droopdetected_dfs_stretch = ~DLL_LEAD[1] ; // 15.00%
    3'b111:  droopdetected_dfs_stretch = ~DLL_LEAD[0] ; // 17.50 %

    default: droopdetected_dfs_stretch = 1'bx;
  endcase


always @(*)
  // STRETCH_THRESH functionality when dfs clock is stretched
  case (UNSTRETCH_THRESH[3:0])
    4'd0 :  droopdetected_dfs_unstretch = ~DLL_LEAD[1] ; // phase 34
    4'd1 :  droopdetected_dfs_unstretch = ~DLL_LEAD[2] ; // phase 35
    4'd2 :  droopdetected_dfs_unstretch = ~DLL_LEAD[3] ; // phase 36
    4'd3 :  droopdetected_dfs_unstretch = ~DLL_LEAD[4] ; // phase 37
    4'd4 :  droopdetected_dfs_unstretch = ~DLL_LEAD[5] ; // phase 38
    4'd5 :  droopdetected_dfs_unstretch = ~DLL_LEAD[6] ; // phase 39
    4'd6 :  droopdetected_dfs_unstretch = ~DLL_LEAD[7] ; // phase 40

    4'd7  : droopdetected_dfs_unstretch = ~DLL_LEAD[8] ; // phase 41
    4'd8  : droopdetected_dfs_unstretch = ~DLL_LEAD[9] ; // phase 42
    4'd9  : droopdetected_dfs_unstretch = ~DLL_LEAD[10] ; // phase 43
    4'd10 : droopdetected_dfs_unstretch = ~DLL_LEAD[11] ; // phase 44
    4'd11 : droopdetected_dfs_unstretch = ~DLL_LEAD[12] ; // phase 45 
    4'd12 : droopdetected_dfs_unstretch = ~DLL_LEAD[13] ; // phase 46
    4'd13 : droopdetected_dfs_unstretch = ~DLL_LEAD[14] ; // phase 47
    4'd14 : droopdetected_dfs_unstretch = ~DLL_LEAD[15] ; // phase 48
    4'd15 : droopdetected_dfs_unstretch = ~DLL_LEAD[16] ; // phase 49

    default: droopdetected_dfs_unstretch = 1'bx ; 
  endcase


// ECO: DECTSCK-266
// assign droopdetected = dfsstretch_d2 ?  droopdetected_dfs_unstretch : droopdetected_dfs_stretch ;
assign droopdetected = dfsstretch_d2d3 ?  droopdetected_dfs_unstretch : droopdetected_dfs_stretch ;
// end of ECO

//Limiting the value of DFSTRETCH_AMOUNT local to 10 to prevent overflow on DLL_LEAD
assign dfsstretch_amount_local[3:0] = ( DFSSTRETCH_AMOUNT[3:0] > 10) ? 4'd0 :  DFSSTRETCH_AMOUNT[3:0] ;


//2flops delay for PD_d0
always@(posedge dfs_clk) begin
  if(RESET) begin
    dfsstretch_d1 <= 1'b0;
    dfsstretch_d2 <= 1'b0;
  end
  else  begin
    dfsstretch_d1 <= DFSSTRETCH;
    dfsstretch_d2 <= dfsstretch_d1;
  end
end

// ECO: DECTSCK-266
always@(posedge dfs_clk) begin
  if(RESET) begin
    dfsstretch_d3 <= 1'b0;
    dfsstretch_d4 <= 1'b0;
  end
  else  begin
    dfsstretch_d3 <= dfsstretch_d2;
    dfsstretch_d4 <= dfsstretch_d3;
  end
end
// end of ECO


always @(*)
// ECO: DECTSCK-266
//  casez ({dll_lock,dfsstretch_d2,CONSERVATIVE_LOCK[1], CONSERVATIVE_LOCK[0]})
  casez ({dll_lock,dfsstretch_d3d4,CONSERVATIVE_LOCK[1], CONSERVATIVE_LOCK[0]})
// end of ECO
    4'b0001:  PD_d0 = ~RESET & DLL_LEAD[5]; // 38th delay chain cell- for 95%
    4'b0010:  PD_d0 = ~RESET & DLL_LEAD[3]; // 36th delay chain cell- for 90%
    4'b0101:  PD_d0 = ~RESET & DLL_LEAD[5+ dfsstretch_amount_local[3:0]]; // 38th + DFSSTRETCH_AMOUNT delay chain cell - for 95%
    4'b0110:  PD_d0 = ~RESET & DLL_LEAD[3+ dfsstretch_amount_local[3:0]]; // 36th + DFSSTRETCH_AMOUNT delay chain cell- for 90%
    4'b0000,
    4'b10??:  PD_d0 = ~RESET & DLL_LEAD[7];
    4'b11??:  PD_d0 = ~RESET & DLL_LEAD[7+ dfsstretch_amount_local[3:0]];
    default:  PD_d0 = 1'bx;
  endcase


endmodule
