//==================================================================================================
//
// mbdroop_sapr_top.v
// 
// Copyright (c) 2002 by Advanced Micro Devices, Inc.
//
// This file is protected by Federal Copyright Law, with all rights
// reserved. No part of this file may be reproduced, stored in a
// retrieval system, translated, transcribed, or transmitted, in any
// form, or by any means manual, electric, electronic, mechanical,
// electro-magnetic, chemical, optical, or otherwise, without prior
// explicit written permission from Advanced Micro Devices, Inc.
//
//==================================================================================================
//
//
//==================================================================================================
//
// Module Description
//
// Droop Detect SAPR 
//
//==================================================================================================

`timescale 1ps/1ps

module mbdroop_sapr_top (

input           VDD, 
input           VSS,

input           dfs_clk,
input           ENABLE_DD,
input           STRETCH_ENABLE,
input           staticEnable,
input           masterReset,

input           NO_EXTRA_COARSE_STEP,
input     [2:0] STRETCH_THRESH,
input     [1:0] CONSERVATIVE_LOCK,
input     [6:0] LOW_BW_CONFIG,
input     [6:0] LOW_BW_CONFIG_UP,
input     [2:0] LOW_BW_SKEW_EN,
input     [3:0] FSM_WAIT_CYCLES,
input     [3:0] SAMPLE_SIZE,
input     [1:0] FREQ_MODE,
input     [1:0] PD_SYNC_SEL,        
input     [15:0] RESERVE,


input           lead_ph40_mf,
input           lead_ph39_mf,
input           lead_ph38_mf,
input           lead_ph37_mf,
input           lead_ph36_mf,
input           lead_ph35_mf,
input           lead_ph34_mf,
input           lead_ph33_mf,
input           lead_ph50_hf,
input           lead_ph49_hf,
input           lead_ph48_hf,
input           lead_ph47_hf,
input           lead_ph46_hf,
input           lead_ph45_hf,
input           lead_ph44_hf,
input           lead_ph43_hf,
input           lead_ph42_hf,
input           lead_ph41_hf,
input           lead_ph40_hf,
input           lead_ph39_hf,
input           lead_ph38_hf,
input           lead_ph37_hf,
input           lead_ph36_hf,
input           lead_ph35_hf,
input           lead_ph34_hf,
input           lead_ph33_hf,

input     [3:0] CHARZ_SEL_A,
input     [3:0] CHARZ_SEL_B,
input     [1:0] CHARZ_DIV_A,
input     [1:0] CHARZ_DIV_B,
input     [1:0] FSM_STEP_SIZE,
input     [1:0] FSM_CODE_MARGIN,
input           CHARZ_DEBUG_MODE,
input     [1:0] FSM_DEBUG_SEL,
input           FSM_OVERRIDE_EN,
input     [3:0] FSM_OVERRIDE_VAL,
input           test_clk_ref,
input           test_clk_fb40,

input     [4:0] UNSTRETCH_THRESH,        // Specifies trigger threshold when DFSSTRETCH=1. (FREQ_MODE=10, CPU only)
input           DFSSTRETCH,              // 1 means dfs_clk is stretched    
input     [3:0] DFSSTRETCH_AMOUNT,       // specifies stretch percentage seen on dfs_clk. (FREQ_MODE=10, CPU only)
input           CT_CpuPwrOkVDD,         // Isolation for the dfs_clk to be usd by CPU else connected to 1'b1 at higher level

output          charz_observ_a,
output          charz_observ_b,
output    [1:0] test_clk_sel,

output    [3:0] c0P,
output    [3:0] c1P,
output    [3:0] c2P,
output    [3:0] c3P,
output    [3:0] c4P,
output    [3:0] c5P,
output    [3:0] c6P,
output    [3:0] c7P,
output    [3:0] c8P,
output    [3:0] c9P,
output    [3:0] c10P,
output    [3:0] c11P,
output    [3:0] c12P,
output    [3:0] c13P,
output    [3:0] c14P,
output    [3:0] c15P,
output    [3:0] c16P,
output    [3:0] c17P,
output    [3:0] c18P,
output    [3:0] c19P,
output    [3:0] c20P,
output    [3:0] c21P,
output    [3:0] c22P,
output    [3:0] c23P,
output    [3:0] c24P,
output    [3:0] c25P,
output    [3:0] c26P,
output    [3:0] c27P,
output    [3:0] c28P,
output    [3:0] c29P,
output    [3:0] c30P,
output    [3:0] c31P,
output    [3:0] c32P,
output    [3:0] c33P,
output    [3:0] c34P,
output    [3:0] c35P,
output    [3:0] c36P,
output    [3:0] c37P,
output    [3:0] c38P,
output    [3:0] c39P,

output          dd_dll_lock,
output [7:0] phase_lead_lag,
output reg [3:0] debug_c0,
output reg [3:0] debug_c19,
output reg [3:0] debug_c29,
output reg [3:0] debug_c39,

output          droopdetected,
output          dfs_clk_gated_mf,
output          dfs_clk_gated_hf
);


wire            reset;
wire            dd_dll_lock_int;

wire      [17:0] dd_lead; 
wire      [7:0] dd_lead_mf; 
wire      [7:0] dd_lead_hf; 

wire           cdc_ENABLE_DD;
wire           cdc_STRETCH_ENABLE;
wire           cdc_staticEnable;
wire           cdc_masterReset;
wire           cdc_FSM_OVERRIDE_EN;

wire           droopdetected_int;

wire            MET_CTRL_buf;
wire            NO_EXTRA_COARSE_STEP_buf;
wire      [2:0] STRETCH_THRESH_buf;
wire      [1:0] CONSERVATIVE_LOCK_buf;
wire      [6:0] LOW_BW_CONFIG_buf;
wire      [6:0] LOW_BW_CONFIG_UP_buf;
wire            LOW_BW_SKEW_EN_buf;
wire      [3:0] FSM_WAIT_CYCLES_buf;
wire      [3:0] SAMPLE_SIZE_buf;
wire      [1:0] FREQ_MODE_buf;
wire      [1:0] PD_SYNC_SEL_buf;
wire      [7:0] RESERVE_buf;

wire      [4:0] UNSTRETCH_THRESH_buf;
wire      [3:0] DFSSTRETCH_AMOUNT_buf;

wire      [2:0] state;
wire      [6:0] delay_conf_ptr;
wire      [3:0] delay_conf_left;
wire     [15:0] low_bw_ctr;
wire     [15:0] phase_accum;

wire            droopdetected_func;
wire            dd_dll_lock_func;
wire            charz_observ_a_int;
wire            charz_observ_b_int;
wire            PD_d0;
reg             dfs_clk_iso; 
//==================================================================

always @*
 dfs_clk_iso = dfs_clk & CT_CpuPwrOkVDD;

//=================================================================
mbclkds_clkgater I_mbclkds_clkgater_mf (
        .VDD        (VDD),
        .VSS        (VSS),
        .C          (dfs_clk_iso),
        .EN         (cdc_staticEnable & cdc_ENABLE_DD & FREQ_MODE_buf[0] & ~FREQ_MODE_buf[1]),  
        .CG         (dfs_clk_gated_mf)
        );

mbclkds_clkgater I_mbclkds_clkgater_hf (
        .VDD        (VDD),
        .VSS        (VSS),
        .C          (dfs_clk_iso),
        .EN         (cdc_staticEnable & cdc_ENABLE_DD & FREQ_MODE_buf[1]),  
        .CG         (dfs_clk_gated_hf)
        );

mbclkds_clkgater I_mbclkds_clkgater_fsm(
        .VDD        (VDD),
        .VSS        (VSS),
        .C          (dfs_clk_iso),
        .EN         (cdc_staticEnable & cdc_ENABLE_DD),  
        .CG         (dfs_clk_gated)
        );

//==================================================================
mbdroop_cdc I_mbdroop_cdc (
        .VDD (VDD),
        .VSS (VSS),
        .dfs_clk(dfs_clk_iso),
        .ENABLE_DD(ENABLE_DD),
        .STRETCH_ENABLE(STRETCH_ENABLE),
        .staticEnable(staticEnable),
        .masterReset(masterReset),
        .FSM_OVERRIDE_EN(FSM_OVERRIDE_EN),
        .cdc_ENABLE_DD(cdc_ENABLE_DD),
        .cdc_STRETCH_ENABLE(cdc_STRETCH_ENABLE),
        .cdc_staticEnable(cdc_staticEnable),
        .cdc_masterReset(cdc_masterReset),
        .cdc_FSM_OVERRIDE_EN(cdc_FSM_OVERRIDE_EN)
        );

//==================================================================

mbdroop_droop_dll_control I_DD_droop_dll_control (
        .VDD                (VDD),
        .VSS                (VSS),

        .dfs_clk             (dfs_clk_gated),
        .RESET               (reset),
        .DLL_LEAD            ({dd_lead[17:0]}),          // phase detector output, only the msb is used in the stretcher
        .STRETCH_THRESH      (STRETCH_THRESH_buf),         // can be any number - the outcome is not used in the stretcher
        .UNSTRETCH_THRESH    (UNSTRETCH_THRESH_buf),           
        .CONSERVATIVE_LOCK   (CONSERVATIVE_LOCK_buf),
        .DFSSTRETCH          (DFSSTRETCH),
        .DFSSTRETCH_AMOUNT   (DFSSTRETCH_AMOUNT_buf),          
        // From DLL chanin
        // From FSM
        .dll_lock  (dd_dll_lock_int),
        
        .PD_d0         (PD_d0), 
        .droopdetected (droopdetected_int) // output: high when droop below the threshold is detected - not used in the stretcher
        );

mbdroopfsm I_FSM (
        .VDD (VDD),
        .VSS (VSS),
        
        //.STRETCHER_FSM (1'b0),                    // STRETCHER (1'b1) vs DROOPDETECT (1'b0) strap
        .CCLK (dfs_clk_gated),                
        .RESET (reset),                           // Stretcher enable
        .LOW_BW_CONFIG (LOW_BW_CONFIG_buf),           // has to be 3'b001 in the stretcher
        .LOW_BW_CONFIG_UP (LOW_BW_CONFIG_UP_buf),           // has to be 3'b001 in the stretcher
        .LOW_BW_SKEW_EN (LOW_BW_SKEW_EN_buf), 
        .FSM_WAIT_CYCLES(FSM_WAIT_CYCLES_buf),
        .NO_EXTRA_COARSE_STEP (NO_EXTRA_COARSE_STEP_buf),
        .SAMPLE_SIZE(SAMPLE_SIZE_buf),
        .PD_d0         (PD_d0),
        .PD_SYNC_SEL (PD_SYNC_SEL_buf),
        .OVERRIDE_DLY_EN (cdc_FSM_OVERRIDE_EN),
        .OVERRIDE_DLY_VAL (FSM_OVERRIDE_VAL[3:0]),
        .use_ph28 (1'b0),
        .FSM_STEP_SIZE (FSM_STEP_SIZE),
        .FSM_CODE_MARGIN (FSM_CODE_MARGIN),
        
        .c_0 (c0P[3:0]),
        .c_1 (c1P[3:0]),
        .c_2 (c2P[3:0]),
        .c_3 (c3P[3:0]),
        .c_4 (c4P[3:0]),
        .c_5 (c5P[3:0]),
        .c_6 (c6P[3:0]),
        .c_7 (c7P[3:0]),
        .c_8 (c8P[3:0]),
        .c_9 (c9P[3:0]),
        .c_10 (c10P[3:0]),
        .c_11 (c11P[3:0]),
        .c_12 (c12P[3:0]),
        .c_13 (c13P[3:0]),
        .c_14 (c14P[3:0]),
        .c_15 (c15P[3:0]),
        .c_16 (c16P[3:0]),
        .c_17 (c17P[3:0]),
        .c_18 (c18P[3:0]),
        .c_19 (c19P[3:0]),
        .c_20 (c20P[3:0]),
        .c_21 (c21P[3:0]),
        .c_22 (c22P[3:0]),
        .c_23 (c23P[3:0]),
        .c_24 (c24P[3:0]),
        .c_25 (c25P[3:0]),
        .c_26 (c26P[3:0]),
        .c_27 (c27P[3:0]),
        .c_28 (c28P[3:0]),
        .c_29 (c29P[3:0]),
        .c_30 (c30P[3:0]),
        .c_31 (c31P[3:0]),
        .c_32 (c32P[3:0]),
        .c_33 (c33P[3:0]),
        .c_34 (c34P[3:0]),
        .c_35 (c35P[3:0]),
        .c_36 (c36P[3:0]),
        .c_37 (c37P[3:0]),
        .c_38 (c38P[3:0]),
        .c_39 (c39P[3:0]),

        .dll_lock (dd_dll_lock_int),         // phase-gen dll lock indicator
        .state (state),
        .delay_conf_ptr (delay_conf_ptr),
        .delay_conf_left (delay_conf_left),
        .low_bw_ctr (low_bw_ctr),
        .phase_accum (phase_accum)
        );


//==================================================================
// Top level logic

// delay reset into fsm so that the state machine can reset itself when staticEnable is turned back on
reg cdc_staticEnable_d1, cdc_staticEnable_d2, cdc_staticEnable_d3;
//always @(posedge dfs_clk_gated) begin
always @(posedge dfs_clk_iso) begin
  cdc_staticEnable_d1 <= cdc_staticEnable;
  cdc_staticEnable_d2 <= cdc_staticEnable_d1;
  cdc_staticEnable_d3 <= cdc_staticEnable_d2;
end


// Reset logic
assign reset = cdc_masterReset | ~cdc_ENABLE_DD | ~cdc_STRETCH_ENABLE | ~cdc_staticEnable_d3 | ~cdc_staticEnable ; // rilau 20121218

// Lock indicator
assign dd_dll_lock_func = dd_dll_lock_int & ~reset; // rilau 20121218

// DroopDetected Output
assign droopdetected_func = droopdetected_int & (~reset & dd_dll_lock_int);


//======================================
// FSM_DEBUG_SEL[1:0] to choose signals
// to output on debug_c*[3:0]
//======================================
wire [3:0] debug_00_c0;
wire [3:0] debug_00_c19;
wire [3:0] debug_00_c29;
wire [3:0] debug_00_c39;
wire [3:0] debug_01_c0;
wire [3:0] debug_01_c19;
wire [3:0] debug_01_c29;
wire [3:0] debug_01_c39;
wire [3:0] debug_10_c0;
wire [3:0] debug_10_c19;
wire [3:0] debug_10_c29;
wire [3:0] debug_10_c39;
wire [3:0] debug_11_c0;
wire [3:0] debug_11_c19;
wire [3:0] debug_11_c29;
wire [3:0] debug_11_c39;

assign debug_00_c0  = c0P; 
assign debug_00_c19 = c19P;
assign debug_00_c29 = c29P;
assign debug_00_c39 = c39P;

assign debug_01_c0  = {1'b0,state[2:0]};
assign debug_01_c19 = {1'b0,delay_conf_ptr[6:4]};
assign debug_01_c29 = delay_conf_ptr[3:0];
assign debug_01_c39 = delay_conf_left[3:0];

assign debug_10_c0  = low_bw_ctr[15:12];
assign debug_10_c19 = low_bw_ctr[11:8];
assign debug_10_c29 = low_bw_ctr[7:4]; 
assign debug_10_c39 = low_bw_ctr[3:0];

assign debug_11_c0  = phase_accum[15:12];
assign debug_11_c19 = phase_accum[11:8];
assign debug_11_c29 = phase_accum[7:4]; 
assign debug_11_c39 = phase_accum[3:0];

always @(*) begin
  case (FSM_DEBUG_SEL[1:0])
    2'b00 : begin 
            debug_c0  = debug_00_c0;
            debug_c19 = debug_00_c19;
            debug_c29 = debug_00_c29;
            debug_c39 = debug_00_c39;
            end
    2'b01 : begin
            debug_c0  = debug_01_c0;
            debug_c19 = debug_01_c19;
            debug_c29 = debug_01_c29;
            debug_c39 = debug_01_c39;
            end
    2'b10 : begin
            debug_c0  = debug_10_c0;
            debug_c19 = debug_10_c19;
            debug_c29 = debug_10_c29;
            debug_c39 = debug_10_c39;
            end
    2'b11 : begin
            debug_c0  = debug_11_c0;
            debug_c19 = debug_11_c19;
            debug_c29 = debug_11_c29;
            debug_c39 = debug_11_c39;
            end
    default : begin
            debug_c0  = 4'bxxxx;
            debug_c19 = 4'bxxxx;
            debug_c29 = 4'bxxxx;
            debug_c39 = 4'bxxxx;
            end
  endcase
end

//======================================
// Muxes to select between LF, MF, HF
//======================================

assign dd_lead_mf = {lead_ph40_mf, lead_ph39_mf, lead_ph38_mf, lead_ph37_mf, lead_ph36_mf, lead_ph35_mf, lead_ph34_mf, lead_ph33_mf}; 

assign dd_lead_hf = {lead_ph40_hf, lead_ph39_hf, lead_ph38_hf, lead_ph37_hf, lead_ph36_hf, lead_ph35_hf, lead_ph34_hf, lead_ph33_hf}; 

assign dd_lead[17:8] = {lead_ph50_hf, lead_ph49_hf,lead_ph48_hf,lead_ph47_hf,lead_ph46_hf,lead_ph45_hf,lead_ph44_hf,lead_ph43_hf,lead_ph42_hf,lead_ph41_hf};


mbstretch_clkmux_cpu I_mux_dd_lead_hf [7:0] (
    .VDD(VDD),
    .VSS(VSS),
    .IN0(dd_lead_mf[7:0]),
    .IN1(dd_lead_hf[7:0]),
    .SEL(FREQ_MODE_buf[1]),
    .OUT(dd_lead[7:0])
);


//Output for Debug purpose; feads in debug bus at SMU
assign phase_lead_lag[7:0] = dd_lead[7:0] ;

//======================================
// charz_observ_a/b
//======================================
mbclkds_charz_observ I_charz_observ (
    .VDD                (VDD),
    .VSS                (VSS),
    .CHARZ_SEL_A        (CHARZ_SEL_A),
    .CHARZ_SEL_B        (CHARZ_SEL_B),
    .CHARZ_DIV_A        (CHARZ_DIV_A),
    .CHARZ_DIV_B        (CHARZ_DIV_B),
    .ASYNC_RESETB       (cdc_staticEnable),
    .IN_0               (droopdetected_func),
    .IN_1               (dd_dll_lock_func),
    .IN_2               (dd_lead[0]),
    .IN_3               (dfs_clk_iso),
    .IN_4               (dd_lead[1]),
    .IN_5               (test_clk_ref),
    .IN_6               (dd_lead[2]),
    .IN_7               (dd_lead[3]),
    .IN_8               (dd_lead[4]),
    .IN_9               (dd_lead[5]),
    .IN_10              (test_clk_fb40),
    .IN_11              (debug_c0[0]),
    .IN_12              (reset),
    .IN_13              (dd_lead[6]),
    .IN_14              (dd_lead[7]),
    .IN_15              (dfs_clk_gated),
    .charz_observ_a     (charz_observ_a_int),
    .charz_observ_b     (charz_observ_b_int) 
);


`ifdef CLKDS_SYNTHESIS
		HDBSVT08_MUX2_CK_2 I_droopdetected_mux ( 
            .CLK1(droopdetected_func), 
    				.CLK2(charz_observ_a_int), 
            .CLKSEL(CHARZ_DEBUG_MODE),
            .Z(droopdetected)
            );				
     HDBSVT08_MUX2_CK_2 I_dll_lock_mux ( 
            .CLK1(dd_dll_lock_func), 
    				.CLK2(charz_observ_b_int), 
            .CLKSEL(CHARZ_DEBUG_MODE),
            .Z(dd_dll_lock)
            );   
    HDBSVT08_BUF_CK_10 I_charz_observ_a_buf (.CLK(charz_observ_a_int), .Z(charz_observ_a) );
    HDBSVT08_BUF_CK_10  I_charz_observ_b_buf (.CLK(charz_observ_b_int), .Z(charz_observ_b) );

`else
assign droopdetected = CHARZ_DEBUG_MODE ? charz_observ_a_int : droopdetected_func;
assign dd_dll_lock   = CHARZ_DEBUG_MODE ? charz_observ_b_int : dd_dll_lock_func;

assign charz_observ_a = charz_observ_a_int;
assign charz_observ_b = charz_observ_b_int;

`endif //CLKDS_SYNTHESIS

assign test_clk_sel = FREQ_MODE[1:0];

//===========================================
// CDC Buffers
mbdroop_sapr_top_cdc_container I_mbdroop_sapr_top_cdc_container (
`ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
.dfs_clk_gated(dfs_clk_gated ),
`endif
.NO_EXTRA_COARSE_STEP(NO_EXTRA_COARSE_STEP  ),
.FREQ_MODE(FREQ_MODE  ),
.STRETCH_THRESH(STRETCH_THRESH  ),
.CONSERVATIVE_LOCK(CONSERVATIVE_LOCK  ),
.LOW_BW_SKEW_EN (LOW_BW_SKEW_EN  ),
.LOW_BW_CONFIG_UP(LOW_BW_CONFIG_UP  ),
.LOW_BW_CONFIG (LOW_BW_CONFIG  ),
.FSM_WAIT_CYCLES(FSM_WAIT_CYCLES  ),
.SAMPLE_SIZE (SAMPLE_SIZE  ),
.PD_SYNC_SEL(PD_SYNC_SEL  ),
.UNSTRETCH_THRESH (UNSTRETCH_THRESH),
.DFSSTRETCH_AMOUNT (DFSSTRETCH_AMOUNT ),

//output
.NO_EXTRA_COARSE_STEP_buf(NO_EXTRA_COARSE_STEP_buf  ),
.FREQ_MODE_buf (FREQ_MODE_buf  ),
.STRETCH_THRESH_buf(STRETCH_THRESH_buf  ),
.CONSERVATIVE_LOCK_buf(CONSERVATIVE_LOCK_buf  ),
.LOW_BW_SKEW_EN_buf(LOW_BW_SKEW_EN_buf  ),
.LOW_BW_CONFIG_UP_buf(LOW_BW_CONFIG_UP_buf  ),
.LOW_BW_CONFIG_buf (LOW_BW_CONFIG_buf  ),
.FSM_WAIT_CYCLES_buf (FSM_WAIT_CYCLES_buf  ),
.SAMPLE_SIZE_buf(SAMPLE_SIZE_buf  ),
.PD_SYNC_SEL_buf (PD_SYNC_SEL_buf ),
.UNSTRETCH_THRESH_buf (UNSTRETCH_THRESH_buf),
.DFSSTRETCH_AMOUNT_buf (DFSSTRETCH_AMOUNT_buf)


);


// -- Top level logic
//==================================================================


endmodule
