//==================================================================================================
//
// mbdroopstretchfsm.v
// 
// Copyright (c) 2002 by Advanced Micro Devices, Inc.
//
// This file is protected by Federal Copyright Law, with all rights
// reserved. No part of this file may be reproduced, stored in a
// retrieval system, translated, transcribed, or transmitted, in any
// form, or by any means manual, electric, electronic, mechanical,
// electro-magnetic, chemical, optical, or otherwise, without prior
// explicit written permission from Advanced Micro Devices, Inc.
//
//==================================================================================================
//
// INTERESTED PARTIES
//
// Circuit              Paras Shah
// RTL                  Faisal Syed      
// Verification         Sean Sequeira 
//
//==================================================================================================
//
// Module Description
//
// Voltage droop detect fsm
//
//==================================================================================================


`timescale 1ps/1ps

// FSM for DLL control
`define DROOPSTRETCHFSM_IDLE                     3'b000
`define DROOPSTRETCHFSM_LF1                      3'b001
`define DROOPSTRETCHFSM_UNCOND_COARSE_INC        3'b010
`define DROOPSTRETCHFSM_COARSE_INC               3'b011
`define DROOPSTRETCHFSM_FINE_DEC                 3'b100
`define DROOPSTRETCHFSM_LOW_BW_UPDATE            3'b101

//`define DROOPSTRETCHFSM_NUM_DELAY_CELLS          6'b101000 // 40
`define DROOPSTRETCHFSM_NUM_DELAY_CELLS_28  7'd28
`define DROOPSTRETCHFSM_NUM_DELAY_CELLS_40  7'd40


module mbdroopstretchfsm (

input        VDD, 
input        VSS,

//input        STRETCHER_FSM,         // STRETCHER (1'b1) vs DROOPDETECT (1'b0) strap
input        CCLK,                  // Buffered Pll clock
input        RESET,                 // 
input  [6:0] LOW_BW_CONFIG,  
input  [6:0] LOW_BW_CONFIG_UP,
input        LOW_BW_SKEW_EN,
input        PD_d0,     
input  [3:0] FSM_WAIT_CYCLES,       // Number of cycles to wait after each delay setting change
input  [3:0] SAMPLE_SIZE,           // Number of samples to take to determine Lead/Lag
input        NO_EXTRA_COARSE_STEP,  // If 1, FSM will increase delay by one coarse step going into FINE_DEC state.  Default = 1.  Default = 0.
input        use_ph28,              // 1 = use 28 stages for locking  

input  [1:0] PD_SYNC_SEL,           // 0,3 = 2 flops sync cell; 1 = 1 flop sync cell; 2 = no sync cell
input        OVERRIDE_DLY_EN,       // For characterization - overrides values to delay cells
input  [3:0] OVERRIDE_DLY_VAL,      // For characterization - if enabled, delay cell values will be overriden with this register

input  [1:0] FSM_STEP_SIZE,         // Programmable coarse_inc step size when FSM transition from LF1 to COARSE_INC state
input  [1:0] FSM_CODE_MARGIN,       // FSM will not assert lock if final setting of c0[3:0] is outside of this range

output reg [3:0] c_0,
output reg [3:0] c_1,
output reg [3:0] c_2,
output reg [3:0] c_3,
output reg [3:0] c_4,
output reg [3:0] c_5,
output reg [3:0] c_6,
output reg [3:0] c_7,
output reg [3:0] c_8,
output reg [3:0] c_9,
output reg [3:0] c_10,
output reg [3:0] c_11,
output reg [3:0] c_12,
output reg [3:0] c_13,
output reg [3:0] c_14,
output reg [3:0] c_15,
output reg [3:0] c_16,
output reg [3:0] c_17,
output reg [3:0] c_18,
output reg [3:0] c_19,
output reg [3:0] c_20,
output reg [3:0] c_21,
output reg [3:0] c_22,
output reg [3:0] c_23,
output reg [3:0] c_24,
output reg [3:0] c_25,
output reg [3:0] c_26,
output reg [3:0] c_27,
output reg [3:0] c_28,
output reg [3:0] c_29,
output reg [3:0] c_30,
output reg [3:0] c_31,
output reg [3:0] c_32,
output reg [3:0] c_33,
output reg [3:0] c_34,
output reg [3:0] c_35,
output reg [3:0] c_36,
output reg [3:0] c_37,
output reg [3:0] c_38,
output reg [3:0] c_39,

output reg        dll_lock,         // signal indicating dll lock

output reg [2:0] state,
output reg [6:0] delay_conf_ptr,
output reg [3:0] delay_conf_left,
output reg [15:0] low_bw_ctr,
output reg [15:0] phase_accum
);

wire [3:0] c_int_0;
wire [3:0] c_int_1;
wire [3:0] c_int_2;
wire [3:0] c_int_3;
wire [3:0] c_int_4;
wire [3:0] c_int_5;
wire [3:0] c_int_6;
wire [3:0] c_int_7;
wire [3:0] c_int_8;
wire [3:0] c_int_9;
wire [3:0] c_int_10;
wire [3:0] c_int_11;
wire [3:0] c_int_12;
wire [3:0] c_int_13;
wire [3:0] c_int_14;
wire [3:0] c_int_15;
wire [3:0] c_int_16;
wire [3:0] c_int_17;
wire [3:0] c_int_18;
wire [3:0] c_int_19;
wire [3:0] c_int_20;
wire [3:0] c_int_21;
wire [3:0] c_int_22;
wire [3:0] c_int_23;
wire [3:0] c_int_24;
wire [3:0] c_int_25;
wire [3:0] c_int_26;
wire [3:0] c_int_27;
wire [3:0] c_int_28;
wire [3:0] c_int_29;
wire [3:0] c_int_30;
wire [3:0] c_int_31;
wire [3:0] c_int_32;
wire [3:0] c_int_33;
wire [3:0] c_int_34;
wire [3:0] c_int_35;
wire [3:0] c_int_36;
wire [3:0] c_int_37;
wire [3:0] c_int_38;
wire [3:0] c_int_39;

wire PD_LF1_TO_UNCOND_COARSE_INC;
wire PD_UNCOND_COARSE_INC_TO_FINE_DEC;
wire PD_COARSE_INC_TO_FINE_DEC;
wire PD_FINE_DEC_TO_LOW_BW_UPDATE;

// ************ synopsys pragma start ************** 
//vrq translate_off 
// synopsys sync_set_reset "RESET, Rst_PD_sum, new_RESET_28, new_RESET_40"
//vrq translate_on
// ************ synopsys pragma end **************

`ifndef MACRO_HEADER_ONLY

//! unused VDD
//! unused VSS

// === define number of delay cells ===
wire [6:0] DROOPSTRETCHFSM_NUM_DELAY_CELLS; 
assign DROOPSTRETCHFSM_NUM_DELAY_CELLS = use_ph28 ? `DROOPSTRETCHFSM_NUM_DELAY_CELLS_28 : `DROOPSTRETCHFSM_NUM_DELAY_CELLS_40;

reg  [3:0] PD_sum;
reg  [3:0] sample_count;


// CONSERVATIVE_LOCK functionality
reg    PD_d2;


// Synchrnoizer for PD_d0
mbclkds_cdccell_sync2 I_CDC_PD_d2_sync2 (
        .VDD(VDD), 
        .VSS(VSS),
        .CLK(CCLK),
        .DATAIN(PD_d0),
        .RESET(RESET),
        .DATAOUT(PD_sync2)
        );

mbclkds_cdccell_sync1 I_CDC_PD_d2_sync1 (
        .VDD(VDD), 
        .VSS(VSS),
        .CLK(CCLK),
        .DATAIN(PD_d0),
        .RESET(RESET),
        .DATAOUT(PD_sync1)
        );

mbclkds_cdccell_sync4 I_CDC_PD_d2_sync4 (
        .VDD(VDD), 
        .VSS(VSS),
        .CLK(CCLK),
        .DATAIN(PD_d0),
        .RESET(RESET),
        .DATAOUT(PD_sync4)
        );



//always @(*) begin
//  case (PD_SYNC_SEL)
//    2'b00,2'b11 : PD_d2 = PD_sync2;
//          2'b01 : PD_d2 = PD_sync1;
//          2'b10 : PD_d2 = PD_d0;
//        default : PD_d2 = PD_sync2;
//  endcase
//end

always @(*) begin
  case (PD_SYNC_SEL)
    			2'b00 : PD_d2 = PD_sync2;
          2'b01 : PD_d2 = PD_sync1;
          2'b10 : PD_d2 = PD_d0;
					2'b11 : PD_d2 = PD_sync4;
        default : PD_d2 = PD_sync2;
  endcase
end



//*** Reset logic ***//
reg RESET_1q;
reg RESET_2q;
reg RESET_3q;

always @(posedge CCLK) begin
    RESET_1q <= RESET;
    RESET_2q <= RESET_1q;
    RESET_3q <= RESET_2q;
end


reg   [3:0] delay_conf_right;
reg   [3:0] waitCounter;
reg   [2:0] nextState;
wire        next_dll_lock;
reg   [3:0] next_delay_conf_left;
wire  [3:0] next_delay_conf_right;
reg   [6:0] next_delay_conf_ptr;
reg   [3:0] next_waitCounter;
reg  [15:0] next_low_bw_ctr;
reg  [16:0] next_phase_accum_int;
reg  [16:0] phase_accum_int;
reg  [15:0] low_bw_ctr_maxval;
wire [15:0] low_bw_ctr_decr;
reg  [15:0] low_bw_ctr_incr;
wire [16:0] low_bw_ctr_trip_pt;
wire [16:0] low_bw_ctr_upper_lmt;
wire        code_invalid;


// Bug 489170: delay_conf_ptr stuck at X in PAGsim
// Solution suggested by Synopsys
 
wire new_RESET_28, new_RESET_40;
assign new_RESET_28 = RESET_3q & use_ph28;
assign new_RESET_40 = RESET_3q & ~use_ph28;

always @* phase_accum[15:0] = phase_accum_int[15:0];

always @(posedge CCLK) begin
  if(new_RESET_28) 
    delay_conf_ptr[6:0] <= `DROOPSTRETCHFSM_NUM_DELAY_CELLS_28;
  else if(new_RESET_40) 
    delay_conf_ptr[6:0] <= `DROOPSTRETCHFSM_NUM_DELAY_CELLS_40;
  else 
    delay_conf_ptr[6:0] <= next_delay_conf_ptr[6:0];
end

//Using Delayed version of reset so tht PD synchornizer comes out of reset and FSM samples correct PD value
always @(posedge CCLK)
  if(RESET_3q)
    begin
      state[2:0]               <= `DROOPSTRETCHFSM_IDLE;
      dll_lock                 <= 1'b0;
      waitCounter[3:0]         <= 4'b0000;
      low_bw_ctr[15:0]         <= 16'd0;
      phase_accum_int[16:0]    <= 17'd0;
      delay_conf_left[3:0]     <= 4'b0000;
      delay_conf_right[3:0]    <= 4'b0001;
      //delay_conf_ptr[6:0]      <= DROOPSTRETCHFSM_NUM_DELAY_CELLS;
    end
  else
    begin
      state[2:0]               <= nextState[2:0];
      dll_lock                 <= next_dll_lock;
      waitCounter[3:0]         <= next_waitCounter[3:0];
      low_bw_ctr[15:0]         <= next_low_bw_ctr[15:0];
      phase_accum_int[16:0]    <= next_phase_accum_int[16:0];
      delay_conf_left[3:0]     <= next_delay_conf_left[3:0];
      delay_conf_right[3:0]    <= next_delay_conf_right[3:0];
      //delay_conf_ptr[6:0]      <= next_delay_conf_ptr[6:0];
    end


wire override_c_code = OVERRIDE_DLY_EN & (state==`DROOPSTRETCHFSM_LOW_BW_UPDATE);

// Delay chain configuration bits
assign  c_int_0[3:0] = override_c_code ? OVERRIDE_DLY_VAL[3:0] : delay_conf_left[3:0];
assign  c_int_1[3:0] = override_c_code ? OVERRIDE_DLY_VAL[3:0] : (delay_conf_ptr[6:0]>1)  ? delay_conf_left[3:0] : delay_conf_right[3:0];
assign  c_int_2[3:0] = override_c_code ? OVERRIDE_DLY_VAL[3:0] : (delay_conf_ptr[6:0]>2)  ? delay_conf_left[3:0] : delay_conf_right[3:0];
assign  c_int_3[3:0] = override_c_code ? OVERRIDE_DLY_VAL[3:0] : (delay_conf_ptr[6:0]>3)  ? delay_conf_left[3:0] : delay_conf_right[3:0];
assign  c_int_4[3:0] = override_c_code ? OVERRIDE_DLY_VAL[3:0] : (delay_conf_ptr[6:0]>4)  ? delay_conf_left[3:0] : delay_conf_right[3:0];
assign  c_int_5[3:0] = override_c_code ? OVERRIDE_DLY_VAL[3:0] : (delay_conf_ptr[6:0]>5)  ? delay_conf_left[3:0] : delay_conf_right[3:0];
assign  c_int_6[3:0] = override_c_code ? OVERRIDE_DLY_VAL[3:0] : (delay_conf_ptr[6:0]>6)  ? delay_conf_left[3:0] : delay_conf_right[3:0];
assign  c_int_7[3:0] = override_c_code ? OVERRIDE_DLY_VAL[3:0] : (delay_conf_ptr[6:0]>7)  ? delay_conf_left[3:0] : delay_conf_right[3:0];
assign  c_int_8[3:0] = override_c_code ? OVERRIDE_DLY_VAL[3:0] : (delay_conf_ptr[6:0]>8)  ? delay_conf_left[3:0] : delay_conf_right[3:0];
assign  c_int_9[3:0] = override_c_code ? OVERRIDE_DLY_VAL[3:0] : (delay_conf_ptr[6:0]>9)  ? delay_conf_left[3:0] : delay_conf_right[3:0];
assign c_int_10[3:0] = override_c_code ? OVERRIDE_DLY_VAL[3:0] : (delay_conf_ptr[6:0]>10) ? delay_conf_left[3:0] : delay_conf_right[3:0];
assign c_int_11[3:0] = override_c_code ? OVERRIDE_DLY_VAL[3:0] : (delay_conf_ptr[6:0]>11) ? delay_conf_left[3:0] : delay_conf_right[3:0];
assign c_int_12[3:0] = override_c_code ? OVERRIDE_DLY_VAL[3:0] : (delay_conf_ptr[6:0]>12) ? delay_conf_left[3:0] : delay_conf_right[3:0];
assign c_int_13[3:0] = override_c_code ? OVERRIDE_DLY_VAL[3:0] : (delay_conf_ptr[6:0]>13) ? delay_conf_left[3:0] : delay_conf_right[3:0];
assign c_int_14[3:0] = override_c_code ? OVERRIDE_DLY_VAL[3:0] : (delay_conf_ptr[6:0]>14) ? delay_conf_left[3:0] : delay_conf_right[3:0];
assign c_int_15[3:0] = override_c_code ? OVERRIDE_DLY_VAL[3:0] : (delay_conf_ptr[6:0]>15) ? delay_conf_left[3:0] : delay_conf_right[3:0];
assign c_int_16[3:0] = override_c_code ? OVERRIDE_DLY_VAL[3:0] : (delay_conf_ptr[6:0]>16) ? delay_conf_left[3:0] : delay_conf_right[3:0];
assign c_int_17[3:0] = override_c_code ? OVERRIDE_DLY_VAL[3:0] : (delay_conf_ptr[6:0]>17) ? delay_conf_left[3:0] : delay_conf_right[3:0];
assign c_int_18[3:0] = override_c_code ? OVERRIDE_DLY_VAL[3:0] : (delay_conf_ptr[6:0]>18) ? delay_conf_left[3:0] : delay_conf_right[3:0];
assign c_int_19[3:0] = override_c_code ? OVERRIDE_DLY_VAL[3:0] : (delay_conf_ptr[6:0]>19) ? delay_conf_left[3:0] : delay_conf_right[3:0];
assign c_int_20[3:0] = override_c_code ? OVERRIDE_DLY_VAL[3:0] : (delay_conf_ptr[6:0]>20) ? delay_conf_left[3:0] : delay_conf_right[3:0];
assign c_int_21[3:0] = override_c_code ? OVERRIDE_DLY_VAL[3:0] : (delay_conf_ptr[6:0]>21) ? delay_conf_left[3:0] : delay_conf_right[3:0];
assign c_int_22[3:0] = override_c_code ? OVERRIDE_DLY_VAL[3:0] : (delay_conf_ptr[6:0]>22) ? delay_conf_left[3:0] : delay_conf_right[3:0];
assign c_int_23[3:0] = override_c_code ? OVERRIDE_DLY_VAL[3:0] : (delay_conf_ptr[6:0]>23) ? delay_conf_left[3:0] : delay_conf_right[3:0];
assign c_int_24[3:0] = override_c_code ? OVERRIDE_DLY_VAL[3:0] : (delay_conf_ptr[6:0]>24) ? delay_conf_left[3:0] : delay_conf_right[3:0];
assign c_int_25[3:0] = override_c_code ? OVERRIDE_DLY_VAL[3:0] : (delay_conf_ptr[6:0]>25) ? delay_conf_left[3:0] : delay_conf_right[3:0];
assign c_int_26[3:0] = override_c_code ? OVERRIDE_DLY_VAL[3:0] : (delay_conf_ptr[6:0]>26) ? delay_conf_left[3:0] : delay_conf_right[3:0];
assign c_int_27[3:0] = override_c_code ? OVERRIDE_DLY_VAL[3:0] : (delay_conf_ptr[6:0]>27) ? delay_conf_left[3:0] : delay_conf_right[3:0];
assign c_int_28[3:0] = override_c_code ? OVERRIDE_DLY_VAL[3:0] : (delay_conf_ptr[6:0]>28) ? delay_conf_left[3:0] : delay_conf_right[3:0];
assign c_int_29[3:0] = override_c_code ? OVERRIDE_DLY_VAL[3:0] : (delay_conf_ptr[6:0]>29) ? delay_conf_left[3:0] : delay_conf_right[3:0];
assign c_int_30[3:0] = override_c_code ? OVERRIDE_DLY_VAL[3:0] : (delay_conf_ptr[6:0]>30) ? delay_conf_left[3:0] : delay_conf_right[3:0];
assign c_int_31[3:0] = override_c_code ? OVERRIDE_DLY_VAL[3:0] : (delay_conf_ptr[6:0]>31) ? delay_conf_left[3:0] : delay_conf_right[3:0];
assign c_int_32[3:0] = override_c_code ? OVERRIDE_DLY_VAL[3:0] : (delay_conf_ptr[6:0]>32) ? delay_conf_left[3:0] : delay_conf_right[3:0];
assign c_int_33[3:0] = override_c_code ? OVERRIDE_DLY_VAL[3:0] : (delay_conf_ptr[6:0]>33) ? delay_conf_left[3:0] : delay_conf_right[3:0];
assign c_int_34[3:0] = override_c_code ? OVERRIDE_DLY_VAL[3:0] : (delay_conf_ptr[6:0]>34) ? delay_conf_left[3:0] : delay_conf_right[3:0];
assign c_int_35[3:0] = override_c_code ? OVERRIDE_DLY_VAL[3:0] : (delay_conf_ptr[6:0]>35) ? delay_conf_left[3:0] : delay_conf_right[3:0];
assign c_int_36[3:0] = override_c_code ? OVERRIDE_DLY_VAL[3:0] : (delay_conf_ptr[6:0]>36) ? delay_conf_left[3:0] : delay_conf_right[3:0];
assign c_int_37[3:0] = override_c_code ? OVERRIDE_DLY_VAL[3:0] : (delay_conf_ptr[6:0]>37) ? delay_conf_left[3:0] : delay_conf_right[3:0];
assign c_int_38[3:0] = override_c_code ? OVERRIDE_DLY_VAL[3:0] : (delay_conf_ptr[6:0]>38) ? delay_conf_left[3:0] : delay_conf_right[3:0];
assign c_int_39[3:0] = override_c_code ? OVERRIDE_DLY_VAL[3:0] : (delay_conf_ptr[6:0]>39) ? delay_conf_left[3:0] : delay_conf_right[3:0];

always @(posedge CCLK) begin
     c_0[3:0] <= c_int_0[3:0];
     c_1[3:0] <= c_int_1[3:0];
     c_2[3:0] <= c_int_2[3:0];
     c_3[3:0] <= c_int_3[3:0];
     c_4[3:0] <= c_int_4[3:0];
     c_5[3:0] <= c_int_5[3:0];
     c_6[3:0] <= c_int_6[3:0];
     c_7[3:0] <= c_int_7[3:0];
     c_8[3:0] <= c_int_8[3:0];
     c_9[3:0] <= c_int_9[3:0];
    c_10[3:0] <= c_int_10[3:0];
    c_11[3:0] <= c_int_11[3:0];
    c_12[3:0] <= c_int_12[3:0];
    c_13[3:0] <= c_int_13[3:0];
    c_14[3:0] <= c_int_14[3:0];
    c_15[3:0] <= c_int_15[3:0];
    c_16[3:0] <= c_int_16[3:0];
    c_17[3:0] <= c_int_17[3:0];
    c_18[3:0] <= c_int_18[3:0];
    c_19[3:0] <= c_int_19[3:0];
    c_20[3:0] <= c_int_20[3:0];
    c_21[3:0] <= c_int_21[3:0];
    c_22[3:0] <= c_int_22[3:0];
    c_23[3:0] <= c_int_23[3:0];
    c_24[3:0] <= c_int_24[3:0];
    c_25[3:0] <= c_int_25[3:0];
    c_26[3:0] <= c_int_26[3:0];
    c_27[3:0] <= c_int_27[3:0];
    c_28[3:0] <= c_int_28[3:0];
    c_29[3:0] <= c_int_29[3:0];
    c_30[3:0] <= c_int_30[3:0];
    c_31[3:0] <= c_int_31[3:0];
    c_32[3:0] <= c_int_32[3:0];
    c_33[3:0] <= c_int_33[3:0];
    c_34[3:0] <= c_int_34[3:0];
    c_35[3:0] <= c_int_35[3:0];
    c_36[3:0] <= c_int_36[3:0];
    c_37[3:0] <= c_int_37[3:0];
    c_38[3:0] <= c_int_38[3:0];
    c_39[3:0] <= c_int_39[3:0];
end

// Choose between high bandwidht or low bandwidth update
// 16 bit counter for high/low bandwidth update
always @(*)
    casez (LOW_BW_CONFIG[6:0])                                     // fine incr/decr     
      7'b0000000:  low_bw_ctr_maxval[15:0] = 16'b0000000000000000; // disabled
      7'b0001???:  low_bw_ctr_maxval[15:0] = 16'b0000000000000001; // every     2 cycles
      7'b0010???:  low_bw_ctr_maxval[15:0] = 16'b0000000000000010; // every     3 cycles
      7'b0011???:  low_bw_ctr_maxval[15:0] = 16'b0000000000000011; // every     4 cycles
      7'b0100???:  low_bw_ctr_maxval[15:0] = 16'b0000000000000101; // every     6 cycles
      7'b0101???:  low_bw_ctr_maxval[15:0] = 16'b0000000000000111; // every     8 cycles
      7'b0110???:  low_bw_ctr_maxval[15:0] = 16'b0000000000001011; // every    12 cycles
      7'b0111???:  low_bw_ctr_maxval[15:0] = 16'b0000000000001111; // every    16 cycles
      7'b1000???:  low_bw_ctr_maxval[15:0] = 16'b0000000000010111; // every    24 cycles
      7'b1001???:  low_bw_ctr_maxval[15:0] = 16'b0000000000011111; // every    32 cycles
      7'b1010???:  low_bw_ctr_maxval[15:0] = 16'b0000000000101111; // every    48 cycles
      7'b1011???:  low_bw_ctr_maxval[15:0] = 16'b0000000000111111; // every    64 cycles
      7'b1100???:  low_bw_ctr_maxval[15:0] = 16'b0000000001011111; // every    96 cycles
      7'b1101???:  low_bw_ctr_maxval[15:0] = 16'b0000000001111111; // every   128 cycles
      7'b1110???:  low_bw_ctr_maxval[15:0] = 16'b0000000011111111; // every   256 cycles
      7'b1111???:  low_bw_ctr_maxval[15:0] = 16'b0000000111111111; // every   512 cycles
                                                                   
      7'b0000001:  low_bw_ctr_maxval[15:0] = 16'b0000001111111111; // every  1024 cycles
      7'b0000010:  low_bw_ctr_maxval[15:0] = 16'b0000011111111111; // every  2048 cycles
      7'b0000011:  low_bw_ctr_maxval[15:0] = 16'b0000111111111111; // every  4096 cycles
      7'b0000100:  low_bw_ctr_maxval[15:0] = 16'b0001111111111111; // every  8192 cycles
      7'b0000101:  low_bw_ctr_maxval[15:0] = 16'b0011111111111111; // every 16384 cycles
      7'b0000110:  low_bw_ctr_maxval[15:0] = 16'b0111111111111111; // every 32768 cycles
      7'b0000111:  low_bw_ctr_maxval[15:0] = 16'b1111111111111111; // every 65536 cycles
         default:  low_bw_ctr_maxval[15:0] = 16'bx;
    endcase

//Used when LOW_BW_SKEW_EN== 1'b1
always @(*)
    casez (LOW_BW_CONFIG_UP[6:0])                                     // fine incr     
      7'b0000000:  low_bw_ctr_incr[15:0] = 16'b0000000000000000; // disabled
      7'b0001???:  low_bw_ctr_incr[15:0] = 16'b0000000000000001; // every     2 cycles
      7'b0010???:  low_bw_ctr_incr[15:0] = 16'b0000000000000010; // every     3 cycles
      7'b0011???:  low_bw_ctr_incr[15:0] = 16'b0000000000000011; // every     4 cycles
      7'b0100???:  low_bw_ctr_incr[15:0] = 16'b0000000000000101; // every     6 cycles
      7'b0101???:  low_bw_ctr_incr[15:0] = 16'b0000000000000111; // every     8 cycles
      7'b0110???:  low_bw_ctr_incr[15:0] = 16'b0000000000001011; // every    12 cycles
      7'b0111???:  low_bw_ctr_incr[15:0] = 16'b0000000000001111; // every    16 cycles
      7'b1000???:  low_bw_ctr_incr[15:0] = 16'b0000000000010111; // every    24 cycles
      7'b1001???:  low_bw_ctr_incr[15:0] = 16'b0000000000011111; // every    32 cycles
      7'b1010???:  low_bw_ctr_incr[15:0] = 16'b0000000000101111; // every    48 cycles
      7'b1011???:  low_bw_ctr_incr[15:0] = 16'b0000000000111111; // every    64 cycles
      7'b1100???:  low_bw_ctr_incr[15:0] = 16'b0000000001011111; // every    96 cycles
      7'b1101???:  low_bw_ctr_incr[15:0] = 16'b0000000001111111; // every   128 cycles
      7'b1110???:  low_bw_ctr_incr[15:0] = 16'b0000000011111111; // every   256 cycles
      7'b1111???:  low_bw_ctr_incr[15:0] = 16'b0000000111111111; // every   512 cycles
                                                                   
      7'b0000001:  low_bw_ctr_incr[15:0] = 16'b0000001111111111; // every  1024 cycles
      7'b0000010:  low_bw_ctr_incr[15:0] = 16'b0000011111111111; // every  2048 cycles
      7'b0000011:  low_bw_ctr_incr[15:0] = 16'b0000111111111111; // every  4096 cycles
      7'b0000100:  low_bw_ctr_incr[15:0] = 16'b0001111111111111; // every  8192 cycles
      7'b0000101:  low_bw_ctr_incr[15:0] = 16'b0011111111111111; // every 16384 cycles
      7'b0000110:  low_bw_ctr_incr[15:0] = 16'b0111111111111111; // every 32768 cycles
      7'b0000111:  low_bw_ctr_incr[15:0] = 16'b1111111111111111; // every 65536 cycles
         default:  low_bw_ctr_incr[15:0] = 16'bx;
    endcase


// Trip point for increment and decrement 
//assign low_bw_ctr_maxval_half = (low_bw_ctr_maxval[15:0] > 16'd1) ? low_bw_ctr_maxval >> 1 : low_bw_ctr_maxval;
// In case of skew enable : 
//>0--------------------LOW_BW_CONFIG----------------LOW_BW_CONFIG_UP <
//>0-------------------low_bw_ctr_decr--------------low_bw_ctr_incr   <
//>        PD_d2==0 <-------------------->  PD_d2==1                  <

assign low_bw_ctr_decr[15:0] = low_bw_ctr_maxval[15:0];

assign low_bw_ctr_trip_pt = LOW_BW_SKEW_EN ? low_bw_ctr_decr[15:0] :
                           (low_bw_ctr_maxval[15:0] > 16'd1) ? low_bw_ctr_maxval >> 1 : low_bw_ctr_maxval;
assign low_bw_ctr_upper_lmt = low_bw_ctr_incr + low_bw_ctr_decr ;
reg  low_update;
reg  low_update_1q;
reg  low_update_2q;
reg  low_update_3q;
reg  low_update_extended;


always @(*)
  begin
    case(LOW_BW_SKEW_EN)
    1'b0 : low_update = low_bw_ctr_maxval[15:0] == 16'd0 ? 1'b0 : 
                        low_bw_ctr_maxval[15:0] == 16'd1 ? (low_bw_ctr[15:0] == low_bw_ctr_maxval[15:0]) :
                        (low_bw_ctr[15:0] == (low_bw_ctr_maxval[15:0] - 1'b1)) ;

    1'b1 : low_update =  low_bw_ctr_trip_pt[15:0] == 16'd0 ? 1'b0 : 
                         (state[2:0]==`DROOPSTRETCHFSM_LOW_BW_UPDATE) &(( phase_accum_int[16:0]  == 0) || ( phase_accum_int[16:0] ==  low_bw_ctr_upper_lmt ));
    endcase
  end  
                         

always @(posedge CCLK) begin
  if (RESET_3q) begin
     low_update_1q <= 1'b0;
     low_update_2q <= 1'b0;
     low_update_3q <= 1'b0;
  end
  else begin
     low_update_1q <= low_update;
     low_update_2q <= low_update_1q;
     low_update_3q <= low_update_2q;
  end
end

always @(*) begin
  if (low_bw_ctr_maxval[15:0] > 16'd1)
    case (PD_SYNC_SEL)
      2'b00,2'b11 : low_update_extended = low_update | low_update_1q | low_update_2q | low_update_3q;
            2'b01 : low_update_extended = low_update | low_update_1q | low_update_2q;
            2'b10 : low_update_extended = low_update | low_update_1q;
          default : low_update_extended = low_update | low_update_1q | low_update_2q | low_update_3q;
    endcase
  else
    case (PD_SYNC_SEL)
      2'b00,2'b11 : low_update_extended = low_update | low_update_1q | low_update_2q;
            2'b01 : low_update_extended = low_update | low_update_1q;
            2'b10 : low_update_extended = low_update;
          default : low_update_extended = low_update | low_update_1q | low_update_2q;
    endcase
end

always @(*)
  if((state[2:0] != `DROOPSTRETCHFSM_LOW_BW_UPDATE) | low_update_extended) begin
      if(LOW_BW_SKEW_EN)
        begin
          next_low_bw_ctr[15:0]  = 16'd0;
          next_phase_accum_int[16:0] =  {1'b0,low_bw_ctr_trip_pt[15:0]}; // Resetting to middle or trip point value
        end  
      else
        begin
          next_low_bw_ctr[15:0] = 16'd0;
          next_phase_accum_int[16:0] = 17'd0;
        end
    end
  else begin
      if(LOW_BW_SKEW_EN)
          begin
            next_low_bw_ctr[15:0]  = 16'd0;
            if (PD_d2)
              next_phase_accum_int[16:0] = phase_accum_int[16:0] + 1'b1 ;
            else
              if(phase_accum_int[16:0] != 0)
               next_phase_accum_int[16:0] = phase_accum_int[16:0] - 1'b1 ;
              else
                next_phase_accum_int[16:0] = phase_accum_int[16:0] ;
          end
      else
          begin
            next_phase_accum_int[16:0] = phase_accum_int[16:0] + PD_d2;
            next_low_bw_ctr[15:0]  = low_bw_ctr[15:0] + 16'd1;
          end        
  end


// DLL state machine logic
reg fine_dec;
reg fine_inc;
wire coarse_inc;

wire skip_coarse_inc = NO_EXTRA_COARSE_STEP & (state==`DROOPSTRETCHFSM_COARSE_INC) & (nextState==`DROOPSTRETCHFSM_FINE_DEC);

always @(*)
  begin 
     case (LOW_BW_SKEW_EN)
       1'b0 : begin 
                 fine_dec = (low_bw_ctr_maxval[15:0] > 16'd1) ? 
                            ((state[2:0]==`DROOPSTRETCHFSM_FINE_DEC) & (waitCounter[3:0]==FSM_WAIT_CYCLES[3:0]) & (sample_count==0)) | (low_update & ((phase_accum_int[16:0] + PD_d2) < low_bw_ctr_trip_pt)) :
                            ((state[2:0]==`DROOPSTRETCHFSM_FINE_DEC) & (waitCounter[3:0]==FSM_WAIT_CYCLES[3:0]) & (sample_count==0)) | (low_update & ~PD_d2);
 
                 fine_inc = (low_bw_ctr_maxval[15:0] > 16'd1) ?
                         low_update & ((phase_accum_int[16:0] + PD_d2) > low_bw_ctr_trip_pt) : low_update & PD_d2;
              end
                      

       1'b1 :  begin 
                  fine_dec = (low_bw_ctr_trip_pt[15:0] > 16'd1) ? 
                             ((state[2:0]==`DROOPSTRETCHFSM_FINE_DEC) & (waitCounter[3:0]==FSM_WAIT_CYCLES[3:0]) & (sample_count==0)) | (low_update & ((phase_accum_int[16:0] ) < low_bw_ctr_trip_pt)) :
                             ((state[2:0]==`DROOPSTRETCHFSM_FINE_DEC) & (waitCounter[3:0]==FSM_WAIT_CYCLES[3:0]) & (sample_count==0)) | (low_update & ~PD_d2);


                  fine_inc =  (low_bw_ctr_trip_pt[15:0] > 16'd1) ? 
                              low_update & ((phase_accum_int[16:0] ) > low_bw_ctr_trip_pt): low_update & PD_d2 ;
               end              
     endcase
 end    



                         

assign coarse_inc = ((state[2:0]==`DROOPSTRETCHFSM_LF1) | ( (state[2:0]==`DROOPSTRETCHFSM_UNCOND_COARSE_INC)&&PD_d2) | ( (state[2:0]==`DROOPSTRETCHFSM_COARSE_INC)&&PD_d2) ) & (waitCounter[3:0]==FSM_WAIT_CYCLES[3:0]) & (sample_count==0) & ~skip_coarse_inc;

always @(*)
  case ({coarse_inc,fine_inc,fine_dec})
    3'b001: 
      if(delay_conf_ptr[6:0]==DROOPSTRETCHFSM_NUM_DELAY_CELLS)
        begin
          next_delay_conf_ptr[6:0] = 7'b0000000;
          next_delay_conf_left[3:0] = (delay_conf_left[3:0]==4'b0000)? delay_conf_left[3:0]: delay_conf_left[3:0]-4'b0001;
        end
      else
        begin
          next_delay_conf_ptr[6:0] = delay_conf_ptr[6:0]+7'b0000001;
          next_delay_conf_left[3:0] = delay_conf_left[3:0];
        end
    3'b010:
      if(delay_conf_ptr[6:0]==7'b0000000)
        begin
          next_delay_conf_ptr[6:0]  = DROOPSTRETCHFSM_NUM_DELAY_CELLS;
          next_delay_conf_left[3:0] = (delay_conf_left[3:0]==4'b1111)? delay_conf_left[3:0]: delay_conf_left[3:0]+4'b0001;
        end
      else
        begin
          next_delay_conf_ptr[6:0] = delay_conf_ptr[6:0]-7'b0000001;
          next_delay_conf_left[3:0] = delay_conf_left[3:0];
        end
    3'b100: begin
      next_delay_conf_ptr[6:0] = delay_conf_ptr[6:0];
      if(PD_LF1_TO_UNCOND_COARSE_INC && state[2:0]==`DROOPSTRETCHFSM_LF1)
        if(delay_conf_left[3:0] < (4'b1111-FSM_STEP_SIZE[1:0]) )
            next_delay_conf_left[3:0] = delay_conf_left[3:0]+4'b0001+FSM_STEP_SIZE[1:0];
          else
            next_delay_conf_left[3:0] = 4'b1111;
      else
        if(delay_conf_left[3:0] != 4'b1111)
          next_delay_conf_left[3:0] = delay_conf_left[3:0]+4'b0001;
        else
          next_delay_conf_left[3:0] = delay_conf_left[3:0];
    end
    default: begin 
      next_delay_conf_ptr[6:0] = delay_conf_ptr[6:0];
      next_delay_conf_left[3:0] = delay_conf_left[3:0];
    end
  endcase

assign next_delay_conf_right[3:0] = (next_delay_conf_left[3:0]==4'b1111) ? next_delay_conf_left[3:0] : next_delay_conf_left[3:0] + 4'b0001;
assign next_dll_lock = (RESET_3q | code_invalid) ? 1'b0 : (state[2:0] == `DROOPSTRETCHFSM_LOW_BW_UPDATE);

assign code_invalid = (delay_conf_left[3:0] < FSM_CODE_MARGIN[1:0]) | (delay_conf_left[3:0] > (4'hf-FSM_CODE_MARGIN[1:0]));

//===========================================
// FSM
//===========================================
wire [3:0] half_sample_size;

assign PD_LF1_TO_UNCOND_COARSE_INC              = (SAMPLE_SIZE[3:1]==3'b000) ? PD_d2 : PD_sum==SAMPLE_SIZE;
assign PD_UNCOND_COARSE_INC_TO_FINE_DEC         = (SAMPLE_SIZE[3:1]==3'b000) ? ~PD_d2 : PD_sum==0;
assign PD_COARSE_INC_TO_FINE_DEC                = (SAMPLE_SIZE[3:1]==3'b000) ? ~PD_d2 : PD_sum==0;
assign PD_FINE_DEC_TO_LOW_BW_UPDATE             = (SAMPLE_SIZE[3:1]==3'b000) ? PD_d2 : PD_sum >= half_sample_size;

wire READY_FOR_DECISION = (waitCounter[3:0]==FSM_WAIT_CYCLES[3:0]) & (sample_count==0);

always @(*)
  case(state[2:0])
    `DROOPSTRETCHFSM_IDLE:
        case(PD_d2)
          1'b0:    nextState[2:0] = `DROOPSTRETCHFSM_LF1;
          1'b1:    nextState[2:0] = `DROOPSTRETCHFSM_UNCOND_COARSE_INC;
          default: nextState[2:0] = `DROOPSTRETCHFSM_IDLE;
        endcase 

    `DROOPSTRETCHFSM_LF1: 
      if (READY_FOR_DECISION)
        if (PD_LF1_TO_UNCOND_COARSE_INC)
            nextState[2:0] = `DROOPSTRETCHFSM_UNCOND_COARSE_INC;
        else
            nextState[2:0] = `DROOPSTRETCHFSM_LF1;
      else
        nextState[2:0] = `DROOPSTRETCHFSM_LF1;

    `DROOPSTRETCHFSM_UNCOND_COARSE_INC:
      if (READY_FOR_DECISION)
        if (PD_UNCOND_COARSE_INC_TO_FINE_DEC)
          nextState[2:0] = `DROOPSTRETCHFSM_FINE_DEC;
        else
          nextState[2:0] = `DROOPSTRETCHFSM_COARSE_INC;
      else
        nextState[2:0] = `DROOPSTRETCHFSM_UNCOND_COARSE_INC;

    `DROOPSTRETCHFSM_COARSE_INC:
      if (READY_FOR_DECISION)
        if (PD_COARSE_INC_TO_FINE_DEC)
          nextState[2:0] = `DROOPSTRETCHFSM_FINE_DEC;
        else
          nextState[2:0] = `DROOPSTRETCHFSM_COARSE_INC;
      else
        nextState[2:0] = `DROOPSTRETCHFSM_COARSE_INC;

    `DROOPSTRETCHFSM_FINE_DEC:
      if (READY_FOR_DECISION)
        if (PD_FINE_DEC_TO_LOW_BW_UPDATE)
            nextState[2:0] = `DROOPSTRETCHFSM_LOW_BW_UPDATE;
        else
          nextState[2:0] = `DROOPSTRETCHFSM_FINE_DEC;
      else
        nextState[2:0] = `DROOPSTRETCHFSM_FINE_DEC;

    `DROOPSTRETCHFSM_LOW_BW_UPDATE:
      nextState[2:0] = `DROOPSTRETCHFSM_LOW_BW_UPDATE;

    default:
      nextState[2:0] = `DROOPSTRETCHFSM_IDLE;
  endcase



//===== SAMPLE_SIZE =====
assign half_sample_size = SAMPLE_SIZE >> 1;

// synopsys sync_set_reset "RESET, Rst_PD_sum, new1_RESET_28, new1_RESET_40, new1_RESET_0, new1_RESET_1, new1_RESET_2, new1_RESET_3, new1_RESET_4, new1_RESET_5, new1_RESET_6, new1_RESET_7, new1_RESET_8, new1_RESET_9, new1_RESET_10, new1_RESET_11, new1_RESET_12, new1_RESET_13, new1_RESET_14, new1_RESET_15"
 
wire new1_RESET_0, new1_RESET_1, new1_RESET_2, new1_RESET_3, new1_RESET_4, new1_RESET_5, new1_RESET_6, new1_RESET_7;
wire new1_RESET_8, new1_RESET_9, new1_RESET_10, new1_RESET_11, new1_RESET_12, new1_RESET_13, new1_RESET_14, new1_RESET_15;
 
assign new1_RESET_0  = RESET_3q  &  (SAMPLE_SIZE==4'd0);
assign new1_RESET_1  = RESET_3q  &  (SAMPLE_SIZE==4'd1);
assign new1_RESET_2  = RESET_3q  &  (SAMPLE_SIZE==4'd2);
assign new1_RESET_3  = RESET_3q  &  (SAMPLE_SIZE==4'd3);
assign new1_RESET_4  = RESET_3q  &  (SAMPLE_SIZE==4'd4);
assign new1_RESET_5  = RESET_3q  &  (SAMPLE_SIZE==4'd5);
assign new1_RESET_6  = RESET_3q  &  (SAMPLE_SIZE==4'd6);
assign new1_RESET_7  = RESET_3q  &  (SAMPLE_SIZE==4'd7);
assign new1_RESET_8  = RESET_3q  &  (SAMPLE_SIZE==4'd8);
assign new1_RESET_9  = RESET_3q  &  (SAMPLE_SIZE==4'd9);
assign new1_RESET_10 = RESET_3q  &  (SAMPLE_SIZE==4'd10);
assign new1_RESET_11 = RESET_3q  &  (SAMPLE_SIZE==4'd11);
assign new1_RESET_12 = RESET_3q  &  (SAMPLE_SIZE==4'd12);
assign new1_RESET_13 = RESET_3q  &  (SAMPLE_SIZE==4'd13);
assign new1_RESET_14 = RESET_3q  &  (SAMPLE_SIZE==4'd14);
assign new1_RESET_15 = RESET_3q  &  (SAMPLE_SIZE==4'd15);
 
always @(posedge CCLK) begin
  if (new1_RESET_0)       sample_count[3:0] <= 4'd0;
  else if (new1_RESET_1)  sample_count[3:0] <= 4'd1;
  else if (new1_RESET_2)  sample_count[3:0] <= 4'd2;
  else if (new1_RESET_3)  sample_count[3:0] <= 4'd3;
  else if (new1_RESET_4)  sample_count[3:0] <= 4'd4;
  else if (new1_RESET_5)  sample_count[3:0] <= 4'd5;
  else if (new1_RESET_6)  sample_count[3:0] <= 4'd6;
  else if (new1_RESET_7)  sample_count[3:0] <= 4'd7;
  else if (new1_RESET_8)  sample_count[3:0] <= 4'd8;
  else if (new1_RESET_9)  sample_count[3:0] <= 4'd9;
  else if (new1_RESET_10) sample_count[3:0] <= 4'd10;
  else if (new1_RESET_11) sample_count[3:0] <= 4'd11;
  else if (new1_RESET_12) sample_count[3:0] <= 4'd12;
  else if (new1_RESET_13) sample_count[3:0] <= 4'd13;
  else if (new1_RESET_14) sample_count[3:0] <= 4'd14;
  else if (new1_RESET_15) sample_count[3:0] <= 4'd15;
  else if (SAMPLE_SIZE[3:1]==3'b000)
    sample_count <= 0;
  else
    if (sample_count==0 | waitCounter[3:0]!=FSM_WAIT_CYCLES[3:0])
        sample_count <= SAMPLE_SIZE;
    else
        sample_count <= sample_count - 1;
end

//always @(posedge CCLK) begin
//  if (RESET) 
//    sample_count <= SAMPLE_SIZE;
//  else if (SAMPLE_SIZE[3:1]==3'b000)
//    sample_count <= 0;
//  else 
//    if (sample_count==0 | waitCounter[3:0]!=FSM_WAIT_CYCLES[3:0])
//        sample_count <= SAMPLE_SIZE;
//    else
//        sample_count <= sample_count - 1;
//end

wire Rst_PD_sum = RESET_3q | sample_count==0 | waitCounter[3:0]!=FSM_WAIT_CYCLES[3:0];
always @(posedge CCLK) begin
    if (Rst_PD_sum)
        PD_sum <= 0;
    else
        PD_sum <= PD_sum + {1'b0,1'b0,1'b0,PD_d2};
end


//==== FSM Wait Counter =====
always @(*)
  if (waitCounter[3:0]==FSM_WAIT_CYCLES[3:0] && sample_count!=0)
    next_waitCounter[3:0] = waitCounter[3:0];
  else
    if((state[2:0] == `DROOPSTRETCHFSM_LF1) | (state[2:0] == `DROOPSTRETCHFSM_UNCOND_COARSE_INC) | (state[2:0] == `DROOPSTRETCHFSM_COARSE_INC) | (state[2:0] == `DROOPSTRETCHFSM_FINE_DEC))
      next_waitCounter[3:0] = (waitCounter[3:0] == FSM_WAIT_CYCLES[3:0])? 4'b0000 : waitCounter[3:0] + 4'b0001;
    else
      next_waitCounter[3:0] = waitCounter[3:0];


`endif   // MACRO_HEADER_ONLY

endmodule // mbdroopstretchfsm
