//==================================================================================================
//
// mbstretch_sapr_top.v
// 
// Copyright (c) 2002 by Advanced Micro Devices, Inc.
//
// This file is protected by Federal Copyright Law, with all rights
// reserved. No part of this file may be reproduced, stored in a
// retrieval system, translated, transcribed, or transmitted, in any
// form, or by any means manual, electric, electronic, mechanical,
// electro-magnetic, chemical, optical, or otherwise, without prior
// explicit written permission from Advanced Micro Devices, Inc.
//
//==================================================================================================
//
// INTERESTED PARTIES
//
// Circuit              Ju Tung Ng 
// RTL                  Ricky Lau      
// Verification         Sean Sequeira 
//
//==================================================================================================
//
// Module Description
//
// Droop Detect and Stretcher CDC
//
//==================================================================================================

`timescale 1ps/1ps

module mbstretch_sapr_top (

input           VDD, 
input           VSS,

input           dfs_clk,
input           ckout_clkpicker_lf,
input           ckout_clkpicker_mf,
input           ckout_clkpicker_hf,
input           STRETCH_ENABLE,
input           masterReset,
input           LDO_EN,

input           LDO_COMP_HI,
input           LDO_COMP_LO,

input     [3:0] STRETCH_AMOUNT,
input     [3:0] STRETCH_AMOUNT_PCC,
input     [3:0] FSM_WAIT_CYCLES,
input     [3:0] SAMPLE_SIZE,
input     [6:0] LOW_BW_CONFIG,
input     [6:0] LOW_BW_CONFIG_UP,
input     [2:0] LOW_BW_SKEW_EN,

input     [1:0] CONSERVATIVE_LOCK,
input           NO_EXTRA_COARSE_STEP, 
input           skip_phase_bypass,

input           lead_for_lock_lf,
input           lead_for_lock_mf,
input           lead_for_lock_hf,

input           droopdetected,        
input           pcc_detected,
input           ck_en0_lf,
input           ck_en0_mf,
input           ck_en0_hf,

input     [7:0] LDO_READY_COUNT_VAL,
input     [14:0] SyncBusIn,
input     [15:0] RESERVE,
input     [1:0] FREQ_MODE,
input     [1:0] PD_SYNC_SEL,
input           BYPASS_LDO,

input     [3:0] CHARZ_SEL_A,
input     [3:0] CHARZ_SEL_B,
input     [1:0] CHARZ_DIV_A,
input     [1:0] CHARZ_DIV_B,
input     [1:0] FSM_STEP_SIZE,
input     [1:0] FSM_CODE_MARGIN,
input           CHARZ_DEBUG_MODE,
input     [1:0] FSM_DEBUG_SEL,
input           FSM_OVERRIDE_EN,
input     [3:0] FSM_OVERRIDE_VAL,
input           test_clk_ref,
input           test_clk_fb,
input           test_dd_sync,

output          charz_observ_a,
output          charz_observ_b,
output    [1:0] test_clk_sel,

output    [3:0] c0,
output    [3:0] c1,
output    [3:0] c2,
output    [3:0] c3,
output    [3:0] c4,
output    [3:0] c5,
output    [3:0] c6,
output    [3:0] c7,
output    [3:0] c8,
output    [3:0] c9,
output    [3:0] c10,
output    [3:0] c11,
output    [3:0] c12,
output    [3:0] c13,
output    [3:0] c14,
output    [3:0] c15,
output    [3:0] c16,
output    [3:0] c17,
output    [3:0] c18,
output    [3:0] c19,
output    [3:0] c20,
output    [3:0] c21,
output    [3:0] c22,
output    [3:0] c23,
output    [3:0] c24,
output    [3:0] c25,
output    [3:0] c26,
output    [3:0] c27,
output    [3:0] c28,
output    [3:0] c29,
output    [3:0] c30,
output    [3:0] c31,
output    [3:0] c32,
output    [3:0] c33,
output    [3:0] c34,
output    [3:0] c35,
output    [3:0] c36,
output    [3:0] c37,
output    [3:0] c38,
output    [3:0] c39,

output          LDO_COMP_EN,
output          skip_phase,           // 1 toggle per base stretch period
output          stretch_clock,
output          pcc_detected_qual,
output          cks_dll_lock,
output          clkpicker_reset,
output          clkpicker_enable,
output          clkpick_sel_skip2,
output          clkpick_sel_skip4,
output          use_ph28 ,
output          clkpick_sel_skip2_pcc,
output          clkpick_sel_skip4_pcc,
output          use_ph28_pcc,
output reg [3:0] debug_c0,
output reg [3:0] debug_c19,
output reg [3:0] debug_c29,
output reg [3:0] debug_c39,
output          debug_LDO_READY,
output    [14:0] SyncBusOut,


output          dfs_clk_gated_lf,
output          dfs_clk_gated_mf,
output          dfs_clk_gated_hf,

output          sclk
);


reg             reset_1q;
reg             reset_2q;
reg             reset_3q;
reg             reset_4q;
reg             reset_5q;
reg             reset_6q;
wire            reset;
wire            FSM_reset;
wire            cks_dll_lock_int;

wire           cdc_LDO_COMP_HI;
wire           cdc_LDO_COMP_LO;
wire           cdc_masterReset;
wire           cdc_STRETCH_ENABLE;

wire            LDO_READY_final;
reg             LDO_READY_reg;
reg       [7:0] LDO_READY_cnt;
wire            LDO_READY_sticky;

wire      [3:0] STRETCH_AMOUNT_buf;
wire      [3:0] STRETCH_AMOUNT_PCC_buf;
wire      [3:0] FSM_WAIT_CYCLES_buf;
wire      [3:0] SAMPLE_SIZE_buf;
wire            NO_EXTRA_COARSE_STEP_buf; 
wire            USE_FOR_LOW_FREQ_buf;
wire            skip_phase_bypass_buf;
wire      [7:0] LDO_READY_COUNT_VAL_buf;
wire            ck_en0_buf;
wire      [1:0] CONSERVATIVE_LOCK_buf;
wire      [6:0] LOW_BW_CONFIG_buf;
wire      [6:0] LOW_BW_CONFIG_UP_buf;
wire            LOW_BW_SKEW_EN_buf;
wire      [1:0] FREQ_MODE_buf;
wire      [1:0] PD_SYNC_SEL_buf;

wire      [2:0] state;
wire      [6:0] delay_conf_ptr;
wire      [3:0] delay_conf_left;
wire     [15:0] low_bw_ctr;
wire     [15:0] phase_accum;
wire            sel_final;
wire            PD_d0;

//==================================================================
// Output clock mux and Clock Walker
mbstretch_outmux_clkwalk I_outmux_clkwalk (
        .VDD            (VDD),
        .VSS            (VSS),
        .cdc_LDO_EN     (cdc_LDO_EN),
        .dfs_clk        (dfs_clk),
        .ckout_clkpicker (ckout_clkpicker),
        .sel            (cks_dll_lock),
        .RESERVE        (RESERVE[1:0]),
        .SyncBusIn      (SyncBusIn),
        .SyncBusOut     (SyncBusOut),
        .sclk           (sclk),
        .sel_final      (sel_final)
        );

//==================================================================
mbclkds_clkgater I_mbclkds_clkgater_lf (
        .VDD        (VDD),
        .VSS        (VSS),
        .C          (dfs_clk),
        .EN         (cdc_LDO_EN & ~FREQ_MODE_buf[0] & ~FREQ_MODE_buf[1]),  
        .CG         (dfs_clk_gated_lf)
        );


mbclkds_clkgater I_mbclkds_clkgater_mf (
        .VDD        (VDD),
        .VSS        (VSS),
        .C          (dfs_clk),
        .EN         (cdc_LDO_EN & FREQ_MODE_buf[0] & ~FREQ_MODE_buf[1]),  
        .CG         (dfs_clk_gated_mf)
        );

mbclkds_clkgater I_mbclkds_clkgater_hf (
        .VDD        (VDD),
        .VSS        (VSS),
        .C          (dfs_clk),
        .EN         (cdc_LDO_EN & FREQ_MODE_buf[1]),  
        .CG         (dfs_clk_gated_hf)
        );

mbclkds_clkgater I_mbclkds_clkgater_fsm(
        .VDD        (VDD),
        .VSS        (VSS),
        .C          (dfs_clk),
        .EN         (cdc_LDO_EN),  
        .CG         (dfs_clk_gated)
        );

//==================================================================
mbstretch_cdc I_mbstretch_cdc (
        .VDD (VDD),
        .VSS (VSS),
        .dfs_clk(dfs_clk),
        .STRETCH_ENABLE(STRETCH_ENABLE),
        .masterReset(masterReset),
        .LDO_COMP_HI(LDO_COMP_HI),
        .LDO_COMP_LO(LDO_COMP_LO),
        .LDO_EN(LDO_EN),
        .FSM_OVERRIDE_EN(FSM_OVERRIDE_EN),
        .cdc_STRETCH_ENABLE(cdc_STRETCH_ENABLE),
        .cdc_masterReset(cdc_masterReset),
        .cdc_LDO_COMP_HI(cdc_LDO_COMP_HI),
        .cdc_LDO_COMP_LO(cdc_LDO_COMP_LO),
        .cdc_LDO_EN(cdc_LDO_EN),
        .cdc_FSM_OVERRIDE_EN(cdc_FSM_OVERRIDE_EN)
        );

//==================================================================

mbdroop_droop_dll_control I_CS_droop_dll_control (
        .VDD                (VDD),
        .VSS                (VSS),

        .dfs_clk             (dfs_clk_gated),
        .RESET               (FSM_reset),
        .DLL_LEAD            ({10'd0,lead_for_lock,7'd0}), // phase detector output, only the msb is used in the stretcher 
        .STRETCH_THRESH      (3'b001),         // can be any number - the outcome is not used in the stretcher
        .UNSTRETCH_THRESH    (5'b00111),           //BOZO : need to be CDC buffed
        .CONSERVATIVE_LOCK   (CONSERVATIVE_LOCK_buf),
        .DFSSTRETCH          (1'b0),
        .DFSSTRETCH_AMOUNT   (4'b0000),          //BOZO : need to be cdc buffed
        // From DLL chanin
        // From FSM
        .dll_lock  (cks_dll_lock_int),
      
        .PD_d0          (PD_d0),
        .droopdetected () // not used in the stretcher
        );


//====================================================================
mbdroopstretchfsm I_FSM (
        .VDD (VDD),
        .VSS (VSS),
        
        .CCLK (dfs_clk_gated),                
        .RESET (FSM_reset),                   // Stretcher enable
        .LOW_BW_CONFIG (LOW_BW_CONFIG),         
        .LOW_BW_CONFIG_UP (LOW_BW_CONFIG_UP_buf),           // has to be 3'b001 in the stretcher
        .LOW_BW_SKEW_EN (LOW_BW_SKEW_EN_buf), 
        .FSM_WAIT_CYCLES(FSM_WAIT_CYCLES_buf),
        .NO_EXTRA_COARSE_STEP (NO_EXTRA_COARSE_STEP_buf), 
        .SAMPLE_SIZE(SAMPLE_SIZE_buf),
        .PD_d0       (PD_d0),
        .PD_SYNC_SEL (PD_SYNC_SEL_buf),
        .OVERRIDE_DLY_EN (cdc_FSM_OVERRIDE_EN),
        .OVERRIDE_DLY_VAL (FSM_OVERRIDE_VAL[3:0]),
        .use_ph28(use_ph28),
        .FSM_STEP_SIZE (FSM_STEP_SIZE),
        .FSM_CODE_MARGIN (FSM_CODE_MARGIN),
        
        .c_0 (c0[3:0]),
        .c_1 (c1[3:0]),
        .c_2 (c2[3:0]),
        .c_3 (c3[3:0]),
        .c_4 (c4[3:0]),
        .c_5 (c5[3:0]),
        .c_6 (c6[3:0]),
        .c_7 (c7[3:0]),
        .c_8 (c8[3:0]),
        .c_9 (c9[3:0]),
        .c_10 (c10[3:0]),
        .c_11 (c11[3:0]),
        .c_12 (c12[3:0]),
        .c_13 (c13[3:0]),
        .c_14 (c14[3:0]),
        .c_15 (c15[3:0]),
        .c_16 (c16[3:0]),
        .c_17 (c17[3:0]),
        .c_18 (c18[3:0]),
        .c_19 (c19[3:0]),
        .c_20 (c20[3:0]),
        .c_21 (c21[3:0]),
        .c_22 (c22[3:0]),
        .c_23 (c23[3:0]),
        .c_24 (c24[3:0]),
        .c_25 (c25[3:0]),
        .c_26 (c26[3:0]),
        .c_27 (c27[3:0]),
        .c_28 (c28[3:0]),
        .c_29 (c29[3:0]),
        .c_30 (c30[3:0]),
        .c_31 (c31[3:0]),
        .c_32 (c32[3:0]),
        .c_33 (c33[3:0]),
        .c_34 (c34[3:0]),
        .c_35 (c35[3:0]),
        .c_36 (c36[3:0]),
        .c_37 (c37[3:0]),
        .c_38 (c38[3:0]),
        .c_39 (c39[3:0]),

        .dll_lock (cks_dll_lock_int),         // phase-gen dll lock indicator
        .state (state),
        .delay_conf_ptr (delay_conf_ptr),
        .delay_conf_left (delay_conf_left),
        .low_bw_ctr (low_bw_ctr),
        .phase_accum (phase_accum)
        );


//==================================================================
// Top level logic

assign stretch_clock = (skip_phase_bypass_buf | droopdetected) & cks_dll_lock_int & |STRETCH_AMOUNT_buf;
assign pcc_detected_qual = (pcc_detected)& cks_dll_lock_int & | STRETCH_AMOUNT_PCC_buf;
//*** Reset logic ***//
always @(posedge dfs_clk) begin
    reset_1q <= reset;
    reset_2q <= reset_1q;
    reset_3q <= reset_2q;
end

reg LDO_READY_sticky_q;
reg LDO_READY_final_q;
always @(posedge dfs_clk) begin
    LDO_READY_sticky_q <= LDO_READY_sticky;
    LDO_READY_final_q  <= LDO_READY_final;
end

// ~LDO_READY_sticky is needed to hold FSM reset=1 during the time when staticEnable asserts and VDDREG is still ramping to 1'b1 during power-up. Else FSM reset will de-assert once cdc_LDO_EN=1 until the after VDDREG=1 during power-up.
assign reset = cdc_masterReset | ~((cdc_STRETCH_ENABLE & LDO_READY_final_q) | ~ck_en0_buf)  | ~cdc_LDO_EN | ~LDO_READY_sticky_q; 

// delay assertion of FSM reset by 3 cycles
assign FSM_reset = reset & reset_3q; 

// Lock indicator
assign cks_dll_lock = cks_dll_lock_int & ~(reset & reset_3q) & ~cdc_FSM_OVERRIDE_EN; 


// Clock picker reset and enable
assign clkpicker_reset  = cdc_masterReset | ~LDO_READY_sticky | ~cks_dll_lock_int;
assign clkpicker_enable = cdc_STRETCH_ENABLE & LDO_READY_final_q;


// skip_phase generation
// BOZO: The schematics have masterReset (and not ~reset_x) in there.
// Replaced with a ASYNCMSFLOP
// BOZO: Check why this is negedge of clock
// always @(negedge clk_en[0] or negedge ~reset_x)
//   if(~~reset_x)
//     skip_phase <= 1'b0;
//   else
//     skip_phase <= ~skip_phase;

ASYNCMSFLOP I_skip_phase (.Q(skip_phase),
                          .D(~skip_phase),
                          .C(~ck_en0),
                          .E(1'b1),
                          .R(reset));


reg [2:0] stretch_amount_case;
assign {use_ph28, clkpick_sel_skip2, clkpick_sel_skip4} = stretch_amount_case[2:0];

always @(*) begin
        case (STRETCH_AMOUNT_buf[3:0])
          4'b0000 : stretch_amount_case = 3'b000;      // 0%
          4'b0001 : stretch_amount_case = 3'b000;      // 5%
          4'b0010 : stretch_amount_case = 3'b100;      // 7.14%
          4'b0011 : stretch_amount_case = 3'b010;      // 10%
          4'b0100 : stretch_amount_case = 3'b110;      // 14.28%
          4'b0101 : stretch_amount_case = 3'b001;      // 20%
          default : stretch_amount_case = 3'b000;
        endcase
end

reg [2:0] stretch_amount_pcc_case;
assign {use_ph28_pcc, clkpick_sel_skip2_pcc, clkpick_sel_skip4_pcc} = stretch_amount_pcc_case[2:0];

always @(*) begin
        case (STRETCH_AMOUNT_PCC_buf[3:0])
          4'b0000 : stretch_amount_pcc_case = 3'b000;      // 0%
          4'b0001 : stretch_amount_pcc_case = 3'b000;      // 5%
          4'b0010 : stretch_amount_pcc_case = 3'b100;      // 7.14%
          4'b0011 : stretch_amount_pcc_case = 3'b010;      // 10%
          4'b0100 : stretch_amount_pcc_case = 3'b110;      // 14.28%
          4'b0101 : stretch_amount_pcc_case = 3'b001;      // 20%
          default : stretch_amount_pcc_case = 3'b000;
        endcase
end

//======================================
// LDO Logic
wire LDO_READY_zero_cnt;
reg cdc_LDO_EN_1q;
reg cdc_LDO_EN_2q;

// for the case LDO_READY_COUNT_VAL=0, cdc_LDO_EN_2q is to give clock picker 
// at least 1 clock edge to reset before LDO_READY is asserted.  
// cks_dll_lock_int cannot reset the clock picker because it is X until 
// FSM resets, but FSM will not reset if ck_en0=X.  Thus it will be stuck at X.
always @(posedge dfs_clk) begin
    cdc_LDO_EN_1q <= cdc_LDO_EN;
    cdc_LDO_EN_2q <= cdc_LDO_EN_1q;
end

assign LDO_COMP_EN = ~cks_dll_lock_int & cdc_LDO_EN & cdc_STRETCH_ENABLE;

wire LDO_in_range = BYPASS_LDO ? 1'b1 : ~cdc_LDO_COMP_HI & cdc_LDO_COMP_LO;


always @(posedge dfs_clk) begin
  if (~cdc_LDO_EN_2q | ~cdc_STRETCH_ENABLE)
      LDO_READY_cnt <= LDO_READY_COUNT_VAL_buf;
  else
    if (LDO_READY_cnt!=8'd0)
      LDO_READY_cnt <= LDO_in_range ? LDO_READY_cnt - 8'b1 : LDO_READY_COUNT_VAL_buf;
    else
      LDO_READY_cnt <= LDO_READY_cnt;
end

assign LDO_READY_final = (LDO_READY_COUNT_VAL_buf == 8'd0) ? LDO_READY_zero_cnt : 
                                                            (LDO_READY_cnt==8'd0 & LDO_READY_zero_cnt);

mbclkds_srlatch I_LDO_READY_sticky_zero_cnt (.S(LDO_in_range), .R(~(cdc_LDO_EN_2q & cdc_STRETCH_ENABLE)), .Q(LDO_READY_zero_cnt));
mbclkds_srlatch I_LDO_READY_sticky_latch (.S(LDO_READY_final), .R(~cdc_LDO_EN_2q), .Q(LDO_READY_sticky));

assign debug_LDO_READY = LDO_READY_final_q;


//======================================
// FSM_DEBUG_SEL[1:0] to choose signals
// to output on debug_c*[3:0]
//======================================
wire [3:0] debug_00_c0;
wire [3:0] debug_00_c19;
wire [3:0] debug_00_c29;
wire [3:0] debug_00_c39;
wire [3:0] debug_01_c0;
wire [3:0] debug_01_c19;
wire [3:0] debug_01_c29;
wire [3:0] debug_01_c39;
wire [3:0] debug_10_c0;
wire [3:0] debug_10_c19;
wire [3:0] debug_10_c29;
wire [3:0] debug_10_c39;
wire [3:0] debug_11_c0;
wire [3:0] debug_11_c19;
wire [3:0] debug_11_c29;
wire [3:0] debug_11_c39;

assign debug_00_c0  = c0;
assign debug_00_c19 = use_ph28 ? c13 :c19;
assign debug_00_c29 = use_ph28 ? c20 :c29;
assign debug_00_c39 = use_ph28 ? c27 :c39;

assign debug_01_c0  = {1'b0,state[2:0]};
assign debug_01_c19 = {1'b0,delay_conf_ptr[6:4]};
assign debug_01_c29 = delay_conf_ptr[3:0];
assign debug_01_c39 = delay_conf_left[3:0];

assign debug_10_c0  = low_bw_ctr[15:12];
assign debug_10_c19 = low_bw_ctr[11:8];
assign debug_10_c29 = low_bw_ctr[7:4]; 
assign debug_10_c39 = low_bw_ctr[3:0];

assign debug_11_c0  = phase_accum[15:12];
assign debug_11_c19 = phase_accum[11:8];
assign debug_11_c29 = phase_accum[7:4]; 
assign debug_11_c39 = phase_accum[3:0];

always @(*) begin
  case (FSM_DEBUG_SEL[1:0])
    2'b00 : begin 
            debug_c0  = debug_00_c0;
            debug_c19 = debug_00_c19;
            debug_c29 = debug_00_c29;
            debug_c39 = debug_00_c39;
            end
    2'b01 : begin
            debug_c0  = debug_01_c0;
            debug_c19 = debug_01_c19;
            debug_c29 = debug_01_c29;
            debug_c39 = debug_01_c39;
            end
    2'b10 : begin
            debug_c0  = debug_10_c0;
            debug_c19 = debug_10_c19;
            debug_c29 = debug_10_c29;
            debug_c39 = debug_10_c39;
            end
    2'b11 : begin
            debug_c0  = debug_11_c0;
            debug_c19 = debug_11_c19;
            debug_c29 = debug_11_c29;
            debug_c39 = debug_11_c39;
            end
  endcase
end


//======================================
// Muxes to select between LF, MF, HF
//======================================
mbstretch_clkmux_cpu I_mux_ckout_clkpicker_lf_mf (
    .VDD(VDD),
    .VSS(VSS),
    .IN0(ckout_clkpicker_lf),
    .IN1(ckout_clkpicker_mf),
    .SEL(FREQ_MODE_buf[0]),
    .OUT(ckout_clkpicker_lf_or_mf)
);

mbstretch_clkmux_cpu I_mux_ckout_clkpicker_hf (
    .VDD(VDD),
    .VSS(VSS),
    .IN0(ckout_clkpicker_lf_or_mf),
    .IN1(ckout_clkpicker_hf),
    .SEL(FREQ_MODE_buf[1]),
    .OUT(ckout_clkpicker)
);

mbstretch_clkmux_cpu I_mux_ck_en0_lf_mf (
    .VDD(VDD),
    .VSS(VSS),
    .IN0(ck_en0_lf),
    .IN1(ck_en0_mf),
    .SEL(FREQ_MODE_buf[0]),
    .OUT(ck_en0_lf_or_mf)
);

mbstretch_clkmux_cpu I_mux_ck_en0_hf (
    .VDD(VDD),
    .VSS(VSS),
    .IN0(ck_en0_lf_or_mf),
    .IN1(ck_en0_hf),
    .SEL(FREQ_MODE_buf[1]),
    .OUT(ck_en0)
);

mbstretch_clkmux_cpu I_mux_lead_for_lock_lf_mf (
    .VDD(VDD),
    .VSS(VSS),
    .IN0(lead_for_lock_lf),
    .IN1(lead_for_lock_mf),
    .SEL(FREQ_MODE_buf[0]),
    .OUT(lead_for_lock_lf_or_mf)
);

mbstretch_clkmux_cpu I_mux_lead_for_lock_hf (
    .VDD(VDD),
    .VSS(VSS),
    .IN0(lead_for_lock_lf_or_mf),
    .IN1(lead_for_lock_hf),
    .SEL(FREQ_MODE_buf[1]),
    .OUT(lead_for_lock)
);

//======================================
// charz_observ_a/b
//======================================
mbclkds_charz_observ I_charz_observ (
    .VDD                (VDD),
    .VSS                (VSS),
    .CHARZ_SEL_A        (CHARZ_SEL_A),
    .CHARZ_SEL_B        (CHARZ_SEL_B),
    .CHARZ_DIV_A        (CHARZ_DIV_A),
    .CHARZ_DIV_B        (CHARZ_DIV_B),
    .ASYNC_RESETB       (cdc_LDO_EN),
    .IN_0               (FSM_reset),
    .IN_1               (test_dd_sync),
    .IN_2               (cks_dll_lock),
    .IN_3               (sclk),
    .IN_4               (LDO_EN),
    .IN_5               (test_clk_ref),
    .IN_6               (LDO_READY_final_q),
    .IN_7               (skip_phase),
    .IN_8               (sel_final),
    .IN_9               (LDO_COMP_HI),
    .IN_10              (test_clk_fb),
    .IN_11              (LDO_COMP_LO),
    .IN_12              (lead_for_lock),
    .IN_13              (clkpicker_enable),
    .IN_14              (clkpicker_reset),
    .IN_15              (dfs_clk),
    .charz_observ_a     (charz_observ_a),
    .charz_observ_b     (charz_observ_b) 
);


assign test_clk_sel = FREQ_MODE[1:0];

//======================================
// CDC Buffers
// conbine all cdc buf instances into a contaioner module
mbstretch_sapr_top_cdc_container I_mbstretch_sapr_top_cdc_container ( 
`ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
.dfs_clk(dfs_clk ),
.dfs_clk_gated(dfs_clk_gated ),
`endif

.ck_en0( ck_en0),
.STRETCH_AMOUNT(STRETCH_AMOUNT ),
.STRETCH_AMOUNT_PCC(STRETCH_AMOUNT_PCC ),
.FSM_WAIT_CYCLES(FSM_WAIT_CYCLES ),
.SAMPLE_SIZE(SAMPLE_SIZE ),
.NO_EXTRA_COARSE_STEP(NO_EXTRA_COARSE_STEP ),
.skip_phase_bypass(skip_phase_bypass ),
.LDO_READY_COUNT_VAL(LDO_READY_COUNT_VAL ),
.CONSERVATIVE_LOCK(CONSERVATIVE_LOCK ),
.LOW_BW_CONFIG(LOW_BW_CONFIG ),
.LOW_BW_CONFIG_UP(LOW_BW_CONFIG_UP ),
.LOW_BW_SKEW_EN(LOW_BW_SKEW_EN ),
.PD_SYNC_SEL(PD_SYNC_SEL ),
.FREQ_MODE(FREQ_MODE ),

.ck_en0_buf(ck_en0_buf ),
.STRETCH_AMOUNT_buf(STRETCH_AMOUNT_buf ),
.STRETCH_AMOUNT_PCC_buf(STRETCH_AMOUNT_PCC_buf ),
.FSM_WAIT_CYCLES_buf(FSM_WAIT_CYCLES_buf ),
.SAMPLE_SIZE_buf(SAMPLE_SIZE_buf ),
.NO_EXTRA_COARSE_STEP_buf(NO_EXTRA_COARSE_STEP_buf ),
.skip_phase_bypass_buf(skip_phase_bypass_buf ),
.LDO_READY_COUNT_VAL_buf(LDO_READY_COUNT_VAL_buf ),
.CONSERVATIVE_LOCK_buf(CONSERVATIVE_LOCK_buf ),
.LOW_BW_CONFIG_buf(LOW_BW_CONFIG_buf ),
.LOW_BW_CONFIG_UP_buf(LOW_BW_CONFIG_UP_buf ),
.LOW_BW_SKEW_EN_buf(LOW_BW_SKEW_EN_buf ),
.PD_SYNC_SEL_buf(PD_SYNC_SEL_buf ),
.FREQ_MODE_buf(FREQ_MODE_buf )

 );

// -- Top level logic
//==================================================================



endmodule
