`timescale 1ps/1ps

module mbstretch_sapr_top_cdc_container (

`ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
input dfs_clk,
input dfs_clk_gated,
`endif

input        ck_en0,
input  [3:0] STRETCH_AMOUNT,
input  [3:0] STRETCH_AMOUNT_PCC,
input  [3:0] FSM_WAIT_CYCLES,
input  [3:0] SAMPLE_SIZE,
input        NO_EXTRA_COARSE_STEP,
input        skip_phase_bypass,
input  [7:0] LDO_READY_COUNT_VAL,
input  [1:0] CONSERVATIVE_LOCK,
input  [6:0] LOW_BW_CONFIG,
input  [6:0] LOW_BW_CONFIG_UP,
input  [2:0] LOW_BW_SKEW_EN,
input  [1:0] PD_SYNC_SEL,
input  [1:0] FREQ_MODE,

output       ck_en0_buf,
output [3:0] STRETCH_AMOUNT_buf,
output [3:0] STRETCH_AMOUNT_PCC_buf,
output [3:0] FSM_WAIT_CYCLES_buf,
output [3:0] SAMPLE_SIZE_buf,
output       NO_EXTRA_COARSE_STEP_buf,
output       skip_phase_bypass_buf,
output [7:0] LDO_READY_COUNT_VAL_buf,
output [1:0] CONSERVATIVE_LOCK_buf,
output [6:0] LOW_BW_CONFIG_buf,
output [6:0] LOW_BW_CONFIG_UP_buf,
output       LOW_BW_SKEW_EN_buf,
output [1:0] PD_SYNC_SEL_buf,
output [1:0] FREQ_MODE_buf

);

//CDCBUFCLK instantiation to prove that ck_en0 path doesn't need any synchronization
//CDCBUFCLK injects X on ck_en0_buf for 2 dfs_clk clock cycles whenever ck_en0 toggles
 
//~((cdc_STRETCH_ENABLE & LDO_READY_final) | ~ck_en0_buf)
//When cdc_STRETCH_ENABLE=1 and LDO_READY_final=1, we don't care for ck_en0_buf (because of "OR"ing)
//We care for ck_en0_buf only when cdc_STRETCH_ENABLE de-asserts (=0)
//CDCBUFCLK #(.Setup(0),.Cycles(2)) I_CDCBUF1 (
CDCBUFCLK #(.Setup(0),.Cycles(0)) I_CDCBUF1 (
           `ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
           .DSTCLK(dfs_clk),
           `endif
           .A(ck_en0),
           .Z(ck_en0_buf) );
 
mbclkds_CDCBUF I_STRETCH_AMOUNT_0 (
           `ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
           .DSTCLK(dfs_clk_gated),
           `endif
           .A(STRETCH_AMOUNT[0]), 
           .Z(STRETCH_AMOUNT_buf[0]) );
mbclkds_CDCBUF I_STRETCH_AMOUNT_1 (
           `ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
           .DSTCLK(dfs_clk_gated),
           `endif
           .A(STRETCH_AMOUNT[1]), 
           .Z(STRETCH_AMOUNT_buf[1]) );
mbclkds_CDCBUF I_STRETCH_AMOUNT_2 (
           `ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
           .DSTCLK(dfs_clk_gated),
           `endif
           .A(STRETCH_AMOUNT[2]), 
           .Z(STRETCH_AMOUNT_buf[2]) );
mbclkds_CDCBUF I_STRETCH_AMOUNT_3 (
           `ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
           .DSTCLK(dfs_clk_gated),
           `endif
           .A(STRETCH_AMOUNT[3]), 
           .Z(STRETCH_AMOUNT_buf[3]) );
mbclkds_CDCBUF I_STRETCH_AMOUNT_PCC_0 (
           `ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
           .DSTCLK(dfs_clk_gated),
           `endif
           .A(STRETCH_AMOUNT_PCC[0]), 
           .Z(STRETCH_AMOUNT_PCC_buf[0]) );
mbclkds_CDCBUF I_STRETCH_AMOUNT_PCC_1 (
           `ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
           .DSTCLK(dfs_clk_gated),
           `endif
           .A(STRETCH_AMOUNT_PCC[1]), 
           .Z(STRETCH_AMOUNT_PCC_buf[1]) );
mbclkds_CDCBUF I_STRETCH_AMOUNT_PCC_2 (
           `ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
           .DSTCLK(dfs_clk_gated),
           `endif
           .A(STRETCH_AMOUNT_PCC[2]), 
           .Z(STRETCH_AMOUNT_PCC_buf[2]) );
mbclkds_CDCBUF I_STRETCH_AMOUNT_PCC_3 (
           `ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
           .DSTCLK(dfs_clk_gated),
           `endif
           .A(STRETCH_AMOUNT_PCC[3]), 
           .Z(STRETCH_AMOUNT_PCC_buf[3]) );
mbclkds_CDCBUF I_FSM_WAIT_CYCLES_0 (
           `ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
           .DSTCLK(dfs_clk_gated),
           `endif
           .A(FSM_WAIT_CYCLES[0]), 
           .Z(FSM_WAIT_CYCLES_buf[0]) );
mbclkds_CDCBUF I_FSM_WAIT_CYCLES_1 (
           `ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
           .DSTCLK(dfs_clk_gated),
           `endif
           .A(FSM_WAIT_CYCLES[1]), 
           .Z(FSM_WAIT_CYCLES_buf[1]) );
mbclkds_CDCBUF I_FSM_WAIT_CYCLES_2 (
           `ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
           .DSTCLK(dfs_clk_gated),
           `endif
           .A(FSM_WAIT_CYCLES[2]), 
           .Z(FSM_WAIT_CYCLES_buf[2]) );
mbclkds_CDCBUF I_FSM_WAIT_CYCLES_3 (
           `ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
           .DSTCLK(dfs_clk_gated),
           `endif
           .A(FSM_WAIT_CYCLES[3]), 
           .Z(FSM_WAIT_CYCLES_buf[3]) );
mbclkds_CDCBUF I_SAMPLE_SIZE_0 (
           `ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
           .DSTCLK(dfs_clk_gated),
           `endif
           .A(SAMPLE_SIZE[0]), 
           .Z(SAMPLE_SIZE_buf[0]) );
mbclkds_CDCBUF I_SAMPLE_SIZE_1 (
           `ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
           .DSTCLK(dfs_clk_gated),
           `endif
           .A(SAMPLE_SIZE[1]), 
           .Z(SAMPLE_SIZE_buf[1]) );
mbclkds_CDCBUF I_SAMPLE_SIZE_2 (
           `ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
           .DSTCLK(dfs_clk_gated),
           `endif
           .A(SAMPLE_SIZE[2]), 
           .Z(SAMPLE_SIZE_buf[2]) );
mbclkds_CDCBUF I_SAMPLE_SIZE_3 (
           `ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
           .DSTCLK(dfs_clk_gated),
           `endif
           .A(SAMPLE_SIZE[3]), 
           .Z(SAMPLE_SIZE_buf[3]) );
mbclkds_CDCBUF I_NO_EXTRA_COARSE_STEP (
           `ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
           .DSTCLK(dfs_clk_gated),
           `endif
           .A(NO_EXTRA_COARSE_STEP), 
           .Z(NO_EXTRA_COARSE_STEP_buf) );
mbclkds_CDCBUF I_skip_phase_bypass (
           `ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
           .DSTCLK(dfs_clk_gated),
           `endif
           .A(skip_phase_bypass), 
           .Z(skip_phase_bypass_buf) );
mbclkds_CDCBUF I_LDO_READY_COUNT_VAL_0 (
           `ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
           .DSTCLK(dfs_clk_gated),
           `endif
           .A(LDO_READY_COUNT_VAL[0]), 
           .Z(LDO_READY_COUNT_VAL_buf[0]) );
mbclkds_CDCBUF I_LDO_READY_COUNT_VAL_1 (
           `ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
           .DSTCLK(dfs_clk_gated),
           `endif
           .A(LDO_READY_COUNT_VAL[1]), 
           .Z(LDO_READY_COUNT_VAL_buf[1]) );
mbclkds_CDCBUF I_LDO_READY_COUNT_VAL_2 (
           `ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
           .DSTCLK(dfs_clk_gated),
           `endif
           .A(LDO_READY_COUNT_VAL[2]), 
           .Z(LDO_READY_COUNT_VAL_buf[2]) );
mbclkds_CDCBUF I_LDO_READY_COUNT_VAL_3 (
           `ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
           .DSTCLK(dfs_clk_gated),
           `endif
           .A(LDO_READY_COUNT_VAL[3]), 
           .Z(LDO_READY_COUNT_VAL_buf[3]) );
mbclkds_CDCBUF I_LDO_READY_COUNT_VAL_4 (
           `ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
           .DSTCLK(dfs_clk_gated),
           `endif
           .A(LDO_READY_COUNT_VAL[4]), 
           .Z(LDO_READY_COUNT_VAL_buf[4]) );
mbclkds_CDCBUF I_LDO_READY_COUNT_VAL_5 (
           `ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
           .DSTCLK(dfs_clk_gated),
           `endif
           .A(LDO_READY_COUNT_VAL[5]), 
           .Z(LDO_READY_COUNT_VAL_buf[5]) );
mbclkds_CDCBUF I_LDO_READY_COUNT_VAL_6 (
           `ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
           .DSTCLK(dfs_clk_gated),
           `endif
           .A(LDO_READY_COUNT_VAL[6]), 
           .Z(LDO_READY_COUNT_VAL_buf[6]) );
mbclkds_CDCBUF I_LDO_READY_COUNT_VAL_7 (
           `ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
           .DSTCLK(dfs_clk_gated),
           `endif
           .A(LDO_READY_COUNT_VAL[7]), 
           .Z(LDO_READY_COUNT_VAL_buf[7]) );
mbclkds_CDCBUF I_CONSERVATIVE_LOCK_1 (
           `ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
           .DSTCLK(dfs_clk_gated),
           `endif
           .A(CONSERVATIVE_LOCK[1]), 
           .Z(CONSERVATIVE_LOCK_buf[1]) );
mbclkds_CDCBUF I_CONSERVATIVE_LOCK_0 (
           `ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
           .DSTCLK(dfs_clk_gated),
           `endif
           .A(CONSERVATIVE_LOCK[0]), 
           .Z(CONSERVATIVE_LOCK_buf[0]) );
mbclkds_CDCBUF I_LOW_BW_CONFIG_6 (
           `ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
           .DSTCLK(dfs_clk_gated),
           `endif
           .A(LOW_BW_CONFIG[6]), 
           .Z(LOW_BW_CONFIG_buf[6]) );
mbclkds_CDCBUF I_LOW_BW_CONFIG_5 (
           `ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
           .DSTCLK(dfs_clk_gated),
           `endif
           .A(LOW_BW_CONFIG[5]), 
           .Z(LOW_BW_CONFIG_buf[5]) );
mbclkds_CDCBUF I_LOW_BW_CONFIG_4 (
           `ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
           .DSTCLK(dfs_clk_gated),
           `endif
           .A(LOW_BW_CONFIG[4]), 
           .Z(LOW_BW_CONFIG_buf[4]) );
mbclkds_CDCBUF I_LOW_BW_CONFIG_3 (
           `ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
           .DSTCLK(dfs_clk_gated),
           `endif
           .A(LOW_BW_CONFIG[3]), 
           .Z(LOW_BW_CONFIG_buf[3]) );
mbclkds_CDCBUF I_LOW_BW_CONFIG_2 (
           `ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
           .DSTCLK(dfs_clk_gated),
           `endif
           .A(LOW_BW_CONFIG[2]), 
           .Z(LOW_BW_CONFIG_buf[2]) );
mbclkds_CDCBUF I_LOW_BW_CONFIG_1 (
           `ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
           .DSTCLK(dfs_clk_gated),
           `endif
           .A(LOW_BW_CONFIG[1]), 
           .Z(LOW_BW_CONFIG_buf[1]) );
mbclkds_CDCBUF I_LOW_BW_CONFIG_0 (
           `ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
           .DSTCLK(dfs_clk_gated),
           `endif
           .A(LOW_BW_CONFIG[0]), 
           .Z(LOW_BW_CONFIG_buf[0]) );

mbclkds_CDCBUF I_LOW_BW_CONFIG_UP[6:0] (
           `ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
           .DSTCLK(dfs_clk_gated),
           `endif
           .A(LOW_BW_CONFIG_UP[6:0]), 
           .Z(LOW_BW_CONFIG_UP_buf[6:0]) );

mbclkds_CDCBUF I_LOW_BW_SKEW_EN (
           `ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
           .DSTCLK(dfs_clk_gated),
           `endif
           .A(LOW_BW_SKEW_EN[0]), 
           .Z(LOW_BW_SKEW_EN_buf) );


                   
mbclkds_CDCBUF I_PD_SYNC_SEL_1 (
           `ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
           .DSTCLK(dfs_clk_gated),
           `endif
           .A(PD_SYNC_SEL[1]), 
           .Z(PD_SYNC_SEL_buf[1]) );
mbclkds_CDCBUF I_PD_SYNC_SEL_0 (
           `ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
           .DSTCLK(dfs_clk_gated),
           `endif
           .A(PD_SYNC_SEL[0]), 
           .Z(PD_SYNC_SEL_buf[0]) );
mbclkds_CDCBUF I_FREQ_MODE_1 (
           `ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
           .DSTCLK(dfs_clk_gated),
           `endif
           .A(FREQ_MODE[1]), 
           .Z(FREQ_MODE_buf[1]) );
mbclkds_CDCBUF I_FREQ_MODE_0 (
           `ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
           .DSTCLK(dfs_clk_gated),
           `endif
           .A(FREQ_MODE[0]), 
           .Z(FREQ_MODE_buf[0]) );


endmodule
