//                              -*- Mode: Verilog -*-

////////////////////////////////////////////////////////////////////////////
/*
 *
 * $Id: //sysip/dfx/dfx-4/4.1/branches/dfxip4_gfxip_gfx10/srf/src/rtl/misr_24bit.v.dpl#2 $
 * $Revision: #2 $
 * $DateTime: 2017/06/27 21:37:00 $
 * $Author: couyang $
 *
 * Copyright (c) 2014 by Advanced Micro Devices, Inc.
 *
 * This file is protected by Federal Copyright Law, with all rights
 * reserved. No part of this file may be reproduced, stored in a
 * retrieval system, translated, transcribed, or transmitted, in any
 * form, or by any means manual, electric, electronic, mechanical,
 * electro-magnetic, chemical, optical, or otherwise, without prior
 * explicit written permission from Advanced Micro Devices, Inc.
 *
 */
////////////////////////////////////////////////////////////////////////////
//
// Description : 24bits MISR
//
////////////////////////////////////////////////////////////////////////////

module misr_24bit_gc10 (
        clock,
        reset,
        misr_signature,
        tst_edt_channel_out);

// Clock port
input clock;            // MISR clock

input reset;           //Initizlize misr in lbist mode 

input  [22:0]  tst_edt_channel_out; //edt channel output from tile

output [23:0]  misr_signature;   //Misr signature to tstxx_p1500 for tst capture

reg feedback;
reg [23:0] misr_reg, next_misr_reg;


//=================================================================================================
//
// GPU MISR for LBIST mode
//
//=================================================================================================

integer i;

//Misr function
always @(negedge reset or posedge clock) begin
  if (!reset)
    misr_reg[23:0] <= 24'h000000;	// initial seed value
  else
    misr_reg[23:0] <= next_misr_reg[23:0];
end

always @(misr_reg or tst_edt_channel_out) begin
  feedback = misr_reg[23] ^ misr_reg[22] ^ misr_reg[21] ^ misr_reg[16];
  for (i=23;i>=1;i=i-1)
    next_misr_reg[i] = misr_reg[i-1] ^ tst_edt_channel_out[i-1];
  next_misr_reg[0] = feedback;
  end

assign misr_signature = misr_reg;

endmodule

