// Accellera Standard V2.8.1 Open Verification Library (OVL).
// Accellera Copyright (c) 2005-2014. All rights reserved.

`include "std_ovl_defines.h"

`module ovl_never (clock, reset, enable, test_expr, fire);

  parameter severity_level = `OVL_SEVERITY_DEFAULT;
  parameter property_type  = `OVL_PROPERTY_DEFAULT;
  parameter msg            = `OVL_MSG_DEFAULT;
  parameter coverage_level = `OVL_COVER_DEFAULT;

  parameter clock_edge     = `OVL_CLOCK_EDGE_DEFAULT;
  parameter reset_polarity = `OVL_RESET_POLARITY_DEFAULT;
  parameter gating_type    = `OVL_GATING_TYPE_DEFAULT;

  input                          clock, reset, enable;
  input                          test_expr;
  output [`OVL_FIRE_WIDTH-1:0]   fire;

  // Parameters that should not be edited
  parameter assert_name = "OVL_NEVER";

  `include "std_ovl_reset.h"
  `include "std_ovl_clock.h"
  `include "std_ovl_cover.h"
  `include "std_ovl_task.h"
  `include "std_ovl_init.h"

`ifdef OVL_VERILOG
  `include "./vlog95/ovl_never_logic.v"
`endif

`ifdef OVL_SVA
`ifdef OVL_NEVER_SIMPLE
  bit fire_2state = 0;
  bit fire_xcheck = 0;
  bit fire_cover = 0;
  always @(posedge clock) begin
`ifdef OVL_SHARED_CODE
`ifdef OVL_GATING_OFF
    if (reset^reset_polarity) begin
`else
    if ((reset&enable)^reset_polarity) begin
`endif
`else
    if (reset) begin
`endif
      assert ((test_expr) !== 1'b1) 
      else begin
        fire_2state <= 1'b1;
        `AMD_ASSERT_ERROR("OVL_NEVER", msg);
      end
    end
  end
`else
  `include "./sva05/ovl_never_logic.sv"
`endif
`endif

`ifdef OVL_PSL
  `include "./psl05/assert_never_psl_logic.v"

`else
  assign fire = {fire_cover, fire_xcheck, fire_2state};
  `endmodule // ovl_never
`endif
