//----------------------------------------------------------------------------
//
// Owner :      Yun Tian, 02A113 (Shanghai - Irving Zhou), 24222
//
// Notes  :     Please contact Yun Tian for any idea or improvement about the PERFMON modules
//----------------------------------------------------------------------------

`include "gc__cp_reg.v"
`include "gc_register_addr.v"

module perfmon_16b_counter (
                /*AUTOARG*/
   // Outputs
   counter_incr_pre,
   carry_out,
   perfcounter_q,
   // Inputs
   sclk,  state_en,
   perfmon_reset, perfmon_spm_reset,
   incrvalue_d, state_d, usestate_d,
   perfmon_state, perfmon_enable_mode, 
   counter_mode,
   spm_mode,
   reg_we,
   reg_wd,
   carry_in,
   reg_addr_cntr_sel,
   spm_sample_strobe_q,
   incr_ge_counter
   );

   parameter SEL_WIDTH=8;
   parameter INCR_WIDTH=10;
   parameter CNTR_WIDTH=64;
   parameter DATA_WIDTH=32;
   parameter SIGNED=0;
   parameter CNTR_ID=0;
   parameter CLKGATE_EN=1'b1;
   
   localparam CNTR_WIDTH_PLUS_ONE = CNTR_WIDTH + 1;
   
   localparam COUNTER_MODE_WIDTH=4;
`include "gc_features.vh"
`ifdef GC__NUM_STATES__4
   localparam STATES_WIDTH = 2 ;
`else
   localparam STATES_WIDTH = 3 ;
`endif

   input         sclk; // should be a permanment sclk
   input         perfmon_reset, perfmon_spm_reset;
   input [7:0]   state_en;
   input [3:0]   spm_mode;
   input         carry_in;
   output reg    carry_out;
   
   output  [CNTR_WIDTH-1:0] 		 perfcounter_q;

   input [CNTR_WIDTH-1 : 0] reg_addr_cntr_sel;

   input [1:0]   reg_we;
   input [CNTR_WIDTH-1 : 0] reg_wd;

   input spm_sample_strobe_q;

   input [INCR_WIDTH-1:0] incrvalue_d;
   input [STATES_WIDTH-1:0] state_d;
   input                  usestate_d;
   input         incr_ge_counter;
   output [INCR_WIDTH-1 :0] counter_incr_pre;

   input [COUNTER_MODE_WIDTH-1 : 0 ] counter_mode;

   input [`CP_PERFMON_CNTL_PERFMON_STATE_SIZE-1:0] perfmon_state;
   input [`CP_PERFMON_CNTL_PERFMON_ENABLE_MODE_SIZE-1:0] perfmon_enable_mode;

   reg  [CNTR_WIDTH:0] 		     perfcounter_pre;
   reg  [CNTR_WIDTH-1:0] 		 perfcounter_q;

   reg [INCR_WIDTH-1:0] incrvalue_q;
   reg [STATES_WIDTH-1:0] state_q;
   reg                  usestate_q;


   // flop the inputs after the mux: 
   always @(posedge sclk)
     begin
	  begin
             if (!CLKGATE_EN || perfmon_reset || (incrvalue_q != 0 || incrvalue_d != 0) || (usestate_q != 0 || usestate_d != 0))
               begin
                  incrvalue_q <= incrvalue_d;
                  state_q  <= state_d;
                  usestate_q  <= usestate_d;
               end
	  end
     end
   
   // and now we do the real work:
   wire localstate = state_en[state_q] ^ perfmon_enable_mode[0];
   wire [INCR_WIDTH-1:0] incr_zero = {INCR_WIDTH {1'b0}};
   wire [INCR_WIDTH-1:0] incr_one  = {incr_zero + 1'b1};
   reg [INCR_WIDTH-1:0] counter_incr, counter_incr_pre;

always @*
begin
 // filter out events whose state doesn't match
   if (~localstate & usestate_q
       & (perfmon_enable_mode == `CP_PERFMON_ENABLE_MODE_COUNT_CONTEXT_TRUE ||
       perfmon_enable_mode == `CP_PERFMON_ENABLE_MODE_COUNT_CONTEXT_FALSE))
         counter_incr_pre = {(INCR_WIDTH){1'b0}};
   else
         counter_incr_pre = incrvalue_q;

       case (counter_mode)
          `PERFMON_COUNTER_MODE_ACCUM,
          `PERFMON_COUNTER_MODE_MAX,
          `PERFMON_COUNTER_MODE_SAMPLE:
              counter_incr = counter_incr_pre;
          `PERFMON_COUNTER_MODE_ACTIVE_CYCLES,
          `PERFMON_COUNTER_MODE_INACTIVE_CYCLES,
          `PERFMON_COUNTER_MODE_DIRTY:
             counter_incr = ((|counter_incr_pre) ^ (counter_mode == `PERFMON_COUNTER_MODE_INACTIVE_CYCLES)) ? incr_one : incr_zero;
          `PERFMON_COUNTER_MODE_CYCLES_SINCE_FIRST_EVENT,
          `PERFMON_COUNTER_MODE_CYCLES_SINCE_LAST_EVENT:
             counter_incr = (|perfcounter_q) ? incr_one : (|counter_incr_pre) ? incr_one : incr_zero;
          `PERFMON_COUNTER_MODE_CYCLES_GE_HI:
             counter_incr = incr_ge_counter ? incr_one : incr_zero;
          `PERFMON_COUNTER_MODE_CYCLES_EQ_HI:
             counter_incr = (counter_incr_pre == perfcounter_q[CNTR_WIDTH-1 : CNTR_WIDTH/2]) ? incr_one : incr_zero;
          default: counter_incr = counter_incr_pre;
       endcase // case(counter_mode)
 end // always @ *

   wire reset_strobe = (spm_mode != `PERFMON_SPM_MODE_OFF) ? (perfmon_spm_reset || spm_sample_strobe_q) : perfmon_reset;

   always @(*)
     begin
     if (reset_strobe)
       perfcounter_pre = '0;
     else
       perfcounter_pre = {1'b0, perfcounter_q};

     if (((spm_mode == `PERFMON_SPM_MODE_OFF) && (CNTR_ID == 0)) ||
        (((spm_mode == `PERFMON_SPM_MODE_32BIT_CLAMP) || (spm_mode == `PERFMON_SPM_MODE_32BIT_NO_CLAMP)) && (CNTR_ID%2==0)) ||
        ((spm_mode == `PERFMON_SPM_MODE_16BIT_CLAMP) || (spm_mode == `PERFMON_SPM_MODE_16BIT_NO_CLAMP)))
           case(perfmon_state)
	        `CP_PERFMON_STATE_START_COUNTING:
	         begin
	            case (counter_mode)
	              `PERFMON_COUNTER_MODE_ACCUM,
   	                `PERFMON_COUNTER_MODE_ACTIVE_CYCLES,
	                `PERFMON_COUNTER_MODE_CYCLES_SINCE_FIRST_EVENT,
	                `PERFMON_COUNTER_MODE_CYCLES_SINCE_LAST_EVENT,
	                `PERFMON_COUNTER_MODE_CYCLES_GE_HI,
	                `PERFMON_COUNTER_MODE_CYCLES_EQ_HI:
	         	 begin
	         	    if (counter_mode == `PERFMON_COUNTER_MODE_CYCLES_SINCE_LAST_EVENT && (|counter_incr_pre))

	         	      //perfcounter_pre = {{(CNTR_WIDTH) {1'b0}},1'b1};
	         	      perfcounter_pre = CNTR_WIDTH_PLUS_ONE'(1'h1); //LEDAFIX (tzohner) cast to correct length
	         	    else

                begin
                    if(SIGNED) //LEDAFIX (tzohner) get rid of zero length concatenation
                      perfcounter_pre = perfcounter_pre + CNTR_WIDTH_PLUS_ONE'($signed(counter_incr)); //FORMALITY FIX (tzohner)
                    else
                      perfcounter_pre = perfcounter_pre + CNTR_WIDTH_PLUS_ONE'(counter_incr);
	         	      //perfcounter_pre = (perfcounter_pre + {{(CNTR_WIDTH+1-INCR_WIDTH){SIGNED?counter_incr[INCR_WIDTH-1]:1'b0}},counter_incr});
               end
	         	 end
	              `PERFMON_COUNTER_MODE_DIRTY:
	                if (counter_incr[0])
	         	       perfcounter_pre = {{{(CNTR_WIDTH){1'b0}},1'b1}};
	              `PERFMON_COUNTER_MODE_MAX:
	                if (incr_ge_counter)

                    perfcounter_pre = CNTR_WIDTH_PLUS_ONE'(counter_incr); //LEDAFIX (tzohner) Cast to correct length, avoid zero length concat
	         	        //perfcounter_pre = {{(CNTR_WIDTH+1-INCR_WIDTH){1'b0}}, counter_incr};
	              `PERFMON_COUNTER_MODE_SAMPLE:

                    perfcounter_pre = CNTR_WIDTH_PLUS_ONE'(counter_incr); //LEDAFIX (tzohner) Cast to correct length, avoid zero length concat
	                  //perfcounter_pre = {{(CNTR_WIDTH+1-INCR_WIDTH){1'b0}}, counter_incr};
	              default:;
	            endcase // case(counter_mode)
	         end // case: `CP_PERFMON_STATE_START_COUNTING
	       `CP_PERFMON_STATE_STOP_COUNTING:;
               `CP_PERFMON_STATE_DISABLE_AND_RESET: perfcounter_pre = {(CNTR_WIDTH+1) {1'b0}};
                 default:;
             endcase // case (perfmon_state)
     else if (perfmon_state != `CP_PERFMON_STATE_DISABLE_AND_RESET) // don't propagate carry when we enter disabled state
       perfcounter_pre = perfcounter_pre + {{(CNTR_WIDTH-1) {1'b0}}, carry_in};
        
     end // always @ (posedge sclk)

     always @ (posedge sclk)
     begin
      carry_out <= perfcounter_pre[CNTR_WIDTH];
      if (| reg_we)
      begin
        if (reg_we[0])
          perfcounter_q[(CNTR_WIDTH/2)-1 : 0] <= reg_wd[CNTR_WIDTH/2 - 1: 0];
        if (reg_we[1])
          perfcounter_q[CNTR_WIDTH-1 : CNTR_WIDTH/2] <= reg_wd[CNTR_WIDTH - 1: CNTR_WIDTH/2];
      end
      else if (reset_strobe || perfmon_state != `CP_PERFMON_STATE_DISABLE_AND_RESET)
        case (spm_mode)
        `PERFMON_SPM_MODE_OFF: begin
                                 perfcounter_q <= perfcounter_pre[CNTR_WIDTH-1 : 0];
                                 carry_out     <= perfcounter_pre[CNTR_WIDTH];
                               end
        `PERFMON_SPM_MODE_16BIT_CLAMP: begin
                                if (perfcounter_pre[CNTR_WIDTH])
                                begin
                                 perfcounter_q <= '1; //simplify (tzohner)
                                 //perfcounter_q <= {CNTR_WIDTH {1'b1}};
                                 carry_out     <= 1'b0;
                                end
                                else
                                begin
                                 perfcounter_q <= perfcounter_pre[CNTR_WIDTH-1 : 0] ;
                                 carry_out     <= perfcounter_pre[CNTR_WIDTH];
                                end
                               end
        `PERFMON_SPM_MODE_16BIT_NO_CLAMP: begin
                                 perfcounter_q <= perfcounter_pre[CNTR_WIDTH-1 : 0] ;
                                 carry_out     <= perfcounter_pre[CNTR_WIDTH];
                                end
        `PERFMON_SPM_MODE_32BIT_CLAMP: begin
                                if (perfcounter_pre[CNTR_WIDTH])
                                begin
                                 perfcounter_q <= '1; //simplify (tzohner)
                                 //perfcounter_q <= {CNTR_WIDTH {1'b1}};
                                 carry_out     <= 1'b0;
                                end
                                else
                                begin
                                 perfcounter_q <= perfcounter_pre[CNTR_WIDTH-1 : 0] ;
                                 carry_out     <= perfcounter_pre[CNTR_WIDTH];
                                end
                               end
        `PERFMON_SPM_MODE_32BIT_NO_CLAMP: begin
                                 perfcounter_q <= perfcounter_pre[CNTR_WIDTH-1 : 0] ;
                                 carry_out     <= perfcounter_pre[CNTR_WIDTH];
                                 end
        `PERFMON_SPM_MODE_TEST_MODE_0: begin
                                 perfcounter_q <= reg_addr_cntr_sel;
                                 carry_out     <= 1'b0;
                                 end
        `PERFMON_SPM_MODE_TEST_MODE_1: begin
                                 perfcounter_q <= perfcounter_q + {{CNTR_WIDTH-1{1'b0}}, 1'b1}; //LEDAFIX (tzohner) cast increment to correct length
                                 carry_out     <= 1'b0;
                                 end
        `PERFMON_SPM_MODE_TEST_MODE_2: begin
                                 if (spm_sample_strobe_q)
                                  begin
                                    perfcounter_q <= perfcounter_q + {{CNTR_WIDTH-1{1'b0}}, 1'b1}; //LEDAFIX (tzohner) cast increment to correct length
                                    carry_out     <= 1'b0;
                                  end
                                  end
        default: begin
                   perfcounter_q <= perfcounter_q;
                   carry_out     <= carry_out;
                   end
     
       endcase
     end



     endmodule
