//----------------------------------------------------------------------------
//
// Owner :      Yun Tian, 02A113 (Shanghai - Irving Zhou), 24222
//
// Notes  :     Please contact Yun Tian for any idea or improvement about the PERFMON modules
//----------------------------------------------------------------------------

`include "gc__cp_reg.v"
`include "gc__pa_reg.v"
`include "gc_register_addr.v"

module perfmon_control
  (
   /*AUTOARG*/
   // Outputs
   regbus_rd, regbus_rs, state_en, perfmon_reset, perfmon_state, spm_perfmon_state, 
   perfmon_spm_reset,
   perfmon_enable_mode, sample_strobe, 
   // Inputs
   regbus_we, regbus_re, regbus_a, regbus_wd, sclk, reset, sample_en
   );
   
   parameter ADDR_WIDTH=18;
   parameter RESET_TO_ENABLED=1'b0;

   output [31:0] regbus_rd;
   output        regbus_rs;
   input         regbus_we;
   input         regbus_re;
   input [ADDR_WIDTH-1:2]  regbus_a;
   input [31:0]  regbus_wd;
   input         sclk; // should be a permanment sclk
   input         reset;
   output [7:0]  state_en;

   input 	 sample_en;
   output    perfmon_spm_reset;

   output 	 perfmon_reset;
   output [`CP_PERFMON_CNTL_PERFMON_STATE_SIZE-1:0] perfmon_state;
   output [`CP_PERFMON_CNTL_SPM_PERFMON_STATE_SIZE-1:0] spm_perfmon_state;
   output [`CP_PERFMON_CNTL_PERFMON_ENABLE_MODE_SIZE-1:0] perfmon_enable_mode;
   
   output 	 sample_strobe;

   assign 	 regbus_rd = 32'd0;
   assign 	 regbus_rs = 1'b0;

   // bad fix for regester awkwardness:
   wire [ADDR_WIDTH-1:0] 	 regbus_aa;
   assign 	 regbus_aa = {regbus_a,2'd0};
   // end of IO definition
   // these are the shared registers amoung all blocks
   reg [9:0] 	 cntl_q;
   wire 	 sample_strobe;
   reg 		 sample_en_q;

   assign 	 perfmon_state = cntl_q[`CP_PERFMON_CNTL_PERFMON_STATE];
   assign 	 spm_perfmon_state = cntl_q[`CP_PERFMON_CNTL_SPM_PERFMON_STATE];
   assign 	 perfmon_enable_mode = cntl_q[`CP_PERFMON_CNTL_PERFMON_ENABLE_MODE];
   
   wire 	 reg_addr_is_cntl = (regbus_aa == `CP_PERFMON_CNTL);

   // this is the write interface to the global control register 
   assign 	 sample_strobe = (reset ? 1'b0
				  : ((regbus_we ? reg_addr_is_cntl ? regbus_wd[`CP_PERFMON_CNTL_PERFMON_SAMPLE_ENABLE] : 1'b0 : 1'b0)
				     || sample_en_q));

   reg 		 perfmon_reset_d;
   reg 		 perfmon_reset;
   reg 		 perfmon_spm_reset_d;
   reg 		 perfmon_spm_reset;

   always @(*)
   begin
      perfmon_reset_d = 1'b0;
      perfmon_spm_reset_d = 1'b0;
   
	  if (regbus_we)
	    if (reg_addr_is_cntl)
	    begin
		   perfmon_reset_d = (regbus_wd[`CP_PERFMON_CNTL_PERFMON_STATE] == `CP_PERFMON_STATE_DISABLE_AND_RESET);
		   perfmon_spm_reset_d = (regbus_wd[`CP_PERFMON_CNTL_SPM_PERFMON_STATE] == `STRM_PERFMON_STATE_DISABLE_AND_RESET);
	    end
   end
   
   always @(posedge sclk)
     begin
	sample_en_q <= sample_en;

	if (reset)
	  begin
	     cntl_q <= 10'd0;
         perfmon_reset <= 1'b1;
         perfmon_spm_reset <= 1'b1;
	  end
	else
      begin
        perfmon_reset <= perfmon_reset_d;
        perfmon_spm_reset <= perfmon_spm_reset_d;
	    if (regbus_we)
	      if (reg_addr_is_cntl)
	      begin
		      cntl_q <= regbus_wd[9:0];
	      end
      end
    end // always @ (posedge sclk)

   wire sr_select,sr_cpy;
   assign sr_select = ((regbus_aa[ADDR_WIDTH-1:0]&18'b111000111111111100) == `CP_PERFMON_CNTX_CNTL);
   assign sr_cpy =    ((regbus_aa[ADDR_WIDTH-1:0]&18'b111000111111111100) == `GFX_COPY_STATE);
   ati_8rp_state_storage_with_reset #(.WIDTH(1),
				      .RESET_ENABLE(1'b1),
				      .RESET_DATA(RESET_TO_ENABLED))
   thestate (.st_data0(state_en[0]), 
             .st_data1(state_en[1]),
             .st_data2(state_en[2]),
             .st_data3(state_en[3]),
             .st_data4(state_en[4]),
             .st_data5(state_en[5]),
             .st_data6(state_en[6]),
             .st_data7(state_en[7]),
	     .rst(reset),
	     .re(1'b0),
             .r_data(),
             .w_data(regbus_wd[`CP_PERFMON_CNTX_CNTL_PERFMON_ENABLE]),
             .w_addr(regbus_a[14:12]),
             .r_addr(regbus_wd[2:0]), // since we do not use the read port, we can wire the bits without a mux
             .we(regbus_we),
             .sel(sr_select),
             .cpy(sr_cpy),
             .clk(sclk )); // Next is the code to implement the counters
   
endmodule // perfmon_control

