//----------------------------------------------------------------------------
//
// Owner :      Yun Tian, 02A113 (Shanghai - Irving Zhou), 24222
//
// Notes  :     Please contact Yun Tian for any idea or improvement about the PERFMON modules
//----------------------------------------------------------------------------

`include "gc__cp_reg.v"
`include "gc_register_addr.v"
`include "gc__sq_reg.v"

module perfmon_counter (
                /*AUTOARG*/
   // Outputs
   regbus_rd, regbus_rs, perfcounter_q, cntr_select,
   spm_cntr_pair0, 
   // Inputs
   regbus_we, regbus_re, regbus_a, regbus_wd, sclk, reset, state_en,
   sample_strobe, strobe_d, incrvalue_d, state_d, usestate_d,
   spm_sample_strobe,
   perfmon_state, perfmon_enable_mode, perfmon_reset,
   spm_perfmon_state,
   spm_perfmon_reset,
   reg_addr_cntr_sel, reg_addr_cntr_lo, reg_addr_cntr_hi
   );

   parameter SEL_WIDTH=8;
   parameter INCR_WIDTH=10;
   parameter CNTR_WIDTH=64;
   parameter ADDR_WIDTH=18;
   parameter SIGNED=0;
   parameter CLKGATE_EN=1'b1;
   parameter DISABLE_SPM=0;
   parameter IS_SQG=0;
   
   localparam COUNTER_MODE_WIDTH=4;
   localparam IS_SPM=0;

   localparam CNTR_SIZE = IS_SPM? 16 : 64;
   localparam NUM_CNTRS = IS_SPM? 4  : 1;

`include "gc_features.vh"
`ifdef GC__NUM_STATES__4
   localparam STATES_WIDTH = 2 ;
`else
   localparam STATES_WIDTH = 3 ;
`endif

   output [31:0] regbus_rd;
   output        regbus_rs;
   input         regbus_we;
   input         regbus_re;
   input [ADDR_WIDTH-1:2] regbus_a;
   input [31:0]  regbus_wd;
   input         sclk; // should be a permanment sclk
   input         reset;
   input [7:0]   state_en;
   input         sample_strobe;
   input         spm_sample_strobe;

   input         strobe_d;
   input [INCR_WIDTH-1:0] incrvalue_d;
   input [STATES_WIDTH-1:0] state_d;
   input                  usestate_d;

   input [`CP_PERFMON_CNTL_PERFMON_STATE_SIZE-1:0] perfmon_state;
   input [`CP_PERFMON_CNTL_SPM_PERFMON_STATE_SIZE-1:0] spm_perfmon_state;
   input [`CP_PERFMON_CNTL_PERFMON_ENABLE_MODE_SIZE-1:0] perfmon_enable_mode;
   input 						 perfmon_reset;
   input 						 spm_perfmon_reset;

   output [CNTR_SIZE-1:0] 				 perfcounter_q;
   output [SEL_WIDTH-1:0] cntr_select;
   output                 spm_cntr_pair0;

   input [ADDR_WIDTH-1:0] reg_addr_cntr_sel,
                          reg_addr_cntr_lo,
                          reg_addr_cntr_hi;

// This is the start of the code to implement counter
//   reg [INCR_WIDTH-1:0]   incrvalue_q;
//   reg [STATES_WIDTH-1:0] state_q;
//   reg                    usestate_q;
   wire [CNTR_SIZE-1:0]   perfcounter_q;
   reg  [CNTR_SIZE-1:0]   perfcounter_shadow_q;
   reg                    perfcounter_shadow_strobe;
   
   reg [SEL_WIDTH-1:0]    cntr_select;
   reg [COUNTER_MODE_WIDTH-1:0] counter_mode;

   reg [3:0]              spm_mode;
   reg                    spm_sample_strobe_q;

   reg [3:0]              sqc_bank_mask;
   wire [31:0]            cntr_sel_data;

   wire                   carry_out_0;

   wire [SEL_WIDTH-1:0]    cntr_select_reset;
   wire [COUNTER_MODE_WIDTH-1:0] counter_mode_reset;
   wire [3:0]              spm_mode_reset;

   // flop the inputs after the mux: 
   always @(posedge sclk)
     begin
//	if (!perfmon_reset)
	  begin
             perfcounter_shadow_strobe <= strobe_d;
             spm_sample_strobe_q <= spm_sample_strobe;
	  end
     end
   
   
   // bad fix for register awkwardness:
   wire [ADDR_WIDTH-1:0] regbus_aa;
   assign                regbus_aa = {regbus_a,2'd0};
   // end of IO definition
   
   wire reg_addr_is_cntr_hi = (regbus_aa==reg_addr_cntr_hi);
   wire reg_addr_is_cntr_lo = (regbus_aa==reg_addr_cntr_lo);
   wire reg_addr_is_cntr_sel = (regbus_aa==reg_addr_cntr_sel);
   wire reg_addr_is_cntr_hi_we   = (regbus_aa==reg_addr_cntr_hi) & regbus_we;
   wire reg_addr_is_cntr_lo_we   = (regbus_aa==reg_addr_cntr_lo) & regbus_we;
   
generate
   if (IS_SQG == 0) begin
      assign cntr_select_reset = {SEL_WIDTH {1'b1}};
      assign counter_mode_reset = `PERFMON_COUNTER_MODE_ACCUM;
      assign spm_mode_reset = 4'h0;
   end
   else begin
      assign cntr_select_reset = `SQ_PERFCOUNTER0_SELECT_PERF_SEL_DEFAULT;
      assign counter_mode_reset = `SQ_PERFCOUNTER0_SELECT_PERF_MODE_DEFAULT;
      assign spm_mode_reset = `SQ_PERFCOUNTER0_SELECT_SPM_MODE_DEFAULT;
   end
endgenerate

   // register write interface;
   always @(posedge sclk)
     begin
        if (reset)
	    begin
           cntr_select <= cntr_select_reset;
           counter_mode <= counter_mode_reset;
           spm_mode     <= spm_mode_reset;
	    end
        else if ((regbus_we) ? (reg_addr_is_cntr_sel) : 1'b0)
	    begin
           cntr_select <= regbus_wd[SEL_WIDTH-1:0];
           counter_mode <= regbus_wd[31-:COUNTER_MODE_WIDTH];
           if (!DISABLE_SPM)
              spm_mode <= regbus_wd[23:20];
	    end
     end

generate
   if (IS_SQG == 0) begin
      assign cntr_sel_data = {counter_mode,4'h0, spm_mode, {(20-SEL_WIDTH){1'b0}},cntr_select[SEL_WIDTH-1:0]};
   end
   else begin
      always @(posedge sclk) begin
        if (reset) begin
           sqc_bank_mask <= `SQ_PERFCOUNTER0_SELECT_SQC_BANK_MASK_DEFAULT;
   	    end
        else if ((regbus_we) ? (reg_addr_is_cntr_sel) : 1'b0) begin
           sqc_bank_mask <= regbus_wd[15:12];
   	    end
      end
      assign cntr_sel_data = {counter_mode,4'h0, spm_mode,4'h0,sqc_bank_mask,3'h0,cntr_select[SEL_WIDTH-1:0]};
   end
endgenerate

   // register read interface interface
   assign regbus_rd = (!regbus_re ? 32'd0 :
                      reg_addr_is_cntr_hi ?  perfcounter_shadow_q[CNTR_SIZE-1:32] :
                      reg_addr_is_cntr_lo ?  perfcounter_shadow_q[31:0] :
                      reg_addr_is_cntr_sel ? cntr_sel_data[31:0] :
                      32'd0);
   assign regbus_rs = (regbus_re ? (reg_addr_is_cntr_hi || reg_addr_is_cntr_lo || reg_addr_is_cntr_sel) : 1'b0);

//   wire [INCR_WIDTH-1:0] counter_incr;
   wire [INCR_WIDTH-1:0] counter_incr_pre;
   
   wire 		incr_ge_counter = (counter_incr_pre >= (counter_mode == `PERFMON_COUNTER_MODE_CYCLES_GE_HI
								? {{(CNTR_SIZE-32) {1'b0}}, perfcounter_q[CNTR_SIZE-1:32]}
								: perfcounter_q));

   wire [INCR_WIDTH-1:0] incr_zero = {INCR_WIDTH {1'b0}};
   wire [INCR_WIDTH-1:0] incr_one = {incr_zero + 1'b1};
//   wire [16:0] perfcounter_pre;
   wire [`CP_PERFMON_CNTL_PERFMON_STATE_SIZE-1:0] perfmon_16b_state;
   assign perfmon_16b_state = (spm_mode == 4'h0)? perfmon_state : spm_perfmon_state;

perfmon_16b_counter #(.SEL_WIDTH(SEL_WIDTH),
                      .INCR_WIDTH(INCR_WIDTH),
                      .CNTR_WIDTH(CNTR_SIZE),
                      .SIGNED(SIGNED),
                      .CLKGATE_EN(CLKGATE_EN))
                      uperf_counter_0 (
                /*AUTOARG*/
   // Outputs
  .counter_incr_pre     ( counter_incr_pre ),
  .carry_out            ( carry_out_0 ),
  .perfcounter_q        ( perfcounter_q[63:0]),
  // Inputs
  .sclk                 ( sclk ),  
  .state_en             ( state_en ),
  .perfmon_reset        ( perfmon_reset ), 
  .perfmon_spm_reset    ( spm_perfmon_reset ),
  .incrvalue_d          ( incrvalue_d ), 
  .state_d              ( state_d ), 
  .usestate_d           ( usestate_d ),
  .perfmon_state        ( perfmon_16b_state ), 
  .perfmon_enable_mode  ( perfmon_enable_mode ), 
  .counter_mode         ( counter_mode ),
  .spm_mode             ( spm_mode ),
  .reg_we               ( {reg_addr_is_cntr_hi_we, reg_addr_is_cntr_lo_we}),
  .reg_wd               ( {regbus_wd,regbus_wd} ),
  .carry_in             ( 1'b0 ),
  .reg_addr_cntr_sel    ( {4{reg_addr_cntr_sel[ADDR_WIDTH-1:2]}}),
  .spm_sample_strobe_q  ( spm_sample_strobe_q ),
  .incr_ge_counter      ( incr_ge_counter )
   );

   wire strobe = (spm_mode == 4'h0 ? perfcounter_shadow_strobe | sample_strobe : spm_sample_strobe_q);
   
   always @(posedge sclk)
     begin
        if (reset || (spm_mode != 4'h0 && spm_perfmon_reset))
          perfcounter_shadow_q <= {CNTR_SIZE{1'b0}};
        else if (strobe)
          perfcounter_shadow_q <= perfcounter_q;
        else if (spm_mode != 4'h0 && spm_perfmon_state != `CP_PERFMON_STATE_DISABLE_AND_RESET)
          perfcounter_shadow_q <= {perfcounter_shadow_q[CNTR_SIZE-1 : 32], perfcounter_shadow_q[0], perfcounter_shadow_q[31:1]};
     end

     assign spm_cntr_pair0 = (spm_mode == 4'h0) ? 1'b0 : perfcounter_shadow_q[0];
   
endmodule // perfmon_counter

