//----------------------------------------------------------------------------
//
// Owner :      Yun Tian, 02A113 (Shanghai - Irving Zhou), 24222
//
// Notes  :     Please contact Yun Tian for any idea or improvement about the PERFMON modules
//----------------------------------------------------------------------------

`include "gc__cp_reg.v"
`include "gc_register_addr.v"

module perfmon_counter_spm (
                /*AUTOARG*/
   // Outputs
   regbus_rd, regbus_rs, perfcounter_q, 
   cntr_select_0, cntr_select_1, cntr_select_2, cntr_select_3,
   spm_cntr_pair0, spm_cntr_pair1,
   // Inputs
   regbus_we, regbus_re, regbus_a, regbus_wd, sclk, reset, 
   state_en_0, state_en_1, state_en_2, state_en_3,
   sample_strobe, spm_sample_strobe, 
   strobe_d_0, incrvalue_d_0, state_d_0, usestate_d_0,
   strobe_d_1, incrvalue_d_1, state_d_1, usestate_d_1,
   strobe_d_2, incrvalue_d_2, state_d_2, usestate_d_2,
   strobe_d_3, incrvalue_d_3, state_d_3, usestate_d_3,
   perfmon_state, spm_perfmon_state, perfmon_enable_mode, perfmon_reset,
   perfmon_spm_reset,
   reg_addr_cntr_sel, reg_addr_cntr_sel1,  reg_addr_cntr_lo, reg_addr_cntr_hi
   );

   parameter SEL_WIDTH=10;
   parameter INCR_WIDTH=10;
   parameter CNTR_WIDTH=64;
   parameter ADDR_WIDTH=18;
   parameter SIGNED=0;
   parameter CLKGATE_EN=1'b1;
   
   localparam COUNTER_MODE_WIDTH=4;

   localparam IS_SPM=1;

   localparam CNTR_SIZE = IS_SPM? 16 : CNTR_WIDTH;
   localparam NUM_CNTRS = IS_SPM? 4  : 1;

`include "gc_features.vh"
`ifdef GC__NUM_STATES__4
   localparam STATES_WIDTH = 2 ;
`else
   localparam STATES_WIDTH = 3 ;
`endif

   output [31:0] regbus_rd;
   output        regbus_rs;
   input         regbus_we;
   input         regbus_re;
   input [ADDR_WIDTH-1:2] regbus_a;
   input [31:0]  regbus_wd;
   input         sclk; // should be a permanment sclk
   input         reset;
   input [7:0]   state_en_0;
   input [7:0]   state_en_1;
   input [7:0]   state_en_2;
   input [7:0]   state_en_3;
   input         sample_strobe;
   input         spm_sample_strobe;

   input                  strobe_d_0;
   input [INCR_WIDTH-1:0] incrvalue_d_0;
   input [STATES_WIDTH-1:0] state_d_0;
   input                  usestate_d_0;
   input                  strobe_d_1;
   input [INCR_WIDTH-1:0] incrvalue_d_1;
   input [STATES_WIDTH-1:0] state_d_1;
   input                  usestate_d_1;
   input                  strobe_d_2;
   input [INCR_WIDTH-1:0] incrvalue_d_2;
   input [STATES_WIDTH-1:0] state_d_2;
   input                  usestate_d_2;
   input                  strobe_d_3;
   input [INCR_WIDTH-1:0] incrvalue_d_3;
   input [STATES_WIDTH-1:0] state_d_3;
   input                  usestate_d_3;

   input [`CP_PERFMON_CNTL_PERFMON_STATE_SIZE-1:0] perfmon_state;
   input [`CP_PERFMON_CNTL_SPM_PERFMON_STATE_SIZE-1:0] spm_perfmon_state;
   input [`CP_PERFMON_CNTL_PERFMON_ENABLE_MODE_SIZE-1:0] perfmon_enable_mode;
   input 						 perfmon_reset;
   input 						 perfmon_spm_reset;

   output [CNTR_WIDTH-1:0] 				 perfcounter_q;
   output [SEL_WIDTH-1:0] cntr_select_0;
   output [SEL_WIDTH-1:0] cntr_select_1;
   output [SEL_WIDTH-1:0] cntr_select_2;
   output [SEL_WIDTH-1:0] cntr_select_3;

   input [ADDR_WIDTH-1:0] reg_addr_cntr_sel, reg_addr_cntr_sel1,
                          reg_addr_cntr_lo,
                          reg_addr_cntr_hi;

   output                 spm_cntr_pair0, spm_cntr_pair1;                          

// This is the start of the code to implement counter
/*   reg [INCR_WIDTH-1:0]   incrvalue_q_0;
   reg [STATES_WIDTH-1:0] state_q_0;
   reg                    usestate_q_0;
   reg [INCR_WIDTH-1:0]   incrvalue_q_1;
   reg [STATES_WIDTH-1:0] state_q_1;
   reg                    usestate_q_1;
   reg [INCR_WIDTH-1:0]   incrvalue_q_2;
   reg [STATES_WIDTH-1:0] state_q_2;
   reg                    usestate_q_2;
   reg [INCR_WIDTH-1:0]   incrvalue_q_3;
   reg [STATES_WIDTH-1:0] state_q_3;
   reg                    usestate_q_3;*/

   reg                    spm_sample_strobe_q;

/*   reg [16:0]             perfcounter_0_q;
   reg [16:0]             perfcounter_1_q;
   reg [16:0]             perfcounter_2_q;
   reg [16:0]             perfcounter_3_q;*/
   wire [63:0]             perfcounter_shadow_q;
   reg [31:0]             perfcounter_shadow_lo_q;
   reg [31:0]             perfcounter_shadow_hi_q;
   reg                    perfcounter_shadow_strobe_0;
   reg                    perfcounter_shadow_strobe_1;
   reg                    perfcounter_shadow_strobe_2;
   reg                    perfcounter_shadow_strobe_3;
   wire                   perfcounter_shadow_strobe;
   
   reg [3:0]  spm_mode;
   reg [SEL_WIDTH-1:0]    cntr_select_0;
   reg [SEL_WIDTH-1:0]    cntr_select_1;
   reg [SEL_WIDTH-1:0]    cntr_select_2;
   reg [SEL_WIDTH-1:0]    cntr_select_3;
   reg [COUNTER_MODE_WIDTH-1:0] counter_mode_0;
   reg [COUNTER_MODE_WIDTH-1:0] counter_mode_1;
   reg [COUNTER_MODE_WIDTH-1:0] counter_mode_2;
   reg [COUNTER_MODE_WIDTH-1:0] counter_mode_3;
   wire [`CP_PERFMON_CNTL_PERFMON_STATE_SIZE-1:0] perfmon_16b_state;

   wire [INCR_WIDTH-1:0]  counter_incr_pre_0;
   wire [INCR_WIDTH-1:0]  counter_incr_pre_1;
   wire [INCR_WIDTH-1:0]  counter_incr_pre_2;
   wire [INCR_WIDTH-1:0]  counter_incr_pre_3;

   // flop the inputs after the mux: 
   always @(posedge sclk)
     begin
//	if (!perfmon_reset)
	  begin
             perfcounter_shadow_strobe_0 <= strobe_d_0;
             perfcounter_shadow_strobe_1 <= strobe_d_1;
             perfcounter_shadow_strobe_2 <= strobe_d_2;
             perfcounter_shadow_strobe_3 <= strobe_d_3;

             spm_sample_strobe_q <= spm_sample_strobe;
	  end
     end
   
   // and now we do the real work:
   
   // bad fix for register awkwardness:
   wire [ADDR_WIDTH-1:0] regbus_aa;
   assign                regbus_aa = {regbus_a,2'd0};
   // end of IO definition
   
   wire reg_addr_is_cntr_hi   = (regbus_aa==reg_addr_cntr_hi);
   wire reg_addr_is_cntr_lo   = (regbus_aa==reg_addr_cntr_lo);
   wire reg_addr_is_cntr_sel  = (regbus_aa==reg_addr_cntr_sel);
   wire reg_addr_is_cntr_sel1 = (regbus_aa==reg_addr_cntr_sel1);
   wire reg_addr_is_cntr_hi_we   = (regbus_aa==reg_addr_cntr_hi) & regbus_we;
   wire reg_addr_is_cntr_lo_we   = (regbus_aa==reg_addr_cntr_lo) & regbus_we;
   
   // register write interface;
   always @(posedge sclk)
     begin
        if (reset)
	  begin
             cntr_select_0  <= {SEL_WIDTH {1'b1}};
	         counter_mode_0 <= `PERFMON_COUNTER_MODE_ACCUM;
             cntr_select_1  <= {SEL_WIDTH {1'b1}};
	         counter_mode_1 <= `PERFMON_COUNTER_MODE_ACCUM;
             cntr_select_2  <= {SEL_WIDTH {1'b1}};
	         counter_mode_2 <= `PERFMON_COUNTER_MODE_ACCUM;
             cntr_select_3  <= {SEL_WIDTH {1'b1}};
	         counter_mode_3 <= `PERFMON_COUNTER_MODE_ACCUM;
             spm_mode       <= 4'h0;
	  end
        else if ((regbus_we) ? (reg_addr_is_cntr_sel) : 1'b0)
	  begin
             cntr_select_0  <= regbus_wd[9:0];
             counter_mode_0 <= regbus_wd[31:28];
             cntr_select_1  <= regbus_wd[19:10];
             counter_mode_1 <= regbus_wd[27:24];
             spm_mode       <= regbus_wd[23:20];

	  end
        else if ((regbus_we) ? (reg_addr_is_cntr_sel1) : 1'b0)
	  begin
             cntr_select_2  <= regbus_wd[9:0];
             counter_mode_2 <= regbus_wd[31:28];
             cntr_select_3  <= regbus_wd[19:10];
             counter_mode_3 <= regbus_wd[27:24];

	  end
     end

   // register read interface interface
   assign regbus_rd = (!regbus_re ? 32'd0 :
                      reg_addr_is_cntr_hi   ?  perfcounter_shadow_q[CNTR_WIDTH-1:32] :
                      reg_addr_is_cntr_lo   ?  perfcounter_shadow_q[31:0] :
                      reg_addr_is_cntr_sel  ?  {counter_mode_0, counter_mode_1, spm_mode, cntr_select_1, cntr_select_0}:
                      reg_addr_is_cntr_sel1 ?  {counter_mode_2, counter_mode_3, 4'h0, cntr_select_3, cntr_select_2}:
                      32'd0);
   assign regbus_rs = (regbus_re ? (reg_addr_is_cntr_hi || reg_addr_is_cntr_lo || reg_addr_is_cntr_sel || reg_addr_is_cntr_sel1) : 1'b0);

   wire [16:0] perfcounter_0_pre;
   wire [16:0] perfcounter_1_pre;
   wire [16:0] perfcounter_2_pre;
   wire [16:0] perfcounter_3_pre;
   
   wire 		incr_ge_counter = (counter_incr_pre_0 >= (counter_mode_0 == `PERFMON_COUNTER_MODE_CYCLES_GE_HI
								? {{(CNTR_WIDTH-32) {1'b0}}, perfcounter_q[CNTR_WIDTH-1:32]}
								: perfcounter_q));

   wire [INCR_WIDTH-1:0] incr_zero = {INCR_WIDTH {1'b0}};
   wire [INCR_WIDTH-1:0] incr_one  = {incr_zero + 1'b1};

   assign perfmon_16b_state = (spm_mode == 4'h0)? perfmon_state : spm_perfmon_state;

   wire carry_out_0;
   wire carry_out_1;
   wire carry_out_2;
   wire carry_out_3;

perfmon_16b_counter #(.SEL_WIDTH(SEL_WIDTH),
                      .INCR_WIDTH(INCR_WIDTH),
                      .CNTR_WIDTH(CNTR_SIZE),
                      .CNTR_ID(0),
                      .SIGNED(SIGNED),
                      .CLKGATE_EN(CLKGATE_EN))
                      uperf_counter_0 (
                /*AUTOARG*/
   // Outputs
  .counter_incr_pre     ( counter_incr_pre_0 ),
  .carry_out            ( carry_out_0 ),
  .perfcounter_q        ( perfcounter_q[15:0]),
  // Inputs
  .sclk                 ( sclk ),  
  .state_en             ( state_en_0 ),
  .perfmon_reset        ( perfmon_reset ), 
  .perfmon_spm_reset    ( perfmon_spm_reset ),
  .incrvalue_d          ( incrvalue_d_0 ), 
  .state_d              ( state_d_0 ), 
  .usestate_d           ( usestate_d_0 ),
  .perfmon_state        ( perfmon_16b_state ), 
  .perfmon_enable_mode  ( perfmon_enable_mode ), 
  .counter_mode         ( counter_mode_0 ),
  .spm_mode             ( spm_mode ),
  .reg_we               ( {reg_addr_is_cntr_lo_we, reg_addr_is_cntr_lo_we}),
  .reg_wd               ( regbus_wd[15:0] ),
  .carry_in             ( 1'b0 ),
  .reg_addr_cntr_sel    ( reg_addr_cntr_sel[ADDR_WIDTH-1:2]),
  .spm_sample_strobe_q  ( spm_sample_strobe_q ),
  .incr_ge_counter      ( incr_ge_counter )
   );
perfmon_16b_counter #(.SEL_WIDTH(SEL_WIDTH),
                      .INCR_WIDTH(INCR_WIDTH),
                      .CNTR_WIDTH(CNTR_SIZE),
                      .CNTR_ID(1),
                      .SIGNED(SIGNED))
                      uperf_counter_1 (
                /*AUTOARG*/
   // Outputs
  .counter_incr_pre     ( counter_incr_pre_1 ),
  .carry_out            ( carry_out_1 ),
  .perfcounter_q        ( perfcounter_q[31:16] ),
  // Inputs
  .sclk                 ( sclk ),  
  .state_en             ( state_en_1 ),
  .perfmon_reset        ( perfmon_reset ), 
  .perfmon_spm_reset    ( perfmon_spm_reset ),
  .incrvalue_d          ( incrvalue_d_1 ), 
  .state_d              ( state_d_1 ), 
  .usestate_d           ( usestate_d_1 ),
  .perfmon_state        ( perfmon_16b_state ), 
  .perfmon_enable_mode  ( perfmon_enable_mode ), 
  .counter_mode         ( counter_mode_1 ),
  .spm_mode             ( spm_mode ),
  .reg_we               ( {reg_addr_is_cntr_lo_we, reg_addr_is_cntr_lo_we}),
  .reg_wd               ( regbus_wd[31:16] ),
  .carry_in             ( carry_out_0 ),
  .reg_addr_cntr_sel    ( reg_addr_cntr_sel[ADDR_WIDTH-1:2]),
  .spm_sample_strobe_q  ( spm_sample_strobe_q ),
  .incr_ge_counter      ( 1'b0 )
   );
perfmon_16b_counter #(.SEL_WIDTH(SEL_WIDTH),
                      .INCR_WIDTH(INCR_WIDTH),
                      .CNTR_WIDTH(CNTR_SIZE),
                      .CNTR_ID(2),
                      .SIGNED(SIGNED))
                      uperf_counter_2 (
                /*AUTOARG*/
   // Outputs
  .counter_incr_pre     ( counter_incr_pre_2 ),
  .carry_out            ( carry_out_2 ),
  .perfcounter_q        ( perfcounter_q[47:32] ), 
  // Inputs
  .sclk                 ( sclk ),  
  .state_en             ( state_en_2 ),
  .perfmon_reset        ( perfmon_reset ), 
  .perfmon_spm_reset    ( perfmon_spm_reset ),
  .incrvalue_d          ( incrvalue_d_2 ), 
  .state_d              ( state_d_2 ), 
  .usestate_d           ( usestate_d_2 ),
  .perfmon_state        ( perfmon_16b_state ), 
  .perfmon_enable_mode  ( perfmon_enable_mode ), 
  .counter_mode         ( counter_mode_2 ),
  .spm_mode             ( spm_mode ),
  .reg_we               ( {reg_addr_is_cntr_hi_we, reg_addr_is_cntr_hi_we}),
  .reg_wd               ( regbus_wd[15:0] ),
  .carry_in             ( carry_out_1 ),
  .reg_addr_cntr_sel    ( reg_addr_cntr_sel1[ADDR_WIDTH-1:2]),
  .spm_sample_strobe_q  ( spm_sample_strobe_q ),
  .incr_ge_counter      ( 1'b0 )
   );
perfmon_16b_counter #(.SEL_WIDTH(SEL_WIDTH),
                      .INCR_WIDTH(INCR_WIDTH),
                      .CNTR_WIDTH(CNTR_SIZE),
                      .CNTR_ID(3),
                      .SIGNED(SIGNED))
                      uperf_counter_3 (
                /*AUTOARG*/
   // Outputs
  .counter_incr_pre     ( counter_incr_pre_3 ),
  .carry_out            ( carry_out_3 ),
  .perfcounter_q        ( perfcounter_q[63:48] ),
  // Inputs
  .sclk                 ( sclk ),  
  .state_en             ( state_en_3 ),
  .perfmon_reset        ( perfmon_reset ), 
  .perfmon_spm_reset    ( perfmon_spm_reset ),
  .incrvalue_d          ( incrvalue_d_3 ), 
  .state_d              ( state_d_3 ), 
  .usestate_d           ( usestate_d_3 ),
  .perfmon_state        ( perfmon_16b_state ), 
  .perfmon_enable_mode  ( perfmon_enable_mode ), 
  .counter_mode         ( counter_mode_3 ),
  .spm_mode             ( spm_mode ),
  .reg_we               ( {reg_addr_is_cntr_hi_we, reg_addr_is_cntr_hi_we}),
  .reg_wd               ( regbus_wd[31:16] ),
  .carry_in             ( carry_out_2 ), 
  .reg_addr_cntr_sel    ( reg_addr_cntr_sel1[ADDR_WIDTH-1:2]),
  .spm_sample_strobe_q  ( spm_sample_strobe_q ),
  .incr_ge_counter      ( incr_ge_counter )
   );
   assign perfcounter_shadow_q = {perfcounter_shadow_hi_q, perfcounter_shadow_lo_q}; 
   assign spm_cntr_pair0 = spm_mode == 4'h0 ? 1'b0 : perfcounter_shadow_lo_q[0];
   assign spm_cntr_pair1 = spm_mode == 4'h0 ? 1'b0 : perfcounter_shadow_hi_q[0];
   assign perfcounter_shadow_strobe = perfcounter_shadow_strobe_0 | perfcounter_shadow_strobe_1 | perfcounter_shadow_strobe_2 | perfcounter_shadow_strobe_3;
   wire strobe = (spm_mode == 4'h0 ? perfcounter_shadow_strobe | sample_strobe : spm_sample_strobe_q);

   always @(posedge sclk)
     begin
        if (reset || (spm_mode != 4'h0 && perfmon_spm_reset))
          begin
             perfcounter_shadow_lo_q <= 0;
             perfcounter_shadow_hi_q <= 0;
          end
        else if (strobe)
          begin
             perfcounter_shadow_lo_q <= perfcounter_q[31:0];
             perfcounter_shadow_hi_q <= perfcounter_q[63:32];
          end
        else if (spm_mode != 4'h0 && spm_perfmon_state != `CP_PERFMON_STATE_DISABLE_AND_RESET)
          begin
             perfcounter_shadow_lo_q <= {perfcounter_shadow_lo_q[0], perfcounter_shadow_lo_q[31:1]};
             perfcounter_shadow_hi_q <= {perfcounter_shadow_hi_q[0], perfcounter_shadow_hi_q[31:1]};
          end
     end
   
endmodule // perfmon_counter_spm

