//----------------------------------------------------------------------------
//
// Owner :      Yun Tian, 02A113 (Shanghai - Irving Zhou), 24222
//
// Notes  :     Please contact Yun Tian for any idea or improvement about the PERFMON modules
//----------------------------------------------------------------------------

module perfmon_file_counter
  (
//VCS coverage off

   /*AUTOARG*/
   // Outputs
   sum, active_cycles, max, dirty, cycles_since_first_event,
   cycles_since_last_event,
   // Inputs
   sclk, reset, incr
   );
   input sclk;
   input reset;

   parameter INCR_WIDTH=10;
   parameter CNTR_WIDTH=64;

   input [INCR_WIDTH-1:0] incr;
   output [CNTR_WIDTH-1:0] sum;
   output [CNTR_WIDTH-1:0] active_cycles;
   output [INCR_WIDTH-1:0] max;
   output 		   dirty;
   output [CNTR_WIDTH-1:0] cycles_since_first_event;
   output [CNTR_WIDTH-1:0] cycles_since_last_event;

   reg [INCR_WIDTH-1:0]    max;
   reg [CNTR_WIDTH-1:0] sum,
			active_cycles,
			cycles_since_first_event,
			cycles_since_last_event;
   reg 			dirty;

   always @(posedge sclk)
     begin
	if (reset)
	  begin
	     max <= {INCR_WIDTH {1'b0}};
	     dirty <= 1'b0;
	     sum <= {CNTR_WIDTH {1'b0}};
	     active_cycles <= {CNTR_WIDTH {1'b0}};
	     cycles_since_first_event <= {CNTR_WIDTH {1'b0}};
	     cycles_since_last_event <= {CNTR_WIDTH {1'b0}};
	  end
	else
	  begin
	     if (incr > 0)
	       begin
		  if (incr > max)
		    max <= incr;
		  if (dirty === 1'b0)
		    dirty <= 1'b1;
		  sum <= sum + incr;
		  active_cycles <= active_cycles + (|incr);
		  cycles_since_last_event <= {CNTR_WIDTH {1'b0}} + 1'b1;
	       end
	     else
	       begin
		  if (^incr === 1'bx)
		    dirty <= 1'bx;
		  if (|cycles_since_last_event)
		    cycles_since_last_event <= cycles_since_last_event + 1;
	       end // else: !if(incr > 0)
	     
	     if ((|cycles_since_first_event) || (|incr))
	       cycles_since_first_event <= cycles_since_first_event + 1;

	  end
     end // always @ (posedge sclk)
//VCS coverage on
`ifdef PERFMON_FILE_COUNTER_COVER
   // psl perfmon_event_incremented: cover {incr > 0} @(posedge sclk);
`endif
endmodule
