//
//                              -*- Mode: Verilog -*-
// Filename              : tcd_ipbuf.v
// Perforce revision     : $Id: //depot/xenos/devel/parts_lib/src/gfx/tc/tcd/tcd_ipbuf.v#5 $
// Perforce changelist   : $Change: 137427 $
//
// Copyright             : Trade secret of ATI Technologies, Inc.
//                         Copyright 2001-2002, ATI Technologies, Inc., (unpublished) 
//                         All rights reserved.  This notice is intended as a precaution against
//                         inadvertent publication and does not imply publication or any waiver
//                         of confidentiality.  The year included in the foregoing notice is the
//                         year of creation of the work.

module pingpong_buffer (
			iCLK,
			iRESET,
			
			// Ping pong buffer idle (i.e. empty) signal
			oIDLE,
			

			// Data input inteface
			iRTS,
			oRTR,

			iDATA,

			// Data output interface
			oRTS,
			iRTR,
			oDATA
			
			);

   parameter WIDTH = 2;
   
   input iCLK;
   input iRESET;
   output oIDLE;

   input  iRTS;
   output oRTR;
   input [WIDTH-1:0] iDATA;

   output 	     oRTS;
   input 	     iRTR;
   output [WIDTH-1:0] oDATA;

   // Internal signals
   wire 	PPB_Push;
   wire 	PPB_Pop;
   wire 	PPB_LdReg0;
   wire 	PPB_LdReg1;

   // Registers
   wire 	PPB_RTR_d;
   wire 	PPB_RTS_d;
   wire 	PPB_RdPtr_d;
   wire 	PPB_WrPtr_d;
   
   reg 		PPB_RTR_q;
   reg 		PPB_RTS_q;	
   reg 		PPB_RdPtr_q;	
   reg 		PPB_WrPtr_q;	
   
   
   assign 	PPB_Push   = PPB_RTR_q && iRTS;
   assign 	PPB_Pop    = PPB_RTS_q && iRTR;
   assign 	PPB_RTR_d  = iRTR || (!PPB_RTS_q) || (PPB_RTR_q &&  !iRTS);
   assign 	PPB_RTS_d  = iRTS || (!PPB_RTR_q) || (PPB_RTS_q && (!iRTR));
   
   assign 	PPB_RdPtr_d	 = PPB_Pop  ^ PPB_RdPtr_q;
   assign 	PPB_WrPtr_d	 = PPB_Push ^ PPB_WrPtr_q;
   
   assign 	PPB_LdReg0	 = !PPB_WrPtr_q && PPB_Push;
   assign 	PPB_LdReg1	 =  PPB_WrPtr_q && PPB_Push;
   assign 	oIDLE            = !PPB_RTS_q;

   
   // Data registers
   reg [WIDTH-1:0] DATA_q0, DATA_q1;
   wire [WIDTH-1:0] DATA_d0 = PPB_LdReg0 ? iDATA : DATA_q0;   
   wire [WIDTH-1:0] DATA_d1 = PPB_LdReg1 ? iDATA : DATA_q1;
      
   always @(posedge iCLK) begin
      if (iRESET) begin
	 PPB_RTR_q <= 1'b1;
         PPB_RTS_q <= 1'b0;
         PPB_RdPtr_q <= 1'b0;
         PPB_WrPtr_q <= 1'b0;
      end
      else begin
	 PPB_RTR_q     <= PPB_RTR_d;
         PPB_RTS_q     <= PPB_RTS_d;
         PPB_RdPtr_q   <= PPB_RdPtr_d;
         PPB_WrPtr_q   <= PPB_WrPtr_d;
      end // else: !if(iRESET)

      DATA_q0 <= DATA_d0;
      DATA_q1 <= DATA_d1;
   end // always @ (posedge iCLK)
   
   //output
   assign oRTR  = PPB_RTR_q;
   assign oRTS  = PPB_RTS_q;
   
   assign oDATA = PPB_RdPtr_q ? DATA_q1 : DATA_q0;


endmodule // pingpong_buffer
