///////////////////////////////////////////////////////////////////////////////////////////
/////  rm_f_sync.v
/////  This module synchronizes iRLC_GFX_rm_valid input, and uses that to sample and hold
/////  the irm[] inputs.
/////  The flops that sample and hold the irm[] inputs are fine-grain-clock-gated where
/////  the synchronized version of iRLC_GFX_rm_valid is the clocking enable.
//////////////////////////////////////////////////////////////////////////////////////////

/////  The input ports irm[] and # of sample/hold flops are parameterized.
/////  No reset is implemented at this time.

module rm_f_sync
(
      // Inputs
      clk,
      rst,
      irm, 
      iRLC_GFX_rm_valid,

      //Outputs
      orm_sync,
      oCG_override
);

// default parameters
parameter WIDTH = 9;

input                  clk;
input                  rst;
input [WIDTH-1:0]      irm;
input                  iRLC_GFX_rm_valid;

output reg [WIDTH-1:0] orm_sync;
output reg             oCG_override;

always @ (posedge clk)
begin
  if(oCG_override | rst)
  begin
    orm_sync[WIDTH-1:0] <= irm[WIDTH-1:0];
  end
end

// synchronize the valid signal
//sync_wrapper rm_valid_synchronizer ( .D(iRLC_GFX_rm_valid), .CK(clk), .DBL_FLOP_EN(1'b1), .Q(oCG_override) );
techind_sync_icd #(.depth(3)) rm_valid_synchronizer (.CKdst(clk), .Dsrc(iRLC_GFX_rm_valid), .Qdst(oCG_override),
                              .SB(1'b1), .RB(1'b1), .FLUSH(1'b0), .CKsrc(1'b0), .EN(1'b1), .Qsrc(),
                              .SC1(1'b0), .SC2(1'b0), .SE(1'b0), .SI(1'b0), .SO()
`ifdef ABV
                              ,.InputChkDis(1'b0)
`endif
);


endmodule


