//                              -*- Mode: Verilog -*-
////////////////////////////////////////////////////////////////////////////
///*
// *
// *  Trade secret of ATI Technologies, Inc.
// *  Copyright 2005, ATI Technologies, Inc., (unpublished)
// *
// *  All rights reserved.  This notice is intended as a precaution against
// *  inadvertent publication and does not imply publication or any waiver
// *  of confidentiality.  The year included in the foregoing notice is the
// *  year of creation of the work.
// *
// */
////////////////////////////////////////////////////////////////////////////
//
// Filename    : ros_cntl.v
// Description : Generate Ring-osc_en for ROS macro.
//
////////////////////////////////////////////////////////////////////////////

//`include "ros_cntl_features.vh"


module ros_cntl_gc10 
#(
parameter USE_COUNTER_CLK = 0
)
  (


//Broadcast to All TDRs
input wire Tdr_Tck,
input wire Tdr_Trstb,
input wire Tdr_Tdi,
input wire Tdr_Shift,
input wire Tdr_Capture,
input wire Tdr_Update,

input wire Tdr_ros_cntl_counter_En,
output wire Tdr_ros_cntl_counter_Tdo,

input wire Tdr_ros_cntl_rosen_En,
output wire Tdr_ros_cntl_rosen_Tdo,

input wire Counter_Clk,

output wire      Ros_En
   
   
   );

   
wire cnt_clk;
wire cnt_rstb;
wire cnt_load;
reg [8-1:0] rosen_cnt;
reg rosen;
reg rosen_tdr_ff;
   //////////////////////////////////////////////////////////////////////////////////

wire [8-1-0+1-1:0] rosen_counter;

ros_cntl_counter_tdr_gc10 ros_cntl_counter_tdr (

 .Tdr_Tck               (Tdr_Tck) ,
 .Tdr_Trstb             (Tdr_Trstb),
 .Tdr_Tdi               (Tdr_Tdi),
 .Tdr_Shift             (Tdr_Shift),
 .Tdr_Capture           (Tdr_Capture),
 .Tdr_Update            (Tdr_Update)


//TDR --> ROS_CNTL_COUNTER
 ,.rosen_counter                 (rosen_counter[8-1-0+1-1:0])
 ,.Tdr_ros_cntl_counter_En              (Tdr_ros_cntl_counter_En)
 ,.Tdr_ros_cntl_counter_Tdo             (Tdr_ros_cntl_counter_Tdo)
);


// TDR output unused. 
wire [1-1:0] rosen_rosen;
reg  rosen_ff;
ros_cntl_rosen_tdr_gc10 ros_cntl_rosen_tdr ( 

 .Tdr_Tck               (Tdr_Tck) ,
 .Tdr_Trstb             (Tdr_Trstb),
 .Tdr_Tdi               (Tdr_Tdi),
 .Tdr_Shift             (Tdr_Shift),
 .Tdr_Capture           (Tdr_Capture),
 .Tdr_Update            (Tdr_Update)


//TDR --> ROS_CNTL_ROSEN
 ,.rosen_rosen           (rosen_rosen[1-1:0])
 ,.Tdr_ros_cntl_rosen_En                (Tdr_ros_cntl_rosen_En)
 ,.Tdr_ros_cntl_rosen_Tdo               (Tdr_ros_cntl_rosen_Tdo)
); 


   //////////////////////////////////////////////////////////////////////////////////

generate
if(USE_COUNTER_CLK == 1 ) begin

  assign cnt_clk = Counter_Clk;

  dfx_async_rst_sync3_gc10 rst_sync_trstb (
    .clk      (cnt_clk),
    .rstinB   (Tdr_Trstb),
    .rstoutB  (cnt_rstb)
    );

  dfx_sync3_gc10 #(
  .TransportDelay(0),
  .TransportCycle(2)
  ) sync_rosen (
    .CK(cnt_clk),
    .D(rosen_rosen),
    .Q(rosen_sync)
    );
   
  always @(posedge cnt_clk)
   rosen_tdr_ff <= rosen_sync;

  assign cnt_load = rosen_sync & ~rosen_tdr_ff;

end else begin

  assign cnt_clk = Tdr_Tck;
  assign cnt_rstb = Tdr_Trstb;

  always @(posedge cnt_clk)
   rosen_tdr_ff <= rosen_rosen;
    
  //assign cnt_load = Tdr_Capture & Tdr_ros_cntl_rosen_En; //Tdr_Capture on negedge.
  //one cycle pulse
  assign cnt_load = rosen_rosen & ~rosen_tdr_ff;

  end
endgenerate


   always @ (posedge cnt_clk or negedge cnt_rstb)
     begin
        if(~cnt_rstb)
          begin
             rosen_cnt <= {(8){1'b0}};
             rosen <= 1'b0;
          end
        else if (cnt_load)  //One cycle pulse             
          begin
             rosen_cnt <= rosen_counter;
             rosen <= 1'b0;
          end
        else if (rosen_cnt == {(8){1'b0}})
          begin
             rosen_cnt   <= rosen_cnt;
             rosen <= 1'b0;
          end
        else
          begin
             rosen_cnt <= rosen_cnt - {{(8-1){1'd0}}, 1'd1};
             rosen <= 1'b1;
          end
     end

   assign     Ros_En =  rosen; 


   //////////////////////////////////////////////////////////////////////////////////
   
endmodule // ros_cntl

