//                              -*- Mode: Verilog -*-
////////////////////////////////////////////////////////////////////////////
/*
 *
 *  Trade secret of ATI Technologies, Inc.
 *  Copyright 2005, ATI Technologies, Inc., (unpublished)
 *
 *  All rights reserved.  This notice is intended as a precaution against
 *  inadvertent publication and does not imply publication or any waiver
 *  of confidentiality.  The year included in the foregoing notice is the
 *  year of creation of the work.
 *
 */
////////////////////////////////////////////////////////////////////////////
//
// Filename    : ros.v
// Description : Performance Ring Oscillator
//
////////////////////////////////////////////////////////////////////////////



//`include "ros_features.vh"


module ros_gc10 
(
 //Broadcast to All TDRs
 input wire Tdr_Tck,
 input wire Tdr_Trstb,
 input wire Tdr_Shift,
 input wire Tdr_Capture,
 input wire Tdr_Update
 
 ,input wire Tdr_ros_setup_En
 ,input wire Tdr_ros_setup_Tdi
 ,output wire Tdr_ros_setup_Tdo
 
 ,input wire Tdr_ros_status_En
 ,input wire Tdr_ros_status_Tdi
 ,output wire Tdr_ros_status_Tdo
 
 ,input wire Tile_Shift_Clk
 ,input wire Tile_ScanEn
 ,input wire Scm_Mode
 ,input wire Single_Chain_Daisy_In
 ,output wire Single_Chain_Daisy_Out
 
 
 ,input	wire Ext_Clk
 ,input	wire Ros_En
 );
   
   wire [4-1:0] cfg_bits;
   wire [1-1:0] counter_reset;
   wire [1-1:0] enable_all;
   wire [2-1:0] ros_size;
   wire [29-1:0] counter_bits;
   wire [1-1:0] overflow;
   wire [1-1:0] ros_clk_sel;
   wire Ext_Clk_gated;
   wire rosen;
   wire Scm_Mode_buf;


ati_buffer1 mrk_scm_mode          (.A(Scm_Mode),             .Y(Scm_Mode_buf));

   ros_tdrs_gc10 ros_tdrs 
     (
      .Tile_Shift_Clk       (Tile_Shift_Clk),
      .Tile_ScanEn          (Tile_ScanEn),
      .Scm_Mode             (Scm_Mode_buf),
      .Single_Chain_Daisy_In (Single_Chain_Daisy_In),
      .Single_Chain_Daisy_Out (Single_Chain_Daisy_Out),
      
      .Tdr_Tck               (Tdr_Tck) ,
      .Tdr_Trstb             (Tdr_Trstb),
      .Tdr_ros_setup_Tdi     (Tdr_ros_setup_Tdi),
      .Tdr_ros_status_Tdi    (Tdr_ros_status_Tdi),
      .Tdr_Shift             (Tdr_Shift),
      .Tdr_Capture           (Tdr_Capture),
      .Tdr_Update            (Tdr_Update)
      
      
      //TDR --> ROS_SETUP
      ,.ros_clk_sel           (ros_clk_sel[1-1:0])
      ,.cfg_bits              (cfg_bits[4-1:0])
      ,.counter_reset         (counter_reset[1-1:0])
      ,.enable_all            (enable_all[1-1:0])
      ,.ros_size              (ros_size[2-1:0])
      ,.Tdr_ros_setup_En      (Tdr_ros_setup_En)
      ,.Tdr_ros_setup_Tdo     (Tdr_ros_setup_Tdo)
      
      //TDR --> ROS_STATUS
      ,.counter_bits         (counter_bits[29-1:0])
      ,.overflow             (overflow)
      ,.Tdr_ros_status_En            (Tdr_ros_status_En)
      ,.Tdr_ros_status_Tdo           (Tdr_ros_status_Tdo)

      ,.Ext_Clk           (Ext_Clk)
      ,.Ext_Clk_gated           (Ext_Clk_gated)
      ,.Ros_En           (Ros_En)
      ,.rosen           (rosen)
      );
   
   
   ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
   //PERFROKABA1 ros_macro (
   //PERFRO16_14LPE ros_macro (
   PERFRO16_TSMCN7B1 perfro_macro 
     (

`ifdef DFX_MACRO_EXPOSE_PWR_PINS
`ifndef SYNTHESIS
   .VDD_RO (1'b1),
   .VDD_CNTRL (1'b1),
   .VSS (1'b0),
`endif
`else
`endif

      .ext_clk (Ext_Clk_gated),
      .clk_sel (ros_clk_sel[1-1:0]),
      .RO_en(rosen),
      .RO_en_all(enable_all),
      .RO_cntr_reset(counter_reset),
      .RO_cntr_ovflw(overflow),
      .RO_cntr(counter_bits[29-1:0]),
      .RO_ocv_sel(ros_size[1:0]),
      .RO_sel(    cfg_bits[3:0])

      );
endmodule // ros


