//                              -*- Mode: Verilog -*-
////////////////////////////////////////////////////////////////////////////
/*
 *
 *  Trade secret of ATI Technologies, Inc.
 *  Copyright 2005, ATI Technologies, Inc., (unpublished)
 *
 *  All rights reserved.  This notice is intended as a precaution against
 *  inadvertent publication and does not imply publication or any waiver
 *  of confidentiality.  The year included in the foregoing notice is the
 *  year of creation of the work.
 *
 */
////////////////////////////////////////////////////////////////////////////
//
// Filename    : ros.v
// Description : Ring Oscillator
//
////////////////////////////////////////////////////////////////////////////



//`include "ros_features.vh"


module ros_tdrs_gc10 
(
 //Broadcast to All TDRs
 input wire Tdr_Tck,
 input wire Tdr_Trstb,
 input wire Tdr_Shift,
 input wire Tdr_Capture,
 input wire Tdr_Update
 
 ,input wire Tdr_ros_setup_En
 ,input wire Tdr_ros_setup_Tdi
 ,output wire Tdr_ros_setup_Tdo
 
 ,input wire Tdr_ros_status_En
 ,input wire Tdr_ros_status_Tdi
 ,output wire Tdr_ros_status_Tdo
 
 ,input wire Tile_Shift_Clk
 ,input wire Tile_ScanEn
 ,input wire Scm_Mode
 ,input wire Single_Chain_Daisy_In
 ,output wire Single_Chain_Daisy_Out
 
 ,output   wire [4-1:0] cfg_bits
 ,output   wire [1-1:0] counter_reset
 ,output   wire [1-1:0] enable_all
 ,output   wire [2-1:0] ros_size
 ,input   wire [29-1:0] counter_bits
 ,input   wire [1-1:0] overflow
 ,output   wire [1-1:0] ros_clk_sel
 
 ,input	 wire Ext_Clk
 ,output wire Ext_Clk_gated
 ,input	wire Ros_En
 ,output	wire rosen


 );
   
   ati_buffer1  mrk_rosen (.A(Ros_En), .Y(rosen));
   
   ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
     //SCM Marker Logic
   wire  rosen_sync;
   wire [29-1:0] counter_bits_muxed;
   wire Tdr_Tck_muxed;
   wire overflow_muxed;
   wire Tdr_ros_status_En_muxed;
   wire Tdr_Shift_muxed;
   wire Tdr_ros_status_Tdi_muxed;
   
   
   
   //Default load the scm_mrk reset value.
   assign counter_bits_muxed[29-1:0]  = ~Tdr_ros_status_En ? 
	  { 23'b110_1011_1100_0101_1111_0110 , cfg_bits, ros_size } :
          counter_bits;
   
   assign overflow_muxed = ~Tdr_ros_status_En ? enable_all : overflow;
   
   aticlk_mux2 clk_mux         
     ( 
       .clk_o(Tdr_Tck_muxed),
       .clk_i({Tile_Shift_Clk, Tdr_Tck}),
       .sel(Scm_Mode) );    //CDC waiver. Shift_clk not toggling when switch happen.
   
   assign Tdr_ros_status_Tdi_muxed  = Scm_Mode ? Single_Chain_Daisy_In : Tdr_ros_status_Tdi;

   //For secure scan_cntl, Scm_Mode can't be programmed and 1. For such tiles, bypass ros markers, else scm chain would be broken.
   assign Single_Chain_Daisy_Out    = (counter_reset & Scm_Mode) ? Tdr_ros_status_Tdo : Single_Chain_Daisy_In; // counter_reset default 1
   
 //Load mrk_flops during scan preamble.
   assign Tdr_ros_status_En_muxed   = Scm_Mode | Tdr_ros_status_En | Tdr_Capture;
   assign Tdr_Shift_muxed           = Scm_Mode ? Tile_ScanEn : Tdr_Shift; //CDC waiver. 
   
   
   ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
   

   ros_setup_tdr_gc10 ros_setup_tdr 
     (
      
      .Tdr_Tck               (Tdr_Tck) ,
      .Tdr_Trstb             (Tdr_Trstb),
      .Tdr_Tdi               (Tdr_ros_setup_Tdi),
      .Tdr_Shift             (Tdr_Shift),
      .Tdr_Capture           (Tdr_Capture),
      .Tdr_Update            (Tdr_Update)
      
      
      //TDR --> ROS_SETUP
      ,.ros_clk_sel           (ros_clk_sel[1-1:0])
      ,.cfg_bits              (cfg_bits[4-1:0])
      ,.counter_reset         (counter_reset[1-1:0])
      ,.enable_all            (enable_all[1-1:0])
      ,.ros_size              (ros_size[2-1:0])
      ,.Tdr_ros_setup_En      (Tdr_ros_setup_En)
      ,.Tdr_ros_setup_Tdo     (Tdr_ros_setup_Tdo)
      );
   
   
   ros_status_tdr_gc10 ros_status_tdr 
     (
      
      .Tdr_Tck               (Tdr_Tck_muxed) ,
      .Tdr_Trstb             (Tdr_Trstb),
      .Tdr_Tdi               (Tdr_ros_status_Tdi_muxed),
      .Tdr_Shift             (Tdr_Shift_muxed),
      .Tdr_Capture           (Tdr_Capture),
      .Tdr_Update            (Tdr_Update)
      
      
      //TDR --> ROS_STATUS
      ,.counter_bits         (counter_bits_muxed[29-1:0])
      ,.overflow             (overflow_muxed)
      ,.Tdr_ros_status_En            (Tdr_ros_status_En_muxed)
      ,.Tdr_ros_status_Tdo           (Tdr_ros_status_Tdo)
      );
   
   
   
   ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

   dfx_sync3_gc10 #(
      .TransportDelay(0),
      .TransportCycle(10)   // rosen routed from ros_cntl to all ros in container. 10 func_clk cycle.
      ) sync_rosen
     (
      .CK(Ext_Clk),
      .D(rosen),
      .Q(rosen_sync));
   
   ati_clock_gate ext_DfxCLOCKGATER
     (.clk_out   (Ext_Clk_gated),
      .clk_src   (Ext_Clk),
      .clk_en    (rosen_sync),
      .TE        (1'b0));
   
   
   
   
   ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
endmodule // ros_tdrs


