
module rst_combine_4 (clk,
                      // input resetb
                      RCU_blk_hard_resetb,
                      xRBM_blk_soft_0_resetb,
                      xRBM_blk_soft_1_resetb,
                      xRBM_blk_soft_2_resetb,
                      xRBM_blk_soft_3_resetb,
                      hard_resetb,
                      // output Combined resetb
                      combined_0_resetb,
                      combined_1_resetb,
                      combined_2_resetb,
                      combined_3_resetb);

   //Local clock of the block
   input  clk;            

   //hard_resetb from RCU
   input  RCU_blk_hard_resetb;

   //soft_resetb from SRBM or GRBM
   input  xRBM_blk_soft_0_resetb;
   input  xRBM_blk_soft_1_resetb;
   input  xRBM_blk_soft_2_resetb;
   input  xRBM_blk_soft_3_resetb;
  
   //Synchronized version of RCU_blk_hard_resetb
   output hard_resetb;

   //Combined (hard & soft) resetb
   output combined_0_resetb;
   output combined_1_resetb;
   output combined_2_resetb;
   output combined_3_resetb;


   
   wire   RCU_blk_hard_resetb_sync;
   wire   xRBM_blk_soft_0_resetb_sync;
   wire   xRBM_blk_soft_1_resetb_sync;
   wire   xRBM_blk_soft_2_resetb_sync;
   wire   xRBM_blk_soft_3_resetb_sync;
   
   techind_sync_icd #(.depth(3)) sync_hard   (.CKdst(clk), .Dsrc(RCU_blk_hard_resetb), .Qdst(RCU_blk_hard_resetb_sync),
                                 .SB(1'b1), .RB(1'b1), .FLUSH(1'b0), .CKsrc(1'b0), .EN(1'b1), .Qsrc(), 
                                 .SC1(1'b0), .SC2(1'b0), .SE(1'b0), .SI(1'b0), .SO()
   `ifdef ABV
                                 ,.InputChkDis(1'b1)
   `endif
   );

   techind_sync_icd #(.depth(3)) sync_soft_0   (.CKdst(clk), .Dsrc(xRBM_blk_soft_0_resetb), .Qdst(xRBM_blk_soft_0_resetb_sync),
                                   .SB(1'b1), .RB(1'b1), .FLUSH(1'b0), .CKsrc(1'b0), .EN(1'b1), .Qsrc(), 
                                   .SC1(1'b0), .SC2(1'b0), .SE(1'b0), .SI(1'b0), .SO()
   `ifdef ABV
                                 ,.InputChkDis(1'b1)
   `endif
   );
   techind_sync_icd #(.depth(3)) sync_soft_1   (.CKdst(clk), .Dsrc(xRBM_blk_soft_1_resetb), .Qdst(xRBM_blk_soft_1_resetb_sync),
                                   .SB(1'b1), .RB(1'b1), .FLUSH(1'b0), .CKsrc(1'b0), .EN(1'b1), .Qsrc(), 
                                   .SC1(1'b0), .SC2(1'b0), .SE(1'b0), .SI(1'b0), .SO()
   `ifdef ABV
                                 ,.InputChkDis(1'b1)
   `endif
   );
   techind_sync_icd #(.depth(3)) sync_soft_2   (.CKdst(clk), .Dsrc(xRBM_blk_soft_2_resetb), .Qdst(xRBM_blk_soft_2_resetb_sync),
                                   .SB(1'b1), .RB(1'b1), .FLUSH(1'b0), .CKsrc(1'b0), .EN(1'b1), .Qsrc(), 
                                   .SC1(1'b0), .SC2(1'b0), .SE(1'b0), .SI(1'b0), .SO()
   `ifdef ABV
                                 ,.InputChkDis(1'b1)
   `endif
   );
   techind_sync_icd #(.depth(3)) sync_soft_3   (.CKdst(clk), .Dsrc(xRBM_blk_soft_3_resetb), .Qdst(xRBM_blk_soft_3_resetb_sync),
                                   .SB(1'b1), .RB(1'b1), .FLUSH(1'b0), .CKsrc(1'b0), .EN(1'b1), .Qsrc(), 
                                   .SC1(1'b0), .SC2(1'b0), .SE(1'b0), .SI(1'b0), .SO()
   `ifdef ABV
                                 ,.InputChkDis(1'b1)
   `endif
   );




   //Adding one stage of flops, so that CDC tools do not complain of ANDing two sync cell outputs
   reg    RCU_blk_hard_resetb_sync_ff;
   reg    hard_resetb;

   reg    xRBM_blk_soft_0_resetb_sync_ff;
   reg    xRBM_blk_soft_1_resetb_sync_ff;
   reg    xRBM_blk_soft_2_resetb_sync_ff;
   reg    xRBM_blk_soft_3_resetb_sync_ff;

   reg    combined_0_resetb;
   reg    combined_1_resetb;
   reg    combined_2_resetb;
   reg    combined_3_resetb;

   always @ (posedge clk) begin
      RCU_blk_hard_resetb_sync_ff    <= RCU_blk_hard_resetb_sync;
      hard_resetb                    <= RCU_blk_hard_resetb_sync_ff;

      xRBM_blk_soft_0_resetb_sync_ff   <= xRBM_blk_soft_0_resetb_sync;
      xRBM_blk_soft_1_resetb_sync_ff   <= xRBM_blk_soft_1_resetb_sync;
      xRBM_blk_soft_2_resetb_sync_ff   <= xRBM_blk_soft_2_resetb_sync;
      xRBM_blk_soft_3_resetb_sync_ff   <= xRBM_blk_soft_3_resetb_sync;

      combined_0_resetb                <= RCU_blk_hard_resetb_sync_ff & xRBM_blk_soft_0_resetb_sync_ff;
      combined_1_resetb                <= RCU_blk_hard_resetb_sync_ff & xRBM_blk_soft_1_resetb_sync_ff;
      combined_2_resetb                <= RCU_blk_hard_resetb_sync_ff & xRBM_blk_soft_2_resetb_sync_ff;
      combined_3_resetb                <= RCU_blk_hard_resetb_sync_ff & xRBM_blk_soft_3_resetb_sync_ff;
   end
         
endmodule // rst_combine

