
//================================================================================================================================
//
// Copyright (c) 2002 by Advanced Micro Devices, Inc.
//
// This file is protected by Federal Copyright Law, with all rights reserved. No part of this file may be reproduced, stored in a
// retrieval system, translated, transcribed, or transmitted, in any form, or by any means manual, electric, electronic,
// mechanical, electro-magnetic, chemical, optical, or otherwise, without prior explicit written permission from Advanced Micro
// Devices, Inc.
//
//================================================================================================================================

//================================================================================================================================
//
// <REFERENCES>
//
//  1 http://twiki.amd.com/twiki/pub/DFTCOE/Meeting_110912/Overview_of_2014_Hierarchical_DFx_Architecture5.pdf
//  2 http://twiki.amd.com/twiki/bin/view/DFTCOE/HierarchicalModularJtag30
//  3 See UBTS 436890 - ALTTAP/P1687 feature
//  4 See UBTS 448813 - SIB Query Update feature
//  5 See UBTS 448812 - 1500 Broadcast feature
//  6 http://twiki.amd.com/twiki/bin/viewauth/DFTCOE/HierarchicalAltTapEn
//  7 See UBTS 459043 - SingleChain feature
//  8 http://twiki.amd.com/twiki/pub/SOCArch/SOC15DFX/STAC_Architecture_Overview4.pdf
//  9 See UBTS 465634 - SJC override feature
// 10 See UBTS 470355 - 1500 Selective Broadcast feature
// 11 See UBTS 472538 - STAC router feature
// 12 See UBTS 473811 - AEB security feature
// 13 See UBTS 476786 - Remove private bscan instructions from STAC controllers
// 14 See UBTS 484316 - STAC TAP (IP) controller should have PipelineEn input
// 15 See UBTS 556234 - Timing Loop in JTAG flop Reset -> Q -> Reset
// 16 See UBTS 558454 - STAC should have pipeline flop on WSO enabled by PipelineEn input
// 17 See UBTS 550459 - STAC ALTTAP MODE - Resets ALTTAPEN tdr after reset
// 17 See UBTS 587368 - Timing loop through STAC router and child STAC
// 18 See http://ontrack-internal.amd.com/browse/DECDFXIP41-141
//
//================================================================================================================================

//================================================================================================================================
//
// <PERLOG>
//
//  This file was generated using perlog.
//
//  Run directory:
//    /proj/gfxip_gcregr_1/user/orlvalid/gfx10.1_navi10_rtl_freeze_atl/import/dfx/stac/src/rtl
//
//  Commandline:
//    /proj/verif_release_ro/perlog/40/perlog.pl -C /proj/gfxip_gcregr_1/user/orlvalid/gfx10.1_navi10_rtl_freeze_atl/out/linux_2.6.32_64.VCS/navi10/library/stac-gc10_sipr16-dfxip4-gc10/pub/src/meta/tools/staccfg/default_dfx_stac.plcfg -C /proj/gfxip_gcregr_1/user/orlvalid/gfx10.1_navi10_rtl_freeze_atl/out/linux_2.6.32_64.VCS/navi10/library/stac-gc10_sipr16-dfxip4-gc10/pub/src/meta/cfg4dfx/rtl_dfx_stac_gc10_sipr16.plcfg -D LIB_PRFX=DFXIP4_GC10
//
//  Config variables:
//    IP_NAME         : gc10_sipr16
//    LIB_PRFX        : DFXIP4_GC10
//    AEB             : 0
//    NUM_1500        : 0
//    NUM_ALTTAP      : 0
//    NUM_SLAVETAP    : 0
//    NUM_STAC        : 16
//    PIPELINE        : 1
//    RESYNC_STAC     : ()
//    ROUTER          : 1
//    SINGLECHAIN     : 0
//    RESET_BC1500    : ()
//    NUM_TDR_RESETS  : 0
//    EARLY_1500      : ()
//
//
//  Supported instruction opcodes:
//    8'h00   WBYPASS
//    8'h02   IDCODE
//    8'h03   BC1500
//    8'h04   BC1500_QRYUPD
//    8'h05   ALTTAPEN
//    8'h07   SYSSCANSHIFT
//    8'h09   SCANCONFIG
//    8'h0A   SELECTDIE
//    8'h0B   BROADCAST
//    8'h13   P1687A
//    8'h14   P1687A_QRYUPD
//    8'h20   SYSSCANSHIFT2
//    8'h21   SYSSCAN
//    8'h25   QRYUPD
//    8'hFF   BYPASS
//
//================================================================================================================================

//================================================================================================================================
//
// <DESCRIPTION>
//
// Useful Abbreviations
//
//  SWB - SelectWIR Bit
//  WIR - (IEEE1500) Wrapper Instruction Register
//  WDR - (IEEE1500) Wrapper Data Register
//  TDR - (IEEE1149.1) Test Data Register
//  BC  - BroadCast
//  SBC - Selective BroadCast
//  SIB - Segment Inclusion Bit
//  SEB - SIB Exclusion Bit
//  WSP - Wrapper Serial Port
//  AEB - Access Enablement Bit
//  SJC - SMN to JTAG Converter
//
// Mode1 - STAC (1500 + FSM)
//          Use the SWB to switch between the 'instruction' or 'data'.
//          The FSM stays in the 1149.1 'data' stack.
//          JTAG1687A can be used to access additional child STACs.
//    Stac_AltTapEn == 0
//    WSI -> LocCORE_Wsi -> SWB (SelectWIR) -> WIR                 -> LocCORE_Wso -> WSO
//                                          -> WBYPASS   -> MuxWso ->
//                                          -> JTAG1687A ->
//                                          -> WDR       ->
// Mode2 - STAC (1149.1)
//          Use the FSM to switch between the 'instruction' or 'data',
//          by traversing through the 1149.1 'instruction or 'data' stack, respectively.
//    Stac_AltTapEn == 1
//    AltTapEnReg[0] == 1
//    WSI -> AltTapTdi[0] -> LocCORE_Wsi -> WIR               -> LocCORE_Wso -> AltTapDrvTdo -> WSO
//                                       -> WBYPASS -> MuxWso ->
//                                       -> WDR     ->
//
// Mode3 - ALTTAP (1149.1)
//          Daisy chain up all the enabled ALTTAP.
//          Freeze the STAC and FSM in RTI by disabling its clock and reset.
//    Stac_AltTapEn == 1
//    AltTapEnReg[0] == 0
//    AltTapEnReg[17-1:1] == 1
//    WSI -> ALTTAP[17-1] -> ... -> ALTTAP[2] -> ALTTAP[1] -> AltTapDrvTdo -> WSO
//
// Mode4 - ALTTAP (1149.1) plus local STAC (1149.1
//          Daisy chain up all the enabled ALTTAP including the local STAC.
//          Use the FSM to switch between the 'instruction' or 'data',
//          by traversing through the 1149.1 'instruction or 'data' stack, respectively.
//    Stac_AltTapEn == 1
//    AltTapEnReg[0] == 1
//    AltTapEnReg[17-1:1] == 1
//    WSI -> ALTTAP[17-1] -> ... -> ALTTAP[1] -> LocCORE_Wsi -> WIR               -> LocCORE_Wso -> AltTapDrvTdo -> WSO
//                                                                        -> WBYPASS -> MuxWso ->
//                                                                        -> WDR     ->
//
//
//================================================================================================================================
// leda B_1001 off
// leda B_1011 off
//~ format_rtl 1
//~ format_inst narrow

`timescale 1ps/1ps

module gc10_sipr16_STAC (

// Interface to the STAC's child STACs
  output  wire                                  STAC_0_SelectWSP,         // Connect this to the child's SelectWSP
  output  wire                                  STAC_0_Tms,               // Connect this to the child's TMS
  output  wire                                  STAC_0_Wrck,              // Connect this to the child's WRCK
  output  wire                                  STAC_0_WrstN,             // Connect this to the child's WRSTN
  output  wire                                  STAC_0_Wsi,               // Connect this to the child's WSI
  input                                         STAC_0_Wso,               // Connect this to the child's WSO
  input                                         STAC_0_AltTdoEn,          // Connect this to the child's AltTdoEn
  output wire                                   STAC_0_AltTapEn,          // Broadcast from MTAP.
  output  wire                                  STAC_0_PipelineEn,
// Interface to the STAC's child STACs
  output  wire                                  STAC_1_SelectWSP,         // Connect this to the child's SelectWSP
  output  wire                                  STAC_1_Tms,               // Connect this to the child's TMS
  output  wire                                  STAC_1_Wrck,              // Connect this to the child's WRCK
  output  wire                                  STAC_1_WrstN,             // Connect this to the child's WRSTN
  output  wire                                  STAC_1_Wsi,               // Connect this to the child's WSI
  input                                         STAC_1_Wso,               // Connect this to the child's WSO
  input                                         STAC_1_AltTdoEn,          // Connect this to the child's AltTdoEn
  output wire                                   STAC_1_AltTapEn,          // Broadcast from MTAP.
  output  wire                                  STAC_1_PipelineEn,
// Interface to the STAC's child STACs
  output  wire                                  STAC_2_SelectWSP,         // Connect this to the child's SelectWSP
  output  wire                                  STAC_2_Tms,               // Connect this to the child's TMS
  output  wire                                  STAC_2_Wrck,              // Connect this to the child's WRCK
  output  wire                                  STAC_2_WrstN,             // Connect this to the child's WRSTN
  output  wire                                  STAC_2_Wsi,               // Connect this to the child's WSI
  input                                         STAC_2_Wso,               // Connect this to the child's WSO
  input                                         STAC_2_AltTdoEn,          // Connect this to the child's AltTdoEn
  output wire                                   STAC_2_AltTapEn,          // Broadcast from MTAP.
  output  wire                                  STAC_2_PipelineEn,
// Interface to the STAC's child STACs
  output  wire                                  STAC_3_SelectWSP,         // Connect this to the child's SelectWSP
  output  wire                                  STAC_3_Tms,               // Connect this to the child's TMS
  output  wire                                  STAC_3_Wrck,              // Connect this to the child's WRCK
  output  wire                                  STAC_3_WrstN,             // Connect this to the child's WRSTN
  output  wire                                  STAC_3_Wsi,               // Connect this to the child's WSI
  input                                         STAC_3_Wso,               // Connect this to the child's WSO
  input                                         STAC_3_AltTdoEn,          // Connect this to the child's AltTdoEn
  output wire                                   STAC_3_AltTapEn,          // Broadcast from MTAP.
  output  wire                                  STAC_3_PipelineEn,
// Interface to the STAC's child STACs
  output  wire                                  STAC_4_SelectWSP,         // Connect this to the child's SelectWSP
  output  wire                                  STAC_4_Tms,               // Connect this to the child's TMS
  output  wire                                  STAC_4_Wrck,              // Connect this to the child's WRCK
  output  wire                                  STAC_4_WrstN,             // Connect this to the child's WRSTN
  output  wire                                  STAC_4_Wsi,               // Connect this to the child's WSI
  input                                         STAC_4_Wso,               // Connect this to the child's WSO
  input                                         STAC_4_AltTdoEn,          // Connect this to the child's AltTdoEn
  output wire                                   STAC_4_AltTapEn,          // Broadcast from MTAP.
  output  wire                                  STAC_4_PipelineEn,
// Interface to the STAC's child STACs
  output  wire                                  STAC_5_SelectWSP,         // Connect this to the child's SelectWSP
  output  wire                                  STAC_5_Tms,               // Connect this to the child's TMS
  output  wire                                  STAC_5_Wrck,              // Connect this to the child's WRCK
  output  wire                                  STAC_5_WrstN,             // Connect this to the child's WRSTN
  output  wire                                  STAC_5_Wsi,               // Connect this to the child's WSI
  input                                         STAC_5_Wso,               // Connect this to the child's WSO
  input                                         STAC_5_AltTdoEn,          // Connect this to the child's AltTdoEn
  output wire                                   STAC_5_AltTapEn,          // Broadcast from MTAP.
  output  wire                                  STAC_5_PipelineEn,
// Interface to the STAC's child STACs
  output  wire                                  STAC_6_SelectWSP,         // Connect this to the child's SelectWSP
  output  wire                                  STAC_6_Tms,               // Connect this to the child's TMS
  output  wire                                  STAC_6_Wrck,              // Connect this to the child's WRCK
  output  wire                                  STAC_6_WrstN,             // Connect this to the child's WRSTN
  output  wire                                  STAC_6_Wsi,               // Connect this to the child's WSI
  input                                         STAC_6_Wso,               // Connect this to the child's WSO
  input                                         STAC_6_AltTdoEn,          // Connect this to the child's AltTdoEn
  output wire                                   STAC_6_AltTapEn,          // Broadcast from MTAP.
  output  wire                                  STAC_6_PipelineEn,
// Interface to the STAC's child STACs
  output  wire                                  STAC_7_SelectWSP,         // Connect this to the child's SelectWSP
  output  wire                                  STAC_7_Tms,               // Connect this to the child's TMS
  output  wire                                  STAC_7_Wrck,              // Connect this to the child's WRCK
  output  wire                                  STAC_7_WrstN,             // Connect this to the child's WRSTN
  output  wire                                  STAC_7_Wsi,               // Connect this to the child's WSI
  input                                         STAC_7_Wso,               // Connect this to the child's WSO
  input                                         STAC_7_AltTdoEn,          // Connect this to the child's AltTdoEn
  output wire                                   STAC_7_AltTapEn,          // Broadcast from MTAP.
  output  wire                                  STAC_7_PipelineEn,
// Interface to the STAC's child STACs
  output  wire                                  STAC_8_SelectWSP,         // Connect this to the child's SelectWSP
  output  wire                                  STAC_8_Tms,               // Connect this to the child's TMS
  output  wire                                  STAC_8_Wrck,              // Connect this to the child's WRCK
  output  wire                                  STAC_8_WrstN,             // Connect this to the child's WRSTN
  output  wire                                  STAC_8_Wsi,               // Connect this to the child's WSI
  input                                         STAC_8_Wso,               // Connect this to the child's WSO
  input                                         STAC_8_AltTdoEn,          // Connect this to the child's AltTdoEn
  output wire                                   STAC_8_AltTapEn,          // Broadcast from MTAP.
  output  wire                                  STAC_8_PipelineEn,
// Interface to the STAC's child STACs
  output  wire                                  STAC_9_SelectWSP,         // Connect this to the child's SelectWSP
  output  wire                                  STAC_9_Tms,               // Connect this to the child's TMS
  output  wire                                  STAC_9_Wrck,              // Connect this to the child's WRCK
  output  wire                                  STAC_9_WrstN,             // Connect this to the child's WRSTN
  output  wire                                  STAC_9_Wsi,               // Connect this to the child's WSI
  input                                         STAC_9_Wso,               // Connect this to the child's WSO
  input                                         STAC_9_AltTdoEn,          // Connect this to the child's AltTdoEn
  output wire                                   STAC_9_AltTapEn,          // Broadcast from MTAP.
  output  wire                                  STAC_9_PipelineEn,
// Interface to the STAC's child STACs
  output  wire                                  STAC_10_SelectWSP,         // Connect this to the child's SelectWSP
  output  wire                                  STAC_10_Tms,               // Connect this to the child's TMS
  output  wire                                  STAC_10_Wrck,              // Connect this to the child's WRCK
  output  wire                                  STAC_10_WrstN,             // Connect this to the child's WRSTN
  output  wire                                  STAC_10_Wsi,               // Connect this to the child's WSI
  input                                         STAC_10_Wso,               // Connect this to the child's WSO
  input                                         STAC_10_AltTdoEn,          // Connect this to the child's AltTdoEn
  output wire                                   STAC_10_AltTapEn,          // Broadcast from MTAP.
  output  wire                                  STAC_10_PipelineEn,
// Interface to the STAC's child STACs
  output  wire                                  STAC_11_SelectWSP,         // Connect this to the child's SelectWSP
  output  wire                                  STAC_11_Tms,               // Connect this to the child's TMS
  output  wire                                  STAC_11_Wrck,              // Connect this to the child's WRCK
  output  wire                                  STAC_11_WrstN,             // Connect this to the child's WRSTN
  output  wire                                  STAC_11_Wsi,               // Connect this to the child's WSI
  input                                         STAC_11_Wso,               // Connect this to the child's WSO
  input                                         STAC_11_AltTdoEn,          // Connect this to the child's AltTdoEn
  output wire                                   STAC_11_AltTapEn,          // Broadcast from MTAP.
  output  wire                                  STAC_11_PipelineEn,
// Interface to the STAC's child STACs
  output  wire                                  STAC_12_SelectWSP,         // Connect this to the child's SelectWSP
  output  wire                                  STAC_12_Tms,               // Connect this to the child's TMS
  output  wire                                  STAC_12_Wrck,              // Connect this to the child's WRCK
  output  wire                                  STAC_12_WrstN,             // Connect this to the child's WRSTN
  output  wire                                  STAC_12_Wsi,               // Connect this to the child's WSI
  input                                         STAC_12_Wso,               // Connect this to the child's WSO
  input                                         STAC_12_AltTdoEn,          // Connect this to the child's AltTdoEn
  output wire                                   STAC_12_AltTapEn,          // Broadcast from MTAP.
  output  wire                                  STAC_12_PipelineEn,
// Interface to the STAC's child STACs
  output  wire                                  STAC_13_SelectWSP,         // Connect this to the child's SelectWSP
  output  wire                                  STAC_13_Tms,               // Connect this to the child's TMS
  output  wire                                  STAC_13_Wrck,              // Connect this to the child's WRCK
  output  wire                                  STAC_13_WrstN,             // Connect this to the child's WRSTN
  output  wire                                  STAC_13_Wsi,               // Connect this to the child's WSI
  input                                         STAC_13_Wso,               // Connect this to the child's WSO
  input                                         STAC_13_AltTdoEn,          // Connect this to the child's AltTdoEn
  output wire                                   STAC_13_AltTapEn,          // Broadcast from MTAP.
  output  wire                                  STAC_13_PipelineEn,
// Interface to the STAC's child STACs
  output  wire                                  STAC_14_SelectWSP,         // Connect this to the child's SelectWSP
  output  wire                                  STAC_14_Tms,               // Connect this to the child's TMS
  output  wire                                  STAC_14_Wrck,              // Connect this to the child's WRCK
  output  wire                                  STAC_14_WrstN,             // Connect this to the child's WRSTN
  output  wire                                  STAC_14_Wsi,               // Connect this to the child's WSI
  input                                         STAC_14_Wso,               // Connect this to the child's WSO
  input                                         STAC_14_AltTdoEn,          // Connect this to the child's AltTdoEn
  output wire                                   STAC_14_AltTapEn,          // Broadcast from MTAP.
  output  wire                                  STAC_14_PipelineEn,
// Interface to the STAC's child STACs
  output  wire                                  STAC_15_SelectWSP,         // Connect this to the child's SelectWSP
  output  wire                                  STAC_15_Tms,               // Connect this to the child's TMS
  output  wire                                  STAC_15_Wrck,              // Connect this to the child's WRCK
  output  wire                                  STAC_15_WrstN,             // Connect this to the child's WRSTN
  output  wire                                  STAC_15_Wsi,               // Connect this to the child's WSI
  input                                         STAC_15_Wso,               // Connect this to the child's WSO
  input                                         STAC_15_AltTdoEn,          // Connect this to the child's AltTdoEn
  output wire                                   STAC_15_AltTapEn,          // Broadcast from MTAP.
  output  wire                                  STAC_15_PipelineEn,

// STAC Interface to the SOC
  input                                         Stac_PipelineEn,
  input                                         Stac_SelectWSP,     // Master enable for the entire STAC
  input                                         Stac_Tms,
  input                                         Stac_Wrck,
  input                                         Stac_WrstN,
  input                                         Stac_Wsi,
  output  wire                                  Stac_Wso,
  input                                         Stac_AltTapEn,
  output  wire                                  Stac_AltTdoEn
);

// Sticking all the wire/reg declaration in one spot to
// keep the rest of the file from clutter.

  wire                                          PipelineWso;
  wire                                          NonPipelineWso;
  wire                                          LocCORE_Tms;
  wire                                          LocCORE_Wrck;
  wire                                          LocCORE_Wrst;
  wire                                          LocCORE_SyncWrst;
  wire                                          LocCORE_Wsi;
  wire                                          LocCORE_Wso;
  wire                                          LocCORE_WsoEn;
  wire                                          QRYUPD_En; // UBTS 528914
  wire                                          P1687A_En;
  wire                                          P1687A_QRYUPD_En;
  wire                                          P1687A_QRYUPD_only;
  wire                                          P1687A_QRYUPD_En_d1;
  wire                                          Jtag1687aSbc;
  wire                                          Jtag1687aSbc_m1;
  wire                                          Jtag1687aSbcTdo;
  wire                                          Jtag1687aSeb_m1;
  wire                                          Jtag1687aSeb;
  wire                                          Jtag1687aSebTdo;
  wire                                          Jtag1687aSibsTdo;
  wire                                          Jtag1687aSebSibsTdo;
  wire                                          Jtag1687aDrvTdi;
  wire                                          Jtag1687aDrvTdo;
  wire               [16-1:0]     Jtag1687aReg_m1;
  wire               [16-1:0]     Jtag1687aReg;
  wire               [16-1:0]     Jtag1687aAeb;
  wire               [16-1:0]     Jtag1687aSib;
  wire               [16-1:0]     Jtag1687aTms_pre;
  wire               [16-1:0]     Jtag1687aTms;
  wire               [16-1:0]     Jtag1687aTrst_pre;
  wire               [16-1:0]     Jtag1687aTrst;
  wire               [16-1:0]     Jtag1687aTdi;
  wire               [16-1:0]     Jtag1687aTdo;
  wire                                          AltTapTdi0;
  wire                                          ALTTAPEN_En;
  wire                                          AltTapEnTdo;
  wire                                          AltTapDrvTdo;
  reg                                           AltTapEnFall;
  wire                                          AltTapEn_d1;
  wire                [17-1:0]     AltTapEnReg;
  wire                [17-1:0]     AltTapEnAeb;
  wire                [17-1:0]     AltTapEnBit;
  wire                [17-1:0]     AltTapTms;
  wire                [17-1:0]     AltTapTckEn;
  wire                [17-1:0]     AltTapTrst;
  wire                [17-1:0]     AltTapTdi;
  wire                [17-1:0]     AltTapTdo;
  wire                [17-1:0]     AltTapTdoEn;
  wire                                          SYSSCANSHIFT_En;
  wire                                          SYSSCANSHIFT2_En;
  wire                                          SingleChainLocalSdo;
  wire                                          SysScanShiftMode;
  wire                                          AltTapEn_FsmEn;
  wire                                          AltTapEn_FsmDis;
  wire                                          AltTapEnReg0;
  wire                                          AltTapEnRegTrst;
  wire                                          TapInTlr;
  wire                                          TapInCapDR;
  wire                                          TapInShfDR;
  wire                                          TapInUpdDR;
  wire                                          TapInUpdIR;
  wire                                          BypassTdo;
  wire                                          Bypass_UNUSED;
  reg                                           MuxWso;
  wire                                [7:0]     WIR;
  wire                                [7:0]     WirShiftReg;
  wire                                          WirUpdate;
  wire                                          WdrCapture;
  wire                                          WdrShift;
  wire                                          WdrUpdate;
  wire                                          WdrWrck;
  wire                                          WdrWrst;
  wire                                          WdrWsi;

  //Route this signal through STAC routers.
assign STAC_0_AltTapEn   = Stac_AltTapEn;
assign STAC_0_PipelineEn = Stac_PipelineEn;
assign STAC_1_AltTapEn   = Stac_AltTapEn;
assign STAC_1_PipelineEn = Stac_PipelineEn;
assign STAC_2_AltTapEn   = Stac_AltTapEn;
assign STAC_2_PipelineEn = Stac_PipelineEn;
assign STAC_3_AltTapEn   = Stac_AltTapEn;
assign STAC_3_PipelineEn = Stac_PipelineEn;
assign STAC_4_AltTapEn   = Stac_AltTapEn;
assign STAC_4_PipelineEn = Stac_PipelineEn;
assign STAC_5_AltTapEn   = Stac_AltTapEn;
assign STAC_5_PipelineEn = Stac_PipelineEn;
assign STAC_6_AltTapEn   = Stac_AltTapEn;
assign STAC_6_PipelineEn = Stac_PipelineEn;
assign STAC_7_AltTapEn   = Stac_AltTapEn;
assign STAC_7_PipelineEn = Stac_PipelineEn;
assign STAC_8_AltTapEn   = Stac_AltTapEn;
assign STAC_8_PipelineEn = Stac_PipelineEn;
assign STAC_9_AltTapEn   = Stac_AltTapEn;
assign STAC_9_PipelineEn = Stac_PipelineEn;
assign STAC_10_AltTapEn   = Stac_AltTapEn;
assign STAC_10_PipelineEn = Stac_PipelineEn;
assign STAC_11_AltTapEn   = Stac_AltTapEn;
assign STAC_11_PipelineEn = Stac_PipelineEn;
assign STAC_12_AltTapEn   = Stac_AltTapEn;
assign STAC_12_PipelineEn = Stac_PipelineEn;
assign STAC_13_AltTapEn   = Stac_AltTapEn;
assign STAC_13_PipelineEn = Stac_PipelineEn;
assign STAC_14_AltTapEn   = Stac_AltTapEn;
assign STAC_14_PipelineEn = Stac_PipelineEn;
assign STAC_15_AltTapEn   = Stac_AltTapEn;
assign STAC_15_PipelineEn = Stac_PipelineEn;

//=============================================================================
// STAC FSM, SWB, and WIR
//=============================================================================
// Pipeline through this flop to prevent false combinatorial loop. UBTS 556234

DFXIP4_GC10_IP_JTAGFLOP I_ALTTAPENREG0PIPL (
  .ASYNCCLK                ( 1'b0           ), // intentionally no reset
  .ASYNCVAL                ( 1'b0           ),
  .CLK                     ( WdrWrck        ),
  .D                       ( AltTapEnReg[0] ),
  .Q                       ( AltTapEnReg0   )
);

// Synchronizing reset deassertion for logic outside TAPFSM. UBTS 598071
DFXIP4_GC10_IP_RST_SYNC I_WRST_SYNC (
  .SampleClk ( ~Stac_Wrck  ), // Falling edge deasserts half clk earlier, minimizing impact of synchronizer delay. TapInTlr asserts on falling edge as well.
  .RstIn     ( ~Stac_WrstN ),
  .RstOut    (  SyncWrst   )
);

// We're in Stac_AltTapEn but since bit zero was set, we enable the
// local STAC in 1149.1 mode as if it was another ALTTAP.
assign AltTapEn_FsmEn  = Stac_AltTapEn &  AltTapEnReg0;

// We're in Stac_AltTapEn but since bit zero was not set,
// we disable the local STAC and keep it in RTI.
assign AltTapEn_FsmDis = Stac_AltTapEn & ~AltTapEnReg0;

// Intercept the controls to this local STAC depending on the mode.

// Extra gating is causing a timing loop during synthesis.
// Discussed this with Sopho and the gating on the clock
// is not really needed as long as we gate off the TMS and
// WRSTN to keep the FSM in RTI.  UBTS 442008
//! unused AltTapTckEn[0]
//! unused AltTapTms[0]
//! unused AltTapTrst[0]
// Marker buffer to use in disable_timing to break false combinatorial loop, see of UBTS 587368
DFXIP4_GC10_IP_MARKER I_Mrk_AltTapTdi0 (
  .A(AltTapTdi[0]),
  .Z(AltTapTdi0  )
);
assign LocCORE_Wsi     = AltTapEn_FsmEn ? AltTapTdi0   : Stac_Wsi;
assign LocCORE_Wrck    =                                  Stac_Wrck;
assign LocCORE_Tms     =                                  (Stac_Tms    & ~AltTapEn_FsmDis);
assign LocCORE_Wrst    =                                  (~Stac_WrstN & ~AltTapEn_FsmDis);
assign LocCORE_SyncWrst =                                 (  SyncWrst  & ~AltTapEn_FsmDis);
assign AltTapTdo[0]    =  AltTapEn_FsmEn ? LocCORE_Wso   : 1'b0;
assign AltTapTdoEn[0]  =  AltTapEn_FsmEn ? LocCORE_WsoEn : 1'b0;
assign AltTapEnRegTrst = ~AltTapEn_FsmEn ? LocCORE_Wrst  : 1'b0; //UBTS 550459

gc10_sipr16_STAC_CORE I_CORE (
  .Mode1149                ( AltTapEn_FsmEn            ),
  .ModeSingleChain         ( SYSSCANSHIFT_En           ),    // Don't use the SYSSCANSHIFT2
  .SelectWSP               ( Stac_SelectWSP            ),
  .Tms                     ( LocCORE_Tms               ),
  .Wrst                    ( LocCORE_Wrst              ),
  .SyncWrst                ( LocCORE_SyncWrst          ),
  .Wrck                    ( LocCORE_Wrck              ),
  .Wsi                     ( LocCORE_Wsi               ),
  .Wso                     ( LocCORE_Wso               ),
  .WsoEn                   ( LocCORE_WsoEn             ),
  .PipelineEn              ( Stac_PipelineEn           ),
  .TapInCapDR              ( TapInCapDR                ),
  .TapInShfDR              ( TapInShfDR                ),
  .TapInUpdDR              ( TapInUpdDR                ),
  .TapInUpdIR              ( TapInUpdIR                ),
  .TapInTlr                ( TapInTlr                  ),
  .TapInRti                (                           ),
  .TapInRtiNxt             (                           ),
  .JTAGFF_WdrUpdate        (                           ),
  .WdrCapture              ( WdrCapture                ),
  .WdrShift                ( WdrShift                  ),
  .WdrUpdate               ( WdrUpdate                 ),
  .WirShiftReg             ( WirShiftReg[7:0]          ),
  .WirUpdate               ( WirUpdate                 ),
  .WIR                     ( WIR[7:0]                  ),
  .WdrWrck                 ( WdrWrck                   ),
  .WdrWrst                 ( WdrWrst                   ),
  .WdrWsi                  ( WdrWsi                    ),
  .MuxWso                  ( MuxWso                    )
);


//=============================================================================
// WSI/WSO muxing
//=============================================================================

always @* begin
  case (^WIR[7:0])
    1'b0, 1'b1:
    begin
      case (WIR[7:0])
        // EMBEDDED_INSTR_OPCODE
        8'h13            : MuxWso = Jtag1687aSebSibsTdo; // P1687A         // UBTS 470941
        8'h14     : MuxWso = Jtag1687aSebSibsTdo; // P1687A_QRYUPD  // UBTS 470941
        8'h25            : MuxWso = Jtag1687aSebSibsTdo; // QRYUPD         // UBTS 528914
        8'h05          : MuxWso = AltTapEnTdo;         // ALTTAPEN
        8'h07      : MuxWso = SingleChainLocalSdo; // SYSSCANSHIFT   // UBTS 470941
        8'h20     : MuxWso = SingleChainLocalSdo; // SYSSCANSHIFT2  // UBTS 470941
        8'hFF            : MuxWso = BypassTdo;           // BYPASS
        8'h00           : MuxWso = BypassTdo;           // WBYPASS
        // DEFAULT_INSTR_OPCODE
        // S_USER_INSTR_OPCODE
        default                                     : MuxWso = BypassTdo;
      endcase
    end
    default                                         : MuxWso = 1'bx;
  endcase
end

// Stitch in the child STACs SIB muxing and 1500 clients SIB muxing.  UBTS 470941
// WSI -> Local STAC -> child STACs -> 1500 clients -> WSO

// Stitch in the child STACs after the local STAC (i.e. LocCORE_Wso)
// Marker buffer to use in disable_timing to break false combinatorial loop, see of UBTS 587368
DFXIP4_GC10_IP_MARKER I_Mrk_LocWso2Wsi (
  .Z(Jtag1687aDrvTdi),
  .A(LocCORE_Wso)
);

// Bypass the connection since no 1500 clients exist
  wire                                          Jtag1500bcDrvTdo;
assign Jtag1500bcDrvTdo                       = Jtag1687aDrvTdo;

// UBTS 558454
DFXIP4_GC10_IP_MUX I_WSO_MUX (
  .Sel ( Stac_PipelineEn ),
  .In0 (  NonPipelineWso ),
  .In1 (     PipelineWso ),
  .Out (        Stac_Wso )
);

assign NonPipelineWso = Stac_AltTapEn ? AltTapDrvTdo : Jtag1500bcDrvTdo;

DFXIP4_GC10_JTAGTDR_NO_UPD #(.LENGTH(1)) I_WSO_PIPELINE (
  .RSTVAL                  ( 1'b0                      ),
  .CAP_D                   ( 1'b0                      ),
  .TDR_Q                   ( Pipeline_UNUSED           ),
  .CAP                     ( 1'b0                      ),
  .SHF                     ( 1'b1                      ),
  .TRST                    ( ~Stac_WrstN               ),
  .TCK                     ( Stac_Wrck                 ),
  .TDI                     ( NonPipelineWso            ),
  .TDO                     (    PipelineWso            ),
  .SEL                     ( 1'b1                      )
);
//! unused Pipeline_UNUSED
//=============================================================================
// BYPASS
//=============================================================================

// The BYPASS is part of the default instruction set.

DFXIP4_GC10_JTAGTDR_NO_UPD #(.LENGTH(1)) I_BYPASS (
  .RSTVAL                  ( 1'b0                      ),
  .CAP_D                   ( 1'b0                      ),
  .TDR_Q                   ( Bypass_UNUSED             ),
  .CAP                     ( WdrCapture                ),
  .SHF                     ( WdrShift                  ),
  .TRST                    ( WdrWrst                   ),
  .TCK                     ( WdrWrck                   ),
  .TDI                     ( WdrWsi                    ),
  .TDO                     ( BypassTdo                 ),
  .SEL                     ( 1'b1                      )
);
//! unused Bypass_UNUSED

//=============================================================================
// DEFAULT instructions
//=============================================================================


//=============================================================================
// USER instructions
//=============================================================================


//=============================================================================
// SYSSCANSHIFT
//=============================================================================

// The SYSSCANSHIFT is part of the default instruction set.
// This bypasses the SWB, so we just have to time SC2 for scan shift.

DFXIP4_GC10_JTAGTDR_UPD I_SYSSCANSHIFT_En (
  .RSTVAL                  ( 1'b0                      ),
  .UPD_D                   ( WirShiftReg[7:0] == 8'h07 ),
  .TDR_Q                   ( SYSSCANSHIFT_En           ),
  .UPD                     ( WirUpdate                 ),
  .TRST                    ( WdrWrst | TapInTlr        ),
  .TCK                     ( WdrWrck                   ),
  .SEL                     ( 1'b1                      )
);

//=============================================================================
// SYSSCANSHIFT2
//=============================================================================

// The SYSSCANSHIFT2 is part of the default instruction set.
// Doesn't bypass the SWB, so we have to time TCK/WRCK/SC2 for scan shift.

DFXIP4_GC10_JTAGTDR_UPD I_SYSSCANSHIFT2_En (
  .RSTVAL                  ( 1'b0                      ),
  .UPD_D                   ( WirShiftReg[7:0] == 8'h20 ),
  .TDR_Q                   ( SYSSCANSHIFT2_En          ),
  .UPD                     ( WirUpdate                 ),
  .TRST                    ( WdrWrst | TapInTlr        ),
  .TCK                     ( WdrWrck                   ),
  .SEL                     ( 1'b1                      )
);

assign SysScanShiftMode = SYSSCANSHIFT_En | SYSSCANSHIFT2_En;

//=============================================================================
// ALTTAPEN
//=============================================================================

// The ALTTAPEN is part of the default instruction set.
// The length of this register is NUM_STAC (upper portion) + NUM_ALTTAP (lower portion) + 1 (for local STAC).
// Also remember that bit 0 is reserved to access this local STAC.

assign ALTTAPEN_En = (WIR[7:0] == 8'h05);

DFXIP4_GC10_JTAGTDR #(.LENGTH(17)) I_ALTTAPEN (
  .RSTVAL                  ( {17{1'b0}}          ),
  .CAP_D                   ( AltTapEnReg[17-1:0] ),
  .TDR_Q                   ( AltTapEnReg[17-1:0] ),
  .CAP                     ( WdrCapture                       ),
  .SHF                     ( WdrShift                         ),
  .UPD                     ( WdrUpdate                        ),
  .TRST                    ( AltTapEnRegTrst | AltTapEnFall   ),    // Resets when MTAP deasserts AltTapEn
  .TCK                     ( WdrWrck                          ),
  .TDI                     ( WdrWsi                           ),
  .TDO                     ( AltTapEnTdo                      ),
  .SEL                     ( ALTTAPEN_En                      )
);

DFXIP4_GC10_IP_JTAGFLOP I_ALTTAPEN_D1 (
  .ASYNCCLK                ( WdrWrst       ),
  .ASYNCVAL                ( 1'b0          ),
  .CLK                     ( WdrWrck       ),
  .D                       ( Stac_AltTapEn ),
  .Q                       ( AltTapEn_d1   )
);

always @* begin
  AltTapEnFall = ~Stac_AltTapEn & AltTapEn_d1;
end

// AEB feature was set to 0 so we'll enable everything
assign AltTapEnAeb[17-1:0] = {17{1'b1}};

assign QRYUPD_En   = (WIR[7:0] == 8'h25);        // UBTS 528914


//=============================================================================
// JTAG1687A
//=============================================================================

// The JTAG1687A is part of the default instruction set.
// The length of this register is just 16 + 1 SEB.
// Both TLR (TapInTlr) and TRST will reset SIBs, SWBs and any other network configuration bits.

assign P1687A_En          = (WIR[7:0] == 8'h13);
assign P1687A_QRYUPD_only = (WIR[7:0] == 8'h14); // UBTS 528914
assign P1687A_QRYUPD_En   = P1687A_QRYUPD_only | QRYUPD_En;                        // UBTS 528914

// UBTS 475115
DFXIP4_GC10_IP_JTAGFLOP I_P1687A_QRYUPD_EN_D1 (
  .ASYNCCLK                ( WdrWrst | TapInTlr        ),
  .ASYNCVAL                ( 1'b0                      ),
  .CLK                     ( WdrWrck                   ),
  .D                       ( P1687A_QRYUPD_En          ),
  .Q                       ( P1687A_QRYUPD_En_d1       )
);

// Selective Broadcast bit (for JTAG1687A) UBTS 504749
//  1 : broadcast mode
//  0 : default, serial mode

DFXIP4_GC10_JTAGTDR #(.LENGTH(1)) I_JTAG1687A_SBC (
  .RSTVAL                  ( 1'b0                         ),
  .CAP_D                   ( Jtag1687aSbc                 ),
  .TDR_Q                   ( Jtag1687aSbc_m1              ),
  .CAP                     ( WdrCapture                   ),
  .SHF                     ( WdrShift                     ),
  .UPD                     ( WdrUpdate                    ),
  .TRST                    ( WdrWrst | TapInTlr           ),
  .TCK                     ( WdrWrck                      ),
  .TDI                     ( WdrWsi                       ),
  .TDO                     ( Jtag1687aSbcTdo              ),
  .SEL                     ( P1687A_En | P1687A_QRYUPD_En )
);

DFXIP4_GC10_IP_JTAGFLOP I_JTAG1687A_SBC_D0 (
  .ASYNCCLK                ( WdrWrst | TapInTlr      ),
  .ASYNCVAL                ( 1'b0                    ),
  .CLK                     ( WdrWrck                 ),
  .D                       ( Jtag1687aSbc_m1         ),
  .Q                       ( Jtag1687aSbc            )
);
// SIB exclusion bit (for JTAG1687A)
//  1 : bypass & disable the SIBs
//  0 : default, include the SIBs

DFXIP4_GC10_JTAGTDR #(.LENGTH(1)) I_JTAG1687A_SEB (
  .RSTVAL                  ( 1'b0                      ),
  .CAP_D                   ( Jtag1687aSeb              ),
  .TDR_Q                   ( Jtag1687aSeb_m1           ),
  .CAP                     ( WdrCapture                ),
  .SHF                     ( WdrShift                  ),
  .UPD                     ( WdrUpdate                 ),
  .TRST                    ( WdrWrst | TapInTlr        ),
  .TCK                     ( WdrWrck                   ),
  .TDI                     ( Jtag1687aSbcTdo           ),
  .TDO                     ( Jtag1687aSebTdo           ),
  .SEL                     ( P1687A_En | P1687A_QRYUPD_En )
);

DFXIP4_GC10_IP_JTAGFLOP I_JTAG1687A_SEB_D0 (
  .ASYNCCLK                ( WdrWrst | TapInTlr        ),
  .ASYNCVAL                ( 1'b0                      ),
  .CLK                     ( WdrWrck                   ),
  .D                       ( Jtag1687aSeb_m1           ),
  .Q                       ( Jtag1687aSeb              )
);

// SIBs (for JTAG1687A)
//  1 : include the segment
//  0 : default, bypass the segment

DFXIP4_GC10_JTAGTDR #(.LENGTH(16)) I_JTAG1687A (
  .RSTVAL                  ( {16{1'b0}}  ),
  .CAP_D                   ( Jtag1687aReg[16-1:0] ),
  .TDR_Q                   ( Jtag1687aReg_m1[16-1:0] ),
  .CAP                     ( WdrCapture                ),
  .SHF                     ( WdrShift                  ),
  .UPD                     ( WdrUpdate                 ),
  .TRST                    ( WdrWrst | TapInTlr        ),
  .TCK                     ( WdrWrck                   ),
  .TDI                     ( Jtag1687aSebTdo           ),
  .TDO                     ( Jtag1687aSibsTdo          ),
//.SEL                     ( ~Jtag1687aSeb & (P1687A_En | P1687A_QRYUPD_En )     )   // UBTS 469977
  .SEL                     ( P1687A_QRYUPD_En | (~Jtag1687aSeb & P1687A_En )     )   // UBTS 474933
);

// UBTS 467380
DFXIP4_GC10_IP_JTAGFLOP I_JTAG1687A_D0[16-1:0] (
  .ASYNCCLK                ( {16{WdrWrst | TapInTlr}} ),
  .ASYNCVAL                ( {16{1'b0}}  ),
  .CLK                     ( {16{WdrWrck}} ),
  .D                       ( Jtag1687aReg_m1[16-1:0] ),
  .Q                       ( Jtag1687aReg[16-1:0] )
);

assign Jtag1687aSebSibsTdo = (P1687A_QRYUPD_En | ~Jtag1687aSeb) ? Jtag1687aSibsTdo : Jtag1687aSebTdo;

// AEB feature was set to 0 so we'll enable everything
assign Jtag1687aAeb[16-1:0] = {16{1'b1}};



//=============================================================================
// STAC Drivers
//=============================================================================

// Generate the control signals for the child STACs.

gc10_sipr16_STAC_DRIVER #(.NUM_DRIVER(16)) I_STAC_DRIVER (

  .DriverEn                ( Stac_SelectWSP            ),
  .AltTapEn                ( AltTapEn_FsmDis           ),
  .QryUpdEn                ( P1687A_QRYUPD_En_d1       ),
  .Trst                    ( ~Stac_WrstN               ),
  .Tms                     ( Stac_Tms                  ),
  .SyncClk                 ( Stac_Wrck                 ),
  .Jtag1687aAeb            ( Jtag1687aAeb[16-1:0] ),
  .Jtag1687aReg            ( Jtag1687aReg[16-1:0] ),
  .Jtag1687aSbc            ( Jtag1687aSbc              ), // UBTS 504749
  .Jtag1687aDrvTdi         ( Jtag1687aDrvTdi           ),
  .Jtag1687aDrvTdo         ( Jtag1687aDrvTdo           ),

// Intentionally left unused.  UBTS 470941
  .SingleChainEn           ( 1'b0                      ),
  .SingleChainSdi          ( 1'b0                      ),
  .SingleChainSdo          (                           ),

  .Jtag1687aSib            ( Jtag1687aSib[16-1:0] ),
  .Jtag1687aTms            ( Jtag1687aTms_pre[16-1:0] ),
  .Jtag1687aTrst           ( Jtag1687aTrst_pre[16-1:0] ),
  .Jtag1687aTdi            ( Jtag1687aTdi[16-1:0] ),
  .Jtag1687aTdo            ( Jtag1687aTdo[16-1:0] )
);



//=============================================================================
// STAC Interface for STAC_0
//=============================================================================

// Connect up the STAC ports to the STAC internal drivers.
// Also connect up the STAC ports to the ALTTAP internal drivers.
// Rename STAC ports for BIA compliance.

// Add one more to NUM_ALTTAP to account for the MTAP which uses bit 0.
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_0_SelectWSP (
  .Z(STAC_0_SelectWSP),
  .A(Stac_AltTapEn ?  AltTapEnBit[0+1+0] :  Jtag1687aSib[0])
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_0_Tms (
  .Z(STAC_0_Tms),
  .A(Stac_AltTapEn ?  AltTapTms[0+1+0]   :  Jtag1687aTms[0])
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_0_WrstN (
  .Z(STAC_0_WrstN),
  .A(Stac_AltTapEn ? ~AltTapTrst[0+1+0]  : ~Jtag1687aTrst[0])
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_0_Wsi (
  .Z(STAC_0_Wsi),
  .A(Stac_AltTapEn ?  AltTapTdi[0+1+0]   :  Jtag1687aTdi[0])
);
DFXIP4_GC10_IP_CLKMARKER I_Mrk_STAC_0_Wrck (
  .Z                         ( STAC_0_Wrck ),
  .A                         ( Stac_Wrck         )

);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_0_WsoStac (
  .Z(Jtag1687aTdo[0]),
  .A(STAC_0_Wso)
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_0_WsoAltTap (
  .Z(AltTapTdo[0+1+0]),
  .A(STAC_0_Wso)
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_0_AltTdoEn (
  .Z(AltTapTdoEn[0+1+0]),
  .A(STAC_0_AltTdoEn)
);

assign Jtag1687aTrst[0] = Jtag1687aTrst_pre[0];
assign Jtag1687aTms[0]  = Jtag1687aTms_pre[0];
//! unused STAC_0_Resync

//=============================================================================
// STAC Interface for STAC_1
//=============================================================================

// Connect up the STAC ports to the STAC internal drivers.
// Also connect up the STAC ports to the ALTTAP internal drivers.
// Rename STAC ports for BIA compliance.

// Add one more to NUM_ALTTAP to account for the MTAP which uses bit 0.
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_1_SelectWSP (
  .Z(STAC_1_SelectWSP),
  .A(Stac_AltTapEn ?  AltTapEnBit[0+1+1] :  Jtag1687aSib[1])
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_1_Tms (
  .Z(STAC_1_Tms),
  .A(Stac_AltTapEn ?  AltTapTms[0+1+1]   :  Jtag1687aTms[1])
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_1_WrstN (
  .Z(STAC_1_WrstN),
  .A(Stac_AltTapEn ? ~AltTapTrst[0+1+1]  : ~Jtag1687aTrst[1])
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_1_Wsi (
  .Z(STAC_1_Wsi),
  .A(Stac_AltTapEn ?  AltTapTdi[0+1+1]   :  Jtag1687aTdi[1])
);
DFXIP4_GC10_IP_CLKMARKER I_Mrk_STAC_1_Wrck (
  .Z                         ( STAC_1_Wrck ),
  .A                         ( Stac_Wrck         )

);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_1_WsoStac (
  .Z(Jtag1687aTdo[1]),
  .A(STAC_1_Wso)
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_1_WsoAltTap (
  .Z(AltTapTdo[0+1+1]),
  .A(STAC_1_Wso)
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_1_AltTdoEn (
  .Z(AltTapTdoEn[0+1+1]),
  .A(STAC_1_AltTdoEn)
);

assign Jtag1687aTrst[1] = Jtag1687aTrst_pre[1];
assign Jtag1687aTms[1]  = Jtag1687aTms_pre[1];
//! unused STAC_1_Resync

//=============================================================================
// STAC Interface for STAC_2
//=============================================================================

// Connect up the STAC ports to the STAC internal drivers.
// Also connect up the STAC ports to the ALTTAP internal drivers.
// Rename STAC ports for BIA compliance.

// Add one more to NUM_ALTTAP to account for the MTAP which uses bit 0.
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_2_SelectWSP (
  .Z(STAC_2_SelectWSP),
  .A(Stac_AltTapEn ?  AltTapEnBit[0+1+2] :  Jtag1687aSib[2])
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_2_Tms (
  .Z(STAC_2_Tms),
  .A(Stac_AltTapEn ?  AltTapTms[0+1+2]   :  Jtag1687aTms[2])
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_2_WrstN (
  .Z(STAC_2_WrstN),
  .A(Stac_AltTapEn ? ~AltTapTrst[0+1+2]  : ~Jtag1687aTrst[2])
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_2_Wsi (
  .Z(STAC_2_Wsi),
  .A(Stac_AltTapEn ?  AltTapTdi[0+1+2]   :  Jtag1687aTdi[2])
);
DFXIP4_GC10_IP_CLKMARKER I_Mrk_STAC_2_Wrck (
  .Z                         ( STAC_2_Wrck ),
  .A                         ( Stac_Wrck         )

);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_2_WsoStac (
  .Z(Jtag1687aTdo[2]),
  .A(STAC_2_Wso)
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_2_WsoAltTap (
  .Z(AltTapTdo[0+1+2]),
  .A(STAC_2_Wso)
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_2_AltTdoEn (
  .Z(AltTapTdoEn[0+1+2]),
  .A(STAC_2_AltTdoEn)
);

assign Jtag1687aTrst[2] = Jtag1687aTrst_pre[2];
assign Jtag1687aTms[2]  = Jtag1687aTms_pre[2];
//! unused STAC_2_Resync

//=============================================================================
// STAC Interface for STAC_3
//=============================================================================

// Connect up the STAC ports to the STAC internal drivers.
// Also connect up the STAC ports to the ALTTAP internal drivers.
// Rename STAC ports for BIA compliance.

// Add one more to NUM_ALTTAP to account for the MTAP which uses bit 0.
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_3_SelectWSP (
  .Z(STAC_3_SelectWSP),
  .A(Stac_AltTapEn ?  AltTapEnBit[0+1+3] :  Jtag1687aSib[3])
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_3_Tms (
  .Z(STAC_3_Tms),
  .A(Stac_AltTapEn ?  AltTapTms[0+1+3]   :  Jtag1687aTms[3])
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_3_WrstN (
  .Z(STAC_3_WrstN),
  .A(Stac_AltTapEn ? ~AltTapTrst[0+1+3]  : ~Jtag1687aTrst[3])
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_3_Wsi (
  .Z(STAC_3_Wsi),
  .A(Stac_AltTapEn ?  AltTapTdi[0+1+3]   :  Jtag1687aTdi[3])
);
DFXIP4_GC10_IP_CLKMARKER I_Mrk_STAC_3_Wrck (
  .Z                         ( STAC_3_Wrck ),
  .A                         ( Stac_Wrck         )

);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_3_WsoStac (
  .Z(Jtag1687aTdo[3]),
  .A(STAC_3_Wso)
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_3_WsoAltTap (
  .Z(AltTapTdo[0+1+3]),
  .A(STAC_3_Wso)
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_3_AltTdoEn (
  .Z(AltTapTdoEn[0+1+3]),
  .A(STAC_3_AltTdoEn)
);

assign Jtag1687aTrst[3] = Jtag1687aTrst_pre[3];
assign Jtag1687aTms[3]  = Jtag1687aTms_pre[3];
//! unused STAC_3_Resync

//=============================================================================
// STAC Interface for STAC_4
//=============================================================================

// Connect up the STAC ports to the STAC internal drivers.
// Also connect up the STAC ports to the ALTTAP internal drivers.
// Rename STAC ports for BIA compliance.

// Add one more to NUM_ALTTAP to account for the MTAP which uses bit 0.
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_4_SelectWSP (
  .Z(STAC_4_SelectWSP),
  .A(Stac_AltTapEn ?  AltTapEnBit[0+1+4] :  Jtag1687aSib[4])
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_4_Tms (
  .Z(STAC_4_Tms),
  .A(Stac_AltTapEn ?  AltTapTms[0+1+4]   :  Jtag1687aTms[4])
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_4_WrstN (
  .Z(STAC_4_WrstN),
  .A(Stac_AltTapEn ? ~AltTapTrst[0+1+4]  : ~Jtag1687aTrst[4])
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_4_Wsi (
  .Z(STAC_4_Wsi),
  .A(Stac_AltTapEn ?  AltTapTdi[0+1+4]   :  Jtag1687aTdi[4])
);
DFXIP4_GC10_IP_CLKMARKER I_Mrk_STAC_4_Wrck (
  .Z                         ( STAC_4_Wrck ),
  .A                         ( Stac_Wrck         )

);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_4_WsoStac (
  .Z(Jtag1687aTdo[4]),
  .A(STAC_4_Wso)
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_4_WsoAltTap (
  .Z(AltTapTdo[0+1+4]),
  .A(STAC_4_Wso)
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_4_AltTdoEn (
  .Z(AltTapTdoEn[0+1+4]),
  .A(STAC_4_AltTdoEn)
);

assign Jtag1687aTrst[4] = Jtag1687aTrst_pre[4];
assign Jtag1687aTms[4]  = Jtag1687aTms_pre[4];
//! unused STAC_4_Resync

//=============================================================================
// STAC Interface for STAC_5
//=============================================================================

// Connect up the STAC ports to the STAC internal drivers.
// Also connect up the STAC ports to the ALTTAP internal drivers.
// Rename STAC ports for BIA compliance.

// Add one more to NUM_ALTTAP to account for the MTAP which uses bit 0.
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_5_SelectWSP (
  .Z(STAC_5_SelectWSP),
  .A(Stac_AltTapEn ?  AltTapEnBit[0+1+5] :  Jtag1687aSib[5])
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_5_Tms (
  .Z(STAC_5_Tms),
  .A(Stac_AltTapEn ?  AltTapTms[0+1+5]   :  Jtag1687aTms[5])
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_5_WrstN (
  .Z(STAC_5_WrstN),
  .A(Stac_AltTapEn ? ~AltTapTrst[0+1+5]  : ~Jtag1687aTrst[5])
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_5_Wsi (
  .Z(STAC_5_Wsi),
  .A(Stac_AltTapEn ?  AltTapTdi[0+1+5]   :  Jtag1687aTdi[5])
);
DFXIP4_GC10_IP_CLKMARKER I_Mrk_STAC_5_Wrck (
  .Z                         ( STAC_5_Wrck ),
  .A                         ( Stac_Wrck         )

);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_5_WsoStac (
  .Z(Jtag1687aTdo[5]),
  .A(STAC_5_Wso)
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_5_WsoAltTap (
  .Z(AltTapTdo[0+1+5]),
  .A(STAC_5_Wso)
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_5_AltTdoEn (
  .Z(AltTapTdoEn[0+1+5]),
  .A(STAC_5_AltTdoEn)
);

assign Jtag1687aTrst[5] = Jtag1687aTrst_pre[5];
assign Jtag1687aTms[5]  = Jtag1687aTms_pre[5];
//! unused STAC_5_Resync

//=============================================================================
// STAC Interface for STAC_6
//=============================================================================

// Connect up the STAC ports to the STAC internal drivers.
// Also connect up the STAC ports to the ALTTAP internal drivers.
// Rename STAC ports for BIA compliance.

// Add one more to NUM_ALTTAP to account for the MTAP which uses bit 0.
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_6_SelectWSP (
  .Z(STAC_6_SelectWSP),
  .A(Stac_AltTapEn ?  AltTapEnBit[0+1+6] :  Jtag1687aSib[6])
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_6_Tms (
  .Z(STAC_6_Tms),
  .A(Stac_AltTapEn ?  AltTapTms[0+1+6]   :  Jtag1687aTms[6])
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_6_WrstN (
  .Z(STAC_6_WrstN),
  .A(Stac_AltTapEn ? ~AltTapTrst[0+1+6]  : ~Jtag1687aTrst[6])
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_6_Wsi (
  .Z(STAC_6_Wsi),
  .A(Stac_AltTapEn ?  AltTapTdi[0+1+6]   :  Jtag1687aTdi[6])
);
DFXIP4_GC10_IP_CLKMARKER I_Mrk_STAC_6_Wrck (
  .Z                         ( STAC_6_Wrck ),
  .A                         ( Stac_Wrck         )

);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_6_WsoStac (
  .Z(Jtag1687aTdo[6]),
  .A(STAC_6_Wso)
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_6_WsoAltTap (
  .Z(AltTapTdo[0+1+6]),
  .A(STAC_6_Wso)
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_6_AltTdoEn (
  .Z(AltTapTdoEn[0+1+6]),
  .A(STAC_6_AltTdoEn)
);

assign Jtag1687aTrst[6] = Jtag1687aTrst_pre[6];
assign Jtag1687aTms[6]  = Jtag1687aTms_pre[6];
//! unused STAC_6_Resync

//=============================================================================
// STAC Interface for STAC_7
//=============================================================================

// Connect up the STAC ports to the STAC internal drivers.
// Also connect up the STAC ports to the ALTTAP internal drivers.
// Rename STAC ports for BIA compliance.

// Add one more to NUM_ALTTAP to account for the MTAP which uses bit 0.
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_7_SelectWSP (
  .Z(STAC_7_SelectWSP),
  .A(Stac_AltTapEn ?  AltTapEnBit[0+1+7] :  Jtag1687aSib[7])
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_7_Tms (
  .Z(STAC_7_Tms),
  .A(Stac_AltTapEn ?  AltTapTms[0+1+7]   :  Jtag1687aTms[7])
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_7_WrstN (
  .Z(STAC_7_WrstN),
  .A(Stac_AltTapEn ? ~AltTapTrst[0+1+7]  : ~Jtag1687aTrst[7])
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_7_Wsi (
  .Z(STAC_7_Wsi),
  .A(Stac_AltTapEn ?  AltTapTdi[0+1+7]   :  Jtag1687aTdi[7])
);
DFXIP4_GC10_IP_CLKMARKER I_Mrk_STAC_7_Wrck (
  .Z                         ( STAC_7_Wrck ),
  .A                         ( Stac_Wrck         )

);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_7_WsoStac (
  .Z(Jtag1687aTdo[7]),
  .A(STAC_7_Wso)
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_7_WsoAltTap (
  .Z(AltTapTdo[0+1+7]),
  .A(STAC_7_Wso)
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_7_AltTdoEn (
  .Z(AltTapTdoEn[0+1+7]),
  .A(STAC_7_AltTdoEn)
);

assign Jtag1687aTrst[7] = Jtag1687aTrst_pre[7];
assign Jtag1687aTms[7]  = Jtag1687aTms_pre[7];
//! unused STAC_7_Resync

//=============================================================================
// STAC Interface for STAC_8
//=============================================================================

// Connect up the STAC ports to the STAC internal drivers.
// Also connect up the STAC ports to the ALTTAP internal drivers.
// Rename STAC ports for BIA compliance.

// Add one more to NUM_ALTTAP to account for the MTAP which uses bit 0.
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_8_SelectWSP (
  .Z(STAC_8_SelectWSP),
  .A(Stac_AltTapEn ?  AltTapEnBit[0+1+8] :  Jtag1687aSib[8])
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_8_Tms (
  .Z(STAC_8_Tms),
  .A(Stac_AltTapEn ?  AltTapTms[0+1+8]   :  Jtag1687aTms[8])
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_8_WrstN (
  .Z(STAC_8_WrstN),
  .A(Stac_AltTapEn ? ~AltTapTrst[0+1+8]  : ~Jtag1687aTrst[8])
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_8_Wsi (
  .Z(STAC_8_Wsi),
  .A(Stac_AltTapEn ?  AltTapTdi[0+1+8]   :  Jtag1687aTdi[8])
);
DFXIP4_GC10_IP_CLKMARKER I_Mrk_STAC_8_Wrck (
  .Z                         ( STAC_8_Wrck ),
  .A                         ( Stac_Wrck         )

);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_8_WsoStac (
  .Z(Jtag1687aTdo[8]),
  .A(STAC_8_Wso)
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_8_WsoAltTap (
  .Z(AltTapTdo[0+1+8]),
  .A(STAC_8_Wso)
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_8_AltTdoEn (
  .Z(AltTapTdoEn[0+1+8]),
  .A(STAC_8_AltTdoEn)
);

assign Jtag1687aTrst[8] = Jtag1687aTrst_pre[8];
assign Jtag1687aTms[8]  = Jtag1687aTms_pre[8];
//! unused STAC_8_Resync

//=============================================================================
// STAC Interface for STAC_9
//=============================================================================

// Connect up the STAC ports to the STAC internal drivers.
// Also connect up the STAC ports to the ALTTAP internal drivers.
// Rename STAC ports for BIA compliance.

// Add one more to NUM_ALTTAP to account for the MTAP which uses bit 0.
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_9_SelectWSP (
  .Z(STAC_9_SelectWSP),
  .A(Stac_AltTapEn ?  AltTapEnBit[0+1+9] :  Jtag1687aSib[9])
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_9_Tms (
  .Z(STAC_9_Tms),
  .A(Stac_AltTapEn ?  AltTapTms[0+1+9]   :  Jtag1687aTms[9])
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_9_WrstN (
  .Z(STAC_9_WrstN),
  .A(Stac_AltTapEn ? ~AltTapTrst[0+1+9]  : ~Jtag1687aTrst[9])
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_9_Wsi (
  .Z(STAC_9_Wsi),
  .A(Stac_AltTapEn ?  AltTapTdi[0+1+9]   :  Jtag1687aTdi[9])
);
DFXIP4_GC10_IP_CLKMARKER I_Mrk_STAC_9_Wrck (
  .Z                         ( STAC_9_Wrck ),
  .A                         ( Stac_Wrck         )

);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_9_WsoStac (
  .Z(Jtag1687aTdo[9]),
  .A(STAC_9_Wso)
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_9_WsoAltTap (
  .Z(AltTapTdo[0+1+9]),
  .A(STAC_9_Wso)
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_9_AltTdoEn (
  .Z(AltTapTdoEn[0+1+9]),
  .A(STAC_9_AltTdoEn)
);

assign Jtag1687aTrst[9] = Jtag1687aTrst_pre[9];
assign Jtag1687aTms[9]  = Jtag1687aTms_pre[9];
//! unused STAC_9_Resync

//=============================================================================
// STAC Interface for STAC_10
//=============================================================================

// Connect up the STAC ports to the STAC internal drivers.
// Also connect up the STAC ports to the ALTTAP internal drivers.
// Rename STAC ports for BIA compliance.

// Add one more to NUM_ALTTAP to account for the MTAP which uses bit 0.
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_10_SelectWSP (
  .Z(STAC_10_SelectWSP),
  .A(Stac_AltTapEn ?  AltTapEnBit[0+1+10] :  Jtag1687aSib[10])
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_10_Tms (
  .Z(STAC_10_Tms),
  .A(Stac_AltTapEn ?  AltTapTms[0+1+10]   :  Jtag1687aTms[10])
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_10_WrstN (
  .Z(STAC_10_WrstN),
  .A(Stac_AltTapEn ? ~AltTapTrst[0+1+10]  : ~Jtag1687aTrst[10])
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_10_Wsi (
  .Z(STAC_10_Wsi),
  .A(Stac_AltTapEn ?  AltTapTdi[0+1+10]   :  Jtag1687aTdi[10])
);
DFXIP4_GC10_IP_CLKMARKER I_Mrk_STAC_10_Wrck (
  .Z                         ( STAC_10_Wrck ),
  .A                         ( Stac_Wrck         )

);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_10_WsoStac (
  .Z(Jtag1687aTdo[10]),
  .A(STAC_10_Wso)
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_10_WsoAltTap (
  .Z(AltTapTdo[0+1+10]),
  .A(STAC_10_Wso)
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_10_AltTdoEn (
  .Z(AltTapTdoEn[0+1+10]),
  .A(STAC_10_AltTdoEn)
);

assign Jtag1687aTrst[10] = Jtag1687aTrst_pre[10];
assign Jtag1687aTms[10]  = Jtag1687aTms_pre[10];
//! unused STAC_10_Resync

//=============================================================================
// STAC Interface for STAC_11
//=============================================================================

// Connect up the STAC ports to the STAC internal drivers.
// Also connect up the STAC ports to the ALTTAP internal drivers.
// Rename STAC ports for BIA compliance.

// Add one more to NUM_ALTTAP to account for the MTAP which uses bit 0.
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_11_SelectWSP (
  .Z(STAC_11_SelectWSP),
  .A(Stac_AltTapEn ?  AltTapEnBit[0+1+11] :  Jtag1687aSib[11])
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_11_Tms (
  .Z(STAC_11_Tms),
  .A(Stac_AltTapEn ?  AltTapTms[0+1+11]   :  Jtag1687aTms[11])
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_11_WrstN (
  .Z(STAC_11_WrstN),
  .A(Stac_AltTapEn ? ~AltTapTrst[0+1+11]  : ~Jtag1687aTrst[11])
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_11_Wsi (
  .Z(STAC_11_Wsi),
  .A(Stac_AltTapEn ?  AltTapTdi[0+1+11]   :  Jtag1687aTdi[11])
);
DFXIP4_GC10_IP_CLKMARKER I_Mrk_STAC_11_Wrck (
  .Z                         ( STAC_11_Wrck ),
  .A                         ( Stac_Wrck         )

);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_11_WsoStac (
  .Z(Jtag1687aTdo[11]),
  .A(STAC_11_Wso)
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_11_WsoAltTap (
  .Z(AltTapTdo[0+1+11]),
  .A(STAC_11_Wso)
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_11_AltTdoEn (
  .Z(AltTapTdoEn[0+1+11]),
  .A(STAC_11_AltTdoEn)
);

assign Jtag1687aTrst[11] = Jtag1687aTrst_pre[11];
assign Jtag1687aTms[11]  = Jtag1687aTms_pre[11];
//! unused STAC_11_Resync

//=============================================================================
// STAC Interface for STAC_12
//=============================================================================

// Connect up the STAC ports to the STAC internal drivers.
// Also connect up the STAC ports to the ALTTAP internal drivers.
// Rename STAC ports for BIA compliance.

// Add one more to NUM_ALTTAP to account for the MTAP which uses bit 0.
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_12_SelectWSP (
  .Z(STAC_12_SelectWSP),
  .A(Stac_AltTapEn ?  AltTapEnBit[0+1+12] :  Jtag1687aSib[12])
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_12_Tms (
  .Z(STAC_12_Tms),
  .A(Stac_AltTapEn ?  AltTapTms[0+1+12]   :  Jtag1687aTms[12])
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_12_WrstN (
  .Z(STAC_12_WrstN),
  .A(Stac_AltTapEn ? ~AltTapTrst[0+1+12]  : ~Jtag1687aTrst[12])
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_12_Wsi (
  .Z(STAC_12_Wsi),
  .A(Stac_AltTapEn ?  AltTapTdi[0+1+12]   :  Jtag1687aTdi[12])
);
DFXIP4_GC10_IP_CLKMARKER I_Mrk_STAC_12_Wrck (
  .Z                         ( STAC_12_Wrck ),
  .A                         ( Stac_Wrck         )

);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_12_WsoStac (
  .Z(Jtag1687aTdo[12]),
  .A(STAC_12_Wso)
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_12_WsoAltTap (
  .Z(AltTapTdo[0+1+12]),
  .A(STAC_12_Wso)
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_12_AltTdoEn (
  .Z(AltTapTdoEn[0+1+12]),
  .A(STAC_12_AltTdoEn)
);

assign Jtag1687aTrst[12] = Jtag1687aTrst_pre[12];
assign Jtag1687aTms[12]  = Jtag1687aTms_pre[12];
//! unused STAC_12_Resync

//=============================================================================
// STAC Interface for STAC_13
//=============================================================================

// Connect up the STAC ports to the STAC internal drivers.
// Also connect up the STAC ports to the ALTTAP internal drivers.
// Rename STAC ports for BIA compliance.

// Add one more to NUM_ALTTAP to account for the MTAP which uses bit 0.
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_13_SelectWSP (
  .Z(STAC_13_SelectWSP),
  .A(Stac_AltTapEn ?  AltTapEnBit[0+1+13] :  Jtag1687aSib[13])
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_13_Tms (
  .Z(STAC_13_Tms),
  .A(Stac_AltTapEn ?  AltTapTms[0+1+13]   :  Jtag1687aTms[13])
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_13_WrstN (
  .Z(STAC_13_WrstN),
  .A(Stac_AltTapEn ? ~AltTapTrst[0+1+13]  : ~Jtag1687aTrst[13])
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_13_Wsi (
  .Z(STAC_13_Wsi),
  .A(Stac_AltTapEn ?  AltTapTdi[0+1+13]   :  Jtag1687aTdi[13])
);
DFXIP4_GC10_IP_CLKMARKER I_Mrk_STAC_13_Wrck (
  .Z                         ( STAC_13_Wrck ),
  .A                         ( Stac_Wrck         )

);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_13_WsoStac (
  .Z(Jtag1687aTdo[13]),
  .A(STAC_13_Wso)
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_13_WsoAltTap (
  .Z(AltTapTdo[0+1+13]),
  .A(STAC_13_Wso)
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_13_AltTdoEn (
  .Z(AltTapTdoEn[0+1+13]),
  .A(STAC_13_AltTdoEn)
);

assign Jtag1687aTrst[13] = Jtag1687aTrst_pre[13];
assign Jtag1687aTms[13]  = Jtag1687aTms_pre[13];
//! unused STAC_13_Resync

//=============================================================================
// STAC Interface for STAC_14
//=============================================================================

// Connect up the STAC ports to the STAC internal drivers.
// Also connect up the STAC ports to the ALTTAP internal drivers.
// Rename STAC ports for BIA compliance.

// Add one more to NUM_ALTTAP to account for the MTAP which uses bit 0.
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_14_SelectWSP (
  .Z(STAC_14_SelectWSP),
  .A(Stac_AltTapEn ?  AltTapEnBit[0+1+14] :  Jtag1687aSib[14])
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_14_Tms (
  .Z(STAC_14_Tms),
  .A(Stac_AltTapEn ?  AltTapTms[0+1+14]   :  Jtag1687aTms[14])
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_14_WrstN (
  .Z(STAC_14_WrstN),
  .A(Stac_AltTapEn ? ~AltTapTrst[0+1+14]  : ~Jtag1687aTrst[14])
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_14_Wsi (
  .Z(STAC_14_Wsi),
  .A(Stac_AltTapEn ?  AltTapTdi[0+1+14]   :  Jtag1687aTdi[14])
);
DFXIP4_GC10_IP_CLKMARKER I_Mrk_STAC_14_Wrck (
  .Z                         ( STAC_14_Wrck ),
  .A                         ( Stac_Wrck         )

);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_14_WsoStac (
  .Z(Jtag1687aTdo[14]),
  .A(STAC_14_Wso)
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_14_WsoAltTap (
  .Z(AltTapTdo[0+1+14]),
  .A(STAC_14_Wso)
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_14_AltTdoEn (
  .Z(AltTapTdoEn[0+1+14]),
  .A(STAC_14_AltTdoEn)
);

assign Jtag1687aTrst[14] = Jtag1687aTrst_pre[14];
assign Jtag1687aTms[14]  = Jtag1687aTms_pre[14];
//! unused STAC_14_Resync

//=============================================================================
// STAC Interface for STAC_15
//=============================================================================

// Connect up the STAC ports to the STAC internal drivers.
// Also connect up the STAC ports to the ALTTAP internal drivers.
// Rename STAC ports for BIA compliance.

// Add one more to NUM_ALTTAP to account for the MTAP which uses bit 0.
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_15_SelectWSP (
  .Z(STAC_15_SelectWSP),
  .A(Stac_AltTapEn ?  AltTapEnBit[0+1+15] :  Jtag1687aSib[15])
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_15_Tms (
  .Z(STAC_15_Tms),
  .A(Stac_AltTapEn ?  AltTapTms[0+1+15]   :  Jtag1687aTms[15])
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_15_WrstN (
  .Z(STAC_15_WrstN),
  .A(Stac_AltTapEn ? ~AltTapTrst[0+1+15]  : ~Jtag1687aTrst[15])
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_15_Wsi (
  .Z(STAC_15_Wsi),
  .A(Stac_AltTapEn ?  AltTapTdi[0+1+15]   :  Jtag1687aTdi[15])
);
DFXIP4_GC10_IP_CLKMARKER I_Mrk_STAC_15_Wrck (
  .Z                         ( STAC_15_Wrck ),
  .A                         ( Stac_Wrck         )

);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_15_WsoStac (
  .Z(Jtag1687aTdo[15]),
  .A(STAC_15_Wso)
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_15_WsoAltTap (
  .Z(AltTapTdo[0+1+15]),
  .A(STAC_15_Wso)
);
DFXIP4_GC10_IP_MARKER I_Mrk_STAC_15_AltTdoEn (
  .Z(AltTapTdoEn[0+1+15]),
  .A(STAC_15_AltTdoEn)
);

assign Jtag1687aTrst[15] = Jtag1687aTrst_pre[15];
assign Jtag1687aTms[15]  = Jtag1687aTms_pre[15];
//! unused STAC_15_Resync



//=============================================================================
// ALTTAP Drivers
//=============================================================================

// Generate the control signals for the ALTTAPs.

gc10_sipr16_ALTTAP_DRIVER #(.NUM_DRIVER(17),.NUM_ALTTAP(0)) I_ALTTAP_DRIVER (
  .AltTapEn                ( Stac_AltTapEn                  ),
  .Trst                    ( ~Stac_WrstN                    ),
  .Tms                     ( Stac_Tms                       ),
  .SyncClk                 ( Stac_Wrck                      ),
  .AltTapEnAeb             ( AltTapEnAeb[17-1:0] ),
  .AltTapEnReg             ( AltTapEnReg[17-1:0] ),
  .AltTapDrvTdi            ( Stac_Wsi                       ),
  .AltTapDrvTdo            ( AltTapDrvTdo                   ),

  .AltTapEnBit             ( AltTapEnBit[17-1:0] ),
  .AltTapTrst              ( AltTapTrst[17-1:0] ),
  .AltTapTms               ( AltTapTms[17-1:0] ),
//! disable checks_for_cycles
  .AltTapTdi               ( AltTapTdi[17-1:0] ),
  .AltTapTdo               ( AltTapTdo[17-1:0] )
//! enable checks_for_cycles
);

// Add one more to NUM_ALTTAP to account for the MTAP which uses bit 0.

assign Stac_AltTdoEn = Stac_AltTapEn & (|AltTapTdoEn[17-1:0]);

//=============================================================================
// SingleChain Drivers
//=============================================================================

// Stitch up the single chain segments from the various child i/f
// Note that the single chain from the child STACs and 1500 clients
// are stitched between LocCORE_Wso and WSO.  UBTS 470941

assign SingleChainLocalSdo  = WdrWsi;

//=============================================================================
// Misc
//=============================================================================

// Just a way to get rid of the RPL warnings/errors
//! unused SysScanShiftMode
//! unused TapInCapDR
//! unused TapCapDR
//! unused TapInShfDR
//! unused TapShfDR
//! unused TapInUpdDR
//! unused TapInUpdIR
  wire                                          AltTapEnBit_UNUSED;
assign AltTapEnBit_UNUSED = |AltTapEnBit[17-1:0];
//! unused AltTapEnBit_UNUSED

`include "stac_assertions_gc10_sipr16.vh"

endmodule

// leda B_1001 on
// leda B_1011 on
