
//================================================================================================================================
//
// Copyright (c) 2002 by Advanced Micro Devices, Inc.
//
// This file is protected by Federal Copyright Law, with all rights reserved. No part of this file may be reproduced, stored in a
// retrieval system, translated, transcribed, or transmitted, in any form, or by any means manual, electric, electronic,
// mechanical, electro-magnetic, chemical, optical, or otherwise, without prior explicit written permission from Advanced Micro
// Devices, Inc.
//
//================================================================================================================================

//================================================================================================================================
//
// <REFERENCES>
//
//  1 http://twiki.amd.com/twiki/pub/DFTCOE/Meeting_110912/Overview_of_2014_Hierarchical_DFx_Architecture5.pdf
//  2 http://twiki.amd.com/twiki/bin/view/DFTCOE/HierarchicalModularJtag30
//  3 See UBTS 436890 - ALTTAP/P1687 feature
//  4 See UBTS 448813 - SIB Query Update feature
//  5 See UBTS 448812 - 1500 Broadcast feature
//  6 http://twiki.amd.com/twiki/bin/viewauth/DFTCOE/HierarchicalAltTapEn
//  7 See UBTS 459043 - SingleChain feature
//  8 http://twiki.amd.com/twiki/pub/SOCArch/SOC15DFX/STAC_Architecture_Overview4.pdf
//  9 See UBTS 465634 - SJC override feature
// 10 See UBTS 470355 - 1500 Selective Broadcast feature
// 11 See UBTS 472538 - STAC router feature
// 12 See UBTS 473811 - AEB security feature
// 13 See UBTS 476786 - Remove private bscan instructions from STAC controllers
// 14 See UBTS 484316 - STAC TAP (IP) controller should have PipelineEn input
// 15 See UBTS 556234 - Timing Loop in JTAG flop Reset -> Q -> Reset
// 16 See UBTS 558454 - STAC should have pipeline flop on WSO enabled by PipelineEn input
// 17 See UBTS 550459 - STAC ALTTAP MODE - Resets ALTTAPEN tdr after reset
// 17 See UBTS 587368 - Timing loop through STAC router and child STAC
// 18 See http://ontrack-internal.amd.com/browse/DECDFXIP41-141
//
//================================================================================================================================

//================================================================================================================================
//
// <PERLOG>
//
//  This file was generated using perlog.
//
//  Run directory:
//    /proj/gfxip_gcregr_1/user/orlvalid/gfx10.1_navi10_rtl_freeze_atl/import/dfx/stac/src/rtl
//
//  Commandline:
//    /proj/verif_release_ro/perlog/40/perlog.pl -C /proj/gfxip_gcregr_1/user/orlvalid/gfx10.1_navi10_rtl_freeze_atl/out/linux_2.6.32_64.VCS/navi10/library/stac-gc12-dfxip4-gc10/pub/src/meta/tools/staccfg/default_dfx_stac.plcfg -C /proj/gfxip_gcregr_1/user/orlvalid/gfx10.1_navi10_rtl_freeze_atl/out/linux_2.6.32_64.VCS/navi10/common/pub/src/meta/tools/staccfg/rtl_dfx_stac_gc12.plcfg -D LIB_PRFX=DFXIP4_GC10
//
//  Config variables:
//    IP_NAME         : gc12
//    LIB_PRFX        : DFXIP4_GC10
//    AEB             : 1
//    NUM_1500        : 1
//    NUM_ALTTAP      : 0
//    NUM_SLAVETAP    : 0
//    NUM_STAC        : 0
//    PIPELINE        : 1
//    RESYNC_STAC     : ()
//    ROUTER          : 0
//    SINGLECHAIN     : 1
//    RESET_BC1500    : ()
//    NUM_TDR_RESETS  : 0
//    EARLY_1500      : (0)
//
//
//  Supported instruction opcodes:
//    8'h00   WBYPASS
//    8'h02   IDCODE
//    8'h03   BC1500
//    8'h04   BC1500_QRYUPD
//    8'h05   ALTTAPEN
//    8'h07   SYSSCANSHIFT
//    8'h09   SCANCONFIG
//    8'h0A   SELECTDIE
//    8'h0B   BROADCAST
//    8'h13   P1687A
//    8'h14   P1687A_QRYUPD
//    8'h20   SYSSCANSHIFT2
//    8'h21   SYSSCAN
//    8'h25   QRYUPD
//    8'h34   SMSG_CONTROL
//    8'h35   SMSG_STATUS
//    8'h36   JPC_WIR
//    8'h37   JPC_WDR
//    8'h38   SMS_WIR
//    8'h39   SMS_WDR
//    8'h3A   FUSE_IF_ATE_MODE
//    8'h3B   FUSE_IF_SETUP
//    8'h3C   FUSE_IF_STATUS
//    8'hFF   BYPASS
//
//================================================================================================================================

//================================================================================================================================
//
// <DESCRIPTION>
//
// Useful Abbreviations
//
//  SWB - SelectWIR Bit
//  WIR - (IEEE1500) Wrapper Instruction Register
//  WDR - (IEEE1500) Wrapper Data Register
//  TDR - (IEEE1149.1) Test Data Register
//  BC  - BroadCast
//  SBC - Selective BroadCast
//  SIB - Segment Inclusion Bit
//  SEB - SIB Exclusion Bit
//  WSP - Wrapper Serial Port
//  AEB - Access Enablement Bit
//  SJC - SMN to JTAG Converter
//
// Mode1 - STAC (1500 + FSM)
//          Use the SWB to switch between the 'instruction' or 'data'.
//          The FSM stays in the 1149.1 'data' stack.
//          JTAG1687A can be used to access additional child STACs.
//    Stac_AltTapEn == 0
//    WSI -> LocCORE_Wsi -> SWB (SelectWIR) -> WIR                 -> LocCORE_Wso -> WSO
//                                          -> WBYPASS   -> MuxWso ->
//                                          -> JTAG1687A ->
//                                          -> WDR       ->
// Mode2 - STAC (1149.1)
//          Use the FSM to switch between the 'instruction' or 'data',
//          by traversing through the 1149.1 'instruction or 'data' stack, respectively.
//    Stac_AltTapEn == 1
//    AltTapEnReg[0] == 1
//    WSI -> AltTapTdi[0] -> LocCORE_Wsi -> WIR               -> LocCORE_Wso -> AltTapDrvTdo -> WSO
//                                       -> WBYPASS -> MuxWso ->
//                                       -> WDR     ->
//
// Mode3 - ALTTAP (1149.1)
//          Daisy chain up all the enabled ALTTAP.
//          Freeze the STAC and FSM in RTI by disabling its clock and reset.
//    Stac_AltTapEn == 1
//    AltTapEnReg[0] == 0
//    AltTapEnReg[1-1:1] == 1
//    WSI -> ALTTAP[1-1] -> ... -> ALTTAP[2] -> ALTTAP[1] -> AltTapDrvTdo -> WSO
//
// Mode4 - ALTTAP (1149.1) plus local STAC (1149.1
//          Daisy chain up all the enabled ALTTAP including the local STAC.
//          Use the FSM to switch between the 'instruction' or 'data',
//          by traversing through the 1149.1 'instruction or 'data' stack, respectively.
//    Stac_AltTapEn == 1
//    AltTapEnReg[0] == 1
//    AltTapEnReg[1-1:1] == 1
//    WSI -> ALTTAP[1-1] -> ... -> ALTTAP[1] -> LocCORE_Wsi -> WIR               -> LocCORE_Wso -> AltTapDrvTdo -> WSO
//                                                                        -> WBYPASS -> MuxWso ->
//                                                                        -> WDR     ->
//
//
//================================================================================================================================
// leda B_1001 off
// leda B_1011 off
//~ format_rtl 1
//~ format_inst narrow

`timescale 1ps/1ps

module gc12_STAC (

// Interface to the STAC's 1500 client clusters
  output  wire                                  BC1500_0_select_wir,
  output  wire                                  BC1500_0_shift,
  output  wire                                  BC1500_0_capture,
  output  wire                                  BC1500_0_update,
  output  wire                                  BC1500_0_tlr,
  output  wire                                  BC1500_0_idle,
  output  wire                                  BC1500_0_tresetb,
  output  wire                                  BC1500_0_Wrck,
  output  wire                                  BC1500_0_Wsi,
  input                                         BC1500_0_Wso,
  input                                         BC1500_0_AEB,
// Interface to the STAC's single chain
  output  wire                                  SingleChainEn,
  output  wire                                  SingleChainSdi,
  input                                         SingleChainSdo,
// Interface to the STAC's AEB
  input                                         AEB,                            // Used to gate the private instructions
// Interface to the STAC's TDR/WDR
  output  wire                                  TapInRti,
  output  wire                                  TapInRtiNxt,
  output   reg                                  WDR_capture,                    // WDR's CAPTURE-DR
  output   reg                                  WDR_shift,                      // WDR's SHIFT-DR
  output   reg                                  WDR_update,                     // WDR's UPDATE-DR
  output   reg                                  WDR_JTAGFF_Update,              // WDR's UPDATE-DR (for JTAGFLOP)
  output   reg                                  WDR_Wrck,                       // WDR's WRCK
  output   reg                                  WDR_Wrst,                       // WDR's WRST
  output   reg                                  WDR_WrstN,                      // WDR's WRST inverted  UBTS 568925
  output   reg                                  WDR_tdi,                        // WDR's WSI
  output  wire                                  SCANCONFIG_En,
  input                                         SCANCONFIG_Tdo,
  output  wire                                  SYSSCAN_En,
  input                                         SYSSCAN_Tdo,
  output  wire                                  FUSE_IF_ATE_MODE_En,
  input                                         FUSE_IF_ATE_MODE_Tdo,
  output  wire                                  FUSE_IF_SETUP_En,
  input                                         FUSE_IF_SETUP_Tdo,
  output  wire                                  FUSE_IF_STATUS_En,
  input                                         FUSE_IF_STATUS_Tdo,
  output  wire                                  JPC_WDR_En,
  input                                         JPC_WDR_Tdo,
  output  wire                                  JPC_WIR_En,
  input                                         JPC_WIR_Tdo,
  output  wire                                  SMSG_CONTROL_En,
  input                                         SMSG_CONTROL_Tdo,
  output  wire                                  SMSG_STATUS_En,
  input                                         SMSG_STATUS_Tdo,
  output  wire                                  SMS_WDR_En,
  input                                         SMS_WDR_Tdo,
  output  wire                                  SMS_WIR_En,
  input                                         SMS_WIR_Tdo,

// STAC Interface to the SOC
  input                                         Stac_PipelineEn,
  input                                         Stac_SelectWSP,     // Master enable for the entire STAC
  input                                         Stac_Tms,
  input                                         Stac_Wrck,
  input                                         Stac_WrstN,
  input                                         Stac_Wsi,
  output  wire                                  Stac_Wso,
  input                                         Stac_AltTapEn,
  output  wire                                  Stac_AltTdoEn
);

// Sticking all the wire/reg declaration in one spot to
// keep the rest of the file from clutter.

  wire                                          PipelineWso;
  wire                                          NonPipelineWso;
  wire                                          LocCORE_Tms;
  wire                                          LocCORE_Wrck;
  wire                                          LocCORE_Wrst;
  wire                                          LocCORE_SyncWrst;
  wire                                          LocCORE_Wsi;
  wire                                          LocCORE_Wso;
  wire                                          LocCORE_WsoEn;
  wire                                          QRYUPD_En; // UBTS 528914
  wire                                          ALTTAPEN_En;
  wire                                          AltTapEnTdo;
  wire                                          AltTapDrvTdo;
  reg                                           AltTapEnFall;
  wire                                          AltTapEn_d1;
  wire                [1-1:0]     AltTapEnReg;
  wire                [1-1:0]     AltTapEnAeb;
  wire                [1-1:0]     AltTapEnBit;
  wire                [1-1:0]     AltTapTms;
  wire                [1-1:0]     AltTapTckEn;
  wire                [1-1:0]     AltTapTrst;
  wire                [1-1:0]     AltTapTdi;
  wire                [1-1:0]     AltTapTdo;
  wire                [1-1:0]     AltTapTdoEn;
  wire                                          BC1500_En;
  wire                                          BC1500_QRYUPD_En;
  wire                                          BC1500_QRYUPD_only;
  wire                                          BC1500_QRYUPD_En_d1;
  wire                                          Jtag1500bcSbc_m1;
  wire                                          Jtag1500bcSbc;
  wire                                          Jtag1500bcSbcTdo;
  wire                                          Jtag1500bcSeb_m1;
  wire                                          Jtag1500bcSeb;
  wire                                          Jtag1500bcSebTdo;
  wire                                          Jtag1500bcSibsTdo;
  wire                                          Jtag1500bcSebSibsTdo;
  wire                                          Jtag1500bcDrvTdi;
  wire                                          Jtag1500bcDrvTdo;
  wire              [1-1:0]     Jtag1500bcReg_m1;
  wire              [1-1:0]     Jtag1500bcReg;
  wire              [1-1:0]     Jtag1500bcAeb;
  wire              [1-1:0]     Jtag1500bcSib;
  wire              [1-1:0]     Jtag1500bcTrst;
  wire              [1-1:0]     Jtag1500bcTdi;
  wire              [1-1:0]     Jtag1500bcTdo;
  wire              [1-1:0]     Jtag1500bcSelectWIR;
  wire              [1-1:0]     Jtag1500bcShiftWR;
  wire              [1-1:0]     Jtag1500bcCaptureWR;
  wire              [1-1:0]     Jtag1500bcUpdateWR;
  wire              [1-1:0]     Jtag1500bcTlr;
  wire              [1-1:0]     Jtag1500bcRti;
  wire              [1-1:0]     Jtag1500SwbShiftWR;  // DECDFXIP41-168
  wire              [1-1:0]     Jtag1500SwbCaptureWR;
  wire                                          SYSSCANSHIFT_En;
  wire                                          SYSSCANSHIFT2_En;
  wire                                          SingleChainLocalSdo;
  wire                                          SysScanShiftMode;
  wire                                          AltTapEn_FsmEn;
  wire                                          AltTapEn_FsmDis;
  wire                                          AltTapEnReg0;
  wire                                          AltTapEnRegTrst;
  wire                                          TapInTlr;
  wire                                          TapInCapDR;
  wire                                          TapCapDR;
  wire                                          TapShfDR;
  wire                                          TapInShfDR;
  wire                                          TapInUpdDR;
  wire                                          TapInUpdIR;
  wire                                          BypassTdo;
  wire                                          Bypass_UNUSED;
  reg                                           MuxWso;
  wire                                [7:0]     WIR;
  wire                                [7:0]     WirShiftReg;
  wire                                          WirUpdate;
  wire                                          WdrCapture;
  wire                                          WdrShift;
  wire                                          WdrUpdate;
  wire                                          JTAGFF_WdrUpdate;
  wire                                          WdrWrck;
  wire                                          WdrWrst;
  wire                                          WdrWsi;

  //Route this signal through STAC routers.

//=============================================================================
// STAC FSM, SWB, and WIR
//=============================================================================
// Pipeline through this flop to prevent false combinatorial loop. UBTS 556234

DFXIP4_GC10_IP_JTAGFLOP I_ALTTAPENREG0PIPL (
  .ASYNCCLK                ( 1'b0           ), // intentionally no reset
  .ASYNCVAL                ( 1'b0           ),
  .CLK                     ( WdrWrck        ),
  .D                       ( AltTapEnReg[0] ),
  .Q                       ( AltTapEnReg0   )
);

// Synchronizing reset deassertion for logic outside TAPFSM. UBTS 598071
DFXIP4_GC10_IP_RST_SYNC I_WRST_SYNC (
  .SampleClk ( ~Stac_Wrck  ), // Falling edge deasserts half clk earlier, minimizing impact of synchronizer delay. TapInTlr asserts on falling edge as well.
  .RstIn     ( ~Stac_WrstN ),
  .RstOut    (  SyncWrst   )
);

// We're in Stac_AltTapEn but since bit zero was set, we enable the
// local STAC in 1149.1 mode as if it was another ALTTAP.
assign AltTapEn_FsmEn  = Stac_AltTapEn &  AltTapEnReg0;

// We're in Stac_AltTapEn but since bit zero was not set,
// we disable the local STAC and keep it in RTI.
assign AltTapEn_FsmDis = Stac_AltTapEn & ~AltTapEnReg0;

// Intercept the controls to this local STAC depending on the mode.

// Extra gating is causing a timing loop during synthesis.
// Discussed this with Sopho and the gating on the clock
// is not really needed as long as we gate off the TMS and
// WRSTN to keep the FSM in RTI.  UBTS 442008
//! unused AltTapTckEn[0]
//! unused AltTapTms[0]
//! unused AltTapTrst[0]
assign LocCORE_Wsi     = AltTapEn_FsmEn ? AltTapTdi[0] : Stac_Wsi;
assign LocCORE_Wrck    =                                  Stac_Wrck;
assign LocCORE_Tms     =                                  (Stac_Tms    & ~AltTapEn_FsmDis);
assign LocCORE_Wrst    =                                  (~Stac_WrstN & ~AltTapEn_FsmDis);
assign LocCORE_SyncWrst =                                 (  SyncWrst  & ~AltTapEn_FsmDis);
assign AltTapTdo[0]    =  AltTapEn_FsmEn ? LocCORE_Wso   : 1'b0;
assign AltTapTdoEn[0]  =  AltTapEn_FsmEn ? LocCORE_WsoEn : 1'b0;
assign AltTapEnRegTrst = ~AltTapEn_FsmEn ? LocCORE_Wrst  : 1'b0; //UBTS 550459

gc12_STAC_CORE I_CORE (
  .Mode1149                ( AltTapEn_FsmEn            ),
  .ModeSingleChain         ( SYSSCANSHIFT_En           ),    // Don't use the SYSSCANSHIFT2
  .SelectWSP               ( Stac_SelectWSP            ),
  .Tms                     ( LocCORE_Tms               ),
  .Wrst                    ( LocCORE_Wrst              ),
  .SyncWrst                ( LocCORE_SyncWrst          ),
  .Wrck                    ( LocCORE_Wrck              ),
  .Wsi                     ( LocCORE_Wsi               ),
  .Wso                     ( LocCORE_Wso               ),
  .WsoEn                   ( LocCORE_WsoEn             ),
  // This comes from the Secure fuse, which STAC doesn't get.
  // Tying high so we'll only depend on the AEB value.
  .SecureMode              ( 1'b1                      ),
  .AEB                     ( AEB                       ),
  .PipelineEn              ( Stac_PipelineEn           ),
  .TapCapDR                ( TapCapDR                  ),
  .TapShfDR                ( TapShfDR                  ),
  .TapInCapDR              ( TapInCapDR                ),
  .TapInShfDR              ( TapInShfDR                ),
  .TapInUpdDR              ( TapInUpdDR                ),
  .TapInUpdIR              ( TapInUpdIR                ),
  .TapInTlr                ( TapInTlr                  ),
  .TapInRti                ( TapInRti                  ),
  .TapInRtiNxt             ( TapInRtiNxt               ),
  .JTAGFF_WdrUpdate        ( JTAGFF_WdrUpdate          ),
  .WdrCapture              ( WdrCapture                ),
  .WdrShift                ( WdrShift                  ),
  .WdrUpdate               ( WdrUpdate                 ),
  .WirShiftReg             ( WirShiftReg[7:0]          ),
  .WirUpdate               ( WirUpdate                 ),
  .WIR                     ( WIR[7:0]                  ),
  .WdrWrck                 ( WdrWrck                   ),
  .WdrWrst                 ( WdrWrst                   ),
  .WdrWsi                  ( WdrWsi                    ),
  .MuxWso                  ( MuxWso                    )
);

// Rename WDR ports for BIA compliance
// adapting to BIA-friemdly names, based on IEEE1500_TARG_tdr ifrit class
always @* begin
  WDR_capture       = WdrCapture       ;  // WDR's CAPTURE-DR
  WDR_shift         = WdrShift         ;  // WDR's SHIFT-DR
  WDR_update        = WdrUpdate        ;  // WDR's UPDATE-DR
  WDR_JTAGFF_Update = JTAGFF_WdrUpdate ;  // WDR's UPDATE-DR (for JTAGFLOP)
  WDR_Wrck          = WdrWrck          ;  // WDR's WRCK
  WDR_Wrst          = WdrWrst          ;  // WDR's WRST
  WDR_WrstN         = ~WdrWrst         ;  // WDR's WRST inverted  UBTS 568925
  WDR_tdi           = WdrWsi           ;  // WDR's WSI
end

//=============================================================================
// WSI/WSO muxing
//=============================================================================

always @* begin
  case (^WIR[7:0])
    1'b0, 1'b1:
    begin
      case (WIR[7:0])
        // EMBEDDED_INSTR_OPCODE
        8'h03            : MuxWso = Jtag1500bcSebSibsTdo;// BC1500         // UBTS 470941
        8'h04     : MuxWso = Jtag1500bcSebSibsTdo;// BC1500_QRYUPD  // UBTS 470941
        8'h25            : MuxWso = Jtag1500bcSebSibsTdo;// QRYUPD         // UBTS 528914
        8'h05          : MuxWso = AltTapEnTdo;         // ALTTAPEN
        8'h07      : MuxWso = SingleChainLocalSdo; // SYSSCANSHIFT   // UBTS 470941
        8'h20     : MuxWso = SingleChainLocalSdo; // SYSSCANSHIFT2  // UBTS 470941
        8'hFF            : MuxWso = BypassTdo;           // BYPASS
        8'h00           : MuxWso = BypassTdo;           // WBYPASS
        // DEFAULT_INSTR_OPCODE
        8'h09                                   : MuxWso = SCANCONFIG_Tdo;
        8'h21                                   : MuxWso = SYSSCAN_Tdo;
        // S_USER_INSTR_OPCODE
        8'h34                                   : MuxWso = SMSG_CONTROL_Tdo;
        8'h35                                   : MuxWso = SMSG_STATUS_Tdo;
        8'h36                                   : MuxWso = JPC_WIR_Tdo;
        8'h37                                   : MuxWso = JPC_WDR_Tdo;
        8'h38                                   : MuxWso = SMS_WIR_Tdo;
        8'h39                                   : MuxWso = SMS_WDR_Tdo;
        8'h3A                                   : MuxWso = FUSE_IF_ATE_MODE_Tdo;
        8'h3B                                   : MuxWso = FUSE_IF_SETUP_Tdo;
        8'h3C                                   : MuxWso = FUSE_IF_STATUS_Tdo;
        default                                     : MuxWso = BypassTdo;
      endcase
    end
    default                                         : MuxWso = 1'bx;
  endcase
end

// Stitch in the child STACs SIB muxing and 1500 clients SIB muxing.  UBTS 470941
// WSI -> Local STAC -> child STACs -> 1500 clients -> WSO

// Bypass the connection since no child STACs exist
  wire                                          Jtag1687aDrvTdo;
assign Jtag1687aDrvTdo                        = LocCORE_Wso;
// Stitch in the 1500 clients after the child STACs (i.e. Jtag1687aDrvTdo)
assign Jtag1500bcDrvTdi                       = Jtag1687aDrvTdo;

// UBTS 558454
DFXIP4_GC10_IP_MUX I_WSO_MUX (
  .Sel ( Stac_PipelineEn ),
  .In0 (  NonPipelineWso ),
  .In1 (     PipelineWso ),
  .Out (        Stac_Wso )
);

assign NonPipelineWso = Stac_AltTapEn ? AltTapDrvTdo : Jtag1500bcDrvTdo;

DFXIP4_GC10_JTAGTDR_NO_UPD #(.LENGTH(1)) I_WSO_PIPELINE (
  .RSTVAL                  ( 1'b0                      ),
  .CAP_D                   ( 1'b0                      ),
  .TDR_Q                   ( Pipeline_UNUSED           ),
  .CAP                     ( 1'b0                      ),
  .SHF                     ( 1'b1                      ),
  .TRST                    ( ~Stac_WrstN               ),
  .TCK                     ( Stac_Wrck                 ),
  .TDI                     ( NonPipelineWso            ),
  .TDO                     (    PipelineWso            ),
  .SEL                     ( 1'b1                      )
);
//! unused Pipeline_UNUSED
//=============================================================================
// BYPASS
//=============================================================================

// The BYPASS is part of the default instruction set.

DFXIP4_GC10_JTAGTDR_NO_UPD #(.LENGTH(1)) I_BYPASS (
  .RSTVAL                  ( 1'b0                      ),
  .CAP_D                   ( 1'b0                      ),
  .TDR_Q                   ( Bypass_UNUSED             ),
  .CAP                     ( WdrCapture                ),
  .SHF                     ( WdrShift                  ),
  .TRST                    ( WdrWrst                   ),
  .TCK                     ( WdrWrck                   ),
  .TDI                     ( WdrWsi                    ),
  .TDO                     ( BypassTdo                 ),
  .SEL                     ( 1'b1                      )
);
//! unused Bypass_UNUSED

//=============================================================================
// DEFAULT instructions
//=============================================================================

// The SCANCONFIG is part of the DEFAULT_INSTR_OPCODE set.
DFXIP4_GC10_JTAGTDR_UPD I_SCANCONFIG_En (
  .RSTVAL                  ( 1'b0                      ),
  .UPD_D                   ( WirShiftReg[7:0] == 8'h09 ),
  .TDR_Q                   ( SCANCONFIG_En               ),
  .UPD                     ( WirUpdate                 ),
  .TRST                    ( WdrWrst | TapInTlr        ),
  .TCK                     ( WdrWrck                   ),
  .SEL                     ( 1'b1                      )
);
// The SYSSCAN is part of the DEFAULT_INSTR_OPCODE set.
DFXIP4_GC10_JTAGTDR_UPD I_SYSSCAN_En (
  .RSTVAL                  ( 1'b0                      ),
  .UPD_D                   ( WirShiftReg[7:0] == 8'h21 ),
  .TDR_Q                   ( SYSSCAN_En               ),
  .UPD                     ( WirUpdate                 ),
  .TRST                    ( WdrWrst | TapInTlr        ),
  .TCK                     ( WdrWrck                   ),
  .SEL                     ( 1'b1                      )
);

//=============================================================================
// USER instructions
//=============================================================================

// The FUSE_IF_ATE_MODE is part of the S_USER_INSTR_OPCODE set.
DFXIP4_GC10_JTAGTDR_UPD I_FUSE_IF_ATE_MODE_En (
  .RSTVAL                  ( 1'b0                      ),
  .UPD_D                   ( WirShiftReg[7:0] == 8'h3A ),
  .TDR_Q                   ( FUSE_IF_ATE_MODE_En               ),
  .UPD                     ( WirUpdate                 ),
  .TRST                    ( WdrWrst | TapInTlr        ),
  .TCK                     ( WdrWrck                   ),
  .SEL                     ( 1'b1                      )
);
// The FUSE_IF_SETUP is part of the S_USER_INSTR_OPCODE set.
DFXIP4_GC10_JTAGTDR_UPD I_FUSE_IF_SETUP_En (
  .RSTVAL                  ( 1'b0                      ),
  .UPD_D                   ( WirShiftReg[7:0] == 8'h3B ),
  .TDR_Q                   ( FUSE_IF_SETUP_En               ),
  .UPD                     ( WirUpdate                 ),
  .TRST                    ( WdrWrst | TapInTlr        ),
  .TCK                     ( WdrWrck                   ),
  .SEL                     ( 1'b1                      )
);
// The FUSE_IF_STATUS is part of the S_USER_INSTR_OPCODE set.
DFXIP4_GC10_JTAGTDR_UPD I_FUSE_IF_STATUS_En (
  .RSTVAL                  ( 1'b0                      ),
  .UPD_D                   ( WirShiftReg[7:0] == 8'h3C ),
  .TDR_Q                   ( FUSE_IF_STATUS_En               ),
  .UPD                     ( WirUpdate                 ),
  .TRST                    ( WdrWrst | TapInTlr        ),
  .TCK                     ( WdrWrck                   ),
  .SEL                     ( 1'b1                      )
);
// The JPC_WDR is part of the S_USER_INSTR_OPCODE set.
DFXIP4_GC10_JTAGTDR_UPD I_JPC_WDR_En (
  .RSTVAL                  ( 1'b0                      ),
  .UPD_D                   ( WirShiftReg[7:0] == 8'h37 ),
  .TDR_Q                   ( JPC_WDR_En               ),
  .UPD                     ( WirUpdate                 ),
  .TRST                    ( WdrWrst | TapInTlr        ),
  .TCK                     ( WdrWrck                   ),
  .SEL                     ( 1'b1                      )
);
// The JPC_WIR is part of the S_USER_INSTR_OPCODE set.
DFXIP4_GC10_JTAGTDR_UPD I_JPC_WIR_En (
  .RSTVAL                  ( 1'b0                      ),
  .UPD_D                   ( WirShiftReg[7:0] == 8'h36 ),
  .TDR_Q                   ( JPC_WIR_En               ),
  .UPD                     ( WirUpdate                 ),
  .TRST                    ( WdrWrst | TapInTlr        ),
  .TCK                     ( WdrWrck                   ),
  .SEL                     ( 1'b1                      )
);
// The SMSG_CONTROL is part of the S_USER_INSTR_OPCODE set.
DFXIP4_GC10_JTAGTDR_UPD I_SMSG_CONTROL_En (
  .RSTVAL                  ( 1'b0                      ),
  .UPD_D                   ( WirShiftReg[7:0] == 8'h34 ),
  .TDR_Q                   ( SMSG_CONTROL_En               ),
  .UPD                     ( WirUpdate                 ),
  .TRST                    ( WdrWrst | TapInTlr        ),
  .TCK                     ( WdrWrck                   ),
  .SEL                     ( 1'b1                      )
);
// The SMSG_STATUS is part of the S_USER_INSTR_OPCODE set.
DFXIP4_GC10_JTAGTDR_UPD I_SMSG_STATUS_En (
  .RSTVAL                  ( 1'b0                      ),
  .UPD_D                   ( WirShiftReg[7:0] == 8'h35 ),
  .TDR_Q                   ( SMSG_STATUS_En               ),
  .UPD                     ( WirUpdate                 ),
  .TRST                    ( WdrWrst | TapInTlr        ),
  .TCK                     ( WdrWrck                   ),
  .SEL                     ( 1'b1                      )
);
// The SMS_WDR is part of the S_USER_INSTR_OPCODE set.
DFXIP4_GC10_JTAGTDR_UPD I_SMS_WDR_En (
  .RSTVAL                  ( 1'b0                      ),
  .UPD_D                   ( WirShiftReg[7:0] == 8'h39 ),
  .TDR_Q                   ( SMS_WDR_En               ),
  .UPD                     ( WirUpdate                 ),
  .TRST                    ( WdrWrst | TapInTlr        ),
  .TCK                     ( WdrWrck                   ),
  .SEL                     ( 1'b1                      )
);
// The SMS_WIR is part of the S_USER_INSTR_OPCODE set.
DFXIP4_GC10_JTAGTDR_UPD I_SMS_WIR_En (
  .RSTVAL                  ( 1'b0                      ),
  .UPD_D                   ( WirShiftReg[7:0] == 8'h38 ),
  .TDR_Q                   ( SMS_WIR_En               ),
  .UPD                     ( WirUpdate                 ),
  .TRST                    ( WdrWrst | TapInTlr        ),
  .TCK                     ( WdrWrck                   ),
  .SEL                     ( 1'b1                      )
);

//=============================================================================
// SYSSCANSHIFT
//=============================================================================

// The SYSSCANSHIFT is part of the default instruction set.
// This bypasses the SWB, so we just have to time SC2 for scan shift.

DFXIP4_GC10_JTAGTDR_UPD I_SYSSCANSHIFT_En (
  .RSTVAL                  ( 1'b0                      ),
  .UPD_D                   ( WirShiftReg[7:0] == 8'h07 ),
  .TDR_Q                   ( SYSSCANSHIFT_En           ),
  .UPD                     ( WirUpdate                 ),
  .TRST                    ( WdrWrst | TapInTlr        ),
  .TCK                     ( WdrWrck                   ),
  .SEL                     ( 1'b1                      )
);

//=============================================================================
// SYSSCANSHIFT2
//=============================================================================

// The SYSSCANSHIFT2 is part of the default instruction set.
// Doesn't bypass the SWB, so we have to time TCK/WRCK/SC2 for scan shift.

DFXIP4_GC10_JTAGTDR_UPD I_SYSSCANSHIFT2_En (
  .RSTVAL                  ( 1'b0                      ),
  .UPD_D                   ( WirShiftReg[7:0] == 8'h20 ),
  .TDR_Q                   ( SYSSCANSHIFT2_En          ),
  .UPD                     ( WirUpdate                 ),
  .TRST                    ( WdrWrst | TapInTlr        ),
  .TCK                     ( WdrWrck                   ),
  .SEL                     ( 1'b1                      )
);

assign SysScanShiftMode = SYSSCANSHIFT_En | SYSSCANSHIFT2_En;

//=============================================================================
// ALTTAPEN
//=============================================================================

// The ALTTAPEN is part of the default instruction set.
// The length of this register is NUM_STAC (upper portion) + NUM_ALTTAP (lower portion) + 1 (for local STAC).
// Also remember that bit 0 is reserved to access this local STAC.

assign ALTTAPEN_En = (WIR[7:0] == 8'h05);

DFXIP4_GC10_JTAGTDR #(.LENGTH(1)) I_ALTTAPEN (
  .RSTVAL                  ( {1{1'b0}}          ),
  .CAP_D                   ( AltTapEnReg[1-1:0] ),
  .TDR_Q                   ( AltTapEnReg[1-1:0] ),
  .CAP                     ( WdrCapture                       ),
  .SHF                     ( WdrShift                         ),
  .UPD                     ( WdrUpdate                        ),
  .TRST                    ( AltTapEnRegTrst | AltTapEnFall   ),    // Resets when MTAP deasserts AltTapEn
  .TCK                     ( WdrWrck                          ),
  .TDI                     ( WdrWsi                           ),
  .TDO                     ( AltTapEnTdo                      ),
  .SEL                     ( ALTTAPEN_En                      )
);

DFXIP4_GC10_IP_JTAGFLOP I_ALTTAPEN_D1 (
  .ASYNCCLK                ( WdrWrst       ),
  .ASYNCVAL                ( 1'b0          ),
  .CLK                     ( WdrWrck       ),
  .D                       ( Stac_AltTapEn ),
  .Q                       ( AltTapEn_d1   )
);

always @* begin
  AltTapEnFall = ~Stac_AltTapEn & AltTapEn_d1;
end

// AEB - access enablement bit
//  1 : enables access
//  0 : disables access

// Always enable the AltTap AEB for the local STAC since
// the local STAC has an AEB gating the WIR directly.
assign AltTapEnAeb[0] = 1'b1;
// Use the STACs' AEBs when trying to access the STACs via ALTTAPEN

assign QRYUPD_En   = (WIR[7:0] == 8'h25);        // UBTS 528914



//=============================================================================
// JTAG1500BC
//=============================================================================

// The JTAG1500BC is part of the default instruction set.
// The length of this register is just 1 + 1 SEB + 1 SBC.
// Both TLR (TapInTlr) and TRST will reset SIBs, SWBs and any other network configuration bits.

assign BC1500_En           = (WIR[7:0] == 8'h03);
assign BC1500_QRYUPD_only  = (WIR[7:0] == 8'h04); // UBTS 528914
assign BC1500_QRYUPD_En    = BC1500_QRYUPD_only | QRYUPD_En;                        // UBTS 528914

// UBTS 475115
DFXIP4_GC10_IP_JTAGFLOP I_BC1500_QRYUPD_EN_D1 (
  .ASYNCCLK                ( WdrWrst | TapInTlr        ),
  .ASYNCVAL                ( 1'b0                      ),
  .CLK                     ( WdrWrck                   ),
  .D                       ( BC1500_QRYUPD_En          ),
  .Q                       ( BC1500_QRYUPD_En_d1       )
);

// Selective Broadcast bit (for JTAG1500BC)
//  1 : broadcast mode
//  0 : default, serial mode

DFXIP4_GC10_JTAGTDR #(.LENGTH(1)) I_JTAG1500BC_SBC (
  .RSTVAL                  ( 1'b0                      ),
  .CAP_D                   ( Jtag1500bcSbc             ),
  .TDR_Q                   ( Jtag1500bcSbc_m1          ),
  .CAP                     ( WdrCapture                ),
  .SHF                     ( WdrShift                  ),
  .UPD                     ( WdrUpdate                 ),
  .TRST                    ( WdrWrst | TapInTlr        ),
  .TCK                     ( WdrWrck                   ),
  .TDI                     ( WdrWsi                    ),
  .TDO                     ( Jtag1500bcSbcTdo          ),
  .SEL                     ( BC1500_En | BC1500_QRYUPD_En )
);

DFXIP4_GC10_IP_JTAGFLOP I_JTAG1500BC_SBC_D0 (
  .ASYNCCLK                ( WdrWrst | TapInTlr        ),
  .ASYNCVAL                ( 1'b0                      ),
  .CLK                     ( WdrWrck                   ),
  .D                       ( Jtag1500bcSbc_m1          ),
  .Q                       ( Jtag1500bcSbc             )
);

// SIB exclusion bit (for JTAG1500BC)
//  1 : bypass & disable the SIBs
//  0 : default, include the SIBs

DFXIP4_GC10_JTAGTDR #(.LENGTH(1)) I_JTAG1500BC_SEB (
  .RSTVAL                  ( 1'b0                      ),
  .CAP_D                   ( Jtag1500bcSeb             ),
  .TDR_Q                   ( Jtag1500bcSeb_m1          ),
  .CAP                     ( WdrCapture                ),
  .SHF                     ( WdrShift                  ),
  .UPD                     ( WdrUpdate                 ),
  .TRST                    ( WdrWrst | TapInTlr        ),
  .TCK                     ( WdrWrck                   ),
  .TDI                     ( Jtag1500bcSbcTdo          ),
  .TDO                     ( Jtag1500bcSebTdo          ),
  .SEL                     ( BC1500_En | BC1500_QRYUPD_En )
);

DFXIP4_GC10_IP_JTAGFLOP I_JTAG1500BC_SEB_D0 (
  .ASYNCCLK                ( WdrWrst | TapInTlr        ),
  .ASYNCVAL                ( 1'b0                      ),
  .CLK                     ( WdrWrck                   ),
  .D                       ( Jtag1500bcSeb_m1          ),
  .Q                       ( Jtag1500bcSeb             )
);

// SIBs (for JTAG1500BC)
//  1 : include the segment
//  0 : default, bypass the segment

DFXIP4_GC10_JTAGTDR #(.LENGTH(1)) I_JTAG1500BC (
  .RSTVAL                  ( {1{1'b0}} ),
  .CAP_D                   ( Jtag1500bcReg[1-1:0] ),
  .TDR_Q                   ( Jtag1500bcReg_m1[1-1:0] ),
  .CAP                     ( WdrCapture                ),
  .SHF                     ( WdrShift                  ),
  .UPD                     ( WdrUpdate                 ),
  .TRST                    ( WdrWrst | TapInTlr        ),
  .TCK                     ( WdrWrck                   ),
  .TDI                     ( Jtag1500bcSebTdo          ),
  .TDO                     ( Jtag1500bcSibsTdo         ),
//.SEL                     ( ~Jtag1500bcSeb & (BC1500_En | BC1500_QRYUPD_En )     )   // UBTS 469977
  .SEL                     ( BC1500_QRYUPD_En | (~Jtag1500bcSeb & BC1500_En )     )   // UBTS 474933
);

// UBTS 467380
DFXIP4_GC10_IP_JTAGFLOP I_JTAG1500BC_D0[1-1:0] (
  .ASYNCCLK                ( {1{WdrWrst | TapInTlr}} ),
  .ASYNCVAL                ( {1{1'b0}} ),
  .CLK                     ( {1{WdrWrck}} ),
  .D                       ( Jtag1500bcReg_m1[1-1:0] ),
  .Q                       ( Jtag1500bcReg[1-1:0] )
);

assign Jtag1500bcSebSibsTdo = (BC1500_QRYUPD_En | ~Jtag1500bcSeb) ? Jtag1500bcSibsTdo : Jtag1500bcSebTdo;

// AEB - access enablement bit
//  1 : enables access
//  0 : disables access

assign Jtag1500bcAeb[0] = BC1500_0_AEB;




//=============================================================================
// 1500 Broadcast Drivers
//=============================================================================

// Generate the control signals for the 1500 client clusters.

gc12_BC1500_DRIVER #(.NUM_DRIVER(1)) I_BC1500_DRIVER (
  .DriverEn                ( Stac_SelectWSP            ),
  .AltTapEn                ( AltTapEn_FsmDis           ),
  .QryUpdEn                ( BC1500_QRYUPD_En_d1       ),
  .Trst                    ( SyncWrst                  ),
  .TapInRti                ( TapInRtiNxt               ),    // They use this for the ROs which need it early.  UBTS 468324
  .TapInTlr                ( TapInTlr                  ),
  .TapInCapDR              ( TapInCapDR                ),
  .TapCapDR                ( TapCapDR                  ),
  .TapShfDR                ( TapShfDR                  ),
  .Jtag1500SwbCaptureWR    ( Jtag1500SwbCaptureWR[1-1:0] ), // DECDFXIP41-168
  .Jtag1500SwbShiftWR      ( Jtag1500SwbShiftWR[1-1:0]   ),
  .TapInShfDR              ( TapInShfDR                ),
  .TapInUpdDR              ( TapInUpdDR                ),
  .SyncClk                 ( Stac_Wrck                 ),
  .Jtag1500bcAeb           ( Jtag1500bcAeb[1-1:0] ),
  .Jtag1500bcReg           ( Jtag1500bcReg[1-1:0] ),
  .Jtag1500bcSbc           ( Jtag1500bcSbc             ),
  .Jtag1500bcDrvTdi        ( Jtag1500bcDrvTdi          ),
  .Jtag1500bcDrvTdo        ( Jtag1500bcDrvTdo          ),

// Intentionally left unused.  UBTS 470941
  .SingleChainEn           ( 1'b0                      ),
  .SingleChainSdi          ( 1'b0                      ),
  .SingleChainSdo          (                           ),

  .Jtag1500bcSib           ( Jtag1500bcSib[1-1:0]       ),
  .Jtag1500bcShiftWR       ( Jtag1500bcShiftWR[1-1:0]   ),
  .Jtag1500bcCaptureWR     ( Jtag1500bcCaptureWR[1-1:0] ),
  .Jtag1500bcUpdateWR      ( Jtag1500bcUpdateWR[1-1:0]  ),
  .Jtag1500bcTlr           ( Jtag1500bcTlr[1-1:0]       ),
  .Jtag1500bcRti           ( Jtag1500bcRti[1-1:0]       ),
  .Jtag1500bcTrst          ( Jtag1500bcTrst[1-1:0]      ),
  .Jtag1500bcTdi           ( Jtag1500bcTdi[1-1:0]       ),
  .Jtag1500bcTdo           ( Jtag1500bcTdo[1-1:0]       )
);



//=============================================================================
// 1500 Broadcast Interface for BC1500_0
//=============================================================================

// Connect up the 1500 ports to the 1500 internal drivers.
// Rename 1500 ports for BIA compliance.

DFXIP4_GC10_IP_MARKER I_Mrk_BC1500_0_Wso        ( .Z(Jtag1500bcTdo[0]),     .A(BC1500_0_Wso)           );
DFXIP4_GC10_IP_MARKER I_Mrk_BC1500_0_select_wir ( .Z(BC1500_0_select_wir),  .A(Jtag1500bcSelectWIR[0]) );
DFXIP4_GC10_IP_MARKER I_Mrk_BC1500_0_shift      ( .Z(BC1500_0_shift),       .A(Jtag1500bcShiftWR[0])   );
DFXIP4_GC10_IP_MARKER I_Mrk_BC1500_0_capture    ( .Z(BC1500_0_capture),     .A(Jtag1500bcCaptureWR[0]) );
DFXIP4_GC10_IP_MARKER I_Mrk_BC1500_0_update     ( .Z(BC1500_0_update),      .A(Jtag1500bcUpdateWR[0])  );
DFXIP4_GC10_IP_MARKER I_Mrk_BC1500_0_tlr        ( .Z(BC1500_0_tlr),         .A(Jtag1500bcTlr[0])       );
DFXIP4_GC10_IP_MARKER I_Mrk_BC1500_0_idle       ( .Z(BC1500_0_idle),        .A(Jtag1500bcRti[0])       );
DFXIP4_GC10_IP_MARKER I_Mrk_BC1500_0_tresetb    ( .Z(BC1500_0_tresetb),     .A(~Jtag1500bcTrst[0])     );

DFXIP4_GC10_IP_CLKMARKER I_Mrk_BC1500_0_Wrck (
  .Z                         ( BC1500_0_Wrck ),
  .A                         ( Stac_Wrck             )
);

// SWB bit for a cluster of 1500 clients.
// Freeze the SWB in single chain mode.
// Both TLR (TapInTlr) and TRST will reset SIBs, SWBs and any other network configuration bits.

gc12_SWB 
#(
  .NEGEDGE_SWB ( 0 )
) I_JTAG1500BC_SWB_0 (
  .ZERO_BIT_SHIFT_IR       ( 1'b0                                 ), // UBTS 528914 TODO: OK to tie off?
  .SYSSCANSHIFT_En         ( SYSSCANSHIFT_En                      ), // Don't use the SYSSCANSHIFT2
  .SwbEn                   ( Jtag1500bcSib[0]             ),
  .SwbCapture              ( Jtag1500SwbCaptureWR[0]      ),// DECDFXIP41-168
  .SwbShift                ( Jtag1500SwbShiftWR[0]        ),
  .SwbUpdate               ( Jtag1500bcUpdateWR[0]        ),
  .TapInUpdIR              ( TapInUpdIR                           ),
  .Trst                    ( Jtag1500bcTrst[0] | TapInTlr ),
  .Tck                     ( BC1500_0_Wrck                ),
  .Tdi                     ( Jtag1500bcTdi[0]             ),
  .Tdo                     ( BC1500_0_Wsi                 ),
  .SWB                     ( Jtag1500bcSelectWIR[0]       )
);


//=============================================================================
// ALTTAP Drivers
//=============================================================================

// Generate the control signals for the ALTTAPs.

gc12_ALTTAP_DRIVER #(.NUM_DRIVER(1),.NUM_ALTTAP(0)) I_ALTTAP_DRIVER (
  .AltTapEn                ( Stac_AltTapEn                  ),
  .Trst                    ( ~Stac_WrstN                    ),
  .Tms                     ( Stac_Tms                       ),
  .SyncClk                 ( Stac_Wrck                      ),
  .AltTapEnAeb             ( AltTapEnAeb[1-1:0] ),
  .AltTapEnReg             ( AltTapEnReg[1-1:0] ),
  .AltTapDrvTdi            ( Stac_Wsi                       ),
  .AltTapDrvTdo            ( AltTapDrvTdo                   ),

  .AltTapEnBit             ( AltTapEnBit[1-1:0] ),
  .AltTapTrst              ( AltTapTrst[1-1:0] ),
  .AltTapTms               ( AltTapTms[1-1:0] ),
//! disable checks_for_cycles
  .AltTapTdi               ( AltTapTdi[1-1:0] ),
  .AltTapTdo               ( AltTapTdo[1-1:0] )
//! enable checks_for_cycles
);

// Add one more to NUM_ALTTAP to account for the MTAP which uses bit 0.

assign Stac_AltTdoEn = Stac_AltTapEn & (|AltTapTdoEn[1-1:0]);

//=============================================================================
// SingleChain Drivers
//=============================================================================

// Stitch up the single chain segments from the various child i/f
// Note that the single chain from the child STACs and 1500 clients
// are stitched between LocCORE_Wso and WSO.  UBTS 470941

// The single chain from this STAC goes first
assign SingleChainEn        = SysScanShiftMode;
assign SingleChainSdi       = WdrWsi;
DFXIP4_GC10_IP_MARKER I_Mrk_ScmSdo (
  .Z ( SingleChainLocalSdo ),
  .A ( SingleChainSdo      )
); // Eswar requested

//=============================================================================
// Misc
//=============================================================================

// Just a way to get rid of the RPL warnings/errors
  wire                                          AltTapEnBit_UNUSED;
assign AltTapEnBit_UNUSED = |AltTapEnBit[1-1:0];
//! unused AltTapEnBit_UNUSED

`include "stac_assertions_gc12.vh"

endmodule

// leda B_1001 on
// leda B_1011 on
