

//================================================================================================================================
//
// Copyright (c) 2002 by Advanced Micro Devices, Inc.
//
// This file is protected by Federal Copyright Law, with all rights reserved. No part of this file may be reproduced, stored in a
// retrieval system, translated, transcribed, or transmitted, in any form, or by any means manual, electric, electronic,
// mechanical, electro-magnetic, chemical, optical, or otherwise, without prior explicit written permission from Advanced Micro
// Devices, Inc.
//
//================================================================================================================================
//! disable one_module_per_file
// leda B_1001 off
//~ format_rtl 1
//~ format_inst narrow

`timescale 1ps/1ps

module gc11_ALTTAP_DRIVER #(parameter
  NUM_DRIVER = 10,
  NUM_ALTTAP = 0
)(

  input                                         AltTapEn,
  input                                         Trst,
  input                                         Tms,
  input                                         SyncClk,
  input                    [NUM_DRIVER-1:0]     AltTapEnAeb,
  input                    [NUM_DRIVER-1:0]     AltTapEnReg,
  input                                         AltTapDrvTdi,
  output   reg                                  AltTapDrvTdo,

  output   reg             [NUM_DRIVER-1:0]     AltTapEnBit,
  output   reg             [NUM_DRIVER-1:0]     AltTapTrst,
  output   reg             [NUM_DRIVER-1:0]     AltTapTms,
  output   reg             [NUM_DRIVER-1:0]     AltTapTdi,
  input                    [NUM_DRIVER-1:0]     AltTapTdo

);

  wire                     [NUM_DRIVER-1:0]     AltTapEnAebSync;
  reg                      [NUM_DRIVER-1:0]     AltTapTdoMux;
  integer                                       i;

DFXIP4_GC10_IP_SYNC I_SYNC_AEB[NUM_DRIVER-1:0] (
  .SampleClk               ( {NUM_DRIVER{SyncClk}}     ),
  .DataIn                  ( AltTapEnAeb[NUM_DRIVER-1:0] ),
  .DataOut                 ( AltTapEnAebSync[NUM_DRIVER-1:0] )
);

always @* begin
  for (i=0; i<NUM_DRIVER; i=i+1) begin
    if (i>=1 & i<NUM_ALTTAP+1) begin
      // This index range is for the actual ALTTAPs.  For these
      // we hold them in reset until the global AltTapEn asserts
      // which should be on negedge after entering RTI.
      AltTapEnBit[i]  = AltTapEn &  AltTapEnReg[i] & AltTapEnAebSync[i];
      AltTapTrst[i]   = Trst     | ~AltTapEnBit[i]; // Keep ALTTAP in TLR when not being used
      AltTapTms[i]    = Tms      &  AltTapEnBit[i];
    end else begin
      // Everything else is for the child STACs.  For these we "enable"
      // the drivers as soon as their respective AltTapEn bit is set.
      // Then once the global AltTapEn asserts, it'll switch a
      // mux and allow the AltTap drivers to control the child STACs.
      AltTapEnBit[i]  = AltTapEn &  AltTapEnReg[i] & AltTapEnAebSync[i];
      AltTapTrst[i]   = Trst     &  AltTapEnReg[i]; // Keep child STACs in RTI when in AltTap mode and not being used
      AltTapTms[i]    = Tms      &  AltTapEnReg[i];
    end
  end
end

// Stitch the ALTTAPs and STACs together.  Note that this local STAC will
// be stitched at the very end of the chain (closest to WSO).

// Example: NUM_STAC==3, NUM_ALTTAP==3
//  WSI -> AltTapTdi[6] -> STAC_Wsi[2]    -> [STAC] ->    STAC_Wso[2] -> AltTapTdo[6] -> AltTapTdoMux[6]
//      -> AltTapTdi[5] -> STAC_Wsi[1]    -> [STAC] ->    STAC_Wso[1] -> AltTapTdo[5] -> AltTapTdoMux[5]
//      -> AltTapTdi[4] -> STAC_Wsi[0]    -> [STAC] ->    STAC_Wso[0] -> AltTapTdo[4] -> AltTapTdoMux[4]
//      -> AltTapTdi[3] -> ALTTAP_Tdi[3] -> [ALTTAP] -> ALTTAP_Tdo[3] -> AltTapTdo[3] -> AltTapTdoMux[3]
//      -> AltTapTdi[2] -> ALTTAP_Tdi[2] -> [ALTTAP] -> ALTTAP_Tdo[2] -> AltTapTdo[2] -> AltTapTdoMux[2]
//      -> AltTapTdi[1] -> ALTTAP_Tdi[1] -> [ALTTAP] -> ALTTAP_Tdo[1] -> AltTapTdo[1] -> AltTapTdoMux[1]
//      -> AltTapTdi[0] -> LocSTAC_Wsi -> [local STAC] -> LocSTAC_Wso -> AltTapTdo[0] -> AltTapTdoMux[0]
//      -> AltTapDrvTdo
//      -> WSO

// I don't believe there is a combination loop here, but RPL is still complaining about it. -pphan
//! disable checks_for_cycles
always @*
  // The first TAP (MSB) always get the AltTapDrvTdi
  // The last TAP (LSB) drives the AltTapDrvTdo
  {AltTapTdi[NUM_DRIVER-1:0],AltTapDrvTdo} = {AltTapDrvTdi,AltTapTdoMux[NUM_DRIVER-1:0]};
//! enable checks_for_cycles

always @*
  for (i=NUM_DRIVER-1; i>=0; i=i-1) begin
    AltTapTdoMux[i] = AltTapEnBit[i] ? AltTapTdo[i] : AltTapTdi[i];
  end

endmodule


module gc11_STAC_DRIVER #(parameter
  NUM_DRIVER = 10
)(

  input                                         DriverEn,
  input                                         AltTapEn,
  input                                         QryUpdEn,
  input                                         Trst,
  input                                         Tms,
  input                                         SyncClk,
  input                    [NUM_DRIVER-1:0]     Jtag1687aAeb,
  input                    [NUM_DRIVER-1:0]     Jtag1687aReg,
  input                                         Jtag1687aSbc, // Selective Broadcast Enable UBTS 504749
  input                                         Jtag1687aDrvTdi,
  output   reg                                  Jtag1687aDrvTdo,

  input                                         SingleChainEn,
  input                                         SingleChainSdi,
  output   reg                                  SingleChainSdo,

  output   reg             [NUM_DRIVER-1:0]     Jtag1687aSib,
  output   reg             [NUM_DRIVER-1:0]     Jtag1687aTms,
  output   reg             [NUM_DRIVER-1:0]     Jtag1687aTrst,
  output   reg             [NUM_DRIVER-1:0]     Jtag1687aTdi,
  input                    [NUM_DRIVER-1:0]     Jtag1687aTdo

);

  reg                                           Jtag1687aDrvEn;
  reg                                           JtagOrScanSdiMux;
  wire                     [NUM_DRIVER-1:0]     Jtag1687aAebSync;
  reg                      [NUM_DRIVER-1:0]     Jtag1687aTdoMux;
  reg                      [NUM_DRIVER-1:0]     Jtag1687aPreSbcTdi;
  integer                                       i;

DFXIP4_GC10_IP_SYNC I_SYNC_AEB[NUM_DRIVER-1:0] (
  .SampleClk               ( {NUM_DRIVER{SyncClk}}     ),
  .DataIn                  ( Jtag1687aAeb[NUM_DRIVER-1:0] ),
  .DataOut                 ( Jtag1687aAebSync[NUM_DRIVER-1:0] )
);

always @* begin
  Jtag1687aDrvEn      = (DriverEn | SingleChainEn) & ~QryUpdEn;
  for (i=0; i<NUM_DRIVER; i=i+1) begin
    // Freeze all of this when AltTapEn is asserted.
    Jtag1687aSib[i]   = Jtag1687aDrvEn & Jtag1687aReg[i] & ~AltTapEn & Jtag1687aAebSync[i];
    Jtag1687aTrst[i]  = Trst                             & ~AltTapEn;
    Jtag1687aTms[i]   = Tms                              & ~AltTapEn;   // UBTS 470579
  end
end

// Stitch the SIBs and child STACs together.  All the SIBs go first (closest to WSI).

// Example: NUM_STAC==4
//  WSI -> WdrWsi -> [I_JTAG1687A_SEB] -> [I_JTAG1687A] -> Jtag1687aDrvTdi
//      -> Jtag1687aTdi[3] -> STAC_Wsi[3] -> [STAC3] -> STAC_Wso[3] -> Jtag1687aTdo[3] -> Jtag1687aTdoMux[3]
//      -> Jtag1687aTdi[2] -> STAC_Wsi[2] -> [STAC2] -> STAC_Wso[2] -> Jtag1687aTdo[2] -> Jtag1687aTdoMux[2]
//      -> Jtag1687aTdi[1] -> STAC_Wsi[1] -> [STAC1] -> STAC_Wso[1] -> Jtag1687aTdo[1] -> Jtag1687aTdoMux[1]
//      -> Jtag1687aTdi[0] -> STAC_Wsi[0] -> [STAC0] -> STAC_Wso[0] -> Jtag1687aTdo[0] -> Jtag1687aTdoMux[0]
//      -> Jtag1687aDrvTdo
//      -> MuxWso
//      -> LocSTAC_Wso
//      -> WSO

// I don't believe there is a combination loop here, but RPL is still complaining about it. -pphan
//! disable checks_for_cycles
always @*
  // The first TAP (MSB) always get the TDO from the group of SIBs
  // The last TAP (LSB) drives the Jtag1687aDrvTdo
  {Jtag1687aPreSbcTdi[NUM_DRIVER-1:0],Jtag1687aDrvTdo} = {JtagOrScanSdiMux, Jtag1687aTdoMux[NUM_DRIVER-1:0]};
//! enable checks_for_cycles

always @*
  for (i=NUM_DRIVER-1; i>=0; i=i-1) begin // UBTS 504749
    Jtag1687aTdi[i]    = (Jtag1687aSib[i] & Jtag1687aSbc) ? JtagOrScanSdiMux : Jtag1687aPreSbcTdi[i];
    Jtag1687aTdoMux[i] =  Jtag1687aSib[i]                 ? Jtag1687aTdo[i]  : Jtag1687aPreSbcTdi[i];
  end

// Intercept the SIB stitching for our single chain mode

always @* begin
  JtagOrScanSdiMux  = SingleChainEn ? SingleChainSdi : Jtag1687aDrvTdi;
  SingleChainSdo    = Jtag1687aDrvTdo;
end

endmodule

module gc11_BC1500_DRIVER #(parameter
  NUM_DRIVER = 10
)(

  input                                         DriverEn,
  input                                         AltTapEn,
  input                                         QryUpdEn,
  input                                         Trst,
  input                                         TapInRti,
  input                                         TapInTlr,
  input                                         TapInCapDR,
  input                                         TapInShfDR,
  input                                         TapCapDR,
  input                                         TapShfDR,
  output   reg             [NUM_DRIVER-1:0]     Jtag1500SwbShiftWR,  // DECDFXIP41-168
  output   reg             [NUM_DRIVER-1:0]     Jtag1500SwbCaptureWR,
  input                                         TapInUpdDR,
  input                                         SyncClk,
  input                    [NUM_DRIVER-1:0]     Jtag1500bcAeb,
  input                    [NUM_DRIVER-1:0]     Jtag1500bcReg,
  input                                         Jtag1500bcSbc,       // Selective Broadcast Enable .  UBTS 470355
  input                                         Jtag1500bcDrvTdi,
  output   reg                                  Jtag1500bcDrvTdo,

  input                                         SingleChainEn,
  input                                         SingleChainSdi,
  output   reg                                  SingleChainSdo,

  output   reg             [NUM_DRIVER-1:0]     Jtag1500bcSib,
  output   reg             [NUM_DRIVER-1:0]     Jtag1500bcShiftWR,
  output   reg             [NUM_DRIVER-1:0]     Jtag1500bcCaptureWR,
  output   reg             [NUM_DRIVER-1:0]     Jtag1500bcUpdateWR,
  output   reg             [NUM_DRIVER-1:0]     Jtag1500bcTlr,
  output   reg             [NUM_DRIVER-1:0]     Jtag1500bcRti,
  output   reg             [NUM_DRIVER-1:0]     Jtag1500bcTrst,
  output   reg             [NUM_DRIVER-1:0]     Jtag1500bcTdi,
  input                    [NUM_DRIVER-1:0]     Jtag1500bcTdo

);

  reg                                           Jtag1500bcDrvEn;
  reg                                           JtagOrScanSdiMux;
  wire                     [NUM_DRIVER-1:0]     Jtag1500bcAebSync;
  reg                      [NUM_DRIVER-1:0]     Jtag1500bcTdoMux;
  reg                      [NUM_DRIVER-1:0]     Jtag1500bcPreSbcTdi;
  integer                                       i;
  wire                     [NUM_DRIVER-1:0]     early;
assign                                          early[0] = 1'b1;

DFXIP4_GC10_IP_SYNC I_SYNC_AEB[NUM_DRIVER-1:0] (
  .SampleClk               ( {NUM_DRIVER{SyncClk}}     ),
  .DataIn                  ( Jtag1500bcAeb[NUM_DRIVER-1:0] ),
  .DataOut                 ( Jtag1500bcAebSync[NUM_DRIVER-1:0] )
);

always @* begin
  Jtag1500bcDrvEn           = (DriverEn | SingleChainEn) & ~QryUpdEn;
  for (i=0; i<NUM_DRIVER; i=i+1) begin
    // Freeze all of this when AltTapEn is asserted.
    Jtag1500bcSib[i]        = Jtag1500bcDrvEn & Jtag1500bcReg[i] & ~AltTapEn & Jtag1500bcAebSync[i];
    Jtag1500bcCaptureWR[i]  = (early[i] ? TapCapDR : TapInCapDR) & Jtag1500bcSib[i];
    Jtag1500bcShiftWR[i]    = (early[i] ? TapShfDR : TapInShfDR) & Jtag1500bcSib[i];
    Jtag1500SwbCaptureWR[i] =                        TapInCapDR  & Jtag1500bcSib[i]; // DECDFXIP41-168
    Jtag1500SwbShiftWR[i]   =                        TapInShfDR  & Jtag1500bcSib[i];
    Jtag1500bcUpdateWR[i]   =                        TapInUpdDR  & Jtag1500bcSib[i];
    Jtag1500bcRti[i]        = TapInRti                           & ~AltTapEn;
    Jtag1500bcTlr[i]        = TapInTlr                           & ~AltTapEn;
    Jtag1500bcTrst[i]       = Trst                               & ~AltTapEn;
  end
end

// Stitch the SIBs for each 1500 cluster together.  All the SIBs go first (closest to WSI).

// Example: NUM_1500==4
//  WSI -> WdrWsi -> [I_JTAG1500BC_SEB] -> [I_JTAG1500BC] -> Jtag1500bcDrvTdi
//      -> Jtag1500bcTdi[3] -> [I_JTAG1500BC_SWB3] -> BC1500_Wsi[3] -> [1500CLUSTER3] -> BC1500_Wso[3] -> Jtag1500bcTdo[3] -> Jtag1500bcTdoMux[3]
//      -> Jtag1500bcTdi[2] -> [I_JTAG1500BC_SWB2] -> BC1500_Wsi[2] -> [1500CLUSTER2] -> BC1500_Wso[2] -> Jtag1500bcTdo[2] -> Jtag1500bcTdoMux[2]
//      -> Jtag1500bcTdi[1] -> [I_JTAG1500BC_SWB1] -> BC1500_Wsi[1] -> [1500CLUSTER1] -> BC1500_Wso[1] -> Jtag1500bcTdo[1] -> Jtag1500bcTdoMux[1]
//      -> Jtag1500bcTdi[0] -> [I_JTAG1500BC_SWB0] -> BC1500_Wsi[0] -> [1500CLUSTER0] -> BC1500_Wso[0] -> Jtag1500bcTdo[0] -> Jtag1500bcTdoMux[0]
//      -> Jtag1500bcDrvTdo
//      -> MuxWso
//      -> LocSTAC_Wso
//      -> WSO

// I don't believe there is a combination loop here, but RPL is still complaining about it. -pphan
//! disable checks_for_cycles
always @*
  // The first TAP (MSB) always get the TDO from the group of SIBs
  // The last TAP (LSB) drives the Jtag1500bcDrvTdo
  {Jtag1500bcPreSbcTdi[NUM_DRIVER-1:0],Jtag1500bcDrvTdo} = {JtagOrScanSdiMux, Jtag1500bcTdoMux[NUM_DRIVER-1:0]};
//! enable checks_for_cycles

always @*
  for (i=NUM_DRIVER-1; i>=0; i=i-1) begin
    Jtag1500bcTdi[i]     = (Jtag1500bcSib[i] & Jtag1500bcSbc) ? JtagOrScanSdiMux : Jtag1500bcPreSbcTdi[i];
    Jtag1500bcTdoMux[i]  =  Jtag1500bcSib[i]                  ? Jtag1500bcTdo[i] : Jtag1500bcPreSbcTdi[i];
  end

// Intercept the SIB stitching for our single chain mode

always @* begin
  JtagOrScanSdiMux  = SingleChainEn ? SingleChainSdi : Jtag1500bcDrvTdi;
  SingleChainSdo    = Jtag1500bcDrvTdo;
end

endmodule

module gc11_TAP_RESYNC (

  input         ResyncTrigger,
  input         Trst,
  input         Tck,
  input         TapInRti,

  input         TRST_in,
  input         TMS_in,
  output   reg  TRST_sync,
  output   reg  TMS_sync

);

  wire          GateTapTms;
  wire          GateTapInRti;
  wire          ResyncTrigger_d1;
  wire          GateTapInRti_d1;
  wire          GateTapInRti_d2;
  reg           RstGateTapInRti;
  reg           RstGateTapTms;

//////////////////////////////////////////////////////////////////////////////////////////////////////////
reg  PowerNotOk;
wire PowerNotOkSync;
wire GatedTck;
// ResyncTrigger is intended to be driven by a PowerOK status signal

DFXIP4_GC10_IP_JTAGFLOP I_ResyncTriggerDelay (
  .ASYNCCLK ( PowerNotOk       ),
  .ASYNCVAL ( 1'b1             ),
  .CLK      ( ~Tck             ),
  .D        ( 1'b0             ),
  .Q        ( ResyncTrigger_d1 )
);
DFXIP4_GC10_IP_JTAGFLOP I_ResyncTriggerDelay2 (
  .ASYNCCLK ( PowerNotOk        ),
  .ASYNCVAL ( 1'b1              ),
  .CLK      ( ~Tck              ),
  .D        ( ResyncTrigger_d1  ),
  .Q        ( PowerNotOkSync    )
);

DFXIP4_GC10_IP_JTAGFLOP GATETAPINRTI_A (
  .D        ( GateTapInRti    ),
  .CLK      ( GatedTck        ),
  .Q        ( GateTapInRti_d1 ),
  .ASYNCVAL ( 1'b0            ),
  .ASYNCCLK ( PowerNotOk      )
);
DFXIP4_GC10_IP_JTAGFLOP GATETAPINRTI_B (
  .D        ( GateTapInRti_d1 ),
  .CLK      ( GatedTck        ),
  .Q        ( GateTapInRti_d2 ),
  .ASYNCVAL ( 1'b0            ),
  .ASYNCCLK ( PowerNotOk      )
);
DFXIP4_GC10_IP_JTAGFLOP GATETAPINRTI_C (
  .D        ( GateTapInRti_d2 ),
  .CLK      ( ~GatedTck       ),
  .Q        ( RstGateTapInRti ),
  .ASYNCVAL ( 1'b0            ),
  .ASYNCCLK ( PowerNotOk      )
);

DFXIP4_GC10_IP_SRLATCH GATETAPINRTI (
  .Q        ( GateTapInRti    ),
  .R        ( RstGateTapInRti ),
  .S        ( PowerNotOk      )
);
DFXIP4_GC10_IP_SRLATCH GATETAPTMS (
  .Q        ( GateTapTms     ),
  .R        ( RstGateTapTms  ),
  .S        ( PowerNotOkSync )
);
DFXIP4_GC10_IP_CLKGATER GateTck (
  .C        ( Tck             ),
  .D        ( ~PowerNotOkSync ),
  .CG       ( GatedTck        )
);

always @* begin
  PowerNotOk       = ~ResyncTrigger;
  RstGateTapTms    = TapInRti & ~GateTapInRti;
  TRST_sync        = TRST_in | PowerNotOkSync;
  TMS_sync         = TMS_in  & ~GateTapTms;
end

endmodule

module gc11_SWB 
#( parameter NEGEDGE_SWB = 0)
(
  input        ZERO_BIT_SHIFT_IR, // UBTS 528914
  input        SYSSCANSHIFT_En,
  input        SwbEn,
  input        SwbCapture,
  input        SwbShift,
  input        SwbUpdate,
  input        TapInUpdIR,
  input        Trst,
  input        Tck,
  input        Tdi,
  output  wire Tdo,
  output  wire SWB                             // Effective value of the SWB
);

  wire         SwbTdo;
  wire         SwbBypassNxt;
  wire         SwbBypass;                      // Used to bypass the SWB
  wire         SwbBypass_buf;
  wire         SwbUpdateReg_m1;
  wire         SwbUpdateReg_sh;
  wire         SWB_m1;
  wire         Update;
  wire         SwbTck;
  reg          TdrNxt;

DFXIP4_GC10_JTAGTDR_NO_UPD I_SWB_Shift (
  .RSTVAL ( 1'b1                ),    // UBTS 467381
  .CAP_D  ( SwbUpdateReg_m1     ),
  .TDR_Q  ( SwbUpdateReg_sh     ),
  .CAP    ( SwbCapture          ),
  .SHF    ( SwbShift            ),
  .TRST   ( Trst                ),
  .TCK    ( Tck                 ),
  .TDI    ( Tdi                 ),
  .TDO    ( SwbTdo              ),
  .SEL    ( SwbEn & ~SwbBypass  )     // Disable SWB if it's been bypassed
);

// UBTS 528914 fix: force DR on next shift
assign Update = ZERO_BIT_SHIFT_IR ? TapInUpdIR : SwbUpdate;
always @*
  TdrNxt = Update ? (SwbUpdateReg_sh & ~ZERO_BIT_SHIFT_IR) : SwbUpdateReg_m1;

DFXIP4_GC10_JTAGTDR_UPD I_SWB (
  .RSTVAL ( 1'b1               ),
  .UPD_D  ( TdrNxt             ),
  .TDR_Q  ( SwbUpdateReg_m1    ),
  .UPD    ( Update             ),
  .TRST   ( Trst               ),
  .TCK    ( Tck                ),
  .SEL    ( SwbEn & ~SwbBypass ) // Disable SWB if it's been bypassed
);

DFXIP4_GC10_IP_MARKER I_Mrk_SwbBypass ( .Z(SwbBypass_buf), .A(SwbBypass) );
assign Tdo = SwbBypass_buf ? Tdi : SwbTdo;

// Track the UPDATE_IR and bypass the SWB if we detect that we're in SYSSCANSHIFT (and not SYSSCANSHIFT2).

assign SwbBypassNxt = SYSSCANSHIFT_En & (TapInUpdIR ^ SwbBypass);

DFXIP4_GC10_IP_JTAGFLOP I_SWB_BYP (
  .ASYNCCLK ( Trst         ),
  .ASYNCVAL ( 1'b0         ),
  .CLK      ( Tck          ),
  .D        ( SwbBypassNxt ),
  .Q        ( SwbBypass    )
);

// Force the SWB to 0 so that we select the MuxWso (has the single chain) instead of the WIR,
// i.e. when we are shifting the single chain.
// Note that this only applies to the SYSSCANSHIFT (and not SYSSCANSHIFT2).

assign SWB_m1 = SwbUpdateReg_m1 & ~SwbBypass;

// http://ontrack-internal.amd.com/browse/DECDFXIP41-141
generate
if (NEGEDGE_SWB == 1) begin 
 assign SwbTck = ~Tck; // better margins and hold relative to IEEE1500 SelectWIR timing, sampled on posedge
end else begin
 assign SwbTck =  Tck; // legacy
end
endgenerate

// UBTS 467380
DFXIP4_GC10_IP_JTAGFLOP I_SWB_D0 (
  .ASYNCCLK ( Trst   ),
  .ASYNCVAL ( 1'b1   ),    // UBTS 467381
  .CLK      ( SwbTck ),    // DECDFXIP41-141
  .D        ( SWB_m1 ),
  .Q        ( SWB    )
);

endmodule

// leda B_1001 on
//! enable one_module_per_file
