
//================================================================================================================================
//
// Copyright (c) 2002 by Advanced Micro Devices, Inc.
//
// This file is protected by Federal Copyright Law, with all rights reserved. No part of this file may be reproduced, stored in a
// retrieval system, translated, transcribed, or transmitted, in any form, or by any means manual, electric, electronic,
// mechanical, electro-magnetic, chemical, optical, or otherwise, without prior explicit written permission from Advanced Micro
// Devices, Inc.
//
//================================================================================================================================
//=============================================================================
// This is the standard 1149.1 FSM
// This is used in the master tap as well as the STAC.
//=============================================================================
// leda B_1001 off
//~ format_rtl 1
//~ format_inst narrow

`timescale 1ps/1ps

module gc3_TAPFSM (
  input                                         TRST,
  input                                         TMS,
  input                                         TCK,
                                                                                // IEEE 1149.1 timing : JTAGFLOP timing
                                                                                //-----------------------------------------
  output  wire                                  CAPTURE_DR,                     // Negedge CAP-DR     : Negedge CAP-DR
  output  wire                                  CAPTURE_IR,                     // Negedge CAP-IR     : Negedge CAP-IR
  output  wire                                  SHIFT_DR,                       // Negedge SHF-DR     : Negedge SHF-DR
  output  wire                                  SHIFT_IR,                       // Negedge SHF-IR     : Negedge SHF-IR
  output  wire                                  JTAGFF_UPDATE_DR,               // Posedge UPD-DR-NXT : Negedge UPD-DR-NXT
  output  wire                                  UPDATE_DR,                      // Posegge UPD-DR     : Negedge UPD-DR-NXT
  output  wire                                  UPDATE_IR,                      // Posedge UDP-IR     : Negedge UPD-IR-NXT
  output  wire                                  TEST_LOGIC_RESET,               // Negedge TLR        : Negedge TLR
  output  wire                                  RUN_TEST_IDLE,                  // Negedge RTI        : Negedge RTI
  output  wire                                  RUN_TEST_IDLE_NXT,              // Posedge RTI-NXT    : Negedge RTI-NXT
  output  wire                            early_CAPTURE_DR,
  output  wire                            early_SHIFT_DR,
  output  wire                                  ZERO_BIT_SHIFT_IR              // when transitioning from CAPTURE_IR to EXIT1_IR
);

//=============================================================================
// Test Access Port (TAP) State Machine
//=============================================================================

// Create defines automatically from perlog so we can use it
// in the code below to make it easier to debug and trace the states.
`define gc3_CAPTURE_DR  4'b0001
`define gc3_CAPTURE_IR  4'b1001
`define gc3_EXIT1_DR  4'b0010
`define gc3_EXIT1_IR  4'b1010
`define gc3_EXIT2_DR  4'b0111
`define gc3_EXIT2_IR  4'b1111
`define gc3_PAUSE_DR  4'b0110
`define gc3_PAUSE_IR  4'b1110
`define gc3_RUN_TEST_IDLE  4'b1100
`define gc3_SELECT_DR_SCAN  4'b0000
`define gc3_SELECT_IR_SCAN  4'b1000
`define gc3_SHIFT_DR  4'b0011
`define gc3_SHIFT_IR  4'b1011
`define gc3_TEST_LOGIC_RESET  4'b0100
`define gc3_UPDATE_DR  4'b0101
`define gc3_UPDATE_IR  4'b1101

  reg                                 [3:0]     NxtState;
  wire                                [3:0]     CurState;
  wire                                          ZERO_BIT_SHIFT_IR_detected;

always @* begin
  case (^{TMS,CurState[3:0]})
    1'b0, 1'b1:
    begin
      if (TRST==1'b1)
        NxtState[3:0] = `gc3_TEST_LOGIC_RESET;
      else
        case (CurState[3:0])
          `gc3_TEST_LOGIC_RESET  : NxtState[3:0] = TMS ? `gc3_TEST_LOGIC_RESET : `gc3_RUN_TEST_IDLE ;
          `gc3_RUN_TEST_IDLE     : NxtState[3:0] = TMS ? `gc3_SELECT_DR_SCAN   : `gc3_RUN_TEST_IDLE ;
          `gc3_SELECT_DR_SCAN    : NxtState[3:0] = TMS ? `gc3_SELECT_IR_SCAN   : `gc3_CAPTURE_DR    ;
          `gc3_CAPTURE_DR        : NxtState[3:0] = TMS ? `gc3_EXIT1_DR         : `gc3_SHIFT_DR      ;
          `gc3_SHIFT_DR          : NxtState[3:0] = TMS ? `gc3_EXIT1_DR         : `gc3_SHIFT_DR      ;
          `gc3_EXIT1_DR          : NxtState[3:0] = TMS ? `gc3_UPDATE_DR        : `gc3_PAUSE_DR      ;
          `gc3_PAUSE_DR          : NxtState[3:0] = TMS ? `gc3_EXIT2_DR         : `gc3_PAUSE_DR      ;
          `gc3_EXIT2_DR          : NxtState[3:0] = TMS ? `gc3_UPDATE_DR        : `gc3_SHIFT_DR      ;
          `gc3_UPDATE_DR         : NxtState[3:0] = TMS ? `gc3_SELECT_DR_SCAN   : `gc3_RUN_TEST_IDLE ;
          `gc3_SELECT_IR_SCAN    : NxtState[3:0] = TMS ? `gc3_TEST_LOGIC_RESET : `gc3_CAPTURE_IR    ;
          `gc3_CAPTURE_IR        : NxtState[3:0] = TMS ? `gc3_EXIT1_IR         : `gc3_SHIFT_IR      ;
          `gc3_SHIFT_IR          : NxtState[3:0] = TMS ? `gc3_EXIT1_IR         : `gc3_SHIFT_IR      ;
          `gc3_EXIT1_IR          : NxtState[3:0] = TMS ? `gc3_UPDATE_IR        : `gc3_PAUSE_IR      ;
          `gc3_PAUSE_IR          : NxtState[3:0] = TMS ? `gc3_EXIT2_IR         : `gc3_PAUSE_IR      ;
          `gc3_EXIT2_IR          : NxtState[3:0] = TMS ? `gc3_UPDATE_IR        : `gc3_SHIFT_IR      ;
          `gc3_UPDATE_IR         : NxtState[3:0] = TMS ? `gc3_SELECT_DR_SCAN   : `gc3_RUN_TEST_IDLE ;
          default          : NxtState[3:0] = {4{1'bx}};
      endcase
    end
    default                : NxtState[3:0] = {4{1'bx}};
  endcase
end

DFXIP4_GC10_IP_JTAGFLOP I_CurState[3:0] (
  .ASYNCCLK                ( {4{TRST}}                 ),
  .ASYNCVAL                ( `gc3_TEST_LOGIC_RESET ),
  .CLK                     ( {4{TCK}}                  ),
  .D                       ( NxtState[3:0]             ),
  .Q                       ( CurState[3:0]             )
);

// UBTS 528914 fix:
assign ZERO_BIT_SHIFT_IR_detected = (CurState[3:0] == `gc3_CAPTURE_IR) & (NxtState[3:0] == `gc3_EXIT1_IR);
DFXIP4_GC10_IP_JTAGFLOP I_ZERO_BIT_SHIFT_IR (
  .ASYNCCLK                ( TRST              ),
  .ASYNCVAL                ( 1'b0              ),
  .CLK                     ( TCK               ),
  .D                       ( (CurState[3:0] == `gc3_UPDATE_IR) ? 1'b0 : (ZERO_BIT_SHIFT_IR_detected | ZERO_BIT_SHIFT_IR) ),
  .Q                       ( ZERO_BIT_SHIFT_IR )
);

DFXIP4_GC10_IP_JTAGFLOP I_TLR (
  .ASYNCCLK                ( TRST                      ),
  .ASYNCVAL                ( 1'b1                      ),
  .CLK                     ( ~TCK                      ),
  .D                       ( CurState[3:0] == `gc3_TEST_LOGIC_RESET ),
  .Q                       ( TEST_LOGIC_RESET          )
);

DFXIP4_GC10_IP_JTAGFLOP I_RTI (
  .ASYNCCLK                ( TRST                      ),
  .ASYNCVAL                ( 1'b0                      ),
  .CLK                     ( ~TCK                      ),
  .D                       ( CurState[3:0] == `gc3_RUN_TEST_IDLE ),
  .Q                       ( RUN_TEST_IDLE             )
);

DFXIP4_GC10_IP_JTAGFLOP I_CAP_DR (
  .ASYNCCLK                ( TRST                      ),
  .ASYNCVAL                ( 1'b0                      ),
  .CLK                     ( ~TCK                      ),
  .D                       ( CurState[3:0] == `gc3_CAPTURE_DR ),
  .Q                       ( CAPTURE_DR                )
);

DFXIP4_GC10_IP_JTAGFLOP I_CAP_IR (
  .ASYNCCLK                ( TRST                      ),
  .ASYNCVAL                ( 1'b0                      ),
  .CLK                     ( ~TCK                      ),
  .D                       ( CurState[3:0] == `gc3_CAPTURE_IR ),
  .Q                       ( CAPTURE_IR                )
);

DFXIP4_GC10_IP_JTAGFLOP I_SHF_DR (
  .ASYNCCLK                ( TRST                      ),
  .ASYNCVAL                ( 1'b0                      ),
  .CLK                     ( ~TCK                      ),
  .D                       ( CurState[3:0] == `gc3_SHIFT_DR ),
  .Q                       ( SHIFT_DR                  )
);

DFXIP4_GC10_IP_JTAGFLOP I_SHF_IR (
  .ASYNCCLK                ( TRST                      ),
  .ASYNCVAL                ( 1'b0                      ),
  .CLK                     ( ~TCK                      ),
  .D                       ( CurState[3:0] == `gc3_SHIFT_IR ),
  .Q                       ( SHIFT_IR                  )
);

// These are all driven on posedge TCK because the state machine
// is built out of regular flops
assign JTAGFF_UPDATE_DR   = (NxtState[3:0] == `gc3_UPDATE_DR);   // UBTS 497290
assign RUN_TEST_IDLE_NXT  = (NxtState[3:0] == `gc3_RUN_TEST_IDLE);
assign UPDATE_DR          = (CurState[3:0] == `gc3_UPDATE_DR);
assign UPDATE_IR          = (CurState[3:0] == `gc3_UPDATE_IR);
assign early_CAPTURE_DR   = (CurState[3:0] == `gc3_CAPTURE_DR);
assign early_SHIFT_DR     = (CurState[3:0] == `gc3_SHIFT_DR);

endmodule

// leda B_1001 on
