// spyglass disable_block ALL
//================================================================================================================================
//
// $RCSfile$
//
// $Revision: #108 $
// $Date: 2014/10/22 $
// $Author: pmandrek $
// $State$
// $Locker$
//
// Copyright (c) 2004 by Advanced Micro Devices, Inc.
//
// This file is protected by Federal Copyright Law, with all rights reserved. No part of this file may be reproduced, stored in a
// retrieval system, translated, transcribed, or transmitted, in any  form, or by any means manual, electric, electronic, 
// mechanical, electro-magnetic, chemical, optical, or otherwise, without prior explicit written permission from Advanced Micro
// Devices, Inc.
//
//================================================================================================================================

//==============================================================================
// SEARCH TAGS
// <Module Description>
// <AMXDEFLOP>
// <ASYNCFLOP>     - normal flop with async reset
// <ASYNCMSFLOP>   - flop with async reset (modeled as latches)
// <ASYNCMSFLOPE>  - flop with async reset and latched enable (modeled as latches)
// <ASYNCSRFLOP>   - Async Set/Reset Flop (reset wins) (modeled as flop)
// <ASYNCSRLAT>    - Async Set/Reset Latch
// <ASYNCSRLAT2>   - Async Set/Reset Latch
// <BMSFLOP>       - negedge flop modeled with latches
// <CDCBUF>        - Clock Domain Crossing Buffer
// <CDCBUFEN>      - Clock Domain Crossing Buffer with enable (really inverting)
// <C2PVTT>
// <CLKGATER>
// <D3SSF>
// <DEFLOPNE>
// <DEFLOP>
// <DLLFLOP>
// <DPFLOP>
// <FUSE>          - GH Rev A and Rev B fuse (deprecated)
// <FUSE0>         - simple fuse, init to 0
// <FUSE1>         - simple fuse, init to 1
// <GLR>           - glitch latch with RESET dominant
// <GLRX>          - glitch latch with RESET dominant and state point inverted
// <GLS>           - glitch latch with SET   dominant
// <GLSX>          - glitch latch with SET   dominant and state point inverted
// <JTAGFLOP>
// <JTAGFLOPD3>
// <JTAGLATRET>
// <JTAGSETFLOP>
// <LATCH>
// <SLATCH>
// <LOCALSCANEN>
// <MSF2>
// <MSFLOP>
// <OBLATCH>
// <P2CVDD>
// <RFLOP>
// <RSTFQ>
// <SCANCTL>
// <SCANCTLA>
// <SCFLOP>
// <SCLATCH>
// <SRMSFLOP>
// <SSF12EN>
// <SSF8EN>
// <VOTINGFUSE>
// <VTT2VDDIO>
// <WRCELL>        - NB-Core scan wrapper base cell
// <PRIOLAT2>      - 2 input latch with prioritized clock inputs
// <SYNC2OR>
// <SYNC3OR>
// <ROXFER>        - Ring Oscilator transition controll
// <ICG>           - Coarse gater last stage (B-latch)
// <T9MSFQBNSX1_DFT>  - Non Scan Flop with different test view 
// <LSI>           - Inverting Level Shifters
// <ISO>           - Isolation Cells
//==============================================================================

//==============================================================================
// <Module Description>
// 
// This file is intended to fully describe all the simple modules instantiated
// in the k9 RTL which are intended to be implemented in Standard Cells.  These
// modules are different from the mac_*.v macro models because they will not be
// implemented as black-box macros by the circuit team.  The modules are not
// included inline with normal k9 RTL because they manipulate signals on a
// clock-phase granularity rather than a full clock period.
//
//==============================================================================

`timescale 1ps/1ps

//! disable illegal_includes
`ifdef ABV
`include "amd_proj_global.svh"
`endif
//! enable illegal_includes

//! disable one_module_per_file
// `include "rtl_cpu.d"


// JPC: disable Leda E_122 and E_169 for this entire file, there
//      are lots of violations. I am not expert but they look
//      harmless -- perfectly legal verilog.
// leda E_122 off
// leda E_169 off
// leda E_001 off

// Add the `celldefine compiler directive for improved simulation
// performance. UBTS:393082
`ifdef RTL_STDLIB_CELLDEFINES
`celldefine
`endif


// sater: Leda doesn't recognize our ternary latch style. Turning off 
//        non-blocking combinational block checks and ternary xprop checks. 
//        That shouldn't be applied to our ternary latches.
// 7/9/09 Turning off Leda where vrq-xprop is applied.
//        Leda xprop checks are changing too frequently.
// leda FM_2_16 off
// leda XPROP_TERN3 off
// leda XPROP_TERN1 off
//        Using vrq to fix the ifs
// leda XPROP_IF2 off
//        Enabling vrq xprop to fix flops, default is off to avoid making
//        latches too pessimistic.
// leda C_1000 off
// leda NTL_STR47 off
// leda NTL_STR33 off
// leda VER_3_2_4_2A off
// leda VER_1_1_1_1 off
// leda XPROP_TERN_RHS off
//        Leda dislikes our ternary latch style
// leda W446 off
// leda NTL_STR61 off

// vrq translate_off

// Added granularity for individual modules modeling as dff's
`ifdef MODEL_AS_DFF
   `define MODEL_ASYNCMSFLOP_AS_DFF 1
   `define MODEL_JTAGSETFLOP_AS_DFF 1
   `define MODEL_SYNC4OR_AS_DFF     1
`endif

//==============================================================================
// LATCH - a simple latch with size parameter
//==============================================================================

module LATCH ( Q, D, C);

parameter  size = 1;

output reg [size-1:0] Q;
input      [size-1:0] D;
input                 C;

// leda SYN9_5 off
// leda W410 off

`ifdef NO_CUTPOINTS
//! latch
always @*
  if ( C ) Q[size-1:0] <= D[size-1:0];
`else
//! latch
always @*
  Q[size-1:0] <= C ? D[size-1:0] : Q[size-1:0];
`endif

// leda SYN9_5 on
// leda W410 on

endmodule

//==============================================================================
// SLATCH - Non-Scannable Async Set Latch
// This is intended to map to a non-scannable latch, with asynchronous preset,
// from the stdcell library (e.g. slaqnsx1)
//==============================================================================

module SLATCH ( Q, D, C, S );

parameter  size = 1;

output reg [size-1:0] Q;     // output
input      [size-1:0] D;     // data
input                 C;     // clock
input                 S;     // set

// leda SYN9_5 off
// leda W410 off

//! latch
always @* begin
`ifdef NO_CUTPOINTS
  if      (S) Q[size-1:0] <= {size{1'b1}};
  else if (C) Q[size-1:0] <= D[size-1:0];
`else
  Q[size-1:0] <= S ? {size{1'b1}} : C ? D[size-1:0] : Q[size-1:0];
`endif
end

// leda SYN9_5 on
// leda W410 on

endmodule

//==============================================================================
// RFLOP - synthesizable flop with reset & enable
//==============================================================================
// vrq translate_on
module RFLOP ( Q, D, C, R, E );

parameter  size = 1;
output reg [size-1:0] Q;     // output
input      [size-1:0] D;     // data
input                 C;     // clock
input                 R;     // set
input                 E;     // enable

// using VRQ xprop
// leda off
always @(posedge C) begin
  if (E) begin
    if (R) Q[size-1:0] <= {size{1'b0}};
    else   Q[size-1:0] <= D[size-1:0];
  end
end
// leda on

endmodule

//==============================================================================
// RSTFQ - equivalent to rstfqx
//==============================================================================

module RSTFQ (Q,D,ColdReset,CLK);

output wire Q;     // Reset Output, asserts asynchronously
input  D;
input  ColdReset;  // Asynchronous Reset Input
input  CLK;

reg Qint;

always @(posedge CLK) begin
  Qint <= D;
end

assign Q = (ColdReset | Qint);

endmodule // RSTFQ
// vrq translate_off

//==============================================================================
// CLKGATER - a simple latch and clock gater
//==============================================================================

module CLKGATER ( CG, D, C);

parameter  size = 1;

output wire [size-1:0] CG;   // gated clock (output)
input       [size-1:0] D;    // input to latch
input                  C;    // input clock
reg         [size-1:0] Q;    // internal latch state point

// leda SYN9_5 off
// leda W410 off

// BOZO - this is really just a LATCH, but RPL won't let me change parameters in this file
`ifdef NO_CUTPOINTS
//! latch
always @*
  if ( ~C ) Q[size-1:0] <= D[size-1:0];
`else
//! latch
always @*
  Q[size-1:0] <= ~C ? D[size-1:0] : Q[size-1:0];
`endif
assign CG[size-1:0] = {size{C}} & Q[size-1:0];

// leda SYN9_5 on
// leda W410 on

endmodule

//===============================================================
// CKAN2 - equivalent to ckan2 stdcell
//===============================================================

module CKAN2 (Z,EN,CLK);

output wire Z;
input  EN;
input  CLK;       

assign Z = CLK & EN;

endmodule // CKAN2


//==============================================================================
// ASYNCSRFLOP - Async Set/Reset Flop (reset wins)
//==============================================================================

// Warning - This doesn't match the stdcell in AMD green LEC, and is not X-uglified.
// Please use SRMSFLOP instead.

// vrq translate_on
module ASYNCSRFLOP ( Q, D, C, E, S, R );

parameter  size = 1;

output reg [size-1:0] Q;     // output
input      [size-1:0] D;     // data
input                 C;     // clock
input                 E;     // enable
input                 R;     // reset
input                 S;     // set

// leda off
always @ (posedge C or posedge S or posedge R) begin
  if      (R) Q[size-1:0] <= {size{1'b0}};
  else if (S) Q[size-1:0] <= {size{1'b1}};
  else if (E) Q[size-1:0] <= D[size-1:0];
end
// leda on

endmodule
// vrq translate_off


//==============================================================================
// ASYNCSRLAT - Async Set/Reset Latch
//==============================================================================

// Warning - even though this is written so that it looks like Reset wins, such
// behavior is not reliable in simulation.  If set/reset/clock come from different
// blocks of code, even though they may appear to switch at exactly the same
// simulation time, one of them may be (deterministically or randomly) later than
// the other in arriving here.  (Think non-blocking assign here - it can delay
// a signal from the simulator's viewpoint but not make that visible to the
// waveform).

module ASYNCSRLAT ( Q, D, C, S, R );

parameter  size = 1;

output reg [size-1:0] Q;     // output
input      [size-1:0] D;     // data
input                 C;     // clock
input                 R;     // reset
input                 S;     // set

// leda SYN9_5 off
// leda W410 off

//! latch
always @* begin
`ifdef NO_CUTPOINTS
  if      (R) Q[size-1:0] <= {size{1'b0}};
  else if (S) Q[size-1:0] <= {size{1'b1}};
  else if (C) Q[size-1:0] <= D[size-1:0];
`else
  Q[size-1:0] <= R ? {size{1'b0}} : S ? {size{1'b1}} : C ? D[size-1:0] : Q[size-1:0];
`endif
end

// leda SYN9_5 on
// leda W410 on

endmodule


//==============================================================================
// ASYNCMSFLOP - normal flop with async reset (modeled with latches)
//==============================================================================

module ASYNCMSFLOP ( Q, D, C, E, R );

parameter  size = 1;

output reg [size-1:0] Q;   // output
input      [size-1:0] D;   // data
input      C;              // clock
input      E;              // enable
input      R;              // reset

reg        [size-1:0] QM;  // master latch

// The ASYNCMSFLOP component was built as a proxy for the rstmsfqbnx standard cell library element
// Brian has added an enable to make this look like ASYNCFLOP so that they are interchangeable.

// leda SYN9_5 off
// leda W410 off


`ifdef ONLY_FOR_STATIC_ZEROIN_CDC_CHECK //remove this when 0in CDC recognizes latches
always @(posedge C or posedge R)
  if (R)
    Q[size-1:0] <= {size{1'b0}};
  else if (E)
    Q[size-1:0] <= D[size-1:0];

`elsif MODEL_ASYNCMSFLOP_AS_DFF
wire CLK;
assign CLK = C & E;

always @ (posedge CLK or posedge R) begin
  if (R)
     Q[size-1:0] <= {size{1'b0}};
  else
     Q[size-1:0] <= D[size-1:0];
end

`else // MODEL_ASYNCMSFLOP_AS_DFF
//  the master latch with an asynchronous clear
`ifdef NO_CUTPOINTS
//! latch
always @*
  if      (R)      QM[size-1:0] <= {size{1'b0}};
  else if (~C & E) QM[size-1:0] <= D[size-1:0];
`else
//! latch
always @* QM[size-1:0] <= (R)      ? {size{1'b0}} :
                          (~C & E) ? D[size-1:0] : QM[size-1:0];
`endif

//  the slave latch with an asynchronous clear
`ifdef NO_CUTPOINTS
//! latch
always @*
  if      (R) Q[size-1:0] <= {size{1'b0}};
  else if (C) Q[size-1:0] <= QM[size-1:0];
`else
//! latch
always @* Q[size-1:0] <= (R) ? {size{1'b0}} :
                         (C) ? QM[size-1:0] : Q[size-1:0];
`endif
`endif //ONLY_FOR_STATIC_ZEROIN_CDC_CHECK

// leda SYN9_5 on
// leda W410 on

endmodule


//======================================================================================
// ASYNCMSFLOPE - normal flop with async reset and latched enable (modeled with latches)
//======================================================================================

module ASYNCMSFLOPE ( Q, D, C, E, R );

parameter  size = 1;

output reg [size-1:0] Q;   // output
input      [size-1:0] D;   // data
input      C;              // clock
input      E;              // enable
input      R;              // reset

reg        [size-1:0] QM;  // master latch
reg        EL;             // latched enable
// The ASYNCMSFLOP component was built as a proxy for the rstmsfqbnx standard cell library element
// Brian has added an enable to make this look like ASYNCFLOP so that they are interchangeable.

// ASYNCMSFLOPE was modified to look to LEC like an async clear flop driven by clock with
// latched enable.

// leda SYN9_5 off
// leda W410 off

`ifdef NO_CUTPOINTS
// latching the enable to replicate the external gater function
//! latch
always @*
  if (~C)  EL <= E;
//  the master latch with an asynchronous clear
//! latch
always @*
  if      (R)      QM[size-1:0] <= {size{1'b0}};
  else if (~C | ~EL) QM[size-1:0] <= D[size-1:0];
`else
//! latch
always @* EL <= C ? EL : E;
//! latch
always @* QM[size-1:0] <= (R)      ? {size{1'b0}} :
                          (~C | ~EL) ? D[size-1:0] : QM[size-1:0];
`endif

//  the slave latch with an asynchronous clear
`ifdef NO_CUTPOINTS
//! latch
always @*
  if      (R) Q[size-1:0] <= {size{1'b0}};
  else if (C & EL) Q[size-1:0] <= QM[size-1:0];
`else
//! latch
always @* Q[size-1:0] <= (R) ? {size{1'b0}} :
                         (C & EL) ? QM[size-1:0] : Q[size-1:0];
`endif

// leda SYN9_5 on
// leda W410 on

endmodule


//==============================================================================
// MSF2 - master/slave dual-input flop (looks like a stdcell component)
//==============================================================================

module MSF2 (

input      C0,
input      C1,
input      D0,
input      D1,
output reg Q             // Latch value
);

// WARNING!!!!!
// This module is not intended for direct instantiation at the RTL level.
// It will work just fine at that level, but the RTL writer will then be
// taking on the burden of making sure that he writes stuff that is both
// functionally correct AND plays well with Verplex LEC.  I created this
// module by request of some of the implementation team who wanted to be
// able to "see" the actual stdcell flop broken out in this logic.

// BOZO - there really isn't an ova clock here
// ova clock posedge C0;

wire   M0, M1;

`ifdef MACRO_FAST_SIM
// The RTL simulation should work correctly regardless of
// how this switch is set.  This switch is in here because
// Bill Chandler discovered that the latch version of MSF2
// was 10% slower than the flop version when running the
// NB whacker.
//! disable no_multiple_drivers
always @(posedge C0) Q <= D0;
always @(posedge C1) Q <= D1;
//! enable no_multiple_drivers

`else
// This version looks like the hardware
// master latches for both data
LATCH M0lat ( .Q(M0), .D(D0), .C(~C0) );
LATCH M1lat ( .Q(M1), .D(D1), .C(~C1) );

// The final dual-input slave latch.
// Because I'm using a full case statement here, there is no cutpoint.

// leda off

//! latch
always @*
//  Q <= (C0) ? M0 : (C1) ? M1 : Q;
  case ( {C0, C1} )
    2'b00: ;
    2'b10: Q <= M0;
    2'b01: Q <= M1;
    default: Q <= 1'bx;   
  endcase

`endif

// leda on

endmodule

//==============================================================================
// DEFLOP - dual edge flop - a flop that uses both edges of the clock
//        - Note that long ago (before July 2004), there was another component
//          with this same name.  This is NOT that component but we're using
//          the name again.
//        - This version differs from (the now deprecated) DEFLOPNE in that
//          DEFLOPNE assumed that the FineEnN came from a negedge flop and
//          required a glitch elimination latch before being ANDed with ~CLK
//          while the CrseEnN came from a posedge flop and needed to be delayed
//          to match up with FineEnN.  DEFLOP now allows both enables to come
//          from negedge flops if desired.  Note that the CrseEnN is now used
//          one full cycle earlier than it would be in DEFLOPNE and a ~CLK latch
//          is required outside the DEFLOP to get the old DEFLOPNE behaviour.
//==============================================================================

module DEFLOP (

input       CLK,          // Clock (this is CCLK, NCLK, BCLK, GCLK, MCLK, etc.)
input       DataP,        // Flop this on posedge of clock
input       DataN,        // Flop this on negedge of clock
input       FineEnP,      // sampled during phase B, enables DataA on posedge
input       CrseEnP,      // sampled during phase B, enables DataA on posedge (*_cgN signal)
input       FineEnN,      // sampled during phase A, enables DataB on negedge
input       CrseEnN,      // sampled during phase A, enables DataB on negedge (*_cgN signal)
output wire Q             // Latch value
);

wire   EnLatP;
wire   EnLatN;
wire   CLKP, CLKN;

// delay and stabilize the enables
LATCH EnLatPlat  ( .Q(EnLatP),  .D(CrseEnP & FineEnP), .C(~CLK) );
LATCH EnLatNlat  ( .Q(EnLatN),  .D(CrseEnN & FineEnN), .C( CLK) );

// create gated clocks  
assign CLKP =  CLK & EnLatP;
assign CLKN = ~CLK & EnLatN;

// The following not_half_cycle tag applies to the flop assignments
// in the MSF2 instance below. The DEFLOPNE instances in the RTL have
// been declared by Phil to not have any half-cycle paths from the
// Data inputs to the DEFLOPNE relative to the clock edge on which they
// are enabled. There will be an assertion which backs up this declaration
// once issues in the macros instances of this flop have been addressed.
// Bottom line - this tag causes an assertion to be put in at run time.
//
//! not_half_cycle

// the actual dual ported "flop"
MSF2 Qlat ( .Q(Q), .C0(CLKP), .C1(CLKN), .D0(DataP), .D1(DataN) );

endmodule



//==============================================================================
// DEFLOPNE - dual edge flop - a flop that uses both edges of the clock
//          - this version differs from the original DEFLOP in that the
//            negedge fine enable is assumed to come from a negedge flop
//          - note that the *original* DEFLOP is not the same as the one
//            in this file today.  The *original* DEFLOP was deleted from
//            this file before July 2004.
// Deprecated - new work should seriously consider using DEFLOP instead.
//==============================================================================

module DEFLOPNE (

input       CLK,          // Clock (this is CCLK, NCLK, BCLK, GCLK, MCLK, etc.)
input       DataP,        // Flop this on posedge of clock
input       DataN,        // Flop this on negedge of clock
input       FineEnP,      // sampled during phase B, enables DataA on posedge
input       CrseEnP,      // sampled during phase B, enables DataA on posedge (*_cgN signal)
input       FineEnN,      //                         delayed by phase A, enables DataB on negedge
input       CrseEnN,      // sampled during phase B, delayed by phase A, enables DataB on negedge (*_cgN signal)
output wire Q             // Latch value
);

wire   EnLatN1;

LATCH EnLatN1lat ( .Q(EnLatN1), .D(CrseEnN), .C(~CLK) );

DEFLOP DEF (
            .CLK      (CLK),
            .DataP    (DataP),
            .DataN    (DataN),
            .FineEnP  (FineEnP),
            .CrseEnP  (CrseEnP),
            .FineEnN  (FineEnN),
            .CrseEnN  (EnLatN1),
            .Q        (Q)
            );

endmodule


// DEPRECATED modules are marked for deletion in a future release. Please contact sean.ater@amd.com if you use this define.
`ifdef DEPRECATED_AMXDEFLOP
module AMXDEFLOP (CLK, CG_7, D, AMxData, AMxSel, En_A, En_B, Q);

parameter size = 1;

input                 CLK;      // Raw Clock (this is GCLK or NCLK)
input                 CG_7;     // Coarse gater stage 7
input [size-1:0]      D;        // Input data
input [size-1:0]      AMxData;  // Asynchronous output data
input                 AMxSel;   // Mux select for AMxData
input                 En_A;     // Clock the data through on a rising edge of CLK, comes from posedge flop
input                 En_B;     // Clock the data through on a falling edge of CLK, comes from negedge flop
output reg [size-1:0] Q;        // Flop output value

wire                  EnA_B;          // A clk enable
wire                  EnB_A;          // B clk enable
wire                  ClkA;           // A clk
wire                  ClkB;           // B clk
wire                  ClkAB;          // A or B clk
reg  [size-1:0]       MQ;             // Master A or B phase latch data

//! unused CG_7
// // ova clock posedge CLK;
// // ova forbid_bool( En_A & En_B,"ERROR: AMXDEFLOP has En_A and En_B active on a rising CLK edge");
// // ova forbid_bool( En_A & En_B,"ERROR: AMXDEFLOP has En_A and En_B active on a falling CLK edge", negedge CLK);

LATCH EnABlat (.Q(EnA_B), .D(En_A), .C(~CLK));  // Fine enabled clk gater for A

assign ClkA = EnA_B & CLK;                      // Aphase (posedge) gated clock

LATCH EnBAlat (.Q(EnB_A), .D(En_B), .C(CLK));   // Fine enabled clk gater for B

assign ClkB = EnB_A & ~CLK;                     // Bphase (negedge) gated clock, still active high

assign ClkAB = ClkA | ClkB;

// leda SYN9_5 off
// leda W410 off

// BOZO local Latch due to parameter issue and RPL:
`ifdef NO_CUTPOINTS
//! latch
always @* 
  if (~ClkAB) MQ[size-1:0] <= D[size-1:0];
`else
//! latch
always @* 
  MQ[size-1:0] <= ~ClkAB ? D[size-1:0] : MQ[size-1:0];
`endif

//! latch
always @* begin             // Slave latch with Async Data path
  casez ({ClkAB, AMxSel})
    2'b00:    ;   // Do nothing
    2'b10:   Q[size-1:0]  <= MQ[size-1:0];
    2'b?1:   Q[size-1:0]  <= AMxData[size-1:0];
    default: Q[size-1:0]  <= {size{1'bx}};
  endcase
end

// leda SYN9_5 on
// leda W410 on

endmodule
`endif //DEPRECATED_AMXDEFLOP

// DEPRECATED modules are marked for deletion in a future release. Please contact sean.ater@amd.com if you use this define.
`ifdef DEPRECATED_DLLFLOP
//==============================================================================
// DLLFLOP  -  A pulse (edge detect) flop with set/reset control as well as a
//             level sensitive clear signal. This is a restricted device used only
//             in the DDR2_DLL macro
//==============================================================================


// vrq translate_on
module DLLFLOP ( Q, PS, PR, CLR );

output     Q;
input      PS;  // Set Pulse
input      PR;  // Reset Pulse
input      CLR; // Clear

// leda off
reg Q;
//! disable no_multiple_drivers
// verplex multi_port "Q"
always @ (posedge PS or posedge CLR)
  if (CLR) Q <= 1'b0;
  else     Q <= 1'b1;

always @ (posedge PR or posedge CLR)
  if (CLR) Q <= 1'b0;
  else     Q <= 1'b0;
//! enable no_multiple_drivers
// leda on

endmodule // DLLFLOP
// vrq translate_off
`endif //DEPRECATED_DLLFLOP

//==============================================================================
// DPFLOP - dual port flop - this component is VERY similar to the dual edge flop
//          but it assumes that it needs two clocks and is posedge active on both
// BOZO - looks like this flop is unused (2005-11-07)
//==============================================================================

module DPFLOP (

input       CLK,          // Clock
input       DH,           // Flop this on first  slow clock
input       DH2,          // Flop this on second slow clock
input       CgH,          // Coarse gater for first slow clock
input       CgH2,         // Coarse gater for second slow clock
input       FgH,          // Fine gater
input       FgH2,         // Fine gater
output wire Q             // Latch value
);

wire   FCgH, FCgH2;
wire   CLKH, CLKH2;

// In an ideal world, the follwing clock gater stuff wouldn't be included
// in this module and only the actual dual ported flop would be here.
// Because the original DEFLOP was written long before a gate library
// existed and because Verplex LEC is very picky about things that drive
// latches, we decided to put ALL of the details of the DEFLOP into the
// DEFLOP "component".  Now that we know what the gate library looks like,
// it's clear that we would do things differently if we did them today.
// But that's not how we got here so we're stuck with making DPFLOP look
// like DEFLOP and replicating all of the gater latches.  Sigh.

// delay and stabilize the enables
LATCH FCgHlat  ( .Q(FCgH),  .D(CgH  & FgH),  .C(~CLK) );
LATCH FCgH2lat ( .Q(FCgH2), .D(CgH2 & FgH2), .C(~CLK) );

// create gated clocks  
assign CLKH  = CLK & FCgH;
assign CLKH2 = CLK & FCgH2;

// the actual dual ported "flop"
MSF2 Qlat ( .Q(Q), .C0(CLKH), .C1(CLKH2), .D0(DH), .D1(DH2) );

endmodule


//==============================================================================
// SCLATCH - a scannable latch (control & observe)
//==============================================================================

module SCLATCH ( Q, SDO, SDI, SC1, SC2, D, C );

parameter  size = 1;

output reg [size-1:0] Q;
output reg [size-1:0] SDO;
input      [size-1:0] SDI;
input                 SC1;
input                 SC2;
input      [size-1:0] D;
input                 C;

`ifdef ABV
ovl_never #(.msg("ERROR: SCLATCH has SC1 and C active at the same time.")) rtl_stdlib_sclatch (
    .clock(C), .reset(1'b1), .enable(`ABV_PWROK_EN), .test_expr( SC1 & C ), .fire());
`else
// ova clock posedge C;
// ova forbid_bool (SC1 & C,"ERROR: SCLATCH has SC1 and C active at the same time.");
`endif

`ifdef NO_CUTPOINTS
// Scan stuff & no cutpoints
// verplex multi_port "Q"
//! latch
always @* begin
  if (SC1)
    Q[size-1:0] <= SDI[size-1:0];
  else if (C)
    Q[size-1:0] <= D[size-1:0];

  if (SC2) SDO[size-1:0] <= Q[size-1:0];
end

`else
// with cutpoints
//! latch
always @* begin
  Q[size-1:0]   <= (C | SC1) ? ({size{C}} & D[size-1:0]) | ({size{SC1}} & SDI[size-1:0]) : Q[size-1:0];
  SDO[size-1:0] <=   SC2     ? Q[size-1:0]   : SDO[size-1:0];
end
`endif

endmodule   //  SCLATCH


// DEPRECATED modules are marked for deletion in a future release. Please contact sean.ater@amd.com if you use this define.
`ifdef DEPRECATED_OBLATCH
//==============================================================================
// OBLATCH - a scan observable simple latch
// BOZO - looks like this flop is unused (2005-11-07)
//==============================================================================

// BOZO the scan portion is probably wrong!
module OBLATCH ( Q, SDO, SDI, SC1, SC2, D, C );

parameter size = 1;

output reg [size-1:0] Q;
output reg [size-1:0] SDO;
input      [size-1:0] SDI;
input                 SC1;
input                 SC2;
input      [size-1:0] D;
input                 C;

reg [size-1:0] SD;

`ifdef NO_CUTPOINTS
// Scan stuff & cutpoints
//! latch
always @* begin
  if ( C ) Q[size-1:0]    <= D[size-1:0] ;
  if (SC1) SD[size-1:0]   <= SDI[size-1:0] ;
  if ( C ) SDO[size-1:0]  <= D[size-1:0] ;
  if (SC2) SDO[size-1:0]  <= SD[size-1:0] ;
end
`else
// with cutpoints
//! latch
always @* begin
  Q[size-1:0]    <= C   ? D[size-1:0]    : Q[size-1:0] ;
  SD[size-1:0]   <= SC1 ? SDI[size-1:0]  : SD[size-1:0] ;
  SDO[size-1:0]  <= (C | SC2) ? (D[size-1:0]  & {size{C}}) | ({size{SC2}} & SD[size-1:0] ) : SDO[size-1:0] ;
end
`endif

endmodule
`endif //DEPRECATED_OBLATCH

//==============================================================================
// SCFLOP - scan observe and launch flop
// 
//  *** This component is NOT intended for use as a standard flop. ***
// 
//  This component is useful to force the creation of a state point which can
//    only be modified via the scan chain.  In some physical component libraries,
//    this structure is smaller than the two latches that otherwise would
//    represent such a point.
//  This component is also sometimes used to force the creation of a state point
//    which can only be read via the scan chain but this use is a bit more
//    suspect since such a flop must also have a normal clock.
//==============================================================================

// vrq translate_on
module SCFLOP ( Q, SDO, SDI, SC1, SC2, D, CLK );

output reg            Q;
output wire           SDO;
input                 SDI;
input                 SC1;
input                 SC2;
input                 D;
input                 CLK;

// The obvious way to code this (with two always blocks, one for the latch and one for the flop) does
// not work in LEC because LEC is unable to collapse a DLAT and DFF into one state point.  This leaves
// us with two choices.
// 1) Master/Slave structure 
//    - does not match the gv_fqs component of ckt library
//    - does not behave properly if scan clock is pulsed while slave is open
// 2) Bring scan data in on set/reset ports of flop
//    - this is awkward but I think it will for both LEC and simulation

// leda off
wire SC1SDI0;
wire SC1SDI1;
assign SC1SDI0 = SC1 & ~SDI;
assign SC1SDI1 = SC1 &  SDI;
always @(posedge CLK or posedge SC1SDI0 or posedge SC1SDI1) begin
  if      (SC1SDI0) Q <= 1'b0;
  else if (SC1SDI1) Q <= 1'b1;
  else              Q <= D;
end

LATCH I_SDO ( .Q(SDO), .D(Q), .C(SC2) );
// leda on

endmodule
// vrq translate_off

//==============================================================================
// MSFLOP - a scannable flop (control & observe)
//==============================================================================

module MSFLOP ( Q, CK1, CK2, D );

output reg            Q;
input                 CK1;
input                 CK2;
input                 D;

reg Qint;

// leda SYN9_5 off
// leda W410 off

`ifdef ONLY_FOR_STATIC_ZEROIN_CDC_CHECK //remove this when 0in CDC recognizes latches
always @(posedge CK1) Qint <= D;
always @(posedge CK2) Q <= Qint;
`else
`ifdef NO_CUTPOINTS
// Scan stuff & no cutpoints
//! latch
always @* begin
  if (CK1) Qint <= D;
end
//! latch
always @* begin
  if (CK2) Q <= Qint;
end
`else
// with cutpoints
//! latch
always @* begin
  Qint <= CK1 ? D : Qint;
end
//! latch
always @* begin
  Q <= CK2 ? Qint : Q;
end
`endif
`endif

// leda SYN9_5 on
// leda W410 on

endmodule

// DEPRECATED modules are marked for deletion in a future release. Please contact sean.ater@amd.com if you use this define.
`ifdef DEPRECATED_LOCALSCANEN
//==============================================================================
// LOCALSCANEN - used to control scan launch mode in standard cell rtl.
//==============================================================================

module LOCALSCANEN (CLK, LBistEn, SC1, SC2, SDI, SDO, ScanLBist_A, ScanLBist_B);

output wire           SDO;
output reg            ScanLBist_A;
output wire           ScanLBist_B;
input                 CLK;
input                 LBistEn;
input                 SC1;
input                 SC2;
input                 SDI;

reg [2:0] Q;
wire      QF;

`ifdef ABV
ovl_never #(.msg("ERROR: LOCALSCANEN has SC1 and CLK active at the same time.")) rtl_stdlib_localscanen (
    .clock(C), .reset(1'b1), .enable(`ABV_PWROK_EN), .test_expr( SC1 & CLK ), .fire());
`else
// ova clock posedge CLK;
// ova forbid_bool (SC1 & CLK,"ERROR: LOCALSCANEN has SC1 and CLK active at the same time.");
`endif

`ifdef NO_CUTPOINTS
// Scan stuff & no cutpoints
// verplex multi_port "Q"
// leda FM_1_7 off
always @(posedge CLK) begin
  Q[2:0] <= {Q[1:0],1'b0};
  if (SC1) Q[2:0] <= {3{QF}};
end
// leda FM_1_7 on
`else
// with cutpoints
always @(posedge CLK) begin
  Q[2:0] <= (CLK | SC1) ? ({3{CLK}} & {Q[1:0],1'b0}) | ({3{SC1}} & {3{QF}}) : Q[2:0];
end
`endif

LATCH SCANLBISTB (.Q(ScanLBist_B), .D(ScanLBist_A), .C(~CLK)); 

LATCH SCANMSTR (.Q(QF), .D(SDI), .C(SC1)); 
LATCH SCANSLV  (.Q(SDO), .D(QF), .C(SC2)); 

always @* ScanLBist_A = LBistEn | Q[2];

endmodule
`endif //DEPRECATED_LOCALSCANEN

// ======================================================================================================
//
// Glitch latches -
//
// There are 16 potential glitch latches (set vs. setx, reset vs. resetx, Q vs. QX, R-dominant vs. S-dominant).
// In theory, if you're willing to torture the mappings, one variant is sufficient.
// In practice, it really helps LEC debug if you can map the state node with no inversions.
// It also helps reduce the debug pain if you can make the "reset" input force the state point low and the
// "set" input force the state node high.  (There are decent arguments for why this might be considered
// backwards but I'll stick with debug simplicity.
//
// GLS  - is a Q  latch with set   dominant (set makes Q =1, reset makes Q =0)
// GLR  - is a Q  latch with reset dominant (set makes Q =1, reset makes Q =0)
// GLSX - is a QX latch with set   dominant (set makes QX=1, reset makes QX=0)
// GLRX - is a QX latch with reset dominant (set makes QX=1, reset makes QX=0)
//
// To select the proper version, I suggest that you start by looking to see whether the hardware maps
// the state point as true or complement.  This is one place I'm willing to compromise on inverted state
// nodes and go ahead and create an inverted RTL state node if that maps to the hardware better.  Once
// you've figured out whether the state node is true or false, then check to see what happens to the
// output if both set and reset are asserted at the same time and pick the appropriate latch.
// 
// ======================================================================================================

// GLITCH LATCH with SET dominant
module GLS ( Q, S, R);

parameter  size = 1;

output reg [size-1:0] Q;
input      [size-1:0] S;
input      [size-1:0] R;

// BOZO - this constraint on GLS pretty much must be hand checked.  Ouch.
// WARNING - this is VERY dangerous component!
//
// This component is often used to represent a glitch latch at the output of a RAM.  It is often
// the case that both the set and reset lines to this latch are basically gated clocks that appear
// to be IN THE SAME CLOCK PHASE in a digital simulation environment.  In the analog world, the
// reset signal both arrives and leaves before the set signal.  (That's why this is written as a
// set dominant glitch latch.)  To mimic this behaviour in the digital world, you must insure
// that the reset signal arrives before the set signal.  If you don't, this latch will never clear
// out all the X's.  Since set overrides reset here, it is not critical to make sure that the set
// signal lasts longer than the reset signal (but that is often a side effect of making the reset
// signal arrive first).  One way to get the set/reset to arrive correctly is to make the reset
// signal a direct AND of clock and control (using always @* and a blocking assign) and make the
// set signal be assigned by a non-blocking assign.  It's ugly but it seems to work.

// leda SYN9_5 off
// leda W410 off

`ifdef NO_CUTPOINTS
integer i;
//! glitch_latch
always @* begin                            // See WARNING above!
  for (i=0; i<size; i=i+1) begin           // See WARNING above!
    if      ( S[i] ) Q[i] <= 1'b1;         // See WARNING above!
    else if ( R[i] ) Q[i] <= 1'b0;         // See WARNING above!
  end                                      // See WARNING above!
end                                        // See WARNING above!
`else
//! glitch_latch
always @*                                  // See WARNING above!
  Q[size-1:0] <= S[size-1:0] | ( ~R[size-1:0] & Q[size-1:0] );
`endif

// leda SYN9_5 on
// leda W410 on

endmodule



// GLITCH LATCH with RESET dominant
module GLR ( Q, S, R);

parameter  size = 1;

output reg [size-1:0] Q;
input      [size-1:0] S;
input      [size-1:0] R;

// BOZO - this constraint on GLR pretty much must be hand checked.  Ouch.
// WARNING - this is VERY dangerous component!
//
// See the massive comments in the GLS above.  This is essentially the exact same structure
// except it has reset dominant.  See the large block comment at the start of the glitch latch
// section to help decide which one to use.

// leda SYN9_5 off
// leda W410 off

`ifdef NO_CUTPOINTS
integer i;
//! glitch_latch
always @* begin                            // See WARNING above!
  for (i=0; i<size; i=i+1) begin           // See WARNING above!
    if      ( R[i] ) Q[i] <= 1'b0;         // See WARNING above!
    else if ( S[i] ) Q[i] <= 1'b1;         // See WARNING above!
  end                                      // See WARNING above!
end                                        // See WARNING above!
`else
//! glitch_latch
always @*                                  // See WARNING above!
  Q[size-1:0] <= ~R[size-1:0] & ( S[size-1:0] | Q[size-1:0] );
`endif

// leda SYN9_5 on
// leda W410 on

endmodule



// GLITCH LATCH with SET dominant and state point inverted
module GLSX ( QX, S, R);

parameter  size = 1;

output reg [size-1:0] QX;
input      [size-1:0] S;
input      [size-1:0] R;

// leda SYN9_5 off
// leda W410 off

`ifdef NO_CUTPOINTS
integer i;
//! glitch_latch
always @* begin                            // See WARNING above!
  for (i=0; i<size; i=i+1) begin           // See WARNING above!
    if      ( S[i] ) QX[i] <= 1'b1;        // See WARNING above!
    else if ( R[i] ) QX[i] <= 1'b0;        // See WARNING above!
  end                                      // See WARNING above!
end                                        // See WARNING above!
`else
//! glitch_latch
always @*                                  // See WARNING above!
  QX[size-1:0] <= S[size-1:0] | ( ~R[size-1:0] & QX[size-1:0] );
`endif

// leda SYN9_5 on
// leda W410 on

endmodule



// GLITCH LATCH with RESET dominant
module GLRX ( QX, S, R);

parameter  size = 1;

output reg [size-1:0] QX;
input      [size-1:0] S;
input      [size-1:0] R;

// leda SYN9_5 off
// leda W410 off

`ifdef NO_CUTPOINTS
integer i;
//! glitch_latch
always @* begin                            // See WARNING above!
  for (i=0; i<size; i=i+1) begin           // See WARNING above!
    if      ( R[i] ) QX[i] <= 1'b0;        // See WARNING above!
    else if ( S[i] ) QX[i] <= 1'b1;        // See WARNING above!
  end                                      // See WARNING above!
end                                        // See WARNING above!
`else
//! glitch_latch
always @*                                  // See WARNING above!
  QX[size-1:0] <= ~R[size-1:0] & ( S[size-1:0] | QX[size-1:0] );
`endif

// leda SYN9_5 on
// leda W410 on

endmodule


module JTAGSETFLOP ( CLK, D, SX, Q);
parameter size = 1;

input                 CLK;     // should be connected to JTAG clock - TCK
input      [size-1:0] D;       // data input
input                 SX;      // asynchronous set
output reg [size-1:0] Q;       // data output

`ifdef ONLY_FOR_STATIC_ZEROIN_CDC_CHECK //remove this when 0in CDC recognizes latches
always @(posedge CLK or negedge SX)
  if(~SX) Q[size-1:0] <= {size{1'b1}};
  else Q[size-1:0] <= D[size-1:0];
`elsif HWEMUL // for emulation, stop combo flow through path by creating f->l instead of l->l->l
reg [size-1:0] Qflop;
always @(posedge CLK or negedge SX)
  if ( ~SX ) Qflop[size-1:0] <= {size{1'b1}};
  else Qflop[size-1:0] <= D[size-1:0];

//! latch
always @*
  if (~SX) Q[size-1:0] <= {size{1'b1}};
  else if (~CLK) Q[size-1:0] <= Qflop[size-1:0];
`else

`ifdef MODEL_JTAGSETFLOP_AS_DFF
reg [size-1:0] QF;
reg [size-1:0] QS;
wire S;

assign S = ~SX;
always @ (posedge CLK or posedge S) begin
  if (S)
     QF[size-1:0] <= {size{1'b1}};
  else
     QF[size-1:0] <= D[size-1:0];
end

`else //MODEL_JTAGSETFLOP_AS_DFF

//  The JTAGSETFLOP component was built as a proxy for the jtsfq standard cell library element
reg [size-1:0] QM;
reg [size-1:0] QS;

// leda SYN9_5 off
// leda W410 off

// this represents a flop with an asynchronous set. this is modeled as a master/slave
// latch set

//  this represents the master latch with an asynchronous set
`ifdef NO_CUTPOINTS
//! LATCH
always @*
  if (~SX)
    QM[size-1:0] <= {size{1'b1}};
  else if (~CLK)          // asynchronous set
    QM[size-1:0] <= D[size-1:0];
`else
//! LATCH
always @* QM[size-1:0] <= (~SX) ? {size{1'b1}} : (~CLK) ? D[size-1:0] : QM[size-1:0];
`endif

//  this represents the slave latch with an asynchronous set
`ifdef NO_CUTPOINTS
//! LATCH
always @*
  if (~SX)
    QS[size-1:0] <= {size{1'b1}};
  else if (CLK)          // asynchronous set
    QS[size-1:0] <= QM[size-1:0];
`else
//! LATCH
always @* QS[size-1:0] <= (~SX) ? {size{1'b1}} : (CLK) ? QM[size-1:0] : QS[size-1:0];
`endif
 
`endif //MODEL_JTAGSETFLOP_AS_DFF



//  this represents a latch with an asynchronous set
`ifdef MODEL_JTAGSETFLOP_AS_DFF
//! LATCH
always @*
  if (S)
    Q[size-1:0] <= {size{1'b1}};
  else if (~CLK)          // asynchronous set
    Q[size-1:0] <= QF[size-1:0];
`else //MODEL_JTAGSETFLOP_AS_DFF
`ifdef NO_CUTPOINTS
//! LATCH
always @*
  if (~SX)
    Q[size-1:0] <= {size{1'b1}};
  else if (~CLK)          // asynchronous set
    Q[size-1:0] <= QS[size-1:0];
`else
//! LATCH
always @* Q[size-1:0] <= (~SX) ? {size{1'b1}} : (~CLK) ? QS[size-1:0] : Q[size-1:0];
`endif
`endif //MODEL_JTAGSETFLOP_AS_DFF

// leda SYN9_5 on
// leda W410 on

`endif //ONLY_FOR_STATIC_ZEROIN_CDC_CHECK

endmodule

module JTAGFLOP ( CLK, D, ASYNCCLK, ASYNCVAL, Q);
parameter size = 1;

input                  CLK;     // should be connected to JTAG clock - TCK
input       [size-1:0] D;       // data input
input                  ASYNCCLK;// asynchronous clock
input       [size-1:0] ASYNCVAL;// asynchronous data
output wire [size-1:0] Q;       // data output

wire [size-1:0] jtq; 
wire [size-1:0] jtd;
wire jtsx;

assign jtd[size-1:0]  = ASYNCVAL[size-1:0] ^~ D[size-1:0];
assign jtsx = ~ASYNCCLK;

JTAGSETFLOP #(.size(size)) jtflop (
  .CLK(CLK),
  .D(jtd[size-1:0]),
  .SX(jtsx),
  .Q(jtq[size-1:0])
);

assign Q[size-1:0] = ASYNCVAL[size-1:0] ^~ jtq[size-1:0];

endmodule



// vrq translate_on
module JTAGFLOPD3 (
input       CLK,     // should be connected to JTAG clock - TCK
input       D0,      // data input
input       D1,      // data input
input       Sel,     // data select
output reg  Q        // data output
);

reg Q0, Q1;

// leda off
//! latch
always @* begin
  if (~CLK) begin
    Q0 <= D0;
  end
end

//! latch
always @* begin
  if (CLK) begin
    if (Sel)
      Q1 <= D1;
    else
      Q1 <= Q0;
  end
end

//! latch
always @* begin
  if (~CLK) begin
    Q <= Q1;
  end
end
// leda on

endmodule
// vrq translate_off



// Added 11/14/2011 in response to UBTS:368920 - mreid
module JTAGLATRET ( jtagQ, LatQ, jtagD, jtagC, jtagAV, jtagAC, LatD, LatC );

parameter size = 1;

  output wire [size-1:0]  jtagQ; // JTAG Q
  output wire [size-1:0]  LatQ; // Lat Q

  input [size-1:0] jtagD;
  input            jtagC;
  input [size-1:0] jtagAV;
  input            jtagAC;
  input [size-1:0] LatD;
  input            LatC;


JTAGFLOP #(.size(size)) jflop ( .ASYNCCLK(jtagAC), .ASYNCVAL(jtagAV[size-1:0]), .CLK(jtagC), .D(jtagD[size-1:0]),
.Q(jtagQ[size-1:0]) );
LATCH    #(.size(size)) lat   ( .Q(LatQ[size-1:0]), .D(LatD[size-1:0]), .C(LatC) );

endmodule



// DEPRECATED modules are marked for deletion in a future release. Please contact sean.ater@amd.com if you use this define.
`ifdef DEPRECATED_FUSE
// FUSE - this code models the original Greyhound fuse.
// 
// NOTE!  Use of FUSE is deprecated.  Use FUSE0 or FUSE1 for future work.
// 
//    This model contains ALL of the guts of the fuse and it worked fine for Rev A and B.
// However, in Rev C, the behavior of the fuse was changed (S no longer forced output high)
// and that required either modifying the FUSE module or coming up with a different plan
// for how to model fuses.
//    There are probably LOTS of different ways to build the stuff that surrounds a fuse
// but it's not the place of rtl_stdlib.v to try to represent each of those models.  Instead,
// rtl_stdlib.v should provide the guts of a fuse (the part that needs special initialization)
// and then allow the RTL writer to build the rest of the magic in his/her own logic.  One
// possible argument against this is that the sense and capture circuitry is very unique to
// the fuse and should be modeled as part of the library.  I would counter that by pointing
// out that even in this model, the sense and capture circuit is modeled with normal logic
// and thus can be modeled in the normal RTL.
module FUSE (
output wire Q, // Level-shifted latch output, ORed with Sense
input       ProgEn_L, // Global Program Enable directly from pad
input       PwrOk,    // Power OK signal to control level shifter on data input and output
input       D,  // Data input
input       P,  // Program input
input       S,  // Sense input
input       C   // Capture input
);

reg  Q0,    // Fuse state element
     Q1;    // Latch state element
wire F;     // Fuse - only valid with capture active 
wire Prog, Data;

assign Prog = ~ProgEn_L & P;
assign Data = D & PwrOk;

//synopsys translate_off
// leda off
//! disable parser
initial begin
  Q0 = 1'b0;   // initial value set to zero
  #1;
  Q0 = 1'b0;   // initial value set to zero (again, to work around simulator weirdness)
end
//! enable parser
// leda on
//synopsys translate_on

// leda SYN9_5 off
// leda W410 off

`ifdef NO_CUTPOINTS
//! latch
always @*
  if ( Prog ) Q0 <= (Data) ? 1'b1 : Q0;
`else
//! latch
always @*
  Q0 <= Prog ? (Data | Q0) : Q0;
`endif

assign F = C ? Q0 : 1'b0;

`ifdef NO_CUTPOINTS
//! latch
always @*
  if ( S ) Q1 <= F;
`else
//! latch
always @*
  Q1 <= S ? F : Q1;
`endif

// leda SYN9_5 on
// leda W410 on

assign Q = Q1 | S; // latch output masked during sense

endmodule
`endif //DEPRECATED_FUSE


// FUSE0 - simple fuse model - init to zero
module FUSE0 (
output reg Q,     // Raw Fuse value
input      Set    // Blow the fuse (set it to 1)
);

//synopsys translate_off
// leda off
//! disable parser
initial begin
  Q = 1'b0;   // initial value set to zero
  #1;
  Q = 1'b0;   // initial value set to zero (again, to work around simulator weirdness)
end
//! enable parser
// leda on
//synopsys translate_on

// leda SYN9_5 off
// leda W410 off

`ifdef NO_CUTPOINTS
//! latch
always @*
  if ( Set ) Q <= 1'b1;
`else
//! latch
always @*
  Q <= Set ? 1'b1 : Q;
`endif

// leda SYN9_5 on
// leda W410 on

endmodule


// FUSE1 - simple fuse model - init to one
module FUSE1 (
output reg Q,     // Raw Fuse value
input      Reset  // Blow the fuse (set it to 0)
);

//synopsys translate_off
// leda off
//! disable parser
initial begin
  Q = 1'b1;   // initial value set to one
  #1;
  Q = 1'b1;   // initial value set to one (again, to work around simulator weirdness)
end
//! enable parser
// leda on
//synopsys translate_on

// leda SYN9_5 off
// leda W410 off

`ifdef NO_CUTPOINTS
//! latch
always @*
  if ( Reset ) Q <= 1'b0;
`else
//! latch
always @*
  Q <= Reset ? 1'b0 : Q;
`endif

// leda SYN9_5 on
// leda W410 on

endmodule

// DEPRECATED modules are marked for deletion in a future release. Please contact sean.ater@amd.com if you use this define.
`ifdef DEPRECATED_VOTINGFUSE
module VOTINGFUSE (
output wire Q, // Value output
input       ProgEn_L, // Global Program Enable directly from pad
input       PwrOk,
input       D, // Data input
input       P, // Program input
input       S, // Sense input
input       C  // Capture input
);

wire Q0, Q1, Q2;

FUSE element1 (
  .Q(Q0),
  .ProgEn_L(ProgEn_L),
  .PwrOk(PwrOk),
  .D(D),
  .P(P),
  .S(S),
  .C(C)
);
FUSE element2 (
  .Q(Q1),
  .ProgEn_L(ProgEn_L),
  .PwrOk(PwrOk),
  .D(D),
  .P(P),
  .S(S),
  .C(C)
);
FUSE element3 (
  .Q(Q2),
  .ProgEn_L(ProgEn_L),
  .PwrOk(PwrOk),
  .D(D),
  .P(P),
  .S(S),
  .C(C)
);

assign Q = Q0 & Q1 | Q1 & Q2 | Q0 & Q2;

endmodule
`endif //DEPRECATED_VOTINGFUSE

// DEPRECATED modules are marked for deletion in a future release. Please contact sean.ater@amd.com if you use this define.
`ifdef DEPRECATED_C2PVTT
// vrq translate_on
module C2PVTT (
CoreSig,
Ctrl,
Default,
Sig_L,
Sig_LX
);

parameter size = 1;
input [size-1:0] CoreSig;
input Ctrl;                         // If Ctrl is 1, allow normal function
input [size-1:0] Default;
output [size-1:0] Sig_L;
output [size-1:0] Sig_LX;

wire [size-1:0] Sig_L;
wire [size-1:0] Sig_LX;

// leda off
assign Sig_L[size-1:0]  = Ctrl ? CoreSig[size-1:0] : Default[size-1:0];
assign Sig_LX[size-1:0] = ~Sig_L[size-1:0];
// leda on

endmodule
// vrq translate_off
`endif //DEPRECATED_C2PVTT

// DEPRECATED modules are marked for deletion in a future release. Please contact sean.ater@amd.com if you use this define.
`ifdef DEPRECATED_P2CVDD
module P2CVDD (
PadSig_L,
Ctrl,
Sig
);

parameter size = 1;
input [size-1:0] PadSig_L;
input Ctrl;
output [size-1:0] Sig;
wire [size-1:0] Sig;

assign Sig[size-1:0]  = Ctrl ? PadSig_L[size-1:0]  : {size{1'b0}};

endmodule
`endif //DEPRECATED_P2CVDD

// DEPRECATED modules are marked for deletion in a future release. Please contact sean.ater@amd.com if you use this define.
`ifdef DEPRECATED_VTT2VDDIO
module VTT2VDDIO (
VttSig_L,
Sig_H
);

parameter size = 1;
input [size-1:0] VttSig_L;
output [size-1:0] Sig_H;
wire [size-1:0] Sig_H;

assign Sig_H[size-1:0]  =  VttSig_L[size-1:0];

endmodule
`endif //DEPRECATED_VTT2VDDIO

// DEPRECATED modules are marked for deletion in a future release. Please contact sean.ater@amd.com if you use this define.
`ifdef DEPRECATED_D3SSF
// vrq translate_on
module D3SSF (SSClk,RdPtrClk,RdPtrInit,RdPtrInitVal,WrPtrInitVal,DataIn,DataOut);
parameter size = 1;
input               SSClk;          // Source Synchronous Clock
input               RdPtrClk;       // Read Pointer Clock
input  [2:0]        RdPtrInit;      // Read Pointer Init - value to initialize the read pointer to (programmable via fuses?)
input               RdPtrInitVal;   // Read Pointer Init Valid - initialization signal
input               WrPtrInitVal;
input  [size-1:0]   DataIn;         // Input Data synchronous to SSCLK
output [size-1:0]   DataOut;        // Output Data synchronous to RdPtrClk

wire                SSClk;
wire                RdPtrClk;
wire   [2:0]        RdPtrInit;
wire                RdPtrInitVal;
wire   [size-1:0]   DataIn;
wire   [size-1:0]   DataOut;
reg    [size-1:0]   SSFifo[7:0];     // the FIFO
reg    [2:0]        WrPtr;           // Write Pointer to the FIFO
reg    [2:0]        RdPtr;           // Read Pointer to the FIFO

// leda off
assign DataOut[size-1:0] = SSFifo[ RdPtr[2:0] ][size-1:0];

// DataIn[size] is the initialization signal.  I expect it to be high during Sys Reset, once Sys Reset deasserts
// then it should go low and stay low until next Sys Reset.
always @(posedge SSClk)
begin
   if (WrPtrInitVal)
      WrPtr[2:0] <= 3'b000;
   else
      WrPtr[2:0] <= WrPtr[2:0] + 3'b001;
   SSFifo[ WrPtr[2:0] ][size-1:0] <= DataIn[size-1:0];
end

always @(posedge RdPtrClk or posedge RdPtrInitVal)
   if (RdPtrInitVal)
      RdPtr[2:0] <= RdPtrInit[2:0];
   else
      RdPtr[2:0] <= RdPtr[2:0] + 3'b001;
// leda on

endmodule
// vrq translate_off
`endif //DEPRECATED_D3SSF


// DEPRECATED modules are marked for deletion in a future release. Please contact sean.ater@amd.com if you use this define.
`ifdef DEPRECATED_SSF16EN
// vrq translate_on
module SSF16EN( // 16 deep.
input               SSClk,          // Source Synchronous Clock
input               RdPtrClk,       // Read Pointer Clock
input       [3:0]   RdPtrInit,      // Read Pointer Init - value to initialize the read pointer to (programmable via fuses?)
input               RdPtrInitVal,   // Read Pointer Init Valid - initialization signal
input               RdPtrEn,        // Read Pointer enable.
input       [4:0]   DataIn,         // Input Data synchronous to SSCLK
input               WrValid,        // Signals valid data in FIFO.

output wire [2:0]   WrPtrState,     // Output low 3 bits of wrptr
output wire [3:0]   DataOut         // Output Data synchronous to RdPtrClk
);

reg    [3:0]        SSFifo[15:0];    // the FIFO
wire   [3:0]        WrPtr;           // Write Pointer to the FIFO
reg    [3:0]        WrPtrNxt;        // Write Pointer+1 to the FIFO
reg    [3:0]        RdPtr;           // Read Pointer to the FIFO

wire                Reset;

// leda off
assign DataOut[3:0] = SSFifo[RdPtr[3:0]][3:0];
assign WrPtrState[2:0] = WrPtr[2:0];

assign Reset = DataIn[4];

//! unused WrValid

// DataIn[4] is the initialization signal.  I expect it to be high during Sys Reset, once Sys Reset deasserts
// then it should go low and stay low until next Sys Reset.

always @* WrPtrNxt[3:0] = WrPtr[3:0] + 4'b0001;

ASYNCMSFLOP I_WrPtr[3:0] ( .D(WrPtrNxt[3:0]), 
                           .Q(WrPtr[3:0]),
                           .C(SSClk),
                           .R(Reset),
                           .E(4'b1111));


always @(posedge SSClk) begin
  SSFifo[WrPtr[3:0]][3:0] <= DataIn[3:0];
end

always @(posedge RdPtrClk)
  if (RdPtrInitVal)
    RdPtr[3:0] <= RdPtrInit[3:0];
  else if(RdPtrEn)
    RdPtr[3:0] <= RdPtr[3:0] + 4'b0001;

//synopsys translate_off
//! disable parser 
//  This will fire when the fifo has more than 13 entries.
//  Tahsin wants it to be this way, even though we are using 16 deep fifo
//  Tahsin 3/17/06: we have problem with 12 at low NCLK:Memclk ratio, high
//                  difference of DqsRcvEn delay and the fact that there
//                  might be a delay between 1st data beat read and 2nd data beat read
//  Rob 10/1/06: we have a problem with 13 at low NCLK:Memclk ratios when
//               we are in a NB low-power P-state because the DCT adds 2 NCLKs
//               to the divided down MaxRdLatency instead of just 1 NCLK (or none
//               in the case of an even # of MaxRdLatency) See Bugs#15314, #15862
// Rdptrclk is used to reset the fifo wrpte as the fifo is change to Async reset fifo for 
// implementation bug.
// ova bind ova_dual_clk_fifo dual_clk_fifo
//// (depth=14,elem size=4,,,,oflow chk=1,uflow chk=1,val chk=0,enqedge=0(pos),deqedge=0(pos),msg,severity,category)
//  #(14,4,,,,1,1,0,0,0,"DCT RCVFIFO PROBLEM",`NOASSERT_IN_SYSRESET,,)
//  (Reset|RdPtrInitVal, (~Reset &SSClk) |(Reset&RdPtrClk),WrValid,DataIn[3:0],RdPtrClk,RdPtrEn,DataOut);
//! enable parser
// leda on
//synopsys translate_on
endmodule
// vrq translate_off
`endif //DEPRECATED_SSF16EN


// SCANCTL - Generates three internal LCLK pulses when set up by scan and triggered by MCLKGater
// 45nm projects will need to define DEPRECATED_45NM if they upgrade to new rtl_stdlib versions.
`ifdef DEPRECATED_45NM
module SCANCTL
(
input           LBistEn,            // Logic BIST enable
input           MCLKGater_cg10,     // Clock gater, stage 10
input           MCLK,               // Free-running main clock
input           SDI,                // Scan data-in
input           SC1,                // Scan master clock
input           SC2,                // Scan slave clock (observe)
output  wire    SDO,                // Scan data-out
output  reg     LM_A,               // Launch mode enable, sets up on A edge
output  wire    LM_B,               // Launch mode enable, sets up on B edge
output  reg     EnableB_B           // Fine enable to all B coarse gaters, sets up on B edge
);


reg     [1:0]   cnt;                // 2-bit finite state machine (scannable flip-flop)
reg     [1:0]   cnt_next;           // Next state for cnt[1 : 0]
reg             cntR1;              // Asynchronous reset for cnt[1]
reg             cntR0;              // Asynchronous reset for cnt[0]
reg             cntS1;              // Asynchronous set for cnt[1]
reg             cntS0;              // Asynchronous set for cnt[0]
wire            LCLK;               // gated version of MCLK
wire            MCLKGater_cg10_B;
wire            DisableB;

// BOZO - bug 6517
// BOZO - these asserts (and the corresponding LEC constraints) should probably be considered
//        as global constraints and moved to some global constraint file for simulation and just
//        coded in the LEC constraints file directly.  Seems inefficient to test them in every
//        instance of a shared like this.
// ova forbid_bool (SC1 & MCLK, "ERROR: SCAN_CTRL has SC1 and MCLK active at the same time.", posedge MCLK );
// ova forbid_bool (SC1 & MCLK, "ERROR: SCAN_CTRL has SC1 and MCLK active at the same time.", posedge SC1 );
// ova forbid_bool (SC2 & MCLK, "ERROR: SCAN_CTRL has SC2 and MCLK active at the same time.", posedge MCLK );
// ova forbid_bool (SC2 & MCLK, "ERROR: SCAN_CTRL has SC2 and MCLK active at the same time.", posedge SC2 );

`ifdef VERPLEX_LEC
$constraint( ~(SC1 & MCLK) );
$constraint( ~(SC2 & MCLK) );
`endif

//----------------------------------------------------------------------------
// Generate the local clocks using the coarse gater
// Also bring in an internal fine enable (LM_A) to reduce power consumption
// when this block is "done".
//----------------------------------------------------------------------------
LATCH I_MCLKGater_cg10_B ( .Q(MCLKGater_cg10_B), .D (MCLKGater_cg10 & LM_A), .C (~MCLK) );
assign LCLK = MCLKGater_cg10_B & MCLK;

//----------------------------------------------------------------------------
// Resets and sets for the 2-bit finite state machine. These are used to implement
// the scan function of the flip-flops.
//----------------------------------------------------------------------------
always @* cntR1 = SC1 & ~LCLK & ~SDI;
always @* cntR0 = SC2 & ~LCLK & ~cnt[1];
always @* cntS1 = SC1 & ~LCLK & SDI;
always @* cntS0 = SC2 & ~LCLK & cnt[1];

//----------------------------------------------------------------------------
// 2-bit decrementer. Its state transitions are 3->2->1->0 when clocked by LCLK. It
// is also scannable using scan clocks SC1 and SC2 with SDI as the scan data input.
//----------------------------------------------------------------------------

// leda B_1205 off

always @* cnt_next[1 : 0] = (cnt[1 : 0] == 2'd0) ? 2'd0: (cnt[1 : 0] - 2'd1);

always @(posedge LCLK or posedge cntR1 or posedge cntS1) begin
  if (cntR1) cnt[1] <= 1'b0;
  else if (cntS1) cnt[1] <= 1'b1;
  else cnt[1] <= cnt_next[1];
end

always @(posedge LCLK or posedge cntR0 or posedge cntS0) begin
  if (cntR0) cnt[0] <= 1'b0;
  else if (cntS0) cnt[0] <= 1'b1;
  else cnt[0] <= cnt_next[0];
end

// leda B_1205 on

//----------------------------------------------------------------------------
// Instantiate the DisableB scan bit used to enable/disable B gaters
//----------------------------------------------------------------------------
LATCH I_DisableB ( .Q(DisableB), .D (cnt[0]), .C (SC1) );
LATCH I_SDO ( .Q(SDO), .D (DisableB), .C (SC2) );

//----------------------------------------------------------------------------
// These are all the outputs.
//----------------------------------------------------------------------------
always @* begin
  LM_A = LBistEn | cnt[1] | cnt[0];             // OR gate
  EnableB_B = ~(DisableB & LM_B);               // NAND gate
end

// clock-off latch
LATCH I_LM_B ( .Q(LM_B), .D(LM_A), .C(~LCLK) );

// synopsys translate_off 
// leda off
//! disable parser              // this is sometimes needed for RPL
`ifndef VERPLEX_LEC             // LEC barfs at the $activeinst syntax
integer   RandSeed;
reg [1:0] temp_cnt;
reg       temp_Gater;
reg       temp_Disable;
reg       temp_SDO;
reg       temp_LM_B;
initial begin
  `ifdef USE_FIXED_INITIAL_VALUES // should only be required in macros, this
                                //   hides PLI calls in stand-alone compile
  // I haven't put much thought into how this should be done in the gate environment.
  // For now, I'll just slam everything to zero using exactly the same structure
  // that I use for the RTL.
  force cnt[1:0]             = 2'b00;
  force I_MCLKGater_cg10_B.Q = 1'b0;
  force I_DisableB.Q         = 1'b0;
  force I_SDO.Q              = 1'b0;
  force I_LM_B.Q             = 1'b0;
  wait (MCLKGater_cg10 !== 1'bx);
  wait (MCLK === 1'b0);
  wait (MCLK === 1'b1);
  force LCLK = 1'b1;   // If LM_A and LCLK both power up to zero, they will
                       // never change.  With no changes on it's input,
                       // the latch on LM_B will not update even though it
                       // is open.  By forcing a pulse on LCLK while holding
                       // everything else, I can trigger the required update.
  wait (MCLK === 1'b0);
  release cnt[1:0];
  release I_MCLKGater_cg10_B.Q;
  release I_DisableB.Q;
  release I_SDO.Q;
  release I_LM_B.Q;
  release LCLK;        // put the clock edge last
  `elsif C_CALLS_NOT_SUPPORTED
    // There are places where this switch is used and USE_FIXED_INITIAL_VALUES is NOT used (for example, LEC)
  force cnt[1:0]             = 2'b00;
  force I_MCLKGater_cg10_B.Q = 1'b0;
  force I_DisableB.Q         = 1'b0;
  force I_SDO.Q              = 1'b0;
  force I_LM_B.Q             = 1'b0;
  wait (MCLKGater_cg10 !== 1'bx);
  wait (MCLK === 1'b0);
  wait (MCLK === 1'b1);
  force LCLK = 1'b1;
  wait (MCLK === 1'b0);
  release cnt[1:0];
  release I_MCLKGater_cg10_B.Q;
  release I_DisableB.Q;
  release I_SDO.Q;
  release I_LM_B.Q;
  release LCLK;
  `else
  RandSeed = get_random_id($activeinst);     // unique id for this inst
                                             //    (done only once per module)
  // For reasons I don't understand, force foo = random32(...) causes VCS to call random32 before get_random_id.
  // The following temp variables appears to get around this apparent bug.
  temp_cnt[1:0] = random32(RandSeed);
  temp_Gater    = random32(RandSeed);
  temp_Disable  = random32(RandSeed);
  temp_SDO      = random32(RandSeed);
  temp_LM_B     = random32(RandSeed);
  force cnt[1:0]             = temp_cnt[1:0];
  force I_MCLKGater_cg10_B.Q = temp_Gater;
  force I_DisableB.Q         = temp_Disable;
  force I_SDO.Q              = temp_SDO;
  force I_LM_B.Q             = temp_LM_B;
  wait (MCLKGater_cg10 !== 1'bx);
  wait (MCLK === 1'b0);
  wait (MCLK === 1'b1);
  force LCLK = 1'b1;   // If LM_A and LCLK both power up to zero, they will
                       // never change.  With no changes on it's input,
                       // the latch on LM_B will not update even though it
                       // is open.  By forcing a pulse on LCLK while holding
                       // everything else, I can trigger the required update.
  wait (MCLK === 1'b0);
  release cnt[1:0];
  release I_MCLKGater_cg10_B.Q;
  release I_DisableB.Q;
  release I_SDO.Q;
  release I_LM_B.Q;
  release LCLK;        // put the clock edge last
  `endif
end
`endif
//! enable parser
// leda on
// synopsys translate_on

endmodule
`endif // ifdef DEPRECATED_45NM

//==============================================================================
// SCANCTLA - Simplified scan_ctrl, uses ColdReset to clear the scannable latch
//   that controls LM. LM will be a drop-in replacement for either LM_A or LM_B.
//   The EnableB output will also be effectively static, so it will replace the
//   EnableB_B output of the old scan_ctrl block. 
//==============================================================================

module SCANCTLA ( EnableB, LM, SDO, ColdReset, LBistEn, SC1, SC2, SDI );

output  wire    EnableB;
output  wire    LM;
output  wire    SDO;
input           ColdReset;
input           LBistEn;
input           SC1;
input           SC2;
input           SDI;

reg LM_En;
reg ScanSlave;
wire DisableB;


`ifdef ABV
//! disable_parser
ovl_never #(.msg("ERROR: SCLATCH has SC1 and C active at the same time.")) rtl_stdlib_scanctla0 (
    .clock(ColdReset), .reset(1'b1), .enable(`ABV_ALWAYS_EN), .test_expr( (SC1 & ColdReset) === 1'b1 ), .fire());
ovl_never #(.msg("ERROR: SCLATCH has SC1 and C active at the same time.")) rtl_stdlib_scanctla1 (
    .clock(SC1), .reset(1'b1), .enable(`ABV_ALWAYS_EN), .test_expr( (SC1 & ColdReset) === 1'b1 ), .fire());
ovl_never #(.msg("ERROR: SCLATCH has SC1 and C active at the same time.")) rtl_stdlib_scanctla2 (
    .clock(ColdReset), .reset(1'b1), .enable(`ABV_PWROK_EN), .test_expr( (SC1 & ColdReset) === 1'bx ), .fire());
ovl_never #(.msg("ERROR: SCLATCH has SC1 and C active at the same time.")) rtl_stdlib_scanctla3 (
    .clock(SC1), .reset(1'b1), .enable(`ABV_PWROK_EN), .test_expr( (SC1 & ColdReset) === 1'bx ), .fire());
//! enable_parser
`else
// ova forbid_bool ((SC1 & ColdReset) === 1'b1, "ERROR: SCANCTRLA has SC1 and ColdReset active at the same time.", posedge ColdReset );
// ova forbid_bool ((SC1 & ColdReset) === 1'b1, "ERROR: SCANCTRLA has SC1 and ColdReset active at the same time.", posedge SC1 );
// using ifdef here to avoid including a define file in rtl_stdlib, which could be disruptive to cad flows.
`ifdef NOASSERT_IN_PWROK_DEASSERT
// ova forbid_bool ((SC1 & ColdReset) === 1'bx, "ERROR: SCANCTRLA has SC1 and ColdReset active at the same time.(X)", posedge ColdReset, `NOASSERT_IN_PWROK_DEASSERT );
// ova forbid_bool ((SC1 & ColdReset) === 1'bx, "ERROR: SCANCTRLA has SC1 and ColdReset active at the same time.(X)", posedge SC1, `NOASSERT_IN_PWROK_DEASSERT );
`endif
`endif //ifdef ABV

// leda off

`ifdef NO_CUTPOINTS
// Scan stuff & no cutpoints
// verplex multi_port "LM_En"
//! latch
always @* begin
  if (SC1)
    LM_En <= ~SDI;
  else if ( ColdReset )
    LM_En <= 1'b0;

  if (SC2) ScanSlave <= ~LM_En;
end
`else
// with cutpoints
//! latch
always @* begin
  LM_En <= (ColdReset | SC1) ? (SC1 & ~SDI) : LM_En;
  ScanSlave <= SC2 ? ~LM_En : ScanSlave;
end
`endif

// leda on

LATCH IDisableB ( .Q(DisableB), .D(ScanSlave), .C(SC1) ); 
LATCH IDB ( .Q(SDO), .D(DisableB), .C(SC2) ); 

assign EnableB = ~(LM & DisableB);
assign LM = LM_En | LBistEn;

endmodule


//==============================================================================
// ADDRDECLAT2 - crack, latch and re-combine a 2-bit address
//   This module is used in macros when an address or index is decoded and then
//     the decoded version is latched.  For LEC, we must model the cracked address
//     but for logic flow, we're not really interested in the details.
//   This component also puts the $constraint in a place where we can't mess it up.
//==============================================================================

module ADDRDECLAT2
(
output wire [1:0]   Q,
input       [1:0]   D,
input               C
);

wire [3:0] Dec;

LATCH #(.size(4)) I_Dec ( .Q(Dec[3:0]), .D( 4'b0001 << D[1:0] ), .C(C) );
`ifdef VERPLEX_LEC
$constraint( $one_hot(Dec[3:0]) );
`endif
assign Q[1:0] = Dec[3:0] == 4'b0001 ? 2'b00 :
                Dec[3:0] == 4'b0010 ? 2'b01 :
                Dec[3:0] == 4'b0100 ? 2'b10 :
                Dec[3:0] == 4'b1000 ? 2'b11 : 2'bxx;

endmodule

//==============================================================================
// ADDRDECLAT3 - crack, latch and re-combine a 3-bit address
//   This module is used in macros when an address or index is decoded and then
//     the decoded version is latched.  For LEC, we must model the cracked address
//     but for logic flow, we're not really interested in the details.
//   This component also puts the $constraint in a place where we can't mess it up.
//==============================================================================

module ADDRDECLAT3
(
output wire [2:0]   Q,
input       [2:0]   D,
input               C
);

wire [7:0] Dec;

LATCH #(.size(8)) I_Dec ( .Q(Dec[7:0]), .D( 8'b00000001 << D[2:0] ), .C(C) );
`ifdef VERPLEX_LEC
$constraint( $one_hot(Dec[7:0]) );
`endif
assign Q[2:0] = Dec[7:0] == 8'b00000001 ? 3'b000 :
                Dec[7:0] == 8'b00000010 ? 3'b001 :
                Dec[7:0] == 8'b00000100 ? 3'b010 :
                Dec[7:0] == 8'b00001000 ? 3'b011 :
                Dec[7:0] == 8'b00010000 ? 3'b100 :
                Dec[7:0] == 8'b00100000 ? 3'b101 :
                Dec[7:0] == 8'b01000000 ? 3'b110 :
                Dec[7:0] == 8'b10000000 ? 3'b111 : 3'bxxx;

endmodule

//==============================================================================
// ADDRDECLAT4 - crack, latch and re-combine a 4-bit address
//   This module is used in macros when an address or index is decoded and then
//     the decoded version is latched.  For LEC, we must model the cracked address
//     but for logic flow, we're not really interested in the details.
//   This component also puts the $constraint in a place where we can't mess it up.
//==============================================================================

module ADDRDECLAT4
(
output wire [3:0]   Q,
input       [3:0]   D,
input               C
);

wire [15:0] Dec;

LATCH #(.size(16)) I_Dec ( .Q(Dec[15:0]), .D( 16'h0001 << D[3:0] ), .C(C) );
`ifdef VERPLEX_LEC
$constraint( $one_hot(Dec[15:0]) );
`endif
assign Q[3:0] = Dec[15:0] == 16'h0001 ? 4'h0 :
                Dec[15:0] == 16'h0002 ? 4'h1 :
                Dec[15:0] == 16'h0004 ? 4'h2 :
                Dec[15:0] == 16'h0008 ? 4'h3 :
                Dec[15:0] == 16'h0010 ? 4'h4 :
                Dec[15:0] == 16'h0020 ? 4'h5 :
                Dec[15:0] == 16'h0040 ? 4'h6 :
                Dec[15:0] == 16'h0080 ? 4'h7 :
                Dec[15:0] == 16'h0100 ? 4'h8 :
                Dec[15:0] == 16'h0200 ? 4'h9 :
                Dec[15:0] == 16'h0400 ? 4'ha :
                Dec[15:0] == 16'h0800 ? 4'hb :
                Dec[15:0] == 16'h1000 ? 4'hc :
                Dec[15:0] == 16'h2000 ? 4'hd :
                Dec[15:0] == 16'h4000 ? 4'he :
                Dec[15:0] == 16'h8000 ? 4'hf : 4'bxxxx;

endmodule

//==============================================================================
// ADDRDECFLOP2 - crack, flop and re-combine a 2-bit address
//   This module is used in macros when an address or index is decoded and then
//     the decoded version is stored.  For LEC, we must model the cracked address
//     but for logic flow, we're not really interested in the details.
//   This component also puts the $constraint in a place where we can't mess it up.
//==============================================================================

module ADDRDECFLOP2
(
output wire [1:0]   Q,
input       [1:0]   D,
input               C
);

reg [3:0] Dec;

always @(posedge C) begin
Dec[3:0] <=  (4'b0001 << D[1:0] );
end

`ifdef VERPLEX_LEC
$constraint( $one_hot(Dec[3:0]) );
`endif
assign Q[1:0] = Dec[3:0] == 4'b0001 ? 2'b00 :
                Dec[3:0] == 4'b0010 ? 2'b01 :
                Dec[3:0] == 4'b0100 ? 2'b10 :
                Dec[3:0] == 4'b1000 ? 2'b11 : 2'bxx;

endmodule

//==============================================================================
// ASYNCFLOP - normal flop with async reset
//==============================================================================

// vrq translate_on
module ASYNCFLOP ( Q, D, C, E, R );

parameter  size = 1;

output reg [size-1:0] Q;   // output
input      [size-1:0] D;   // data
input      C;              // clock
input      E;              // enable
input      R;              // reset

// leda off
always @(posedge C or posedge R)
  if (R)
    Q[size-1:0] <= {size{1'b0}};
  else if (E)
    Q[size-1:0] <= D[size-1:0];
// leda on

endmodule
// vrq translate_off

//==============================================================================
// ASYNCSRLAT2 - Async Set/Reset Latch
//==============================================================================

// Warning - even though this is written so that it looks like set wins, such
// behavior is not reliable in simulation.  If set/reset/clock come from different
// blocks of code, even though they may appear to switch at exactly the same
// simulation time, one of them may be (deterministically or randomly) later than
// the other in arriving here.  (Think non-blocking assign here - it can delay
// a signal from the simulator's viewpoint but not make that visible to the
// waveform).

module ASYNCSRLAT2 ( Q, D, C, S, R );

parameter  size = 1;

output reg [size-1:0] Q;     // output
input      [size-1:0] D;     // data
input                 C;     // clock
input                 R;     // reset
input                 S;     // set

// leda SYN9_5 off
// leda W410 off

//! latch
always @* begin
`ifdef NO_CUTPOINTS
  if      (S) Q[size-1:0] <= {size{1'b1}};
  else if (R) Q[size-1:0] <= {size{1'b0}};
  else if (C) Q[size-1:0] <= D[size-1:0];
`else
  Q[size-1:0] <= S ? {size{1'b1}} : R ? {size{1'b0}} : C ? D[size-1:0] : Q[size-1:0];
`endif
end

// leda SYN9_5 on
// leda W410 on

endmodule

//==============================================================================
// SRMSFLOP - flop with async set and reset (modeled with latches)
//==============================================================================

module SRMSFLOP ( Q, D, C, S, R );

parameter  size = 1;

output  wire   [size-1:0] Q;   // output
input      [size-1:0] D;   // data
input      C;              // clock
input      S;              // enable
input      R;              // reset

wire        [size-1:0] QM;  // master latch

// The SRMSFLOP component was built as a proxy for the srmsfqbnsx2 standard cell library element
  ASYNCSRLAT2 #(.size(size)) QMlat (.Q(QM[size-1:0]), .D(D[size-1:0]), .C(~C), .S(S), .R(R));
//  the slave latch with an asynchronous set and reset
//  priority order(highest first):  set, reset, data

  ASYNCSRLAT2 #(.size(size)) Qlat (.Q(Q[size-1:0]), .D(QM[size-1:0]), .C(C), .S(S), .R(R));

endmodule

//==============================================================================
// BMSFLOP - negedge flop modeled with latches -matches bmsfqb in Griffin lib
//==============================================================================

module BMSFLOP ( Q, D, C);

parameter  size = 1;

output  wire   [size-1:0] Q;   // output
input      [size-1:0] D;   // data
input      C;              // clock

wire        [size-1:0] QM;  // master latch


//  the master latch  
  LATCH #(.size(size)) QMlat ( .Q(QM[size-1:0]), .D(D[size-1:0]), .C(C) );

//  the slave latch
 LATCH #(.size(size)) Qlat ( .Q(Q[size-1:0]), .D(QM[size-1:0]), .C(~C) );

endmodule

//==============================================================================
// <WRCELL>        - NB-Core scan wrapper base cell        
// This module is the base cell for the NB-Core scan wrapper.
//==============================================================================

module WRCELL (
input         CCLK12,     // Core clock on gater 12
input         CpuToNbIn,  // Data traveling from the CPU to the NB
output  wire  CpuToNbOut, // Data traveling from the CPU to the NB
input         NbToCpuIn,  // Data traveling from the NB to the CPU
output  wire  NbToCpuOut, // Data traveling from the NB to the CPU
input         WrpEn,      // Enable signal for the test wrapper.
input         IOFace,     // Internal (1) or External (0) test mode.
output  wire  SDO,        // Scan data output
input         SDI,        // Sac data input
input         SC1,        // Scan clock 1
input         SC2         // Scan clock 2
);

// leda DCVER_154 off
// leda W287 off

assign SDO = 1'b0;
//! unused SDI

wire  FlopAIn;
wire  FlopAOut;
wire  FlopBOut;

SCFLOP FlopA( .Q (FlopAOut), .D (FlopAIn),  .SDO (), .SDI (1'b0), .SC1 (SC1), .SC2 (SC2), .CLK (CCLK12) );
SCFLOP FlopB( .Q (FlopBOut), .D (FlopAOut), .SDO (), .SDI (1'b0), .SC1 (SC1), .SC2 (SC2), .CLK (CCLK12) );

`ifdef NO_CUTPOINTS
assign FlopAIn = IOFace ? CpuToNbIn : NbToCpuIn;   // FlopA input mux
assign CpuToNbOut  = WrpEn ? FlopBOut : CpuToNbIn; // Core to NB mux
assign NbToCpuOut  = WrpEn ? FlopBOut : NbToCpuIn; // NB to Core mux
`else
assign FlopAIn = IOFace ? CpuToNbIn : ~IOFace ? NbToCpuIn : 1'bx;   // FlopA input mux
assign CpuToNbOut  = WrpEn ? FlopBOut : ~WrpEn ? CpuToNbIn : 1'bx; // Core to NB mux
assign NbToCpuOut  = WrpEn ? FlopBOut : ~WrpEn ? NbToCpuIn : 1'bx; // NB to Core mux
`endif

// leda DCVER_154 on
// leda W287 on

endmodule

//==============================================================================
// PRIOLAT2 - 2 input latch with prioritized clock inputs (C0/D0 wins)
//==============================================================================

module PRIOLAT2 ( Q, D0, C0, D1, C1);

parameter  size = 1;

output reg [size-1:0] Q;
input      [size-1:0] D0;
input                 C0;
input      [size-1:0] D1;
input                 C1;

// leda SYN9_5 off
// leda W410 off

`ifdef NO_CUTPOINTS
//! latch
always @*
  if      ( C0 ) Q[size-1:0] <= D0[size-1:0];
  else if ( C1 ) Q[size-1:0] <= D1[size-1:0];
`else
//! latch
always @*
  Q[size-1:0] <= C0 ? D0[size-1:0] : ( C1 ? D1[size-1:0] : Q[size-1:0] );
`endif

// leda SYN9_5 on
// leda W410 on

endmodule

//============================================================================
// rtl_stdlib_internal_met_model
//   WARNING: This is intended to be an rtl_stdlib internal module, please
//     do not use it unless you work closely with it's author.
//   This is a seperate module so the meta_rand reg can remain active in MVSIM
//   Disabling RPL for the entire module to avoid unassigned warnings
//============================================================================
//! disable parser
module rtl_stdlib_internal_met_model (
   D,
   EN,
   Dsync,
   RandomExtraCycle
);

//VCS coverage exclude_module
// Inputs and Ouputs
input D;
input EN;
output reg Dsync;
output reg RandomExtraCycle;

//synopsys translate_off
reg  [31:0]       meta_rand;

//! unassigned Dsync
//! unassigned RandomExtraCycle
// leda off
`ifdef METASTABLE_MODEL
`ifndef COSIM_LIB_UNAVAILABLE
integer RandSeed;
initial begin
     if (check_option_switch("Sync2OrSeedOverride")) begin
        if (get_option_switch("Sync2OrSeedOverride")) begin
          RandSeed[31:0] = get_random_id_seed_override($activeinst,get_option_value("Sync2OrSeed"));
        end else begin
          RandSeed[31:0] = get_random_id($activeinst);
        end
     end
     else begin
       RandSeed[31:0] = get_random_id($activeinst);
     end
     meta_rand[31:0] = random32(RandSeed);
     RandomExtraCycle = random32(RandSeed);
end
`else // AMD Red LFSR Seed Initialization

   initial begin
     `ifdef AMD_RED_GLOBAL_RAND_SEED_LOC
        `ifdef RPL_TOOL
        meta_rand[31:0] = 32'hDEAD_BEEF;
        `else
        meta_rand[31:0] = $random(`AMD_RED_GLOBAL_RAND_SEED_LOC); //only want to call this random once so result can be duplicated
        `endif
     `else //AMD_RED_GLOBAL_RAND_SEED_LOC
        meta_rand[31:0] = 32'hDEAD_BEEF;
     `endif //AMD_RED_GLOBAL_RAND_SEED_LOC
      #1;
      RandomExtraCycle = 1'b0;
      Dsync = D;
   end
`endif // COSIM_LIB_UNAVAILABLE
`endif // METASTABLE_MODEL

//LFSR
always @(D) begin
   Dsync <= D; //transitions at the same time as RandomExtraCycle
   if(EN) begin
      RandomExtraCycle <= meta_rand[0];
      { meta_rand[31:8], meta_rand[5:3], meta_rand[1] } <= { meta_rand[30:7], meta_rand[4:2], meta_rand[0] };
      meta_rand[0] <= meta_rand[31];
      meta_rand[2] <= meta_rand[31] ^ meta_rand[1];
      meta_rand[6] <= meta_rand[31] ^ meta_rand[5];
      meta_rand[7] <= meta_rand[31] ^ meta_rand[6];
   end
end
// leda on
//synopsys translate_on

endmodule
//! enable parser

//============================================================================
// rtl_stdlib_internal_syncff_rand_model
//   WARNING: This is intended to be an rtl_stdlib internal module, please
//     do not use it unless you work closely with it's author.
//============================================================================
module rtl_stdlib_internal_syncff_rand_model (
   D,
   EN,
   CK,
   EnableRand,
`ifdef assertions
   InputChkDis,
`endif
   MetWindowMode,
   Q
);

//VCS coverage exclude_module

////////////////////
// IO declarations
////////////////////
input  D;
input  EN;
input  CK;
input  EnableRand;  // Randomization enable, simulation only
`ifdef assertions
input  InputChkDis; // OVA disable, simulation only
`endif
input  MetWindowMode;
output wire Q;

parameter metastable = 1'b1;

//! unused InputChkDis
wire     RandomExtraCycle;
reg      DelayFF;
wire     D_delayed;
wire     Dsync;
integer  LocalMetaWindowSize;

// leda off
initial begin
`ifdef TECH__SYNC__META_WINDOW_SIZE
  LocalMetaWindowSize[31:0] = `TECH__SYNC__META_WINDOW_SIZE;
`else
  LocalMetaWindowSize[31:0] = 180;
`endif
end
// leda on
//! unused LocalMetaWindowSize[31:0]


////////////////////////////////////////////////////////////////////////////////
// Window mode randomly delays the data by LocalMetaWindowSize. Use of this mode
// requires a simulation env that models jitter/frequency drift on clocks.
////////////////////////////////////////////////////////////////////////////////

// leda off
//! disable delay_restrictions
assign #(LocalMetaWindowSize[31:0]) D_delayed = D;
//! enable delay_restrictions
// leda on

// Random bit stream generator
rtl_stdlib_internal_met_model rtl_stdlib_internal_met_model (
   .D(D),
   .EN(EN),
   .Dsync(Dsync),
   .RandomExtraCycle(RandomExtraCycle)
);

// leda off
// Delay flip flop
always @(posedge CK) begin
   if(EN) DelayFF <= Dsync;
   `ifdef SYNC_WINDOW_DISPLAY_RANDOMIZATION
   if(MetWindowMode & RandomExtraCycle & EnableRand & (D_delayed ^ D))
      $display("INFO : %m : Synchronizer Meta-stability Window hit at Time=%t", $time);
   `endif
end

//synopsys translate_off
//! disable parser
always @(posedge EN) begin
   DelayFF <= Dsync;
end
//! enable parser
//synopsys translate_on

// leda on

// Using Dsync to prevent glitching when D changes then RandomExtraCycle changes
assign Q = (metastable & RandomExtraCycle & EnableRand) ? ( MetWindowMode ? D_delayed : DelayFF ) : Dsync;

//! unassigned InputChkDis
// This is RTL Simulation-ONLY logic used to model meta-stable events
//   If you get an ova compile error here, then you need to define assertions. This is not a bug!
// WARNING: For some paths it is probably not legal to disable this under NbIReset (i.e HT Config)
`ifdef ABV
//synopsys translate_off
//! disable parser
ovl_never #(.msg("DataIn is changing too quickly for the synchronizer.")) Assert_SYNCFF (
    .clock (CK), .reset(1'b1), .enable(~InputChkDis & ~MetWindowMode), 
    .test_expr( ~((D === DelayFF) | (DelayFF === $past(DelayFF,1,,@(posedge CK)))) &
                 (DelayFF !== 1'bx) & (Dsync !== 1'bx)), .fire());
//! enable parser
//synopsys translate_on
`else
// ova clock posedge CK;
// ova forbid_bool((~((D === DelayFF) | (DelayFF === DelayFF[#1])) & (DelayFF !== 1'bx) & (Dsync !== 1'bx) & ~InputChkDis & ~MetWindowMode), "SYNCFF Error: DataIn is changing too quickly for the synchronizer.");
`endif

////////////////////////////////////////////////////////////////////////////////
// Pulse Width Check for metastable window mode
//   - Data should be held long enough to hit 2 clock edges if shifted to cover
//     either destclk period to guarantee capture
////////////////////////////////////////////////////////////////////////////////

`ifdef RTL_ONLY
`elsif assertions
//synopsys translate_off
// leda off
//! disable parser

reg DataCaptured;
time lastCK, lastD, DestClockPeriod, DataPulseWidth, LastClockPeriod;

initial begin
  DataPulseWidth = 0; DestClockPeriod = 0; LastClockPeriod = 0; lastCK = 0; lastD = 0; DataCaptured = 0;
end

always @(negedge InputChkDis) begin // reset
  DataPulseWidth = 0; DestClockPeriod = 0; LastClockPeriod = 0; lastCK = 0; lastD = 0; DataCaptured = 0;
end

////////////////////////////////////////////////////////////////////////////////
// ClockCheck - measures last clock period, then compares the most recent data
//   against the last 2 clock periods. Data pulses smaller than either period 
//   could potentially drift and be lost.
// Disabled using InputChkDis (this should NOT be disabled all the time!!!!!)
////////////////////////////////////////////////////////////////////////////////
always @(posedge CK) begin : ClockCheck
    reg Last_Fail, This_Fail;
    if(MetWindowMode & ~InputChkDis) begin
        LastClockPeriod = DestClockPeriod;
        DestClockPeriod = $time - lastCK;
        if(DataPulseWidth) begin
            Last_Fail = DataCaptured && (DataPulseWidth < (LastClockPeriod + LocalMetaWindowSize));
            This_Fail = DataCaptured && (DataPulseWidth < (DestClockPeriod + LocalMetaWindowSize));
            if(Last_Fail & This_Fail ) begin
                $display("ERROR : %m : %0t D input pulse < %0t CLK period + %0t metastable window Time=%t", 
                    DataPulseWidth, DestClockPeriod, LocalMetaWindowSize,$time);
                `ifndef SYNC_WINDOW_DISABLE_FATAL_ERRORS
                $finish;
                `endif
            `ifndef SYNC_WINDOW_DISABLE_WARNINGS
            end else if( Last_Fail ) begin
                $display("WARNING : %m : %0t D input pulse < previous %0t CLK period + %0t metastable window Time=%t", 
                    DataPulseWidth, LastClockPeriod, LocalMetaWindowSize,$time);
            end else if( This_Fail ) begin
                $display("WARNING : %m : %0t D input pulse < %0t CLK period + %0t metastable window Time=%t", 
                    DataPulseWidth, DestClockPeriod, LocalMetaWindowSize,$time);
            `endif
            end
        end
        DataPulseWidth = 0; // last pulse checked
        lastCK = $time;
    end
end

////////////////////////////////////////////////////////////////////////////////
// DataCheck - measures data pulses, catches pulses smaller than a single clock
// Disabled using InputChkDis (this should NOT be disabled all the time!!!!!)
////////////////////////////////////////////////////////////////////////////////
always @(D) begin : DataCheck
    if(MetWindowMode & ~InputChkDis) begin
        if(($time - lastD) > 5) begin //smooth out any data glitches
            if(DataPulseWidth) begin // no clock edge occurred in this pulse, still need to check last pulse
                if(DataCaptured) begin // last pulse included a posedge clock, need to check it.
                    if(DataPulseWidth < (DestClockPeriod + LocalMetaWindowSize)) begin
                        $display("ERROR : %m : %0t D input pulse < %0t CLK period + %0t metastable window Time=%t", 
                            DataPulseWidth, DestClockPeriod, LocalMetaWindowSize,$time);
                        `ifndef SYNC_WINDOW_DISABLE_FATAL_ERRORS
                        $finish;
                        `endif
                    end
                end
                `ifndef SYNC_WINDOW_DISABLE_WARNINGS
                $display("WARNING : %m : %0t D pulse lost at Time=%t", 
                    DataPulseWidth,$time);
                `endif
            end
            DataCaptured = (lastD < lastCK) & (D_delayed !== 1'bx) & (D !== 1'bx);
            DataPulseWidth = $time - lastD;
        end
        lastD = $time;
    end
end
// leda on
//! enable parser
//synopsys translate_on
`endif

endmodule //rtl_stdlib_internal_syncff_rand_model


//============================================================================
// rtl_stdlib_internal_sync_multibit
//   WARNING: This is intended to be an rtl_stdlib internal module, please
//     do not use it unless you work closely with it's author.
//   Disabling RPL, this is a simulation-only module
//============================================================================
//! disable parser
module rtl_stdlib_internal_sync_multibit (
   EN,
   D,
   CK,
`ifdef assertions
   InputChkDis, // OVA disable, simulation only
`endif

   Q
);
 
//VCS coverage exclude_module
parameter WIDTH      = 1;
parameter metastable = 1'b1;
parameter transport_delay = 0;

input               EN;
input  [WIDTH-1:0]  D;
input               CK;
`ifdef assertions
input               InputChkDis; // OVA disable, simulation only
`endif
output wire [WIDTH-1:0]  Q;

// leda off

`ifdef METASTABLE_MODEL
//synopsys translate_off
reg [WIDTH-1:0] TRANSPORT_D;
reg  [WIDTH-1:0] EnableRand;
reg MetWindowMode;
reg transport_rand;
integer TransportDelay;
integer TransportDelay_random;
integer RandSeed;


`ifdef DISPLAY_CDC_OVERRIDES
always @(TransportDelay) if($test$plusargs("logCdcBufTiming")) $display("info: SYNC %m.TransportDelay\t = %d : %t", TransportDelay, $time);
`endif

initial begin
    MetWindowMode = 1'b0;
    TransportDelay[31:0] = transport_delay;
    TransportDelay_random[31:0] = 0;

`ifndef COSIM_LIB_UNAVAILABLE
    if (check_option_switch("Sync2OrSeedOverride")) begin
        if (get_option_switch("Sync2OrSeedOverride")) begin
            RandSeed[31:0] = get_random_id_seed_override($activeinst,get_option_value("Sync2OrSeed"));
        end else begin
            RandSeed[31:0] = get_random_id($activeinst);
        end
    end else begin
        RandSeed[31:0] = get_random_id($activeinst);
    end
    transport_rand = random32(RandSeed);
`else
    `ifdef AMD_RED_GLOBAL_RAND_SEED_LOC
        `ifdef RPL_TOOL
        transport_rand = 1'b1;
        `else
        transport_rand = $random(`AMD_RED_GLOBAL_RAND_SEED_LOC); //only want to call this random once so result can be duplicated
        `endif
    `else
        transport_rand = 1'b1;
    `endif
`endif // COSIM_LIB_UNAVAILABLE
    if(transport_rand) TransportDelay_random = TransportDelay;
end

wire [WIDTH-1:0] Old_D;

always @(D[WIDTH-1:0]) begin //avoid producing #0 delays
    if(TransportDelay_random) #(TransportDelay_random) TRANSPORT_D[WIDTH-1:0] = D[WIDTH-1:0];
    else TRANSPORT_D[WIDTH-1:0] = D[WIDTH-1:0];
end


assign #1 Old_D[WIDTH-1:0] = TRANSPORT_D[WIDTH-1:0];

integer i;
always @(posedge CK or TRANSPORT_D[WIDTH-1:0] or negedge EN) begin
   if(MetWindowMode)
      EnableRand[WIDTH-1:0] = {WIDTH{EN}};
   else begin
      EnableRand[WIDTH-1:0] = {WIDTH{1'b0}};
      if(EN) for(i=0;i<WIDTH;i=i+1) begin
         if(TRANSPORT_D[i] != Old_D[i]) EnableRand[i] = 1'b1;
         else                 EnableRand[i] = 1'b0;
      end
   end
end

always @* //in case TransportDelay is forced late or changes with voltage
    if(transport_rand) TransportDelay_random = TransportDelay;

rtl_stdlib_internal_syncff_rand_model #(.metastable(metastable)) syncff_rand [WIDTH-1:0] (
   .D         (TRANSPORT_D[WIDTH-1:0]         ),
   .EN        (EN        ),
   .CK        (CK        ),
   .EnableRand(EnableRand[WIDTH-1:0]),
`ifdef assertions
   .InputChkDis(InputChkDis), // OVA disable, simulation only
`endif
   .MetWindowMode(MetWindowMode     ),
   .Q         (Q[WIDTH-1:0]         )
);
//synopsys translate_on
`else // METASTABLE_MODEL
    `ifdef SYNC_DELAY_DATA
        // select D after C to provide determinism in gatesim.
        assign #0 Q[WIDTH-1:0] = D[WIDTH-1:0];
    `else
        assign Q[WIDTH-1:0] = D[WIDTH-1:0];
    `endif
`endif // METASTABLE_MODEL
// leda on

endmodule //rtl_stdlib_internal_sync_multibit
//! enable parser

//===========================================================================
// Standard Ring Oscillator
//
//   A simple ring oscillator that operates somewhere between 100-800 MHz.
// This is used to provide a slow clock to initialize synchronously-reset
// state when the PLL is unavailable, and will be paired with a standard cell
// clock mux that will divide down the ringo output and then provide a
// glitchless mux between the divided ringo output and a PLL output.
//
//===========================================================================
module AMS_STRO ( 
input               VPWR,
input               VGND,
`ifdef DEPRECATED_32NM
input               FakeClk,
`endif
output wire         OutClk
);

`ifndef HWEMUL

//! unused FakeClk
//! unassigned OutClkRaw
reg            OutClkRaw;

//! disable parser
// leda off
integer        RandSeedRO;
reg     [31:0] RandPeriodRO;
initial begin
  `ifdef COSIM_LIB_UNAVAILABLE
  RandPeriodRO[31:0] = 32'd761;
  `elsif USE_FIXED_INITIAL_VALUES
  RandPeriodRO[31:0] = 32'd761;
  `elsif RTL_ONLY
  RandPeriodRO[31:0] = 32'd761;
  `else
  RandSeedRO         = get_random_id($activeinst);
  RandPeriodRO[31:0] = (random32(RandSeedRO) % 4375) + 625; //100-800Mhz
  `endif 
  wait (VPWR===1'b1); // wait for power plane rise to avoid 0 time race on X-ing
  `ifdef COSIM_LIB_UNAVAILABLE
  OutClkRaw = 1'b0;
  `elsif USE_FIXED_INITIAL_VALUES
  OutClkRaw = 1'b0;
  `elsif RTL_ONLY
  OutClkRaw = 1'b0;
  `else
  OutClkRaw = random32(RandSeedRO);
  `endif
end
// leda on
//! enable parser

assign OutClk = (VPWR & ~VGND) ? OutClkRaw : 1'bx;

//! disable parser
// leda off
always @* begin
  OutClkRaw <= #(RandPeriodRO[31:0]) ~OutClkRaw;
end
// leda on
//! enable parser

`else //HWEMUL

wire BaseClock;
assign OutClk = BaseClock;
//! unassigned BaseClock

`endif //HWEMUL
endmodule

//===========================================================================
// Standard Ring Oscillator
//
// A simple ring oscillator that operates at a default 10-300 MHz.
//===========================================================================

module STD_ROEN ( VPWR, VGND, EN, OutClk);
input               VPWR;
input               VGND;
input               EN;
output wire         OutClk;

`ifndef HWEMUL

parameter min_period_ps = 3333;   //300Mhz
parameter max_period_ps = 100000; //10Mhz

//! unused EN
//! unassigned OutClkRaw
reg            OutClkRaw;
//! disable parser
// leda off
integer        RandSeedRO;
reg     [31:0] RandHalfPeriodRO;
initial begin
  `ifdef COSIM_LIB_UNAVAILABLE
  RandHalfPeriodRO[31:0] = (max_period_ps + min_period_ps) / 4;
  `elsif USE_FIXED_INITIAL_VALUES
  RandHalfPeriodRO[31:0] = (max_period_ps + min_period_ps) / 4;
  `elsif RTL_ONLY
  RandHalfPeriodRO[31:0] = (max_period_ps + min_period_ps) / 4;
  `else
  RandSeedRO             = get_random_id($activeinst);
  RandHalfPeriodRO[31:0] = (random32(RandSeedRO) % (max_period_ps/2)) + (min_period_ps/2);
  `endif 
  `ifndef RTL_ONLY
  wait (VPWR===1'b1); // wait for power plane rise to avoid 0 time race on X-ing
  `endif
  `ifdef COSIM_LIB_UNAVAILABLE
  OutClkRaw = 1'b0;
  `elsif USE_FIXED_INITIAL_VALUES
  OutClkRaw = 1'b0;
  `elsif RTL_ONLY
  OutClkRaw = 1'b0;
  `else
  OutClkRaw = random32(RandSeedRO);
  `endif
end
// leda on
//! enable parser

assign OutClk = (VPWR & ~VGND) ? OutClkRaw : 1'bx;

//! disable parser
// leda off
always @* begin
  OutClkRaw <= #(RandHalfPeriodRO[31:0]) EN ? ~OutClkRaw : 1'b1;
end
// leda on
//! enable parser

`else //HWEMUL

wire BaseClock;
assign OutClk = BaseClock;
//! unassigned BaseClock

`endif //HWEMUL
endmodule



//==============================================================================
// STD_CLKDIV - simple clock divider that handles initialization
//   
//==============================================================================

module STD_CLKDIV ( C, CDiv);

// no parameter for now, parameterizing require use of an ugly generate block

input      C;
output reg CDiv;

//synopsys translate_off
// leda off
//! disable parser
integer RandSeed;
initial begin
  `ifdef COSIM_LIB_UNAVAILABLE
  CDiv = 1'b0;
  `elsif USE_FIXED_INITIAL_VALUES
  CDiv = 1'b0;
  `elsif RTL_ONLY
  CDiv = 1'b0;
  `else
  RandSeed = get_random_id($activeinst);
  CDiv = random32(RandSeed);
  `endif 
end
//! enable parser
// leda on
//synopsys translate_on

always @(negedge C) CDiv <= ~CDiv;

endmodule


//==============================================================================
// ROXFER - Ring Oscillator vs ClockIn control logic & clock divider
//   ROClkOut  - PllROClk divided by 8, gated by ROOff
//   VCOEnSync - Synchronized to ClockIn, Use a b latch in the downstream module
//               to cleanly transition to the external clock.
//   ROClkDiv2, ROClkDiv4 & ROClkDiv8 need to be randomly initialized
//==============================================================================
module ROXFER ( Vpwr, Vgnd, PwrOk, PllROClk, ClockIn, ROEn, VCOEnSync, ROClkOut );

input        Vpwr;
input        Vgnd;
input        PwrOk;
input        PllROClk;
input        ClockIn;
output wire  ROEn;
output wire  VCOEnSync;
output wire  ROClkOut;

wire PwrOkROX,ROOff;
reg RODis, VCOEn, ROClkDiv2, ROClkDiv4,  ROClkDiv8;

//synopsys translate_off
// leda off
//! disable parser
integer RandSeed;
`ifdef ARCHPRO
always @(posedge Vpwr) begin
`else
initial begin
`endif
  `ifdef COSIM_LIB_UNAVAILABLE
  {ROClkDiv2,ROClkDiv4,ROClkDiv8} = 3'b000;
  `elsif USE_FIXED_INITIAL_VALUES
  {ROClkDiv2,ROClkDiv4,ROClkDiv8} = 3'b000;
  `elsif RTL_ONLY
  {ROClkDiv2,ROClkDiv4,ROClkDiv8} = 3'b000;
  `else
  RandSeed = get_random_id($activeinst);
  {ROClkDiv2,ROClkDiv4,ROClkDiv8} = random32(RandSeed);
  `endif 
end
//! enable parser
// leda on
//synopsys translate_on

// leda B_1205 off
always @(negedge PllROClk)  ROClkDiv2  <= ~ROClkDiv2;
always @(negedge ROClkDiv2) ROClkDiv4  <= ~ROClkDiv4;
always @(negedge ROClkDiv4) ROClkDiv8  <= ~ROClkDiv8;
assign ROClkOut = ~ROOff & ROClkDiv8;

`ifdef assertions
// leda off
wire OvaEnable;
wire OvaEnable_dly;
wire OvaEnable_nodly;
//synopsys translate_off
//! disable parser
assign #1 OvaEnable_dly = (PwrOk===1'bx) ? 1'b0 : 1'b1;
assign OvaEnable_nodly = (PwrOk===1'bx) ? 1'b0 : 1'b1;
assign OvaEnable = OvaEnable_nodly & OvaEnable_dly;
//! enable parser
// leda on
//synopsys translate_on
//! unassigned OvaEnable
//! unassigned OvaEnable_dly
//! unassigned OvaEnable_nodly
`endif

SYNC3OR I_PwrOkROSync (
       .SampleClk(ROClkDiv8),
       .SampleClkEn(Vpwr),
       .DataIn(~PwrOk),
       .AsyncOR(~PwrOk),
`ifdef assertions
       .OrChkDis(~OvaEnable),
       .InputChkDis(~OvaEnable),
`endif
       .DataOut(PwrOkROX)
);

// 1 cycle of delay between RO disable and RO off / VCO enable 
always @(negedge ROClkDiv8) begin
    RODis <= (Vpwr & ~Vgnd) ? ~PwrOkROX & ~RODis & ~VCOEn : 1'bx;
    VCOEn <= (Vpwr & ~Vgnd) ? ~PwrOkROX & (RODis | VCOEn) : 1'bx;
end
// leda B_1205 on

assign ROOff = VCOEn & PwrOk;
assign ROEn = ~VCOEn & ~RODis;

SYNC3OR I_VCORefClkSync (
       .SampleClk(ClockIn),
       .SampleClkEn(Vpwr),
       .DataIn(ROOff),
       .AsyncOR(Vgnd),
`ifdef assertions
       .OrChkDis(~OvaEnable),
       .InputChkDis(Vgnd),
`endif
       .DataOut(VCOEnSync)
);

endmodule //ROXFER

//============================================================================
// MSSCANLATCH - 
//============================================================================

// vrq translate_on
module MSSCANLATCH (Q,SDO,SDI,SC1,SC2);

output reg       Q;
output reg       SDO;
input            SDI;
input            SC1;
input            SC2;

// leda off
//! latch
always @* begin
  if (SC1) Q <= SDI;
end
//! latch
always @* begin
  if (SC2) SDO <= Q;
end
// leda on

endmodule // MSSCANLATCH
// vrq translate_off


//============================================================================
// ICG - 
//============================================================================

module ICG (Z,E,TE,CLK);
    output Z;     // Gated clock output
    input  E;     // Enable
    input  TE;    // Test enable
    input  CLK;

// This will be mapped by implementation 

wire LQB;
wire Z;

   
LATCH I_CoarseGater_next_B ( .Q(LQB), .D (E | TE), .C (~CLK) ); 

assign Z = LQB & CLK;


endmodule // ICG

//===========================================================================
// T9MSFQBNSX1_DFT -non scan flop to map with cell with different test view
//===========================================================================
module T9MSFQBNSX1_DFT (CLK, D, QB);

   input CLK;    
   input D ;
   output reg QB;

 always @(posedge CLK)

   QB <= (~D);
   
endmodule // T9MSFQBNSX1_DFT

//===========================================================================
// AMS_POWERSNIFFER - AMS COE's power sniffer
//===========================================================================
module AMS_POWERSNIFFER (VDDIN, VDDOUT, VSS, VDDINOk );
input                 VDDIN;
input                 VDDOUT;
input                 VSS;
output wire           VDDINOk;

//   In silicon, supplies have a non-zero ramp time and the sniffer circuit is
// designed to only assert VDDINOk well after the VDDIN drivers are
// sufficiently powered to drive valid signals into downstream domains.
//
//   In RTL sims we model power plane ramp time as instantaneous, so we have to
// add delay within the sniffer model to prevent a 0-time race at the firewall.
// Otherwise, VDDIN and VDDINOk would rise in the same timestep that
// VDDIN-based signals went from X to valid and the behavior of the firewall
// can not be predicted.  See UBTS 192164.            -- humphries  1 Dec 2009

`ifdef RTL_ONLY
`ifdef RTL_ONLY_DVT
wire    VDDIN_del;
//! disable delay_restrictions
// leda off
assign #1 VDDIN_del = VDDIN;
// leda on
//! enable delay_restrictions

assign VDDINOk = (VDDOUT & ~VSS) ? (VDDIN & VDDIN_del): 1'bx;
`else
assign VDDINOk = (VDDOUT & ~VSS) ? VDDIN : 1'bx;
`endif
`else
wire    VDDIN_del;
//! disable delay_restrictions
// leda off
assign #1 VDDIN_del = VDDIN;
// leda on
//! enable delay_restrictions

assign VDDINOk = (VDDOUT & ~VSS) ? (VDDIN & VDDIN_del): 1'bx;
`endif

endmodule

//===========================================================================
// Isolation Cells
//===========================================================================

module ISO_AN2 ( ISO_X, A, Z, VDD, VSS );
parameter size = 1;

input                  ISO_X;
input                  VDD;
input                  VSS;
input       [size-1:0] A;
output wire [size-1:0] Z;

assign Z[size-1:0] = ({size{ISO_X}} & A[size-1:0]) ^~ {size{(VDD & ~VSS) | 1'bx}};

endmodule

module ISO_ND2 ( ISO_X, A, Z, VDD, VSS );
parameter size = 1;

input                  ISO_X;
input                  VDD;
input                  VSS;
input       [size-1:0] A;
output wire [size-1:0] Z;

assign Z[size-1:0] = ~({size{ISO_X}} & A[size-1:0]) ^~ {size{(VDD & ~VSS) | 1'bx}};

endmodule

module ISO_OR2 ( ISO, A, Z, VDD, VSS ); parameter size = 1;

input                  ISO;
input                  VDD;
input                  VSS;
input       [size-1:0] A;
output wire [size-1:0] Z;

assign Z[size-1:0] = ({size{ISO}} | A[size-1:0]) ^~ {size{(VDD & ~VSS) | 1'bx}};

endmodule

module ISO_NR2 ( ISO, A, Z, VDD, VSS );
parameter size = 1;

input                  ISO;
input                  VDD;
input                  VSS;
input       [size-1:0] A;
output wire [size-1:0] Z;

assign Z[size-1:0] = ~({size{ISO}} | A[size-1:0]) ^~ {size{(VDD & ~VSS) | 1'bx}};

endmodule

module ISO_CKAN2 ( ISO_X, CLK, Z, VDD, VSS );

input                  ISO_X;
input                  VDD;
input                  VSS;
input                  CLK;
output wire            Z;

assign Z = (ISO_X & CLK) ^~ ((VDD & ~VSS) | 1'bx);

endmodule


//===========================================================================
// Level Shifters
//  Naming convention: LS(I|)(LL|LH|HL)(CK|)(NE|EQ)(0|1)(RST|)
//  LS - Level Shifter
//  I - Inverting
//  LL - Low to Low; LH - Low to High; HL - High to Low
//  NE - Back-end not Equalized; EQ - back-end delay equalized
//  CK - high speed
//  0/1 - default output when pwrok is not asserted
//  RST - models state that is reset/set by pwrok, clocked by vpwr1
//
//  vpwr1pwrok_vpwr2 - isolation enable pin. 
//    Typically indicates when input power domain is ok & pwrok voltage
//    conforms to the output power domain. There may be additional reasons 
//    that you would want to disable the output.
//===========================================================================

//===========================================================================
module LSILLEQ0 ( in_vpwr1, vpwr1pwrok_vpwr2, VPWR1, VPWR2, VSS, out_vpwr2_l);

parameter  size = 1;

input       [size-1:0] in_vpwr1;
input                  vpwr1pwrok_vpwr2;
input                  VPWR1;
input                  VPWR2;
input                  VSS;
output wire [size-1:0] out_vpwr2_l;

wire [size-1:0] tmp;
assign tmp[size-1:0] = ~in_vpwr1[size-1:0] ^~ {size{(VPWR1 | 1'bx)}};
assign out_vpwr2_l[size-1:0] = (tmp[size-1:0] & {size{vpwr1pwrok_vpwr2}}) ^~ {size{(VPWR2 & ~VSS) | 1'bx}};

endmodule

//===========================================================================
module LSILLNE0 ( in_vpwr1, vpwr1pwrok_vpwr2, VPWR1, VPWR2, VSS, out_vpwr2_l);

parameter  size = 1;

input       [size-1:0] in_vpwr1;
input                  vpwr1pwrok_vpwr2;
input                  VPWR1;
input                  VPWR2;
input                  VSS;
output wire [size-1:0] out_vpwr2_l;

wire [size-1:0] tmp;
assign tmp[size-1:0] = ~in_vpwr1[size-1:0]  ^~ {size{(VPWR1 | 1'bx)}};
assign out_vpwr2_l[size-1:0] = (tmp[size-1:0] & {size{vpwr1pwrok_vpwr2}}) ^~ {size{(VPWR2 & ~VSS) | 1'bx}};

endmodule

//===========================================================================
module LSILHEQ0 ( in_vpwr1, vpwr1pwrok_vpwr2, VPWR1, VPWR2, VSS, out_vpwr2_l);

parameter  size = 1;

input       [size-1:0] in_vpwr1;
input                  vpwr1pwrok_vpwr2;
input                  VPWR1;
input                  VPWR2;
input                  VSS;
output wire [size-1:0] out_vpwr2_l;

wire [size-1:0] tmp;
assign tmp[size-1:0] = ~in_vpwr1[size-1:0]  ^~ {size{(VPWR1 | 1'bx)}};
assign out_vpwr2_l[size-1:0] = (tmp[size-1:0] & {size{vpwr1pwrok_vpwr2}}) ^~ {size{(VPWR2 & ~VSS) | 1'bx}};

endmodule

//===========================================================================
module LSILHNE0 ( in_vpwr1, vpwr1pwrok_vpwr2, VPWR1, VPWR2, VSS, out_vpwr2_l);

parameter  size = 1;

input       [size-1:0] in_vpwr1;
input                  vpwr1pwrok_vpwr2;
input                  VPWR1;
input                  VPWR2;
input                  VSS;
output wire [size-1:0] out_vpwr2_l;

wire [size-1:0] tmp;
assign tmp[size-1:0] = ~in_vpwr1[size-1:0]  ^~ {size{(VPWR1 | 1'bx)}};
assign out_vpwr2_l[size-1:0] = (tmp[size-1:0] & {size{vpwr1pwrok_vpwr2}}) ^~ {size{(VPWR2 & ~VSS) | 1'bx}};

endmodule

//===========================================================================
module LSIHLEQ0 ( in_vpwr1, vpwr1pwrok_vpwr2, VPWR1, VPWR2, VSS, out_vpwr2_l);

parameter  size = 1;

input       [size-1:0] in_vpwr1;
input                  vpwr1pwrok_vpwr2;
input                  VPWR1;
input                  VPWR2;
input                  VSS;
output wire [size-1:0] out_vpwr2_l;

wire [size-1:0] tmp;
assign tmp[size-1:0] = ~in_vpwr1[size-1:0]  ^~ {size{(VPWR1 | 1'bx)}};
assign out_vpwr2_l[size-1:0] = (tmp[size-1:0] & {size{vpwr1pwrok_vpwr2}}) ^~ {size{(VPWR2 & ~VSS) | 1'bx}};

endmodule

//===========================================================================
module LSIHLNE0 ( in_vpwr1, vpwr1pwrok_vpwr2, VPWR1, VPWR2, VSS, out_vpwr2_l);

parameter  size = 1;

input       [size-1:0] in_vpwr1;
input                  vpwr1pwrok_vpwr2;
input                  VPWR1;
input                  VPWR2;
input                  VSS;
output wire [size-1:0] out_vpwr2_l;

wire [size-1:0] tmp;
assign tmp[size-1:0] = ~in_vpwr1[size-1:0] ^~ {size{(VPWR1 | 1'bx)}};
assign out_vpwr2_l[size-1:0] = (tmp[size-1:0] & {size{vpwr1pwrok_vpwr2}}) ^~ {size{(VPWR2 & ~VSS) | 1'bx}};

endmodule

//===========================================================================
module LSILLEQ1 ( in_vpwr1, vpwr1pwrok_vpwr2, VPWR1, VPWR2, VSS, out_vpwr2_l);

parameter  size = 1;

input       [size-1:0] in_vpwr1;
input                  vpwr1pwrok_vpwr2;
input                  VPWR1;
input                  VPWR2;
input                  VSS;
output wire [size-1:0] out_vpwr2_l;

wire [size-1:0] tmp;
assign tmp[size-1:0] = ~in_vpwr1[size-1:0] ^~ {size{(VPWR1 | 1'bx)}};
assign out_vpwr2_l[size-1:0] = (tmp[size-1:0] | {size{~vpwr1pwrok_vpwr2}}) ^~ {size{(VPWR2 & ~VSS) | 1'bx}};
   
endmodule

//===========================================================================
module LSILLNE1 ( in_vpwr1, vpwr1pwrok_vpwr2, VPWR1, VPWR2, VSS, out_vpwr2_l);

parameter  size = 1;

input       [size-1:0] in_vpwr1;
input                  vpwr1pwrok_vpwr2;
input                  VPWR1;
input                  VPWR2;
input                  VSS;
output wire [size-1:0] out_vpwr2_l;

wire [size-1:0] tmp;
assign tmp[size-1:0] = ~in_vpwr1[size-1:0] ^~ {size{(VPWR1 | 1'bx)}};
assign out_vpwr2_l[size-1:0] = (tmp[size-1:0] | {size{~vpwr1pwrok_vpwr2}}) ^~ {size{(VPWR2 & ~VSS) | 1'bx}};
   
endmodule

//===========================================================================
module LSILHEQ1 ( in_vpwr1, vpwr1pwrok_vpwr2, VPWR1, VPWR2, VSS, out_vpwr2_l);

parameter  size = 1;

input       [size-1:0] in_vpwr1;
input                  vpwr1pwrok_vpwr2;
input                  VPWR1;
input                  VPWR2;
input                  VSS;
output wire [size-1:0] out_vpwr2_l;

wire [size-1:0] tmp;
assign tmp[size-1:0] = ~in_vpwr1[size-1:0] ^~ {size{(VPWR1 | 1'bx)}};
assign out_vpwr2_l[size-1:0] = (tmp[size-1:0] | {size{~vpwr1pwrok_vpwr2}}) ^~ {size{(VPWR2 & ~VSS) | 1'bx}};
   
endmodule

//===========================================================================
module LSILHNE1 ( in_vpwr1, vpwr1pwrok_vpwr2, VPWR1, VPWR2, VSS, out_vpwr2_l);

parameter  size = 1;

input       [size-1:0] in_vpwr1;
input                  vpwr1pwrok_vpwr2;
input                  VPWR1;
input                  VPWR2;
input                  VSS;
output wire [size-1:0] out_vpwr2_l;

wire [size-1:0] tmp;
assign tmp[size-1:0] = ~in_vpwr1[size-1:0] ^~ {size{(VPWR1 | 1'bx)}};
assign out_vpwr2_l[size-1:0] = (tmp[size-1:0] | {size{~vpwr1pwrok_vpwr2}}) ^~ {size{(VPWR2 & ~VSS) | 1'bx}};
   
endmodule

//===========================================================================
module LSIHLEQ1 ( in_vpwr1, vpwr1pwrok_vpwr2, VPWR1, VPWR2, VSS, out_vpwr2_l);

parameter  size = 1;

input       [size-1:0] in_vpwr1;
input                  vpwr1pwrok_vpwr2;
input                  VPWR1;
input                  VPWR2;
input                  VSS;
output wire [size-1:0] out_vpwr2_l;

wire [size-1:0] tmp;
assign tmp[size-1:0] = ~in_vpwr1[size-1:0] ^~ {size{(VPWR1 | 1'bx)}};
assign out_vpwr2_l[size-1:0] = (tmp[size-1:0] | {size{~vpwr1pwrok_vpwr2}}) ^~ {size{(VPWR2 & ~VSS) | 1'bx}};
   
endmodule

//===========================================================================
module LSIHLNE1 ( in_vpwr1, vpwr1pwrok_vpwr2, VPWR1, VPWR2, VSS, out_vpwr2_l);

parameter  size = 1;

input       [size-1:0] in_vpwr1;
input                  vpwr1pwrok_vpwr2;
input                  VPWR1;
input                  VPWR2;
input                  VSS;
output wire [size-1:0] out_vpwr2_l;

wire [size-1:0] tmp;
assign tmp[size-1:0] = ~in_vpwr1[size-1:0] ^~ {size{(VPWR1 | 1'bx)}};
assign out_vpwr2_l[size-1:0] = (tmp[size-1:0] | {size{~vpwr1pwrok_vpwr2}}) ^~ {size{(VPWR2 & ~VSS) | 1'bx}};
    
endmodule

//===========================================================================
module LSLHEQ0 ( DinVPWR1, EnPWR2, VPWR1, VPWR2, VSS, DoutPWR2);

parameter  size = 1;

input       [size-1:0] DinVPWR1;
input                  EnPWR2;
input                  VPWR1;
input                  VPWR2;
input                  VSS;
output wire [size-1:0] DoutPWR2;

wire [size-1:0] tmp;
assign tmp[size-1:0] = DinVPWR1[size-1:0]  ^~ {size{(VPWR1 | 1'bx)}};
assign DoutPWR2[size-1:0] = (tmp[size-1:0] & {size{EnPWR2}}) ^~ {size{(VPWR2 & ~VSS) | 1'bx}};

endmodule

//===========================================================================
module LSLHEQ1 ( DinVPWR1, EnPWR2, VPWR1, VPWR2, VSS, DoutPWR2);

parameter  size = 1;

input       [size-1:0] DinVPWR1;
input                  EnPWR2;
input                  VPWR1;
input                  VPWR2;
input                  VSS;
output wire [size-1:0] DoutPWR2;

wire [size-1:0] tmp;
assign tmp[size-1:0] = DinVPWR1[size-1:0] ^~ {size{(VPWR1 | 1'bx)}};
assign DoutPWR2[size-1:0] = (tmp[size-1:0] | {size{~EnPWR2}}) ^~ {size{(VPWR2 & ~VSS) | 1'bx}};
   
endmodule

//===========================================================================
module LSIHLEQ ( A, VDD, VSS, Z);

parameter  size = 1;

input       [size-1:0] A;
input                  VDD;
input                  VSS;
output wire [size-1:0] Z;

assign Z[size-1:0] = ~A[size-1:0] ^~ {size{(VDD & ~VSS) | 1'bx}};

endmodule

//===========================================================================
module LSILLNE1RST (in_vpwr1, vpwr1pwrok_vpwr2, VPWR1, VPWR2, VSS, out_vpwr2_l );

parameter size = 1;

input      [size-1:0] in_vpwr1 ;
input                 vpwr1pwrok_vpwr2 ;
input                 VPWR1;
input                 VPWR2;
input                 VSS;
output reg [size-1:0] out_vpwr2_l ;

//! latch
always @* begin
  out_vpwr2_l[size-1:0] <= ~(VPWR2 & ~VSS) ?  {size{1'bx}} : 
    ( ~vpwr1pwrok_vpwr2 ? {size{1'b1}} : ( VPWR1 ? ~in_vpwr1[size-1:0] : out_vpwr2_l[size-1:0] ) );
end

endmodule

module LSLLCKEQ0 ( in_vpwr1, vpwr1pwrok_vpwr2, VPWR1, VPWR2, VSS, out_vpwr2_l);

parameter  size = 1;

input       [size-1:0] in_vpwr1;
input                  vpwr1pwrok_vpwr2;
input                  VPWR1;
input                  VPWR2;
input                  VSS;
output wire [size-1:0] out_vpwr2_l;

wire [size-1:0] tmp;
assign out_vpwr2_l[size-1:0] = ~((tmp[size-1:0] | ~{size{vpwr1pwrok_vpwr2}}) ^~ {size{(VPWR2 & ~VSS) | 1'bx}});
assign tmp[size-1:0] = ~in_vpwr1[size-1:0] ^~ {size{(VPWR1 | 1'bx)}};

endmodule

///////////////////////////////////////////////
// New level shifters added from bug 394699
///////////////////////////////////////////////

module LSIHLNE1RST (in_vpwr1, vpwr1pwrok_vpwr2, VPWR1, VPWR2, VSS, out_vpwr2_l );

parameter size = 1;

input      [size-1:0] in_vpwr1 ;
input                 vpwr1pwrok_vpwr2 ;
input                 VPWR1;
input                 VPWR2;
input                 VSS;
output reg [size-1:0] out_vpwr2_l ;

//! latch
always @* begin
  out_vpwr2_l[size-1:0] <= ~(VPWR2 & ~VSS) ?  {size{1'bx}} :
    ( ~vpwr1pwrok_vpwr2 ? {size{1'b1}} : ( VPWR1 ? ~in_vpwr1[size-1:0] : out_vpwr2_l[size-1:0] ) );
end

endmodule

module LSILHNE1RST (in_vpwr1, vpwr1pwrok_vpwr2, VPWR1, VPWR2, VSS, out_vpwr2_l );

parameter size = 1;

input      [size-1:0] in_vpwr1 ;
input                 vpwr1pwrok_vpwr2 ;
input                 VPWR1;
input                 VPWR2;
input                 VSS;
output reg [size-1:0] out_vpwr2_l ;

//! latch
always @* begin
  out_vpwr2_l[size-1:0] <= ~(VPWR2 & ~VSS) ?  {size{1'bx}} :
    ( ~vpwr1pwrok_vpwr2 ? {size{1'b1}} : ( VPWR1 ? ~in_vpwr1[size-1:0] : out_vpwr2_l[size-1:0] ) );
end

endmodule

module LSILHEQ1RST (in_vpwr1, vpwr1pwrok_vpwr2, VPWR1, VPWR2, VSS, out_vpwr2_l );

parameter size = 1;

input      [size-1:0] in_vpwr1 ;
input                 vpwr1pwrok_vpwr2 ;
input                 VPWR1;
input                 VPWR2;
input                 VSS;
output reg [size-1:0] out_vpwr2_l ;

//! latch
always @* begin
  out_vpwr2_l[size-1:0] <= ~(VPWR2 & ~VSS) ?  {size{1'bx}} :
    ( ~vpwr1pwrok_vpwr2 ? {size{1'b1}} : ( VPWR1 ? ~in_vpwr1[size-1:0] : out_vpwr2_l[size-1:0] ) );
end

endmodule

module LSILLNE0RST (in_vpwr1, vpwr1pwrok_vpwr2, VPWR1, VPWR2, VSS, out_vpwr2_l );

parameter size = 1;

input      [size-1:0] in_vpwr1 ;
input                 vpwr1pwrok_vpwr2 ;
input                 VPWR1;
input                 VPWR2;
input                 VSS;
output reg [size-1:0] out_vpwr2_l ;

//! latch
always @* begin
  out_vpwr2_l[size-1:0] <= ~(VPWR2 & ~VSS) ?  {size{1'bx}} :
    ( ~vpwr1pwrok_vpwr2 ? {size{1'b0}} : ( VPWR1 ? ~in_vpwr1[size-1:0] : out_vpwr2_l[size-1:0] ) );
end

endmodule

module LSILHNE0RST (in_vpwr1, vpwr1pwrok_vpwr2, VPWR1, VPWR2, VSS, out_vpwr2_l );

parameter size = 1;

input      [size-1:0] in_vpwr1 ;
input                 vpwr1pwrok_vpwr2 ;
input                 VPWR1;
input                 VPWR2;
input                 VSS;
output reg [size-1:0] out_vpwr2_l ;

//! latch
always @* begin
  out_vpwr2_l[size-1:0] <= ~(VPWR2 & ~VSS) ?  {size{1'bx}} :
    ( ~vpwr1pwrok_vpwr2 ? {size{1'b0}} : ( VPWR1 ? ~in_vpwr1[size-1:0] : out_vpwr2_l[size-1:0] ) );
end

endmodule

module LSILHEQ0RST (in_vpwr1, vpwr1pwrok_vpwr2, VPWR1, VPWR2, VSS, out_vpwr2_l );

parameter size = 1;

input      [size-1:0] in_vpwr1 ;
input                 vpwr1pwrok_vpwr2 ;
input                 VPWR1;
input                 VPWR2;
input                 VSS;
output reg [size-1:0] out_vpwr2_l ;

//! latch
always @* begin
  out_vpwr2_l[size-1:0] <= ~(VPWR2 & ~VSS) ?  {size{1'bx}} :
    ( ~vpwr1pwrok_vpwr2 ? {size{1'b0}} : ( VPWR1 ? ~in_vpwr1[size-1:0] : out_vpwr2_l[size-1:0] ) );
end

endmodule

module LSIHLNE0RST (in_vpwr1, vpwr1pwrok_vpwr2, VPWR1, VPWR2, VSS, out_vpwr2_l );

parameter size = 1;

input      [size-1:0] in_vpwr1 ;
input                 vpwr1pwrok_vpwr2 ;
input                 VPWR1;
input                 VPWR2;
input                 VSS;
output reg [size-1:0] out_vpwr2_l ;

//! latch
always @* begin
  out_vpwr2_l[size-1:0] <= ~(VPWR2 & ~VSS) ?  {size{1'bx}} :
    ( ~vpwr1pwrok_vpwr2 ? {size{1'b0}} : ( VPWR1 ? ~in_vpwr1[size-1:0] : out_vpwr2_l[size-1:0] ) );
end

endmodule

//==============================================================================
// SSBRETIMER: This module is used in SSB type interfaces.It models
// the function of a retimer cell that is used to match Clock delay to data.
// UBTS:401322
//==============================================================================
module SSBRETIMER ( CLK, CLKOUT, CLKOUTB );
   input CLK;           //Incoming forwarded clock.
   output wire CLKOUT;  //Clock output to current flop stage.
   output wire CLKOUTB; //Inverted Clock output; forward to next flop stage


   assign CLKOUT = CLK;
   assign CLKOUTB = ~CLK;

endmodule // SSBRETIMER

//==============================================================================
// MRKBUF: This module is used as a marker buffer, to preserve logic
// and prevent optimization. Typically placed in IP that is shared
// with other teams. UBTS:495090
//==============================================================================
module MRKBUF( A, Y );
 input       A;
 output wire Y;

 assign Y = A;

endmodule // MRKBUF

// leda E_122 on
// leda E_169 on
// leda E_001 on

// leda XPROP_TERN1 on

// leda XPROP_IF2 on
// leda XPROP_TERN3 on
// leda FM_2_16 on
// leda C_1000 on
// leda NTL_STR47 on
// leda NTL_STR33 on
// leda VER_3_2_4_2A on
// leda VER_1_1_1_1 on
// leda XPROP_TERN_RHS on
// leda W446 on
// leda NTL_STR61 on

// vrq translate_on

// Add the `celldefine compiler directive for improved simulation
// performance. UBTS:393082
`ifdef RTL_STDLIB_CELLDEFINES
`endcelldefine
`endif

//! enable one_module_per_file
// spyglass enable_block ALL

