//                              -*- Mode: Verilog -*-

////////////////////////////////////////////////////////////////////////////
/*
 *
 *  Trade secret of ATI Technologies, Inc.
 *  Copyright 2005, ATI Technologies, Inc., (unpublished)
 *
 *  All rights reserved.  This notice is intended as a precaution against
 *  inadvertent publication and does not imply publication or any waiver
 *  of confidentiality.  The year included in the foregoing notice is the
 *  year of creation of the work.
 *
 */
////////////////////////////////////////////////////////////////////////////
//
// Filename    : scan_cntl.v
// Description : Scan control 
//
////////////////////////////////////////////////////////////////////////////

//`include "scan_cntl_features.vh"

//;use comp_config;


module scan_cntl_gc10 
# (
   parameter TILE_SCAN_IN_CHANNEL_WIDTH = 1,
   parameter TILE_SCAN_OUT_CHANNEL_WIDTH = 1,
   parameter GFX_HARVEST_PRESENT = 1,
   parameter NO_SHIFT_CLK = 0
)
  (
   //Broadcast to All TDRs
   input wire Tdr_Tck,
   input wire Tdr_Tdi,
   input wire Tdr_Trstb,
   input wire Tdr_Shift,
   input wire Tdr_Capture,
   input wire Tdr_Update,
   
   //Tdr
   input wire Tdr_byp_scm_segment_En,
   output wire Tdr_byp_scm_segment_Tdo,
   
   input wire Tdr_scan_setup_En,
   output wire Tdr_scan_setup_Tdo,
   
   input wire Tdr_shift_cnt_En,
   output wire Tdr_shift_cnt_Tdo,
   
   input wire Tdr_gfx_harvest_En,
   output wire Tdr_gfx_harvest_Tdo,
   
   //CLK
   input   wire Shift_Clk,
   output  wire Tile_Shift_Clk,
   output wire Edt_Clk,
   output wire Scan_Pipeline_Clk,
   
   // SCAN
   input  wire ScanEn,
   output wire Tile_ScanEn,
   output wire Tile_ScanEn_Neg,
   output wire Tile_DC_ScanEn,
   output wire Tile_GaterEn,    // Connect to CGCG TE pin.
   output wire Scan_Mode,
   output wire Scm_Mode,
   output wire Scan_Cntl_Pipe_En,
   output wire Scan_Cntl_Low_Power_Edt,
   output wire  Extest_Mode,
   output wire Gater_Table_Mode,
   output wire Stuck_At_Mode,
   //Harvesting
   output wire [1-1:0] rep_fault_byp_mux_in,
   output wire [1-1:0] rep_fault_byp_mux_sel,
   
   output wire ScanEn_CP_Override,
   output wire ScanEn_IP_Override,
   output wire ScanEn_OP_Override,
   output wire SE_Sel,
   output wire LOS_Sel,
   output wire TEST_CLK_EN,
   output wire DC_TEST_CLK_EN,
   output wire Async_Rstb_Scan_Override,
   output wire Async_Rstb_Scan_Override_Sel,
   
   input wire Gater_Flop_ScanIn,
   output wire Gater_Flop_ScanOut,
   
   //PGFSM Reset
   input wire PGFSM_P0_resetb,
   input wire PGFSM_P1_resetb,
   input wire PGFSM_P2_resetb,
   input wire PGFSM_P3_resetb,
   input wire PGFSM_P4_resetb,
   input wire PGFSM_P5_resetb,
   input wire PGFSM_P6_resetb,
   input wire PGFSM_P7_resetb,
   
   
   //edt
   input  wire IO_Edt_Update,
   output wire Edt_Update,
   output wire Edt_Bypass,
   
   //Scan Channels
   input  wire Tile_ScanChannelIn_0,
   output wire Tile_ScanChannelOut_0,
   input  wire ScanChannelFromEdt_0,
   output wire ScanChannelToEdt_0,
   output wire ScanChannelMask_0,					  
   input  wire Tile_ScanChannelIn_1,
   output wire Tile_ScanChannelOut_1,
   input  wire ScanChannelFromEdt_1,
   output wire ScanChannelToEdt_1,
   output wire ScanChannelMask_1,					  
   input  wire Tile_ScanChannelIn_2,
   output wire Tile_ScanChannelOut_2,
   input  wire ScanChannelFromEdt_2,
   output wire ScanChannelToEdt_2,
   output wire ScanChannelMask_2,					  
   input  wire Tile_ScanChannelIn_3,
   output wire Tile_ScanChannelOut_3,
   input  wire ScanChannelFromEdt_3,
   output wire ScanChannelToEdt_3,
   output wire ScanChannelMask_3,					  
   input  wire Tile_ScanChannelIn_4,
   output wire Tile_ScanChannelOut_4,
   input  wire ScanChannelFromEdt_4,
   output wire ScanChannelToEdt_4,
   output wire ScanChannelMask_4,					  
   input  wire Tile_ScanChannelIn_5,
   output wire Tile_ScanChannelOut_5,
   input  wire ScanChannelFromEdt_5,
   output wire ScanChannelToEdt_5,
   output wire ScanChannelMask_5,					  
   input  wire Tile_ScanChannelIn_6,
   output wire Tile_ScanChannelOut_6,
   input  wire ScanChannelFromEdt_6,
   output wire ScanChannelToEdt_6,
   output wire ScanChannelMask_6,					  
   input  wire Tile_ScanChannelIn_7,
   output wire Tile_ScanChannelOut_7,
   input  wire ScanChannelFromEdt_7,
   output wire ScanChannelToEdt_7,
   output wire ScanChannelMask_7,					  
   input  wire Tile_ScanChannelIn_8,
   output wire Tile_ScanChannelOut_8,
   input  wire ScanChannelFromEdt_8,
   output wire ScanChannelToEdt_8,
   output wire ScanChannelMask_8,					  
   input  wire Tile_ScanChannelIn_9,
   output wire Tile_ScanChannelOut_9,
   input  wire ScanChannelFromEdt_9,
   output wire ScanChannelToEdt_9,
   output wire ScanChannelMask_9,					  
   input  wire Tile_ScanChannelIn_10,
   output wire Tile_ScanChannelOut_10,
   input  wire ScanChannelFromEdt_10,
   output wire ScanChannelToEdt_10,
   output wire ScanChannelMask_10,					  
   input  wire Tile_ScanChannelIn_11,
   output wire Tile_ScanChannelOut_11,
   input  wire ScanChannelFromEdt_11,
   output wire ScanChannelToEdt_11,
   output wire ScanChannelMask_11,					  
   input  wire Tile_ScanChannelIn_12,
   output wire Tile_ScanChannelOut_12,
   input  wire ScanChannelFromEdt_12,
   output wire ScanChannelToEdt_12,
   output wire ScanChannelMask_12,					  
   input  wire Tile_ScanChannelIn_13,
   output wire Tile_ScanChannelOut_13,
   input  wire ScanChannelFromEdt_13,
   output wire ScanChannelToEdt_13,
   output wire ScanChannelMask_13,					  
   input  wire Tile_ScanChannelIn_14,
   output wire Tile_ScanChannelOut_14,
   input  wire ScanChannelFromEdt_14,
   output wire ScanChannelToEdt_14,
   output wire ScanChannelMask_14,					  
   input  wire Tile_ScanChannelIn_15,
   output wire Tile_ScanChannelOut_15,
   input  wire ScanChannelFromEdt_15,
   output wire ScanChannelToEdt_15,
   output wire ScanChannelMask_15,					  
   input  wire Tile_ScanChannelIn_16,
   output wire Tile_ScanChannelOut_16,
   input  wire ScanChannelFromEdt_16,
   output wire ScanChannelToEdt_16,
   output wire ScanChannelMask_16,					  
 
 //Scm
   input  wire Single_Chain_Daisy_In,
   output wire  Single_Chain_Daisy_Out,
   output wire Scm_Async_Reset_Pulse,
   
   //SMS
   output  wire [1-1:0] dm0,
   output  wire [1-1:0] dm1,
   output  wire [1-1:0] dm2
   
   //Harvesting
   );
   
   
   // Bypass channel number decided by smaller number of channels in async edt mode.
   // Input Channel >  Output Channel -> Out_chan
   // Input Channel <  Output Channel -> In_chan
   
   localparam TILE_SCAN_CHANNEL_WIDTH = 
	      (TILE_SCAN_IN_CHANNEL_WIDTH > TILE_SCAN_OUT_CHANNEL_WIDTH) ? 
              TILE_SCAN_OUT_CHANNEL_WIDTH : TILE_SCAN_IN_CHANNEL_WIDTH;
   
   /////////////////////////// wire ///////////////////////////////////////////////////
   
   wire [1-1:0] los_sel;
   wire [1-1:0] byp_scm_seg_condition_override;
   wire [2-1:0] cap_disable;
   //wire [1-1:0] dm0;
   //wire [1-1:0] dm1;
   //wire [1-1:0] dm2;
   wire [1-1:0] edt_bypass;
   wire [1-1:0] edt_update_sel;
   wire [1-1:0] extest_mode;
   wire [1-1:0] gater_table_mode;
   wire [1-1:0] low_power_edt;
   wire [1-1:0] pipe_en;
   wire [1-1:0] scan_en_cp_override;
   wire [1-1:0] scan_en_ip_override;
   wire [1-1:0] scan_en_op_override;
   wire [1-1:0] scan_mode;
   wire [1-1:0] scm_mode;
   wire [1-1:0] se_sel;
   wire [1-1:0] shift_clk_stop;
   wire [1-1:0] stuck_at_mode;
   wire [1-1:0] test_clk_en;
   wire [1-1:0] test_clk_en_sel;
   wire         sleep_en;
   
   wire [17-1:0] byp_scm_segment_to_scm_mux; 
   wire [17-1:0] byp_scm_segment_to_scan_channel; 
   
   wire Shift_Clk_Raw;
   wire gFunc_Clk;
   wire gShift_Clk;
   wire gShift_Clk_1;
   wire scan_mode_sync;
   wire scan_pipeline_clk_en;
   wire gEdt_Clk;
   wire [16-1:0] shift_cnt;
   wire ScanEn_Sync_1;
   wire ScanEn_Sync;
   wire ScanEn_Muxed_pre;
   
   wire [17-1:0] Tile_ScanChannelIn;
   wire [17-1:0] Tile_ScanChannelOut;
   wire [17-1:0] ScanChannelFromEdt;
   wire [17-1:0] ScanChannelToEdt;
   
   wire async_rstb_override_pre, async_rstb_override_sel_pre;
   wire scan_mode_pulse;
   wire gater_ff_async_rstb;
   
   /////////////////////////// TDR Instances Start /////////////////////////////////////
   
    
    scan_setup_tdr_gc10 scan_setup_tdr 
	(
	 
	 .Tdr_Tck               (Tdr_Tck) ,
	 .Tdr_Trstb             (Tdr_Trstb),
	 .Tdr_Tdi               (Tdr_Tdi),
	 .Tdr_Shift             (Tdr_Shift),
	 .Tdr_Capture           (Tdr_Capture),
	 .Tdr_Update            (Tdr_Update)
	 
	 
	 //TDR --> SCAN_SETUP
	 ,.los_sel             (los_sel[1-1:0])
	 ,.byp_scm_seg_condition_override                (byp_scm_seg_condition_override[1-1:0])
     ,.cap_disable   (cap_disable[2-1:0])
	 ,.dm0           (dm0[1-1:0])
	 ,.dm1           (dm1[1-1:0])
	 ,.dm2           (dm2[1-1:0])
	 ,.edt_bypass            (edt_bypass[1-1:0])
	 ,.edt_update_sel                (edt_update_sel[1-1:0])
	 ,.extest_mode           (extest_mode[1-1:0])
	 ,.gater_table_mode              (gater_table_mode[1-1:0])
	 ,.low_power_edt                 (low_power_edt[1-1:0])
	 ,.pipe_en               (pipe_en[1-1:0])
	 ,.scan_en_cp_override           (scan_en_cp_override[1-1:0])
	 ,.scan_en_ip_override           (scan_en_ip_override[1-1:0])
	 ,.scan_en_op_override           (scan_en_op_override[1-1:0])
	 ,.scan_mode             (scan_mode[1-1:0])
	 ,.scm_mode              (scm_mode[1-1:0])
	 ,.se_sel                (se_sel[1-1:0])
	 ,.shift_clk_stop                (shift_clk_stop[1-1:0])
	 ,.stuck_at_mode                 (stuck_at_mode[1-1:0])
	 ,.test_clk_en           (test_clk_en[1-1:0])
	 ,.test_clk_en_sel               (test_clk_en_sel[1-1:0])
	 ,.Tdr_scan_setup_En            (Tdr_scan_setup_En)
	 ,.Tdr_scan_setup_Tdo           (Tdr_scan_setup_Tdo)
	 );
   

   
   
   generate 
    if(TILE_SCAN_CHANNEL_WIDTH == 0) 
	begin
       assign Tdr_byp_scm_segment_Tdo = 1'b0;
	   assign byp_scm_segment_to_scm_mux = {(17){1'd1}};
	   assign byp_scm_segment_to_scan_channel = {(17){1'd1}};
	end
    else
    begin
       wire [TILE_SCAN_CHANNEL_WIDTH-1:0] bypass_scm_segment;

      if(TILE_SCAN_CHANNEL_WIDTH == 17)
      begin
	   assign byp_scm_segment_to_scm_mux = bypass_scm_segment[TILE_SCAN_CHANNEL_WIDTH-1:0];
	   assign byp_scm_segment_to_scan_channel = bypass_scm_segment[TILE_SCAN_CHANNEL_WIDTH-1:0];
      end
      else
	  begin
	   assign byp_scm_segment_to_scm_mux = { {(17-TILE_SCAN_CHANNEL_WIDTH){1'd1}}, bypass_scm_segment[TILE_SCAN_CHANNEL_WIDTH-1:0] };
	   assign byp_scm_segment_to_scan_channel = { {(17-TILE_SCAN_CHANNEL_WIDTH){1'd1}}, bypass_scm_segment[TILE_SCAN_CHANNEL_WIDTH-1:0] };
      end

	   // BYPASS_SCM_SEGMENT TDR
	   byp_scm_segment_tdr_gc10 
	     #(
	       .TILE_SCAN_CHANNEL_WIDTH (TILE_SCAN_CHANNEL_WIDTH)
	       ) 
	   byp_scm_segment_tdr 
	     (
	      
	      .Tdr_Tck               (Tdr_Tck) ,
	      .Tdr_Trstb             (Tdr_Trstb),
	      .Tdr_Tdi               (Tdr_Tdi),
	      .Tdr_Shift             (Tdr_Shift),
	      .Tdr_Capture           (Tdr_Capture),
	      .Tdr_Update            (Tdr_Update)
	      
	      
	      //TDR --> BYP_SCM_SEGMENT
	      ,.bypass_scm_segment            (bypass_scm_segment[TILE_SCAN_CHANNEL_WIDTH-1:0])
	      ,.Tdr_byp_scm_segment_En               (Tdr_byp_scm_segment_En)
	      ,.Tdr_byp_scm_segment_Tdo              (Tdr_byp_scm_segment_Tdo)
	      );
   
	end // else: !if(TILE_SCAN_CHANNEL_WIDTH == 0)    
   endgenerate
 
   
   // SHIFT_CNT TDR
   shift_cnt_tdr_gc10 shift_cnt_tdr 
     (
      .Tdr_Tck               (Tdr_Tck) ,
      .Tdr_Trstb             (Tdr_Trstb),
      .Tdr_Tdi               (Tdr_Tdi),
      .Tdr_Shift             (Tdr_Shift),
      .Tdr_Capture           (Tdr_Capture),
      .Tdr_Update            (Tdr_Update)
      
      
      //TDR --> SHIFT_CNT
      ,.shift_cnt             (shift_cnt[16-1:0])
      ,.Tdr_shift_cnt_En             (Tdr_shift_cnt_En)
      ,.Tdr_shift_cnt_Tdo            (Tdr_shift_cnt_Tdo)
      );
   
   
   generate
      if(GFX_HARVEST_PRESENT == 1 ) 
	begin
	   gfx_harvest_tdr_gc10 gfx_harvest_tdr 
	     (
	      .Tdr_Tck               (Tdr_Tck) ,
	      .Tdr_Trstb             (Tdr_Trstb),
	      .Tdr_Tdi               (Tdr_Tdi),
	      .Tdr_Shift             (Tdr_Shift),
	      .Tdr_Capture           (Tdr_Capture),
	      .Tdr_Update            (Tdr_Update)
	      
	      
	      //TDR --> GFX_HARVEST
	      ,.rep_fault_byp_mux_in          (rep_fault_byp_mux_in[1-1:0])
	      ,.rep_fault_byp_mux_sel                 (rep_fault_byp_mux_sel[1-1:0])
	      ,.Tdr_gfx_harvest_En           (Tdr_gfx_harvest_En)
	      ,.Tdr_gfx_harvest_Tdo          (Tdr_gfx_harvest_Tdo)
	      );
	   
	end
      else 
	begin
	   assign Tdr_gfx_harvest_Tdo = 1'b0;
	   assign rep_fault_byp_mux_in[1-1:0] = 1'b0;
	   assign rep_fault_byp_mux_sel[1-1:0] = 1'b0;
	end // else: !if(GFX_HARVEST_PRESENT == 1 )
    
   endgenerate
  


   assign Scan_Cntl_Pipe_En        =   pipe_en;
   assign Scan_Cntl_Low_Power_Edt  =   low_power_edt;
   assign Edt_Bypass               =   edt_bypass;
   assign Extest_Mode              =   extest_mode;
   assign Scan_Mode                = scan_mode;
   assign Scm_Mode                 = scm_mode;
   assign ScanEn_CP_Override       = scan_en_cp_override;
   assign ScanEn_IP_Override       = scan_en_ip_override;
   assign ScanEn_OP_Override       = scan_en_op_override;
   assign SE_Sel                   = se_sel;
   assign LOS_Sel                  = los_sel;
   assign Gater_Table_Mode         = gater_table_mode;
   assign Stuck_At_Mode            = stuck_at_mode;
   
   /////////////////////////// TDR Instances End   /////////////////////////////////////
   
    /////////////////////////// Clock Gating   /////////////////////////////////////
generate
if(NO_SHIFT_CLK == 1 ) begin

// When shift clock is sourced from func clock, add clock-gating logic.
   dfx_async_rst_sync3_gc10 rst_sync_trstb
    (
     .clk      (Shift_Clk),
     .rstinB   (Tdr_Trstb),
     .rstoutB  (trstb_sync_func_clk)    // func clock is sourced to shift clock tree.
    );

// mission mode- Tdr_Trstb=0, gate Shift_Clk(Func_Clk sourced here) always
// ATE mode- Tdr_Trstb=1 always befroe scan_mode become-1
// Hence, clock ungated when scan_mode=0. this would reset synchronous logic properly.

   ati_clock_gate Func_Clk_DfxCLOCKGATER
     (.clk_out   (gFunc_Clk),
      .clk_src   (Shift_Clk),
      .clk_en    (trstb_sync_func_clk),
      .TE        (1'b0));

end else begin
assign gFunc_Clk = Shift_Clk;
end
endgenerate

   aticlk_buf mrk_shift_clk_cts 
   ( 
     .A(gFunc_Clk),
     .Y(Shift_Clk_Raw)
     );



   // edt_clk
   assign scan_pipeline_clk_en = scan_mode ? Tile_ScanEn : 1'b1; // clock would be gated at root.
   ati_clock_gate Scan_Pipeline_Edt_DfxCLOCKGATER 
     (
      .clk_out(gShift_Clk),
      .clk_src(Shift_Clk_Raw),
      .clk_en(scan_pipeline_clk_en),
      .TE(1'b0));
   
   ati_clock_gate Scan_Pipeline_DfxCLOCKGATER 
     (
      .clk_out(gShift_Clk_1),
      .clk_src(gShift_Clk),
      .clk_en(~Edt_Update), 
      .TE(1'b0));
   
   
   aticlk_buf mrk_Scan_Pipeline_Clk_src 
     ( 
       .A(gShift_Clk_1),
       .Y(Scan_Pipeline_Clk)
       );
   
   
   ati_clock_gate Edt_DfxCLOCKGATER 
     (
      .clk_out(gEdt_Clk),
      .clk_src(gShift_Clk),
      .clk_en(~shift_clk_stop),
      .TE(1'b0));
   
   
   aticlk_buf mrk_Edt_Clk_Src 
     ( 
       .A(gEdt_Clk),
       .Y(Edt_Clk)
       );
   
   //Tile_Shift_Clk
   ati_clock_gate Shift_DfxCLOCKGATER 
     (
      .clk_out(gTile_Shift_Clk_pre),
      .clk_src(Shift_Clk_Raw),
      .clk_en(~Edt_Update), 
      .TE(1'b0));
   
   ati_clock_gate Tile_Shift_DfxCLOCKGATER 
     (
      .clk_out(gTile_Shift_Clk),
      .clk_src(gTile_Shift_Clk_pre),
      .clk_en(~shift_clk_stop), 
      .TE(1'b0));
   
   aticlk_buf mrk_Tile_Shift_Clk 
     ( 
       .A(gTile_Shift_Clk),
       .Y(Tile_Shift_Clk)
       );
   
   /////////////////////////// Scan_Mode Sync   /////////////////////////////////////
   // Note: need few clock pulsed before scan shift starts to flush out the synchronizer.
       // Enable scan mode first, then few shift clock pulses to reset the flop
       
       dfx_async_rst_sync3_gc10 rst_sync_scan_mode
	 (
	  .clk      (Shift_Clk_Raw),
	  .rstinB   (scan_mode),
	  .rstoutB  (scan_mode_sync)
	  );
   
   
   
   /////////////////////////// ScanEn Sync   /////////////////////////////////////
   //async set, sync de-assertion
   // Note: Give few clock pulsed to flush out the sync before starting scan shift.

   //Mission mode: ScanEn=0, 
   dfx_async_set_sync3_gc10 rst_sync_ScanEn 
     (
      .clk(Shift_Clk_Raw),
      .rstinB(~ScanEn),         //False path
      .rstout(ScanEn_Sync));  //Real path
   
   ati_mux2 mux_ScanEn  
     (
      .i0(ScanEn_Sync),
      .i1(ScanEn),       // ScanEn from TAP FSM in scm mode.
      .s(scm_mode | ~se_sel),
      .z(ScanEn_Muxed_pre));
   
   
   // TO keep scan-en 0 during mission mode.
   dfx_mrk_and_gc10 d0nt_SE_and_SM 
     ( 
       .A(scan_mode),          //CDC waiver
       .B(ScanEn_Muxed_pre), 
       .Y(ScanEn_Muxed) );     
   
   
   /////////////////////////// Gater Flops   /////////////////////////////////////
       
   //MCM= out of reset after sync output become 1.
   //SCM= out of reset as soon as scm_mode TDR is 1. no additional shift clock pulses after TDR set to 1 and shift starts.
   //assign gater_ff_async_rstb = scan_mode_sync | scm_mode;
   //Insted of above, using non-sync scan_mode.

   assign gater_ff_async_rstb = scan_mode;  // CDC waiver
   //Bug:568312
   reg 	  te_gater_ff;
   always @ (posedge Tile_Shift_Clk or negedge gater_ff_async_rstb)
     begin
	if(~gater_ff_async_rstb)
	  te_gater_ff <= 1'b0;
	else if(Tile_ScanEn)   // TBD: LOS mode, SE de-assert one cycle early
	  te_gater_ff <= Gater_Flop_ScanIn;
	else
	  te_gater_ff <= te_gater_ff;
     end
   
   reg async_rst_gater_ff;
   reg async_rst_gater_ff_sel;
   always @ (posedge Tile_Shift_Clk or negedge gater_ff_async_rstb)
    begin
	if(~gater_ff_async_rstb) begin
	  async_rst_gater_ff <= 1'b0;
	  async_rst_gater_ff_sel <= 1'b0;
	end else if(Tile_ScanEn) begin   // TBD: LOS mode, SE de-assert one cycle early
	  async_rst_gater_ff <= te_gater_ff;
	  async_rst_gater_ff_sel <= async_rst_gater_ff;
	end else begin
	  async_rst_gater_ff <= async_rst_gater_ff;
	  async_rst_gater_ff_sel <= async_rst_gater_ff_sel;
    end 
    end
 
   assign Gater_Flop_ScanOut = async_rst_gater_ff_sel;
   
   
   /////////////////////////// TEST_CLK_EN    /////////////////////////////////////
   
   ati_mux2 mux_test_clk_en 
    (
     .i0(test_clk_en),    //Func mode
     .i1(Tile_ScanEn_Neg | te_gater_ff),
     .s(test_clk_en_sel),
     .z(TEST_CLK_EN));

    //For logic which is only DC scannable, not AC scannable.
     assign DC_TEST_CLK_EN = TEST_CLK_EN & Stuck_At_Mode;

   
   /////////////////////////// Async_Rstb_Scan_Override    /////////////////////////////////////
   //Keep it 0 in functional mode.
   //Keep it 1 in scm_mode,else flops get corrupted. Make sure to assert scm_mode after clock is stopped.
   //assign async_rstb_override_pre = (async_rstb_override_pre_1 & scan_mode) | scm_mode;
   //ati_buffer1 mrk_Async_Rstb_Scan_Override (.A(async_rstb_override_pre), .Y(Async_Rstb_Scan_Override) );

    //Eliminate 0,0 condition which could create glitch on RSTb when SE 0->1 transition happen.
    //When async_rst_gater_ff_sel = 0, keep async_rstb_override to 1 to have clean SE 0->1 transition when flops are out of reset after capture.
   assign async_rstb_override_pre = ( (Tile_ScanEn_Neg | async_rst_gater_ff | ~async_rst_gater_ff_sel) & scan_mode) | scm_mode;
   ati_buffer1 mrk_Async_Rstb_Scan_Override (.A(async_rstb_override_pre), .Y(Async_Rstb_Scan_Override) );

   assign async_rstb_override_sel_pre = ( (Tile_ScanEn_Neg | async_rst_gater_ff_sel) & scan_mode) | scm_mode;
   ati_buffer1 mrk_Async_Rstb_Scan_Override_Sel (.A(async_rstb_override_sel_pre), .Y(Async_Rstb_Scan_Override_Sel) );
   
   
   /////////////////////////// Scm_Async_Reset_Pulse    /////////////////////////////////////
   reg 	  scan_mode_p1;
   reg 	  scan_mode_p2;
   always @ (negedge Tdr_Tck)
     begin
	scan_mode_p1 <= scan_mode;
	scan_mode_p2 <= scan_mode_p1;
     end
   
   dfx_mrk_and_gc10 d0nt_scm_async_reset_pulse 
     (
      .A(scan_mode),          //CDC waiver
      .B(~scan_mode_p2),
      .Y(scan_mode_pulse) );
   
   //Func_mode - 0
   //Entering in scan_mode - 1.
   //AC-scan-shift/capture - 0
   //DC-Scan-shift mode  - 0
   //DC-Scan-capture mode - ~async_rst_gater_ff
   
   assign Scm_Async_Reset_Pulse =  scan_mode  & 
          ( scan_mode_pulse | ~Async_Rstb_Scan_Override );
   
   /////////////////////////// Tile_DC_ScanEn    /////////////////////////////////////
   //Bug: 568314
   assign Tile_DC_ScanEn = Tile_ScanEn_Neg & Stuck_At_Mode;

   /////////////////////////// Tile_GaterEn    /////////////////////////////////////

    // Disable TE pin during Edt. The E pin of gater need to be disabled as well during Edt_udpate stage.
   assign Tile_GaterEn  =  Tile_ScanEn_Neg & ~Edt_Update;   // Connect to CGCG TE pin.

   /////////////////////////// Instances   /////////////////////////////////////
    // edt_update
    gen_edt_update_gc10 gen_edt_update 
	  (
	   .ScanEn             (ScanEn_Muxed),
	   .Shift_Clk          (Shift_Clk_Raw),  // Freerunning Shift clock. 
	   .IO_Edt_Update      (IO_Edt_Update),
	   .edt_update_sel     (edt_update_sel),
	   .Edt_Update         (Edt_Update),
	   .scan_mode          (scan_mode),
	   .scm_mode           (scm_mode)
	   
	   );
   
   gen_scan_en_gc10 gen_scan_en 
     (
      .ScanEn            (ScanEn_Muxed)
      ,.cap_disable      (cap_disable[2-1:0])
      ,.rstb             (scan_mode_sync)
      ,.Shift_Clk        (gTile_Shift_Clk_pre)
      ,.Tile_ScanEn      (Tile_ScanEn)
      ,.Tile_ScanEn_Neg  (Tile_ScanEn_Neg)
      ,.scan_mode        (scan_mode)
      ,.SE_Sel           (SE_Sel)
      ,.shift_cnt_tdr    (shift_cnt[16-1:0])
      );
   
   
   
assign  Tile_ScanChannelIn[0] = Tile_ScanChannelIn_0;
assign  Tile_ScanChannelOut_0 = Tile_ScanChannelOut[0];
assign ScanChannelFromEdt[0] = ScanChannelFromEdt_0;
assign ScanChannelToEdt_0    = ScanChannelToEdt[0];
assign ScanChannelMask_0     = byp_scm_segment_to_scan_channel[0] & ~scm_mode;
assign  Tile_ScanChannelIn[1] = Tile_ScanChannelIn_1;
assign  Tile_ScanChannelOut_1 = Tile_ScanChannelOut[1];
assign ScanChannelFromEdt[1] = ScanChannelFromEdt_1;
assign ScanChannelToEdt_1    = ScanChannelToEdt[1];
assign ScanChannelMask_1     = byp_scm_segment_to_scan_channel[1] & ~scm_mode;
assign  Tile_ScanChannelIn[2] = Tile_ScanChannelIn_2;
assign  Tile_ScanChannelOut_2 = Tile_ScanChannelOut[2];
assign ScanChannelFromEdt[2] = ScanChannelFromEdt_2;
assign ScanChannelToEdt_2    = ScanChannelToEdt[2];
assign ScanChannelMask_2     = byp_scm_segment_to_scan_channel[2] & ~scm_mode;
assign  Tile_ScanChannelIn[3] = Tile_ScanChannelIn_3;
assign  Tile_ScanChannelOut_3 = Tile_ScanChannelOut[3];
assign ScanChannelFromEdt[3] = ScanChannelFromEdt_3;
assign ScanChannelToEdt_3    = ScanChannelToEdt[3];
assign ScanChannelMask_3     = byp_scm_segment_to_scan_channel[3] & ~scm_mode;
assign  Tile_ScanChannelIn[4] = Tile_ScanChannelIn_4;
assign  Tile_ScanChannelOut_4 = Tile_ScanChannelOut[4];
assign ScanChannelFromEdt[4] = ScanChannelFromEdt_4;
assign ScanChannelToEdt_4    = ScanChannelToEdt[4];
assign ScanChannelMask_4     = byp_scm_segment_to_scan_channel[4] & ~scm_mode;
assign  Tile_ScanChannelIn[5] = Tile_ScanChannelIn_5;
assign  Tile_ScanChannelOut_5 = Tile_ScanChannelOut[5];
assign ScanChannelFromEdt[5] = ScanChannelFromEdt_5;
assign ScanChannelToEdt_5    = ScanChannelToEdt[5];
assign ScanChannelMask_5     = byp_scm_segment_to_scan_channel[5] & ~scm_mode;
assign  Tile_ScanChannelIn[6] = Tile_ScanChannelIn_6;
assign  Tile_ScanChannelOut_6 = Tile_ScanChannelOut[6];
assign ScanChannelFromEdt[6] = ScanChannelFromEdt_6;
assign ScanChannelToEdt_6    = ScanChannelToEdt[6];
assign ScanChannelMask_6     = byp_scm_segment_to_scan_channel[6] & ~scm_mode;
assign  Tile_ScanChannelIn[7] = Tile_ScanChannelIn_7;
assign  Tile_ScanChannelOut_7 = Tile_ScanChannelOut[7];
assign ScanChannelFromEdt[7] = ScanChannelFromEdt_7;
assign ScanChannelToEdt_7    = ScanChannelToEdt[7];
assign ScanChannelMask_7     = byp_scm_segment_to_scan_channel[7] & ~scm_mode;
assign  Tile_ScanChannelIn[8] = Tile_ScanChannelIn_8;
assign  Tile_ScanChannelOut_8 = Tile_ScanChannelOut[8];
assign ScanChannelFromEdt[8] = ScanChannelFromEdt_8;
assign ScanChannelToEdt_8    = ScanChannelToEdt[8];
assign ScanChannelMask_8     = byp_scm_segment_to_scan_channel[8] & ~scm_mode;
assign  Tile_ScanChannelIn[9] = Tile_ScanChannelIn_9;
assign  Tile_ScanChannelOut_9 = Tile_ScanChannelOut[9];
assign ScanChannelFromEdt[9] = ScanChannelFromEdt_9;
assign ScanChannelToEdt_9    = ScanChannelToEdt[9];
assign ScanChannelMask_9     = byp_scm_segment_to_scan_channel[9] & ~scm_mode;
assign  Tile_ScanChannelIn[10] = Tile_ScanChannelIn_10;
assign  Tile_ScanChannelOut_10 = Tile_ScanChannelOut[10];
assign ScanChannelFromEdt[10] = ScanChannelFromEdt_10;
assign ScanChannelToEdt_10    = ScanChannelToEdt[10];
assign ScanChannelMask_10     = byp_scm_segment_to_scan_channel[10] & ~scm_mode;
assign  Tile_ScanChannelIn[11] = Tile_ScanChannelIn_11;
assign  Tile_ScanChannelOut_11 = Tile_ScanChannelOut[11];
assign ScanChannelFromEdt[11] = ScanChannelFromEdt_11;
assign ScanChannelToEdt_11    = ScanChannelToEdt[11];
assign ScanChannelMask_11     = byp_scm_segment_to_scan_channel[11] & ~scm_mode;
assign  Tile_ScanChannelIn[12] = Tile_ScanChannelIn_12;
assign  Tile_ScanChannelOut_12 = Tile_ScanChannelOut[12];
assign ScanChannelFromEdt[12] = ScanChannelFromEdt_12;
assign ScanChannelToEdt_12    = ScanChannelToEdt[12];
assign ScanChannelMask_12     = byp_scm_segment_to_scan_channel[12] & ~scm_mode;
assign  Tile_ScanChannelIn[13] = Tile_ScanChannelIn_13;
assign  Tile_ScanChannelOut_13 = Tile_ScanChannelOut[13];
assign ScanChannelFromEdt[13] = ScanChannelFromEdt_13;
assign ScanChannelToEdt_13    = ScanChannelToEdt[13];
assign ScanChannelMask_13     = byp_scm_segment_to_scan_channel[13] & ~scm_mode;
assign  Tile_ScanChannelIn[14] = Tile_ScanChannelIn_14;
assign  Tile_ScanChannelOut_14 = Tile_ScanChannelOut[14];
assign ScanChannelFromEdt[14] = ScanChannelFromEdt_14;
assign ScanChannelToEdt_14    = ScanChannelToEdt[14];
assign ScanChannelMask_14     = byp_scm_segment_to_scan_channel[14] & ~scm_mode;
assign  Tile_ScanChannelIn[15] = Tile_ScanChannelIn_15;
assign  Tile_ScanChannelOut_15 = Tile_ScanChannelOut[15];
assign ScanChannelFromEdt[15] = ScanChannelFromEdt_15;
assign ScanChannelToEdt_15    = ScanChannelToEdt[15];
assign ScanChannelMask_15     = byp_scm_segment_to_scan_channel[15] & ~scm_mode;
assign  Tile_ScanChannelIn[16] = Tile_ScanChannelIn_16;
assign  Tile_ScanChannelOut_16 = Tile_ScanChannelOut[16];
assign ScanChannelFromEdt[16] = ScanChannelFromEdt_16;
assign ScanChannelToEdt_16    = ScanChannelToEdt[16];
assign ScanChannelMask_16     = byp_scm_segment_to_scan_channel[16] & ~scm_mode;

 
   // 0 when any Power domain is down. 
   assign  sleep_en = PGFSM_P0_resetb &
       PGFSM_P1_resetb &
       PGFSM_P2_resetb &
       PGFSM_P3_resetb &
       PGFSM_P4_resetb &
       PGFSM_P5_resetb &
       PGFSM_P6_resetb &
       PGFSM_P7_resetb;
   
   wire    autobypass_scm_segment = 1'b0;//TBD
   
   scm_mux_gc10 scm_mux 
     (
      .Tile_ScanChannelIn   (Tile_ScanChannelIn[17-1:0])
      ,.Tile_ScanChannelOut  (Tile_ScanChannelOut[17-1:0])
      ,.ScanChannelFromEdt   (ScanChannelFromEdt[17-1:0])
      ,.ScanChannelToEdt     (ScanChannelToEdt[17-1:0])
      
      ,.bypass_scm_segment   (byp_scm_segment_to_scm_mux)
      ,.Single_Chain_Daisy_In (Single_Chain_Daisy_In)
      ,.Single_Chain_Daisy_Out (Single_Chain_Daisy_Out)
      ,.scm_mode               (scm_mode)
      ,.autobypass_scm_segment (autobypass_scm_segment) 
      );

   `include "scan_cntl_assertions_gc10.vh"
   
endmodule
