//                              -*- Mode: Verilog -*-

////////////////////////////////////////////////////////////////////////////
/*
 *
 *  Trade secret of ATI Technologies, Inc.
 *  Copyright 2005, ATI Technologies, Inc., (unpublished)
 *
 *  All rights reserved.  This notice is intended as a precaution against
 *  inadvertent publication and does not imply publication or any waiver
 *  of confidentiality.  The year included in the foregoing notice is the
 *  year of creation of the work.
 *
 */
////////////////////////////////////////////////////////////////////////////

//`include "scan_cntl_features.vh"


module scm_mux_gc10
(
input  wire [17-1:0] Tile_ScanChannelIn,
output wire [17-1:0] Tile_ScanChannelOut,

input  wire [17-1:0] ScanChannelFromEdt,
output wire [17-1:0] ScanChannelToEdt,

input  wire [17-1:0] bypass_scm_segment,
input  Single_Chain_Daisy_In,
output Single_Chain_Daisy_Out,
input scm_mode,
input autobypass_scm_segment

);

wire scm_si, scm_so;
   ati_buffer1 mrk_scm_daisy_in
   (
     .A(Single_Chain_Daisy_In),
     .Y(scm_si)
   );

//   ati_buffer1 mrk_scm_daisy_out
//   (
//     .A(scm_so),
//     .Y(Single_Chain_Daisy_Out)
//   );

 
// Added for secure tile. When tile is secure, scm_mode can't be programmed. In that case, bypass single chain of that tile.
   ati_mux2 mrk_byp_tile_scm (
   .i0(scm_si), // When scm_mode TDR is 0, bypass single chain.
   .i1(scm_so),
   .s(scm_mode),
   .z(Single_Chain_Daisy_Out)
   );


  // bug 566143. ANDed with scm_mode to use byp_scm_segment TDR for other feature in MCM mode.
  wire [17-1:0] bypass_scm_segment_ored =  ( bypass_scm_segment[17-1:0]  | 
                                                                 {(17){autobypass_scm_segment}} ) & 
                                                                 {(17){scm_mode}};


   //Single chain Muxing Logic (SCM)
   wire [17-1:0] ScanChannelInToSCM;
   wire [17-1:0] ScanChannelOutSCM;
   wire [17-1:0] ScanChannelInSCMOut;
   wire [17-1:0] ScanChannelInSCM0Out;


  // ScanChannelOutSCM[17-1] buffer output is not connected.
   ati_buffer1 mrk_tile_scan_channel_out[17-1:0]
   (
     .A(ScanChannelFromEdt[17-1:0]),
     .Y(Tile_ScanChannelOut[17-1:0])
   );

  //From EDT block output.
  // ScanChannelOutSCM[17-1] buffer output is not connected.
   ati_buffer1 mrk_tile_scan_channel_out_from_edt[17-1:0]
   (
     .A(ScanChannelFromEdt[17-1:0]),
     .Y(ScanChannelOutSCM[17-1:0])
   );


   //marker buffers for ScanChannelIn hookup
   ati_buffer1 mrk_tile_scan_channel_in_to_tile[17-1:0]
   (.A(Tile_ScanChannelIn[17-1:0]),
   .Y(ScanChannelInToSCM[17-1:0]));

   //mux0 SCM mode mux to select between ScanChannelIn and ScanChannelOut from previous chain
   ati_mux2 tile_scm_mux0 [17-1:0] (
   .i0(ScanChannelInToSCM[17-1:0]),     // From SRF. Tile Scan Channel Inputs.
   .i1({ScanChannelOutSCM[17-2:0],scm_si}), // Edt output channels. 
                                                            //SCM mode, connect edt_channel output to next edt_chain input
   .s(scm_mode),
   .z(ScanChannelInSCM0Out[17-1:0])
   );

   wire scanin_0;
   ati_mux2 mrk_scanin_0 (
   .i0(ScanChannelInSCM0Out[0]),
   .i1(scm_si),
   .s(autobypass_scm_segment),
   .z(scanin_0)
   );


   //mux1 SCM segment bypass
   ati_mux2 tile_scm_mux1 [17-1:1] (
   .i0(ScanChannelInSCM0Out[17-1:1]),   // From Above mux which select EDT (scm) or I/O channel(mcm)
   .i1({ScanChannelInSCMOut[17-2:1],scanin_0}), //From previous mux output.
   .s(bypass_scm_segment_ored[17-2:0]), // From BYP_SCM_SEGMENT TDR or or autobypass & SCM_MODE.
                                                            // In MCM mode, select mux-leg-0
   .z(ScanChannelInSCMOut[17-1:1]));



   //Final SCM mux
   ati_mux2 tile_scm_final_mux  (
   .i0(ScanChannelOutSCM[17-1]),    //last EDT output Channel.
   .i1(ScanChannelInSCMOut[17-1]),
   .s(bypass_scm_segment_ored[17-1]),
   .z(scm_so));

    //ScanChannelInSCMOut[0] not connected
   ati_buffer1 mrk_tile_scan_channel_in_to_edt[17-1:0]
   (.A({ScanChannelInSCMOut[17-1:1],ScanChannelInSCM0Out[0]}),
   .Y(ScanChannelToEdt[17-1:0]));


endmodule

