/*---------------------------------------------------------------------*/
/*               Copyright(c) Synopsys, Inc.                           */
/*     All Rights reserved - Unpublished -rights reserved under        */
/*     the Copyright laws of the United States of America.             */
/*                                                                     */
/*  U.S. Patents: 7,093,156 B1 and 7,406,620 B2 (and more pending).    */
/*                                                                     */
/*  This file includes the Confidential information of Synopsys, Inc.. */
/*  The receiver of this Confidential Information shall not disclose   */
/*  it to any third party and shall protect its confidentiality by     */
/*  using the same degree of care, but not less then a reasonable      */
/*  degree of care, as the receiver uses to protect receiver's own     */
/*  Confidential Information.                                          */
/*  Licensee acknowledges and agrees that all output generated for     */
/*  Licensee by Synopsys, Inc. as described in the pertinent Program   */
/*  Schedule(s), or generated by Licensee through use of any Compiler  */
/*  licensed hereunder contains information that complies with the     */
/*  Virtual Component Identification Physical Tagging Standard (VCID)  */
/*  as maintained by the Virtual Socket Interface Alliance (VSIA).     */
/*  Such information may be expressed in GDSII Layer 63 or other such  */
/*  layer designated by the VSIA, hardware definition languages, or    */
/*  other formats.  Licensee is not authorized to alter or change any  */
/*  such information.                                                  */
/*---------------------------------------------------------------------*/
/*                                                                     */
/*  Built for linux64 and running on linux64.                          */
/*                                                                     */
/*  Software           : Rev: M-2017.03                                */
/*  Library Format     : Rev: 1.05.00                                  */
/*  Compiler Name      : vl00x000vpnnstsrv000sa31                      */
/*  Platform           : Linux2.6.32-573.el6.x86_64                    */
/*                     : #1 SMP Wed Jul 1 18:23:37 EDT 2015x86_64      */
/*  Date of Generation : Wed Oct 11 01:17:45 EDT 2017                  */
/*                                                                     */
/*---------------------------------------------------------------------*/
`resetall
// T I M E S C A L E ===============================================================================
`ifndef SMS_NO_TIMESCALE
    `timescale 1ns/1ps
`endif
//Driver for AMD random access parallel e-fuse
module smsg_gc13_rp_efuse_drv(
                                clk_sfbp,
                                rst_sfbp,
                                efread,
                                efcompare,
                                ack_ef,
                                dout_ef,
                                data_in,
                                smart_flow,
                                driver_on,
                                cycling,
                                data_shift_en,
                                req_ef,
                                addr_ef,
                                compare_err,
                                early_abort
                                );

input clk_sfbp;
input rst_sfbp;
input efread;
input efcompare;
input ack_ef;
input dout_ef;
input data_in;
input smart_flow;

output driver_on;
output cycling;
output data_shift_en;
output req_ef;
output [9:0]addr_ef;
output compare_err;
output early_abort;

//Parameters for driver state machine
parameter RESET       = 4'd0;
parameter READ        = 4'd1;
parameter FIRSTRCYCLE = 4'd2;
parameter R_DUMMY     = 4'd3;
parameter CAPTURE     = 4'd4;
parameter ADDRINCR    = 4'd5;
parameter WAIT_ACK    = 4'd6;
parameter COMPARE     = 4'd7;
parameter CHECK_1BIT  = 4'd8;

//Internal variables
reg driver_on_r,driver_on_nxt;
reg req_ef_r, req_ef_nxt;
reg compare_err_r,compare_err_nxt;
reg data_en_r,data_en_nxt;
reg cycling_r,cycling_nxt;
reg [3:0]state_r,state_nxt;
reg [9:0]bit_counter_r,bit_counter_nxt;
reg early_abort_r,early_abort_nxt;
reg early_abort_en_r,early_abort_en_nxt;
reg toggle_buf_r;
wire byte_over;
wire sms_read_on;
wire detect_toggle;
wire ready_ef_synch;

//Assignment
assign driver_on = driver_on_r;
assign sms_read_on = (bit_counter_r != 10'd143);
assign early_abort = early_abort_r;
assign req_ef = req_ef_r;
assign addr_ef = bit_counter_r;
assign data_shift_en = data_en_r;
assign cycling = cycling_r;
assign compare_err = compare_err_r;
assign byte_over = (bit_counter_r[2:0] == 3'b111);
assign detect_toggle = toggle_buf_r ^ ready_ef_synch;

srv_async_meta U_smsg_gc13_rp_ready (
             .clk_in     (clk_sfbp),
             .rst_in     (rst_sfbp),
             .signal_in  (ack_ef),
             .signal_out (ready_ef_synch)
);

//Driver state machine
always @ (state_r or efread or efcompare or sms_read_on or byte_over
          or early_abort_en_r or dout_ef or detect_toggle)
  begin
    case(state_r)
      RESET           : begin
                          case({efread,efcompare})
                            2'b10     : state_nxt = READ;
                            2'b01     : state_nxt = COMPARE;
                            default   : state_nxt = RESET;
                          endcase
                        end
      READ            : state_nxt = FIRSTRCYCLE;
      FIRSTRCYCLE     : state_nxt = WAIT_ACK;
      WAIT_ACK        : if (detect_toggle)
                          state_nxt = R_DUMMY;
                        else
                          state_nxt = WAIT_ACK;
      R_DUMMY         : if(early_abort_en_r)
                          state_nxt = CHECK_1BIT;
                        else
                          state_nxt = CAPTURE;
      CHECK_1BIT      : if(dout_ef)
                          state_nxt = CAPTURE;
                        else
                          state_nxt = RESET;
      CAPTURE         : if(sms_read_on)
                          state_nxt = ADDRINCR;
                        else
                          state_nxt = RESET;
      ADDRINCR        : if(byte_over)
                          state_nxt = FIRSTRCYCLE;
                        else
                          state_nxt = CAPTURE;

      COMPARE         : state_nxt = FIRSTRCYCLE;

      default         : state_nxt = FIRSTRCYCLE;
    endcase
  end

//Enable early abort
always @ (state_r or smart_flow or dout_ef or early_abort_en_r)
  if((state_r == RESET) | ((state_r == CHECK_1BIT) & dout_ef))
    early_abort_en_nxt = 1'b0;
  else if(state_r == READ)
    early_abort_en_nxt = smart_flow;
  else
    early_abort_en_nxt = early_abort_en_r;

always @ (state_r or dout_ef or early_abort_r)
  if(state_r == READ)
    early_abort_nxt = 1'b0;
  else if((state_r == CHECK_1BIT) & !dout_ef)
    early_abort_nxt = 1'b1;
  else
    early_abort_nxt = early_abort_r;

//E-fuse control   req_ef_nxt
always @ (state_r or req_ef_r)
  begin
    case(state_r)
      FIRSTRCYCLE     : req_ef_nxt = ~req_ef_r;
      default         : req_ef_nxt = req_ef_r;
    endcase
  end

//Data register control
always @ (state_r or cycling_r)
  begin
    case(state_r)
      RESET           : {cycling_nxt,data_en_nxt} = 2'b00;
      COMPARE         : {cycling_nxt,data_en_nxt} = 2'b10;
      CAPTURE         : {cycling_nxt,data_en_nxt} = {cycling_r,1'b1};
      default         : {cycling_nxt,data_en_nxt} = {cycling_r,1'b0};
    endcase
  end

//Bit counter control
always @ (state_r or bit_counter_r)
  begin
    case(state_r)
      RESET           : bit_counter_nxt = 10'd0;
      ADDRINCR        : bit_counter_nxt = bit_counter_r + 10'd1;
      default         : bit_counter_nxt = bit_counter_r;
    endcase
  end

//E-fuse verification control
always @ (state_r or dout_ef or data_in or compare_err_r
          )
  begin
    case(state_r)
      COMPARE         : compare_err_nxt = 1'b0;
      CAPTURE         : compare_err_nxt = compare_err_r | (dout_ef^data_in);
      default         : compare_err_nxt = compare_err_r;
    endcase
  end

//Readiness control
always @ (state_r or efread or efcompare)
  if((state_r != RESET) | efread | efcompare
      )
    driver_on_nxt = 1'b1;
  else
    driver_on_nxt = 1'b0;

//Non-blocking assignment
always @ (posedge clk_sfbp or negedge rst_sfbp)
  if(!rst_sfbp)
    begin
      state_r                               <=  RESET;
      data_en_r                             <=  1'b0;
      cycling_r                             <=  1'b0;
      compare_err_r                         <=  1'b0;
      driver_on_r                           <=  1'b0;
      bit_counter_r                         <=  10'd0;
      req_ef_r                              <=  1'b0;
      {early_abort_r,early_abort_en_r}      <=  2'd0;
      toggle_buf_r                          <=  1'b0;
    end
  else
    begin
      state_r                               <=  state_nxt;
      data_en_r                             <=  data_en_nxt;
      cycling_r                             <=  cycling_nxt;
      compare_err_r                         <=  compare_err_nxt;
      driver_on_r                           <=  driver_on_nxt;
      bit_counter_r                         <=  bit_counter_nxt;
      req_ef_r                              <=  req_ef_nxt;
      {early_abort_r,early_abort_en_r}      <=  {early_abort_nxt,early_abort_en_nxt};
      toggle_buf_r                          <=  ready_ef_synch;
    end

endmodule
