/*---------------------------------------------------------------------*/
/*               Copyright(c) Synopsys, Inc.                           */
/*     All Rights reserved - Unpublished -rights reserved under        */
/*     the Copyright laws of the United States of America.             */
/*                                                                     */
/*  U.S. Patents: 7,093,156 B1 and 7,406,620 B2 (and more pending).    */
/*                                                                     */
/*  This file includes the Confidential information of Synopsys, Inc.. */
/*  The receiver of this Confidential Information shall not disclose   */
/*  it to any third party and shall protect its confidentiality by     */
/*  using the same degree of care, but not less then a reasonable      */
/*  degree of care, as the receiver uses to protect receiver's own     */
/*  Confidential Information.                                          */
/*  Licensee acknowledges and agrees that all output generated for     */
/*  Licensee by Synopsys, Inc. as described in the pertinent Program   */
/*  Schedule(s), or generated by Licensee through use of any Compiler  */
/*  licensed hereunder contains information that complies with the     */
/*  Virtual Component Identification Physical Tagging Standard (VCID)  */
/*  as maintained by the Virtual Socket Interface Alliance (VSIA).     */
/*  Such information may be expressed in GDSII Layer 63 or other such  */
/*  layer designated by the VSIA, hardware definition languages, or    */
/*  other formats.  Licensee is not authorized to alter or change any  */
/*  such information.                                                  */
/*---------------------------------------------------------------------*/
/*                                                                     */
/*  Built for linux64 and running on linux64.                          */
/*                                                                     */
/*  Software           : Rev: M-2017.03                                */
/*  Library Format     : Rev: 1.05.00                                  */
/*  Compiler Name      : vl00x000vpnnstsrv000sa31                      */
/*  Platform           : Linux2.6.32-573.el6.x86_64                    */
/*                     : #1 SMP Wed Jul 1 18:23:37 EDT 2015x86_64      */
/*  Date of Generation : Sun Dec 24 00:24:05 EST 2017                  */
/*                                                                     */
/*---------------------------------------------------------------------*/
`resetall
// T I M E S C A L E ===============================================================================
`ifndef SMS_NO_TIMESCALE
    `timescale 1ns/1ps
`endif

module smsg_gc7_net_send_engine (
                                    clk_sfbp,
                                    rst_sfbp,
                                    instr2net,
                                    net_send,
                                    power_off,
                                    last_sms,
                                    fast_load,
                                    curr_sms_bypass_ctrl,
                                    decr_sms_cntr,
                                    ld_sms_cntr,
                                    data2sms,                  
                                    instr_ready,
                                    shift_instr
);
input clk_sfbp;
input rst_sfbp;
input [5:0]instr2net;
input net_send;
input power_off;
input last_sms;
input  fast_load;
input  curr_sms_bypass_ctrl;
output decr_sms_cntr;
output ld_sms_cntr;
output data2sms;
output instr_ready;
output shift_instr;

parameter RESET   = 3'd0;
parameter SEND1   = 3'd1;
parameter SEND2   = 3'd2;
parameter SEND3   = 3'd3;
parameter SEND4   = 3'd4;
parameter SEND5   = 3'd5;
parameter CAPTURE = 3'd6;
parameter DUMMY   = 3'd7;

reg last_sms_r;
reg [2:0] state_r,state_nxt;
reg [5:0] instr_r,instr_nxt;
reg  shift_instr_r,shift_instr_nxt;

assign data2sms             = instr_r[5] & (fast_load ? !curr_sms_bypass_ctrl : 1'b1);
assign instr_ready          = (state_r != RESET) | net_send;
assign shift_instr          = shift_instr_r;
assign ld_sms_cntr          = fast_load & (state_nxt == DUMMY);
assign decr_sms_cntr        = fast_load & (state_nxt == CAPTURE) & (state_r != RESET) & (state_r != DUMMY);

//Net send state machine
always @ (state_r or net_send or power_off 
         or last_sms_r
         or fast_load
         )
  begin
    case(state_r)
      RESET       : if(net_send)
                      state_nxt = CAPTURE;
                    else
                      state_nxt = RESET;
      CAPTURE     : if (power_off)
                      state_nxt = DUMMY;
                  else
                      state_nxt = SEND5;
      DUMMY       : if (last_sms_r)
                      state_nxt = RESET;
                    else
                      state_nxt = CAPTURE;
      SEND5       : state_nxt = SEND4;
      SEND4       : state_nxt = SEND3;
      SEND3       : state_nxt = SEND2;
      SEND2       : state_nxt = SEND1;
      SEND1       : if (fast_load & !last_sms_r)
                      state_nxt = CAPTURE;
                    else            
                      state_nxt = RESET;
      default     : state_nxt = RESET;
    endcase
  end

//Capture instruction, shift instruction
always @ (state_r or instr2net or instr_r
          or fast_load
          )
  begin
    case(state_r)
      CAPTURE     : instr_nxt = instr2net;
      SEND1,
      SEND2,
      SEND3,
      SEND4,
      SEND5       : instr_nxt = {instr_r[4:0], fast_load ? instr_r[5] : 1'b0};
      default     : instr_nxt = 6'd0;
    endcase
  end

//Shift instruction
always @ (state_r)
  begin
    case(state_r)
      CAPTURE,
      SEND1,
      SEND2,
      SEND3,
      SEND4,
      SEND5       : shift_instr_nxt = 1'b1;
      default     : shift_instr_nxt = 1'b0;
    endcase
  end

//Non-blocking assignment
always @ (posedge clk_sfbp or negedge rst_sfbp)
  if(!rst_sfbp)
    begin
      state_r               <=  3'd0;
      instr_r               <=  6'd0;
      shift_instr_r         <=  1'b0;
      last_sms_r            <=  1'b0;
    end
  else
    begin
      state_r               <=  state_nxt;
      instr_r               <=  instr_nxt;
      shift_instr_r         <=  shift_instr_nxt;
      last_sms_r            <=  last_sms;
    end

endmodule

