/*---------------------------------------------------------------------*/
/*               Copyright(c) Synopsys, Inc.                           */
/*     All Rights reserved - Unpublished -rights reserved under        */
/*     the Copyright laws of the United States of America.             */
/*                                                                     */
/*  U.S. Patents: 7,093,156 B1 and 7,406,620 B2 (and more pending).    */
/*                                                                     */
/*  This file includes the Confidential information of Synopsys, Inc.. */
/*  The receiver of this Confidential Information shall not disclose   */
/*  it to any third party and shall protect its confidentiality by     */
/*  using the same degree of care, but not less then a reasonable      */
/*  degree of care, as the receiver uses to protect receiver's own     */
/*  Confidential Information.                                          */
/*  Licensee acknowledges and agrees that all output generated for     */
/*  Licensee by Synopsys, Inc. as described in the pertinent Program   */
/*  Schedule(s), or generated by Licensee through use of any Compiler  */
/*  licensed hereunder contains information that complies with the     */
/*  Virtual Component Identification Physical Tagging Standard (VCID)  */
/*  as maintained by the Virtual Socket Interface Alliance (VSIA).     */
/*  Such information may be expressed in GDSII Layer 63 or other such  */
/*  layer designated by the VSIA, hardware definition languages, or    */
/*  other formats.  Licensee is not authorized to alter or change any  */
/*  such information.                                                  */
/*---------------------------------------------------------------------*/
/*                                                                     */
/*  Built for linux64 and running on linux64.                          */
/*                                                                     */
/*  Software           : Rev: M-2017.03                                */
/*  Library Format     : Rev: 1.05.00                                  */
/*  Compiler Name      : vl00x000vpnnstsrv000sa31                      */
/*  Platform           : Linux2.6.32-573.el6.x86_64                    */
/*                     : #1 SMP Wed Jul 1 18:23:37 EDT 2015x86_64      */
/*  Date of Generation : Thu Jul 27 18:25:16 EDT 2017                  */
/*                                                                     */
/*---------------------------------------------------------------------*/

`resetall
// T I M E S C A L E ===============================================================================
`ifndef SMS_NO_TIMESCALE
    `timescale 1ns/1ps
`endif
//  input Ports:
//               wrck                  -  clock input signal
//               rst_sfbp       -  reset input signal
//               data_from_sms_p1500_n -  expanded negedge toggled data serial input (from the SMS group IEEE 1500, last WSO in chain)
//               signature_out         -  compressed data serial input (From the compressed signature data register)
//               start_compress        -  start compress command signal input
//               clean_status          -  keeps overflow flag during STATUS_SEL instruction, otherwise cleans it, to skip wrong sfp_fail signal generation
//               start_expand          -  start expand command signal input
// output Ports:
//               data2sms_compr        -  expanded data serial output (to the SMS group IEEE 1500, first WSI in chain)
//               data2sms_sh_compr     -  expanded data shift enable signal (to the SMS IEEE 1500 ShiftDR)
//               signature_in          -  compressed data serial output (to the compressed signature data register)
//               signature_en          -  compressed data shift enable signal (to the compressed signature data register)
//               compr_expand_on       -  compressor-expander on/off status output signal
//               overflow              -  overflow status output signal, indicates that the compressed data exceeds the size of the non-volatile data container.
//               sms_early_shift       -  expanded data shift enable signal (forward by one clock cycle)
module smsg_gc9_rp_ce (
            wrck,
            rst_sfbp,
            data_from_sms_p1500_n,
            signature_out,
            start_compress,      
            clean_status,
            start_expand,
            mux_data_msb_nxt,
            early_abort,
            efread_flag,
            ef_sign_read,
            driver_on,
            data_sh_en_drv,
            dout_ef,
            sms_cntr,
            ring_bypass_unrep,
            decr_sms_cntr_ce,            
            curr_sms_bypass_ctrl,
            last_dummy_loop,
            data2sms_compr,
            data2sms_sh_compr,
            sms_early_shift,
            signature_in,
            signature_en,
            compr_expand_on,
            overflow
);


input  wrck;
input  rst_sfbp;
input  data_from_sms_p1500_n;
input  signature_out;
input  start_compress;
input  clean_status;
input  start_expand;
input  early_abort;
input  efread_flag;
input  mux_data_msb_nxt;
input  ef_sign_read;
input  driver_on;
input  data_sh_en_drv;
input  dout_ef;
input  [4:0] sms_cntr;
output [1:0] ring_bypass_unrep;
output  decr_sms_cntr_ce;
output curr_sms_bypass_ctrl;
output last_dummy_loop;
output  data2sms_compr;
output  data2sms_sh_compr;
output  sms_early_shift;
output  signature_in;
output  signature_en;
output  compr_expand_on;
output  overflow;

parameter  RESETSTATE        = 6'd0;
parameter  INTERN_RESET      = 6'd1;
parameter  CHECK_DUMMY_OUT   = 6'd2;
parameter  COMPR_PART        = 6'd3;
parameter  WAIT_OUT          = 6'd4;
parameter  PRST_DATAOUT      = 6'd5;
parameter  SMSOUT            = 6'd6;
parameter  PRST_ADDRIN       = 6'd7;
parameter  COMPRIN           = 6'd9;
parameter  NEXTEL            = 6'd10;
parameter  PRSTROTATION      = 6'd13;
parameter  ROTATION          = 6'd14;
parameter  ENDSTATE          = 6'd15;
parameter  CHECK_DUMMY_IN    = 6'd17;
parameter  EXPR_PART         = 6'd18;
parameter  PRST_DATAIN       = 6'd21;
parameter  SMSIN             = 6'd22;
parameter  PRST_ADDROUT      = 6'd23;
parameter  COMPROUT          = 6'd25;
parameter  NEXTEL_EXP        = 6'd26;
parameter  WAIT1_ADR_CHECK   = 6'd27;
parameter  WAIT2_ADR_CHECK   = 6'd28;
parameter  PRSTEXTROTATION   = 6'd29;
parameter  EXTROTATION       = 6'd30;

parameter  NEXT_REPAIR       = 6'd31; 
parameter  LD_REP_EL         = 6'd32;
parameter  SKIP_DATA         = 6'd33;
parameter  LD_DUMMY_DATA     = 6'd34;

parameter  DUMMY_CHAIN       = 3'd1;
parameter  ADDR_SFPREG       = 3'd2;
parameter  SMS_SFPREG        = 3'd4;


wire [4:0] nxt_sms_start_gr;
wire [4:0] nxt_sms_start_el;
integer current;
reg next_sms_bypass_ce;
wire check_sms_repair;
wire decr_gr;
wire load_el;
wire decr_sms_cntr;
wire skip_mux_shift_nxt;
wire ret_check_dummy_in_nxt;
reg ret_check_dummy_in_r;
reg  skip_mux_shift_r;
wire [4:0] sms_cntr_nxt;
reg  [4:0] sms_num;
reg  [4:0] sms_cntr_r;
reg  [31:0] sms_repair_r;
reg  [31:0] sms_repair_nxt;
 
reg  curr_sms_bypass_ce;
reg  curr_sms_bypass_ctrl;
reg  int_rst_r;
reg  [5:0] state_r, state_nxt;
reg  [4:0] el_cntr_r, el_cntr_nxt;
reg  [4:0] group_cntr_r, group_cntr_nxt;
reg  [3:0] shiftcntr_r, shiftcntr_nxt;
reg  [7:0] comprshiftcntr_r, comprshiftcntr_nxt;
reg  [2:0] chain_select_r, chain_select_nxt;
reg  [5:0] addrbuf_r, addrbuf_nxt;
reg  [1:0] recen_flag_r, recen_flag_nxt;
reg  repair_en_r, repair_en_nxt;
reg  seek_flag_r, seek_flag_nxt;
reg  addr_scan_flag_r, addr_scan_flag_nxt;
reg  dummy_flag_r, dummy_flag_nxt;
reg  dummy_flop;
reg  addr_matched_r, addr_matched_nxt;
reg  errflag_r, errflag_nxt;
reg  overflow_r, overflow_nxt;
reg  [3:0] comprdata_size;
reg  [3:0] recsize;
reg  sms_sc_en_r;
reg  compr_sc_en_r, compr_sc_en_nxt;
reg  compr_mode_r, compr_mode_nxt;
reg  sh_rep_en_bit_r, sh_rep_en_bit_nxt;
reg  shift_addrbuf_r;
reg  signature_in;
reg  flag_ext_rot_r;

wire sms_sc_en_nxt;
wire shift_addrbuf_nxt;
wire endshift;
wire end_of_chain;
wire group_change;
wire next_group;
wire addr_match;
wire next_el;
wire end_rotation;
wire err_rotation;
wire endsh_rotation;
wire chain_rotation;
wire shiftcntren;
wire int_rst;
wire compr_err_flag;
wire dummy_data;
wire flag_ext_rot_nxt;

// Repair acceleration logic
assign last_dummy_loop     = !check_sms_repair & !compr_mode_r & efread_flag & ((state_r == ROTATION) | (state_r == ENDSTATE));
assign {nxt_sms_start_gr, nxt_sms_start_el} =
                              (sms_cntr_r == 5'd31) ? {5'd22, 5'd26} : 
                              (sms_cntr_r == 5'd30) ? {5'd22, 5'd18} : 
                              (sms_cntr_r == 5'd29) ? {5'd21, 5'd30} : 
                              (sms_cntr_r == 5'd28) ? {5'd21, 5'd22} : 
                              (sms_cntr_r == 5'd27) ? {5'd21, 5'd14} : 
                              (sms_cntr_r == 5'd26) ? {5'd20, 5'd26} : 
                              (sms_cntr_r == 5'd25) ? {5'd20, 5'd18} : 
                              (sms_cntr_r == 5'd24) ? {5'd20, 5'd10} : 
                              (sms_cntr_r == 5'd23) ? {5'd19, 5'd22} : 
                              (sms_cntr_r == 5'd22) ? {5'd19, 5'd14} : 
                              (sms_cntr_r == 5'd21) ? {5'd19, 5'd6} : 
                              (sms_cntr_r == 5'd20) ? {5'd18, 5'd10} : 
                              (sms_cntr_r == 5'd19) ? {5'd17, 5'd22} : 
                              (sms_cntr_r == 5'd18) ? {5'd17, 5'd14} : 
                              (sms_cntr_r == 5'd17) ? {5'd17, 5'd6} : 
                              (sms_cntr_r == 5'd16) ? {5'd16, 5'd10} : 
                              (sms_cntr_r == 5'd15) ? {5'd15, 5'd22} : 
                              (sms_cntr_r == 5'd14) ? {5'd15, 5'd14} : 
                              (sms_cntr_r == 5'd13) ? {5'd15, 5'd6} : 
                              (sms_cntr_r == 5'd12) ? {5'd14, 5'd10} : 
                              (sms_cntr_r == 5'd11) ? {5'd13, 5'd22} : 
                              (sms_cntr_r == 5'd10) ? {5'd13, 5'd14} : 
                              (sms_cntr_r == 5'd9) ? {5'd11, 5'd26} : 
                              (sms_cntr_r == 5'd8) ? {5'd10, 5'd6} : 
                              (sms_cntr_r == 5'd7) ? {5'd9, 5'd16} : 
                              (sms_cntr_r == 5'd6) ? {5'd8, 5'd26} : 
                              (sms_cntr_r == 5'd5) ? {5'd8, 5'd4} : 
                              (sms_cntr_r == 5'd4) ? {5'd7, 5'd14} : 
                              (sms_cntr_r == 5'd3) ? {5'd6, 5'd9} : 
                              (sms_cntr_r == 5'd2) ? {5'd3, 5'd15} : 
                              (sms_cntr_r == 5'd1) ? {5'd0, 5'd21} : 
                                                     {5'd0, 5'd0};
wire first_dummy_cnt;
wire dummy_cnt;
wire blk_ld_dummy_nxt;
reg  blk_ld_dummy_r;

assign blk_ld_dummy_nxt     = (state_r == LD_DUMMY_DATA) | ((state_r != CHECK_DUMMY_IN) & blk_ld_dummy_r);
assign first_dummy_cnt      = (sms_cntr_r == 5'd31) & curr_sms_bypass_ce ? 1'd0 : 1'd0;
assign dummy_cnt            = 
                              (sms_cntr_r == 5'd31) ? 1'd0:
                              (sms_cntr_r == 5'd30) ? 1'd0:
                              (sms_cntr_r == 5'd29) ? 1'd0:
                              (sms_cntr_r == 5'd28) ? 1'd0:
                              (sms_cntr_r == 5'd27) ? 1'd0:
                              (sms_cntr_r == 5'd26) ? 1'd0:
                              (sms_cntr_r == 5'd25) ? 1'd0:
                              (sms_cntr_r == 5'd24) ? 1'd0:
                              (sms_cntr_r == 5'd23) ? 1'd0:
                              (sms_cntr_r == 5'd22) ? 1'd0:
                              (sms_cntr_r == 5'd21) ? 1'd0:
                              (sms_cntr_r == 5'd20) ? 1'd0:
                              (sms_cntr_r == 5'd19) ? 1'd0:
                              (sms_cntr_r == 5'd18) ? 1'd0:
                              (sms_cntr_r == 5'd17) ? 1'd0:
                              (sms_cntr_r == 5'd16) ? 1'd0:
                              (sms_cntr_r == 5'd15) ? 1'd0:
                              (sms_cntr_r == 5'd14) ? 1'd0:
                              (sms_cntr_r == 5'd13) ? 1'd0:
                              (sms_cntr_r == 5'd12) ? 1'd0:
                              (sms_cntr_r == 5'd11) ? 1'd0:
                              (sms_cntr_r == 5'd10) ? 1'd0:
                              (sms_cntr_r == 5'd9) ? 1'd0:
                              (sms_cntr_r == 5'd8) ? 1'd0:
                              (sms_cntr_r == 5'd7) ? 1'd0:
                              (sms_cntr_r == 5'd6) ? 1'd0:
                              (sms_cntr_r == 5'd5) ? 1'd0:
                              (sms_cntr_r == 5'd4) ? 1'd0:
                              (sms_cntr_r == 5'd3) ? 1'd0:
                              (sms_cntr_r == 5'd2) ? 1'd0:
                              (sms_cntr_r == 5'd1) ? 1'd0:
                              (sms_cntr_r == 5'd0) ? 1'd0 : 1'd0;

assign ring_bypass_unrep    = !efread_flag ? 2'd0 : {
                               !(|sms_repair_r[31:8]),
                               !(|sms_repair_r[7:0]) 
                              };
assign check_sms_repair     = ef_sign_read & driver_on;
assign decr_gr              = !check_sms_repair & ((state_r == NEXT_REPAIR) & endshift | (state_r == LD_REP_EL)) & (group_cntr_r != nxt_sms_start_gr)
                              | check_sms_repair & (state_r == NEXT_REPAIR) & data_sh_en_drv & !dout_ef
                              | (state_r == SKIP_DATA) & (el_cntr_r == 5'd0) & (shiftcntr_r == ~comprdata_size) & data_sh_en_drv;
assign load_el              = !check_sms_repair & (state_r == NEXT_REPAIR) & endshift
                              | check_sms_repair & (state_r == LD_REP_EL) & data_sh_en_drv;
assign decr_sms_cntr        = !check_sms_repair & ((state_r == LD_REP_EL) & (group_cntr_r == nxt_sms_start_gr)  | 
                              ((state_r == EXPR_PART) & !int_rst_r | (state_r == CHECK_DUMMY_IN) | (state_r == WAIT2_ADR_CHECK)) & (sms_cntr_r != sms_num)) & (sms_cntr_r != 5'd0)
                              ;
assign decr_sms_cntr_ce     = decr_sms_cntr;
assign ret_check_dummy_in_nxt = !check_sms_repair & (state_nxt == NEXT_REPAIR) & ((state_r == WAIT2_ADR_CHECK) | (state_r == CHECK_DUMMY_IN)) | !((state_r == LD_DUMMY_DATA) & endshift | int_rst) & ret_check_dummy_in_r;
assign skip_mux_shift_nxt   = !check_sms_repair & decr_gr & mux_data_msb_nxt |  !((state_r == LD_DUMMY_DATA) & endshift | int_rst) & skip_mux_shift_r;
assign sms_cntr_nxt         = int_rst ? 5'd31 : decr_sms_cntr ? sms_cntr_r - 5'd1 : sms_cntr_r;
assign sms_sc_en_nxt        = (state_r == SMSOUT) | (state_r == SMSIN) | !check_sms_repair & efread_flag & ((state_r == NEXT_REPAIR) | (state_r == LD_DUMMY_DATA));
assign endshift             = ((shiftcntr_r == 4'd0) & (!check_sms_repair | data_sh_en_drv));
assign shiftcntren          = shiftcntr_r != 4'd0 & (!check_sms_repair | data_sh_en_drv);

// End of repair acceleration logic
assign data2sms_sh_compr    = sms_sc_en_r;
assign sms_early_shift      = sms_sc_en_nxt;
assign signature_en         = compr_sc_en_r | sh_rep_en_bit_r;
assign data2sms_compr       = addr_matched_r ? ((signature_out & compr_sc_en_r) | recen_flag_r[1]) : 1'b0;
assign compr_err_flag       = data_from_sms_p1500_n & (state_r == PRST_DATAOUT);
assign end_of_chain         = (group_cntr_r == 5'd0) & group_change;
assign group_change         = (el_cntr_r == 5'd0);
assign next_group           = group_change & next_el;
assign addr_match           = (addrbuf_r[4:0] == el_cntr_r);
assign next_el              = (state_r == NEXTEL) | (state_r == NEXTEL_EXP);
assign shift_addrbuf_nxt    = chain_select_r[1] & ((state_r == COMPRIN) | (state_r == COMPROUT));
assign end_rotation         = (comprshiftcntr_r <= 8'd1);
assign endsh_rotation       = (comprshiftcntr_r <= 8'd2);
assign err_rotation         = (comprshiftcntr_r == 8'd0);
assign chain_rotation       = (state_r == ROTATION) | (state_r == EXTROTATION);
assign int_rst              = (state_r == INTERN_RESET);
assign compr_expand_on      = (state_r != RESETSTATE) & (state_r != ENDSTATE);
assign overflow             = overflow_r;
assign dummy_data           = chain_select_r[0];
assign flag_ext_rot_nxt     = state_r[4];

always @(sms_num or state_r or sms_cntr_r or sms_repair_r or start_expand  or sms_cntr
         or shiftcntr_r or comprdata_size or data_sh_en_drv
         or check_sms_repair or int_rst or early_abort
         or efread_flag
         )
  begin
    curr_sms_bypass_ce   = 1'b0;
    next_sms_bypass_ce   = 1'b0;
    curr_sms_bypass_ctrl = 1'b0;   
    sms_repair_nxt       = (check_sms_repair & int_rst | early_abort) ? 32'd0 : (!efread_flag & (state_r == RESETSTATE))? {32{1'b1}}  : sms_repair_r;
    for (current = 0;  current <= 31;  current = current + 1)
      begin
         if ((sms_num == current) & (state_r == SKIP_DATA) & (shiftcntr_r == ~comprdata_size) & data_sh_en_drv)
           sms_repair_nxt[current] = 1'b1;
         if (sms_cntr_r == current) begin
           curr_sms_bypass_ce = start_expand & !sms_repair_r[current];
           next_sms_bypass_ce = (current == 0) ? 1'b0 : start_expand & !sms_repair_r[current - 1];
         end  
         if (sms_cntr == current)
           curr_sms_bypass_ctrl = !sms_repair_r[current];
      end
  end
always @(state_r or start_expand)
  begin
    if ((state_r == RESETSTATE) & start_expand)
      sh_rep_en_bit_nxt = 1'b1;
    else if (state_r == PRSTEXTROTATION)
      sh_rep_en_bit_nxt = 1'b1;
    else
      sh_rep_en_bit_nxt = 1'b0;
  end


always @(state_r or start_compress or compr_mode_r)
  begin
    if (state_r == RESETSTATE)
      begin
        if (start_compress)
          compr_mode_nxt = 1'b1;
        else
          compr_mode_nxt = 1'b0;
      end
    else
      compr_mode_nxt = compr_mode_r;
  end


always @(int_rst or errflag_r or repair_en_r or overflow_r)
  begin
    if (int_rst)
      repair_en_nxt = 1'b0;
    else if (overflow_r)
      repair_en_nxt = 1'b0;
    else if (errflag_r)
      repair_en_nxt = 1'b1;
    else
      repair_en_nxt = repair_en_r;
  end


always @(chain_select_r or state_r or shiftcntr_r or chain_rotation or end_rotation 
        or comprdata_size
        or check_sms_repair or decr_gr or mux_data_msb_nxt or efread_flag
        )
  begin
    case (chain_select_r)
      SMS_SFPREG   : compr_sc_en_nxt = ((state_r == SMSOUT) | ((state_r == SMSIN) & !end_rotation)) & (shiftcntr_r <= comprdata_size);
      default      : compr_sc_en_nxt = ((state_r == COMPRIN) | (state_r == COMPROUT)) | chain_rotation  | efread_flag & !check_sms_repair & decr_gr & !mux_data_msb_nxt;
    endcase
  end


always @(state_r or recen_flag_r)
  begin
    recen_flag_nxt = {recen_flag_r[0], (state_r == PRST_DATAIN)};
  end


always @(state_r or seek_flag_r)
  begin
    if (state_r == PRST_ADDROUT)
      seek_flag_nxt = 1'b1;
    else if (state_r == EXPR_PART)
      seek_flag_nxt = 1'b0;
    else
      seek_flag_nxt = seek_flag_r;
  end


always @(chain_select_r or compr_mode_r or addrbuf_r or data_from_sms_p1500_n or signature_out or
          sh_rep_en_bit_r or repair_en_r or flag_ext_rot_r)
  begin
    if (!compr_mode_r)
      signature_in = signature_out;
    else
      case (chain_select_r)
        ADDR_SFPREG  : signature_in = addrbuf_r[5];
        SMS_SFPREG   : signature_in = data_from_sms_p1500_n;
        default      : if (sh_rep_en_bit_r)
                         signature_in = repair_en_r;
                       else if (flag_ext_rot_r)
                         signature_in = signature_out;
                       else
                         signature_in = 1'b0;
      endcase
  end


always @(state_r or addr_match or addr_matched_r or seek_flag_r)
  begin
    if ((state_r == PRST_DATAIN) & addr_match & seek_flag_r)
      addr_matched_nxt = 1'b1;
    else if (state_r == NEXTEL_EXP)
      addr_matched_nxt = 1'b0;
    else
      addr_matched_nxt = addr_matched_r;
  end


always @(int_rst or compr_sc_en_r or end_rotation or overflow_r or state_r or clean_status)
  begin
    if (int_rst)
      overflow_nxt = 1'b0;
    else if (compr_sc_en_r & end_rotation & (state_r <= 6'd12))
      overflow_nxt = 1'b1;
    else
      overflow_nxt = overflow_r & !clean_status;
  end


always @(int_rst or compr_sc_en_r or comprshiftcntr_r or err_rotation or state_r or chain_rotation)
  begin
    if (int_rst | (state_r == PRSTEXTROTATION))
      comprshiftcntr_nxt = 8'd208;
    else if ((compr_sc_en_r | chain_rotation) & !err_rotation)
      comprshiftcntr_nxt = comprshiftcntr_r - 8'd1;
    else
      comprshiftcntr_nxt = comprshiftcntr_r;
  end

always @(int_rst or state_r or recsize or shiftcntren or shiftcntr_r
        or data_sh_en_drv or check_sms_repair or endshift or efread_flag
        or int_rst_r or first_dummy_cnt or dummy_cnt or decr_sms_cntr or next_sms_bypass_ce
        )
  begin
    if (int_rst)
      shiftcntr_nxt = 4'd0;
    else
    case (state_r)
      PRST_DATAOUT,
      PRST_DATAIN     : shiftcntr_nxt = recsize;
      LD_REP_EL       : shiftcntr_nxt = !check_sms_repair ? {3'd0, dummy_cnt}:
                        data_sh_en_drv ? (shiftcntr_r - 4'd1) :
                        shiftcntr_r;
      PRST_ADDRIN,
      PRST_ADDROUT    : shiftcntr_nxt = 4'd5;
      NEXT_REPAIR     : shiftcntr_nxt = check_sms_repair ? 4'd4 :
                                        !endshift ? (shiftcntr_r - 4'd1) :
                                        shiftcntr_r;
      EXPR_PART       : shiftcntr_nxt = !check_sms_repair ? !efread_flag ? 4'd0 : (int_rst_r ? {3'd0, first_dummy_cnt} : {3'd0, (decr_sms_cntr & next_sms_bypass_ce ? dummy_cnt : 1'd0)}) : shiftcntr_r;
      default         : if (shiftcntren)
                          shiftcntr_nxt = shiftcntr_r - 4'd1;
                        else
                          shiftcntr_nxt = shiftcntr_r;
    endcase
  end


always @(int_rst or next_group or next_el or el_cntr_r
         or load_el or check_sms_repair or data_sh_en_drv or dout_ef or nxt_sms_start_el
        )
  begin
    if (int_rst)
      el_cntr_nxt = 5'd14;    
    else if (load_el & check_sms_repair & data_sh_en_drv)
      el_cntr_nxt = {el_cntr_r[3:0], dout_ef};
    else if (load_el & !check_sms_repair)
      el_cntr_nxt = nxt_sms_start_el;
    else if (next_group)
      el_cntr_nxt = 5'd31;
    else if (next_el)
      el_cntr_nxt = el_cntr_r - 5'd1;
    else
      el_cntr_nxt = el_cntr_r;
  end


always @(int_rst or next_group or group_cntr_r
        or decr_gr
        )
  begin
    if (int_rst)
      group_cntr_nxt = 5'd24;
    else if ((next_group | decr_gr) & (group_cntr_r != 5'd0))
      group_cntr_nxt = group_cntr_r - 5'd1;
    else
      group_cntr_nxt = group_cntr_r;
  end



always @(int_rst or state_r or dummy_flop or dummy_flag_r or blk_ld_dummy_r)
  begin
    if (int_rst)
      dummy_flag_nxt = 1'b0;
    else if (((state_r == CHECK_DUMMY_OUT) | (state_r == CHECK_DUMMY_IN)) & dummy_flop)
      dummy_flag_nxt = !blk_ld_dummy_r;
    else if ((state_r == WAIT_OUT) | (state_r == PRST_DATAIN))
      dummy_flag_nxt = 1'b0;
    else
      dummy_flag_nxt = dummy_flag_r;
  end


always @(state_r or addr_scan_flag_r or endshift)
  begin
    if ((state_r == PRST_ADDRIN) | (state_r == PRST_ADDROUT))
      addr_scan_flag_nxt = 1'b1;
    else if (((state_r == COMPRIN) | (state_r == COMPROUT)) & endshift)
      addr_scan_flag_nxt = 1'b0;
    else
      addr_scan_flag_nxt = addr_scan_flag_r;
  end


always @(state_r or shift_addrbuf_r or signature_out or el_cntr_r or addrbuf_r)
  begin
    if (state_r == PRST_ADDRIN)
      addrbuf_nxt = {1'b1, el_cntr_r};
    else if (shift_addrbuf_r)
      addrbuf_nxt = {addrbuf_r[4:0], signature_out};
    else
      addrbuf_nxt = addrbuf_r;
  end


always @(state_r or compr_err_flag or addr_match or next_el or int_rst or chain_select_r or seek_flag_r)
  begin
    if ((state_r == PRST_ADDRIN) | (state_r == PRST_ADDROUT))
      chain_select_nxt = ADDR_SFPREG;
    else if (compr_err_flag | ((state_r == PRST_DATAIN) & addr_match & seek_flag_r))
      chain_select_nxt = SMS_SFPREG;
    else if (next_el | int_rst)
      chain_select_nxt = DUMMY_CHAIN;
    else
      chain_select_nxt = chain_select_r;
  end


always @(int_rst or errflag_r or next_el or compr_err_flag )
  begin
    if (next_el | int_rst)
      errflag_nxt = 1'b0;
    else if (compr_err_flag )
      errflag_nxt = 1'b1;
    else
      errflag_nxt = errflag_r;
  end


always @(start_compress or start_expand or data_from_sms_p1500_n or dummy_data or endshift
         or addr_scan_flag_r or end_of_chain or signature_out or addr_match or addr_matched_r
         or endsh_rotation or end_rotation or state_r or seek_flag_r or group_change or compr_mode_r or dummy_flop or dummy_flag_r or blk_ld_dummy_r
         or check_sms_repair or data_sh_en_drv or dout_ef or curr_sms_bypass_ce or next_sms_bypass_ce or sms_num or sms_cntr_r or early_abort or ret_check_dummy_in_r
         or nxt_sms_start_gr or group_cntr_r or comprdata_size or shiftcntr_r or efread_flag or int_rst_r or decr_sms_cntr /*or addrbuf_ready_r*/ or skip_mux_shift_r
        )

  begin
    case (state_r)
      RESETSTATE      : if (start_compress | start_expand | (check_sms_repair & !early_abort))
                          state_nxt = INTERN_RESET;
                        else
                          state_nxt = state_r;
      INTERN_RESET    : if (start_compress)
                          state_nxt = CHECK_DUMMY_OUT;
                        else if (check_sms_repair)
                          if (early_abort)
                            state_nxt = RESETSTATE;
                          else if (data_sh_en_drv)
                            state_nxt = NEXT_REPAIR;
                          else
                            state_nxt = INTERN_RESET;
                        else
                          state_nxt = EXPR_PART;
       NEXT_REPAIR    : if (!start_expand & !check_sms_repair)
                          state_nxt = RESETSTATE;  
                        else if (endshift & !check_sms_repair & (sms_cntr_r == 5'd0))
                          state_nxt = PRSTROTATION;
                        else if (check_sms_repair ? (data_sh_en_drv & dout_ef) : endshift)
                          state_nxt = LD_REP_EL;
                        else 
                          state_nxt = NEXT_REPAIR;
       LD_REP_EL      : if (!start_expand & !check_sms_repair)
                          state_nxt = RESETSTATE;  
                        else if (end_of_chain & !check_sms_repair & (sms_cntr_r == 5'd0))
                          state_nxt = PRSTROTATION;
                        else if (!endshift & check_sms_repair | ((group_cntr_r != nxt_sms_start_gr) & !check_sms_repair))
                          state_nxt = LD_REP_EL;
                        else if (check_sms_repair & data_sh_en_drv)
                          state_nxt = SKIP_DATA;
                        else if ((group_cntr_r == nxt_sms_start_gr) & !check_sms_repair & !(end_of_chain & (sms_cntr_r == 5'd0)))
                          state_nxt = next_sms_bypass_ce ? NEXT_REPAIR : LD_DUMMY_DATA;
                        else
                          state_nxt = state_r;
       LD_DUMMY_DATA  : if (!start_expand & !check_sms_repair)
                          state_nxt = RESETSTATE;  
                        else if (endshift)
                          if (ret_check_dummy_in_r & !skip_mux_shift_r)
                            state_nxt = CHECK_DUMMY_IN;
                          else if (signature_out)
                            state_nxt = PRST_ADDROUT;
                          else
                            state_nxt = COMPROUT;
                        else
                          state_nxt = LD_DUMMY_DATA;
       SKIP_DATA      : if (!start_expand & !check_sms_repair)
                          state_nxt = RESETSTATE;  
                        else if ((shiftcntr_r == ~comprdata_size) & data_sh_en_drv)  
                          state_nxt = NEXT_REPAIR;
                        else
                          state_nxt = SKIP_DATA;
      CHECK_DUMMY_OUT : if (dummy_flop)
                          state_nxt = SMSOUT;
                        else
                          state_nxt = COMPR_PART;
      WAIT_OUT        : state_nxt = COMPR_PART;
      COMPR_PART      : if (data_from_sms_p1500_n)
                          state_nxt = PRST_ADDRIN;
                        else
                          state_nxt = PRST_DATAOUT;
      PRST_DATAOUT    : state_nxt = SMSOUT;
      SMSOUT          : if (endshift)
                          begin
                            if (dummy_flag_r)
                              state_nxt = WAIT_OUT;
                            else
                              state_nxt = NEXTEL;
                          end
                        else
                          state_nxt = state_r;
      PRST_ADDRIN     : state_nxt = COMPRIN;
      COMPRIN         : if (endshift)
                          begin
                            if (addr_scan_flag_r)
                              state_nxt = PRST_DATAOUT;
                            else
                              state_nxt = CHECK_DUMMY_OUT;
                          end
                        else
                          state_nxt = state_r;
      NEXTEL          : if (end_of_chain)
                          state_nxt = PRSTROTATION;
                        else if (group_change & dummy_data)
                          state_nxt = COMPRIN;
                        else
                          state_nxt = CHECK_DUMMY_OUT;
      EXPR_PART       : if ((next_sms_bypass_ce & (sms_cntr_r != sms_num) | curr_sms_bypass_ce & int_rst_r) & !check_sms_repair & efread_flag)
                          state_nxt = NEXT_REPAIR;
                        else if (end_rotation)
                          state_nxt = CHECK_DUMMY_IN;
                        else if ((!(next_sms_bypass_ce & decr_sms_cntr) | !efread_flag) & signature_out)
                          state_nxt = PRST_ADDROUT;
                        else
                          state_nxt = COMPROUT;
      PRST_ADDROUT    : state_nxt = COMPROUT;
      COMPROUT        : if (endshift)
                          begin
                            if (addr_scan_flag_r)
                              state_nxt = WAIT1_ADR_CHECK;
                            else
                              state_nxt = CHECK_DUMMY_IN;
                          end
                        else
                          state_nxt = state_r;
      WAIT1_ADR_CHECK : state_nxt = WAIT2_ADR_CHECK;
      WAIT2_ADR_CHECK : if (efread_flag & next_sms_bypass_ce & (sms_cntr_r != sms_num))
                          state_nxt = NEXT_REPAIR;
                        else if (addr_match)
                          state_nxt = CHECK_DUMMY_IN;
                        else
                          state_nxt = CHECK_DUMMY_IN;
      CHECK_DUMMY_IN  :  if (efread_flag & next_sms_bypass_ce & (sms_cntr_r != sms_num))
                          state_nxt = NEXT_REPAIR;
                         else if (dummy_flop & !blk_ld_dummy_r)
                          state_nxt = SMSIN;
                        else
                          state_nxt = PRST_DATAIN;
      PRST_DATAIN     : state_nxt = SMSIN;
      SMSIN           : if (endshift)
                          begin
                            if (dummy_flag_r)
                              state_nxt = PRST_DATAIN;
                            else
                              state_nxt = NEXTEL_EXP;
                          end
                        else
                          state_nxt = state_r;
      NEXTEL_EXP      : if (end_of_chain)
                          state_nxt = PRSTROTATION;
                        else if (/*(*/group_change | addr_matched_r/*) & !addrbuf_ready_r*/)
                          state_nxt = EXPR_PART;
                        else if (seek_flag_r)
                          state_nxt = WAIT2_ADR_CHECK;
                        else
                          state_nxt = CHECK_DUMMY_IN;
      PRSTROTATION    : if (end_rotation & compr_mode_r)
                          state_nxt = PRSTEXTROTATION;
                        else if (end_rotation)
                          state_nxt = ENDSTATE;
                        else
                          state_nxt = ROTATION;
      ROTATION        : if (endsh_rotation)
                          begin
                            if (compr_mode_r)
                              state_nxt = PRSTEXTROTATION;
                            else
                              state_nxt = ENDSTATE;
                          end
                        else
                          state_nxt = state_r;
      PRSTEXTROTATION : state_nxt = EXTROTATION;
      EXTROTATION     : if (endsh_rotation)
                          state_nxt = ENDSTATE;
                        else
                          state_nxt = state_r;
      default         : if (start_compress | start_expand)
                          state_nxt = state_r;
                        else
                          state_nxt = RESETSTATE;
    endcase
  end


always @(group_cntr_r or el_cntr_r)
  begin
    case (group_cntr_r)
      5'd0 :
        case (el_cntr_r)
          5'd0 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd0};
          5'd1 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd0};
          5'd2 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd0};
          5'd3 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd0};
          5'd4 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd0};
          5'd5 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd0};
          5'd6 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd0};
          5'd7 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd0};
          5'd8 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd0};
          5'd9 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd0};
          5'd10 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd0};
          5'd11 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd0};
          5'd12 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd0};
          5'd13 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd0};
          5'd14 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd0};
          5'd15 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd0};
          5'd16 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd0};
          5'd17 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd0};
          5'd18 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd0};
          5'd19 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd0};
          5'd20 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd0};
          5'd21 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd0};
          5'd22 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd1};
          5'd23 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd1};
          5'd24 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd1};
          5'd25 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd1};
          5'd26 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd1};
          5'd27 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd1};
          5'd28 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd1};
          5'd29 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd1};
          5'd30 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd1};
          default : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd1};
        endcase
      5'd1 :
        case (el_cntr_r)
          5'd0 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd1};
          5'd1 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd1};
          5'd2 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd1};
          5'd3 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd1};
          5'd4 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd1};
          5'd5 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd1};
          5'd6 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd1};
          5'd7 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd1};
          5'd8 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd1};
          5'd9 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd1};
          5'd10 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd1};
          5'd11 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd1};
          5'd12 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd1};
          5'd13 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd1};
          5'd14 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd1};
          5'd15 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd1};
          5'd16 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd1};
          5'd17 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd1};
          5'd18 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd1};
          5'd19 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd1};
          5'd20 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd1};
          5'd21 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd1};
          5'd22 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd1};
          5'd23 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd1};
          5'd24 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd1};
          5'd25 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd1};
          5'd26 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd1};
          5'd27 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd1};
          5'd28 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd1};
          5'd29 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd1};
          5'd30 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd1};
          default : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd1};
        endcase
      5'd2 :
        case (el_cntr_r)
          5'd0 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd1};
          5'd1 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd1};
          5'd2 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd1};
          5'd3 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd1};
          5'd4 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd1};
          5'd5 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd1};
          5'd6 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd1};
          5'd7 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd1};
          5'd8 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd1};
          5'd9 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd1};
          5'd10 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd1};
          5'd11 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd1};
          5'd12 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd1};
          5'd13 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd1};
          5'd14 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd1};
          5'd15 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd1};
          5'd16 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd1};
          5'd17 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd1};
          5'd18 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd1};
          5'd19 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd1};
          5'd20 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd1};
          5'd21 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd1};
          5'd22 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd1};
          5'd23 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd1};
          5'd24 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd1};
          5'd25 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd1};
          5'd26 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd1};
          5'd27 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd1};
          5'd28 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd1};
          5'd29 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd1};
          5'd30 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd1};
          default : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd1};
        endcase
      5'd3 :
        case (el_cntr_r)
          5'd0 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd1};
          5'd1 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd1};
          5'd2 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd1};
          5'd3 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd1};
          5'd4 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd1};
          5'd5 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd1};
          5'd6 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd1};
          5'd7 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd1};
          5'd8 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd1};
          5'd9 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd1};
          5'd10 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd1};
          5'd11 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd1};
          5'd12 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd1};
          5'd13 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd1};
          5'd14 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd1};
          5'd15 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd1};
          5'd16 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd2};
          5'd17 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd2};
          5'd18 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd2};
          5'd19 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd2};
          5'd20 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd2};
          5'd21 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd2};
          5'd22 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd2};
          5'd23 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd2};
          5'd24 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd2};
          5'd25 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd2};
          5'd26 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd2};
          5'd27 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd2};
          5'd28 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd2};
          5'd29 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd2};
          5'd30 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd2};
          default : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd2};
        endcase
      5'd4 :
        case (el_cntr_r)
          5'd0 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd2};
          5'd1 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd2};
          5'd2 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd2};
          5'd3 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd2};
          5'd4 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd2};
          5'd5 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd2};
          5'd6 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd2};
          5'd7 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd2};
          5'd8 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd2};
          5'd9 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd2};
          5'd10 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd2};
          5'd11 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd2};
          5'd12 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd2};
          5'd13 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd2};
          5'd14 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd2};
          5'd15 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd2};
          5'd16 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd2};
          5'd17 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd2};
          5'd18 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd2};
          5'd19 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd2};
          5'd20 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd2};
          5'd21 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd2};
          5'd22 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd2};
          5'd23 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd2};
          5'd24 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd2};
          5'd25 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd2};
          5'd26 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd2};
          5'd27 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd2};
          5'd28 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd2};
          5'd29 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd2};
          5'd30 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd2};
          default : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd2};
        endcase
      5'd5 :
        case (el_cntr_r)
          5'd0 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd2};
          5'd1 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd2};
          5'd2 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd2};
          5'd3 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd2};
          5'd4 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd2};
          5'd5 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd2};
          5'd6 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd2};
          5'd7 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd2};
          5'd8 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd2};
          5'd9 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd2};
          5'd10 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd2};
          5'd11 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd2};
          5'd12 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd2};
          5'd13 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd2};
          5'd14 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd2};
          5'd15 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd2};
          5'd16 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd2};
          5'd17 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd2};
          5'd18 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd2};
          5'd19 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd2};
          5'd20 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd2};
          5'd21 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd2};
          5'd22 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd2};
          5'd23 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd2};
          5'd24 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd2};
          5'd25 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd2};
          5'd26 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd2};
          5'd27 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd2};
          5'd28 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd2};
          5'd29 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd2};
          5'd30 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd2};
          default : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd2};
        endcase
      5'd6 :
        case (el_cntr_r)
          5'd0 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd2};
          5'd1 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd2};
          5'd2 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd2};
          5'd3 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd2};
          5'd4 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd2};
          5'd5 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd2};
          5'd6 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd2};
          5'd7 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd2};
          5'd8 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd2};
          5'd9 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd2};
          5'd10 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd3};
          5'd11 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd3};
          5'd12 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd3};
          5'd13 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd3};
          5'd14 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd3};
          5'd15 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd3};
          5'd16 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd3};
          5'd17 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd3};
          5'd18 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd3};
          5'd19 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd3};
          5'd20 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd3};
          5'd21 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd3};
          5'd22 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd3};
          5'd23 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd3};
          5'd24 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd3};
          5'd25 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd3};
          5'd26 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd3};
          5'd27 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd3};
          5'd28 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd3};
          5'd29 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd3};
          5'd30 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd3};
          default : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd3};
        endcase
      5'd7 :
        case (el_cntr_r)
          5'd0 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd3};
          5'd1 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd3};
          5'd2 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd3};
          5'd3 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd3};
          5'd4 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd3};
          5'd5 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd3};
          5'd6 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd3};
          5'd7 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd3};
          5'd8 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd3};
          5'd9 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd3};
          5'd10 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd3};
          5'd11 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd3};
          5'd12 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd3};
          5'd13 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd3};
          5'd14 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd3};
          5'd15 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd4};
          5'd16 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd4};
          5'd17 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd4};
          5'd18 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd4};
          5'd19 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd4};
          5'd20 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd4};
          5'd21 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd4};
          5'd22 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd4};
          5'd23 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd4};
          5'd24 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd4};
          5'd25 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd4};
          5'd26 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd4};
          5'd27 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd4};
          5'd28 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd4};
          5'd29 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd4};
          5'd30 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd4};
          default : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd4};
        endcase
      5'd8 :
        case (el_cntr_r)
          5'd0 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd4};
          5'd1 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd4};
          5'd2 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd4};
          5'd3 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd4};
          5'd4 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd4};
          5'd5 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd5};
          5'd6 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd5};
          5'd7 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd5};
          5'd8 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd5};
          5'd9 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd5};
          5'd10 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd5};
          5'd11 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd5};
          5'd12 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd5};
          5'd13 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd5};
          5'd14 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd5};
          5'd15 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd5};
          5'd16 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd5};
          5'd17 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd5};
          5'd18 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd5};
          5'd19 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd5};
          5'd20 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd5};
          5'd21 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd5};
          5'd22 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd5};
          5'd23 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd5};
          5'd24 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd5};
          5'd25 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd5};
          5'd26 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd5};
          5'd27 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd6};
          5'd28 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd6};
          5'd29 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd6};
          5'd30 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd6};
          default : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd6};
        endcase
      5'd9 :
        case (el_cntr_r)
          5'd0 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd6};
          5'd1 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd6};
          5'd2 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd6};
          5'd3 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd6};
          5'd4 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd6};
          5'd5 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd6};
          5'd6 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd6};
          5'd7 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd6};
          5'd8 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd6};
          5'd9 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd6};
          5'd10 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd6};
          5'd11 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd6};
          5'd12 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd6};
          5'd13 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd6};
          5'd14 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd6};
          5'd15 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd6};
          5'd16 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd6};
          5'd17 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd7};
          5'd18 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd7};
          5'd19 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd7};
          5'd20 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd7};
          5'd21 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd7};
          5'd22 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd7};
          5'd23 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd7};
          5'd24 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd7};
          5'd25 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd7};
          5'd26 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd7};
          5'd27 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd7};
          5'd28 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd7};
          5'd29 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd7};
          5'd30 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd7};
          default : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd7};
        endcase
      5'd10 :
        case (el_cntr_r)
          5'd0 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd7};
          5'd1 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd7};
          5'd2 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd7};
          5'd3 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd7};
          5'd4 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd7};
          5'd5 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd7};
          5'd6 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd7};
          5'd7 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd8};
          5'd8 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd8};
          5'd9 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd8};
          5'd10 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd8};
          5'd11 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd8};
          5'd12 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd8};
          5'd13 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd8};
          5'd14 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd8};
          5'd15 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd8};
          5'd16 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd8};
          5'd17 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd8};
          5'd18 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd8};
          5'd19 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd8};
          5'd20 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd8};
          5'd21 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd8};
          5'd22 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd8};
          5'd23 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd8};
          5'd24 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd8};
          5'd25 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd8};
          5'd26 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd8};
          5'd27 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd8};
          5'd28 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd8};
          5'd29 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd8};
          5'd30 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd8};
          default : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd8};
        endcase
      5'd11 :
        case (el_cntr_r)
          5'd0 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd8};
          5'd1 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd8};
          5'd2 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd8};
          5'd3 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd8};
          5'd4 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd8};
          5'd5 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd8};
          5'd6 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd8};
          5'd7 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd8};
          5'd8 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd8};
          5'd9 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd8};
          5'd10 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd8};
          5'd11 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd8};
          5'd12 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd8};
          5'd13 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd8};
          5'd14 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd8};
          5'd15 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd8};
          5'd16 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd8};
          5'd17 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd8};
          5'd18 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd8};
          5'd19 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd8};
          5'd20 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd8};
          5'd21 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd8};
          5'd22 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd8};
          5'd23 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd8};
          5'd24 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd8};
          5'd25 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd8};
          5'd26 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd8};
          5'd27 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd9};
          5'd28 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd9};
          5'd29 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd9};
          5'd30 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd9};
          default : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd9};
        endcase
      5'd12 :
        case (el_cntr_r)
          5'd0 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd9};
          5'd1 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd9};
          5'd2 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd9};
          5'd3 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd9};
          5'd4 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd9};
          5'd5 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd9};
          5'd6 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd9};
          5'd7 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd9};
          5'd8 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd9};
          5'd9 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd9};
          5'd10 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd9};
          5'd11 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd9};
          5'd12 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd9};
          5'd13 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd9};
          5'd14 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd9};
          5'd15 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd9};
          5'd16 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd9};
          5'd17 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd9};
          5'd18 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd9};
          5'd19 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd9};
          5'd20 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd9};
          5'd21 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd9};
          5'd22 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd9};
          5'd23 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd9};
          5'd24 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd9};
          5'd25 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd9};
          5'd26 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd9};
          5'd27 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd9};
          5'd28 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd9};
          5'd29 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd9};
          5'd30 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd9};
          default : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd9};
        endcase
      5'd13 :
        case (el_cntr_r)
          5'd0 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd9};
          5'd1 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd9};
          5'd2 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd9};
          5'd3 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd9};
          5'd4 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd9};
          5'd5 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd9};
          5'd6 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd9};
          5'd7 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd9};
          5'd8 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd9};
          5'd9 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd9};
          5'd10 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd9};
          5'd11 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd9};
          5'd12 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd9};
          5'd13 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd9};
          5'd14 : {recsize, comprdata_size, sms_num} = {4'd6, 4'd5, 5'd9};
          5'd15 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd10};
          5'd16 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd10};
          5'd17 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd10};
          5'd18 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd10};
          5'd19 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd10};
          5'd20 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd10};
          5'd21 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd10};
          5'd22 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd10};
          5'd23 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd11};
          5'd24 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd11};
          5'd25 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd11};
          5'd26 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd11};
          5'd27 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd11};
          5'd28 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd11};
          5'd29 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd11};
          5'd30 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd11};
          default : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd11};
        endcase
      5'd14 :
        case (el_cntr_r)
          5'd0 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd11};
          5'd1 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd11};
          5'd2 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd11};
          5'd3 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd11};
          5'd4 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd11};
          5'd5 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd11};
          5'd6 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd11};
          5'd7 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd11};
          5'd8 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd11};
          5'd9 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd11};
          5'd10 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd11};
          5'd11 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd12};
          5'd12 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd12};
          5'd13 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd12};
          5'd14 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd12};
          5'd15 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd12};
          5'd16 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd12};
          5'd17 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd12};
          5'd18 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd12};
          5'd19 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd12};
          5'd20 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd12};
          5'd21 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd12};
          5'd22 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd12};
          5'd23 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd12};
          5'd24 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd12};
          5'd25 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd12};
          5'd26 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd12};
          5'd27 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd12};
          5'd28 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd12};
          5'd29 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd12};
          5'd30 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd12};
          default : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd12};
        endcase
      5'd15 :
        case (el_cntr_r)
          5'd0 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd12};
          5'd1 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd12};
          5'd2 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd12};
          5'd3 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd12};
          5'd4 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd12};
          5'd5 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd12};
          5'd6 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd12};
          5'd7 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd13};
          5'd8 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd13};
          5'd9 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd13};
          5'd10 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd13};
          5'd11 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd13};
          5'd12 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd13};
          5'd13 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd13};
          5'd14 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd13};
          5'd15 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd14};
          5'd16 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd14};
          5'd17 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd14};
          5'd18 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd14};
          5'd19 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd14};
          5'd20 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd14};
          5'd21 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd14};
          5'd22 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd14};
          5'd23 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd15};
          5'd24 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd15};
          5'd25 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd15};
          5'd26 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd15};
          5'd27 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd15};
          5'd28 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd15};
          5'd29 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd15};
          5'd30 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd15};
          default : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd15};
        endcase
      5'd16 :
        case (el_cntr_r)
          5'd0 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd15};
          5'd1 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd15};
          5'd2 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd15};
          5'd3 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd15};
          5'd4 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd15};
          5'd5 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd15};
          5'd6 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd15};
          5'd7 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd15};
          5'd8 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd15};
          5'd9 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd15};
          5'd10 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd15};
          5'd11 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd16};
          5'd12 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd16};
          5'd13 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd16};
          5'd14 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd16};
          5'd15 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd16};
          5'd16 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd16};
          5'd17 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd16};
          5'd18 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd16};
          5'd19 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd16};
          5'd20 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd16};
          5'd21 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd16};
          5'd22 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd16};
          5'd23 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd16};
          5'd24 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd16};
          5'd25 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd16};
          5'd26 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd16};
          5'd27 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd16};
          5'd28 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd16};
          5'd29 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd16};
          5'd30 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd16};
          default : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd16};
        endcase
      5'd17 :
        case (el_cntr_r)
          5'd0 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd16};
          5'd1 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd16};
          5'd2 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd16};
          5'd3 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd16};
          5'd4 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd16};
          5'd5 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd16};
          5'd6 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd16};
          5'd7 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd17};
          5'd8 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd17};
          5'd9 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd17};
          5'd10 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd17};
          5'd11 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd17};
          5'd12 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd17};
          5'd13 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd17};
          5'd14 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd17};
          5'd15 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd18};
          5'd16 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd18};
          5'd17 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd18};
          5'd18 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd18};
          5'd19 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd18};
          5'd20 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd18};
          5'd21 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd18};
          5'd22 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd18};
          5'd23 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd19};
          5'd24 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd19};
          5'd25 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd19};
          5'd26 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd19};
          5'd27 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd19};
          5'd28 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd19};
          5'd29 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd19};
          5'd30 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd19};
          default : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd19};
        endcase
      5'd18 :
        case (el_cntr_r)
          5'd0 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd19};
          5'd1 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd19};
          5'd2 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd19};
          5'd3 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd19};
          5'd4 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd19};
          5'd5 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd19};
          5'd6 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd19};
          5'd7 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd19};
          5'd8 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd19};
          5'd9 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd19};
          5'd10 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd19};
          5'd11 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd20};
          5'd12 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd20};
          5'd13 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd20};
          5'd14 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd20};
          5'd15 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd20};
          5'd16 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd20};
          5'd17 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd20};
          5'd18 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd20};
          5'd19 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd20};
          5'd20 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd20};
          5'd21 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd20};
          5'd22 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd20};
          5'd23 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd20};
          5'd24 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd20};
          5'd25 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd20};
          5'd26 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd20};
          5'd27 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd20};
          5'd28 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd20};
          5'd29 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd20};
          5'd30 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd20};
          default : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd20};
        endcase
      5'd19 :
        case (el_cntr_r)
          5'd0 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd20};
          5'd1 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd20};
          5'd2 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd20};
          5'd3 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd20};
          5'd4 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd20};
          5'd5 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd20};
          5'd6 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd20};
          5'd7 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd21};
          5'd8 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd21};
          5'd9 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd21};
          5'd10 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd21};
          5'd11 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd21};
          5'd12 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd21};
          5'd13 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd21};
          5'd14 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd21};
          5'd15 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd22};
          5'd16 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd22};
          5'd17 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd22};
          5'd18 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd22};
          5'd19 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd22};
          5'd20 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd22};
          5'd21 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd22};
          5'd22 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd22};
          5'd23 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd23};
          5'd24 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd23};
          5'd25 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd23};
          5'd26 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd23};
          5'd27 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd23};
          5'd28 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd23};
          5'd29 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd23};
          5'd30 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd23};
          default : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd23};
        endcase
      5'd20 :
        case (el_cntr_r)
          5'd0 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd23};
          5'd1 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd23};
          5'd2 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd23};
          5'd3 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd23};
          5'd4 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd23};
          5'd5 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd23};
          5'd6 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd23};
          5'd7 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd23};
          5'd8 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd23};
          5'd9 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd23};
          5'd10 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd23};
          5'd11 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd24};
          5'd12 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd24};
          5'd13 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd24};
          5'd14 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd24};
          5'd15 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd24};
          5'd16 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd24};
          5'd17 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd24};
          5'd18 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd24};
          5'd19 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd25};
          5'd20 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd25};
          5'd21 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd25};
          5'd22 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd25};
          5'd23 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd25};
          5'd24 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd25};
          5'd25 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd25};
          5'd26 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd25};
          5'd27 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd26};
          5'd28 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd26};
          5'd29 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd26};
          5'd30 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd26};
          default : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd26};
        endcase
      5'd21 :
        case (el_cntr_r)
          5'd0 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd26};
          5'd1 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd26};
          5'd2 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd26};
          5'd3 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd26};
          5'd4 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd26};
          5'd5 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd26};
          5'd6 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd26};
          5'd7 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd26};
          5'd8 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd26};
          5'd9 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd26};
          5'd10 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd26};
          5'd11 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd26};
          5'd12 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd26};
          5'd13 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd26};
          5'd14 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd26};
          5'd15 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd27};
          5'd16 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd27};
          5'd17 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd27};
          5'd18 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd27};
          5'd19 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd27};
          5'd20 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd27};
          5'd21 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd27};
          5'd22 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd27};
          5'd23 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd28};
          5'd24 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd28};
          5'd25 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd28};
          5'd26 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd28};
          5'd27 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd28};
          5'd28 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd28};
          5'd29 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd28};
          5'd30 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd28};
          default : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd29};
        endcase
      5'd22 :
        case (el_cntr_r)
          5'd0 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd29};
          5'd1 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd29};
          5'd2 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd29};
          5'd3 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd29};
          5'd4 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd29};
          5'd5 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd29};
          5'd6 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd29};
          5'd7 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd29};
          5'd8 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd29};
          5'd9 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd29};
          5'd10 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd29};
          5'd11 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd29};
          5'd12 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd29};
          5'd13 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd29};
          5'd14 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd29};
          5'd15 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd29};
          5'd16 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd29};
          5'd17 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd29};
          5'd18 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd29};
          5'd19 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd30};
          5'd20 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd30};
          5'd21 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd30};
          5'd22 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd30};
          5'd23 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd30};
          5'd24 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd30};
          5'd25 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd30};
          5'd26 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd30};
          5'd27 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd31};
          5'd28 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd31};
          5'd29 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd31};
          5'd30 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd31};
          default : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd31};
        endcase
      5'd23 :
        case (el_cntr_r)
          5'd0 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd31};
          5'd1 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd31};
          5'd2 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd31};
          5'd3 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd31};
          5'd4 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd31};
          5'd5 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd31};
          5'd6 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd31};
          5'd7 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd31};
          5'd8 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd31};
          5'd9 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd31};
          5'd10 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd31};
          5'd11 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd31};
          5'd12 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd31};
          5'd13 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd31};
          5'd14 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd31};
          5'd15 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd31};
          5'd16 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd31};
          5'd17 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd31};
          5'd18 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd31};
          5'd19 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd31};
          5'd20 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd31};
          5'd21 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd31};
          5'd22 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd31};
          5'd23 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd31};
          5'd24 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd31};
          5'd25 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd31};
          5'd26 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd31};
          5'd27 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd31};
          5'd28 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd31};
          5'd29 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd31};
          5'd30 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd31};
          default : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd31};
        endcase
      default :
        case (el_cntr_r)
          5'd0 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd31};
          5'd1 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd31};
          5'd2 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd31};
          5'd3 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd31};
          5'd4 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd31};
          5'd5 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd31};
          5'd6 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd31};
          5'd7 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd31};
          5'd8 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd31};
          5'd9 : {recsize, comprdata_size, sms_num} = {4'd9, 4'd8, 5'd31};
          5'd10 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd31};
          5'd11 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd31};
          5'd12 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd31};
          5'd13 : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd31};
          default : {recsize, comprdata_size, sms_num} = {4'd7, 4'd6, 5'd31};
        endcase
    endcase
  end

always @(group_cntr_r or el_cntr_r)
  begin
    case ({group_cntr_r, el_cntr_r})
      {5'd0, 5'd21},
      {5'd3, 5'd15},
      {5'd6, 5'd9},
      {5'd7, 5'd14},
      {5'd8, 5'd4},
      {5'd8, 5'd26},
      {5'd9, 5'd16},
      {5'd10, 5'd6},
      {5'd11, 5'd26},
      {5'd13, 5'd14},
      {5'd13, 5'd22},
      {5'd14, 5'd10},
      {5'd15, 5'd6},
      {5'd15, 5'd14},
      {5'd15, 5'd22},
      {5'd16, 5'd10},
      {5'd17, 5'd6},
      {5'd17, 5'd14},
      {5'd17, 5'd22},
      {5'd18, 5'd10},
      {5'd19, 5'd6},
      {5'd19, 5'd14},
      {5'd19, 5'd22},
      {5'd20, 5'd10},
      {5'd20, 5'd18},
      {5'd20, 5'd26},
      {5'd21, 5'd14},
      {5'd21, 5'd22},
      {5'd21, 5'd30},
      {5'd22, 5'd18},
      {5'd22, 5'd26},
      {5'd24, 5'd14} : dummy_flop = 1'b1;
      default : dummy_flop = 1'b0;
    endcase
  end



always @(posedge wrck or negedge rst_sfbp)
  if (!rst_sfbp)
    begin
      state_r           <=  6'd0;
      el_cntr_r         <=  5'd0;
      group_cntr_r      <=  5'd0;
      shiftcntr_r       <=  4'd0;
      comprshiftcntr_r  <=  8'd0;
      chain_select_r    <=  3'd0;
      addrbuf_r         <=  6'd0;
      recen_flag_r      <=  2'd0;
      addr_scan_flag_r  <=  1'b0;
      dummy_flag_r      <=  1'b0;
      errflag_r         <=  1'd0;
      addr_matched_r    <=  1'b0;
      overflow_r        <=  1'd0;
      sms_sc_en_r       <=  1'b0;
      compr_sc_en_r     <=  1'b0;
      compr_mode_r      <=  1'b0;
      sh_rep_en_bit_r   <=  1'b0;
      shift_addrbuf_r   <=  1'b0;
      seek_flag_r       <=  1'b0;
      repair_en_r       <=  1'b0;
      flag_ext_rot_r    <=  1'b0;
      blk_ld_dummy_r    <=  1'b0;
      int_rst_r         <=  1'b0;
      sms_repair_r      <=  32'd0;
      sms_cntr_r        <=  5'd0;
      ret_check_dummy_in_r  <=  1'b0;
      skip_mux_shift_r   <=  1'b0;
    end
  else
    begin
      state_r           <=  state_nxt;
      el_cntr_r         <=  el_cntr_nxt;
      group_cntr_r      <=  group_cntr_nxt;
      shiftcntr_r       <=  shiftcntr_nxt;
      comprshiftcntr_r  <=  comprshiftcntr_nxt;
      chain_select_r    <=  chain_select_nxt;
      addrbuf_r         <=  addrbuf_nxt;
      recen_flag_r      <=  recen_flag_nxt;
      addr_scan_flag_r  <=  addr_scan_flag_nxt;
      dummy_flag_r      <=  dummy_flag_nxt;
      errflag_r         <=  errflag_nxt;
      addr_matched_r    <=  addr_matched_nxt;
      overflow_r        <=  overflow_nxt;
      sms_sc_en_r       <=  sms_sc_en_nxt;
      compr_sc_en_r     <=  compr_sc_en_nxt;
      compr_mode_r      <=  compr_mode_nxt;
      sh_rep_en_bit_r   <=  sh_rep_en_bit_nxt;
      shift_addrbuf_r   <=  shift_addrbuf_nxt;
      seek_flag_r       <=  seek_flag_nxt;
      repair_en_r       <=  repair_en_nxt;
      flag_ext_rot_r    <=  flag_ext_rot_nxt;
      blk_ld_dummy_r    <=  blk_ld_dummy_nxt;
      int_rst_r         <=  int_rst;
      sms_repair_r      <=  sms_repair_nxt;
      sms_cntr_r        <=  sms_cntr_nxt;
      ret_check_dummy_in_r <=  ret_check_dummy_in_nxt;
      skip_mux_shift_r   <=  skip_mux_shift_nxt;
    end

endmodule

