//================================================================================================================================
//
// Copyright             : Trade secret of ATI Technologies, Inc.
//                         Copyright 2004-...., ATI Technologies, Inc., (unpublished) 
//                         All rights reserved.  This notice is intended as a precaution against
//                         inadvertent publication and does not imply publication or any waiver
//                         of confidentiality.  The year included in the foregoing notice is the
//                         year of creation of the work.
//
// Description           : Parameterized fine grained clock gating for L/Q/P Type 1 or 2 port SRAM. GFX10 feature 10.307 [was 10.260] fgcg_srams
//
//================================================================================================================================

module sram_fgcg_cfg
 #(
   parameter NUM_PORTS = 1, // Number of ports: 0=mem1p, 1=mem2p
   parameter MEM_TYPE = 0 ) // SRAM type: 0=L type, 1=Q type, 2=P type
  
  (// Outputs
   output wire 	       CLKA_gated,
   // Inputs
   input 	       srst,
   input 	       CLKA,
   input 	       RM_VALID,
   input 	       MEA,       // connect to ME for mem1p
   input               MEB,       // tie to 1'b0 for mem1p
   input 	       PIPEMEB ); // tie to 1'b0 for L type

   reg q0_me;
   reg q0_pipemeb;
   wire me_int;
   wire all_me_int;
   reg pipemeb_int;

   assign me_int = NUM_PORTS ? MEB : MEA;

   assign all_me_int = NUM_PORTS ? (MEB | MEA) : MEA;
  
   always @* begin
      pipemeb_int = q0_me;
      case (MEM_TYPE)
	0: pipemeb_int = q0_me;                // L type - internal PIPEMEB which is registered MEB
	1: pipemeb_int = PIPEMEB | q0_pipemeb; // Q type - external registered PIPEMEB
	2: pipemeb_int = PIPEMEB;              // P type - external nonregistered PIPEMEB
	default: ;
      endcase // case (MEM_TYPE)
   end
   
   // register MEB for L type two cycle read, register PIPEMEB for Q type
   always @(posedge CLKA) begin
      if (srst) begin
	 /*AUTORESET*/
	 // Beginning of autoreset for uninitialized flops
	 q0_me <= 1'h0;
	 q0_pipemeb <= 1'h0;
	 // End of automatics
      end
      else begin
	 q0_me <= me_int;
	 q0_pipemeb <= PIPEMEB;
      end
   end

   // RM_VALID allows dynamic RM feature, srst (~resetb) allows Read Margin registers time to set up
   wire clk_en = srst | RM_VALID | all_me_int | pipemeb_int;
   wire dft_te_wire;

   ati_TE_driver u_TE_driver
     (.TEST_CLK_EN(1'b0),
      .DFT_TE(dft_te_wire) );

   ati_clock_gate u_ati_clock_gate
     (// Outputs
      .clk_out				(CLKA_gated),
      // Inputs
      .clk_src				(CLKA),
      .TE				(dft_te_wire),
      /*AUTOINST*/
      // Inputs
      .clk_en				(clk_en));

//added assertion
`include "sram_fgcg_cfg_cov.vh" 

endmodule // sram_fgcg_cfg

// Local Variables:
// verilog-library-directories:("." "./.." "./../../../import/rtllib_soc15/src/rtl/variant/tsmc7n/")
// End: