module sram_fgcg_cfg_cov
(   //input to sram_fgcg_cfg
    CLKA, 
    srst,     
    MEA,
    MEB,
    RM_VALID, 
    CLKA_gated,
    clk_en,
    all_me_int
);
input CLKA;
input srst;
input MEA;
input MEB;
input RM_VALID;
input CLKA_gated;
input clk_en;
input all_me_int;

//////////////////////////

//VCS coverage exclude_module
`ifdef VCS

`ifdef GFX_CG_W2E
reg enable;
initial begin
  if($test$plusargs("sram_fgcg_cfg_assert_enable"))
    enable = 1'b1;
  else
    enable = 1'b0;
end
reg cov_enable;
initial begin
  if($test$plusargs("sram_fgcg_cfg_cov_enable"))
    cov_enable = 1'b1;
  else
    cov_enable = 1'b0;
end

reg hier_enable;
string Hier;
initial begin
  hier_enable = 1'b1;
  $sformat(Hier, "%m");
//`ifdef CG_WORKAROUND_CHECKER
//for(integer i=0;i<Hier.len();i++) 
//begin 
//    if((Hier.substr(i,i+2)==".cb") || (Hier.substr(i,i+2)==".db") || (Hier.substr(i,i+2)=="ge2") || (Hier.substr(i,i+2)=="rmi")
//        || (Hier.substr(i,i+2)==".td") 
//        || (Hier.substr(i,i+3)==".cpc") ||  (Hier.substr(i,i+3)==".cpf") ||   (Hier.substr(i,i+3)==".cpg") ||  (Hier.substr(i,i+4)==".grbm")
//     )     
//	begin 
//           hier_enable=1'b0; 
//           `MSGIO_DISPLAY("%m: WARNING: sram_fgcg_cfg assertion is waived via hierarchy");
//           break; 
//    end
//end
//`endif
end

wire tb_bpm_fgcg_override;
wire tb_static_clk;
assign tb_bpm_fgcg_override = tb.clock_gating_monitor.bpm_fgcg_override;
assign tb_static_clk = tb.clock_gating_monitor.static_clk;


reg RM_VALID_activated;
initial RM_VALID_activated = 0;
always @(posedge tb_static_clk) if(RM_VALID == 1'b1) RM_VALID_activated <= 1;
final if((RM_VALID_activated == 0) && enable && hier_enable) `MSGIO_DISPLAY("%m ERROR: RM_VALID never activated");

//override coverage check
reg RM_VALID_pos_activated;
reg RM_VALID_1d;
initial RM_VALID_pos_activated = 0;
initial RM_VALID_1d = 1;
always @(posedge tb_static_clk) RM_VALID_1d <= RM_VALID;
always @(posedge tb_static_clk) RM_VALID_pos_activated <= ((RM_VALID & (~RM_VALID_1d)) | RM_VALID_pos_activated);
final if((RM_VALID_pos_activated == 0) && enable && hier_enable) `MSGIO_DISPLAY("%m ERROR: RM_VALID never change from 0 to 1, override coverage check");

//override -> clock active, sometime clk source is gated by CGCG or MGCG
gated_clk_active :  assert property (@(negedge tb_static_clk) disable iff((~enable)|| (~hier_enable)) ((clk_en) |=> (CLKA_gated | (~CLKA)))); 
//bpm fgcg override -> clock active
tb_gated_clk_active :     assert property (@(negedge tb_static_clk) disable iff((~enable) || (~hier_enable)) ((tb_bpm_fgcg_override) |=> ##[0:90] (CLKA_gated | (~CLKA)))); 
//bpm fgcg override -> clock_en active
tb_gated_clk_en_active :  assert property (@(negedge tb_static_clk) disable iff((~enable) || (~hier_enable)) ((tb_bpm_fgcg_override) |-> ##[0:90] (clk_en))); 
tb_gated_clk_en_rose :  assert property (@(posedge tb_static_clk) disable iff((~enable) || (~hier_enable)) ((($rose(tb_bpm_fgcg_override)) & (~all_me_int)) |-> ##[0:90] ($rose(clk_en)))); 

//only one case to check the sram_fgcg_cfg ov 
tb_gated_clk_active_cov :     assert property (@(negedge tb_static_clk) disable iff((~enable) || (~hier_enable) || (~cov_enable)) ((tb_bpm_fgcg_override) |=> ##[0:90] (CLKA_gated))); 

`endif //!`ifdef GFX_CG_W2E

`else // !`ifdef VCS

initial begin
  $display("%m at %0t: src/rtl/shared/sram_fgcg_cfg_cov.sv: SVA not yet supported in NCSIM in wekiva scripts", $realtime);
end

`endif // VCS


endmodule // sram_fgcg_cfg_cov
