//================================================================================================================================
//
// Copyright             : Trade secret of ATI Technologies, Inc.
//                         Copyright 2004-...., ATI Technologies, Inc., (unpublished) 
//                         All rights reserved.  This notice is intended as a precaution against
//                         inadvertent publication and does not imply publication or any waiver
//                         of confidentiality.  The year included in the foregoing notice is the
//                         year of creation of the work.
//
// Description           : Generic fine grained clock gating for L/Q/P Type 2 port SRAM for GFX10 feature 10.307 [was 10.260] fgcg_srams
//
//================================================================================================================================

module sram_mem2p_fgcg
  #(parameter MEM_TYPE = 0 ) // SRAM type: 0=L type, 1=Q type, 2=P type
  
  (// Outputs
   output wire CLKA_gated,
   // Inputs
   input       srst,
   input       CLKA,
   input       RM_VALID,
   input       MEA,
   input       MEB,
   input       PIPEMEB );


   reg q0_MEB;
   reg q0_PIPEMEB;
   reg pipemeb_variable;

   always @* begin
     pipemeb_variable = q0_MEB;
     case (MEM_TYPE)
       0: pipemeb_variable = q0_MEB;               // L type - internal PIPEMEB which is registered MEB
       1: pipemeb_variable = PIPEMEB | q0_PIPEMEB; // Q type - external registered PIPEMEB
       2: pipemeb_variable = PIPEMEB;              // P type - external nonregistered PIPEMEB
       default: ;
     endcase // case (MEM_TYPE)
   end
   
   // register MEB for L type two cycle read, register PIPEMEB for Q type
   always @(posedge CLKA) begin
      if (srst) begin
	 /*AUTORESET*/
	 // Beginning of autoreset for uninitialized flops
	 q0_MEB <= 1'h0;
	 q0_PIPEMEB <= 1'h0;
	 // End of automatics
      end
      else begin
	 q0_MEB <= MEB;
	 q0_PIPEMEB <= PIPEMEB;
      end
   end

   // RM_VALID allows dynamic RM feature, srst (~resetb) allows Read Margin registers time to set up
   wire clk_en = srst | RM_VALID | MEA | MEB | pipemeb_variable;
   wire dft_te_wire;

   ati_TE_driver u_TE_driver
     (.TEST_CLK_EN(1'b0),
      .DFT_TE(dft_te_wire) );

   ati_clock_gate u_ati_clock_gate
     (// Outputs
      .clk_out				(CLKA_gated),
      // Inputs
      .clk_src				(CLKA),
      .TE				(dft_te_wire),
      /*AUTOINST*/
      // Inputs
      .clk_en				(clk_en));

//added assertion
`include "sram_fgcg_cov.vh" 

endmodule // sram_mem2p_fgcg

// Local Variables:
// verilog-library-directories:("." "./.." "./../../../import/rtllib_soc15/src/rtl/variant/tsmc7n/")
// End:
