/*---------------------------------------------------------------------*/
/*               Copyright(c) Synopsys, Inc.                           */
/*     All Rights reserved - Unpublished -rights reserved under        */
/*     the Copyright laws of the United States of America.             */
/*                                                                     */
/*  U.S. Patents: 7,093,156 B1 and 7,406,620 B2 (and more pending).    */
/*                                                                     */
/*  This file includes the Confidential information of Synopsys, Inc.. */
/*  The receiver of this Confidential Information shall not disclose   */
/*  it to any third party and shall protect its confidentiality by     */
/*  using the same degree of care, but not less then a reasonable      */
/*  degree of care, as the receiver uses to protect receiver's own     */
/*  Confidential Information.                                          */
/*  Licensee acknowledges and agrees that all output generated for     */
/*  Licensee by Synopsys, Inc. as described in the pertinent Program   */
/*  Schedule(s), or generated by Licensee through use of any Compiler  */
/*  licensed hereunder contains information that complies with the     */
/*  Virtual Component Identification Physical Tagging Standard (VCID)  */
/*  as maintained by the Virtual Socket Interface Alliance (VSIA).     */
/*  Such information may be expressed in GDSII Layer 63 or other such  */
/*  layer designated by the VSIA, hardware definition languages, or    */
/*  other formats.  Licensee is not authorized to alter or change any  */
/*  such information.                                                  */
/*---------------------------------------------------------------------*/
/*                                                                     */
/*  Built for linux64 and running on linux64.                          */
/*                                                                     */
/*  Software           : Rev: L-2016.09                                */
/*  Library Format     : Rev: 1.05.00                                  */
/*  Compiler Name      : vl00x000vpnnstsrv000sa30                      */
/*  Platform           : Linux2.6.18-308.8.2.el5                       */
/*                     : #1 SMP Tue May 29 11:54:17 EDT 2012x86_64     */
/*  Date of Generation : Tue Mar 28 21:21:11 EDT 2017                  */
/*                                                                     */
/*---------------------------------------------------------------------*/

`include "global_features.vh"

`ifdef ENV__FLAVOR__DGPU
    //DGPU
    `ifdef GC__SOC_ENV__1
        //SOC (skip, use SOC version)
    `else
        //IP

`resetall
// T I M E S C A L E ===============================================================================
`ifndef SMS_NO_TIMESCALE
    `timescale 1ns/1ps
`endif

module srv_sync_meta (
                  clk_in,
                  rst_in,
                  signal_in,
                  signal_out
                 );

input   clk_in;
input   rst_in;
input   signal_in;
output  signal_out;

reg   [1:0] dwsms_sync_met_signal_r, dwsms_sync_met_signal_nxt;

assign signal_out = dwsms_sync_met_signal_r[1];

always @(dwsms_sync_met_signal_r or signal_in)
  begin 
    dwsms_sync_met_signal_nxt = {dwsms_sync_met_signal_r[0], signal_in};
  end 

always @( posedge clk_in)
  begin
    if (rst_in)      
      dwsms_sync_met_signal_r <=  2'b00;
    else
      dwsms_sync_met_signal_r <=  dwsms_sync_met_signal_nxt;
  end

endmodule // srv_sync_meta

module srv_async_meta (
                  clk_in,
                  rst_in,
                  signal_in,
                  signal_out
                 );

input   clk_in;
input   rst_in;
input   signal_in;
output  signal_out;
/*
reg   [1:0] dwsms_async_met_signal_r, dwsms_async_met_signal_nxt;

assign signal_out = dwsms_async_met_signal_r[1];

always @(dwsms_async_met_signal_r or signal_in)
  begin
    dwsms_async_met_signal_nxt = {dwsms_async_met_signal_r[0], signal_in};
  end

always @( posedge clk_in or negedge rst_in)
  begin
    if (!rst_in)
      dwsms_async_met_signal_r <=  2'b00;
    else
      dwsms_async_met_signal_r <=  dwsms_async_met_signal_nxt;
  end
*/

techind_sync_icd
# (.depth(2), .asyncReset(1)
) sync_r (
    .SB(1'b1),
    .RB(rst_in),
    .FLUSH(1'b0),
`ifdef ABV
    .InputChkDis(1'b1), // always disable, ok to miss pulses here,
`endif
    .CKsrc(1'b0),
    .CKdst(clk_in),
    .EN(1'b1),
    .Dsrc(signal_in),
    .Qsrc(),
    .Qdst(signal_out),

    .SC1(1'b0),
    .SC2(1'b0),
    .SE(1'b0),
    .SI(1'b0),
    .SO()
);

endmodule // srv_async_meta 

module srv_2to1mux (
                    mux_in0,
                    mux_in1,
                    mux_sel,
                    mux_out
                    );


input mux_in0;
input mux_in1;
input mux_sel;
output mux_out;

reg mux_out;

always @(mux_sel or mux_in0 or mux_in1)
  begin
    case (mux_sel) // synopsys infer_mux infer_mux_override
      1'b0 : mux_out = mux_in0;
      default : mux_out = mux_in1;
    endcase
  end

endmodule // srv_2to1mux


module srv_sync_rst ( 
                     clk_in,
                     rst_in,
                     rst_out   
                       );

input  clk_in;
input  rst_in;
output rst_out;

reg [1:0] dwsms_rst_sync_r, dwsms_rst_sync_nxt;

assign rst_out = dwsms_rst_sync_r[1];

always @(dwsms_rst_sync_r or rst_in)
  begin
    dwsms_rst_sync_nxt = {dwsms_rst_sync_r[0], rst_in};
  end 

always @(posedge clk_in)
  begin
    dwsms_rst_sync_r <=  dwsms_rst_sync_nxt;
  end

endmodule // srv_sync_rst


module srv_async_rst ( 
                     clk_in,
                     rst_in,
                     rst_out   
                       );

input  clk_in;
input  rst_in;
output rst_out;

reg [1:0] dwsms_async_rst_r;

assign rst_out = dwsms_async_rst_r[1];

always @(posedge clk_in or negedge rst_in)
  begin
    if (!rst_in)
      dwsms_async_rst_r  <=  2'b00;
    else
      dwsms_async_rst_r <=  {dwsms_async_rst_r[0], 1'b1};
  end
 
endmodule // srv_async_rst


// posedge sinchronized pipe
module pipe_pos ( 
                     clk_in,
                     rst_in,
                     sig_in,
                     sig_out
                       );

input  clk_in;
input  rst_in;
input  sig_in;
output sig_out;

reg sig_out_r;
assign sig_out = sig_out_r;

always @(posedge clk_in or negedge rst_in)
  begin
    if (!rst_in)
      sig_out_r  <=  1'b0;
    else
      sig_out_r <=  sig_in;
  end
 
endmodule // pipe_pos

// posedge sinchronized pipe with blocking
module pipe_pos_blocked ( 
                        clk_in,
                        rst_in,
                        sig_in,
                        shift_in,
                        sig_out
                        );

input  clk_in;
input  rst_in;
input  sig_in;
input  shift_in;
output sig_out;

reg sig_out_r;
wire sig_nxt;
assign sig_nxt = shift_in ? sig_in : sig_out_r;
assign sig_out = sig_out_r;

 always @(posedge clk_in or negedge rst_in)
  begin
    if (!rst_in)
      sig_out_r <=  1'b0;
    else 
      sig_out_r <=  sig_nxt;    
  end
endmodule // pipe_pos_blocked

// negedge sinchronized pipe
module pipe_neg ( 
                     clk_in,
                     rst_in,
                     sig_in,
                     sig_out
                       );

input  clk_in;
input  rst_in;
input  sig_in;
output sig_out;

reg sig_out_nr;
assign sig_out = sig_out_nr;

always @(negedge clk_in or negedge rst_in)
  begin
    if (!rst_in)
      sig_out_nr  <=  1'b0;
    else
      sig_out_nr  <=  sig_in;
  end
 
endmodule // pipe_neg

// TAP to SERVER pipe1 (regular branch)
module pipe1_to_srv ( 
                     clk_in,
                     rst_in,
                     capture,
                     shift,
                     update,
                     select_jpc_wir,
                     select_jpc_wdr,
                     select_sms_wir,
                     select_sms_wdr,
                     tdi,
                     smsg_data,
                     capture_p,
                     shift_p,
                     update_n,
                     tdi_p,
                     smsg_data_p,
                     select_jpc_wir_p,
                     select_jpc_wdr_p,
                     select_sms_wir_p,
                     select_sms_wdr_p
                       );

input clk_in;
input rst_in;
input capture;
input shift;
input update;
input tdi;
input smsg_data;
input select_jpc_wir;
input select_jpc_wdr;
input select_sms_wir;
input select_sms_wdr;
output capture_p;
output shift_p;
output update_n;
output tdi_p;
output smsg_data_p;
output select_jpc_wir_p;
output select_jpc_wdr_p;
output select_sms_wir_p;
output select_sms_wdr_p;

pipe_pos U_capture_p (
	.clk_in(clk_in),
	.rst_in(rst_in),
	.sig_in(capture),
	.sig_out(capture_p)
);

pipe_pos U_shift_p (
	.clk_in(clk_in),
	.rst_in(rst_in),
	.sig_in(shift),
	.sig_out(shift_p)
);

pipe_neg U_update_n (
	.clk_in(clk_in),
	.rst_in(rst_in),
	.sig_in(update),
	.sig_out(update_n)
);

pipe_pos U_tdi_p (
	.clk_in(clk_in),
	.rst_in(rst_in),
	.sig_in(tdi),
	.sig_out(tdi_p)
);

pipe_pos U_smsg_data_p (
	.clk_in(clk_in),
	.rst_in(rst_in),
	.sig_in(smsg_data),
	.sig_out(smsg_data_p)
);
pipe_pos U_select_jpc_wir_p (
	.clk_in(clk_in),
	.rst_in(rst_in),
	.sig_in(select_jpc_wir),
	.sig_out(select_jpc_wir_p)
);
pipe_pos U_select_jpc_wdr_p (
	.clk_in(clk_in),
	.rst_in(rst_in),
	.sig_in(select_jpc_wdr),
	.sig_out(select_jpc_wdr_p)
);

pipe_pos U_select_sms_wir_p (
	.clk_in(clk_in),
	.rst_in(rst_in),
	.sig_in(select_sms_wir),
	.sig_out(select_sms_wir_p)
);

pipe_pos U_select_sms_wdr_p (
	.clk_in(clk_in),
	.rst_in(rst_in),
	.sig_in(select_sms_wdr),
	.sig_out(select_sms_wdr_p)
);

endmodule // pipe1_to_srv (regular branch)


// TAP to SERVER pipe2 (regular branch)
module pipe2_to_srv ( 
                     clk_in,
                     rst_in,
                     capture,
                     shift,
                     update,
                     select_jpc_wir,
                     select_jpc_wdr,
                     select_sms_wir,
                     select_sms_wdr,
                     tdi,
                     smsg_data,
                     capture_n,
                     shift_n,
                     update_p,
                     tdi_n,
                     smsg_data_n,
                     select_jpc_wir_n,
                     select_jpc_wdr_n,
                     select_sms_wir_n,
                     select_sms_wdr_n
                       );

input clk_in;
input rst_in;
input capture;
input shift;
input update;
input select_jpc_wir;
input select_jpc_wdr;
input select_sms_wir;
input select_sms_wdr;
input tdi;
input smsg_data;
output capture_n;
output shift_n;
output update_p;
output tdi_n;
output smsg_data_n;
output select_jpc_wir_n;
output select_jpc_wdr_n;
output select_sms_wir_n;
output select_sms_wdr_n;

pipe_neg U_capture_n (
	.clk_in(clk_in),
	.rst_in(rst_in),
	.sig_in(capture),
	.sig_out(capture_n)
);
pipe_neg U_shift_n (
	.clk_in(clk_in),
	.rst_in(rst_in),
	.sig_in(shift),
	.sig_out(shift_n)
);
pipe_pos U_update_p (
	.clk_in(clk_in),
	.rst_in(rst_in),
	.sig_in(update),
	.sig_out(update_p)
);
pipe_neg U_tdi_n (
	.clk_in(clk_in),
	.rst_in(rst_in),
	.sig_in(tdi),
	.sig_out(tdi_n)
);
pipe_neg U_smsg_data_n (
	.clk_in(clk_in),
	.rst_in(rst_in),
	.sig_in(smsg_data),
	.sig_out(smsg_data_n)
);
pipe_neg U_select_jpc_wir_n (
	.clk_in(clk_in),
	.rst_in(rst_in),
	.sig_in(select_jpc_wir),
	.sig_out(select_jpc_wir_n)
);
pipe_neg U_select_jpc_wdr_n (
	.clk_in(clk_in),
	.rst_in(rst_in),
	.sig_in(select_jpc_wdr),
	.sig_out(select_jpc_wdr_n)
);
pipe_neg U_select_sms_wir_n (
	.clk_in(clk_in),
	.rst_in(rst_in),
	.sig_in(select_sms_wir),
	.sig_out(select_sms_wir_n)
);
pipe_neg U_select_sms_wdr_n (
	.clk_in(clk_in),
	.rst_in(rst_in),
	.sig_in(select_sms_wdr),
	.sig_out(select_sms_wdr_n)
);

endmodule // pipe2_to_srv (regular branch)


// SERVER to SMS pipe1
module pipe1_to_sms ( 
                     clk_in,
                     rst_in,
                     capture,
                     shift,
                     update,
                     select_wir,
                     tdi,
                     sms_wso,
                     capture_p,
                     shift_p,
                     safe_shift,
                     update_n,
                     select_wir_p,
                     tdi_p,
                     sms_wso_n
                       );

input clk_in;
input rst_in;
input capture;
input shift;
input update;
input select_wir;
input tdi;
input sms_wso;
output capture_p;
output shift_p;
input safe_shift;
output update_n;
output select_wir_p;
output tdi_p;
output sms_wso_n;

pipe_pos U_capture_p (
	.clk_in(clk_in),
	.rst_in(rst_in),
	.sig_in(capture),
	.sig_out(capture_p)
);

pipe_pos U_shift_p (
	.clk_in(clk_in),
	.rst_in(rst_in),
	.sig_in(shift),
	.sig_out(shift_p)
);
pipe_pos U_srv_select_wir_p (
	.clk_in(clk_in),
	.rst_in(rst_in),
	.sig_in(select_wir),
	.sig_out(select_wir_p)
);
pipe_neg U_update_n (
	.clk_in(clk_in),
	.rst_in(rst_in),
	.sig_in(update),
	.sig_out(update_n)
);

pipe_pos_blocked U_tdi_p (
	.clk_in(clk_in),
	.rst_in(rst_in),
        .shift_in(safe_shift),
	.sig_in(tdi),
	.sig_out(tdi_p)
);

pipe_neg U_sms_wso_n (
	.clk_in(clk_in),
	.rst_in(rst_in),
	.sig_in(sms_wso),
	.sig_out(sms_wso_n)
);

endmodule // pipe1_to_sms


// SERVER to SMS pipe2
module pipe2_to_sms ( 
                     clk_in,
                     rst_in,
                     capture,
                     shift,
                     update,
                     select_wir,
                     tdi,
                     sms_wso,
                     capture_n,
                     shift_n,
                     update_p,
                     select_wir_n,
                     tdi_n,
                     sms_wso_p
                       );

input clk_in;
input rst_in;
input capture;
input shift;
input update;
input select_wir;
input tdi;
input sms_wso;
output capture_n;
output shift_n;
output update_p;
output select_wir_n;
output tdi_n;
output sms_wso_p;

pipe_neg U_capture_n (
	.clk_in(clk_in),
	.rst_in(rst_in),
	.sig_in(capture),
	.sig_out(capture_n)
);
pipe_neg U_shift_n (
	.clk_in(clk_in),
	.rst_in(rst_in),
	.sig_in(shift),
	.sig_out(shift_n)
);
pipe_neg U_srv_select_wir_n (
	.clk_in(clk_in),
	.rst_in(rst_in),
	.sig_in(select_wir),
	.sig_out(select_wir_n)
);
pipe_pos U_update_p (
	.clk_in(clk_in),
	.rst_in(rst_in),
	.sig_in(update),
	.sig_out(update_p)
);
pipe_neg U_tdi_n (
	.clk_in(clk_in),
	.rst_in(rst_in),
	.sig_in(tdi),
	.sig_out(tdi_n)
);

pipe_pos_blocked U_sms_wso_p (
	.clk_in(clk_in),
	.rst_in(rst_in),
        .shift_in(shift_n),
	.sig_in(sms_wso),
	.sig_out(sms_wso_p)
);

endmodule // pipe2_to_sms


    `endif  //GC__SOC_ENV__1
`else   //ENV__FLAVOR__DGPU
    //APU

`resetall
// T I M E S C A L E ===============================================================================
`ifndef SMS_NO_TIMESCALE
    `timescale 1ns/1ps
`endif

module srv_sync_meta (
                  clk_in,
                  rst_in,
                  signal_in,
                  signal_out
                 );

input   clk_in;
input   rst_in;
input   signal_in;
output  signal_out;

reg   [1:0] dwsms_sync_met_signal_r, dwsms_sync_met_signal_nxt;

assign signal_out = dwsms_sync_met_signal_r[1];

always @(dwsms_sync_met_signal_r or signal_in)
  begin 
    dwsms_sync_met_signal_nxt = {dwsms_sync_met_signal_r[0], signal_in};
  end 

always @( posedge clk_in)
  begin
    if (rst_in)      
      dwsms_sync_met_signal_r <=  2'b00;
    else
      dwsms_sync_met_signal_r <=  dwsms_sync_met_signal_nxt;
  end

endmodule // srv_sync_meta

module srv_async_meta (
                  clk_in,
                  rst_in,
                  signal_in,
                  signal_out
                 );

input   clk_in;
input   rst_in;
input   signal_in;
output  signal_out;
/*
reg   [1:0] dwsms_async_met_signal_r, dwsms_async_met_signal_nxt;

assign signal_out = dwsms_async_met_signal_r[1];

always @(dwsms_async_met_signal_r or signal_in)
  begin
    dwsms_async_met_signal_nxt = {dwsms_async_met_signal_r[0], signal_in};
  end

always @( posedge clk_in or negedge rst_in)
  begin
    if (!rst_in)
      dwsms_async_met_signal_r <=  2'b00;
    else
      dwsms_async_met_signal_r <=  dwsms_async_met_signal_nxt;
  end
*/

techind_sync_icd
# (.depth(2), .asyncReset(1)
) sync_r (
    .SB(1'b1),
    .RB(rst_in),
    .FLUSH(1'b0),
`ifdef ABV
    .InputChkDis(1'b1), // always disable, ok to miss pulses here,
`endif
    .CKsrc(1'b0),
    .CKdst(clk_in),
    .EN(1'b1),
    .Dsrc(signal_in),
    .Qsrc(),
    .Qdst(signal_out),

    .SC1(1'b0),
    .SC2(1'b0),
    .SE(1'b0),
    .SI(1'b0),
    .SO()
);

endmodule // srv_async_meta 

module srv_2to1mux (
                    mux_in0,
                    mux_in1,
                    mux_sel,
                    mux_out
                    );


input mux_in0;
input mux_in1;
input mux_sel;
output mux_out;

reg mux_out;

always @(mux_sel or mux_in0 or mux_in1)
  begin
    case (mux_sel) // synopsys infer_mux infer_mux_override
      1'b0 : mux_out = mux_in0;
      default : mux_out = mux_in1;
    endcase
  end

endmodule // srv_2to1mux


module srv_sync_rst ( 
                     clk_in,
                     rst_in,
                     rst_out   
                       );

input  clk_in;
input  rst_in;
output rst_out;

reg [1:0] dwsms_rst_sync_r, dwsms_rst_sync_nxt;

assign rst_out = dwsms_rst_sync_r[1];

always @(dwsms_rst_sync_r or rst_in)
  begin
    dwsms_rst_sync_nxt = {dwsms_rst_sync_r[0], rst_in};
  end 

always @(posedge clk_in)
  begin
    dwsms_rst_sync_r <=  dwsms_rst_sync_nxt;
  end

endmodule // srv_sync_rst


module srv_async_rst ( 
                     clk_in,
                     rst_in,
                     rst_out   
                       );

input  clk_in;
input  rst_in;
output rst_out;

reg [1:0] dwsms_async_rst_r;

assign rst_out = dwsms_async_rst_r[1];

always @(posedge clk_in or negedge rst_in)
  begin
    if (!rst_in)
      dwsms_async_rst_r  <=  2'b00;
    else
      dwsms_async_rst_r <=  {dwsms_async_rst_r[0], 1'b1};
  end
 
endmodule // srv_async_rst


// posedge sinchronized pipe
module pipe_pos ( 
                     clk_in,
                     rst_in,
                     sig_in,
                     sig_out
                       );

input  clk_in;
input  rst_in;
input  sig_in;
output sig_out;

reg sig_out_r;
assign sig_out = sig_out_r;

always @(posedge clk_in or negedge rst_in)
  begin
    if (!rst_in)
      sig_out_r  <=  1'b0;
    else
      sig_out_r <=  sig_in;
  end
 
endmodule // pipe_pos

// posedge sinchronized pipe with blocking
module pipe_pos_blocked ( 
                        clk_in,
                        rst_in,
                        sig_in,
                        shift_in,
                        sig_out
                        );

input  clk_in;
input  rst_in;
input  sig_in;
input  shift_in;
output sig_out;

reg sig_out_r;
wire sig_nxt;
assign sig_nxt = shift_in ? sig_in : sig_out_r;
assign sig_out = sig_out_r;

 always @(posedge clk_in or negedge rst_in)
  begin
    if (!rst_in)
      sig_out_r <=  1'b0;
    else 
      sig_out_r <=  sig_nxt;    
  end
endmodule // pipe_pos_blocked

// negedge sinchronized pipe
module pipe_neg ( 
                     clk_in,
                     rst_in,
                     sig_in,
                     sig_out
                       );

input  clk_in;
input  rst_in;
input  sig_in;
output sig_out;

reg sig_out_nr;
assign sig_out = sig_out_nr;

always @(negedge clk_in or negedge rst_in)
  begin
    if (!rst_in)
      sig_out_nr  <=  1'b0;
    else
      sig_out_nr  <=  sig_in;
  end
 
endmodule // pipe_neg

// TAP to SERVER pipe1 (regular branch)
module pipe1_to_srv ( 
                     clk_in,
                     rst_in,
                     capture,
                     shift,
                     update,
                     select_jpc_wir,
                     select_jpc_wdr,
                     select_sms_wir,
                     select_sms_wdr,
                     tdi,
                     smsg_data,
                     capture_p,
                     shift_p,
                     update_n,
                     tdi_p,
                     smsg_data_p,
                     select_jpc_wir_p,
                     select_jpc_wdr_p,
                     select_sms_wir_p,
                     select_sms_wdr_p
                       );

input clk_in;
input rst_in;
input capture;
input shift;
input update;
input tdi;
input smsg_data;
input select_jpc_wir;
input select_jpc_wdr;
input select_sms_wir;
input select_sms_wdr;
output capture_p;
output shift_p;
output update_n;
output tdi_p;
output smsg_data_p;
output select_jpc_wir_p;
output select_jpc_wdr_p;
output select_sms_wir_p;
output select_sms_wdr_p;

pipe_pos U_capture_p (
	.clk_in(clk_in),
	.rst_in(rst_in),
	.sig_in(capture),
	.sig_out(capture_p)
);

pipe_pos U_shift_p (
	.clk_in(clk_in),
	.rst_in(rst_in),
	.sig_in(shift),
	.sig_out(shift_p)
);

pipe_neg U_update_n (
	.clk_in(clk_in),
	.rst_in(rst_in),
	.sig_in(update),
	.sig_out(update_n)
);

pipe_pos U_tdi_p (
	.clk_in(clk_in),
	.rst_in(rst_in),
	.sig_in(tdi),
	.sig_out(tdi_p)
);

pipe_pos U_smsg_data_p (
	.clk_in(clk_in),
	.rst_in(rst_in),
	.sig_in(smsg_data),
	.sig_out(smsg_data_p)
);
pipe_pos U_select_jpc_wir_p (
	.clk_in(clk_in),
	.rst_in(rst_in),
	.sig_in(select_jpc_wir),
	.sig_out(select_jpc_wir_p)
);
pipe_pos U_select_jpc_wdr_p (
	.clk_in(clk_in),
	.rst_in(rst_in),
	.sig_in(select_jpc_wdr),
	.sig_out(select_jpc_wdr_p)
);

pipe_pos U_select_sms_wir_p (
	.clk_in(clk_in),
	.rst_in(rst_in),
	.sig_in(select_sms_wir),
	.sig_out(select_sms_wir_p)
);

pipe_pos U_select_sms_wdr_p (
	.clk_in(clk_in),
	.rst_in(rst_in),
	.sig_in(select_sms_wdr),
	.sig_out(select_sms_wdr_p)
);

endmodule // pipe1_to_srv (regular branch)


// TAP to SERVER pipe2 (regular branch)
module pipe2_to_srv ( 
                     clk_in,
                     rst_in,
                     capture,
                     shift,
                     update,
                     select_jpc_wir,
                     select_jpc_wdr,
                     select_sms_wir,
                     select_sms_wdr,
                     tdi,
                     smsg_data,
                     capture_n,
                     shift_n,
                     update_p,
                     tdi_n,
                     smsg_data_n,
                     select_jpc_wir_n,
                     select_jpc_wdr_n,
                     select_sms_wir_n,
                     select_sms_wdr_n
                       );

input clk_in;
input rst_in;
input capture;
input shift;
input update;
input select_jpc_wir;
input select_jpc_wdr;
input select_sms_wir;
input select_sms_wdr;
input tdi;
input smsg_data;
output capture_n;
output shift_n;
output update_p;
output tdi_n;
output smsg_data_n;
output select_jpc_wir_n;
output select_jpc_wdr_n;
output select_sms_wir_n;
output select_sms_wdr_n;

pipe_neg U_capture_n (
	.clk_in(clk_in),
	.rst_in(rst_in),
	.sig_in(capture),
	.sig_out(capture_n)
);
pipe_neg U_shift_n (
	.clk_in(clk_in),
	.rst_in(rst_in),
	.sig_in(shift),
	.sig_out(shift_n)
);
pipe_pos U_update_p (
	.clk_in(clk_in),
	.rst_in(rst_in),
	.sig_in(update),
	.sig_out(update_p)
);
pipe_neg U_tdi_n (
	.clk_in(clk_in),
	.rst_in(rst_in),
	.sig_in(tdi),
	.sig_out(tdi_n)
);
pipe_neg U_smsg_data_n (
	.clk_in(clk_in),
	.rst_in(rst_in),
	.sig_in(smsg_data),
	.sig_out(smsg_data_n)
);
pipe_neg U_select_jpc_wir_n (
	.clk_in(clk_in),
	.rst_in(rst_in),
	.sig_in(select_jpc_wir),
	.sig_out(select_jpc_wir_n)
);
pipe_neg U_select_jpc_wdr_n (
	.clk_in(clk_in),
	.rst_in(rst_in),
	.sig_in(select_jpc_wdr),
	.sig_out(select_jpc_wdr_n)
);
pipe_neg U_select_sms_wir_n (
	.clk_in(clk_in),
	.rst_in(rst_in),
	.sig_in(select_sms_wir),
	.sig_out(select_sms_wir_n)
);
pipe_neg U_select_sms_wdr_n (
	.clk_in(clk_in),
	.rst_in(rst_in),
	.sig_in(select_sms_wdr),
	.sig_out(select_sms_wdr_n)
);

endmodule // pipe2_to_srv (regular branch)


// SERVER to SMS pipe1
module pipe1_to_sms ( 
                     clk_in,
                     rst_in,
                     capture,
                     shift,
                     update,
                     select_wir,
                     tdi,
                     sms_wso,
                     capture_p,
                     shift_p,
                     safe_shift,
                     update_n,
                     select_wir_p,
                     tdi_p,
                     sms_wso_n
                       );

input clk_in;
input rst_in;
input capture;
input shift;
input update;
input select_wir;
input tdi;
input sms_wso;
output capture_p;
output shift_p;
input safe_shift;
output update_n;
output select_wir_p;
output tdi_p;
output sms_wso_n;

pipe_pos U_capture_p (
	.clk_in(clk_in),
	.rst_in(rst_in),
	.sig_in(capture),
	.sig_out(capture_p)
);

pipe_pos U_shift_p (
	.clk_in(clk_in),
	.rst_in(rst_in),
	.sig_in(shift),
	.sig_out(shift_p)
);
pipe_pos U_srv_select_wir_p (
	.clk_in(clk_in),
	.rst_in(rst_in),
	.sig_in(select_wir),
	.sig_out(select_wir_p)
);
pipe_neg U_update_n (
	.clk_in(clk_in),
	.rst_in(rst_in),
	.sig_in(update),
	.sig_out(update_n)
);

pipe_pos_blocked U_tdi_p (
	.clk_in(clk_in),
	.rst_in(rst_in),
        .shift_in(safe_shift),
	.sig_in(tdi),
	.sig_out(tdi_p)
);

pipe_neg U_sms_wso_n (
	.clk_in(clk_in),
	.rst_in(rst_in),
	.sig_in(sms_wso),
	.sig_out(sms_wso_n)
);

endmodule // pipe1_to_sms


// SERVER to SMS pipe2
module pipe2_to_sms ( 
                     clk_in,
                     rst_in,
                     capture,
                     shift,
                     update,
                     select_wir,
                     tdi,
                     sms_wso,
                     capture_n,
                     shift_n,
                     update_p,
                     select_wir_n,
                     tdi_n,
                     sms_wso_p
                       );

input clk_in;
input rst_in;
input capture;
input shift;
input update;
input select_wir;
input tdi;
input sms_wso;
output capture_n;
output shift_n;
output update_p;
output select_wir_n;
output tdi_n;
output sms_wso_p;

pipe_neg U_capture_n (
	.clk_in(clk_in),
	.rst_in(rst_in),
	.sig_in(capture),
	.sig_out(capture_n)
);
pipe_neg U_shift_n (
	.clk_in(clk_in),
	.rst_in(rst_in),
	.sig_in(shift),
	.sig_out(shift_n)
);
pipe_neg U_srv_select_wir_n (
	.clk_in(clk_in),
	.rst_in(rst_in),
	.sig_in(select_wir),
	.sig_out(select_wir_n)
);
pipe_pos U_update_p (
	.clk_in(clk_in),
	.rst_in(rst_in),
	.sig_in(update),
	.sig_out(update_p)
);
pipe_neg U_tdi_n (
	.clk_in(clk_in),
	.rst_in(rst_in),
	.sig_in(tdi),
	.sig_out(tdi_n)
);

pipe_pos_blocked U_sms_wso_p (
	.clk_in(clk_in),
	.rst_in(rst_in),
        .shift_in(shift_n),
	.sig_in(sms_wso),
	.sig_out(sms_wso_p)
);

endmodule // pipe2_to_sms

`endif  //ENV__FLAVOR__DGPU

