// **********************************************************************************
//                              -*- Mode: Verilog -*-
// Filename        : tbmod_cgts_cntl_init.v
// Description     : For use in a block level testbench to program the cgts_local_cntl unit.
// Author          : Donald Lee
// Created On      : 04/17/2007
// **********************************************************************************

module tbmod_cgts_cntl_init(
  // Outputs
  ls_cntl_out,
  cntl_ls_override_out,
  cntl_out,
  busy_out,

  // Inputs
  srst_in,
  sclk_in,
  mode_en_in,
  mode_in,
  simd_en_in,
  qp_en_in,
  harvest_repair_en_in,
  harvest_repair_in,
  override_in,
  delay_mult_in,
  base_in
);

  input        srst_in;
  input        sclk_in;
  input        mode_en_in;
  input  [1:0] mode_in;
  input        simd_en_in;
  input        qp_en_in;
  input        harvest_repair_en_in;
  input  [3:0] harvest_repair_in;
  input        override_in;
  input  [6:0] delay_mult_in;
  input        base_in;

  output        ls_cntl_out;
  output        cntl_ls_override_out;
  output  [4:0] cntl_out;
  output        busy_out;


  wire         ctrl_program;
  wire         ctrl_read;
  wire         ctrl_off_seq_en;
  wire         ctrl_on_seq_en;
  wire         srst_pulse;
  wire         program_pulse;
  wire         programming;
  reg          srst_d1;
  reg          ctrl_program_d1;
  reg          ctrl_off_seq_en_d1;
  reg          ctrl_on_seq_en_d1;
  wire   [5:0] ge;
  wire         pgm_seq_cnt_incr;

  reg          busy_d1;
  reg          busy_d2;
  reg          busy_d3;
  reg          busy_d4;
  reg    [4:0] pgm_seq_cnt;
  reg    [4:0] pgm_sequence;
  reg          pgm_done;
  reg    [4:0] on_off_sequence;
  reg          on_seq_pulse;
  reg          off_seq_pulse;


  

  assign ctrl_on_seq_en  = mode_en_in & (mode_in == 2'b00);
  assign ctrl_off_seq_en = mode_en_in & (mode_in == 2'b01);
  assign ctrl_program    = mode_en_in & (mode_in == 2'b10);
  assign ctrl_read       = mode_en_in & (mode_in == 2'b11);

  
  assign srst_pulse = ~srst_in & srst_d1;
  assign program_pulse = ctrl_program & ~ctrl_program_d1;


  //----------------------------------------------------------
  // Generate the gate enables
  //----------------------------------------------------------
  //  r7xx
  //assign ge[5] = 1'b0;  // always 0
  //assign ge[4] = (harvest_repair_en_in ? (override_in | simd_en_in | ~harvest_repair_in[3]) : 1'b0);
  //assign ge[3] = (harvest_repair_en_in ? (override_in | simd_en_in | ~harvest_repair_in[2]) : 1'b0);
  //assign ge[2] = (harvest_repair_en_in ? (override_in | simd_en_in | ~harvest_repair_in[1]) : 1'b0);
  //assign ge[1] = override_in | (harvest_repair_en_in ? (simd_en_in | ~harvest_repair_in[0]) : (qp_en_in | simd_en_in));
  //assign ge[0] = 1'b1;  // always 1

  //assign ge[5] = override_in | simd_en_in; 
  //assign ge[4] = 1'b1;
  //assign ge[3] = 1'b0;
  //assign ge[2] = (harvest_repair_en_in ? (override_in | simd_en_in) : 1'b0);
  //assign ge[1] = override_in | (harvest_repair_en_in ? (simd_en_in) : (qp_en_in | simd_en_in));
  //assign ge[0] = 1'b1;  // always 1

  assign ge[5] = override_in | (harvest_repair_en_in ? 1'b0 : simd_en_in);
  assign ge[4] = (harvest_repair_en_in ? (override_in | ~harvest_repair_in[3]) : 1'b1);
  assign ge[3] = (harvest_repair_en_in ? (override_in | ~harvest_repair_in[2]) : 1'b0);
  assign ge[2] = (harvest_repair_en_in ? (override_in | ~harvest_repair_in[1]) : simd_en_in);
  assign ge[1] = override_in | (harvest_repair_en_in ? ~harvest_repair_in[0] : (qp_en_in | simd_en_in));
  assign ge[0] = 1'b1;  // always 1

  //----------------------------------------------------------
  // Step through program sequence based on a program count.
  //----------------------------------------------------------
  assign programming = (pgm_seq_cnt != 5'h00);
  assign pgm_seq_cnt_incr = (srst_pulse | program_pulse) | (~pgm_done & programming);

  always @(posedge sclk_in)
  begin
    srst_d1            <= srst_in;
    ctrl_on_seq_en_d1  <= ctrl_on_seq_en;
    ctrl_off_seq_en_d1 <= ctrl_off_seq_en;
    ctrl_program_d1    <= ctrl_program;
    busy_d1 <= programming;
    busy_d2 <= busy_d1;
    busy_d3 <= busy_d2;
    busy_d4 <= busy_d3;

    if (srst_in | pgm_done)
      pgm_seq_cnt <= 5'h0;
    else if (pgm_seq_cnt_incr)
      pgm_seq_cnt <= pgm_seq_cnt + 1'b1;
  end

  always @(*)
  begin
    case (pgm_seq_cnt)
                             //{override, program, state, valid, data}
      5'h15:  pgm_sequence = {1'b0, 1'b1, 1'b0, 1'b1, 1'b0};              // send a block lock command
      5'h14:  pgm_sequence = {1'b0, 1'b1, 1'b0, 1'b0, 1'b0};              // shift simdbusy_soft_override
      5'h13:  pgm_sequence = {1'b0, 1'b1, 1'b0, 1'b0, 1'b0};              // shift ls_soft_override
      5'h12:  pgm_sequence = {1'b0, 1'b1, 1'b0, 1'b0, 1'b0};              // shift cg_soft_override
      5'h11:  pgm_sequence = {1'b0, 1'b1, 1'b0, 1'b0, 1'b0};              // shift alu override 1  
      5'h10:  pgm_sequence = {1'b0, 1'b1, 1'b0, 1'b0, 1'b0};              // shift alu override 0
      5'h0f:  pgm_sequence = {1'b0, 1'b1, 1'b0, 1'b0, ge[5]};             // shift a bit of gate enable
      5'h0e:  pgm_sequence = {1'b0, 1'b1, 1'b0, 1'b0, ge[4]};             // shift a bit of gate enable
      5'h0d:  pgm_sequence = {1'b0, 1'b1, 1'b0, 1'b0, ge[3]};             // shift a bit of gate enable
      5'h0c:  pgm_sequence = {1'b0, 1'b1, 1'b0, 1'b0, ge[2]};             // shift a bit of gate enable
      5'h0b:  pgm_sequence = {1'b0, 1'b1, 1'b0, 1'b0, ge[1]};             // shift a bit of gate enable
      5'h0a:  pgm_sequence = {1'b0, 1'b1, 1'b0, 1'b0, ge[0]};             // shift a bit of gate enable
      5'h09:  pgm_sequence = {1'b0, 1'b1, 1'b0, 1'b0, base_in};           // shift base=0
      5'h08:  pgm_sequence = {1'b0, 1'b1, 1'b0, 1'b0, delay_mult_in[6]};  // shift a bit of delay_mult
      5'h07:  pgm_sequence = {1'b0, 1'b1, 1'b0, 1'b0, delay_mult_in[5]};  // shift a bit of delay_mult
      5'h06:  pgm_sequence = {1'b0, 1'b1, 1'b0, 1'b0, delay_mult_in[4]};  // shift a bit of delay_mult
      5'h05:  pgm_sequence = {1'b0, 1'b1, 1'b0, 1'b0, delay_mult_in[3]};  // shift a bit of delay_mult
      5'h04:  pgm_sequence = {1'b0, 1'b1, 1'b0, 1'b0, delay_mult_in[2]};  // shift a bit of delay_mult
      5'h03:  pgm_sequence = {1'b0, 1'b1, 1'b0, 1'b0, delay_mult_in[1]};  // shift a bit of delay_mult
      5'h02:  pgm_sequence = {1'b0, 1'b1, 1'b0, 1'b0, delay_mult_in[0]};  // shift a bit of delay_mult
      5'h01:  pgm_sequence = {1'b0, 1'b1, 1'b1, 1'b0, 1'b0};              // clear blk lock so can program again.
      5'h00:  pgm_sequence = {1'b0, 1'b1, 1'b0, 1'b0, 1'b0};  // no command
      default:  pgm_sequence = {1'b0, 1'b1, 1'b0, 1'b0, 1'b0};  // no command
    endcase

    pgm_done = (pgm_seq_cnt == 5'h15);
  end


  //----------------------------------------------------------
  // Turn cgts controller on or off based on pipe_active_in.
  //----------------------------------------------------------
  always @(*)
  begin
    on_seq_pulse  = ctrl_on_seq_en & ~ctrl_on_seq_en_d1;
    off_seq_pulse = ctrl_off_seq_en & ~ctrl_off_seq_en_d1;
    case ({off_seq_pulse, on_seq_pulse})
                             //{override, program, state, valid, data}
      2'b11: on_off_sequence = 5'b01000;  // no command
      2'b10: on_off_sequence = 5'b00000;  // ON command
      2'b01: on_off_sequence = 5'b00100;  // ON command
      2'b00: on_off_sequence = 5'b01000;  // no command
      default:  on_off_sequence = 5'b01000;  // no command
    endcase
  end


  assign ls_cntl_out = ~srst_in;
  assign cntl_ls_override_out = srst_in;
  assign cntl_out = programming ? pgm_sequence : on_off_sequence;
  assign busy_out = busy_d1 | busy_d2 | busy_d3 | busy_d4;

endmodule
