`timescale 1ps/1ps
//////////////////////////////////////////////////////////////////////////////
//  
//  Module Name    : techind_cdcefpm
//  
//  Revision       : $Revision$
//  Release Status : $State$
//  Author         : $Author$
//  
//   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
//  
//    ABSTRACT
//
//    This wrapper module implements a technology independent means to embed
//    a parameterized bus-width of false-path marker into RTL code. With
//    appropriate switches, simulation is augmented with the means to aid the
//    verification of false-path assumptions by incorporating a special
//    simulation model.
//
//    Note, this upper-level wrapper is simply used to perform a bussed
//    instantiation of another wrapper, passing all [except size] parameters
//    down to the lower-level wrapper. The lower-level wrapper actually
//    instantiates either the model or the technology-specific cell. for more
//    details, refer to the 'techind_cdcefpm_single.v' module file.
//
//   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
//  
//    Copyright (C) 2009,2010,2011
//      by Advanced Micro Devices (AMD) Corporation
//  
//    All rights reserved. This source file is considered proprietary and
//    confidential information of AMD Corp. As such, this source file may
//    not be distributed by any means, in whole, or in part, without the
//    express, written approval of AMD Corp.
//  
//   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
//  
//  Revision History :
//  
//  $Log$
//  
//////////////////////////////////////////////////////////////////////////////
module techind_cdcfpm ( A, DSTCLK, Z ); 
  parameter   size               = 1;
  parameter   applyParams        = 0;       // deprecated
  parameter   writeParams        = 1;       // deprecated
  parameter   CdcDestSetup       = 11000;
  parameter   ClockBasedDelay    = 0;       // deprecated
  parameter   DestClkCnt         = 0;
  parameter   driveSel           = 3;
  parameter   mode01             = 0;       // deprecated
  parameter   BufMetaWaiveMode   = 0;
  parameter   SrcPath            = 0;
  input   [size-1:0]  A;
  input               DSTCLK;
  output  [size-1:0]  Z; 
  // -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
  //  we provide a layer of indirection that helps us resolve two problems.
  //
  //      o   for the bussed instantiation, the 'force' statements used to pass
  //          the timing parameters onto the model cannot be unrolled using a
  //          loop. By bus-instantiating another wrapper that instantiates a
  //          single cell, we can avoid this problem.
  //
  //      o   we need to propagate the clock connection through synthesis, but
  //          not to the leaf cell. By creating this extra layer, we can preserve
  //          the wrapper with the clock connection, but it will not actually
  //          be connected to the leaf cell.
  techind_cdcfpm_single
    #(
       .applyParams    ( applyParams  ),
       .CdcDestSetup   ( CdcDestSetup ),
       .DestClkCnt     ( DestClkCnt   ),
       .driveSel       ( driveSel     ),
       .mode01         ( mode01       ),
       .BufMetaWaiveMode (BufMetaWaiveMode),
       .SrcPath        ( SrcPath      )
     )  ufpm [size-1:0]  (
       .A       ( A      ),
       .DSTCLK  ( DSTCLK ),
       .Z       ( Z      )
     );
endmodule
