//                              -*- Mode: Verilog -*-
////////////////////////////////////////////////////////////////////////////
/*
 *
 *  Trade secret of ATI Technologies, Inc.
 *  Copyright 2005, ATI Technologies, Inc., (unpublished)
 *
 *  All rights reserved.  This notice is intended as a precaution against
 *  inadvertent publication and does not imply publication or any waiver
 *  of confidentiality.  The year included in the foregoing notice is the
 *  year of creation of the work.
 *
 */
////////////////////////////////////////////////////////////////////////////
//
// Filename    : techro.v
// Description : Technology Ring Oscillator
//
////////////////////////////////////////////////////////////////////////////

module techro_gc10
(
 //Broadcast to All TDRs
 input wire Tdr_Tck,
 input wire Tdr_Trstb,
 input wire Tdr_Shift,
 input wire Tdr_Capture,
 input wire Tdr_Update

 ,input wire Tdr_techro_setup_En
 ,input wire Tdr_techro_setup_Tdi
 ,output wire Tdr_techro_setup_Tdo

 ,input wire Tdr_techro_status_En
 ,input wire Tdr_techro_status_Tdi
 ,output wire Tdr_techro_status_Tdo

 ,input wire Tile_Shift_Clk
 ,input wire Tile_ScanEn
 ,input wire Scm_Mode
 ,input wire Single_Chain_Daisy_In_0
 ,output wire Single_Chain_Daisy_Out_0
 ,input wire Single_Chain_Daisy_In_1
 ,output wire Single_Chain_Daisy_Out_1

 ,input	wire Ros_En
 );


   //wire [3:0] cfg_bits [4-1:0] ;
   //wire [3:0] counter_reset [1-1:0] ;
   //wire [3:0] enable_all [1-1:0] ;
   //wire [3:0] ros_size [2-1:0] ;
   //wire [3:0] counter_bits [29-1:0] ;
   //wire [3:0] overflow [1-1:0] ;
   //wire [3:0] ros_clk_sel [1-1:0] ;

        //packed          unpacked
   wire [4-1:0] cfg_bits [6:0];
   //   packed  unpacked
   //   000A     0 - cfg_bits[0] = 000, cfg_bigs[0][0] = A
   //   00B0     1 - cfg_bits[1]
   //   0C00     2
   //   D000     3
   wire [1-1:0] counter_reset [6:0];
   wire [1-1:0] enable_all [6:0];
   wire [2-1:0] ros_size [6:0];
   wire [29-1:0] counter_bits [6:0];
   wire [1-1:0] overflow [6:0];
   wire [1-1:0] ros_clk_sel [6:0];

   wire [6:0] rosen;
   wire [6:0] Single_Chain_Daisy_In;
   wire [6:0] Single_Chain_Daisy_Out;

   wire [7:0] ros_setup_tdi_tdo;
   wire [7:0] ros_status_tdi_tdo;

assign ros_setup_tdi_tdo[0]     = Tdr_techro_setup_Tdi;
assign ros_status_tdi_tdo[0]    = Tdr_techro_setup_Tdi;

assign Tdr_techro_setup_Tdo     = ros_setup_tdi_tdo[7];
assign Tdr_techro_status_Tdo    = ros_status_tdi_tdo[7];

assign Single_Chain_Daisy_In[0] = Single_Chain_Daisy_In_0;

assign Single_Chain_Daisy_In[1] = Single_Chain_Daisy_Out[0];
assign Single_Chain_Daisy_In[2] = Single_Chain_Daisy_Out[1];
assign Single_Chain_Daisy_Out_0 = Single_Chain_Daisy_Out[2];

assign Single_Chain_Daisy_In[3] = Single_Chain_Daisy_In_1;

assign Single_Chain_Daisy_In[4] = Single_Chain_Daisy_Out[3];
assign Single_Chain_Daisy_In[5] = Single_Chain_Daisy_Out[4];
assign Single_Chain_Daisy_In[6] = Single_Chain_Daisy_Out[5];

assign Single_Chain_Daisy_Out_1 = Single_Chain_Daisy_Out[6];

generate
 genvar i;
 for(i=0; i < 7; i = i+1) begin : techro_tdrs
   ros_tdrs_gc10 ros_tdrs
     (
      .Tile_Shift_Clk       (Tile_Shift_Clk),
      .Tile_ScanEn          (Tile_ScanEn),
      .Scm_Mode             (Scm_Mode),
      .Single_Chain_Daisy_In (Single_Chain_Daisy_In[i]),
      .Single_Chain_Daisy_Out (Single_Chain_Daisy_Out[i]),

      .Tdr_Tck               (Tdr_Tck) ,
      .Tdr_Trstb             (Tdr_Trstb),
      .Tdr_ros_setup_Tdi     (ros_setup_tdi_tdo[i]),
      .Tdr_ros_status_Tdi    (ros_status_tdi_tdo[i]),
      .Tdr_Shift             (Tdr_Shift),
      .Tdr_Capture           (Tdr_Capture),
      .Tdr_Update            (Tdr_Update)


      //TDR --> ROS_SETUP
      ,.ros_clk_sel           (ros_clk_sel[i])
      ,.cfg_bits              (cfg_bits[i])
      ,.counter_reset         (counter_reset[i])
      ,.enable_all            (enable_all[i])
      ,.ros_size              (ros_size[i])
      ,.Tdr_ros_setup_En      (Tdr_techro_setup_En)
      ,.Tdr_ros_setup_Tdo     (ros_setup_tdi_tdo[i+1])

      //TDR --> ROS_STATUS
      ,.counter_bits         (counter_bits[i])
      ,.overflow             (overflow[i])
      ,.Tdr_ros_status_En            (Tdr_techro_status_En)
      ,.Tdr_ros_status_Tdo           (ros_status_tdi_tdo[i+1])

      ,.Ext_Clk           (1'b0)
      ,.Ext_Clk_gated           ()
      ,.Ros_En           (Ros_En)
      ,.rosen           (rosen[i])

      );

end
endgenerate
   ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

   TECHRO111_TSMCN7B1 techro_macro
     (
`ifndef SYNTHESIS
      .VDD_RO (1'b1),
      .VDD_CNTRL (1'b1),
      .VSS (1'b0),
`endif

      .RO_en_1(rosen[0])
      ,.RO_en_all_1(enable_all[0])
      ,.RO_cntr_reset_1(counter_reset[0])
      ,.RO_cntr_ovflw_1(overflow[0])
      ,.RO_cntr_1(counter_bits[0])
      ,.RO_ocv_sel_1(ros_size[0])
      ,.RO_sel_1(    cfg_bits[0])

      ,
      .RO_en_2(rosen[1])
      ,.RO_en_all_2(enable_all[1])
      ,.RO_cntr_reset_2(counter_reset[1])
      ,.RO_cntr_ovflw_2(overflow[1])
      ,.RO_cntr_2(counter_bits[1])
      ,.RO_ocv_sel_2(ros_size[1])
      ,.RO_sel_2(    cfg_bits[1])

      ,
      .RO_en_3(rosen[2])
      ,.RO_en_all_3(enable_all[2])
      ,.RO_cntr_reset_3(counter_reset[2])
      ,.RO_cntr_ovflw_3(overflow[2])
      ,.RO_cntr_3(counter_bits[2])
      ,.RO_ocv_sel_3(ros_size[2])
      ,.RO_sel_3(    cfg_bits[2])

      ,
      .RO_en_4(rosen[3])
      ,.RO_en_all_4(enable_all[3])
      ,.RO_cntr_reset_4(counter_reset[3])
      ,.RO_cntr_ovflw_4(overflow[3])
      ,.RO_cntr_4(counter_bits[3])
      ,.RO_ocv_sel_4(ros_size[3])
      ,.RO_sel_4(    cfg_bits[3])

      ,
      .RO_en_5(rosen[4])
      ,.RO_en_all_5(enable_all[4])
      ,.RO_cntr_reset_5(counter_reset[4])
      ,.RO_cntr_ovflw_5(overflow[4])
      ,.RO_cntr_5(counter_bits[4])
      ,.RO_ocv_sel_5(ros_size[4])
      ,.RO_sel_5(    cfg_bits[4])

      ,
      .RO_en_6(rosen[5])
      ,.RO_en_all_6(enable_all[5])
      ,.RO_cntr_reset_6(counter_reset[5])
      ,.RO_cntr_ovflw_6(overflow[5])
      ,.RO_cntr_6(counter_bits[5])
      ,.RO_ocv_sel_6(ros_size[5])
      ,.RO_sel_6(    cfg_bits[5])

      ,
      .RO_en_7(rosen[6])
      ,.RO_en_all_7(enable_all[6])
      ,.RO_cntr_reset_7(counter_reset[6])
      ,.RO_cntr_ovflw_7(overflow[6])
      ,.RO_cntr_7(counter_bits[6])
      ,.RO_ocv_sel_7(ros_size[6])
      ,.RO_sel_7(    cfg_bits[6])


      );

/*
   assign counter_bits[3] = 'h0;
   assign counter_bits[2] = 'h0;
   assign counter_bits[1] = 'h0;
   assign counter_bits[0] = 'h0;
   assign overflow[3]  = 'h0;
   assign overflow[2]  = 'h0;
   assign overflow[1]  = 'h0;
   assign overflow[0]  = 'h0;
*/
endmodule // techros


