//timescale 1ns/10ps



module test_ff_async_rst (/*AUTOARG*/
   // Outputs
   q,
   q_posedge,
   // Inputs
   clk,
   rstb,
   d,
   d_rst
   );
   parameter width = 1;

   input     clk;
   input     rstb;
   input [width-1:0] d;
   input [width-1:0] d_rst;
   output [width-1:0] q,
		      q_posedge;

   reg [width-1:0]    q_int,
		      q;

   always @(posedge clk or negedge rstb)
     if (~rstb)
       q_int <= d_rst;
     else
       q_int <= d;

   always @(negedge clk)
       q <= q_int;

   assign 	      q_posedge = q_int;

endmodule

module test_ff_async_rst_pos_neg (/*AUTOARG*/
   // Outputs
   q,
   q_posedge,
   // Inputs
   clk,
   rstb,
   d,
   d_rst
   );
   parameter width = 1;

   input     clk;
   input     rstb;
   input [width-1:0] d;
   input [width-1:0] d_rst;
   output [width-1:0] q,
		      q_posedge;

   reg [width-1:0]    q_int,
		      q;

   always @(posedge clk or negedge rstb)
     if (~rstb)
       q_int <= d_rst;
     else
       q_int <= d;

   always @(negedge clk or negedge rstb)
       if(~rstb)
         q <= d_rst;
       else
         q <= q_int;

   assign 	      q_posedge = q_int;

endmodule

module test_ff_neg_async (/*AUTOARG*/
   // Outputs
   q,
   // Inputs
   clk,
   rstb,
   d,
   d_rst
   );
   parameter width = 1;

   input     clk;
   input     rstb;
   input [width-1:0] d;
   input [width-1:0] d_rst;
   output [width-1:0] q;

   reg [width-1:0]    q_int;

   always @(negedge clk or negedge rstb)
     if (~rstb)
       q_int <= d_rst;
     else
       q_int <= d;

       assign q = q_int;


endmodule



module test_eff (/*AUTOARG*/
   // Outputs
   q,
   q_posedge,
   // Inputs
   e,
   clk,
   d
   );
   parameter width = 1;
   input     e;
   input     clk;
   input [width-1:0] d;
   output [width-1:0] q;
   output [width-1:0] q_posedge;

   reg [width-1:0]    q_int,
		      q;

   always @(posedge clk)
     if (e)
       q_int <= d;
     else
       q_int <= q_int;

   always @(negedge clk)
     q <= q_int;

   assign 	      q_posedge = q_int;

endmodule

module test_pos_eff (/*AUTOARG*/
   // Outputs
   q_posedge,
   // Inputs
   e, clk, d
   );
   parameter width = 1;
   input     e;
   input     clk;
   input [width-1:0] d;
   output [width-1:0] q_posedge;

   reg [width-1:0]    q_int;

   always @(posedge clk)
     if (e)
       q_int <= d;
     else
       q_int <= q_int;

   assign             q_posedge = q_int;

endmodule


////////////////////////////////////////////////////////////////////////////////////
// Read Only TDR
////////////////////////////////////////////////////////////////////////////////////

// shift reg
module test_shift_reg (/*AUTOARG*/
   // Outputs
   so,
   q,
   q_posedge,
   // Inputs
   clk,
   en,
   shift,
   si,
   capture,
   d
   );

   parameter width = 1;

   input clk;
   input en;
   input shift;
   input si;
   output so;
   input capture;

   input [width-1:0] d;
   output q;
   output [width-1:0] q_posedge;

   wire clk_gated;
   ati_clock_gate tdr_shf_DfxCLOCKGATER 
   (.clk_out   (clk_gated),
    .clk_src   (clk),
    .clk_en    (en),
    .TE        (1'b0));

   wire   q;

   wire [width-1:0] d_new = (shift
			     ? (si << (width-1)) | (q_posedge >> 1)
			     : d);
 // river translate_off
generate
  genvar i;
  for(i=1; i < width; i = i+1) begin : eff
   test_pos_eff data (.clk(clk_gated),
                           .e(en && (shift || capture)),
                           .d(d_new[width-i]),
                           .q_posedge(q_posedge[width-i]));
  end
  endgenerate
 // river translate_on

   test_eff #(1) data (.clk(clk_gated),
			   .e(en && (shift || capture)),
			   .d(d_new[0]),
			   .q_posedge(q_posedge[0]),
			   .q(q));

   assign so = q;	// output at negedge

endmodule // test_shift_reg

////////////////////////////////////////////////////////////////////////////////////
// Readable/Writable TDR
////////////////////////////////////////////////////////////////////////////////////

// test register with asynchronous reset
module effarst (
  clk,
  en,
  arstn,
  d,
  drst,
  q
);

  parameter width = 1;

  input              clk;
  input              en;
  input              arstn;
  input  [width-1:0] d;
  input  [width-1:0] drst;
  output [width-1:0] q;


  reg [width-1:0]  q;

  always @(negedge clk or negedge arstn)
   begin
    if (~arstn) begin
      q <= drst;
    end else if (en) begin
      q <= d;
    end else begin
      q <= q;
    end
   end

endmodule

module test_reg_async_rst (/*AUTOARG*/
   // Outputs
   so,
   q,
   q_sr,
   q_sr_posedge,
   // Inputs
   d,
   en,
   capture,
   update,
   shift,
   si,
   clk,
   rstb,
   d_rst
   );

   parameter width = 1;

   input en;
   input capture;
   input update;
   input shift;
   input si;
   output so;
   input clk;
   input rstb;
   input [width-1:0] d;
   input [width-1:0] d_rst;

   output [width-1:0] q;
   output q_sr;
   output [width-1:0] q_sr_posedge;

   wire [width-1:0]   q_sr_posedge;
   wire q_sr;
   wire             trstn;
   wire [width-1:0]   q;
   reg 		     en_q;
   wire clk_upd;

   test_shift_reg #(width) DFT_TDRSHIFTREG
     (
      .clk(clk),
      .d(d),
      .q(q_sr),
      .q_posedge(q_sr_posedge),
      .shift(shift),
      .capture(capture),
      .en(en), 
      .si(si),
      .so(so));

   ati_clock_gate tdr_upd_DfxCLOCKGATER 
   (.clk_out   (clk_upd),
    .clk_src   (clk),
    .clk_en    (en_q),  // using local en_q insted of coming from decoding logic for better timing. 
    .TE        (1'b0));

// For TDR. en_q=1 on the same posedge when update become 1->0
   always @(posedge clk or negedge rstb)
     if (~rstb)
      en_q <= 1'b0;
     else
      en_q <= en;

   wire upd_en;
   assign upd_en = en_q & update;

   ati_buffer1 DFT_TDRTRSTN (.Y(trstn), .A(rstb));

   effarst #(width) DFT_TDRUPDATEREG
   (
     .clk   (clk_upd),
     .en    (upd_en),
     .arstn (trstn),
     .d     (q_sr_posedge),
     .drst  (d_rst),
     .q     (q)
   );


`ifdef ABV
`include "dftmisc_assert_control.vh"

ovl_never_unknown #(
   .msg("TDR enable is X/Z"))
assert_tdr_en_unknown(
   .clock     (clk),
   .reset     (rstb),
   .enable    (`DFTMISC_ABV_PWROK_EN && !$test$plusargs("DFTDV_DISABLE_OVL_TDR_UNKNOWN")),
   .qualifier (rstb),
   .test_expr (en), 
   .fire      ());

`endif //ABV


endmodule // test_reg_async_rst

// test register with CDC bufen.
// Use this only where TDR is needed to work in debug mode (non-scan.)
// If flops in dst_clk are in scan, the TDR functioanlity is broken in scan mode. This TDR can't be used for scan mode intercepts.
module test_reg_async_rst_cdc (/*AUTOARG*/
   // Outputs
   so,
   q,
   q_sr,
   q_sr_posedge,
   dst_q,
   // Inputs
   d,
   en,
   capture,
   update,
   shift,
   si,
   clk,
   rstb,
   d_rst,
   dst_rstb,
   dst_clk

   );

   parameter width = 1;

   input en;
   input capture;
   input update;
   input shift;
   input si;
   output so;
   input clk;
   input rstb;
   input [width-1:0] d;
   input [width-1:0] d_rst;

   output [width-1:0] q;
   output q_sr;
   output [width-1:0] q_sr_posedge;

   input  dst_clk;
   input  dst_rstb; // Active low
   output wire [width-1:0] dst_q;

   wire [width-1:0]   q_sr_posedge;
   wire q_sr;
   wire trstn;
   wire [width-1:0]   q;
   reg 		     en_q;
   wire clk_upd;

   test_shift_reg #(width) DFT_TDRSHIFTREG
     (
      .clk(clk),
      .d(d),
      .q(q_sr),
      .q_posedge(q_sr_posedge),
      .shift(shift),
      .capture(capture),
      .en(en),
      .si(si),
      .so(so));


   ati_clock_gate tdr_upd_DfxCLOCKGATER 
   (.clk_out   (clk_upd),
    .clk_src   (clk),
    .clk_en    (en_q),  // using local en_q insted of coming from decoding logic for better timing. 
    .TE        (1'b0));

   always @(posedge clk or negedge rstb)
     if (~rstb)
      en_q <= 1'b0;
     else
      en_q <= en;

   wire upd_en;
   assign upd_en = en_q & update;

   ati_buffer1 DFT_TDRTRSTN (.Y(trstn), .A(rstb));

   effarst #(width) DFT_TDRUPDATEREG
   (
     .clk   (clk_upd),
     .en    (upd_en),
     .arstn (trstn),
     .d     (q_sr_posedge),
     .drst  (d_rst),
     .q     (q)
   );

   tdr_cdc #(.width(width)) tdr_cdc 
   (
    .clk        (clk),
    .rstb       (rstb),
    .update     (upd_en),
    .en         (1'b1),     // upd_en already ANDed en.
    .d_rst      (d_rst),
    .q          (q),

    .dst_clk    (dst_clk),
    .dst_rstb   (dst_rstb),
    .dst_q      (dst_q)
   );


`ifdef ABV
`include "dftmisc_assert_control.vh"

ovl_never_unknown #(
   .msg("TDR enable is X/Z"))
assert_tdr_en_unknown(
   .clock     (clk),
   .reset     (rstb),
   .enable    (`DFTMISC_ABV_PWROK_EN),
   .qualifier (rstb),
   .test_expr (en), 
   .fire      ());
   
`endif //ABV


endmodule // test_reg_async_rst_cdc



// BW added new module which outputs "cap_pulse"
// Provide pulse when update asserted. The tdr_cdc is outside this module.
module test_reg_async_rst_cap_cdc
(

   en,
   capture,
   update,
   shift,
   si,
   so,
   clk,
   rstb,
   d,
   d_rst,

   q,
   cap_pulse

);

   parameter WIDTH = 1;

   input en;
   input capture;
   input update;
   input shift;
   input si;
   output so;
   input clk;
   input rstb;
   input [WIDTH-1:0] d;
   input [WIDTH-1:0] d_rst;

   output [WIDTH-1:0] q;
   output cap_pulse;


   wire [WIDTH-1:0]   q_sr_posedge;
   wire q_sr;
   wire trstn;
   wire [WIDTH-1:0]   q;
   reg 		     en_q;
   wire clk_upd;

   test_shift_reg #(WIDTH) DFT_TDRSHIFTREG
     (
      .clk(clk),
      .d(d),
      .q(q_sr),
      .q_posedge(q_sr_posedge),
      .shift(shift),
      .capture(capture),
      .en(en),
      .si(si),
      .so(so));


   ati_clock_gate tdr_upd_DfxCLOCKGATER 
   (.clk_out   (clk_upd),
    .clk_src   (clk),
    .clk_en    (en_q),  // using local en_q insted of coming from decoding logic for better timing. 
    .TE        (1'b0));

   always @(posedge clk or negedge rstb)
     if (~rstb)
      en_q <= 1'b0;
     else
      en_q <= en;

   wire upd_en;
   assign upd_en = en_q & update;

   ati_buffer1 DFT_TDRTRSTN (.Y(trstn), .A(rstb));

   effarst #(WIDTH) DFT_TDRUPDATEREG
   (
     .clk   (clk_upd),
     .en    (upd_en),
     .arstn (trstn),
     .d     (q_sr_posedge),
     .drst  (d_rst),
     .q     (q)
   );


//////////////////////// sync to other clock domain ///////////////
reg upd_en_ff_0, upd_en_ff_1;
   always @(negedge clk or negedge rstb)
     if (~rstb)
      upd_en_ff_0 <= 1'b0;
     else
      upd_en_ff_0 <= upd_en;

   always @(negedge clk or negedge rstb)
     if (~rstb)
      upd_en_ff_1 <= 1'b0;
     else
      upd_en_ff_1 <= upd_en_ff_0;

// edge detect in case Update stretches beyond a single cycle pulse
wire rising_edge_det = upd_en_ff_0 & ~upd_en_ff_1;

assign cap_pulse = rising_edge_det;

`ifdef ABV
`include "dftmisc_assert_control.vh"

ovl_never_unknown #(
   .msg("TDR enable is X/Z"))
assert_tdr_en_unknown(
   .clock     (clk),
   .reset     (rstb),
   .enable    (`DFTMISC_ABV_PWROK_EN),
   .qualifier (rstb),
   .test_expr (en), 
   .fire      ());

`endif //ABV



endmodule  // test_reg_async_rst_cap_cdc
 

module tdr_cdc 
# (
   parameter width = 1
   )
(
input wire clk,
input wire rstb,
input wire update,
input wire en,
input wire [width-1:0] d_rst,
input wire [width-1:0]   q,

input wire dst_clk,
input wire dst_rstb, // Active low
output reg [width-1:0] dst_q

);

  wire [width-1:0] q_cdc;
//////////////////////// sync to other clock domain ///////////////
reg upd_en_ff_0, upd_en_ff_1;
   always @(negedge clk or negedge rstb)
     if (~rstb)
      upd_en_ff_0 <= 1'b0;
     else
      upd_en_ff_0 <= en & update;

   always @(negedge clk or negedge rstb)
     if (~rstb)
      upd_en_ff_1 <= 1'b0;
     else
      upd_en_ff_1 <= upd_en_ff_0;

// edge detect in case Update stretches beyond a single cycle pulse
wire rising_edge_det = upd_en_ff_0 & ~upd_en_ff_1;


reg  tog_in;
wire tog_out;
reg  tog_out_d;

   always @(negedge clk or negedge rstb)
     if (~rstb)   
       tog_in <= 1'b0;
     else if (rising_edge_det) 
       tog_in <= ~tog_in;

dfx_sync3 utog_sync (
  .CK(dst_clk),
  .D(tog_in),
  .Q(tog_out)
);


wire dst_rstb_sync;
dfx_sync3 uDST_RESET_sync (
  .CK(dst_clk),
  .D(rstb),
  .Q(dst_rstb_sync)
);

always @(posedge dst_clk) 
  tog_out_d <= tog_out;

reg dst_pulse;
always @(posedge dst_clk) 
  dst_pulse <= (tog_out_d ^ tog_out) & dst_rstb_sync;


techind_cdcefpm #(
   .size(width),
   .applyParams(1),
   .ClockBasedDelay(1),
   .DestClkCnt(2),
   .CdcDestSetup(0),
   .AssertEnable(1)
) data_CDCBUF (
    .DSTCLK(dst_clk),
    .EN(dst_pulse),
    .A(q),
    .Z(q_cdc));

always @(posedge dst_clk)
   if (~dst_rstb) 
    dst_q <= d_rst;
   else if (dst_pulse) 
    dst_q <= q_cdc;



endmodule // tdr_cdc
