// spyglass disable_block ALL
//============================================================================
//Copyright (c) 2004 by Advanced Micro Devices, Inc.
//
// This file is protected by Federal Copyright Law, with all rights reserved. No part of this file may be reproduced, stored in a
// retrieval system, translated, transcribed, or transmitted, in any  form, or by any means manual, electric, electronic, 
// mechanical, electro-magnetic, chemical, optical, or otherwise, without prior explicit written permission from Advanced Micro
// Devices, Inc.
// MCBUFEN  Multi-Cycle buffer
// MCBUF  Multi-Cycle buffer
// <CDCBUF>        - Clock Domain Crossing Buffer
// <SYNC2OR>       - Parameterized Synchronizer with a minimum depth of 2
// <SYNC3OR>       - Parameterized Synchronizer with a minimum depth of 3
// <SYNC4OR>       - Parameterized Synchronizer with a minimum depth of 4
// <SYNC3FLUSH>    - Parameterized depth 3 Synchronizer with Async Flush
// <SYNC4FLUSH>    - Parameterized depth 3 Synchronizer with Async Flush
// <CDCBUFCLK>     - CDCBUF with params / non-optional DSTCLK port
// <CDCBUF01>      -
// <CDCBUFEN>      - Clock Domain Crossing (CDC) Buffer with Enable
// <CDCBUFENCLK>   - CDCBUFEN with params / non-optional DSTCLK port 
// NOTE: This file includes support modules for techind_sync, which is the
// SOC15 preferred synchronizer model.  Future work on this file should
// merge the sync3OR and techind_sync submodules.  (SteveKommrusch)
// NEW modules with version 8 for techind_sync support:
//       cdc_stdlib_internal_meta_rand 
//       cdc_stdlib_internal_multibit_sync_filter 
//       cdc_stdlib_internal_sync_transport 
//       cdc_stdlib_internal_sync_model
//       CDCGENERICSYNC 


//! disable illegal_includes
`ifdef ABV
`include "std_ovl_defines.h"
`include "amd_proj_global.svh"
`endif
//! enable illegal_includes
//! disable one_module_per_file
// `include "rtl_cpu.d"

// JPC: disable Leda E_122 and E_169 for this entire file, there
//      are lots of violations. I am not expert but they look
//      harmless -- perfectly legal verilog.
// leda E_122 off
// leda E_169 off
// leda E_001 off

// Add the `celldefine compiler directive for improved simulation
// performance. UBTS:393082
`ifdef RTL_STDLIB_CELLDEFINES
`celldefine
`endif

//Added granualarity for individual modules modelinf as dff's
//UBTS 483844
`ifdef MODEL_AS_DFF
   `define MODEL_SYNC4OR_AS_DFF 1
`endif

// vrq translate_off

//==============================================================================
// CDC_STDLIB_LATCH - a simple latch with size parameter
//==============================================================================

`timescale 1ps/1ps
module CDC_STDLIB_LATCH ( Q, D, C);

parameter  size = 1;

output reg [size-1:0] Q;
input      [size-1:0] D;
input                 C;

// leda SYN9_5 off
// leda W410 off

`ifdef NO_CUTPOINTS
//! latch
always @*
  if ( C ) Q[size-1:0] <= D[size-1:0];
`else
//! latch
always @*
  Q[size-1:0] <= C ? D[size-1:0] : Q[size-1:0];
`endif

// leda SYN9_5 on
// leda W410 on

endmodule

//============================================================================
// cdc_stdlib_internal_met_model
//   WARNING: This is intended to be an rtl_stdlib internal module, please
//     do not use it unless you work closely with it's author.
//   This is a seperate module so the meta_rand reg can remain active in MVSIM
//   Disabling RPL for the entire module to avoid unassigned warnings
//============================================================================
//! disable parser
module cdc_stdlib_internal_met_model (
   D,
   EN,
   Dsync,
   RandomExtraCycle
);

//VCS coverage exclude_module
// Inputs and Ouputs
input D;
input EN;
output reg Dsync;
output reg RandomExtraCycle;

// synopsys translate_off
reg  [31:0]       meta_rand;

//! unassigned Dsync
//! unassigned RandomExtraCycle
// leda off
`ifdef METASTABLE_MODEL
import "DPI-C" function int unsigned cdcRandomFunc(input int unsigned  seed);
integer RandSeed;
initial begin
        if (!$value$plusargs("seed=%d", RandSeed)) begin
           RandSeed=123456;
        end
        `ifdef SYNC2ORSEED_OVERRIDE
        if (check_option_switch("Sync2OrSeedOverride")) begin
           if (get_option_switch("Sync2OrSeedOverride")) begin
             RandSeed[31:0] = get_option_value("Sync2OrSeed");
           end 
        end
        `endif //SYNC2ORSEED_OVERRIDE
        meta_rand[31:0] = cdcRandomFunc(RandSeed);
        RandomExtraCycle = meta_rand[1];
end
`else // LFSR Seed Initialization

   initial begin
      #1;
      RandomExtraCycle = 1'b0;
      Dsync = D;
   end
`endif // METASTABLE_MODEL

// modified LFSR with 11 LSBs uncorellated from call to call
always @(D) begin
   Dsync <= D; //transitions at the same time as RandomExtraCycle
   if(EN) begin
      RandomExtraCycle <= meta_rand[0];
      meta_rand[31:0] <= { meta_rand[10:0], meta_rand[31:11] } ^ ( meta_rand[0] ? 32'h80080100 : 32'h00 );
   end
end
// leda on
// synopsys translate_on

endmodule

//! enable parser

//============================================================================
// cdc_stdlib_internal_syncff_rand_model
//   WARNING: This is intended to be an rtl_stdlib internal module, please
//     do not use it unless you work closely with it's author.
//============================================================================
module cdc_stdlib_internal_syncff_rand_model (
   D,
   EN,
   CK,
   EnableRand,
`ifdef ABV
   InputChkDis,
`endif
   MetWindowMode,
   Q
);

//VCS coverage exclude_module

////////////////////
// IO declarations
////////////////////
input  D;
input  EN;
input  CK;
input  EnableRand;  // Randomization enable, simulation only
`ifdef ABV
input  InputChkDis; // OVA disable, simulation only
`endif
input  MetWindowMode;
output wire Q;

parameter metastable = 1'b1;

//! unused InputChkDis
wire     RandomExtraCycle;
reg      DelayFF;
wire     D_delayed;
wire     Dsync;
integer  LocalMetaWindowSize;

// leda off
// synopsys translate_off
initial begin
`ifdef TECH__SYNC__META_WINDOW_SIZE
  LocalMetaWindowSize[31:0] = `TECH__SYNC__META_WINDOW_SIZE;
`else
  LocalMetaWindowSize[31:0] = 180;
`endif
end
// synopsys translate_on
// leda on
//! unused LocalMetaWindowSize[31:0]


////////////////////////////////////////////////////////////////////////////////
// Window mode randomly delays the data by LocalMetaWindowSize. Use of this mode
// requires a simulation env that models jitter/frequency drift on clocks.
////////////////////////////////////////////////////////////////////////////////

// leda off
//! disable delay_restrictions
assign #(LocalMetaWindowSize[31:0]) D_delayed = D;
//! enable delay_restrictions
// leda on

// Random bit stream generator
cdc_stdlib_internal_met_model cdc_stdlib_internal_met_model (
   .D(D),
   .EN(EN),
   .Dsync(Dsync),
   .RandomExtraCycle(RandomExtraCycle)
);

// leda off
// Delay flip flop
always @(posedge CK) begin
   if(EN) DelayFF <= Dsync;
   `ifdef SYNC_WINDOW_DISPLAY_RANDOMIZATION
   if(MetWindowMode & RandomExtraCycle & EnableRand & (D_delayed ^ D))
      $display("INFO : %m : Synchronizer Meta-stability Window hit at Time=%t", $time);
   `endif
end

// synopsys translate_off
//! disable parser


 reg firstCK;  // will be valid from t=0 till first posedge of CK. 
 initial firstCK <= 1'b1;
 always @(posedge CK)
     if(CK & firstCK) firstCK <= 1'b0;
 
 // to make sure DelayFF gets Dsync during initial period, when CK is not toggling
 always @(Dsync)   
     if (firstCK)
      DelayFF = Dsync;

always @(posedge EN) begin
   DelayFF <= Dsync;
end
//! enable parser
// synopsys translate_on

// leda on

// Using Dsync to prevent glitching when D changes then RandomExtraCycle changes
assign Q = (metastable & RandomExtraCycle & EnableRand) ? ( MetWindowMode ? D_delayed : DelayFF ) : Dsync;

//! unassigned InputChkDis
// This is RTL Simulation-ONLY logic used to model meta-stable events
//   If you get an ova compile error here, then you need to define assertions. This is not a bug!
// WARNING: For some paths it is probably not legal to disable this under NbIReset (i.e HT Config)
`ifdef ABV
`ifndef REMOVE_OVL_NEVER_CDC_STDLIB_INTERNAL_SYCNFF_RAND_MODEL
// synopsys translate_off
//! disable parser
ovl_never #(.msg("DataIn is changing too quickly for the synchronizer.")) Assert_SYNCFF (
    .clock (CK), .reset(1'b1), .enable(~InputChkDis & ~MetWindowMode & ~firstCK), 
    .test_expr( ~((D === DelayFF) | (DelayFF === $past(DelayFF,1,,@(posedge CK)))) & 
                 (DelayFF !== 1'bx) & (Dsync !== 1'bx)), .fire());
//! enable parser
// synopsys translate_on
`endif   // REMOVE_OVL_NEVER_CDC_STDLIB_INTERNAL_SYCNFF_RAND_MODEL
`endif

////////////////////////////////////////////////////////////////////////////////
// Pulse Width Check for metastable window mode
//   - Data should be held long enough to hit 2 clock edges if shifted to cover
//     either destclk period to guarantee capture
////////////////////////////////////////////////////////////////////////////////

`ifdef RTL_ONLY
`elsif ABV
// synopsys translate_off
// leda off
//! disable parser

reg DataCaptured;
time lastCK, lastD, DestClockPeriod, DataPulseWidth, LastClockPeriod;

initial begin
  DataPulseWidth = 0; DestClockPeriod = 0; LastClockPeriod = 0; lastCK = 0; lastD = 0; DataCaptured = 0;
end

always @(negedge InputChkDis) begin // reset
  DataPulseWidth = 0; DestClockPeriod = 0; LastClockPeriod = 0; lastCK = 0; lastD = 0; DataCaptured = 0;
end

////////////////////////////////////////////////////////////////////////////////
// ClockCheck - measures last clock period, then compares the most recent data
//   against the last 2 clock periods. Data pulses smaller than either period 
//   could potentially drift and be lost.
// Disabled using InputChkDis (this should NOT be disabled all the time!!!!!)
////////////////////////////////////////////////////////////////////////////////
always @(posedge CK) begin : ClockCheck
    reg Last_Fail, This_Fail;
    if(MetWindowMode & ~InputChkDis) begin
        LastClockPeriod = DestClockPeriod;
        DestClockPeriod = $time - lastCK;
        if(DataPulseWidth) begin
            Last_Fail = DataCaptured && (DataPulseWidth < (LastClockPeriod + LocalMetaWindowSize));
            This_Fail = DataCaptured && (DataPulseWidth < (DestClockPeriod + LocalMetaWindowSize));
            if(Last_Fail & This_Fail ) begin
                $display("ERROR : %m : %0t D input pulse < %0t CLK period + %0t metastable window Time=%t", 
                    DataPulseWidth, DestClockPeriod, LocalMetaWindowSize,$time);
                `ifndef SYNC_WINDOW_DISABLE_FATAL_ERRORS
                $finish;
                `endif
            `ifndef SYNC_WINDOW_DISABLE_WARNINGS
            end else if( Last_Fail ) begin
                $display("WARNING : %m : %0t D input pulse < previous %0t CLK period + %0t metastable window Time=%t", 
                    DataPulseWidth, LastClockPeriod, LocalMetaWindowSize,$time);
            end else if( This_Fail ) begin
                $display("WARNING : %m : %0t D input pulse < %0t CLK period + %0t metastable window Time=%t", 
                    DataPulseWidth, DestClockPeriod, LocalMetaWindowSize,$time);
            `endif
            end
        end
        DataPulseWidth = 0; // last pulse checked
        lastCK = $time;
    end
end

////////////////////////////////////////////////////////////////////////////////
// DataCheck - measures data pulses, catches pulses smaller than a single clock
// Disabled using InputChkDis (this should NOT be disabled all the time!!!!!)
////////////////////////////////////////////////////////////////////////////////
always @(D) begin : DataCheck
    if(MetWindowMode & ~InputChkDis) begin
        if(|lastCK & (($time - lastD) > 5)) begin //smooth out any data glitches and ignore check before 1st clock edge
            if(DataPulseWidth) begin // no clock edge occurred in this pulse, still need to check last pulse
                if(DataCaptured) begin // last pulse included a posedge clock, need to check it.
                    if(DataPulseWidth < (DestClockPeriod + LocalMetaWindowSize)) begin
                        $display("ERROR : %m : %0t D input pulse < %0t CLK period + %0t metastable window Time=%t", 
                            DataPulseWidth, DestClockPeriod, LocalMetaWindowSize,$time);
                        `ifndef SYNC_WINDOW_DISABLE_FATAL_ERRORS
                        $finish;
                        `endif
                    end
                end
                `ifndef SYNC_WINDOW_DISABLE_WARNINGS
                $display("WARNING : %m : %0t D pulse lost at Time=%t", 
                    DataPulseWidth,$time);
                `endif
            end
            DataCaptured = (lastD < lastCK) & (D_delayed !== 1'bx) & (D !== 1'bx);
            DataPulseWidth = $time - lastD;
        end
        lastD = $time;
    end
end
// leda on
//! enable parser
// synopsys translate_on
`endif

endmodule



//============================================================================
// cdc_stdlib_internal_sync_multibit
//   WARNING: This is intended to be an rtl_stdlib internal module, please
//     do not use it unless you work closely with it's author.
//   Disabling RPL, this is a simulation-only module
//============================================================================
//! disable parser
module cdc_stdlib_internal_sync_multibit (
   EN,
   D,
   CK,
`ifdef ABV
   InputChkDis, // OVA disable, simulation only
`endif

   Q
);
 
//VCS coverage exclude_module
parameter WIDTH      = 1;
parameter metastable = 1'b1;
parameter transport_delay = 0;
parameter transport_cycle = 0;

input               EN;
input  [WIDTH-1:0]  D;
input               CK;
`ifdef ABV
input               InputChkDis; // OVA disable, simulation only
`endif
output wire [WIDTH-1:0]  Q;

// leda off

reg EnableIPWC;
`ifdef ENABLE_IPWC
initial EnableIPWC=1'b1;
`else
initial EnableIPWC=1'b0;
`endif

`ifdef METASTABLE_MODEL
import "DPI-C" function int unsigned cdcRandomFunc(input int unsigned  seed);
// synopsys translate_off
reg [WIDTH-1:0] TRANSPORT_D;
reg  [WIDTH-1:0] EnableRand;
reg MetWindowMode;
reg transport_rand;
integer TransportDelay;
integer TransportCycle;
integer TransportDelay_random;
integer TransportCycle_random;
integer RandSeed;


`ifdef DISPLAY_CDC_OVERRIDES
always @(TransportDelay) if($test$plusargs("logCdcBufTiming")) $display("info: SYNC %m.TransportDelay\t = %d : %t", TransportDelay, $time);
always @(TransportCycle) if($test$plusargs("logCdcBufTiming")) $display("info: SYNC %m.TransportCycle\t = %d : %t", TransportCycle, $time);
`endif

initial begin
    MetWindowMode = 1'b0;
    TransportDelay[31:0] = transport_delay;
    TransportDelay_random[31:0] = 0;
    TransportCycle[31:0] = transport_cycle;
    TransportCycle_random[31:0] = 0;
    if (!$value$plusargs("seed=%d", RandSeed)) begin
       RandSeed=123456;
    end
    `ifdef SYNC2ORSEED_OVERRIDE
     if (check_option_switch("Sync2OrSeedOverride")) begin
        if (get_option_switch("Sync2OrSeedOverride")) begin
          RandSeed[31:0] = get_option_value("Sync2OrSeed");
        end 
     end
     `endif //SYNC2ORSEED_OVERRIDE
    transport_rand = cdcRandomFunc(RandSeed);
    if(transport_rand) begin
        TransportDelay_random = TransportDelay;
        TransportCycle_random = TransportCycle;
    end
end

wire [WIDTH-1:0] Old_D;

    initial TRANSPORT_D[WIDTH-1:0]  = D[WIDTH-1:0] ; // initialize to D, for when FLUSH is asserted near t=0, when clk=x, ubts 622993

// fix for ubts:523633, where TRANSPORT_D is not getting initialized until CK is seen.
// for this case TRANSPORT_D should be assigned D directly, so that proper value is propagated. 
reg firstCK;  // will be valid from t=0 till first posedge of CK. 
initial firstCK <= 1'b1;
always @(posedge CK)
    if(CK & firstCK) firstCK <= 1'b0;

generate
  if (transport_cycle) begin: TC_block
    reg [transport_cycle:0] TransportCycle_reg;

    always @(D[WIDTH-1:0]) begin
      TransportCycle_reg <= 'b0;
      if (!TransportCycle_random) TRANSPORT_D[WIDTH-1:0] = D[WIDTH-1:0];
      if (firstCK) TRANSPORT_D[WIDTH-1:0] = D[WIDTH-1:0];
    end

    always @(posedge CK) begin
      if (TransportCycle_random) begin
        TransportCycle_reg = {1'b1,TransportCycle_reg[transport_cycle:1]};
        // If transport_cycle=1, then first CK edge changes data to sync model
        if (TransportCycle_reg[1]) TRANSPORT_D[WIDTH-1:0] = D[WIDTH-1:0];
      end
    end
  end else begin: TD_block
    always @(D[WIDTH-1:0]) begin //avoid producing #0 delays
        if (firstCK) TRANSPORT_D[WIDTH-1:0] = D[WIDTH-1:0];
        else if(TransportDelay_random) #(TransportDelay_random) TRANSPORT_D[WIDTH-1:0] = D[WIDTH-1:0];
        else TRANSPORT_D[WIDTH-1:0] = D[WIDTH-1:0];
    end
  end
endgenerate

assign #1 Old_D[WIDTH-1:0] = TRANSPORT_D[WIDTH-1:0];

integer i;
always @(posedge CK or TRANSPORT_D[WIDTH-1:0] or negedge EN) begin
   if(MetWindowMode)
      EnableRand[WIDTH-1:0] = {WIDTH{EN}};
   else begin
      EnableRand[WIDTH-1:0] = {WIDTH{1'b0}};
      if(EN) for(i=0;i<WIDTH;i=i+1) begin
         if(TRANSPORT_D[i] !== Old_D[i]) EnableRand[i] = 1'b1;
         else                 EnableRand[i] = 1'b0;
      end
   end
end

always @* //in case TransportDelay is forced late or changes with voltage
    if(transport_rand) TransportDelay_random = TransportDelay;

cdc_stdlib_internal_syncff_rand_model #(.metastable(metastable)) syncff_rand [WIDTH-1:0] (
   .D         (TRANSPORT_D[WIDTH-1:0]         ),
   .EN        (EN        ),
   .CK        (CK        ),
   .EnableRand(EnableRand[WIDTH-1:0]),
`ifdef ABV
    `ifdef IGNORE_IPWC_FLOATING        
    .InputChkDis(InputChkDis===1'b0 ? 1'b0 : 1'b1), //TEMP DEBUG use only. disable IPWC if InputChkDis is floating
    `else            
    .InputChkDis(InputChkDis & ~EnableIPWC), // OVA disable, simulation only
    `endif            
`endif
   .MetWindowMode(MetWindowMode     ),
   .Q         (Q[WIDTH-1:0]         )
);
// synopsys translate_on
`else // METASTABLE_MODEL
    `ifdef SYNC_DELAY_DATA
        // select D after C to provide determinism in gatesim.
        assign #0 Q[WIDTH-1:0] = D[WIDTH-1:0];
    `else
        assign Q[WIDTH-1:0] = D[WIDTH-1:0];
    `endif
`endif // METASTABLE_MODEL
// leda on

endmodule
//! enable parser

//============================================================================
// MCBUFEN - Multi-Cycle buffer
//============================================================================
module MCBUFEN (
              `ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
              DSTCLK,
              `endif
              EN, 
              `ifdef ABV
              InputChkDis,
              `endif
              A, 
              Z);

parameter  Cycles = 1;
parameter  Setup  = 11000;
parameter en_port = 1'b1;
parameter HoldCheck = 0;

//VCS coverage exclude_module

`ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
input      DSTCLK;           // Destination Clock For Z modification
`endif
input          EN;           // Z Enable
`ifdef ABV
input               InputChkDis; // assertion disable, simulation only
`endif
input          A;            // Signal from the source clock domain
output wire    Z;            // Signal to the destination clock domain


//! unused DSTCLK

`ifdef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
  assign     Z = ~(EN & A);
`else
// leda off
  wire Ax;
// synopsys translate_off
//! disable parser
  reg ForceOutDly, AssertEnable;
  integer CdcDestSetup, ForceOutDstClk, ForceOutDstClkN, DestClkCnt;
  event OutDlyClr;
  reg enable_mcbuf_x;

  initial begin
    AssertEnable = 0;
    ForceOutDstClk = 0;
    ForceOutDstClkN = 0;
    ForceOutDly = 1'b0;
    enable_mcbuf_x = 1'b0;
    CdcDestSetup = Setup; 
    DestClkCnt  = Cycles;
    #1 AssertEnable = en_port;
    enable_mcbuf_x = 1'b1; //enable after 1ps
  end

  `ifdef DISPLAY_CDC_OVERRIDES
  always @(CdcDestSetup) if($test$plusargs("logCdcBufTiming")) $display("info: MCBUF %m.CdcDestSetup\t = %d : %t", CdcDestSetup, $time);
  always @(DestClkCnt) if($test$plusargs("logCdcBufTiming")) $display("info: MCBUF %m.DestClkCnt\t = %d : %t", DestClkCnt, $time);
  always @(AssertEnable) if($test$plusargs("logCdcBufTiming")) $display("info: MCBUF %m.AssertEnable\t =           %d : %t", AssertEnable, $time);
  `endif
wire #0 A1 = A;
  always @(A1) begin
      // om  if(CdcDestSetup || DestClkCnt ) begin
      if ((CdcDestSetup | (DestClkCnt >32'd1)) & enable_mcbuf_x)  begin
          force Ax = 1'bx;
          disable ClearOutDly;
          ForceOutDly = 1'b1;
          if(DSTCLK)
          begin
          ForceOutDstClk = (DestClkCnt>32'd1)? DestClkCnt-1 : 'd0 ;
          end
          else
          begin
          ForceOutDstClkN = (DestClkCnt>32'd1)? DestClkCnt-1 : 'd0 ;
          end
          #0;
          -> OutDlyClr;
      end
  end

  always @(posedge DSTCLK) begin
      if(ForceOutDstClk) ForceOutDstClk = ForceOutDstClk - 1;
  end

  always @(negedge DSTCLK) begin
      if(ForceOutDstClkN) ForceOutDstClkN = ForceOutDstClkN - 1;
  end

  always @(OutDlyClr) begin : ClearOutDly
     #(CdcDestSetup) ForceOutDly = 1'b0;         
  end

  always @* if ((~ForceOutDly) && (ForceOutDstClk == 0) && (ForceOutDstClkN == 0)) release Ax;

//! enable parser
// synopsys translate_on

//! disable delay_restrictions
assign #0 Ax = A;
//! enable delay_restrictions
assign     Z = ~(EN & Ax);

//! unassigned AssertEnable
`ifdef ABV
// synopsys translate_off
//! disable parser
//UBTS 628249
always @(posedge DSTCLK) begin
    if (AssertEnable & (InputChkDis===1'b0) & (EN === 1'b1) & (Ax !== A) ) `AMD_ASSERT_ERROR("%m.MCBUFEN", "Error: EN failing to block metastable X output.");
    // if (AssertEnable & (EN === 1'b1) & (Ax !== A) ) `AMD_ASSERT_ERROR("%m.MCBUFEN", "Error: EN failing to block metastable X output.");
end

// hold check
reg EN_n;
reg [1:0] ck_posedge_cnt;
generate
    if (HoldCheck) begin: HoldCheck_mcbufen
        always @(negedge DSTCLK)
            EN_n    <= EN;
        // ck_posedge_cnt accumulated as 00-01-10 when EN is high. hold check is executed at DSTCLK negedge when cnt=01
        always @(posedge DSTCLK)
            if (!EN)
                ck_posedge_cnt  <= 2'b0;
            else if (|ck_posedge_cnt) 
                ck_posedge_cnt  <= 2'b10;
            else
                ck_posedge_cnt  <= ck_posedge_cnt + 1'b1;

        property n_rtl_stdlib_mcbufenclk;
          @(negedge DSTCLK) disable iff ((!AssertEnable)|(InputChkDis===1'b1)&(!ck_posedge_cnt[0])) ( (EN_n === 1'b1) & (Ax[size-1:0] !== A[size-1:0]) ) !== 1'b1;
        endproperty
        
        hold_rtl_stdlib_mcbufenclk: assert property (n_rtl_stdlib_mcbufenclk) else `AMD_ASSERT_ERROR("%m.MCBUFEN", "Error: EN failing to block hold metastable X output.");
end
endgenerate
//`ifndef CDC_STDLIB_DISABLE_OVL_NEVER_MCBUFEN
//ovl_never #(.msg("MCBUFEN Error: EN failing to block metastable X output.")) rtl_stdlib_cdcbufenclk ( .clock(DSTCLK),
//    .reset(1'b1), .enable(AssertEnable & (InputChkDis===1'b0)), .test_expr( (EN === 1'b1) & (Ax !== A) ), .fire());
//`endif
//! enable parser
// synopsys translate_on
`endif
// leda on 
`endif // TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG

endmodule


//============================================================================
// MCBUF - Multi-Cycle buffer
//============================================================================
module MCBUF (
               `ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
               DSTCLK,
               `endif
               A, 
               Z);

parameter Setup = 11000;
parameter Cycles = 1;
parameter mode01 = 0;
parameter BufMetaWaiveMode = 0;

`ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
input     DSTCLK;           // Destination Clock For Z modification
`endif
input     A;           // Signal from the source clock domain
output    Z;           // Signal to the destination clock domain

`ifdef ABV
    `ifndef RTLLIB_TEMP_KEEP_LEGACY_PARAM        
        initial begin
            if (mode01) `AMD_ASSERT_ERROR("%m.MCBUF","Error: the parameter 'mode01' is renamed as 'BufMetaWaiveMode' to reflect its behavior, update your instantiation accordingly.");
        end
        always@(mode01) begin
            if (mode01) `AMD_ASSERT_ERROR("%m.MCBUF","Error: the parameter 'mode01' is renamed as 'BufMetaWaiveMode' to relfect its behavior, update your instantiation accordingly.");
        end 
    `endif
`endif

wire    Z_INV;
MCBUFEN #(.Setup(Setup), .Cycles(Cycles), .en_port(1'b0), .HoldCheck(0) ) MCBUF_GEN(
                    .A(   A), 
                    .Z( Z_INV), 
                    `ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
                    .DSTCLK(DSTCLK),      
                    `endif
                    `ifdef ABV
                    .InputChkDis(1'b1), // since en_port is tied to 1'b0;
                    `endif
                    .EN(   1'b1));

`ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
  import "DPI-C" function int unsigned cdcRandomFunc(input int unsigned  seed);

  integer transRandomVar;
  integer randSeed; 
  integer RandType;
  integer transportCycle_random;
  integer transportDelay_random;

  initial begin
       if (!$value$plusargs("seed=%d", randSeed)) begin
        randSeed = 123456;
       end
       `ifdef SYNC2ORSEED_OVERRIDE
       if (check_option_switch("Sync2OrSeedOverride")) begin
        if (get_option_switch("Sync2OrSeedOverride")) begin
          randSeed = get_option_value("Sync2OrSeed");
        end 
       end
        `endif

       ///  When we finally get our random seed, we use it to randomly decide whether to
       ///  model the initial transport delay as 0, or as the maximum delay permissible.

       transRandomVar = (32'h7fffffff & cdcRandomFunc(randSeed));

       RandType = transRandomVar%100;

       if (RandType<25) transportCycle_random = 0; 
       else if (RandType<50) transportCycle_random = Cycles; 
       else transportCycle_random = transRandomVar%(Cycles+1);

       if (RandType<25) transportDelay_random = 0; 
       else if (RandType<50) transportDelay_random = Setup; 
       else transportDelay_random = transRandomVar%(Setup+1);
    end

// BufMetaWaiveMode should be used carefully. Z will not output X to model
// the transport delay, instead, output specific sequence.
// we have some requirements to BufMetaWaiveMode:
// 1. A pulse should be longer than CLKDST_PERIOD*Cycles, otherwise the BufMetaWaiveMode
// behavior is not guaranteed;
reg reg_A_BufMetaWaiveMode_1;
reg reg_A_BufMetaWaiveMode_2;
reg A_pre;
reg BufMetaWaiveMode_3_active; 
reg BufMetaWaiveMode_4_active; 
initial begin 
        reg_A_BufMetaWaiveMode_1 = A;
        reg_A_BufMetaWaiveMode_2 = A;
end

integer half_setup;
initial half_setup = Setup/2;
always @(Setup)
    half_setup = Setup/2;

generate
    if (BufMetaWaiveMode) begin: mcbuf_BufMetaWaiveMode
        // BufMetaWaiveMode=1, Z produce square-pulses instead of X
        always @(A) begin
            reg_A_BufMetaWaiveMode_1 = A; 
            #(half_setup)
            if (Setup && (Cycles==0)) // for Setup
                reg_A_BufMetaWaiveMode_1 = !reg_A_BufMetaWaiveMode_1; 
        end
        always @(DSTCLK)    // for Cycles
            if (Cycles)
                reg_A_BufMetaWaiveMode_1  <= !reg_A_BufMetaWaiveMode_1;  

        // BufMetaWaiveMode=2, Z is delayed by [0,CLKDST_PERIOD]*cycles, randomized    
        // BufMetaWaiveMode=3, Z passes transition to 0 instantly, but randomizes rising edge like BufMetaWaiveMode=2 
        //           This can be useful for async falling, synchronous rising signals like PwrOk.
        // BufMetaWaiveMode=4, Z always passes transition to 1 instantly, but randomizes falling edge like BufMetaWaiveMode=2
        //           This can be useful for async rising, synchronous falling signals an active high reset. 
        reg [31:0] rand_cdcbuf_delay;
        reg [31:0] rand_cdcbuf_delay_1;
        initial rand_cdcbuf_delay_1 = 0;
        always @(A) begin
            BufMetaWaiveMode_3_active = 0; 
            BufMetaWaiveMode_4_active = 0; 
            BufMetaWaiveMode_3_active = (!A & A_pre);
            BufMetaWaiveMode_4_active = (A & !A_pre);
            if ( ((BufMetaWaiveMode==3'd3) & BufMetaWaiveMode_3_active) | ((BufMetaWaiveMode==3'd4) & BufMetaWaiveMode_4_active) )
                rand_cdcbuf_delay = 0;
            else
                // rand_cdcbuf_delay={$random}%(Cycles+1); // range: [0, cycle]
                rand_cdcbuf_delay = (|Cycles) ? transportCycle_random : transportDelay_random;
            if (rand_cdcbuf_delay==0)
                rand_cdcbuf_delay_1 = !rand_cdcbuf_delay_1; // A changed and rand is 0
            A_pre <= A;
        end
        always @(posedge DSTCLK)
            rand_cdcbuf_delay <= |rand_cdcbuf_delay ? rand_cdcbuf_delay - 1'b1 : 0;
        always @(rand_cdcbuf_delay or rand_cdcbuf_delay_1)
            if (rand_cdcbuf_delay==0)
                reg_A_BufMetaWaiveMode_2 <= A;
            else
                reg_A_BufMetaWaiveMode_2 <= ~A;
    end
  endgenerate

`endif

assign Z = 

`ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
// synopsys translate_off 
// BufMetaWaiveMode:  
                     ((BufMetaWaiveMode == 3'd1) & (^Z_INV === 1'bx))
                     ? reg_A_BufMetaWaiveMode_1:

                     ((|BufMetaWaiveMode[2:1]) & (^Z_INV === 1'bx)) // BufMetaWaiveMode=2/3/4
                     ? reg_A_BufMetaWaiveMode_2:
// synopsys translate_on 
`endif
                     ~Z_INV;


endmodule



////////////////////////////////////////////////////////////////////////////////
// Clock Domain Crossing (CDC) Buffer with enable
//
//  -This buffer should be used for signals that do not use a standard synchronizer
//   and are considered to be "static" while the "EN" signal is asserted.
//
//   Note : This buffer is implemented as a NAND gate, the output is inverted.
// 
//  -The output will be driven X on every transition on A. The duration of the Xs
//   depend on the configuration, but may be based on the destination clock or and
//   a fixed delay.
//
//                   _____________________________                     ________
//    A ____________|                             |___________________|
//      __________________________                                 ____________
//    Z                           |_______________XXXXXX__________|
//                                  _____________________________
//   EN ___________________________|                             |_____________
//
////////////////////////////////////////////////////////////////////////////////
module CDCBUFEN (
               `ifdef TECH__CDCBUF__DSTCLKPORTEN
               // synopsys translate_off 
                 DSTCLK,
               // synopsys translate_on
               `endif
               EN, 
               A, 
               Z);

`ifdef TECH__CDCBUF__DSTCLKPORTEN
    // synopsys translate_off 
    input      DSTCLK;       // Destination Clock For Z modification
    // synopsys translate_on
`endif
input          EN;           // Z Enable
input          A;            // Signal from the source clock domain
output wire    Z;            // Signal to the destination clock domain

//VCS coverage exclude_module

`ifdef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
  assign     Z = ~(EN & A);
`else
  wire Ax;
// leda off
// synopsys translate_off 
//! disable parser
  ////////////////////////////////////////////////////////////////////////////////
  // This parameter defines the time that the output must be stable
  // before being sampled in the destination domain. The model will drive
  // Xs on its output for this time anytime the input changes.   
  ////////////////////////////////////////////////////////////////////////////////
  `ifdef TECH__CDCBUF__DSTSETUP
    // CDC destination setup is defined 
    integer CdcDestSetup = `TECH__CDCBUF__DSTSETUP;
  `else
    // CCD destination setup is not defined and is
    // either the default value below, or overridden on the
    // command line or instance. 
    integer CdcDestSetup = 11000; 
  `endif

  `ifdef TECH__CDCBUF__DSTCLKDEL
    // CDC destclkDelay is defined 
    `ifdef TECH__CDCBUF__DSTCLKPORTEN
      reg ClockBasedDelay = `TECH__CDCBUF__DSTCLKDEL;
    `else
      reg ClockBasedDelay = 0;
    `endif
  `else
    // CCD destClkDelay is not defined and is
    // either the default value below, or overridden on the
    // command line or instance. 
    reg ClockBasedDelay = 0; 
  `endif

  // Destination clock based X forcing :
  integer ForceOutDstClk, DestClkCnt;
  integer ForceOutDstClkN, DestClkCntN;
  initial ForceOutDstClk = 0;
  initial ForceOutDstClkN = 0;
  initial DestClkCnt = 1;
  initial DestClkCntN = 1;
  `ifdef TECH__CDCBUF__DSTCLKPORTEN
  always @(A) begin
      if (ClockBasedDelay) begin
        force Ax = 1'bx;
          if(DSTCLK)
          begin
          ForceOutDstClk = (DestClkCnt>32'd1)? DestClkCnt-1 : DestClkCnt ; // incase DestClkCntN is forced
          ForceOutDstClkN = DestClkCntN ;
          end
          else
          begin
          ForceOutDstClk = DestClkCntN ;
          ForceOutDstClkN = (DestClkCnt>32'd1)? DestClkCnt-1 : DestClkCnt ; // incase DestClkCntN is forced
          end
      end
  end

  always @(posedge DSTCLK) begin
      if(ForceOutDstClk) ForceOutDstClk = ForceOutDstClk - 1;
  end

  always @(negedge DSTCLK) begin
      if(ForceOutDstClkN) ForceOutDstClkN = ForceOutDstClkN - 1;
  end


  `ifdef DISPLAY_CDC_OVERRIDES
  always @(ClockBasedDelay) if($test$plusargs("logCdcBufTiming")) $display("info: CDCBUF %m.ClockBasedDelay\t =           %d : %t", ClockBasedDelay, $time);
  always @(DestClkCnt) if($test$plusargs("logCdcBufTiming")) $display("info: CDCBUF %m.DestClkCnt\t = %d : %t", DestClkCnt, $time);
  `endif
  `endif  //TECH__CDCBUF__DSTCLKPORTEN

  `ifdef DISPLAY_CDC_OVERRIDES
  always @(CdcDestSetup) if($test$plusargs("logCdcBufTiming")) $display("info: CDCBUF %m.CdcDestSetup\t = %d : %t", CdcDestSetup, $time);
  `endif

  // Fixed delay based X forcing :
  reg     ForceOutDly;
  event OutDlyClr;

  always @(A) begin
      force Ax = 1'bx;
      ForceOutDly = 1'b1;
      disable ClearOutDly;
      #0;
      -> OutDlyClr;
  end

  always @(OutDlyClr) begin : ClearOutDly
     #(CdcDestSetup) ForceOutDly = 1'b0;         
  end

 always @* if (~ForceOutDly && (ForceOutDstClk == 0) && (ForceOutDstClkN == 0)) release Ax;

//! enable parser
// synopsys translate_on

//! disable delay_restrictions
assign #0 Ax = A;
//! enable delay_restrictions
assign     Z = ~(EN & Ax);
`endif // TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
// leda on 

endmodule


////////////////////////////////////////////////////////////////////////////////
// Clock Domain Crossing (CDC) Buffer
//
//  -This buffer should be used for signals that do not use a standard synchronizer
//   and are considered to be "static" during operation. Fuses and other configuration
//   signals set during reset are good examples.
//
//  -The output will be driven X on every transition on A. The duration of the Xs
//   depend on the configuration, but may be based on the destination clock and a
//   fixed delay.
//
//                   ____________________________________
//   A ____________|                                    |_____________________
//                          _____________________________
//   Z ____________XXXXXXXX                             XXXXXXXX______________
//  
//
//
////////////////////////////////////////////////////////////////////////////////
module CDCBUF (
               `ifdef TECH__CDCBUF__DSTCLKPORTEN
               // synopsys translate_off 
               DSTCLK,
               // synopsys translate_on
               `endif
               A, 
               Z);

`ifdef TECH__CDCBUF__DSTCLKPORTEN
// synopsys translate_off 
input      DSTCLK;       // Destination Clock For Z modification
// synopsys translate_on 
`endif
input          A;           // Signal from the source clock domain
output wire    Z;           // Signal to the destination clock domain

wire Z_INV;
CDCBUFEN CDCBUF_GEN(.A(   A), 
                    .Z( Z_INV), 
                    `ifdef TECH__CDCBUF__DSTCLKPORTEN
                    // synopsys translate_off 
                    .DSTCLK(DSTCLK),      
                    // synopsys translate_on
                    `endif
                    .EN(   1'b1));

assign Z = ~Z_INV;
endmodule
// vrq translate_on




//============================================================================
// SYNC2OR - parameterized synchronizer with a minimum depth of 2
//============================================================================

//============================================================================
//
// Module Description
//  This is a parameterized synchronizer with a minimum (default) depth of 2:
//  1 - capture flop
//  1 - synchronization flop
//
//  A random variable is used to create a 1 flop uncertainty on the 
//  synchronizer flop chain depth.  Since RTL is typically run in a 0-time
//  environment, no attempt was made to model setup & hold violations.  The
//  logic merely randomly delays the output by an extra clock. This logic has
//  the side affect of potentially squashing a pulse if it is held for only
//  one clock period (as measured in the sampling clock domain).
//  
//  An ova is present which ensures that the input is always held for two
//  of the sampling clock periods.  This should help avoid difficult debug
//  failures where the pulse was lost due to the metastability logic and
//  created a non-obvious failure in the downstream logic.
//============================================================================

module SYNC2OR (
       SampleClk,      // Receiving Clock domain
       SampleClkEn,    // Receiving Clock Enable
       DataIn,         // Input data
       AsyncOR,        // This provides an Asynchronous OR path for Reset-like functionality.
`ifdef ABV
       OrChkDis,       // Disables the OVA for OR path checking
       InputChkDis,    // Disables the OVA for input pulse width checking.
`endif

       DataOut         // Output data
);

parameter depth = 2;    // Synchronizer depth, minimum of 2.

input      SampleClk;
input      SampleClkEn;
input      DataIn;
input      AsyncOR;
`ifdef ABV
input      OrChkDis;
input      InputChkDis;
`endif
output reg DataOut;

wire   CLK;
`ifdef SYNC_NO_CLOCK_DELAY
assign    CLK = SampleClk;
`elsif METASTABLE_MODEL
// Delay clock to ensure the metastability model tests the correct edge during races. 
// Without this delay we model metastability one cycle too late.
//! disable delay_restrictions
assign #0 CLK = SampleClk;
//! enable delay_restrictions
`else
assign    CLK = SampleClk;
`endif

reg [depth-2:0] FlopChain;
reg PreDataOut;
wire MetaData;
//! static_check (depth >= 2)


`ifdef METASTABLE_MODEL
cdc_stdlib_internal_sync_multibit sync_multibit (
   .D  (DataIn             ),
   .CK (CLK          ),
   .EN (SampleClkEn        ),
`ifdef ABV
   .InputChkDis(InputChkDis), // OVA disable, simulation only
`endif
   .Q  (MetaData           )
);
`else // METASTABLE_MODEL
    //! unused InputChkDis
    `ifdef SYNC_DELAY_DATA
        // select D after C to provide determinism in gatesim.
        //! disable delay_restrictions
        assign #0 MetaData = DataIn;
        //! enable delay_restrictions
    `else
        assign MetaData = DataIn;
    `endif
`endif // METASTABLE_MODEL

// This is the actual implemented gates
// leda off
always @(posedge CLK)
  if (SampleClkEn)
    {PreDataOut, FlopChain[depth-2:0]} <= {FlopChain[depth-2:0], MetaData};
// leda on

//! unused OrChkDis
//! unassigned OrChkDis
// We need to ensure that the logic is synchronous on the deasserting edge
// if you get an ova compile error here then you need to define assertions!
`ifdef ABV
// synopsys translate_off
// vrq translate_off
//! disable parser
ovl_never #(.msg("AsyncOR deasserted when the synchronous output was 0.")) rtl_stdlib_sync4or ( 
    .clock (CLK), .reset(1'b1), .enable(~OrChkDis), 
    .test_expr( ($past(AsyncOR,1,,@(posedge CLK)) === 1'b1) & (AsyncOR === 1'b0) & ( PreDataOut === 1'b0 )), .fire());
//! enable parser
// vrq translate_on
// synopsys translate_on
`endif

`ifdef CDC_SPYGLASS_WORKAROUND
//! unused AsyncOR
// leda off
always @*
  DataOut = PreDataOut;
`else
always @*
  DataOut = PreDataOut | AsyncOR;
`endif
// leda on
endmodule

//============================================================================
// SYNC3FLUSH - parameterized depth 3 synchronizer w/ async flush
//============================================================================

module SYNC3FLUSH (
       SampleClk,      // Receiving Clock domain
       DataIn,         // Input data
       Flush,          // Asynchronous Flush
`ifdef ABV
       InputChkDis,    // Disables the OVA for input pulse width checking.
`endif
       DataOut         // Output data
);

parameter GRAY_CODE_PTR_WIDTH = 1;
// CAUTION: only use settings other than 1 for GRAY_CODE_PTR_WIDTH in case that
// particular instance of SYNC3OR is used to synchronize a grey coded pointer or a
// similar, related, group of bits. If the group of bits is not gray coded, it must be coded
// using another one-step code.
//
// This parameter is to enable instantiating GRAY_CODE_PTR_WIDTH parallel synchronizers
// for synchronizing groups of RELATED bits, for example gray coded pointers
// this method as opposed to using N instances of SYNC3OR allows proper randomization
// of pointer bit delays through the synchronizer.
//
// Using N instances to synchronize an N bit gray coded fifo pointer will generally not work
// correctly if randomization is enabled.

parameter metastable = 1'b1;
// Set metastable to 1'b0 on the last synchronizer(s) in a chain of synchronizers, this should NOT
// be modified for a standalone synchronizer or the first synchronizer in a chain. 

parameter flush_assertion_radius = 11000;
// This can be overridden if you have other means to verify that Data is stable when Flush de-asserts
// The assertion is disabled when radius = 0

parameter transport_delay = 0;

input                                 SampleClk;
input       [GRAY_CODE_PTR_WIDTH-1:0] DataIn;
input                                 Flush;
`ifdef ABV
input                                 InputChkDis;
`endif
output wire [GRAY_CODE_PTR_WIDTH-1:0] DataOut;

wire [GRAY_CODE_PTR_WIDTH-1:0] MetaData;


wire   CLK;
`ifdef SYNC_NO_CLOCK_DELAY
assign    CLK = SampleClk;
`elsif METASTABLE_MODEL
// Delay clock to ensure the metastability model tests the correct edge during races. 
// Without this delay we model metastability one cycle too late.
//! disable delay_restrictions
assign #0 CLK = SampleClk;
//! enable delay_restrictions
`else
assign    CLK = SampleClk;
`endif

`ifdef METASTABLE_MODEL
cdc_stdlib_internal_sync_multibit #(.WIDTH(GRAY_CODE_PTR_WIDTH), .metastable(metastable), .transport_delay(transport_delay)) sync_multibit (
   .D  (DataIn[GRAY_CODE_PTR_WIDTH-1:0]     ),
   .CK (CLK                           ),
   .EN (~Flush                        ),
`ifdef ABV
   .InputChkDis(InputChkDis), // OVA disable, simulation only
`endif
   .Q  (MetaData[GRAY_CODE_PTR_WIDTH-1:0]   )
);
`else
    //! unused InputChkDis
    `ifdef SYNC_DELAY_DATA
        // select D after C to provide determinism in gatesim.
        //! disable delay_restrictions
        assign #0 MetaData[GRAY_CODE_PTR_WIDTH-1:0] = DataIn[GRAY_CODE_PTR_WIDTH-1:0];
        //! enable delay_restrictions
    `else
        assign MetaData[GRAY_CODE_PTR_WIDTH-1:0] = DataIn[GRAY_CODE_PTR_WIDTH-1:0];
    `endif
`endif


// leda off
`ifdef NO_CUTPOINTS
reg  [GRAY_CODE_PTR_WIDTH-1:0] Lat0; 
reg  [GRAY_CODE_PTR_WIDTH-1:0] Lat1; 
reg  [GRAY_CODE_PTR_WIDTH-1:0] Lat2; 
reg  [GRAY_CODE_PTR_WIDTH-1:0] Lat3; 
reg  [GRAY_CODE_PTR_WIDTH-1:0] Lat4; 
reg  [GRAY_CODE_PTR_WIDTH-1:0] Lat5; 
//! latch
always @* begin
  if(~CLK | Flush ) Lat0[GRAY_CODE_PTR_WIDTH-1:0] <= MetaData[GRAY_CODE_PTR_WIDTH-1:0];
  if( CLK | Flush ) Lat1[GRAY_CODE_PTR_WIDTH-1:0] <= Lat0[GRAY_CODE_PTR_WIDTH-1:0];
  if(~CLK | Flush ) Lat2[GRAY_CODE_PTR_WIDTH-1:0] <= Lat1[GRAY_CODE_PTR_WIDTH-1:0];
  if( CLK | Flush ) Lat3[GRAY_CODE_PTR_WIDTH-1:0] <= Lat2[GRAY_CODE_PTR_WIDTH-1:0];
  if(~CLK | Flush ) Lat4[GRAY_CODE_PTR_WIDTH-1:0] <= Lat3[GRAY_CODE_PTR_WIDTH-1:0];
  if( CLK | Flush ) Lat5[GRAY_CODE_PTR_WIDTH-1:0] <= Lat4[GRAY_CODE_PTR_WIDTH-1:0];
end
assign DataOut[GRAY_CODE_PTR_WIDTH-1:0] = Lat5[GRAY_CODE_PTR_WIDTH-1:0];
`else
reg  [GRAY_CODE_PTR_WIDTH-1:0] Stage0; 
reg  [GRAY_CODE_PTR_WIDTH-1:0] Stage1; 
reg  [GRAY_CODE_PTR_WIDTH-1:0] Stage2; 

//! disable no_multiple_drivers
`ifdef RTL_STDLIB_NO_CUSTOM_XPROP
always @(posedge CLK) begin
    //remove this section if no tool complains about the default code
    //tell the rtl_stdlib maintainer if you use this define or it will disappear!
    Stage0[GRAY_CODE_PTR_WIDTH-1:0] <= MetaData[GRAY_CODE_PTR_WIDTH-1:0];
    Stage1[GRAY_CODE_PTR_WIDTH-1:0] <= Stage0[GRAY_CODE_PTR_WIDTH-1:0];
    Stage2[GRAY_CODE_PTR_WIDTH-1:0] <= Stage1[GRAY_CODE_PTR_WIDTH-1:0];
end
`else
// vrq translate_off
always @(posedge CLK) begin
    Stage0[GRAY_CODE_PTR_WIDTH-1:0] <= (Flush | CLK) ? MetaData[GRAY_CODE_PTR_WIDTH-1:0] : Stage0[GRAY_CODE_PTR_WIDTH-1:0];
    Stage1[GRAY_CODE_PTR_WIDTH-1:0] <= Flush ? MetaData[GRAY_CODE_PTR_WIDTH-1:0] : (CLK ? Stage0[GRAY_CODE_PTR_WIDTH-1:0] : Stage1[GRAY_CODE_PTR_WIDTH-1:0]);
    Stage2[GRAY_CODE_PTR_WIDTH-1:0] <= Flush ? MetaData[GRAY_CODE_PTR_WIDTH-1:0] : (CLK ? Stage1[GRAY_CODE_PTR_WIDTH-1:0] : Stage2[GRAY_CODE_PTR_WIDTH-1:0]);
end
// vrq translate_on
`endif

// vrq translate_off
//! disable parser
//! latch
always @* begin
    Stage0[GRAY_CODE_PTR_WIDTH-1:0] = Flush ? MetaData[GRAY_CODE_PTR_WIDTH-1:0] : Stage0[GRAY_CODE_PTR_WIDTH-1:0];
    Stage1[GRAY_CODE_PTR_WIDTH-1:0] = Flush ? MetaData[GRAY_CODE_PTR_WIDTH-1:0] : Stage1[GRAY_CODE_PTR_WIDTH-1:0];
    Stage2[GRAY_CODE_PTR_WIDTH-1:0] = Flush ? MetaData[GRAY_CODE_PTR_WIDTH-1:0] : Stage2[GRAY_CODE_PTR_WIDTH-1:0];
end
//! enable parser
//! enable no_multiple_drivers
// vrq translate_on

assign DataOut[GRAY_CODE_PTR_WIDTH-1:0] = Stage2[GRAY_CODE_PTR_WIDTH-1:0];
`endif
// leda on

// synopsys translate_off
// vrq translate_off
//! disable parser
//  if you get an ova compile error here then you need to define assertions!
`ifdef ABV
ovl_win_unchange #(.msg("DataIn transitioning while Flush is enabled and SampleClk is toggling, metastability may occur!"), .width(GRAY_CODE_PTR_WIDTH)) 
    rtl_stdlib_sync3flush_window (  .clock (SampleClk), .reset(1'b1), .enable( ~InputChkDis ), .start_event( Flush === 1'b1), .end_event( Flush === 1'b0 ),
    .test_expr( DataIn[GRAY_CODE_PTR_WIDTH-1:0] ), .fire());
`ifndef HWEMUL
// Flush falling is equivalent to a latch closing. DataIn needs to be stable on Flush's falling edge.
wire [GRAY_CODE_PTR_WIDTH-1:0] DataInDelay;
wire FlushDelay;
assign #(flush_assertion_radius * 2) DataInDelay[GRAY_CODE_PTR_WIDTH-1:0] = DataIn[GRAY_CODE_PTR_WIDTH-1:0];
assign #(flush_assertion_radius) FlushDelay = Flush;
//
//UBTS 628249
//ovl_never #(.msg("DataIn transitioned close to the deasserting flush edge, metastability may occur!"), .clock_edge(`OVL_NEGEDGE)) rtl_stdlib_sync3flush ( 
//    .clock (FlushDelay), .reset(1'b1), .enable( ~InputChkDis ),
//    .test_expr( |(DataIn[GRAY_CODE_PTR_WIDTH-1:0] ^ DataInDelay[GRAY_CODE_PTR_WIDTH-1:0]) === 1'b1 ), .fire());
always@(negedge (FlushDelay)) begin
    if (~InputChkDis & (|(DataIn[GRAY_CODE_PTR_WIDTH-1:0] ^DataInDelay[GRAY_CODE_PTR_WIDTH-1:0]) === 1'b1 ) ) `AMD_ASSERT_ERROR("%m.sync3flush","Error: DataIn transitioned close to the deasserting flush edge, metastability may occur!");
end 
`endif
`endif
//! enable parser
// vrq translate_on
// synopsys translate_on

endmodule



//============================================================================
// SYNC4FLUSH - parameterized depth 4 synchronizer w/ async flush
//============================================================================

module SYNC4FLUSH (
       SampleClk,      // Receiving Clock domain
       DataIn,         // Input data
       Flush,          // Asynchronous Flush
`ifdef ABV
       InputChkDis,    // Disables the OVA for input pulse width checking.
`endif
       DataOut         // Output data
);

parameter GRAY_CODE_PTR_WIDTH = 1;
// CAUTION: only use settings other than 1 for GRAY_CODE_PTR_WIDTH in case that
// particular instance of SYNC3OR is used to synchronize a grey coded pointer or a
// similar, related, group of bits. If the group of bits is not gray coded, it must be coded
// using another one-step code.
//
// This parameter is to enable instantiating GRAY_CODE_PTR_WIDTH parallel synchronizers
// for synchronizing groups of RELATED bits, for example gray coded pointers
// this method as opposed to using N instances of SYNC3OR allows proper randomization
// of pointer bit delays through the synchronizer.
//
// Using N instances to synchronize an N bit gray coded fifo pointer will generally not work
// correctly if randomization is enabled.

parameter metastable = 1'b1;
// Set this to 1'b0 on the last synchronizer(s) in a chain of synchronizers, this should NOT
// be modified for a standalone synchronizer or the first synchronizer in a chain. 

parameter flush_assertion_radius = 11000;
// This can be overridden if you have other means to verify that Data is stable when Flush de-asserts
// The assertion is disabled when radius = 0

parameter transport_delay = 0;

input                                 SampleClk;
input       [GRAY_CODE_PTR_WIDTH-1:0] DataIn;
input                                 Flush;
`ifdef ABV
input                                 InputChkDis;
`endif
output wire [GRAY_CODE_PTR_WIDTH-1:0] DataOut;

wire [GRAY_CODE_PTR_WIDTH-1:0] MetaData;

wire   CLK;
`ifdef SYNC_NO_CLOCK_DELAY
assign    CLK = SampleClk;
`elsif METASTABLE_MODEL
// Delay clock to ensure the metastability model tests the correct edge during races. 
// Without this delay we model metastability one cycle too late.
//! disable delay_restrictions
assign #0 CLK = SampleClk;
//! enable delay_restrictions
`else
assign    CLK = SampleClk;
`endif

`ifdef METASTABLE_MODEL
cdc_stdlib_internal_sync_multibit #(.WIDTH(GRAY_CODE_PTR_WIDTH), .metastable(metastable), .transport_delay(transport_delay)) sync_multibit (
   .D  (DataIn[GRAY_CODE_PTR_WIDTH-1:0]     ),
   .CK (CLK                           ),
   .EN (~Flush                        ),
`ifdef ABV
   .InputChkDis(InputChkDis), // OVA disable, simulation only
`endif
   .Q  (MetaData[GRAY_CODE_PTR_WIDTH-1:0]   )
);
`else
    //! unused InputChkDis
    `ifdef SYNC_DELAY_DATA
        // select D after C to provide determinism in gatesim.
        //! disable delay_restrictions
        assign #0 MetaData[GRAY_CODE_PTR_WIDTH-1:0] = DataIn[GRAY_CODE_PTR_WIDTH-1:0];
        //! enable delay_restrictions
    `else
        assign MetaData[GRAY_CODE_PTR_WIDTH-1:0] = DataIn[GRAY_CODE_PTR_WIDTH-1:0];
    `endif
`endif


// leda off
`ifdef NO_CUTPOINTS
reg  [GRAY_CODE_PTR_WIDTH-1:0] Lat0; 
reg  [GRAY_CODE_PTR_WIDTH-1:0] Lat1; 
reg  [GRAY_CODE_PTR_WIDTH-1:0] Lat2; 
reg  [GRAY_CODE_PTR_WIDTH-1:0] Lat3; 
reg  [GRAY_CODE_PTR_WIDTH-1:0] Lat4; 
reg  [GRAY_CODE_PTR_WIDTH-1:0] Lat5; 
reg  [GRAY_CODE_PTR_WIDTH-1:0] Lat6; 
reg  [GRAY_CODE_PTR_WIDTH-1:0] Lat7; 
//! latch
always @* begin
  if(~CLK | Flush ) Lat0[GRAY_CODE_PTR_WIDTH-1:0] <= MetaData[GRAY_CODE_PTR_WIDTH-1:0];
  if( CLK | Flush ) Lat1[GRAY_CODE_PTR_WIDTH-1:0] <= Lat0[GRAY_CODE_PTR_WIDTH-1:0];
  if(~CLK | Flush ) Lat2[GRAY_CODE_PTR_WIDTH-1:0] <= Lat1[GRAY_CODE_PTR_WIDTH-1:0];
  if( CLK | Flush ) Lat3[GRAY_CODE_PTR_WIDTH-1:0] <= Lat2[GRAY_CODE_PTR_WIDTH-1:0];
  if(~CLK | Flush ) Lat4[GRAY_CODE_PTR_WIDTH-1:0] <= Lat3[GRAY_CODE_PTR_WIDTH-1:0];
  if( CLK | Flush ) Lat5[GRAY_CODE_PTR_WIDTH-1:0] <= Lat4[GRAY_CODE_PTR_WIDTH-1:0];
  if(~CLK | Flush ) Lat6[GRAY_CODE_PTR_WIDTH-1:0] <= Lat5[GRAY_CODE_PTR_WIDTH-1:0];
  if( CLK | Flush ) Lat7[GRAY_CODE_PTR_WIDTH-1:0] <= Lat6[GRAY_CODE_PTR_WIDTH-1:0];
end
assign DataOut[GRAY_CODE_PTR_WIDTH-1:0] = Lat7[GRAY_CODE_PTR_WIDTH-1:0];
`else
reg  [GRAY_CODE_PTR_WIDTH-1:0] Stage0; 
reg  [GRAY_CODE_PTR_WIDTH-1:0] Stage1; 
reg  [GRAY_CODE_PTR_WIDTH-1:0] Stage2; 
reg  [GRAY_CODE_PTR_WIDTH-1:0] Stage3; 

//! disable no_multiple_drivers
`ifdef RTL_STDLIB_NO_CUSTOM_XPROP
//remove this section if no tool complains about the default code
//tell the rtl_stdlib maintainer if you use this define or it will disappear!
always @(posedge CLK) begin
    Stage0[GRAY_CODE_PTR_WIDTH-1:0] <= MetaData[GRAY_CODE_PTR_WIDTH-1:0];
    Stage1[GRAY_CODE_PTR_WIDTH-1:0] <= Stage0[GRAY_CODE_PTR_WIDTH-1:0];
    Stage2[GRAY_CODE_PTR_WIDTH-1:0] <= Stage1[GRAY_CODE_PTR_WIDTH-1:0];
    Stage3[GRAY_CODE_PTR_WIDTH-1:0] <= Stage2[GRAY_CODE_PTR_WIDTH-1:0];
end
`else
// vrq translate_off
always @(posedge CLK) begin
    Stage0[GRAY_CODE_PTR_WIDTH-1:0] <= (Flush | CLK) ? MetaData[GRAY_CODE_PTR_WIDTH-1:0] : Stage0[GRAY_CODE_PTR_WIDTH-1:0];
    Stage1[GRAY_CODE_PTR_WIDTH-1:0] <= Flush ? MetaData[GRAY_CODE_PTR_WIDTH-1:0] : (CLK ? Stage0[GRAY_CODE_PTR_WIDTH-1:0] : Stage1[GRAY_CODE_PTR_WIDTH-1:0]);
    Stage2[GRAY_CODE_PTR_WIDTH-1:0] <= Flush ? MetaData[GRAY_CODE_PTR_WIDTH-1:0] : (CLK ? Stage1[GRAY_CODE_PTR_WIDTH-1:0] : Stage2[GRAY_CODE_PTR_WIDTH-1:0]);
    Stage3[GRAY_CODE_PTR_WIDTH-1:0] <= Flush ? MetaData[GRAY_CODE_PTR_WIDTH-1:0] : (CLK ? Stage2[GRAY_CODE_PTR_WIDTH-1:0] : Stage3[GRAY_CODE_PTR_WIDTH-1:0]);
end
// vrq translate_on
`endif

// vrq translate_off
//! disable parser
//! latch
always @* begin
    Stage0[GRAY_CODE_PTR_WIDTH-1:0] = Flush ? MetaData[GRAY_CODE_PTR_WIDTH-1:0] : Stage0[GRAY_CODE_PTR_WIDTH-1:0];
    Stage1[GRAY_CODE_PTR_WIDTH-1:0] = Flush ? MetaData[GRAY_CODE_PTR_WIDTH-1:0] : Stage1[GRAY_CODE_PTR_WIDTH-1:0];
    Stage2[GRAY_CODE_PTR_WIDTH-1:0] = Flush ? MetaData[GRAY_CODE_PTR_WIDTH-1:0] : Stage2[GRAY_CODE_PTR_WIDTH-1:0];
    Stage3[GRAY_CODE_PTR_WIDTH-1:0] = Flush ? MetaData[GRAY_CODE_PTR_WIDTH-1:0] : Stage3[GRAY_CODE_PTR_WIDTH-1:0];
end
//! enable parser
//! enable no_multiple_drivers
// vrq translate_on

assign DataOut[GRAY_CODE_PTR_WIDTH-1:0] = Stage3[GRAY_CODE_PTR_WIDTH-1:0];
`endif
// leda on

// synopsys translate_off
// vrq translate_off
//! disable parser
//  if you get an ova compile error here then you need to define assertions!
`ifdef ABV
ovl_win_unchange #(.msg("DataIn transitioning while Flush is enabled and SampleClk is toggling, metastability may occur!"), .width(GRAY_CODE_PTR_WIDTH)) 
    rtl_stdlib_sync4flush_window (  .clock (SampleClk), .reset(1'b1), .enable( ~InputChkDis ), .start_event( Flush === 1'b1), .end_event( Flush === 1'b0 ),
    .test_expr( DataIn[GRAY_CODE_PTR_WIDTH-1:0] ), .fire());
`ifndef HWEMUL
// Flush falling is equivalent to a latch closing. DataIn needs to be stable on Flush's falling edge.
wire [GRAY_CODE_PTR_WIDTH-1:0] DataInDelay;
wire FlushDelay;
assign #(flush_assertion_radius * 2) DataInDelay[GRAY_CODE_PTR_WIDTH-1:0] = DataIn[GRAY_CODE_PTR_WIDTH-1:0];
assign #(flush_assertion_radius) FlushDelay = Flush;
// UBTS 628249
//ovl_never #(.msg("DataIn transitioned close to the deasserting flush edge, metastability may occur!"), .clock_edge(`OVL_NEGEDGE)) rtl_stdlib_sync4flush ( 
//    .clock (FlushDelay), .reset(1'b1), .enable( ~InputChkDis ),
//    .test_expr( |(DataIn[GRAY_CODE_PTR_WIDTH-1:0] ^ DataInDelay[GRAY_CODE_PTR_WIDTH-1:0]) === 1'b1 ), .fire());
always@(negedge (FlushDelay)) begin
    if (~InputChkDis & (|(DataIn[GRAY_CODE_PTR_WIDTH-1:0] ^ DataInDelay[GRAY_CODE_PTR_WIDTH-1:0]) === 1'b1 ) ) `AMD_ASSERT_ERROR("%m.sync4flush","Error: DataIn transitioned close to the deasserting flush edge, metastability may occur!");
end

`endif
`endif
//! enable parser
// vrq translate_on
// synopsys translate_on

endmodule



//============================================================================
// SYNC3OR - parameterized depth 3 synchronizer
//============================================================================

//============================================================================
//
// Module Description
//  This is a depth 3 synchronizer
//  2 - capture flop
//  1 - synchronization flop
//
//  A random variable is used to create a 1 flop uncertainty on the 
//  synchronizer flop chain depth.  Since RTL is typically run in a 0-time
//  environment, no attempt was made to model setup & hold violations.  The
//  logic merely randomly delays the output by an extra clock. This logic has
//  the side affect of potentially squashing a pulse if it is held for only
//  one clock period (as measured in the sampling clock domain).
//  
//  An ova is present which ensures that the input is always held for two
//  of the sampling clock periods.  This should help avoid difficult debug
//  failures where the pulse was lost due to the metastability logic and
//  created a non-obvious failure in the downstream logic.
//============================================================================

module SYNC3OR (
       SampleClk,      // Receiving Clock domain
       SampleClkEn,    // Receiving Clock Enable
       DataIn,         // Input data
       AsyncOR,        // This provides an Asynchronous OR path for Reset-like functionality.
`ifdef ABV
       OrChkDis,       // Disables the OVA for OR path checking
       InputChkDis,    // Disables the OVA for input pulse width checking.
`endif

       DataOut         // Output data
);

parameter GRAY_CODE_PTR_WIDTH = 1;
// CAUTION: only use settings other than 1 for GRAY_CODE_PTR_WIDTH in case that
// particular instance of SYNC3OR is used to synchronize a grey coded pointer or a
// similar, related, group of bits. If the group of bits is not gray coded, it must be coded
// using another one-step code.
//
// This parameter is to enable instantiating GRAY_CODE_PTR_WIDTH parallel synchronizers
// for synchronizing groups of RELATED bits, for example gray coded pointers
// this method as opposed to using N instances of SYNC3OR allows proper randomization
// of pointer bit delays through the synchronizer.
//
// Using N instances to synchronize an N bit gray coded fifo pointer will generally not work
// correctly if randomization is enabled.

parameter metastable = 1'b1;
// Set this to 1'b0 on the last synchronizer(s) in a chain of synchronizers, this should NOT
// be modified for a standalone synchronizer or the first synchronizer in a chain. 

parameter transport_delay = 0;

input                                 SampleClk;
input                                 SampleClkEn;
input       [GRAY_CODE_PTR_WIDTH-1:0] DataIn;
input                                 AsyncOR;
`ifdef ABV
input                                 OrChkDis;
input                                 InputChkDis;
`endif
output wire [GRAY_CODE_PTR_WIDTH-1:0] DataOut;

wire CK_EN;
wire CKGaterLatch;

wire   CLK;
`ifdef SYNC_NO_CLOCK_DELAY
assign    CLK = SampleClk;
`elsif METASTABLE_MODEL
// Delay clock to ensure the metastability model tests the correct edge during races. 
// Without this delay we model metastability one cycle too late.
//! disable delay_restrictions
assign #0 CLK = SampleClk;
//! enable delay_restrictions
`else
assign    CLK = SampleClk;
`endif

CDC_STDLIB_LATCH CKGaterLatchInst (.Q(CKGaterLatch), .D(SampleClkEn), .C(~CLK));
assign CK_EN = CLK & CKGaterLatch;


`ifdef DEPRECATED_32NM

wire [GRAY_CODE_PTR_WIDTH-1:0] MetaData;
reg  [GRAY_CODE_PTR_WIDTH-1:0] Stage0; 
reg  [GRAY_CODE_PTR_WIDTH-1:0] Stage1; 
reg  [GRAY_CODE_PTR_WIDTH-1:0] Stage2; 

`ifdef METASTABLE_MODEL
cdc_stdlib_internal_sync_multibit #(.WIDTH(GRAY_CODE_PTR_WIDTH), .metastable(metastable), .transport_delay(transport_delay)) sync_multibit (
   .D  (DataIn[GRAY_CODE_PTR_WIDTH-1:0]     ),
   .CK (CLK                           ),
   .EN (SampleClkEn                         ),
`ifdef ABV
   .InputChkDis(InputChkDis), // OVA disable, simulation only
`endif
   .Q  (MetaData[GRAY_CODE_PTR_WIDTH-1:0]   )
);
`else
    //! unused InputChkDis
    `ifdef SYNC_DELAY_DATA
        // select D after C to provide determinism in gatesim.
        //! disable delay_restrictions
        assign #0 MetaData[GRAY_CODE_PTR_WIDTH-1:0] = DataIn[GRAY_CODE_PTR_WIDTH-1:0];
        //! enable delay_restrictions
    `else
        assign MetaData[GRAY_CODE_PTR_WIDTH-1:0] = DataIn[GRAY_CODE_PTR_WIDTH-1:0];
    `endif
`endif


// leda off
always @(posedge CK_EN) begin
    Stage0[GRAY_CODE_PTR_WIDTH-1:0] <= MetaData[GRAY_CODE_PTR_WIDTH-1:0];
    Stage1[GRAY_CODE_PTR_WIDTH-1:0] <= Stage0[GRAY_CODE_PTR_WIDTH-1:0];
    Stage2[GRAY_CODE_PTR_WIDTH-1:0] <= Stage1[GRAY_CODE_PTR_WIDTH-1:0];
end
// leda on

`else //28NM and beyond, only flushable synchronizers exist

wire  [GRAY_CODE_PTR_WIDTH-1:0] Stage2; 
SYNC3FLUSH #(.GRAY_CODE_PTR_WIDTH(GRAY_CODE_PTR_WIDTH), .metastable(metastable)) sync3fl (
       .SampleClk(CK_EN),
       .DataIn(DataIn[GRAY_CODE_PTR_WIDTH-1:0]),
       .Flush(1'b0),
`ifdef ABV
       .InputChkDis(InputChkDis),
`endif
       .DataOut(Stage2[GRAY_CODE_PTR_WIDTH-1:0])
);

`endif //DEPRECATED_32NM

assign DataOut[GRAY_CODE_PTR_WIDTH-1:0] = {GRAY_CODE_PTR_WIDTH{AsyncOR}} | Stage2[GRAY_CODE_PTR_WIDTH-1:0];
//! unused OrChkDis
//! unassigned OrChkDis
// We need to ensure that the logic is synchronous on the deasserting edge
// synopsys translate_off
//! disable parser
//  if you get an ova compile error here then you need to define assertions!
`ifdef ABV
// vrq translate_off
ovl_never #(.msg("AsyncOR deasserted when the synchronous output was 0.")) rtl_stdlib_sync3or ( 
    .clock (CLK), .reset(1'b1), .enable(~OrChkDis), 
    .test_expr( ($past(AsyncOR,1,,@(posedge CLK)) === 1'b1) & (AsyncOR === 1'b0) & ( &Stage2[GRAY_CODE_PTR_WIDTH-1:0] === 1'b0 )), .fire());
// vrq translate_on
`endif
//! enable parser
// synopsys translate_on

endmodule




//============================================================================
// SYNC4OR - parameterized depth 3 synchronizer
//============================================================================

module SYNC4OR (
       SampleClk,      // Receiving Clock domain
       SampleClkEn,    // Receiving Clock Enable
       DataIn,         // Input data
       AsyncOR,        // This provides an Asynchronous OR path for Reset-like functionality.
`ifdef ABV
       OrChkDis,       // Disables the OVA for OR path checking
       InputChkDis,    // Disables the OVA for input pulse width checking.
`endif

       DataOut         // Output data
);

parameter GRAY_CODE_PTR_WIDTH = 1;
// CAUTION: only use settings other than 1 for GRAY_CODE_PTR_WIDTH in case that
// particular instance of SYNC4OR is used to synchronize a grey coded pointer or a
// similar, related, group of bits. If the group of bits is not gray coded, it must be coded
// using another one-step code.
//
// This parameter is to enable instantiating GRAY_CODE_PTR_WIDTH parallel synchronizers
// for synchronizing groups of RELATED bits, for example gray coded pointers
// this method as opposed to using N instances of SYNC4OR allows proper randomization
// of pointer bit delays through the synchronizer.
//
// Using N instances to synchronize an N bit gray coded fifo pointer will generally not work
// correctly if randomization is enabled.

parameter metastable = 1'b1;
// Set this to 1'b0 on the last synchronizer(s) in a chain of synchronizers, this should NOT
// be modified for a standalone synchronizer or the first synchronizer in a chain. 

parameter transport_delay = 0;

input                                 SampleClk;
input                                 SampleClkEn;
input       [GRAY_CODE_PTR_WIDTH-1:0] DataIn;
input                                 AsyncOR;
`ifdef ABV
input                                 OrChkDis;
input                                 InputChkDis;
`endif
output wire [GRAY_CODE_PTR_WIDTH-1:0] DataOut;

wire CK_EN;
wire CKGaterLatch;

wire   CLK;
`ifdef SYNC_NO_CLOCK_DELAY
assign    CLK = SampleClk;
`elsif METASTABLE_MODEL
// Delay clock to ensure the metastability model tests the correct edge during races. 
// Without this delay we model metastability one cycle too late.
//! disable delay_restrictions
assign #0 CLK = SampleClk;
//! enable delay_restrictions
`else
assign    CLK = SampleClk;
`endif

CDC_STDLIB_LATCH CKGaterLatchInst (.Q(CKGaterLatch), .D(SampleClkEn), .C(~CLK));
assign CK_EN = CLK & CKGaterLatch;

`ifdef MODEL_SYNC4OR_AS_DFF

wire [GRAY_CODE_PTR_WIDTH-1:0] MetaData;
reg  [GRAY_CODE_PTR_WIDTH-1:0] Stage0; 
reg  [GRAY_CODE_PTR_WIDTH-1:0] Stage1; 
reg  [GRAY_CODE_PTR_WIDTH-1:0] Stage2; 
reg  [GRAY_CODE_PTR_WIDTH-1:0] Stage3; 

`ifdef METASTABLE_MODEL
cdc_stdlib_internal_sync_multibit #(.WIDTH(GRAY_CODE_PTR_WIDTH), .metastable(metastable)) sync_multibit (
   .D  (DataIn[GRAY_CODE_PTR_WIDTH-1:0]     ),
   .CK (CLK                           ),
   .EN (SampleClkEn                         ),
`ifdef ABV
   .InputChkDis(InputChkDis), // OVA disable, simulation only
`endif
   .Q  (MetaData[GRAY_CODE_PTR_WIDTH-1:0]   )
);
`else
    //! unused InputChkDis
    `ifdef SYNC_DELAY_DATA
        // select D after C to provide determinism in gatesim.
        //! disable delay_restrictions
        assign #0 MetaData[GRAY_CODE_PTR_WIDTH-1:0] = DataIn[GRAY_CODE_PTR_WIDTH-1:0];
        //! enable delay_restrictions
    `else
        assign MetaData[GRAY_CODE_PTR_WIDTH-1:0] = DataIn[GRAY_CODE_PTR_WIDTH-1:0];
    `endif
`endif

// leda off
always @(posedge CK_EN) begin
    Stage0[GRAY_CODE_PTR_WIDTH-1:0] <= MetaData[GRAY_CODE_PTR_WIDTH-1:0];
    Stage1[GRAY_CODE_PTR_WIDTH-1:0] <= Stage0[GRAY_CODE_PTR_WIDTH-1:0];
    Stage2[GRAY_CODE_PTR_WIDTH-1:0] <= Stage1[GRAY_CODE_PTR_WIDTH-1:0];
    Stage3[GRAY_CODE_PTR_WIDTH-1:0] <= Stage2[GRAY_CODE_PTR_WIDTH-1:0];
end
// leda on


`elsif DEPRECATED_32NM  //MODEL_AS_DFF

wire [GRAY_CODE_PTR_WIDTH-1:0] MetaData;
reg  [GRAY_CODE_PTR_WIDTH-1:0] Stage0; 
reg  [GRAY_CODE_PTR_WIDTH-1:0] Stage1; 
reg  [GRAY_CODE_PTR_WIDTH-1:0] Stage2; 
reg  [GRAY_CODE_PTR_WIDTH-1:0] Stage3; 

`ifdef METASTABLE_MODEL
cdc_stdlib_internal_sync_multibit #(.WIDTH(GRAY_CODE_PTR_WIDTH), .metastable(metastable), .transport_delay(transport_delay)) sync_multibit (
   .D  (DataIn[GRAY_CODE_PTR_WIDTH-1:0]     ),
   .CK (CLK                           ),
   .EN (SampleClkEn                         ),
`ifdef ABV
   .InputChkDis(InputChkDis), // OVA disable, simulation only
`endif
   .Q  (MetaData[GRAY_CODE_PTR_WIDTH-1:0]   )
);
`else
    //! unused InputChkDis
    `ifdef SYNC_DELAY_DATA
        // select D after C to provide determinism in gatesim.
        //! disable delay_restrictions
        assign #0 MetaData[GRAY_CODE_PTR_WIDTH-1:0] = DataIn[GRAY_CODE_PTR_WIDTH-1:0];
        //! enable delay_restrictions
    `else
        assign MetaData[GRAY_CODE_PTR_WIDTH-1:0] = DataIn[GRAY_CODE_PTR_WIDTH-1:0];
    `endif
`endif

// leda off
always @(posedge CK_EN) begin
    Stage0[GRAY_CODE_PTR_WIDTH-1:0] <= MetaData[GRAY_CODE_PTR_WIDTH-1:0];
    Stage1[GRAY_CODE_PTR_WIDTH-1:0] <= Stage0[GRAY_CODE_PTR_WIDTH-1:0];
    Stage2[GRAY_CODE_PTR_WIDTH-1:0] <= Stage1[GRAY_CODE_PTR_WIDTH-1:0];
    Stage3[GRAY_CODE_PTR_WIDTH-1:0] <= Stage2[GRAY_CODE_PTR_WIDTH-1:0];
end
// leda on

`else //28nm only has flushable synchronizers

wire  [GRAY_CODE_PTR_WIDTH-1:0] Stage3; 
SYNC4FLUSH #(.GRAY_CODE_PTR_WIDTH(GRAY_CODE_PTR_WIDTH), .metastable(metastable)) sync3fl (
       .SampleClk(CK_EN),
       .DataIn(DataIn[GRAY_CODE_PTR_WIDTH-1:0]),
       .Flush(1'b0),
`ifdef ABV
       .InputChkDis(InputChkDis),
`endif
       .DataOut(Stage3[GRAY_CODE_PTR_WIDTH-1:0])
);

`endif //DEPRECATED_32NM

assign DataOut[GRAY_CODE_PTR_WIDTH-1:0] = {GRAY_CODE_PTR_WIDTH{AsyncOR}} | Stage3[GRAY_CODE_PTR_WIDTH-1:0];
//! unused OrChkDis
//! unassigned OrChkDis
// We need to ensure that the logic is synchronous on the deasserting edge
// synopsys translate_off
//! disable parser
//  if you get an ova compile error here then you need to define assertions!
`ifdef ABV
// vrq translate_off
ovl_never #(.msg("AsyncOR deasserted when the synchronous output was 0.")) rtl_stdlib_sync4or ( 
    .clock (CLK), .reset(1'b1), .enable(~OrChkDis), 
    .test_expr( ($past(AsyncOR,1,,@(posedge CLK)) === 1'b1) & (AsyncOR === 1'b0) & ( &Stage3[GRAY_CODE_PTR_WIDTH-1:0] === 1'b0 )), .fire());
// vrq translate_on
`endif
//! enable parser
// synopsys translate_on

endmodule

// vrq translate_off

//============================================================================
// CDCBUF01 - Should not be used for most hardware. It is intended for very 
// limited use in certain very specific cases.  Please see the Twiki 
// (http://twiki.amd.com/twiki/bin/view/Verif/FalsepathMarker_cell).
// If you use this cell in your design, you WILL be audited and will have to 
// justify the usage to a panel of experts in each SOC and document what 
// additional work you have done to prove your usage of the cell is safe.
//============================================================================
module CDCBUF01 ( A, Z);

parameter Setup = 11000;
parameter Monotonic = 1'b0;

input          A;           // Signal from the source clock domain

`ifdef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
output wire    Z;           // Signal to the destination clock domain
assign Z = A;
`else
import "DPI-C" function int unsigned cdcRandomFunc(input int unsigned  seed);
output reg     Z;           // Signal to the destination clock domain
//! unassigned Z
//! unused A
//! disable parser
// synopsys translate_off
// leda off
//VCS coverage exclude_module

integer RandSeed, TransitionDelay, Strobe, CdcDestSetup, RandVal;
reg CdcMonotonic;
wire randbit;
reg inflight,inverted;
wire A_after_randbit;

// reusing synchronizer randomizer. Make sure METASTABLE_MODEL is defined!
cdc_stdlib_internal_met_model randgen ( .D(A), .EN(~CdcMonotonic), .Dsync(A_after_randbit), .RandomExtraCycle(randbit));

initial begin
    if (!$value$plusargs("seed=%d", RandSeed)) begin
      RandSeed = 123456;
    end
    Z = 1'b0;
    CdcDestSetup = Setup;
    CdcMonotonic = Monotonic;
    $display("CDCBUF01 found at %m");
    inflight = 1'b0;
    RandVal = cdcRandomFunc(RandSeed);
end

`ifdef DISPLAY_CDC_OVERRIDES
always @(CdcDestSetup) if($test$plusargs("logCdcBufTiming")) $display("info: CDCBUF01 %m.CdcDestSetup\t = %d : %t", CdcDestSetup, $time);
always @(CdcMonotonic) if($test$plusargs("logCdcBufTiming")) $display("info: CDCBUF01 %m.CdcMonotonic\t =           %d : %t", CdcMonotonic, $time);
`endif

always @(CdcDestSetup) begin
    TransitionDelay = RandVal % CdcDestSetup;
    // RandVal = (RandVal >> 11) ^ ((RandVal & 32'h7ff) << 21) ^ ((RandVal & 1) ? 0x80080100 : 0);
    RandVal = (RandVal >> 11) ^ ((RandVal & 32'h7ff) << 21) ^ ((RandVal & 1) ? 32'h80080100 : 0);
    Strobe = RandVal % (CdcDestSetup-2) + 1;
    // RandVal = (RandVal >> 11) ^ ((RandVal & 32'h7ff) << 21) ^ ((RandVal & 1) ? 0x80080100 : 0);
    RandVal = (RandVal >> 11) ^ ((RandVal & 32'h7ff) << 21) ^ ((RandVal & 1) ? 32'h80080100 : 0);
end


always @(A) disable DelayBlock;

always @(A_after_randbit) begin : DelayBlock
    if(~inflight) begin
        // only update pseudo-random vars after a complete CdcDestSetup to avoid losing glitchy edges
        TransitionDelay = (TransitionDelay + Strobe) % CdcDestSetup;
        inverted = randbit;
        // immediately update if we are inverted
        if(~CdcMonotonic & inverted) Z = A;
        inflight = 1'b1;
    end
    Z <= #(TransitionDelay) (~CdcMonotonic & inverted) ^ A;
    #(CdcDestSetup[31:0]);
    inflight = 1'b0;
    Z = A_after_randbit;
end

// synopsys translate_on
//! enable parser
// leda on

`endif

endmodule



//============================================================================
// CDCBUFENCLK - CDCBUFEN with paremeters & a non-optional DSTCLK port
//============================================================================
module CDCBUFENCLK (
              `ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
              DSTCLK,
              `endif
              EN, 
              `ifdef ABV
              InputChkDis,
              `endif
              A, 
              Z);

parameter  Cycles = 1;
parameter  Setup  = 11000;
parameter en_port = 1'b1;
parameter size = 1;
parameter HoldCheck = 0;
parameter SrcPath = 0;

//VCS coverage exclude_module

`ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
input      DSTCLK;           // Destination Clock For Z modification
`endif
input          EN;           // Z Enable
`ifdef ABV
input       InputChkDis;   ///  disables input pulse-width checking 
`endif
input       [size-1:0]   A;            // Signal from the source clock domain
output wire [size-1:0]   Z;            // Signal to the destination clock domain


//! unused DSTCLK

`ifdef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
  assign     Z[size-1:0] = ~({size{EN}} & A[size-1:0]);
`else
// leda off
reg SrcPathEn;
initial begin SrcPathEn = 1'b0; end
always @*
    if (!SrcPathEn&SrcPath) begin
        $display("Error: %m : parameter SrcPath should not be set 1 when internal register SrcPathEn is 0. Time=%t", $time);
    end

  wire [size-1:0] Ax;
  // om  wire [size-1:0] A1;

//! disable delay_restrictions
  // om  assign #0 A1[size-1:0] = A[size-1:0];
//! enable delay_restrictions

// synopsys translate_off
//! disable parser
  reg ForceOutDly, AssertEnable;
  integer CdcDestSetup, ForceOutDstClk, ForceOutDstClkN, DestClkCnt, DestClkCntN;
  event OutDlyClr;

  integer test;

  initial begin
    AssertEnable = 0;
    ForceOutDstClk = 0;
    ForceOutDstClkN = 0;
    ForceOutDly = 1'b0;
    CdcDestSetup = Setup; 
    // DestClkCnt = Cycles;
    // DestClkCntN = Cycles;
    DestClkCnt  = ( int'(Cycles)==Cycles ) ? Cycles : Cycles - 0.5;
    DestClkCntN = ( int'(Cycles)==Cycles ) ? Cycles : Cycles + 0.5;
    #1 AssertEnable = en_port;
  end

  `ifdef DISPLAY_CDC_OVERRIDES
  always @(CdcDestSetup) if($test$plusargs("logCdcBufTiming")) $display("info: CDCBUFCLK %m.CdcDestSetup\t = %d : %t", CdcDestSetup, $time);
  always @(DestClkCnt) if($test$plusargs("logCdcBufTiming")) $display("info: CDCBUFCLK %m.DestClkCnt\t = %d : %t", DestClkCnt, $time);
  always @(AssertEnable) if($test$plusargs("logCdcBufTiming")) $display("info: CDCBUFCLK %m.AssertEnable\t =           %d : %t", AssertEnable, $time);
  `endif

  // om  always @(A1[size-1:0]) begin // for ubts 630641, so that a glitch is not seen, A1 is not required
  always @(A[size-1:0]) begin
      if(CdcDestSetup || DestClkCnt) begin
          force Ax[size-1:0] = {size{1'bx}};
          disable ClearOutDly;
          ForceOutDly = 1'b1;
          if(DSTCLK)
          begin
          ForceOutDstClk = (DestClkCnt>32'd1)? DestClkCnt-1 : DestClkCnt ; 
          ForceOutDstClkN = DestClkCntN ;
          end
          else
          begin
          ForceOutDstClk = DestClkCntN ;
          ForceOutDstClkN = (DestClkCnt>32'd1)? DestClkCnt-1 : DestClkCnt ;
          end
          #0;
          -> OutDlyClr;
      end
  end

  always @(posedge DSTCLK) begin
      if(ForceOutDstClk) ForceOutDstClk = ForceOutDstClk - 1;
  end

  always @(negedge DSTCLK) begin
      if(ForceOutDstClkN) ForceOutDstClkN = ForceOutDstClkN - 1;
  end

  always @(OutDlyClr) begin : ClearOutDly
     #(CdcDestSetup) ForceOutDly = 1'b0;         
  end

  always @* if (~ForceOutDly && (ForceOutDstClk == 0) && (ForceOutDstClkN == 0)) release Ax;
//! enable parser
// synopsys translate_on

//! disable delay_restrictions
assign #0 Ax[size-1:0] = A[size-1:0];
//! enable delay_restrictions
assign     Z[size-1:0] = ~({size{EN}} & Ax[size-1:0]);

//! unassigned AssertEnable
`ifdef ABV
// synopsys translate_off
//! disable parser

// ovl_never causing performance problems UBTS:401781
// ovl_never #(.msg("CDCBUFENCLK Error: EN failing to block metastable X output.")) rtl_stdlib_cdcbufenclk ( .clock(DSTCLK),
//     .reset(1'b1), .enable(AssertEnable & (InputChkDis===1'b0)), .test_expr( (EN === 1'b1) & (Ax[size-1:0] !== A[size-1:0]) ), .fire());
property p_rtl_stdlib_cdcbufenclk;
  @(posedge DSTCLK) disable iff ((!AssertEnable)|(InputChkDis===1'b1)) ( (EN === 1'b1) & (Ax[size-1:0] !== A[size-1:0]) ) !== 1'b1;
endproperty

rtl_stdlib_cdcbufenclk: assert property (p_rtl_stdlib_cdcbufenclk) else `AMD_ASSERT_ERROR("%m.CDCBUFENCLK", "Error: EN failing to block metastable X output.");

// hold check
reg EN_n;
reg [1:0] ck_posedge_cnt;  
generate
    if (HoldCheck) begin: HoldCheck_cdcbufen
        always @(negedge DSTCLK)
            EN_n    <= EN;
        // ck_posedge_cnt accumulated as 00-01-10 when EN is high. hold check is executed at DSTCLK negedge when cnt=01
        always @(posedge DSTCLK)
            if (!EN)
                ck_posedge_cnt  <= 2'b0;
            else if (|ck_posedge_cnt) 
                ck_posedge_cnt  <= 2'b10;
            else
                ck_posedge_cnt  <= ck_posedge_cnt + 1'b1;

        property n_rtl_stdlib_cdcbufenclk;
          @(negedge DSTCLK) disable iff ((!AssertEnable)|(InputChkDis===1'b1)&(!ck_posedge_cnt[0])) ( (EN_n === 1'b1) & (Ax[size-1:0] !== A[size-1:0]) ) !== 1'b1;
        endproperty
        
        hold_rtl_stdlib_cdcbufenclk: assert property (n_rtl_stdlib_cdcbufenclk) else `AMD_ASSERT_ERROR("%m.CDCBUFENCLK", "Error: EN failing to block hold metastable X output.");
end
endgenerate

//! enable parser
// synopsys translate_on
`endif
// leda on 
`endif // TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG

endmodule



//============================================================================
// CDCBUFCLK - CDCBUF with paremeters & a non-optional DSTCLK port
//============================================================================
module CDCBUFCLK ( `ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
               DSTCLK,
               `endif
               A, 
               Z);

parameter Setup = 11000;
parameter Cycles = 1;
parameter size = 1;
parameter mode01 = 0;
parameter BufMetaWaiveMode = 0;
parameter SrcPath = 0;

`ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
input     DSTCLK;           // Destination Clock For Z modification
`endif
input        [size-1:0]  A;           // Signal from the source clock domain
output wire  [size-1:0]  Z;           // Signal to the destination clock domain

wire [size-1:0] Z_INV;
reg [2:0] mode01_reg;
reg [size-1:0] A_delayedP;
reg [size-1:0] A_delayedN;
reg [size-1:0] A_previous;
wire A_diff;
// wire randbit;
// wire A_after_randbit;

`ifdef ABV
    `ifndef RTLLIB_TEMP_KEEP_LEGACY_PARAM        
        initial begin
            if (mode01) `AMD_ASSERT_ERROR("%m.CDCBUFCLK","Error: the parameter 'mode01' is renamed as 'BufMetaWaiveMode' to reflect its behavior, update your instantiation accordingly.");
        end
        always@(mode01) begin
            if (mode01) `AMD_ASSERT_ERROR("%m.CDCBUFCLK","Error: the parameter 'mode01' is renamed as 'BufMetaWaiveMode' to relfect its behavior, update your instantiation accordingly.");
        end 
    `endif
`endif

CDCBUFENCLK #(.Setup(Setup), .Cycles(Cycles), .en_port(1'b0), .size(size), .HoldCheck(0), .SrcPath(SrcPath) ) CDCBUF_GEN(
                    .A(   A[size-1:0]), 
                    .Z( Z_INV[size-1:0]), 
                    `ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
                    .DSTCLK(DSTCLK),      
                    `endif
                    `ifdef ABV
                    .InputChkDis(1'b1), // since en_port is tied to 0.
                    `endif
                    .EN(   1'b1));

`ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
// synopsys translate_off 

always @ (posedge DSTCLK) 
    A_delayedP[size-1:0] <= A[size-1:0];
always @ (negedge DSTCLK) 
    A_delayedN[size-1:0] <= A[size-1:0];
assign A_diff = (A_delayedN[size-1:0] != A_delayedP[size-1:0]);    
always @ (A_diff)    // latch the previous value of A
    if (A_diff)
    A_previous[size-1:0] <= (A_delayedP[0]^A[0]) ? A_delayedP[size-1:0] : A_delayedN[size-1:0];


initial
begin
A_previous = 1'b0;
mode01_reg = mode01;
end

// synopsys translate_on 
`endif

// mode01_reg doesnt work in always statement, works in initial as used in other modules above,
// since, mode01 is getting properly assigned, dont need any new reg.


//orig // reusing synchronizer randomizer. Make sure METASTABLE_MODEL is defined!
//orig cdc_stdlib_internal_met_model randgen ( .D(((mode01_reg[1:0] != 2'd0) & (Z_INV[0] === 1'bx)) ? 
//orig `ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
//orig                                            DSTCLK ^ 
//orig `endif
//orig                                            1'b1 : 1'b0),
//orig                                         .EN(mode01_reg[1:0] != 2'd0), .Dsync(), .RandomExtraCycle(randbit));


// mode01=0 implies X model, which is most robust.  mode01=1 is random 0/1, mode01=2 is monotonic 0/1.

//orig assign Z[size-1:0] = 
//orig // synopsys translate_off 
//orig                      ((mode01_reg[1:0] != 2'd0) & (Z_INV[0] === 1'bx))
//orig                      ? (randbit ? A[size-1:0] : ~A[size-1:0]) : 
//orig // synopsys translate_on 
//orig                      ~Z_INV[size-1:0];

//  assign Z[size-1:0] = 
//  // synopsys translate_off 
//                       ((mode01_reg[1:0] == 2'd1) & (Z_INV[0] === 1'bx))
//                       ? (randbit ? A[size-1:0] : ~A[size-1:0]) : 
//                       (((mode01_reg[1:0] == 2'd2) & (Z_INV[0] === 1'bx))
//                       ? A_previous[size-1:0] :
//  // synopsys translate_on 
//                       ~Z_INV[size-1:0] );
//  

`ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
  import "DPI-C" function int unsigned cdcRandomFunc(input int unsigned  seed);

  integer transRandomVar;
  integer randSeed; 
  integer RandType;
  integer transportCycle_random;
  integer transportDelay_random;
  integer round_Cycles;

  initial begin
       if (!$value$plusargs("seed=%d", randSeed)) begin
        randSeed = 123456;
       end
       `ifdef SYNC2ORSEED_OVERRIDE
       if (check_option_switch("Sync2OrSeedOverride")) begin
        if (get_option_switch("Sync2OrSeedOverride")) begin
          randSeed = get_option_value("Sync2OrSeed");
        end 
       end
        `endif

       ///  When we finally get our random seed, we use it to randomly decide whether to
       ///  model the initial transport delay as 0, or as the maximum delay permissible.

       transRandomVar = (32'h7fffffff & cdcRandomFunc(randSeed));

       RandType = transRandomVar%100;

       round_Cycles = Cycles;

       if (RandType<25) transportCycle_random = 0; 
       else if (RandType<50) transportCycle_random = Cycles; 
       else transportCycle_random = transRandomVar%(round_Cycles);

       if (RandType<25) transportDelay_random = 0; 
       else if (RandType<50) transportDelay_random = Setup; 
       else transportDelay_random = transRandomVar%(Setup+1);
    end
// BufMetaWaiveMode should be used carefully. Z will not output X to model
// the transport delay, instead, output specific sequence.
// we have some requirements to BufMetaWaiveMode:
// 1. A pulse should be longer than CLKDST_PERIOD*Cycles, otherwise the BufMetaWaiveMode
// behavior is not guaranteed;
integer i, j;
reg [size-1:0] reg_A_BufMetaWaiveMode_1;
reg [size-1:0] reg_A_BufMetaWaiveMode_2;
reg [size-1:0] A_pre;
reg            BufMetaWaiveMode_3_active; 
reg            BufMetaWaiveMode_4_active; 
initial begin 
        reg_A_BufMetaWaiveMode_1 = A;
        force reg_A_BufMetaWaiveMode_2 = A;
end

integer half_setup;
initial half_setup = Setup/2;
always @(Setup)
    half_setup = Setup/2;

generate
    if (BufMetaWaiveMode) begin: cdcbuf_BufMetaWaiveMode
        // BufMetaWaiveMode=1, Z produce square-pulses instead of X
        always @(A) begin
            reg_A_BufMetaWaiveMode_1 = A; 
            #(half_setup)
            if (Setup && (Cycles==0)) // for Setup
                reg_A_BufMetaWaiveMode_1 = !reg_A_BufMetaWaiveMode_1; 
        end
        always @(DSTCLK)    // for Cycles
            if (Cycles)
                reg_A_BufMetaWaiveMode_1  <= !reg_A_BufMetaWaiveMode_1;  
    
    // BufMetaWaiveMode=2, Z is delayed by [0,CLKDST_PERIOD]*cycles, randomized    
    // BufMetaWaiveMode=3, Z passes transition to 0 instantly, but randomizes rising edge like BufMetaWaiveMode=2 
    //           This can be useful for async falling, synchronous rising signals like PwrOk.
    // BufMetaWaiveMode=4, Z always passes transition to 1 instantly, but randomizes falling edge like BufMetaWaiveMode=2
    //           This can be useful for async rising, synchronous falling signals an active high reset. 
    reg [8:0] rand_cdcbuf_delay;
    
    always @(A) begin
        BufMetaWaiveMode_3_active = 0; 
        BufMetaWaiveMode_4_active = 0; 
        for (i=0; i<size; i=i+1) begin
            BufMetaWaiveMode_3_active = (!A[i] & A_pre[i]);
        end
        for (j=0; j<size; j=j+1) begin
            BufMetaWaiveMode_4_active = (A[j] & !A_pre[j]);
        end
        if ( ((BufMetaWaiveMode==3'd3) & BufMetaWaiveMode_3_active) | ((BufMetaWaiveMode==3'd4) & BufMetaWaiveMode_4_active) ) // the edge should be passed on instantly
            rand_cdcbuf_delay = 0;
        else
            // rand_cdcbuf_delay={$random}%(Cycles+1); // range: [0, cycle]
            rand_cdcbuf_delay = |Cycles ? {transportCycle_random[7:0],1'b0} - 1'b1 : transportDelay_random; 
        A_pre <= A;
    end
    always @(DSTCLK)
        if (Cycles)
            rand_cdcbuf_delay <= |rand_cdcbuf_delay ? rand_cdcbuf_delay - 1'b1 : 0;
    always @(*) begin
        release reg_A_BufMetaWaiveMode_2;
        if (Cycles) begin // delay is modeled in manner of dstclk cycles
            if (rand_cdcbuf_delay==0)
                reg_A_BufMetaWaiveMode_2 <= A;
            else
                reg_A_BufMetaWaiveMode_2 <= ~A;
        end else begin // delay is modeled in manner of picosecond
                reg_A_BufMetaWaiveMode_2 <= #rand_cdcbuf_delay A;
        end
    end
end
endgenerate
`endif

assign Z[size-1:0] = 

`ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
// synopsys translate_off 
// BufMetaWaiveMode:  
                     ((BufMetaWaiveMode == 3'd1) & (^Z_INV === 1'bx))
                     ? reg_A_BufMetaWaiveMode_1:

                     ((|BufMetaWaiveMode[2:1]) & (^Z_INV === 1'bx)) // BufMetaWaiveMode=2/3/4
                     ? reg_A_BufMetaWaiveMode_2:
// synopsys translate_on 
`endif
                     ~Z_INV[size-1:0] ;

endmodule



//leda off
//! disable parser

//-----------------------------------------------------------
// File name: FPM_BUF.v
// Module   : FPM_BUF
// Purpose  : False Path Marker Buffer cell
//            Internally instantiates: bfxss4ur
// Project  : Kabini
// Revision History:
//     version 1.0: Initial Version.
//     Imported from Krishna and changed the cell name.
//-----------------------------------------------------------

module FPM_BUF (
               `ifdef TECH__CDCBUF__DSTCLKPORTEN
               // synopsys translate_off 
                 DSTCLK,
               // synopsys translate_on
               `endif
               A,
               Z);
`ifdef TECH__CDCBUF__DSTCLKPORTEN
    // synopsys translate_off 
    input      DSTCLK;       // Destination Clock For Z modification
    // synopsys translate_on
`endif

  input A;
  output Z;

  wire Z;

    wire ZN;

    //assign Z = I;
    CDCBUFEN fpm_buf_en (.A(A),
                         .Z(ZN),
                         `ifdef TECH__CDCBUF__DSTCLKPORTEN
                         // synopsys translate_off 
                         .DSTCLK(DSTCLK),      
                         // synopsys translate_on
                         `endif
                         .EN(1'b1));

    assign Z = ~ZN;


endmodule

//! enable parser
//leda on


//leda off
//-----------------------------------------------------------
// File name: FPM_NAND.v
// Module   : FPM_NAND
// Purpose  : False Path Marker NAND cell
//            Internally instantiates: nd2xss4ur
// Project  : Kabini
// Revision History:
//     version 1.0 Initial Version.
//     Imported from krishna and changed the cell name.
//-----------------------------------------------------------

module FPM_NAND (A, 
                 EN, 
               `ifdef TECH__CDCBUF__DSTCLKPORTEN
               // synopsys translate_off 
                 DSTCLK,
               // synopsys translate_on
               `endif

                 ZN);

`ifdef TECH__CDCBUF__DSTCLKPORTEN
    // synopsys translate_off 
    input      DSTCLK;       // Destination Clock For Z modification
    // synopsys translate_on
`endif

  input A;
  input EN;
  output ZN;

  wire ZN;


    // assign ZN = ~(A1 & A2);
    CDCBUFEN fpm_nand (.A(A), 
                       .EN(EN), 
                    `ifdef TECH__CDCBUF__DSTCLKPORTEN
                    // synopsys translate_off 
                    .DSTCLK(DSTCLK),      
                    // synopsys translate_on
                    `endif

                       .Z(ZN));

endmodule

//leda on
//vrq translate_on

//! enable one_module_per_file

/////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////
///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
///  
///  Module Name    : cdc_stdlib
///  
///  Revision       : $Revision$
///  Release Status : $State$
///  Author         : $Author$
///  
///   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
///  
///    ABSTRACT
///
///    This library file provides a centralized repository for the corporate distribution
///    of simulation models related to both clock-domain crossing (CDC) and multi-cycle
///    timing flows. By centralizing the modelling for these types of cells, special
///    modelling techniques can be consistently deployed across all teams within AMD. In
///    addition, by providing more sophisticated models outside of the technology library
///    flow, we can reduce the burden on the technology libary development, and coordinate
///    its deployment with this library.
///
///    It should be noted that, while the cells in this module are referred to as
///    'user-level' models, it is not intended that any of these cells be instantiated
///    directly by a designer. Rather, the distinction of 'user-level' merely refers to
///    the expectation that modules outside of this library will instantiate them. It is
///    epxected that designers will deploy the cells contained herein through the higher
///    level, technology independent 'wrapper' flow that is associated with this library.
///
///    Through an associated higher-level 'wrapper' flow, each of the 'user-level' cells
///    contained within this library can ultimately be mapped to an equivalent cell [or set
///    of cells] within the target library. For simulation purposes, models from this
///    library will be deployed, whereas synthesis/implementation will deploy cells from
///    the target technology library. For more information, please refer to the 'techind'
///    library of technology independent wrappers.
///
///   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
///
///    SEARCH TAGS :
///
///    The following is a list of the cell models that are made available through this
///    library. As a general rule, the cells in this list are the only modules that should
///    be called out from outside this library. All other modules contained within this
///    library that are not on the following list are intended solely for internal use
///    within this library.
///
///    <ModuleName>            Description
///      --------               ---------
///
///    <CDCGENERICSYNC>     -  generic synchronizer model
///
///   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
///  
///    Copyright (C) 2011
///      by Advanced Micro Devices (AMD) Corporation
///  
///    All rights reserved. This source file is considered proprietary and confidential
///    information of AMD Corp. As such, this source file may not be distributed by any
///    means, in whole, or in part, without the express, written approval of AMD Corp.
///  
///   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
///  
///  Revision History :
///  
///  $Log$
///  
///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
/////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////



/////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////
///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
///
///  This section of our library is dedicated to synchronizers used for clock domain
///  crossings. Since a variety of simulation behaviours/techniques are incoporated into
///  the AMD synchronizer methodology, we have broken down the functionality in order to
///  better manage our code, and its behaviour during during various phases of simulation.
///  Each of these bits of functionality are represented by an explicit sub-module that is
///  solely intended for internal use.
///
///  Ultimately, there is only one 'user' cell within this section of our library:
///
///      'CDCGENERICSYNC'
///
///  However, this 'user-level' model leverages a number of internal modules for the
///  construction of its overall functionality. In particular, these internal modules are:
///
///      'cdc_stdlib_internal_sync_transport'
///      'cdc_stdlib_internal_multibit_sync_filter'
///      'cdc_stdlib_internal_meta_rand'
///      'cdc_stdlib_internal_sync_model'
///

///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
///
///     module:   'first_ck_posedge'
///               detect the first 0->1 on CK
///
///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
`ifdef ABV
`ifndef HWEMUL
// KENDALL
primitive Catch_0_to_1_trans (firstCK, CK );
output firstCK; reg firstCK;
initial firstCK = 1'b0;
input CK;
table
// obtain output on rising edge of clock
// CK    firstCK   firstCK+
   // ignore any other CK transition
    *    : ?       : - ;
   (01)  : ?       : 1 ;
endtable
endprimitive

// primitive first_ck_posedge(cnt, CK); 
// output cnt;
// input CK;
// reg cnt;
// initial cnt = 1'b0;
// 
// table
// // CK CNT CNT+
// (01)    :   ?   :   1;
// (x1)    :   ?   :   -;
// (1?)    :   ?   :   -;
// ?       :   1   :   -;
// endtable
// endprimitive

///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
///
///     module:   'IPWC_UDP'
///               IPWC in UDP style
///
///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
primitive IPWC_UDP (cnt, CK, D); 
output cnt;
input CK, D;
reg cnt;
initial cnt = 1'b0;

table
// CK D CNT CNT+
(01)    ?   :   ?   :   0;  // reset at every CK rising edge
(x1)    ?   :   ?   :   -;  // - at X->1 on CK
?       *   :   0   :   1;  // cnt+1 at D toggle
?       *   :   1   :   x;  // cnt+1 at D toggle
(?0)    ?   :   ?   :   -;  // - at CK falling edge
(?x)    ?   :   ?   :   -;  // - at ->X on CK
// x       ?   :   ?   :   -;  // - at X on CK. needed for CK initial to X scenario
endtable
endprimitive
///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
///
///     module:   'IPWC_UDP_display'
///               msg module for IPWC_UDP
///
///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
module IPWC_UDP_display(
        InputChkDis,
        cnt
);
input InputChkDis, cnt;
always @(cnt) begin
    if ((!InputChkDis) & (cnt===1'bx)) begin
        // $display("ERROR : D pulse lost because of Min pulse width at Time=%t", $time);
        $display("ERROR : %m : D pulse lost because of Min pulse width at Time=%t", $time);
        `ifndef SYNC_WINDOW_DISABLE_FATAL_ERRORS
        $finish;
        `endif
    end
end
endmodule
///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
///
///     module:   'IPWC_data_check'
///               assertion check module used in IPWC
///
///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
module IPWC_data_check(
        CK,
        D,
        input_check,
        InputChkDis
        );

input   CK;
input   D;
input   input_check;
input   InputChkDis;

`ifndef RTLLIB_HACK_DISABLE_IPWC
        ////////////////////////////////////////////////////////////////////////////////
        // Pulse Width Check 
        ////////////////////////////////////////////////////////////////////////////////

        reg DataCaptured;
        time lastCK, LastClockPeriod;
        time lastD, DestClockPeriod, DataPulseWidth; 

        initial begin
          DataPulseWidth = 0; DestClockPeriod = 0; LastClockPeriod = 0; lastCK = 0; lastD = 0; DataCaptured = 0; 
        end

        always @(negedge InputChkDis) begin // reset
          DataPulseWidth = 0; DestClockPeriod = 0; LastClockPeriod = 0; lastCK = 0; lastD = 0; DataCaptured = 0;
        end

        ////////////////////////////////////////////////////////////////////////////////
        // ClockCheck - measures last clock period,  
        ////////////////////////////////////////////////////////////////////////////////
        always @(posedge CK) 
        begin : ClockCheck
             begin
                LastClockPeriod = DestClockPeriod;
                DestClockPeriod = $time - lastCK;
                DataPulseWidth = 0; // last pulse checked
                lastCK = $time;
            end
        end

        ////////////////////////////////////////////////////////////////////////////////
        // DataCheck - measures data pulses, catches pulses smaller than a single clock
        ////////////////////////////////////////////////////////////////////////////////
        always @(D) begin : DataCheck
            if(~InputChkDis & input_check) begin
                if(|lastCK & (($time - lastD) > 5)) begin //smooth out any data glitches and ignore check before 1st clock edge
                    if(DataPulseWidth) begin // no clock edge occurred in this pulse, still need to check last pulse
                        if(DataCaptured) begin // last pulse included a posedge clock, need to check it.
                            if(DataPulseWidth < (DestClockPeriod )) 
                            begin
                            `ifdef RTLLIB_HACK_ASSERT
                                assert_check_ipwc0: assert (DataPulseWidth >= DestClockPeriod) else $display("ERROR : %m : %0t D input pulse < %0t CLK period  Time=%t", DataPulseWidth, DestClockPeriod, $time);
                            `else
                                $display("ERROR : %m : %0t D input pulse < %0t CLK period  Time=%t", 
                                    DataPulseWidth, DestClockPeriod, $time);
                                `ifndef SYNC_WINDOW_DISABLE_FATAL_ERRORS
                                $finish;
                                `endif
                            `endif
        // assert_check_ipwc: assert (DataPulseWidth < DestClockPeriod) else $display("ERROR : %m : %0t D input pulse < %0t CLK period  Time=%t");
                            end
                        end
                     `ifdef RTLLIB_HACK_ASSERT
                        assert_check_ipwc1: assert (!DataPulseWidth) else $display("ERROR : %m : %0t D pulse lost because of Min pulse width at Time=%t", DataPulseWidth,$time);
                     `else
                        $display("ERROR : %m : %0t D pulse lost because of Min pulse width at Time=%t", DataPulseWidth,$time);
                        `ifndef SYNC_WINDOW_DISABLE_FATAL_ERRORS
                        $finish;
                        `endif
                     `endif
                    end
                    DataCaptured = (lastD < lastCK) & (D !== 1'bx);
                    DataPulseWidth = $time - lastD;
                end
                lastD = $time;
            end
        end
`endif

endmodule        
`endif  // `HWEMUL
`endif  // `ABV

///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
///
///     module:   'cdc_stdlib_internal_sync_transport'
///
///  WARNING: This module is intended solely as a 'cdc_stdlib' internal module. Please do
///  not use this module unless you work closely with it's author.
///
///  Since some signals must travel on-trivial distances from one block to another before
///  being sampled by the destination domain, there can also be a non-trivial propagation
///  time associated with the signal flight. From the perspective of design performance,
///  failure to model this timing effect can lead to a less-than-optimal design. Worse,
///  not modelling this timing effect can result in unexpected behaviour where event timing
///  expectations are formed without veryfying against the appropriate degree of pessimism.
///
///  The 'cdc_stdlib_internal_sync_transport' module provides a means to model non-trivial
///  propagation time, of the data-signal to be synchronized, from the source location to
///  the synchronizer's destination-clock sampler. On every transition of the data input of
///  this module, the output signal will undergo an identical transition after a [mildly]
///  randomized delay.
///
///  By randomizing the delay between zero and an instance-specific maximum, repeated
///  simulations are expected to cover a wide variety of propagation-time scenarios. It is
///  expected that this behaviour will eventually expose out performance issues and
///  functional issues that are propagation-time dependent.
///
///   .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .

//! disable parser
module cdc_stdlib_internal_sync_transport (

    CK,
    D,
    `ifdef ABV
    InputChkDis,   ///  disables input pulse-width checking 
    `endif
    transportCycle_assert, 
    TC_holdcheck_assert,
    Dtransport

  );

  //VCS coverage exclude_module

  /// .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
  ///
  ///  While we do support a data-bus 'size' greater than one bit, it is generally expected
  ///  that the size for signals that use this mechanism will be single-bit. If a multi-bit
  ///  data-signal deploys this technique, a top-level skew requirement will still be in
  ///  force [that will not likely close timing in all operational corners].
  ///
  ///  Given the behaviour of the sampler randomization [discussed later], a transport
  ///  delay of up to one source-clock cycle is already intrinsically modelled. Where the
  ///  propagation time to our synchronizer may exceed a single source-clock cycle, it is
  ///  incumbent upon the designer to specify the maximum propagation time through the
  ///  'TransportDelay' and 'TransportCycle' parameter.

  parameter  size           = 1;
  parameter  InputCheck     = 1;
  parameter  TransportDelay = 0;
  parameter  TransportCycle = 1;
  parameter  MetaWindowMode = 0;
  parameter  HoldCheck      = 0;

  `ifdef META_WINDOW_USE
     localparam  MetaWindowUse = 1;
  `else
     localparam  MetaWindowUse = MetaWindowMode;
  `endif

  /// .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .

  input                     CK;
  input       [(size-1):0]  D;
  `ifdef ABV
  input       InputChkDis;   ///  disables input pulse-width checking 
  `endif
  output                    transportCycle_assert; 
  output reg  [(size-1):0]  TC_holdcheck_assert;
  output reg  [(size-1):0]  Dtransport;

  // synopsys translate_off
  // leda off
`ifndef METASTABLE_MODEL // {
always @* Dtransport = D;
`else
// vrq translate_off
import "DPI-C" function int unsigned cdcRandomFunc(input int unsigned  seed);
// vrq translate_on

  /// .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
  ///
  ///  This task implements a modified 32-bit LFSR as its pseudo-random decision, where the low bit
  ///  provides a binary response.  Although all intances utilize the same polynomial for
  ///  their LFSR, it is expected that a 'reasonable' distribution of initial seeds will
  ///  result in relatively uncorrelated random sequences between instances.  As desired, 
  ///  the pattern only repeats after 2^32-1 cycles; a setting of 0 is illegal as per typical 
  ///  LFSR circuits.  Unlike typical LFSR patterns there is an 11-bit rotation with the polynomial
  ///  each cycle - as opposed to a typical 1-bit shift.  This allows for a polynomial with
  ///  bits spread out more than typical and results a better random distribution.

    initial Dtransport = D; // initialize to D, for when FLUSH is asserted near t=0, when clk=x, ubts 622993
    
reg input_check;
initial
  input_check = 1'b0;                      // Initialize to 0 at time 0
always @(negedge CK)
    input_check <= InputCheck ? 1'b1 : 1'b0;  // parameters are 32bit, making it single bit for use 

  task  evalRandomDelay;

    inout   [31:0]  randomVar;    ///  the variable to manage
    output          incrDelay;    ///  the decision to be made

    begin
      incrDelay  =   randomVar[0];
      ///  modified lfsr with 11 LSBs uncorellated from sample to sample
      randomVar  = { randomVar[10:0], randomVar[31:11] } ^ ( randomVar[0] ? 32'h80080100 : 32'h00 );
    end
  endtask

  /// .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .

  reg        randomDelay;
  reg        randomStart;
  integer    transportDelay_random;
  integer    transportCycle_random;
  integer    transRandomVar;

  /// .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
  ///
  ///  In order to facilitate independence between different instances, we provide an
  ///  instance-specific register to store the random variable. Of course, what comes along
  ///  with the storage is the need to initialize the value in a meaningful way. To this
  ///  end, we provide for different, progressively simpler methods of initializing our
  ///  random variables, based on a variety of verification criteria.

     integer randSeed;
     integer RandType;
     integer TransportCycle_int;
     reg     halfcycle;
     // vrq translate_off
     initial begin
       if (!$value$plusargs("seed=%d", randSeed)) begin
        randSeed = 123456;
       end
       `ifdef SYNC2ORSEED_OVERRIDE
       if (check_option_switch("Sync2OrSeedOverride")) begin
        if (get_option_switch("Sync2OrSeedOverride")) begin
          randSeed = get_option_value("Sync2OrSeed");
        end 
       end
        `endif

       ///  When we finally get our random seed, we use it to randomly decide whether to
       ///  model the initial transport delay as 0, or as the maximum delay permissible.

       transRandomVar = (32'h7fffffff & cdcRandomFunc(randSeed));

       RandType = transRandomVar%100;

       // to detect TC includes a half cycle
       TransportCycle_int = int'(TransportCycle);
       halfcycle = TransportCycle_int !== TransportCycle;

       if (TransportCycle==1) begin // to ensure 0/1 is 50%/50% when TC=1
            if (RandType<50) transportCycle_random = 0; 
            else transportCycle_random = 1;
       end else begin
            if (RandType<25) transportCycle_random = 0; // 25% random result is 0 
            else if (RandType<50) transportCycle_random = TransportCycle_int;  // 25% random result is TC
            else transportCycle_random = transRandomVar%(TransportCycle_int+1);
            // 50% random result is wander from [0, TC]
       end

       if (TransportDelay==1) begin
            if (RandType<50) transportDelay_random = 0; 
            else transportDelay_random = 1;
       end else begin
            if (RandType<25) transportDelay_random = 0; 
            else if (RandType<50) transportDelay_random = TransportDelay; 
            else transportDelay_random = transRandomVar%(TransportDelay+1);
       end

     end
     // vrq translate_on

  /// .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .

  always @* begin  ///  in case TransportDelay is forced late, or changes with voltage
       if (TransportDelay==1) begin
            if (RandType<50) transportDelay_random = 0; 
            else transportDelay_random = 1;
       end else begin
            if (RandType<25) transportDelay_random = 0; 
            else if (RandType<50) transportDelay_random = TransportDelay; 
            else transportDelay_random = transRandomVar%(TransportDelay+1);
       end
    end

  always @* begin  ///  in case TransportCycle is forced late, or changes with voltage
       TransportCycle_int = int'(TransportCycle);
       halfcycle = TransportCycle_int !== TransportCycle;
       if (TransportCycle==1) begin
            if (RandType<50) transportCycle_random = 0; 
            else transportCycle_random = 1;
       end else begin
            if (RandType<25) transportCycle_random = 0; 
            else if (RandType<50) transportCycle_random = TransportCycle_int; 
            else transportCycle_random = transRandomVar%(TransportCycle_int+1);
       end
    end

  /// .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
  ///
  ///  When we know the amount of transport delay we want to model, we apply it every time
  ///  our input data transtions. On the input transition, we schedule the delayed event
  ///  using a pure transport mechansim so that we do not lose any transtions.

// fix for ubts:523633, where TRANSPORT_D is not getting initialized until CK is seen.
// for this case TRANSPORT_D should be assigned D directly, so that proper value is propagated. 
reg firstCK;  // will be valid from t=0 till first posedge of CK. 

`ifdef RTLLIB_HACK_UDP
Catch_0_to_1_trans ucatch_first_01 (firstCK, CK );
`else
reg CKd;
always @* CKd  <= `ifdef CDC_RTLLIB_DELTA_DEBUG #1 `else `endif CK;
initial firstCK = 1'b1;
always @ (posedge CK)
// ubts 646242. CK x->1 is also regarded as posedge by simulator.
// suppose CK doesnt toggle after leaving X,
// (firstCK is set to 0), and D leave X later. Dtransport will always be
// X before next clock posedge. this is not what we expected and may cause
// extra X cycles at Q before output correct value. 
// with !CKd, firstCK is able to find the real 0->1.
    if(CKd===1'b0) firstCK <= 1'b0;
`endif    

// in old design, we use [TransportCycle:0] to define TransportCycle_reg's width. note that to support halfcycle TC we have to set TransportCycle_reg as a fix-width(8bit) reg, because TC cannot be used to define reg width as it may be not an integer.
// so now we have a limitation that TC should not exceed 63. it looks enough
// for all current projects.
reg [7:0] TransportCycle_reg;
reg       halfcycle_cut;
initial begin
        TransportCycle_reg = 0;
        halfcycle_cut = 0; 
end

genvar holdcheck_i;
generate
  if (TransportCycle) begin: TC_block
    // TransportCycle is higher priority that transport_delay
    // reg [TransportCycle:0] TransportCycle_reg;

    always @(D) begin
      TransportCycle_reg <= 'b0;
      halfcycle_cut <= CK; // D change when CK is high, in this case TC=N.5 has same behavior with TC=N i.e. the extra 0.5 cycle delay can be cut
      if (!(|transportCycle_random)) Dtransport = D;
      ///  Use a blocking assignment to update Dtransport so that it reflects 'D' in the same
      ///  simulation delta. From a 'D' transition, there is not need to further delay the update.
      if (firstCK)  Dtransport = D;
      // if TC=0.5 and TC_random=1 and halfcycle_cut=1, D should be reflected on Dtransport
      // before next CK rising edge
      if (CK & halfcycle & (TransportCycle_int==1) ) Dtransport = D;
    end

    always @(posedge CK) begin
      if (transportCycle_random) begin
        // TransportCycle_reg[transportCycle_random] = 1'b1;
        // TransportCycle_reg = {1'b1,TransportCycle_reg[TransportCycle_int:1]};
        // ///  If TransportCycle=1, then first CK edge changes data to sync model
        // ///  Since the delay iposed on Dtransport is pushed to the following clock cycle, we want
        // ///  to be sure that we do not introduce a race-condition between the CK and Dtransport. For
        // ///  this reason, we use a non-blocking assigment. This ensures that Dtransport reflects 'D'
        // ///  in the simulation delta following the CK event.
        // if (TransportCycle_reg[0]) Dtransport <= D;
        TransportCycle_reg = TransportCycle_reg + 1'b1;
        if (halfcycle_cut & halfcycle & (transportCycle_random==TransportCycle_int) ) begin
            if (TransportCycle_reg == (transportCycle_random-1)) Dtransport <= D;
        end else begin
            if (TransportCycle_reg == transportCycle_random) Dtransport <= D;
        end
      end
    end

    // when TC is non-zero, the D input of syncp(metastable-model) always
    // change after CK rising edge, thus the metastable is modeled in hold
    // manner at every D toggle. this is not what we expect.
    // here we pass the 'original' D/CK phase relationship to metastable-model
    // when TC is used.
    initial TC_holdcheck_assert = 0;
    if (HoldCheck) begin: HoldCheck_TC
        for (holdcheck_i=0; holdcheck_i<size; holdcheck_i=holdcheck_i+1) begin
            // a potential issue is that if TC is a great value that metastable-model
            // haven't get Dtransport before next D toggle arrive, TC_holdcheck will
            // be refreshed, as unexpected
            always @(D[holdcheck_i]) begin 
                // if D change when CK is high, we suppose the metastable is
                // caused by hold but not seup violation. if TC=N.5
                // holdcheck is asserted when D toggle@CK=0
                TC_holdcheck_assert[holdcheck_i] <= halfcycle ^ CK & transportCycle_assert;
            end
        end
    end
  end else begin: TD_block
    always @( D ) begin

      ///  Here, we only use a non-blocking assignment if the actual delay is zero. We don't
      ///  want to create any wierd race-conditions, and it is a little friendlier on the
      ///  event wheel performance.

      if (firstCK)  Dtransport = D;
      else if ( transportDelay_random ) Dtransport <= #(transportDelay_random) D;
      else                         Dtransport  =                          D;

      ///  As a special tweak, after we have selected our initial transport delay value, we
      ///  have the option of allowing it to randomly wander between 0 and the maximum delay
      ///  specified. While this will spend significantly less time at the extremities, it
      ///  will add coverage across the continuum of timing delays from 0 to the maximum.

      `ifdef  SYNC_TRANSPORT_DELAY_WANDER
  
         evalRandomDelay ( transRandomVar, randomDelay );
         if ( randomDelay ) begin
           if ( transportDelay_random != (TransportDelay + TransportCycle) ) begin
             transportDelay_random = transportDelay_random + 1;
           end
         end else begin
           if ( transportDelay_random != 32'h0 ) begin
             transportDelay_random = transportDelay_random - 1;
           end
         end

      `endif
    end
  end
endgenerate

assign transportCycle_assert = |transportCycle_random;

// if transportCycle_assert, Dtransport toggle always happen when CK is high thus all
// metastable are modeled in hold style. this is not expected. in this
// case we use D but not Dtransport to judge if the metastable should
// be modeled in setup or hold style.

// initial TC_holdcheck_assert = 0;
// reg TC_holdcheck;
// initial TC_holdcheck = 0;

// genvar holdcheck_i;
// generate 
//     if (TransportCycle) begin: holdcheck
//         if (transportCycle_assert)
//             for (holdcheck_i=0; holdcheck_i<size; holdcheck_i=holdcheck_i+1) begin
//                 always @(D[holdcheck_i]) begin 
//                     TC_holdcheck[holdcheck_i] <= CK;
//                 end
//                 always @(posedge CK) begin
//                     if (TransportCycle_reg[1]) TC_holdcheck_assert[holdcheck_i] <= TC_holdcheck[holdcheck_i];
//                 end
//             end
//     end
// endgenerate

generate if (InputCheck) begin: ABV_IPWC
wire [size-1:0] cnt; // IPWC fail flag
wire cnt_ck;
`ifdef ABV
    // `ifdef RTLLIB_IPWC_TIME_STYLE
    `ifndef RTLLIB_IPWC_UDP_STYLE
        IPWC_data_check
        ABV_IPWC_DATA_CHECK[size-1:0](
                .CK             (CK         ),
                .D              (D          ),
                .input_check    (input_check),
                .InputChkDis    (InputChkDis)
                );
    `else // RTLLIB_IPWC_TIME_STYLE
        IPWC_UDP IPWC_UDP[size-1:0](cnt, CK, D);
        first_ck_posedge first_ck_posedge(cnt_ck, CK);
        IPWC_UDP_display IPWC_UDP_display[size-1:0](.cnt (cnt), .InputChkDis(!cnt_ck|InputChkDis));
    `endif // RTLLIB_IPWC_TIME_STYLE
`endif // ABV        
end
endgenerate

`endif // } METASTABLE_MODEL
// leda on
// synopsys translate_on

endmodule

//! enable parser

///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
///
///     module:   'cdc_stdlib_internal_multibit_sync_filter'
///
///  WARNING: This module is intended solely as a 'cdc_stdlib' internal module. Please do
///  not use this module unless you work closely with it's author.
///
///  In order to facilitate an extremely aggressive randomiztion of the synchronization
///  sampling, we need to take special care to not permit over-aggressive behaviour to
///  result in a reordering of events, as can occur when a multi-bit bus is employed.
///
///  In common cases of FIFOs, higher synchronization performance is achieved by signalling
///  a sequence of ordered events pointer increents] such that their synchronization into
///  the destination clock recovers the basic sequence. Typically, this type of signalling
///  is performed as a special unit step-code, but not exclusively. However, in all cases,
///  each event is distinguishable from previous and following events hinged on the
///  assumption that the event sequence order will be respected. As a consequence, where
///  a source domain is capable of launching a new event on every source clock cycle, the
///  skew across the multi-bit value must be less that the launch frequency [ie. one source
///  clock period in this example].
///
///  Since the aggressive randomization algorithm is, if left unchecked, capable of
///  violating the sequence order, we must apply a special filter to the randomization
///  activity. In particular, when that value on a data-bus to be synchronized changes,
///  we shall only permit randomization of those bits that have changed. All preceding bit
///  changes that are stable at the time of the overall bus-value change will be deemed to
///  have settled such that their sample is guaranteed. In addition, once the data-bus has
///  been sampled by the synchronizer, all bits will immediately be marked as valid so that
///  we do not inadvertantly randomize without further change.
///
///  However, in some special cases, the aggressive randomization is still too aggressive.
///  In order to account for these cases, a provision is made availalbe to limit the
///  election to randomize to those bits whose value change within a small window ahead of
///  the sampling edge of the destination clock. In practice, this window needs to be
///  smaller than the clock period of the fastest clock lest the system still fall prey to
///  inadvertant reordering problems.
///
///   .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .

//! disable parser
module cdc_stdlib_internal_multibit_sync_filter (

    CK,
    sourceData,
    SB,
    RB,
    FLUSH,
    `ifdef ABV
    InputChkDis,
    `endif
    enableRand,
    enableRand_IgnorePulseNotSampled,
    transportCycle_assert 

  );

  //VCS coverage exclude_module
  /// .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .

  parameter  size           = 1;
  parameter  halfStage      = 0;
  parameter  MetaWindowMode = 0;
  parameter  IgnorePulseNotSampled = 0; 

  /// .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
  ///
  ///  here we provide two means for enabling a windowed mode of operation. The first is a
  ///  global `define that will cause all synchronizer models to use this mode, and the
  ///  second is an instance specific mode-select [off by default].

  `ifdef META_WINDOW_USE
     localparam  MetaWindowUse = 1;
  `else
     localparam  MetaWindowUse = MetaWindowMode;
  `endif

  /// .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .

  input                     CK;
  input       [(size-1):0]  sourceData;
  input                     SB,   RB,    FLUSH;    
  output reg  [(size-1):0]  enableRand;
  output wire [(size-1):0]  enableRand_IgnorePulseNotSampled;
  input                     transportCycle_assert;
  `ifdef ABV
  input InputChkDis;
  `endif

  // synopsys translate_off
  // leda off

  initial                   enableRand = {size{1'b0}};

  reg         [(size-1):0]  lastD;
  integer                   ilp;

  reg CKd, CKdd, CKddd;
  always @* CKd  <= `ifdef CDC_RTLLIB_DELTA_DEBUG #1 `else `endif CK;
  always @* CKdd <= `ifdef CDC_RTLLIB_DELTA_DEBUG #1 `else `endif CKd;
  always @* CKddd <= `ifdef CDC_RTLLIB_DELTA_DEBUG #1 `else `endif CKdd;

reg firstCK;  // will be valid from t=0 till first posedge of CK. 
initial firstCK <= 1'b1;
always @(posedge CK)
    if(CK & firstCK) firstCK <= 1'b0;

  generate

    ///  If we are using the metastability mode, here we merely override the 'enableRand'
    ///  signal. the synchronizer model itself will make the assessment as to whether the
    ///  transition fell within the window.

    if ( MetaWindowUse ) begin: enableRand_MetaWindowUse
      always @*  enableRand <= {size{1'b1}};

    ///  Otherwise, we compare our current data against our last data. Any bit that is
    ///  found to have changed as compared to the last time we executed our check will be
    ///  elected for randomization. Note, as indicate above, we need to make this
    ///  determiniation whenever the data-bus [as a whole] changes, or on the sampling edge
    ///  of the destination clock.
    ///
    ///  Since our model supports both posedge and negedge sampling, we code up two
    ///  variations of this difference check. The choice of which to use is based on a
    ///  parameter that tells us the synchronization type [posedge or negedge].

    end else begin: enableRand_halfStage
      if ( halfStage ) begin

         `ifndef REDUCE_META_PESSIMISM
          always @( sourceData or negedge CK or RB or SB or FLUSH) begin
         `else
         // om for SATA UBTS 514271 
          always @( sourceData or negedge CK or posedge CK or RB or SB or FLUSH) begin
         `endif

          lastD <= sourceData;
          for ( ilp = 0; ilp < size; ilp=ilp+1 ) begin
            if( (~RB) | (~SB) | FLUSH )  // disable randomization if flop is set/reset/flushed
            enableRand[ilp] <= {size{1'b0}};
            else
            enableRand[ilp] <= ( lastD[ilp] !== sourceData[ilp] );
          end
        end

      end else begin

         `ifndef REDUCE_META_PESSIMISM
         always @( sourceData or posedge CK or RB or SB or FLUSH or posedge CKd ) begin
         `else
         // om for SATA UBTS 514271 
         always @( sourceData or posedge CK or negedge CK or RB or SB or FLUSH or posedge CKd) begin
         `endif

          lastD <= sourceData;
          for ( ilp = 0; ilp < size; ilp=ilp+1 ) begin
            if( (~RB) | (~SB) | FLUSH ) // disable randomization if flop is set/reset/flushed
            enableRand[ilp] <= {size{1'b0}};
            // clear enableRand after CKd. note if transportCycle_assert,
            // which means sourceData and CKd arrive at the same time,
            // enableRand should not be cleared to 0.
            else if (!transportCycle_assert & !CKdd & CKd)
            enableRand[ilp] <= {size{1'b0}};
            else
            enableRand[ilp] <= ( lastD[ilp] !== sourceData[ilp] );
          end
        end

      end
    end
  endgenerate

  // IgnorePulseNotSampled. Short pulse include no clkdst posedge lead to
  // enableRand=0.

  // how many D edges between 2 clkdst posedges
  reg [(size-1):0] sourceData_delta; 
  reg [(size*8-1):0] short_pulse_edge_acc; 

  initial begin 
          short_pulse_edge_acc = 0;
          sourceData_delta = 0;
  end

  genvar i;
  generate
  for (i=0; i<size; i=i+1) begin: D_EDGE_ACC

    always @(sourceData[i])
          sourceData_delta[i] <= sourceData[i];

    always @(sourceData[i] or posedge CKdd)
      if (!CKddd&CKdd) // reset after enableRand sample finished
          short_pulse_edge_acc[i*8+7:i*8] <= 8'b0;
      else if ( (sourceData[i]===1'bx) | (sourceData_delta[i]===1'bx) )
          short_pulse_edge_acc[i*8+7:i*8] <= short_pulse_edge_acc[i*8+7:i*8];
      else
          short_pulse_edge_acc[i*8+7:i*8] <= short_pulse_edge_acc[i*8+7:i*8] + 1'b1;

  end
  endgenerate

wire InputChkDis_wire; 
`ifdef ABV
    assign InputChkDis_wire = (InputChkDis===1'b1) ? 1'b1 : 1'b0;
`else
    assign InputChkDis_wire = 1'b0;
`endif

wire [size-1:0] IPNS_FLAG;

genvar j;
generate 
    if (IgnorePulseNotSampled) begin: enableRand_ignorepulsenotsampled
        for (j=0; j<size; j=j+1) begin
            assign IPNS_FLAG[j] = (InputChkDis_wire & (!short_pulse_edge_acc[j*8]) & (|short_pulse_edge_acc[j*8+7:j*8]) );
            assign enableRand_IgnorePulseNotSampled[j] = (InputChkDis_wire & (!short_pulse_edge_acc[j*8]) & (|short_pulse_edge_acc[j*8+7:j*8]) ) ? 0 : enableRand[j];
        end
    end
    else begin: enableRand_legacy
        assign IPNS_FLAG = 0;
        assign enableRand_IgnorePulseNotSampled = enableRand;
    end
endgenerate

  // debug
  wire [(size-1):0] debug_diff_enableRand = enableRand^enableRand_IgnorePulseNotSampled;
                                             
// leda on
// synopsys translate_on

endmodule

///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
///
///     module:   'cdc_stdlib_internal_multibit_sync_filter_nodelta'
///
///  WARNING: This module is intended solely as a 'cdc_stdlib' internal module. Please do
///  not use this module unless you work closely with it's author.
///
///  the no_delta version of module cdc_stdlib_internal_multibit_sync_filter.
///  randomextracycle generation is moved here from cdc_stdlib_internal_sync_model so we can implement 
///  all comb logic in one always block (use block assignment to align the
///  execute sequence) and no longer need delta delay signals.
///
///   .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .

//! disable parser
module cdc_stdlib_internal_multibit_sync_filter_nodelta (

    CK,
    sourceData,
    SB,
    RB,
    FLUSH,
    `ifdef ABV
    InputChkDis,
    `endif
    randomExtraCycle,
    random_var,
    race_d_late
  );

  //VCS coverage exclude_module
  /// .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .

  parameter  size           = 1;
  parameter  MetaWindowMode = 0;
  parameter  MetaWindowSize = 0;
  parameter  reduce_meta_pessimism = 0;

  /// .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
  ///
  ///  here we provide two means for enabling a windowed mode of operation. The first is a
  ///  global `define that will cause all synchronizer models to use this mode, and the
  ///  second is an instance specific mode-select [off by default].

  `ifdef META_WINDOW_USE
     localparam  MetaWindowUse = 1;
  `else
     localparam  MetaWindowUse = MetaWindowMode;
  `endif

  /// .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .

  input                     CK;
  input       [(size-1):0]  sourceData;
  input                     SB,   RB,    FLUSH;    
  output wire [(size-1):0]  randomExtraCycle;
  output reg  [(size-1):0]  race_d_late;
  output reg  [(size-1):0]  random_var;
  `ifdef ABV
  input InputChkDis;
  `endif

  reg  [(size-1):0]  enableRand;
  reg  [(size-1):0]  randomExtraCycle_reg;
  // synopsys translate_off
  // leda off

  /// .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
  ///
  ///  The main body of this 'cdc_stdlib_internal_meta_rand' module is comprised of three
  ///  main elements, each of which will be presented below:
  ///
  ///     o  helper task to abstract the random decision and advancement of the random variable
  ///     o  initialization code for the random variable
  ///     o  edge-appropriate random decision invocation
  ///
  ///
  /// .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
  ///
  ///  Since there is more than one location within our main code that requires a random
  ///  decision, we have encapsulated the decision management, and the random variable
  ///  advancement within this task. This abstraction should provide consistency in the
  ///  behaviour, and mitigate bug infestation.

  task  evalRandomDelay;

    inout   [31:0]  randomVar;    ///  the variable to manage
    output          extraDelay;   ///  the decision to be made

    begin
      extraDelay =   randomVar[0];
      randomVar  = { randomVar[30:0], 1'b0 } ^ ( randomVar[31] ? 32'hc5 : 32'h00 );
      ///  lfsr polynomial :  0000 0000 0000 0000 0000 0000 1100 0101
    end
  endtask

  /// .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
  ///
  ///  Here is the variable that is, in effect, the sole reason for the existence of this
  ///  module as a separate entity. This variable holds the current lfsr state and will be
  ///  advanced once every time a random decision is pulled from it.

  integer   metaRandomVar;


  /// .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
  ///
  ///  The next chunk of code is all about setting our initial value for the
  ///  'metaRandomVar'. As can be readily seen, means for setting the initial value of our
  ///  random variable has several different modes based on a number of testbench
  ///  verfication criteria.


       integer randSeed;
       // initial randomExtraCycle_reg = {size{1'b0}}; 
       // initial metaRandomVar = 32'h0;

       // vrq translate_off
       `ifdef METASTABLE_MODEL
       import "DPI-C" function int unsigned cdcRandomFunc(input int unsigned  seed);
       initial begin
       if (!$value$plusargs("seed=%d", randSeed)) begin
        randSeed=123456; 
       end
       `ifdef SYNC2ORSEED_OVERRIDE
       if (check_option_switch("Sync2OrSeedOverride")) begin
        if (get_option_switch("Sync2OrSeedOverride")) begin
          randSeed = get_option_value("Sync2OrSeed");
        end 
       end
       `endif //SYNC2ORSEED_OVERRIDE


         metaRandomVar    = cdcRandomFunc(randSeed);
         evalRandomDelay ( metaRandomVar, randomExtraCycle_reg );

       end
       `endif

        `ifndef METASTABLE_MODEL
            initial randomExtraCycle_reg = {size{1'b0}}; 
            initial metaRandomVar = 32'h0;
        `endif
       // vrq translate_on

  initial                   enableRand = {size{1'b0}};
  initial                   random_var = {size{1'b0}};

  reg         [(size-1):0]  lastD;
  integer                   ilp, ilp1, ilp2;

  integer  LocalMetaWindowSize;
  initial begin
        if (MetaWindowSize)
            LocalMetaWindowSize = MetaWindowSize;
        else
            `ifdef TECH__SYNC__META_WINDOW_SIZE
               LocalMetaWindowSize = `TECH__SYNC__META_WINDOW_SIZE;
            `else
               LocalMetaWindowSize = 100;
            `endif
  end

  initial race_d_late = 0;

  generate if (MetaWindowUse) begin: randomExtraCycle_MetaWindowMode

    wire [(size-1):0]  Ddelayed;
    assign #(LocalMetaWindowSize) Ddelayed = sourceData;

    reg [size-1:0] D_diff;
    always@(*) begin
    for (ilp2=0; ilp2<size; ilp2=ilp2+1)
            D_diff[ilp2] = Ddelayed[ilp2] !== sourceData[ilp2];
    end

    cdc_stdlib_internal_meta_rand #(.halfStage (0))
  
    cdc_stdlib_imr[size-1:0](
      .CK               ( CK           ),
      .EN               ( 1'b1         ),
      // .evaluateRand     ( Ddelayed !== sourceData ),
      .evaluateRand     ( D_diff ),
      .randomExtraCycle ( randomExtraCycle )
      );

      end else begin: randomExtraCycle_normal
// enableRand should be set to 1 after D toggle, and cleared to 0 after the
// next CK posedge.              
`ifdef METASTABLE_MODEL
        // enableRand/randomExtraCycle assert
        always @( sourceData or RB or SB or FLUSH ) begin
            for ( ilp = 0; ilp < size; ilp=ilp+1 ) begin
              if( (~RB) | (~SB) | FLUSH ) begin // disable randomization if flop is set/reset/flushed
                  enableRand[ilp] = 1'b0;
                  randomExtraCycle_reg[ilp] = 1'b0;
              end else begin // notice we use blocking-assignment here. the execute sequence need to be kept.
                      enableRand[ilp] = ( lastD[ilp] !== sourceData[ilp] );
                      if ( enableRand[ilp] )  evalRandomDelay ( metaRandomVar, randomExtraCycle_reg[ilp] );
                      else                 randomExtraCycle_reg[ilp] = 0;
                      random_var[ilp] = metaRandomVar[0];
              end
            end
            lastD = sourceData;
        end

        // enableRand/randomExtraCycle de-assert
        always @( posedge CK ) begin
               enableRand <= {size{1'b0}};
               randomExtraCycle_reg <= {size{1'b0}};
        end
       
        // parameter reduce_meta_pessimism
        // only D toggles happen after CLK negedge trigger meta random
        if (reduce_meta_pessimism)
        always @( negedge CK ) begin
               enableRand <= {size{1'b0}};
               randomExtraCycle_reg <= {size{1'b0}};
        end
        // race_d_late will be asserted when CK_posedge/D toggle 
        // simutaneously, but CK posedge was not able to capture the
        // enableRand. in this case we suppose D arrive later than CK posedge.
        // the register stages[size-1] should keep its previous value after
        // the CK posedge.
        //
        // always @( posedge CK or negedge CK ) begin
        //    for ( ilp1 = 0; ilp1 < size; ilp1=ilp1+1 ) begin
        //        if (CK & !enableRand[ilp1])
        //          race_d_late[ilp1] = (sourceData[ilp1]!==lastD[ilp1]);
        //        else if (!CK)
        //          race_d_late[ilp1] = 1'b0;
        //        else;
        //    end
        // end
`endif        

`ifdef METASTABLE_MODEL
        assign randomExtraCycle = randomExtraCycle_reg;
`else        
        assign randomExtraCycle = 0;
`endif        

    end
endgenerate
                                             
// leda on
// synopsys translate_on

endmodule
//! enable parser

///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
///
///     module:   'cdc_stdlib_internal_meta_rand'
///
///  WARNING: This module is intended solely as a 'cdc_stdlib' internal module. Please do
///  not use this module unless you work closely with it's author.
///
///  Used in conjunction with the 'cdc_stdlib_internal_sync_model', this module has been
///  separated out so that the 'metaRandomVariable' register can remain active during MVSIM
///  simulations while the remainder of the 'cdc_stdlib_internal_sync_model' can be
///  rendered inactive.
///
///  The main function of this module is to randomly decide, at the appropriate sample edge
///  of CK, whether the data being presented to the synchronizer is to be sampled
///  immediately, or whether the sample is to be delayed by one clock cycle. Every time a
///  random decision is made, the random variable 'metaRandomVar' will be advanced one step
///  in its pseudo-random sequence. Since general randomization techniques frown upon
///  pulling from the randomization mechanism when the resultant decsion will not be used
///  [lest it detrimentally alter the pseudo-random characteristic of the variable], we
///  have taken steps to ensure that the random decision is only pulled when we truly need
///  a decision, and the random variable is only advanced when a decision is pulled.
///
///   .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .

//! disable parser
module cdc_stdlib_internal_meta_rand (

    CK,
    EN,
    evaluateRand,
    randomExtraCycle

  );

  // VCS coverage exclude_module
  /// .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .

  parameter  halfStage = 0;      ///  used to control which clock edge is the sample edge

  /// .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .

  input       CK;
  input       EN;
  input       evaluateRand;
  output reg  randomExtraCycle;

  // synopsys translate_off
  /// .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
  ///
  ///  The main body of this 'cdc_stdlib_internal_meta_rand' module is comprised of three
  ///  main elements, each of which will be presented below:
  ///
  ///     o  helper task to abstract the random decision and advancement of the random variable
  ///     o  initialization code for the random variable
  ///     o  edge-appropriate random decision invocation
  ///
  ///
  /// .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
  ///
  ///  Since there is more than one location within our main code that requires a random
  ///  decision, we have encapsulated the decision management, and the random variable
  ///  advancement within this task. This abstraction should provide consistency in the
  ///  behaviour, and mitigate bug infestation.

  task  evalRandomDelay;

    inout   [31:0]  randomVar;    ///  the variable to manage
    output          extraDelay;   ///  the decision to be made

    begin
      extraDelay =   randomVar[0];
      randomVar  = { randomVar[30:0], 1'b0 } ^ ( randomVar[31] ? 32'hc5 : 32'h00 );
      ///  lfsr polynomial :  0000 0000 0000 0000 0000 0000 1100 0101
    end
  endtask

  /// .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
  ///
  ///  Here is the variable that is, in effect, the sole reason for the existence of this
  ///  module as a separate entity. This variable holds the current lfsr state and will be
  ///  advanced once every time a random decision is pulled from it.

  integer   metaRandomVar;

  // leda off
  /// .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
  ///
  ///  The next chunk of code is all about setting our initial value for the
  ///  'metaRandomVar'. As can be readily seen, means for setting the initial value of our
  ///  random variable has several different modes based on a number of testbench
  ///  verfication criteria.


       integer randSeed;
       `ifdef METASTABLE_MODEL
       // vrq translate_off
       import "DPI-C" function int unsigned cdcRandomFunc(input int unsigned  seed);
       initial begin
       if (!$value$plusargs("seed=%d", randSeed)) begin
        randSeed=123456; 
       end
       `ifdef SYNC2ORSEED_OVERRIDE
       if (check_option_switch("Sync2OrSeedOverride")) begin
        if (get_option_switch("Sync2OrSeedOverride")) begin
          randSeed = get_option_value("Sync2OrSeed");
        end 
       end
       `endif //SYNC2ORSEED_OVERRIDE


         metaRandomVar    = cdcRandomFunc(randSeed);
         evalRandomDelay ( metaRandomVar, randomExtraCycle );

       end
       // vrq translate_on


     /// .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
     ///
     ///  Now that we have our initial values selected, we shall put in place the algorithm
     ///  that will continually pull a random decision and advance our random variable.
     ///  Since we are to make our decision at the rise or fall edge of our clock,
     ///  depending on the type of synchronizer we are part of, we use a generate statement
     ///  in order to reduce the amount of code we have active. In particular, we do not
     ///  need both posedge and negedge versions  running, we use some parametric
     ///  information to determine which code we want active.

     generate
       if ( halfStage ) begin: evaluateRand_halfStage1
         always @( negedge CK ) begin
           if ( EN ) begin
             if ( evaluateRand )  evalRandomDelay ( metaRandomVar, randomExtraCycle );
             else                 randomExtraCycle = 0;
           end
         end
       end else begin: evaluateRand_halfStage0
         always @( posedge CK ) begin
           if ( EN ) begin
             if ( evaluateRand )  evalRandomDelay ( metaRandomVar, randomExtraCycle );
             else                 randomExtraCycle = 0;
           end
         end
       end
     endgenerate

  `else

     /// .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
     ///
     ///  When we do not support metastability modelling, we merely set our internal
     ///  variables to inert values, and deprecate all other behavioural code so that we
     ///  do not impose any additional tax on the simulation performance.

     initial begin
       metaRandomVar    = 32'h0;
       randomExtraCycle = 0;
     end
  `endif     ///  METASTABLE_MODEL

  // leda on
  // synopsys translate_on

endmodule

//! enable parser

///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
///
///     module:   'cdc_stdlib_internal_sync_model'
///
///  WARNING: This module is intended solely as a 'cdc_stdlib' internal module. Please do
///  not use this module unless you work closely with it's author.
///
///  This module is one of the main components of an overall synchronizer model. It is
///  responsible for modelling all of the internal state-points within the synchronizer
///  model, including all asynchronous set/reset/flush features, and 'X'ing up when any of
///  its main input controls are 'X' for any duration of time.
///
///  In addition, this model is responsible, with aid from a randomization sub-module,
///  for making random decisions as to whether the most recently seen transition will be
///  sampled in the current destination clock cycle, or will be deferred until the next
///  destination clock cycle. Two algorithms are deployed for determining when a transition
///  should be randomized. The first is the 'aggressive' method which applies randomization
///  to any unsampled transition [with the aid of a multi-bit transition filter to suppress
///  reordering of events]. The second is a 'metastability window' mode, where all
///  transitions within a small period of time ahead of the sampling edge are randomized.
///
///   .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .

//! disable parser
module cdc_stdlib_internal_sync_model (

    SB,
    RB,
    FLUSH,
    CK,
    EN,

    enableRand,
    `ifdef ABV
    InputChkDis,
    `endif

    D,
    Q

  );

  //VCS coverage exclude_module
  /// .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .

  parameter   depth          = 2;
  parameter   halfStage      = 0;
  parameter   InputCheck     = 1;
  parameter   MetaWindowMode = 0;

  /// .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
  ///
  ///  here we create a variant of the 'depth' parameter that reflects the maximum depth
  ///  [cycle delay] of the synchronizer pipeline that we are modelling.

  localparam     PipeP      = ( depth + 1 );

  ///  here we provide two means for enabling a windowed mode of operation. The first is a
  ///  global `define that will cause all synchronizer models to use this mode, and the
  ///  second is an instance specific mode-select [off by default].

  // leda off
  `ifdef META_WINDOW_USE
     localparam  MetaWindowUse = 1;
  `else
     localparam  MetaWindowUse = MetaWindowMode;
  `endif

  /// .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .

  input       SB;            ///  active-low, asynchronous set
  input       RB;            ///  active-low, asynchronous reset
  input       FLUSH;         ///  active-high, asynchronous flush
  input       CK;            ///  clock input
  input       EN;            ///  clock enable

  input       enableRand;    ///  specifies whether randomization is permitted 
  `ifdef ABV
  input       InputChkDis;   ///  disables input pulse-width checking 
  `endif

  input       D;             ///  data input
  output      Q;             ///  synchronized data output

  /// .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
  ///
  ///  The main body of our 'cdc_stdlib_internal_sync_model' module can be sub-divided
  ///  into two major elements, each of which will be covered below
  ///
  ///     o  preparation for a random delay decision
  ///     o  the sychronizer pipeline itself
  ///
  ///
  /// .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
  ///
  ///  The preparation for our random delay decision comes in two varieties. The first, and
  ///  less aggressive of these is a "window-mode" whereby a data transition, within a
  ///  specified amount of time prior to the sampling edge of the clock, will yield a
  ///  random decision as to whether or not to delay the sample by an extra clock cycle.
  ///  Data transitions outside this window will always be sampled as soon as possible. It
  ///  should be noted that this mode of operation is really only effective when clock
  ///  jitter / wander is modelled within the design. However, there may be specific cases
  ///  in which a "windowed-mode" may be required for specific instances in order to
  ///  [temporarily] work around ordering issues that the more aggressive [second] mode
  ///  induces.
  ///
  ///  The second mode is used to more aggressively apply randomization decisions regarding
  ///  whether to immediately sample incoming data, or to delay the sample by one clock
  ///  cycle. Due to the aggressiveness of this mechanism, it is necessary for this method
  ///  to be tempered by a multi-bit filter ['cdc_stdlb_mulitbit_transition_filter'] in
  ///  order to respect event-ordering within a group of synchronized bits.
  ///
  ///  Before we begin, however, we need to declare a few signals...

  // synopsys translate_off

  reg      [1:0]     hstage;
  reg  [depth:0]     stages;
  wire               delaySync;
  wire               evaluateRand;

  // declaration of delta-delay versions signals. why they are introduced will
  // be explained when they are used. 
  reg       SBd,   RBd,   FLUSHd,   Dd,   CKd,  ENdel;
  reg       SBdd,  RBdd,  FLUSHdd,  Ddd,  CKdd;
  reg       SBddd, RBddd, FLUSHddd, Dddd, CKddd;

  ///  In support of the 'windowed-mode', we create a local variable to [statically] store
  ///  the size of the window, in picoseconds. It should be noted that this code is
  ///  expected to be used rarely, but it does not represent a significant performance
  ///  issue as it is only excuted when the simulations start up.

  integer  LocalMetaWindowSize;
  initial begin
   `ifdef TECH__SYNC__META_WINDOW_SIZE
      LocalMetaWindowSize = `TECH__SYNC__META_WINDOW_SIZE;
   `else
      LocalMetaWindowSize = 100;
   `endif
  end

  ///  Below we have the main code for putting together our delay randomization decision.
  ///  Since this code is in the performance path of the module, we use generate statements
  ///  to only include the code that is relevant to the behaviour specified by our
  ///  parameters and defines. In particular, our model can support the following methods
  ///  of operation:
  ///
  ///     o  negedge synchronization, windowed mode
  ///     o  negedge synchronization, aggressive mode
  ///     o  posedge synchronization, windowed mode
  ///     o  posedge synchronization, aggressive mode
  ///
  ///  While each of these modes have slightly different code, and only the code relevant
  ///  to that mode mode be compiled into our design instance. However, the distinction
  ///  between posedge and negedge sampling is handled by a subordinate module [as will
  ///  become apparent]

       wire   Ddelayed; // om
  generate
    if ( MetaWindowUse ) begin: evaluateRand_MetaWindowUse_1

      ///  We create a simple, inertially delayed version of our incoming data. We select
      ///  this method of delay since it is more performant, and the expectation is that
      ///  the window size will be sufficiently small as to not cause the event to be
      ///  intertially dequeued. Then, our random delay decision will only be active should
      ///  the sample edge occur before the signal has propagated.

       
      assign #(LocalMetaWindowSize) Ddelayed = D;
      assign evaluateRand = ( D !== Ddelayed );

    end else begin: evaluateRand_MetaWindowUse_0

      ///  In conjunction with the 'enableRand' signal from the multi-bit pre-filter, we
      ///  elect to randomize our delay based merely on whether the current data has
      ///  changed at least once since the last time it was sampled. In effect, this is
      ///  what the 'enableRand' terms tells us, so our decision to evaluate is merely a
      ///  reflection of the filter's information.

      assign evaluateRand = enableRand;

    end
  endgenerate

  reg   evaluateRand_extend;
  initial evaluateRand_extend = 1'b0;
  always @(posedge CK or posedge CKd)
    if (!CKd)
        evaluateRand_extend <= evaluateRand;
    else
        evaluateRand_extend <= 1'b0;

  ///  So far, we have made a coarse determiniation as to whether we are willing to
  ///  randomize the delay of our sample. However, we still need to turn this into a
  ///  concrete decision at the time of our sampling clock edge. For this, we shall
  ///  instantiate a sub-module that handles the final decision, taking the sample edge
  ///  into account. Note, this sub-module has been partitioned off so that the random
  ///  variable that it contains can be mainated during MVSIM while this module is
  ///  deactivated.

  cdc_stdlib_internal_meta_rand #(halfStage) cdc_stdlib_imr (

      // we need to sample evaluateRand to generate delaySync(indicator of whether to 
      // insert the extra delay) in the 1st cycle after data changed.
      // when CK and D switch simutaneously, evaluateRand is asserted after the CK 
      // rising edge, hence if we sample evaluateRand with CK, the delaySync can only be
      // asserted at the 2nd CK cycle, this may cause incorrect pulse at Q.
      // -- ubts 644550
      // so we use CKd instead of CK to make sure evaluateRand is sampled in the 1st cycle.

  // Explanation of why we use evaluateRand|evaluateRand_extend to generate the
  // actual evaluateRand window, sampled by CKd to generate delaySyncP.
  // 
  // Generally there are only 3 kinds of relationship between D and CK:
  // D comes before CK rising edge, D and CK transition simutaneously, or D is
  // 1 delta delay after CK posedge(in transportCycle mode).
  //   ___
  // _|dly|_ represents a delta delay.
  //
  // 1. if D is stable before CK edge, evaluateRand will be clear to 0 after CK
  //    rising edge, thus cannot be sampled by CKd. that's why we introduce
  //    its extend to make sure the pulse would not be missed.
  //                               _________________________
  // D:                   ________|
  //                                            ____
  // CK:                  _____________________|    |_______
  //                                  ____________
  // evaluateRand:        ________dly|         dly|_________
  //                                               ____
  // CKd:                 _____________________dly|    |____
  //                                               ___
  // evaluateRand_extend: _____________________dly|dly|_____
  //                                  ________________
  // evaluateRand_actual: ___________|                |_____
  //
  //
  // 2. if D and CK switch simutaneously, we need to clear evaluateRand after
  //    CKd, otherwise the extend will assert at next CK edge and be sampled
  //    by next CKd -- an unexpected delaySyncP will be generated.
  //                                           ______________________
  // D:                   ____________________|
  //                                           ____       ____
  // CK:                  ____________________|    |_____|    |______
  //                                              ____       ____
  // CKd:                 ____________________dly|    |__dly|    |___
  //                                              ___........
  // evaluateRand:        ____________________dly|dly|___dly..........
  //                                                        .....
  // evaluateRand_extend: _______________________________dly.dly......
  //                                              ___............
  // evaluateRand_actual: _______________________|   |__________.____
  //
  // 3. if transportCycle_assert assert, (transportCycle is enabled and random
  // result is insert the transport cycle), D will always switches after CK posedge, 
  // 1 delta delay later. in this mode the evaluateRand is asserted after CKd,
  // it should not be cleared in this mode.
  //                                           _________________________
  // Dsrc:                _______________//////
  //                                           ____       ____
  // CK:                  ____________________|    |_____|    |_________
  //                                              ______________________
  // D:                   ____________________dly|
  //                                              ____       ____
  // CKd:                 ____________________dly|    |__dly|    |______
  //                                                 __________
  // evaluateRand:        _______________________dly|dly    dly|________
  //                                                        ...
  // evaluateRand_extend: _______________________________dlydly|________
  //                                                 __________
  // evaluateRand_actual: __________________________|          |________

      // .CK               ( CK           ),
      .CK               ( CKd           ),
      .EN               ( EN           ),
      // .evaluateRand     ( evaluateRand ),
      .evaluateRand     ( evaluateRand|evaluateRand_extend  ),
      .randomExtraCycle ( delaySync    )

    );

  /// .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
  ///
  ///  Now, that we have prepared our random delay decision, we move on to the actual
  ///  synchronizer modelling. In the construction of our overall synchronizer pipeline,
  ///  we have a negedge pipeline, and a posedge pipeline. In any given configuration,
  ///  the random delay will only be introduced into one of these two pipelines, but
  ///  depending on mode, could be either.
  ///
  ///  In order to ensure we only introduce delay in the lead pipeline, we create
  ///  separate decision signals that will carry our previous decision, exclusively to
  ///  the pipeline of importance.

  wire    delaySyncN = halfStage ? delaySync : 1'b0;
  wire    delaySyncP = halfStage ? 1'b0      : delaySync;

  ///  Moving on to the meat of our synchronizer model...
  ///
  ///  Whenever one of our asynchronous inputs transitions, we abort our synchronizer
  ///  'always' blocks so that we can re-trigger them from the transitions. However, since
  ///  the 'disable' command is executed in the same simulation-delta as the event itself,
  ///  we are subject to a potential race-condition. To avoid this race, we create
  ///  equivalent events in the next simulation delta, and use those events within our
  ///  synchronizer 'always' blocks.

//   initial begin
//     if (FLUSH | ~SB | ~RB) begin
//       FLUSHd     <= FLUSH;
//       FLUSHdd    <= FLUSH;
//       FLUSHddd   <= FLUSH; 
//       SBd        <= SB;
//       SBdd       <= SB;
//       SBddd      <= SB; 
//       RBd        <= RB;
//       RBdd       <= RB;
//       RBddd      <= RB; 
//       Dd         <= (D & RB) | ~SB;
//       Ddd        <= (D & RB) | ~SB;
//       Dddd       <= (D & RB) | ~SB;
//       hstage <= {2{(D & RB) | ~SB}};
//       stages <= {PipeP{(D & RB) | ~SB}};
//     end
//     #1; // Evaluate at time 0 and time 1 to insure we drive and recieve 0ps values as needed.
//     if (FLUSH | ~SB | ~RB) begin
//       FLUSHd     <= FLUSH;
//       FLUSHdd    <= FLUSH;
//       FLUSHddd   <= FLUSH; 
//       SBd        <= SB;
//       SBdd       <= SB;
//       SBddd      <= SB; 
//       RBd        <= RB;
//       RBdd       <= RB;
//       RBddd      <= RB; 
//       Dd         <= (D & RB) | ~SB;
//       Ddd        <= (D & RB) | ~SB;
//       Dddd       <= (D & RB) | ~SB;
//       hstage <= {2{(D & RB) | ~SB}};
//       stages <= {PipeP{(D & RB) | ~SB}};
//     end
//   end
  always @( SB      ) begin  disable syncp; disable syncn;    SBd      <= `ifdef CDC_RTLLIB_DELTA_DEBUG #1 `else `endif SB;       end
  always @( RB      ) begin  disable syncp; disable syncn;    RBd      <= `ifdef CDC_RTLLIB_DELTA_DEBUG #1 `else `endif RB;       end
  always @( FLUSH   ) begin  disable syncp; disable syncn;    FLUSHd   <= `ifdef CDC_RTLLIB_DELTA_DEBUG #1 `else `endif FLUSH;    end
  always @( D       ) begin  disable syncp; disable syncn;    Dd       <= `ifdef CDC_RTLLIB_DELTA_DEBUG #1 `else `endif D;        end
  always @( EN      ) begin                                   ENdel    <= `ifdef CDC_RTLLIB_DELTA_DEBUG #1 `else `endif EN;       end

  always @( SBd     ) begin                                   SBdd     <= `ifdef CDC_RTLLIB_DELTA_DEBUG #1 `else `endif SBd;      end
  always @( RBd     ) begin                                   RBdd     <= `ifdef CDC_RTLLIB_DELTA_DEBUG #1 `else `endif RBd;      end
  always @( FLUSHd  ) begin                                   FLUSHdd  <= `ifdef CDC_RTLLIB_DELTA_DEBUG #1 `else `endif FLUSHd;   end
  always @( Dd      ) begin                                   Ddd      <= `ifdef CDC_RTLLIB_DELTA_DEBUG #1 `else `endif Dd;       end
  always @( CKd     ) begin                                   CKdd     <= `ifdef CDC_RTLLIB_DELTA_DEBUG #1 `else `endif CKd;      end

  always @( SBdd    ) begin                                   SBddd    <= `ifdef CDC_RTLLIB_DELTA_DEBUG #1 `else `endif SBd;      end
  always @( RBdd    ) begin                                   RBddd    <= `ifdef CDC_RTLLIB_DELTA_DEBUG #1 `else `endif RBd;      end
  always @( FLUSHdd ) begin                                   FLUSHddd <= `ifdef CDC_RTLLIB_DELTA_DEBUG #1 `else `endif FLUSHd;   end
  always @( Ddd     ) begin                                   Dddd     <= `ifdef CDC_RTLLIB_DELTA_DEBUG #1 `else `endif Dd;       end
  always @( CKdd    ) begin                                   CKddd    <= `ifdef CDC_RTLLIB_DELTA_DEBUG #1 `else `endif CKd;      end

  always @( CK    ) begin                                     CKd      <= `ifdef CDC_RTLLIB_DELTA_DEBUG #1 `else `endif CK;    
    if (( CK === 1'bx ) || ( CK === 1'bz )) begin
      disable syncp;  ///  Note, when we detect that our clock has transitioned to an
      disable syncn;  ///  'unknown' state, we need to pollute the synchronizer. In order
    end               ///  for us to do that, we need to disable our synchroner piplines
  end                 ///  and re-trigger them. Similarly to above, we do this via a
  ///                      delta-cycle delayed version of the clock. However, there is a
  ///  second need for this delayed clock. Since the clock event is the time at which we
  ///  determine whether to introduce an extra cycle of delay, we need to be able to pick
  ///  up the decision in the current clock. This we can only do if we use a slightly
  ///  delayed version of the clock. Otherwise, we will end up picking up the decision on
  ///  the following pos/neg edge, not the current one.
  ///
  /// .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
  ///
  ///  Here we have two 'always' blocks that form the synchronizer pipeline. Note, we had
  ///  two options for how to handle with the first latch stage, if present. We could have
  ///  used a 'generate' construct to optionally include this [to improve simulation
  ///  performance slightly], but that would also require us to replicate the previous
  ///  code minus the 'disable syncn' statement. Instead, the method we chose was to have
  ///  the latch-stage always present, but bypass it when the half-stage is not required.
  ///  This reduces the clutter within our code, but we may need to revisit this decision
  ///  based on actual simulation performance.
  ///
  ///  So, we create a term that either sources from the latch output, or bypasses the
  ///  latch and sources directly from our data input [or the delta-cycle delayed version,
  ///  to be more specific].

  wire    Dp = halfStage ? hstage[0] : D;

  ///  Note, rather than use 'always @*', we need to explicitly call out our complete
  ///  sensitivity list. This is necessary in order to cover the 'trigger_me' event that
  ///  we use when the clock transitions to an unknown state.

  always @( SBd or RBd or FLUSHd or CKd or Dd ) begin : syncn
   
    ///  If any of our inputs are unknown, we pollute our state to unknown. Otherwise,
    ///  we handle all of our asynchronous conditions. Note, the async set takes
    ///  precedence over async reset.

         if (( SBd   === 1'bx )  || ( SBd   === 1'bz ))   hstage <= 2'bx;
    else if (( RBd === 1'bx )    || ( RBd === 1'bz ))     hstage <= 2'bx;
    else if (( FLUSHd === 1'bx ) || ( FLUSHd === 1'bz ))  hstage <= 2'bx;
    else if ( ~SBd   )                                    hstage <= 2'b1;
    else if ( ~RBd )                                      hstage <= 2'b0;
    else if (  FLUSHd )                                   hstage <= {2{Dd}};
    else if (( CKd    === 1'bx ) || ( CKd    === 1'bz ))  hstage <= 2'bx;  // ubts 622993

    ///  Once we have handled our async signals, we wait for our edge-sensitive capture.
    ///  However, since we still need to be sensitive to async events, [as seen in earlier
    ///  code] we shall abort this wait if an asynchronous event occurs.
    ///
    ///  It should be noted that a byproduct of this wait/disable functionality is that
    ///  this always block will be triggered every time the clock falls, and every time
    ///  the data transitions.
   
    if ((FLUSHd === 1'b0) & (RBd === 1'b1) & (SBd === 1'b1)) begin
      @( negedge CKd );

      ///  When we have seen our falling edge, we still need to allow the async inputs to
      ///  override our data-update. However, if none of the overrides are active, then we
      ///  can clock in our incoming data.
  
           if (( SBd   === 1'bx )  || ( SBd   === 1'bz ))   hstage <= 2'bx;
      else if (( RBd === 1'bx )    || ( RBd === 1'bz ))     hstage <= 2'bx;
      else if (( FLUSHd === 1'bx ) || ( FLUSHd === 1'bz ))  hstage <= 2'bx;
      else if ( ~SBd   )                                    hstage <= 2'b1;
      else if ( ~RBd )                                      hstage <= 2'b0;
      else if (  FLUSHd )                                   hstage <= {2{Dd}};
      else if (  ENdel  )       hstage <= ( delaySyncN ) ? {Dd,~Dd} : {2{Dd}};
    end

    ///  Note, in the case when we are delaying the the data sample, we populate the
    ///  immediate sample with the inverse of the current data. The rationale for this is
    ///  as follows... Since the data may have transitioned more than once since the last
    ///  sample, we should only be modelling whether the last transtion was successfully
    ///  sampled -- not whether we retain the previous sample. For instance, if our last
    ///  sample was 0, and our data has transitioned to 1 and then back to 0 by the time
    ///  we sample it, a delayed sample should capture a '1' from the previous value, not
    ///  the '0' from the last sample.
    ///
    ///  As a simplification, we assume that this happens outside of situations where the
    ///  data input is unknown. With this simplfication, we assume that the previous value
    ///  is '~Dd' when the current value is 'Dd'.

  end

  /// .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
  ///
  ///  Our main posedge pipeline follows a very similar flow to the negedge latch stage.
  ///  We similarly need to fully specify our sensitivity list so that our process is also
  ///  sensitive to the 'trigger_me' event.

  reg   [7:0] debug_x_status;

  /// always @( SBdd or RBdd or FLUSHdd or CKdd or Ddd or Dp ) begin : syncp
  // always @( SBd or RBd or FLUSHd or CKddd or Ddd ) begin : syncp
  always @( SBd or RBd or FLUSHd or CKddd or Dd ) begin : syncp

    ///  If any of our inputs are unknown, we pollute our state to unknown. Otherwise, we
    ///  handle all of our asynchronous conditions. Note, the async set takes precedence
    ///  over async reset.

    if (( SBd    === 1'bx ) || ( SBd    === 1'bz ))    begin stages <= {PipeP{1'bx}}; debug_x_status <= 8'd0; end
    else if (( RBd    === 1'bx ) || ( RBd    === 1'bz ))    begin stages <= {PipeP{1'bx}}; debug_x_status <= 8'd1; end
    else if (( FLUSHd === 1'bx ) || ( FLUSHd === 1'bz ))    begin stages <= {PipeP{1'bx}}; debug_x_status <= 8'd2; end
    else if ( ~SBd   )                                      stages <= {PipeP{1'b1}};
    else if ( ~RBd )                                        stages <= {PipeP{1'b0}};
    else if (  FLUSHd )                                     stages <= {PipeP{Dd}};
    else if (( CKddd   === 1'bx ) || ( CKddd   === 1'bz ))  begin stages <= {PipeP{1'bx}}; debug_x_status <= 8'd3; end // ubts 622993

    ///  Once we have handled our async signals, we wait for our edge-sensitive capture.
    ///  However, since we still need to be sensitive to async events, [as seen in earlier
    ///  code] we shall abort this wait if an asynchronous event occurs.
    ///
    ///  It should be noted that a byproduct of this wait/disable functionality is that
    ///  this always block will be triggered every time the clock rises, and every time
    ///  the data transitions.
   
   if ((FLUSHd === 1'b0) & (RBd === 1'b1) & (SBd === 1'b1)) begin
     @( posedge CKddd );

      ///  When we have seen our rising edge, we still need to allow the async inputs to
      ///  override our data-update. However, if none of the overrides are active, then we
      ///  can clock our data data through our posedge pipeline. Note, the 'Dp' term [as per
      ///  previous code] will either source from the output of our negedge 'latch' stage,
      ///  or will bypass the latch and source directly from our input.
      ///
      ///  Note, as we shift through our posedge pipeline, we shift through a vector from
      ///  MSB to LSB. By doing this, our final stage will always be the LSB of our pipeline,
      ///  regardless of the pipeline depth.
  
      if (( SBddd    === 1'bx ) || ( SBddd    === 1'bz ))  begin stages <= {PipeP{1'bx}}; debug_x_status <= 8'd4; end
      else if (( RBddd    === 1'bx ) || ( RBddd    === 1'bz ))  begin stages <= {PipeP{1'bx}}; debug_x_status <= 8'd5; end
      else if (( FLUSHddd === 1'bx ) || ( FLUSHddd === 1'bz ))  begin stages <= {PipeP{1'bx}}; debug_x_status <= 8'd6; end
      else if ( ~SBddd             )                            stages <= {PipeP{1'b1}};
      else if ( ~RBddd             )                            stages <= {PipeP{1'b0}};
      else if (  FLUSHddd          )                            stages <= {PipeP{Dddd}};
      else if (  ENdel  ) stages <= ( delaySyncP ) ? {Dddd,~Dddd,stages[(depth-1):1]} : {Dddd,Dddd,stages[(depth-1):1]};
  
      ///  Note, in the case when we are delaying the the data sample, we populate the
      ///  immediate sample with the inverse of the current data. The rationale for this is
      ///  as follows... Since the data may have transitioned more than once since the last
      ///  sample, we should only be modelling whether the last transtion was successfully
      ///  sampled -- not whether we retain the previous sample. For instance, if our last
      ///  sample was 0, and our data has transitioned to 1 and then back to 0 by the time
      ///  we sample it, a delayed sample should capture a '1' from the previous value, not
      ///  the '0' from the last sample.
      ///
      ///  As a simplification, we assume that this happens outside of situations where the
      ///  data input is unknown. With this simplfication, we assume that the previous value
      ///  is '~Dp' when the current value is 'Dp'.
    end
  end

  ///  Finally, as our data flows to the end of our pipeline, we merely need to take the
  ///  last stage of our posedge pipeline and drive it out as our 'Q' output.

  assign  Q = stages[0];


`ifdef ABV

////////////////////////////////////////////////////////////////////////////////
// Pulse Width Check for normal mode, MetaWindowMode not used

// This is RTL Simulation-ONLY logic used to model meta-stable events
//   If you get an ova compile error here, then you need to define ABV. This is not a bug!
// WARNING: For some paths it is probably not legal to disable this under NbIReset (i.e HT Config)


reg DelayFF, Dsync;

always @(D) 
   Dsync <= D; //transitions at the same time as randomExtraCycle

// Delay flip flop
always @(posedge CK) begin
   if(EN) DelayFF <= Dsync;
   `ifdef SYNC_WINDOW_DISPLAY_RANDOMIZATION
//om   if(MetWindowMode & RandomExtraCycle & EnableRand & (D_delayed ^ D))
   if(MetaWindowMode & delaySync & enableRand & (Ddelayed ^ D))
      $display("INFO : %m : Synchronizer Meta-stability Window hit at Time=%t", $time);
   `endif
end

always @(posedge EN) 
   DelayFF <= Dsync;

reg input_check;
wire meta_window_mode = MetaWindowMode ? 1'b1 : 1'b0;
initial
  input_check = 1'b0;                      // Initialize to 0 at time 0
always @(negedge CK)
    input_check <= InputCheck ? 1'b1 : 1'b0;  // parameters are 32bit, making it single bit for use 

// om  `ifdef ABV
`ifndef REMOVE_OVL_NEVER_CDC_STDLIB_INTERNAL_SYCNFF_RAND_MODEL
ovl_never #(.msg("DataIn is changing too quickly for the synchronizer.")) Assert_SYNCFF (
    .clock (CK), .reset((RB===1'b1) & (SB===1'b1) & (FLUSH===1'b0)), .enable(input_check & ~meta_window_mode & ~InputChkDis), 
    .test_expr( (~((D === DelayFF) | (DelayFF === $past(DelayFF,1,,@(posedge CK))))) &
                 (DelayFF !== 1'bx) & (Dsync !== 1'bx) ), .fire());
`endif   // REMOVE_OVL_NEVER_CDC_STDLIB_INTERNAL_SYCNFF_RAND_MODEL
// om  `else
// om  // ova clock posedge CK;
// om  // ova forbid_bool((~((D === DelayFF) | (DelayFF === DelayFF[#1])) & (DelayFF !== 1'bx) & (Dsync !== 1'bx) & input_check & ~MetWindowMode), "SYNCFF Error: DataIn is changing too quickly for the synchronizer.");
// om  `endif

// Pulse Width Check for normal mode, MetaWindowMode not used
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Pulse Width Check for MetaWindowMode mode
//   - Data should be held long enough to hit 2 clock edges if shifted to cover
//     either destclk period to guarantee capture
////////////////////////////////////////////////////////////////////////////////


        reg DataCaptured;
        time lastCK, lastD, DestClockPeriod, DataPulseWidth, LastClockPeriod;

        initial begin
          DataPulseWidth = 0; DestClockPeriod = 0; LastClockPeriod = 0; lastCK = 0; lastD = 0; DataCaptured = 0;
        end

        always @(negedge InputChkDis) begin // reset
          DataPulseWidth = 0; DestClockPeriod = 0; LastClockPeriod = 0; lastCK = 0; lastD = 0; DataCaptured = 0;
        end

        ////////////////////////////////////////////////////////////////////////////////
        // ClockCheck - measures last clock period, then compares the most recent data
        //   against the last 2 clock periods. Data pulses smaller than either period 
        //   could potentially drift and be lost.
        // Disabled using InputCheck (this should NOT be disabled all the time!!!!!)
        ////////////////////////////////////////////////////////////////////////////////
        always @(posedge CK) begin : ClockCheck
            reg Last_Fail, This_Fail;
            if(meta_window_mode & input_check & ~InputChkDis & |lastCK) begin
                LastClockPeriod = DestClockPeriod;
                DestClockPeriod = $time - lastCK;
                if(DataPulseWidth) begin
                    Last_Fail = DataCaptured && (DataPulseWidth < (LastClockPeriod + LocalMetaWindowSize));
                    This_Fail = DataCaptured && (DataPulseWidth < (DestClockPeriod + LocalMetaWindowSize));
                    if(Last_Fail & This_Fail ) begin
                        $display("ERROR : %m : %0t D input pulse < %0t CLK period + %0t metastable window Time=%t", 
                            DataPulseWidth, DestClockPeriod, LocalMetaWindowSize,$time);
                        `ifndef SYNC_WINDOW_DISABLE_FATAL_ERRORS
                        $finish;
                        `endif
                    `ifndef SYNC_WINDOW_DISABLE_WARNINGS
                    end else if( Last_Fail ) begin
                        $display("WARNING : %m : %0t D input pulse < previous %0t CLK period + %0t metastable window Time=%t", 
                            DataPulseWidth, LastClockPeriod, LocalMetaWindowSize,$time);
                    end else if( This_Fail ) begin
                        $display("WARNING : %m : %0t D input pulse < %0t CLK period + %0t metastable window Time=%t", 
                            DataPulseWidth, DestClockPeriod, LocalMetaWindowSize,$time);
                    `endif
                    end
                end
                DataPulseWidth = 0; // last pulse checked
                lastCK = $time;
            end
        end

        ////////////////////////////////////////////////////////////////////////////////
        // DataCheck - measures data pulses, catches pulses smaller than a single clock
        // Disabled using InputCheck (this should NOT be disabled all the time!!!!!)
        ////////////////////////////////////////////////////////////////////////////////
        always @(D) begin : DataCheck
            if(meta_window_mode & input_check & ~InputChkDis) begin
                if(|lastCK & (($time - lastD) > 5)) begin //smooth out any data glitches and ignore check before 1st clock edge
                    if(DataPulseWidth) begin // no clock edge occurred in this pulse, still need to check last pulse
                        if(DataCaptured) begin // last pulse included a posedge clock, need to check it.
                            if(DataPulseWidth < (DestClockPeriod + LocalMetaWindowSize)) begin
                                $display("ERROR : %m : %0t D input pulse < %0t CLK period + %0t metastable window Time=%t", 
                                    DataPulseWidth, DestClockPeriod, LocalMetaWindowSize,$time);
                                `ifndef SYNC_WINDOW_DISABLE_FATAL_ERRORS
                                $finish;
                                `endif
                            end
                        end
                        `ifndef SYNC_WINDOW_DISABLE_WARNINGS
                        $display("WARNING : %m : %0t D pulse lost at Time=%t", DataPulseWidth,$time);
                        `endif
                    end
                    DataCaptured = (lastD < lastCK) & (Ddelayed !== 1'bx) & (D !== 1'bx);
                    DataPulseWidth = $time - lastD;
                end
                lastD = $time;
            end
        end

`endif // `ifdef ABV

////////////////////////////////////////////////////////////////////////////////
// End of Pulse Width Check for MetaWindowMode  mode
//   - Data should be held long enough to hit 2 clock edges if shifted to cover
//     either destclk period to guarantee capture


// leda on
// synopsys translate_on

endmodule

//! enable parser
///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
///
///     module:   'cdc_stdlib_internal_sync_model_nodelta'
///
///  WARNING: This module is intended solely as a 'cdc_stdlib' internal module. Please do
///  not use this module unless you work closely with it's author.
///
///  the no_delta version of module cdc_stdlib_internal_sync_model.
///
///   .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .

//! disable parser
module cdc_stdlib_internal_sync_model_nodelta (

    SB,
    RB,
    FLUSH,
    CK,
    EN,

    race_d_late,
    randomExtraCycle,
    random_var,
    transportCycle_assert,
    TC_holdcheck_assert,
    `ifdef ABV
    InputChkDis,
    `endif

    D,
    Q,

    SI,
    SO,
    SE

  );

  //VCS coverage exclude_module
  /// .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .

  parameter   depth          = 2;
  parameter   InputCheck     = 1;
  parameter   MetaWindowMode = 0;
  parameter   MetaWindowSize = 0;
  parameter   HoldCheck      = 0;
  parameter   DstClkIdleTime = 0;
  parameter   DstClkIdleCycles = 0;

  ///  here we provide two means for enabling a windowed mode of operation. The first is a
  ///  global `define that will cause all synchronizer models to use this mode, and the
  ///  second is an instance specific mode-select [off by default].

  // leda off
  `ifdef META_WINDOW_USE
     localparam  MetaWindowUse = 1;
  `else
     localparam  MetaWindowUse = MetaWindowMode;
  `endif

  /// .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .

  input       SB;            ///  active-low, asynchronous set
  input       RB;            ///  active-low, asynchronous reset
  input       FLUSH;         ///  active-high, asynchronous flush
  input       CK;            ///  clock input
  input       EN;            ///  clock enable

  input       randomExtraCycle;    /// meta randomization result, insert an extra cycle or not 
  input       random_var;    /// will not be clear after CK rising edge
  input       race_d_late;
  input       transportCycle_assert;              
  input       TC_holdcheck_assert;
  `ifdef ABV
  input       InputChkDis;   ///  disables input pulse-width checking 
  `endif

  input       D;             ///  data input
  output      Q;             ///  synchronized data output

  input       SI;
  output      SO;
  input       SE;

  /// .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
  ///
  ///  The main body of our 'cdc_stdlib_internal_sync_model' module can be sub-divided
  ///  into two major elements, each of which will be covered below
  ///
  ///     o  preparation for a random delay decision
  ///     o  the sychronizer pipeline itself
  ///
  ///
  /// .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
  ///
  ///  The preparation for our random delay decision comes in two varieties. The first, and
  ///  less aggressive of these is a "window-mode" whereby a data transition, within a
  ///  specified amount of time prior to the sampling edge of the clock, will yield a
  ///  random decision as to whether or not to delay the sample by an extra clock cycle.
  ///  Data transitions outside this window will always be sampled as soon as possible. It
  ///  should be noted that this mode of operation is really only effective when clock
  ///  jitter / wander is modelled within the design. However, there may be specific cases
  ///  in which a "windowed-mode" may be required for specific instances in order to
  ///  [temporarily] work around ordering issues that the more aggressive [second] mode
  ///  induces.
  ///
  ///  The second mode is used to more aggressively apply randomization decisions regarding
  ///  whether to immediately sample incoming data, or to delay the sample by one clock
  ///  cycle. Due to the aggressiveness of this mechanism, it is necessary for this method
  ///  to be tempered by a multi-bit filter ['cdc_stdlb_mulitbit_transition_filter'] in
  ///  order to respect event-ordering within a group of synchronized bits.
  ///
  ///  Before we begin, however, we need to declare a few signals...

  // synopsys translate_off

  reg  [depth-1:0]     stages;
  reg  [depth-1:0]     stages_dft;

  reg DataCaptured;
  time lastCK, lastD, DestClockPeriod, DataPulseWidth, LastClockPeriod;
  ///  In support of the 'windowed-mode', we create a local variable to [statically] store
  ///  the size of the window, in picoseconds. It should be noted that this code is
  ///  expected to be used rarely, but it does not represent a significant performance
  ///  issue as it is only excuted when the simulations start up.

  integer  LocalMetaWindowSize;
  initial begin
        if (MetaWindowSize)
            LocalMetaWindowSize = MetaWindowSize;
        else
            `ifdef TECH__SYNC__META_WINDOW_SIZE
               LocalMetaWindowSize = `TECH__SYNC__META_WINDOW_SIZE;
            `else
               LocalMetaWindowSize = 100;
            `endif
  end

  /// .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
  ///
  ///  Now, that we have prepared our random delay decision, we move on to the actual
  ///  synchronizer modelling. In the construction of our overall synchronizer pipeline,
  ///  we have a negedge pipeline, and a posedge pipeline. In any given configuration,
  ///  the random delay will only be introduced into one of these two pipelines, but
  ///  depending on mode, could be either.
  ///
  ///  In order to ensure we only introduce delay in the lead pipeline, we create
  ///  separate decision signals that will carry our previous decision, exclusively to
  ///  the pipeline of importance.

  wire    delaySyncP = randomExtraCycle;

  ///  Moving on to the meat of our synchronizer model...
  ///
  ///  Whenever one of our asynchronous inputs transitions, we abort our synchronizer
  ///  'always' blocks so that we can re-trigger them from the transitions. However, since
  ///  the 'disable' command is executed in the same simulation-delta as the event itself,
  ///  we are subject to a potential race-condition. To avoid this race, we create
  ///  equivalent events in the next simulation delta, and use those events within our
  ///  synchronizer 'always' blocks.

  // reg SBd, RBd, FLUSHd, CKd;
  // always @* SBd <= SB;
  // always @* RBd <= RB;
  // always @* FLUSHd <= FLUSH;
  // always @* CKd <= CK;

  // for 0/1->X/Z scenarios
  wire rst_x_or_z = SB===1'bx | SB===1'bz | RB===1'bx | RB===1'bz | FLUSH===1'bx | FLUSH===1'bz; //| CK===1'bx | CK===1'bz;
  wire clk_x_or_z = CK===1'bx | CK===1'bz;
  // // for X/Z->0/1 scenarios
  // wire input_fr_x_or_z = SBd===1'bx | SBd===1'bz | RBd===1'bx | RBd===1'bz | FLUSHd===1'bx | FLUSHd===1'bz | CKd===1'bx | CKd===1'bz;
  // // both scenarios above should trigger X pollution on Q side
  // wire input_x_or_z = input_to_x_or_z | input_fr_x_or_z;
  // wire input_x_or_z = input_to_x_or_z;

  // if D change during FLUSH=1, Q should change accordingly.
  reg flush_d_toggle;
  initial flush_d_toggle = 0;
  always@(D)
        if (FLUSH)
            flush_d_toggle = !flush_d_toggle; 

  // if D change when CK is high, use hold model for metastable
  // note HC is disabled in legacy MetaWindowMode    
  reg holdcheck_assert;
  reg [depth-1:0] stages_n;
  initial holdcheck_assert = 0;
  generate
    if (HoldCheck && (MetaWindowMode==0) ) begin: syncp_hold
        always @(posedge CK)
            holdcheck_assert <= 0; 
        always @(D)
          if (transportCycle_assert)
              holdcheck_assert <= TC_holdcheck_assert;
          else begin
              holdcheck_assert <= CK;
   // This is needed for microtime races when D and CK change at the same moment:
   // D changed at the falling edge of CK, holdcheck_assert
   // got set (should not happen unless delta race happens) but stages_n didn't update and kept X
              if (CK)
                stages_n  <= random_var ? {~D,stages[(depth-1):1]} : {D,stages[(depth-1):1]};
          end
        always @(negedge CK)
            if (holdcheck_assert)
                stages_n  <= random_var ? {~D,stages[(depth-1):1]} : {D,stages[(depth-1):1]};
    end
  endgenerate
        
  // clock gating support
  reg firstCK;  // will be valid from t=0 till first posedge of CK. 
  initial firstCK <= 1'b1;
  initial begin lastCK=0; DestClockPeriod=0; end 

  reg clock_gating_detected;
  initial clock_gating_detected = 1'b0;
  generate 
    if (DstClkIdleTime || DstClkIdleCycles) begin: ClkGatingDetected
        always @(posedge CK)
            if(CK & firstCK) firstCK <= 1'b0;

        always @(posedge CK) begin
            DestClockPeriod = $time - lastCK;
            lastCK = $time;
        end

        always @(D or posedge CK)
            if ( ($time - lastCK > DestClockPeriod*DstClkIdleCycles) & (DstClkIdleCycles > 0) & (!firstCK) )
                clock_gating_detected = 1'b1;
            else if ( ($time - lastCK > DstClkIdleTime) & (DstClkIdleTime > 0) & (!firstCK) )
                clock_gating_detected = 1'b1;
            else
                clock_gating_detected = 1'b0;
    end else
        always @*
            clock_gating_detected = 1'b0;
  endgenerate


  // a potential risk is if both RB/SB are assert, then SB deassert, stages
  // cannot be reset immediately, but at the next CK edge, because 
  // posedge SB is not in sensitive list.
  // this is fine because so far all real cell support only 1 type of clear
  // signal, SB or RB or FLUSH, the scenario above will never happen in real
  // world.
  // use generate to improve sim performance (is this reasonable?)
  initial begin 
    // if (rst_x_or_z)
    if (SB===1'bx | SB===1'bz | RB===1'bx | RB===1'bz | FLUSH===1'bx | FLUSH===1'bz)
          stages  <= {depth{1'bx}};
    else if (!SB)
          stages  <= {depth{1'b1}};
    else if (!RB)
          stages  <= {depth{1'b0}};
    else if (FLUSH)
          stages  <= {depth{D}};
  end

  // rst_x_or_z is necessary because:
  // 1. X can be reflected on stages/Q immediately when X happens on SB/RB/FLUSH
  // 2. for some corner cases. e.g. 0->x->1 on RB, while CK never toggle.
  //    X will not pass to Q without rst_x_or_z in sensitive list
  // 3. why use always@(negedge RB) but not always@(RB): posedge RB should not
  //    trigger stages update
  // clk_x_or_z is necessary because:
  // 1. corner cases. e.g. 1->X on CK
  generate
    if (HoldCheck)
        // always @( negedge SB or negedge RB or posedge FLUSH or posedge CK or posedge race_d_late or flush_d_toggle or posedge rst_x_or_z or posedge clk_x_or_z) begin : syncp
        always @( negedge SB or negedge RB or posedge FLUSH or posedge CK or flush_d_toggle or posedge rst_x_or_z or posedge clk_x_or_z) begin : syncp
                // if (rst_x_or_z)
                if (SB===1'bx | SB===1'bz | RB===1'bx | RB===1'bz | FLUSH===1'bx | FLUSH===1'bz)
                      stages  <= {depth{1'bx}};
                else if (!SB)
                      stages  <= {depth{1'b1}};
                else if (!RB)
                      stages  <= {depth{1'b0}};
                else if (FLUSH)
                      stages  <= {depth{D}};
                else if (clk_x_or_z)
                      stages  <= {depth{1'bx}};
                else if (EN & (clock_gating_detected===1'b0)) begin
                        if (holdcheck_assert && (stages[depth-1]^D)) // HC only works for D change happen before CK rising edge more than 1/2 cycle
                          stages  <= {D,stages_n[(depth-1):1]};
                        else begin
                            if (delaySyncP & (stages[(depth-1)] == stages[(depth-2)] | (stages[depth-1]^D)))
                      // (stages[depth-1]^D) is necessary for D/CK race
                      // (stages[(depth-1)] == stages[(depth-2)]) is aimed to
                      // generate glitch when short pulse D is even not sampled by
                      // CK
                              stages <= {~D,stages[(depth-1):1]};
                            else if (delaySyncP & (stages[depth-1]===1'bx))
                              stages <= {1'bx,stages[(depth-1):1]};
                            else
                              stages <= {D,stages[(depth-1):1]};
                        end
                end
                else;
        end
    else
        // always @( negedge SB or negedge RB or posedge FLUSH or posedge CK or posedge race_d_late or flush_d_toggle or posedge rst_x_or_z or posedge clk_x_or_z) begin : syncp
        always @( negedge SB or negedge RB or posedge FLUSH or posedge CK or flush_d_toggle or posedge rst_x_or_z or posedge clk_x_or_z) begin : syncp
                // if (rst_x_or_z)
                if (SB===1'bx | SB===1'bz | RB===1'bx | RB===1'bz | FLUSH===1'bx | FLUSH===1'bz)
                      stages  <= {depth{1'bx}};
                else if (!SB)
                      stages  <= {depth{1'b1}};
                else if (!RB)
                      stages  <= {depth{1'b0}};
                else if (FLUSH)
                      stages  <= {depth{D}};
                else if (clk_x_or_z)
                      stages  <= {depth{1'bx}};
                else if (EN & (clock_gating_detected===1'b0)) begin
                      // stages  <= ( delaySyncP && (stages[depth-1]^D) ) ? {~D,stages[(depth-1):1]} : {D,stages[(depth-1):1]};
                      if (delaySyncP & (stages[(depth-1)] == stages[(depth-2)] | (stages[depth-1]^D))) 
                      // (stages[depth-1]^D) is necessary for D/CK race
                      // (stages[(depth-1)] == stages[(depth-2)]) is aimed to
                      // generate glitch when short pulse D is even not sampled by
                      // CK
                        stages <= {~D,stages[(depth-1):1]};
                      else if (delaySyncP & (stages[depth-1]===1'bx))
                        stages <= {1'bx,stages[(depth-1):1]};
                      else
                        stages <= {D,stages[(depth-1):1]};
                end
                else;
        end
  endgenerate

  always @(negedge RB or negedge SB or posedge CK)
          if (!SB)
            stages_dft  <= {depth{1'b1}};
          else if (!RB)
            stages_dft  <= {depth{1'b0}};
          else
            stages_dft  <= {SI,stages_dft[(depth-1):1]};
  assign SO = stages_dft[0];



  ///  Finally, as our data flows to the end of our pipeline, we merely need to take the
  ///  last stage of our posedge pipeline and drive it out as our 'Q' output.

   assign  Q = SE===1'b1 ? SO : stages[0];


`ifdef ABV

////////////////////////////////////////////////////////////////////////////////
// Pulse Width Check for normal mode, MetaWindowMode not used

// This is RTL Simulation-ONLY logic used to model meta-stable events
//   If you get an ova compile error here, then you need to define ABV. This is not a bug!
// WARNING: For some paths it is probably not legal to disable this under NbIReset (i.e HT Config)


reg DelayFF, Dsync;
wire #(LocalMetaWindowSize) Ddelayed = D;

always @(D) 
   Dsync <= D; //transitions at the same time as randomExtraCycle

// Delay flip flop
always @(posedge CK) begin
   if(EN) DelayFF <= Dsync;
   `ifdef SYNC_WINDOW_DISPLAY_RANDOMIZATION
//om   if(MetWindowMode & RandomExtraCycle & EnableRand & (D_delayed ^ D))
   if(MetaWindowMode & delaySync & enableRand & (Ddelayed ^ D))
      $display("INFO : %m : Synchronizer Meta-stability Window hit at Time=%t", $time);
   `endif
end

always @(posedge EN) 
   DelayFF <= Dsync;

reg input_check;
wire meta_window_mode = MetaWindowMode ? 1'b1 : 1'b0;
initial
  input_check = 1'b0;                      // Initialize to 0 at time 0
always @(negedge CK)
    input_check <= InputCheck ? 1'b1 : 1'b0;  // parameters are 32bit, making it single bit for use 

reg InputChkDis_d1;
initial InputChkDis_d1 = InputChkDis;
always @(posedge CK)
    InputChkDis_d1 <= InputChkDis;

// om  `ifdef ABV
`ifndef REMOVE_OVL_NEVER_CDC_STDLIB_INTERNAL_SYCNFF_RAND_MODEL
ovl_never #(.msg("DataIn is changing too quickly for the synchronizer.")) Assert_SYNCFF (
    .clock (CK), .reset((RB===1'b1) & (SB===1'b1) & (FLUSH===1'b0)), .enable(input_check & ~meta_window_mode & (~InputChkDis & ~InputChkDis_d1) ), 
    .test_expr( (~((D === DelayFF) | (DelayFF === $past(DelayFF,1,,@(posedge CK))))) &
                 (DelayFF !== 1'bx) & (DelayFF !== 1'bz) & (Dsync !== 1'bx) & (Dsync !== 1'bz)), .fire());
`endif   // REMOVE_OVL_NEVER_CDC_STDLIB_INTERNAL_SYCNFF_RAND_MODEL
// om  `else
// om  // ova clock posedge CK;
// om  // ova forbid_bool((~((D === DelayFF) | (DelayFF === DelayFF[#1])) & (DelayFF !== 1'bx) & (Dsync !== 1'bx) & input_check & ~MetWindowMode), "SYNCFF Error: DataIn is changing too quickly for the synchronizer.");
// om  `endif

// Pulse Width Check for normal mode, MetaWindowMode not used
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Pulse Width Check for MetaWindowMode mode
//   - Data should be held long enough to hit 2 clock edges if shifted to cover
//     either destclk period to guarantee capture
////////////////////////////////////////////////////////////////////////////////


        // reg DataCaptured;
        // time lastCK, lastD, DestClockPeriod, DataPulseWidth, LastClockPeriod;

        initial begin
          DataPulseWidth = 0; DestClockPeriod = 0; LastClockPeriod = 0; lastCK = 0; lastD = 0; DataCaptured = 0;
        end

        always @(negedge InputChkDis) begin // reset
          DataPulseWidth = 0; DestClockPeriod = 0; LastClockPeriod = 0; lastCK = 0; lastD = 0; DataCaptured = 0;
        end

        ////////////////////////////////////////////////////////////////////////////////
        // ClockCheck - measures last clock period, then compares the most recent data
        //   against the last 2 clock periods. Data pulses smaller than either period 
        //   could potentially drift and be lost.
        // Disabled using InputCheck (this should NOT be disabled all the time!!!!!)
        ////////////////////////////////////////////////////////////////////////////////
        always @(posedge CK) begin : ClockCheck
            reg Last_Fail, This_Fail;
            if(meta_window_mode & input_check & ~InputChkDis & |lastCK) begin
                LastClockPeriod = DestClockPeriod;
                DestClockPeriod = $time - lastCK;
                if(DataPulseWidth) begin
                    Last_Fail = DataCaptured && (DataPulseWidth < (LastClockPeriod + LocalMetaWindowSize));
                    This_Fail = DataCaptured && (DataPulseWidth < (DestClockPeriod + LocalMetaWindowSize));
                    if(Last_Fail & This_Fail ) begin
                        $display("ERROR : %m : %0t D input pulse < %0t CLK period + %0t metastable window Time=%t", 
                            DataPulseWidth, DestClockPeriod, LocalMetaWindowSize,$time);
                        `ifndef SYNC_WINDOW_DISABLE_FATAL_ERRORS
                        $finish;
                        `endif
                    `ifndef SYNC_WINDOW_DISABLE_WARNINGS
                    end else if( Last_Fail ) begin
                        $display("WARNING : %m : %0t D input pulse < previous %0t CLK period + %0t metastable window Time=%t", 
                            DataPulseWidth, LastClockPeriod, LocalMetaWindowSize,$time);
                    end else if( This_Fail ) begin
                        $display("WARNING : %m : %0t D input pulse < %0t CLK period + %0t metastable window Time=%t", 
                            DataPulseWidth, DestClockPeriod, LocalMetaWindowSize,$time);
                    `endif
                    end
                end
                DataPulseWidth = 0; // last pulse checked
                lastCK = $time;
            end
        end

        ////////////////////////////////////////////////////////////////////////////////
        // DataCheck - measures data pulses, catches pulses smaller than a single clock
        // Disabled using InputCheck (this should NOT be disabled all the time!!!!!)
        ////////////////////////////////////////////////////////////////////////////////
        always @(D) begin : DataCheck
            if(meta_window_mode & input_check & ~InputChkDis) begin
                if(|lastCK & (($time - lastD) > 5)) begin //smooth out any data glitches and ignore check before 1st clock edge
                    if(DataPulseWidth) begin // no clock edge occurred in this pulse, still need to check last pulse
                        if(DataCaptured) begin // last pulse included a posedge clock, need to check it.
                            if(DataPulseWidth < (DestClockPeriod + LocalMetaWindowSize)) begin
                                $display("ERROR : %m : %0t D input pulse < %0t CLK period + %0t metastable window Time=%t", 
                                    DataPulseWidth, DestClockPeriod, LocalMetaWindowSize,$time);
                                `ifndef SYNC_WINDOW_DISABLE_FATAL_ERRORS
                                $finish;
                                `endif
                            end
                        end
                        `ifndef SYNC_WINDOW_DISABLE_WARNINGS
                        $display("WARNING : %m : %0t D pulse lost at Time=%t", DataPulseWidth,$time);
                        `endif
                    end
                    DataCaptured = (lastD < lastCK) & (Ddelayed !== 1'bx) & (D !== 1'bx);
                    DataPulseWidth = $time - lastD;
                end
                lastD = $time;
            end
        end

`endif // `ifdef ABV

////////////////////////////////////////////////////////////////////////////////
// End of Pulse Width Check for MetaWindowMode  mode
//   - Data should be held long enough to hit 2 clock edges if shifted to cover
//     either destclk period to guarantee capture


// leda on
// synopsys translate_on

endmodule

//! enable parser

///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
///
///     module:   'CDCGENERICSYNC'
///
///  This module implements the 'user' cell that is deployed into the RTL design for the
///  purposes of simulation. In order to effectively model the various physical effects
///  that can manifest in a real design, with real timing, the model has been behaviourally
///  enhanced relative to a simplistic pipeline model. In particular, the following
///  enhancements have been incorporated into the model behaviour:
///
///     o  transport delay modelling to the input of the cell [passed along as a timing constraint]
///     o  randomized synchronizer delay for every input transition seen
///          - multi-bit support to ensure aggressive behaviour, avoids erroneous event reordering
///     o  optional input pulse-width check
///     o  various cell-type features such as:
///          - inclusion of source-flop
///          - variable synchronizer depth [including N+1/2 depths]
///          - async reset/set/flush
///          - clock enable
///
///  Note, while it is valid for individual designers to use this cell directly from this
///  library, this module strictly a simulation model. There is no automated means to map
///  a direct instantiation of this model to a physical cell from the target technology
///  library.
///
///  Instead, a higher-level, technology-independent wrapper flow has been developed to
///  deploy the appropriate synchronizer cells into the design when the design is passed
///  through a synthesis tool chain. It is highly recommended that the higher-level wrapper
///  be the deployment method for all RTL deployments.
///
///  As should be evident from the coding of this module, in order to help us manage our
///  code better, and to compartmentalize specific modelling features, this model is
///  comprised of a cascade of features [as implemented in the previous modules in this
///  section of the cdc_stdlib library].
///
///   .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .

//! disable parser
module  CDCGENERICSYNC (

    SB,
    RB,
    FLUSH,
    CKsrc,
    CKdst,
    EN,
    
    `ifdef ABV
    InputChkDis,
    `endif

    Dsrc,
    Qsrc,
    Qdst,

    SI,
    SO,
    SE

  );

  /// .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .

  parameter   size            = 1;
  parameter   depth           = 2;
  parameter   sourceFlop      = 0;
  parameter   orBypass        = 0;

  parameter   TransportDelay  = 0;
  parameter   TransportCycle  = 1;
  parameter   InputCheck      = 1;
  parameter   MetaWindowMode  = 0;
  parameter   MetaWindowSize  = 0;

  parameter   halfStage       = 0;
  parameter   IgnorePulseNotSampled = 0;
  parameter   reduce_meta_pessimism = 0;
  parameter   HoldCheck = 0;
  parameter   DstClkIdleTime = 0;
  parameter   DstClkIdleCycles = 0;
  parameter   SrcPath = 0;

  //VCS coverage exclude_module
  /// .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .

  input                 SB;            ///  active-low, asynchronous set
  input                 RB;            ///  active-low, asynchronous reset
  input                 FLUSH;         ///  active-high, asynchronous flush
  input                 CKsrc;         ///  clock input (source domain)
  input                 CKdst;         ///  clock input (destination domain)
  input                 EN;            ///  clock enable

  `ifdef ABV
  input                 InputChkDis;   ///  disables input pulse-width checking 
  `endif

  input  [(size-1):0]   Dsrc;          ///  data input (source domain)
  output [(size-1):0]   Qsrc;          ///  data output (source domain)
  output [(size-1):0]   Qdst;          ///  synchronized data output

  // input  [(size-1):0]   SI;          ///  DFT data input
  // output [(size-1):0]   SO;          ///  DFT data output
  input     SI;          ///  DFT data input
  output    SO;          ///  DFT data output
  input                 SE;          ///  DFT enable
  /// .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .

  wire   [(size-1):0]  sourceData;
  wire                 transportCycle_assert;
  wire   [(size-1):0]  TC_holdcheck_assert;

  // leda off
  // synopsys translate_off
  // the parameter priority sequence is: sourceFlop > TC > TD. make assertion
  // check here for any possible override
`ifdef ABV
    `ifndef RTLLIB_TEMP_KEEP_LEGACY_PARAM
        initial begin
              if (sourceFlop && TransportCycle)
                  $display("Error: %m : parameter TransportCycle is overriden by sourceFlop. sourceFlop = %0t TC = %0t TD = %0t Time=%t", 
                  sourceFlop, TransportCycle, TransportDelay, $time);
              if (sourceFlop && TransportDelay)
                  $display("Error: %m : parameter TransportDelay is overriden by sourceFlop. sourceFlop = %0t TC = %0t TD = %0t Time=%t", 
                  sourceFlop, TransportCycle, TransportDelay, $time);
              if (TransportCycle && TransportDelay)
                  $display("Error: %m : parameter TransportDelay is overriden by TransportCycle. sourceFlop = %0t TC = %0t TD = %0t Time=%t", 
                  sourceFlop, TransportCycle, TransportDelay, $time);
              if (halfStage)
                  $display("Error: %m : parameter halfStage is removed, update your instance accordingly. halfStage = %0t Time=%t", halfStage, $time); 
              if (IgnorePulseNotSampled)
                  $display("Error: %m : parameter IgnorePulseNotSampled is removed, update your instance accordingly. IgnorePulseNotSampled = %0t Time=%t", IgnorePulseNotSampled, $time); 
          end
        always @(sourceFlop or TransportCycle or TransportDelay or halfStage or IgnorePulseNotSampled) begin
              if (sourceFlop && TransportCycle)
                  $display("Error: %m : parameter TransportCycle is overriden by sourceFlop. sourceFlop = %0t TC = %0t TD = %0t Time=%t", 
                  sourceFlop, TransportCycle, TransportDelay, $time);
              if (sourceFlop && TransportDelay)
                  $display("Error: %m : parameter TransportDelay is overriden by sourceFlop. sourceFlop = %0t TC = %0t TD = %0t Time=%t", 
                  sourceFlop, TransportCycle, TransportDelay, $time);
              if (TransportCycle && TransportDelay)
                  $display("Error: %m : parameter TransportDelay is overriden by TransportCycle. sourceFlop = %0t TC = %0t TD = %0t Time=%t", 
                  sourceFlop, TransportCycle, TransportDelay, $time);
              if (halfStage)
                  $display("Error: %m : parameter halfStage is removed, update your instance accordingly. halfStage = %0t Time=%t", halfStage, $time); 
              if (IgnorePulseNotSampled)
                  $display("Error: %m : parameter IgnorePulseNotSampled is removed, update your instance accordingly. IgnorePulseNotSampled = %0t Time=%t", IgnorePulseNotSampled, $time); 
          end
    `endif
`endif

reg SrcPathEn;
initial begin SrcPathEn = 1'b0; end
always @*
    if (!SrcPathEn&SrcPath) begin
        $display("Error: %m : parameter SrcPath should not be set 1 when internal register SrcPathEn is 0. Time=%t", $time);
    end

  /// .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
  ///
  ///  When we have a source-flop incorporated into the cell, the effects of transport
  ///  delay are unimportant to the behviour. Consequently, the mechanism for modelling
  ///  transport delay is unneccesary. To avoid impacting simulation performance, through
  ///  a generate foramt, we either include the source flop, or include the transport
  ///  delay modelling mechanism.

  generate
    if ( sourceFlop ) begin: SF_enable

      reg   [(size-1):0]         qSource;
      always @( posedge CKsrc )  qSource <= Dsrc;
      assign  sourceData = qSource;
      assign  Qsrc       = qSource;
      assign  TC_holdcheck_assert = {size{1'b0}};

    end else begin: transport_model

      wire  [(size-1):0]   dTrans;
      assign  sourceData = dTrans;
      assign  Qsrc       = 1'bx;

      cdc_stdlib_internal_sync_transport  #(

          .size            ( size           ),
          .InputCheck      ( InputCheck     ),
          .TransportDelay  ( TransportDelay ),
          .TransportCycle  ( TransportCycle ),
          .MetaWindowMode  ( MetaWindowMode ),
          .HoldCheck       ( HoldCheck      )

        )  syncTransport  (

          .CK              ( CKdst          ),
          .D               ( Dsrc           ),
          `ifdef ABV
          .InputChkDis     ( InputChkDis     ),
          `endif
          .transportCycle_assert (transportCycle_assert),
          .TC_holdcheck_assert ( TC_holdcheck_assert),
          .Dtransport      ( dTrans         )

        );

    end
  endgenerate

  /// .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .

  wire  [(size-1):0]  enableRand;
  wire  [(size-1):0]  randomExtraCycle;
  wire  [(size-1):0]  random_var;
  wire  [(size-1):0]  race_d_late;

  /// .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
  ///
  ///  As has been discussed, where we have multi-bit busses to synchronize, we need to
  ///  put in place a filter mechanism to ensure that we are not overly aggressive with
  ///  our randomization. In particular, we preserve the original event order based on
  ///  the assumption that the physical implementation will meet the following criteria:
  ///
  ///     the skew between all bits, in a synchronized bus, as they arrive at the
  ///     destination sampling stage must be less than one source clock period, margined
  ///     appropriately for clock skew.

`ifdef CDC_RTLLIB_DELTA_ENABLE

  cdc_stdlib_internal_multibit_sync_filter #(

      .size            ( size            ),
      .halfStage       ( halfStage       ),
      .MetaWindowMode  ( MetaWindowMode  ),
      .IgnorePulseNotSampled (IgnorePulseNotSampled )

    )  syncFilter  (

      .CK              ( CKdst           ),
      .sourceData      ( sourceData      ),
      .SB              ( SB             ),
      .RB              ( RB           ),
      .FLUSH           ( FLUSH           ),
      `ifdef ABV
      .InputChkDis     ( InputChkDis     ),
      `endif
//       .enableRand      ( enableRand      ),
      .enableRand      ( ),
      .enableRand_IgnorePulseNotSampled (enableRand), 
      .transportCycle_assert (transportCycle_assert)

    );

`else    

  cdc_stdlib_internal_multibit_sync_filter_nodelta #(

      .size            ( size            ),
      .MetaWindowMode  ( MetaWindowMode  ),
      .MetaWindowSize  ( MetaWindowSize  ),
      .reduce_meta_pessimism ( reduce_meta_pessimism )  

    )  syncFilter  (

      .CK              ( CKdst           ),
      .sourceData      ( sourceData      ),
      .SB              ( SB              ),
      .RB              ( RB              ),
      .FLUSH           ( FLUSH           ),
      `ifdef ABV
      .InputChkDis     ( InputChkDis     ),
      `endif
      .randomExtraCycle( randomExtraCycle),
      .random_var      ( random_var      ),
      .race_d_late     ( race_d_late     )
    );

`endif

  /// .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
  ///
  ///  Finally, we incorporate the actual state-point modelling, complete with the final
  ///  synchronization randomization decision. In the interest of making the model truly
  ///  generic, the state-point model also supports a number of optional features,
  ///  including [as mentioned elsewhere]:
  ///
  ///     o  inclusion of a source flop at the head of the chain
  ///     o  variable depth [including N+1/2 flop-stage equivalent]
  ///     o  asynchronous reset/set/flush
  ///     o  clock enable
  
`ifdef CDC_RTLLIB_DELTA_ENABLE

  cdc_stdlib_internal_sync_model  #(

      .depth           ( depth           ),
      .halfStage       ( halfStage       ),
      .InputCheck      ( InputCheck      ),
      .MetaWindowMode  ( MetaWindowMode  )
  

    )  syncCell[(size-1):0]  (

      .SB              ( SB             ),
      .RB              ( RB           ),
      .FLUSH           ( FLUSH           ),
      .CK              ( CKdst           ),
      .EN              ( EN              ),

      .enableRand      ( enableRand      ),
      `ifdef ABV
      .InputChkDis     ( InputChkDis     ),
      `endif

      .D               ( sourceData      ),
      .Q               ( Qdst            )

    );

`else

  cdc_stdlib_internal_sync_model_nodelta  #(

      .depth           ( depth           ),
      .InputCheck      ( InputCheck      ),
      .MetaWindowMode  ( MetaWindowMode  ),
      .MetaWindowSize  ( MetaWindowSize  ),
      .HoldCheck       ( HoldCheck       ),
      .DstClkIdleTime  ( DstClkIdleTime  ),
      .DstClkIdleCycles( DstClkIdleCycles)
  

    )  syncCell[(size-1):0]  (

      .SB              ( SB              ),
      .RB              ( RB              ),
      .FLUSH           ( FLUSH           ),
      .CK              ( CKdst           ),
      .EN              ( EN              ),

      .randomExtraCycle( randomExtraCycle),
      .random_var      ( random_var      ),
      .race_d_late     ( race_d_late     ),
      .transportCycle_assert ( transportCycle_assert ),
      .TC_holdcheck_assert (TC_holdcheck_assert ), 
      `ifdef ABV
      .InputChkDis     ( InputChkDis     ),
      `endif

      .D               ( sourceData      ),
      .Q               ( Qdst            ), 

      .SI              ( SI              ),
      .SO              ( SO              ),
      .SE              ( SE              )

    );

`endif

// leda on
// synopsys translate_on

endmodule

//! enable parser

/// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////


