`include "grbm_defs.vh"
`include "gc__didt_reg.v"
`include "gc_register_addr.v"

//;my  = 4 * 10 * 2;


module gc_didt_regs
(
   input 								   sclk,
   input                                   resetb,
   input                                   reg_clk,
   input                                   reg_send,
   input                                   reg_op,
   input [`GRBM_TARG_REG_ADDR_RANGE]       reg_addr,
   input [`GRBM_TARG_REG_WD_RANGE]         reg_wd,

   output wire             didt_SQ_didt_ctrl_en,
   output wire [1:0]       didt_SQ_phase_offset,
   output wire             didt_SQ_didt_ctrl_rst,
   output wire             didt_SQ_didt_clk_en_override,
   output wire [5:0]       didt_SQ_didt_max_stalls_allowed_hi,
   output wire [5:0]       didt_SQ_didt_max_stalls_allowed_lo,
   output wire [0:0]       didt_SQ_stall_event_enable,
   output wire [0:0]       didt_SQ_stall_event_counter_clear,
   output wire [0:0]       didt_SQ_didt_rlc_force_stall_en,
   output wire [0:0]       didt_SQ_didt_rlc_stall_level_sel,
   output wire			   didt_SQ_didt_throttle_mode,
   input wire [31:0]	   didt_SQ_stall_event_counter,

                                      // DIDT_SQ_CTRL1
   output wire [15:0]      didt_SQ_min_power,
   output wire [15:0]      didt_SQ_max_power,

                                      // DIDT_SQ_CTRL2
   output wire [13:0]      didt_SQ_max_power_delta,
   output wire [9:0]       didt_SQ_short_term_interval_size,
   output wire [3:0]       didt_SQ_long_term_interval_ratio,

   output wire [15:0]	   didt_SQ_ocp_max_power,

									  // DIDT_SQ_STALL_CTRL
   output wire [0:0]	   didt_SQ_stall_ctrl_enable,
   output wire [5:0]	   didt_SQ_stall_delay_hi,
   output wire [5:0]	   didt_SQ_stall_delay_lo,
   output wire [15:0]	   didt_SQ_hi_power_threshold,
									//DIDT_SQ_TUNING_CTRL
   output wire [0:0]       didt_SQ_tuning_enable,
   output wire [13:0]      didt_SQ_max_power_delta_hi,
   output wire [13:0]      didt_SQ_max_power_delta_lo,
   output wire [0:0]       didt_SQ_auto_mpd_enable,
   									//DIDT_SQ_STALL_AUTO_RELEASE_CTRL
   output wire [0:0]       didt_SQ_stall_auto_release_enable,
   output wire [23:0]      didt_SQ_stall_auto_release_time,

   output wire [3:0]       didt_SQ_mpd_ratio_scale_level1,
   output wire [3:0]       didt_SQ_mpd_ratio_scale_level2,
   output wire [3:0]       didt_SQ_mpd_ratio_scale_level3,
   output wire [3:0]       didt_SQ_mpd_ratio_scale_level4,
   output wire [3:0]       didt_SQ_mpd_scale_level0,
   output wire [3:0]       didt_SQ_mpd_scale_level1,
   output wire [3:0]       didt_SQ_mpd_scale_level2,
   output wire [3:0]       didt_SQ_mpd_scale_level3,

                                    //DIDT_SQ_STALL_RELEASE_CNTL0
   output wire [0:0]       didt_SQ_didt_stall_release_cntl_en,
   output wire [0:0]       didt_SQ_didt_stall_cntl_sel,
   output wire [10:0]      didt_SQ_didt_release_delay_hi,
   output wire [10:0]      didt_SQ_didt_release_delay_lo,

                                    //DIDT_SQ_STALL_RELEASE_CNTL1
   output wire [4:0]       didt_SQ_didt_base_release_allowed_hi,
   output wire [4:0]       didt_SQ_didt_incr_release_allowed_hi,
   output wire [4:0]       didt_SQ_didt_base_release_allowed_lo,
   output wire [4:0]       didt_SQ_didt_incr_release_allowed_lo,

   input wire  [1:0]       didt_SQ_didt_stall_release_cntl_fsm_state,

                                      // DIDT_SQ_WEIGHT0_3
   output wire [7:0]       didt_SQ_weight0,
   output wire [7:0]       didt_SQ_weight1,
   output wire [7:0]       didt_SQ_weight2,
   output wire [7:0]       didt_SQ_weight3,

                                      // DIDT_SQ_WEIGHT4_7
   output wire [7:0]       didt_SQ_weight4,
   output wire [7:0]       didt_SQ_weight5,
   output wire [7:0]       didt_SQ_weight6,
   output wire [7:0]       didt_SQ_weight7,

                                      // DIDT_SQ_WEIGHT8_11
   output wire [7:0]       didt_SQ_weight8,
   output wire [7:0]       didt_SQ_weight9,
   output wire [7:0]       didt_SQ_weight10,
   output wire [7:0]       didt_SQ_weight11,
   
	//New DIDT
	output wire             didt_SQ_gc_didt_enable,
	output wire				didt_SQ_gc_didt_clk_en_override,
	output wire [1:0]		didt_SQ_throttle_policy,
	output wire [4:0]		didt_SQ_didt_trigger_throttle_lowbit,
	output wire [4:0]		didt_SQ_didt_power_level_lowbit,
	output wire [7:0]		didt_SQ_didt_stall_pattern_bit_nums,
	output wire [0:0]		didt_SQ_gc_didt_level_comb_en,
	output wire [0:0]		didt_SQ_se_didt_level_comb_en,
	output wire [0:0]		didt_SQ_qualify_stall_en,
	output wire [1:0]		didt_SQ_didt_stall_sel,
    output wire             didt_SQ_didt_force_stall,
    output wire             didt_SQ_didt_stall_delay_en,
	output wire[14:0]		didt_SQ_stall_pattern1,
    output wire[14:0]		didt_SQ_stall_pattern2,
    output wire[14:0]		didt_SQ_stall_pattern3,
    output wire[14:0]		didt_SQ_stall_pattern4,
    output wire[14:0]		didt_SQ_stall_pattern5,
    output wire[14:0]		didt_SQ_stall_pattern6,
    output wire[14:0]		didt_SQ_stall_pattern7,


  output wire				SQ_edc_en,
  output wire				SQ_edc_sw_rst,
  output wire				SQ_edc_clk_en_override,
  output wire				SQ_edc_force_stall,
  output wire[31:0]			SQ_edc_threshold,
  output wire[4:0]			SQ_edc_trigger_throttle_lowbit,
  output wire[7:0]			SQ_edc_stall_pattern_bit_nums,
  output wire 				SQ_edc_allow_write_pwrdelta,
  output wire				SQ_edc_grbm_clear_overflow_counter,
  output wire[14:0]			SQ_edc_stall_pattern1,
  output wire[14:0]			SQ_edc_stall_pattern2,
  output wire[14:0]			SQ_edc_stall_pattern3,
  output wire[14:0]			SQ_edc_stall_pattern4,
  output wire[14:0]			SQ_edc_stall_pattern5,
  output wire[14:0]			SQ_edc_stall_pattern6,
  output wire[14:0]			SQ_edc_stall_pattern7,
  output wire[13:0]                     SQ_edc_timer_period,
  output wire[`DIDT_SQ_THROTTLE_CTRL_GC_EDC_STALL_EN_SIZE-1:0]		SQ_gc_edc_stall_en,
  output wire[`DIDT_SQ_THROTTLE_CTRL_PCC_STALL_EN_SIZE-1:0]			SQ_pcc_stall_en,
  output wire[`DIDT_SQ_THROTTLE_CTRL_PWRBRK_STALL_EN_SIZE-1:0]		SQ_pwrbrk_stall_en,
  output wire[`DIDT_SQ_THROTTLE_CTRL_GC_EDC_ONLY_MODE_SIZE-1:0]		SQ_gc_edc_only_mode,

  input wire[31:0]			SQ_edc_pcc_perf_counter,
  output wire				rd_SQ_edc_pcc_perf_counter,
  input wire				SQ_edc_fsm_state,
  input wire[2:0]			SQ_edc_throttle_level,
  output wire[6-1:0]			SQ_edc_stall_delay_sig0,
  output wire[6-1:0]			SQ_edc_stall_delay_sig1,
  output wire[6-1:0]			SQ_edc_stall_delay_sig2,
  output wire[6-1:0]			SQ_edc_stall_delay_sig3,
  output wire[6-1:0]			SQ_edc_stall_delay_sig4,
  output wire[6-1:0]			SQ_edc_stall_delay_sig5,
  output wire[6-1:0]			SQ_edc_stall_delay_sig6,
  output wire[6-1:0]			SQ_edc_stall_delay_sig7,
  output wire[6-1:0]			SQ_edc_stall_delay_sig8,
  output wire[6-1:0]			SQ_edc_stall_delay_sig9,
  input wire[15:0]			SQ_edc_throttle_level_overflow_counter,
  input wire				SQ_edc_rolling_power_delta_overflow,
  input wire[31:0]			SQ_edc_rolling_power_delta,
  output wire[31:0]			SQ_edc_grbm_wr_pwrdelta_data,
  output wire				SQ_edc_grbm_wr_pwrdelta_en,
  output wire				SQ_edc_grbm_clear_rolling_power_delta_ov,
  output wire				SQ_gc_edc_en,
  output wire [1:0]			SQ_gc_edc_stall_policy,
  output wire [0:0]			SQ_gc_edc_level_comb_en,
  output wire [0:0]			SQ_se_edc_level_comb_en,
  output wire [0:0]			SQ_edc_level_mode_sel,
  output wire [0:0]			SQ_edc_level_comb_adapt_mode_en,

   output wire             didt_DB_didt_ctrl_en,
   output wire [1:0]       didt_DB_phase_offset,
   output wire             didt_DB_didt_ctrl_rst,
   output wire             didt_DB_didt_clk_en_override,
   output wire [5:0]       didt_DB_didt_max_stalls_allowed_hi,
   output wire [5:0]       didt_DB_didt_max_stalls_allowed_lo,
   output wire [0:0]       didt_DB_stall_event_enable,
   output wire [0:0]       didt_DB_stall_event_counter_clear,
   output wire [0:0]       didt_DB_didt_rlc_force_stall_en,
   output wire [0:0]       didt_DB_didt_rlc_stall_level_sel,
   output wire			   didt_DB_didt_throttle_mode,
   input wire [31:0]	   didt_DB_stall_event_counter,

                                      // DIDT_DB_CTRL1
   output wire [15:0]      didt_DB_min_power,
   output wire [15:0]      didt_DB_max_power,

                                      // DIDT_DB_CTRL2
   output wire [13:0]      didt_DB_max_power_delta,
   output wire [9:0]       didt_DB_short_term_interval_size,
   output wire [3:0]       didt_DB_long_term_interval_ratio,

   output wire [15:0]	   didt_DB_ocp_max_power,

									  // DIDT_DB_STALL_CTRL
   output wire [0:0]	   didt_DB_stall_ctrl_enable,
   output wire [5:0]	   didt_DB_stall_delay_hi,
   output wire [5:0]	   didt_DB_stall_delay_lo,
   output wire [15:0]	   didt_DB_hi_power_threshold,
									//DIDT_DB_TUNING_CTRL
   output wire [0:0]       didt_DB_tuning_enable,
   output wire [13:0]      didt_DB_max_power_delta_hi,
   output wire [13:0]      didt_DB_max_power_delta_lo,
   output wire [0:0]       didt_DB_auto_mpd_enable,
   									//DIDT_DB_STALL_AUTO_RELEASE_CTRL
   output wire [0:0]       didt_DB_stall_auto_release_enable,
   output wire [23:0]      didt_DB_stall_auto_release_time,

   output wire [3:0]       didt_DB_mpd_ratio_scale_level1,
   output wire [3:0]       didt_DB_mpd_ratio_scale_level2,
   output wire [3:0]       didt_DB_mpd_ratio_scale_level3,
   output wire [3:0]       didt_DB_mpd_ratio_scale_level4,
   output wire [3:0]       didt_DB_mpd_scale_level0,
   output wire [3:0]       didt_DB_mpd_scale_level1,
   output wire [3:0]       didt_DB_mpd_scale_level2,
   output wire [3:0]       didt_DB_mpd_scale_level3,

                                    //DIDT_DB_STALL_RELEASE_CNTL0
   output wire [0:0]       didt_DB_didt_stall_release_cntl_en,
   output wire [0:0]       didt_DB_didt_stall_cntl_sel,
   output wire [10:0]      didt_DB_didt_release_delay_hi,
   output wire [10:0]      didt_DB_didt_release_delay_lo,

                                    //DIDT_DB_STALL_RELEASE_CNTL1
   output wire [4:0]       didt_DB_didt_base_release_allowed_hi,
   output wire [4:0]       didt_DB_didt_incr_release_allowed_hi,
   output wire [4:0]       didt_DB_didt_base_release_allowed_lo,
   output wire [4:0]       didt_DB_didt_incr_release_allowed_lo,

   input wire  [1:0]       didt_DB_didt_stall_release_cntl_fsm_state,

                                      // DIDT_DB_WEIGHT0_3
   output wire [7:0]       didt_DB_weight0,
   output wire [7:0]       didt_DB_weight1,
   output wire [7:0]       didt_DB_weight2,
   output wire [7:0]       didt_DB_weight3,

                                      // DIDT_DB_WEIGHT4_7
   output wire [7:0]       didt_DB_weight4,
   output wire [7:0]       didt_DB_weight5,
   output wire [7:0]       didt_DB_weight6,
   output wire [7:0]       didt_DB_weight7,

                                      // DIDT_DB_WEIGHT8_11
   output wire [7:0]       didt_DB_weight8,
   output wire [7:0]       didt_DB_weight9,
   output wire [7:0]       didt_DB_weight10,
   output wire [7:0]       didt_DB_weight11,
   
	//New DIDT
	output wire             didt_DB_gc_didt_enable,
	output wire				didt_DB_gc_didt_clk_en_override,
	output wire [1:0]		didt_DB_throttle_policy,
	output wire [4:0]		didt_DB_didt_trigger_throttle_lowbit,
	output wire [4:0]		didt_DB_didt_power_level_lowbit,
	output wire [7:0]		didt_DB_didt_stall_pattern_bit_nums,
	output wire [0:0]		didt_DB_gc_didt_level_comb_en,
	output wire [0:0]		didt_DB_se_didt_level_comb_en,
	output wire [0:0]		didt_DB_qualify_stall_en,
	output wire [1:0]		didt_DB_didt_stall_sel,
    output wire             didt_DB_didt_force_stall,
    output wire             didt_DB_didt_stall_delay_en,
	output wire[14:0]		didt_DB_stall_pattern1,
    output wire[14:0]		didt_DB_stall_pattern2,
    output wire[14:0]		didt_DB_stall_pattern3,
    output wire[14:0]		didt_DB_stall_pattern4,
    output wire[14:0]		didt_DB_stall_pattern5,
    output wire[14:0]		didt_DB_stall_pattern6,
    output wire[14:0]		didt_DB_stall_pattern7,


  output wire				DB_edc_en,
  output wire				DB_edc_sw_rst,
  output wire				DB_edc_clk_en_override,
  output wire				DB_edc_force_stall,
  output wire[31:0]			DB_edc_threshold,
  output wire[4:0]			DB_edc_trigger_throttle_lowbit,
  output wire[7:0]			DB_edc_stall_pattern_bit_nums,
  output wire 				DB_edc_allow_write_pwrdelta,
  output wire				DB_edc_grbm_clear_overflow_counter,
  output wire[14:0]			DB_edc_stall_pattern1,
  output wire[14:0]			DB_edc_stall_pattern2,
  output wire[14:0]			DB_edc_stall_pattern3,
  output wire[14:0]			DB_edc_stall_pattern4,
  output wire[14:0]			DB_edc_stall_pattern5,
  output wire[14:0]			DB_edc_stall_pattern6,
  output wire[14:0]			DB_edc_stall_pattern7,
  output wire[13:0]                     DB_edc_timer_period,
  output wire[`DIDT_DB_THROTTLE_CTRL_GC_EDC_STALL_EN_SIZE-1:0]		DB_gc_edc_stall_en,
  output wire[`DIDT_DB_THROTTLE_CTRL_PCC_STALL_EN_SIZE-1:0]			DB_pcc_stall_en,
  output wire[`DIDT_DB_THROTTLE_CTRL_PWRBRK_STALL_EN_SIZE-1:0]		DB_pwrbrk_stall_en,
  output wire[`DIDT_DB_THROTTLE_CTRL_GC_EDC_ONLY_MODE_SIZE-1:0]		DB_gc_edc_only_mode,

  input wire[31:0]			DB_edc_pcc_perf_counter,
  output wire				rd_DB_edc_pcc_perf_counter,
  input wire				DB_edc_fsm_state,
  input wire[2:0]			DB_edc_throttle_level,
  output wire[5-1:0]			DB_edc_stall_delay_sig0,
  output wire[5-1:0]			DB_edc_stall_delay_sig1,
  output wire[5-1:0]			DB_edc_stall_delay_sig2,
  output wire[5-1:0]			DB_edc_stall_delay_sig3,
  input wire[15:0]			DB_edc_throttle_level_overflow_counter,
  input wire				DB_edc_rolling_power_delta_overflow,
  input wire[31:0]			DB_edc_rolling_power_delta,
  output wire[31:0]			DB_edc_grbm_wr_pwrdelta_data,
  output wire				DB_edc_grbm_wr_pwrdelta_en,
  output wire				DB_edc_grbm_clear_rolling_power_delta_ov,
  output wire				DB_gc_edc_en,
  output wire [1:0]			DB_gc_edc_stall_policy,
  output wire [0:0]			DB_gc_edc_level_comb_en,
  output wire [0:0]			DB_se_edc_level_comb_en,
  output wire [0:0]			DB_edc_level_mode_sel,
  output wire [0:0]			DB_edc_level_comb_adapt_mode_en,

   output wire             didt_TD_didt_ctrl_en,
   output wire [1:0]       didt_TD_phase_offset,
   output wire             didt_TD_didt_ctrl_rst,
   output wire             didt_TD_didt_clk_en_override,
   output wire [5:0]       didt_TD_didt_max_stalls_allowed_hi,
   output wire [5:0]       didt_TD_didt_max_stalls_allowed_lo,
   output wire [0:0]       didt_TD_stall_event_enable,
   output wire [0:0]       didt_TD_stall_event_counter_clear,
   output wire [0:0]       didt_TD_didt_rlc_force_stall_en,
   output wire [0:0]       didt_TD_didt_rlc_stall_level_sel,
   output wire			   didt_TD_didt_throttle_mode,
   input wire [31:0]	   didt_TD_stall_event_counter,

                                      // DIDT_TD_CTRL1
   output wire [15:0]      didt_TD_min_power,
   output wire [15:0]      didt_TD_max_power,

                                      // DIDT_TD_CTRL2
   output wire [13:0]      didt_TD_max_power_delta,
   output wire [9:0]       didt_TD_short_term_interval_size,
   output wire [3:0]       didt_TD_long_term_interval_ratio,

   output wire [15:0]	   didt_TD_ocp_max_power,

									  // DIDT_TD_STALL_CTRL
   output wire [0:0]	   didt_TD_stall_ctrl_enable,
   output wire [5:0]	   didt_TD_stall_delay_hi,
   output wire [5:0]	   didt_TD_stall_delay_lo,
   output wire [15:0]	   didt_TD_hi_power_threshold,
									//DIDT_TD_TUNING_CTRL
   output wire [0:0]       didt_TD_tuning_enable,
   output wire [13:0]      didt_TD_max_power_delta_hi,
   output wire [13:0]      didt_TD_max_power_delta_lo,
   output wire [0:0]       didt_TD_auto_mpd_enable,
   									//DIDT_TD_STALL_AUTO_RELEASE_CTRL
   output wire [0:0]       didt_TD_stall_auto_release_enable,
   output wire [23:0]      didt_TD_stall_auto_release_time,

   output wire [3:0]       didt_TD_mpd_ratio_scale_level1,
   output wire [3:0]       didt_TD_mpd_ratio_scale_level2,
   output wire [3:0]       didt_TD_mpd_ratio_scale_level3,
   output wire [3:0]       didt_TD_mpd_ratio_scale_level4,
   output wire [3:0]       didt_TD_mpd_scale_level0,
   output wire [3:0]       didt_TD_mpd_scale_level1,
   output wire [3:0]       didt_TD_mpd_scale_level2,
   output wire [3:0]       didt_TD_mpd_scale_level3,

                                    //DIDT_TD_STALL_RELEASE_CNTL0
   output wire [0:0]       didt_TD_didt_stall_release_cntl_en,
   output wire [0:0]       didt_TD_didt_stall_cntl_sel,
   output wire [10:0]      didt_TD_didt_release_delay_hi,
   output wire [10:0]      didt_TD_didt_release_delay_lo,

                                    //DIDT_TD_STALL_RELEASE_CNTL1
   output wire [4:0]       didt_TD_didt_base_release_allowed_hi,
   output wire [4:0]       didt_TD_didt_incr_release_allowed_hi,
   output wire [4:0]       didt_TD_didt_base_release_allowed_lo,
   output wire [4:0]       didt_TD_didt_incr_release_allowed_lo,

   input wire  [1:0]       didt_TD_didt_stall_release_cntl_fsm_state,

                                      // DIDT_TD_WEIGHT0_3
   output wire [7:0]       didt_TD_weight0,
   output wire [7:0]       didt_TD_weight1,
   output wire [7:0]       didt_TD_weight2,
   output wire [7:0]       didt_TD_weight3,

                                      // DIDT_TD_WEIGHT4_7
   output wire [7:0]       didt_TD_weight4,
   output wire [7:0]       didt_TD_weight5,
   output wire [7:0]       didt_TD_weight6,
   output wire [7:0]       didt_TD_weight7,

                                      // DIDT_TD_WEIGHT8_11
   output wire [7:0]       didt_TD_weight8,
   output wire [7:0]       didt_TD_weight9,
   output wire [7:0]       didt_TD_weight10,
   output wire [7:0]       didt_TD_weight11,
   
	//New DIDT
	output wire             didt_TD_gc_didt_enable,
	output wire				didt_TD_gc_didt_clk_en_override,
	output wire [1:0]		didt_TD_throttle_policy,
	output wire [4:0]		didt_TD_didt_trigger_throttle_lowbit,
	output wire [4:0]		didt_TD_didt_power_level_lowbit,
	output wire [7:0]		didt_TD_didt_stall_pattern_bit_nums,
	output wire [0:0]		didt_TD_gc_didt_level_comb_en,
	output wire [0:0]		didt_TD_se_didt_level_comb_en,
	output wire [0:0]		didt_TD_qualify_stall_en,
	output wire [1:0]		didt_TD_didt_stall_sel,
    output wire             didt_TD_didt_force_stall,
    output wire             didt_TD_didt_stall_delay_en,
	output wire[14:0]		didt_TD_stall_pattern1,
    output wire[14:0]		didt_TD_stall_pattern2,
    output wire[14:0]		didt_TD_stall_pattern3,
    output wire[14:0]		didt_TD_stall_pattern4,
    output wire[14:0]		didt_TD_stall_pattern5,
    output wire[14:0]		didt_TD_stall_pattern6,
    output wire[14:0]		didt_TD_stall_pattern7,


  output wire				TD_edc_en,
  output wire				TD_edc_sw_rst,
  output wire				TD_edc_clk_en_override,
  output wire				TD_edc_force_stall,
  output wire[31:0]			TD_edc_threshold,
  output wire[4:0]			TD_edc_trigger_throttle_lowbit,
  output wire[7:0]			TD_edc_stall_pattern_bit_nums,
  output wire 				TD_edc_allow_write_pwrdelta,
  output wire				TD_edc_grbm_clear_overflow_counter,
  output wire[14:0]			TD_edc_stall_pattern1,
  output wire[14:0]			TD_edc_stall_pattern2,
  output wire[14:0]			TD_edc_stall_pattern3,
  output wire[14:0]			TD_edc_stall_pattern4,
  output wire[14:0]			TD_edc_stall_pattern5,
  output wire[14:0]			TD_edc_stall_pattern6,
  output wire[14:0]			TD_edc_stall_pattern7,
  output wire[13:0]                     TD_edc_timer_period,
  output wire[`DIDT_TD_THROTTLE_CTRL_GC_EDC_STALL_EN_SIZE-1:0]		TD_gc_edc_stall_en,
  output wire[`DIDT_TD_THROTTLE_CTRL_PCC_STALL_EN_SIZE-1:0]			TD_pcc_stall_en,
  output wire[`DIDT_TD_THROTTLE_CTRL_PWRBRK_STALL_EN_SIZE-1:0]		TD_pwrbrk_stall_en,
  output wire[`DIDT_TD_THROTTLE_CTRL_GC_EDC_ONLY_MODE_SIZE-1:0]		TD_gc_edc_only_mode,

  input wire[31:0]			TD_edc_pcc_perf_counter,
  output wire				rd_TD_edc_pcc_perf_counter,
  input wire				TD_edc_fsm_state,
  input wire[2:0]			TD_edc_throttle_level,
  output wire[6-1:0]			TD_edc_stall_delay_sig0,
  output wire[6-1:0]			TD_edc_stall_delay_sig1,
  output wire[6-1:0]			TD_edc_stall_delay_sig2,
  output wire[6-1:0]			TD_edc_stall_delay_sig3,
  output wire[6-1:0]			TD_edc_stall_delay_sig4,
  output wire[6-1:0]			TD_edc_stall_delay_sig5,
  output wire[6-1:0]			TD_edc_stall_delay_sig6,
  output wire[6-1:0]			TD_edc_stall_delay_sig7,
  output wire[6-1:0]			TD_edc_stall_delay_sig8,
  output wire[6-1:0]			TD_edc_stall_delay_sig9,
  input wire[15:0]			TD_edc_throttle_level_overflow_counter,
  input wire				TD_edc_rolling_power_delta_overflow,
  input wire[31:0]			TD_edc_rolling_power_delta,
  output wire[31:0]			TD_edc_grbm_wr_pwrdelta_data,
  output wire				TD_edc_grbm_wr_pwrdelta_en,
  output wire				TD_edc_grbm_clear_rolling_power_delta_ov,
  output wire				TD_gc_edc_en,
  output wire [1:0]			TD_gc_edc_stall_policy,
  output wire [0:0]			TD_gc_edc_level_comb_en,
  output wire [0:0]			TD_se_edc_level_comb_en,
  output wire [0:0]			TD_edc_level_mode_sel,
  output wire [0:0]			TD_edc_level_comb_adapt_mode_en,

   output wire             didt_TCP_didt_ctrl_en,
   output wire [1:0]       didt_TCP_phase_offset,
   output wire             didt_TCP_didt_ctrl_rst,
   output wire             didt_TCP_didt_clk_en_override,
   output wire [5:0]       didt_TCP_didt_max_stalls_allowed_hi,
   output wire [5:0]       didt_TCP_didt_max_stalls_allowed_lo,
   output wire [0:0]       didt_TCP_stall_event_enable,
   output wire [0:0]       didt_TCP_stall_event_counter_clear,
   output wire [0:0]       didt_TCP_didt_rlc_force_stall_en,
   output wire [0:0]       didt_TCP_didt_rlc_stall_level_sel,
   output wire			   didt_TCP_didt_throttle_mode,
   input wire [31:0]	   didt_TCP_stall_event_counter,

                                      // DIDT_TCP_CTRL1
   output wire [15:0]      didt_TCP_min_power,
   output wire [15:0]      didt_TCP_max_power,

                                      // DIDT_TCP_CTRL2
   output wire [13:0]      didt_TCP_max_power_delta,
   output wire [9:0]       didt_TCP_short_term_interval_size,
   output wire [3:0]       didt_TCP_long_term_interval_ratio,

   output wire [15:0]	   didt_TCP_ocp_max_power,

									  // DIDT_TCP_STALL_CTRL
   output wire [0:0]	   didt_TCP_stall_ctrl_enable,
   output wire [5:0]	   didt_TCP_stall_delay_hi,
   output wire [5:0]	   didt_TCP_stall_delay_lo,
   output wire [15:0]	   didt_TCP_hi_power_threshold,
									//DIDT_TCP_TUNING_CTRL
   output wire [0:0]       didt_TCP_tuning_enable,
   output wire [13:0]      didt_TCP_max_power_delta_hi,
   output wire [13:0]      didt_TCP_max_power_delta_lo,
   output wire [0:0]       didt_TCP_auto_mpd_enable,
   									//DIDT_TCP_STALL_AUTO_RELEASE_CTRL
   output wire [0:0]       didt_TCP_stall_auto_release_enable,
   output wire [23:0]      didt_TCP_stall_auto_release_time,

   output wire [3:0]       didt_TCP_mpd_ratio_scale_level1,
   output wire [3:0]       didt_TCP_mpd_ratio_scale_level2,
   output wire [3:0]       didt_TCP_mpd_ratio_scale_level3,
   output wire [3:0]       didt_TCP_mpd_ratio_scale_level4,
   output wire [3:0]       didt_TCP_mpd_scale_level0,
   output wire [3:0]       didt_TCP_mpd_scale_level1,
   output wire [3:0]       didt_TCP_mpd_scale_level2,
   output wire [3:0]       didt_TCP_mpd_scale_level3,

                                    //DIDT_TCP_STALL_RELEASE_CNTL0
   output wire [0:0]       didt_TCP_didt_stall_release_cntl_en,
   output wire [0:0]       didt_TCP_didt_stall_cntl_sel,
   output wire [10:0]      didt_TCP_didt_release_delay_hi,
   output wire [10:0]      didt_TCP_didt_release_delay_lo,

                                    //DIDT_TCP_STALL_RELEASE_CNTL1
   output wire [4:0]       didt_TCP_didt_base_release_allowed_hi,
   output wire [4:0]       didt_TCP_didt_incr_release_allowed_hi,
   output wire [4:0]       didt_TCP_didt_base_release_allowed_lo,
   output wire [4:0]       didt_TCP_didt_incr_release_allowed_lo,

   input wire  [1:0]       didt_TCP_didt_stall_release_cntl_fsm_state,

                                      // DIDT_TCP_WEIGHT0_3
   output wire [7:0]       didt_TCP_weight0,
   output wire [7:0]       didt_TCP_weight1,
   output wire [7:0]       didt_TCP_weight2,
   output wire [7:0]       didt_TCP_weight3,

                                      // DIDT_TCP_WEIGHT4_7
   output wire [7:0]       didt_TCP_weight4,
   output wire [7:0]       didt_TCP_weight5,
   output wire [7:0]       didt_TCP_weight6,
   output wire [7:0]       didt_TCP_weight7,

                                      // DIDT_TCP_WEIGHT8_11
   output wire [7:0]       didt_TCP_weight8,
   output wire [7:0]       didt_TCP_weight9,
   output wire [7:0]       didt_TCP_weight10,
   output wire [7:0]       didt_TCP_weight11,
   
	//New DIDT
	output wire             didt_TCP_gc_didt_enable,
	output wire				didt_TCP_gc_didt_clk_en_override,
	output wire [1:0]		didt_TCP_throttle_policy,
	output wire [4:0]		didt_TCP_didt_trigger_throttle_lowbit,
	output wire [4:0]		didt_TCP_didt_power_level_lowbit,
	output wire [7:0]		didt_TCP_didt_stall_pattern_bit_nums,
	output wire [0:0]		didt_TCP_gc_didt_level_comb_en,
	output wire [0:0]		didt_TCP_se_didt_level_comb_en,
	output wire [0:0]		didt_TCP_qualify_stall_en,
	output wire [1:0]		didt_TCP_didt_stall_sel,
    output wire             didt_TCP_didt_force_stall,
    output wire             didt_TCP_didt_stall_delay_en,
	output wire[14:0]		didt_TCP_stall_pattern1,
    output wire[14:0]		didt_TCP_stall_pattern2,
    output wire[14:0]		didt_TCP_stall_pattern3,
    output wire[14:0]		didt_TCP_stall_pattern4,
    output wire[14:0]		didt_TCP_stall_pattern5,
    output wire[14:0]		didt_TCP_stall_pattern6,
    output wire[14:0]		didt_TCP_stall_pattern7,


  output wire				TCP_edc_en,
  output wire				TCP_edc_sw_rst,
  output wire				TCP_edc_clk_en_override,
  output wire				TCP_edc_force_stall,
  output wire[31:0]			TCP_edc_threshold,
  output wire[4:0]			TCP_edc_trigger_throttle_lowbit,
  output wire[7:0]			TCP_edc_stall_pattern_bit_nums,
  output wire 				TCP_edc_allow_write_pwrdelta,
  output wire				TCP_edc_grbm_clear_overflow_counter,
  output wire[14:0]			TCP_edc_stall_pattern1,
  output wire[14:0]			TCP_edc_stall_pattern2,
  output wire[14:0]			TCP_edc_stall_pattern3,
  output wire[14:0]			TCP_edc_stall_pattern4,
  output wire[14:0]			TCP_edc_stall_pattern5,
  output wire[14:0]			TCP_edc_stall_pattern6,
  output wire[14:0]			TCP_edc_stall_pattern7,
  output wire[13:0]                     TCP_edc_timer_period,
  output wire[`DIDT_TCP_THROTTLE_CTRL_GC_EDC_STALL_EN_SIZE-1:0]		TCP_gc_edc_stall_en,
  output wire[`DIDT_TCP_THROTTLE_CTRL_PCC_STALL_EN_SIZE-1:0]			TCP_pcc_stall_en,
  output wire[`DIDT_TCP_THROTTLE_CTRL_PWRBRK_STALL_EN_SIZE-1:0]		TCP_pwrbrk_stall_en,
  output wire[`DIDT_TCP_THROTTLE_CTRL_GC_EDC_ONLY_MODE_SIZE-1:0]		TCP_gc_edc_only_mode,

  input wire[31:0]			TCP_edc_pcc_perf_counter,
  output wire				rd_TCP_edc_pcc_perf_counter,
  input wire				TCP_edc_fsm_state,
  input wire[2:0]			TCP_edc_throttle_level,
  output wire[6-1:0]			TCP_edc_stall_delay_sig0,
  output wire[6-1:0]			TCP_edc_stall_delay_sig1,
  output wire[6-1:0]			TCP_edc_stall_delay_sig2,
  output wire[6-1:0]			TCP_edc_stall_delay_sig3,
  output wire[6-1:0]			TCP_edc_stall_delay_sig4,
  output wire[6-1:0]			TCP_edc_stall_delay_sig5,
  output wire[6-1:0]			TCP_edc_stall_delay_sig6,
  output wire[6-1:0]			TCP_edc_stall_delay_sig7,
  output wire[6-1:0]			TCP_edc_stall_delay_sig8,
  output wire[6-1:0]			TCP_edc_stall_delay_sig9,
  input wire[15:0]			TCP_edc_throttle_level_overflow_counter,
  input wire				TCP_edc_rolling_power_delta_overflow,
  input wire[31:0]			TCP_edc_rolling_power_delta,
  output wire[31:0]			TCP_edc_grbm_wr_pwrdelta_data,
  output wire				TCP_edc_grbm_wr_pwrdelta_en,
  output wire				TCP_edc_grbm_clear_rolling_power_delta_ov,
  output wire				TCP_gc_edc_en,
  output wire [1:0]			TCP_gc_edc_stall_policy,
  output wire [0:0]			TCP_gc_edc_level_comb_en,
  output wire [0:0]			TCP_se_edc_level_comb_en,
  output wire [0:0]			TCP_edc_level_mode_sel,
  output wire [0:0]			TCP_edc_level_comb_adapt_mode_en,

   output wire                              reg_read_valid,
   output wire [`TARG_GRBM_READ_DATA_RANGE] reg_read_data

 );

   localparam DIDT_IND_INDEX_MAX = 255;

   reg [31:0]                        didt_ind_index;
   wire                              reg_read_valid_d;
   reg [`TARG_GRBM_READ_DATA_RANGE]  reg_read_data_d;
   reg                               didt_index_auto_incr_en;

   wire wr_ind_index = reg_send &  reg_op & ({reg_addr, 2'b0} == `DIDT_IND_INDEX);
   wire rd_ind_index = reg_send & ~reg_op & ({reg_addr, 2'b0} == `DIDT_IND_INDEX);
   wire wr_ind_data  = reg_send &  reg_op & ({reg_addr, 2'b0} == `DIDT_IND_DATA);
   wire rd_ind_data  = reg_send & ~reg_op & ({reg_addr, 2'b0} == `DIDT_IND_DATA);
   wire wr_didt_index_auto_incr_en = reg_send &  reg_op & ({reg_addr, 2'b0} == `DIDT_INDEX_AUTO_INCR_EN);
   wire rd_didt_index_auto_incr_en = reg_send & ~reg_op & ({reg_addr, 2'b0} == `DIDT_INDEX_AUTO_INCR_EN);
   wire [`DIDT_INDEX_AUTO_INCR_EN_DIDT_INDEX_AUTO_INCR_EN_SIZE-1:0]  didt_index_auto_incr_en_d = wr_didt_index_auto_incr_en ? reg_wd : didt_index_auto_incr_en;


   reg [`DIDT_SQ_CTRL0_REG]       didt_SQ_ctrl0;
   reg [`DIDT_SQ_CTRL1_REG]       didt_SQ_ctrl1;
   reg [`DIDT_SQ_CTRL2_REG]       didt_SQ_ctrl2;
   reg [`DIDT_SQ_CTRL_OCP_REG]    didt_SQ_ctrl_ocp;
   reg [`DIDT_SQ_CTRL3_REG]       didt_SQ_ctrl3;
   reg [`DIDT_SQ_STALL_CTRL_REG]       didt_SQ_stall_ctrl;
   reg [`DIDT_SQ_TUNING_CTRL_REG]       didt_SQ_tuning_ctrl;
   reg [`DIDT_SQ_STALL_AUTO_RELEASE_CTRL_REG]       didt_SQ_stall_auto_release_ctrl;
   reg [`DIDT_SQ_WEIGHT0_3_REG]   didt_SQ_weight_0;
   reg [`DIDT_SQ_WEIGHT4_7_REG]   didt_SQ_weight_1;
   reg [`DIDT_SQ_WEIGHT8_11_REG]  didt_SQ_weight_2;

   reg [`DIDT_SQ_MPD_SCALE_FACTOR_REG] didt_SQ_mpd_scale_factor;

   reg [`DIDT_SQ_STALL_RELEASE_CNTL0_REG]  didt_SQ_stall_release_cntl0;
   reg [`DIDT_SQ_STALL_RELEASE_CNTL1_REG]  didt_SQ_stall_release_cntl1;

   reg [`DIDT_SQ_STALL_PATTERN_1_2_REG]       didt_SQ_didt_stall_pattern_1_2;
   reg [`DIDT_SQ_STALL_PATTERN_3_4_REG]       didt_SQ_didt_stall_pattern_3_4;
   reg [`DIDT_SQ_STALL_PATTERN_5_6_REG]       didt_SQ_didt_stall_pattern_5_6;
   reg [`DIDT_SQ_STALL_PATTERN_7_REG]         didt_SQ_didt_stall_pattern_7;

   reg [`DIDT_SQ_EDC_CTRL_REG]       SQ_edc_ctrl;
   reg [`DIDT_SQ_EDC_THRESHOLD_REG]       SQ_edc_threshold_reg;
   reg [`DIDT_SQ_EDC_STALL_PATTERN_1_2_REG]       SQ_edc_stall_pattern_1_2;
   reg [`DIDT_SQ_EDC_STALL_PATTERN_3_4_REG]       SQ_edc_stall_pattern_3_4;
   reg [`DIDT_SQ_EDC_STALL_PATTERN_5_6_REG]       SQ_edc_stall_pattern_5_6;
   reg [`DIDT_SQ_EDC_STALL_PATTERN_7_REG]       SQ_edc_stall_pattern_7;
   reg [`DIDT_SQ_EDC_TIMER_PERIOD_EDC_TIMER_PERIOD_SIZE-1:0]          SQ_edc_timer_period_reg;
   reg [`DIDT_SQ_THROTTLE_CTRL_REG]          	SQ_throttle_ctrl_reg;
   reg [`DIDT_SQ_EDC_STALL_DELAY_1_REG]       SQ_edc_stall_delay_1;
   reg [`DIDT_SQ_EDC_STALL_DELAY_2_REG]       SQ_edc_stall_delay_2;
   reg [`DIDT_SQ_EDC_STALL_DELAY_3_REG]       SQ_edc_stall_delay_3;

   wire  wr_didt_SQ_ctrl0    = (wr_ind_data & (didt_ind_index == `DIDT_SQ_CTRL0_DEC_ADDR));
   wire  wr_didt_SQ_ctrl1    = (wr_ind_data & (didt_ind_index == `DIDT_SQ_CTRL1_DEC_ADDR));
   wire  wr_didt_SQ_ctrl2    = (wr_ind_data & (didt_ind_index == `DIDT_SQ_CTRL2_DEC_ADDR));
   wire  wr_didt_SQ_ctrl3    = (wr_ind_data & (didt_ind_index == `DIDT_SQ_CTRL3_DEC_ADDR));
   wire  wr_didt_SQ_ctrl_ocp   = (wr_ind_data & (didt_ind_index == `DIDT_SQ_CTRL_OCP_DEC_ADDR));
   wire  wr_didt_SQ_stall_ctrl    = (wr_ind_data & (didt_ind_index == `DIDT_SQ_STALL_CTRL_DEC_ADDR));
   wire  wr_didt_SQ_tuning_ctrl    = (wr_ind_data & (didt_ind_index == `DIDT_SQ_TUNING_CTRL_DEC_ADDR));
   wire  wr_didt_SQ_stall_auto_release_ctrl    = (wr_ind_data & (didt_ind_index == `DIDT_SQ_STALL_AUTO_RELEASE_CTRL_DEC_ADDR));
   wire  wr_didt_SQ_weight_0 = (wr_ind_data & (didt_ind_index == `DIDT_SQ_WEIGHT0_3_DEC_ADDR));
   wire  wr_didt_SQ_weight_1 = (wr_ind_data & (didt_ind_index == `DIDT_SQ_WEIGHT4_7_DEC_ADDR));
   wire  wr_didt_SQ_weight_2 = (wr_ind_data & (didt_ind_index == `DIDT_SQ_WEIGHT8_11_DEC_ADDR));
   wire  wr_didt_SQ_mpd_scale_factor = (wr_ind_data & (didt_ind_index == `DIDT_SQ_MPD_SCALE_FACTOR_DEC_ADDR));
   wire  wr_didt_SQ_stall_release_cntl0 = (wr_ind_data & (didt_ind_index == `DIDT_SQ_STALL_RELEASE_CNTL0_DEC_ADDR));
   wire  wr_didt_SQ_stall_release_cntl1 = (wr_ind_data & (didt_ind_index == `DIDT_SQ_STALL_RELEASE_CNTL1_DEC_ADDR));

   wire  wr_didt_SQ_didt_stall_pattern_1_2    = (wr_ind_data & (didt_ind_index == `DIDT_SQ_STALL_PATTERN_1_2_DEC_ADDR));
   wire  wr_didt_SQ_didt_stall_pattern_3_4    = (wr_ind_data & (didt_ind_index == `DIDT_SQ_STALL_PATTERN_3_4_DEC_ADDR));
   wire  wr_didt_SQ_didt_stall_pattern_5_6    = (wr_ind_data & (didt_ind_index == `DIDT_SQ_STALL_PATTERN_5_6_DEC_ADDR));
   wire  wr_didt_SQ_didt_stall_pattern_7      = (wr_ind_data & (didt_ind_index == `DIDT_SQ_STALL_PATTERN_7_DEC_ADDR));

   wire  wr_SQ_edc_ctrl    				= (wr_ind_data & (didt_ind_index == `DIDT_SQ_EDC_CTRL_DEC_ADDR));
   wire  wr_SQ_edc_threshold    		= (wr_ind_data & (didt_ind_index == `DIDT_SQ_EDC_THRESHOLD_DEC_ADDR));
   wire  wr_SQ_edc_stall_pattern_1_2    = (wr_ind_data & (didt_ind_index == `DIDT_SQ_EDC_STALL_PATTERN_1_2_DEC_ADDR));
   wire  wr_SQ_edc_stall_pattern_3_4    = (wr_ind_data & (didt_ind_index == `DIDT_SQ_EDC_STALL_PATTERN_3_4_DEC_ADDR));
   wire  wr_SQ_edc_stall_pattern_5_6    = (wr_ind_data & (didt_ind_index == `DIDT_SQ_EDC_STALL_PATTERN_5_6_DEC_ADDR));
   wire  wr_SQ_edc_stall_pattern_7    	= (wr_ind_data & (didt_ind_index == `DIDT_SQ_EDC_STALL_PATTERN_7_DEC_ADDR));
   wire  wr_SQ_edc_timer_period             = (wr_ind_data & (didt_ind_index == `DIDT_SQ_EDC_TIMER_PERIOD_DEC_ADDR));
   wire  wr_SQ_throttle_ctrl             = (wr_ind_data & (didt_ind_index == `DIDT_SQ_THROTTLE_CTRL_DEC_ADDR));
   wire  wr_SQ_edc_stall_delay_1    	= (wr_ind_data & (didt_ind_index == `DIDT_SQ_EDC_STALL_DELAY_1_DEC_ADDR));
   wire  wr_SQ_edc_stall_delay_2    = (wr_ind_data & (didt_ind_index == `DIDT_SQ_EDC_STALL_DELAY_2_DEC_ADDR));
   wire  wr_SQ_edc_stall_delay_3    = (wr_ind_data & (didt_ind_index == `DIDT_SQ_EDC_STALL_DELAY_3_DEC_ADDR));
   wire  wr_SQ_edc_grbm_wr_pwrdelta_en    = (wr_ind_data & (didt_ind_index == `DIDT_SQ_EDC_ROLLING_POWER_DELTA_DEC_ADDR));
   assign SQ_edc_grbm_wr_pwrdelta_en = wr_SQ_edc_grbm_wr_pwrdelta_en;
   assign SQ_edc_grbm_wr_pwrdelta_data = reg_wd;  //reg_wd only be sampled when wr_didt_edc_grbm_wr_pwrdelta_data_en is valid in DIDT_CNTL_CORE
   //write-clear
   wire  wr_SQ_edc_overflow_rolling_power_delta_overflow    = (wr_ind_data & (didt_ind_index == `DIDT_SQ_EDC_OVERFLOW_DEC_ADDR) & (reg_wd[`DIDT_SQ_EDC_OVERFLOW_EDC_ROLLING_POWER_DELTA_OVERFLOW] == 'd1));
   assign SQ_edc_grbm_clear_rolling_power_delta_ov = wr_SQ_edc_overflow_rolling_power_delta_overflow;

   wire [`DIDT_SQ_CTRL0_REG]      didt_SQ_ctrl0_d    = wr_didt_SQ_ctrl0    ? reg_wd : didt_SQ_ctrl0;
   wire [`DIDT_SQ_CTRL1_REG]      didt_SQ_ctrl1_d    = wr_didt_SQ_ctrl1    ? reg_wd : didt_SQ_ctrl1;
   wire [`DIDT_SQ_CTRL2_REG]      didt_SQ_ctrl2_d    = wr_didt_SQ_ctrl2    ? reg_wd : didt_SQ_ctrl2;
   wire [`DIDT_SQ_CTRL3_REG]      didt_SQ_ctrl3_d    = wr_didt_SQ_ctrl3    ? reg_wd[`DIDT_SQ_CTRL3_REG] : didt_SQ_ctrl3;
   wire [`DIDT_SQ_CTRL_OCP_REG]      didt_SQ_ctrl_ocp_d    = wr_didt_SQ_ctrl_ocp    ? reg_wd : didt_SQ_ctrl_ocp;
   wire [`DIDT_SQ_STALL_CTRL_REG]      didt_SQ_stall_ctrl_d    = wr_didt_SQ_stall_ctrl    ? reg_wd : didt_SQ_stall_ctrl;
   wire [`DIDT_SQ_TUNING_CTRL_REG]      didt_SQ_tuning_ctrl_d    = wr_didt_SQ_tuning_ctrl    ? reg_wd[`DIDT_SQ_TUNING_CTRL_REG] : didt_SQ_tuning_ctrl;
   wire [`DIDT_SQ_STALL_AUTO_RELEASE_CTRL_REG]      didt_SQ_stall_auto_release_ctrl_d    = wr_didt_SQ_stall_auto_release_ctrl    ? reg_wd[`DIDT_SQ_STALL_AUTO_RELEASE_CTRL_REG] : didt_SQ_stall_auto_release_ctrl;
   wire [`DIDT_SQ_WEIGHT0_3_REG]  didt_SQ_weight_0_d = wr_didt_SQ_weight_0 ? reg_wd : didt_SQ_weight_0;
   wire [`DIDT_SQ_WEIGHT4_7_REG]  didt_SQ_weight_1_d = wr_didt_SQ_weight_1 ? reg_wd : didt_SQ_weight_1;
   wire [`DIDT_SQ_WEIGHT8_11_REG] didt_SQ_weight_2_d = wr_didt_SQ_weight_2 ? reg_wd : didt_SQ_weight_2;
   wire [`DIDT_SQ_MPD_SCALE_FACTOR_REG] didt_SQ_mpd_scale_factor_d = wr_didt_SQ_mpd_scale_factor ? reg_wd : didt_SQ_mpd_scale_factor;
   wire [`DIDT_SQ_STALL_RELEASE_CNTL0_REG] didt_SQ_stall_release_cntl0_d = wr_didt_SQ_stall_release_cntl0 ? reg_wd : didt_SQ_stall_release_cntl0;
   wire [`DIDT_SQ_STALL_RELEASE_CNTL1_REG] didt_SQ_stall_release_cntl1_d = wr_didt_SQ_stall_release_cntl1 ? reg_wd : didt_SQ_stall_release_cntl1;   
   wire [`DIDT_SQ_STALL_PATTERN_1_2_REG]      didt_SQ_didt_stall_pattern_1_2_d    = wr_didt_SQ_didt_stall_pattern_1_2    ? reg_wd : didt_SQ_didt_stall_pattern_1_2;
   wire [`DIDT_SQ_STALL_PATTERN_3_4_REG]      didt_SQ_didt_stall_pattern_3_4_d    = wr_didt_SQ_didt_stall_pattern_3_4    ? reg_wd : didt_SQ_didt_stall_pattern_3_4;
   wire [`DIDT_SQ_STALL_PATTERN_5_6_REG]      didt_SQ_didt_stall_pattern_5_6_d    = wr_didt_SQ_didt_stall_pattern_5_6    ? reg_wd : didt_SQ_didt_stall_pattern_5_6;
   wire [`DIDT_SQ_STALL_PATTERN_7_REG]        didt_SQ_didt_stall_pattern_7_d    = wr_didt_SQ_didt_stall_pattern_7    ? reg_wd : didt_SQ_didt_stall_pattern_7;


   wire [`DIDT_SQ_EDC_CTRL_REG]      SQ_edc_ctrl_d    = wr_SQ_edc_ctrl    ? reg_wd : SQ_edc_ctrl;
   wire [`DIDT_SQ_EDC_THRESHOLD_REG]      SQ_edc_threshold_d    = wr_SQ_edc_threshold    ? reg_wd : SQ_edc_threshold_reg;
   wire [`DIDT_SQ_EDC_STALL_PATTERN_1_2_REG]      SQ_edc_stall_pattern_1_2_d    = wr_SQ_edc_stall_pattern_1_2    ? reg_wd : SQ_edc_stall_pattern_1_2;
   wire [`DIDT_SQ_EDC_STALL_PATTERN_3_4_REG]      SQ_edc_stall_pattern_3_4_d    = wr_SQ_edc_stall_pattern_3_4    ? reg_wd : SQ_edc_stall_pattern_3_4;
   wire [`DIDT_SQ_EDC_STALL_PATTERN_5_6_REG]      SQ_edc_stall_pattern_5_6_d    = wr_SQ_edc_stall_pattern_5_6    ? reg_wd : SQ_edc_stall_pattern_5_6;
   wire [`DIDT_SQ_EDC_STALL_PATTERN_7_REG]        SQ_edc_stall_pattern_7_d   	= wr_SQ_edc_stall_pattern_7      ? reg_wd : SQ_edc_stall_pattern_7;
   wire [`DIDT_SQ_EDC_TIMER_PERIOD_REG]          SQ_edc_timer_period_d   	= wr_SQ_edc_timer_period         ? reg_wd : SQ_edc_timer_period_reg;
   wire [`DIDT_SQ_THROTTLE_CTRL_REG]          SQ_throttle_ctrl_d   	= wr_SQ_throttle_ctrl         ? reg_wd : SQ_throttle_ctrl_reg;
   assign rd_SQ_edc_pcc_perf_counter		= (rd_ind_data & (didt_ind_index == `DIDT_SQ_EDC_PCC_PERF_COUNTER_DEC_ADDR));
   wire [`DIDT_SQ_EDC_STALL_DELAY_1_REG]      SQ_edc_stall_delay_1_d    = wr_SQ_edc_stall_delay_1    ? reg_wd[`DIDT_SQ_EDC_STALL_DELAY_1_REG] : SQ_edc_stall_delay_1;
   wire [`DIDT_SQ_EDC_STALL_DELAY_2_REG]      SQ_edc_stall_delay_2_d    = wr_SQ_edc_stall_delay_2    ? reg_wd : SQ_edc_stall_delay_2;
   wire [`DIDT_SQ_EDC_STALL_DELAY_3_REG]      SQ_edc_stall_delay_3_d    = wr_SQ_edc_stall_delay_3    ? reg_wd : SQ_edc_stall_delay_3;


   reg [`DIDT_DB_CTRL0_REG]       didt_DB_ctrl0;
   reg [`DIDT_DB_CTRL1_REG]       didt_DB_ctrl1;
   reg [`DIDT_DB_CTRL2_REG]       didt_DB_ctrl2;
   reg [`DIDT_DB_CTRL_OCP_REG]    didt_DB_ctrl_ocp;
   reg [`DIDT_DB_CTRL3_REG]       didt_DB_ctrl3;
   reg [`DIDT_DB_STALL_CTRL_REG]       didt_DB_stall_ctrl;
   reg [`DIDT_DB_TUNING_CTRL_REG]       didt_DB_tuning_ctrl;
   reg [`DIDT_DB_STALL_AUTO_RELEASE_CTRL_REG]       didt_DB_stall_auto_release_ctrl;
   reg [`DIDT_DB_WEIGHT0_3_REG]   didt_DB_weight_0;
   reg [`DIDT_DB_WEIGHT4_7_REG]   didt_DB_weight_1;
   reg [`DIDT_DB_WEIGHT8_11_REG]  didt_DB_weight_2;

   reg [`DIDT_DB_MPD_SCALE_FACTOR_REG] didt_DB_mpd_scale_factor;

   reg [`DIDT_DB_STALL_RELEASE_CNTL0_REG]  didt_DB_stall_release_cntl0;
   reg [`DIDT_DB_STALL_RELEASE_CNTL1_REG]  didt_DB_stall_release_cntl1;

   reg [`DIDT_DB_STALL_PATTERN_1_2_REG]       didt_DB_didt_stall_pattern_1_2;
   reg [`DIDT_DB_STALL_PATTERN_3_4_REG]       didt_DB_didt_stall_pattern_3_4;
   reg [`DIDT_DB_STALL_PATTERN_5_6_REG]       didt_DB_didt_stall_pattern_5_6;
   reg [`DIDT_DB_STALL_PATTERN_7_REG]         didt_DB_didt_stall_pattern_7;

   reg [`DIDT_DB_EDC_CTRL_REG]       DB_edc_ctrl;
   reg [`DIDT_DB_EDC_THRESHOLD_REG]       DB_edc_threshold_reg;
   reg [`DIDT_DB_EDC_STALL_PATTERN_1_2_REG]       DB_edc_stall_pattern_1_2;
   reg [`DIDT_DB_EDC_STALL_PATTERN_3_4_REG]       DB_edc_stall_pattern_3_4;
   reg [`DIDT_DB_EDC_STALL_PATTERN_5_6_REG]       DB_edc_stall_pattern_5_6;
   reg [`DIDT_DB_EDC_STALL_PATTERN_7_REG]       DB_edc_stall_pattern_7;
   reg [`DIDT_DB_EDC_TIMER_PERIOD_EDC_TIMER_PERIOD_SIZE-1:0]          DB_edc_timer_period_reg;
   reg [`DIDT_DB_THROTTLE_CTRL_REG]          	DB_throttle_ctrl_reg;
   reg [`DIDT_DB_EDC_STALL_DELAY_1_REG]       DB_edc_stall_delay_1;
`ifdef GC__NUM_RB_PER_SA__4_PRESENT
   reg [`DIDT_DB_EDC_STALL_DELAY_2_REG]       DB_edc_stall_delay_2;
`endif
`ifdef GC__NUM_RB_PER_SA__8_PRESENT
   reg [`DIDT_DB_EDC_STALL_DELAY_3_REG]       DB_edc_stall_delay_3;
`endif
`ifdef GC__NUM_RB_PER_SA__12_PRESENT
   reg [`DIDT_DB_EDC_STALL_DELAY_4_REG]       DB_edc_stall_delay_4;
`endif

   wire  wr_didt_DB_ctrl0    = (wr_ind_data & (didt_ind_index == `DIDT_DB_CTRL0_DEC_ADDR));
   wire  wr_didt_DB_ctrl1    = (wr_ind_data & (didt_ind_index == `DIDT_DB_CTRL1_DEC_ADDR));
   wire  wr_didt_DB_ctrl2    = (wr_ind_data & (didt_ind_index == `DIDT_DB_CTRL2_DEC_ADDR));
   wire  wr_didt_DB_ctrl3    = (wr_ind_data & (didt_ind_index == `DIDT_DB_CTRL3_DEC_ADDR));
   wire  wr_didt_DB_ctrl_ocp   = (wr_ind_data & (didt_ind_index == `DIDT_DB_CTRL_OCP_DEC_ADDR));
   wire  wr_didt_DB_stall_ctrl    = (wr_ind_data & (didt_ind_index == `DIDT_DB_STALL_CTRL_DEC_ADDR));
   wire  wr_didt_DB_tuning_ctrl    = (wr_ind_data & (didt_ind_index == `DIDT_DB_TUNING_CTRL_DEC_ADDR));
   wire  wr_didt_DB_stall_auto_release_ctrl    = (wr_ind_data & (didt_ind_index == `DIDT_DB_STALL_AUTO_RELEASE_CTRL_DEC_ADDR));
   wire  wr_didt_DB_weight_0 = (wr_ind_data & (didt_ind_index == `DIDT_DB_WEIGHT0_3_DEC_ADDR));
   wire  wr_didt_DB_weight_1 = (wr_ind_data & (didt_ind_index == `DIDT_DB_WEIGHT4_7_DEC_ADDR));
   wire  wr_didt_DB_weight_2 = (wr_ind_data & (didt_ind_index == `DIDT_DB_WEIGHT8_11_DEC_ADDR));
   wire  wr_didt_DB_mpd_scale_factor = (wr_ind_data & (didt_ind_index == `DIDT_DB_MPD_SCALE_FACTOR_DEC_ADDR));
   wire  wr_didt_DB_stall_release_cntl0 = (wr_ind_data & (didt_ind_index == `DIDT_DB_STALL_RELEASE_CNTL0_DEC_ADDR));
   wire  wr_didt_DB_stall_release_cntl1 = (wr_ind_data & (didt_ind_index == `DIDT_DB_STALL_RELEASE_CNTL1_DEC_ADDR));

   wire  wr_didt_DB_didt_stall_pattern_1_2    = (wr_ind_data & (didt_ind_index == `DIDT_DB_STALL_PATTERN_1_2_DEC_ADDR));
   wire  wr_didt_DB_didt_stall_pattern_3_4    = (wr_ind_data & (didt_ind_index == `DIDT_DB_STALL_PATTERN_3_4_DEC_ADDR));
   wire  wr_didt_DB_didt_stall_pattern_5_6    = (wr_ind_data & (didt_ind_index == `DIDT_DB_STALL_PATTERN_5_6_DEC_ADDR));
   wire  wr_didt_DB_didt_stall_pattern_7      = (wr_ind_data & (didt_ind_index == `DIDT_DB_STALL_PATTERN_7_DEC_ADDR));

   wire  wr_DB_edc_ctrl    				= (wr_ind_data & (didt_ind_index == `DIDT_DB_EDC_CTRL_DEC_ADDR));
   wire  wr_DB_edc_threshold    		= (wr_ind_data & (didt_ind_index == `DIDT_DB_EDC_THRESHOLD_DEC_ADDR));
   wire  wr_DB_edc_stall_pattern_1_2    = (wr_ind_data & (didt_ind_index == `DIDT_DB_EDC_STALL_PATTERN_1_2_DEC_ADDR));
   wire  wr_DB_edc_stall_pattern_3_4    = (wr_ind_data & (didt_ind_index == `DIDT_DB_EDC_STALL_PATTERN_3_4_DEC_ADDR));
   wire  wr_DB_edc_stall_pattern_5_6    = (wr_ind_data & (didt_ind_index == `DIDT_DB_EDC_STALL_PATTERN_5_6_DEC_ADDR));
   wire  wr_DB_edc_stall_pattern_7    	= (wr_ind_data & (didt_ind_index == `DIDT_DB_EDC_STALL_PATTERN_7_DEC_ADDR));
   wire  wr_DB_edc_timer_period             = (wr_ind_data & (didt_ind_index == `DIDT_DB_EDC_TIMER_PERIOD_DEC_ADDR));
   wire  wr_DB_throttle_ctrl             = (wr_ind_data & (didt_ind_index == `DIDT_DB_THROTTLE_CTRL_DEC_ADDR));
   wire  wr_DB_edc_stall_delay_1    	= (wr_ind_data & (didt_ind_index == `DIDT_DB_EDC_STALL_DELAY_1_DEC_ADDR));
`ifdef GC__NUM_RB_PER_SA__4_PRESENT
   wire  wr_DB_edc_stall_delay_2    = (wr_ind_data & (didt_ind_index == `DIDT_DB_EDC_STALL_DELAY_2_DEC_ADDR));
`endif
`ifdef GC__NUM_RB_PER_SA__8_PRESENT
   wire  wr_DB_edc_stall_delay_3    = (wr_ind_data & (didt_ind_index == `DIDT_DB_EDC_STALL_DELAY_3_DEC_ADDR));
`endif
`ifdef GC__NUM_RB_PER_SA__12_PRESENT
   wire  wr_DB_edc_stall_delay_4    = (wr_ind_data & (didt_ind_index == `DIDT_DB_EDC_STALL_DELAY_4_DEC_ADDR));
`endif
   wire  wr_DB_edc_grbm_wr_pwrdelta_en    = (wr_ind_data & (didt_ind_index == `DIDT_DB_EDC_ROLLING_POWER_DELTA_DEC_ADDR));
   assign DB_edc_grbm_wr_pwrdelta_en = wr_DB_edc_grbm_wr_pwrdelta_en;
   assign DB_edc_grbm_wr_pwrdelta_data = reg_wd;  //reg_wd only be sampled when wr_didt_edc_grbm_wr_pwrdelta_data_en is valid in DIDT_CNTL_CORE
   //write-clear
   wire  wr_DB_edc_overflow_rolling_power_delta_overflow    = (wr_ind_data & (didt_ind_index == `DIDT_DB_EDC_OVERFLOW_DEC_ADDR) & (reg_wd[`DIDT_DB_EDC_OVERFLOW_EDC_ROLLING_POWER_DELTA_OVERFLOW] == 'd1));
   assign DB_edc_grbm_clear_rolling_power_delta_ov = wr_DB_edc_overflow_rolling_power_delta_overflow;

   wire [`DIDT_DB_CTRL0_REG]      didt_DB_ctrl0_d    = wr_didt_DB_ctrl0    ? reg_wd : didt_DB_ctrl0;
   wire [`DIDT_DB_CTRL1_REG]      didt_DB_ctrl1_d    = wr_didt_DB_ctrl1    ? reg_wd : didt_DB_ctrl1;
   wire [`DIDT_DB_CTRL2_REG]      didt_DB_ctrl2_d    = wr_didt_DB_ctrl2    ? reg_wd : didt_DB_ctrl2;
   wire [`DIDT_DB_CTRL3_REG]      didt_DB_ctrl3_d    = wr_didt_DB_ctrl3    ? reg_wd[`DIDT_DB_CTRL3_REG] : didt_DB_ctrl3;
   wire [`DIDT_DB_CTRL_OCP_REG]      didt_DB_ctrl_ocp_d    = wr_didt_DB_ctrl_ocp    ? reg_wd : didt_DB_ctrl_ocp;
   wire [`DIDT_DB_STALL_CTRL_REG]      didt_DB_stall_ctrl_d    = wr_didt_DB_stall_ctrl    ? reg_wd : didt_DB_stall_ctrl;
   wire [`DIDT_DB_TUNING_CTRL_REG]      didt_DB_tuning_ctrl_d    = wr_didt_DB_tuning_ctrl    ? reg_wd[`DIDT_DB_TUNING_CTRL_REG] : didt_DB_tuning_ctrl;
   wire [`DIDT_DB_STALL_AUTO_RELEASE_CTRL_REG]      didt_DB_stall_auto_release_ctrl_d    = wr_didt_DB_stall_auto_release_ctrl    ? reg_wd[`DIDT_DB_STALL_AUTO_RELEASE_CTRL_REG] : didt_DB_stall_auto_release_ctrl;
   wire [`DIDT_DB_WEIGHT0_3_REG]  didt_DB_weight_0_d = wr_didt_DB_weight_0 ? reg_wd : didt_DB_weight_0;
   wire [`DIDT_DB_WEIGHT4_7_REG]  didt_DB_weight_1_d = wr_didt_DB_weight_1 ? reg_wd : didt_DB_weight_1;
   wire [`DIDT_DB_WEIGHT8_11_REG] didt_DB_weight_2_d = wr_didt_DB_weight_2 ? reg_wd : didt_DB_weight_2;
   wire [`DIDT_DB_MPD_SCALE_FACTOR_REG] didt_DB_mpd_scale_factor_d = wr_didt_DB_mpd_scale_factor ? reg_wd : didt_DB_mpd_scale_factor;
   wire [`DIDT_DB_STALL_RELEASE_CNTL0_REG] didt_DB_stall_release_cntl0_d = wr_didt_DB_stall_release_cntl0 ? reg_wd : didt_DB_stall_release_cntl0;
   wire [`DIDT_DB_STALL_RELEASE_CNTL1_REG] didt_DB_stall_release_cntl1_d = wr_didt_DB_stall_release_cntl1 ? reg_wd : didt_DB_stall_release_cntl1;   
   wire [`DIDT_DB_STALL_PATTERN_1_2_REG]      didt_DB_didt_stall_pattern_1_2_d    = wr_didt_DB_didt_stall_pattern_1_2    ? reg_wd : didt_DB_didt_stall_pattern_1_2;
   wire [`DIDT_DB_STALL_PATTERN_3_4_REG]      didt_DB_didt_stall_pattern_3_4_d    = wr_didt_DB_didt_stall_pattern_3_4    ? reg_wd : didt_DB_didt_stall_pattern_3_4;
   wire [`DIDT_DB_STALL_PATTERN_5_6_REG]      didt_DB_didt_stall_pattern_5_6_d    = wr_didt_DB_didt_stall_pattern_5_6    ? reg_wd : didt_DB_didt_stall_pattern_5_6;
   wire [`DIDT_DB_STALL_PATTERN_7_REG]        didt_DB_didt_stall_pattern_7_d    = wr_didt_DB_didt_stall_pattern_7    ? reg_wd : didt_DB_didt_stall_pattern_7;


   wire [`DIDT_DB_EDC_CTRL_REG]      DB_edc_ctrl_d    = wr_DB_edc_ctrl    ? reg_wd : DB_edc_ctrl;
   wire [`DIDT_DB_EDC_THRESHOLD_REG]      DB_edc_threshold_d    = wr_DB_edc_threshold    ? reg_wd : DB_edc_threshold_reg;
   wire [`DIDT_DB_EDC_STALL_PATTERN_1_2_REG]      DB_edc_stall_pattern_1_2_d    = wr_DB_edc_stall_pattern_1_2    ? reg_wd : DB_edc_stall_pattern_1_2;
   wire [`DIDT_DB_EDC_STALL_PATTERN_3_4_REG]      DB_edc_stall_pattern_3_4_d    = wr_DB_edc_stall_pattern_3_4    ? reg_wd : DB_edc_stall_pattern_3_4;
   wire [`DIDT_DB_EDC_STALL_PATTERN_5_6_REG]      DB_edc_stall_pattern_5_6_d    = wr_DB_edc_stall_pattern_5_6    ? reg_wd : DB_edc_stall_pattern_5_6;
   wire [`DIDT_DB_EDC_STALL_PATTERN_7_REG]        DB_edc_stall_pattern_7_d   	= wr_DB_edc_stall_pattern_7      ? reg_wd : DB_edc_stall_pattern_7;
   wire [`DIDT_DB_EDC_TIMER_PERIOD_REG]          DB_edc_timer_period_d   	= wr_DB_edc_timer_period         ? reg_wd : DB_edc_timer_period_reg;
   wire [`DIDT_DB_THROTTLE_CTRL_REG]          DB_throttle_ctrl_d   	= wr_DB_throttle_ctrl         ? reg_wd : DB_throttle_ctrl_reg;
   assign rd_DB_edc_pcc_perf_counter		= (rd_ind_data & (didt_ind_index == `DIDT_DB_EDC_PCC_PERF_COUNTER_DEC_ADDR));
   wire [`DIDT_DB_EDC_STALL_DELAY_1_REG]      DB_edc_stall_delay_1_d    = wr_DB_edc_stall_delay_1    ? reg_wd[`DIDT_DB_EDC_STALL_DELAY_1_REG] : DB_edc_stall_delay_1;
`ifdef GC__NUM_RB_PER_SA__4_PRESENT
   wire [`DIDT_DB_EDC_STALL_DELAY_2_REG]      DB_edc_stall_delay_2_d    = wr_DB_edc_stall_delay_2    ? reg_wd : DB_edc_stall_delay_2;
`endif
`ifdef GC__NUM_RB_PER_SA__8_PRESENT
   wire [`DIDT_DB_EDC_STALL_DELAY_3_REG]      DB_edc_stall_delay_3_d    = wr_DB_edc_stall_delay_3    ? reg_wd : DB_edc_stall_delay_3;
`endif
`ifdef GC__NUM_RB_PER_SA__12_PRESENT
   wire [`DIDT_DB_EDC_STALL_DELAY_4_REG]      DB_edc_stall_delay_4_d    = wr_DB_edc_stall_delay_4    ? reg_wd : DB_edc_stall_delay_4;
`endif


   reg [`DIDT_TD_CTRL0_REG]       didt_TD_ctrl0;
   reg [`DIDT_TD_CTRL1_REG]       didt_TD_ctrl1;
   reg [`DIDT_TD_CTRL2_REG]       didt_TD_ctrl2;
   reg [`DIDT_TD_CTRL_OCP_REG]    didt_TD_ctrl_ocp;
   reg [`DIDT_TD_CTRL3_REG]       didt_TD_ctrl3;
   reg [`DIDT_TD_STALL_CTRL_REG]       didt_TD_stall_ctrl;
   reg [`DIDT_TD_TUNING_CTRL_REG]       didt_TD_tuning_ctrl;
   reg [`DIDT_TD_STALL_AUTO_RELEASE_CTRL_REG]       didt_TD_stall_auto_release_ctrl;
   reg [`DIDT_TD_WEIGHT0_3_REG]   didt_TD_weight_0;
   reg [`DIDT_TD_WEIGHT4_7_REG]   didt_TD_weight_1;
   reg [`DIDT_TD_WEIGHT8_11_REG]  didt_TD_weight_2;

   reg [`DIDT_TD_MPD_SCALE_FACTOR_REG] didt_TD_mpd_scale_factor;

   reg [`DIDT_TD_STALL_RELEASE_CNTL0_REG]  didt_TD_stall_release_cntl0;
   reg [`DIDT_TD_STALL_RELEASE_CNTL1_REG]  didt_TD_stall_release_cntl1;

   reg [`DIDT_TD_STALL_PATTERN_1_2_REG]       didt_TD_didt_stall_pattern_1_2;
   reg [`DIDT_TD_STALL_PATTERN_3_4_REG]       didt_TD_didt_stall_pattern_3_4;
   reg [`DIDT_TD_STALL_PATTERN_5_6_REG]       didt_TD_didt_stall_pattern_5_6;
   reg [`DIDT_TD_STALL_PATTERN_7_REG]         didt_TD_didt_stall_pattern_7;

   reg [`DIDT_TD_EDC_CTRL_REG]       TD_edc_ctrl;
   reg [`DIDT_TD_EDC_THRESHOLD_REG]       TD_edc_threshold_reg;
   reg [`DIDT_TD_EDC_STALL_PATTERN_1_2_REG]       TD_edc_stall_pattern_1_2;
   reg [`DIDT_TD_EDC_STALL_PATTERN_3_4_REG]       TD_edc_stall_pattern_3_4;
   reg [`DIDT_TD_EDC_STALL_PATTERN_5_6_REG]       TD_edc_stall_pattern_5_6;
   reg [`DIDT_TD_EDC_STALL_PATTERN_7_REG]       TD_edc_stall_pattern_7;
   reg [`DIDT_TD_EDC_TIMER_PERIOD_EDC_TIMER_PERIOD_SIZE-1:0]          TD_edc_timer_period_reg;
   reg [`DIDT_TD_THROTTLE_CTRL_REG]          	TD_throttle_ctrl_reg;
   reg [`DIDT_TD_EDC_STALL_DELAY_1_REG]       TD_edc_stall_delay_1;
   reg [`DIDT_TD_EDC_STALL_DELAY_2_REG]       TD_edc_stall_delay_2;
   reg [`DIDT_TD_EDC_STALL_DELAY_3_REG]       TD_edc_stall_delay_3;

   wire  wr_didt_TD_ctrl0    = (wr_ind_data & (didt_ind_index == `DIDT_TD_CTRL0_DEC_ADDR));
   wire  wr_didt_TD_ctrl1    = (wr_ind_data & (didt_ind_index == `DIDT_TD_CTRL1_DEC_ADDR));
   wire  wr_didt_TD_ctrl2    = (wr_ind_data & (didt_ind_index == `DIDT_TD_CTRL2_DEC_ADDR));
   wire  wr_didt_TD_ctrl3    = (wr_ind_data & (didt_ind_index == `DIDT_TD_CTRL3_DEC_ADDR));
   wire  wr_didt_TD_ctrl_ocp   = (wr_ind_data & (didt_ind_index == `DIDT_TD_CTRL_OCP_DEC_ADDR));
   wire  wr_didt_TD_stall_ctrl    = (wr_ind_data & (didt_ind_index == `DIDT_TD_STALL_CTRL_DEC_ADDR));
   wire  wr_didt_TD_tuning_ctrl    = (wr_ind_data & (didt_ind_index == `DIDT_TD_TUNING_CTRL_DEC_ADDR));
   wire  wr_didt_TD_stall_auto_release_ctrl    = (wr_ind_data & (didt_ind_index == `DIDT_TD_STALL_AUTO_RELEASE_CTRL_DEC_ADDR));
   wire  wr_didt_TD_weight_0 = (wr_ind_data & (didt_ind_index == `DIDT_TD_WEIGHT0_3_DEC_ADDR));
   wire  wr_didt_TD_weight_1 = (wr_ind_data & (didt_ind_index == `DIDT_TD_WEIGHT4_7_DEC_ADDR));
   wire  wr_didt_TD_weight_2 = (wr_ind_data & (didt_ind_index == `DIDT_TD_WEIGHT8_11_DEC_ADDR));
   wire  wr_didt_TD_mpd_scale_factor = (wr_ind_data & (didt_ind_index == `DIDT_TD_MPD_SCALE_FACTOR_DEC_ADDR));
   wire  wr_didt_TD_stall_release_cntl0 = (wr_ind_data & (didt_ind_index == `DIDT_TD_STALL_RELEASE_CNTL0_DEC_ADDR));
   wire  wr_didt_TD_stall_release_cntl1 = (wr_ind_data & (didt_ind_index == `DIDT_TD_STALL_RELEASE_CNTL1_DEC_ADDR));

   wire  wr_didt_TD_didt_stall_pattern_1_2    = (wr_ind_data & (didt_ind_index == `DIDT_TD_STALL_PATTERN_1_2_DEC_ADDR));
   wire  wr_didt_TD_didt_stall_pattern_3_4    = (wr_ind_data & (didt_ind_index == `DIDT_TD_STALL_PATTERN_3_4_DEC_ADDR));
   wire  wr_didt_TD_didt_stall_pattern_5_6    = (wr_ind_data & (didt_ind_index == `DIDT_TD_STALL_PATTERN_5_6_DEC_ADDR));
   wire  wr_didt_TD_didt_stall_pattern_7      = (wr_ind_data & (didt_ind_index == `DIDT_TD_STALL_PATTERN_7_DEC_ADDR));

   wire  wr_TD_edc_ctrl    				= (wr_ind_data & (didt_ind_index == `DIDT_TD_EDC_CTRL_DEC_ADDR));
   wire  wr_TD_edc_threshold    		= (wr_ind_data & (didt_ind_index == `DIDT_TD_EDC_THRESHOLD_DEC_ADDR));
   wire  wr_TD_edc_stall_pattern_1_2    = (wr_ind_data & (didt_ind_index == `DIDT_TD_EDC_STALL_PATTERN_1_2_DEC_ADDR));
   wire  wr_TD_edc_stall_pattern_3_4    = (wr_ind_data & (didt_ind_index == `DIDT_TD_EDC_STALL_PATTERN_3_4_DEC_ADDR));
   wire  wr_TD_edc_stall_pattern_5_6    = (wr_ind_data & (didt_ind_index == `DIDT_TD_EDC_STALL_PATTERN_5_6_DEC_ADDR));
   wire  wr_TD_edc_stall_pattern_7    	= (wr_ind_data & (didt_ind_index == `DIDT_TD_EDC_STALL_PATTERN_7_DEC_ADDR));
   wire  wr_TD_edc_timer_period             = (wr_ind_data & (didt_ind_index == `DIDT_TD_EDC_TIMER_PERIOD_DEC_ADDR));
   wire  wr_TD_throttle_ctrl             = (wr_ind_data & (didt_ind_index == `DIDT_TD_THROTTLE_CTRL_DEC_ADDR));
   wire  wr_TD_edc_stall_delay_1    	= (wr_ind_data & (didt_ind_index == `DIDT_TD_EDC_STALL_DELAY_1_DEC_ADDR));
   wire  wr_TD_edc_stall_delay_2    = (wr_ind_data & (didt_ind_index == `DIDT_TD_EDC_STALL_DELAY_2_DEC_ADDR));
   wire  wr_TD_edc_stall_delay_3    = (wr_ind_data & (didt_ind_index == `DIDT_TD_EDC_STALL_DELAY_3_DEC_ADDR));
   wire  wr_TD_edc_grbm_wr_pwrdelta_en    = (wr_ind_data & (didt_ind_index == `DIDT_TD_EDC_ROLLING_POWER_DELTA_DEC_ADDR));
   assign TD_edc_grbm_wr_pwrdelta_en = wr_TD_edc_grbm_wr_pwrdelta_en;
   assign TD_edc_grbm_wr_pwrdelta_data = reg_wd;  //reg_wd only be sampled when wr_didt_edc_grbm_wr_pwrdelta_data_en is valid in DIDT_CNTL_CORE
   //write-clear
   wire  wr_TD_edc_overflow_rolling_power_delta_overflow    = (wr_ind_data & (didt_ind_index == `DIDT_TD_EDC_OVERFLOW_DEC_ADDR) & (reg_wd[`DIDT_TD_EDC_OVERFLOW_EDC_ROLLING_POWER_DELTA_OVERFLOW] == 'd1));
   assign TD_edc_grbm_clear_rolling_power_delta_ov = wr_TD_edc_overflow_rolling_power_delta_overflow;

   wire [`DIDT_TD_CTRL0_REG]      didt_TD_ctrl0_d    = wr_didt_TD_ctrl0    ? reg_wd : didt_TD_ctrl0;
   wire [`DIDT_TD_CTRL1_REG]      didt_TD_ctrl1_d    = wr_didt_TD_ctrl1    ? reg_wd : didt_TD_ctrl1;
   wire [`DIDT_TD_CTRL2_REG]      didt_TD_ctrl2_d    = wr_didt_TD_ctrl2    ? reg_wd : didt_TD_ctrl2;
   wire [`DIDT_TD_CTRL3_REG]      didt_TD_ctrl3_d    = wr_didt_TD_ctrl3    ? reg_wd[`DIDT_TD_CTRL3_REG] : didt_TD_ctrl3;
   wire [`DIDT_TD_CTRL_OCP_REG]      didt_TD_ctrl_ocp_d    = wr_didt_TD_ctrl_ocp    ? reg_wd : didt_TD_ctrl_ocp;
   wire [`DIDT_TD_STALL_CTRL_REG]      didt_TD_stall_ctrl_d    = wr_didt_TD_stall_ctrl    ? reg_wd : didt_TD_stall_ctrl;
   wire [`DIDT_TD_TUNING_CTRL_REG]      didt_TD_tuning_ctrl_d    = wr_didt_TD_tuning_ctrl    ? reg_wd[`DIDT_TD_TUNING_CTRL_REG] : didt_TD_tuning_ctrl;
   wire [`DIDT_TD_STALL_AUTO_RELEASE_CTRL_REG]      didt_TD_stall_auto_release_ctrl_d    = wr_didt_TD_stall_auto_release_ctrl    ? reg_wd[`DIDT_TD_STALL_AUTO_RELEASE_CTRL_REG] : didt_TD_stall_auto_release_ctrl;
   wire [`DIDT_TD_WEIGHT0_3_REG]  didt_TD_weight_0_d = wr_didt_TD_weight_0 ? reg_wd : didt_TD_weight_0;
   wire [`DIDT_TD_WEIGHT4_7_REG]  didt_TD_weight_1_d = wr_didt_TD_weight_1 ? reg_wd : didt_TD_weight_1;
   wire [`DIDT_TD_WEIGHT8_11_REG] didt_TD_weight_2_d = wr_didt_TD_weight_2 ? reg_wd : didt_TD_weight_2;
   wire [`DIDT_TD_MPD_SCALE_FACTOR_REG] didt_TD_mpd_scale_factor_d = wr_didt_TD_mpd_scale_factor ? reg_wd : didt_TD_mpd_scale_factor;
   wire [`DIDT_TD_STALL_RELEASE_CNTL0_REG] didt_TD_stall_release_cntl0_d = wr_didt_TD_stall_release_cntl0 ? reg_wd : didt_TD_stall_release_cntl0;
   wire [`DIDT_TD_STALL_RELEASE_CNTL1_REG] didt_TD_stall_release_cntl1_d = wr_didt_TD_stall_release_cntl1 ? reg_wd : didt_TD_stall_release_cntl1;   
   wire [`DIDT_TD_STALL_PATTERN_1_2_REG]      didt_TD_didt_stall_pattern_1_2_d    = wr_didt_TD_didt_stall_pattern_1_2    ? reg_wd : didt_TD_didt_stall_pattern_1_2;
   wire [`DIDT_TD_STALL_PATTERN_3_4_REG]      didt_TD_didt_stall_pattern_3_4_d    = wr_didt_TD_didt_stall_pattern_3_4    ? reg_wd : didt_TD_didt_stall_pattern_3_4;
   wire [`DIDT_TD_STALL_PATTERN_5_6_REG]      didt_TD_didt_stall_pattern_5_6_d    = wr_didt_TD_didt_stall_pattern_5_6    ? reg_wd : didt_TD_didt_stall_pattern_5_6;
   wire [`DIDT_TD_STALL_PATTERN_7_REG]        didt_TD_didt_stall_pattern_7_d    = wr_didt_TD_didt_stall_pattern_7    ? reg_wd : didt_TD_didt_stall_pattern_7;


   wire [`DIDT_TD_EDC_CTRL_REG]      TD_edc_ctrl_d    = wr_TD_edc_ctrl    ? reg_wd : TD_edc_ctrl;
   wire [`DIDT_TD_EDC_THRESHOLD_REG]      TD_edc_threshold_d    = wr_TD_edc_threshold    ? reg_wd : TD_edc_threshold_reg;
   wire [`DIDT_TD_EDC_STALL_PATTERN_1_2_REG]      TD_edc_stall_pattern_1_2_d    = wr_TD_edc_stall_pattern_1_2    ? reg_wd : TD_edc_stall_pattern_1_2;
   wire [`DIDT_TD_EDC_STALL_PATTERN_3_4_REG]      TD_edc_stall_pattern_3_4_d    = wr_TD_edc_stall_pattern_3_4    ? reg_wd : TD_edc_stall_pattern_3_4;
   wire [`DIDT_TD_EDC_STALL_PATTERN_5_6_REG]      TD_edc_stall_pattern_5_6_d    = wr_TD_edc_stall_pattern_5_6    ? reg_wd : TD_edc_stall_pattern_5_6;
   wire [`DIDT_TD_EDC_STALL_PATTERN_7_REG]        TD_edc_stall_pattern_7_d   	= wr_TD_edc_stall_pattern_7      ? reg_wd : TD_edc_stall_pattern_7;
   wire [`DIDT_TD_EDC_TIMER_PERIOD_REG]          TD_edc_timer_period_d   	= wr_TD_edc_timer_period         ? reg_wd : TD_edc_timer_period_reg;
   wire [`DIDT_TD_THROTTLE_CTRL_REG]          TD_throttle_ctrl_d   	= wr_TD_throttle_ctrl         ? reg_wd : TD_throttle_ctrl_reg;
   assign rd_TD_edc_pcc_perf_counter		= (rd_ind_data & (didt_ind_index == `DIDT_TD_EDC_PCC_PERF_COUNTER_DEC_ADDR));
   wire [`DIDT_TD_EDC_STALL_DELAY_1_REG]      TD_edc_stall_delay_1_d    = wr_TD_edc_stall_delay_1    ? reg_wd[`DIDT_TD_EDC_STALL_DELAY_1_REG] : TD_edc_stall_delay_1;
   wire [`DIDT_TD_EDC_STALL_DELAY_2_REG]      TD_edc_stall_delay_2_d    = wr_TD_edc_stall_delay_2    ? reg_wd : TD_edc_stall_delay_2;
   wire [`DIDT_TD_EDC_STALL_DELAY_3_REG]      TD_edc_stall_delay_3_d    = wr_TD_edc_stall_delay_3    ? reg_wd : TD_edc_stall_delay_3;


   reg [`DIDT_TCP_CTRL0_REG]       didt_TCP_ctrl0;
   reg [`DIDT_TCP_CTRL1_REG]       didt_TCP_ctrl1;
   reg [`DIDT_TCP_CTRL2_REG]       didt_TCP_ctrl2;
   reg [`DIDT_TCP_CTRL_OCP_REG]    didt_TCP_ctrl_ocp;
   reg [`DIDT_TCP_CTRL3_REG]       didt_TCP_ctrl3;
   reg [`DIDT_TCP_STALL_CTRL_REG]       didt_TCP_stall_ctrl;
   reg [`DIDT_TCP_TUNING_CTRL_REG]       didt_TCP_tuning_ctrl;
   reg [`DIDT_TCP_STALL_AUTO_RELEASE_CTRL_REG]       didt_TCP_stall_auto_release_ctrl;
   reg [`DIDT_TCP_WEIGHT0_3_REG]   didt_TCP_weight_0;
   reg [`DIDT_TCP_WEIGHT4_7_REG]   didt_TCP_weight_1;
   reg [`DIDT_TCP_WEIGHT8_11_REG]  didt_TCP_weight_2;

   reg [`DIDT_TCP_MPD_SCALE_FACTOR_REG] didt_TCP_mpd_scale_factor;

   reg [`DIDT_TCP_STALL_RELEASE_CNTL0_REG]  didt_TCP_stall_release_cntl0;
   reg [`DIDT_TCP_STALL_RELEASE_CNTL1_REG]  didt_TCP_stall_release_cntl1;

   reg [`DIDT_TCP_STALL_PATTERN_1_2_REG]       didt_TCP_didt_stall_pattern_1_2;
   reg [`DIDT_TCP_STALL_PATTERN_3_4_REG]       didt_TCP_didt_stall_pattern_3_4;
   reg [`DIDT_TCP_STALL_PATTERN_5_6_REG]       didt_TCP_didt_stall_pattern_5_6;
   reg [`DIDT_TCP_STALL_PATTERN_7_REG]         didt_TCP_didt_stall_pattern_7;

   reg [`DIDT_TCP_EDC_CTRL_REG]       TCP_edc_ctrl;
   reg [`DIDT_TCP_EDC_THRESHOLD_REG]       TCP_edc_threshold_reg;
   reg [`DIDT_TCP_EDC_STALL_PATTERN_1_2_REG]       TCP_edc_stall_pattern_1_2;
   reg [`DIDT_TCP_EDC_STALL_PATTERN_3_4_REG]       TCP_edc_stall_pattern_3_4;
   reg [`DIDT_TCP_EDC_STALL_PATTERN_5_6_REG]       TCP_edc_stall_pattern_5_6;
   reg [`DIDT_TCP_EDC_STALL_PATTERN_7_REG]       TCP_edc_stall_pattern_7;
   reg [`DIDT_TCP_EDC_TIMER_PERIOD_EDC_TIMER_PERIOD_SIZE-1:0]          TCP_edc_timer_period_reg;
   reg [`DIDT_TCP_THROTTLE_CTRL_REG]          	TCP_throttle_ctrl_reg;
   reg [`DIDT_TCP_EDC_STALL_DELAY_1_REG]       TCP_edc_stall_delay_1;
   reg [`DIDT_TCP_EDC_STALL_DELAY_2_REG]       TCP_edc_stall_delay_2;
   reg [`DIDT_TCP_EDC_STALL_DELAY_3_REG]       TCP_edc_stall_delay_3;

   wire  wr_didt_TCP_ctrl0    = (wr_ind_data & (didt_ind_index == `DIDT_TCP_CTRL0_DEC_ADDR));
   wire  wr_didt_TCP_ctrl1    = (wr_ind_data & (didt_ind_index == `DIDT_TCP_CTRL1_DEC_ADDR));
   wire  wr_didt_TCP_ctrl2    = (wr_ind_data & (didt_ind_index == `DIDT_TCP_CTRL2_DEC_ADDR));
   wire  wr_didt_TCP_ctrl3    = (wr_ind_data & (didt_ind_index == `DIDT_TCP_CTRL3_DEC_ADDR));
   wire  wr_didt_TCP_ctrl_ocp   = (wr_ind_data & (didt_ind_index == `DIDT_TCP_CTRL_OCP_DEC_ADDR));
   wire  wr_didt_TCP_stall_ctrl    = (wr_ind_data & (didt_ind_index == `DIDT_TCP_STALL_CTRL_DEC_ADDR));
   wire  wr_didt_TCP_tuning_ctrl    = (wr_ind_data & (didt_ind_index == `DIDT_TCP_TUNING_CTRL_DEC_ADDR));
   wire  wr_didt_TCP_stall_auto_release_ctrl    = (wr_ind_data & (didt_ind_index == `DIDT_TCP_STALL_AUTO_RELEASE_CTRL_DEC_ADDR));
   wire  wr_didt_TCP_weight_0 = (wr_ind_data & (didt_ind_index == `DIDT_TCP_WEIGHT0_3_DEC_ADDR));
   wire  wr_didt_TCP_weight_1 = (wr_ind_data & (didt_ind_index == `DIDT_TCP_WEIGHT4_7_DEC_ADDR));
   wire  wr_didt_TCP_weight_2 = (wr_ind_data & (didt_ind_index == `DIDT_TCP_WEIGHT8_11_DEC_ADDR));
   wire  wr_didt_TCP_mpd_scale_factor = (wr_ind_data & (didt_ind_index == `DIDT_TCP_MPD_SCALE_FACTOR_DEC_ADDR));
   wire  wr_didt_TCP_stall_release_cntl0 = (wr_ind_data & (didt_ind_index == `DIDT_TCP_STALL_RELEASE_CNTL0_DEC_ADDR));
   wire  wr_didt_TCP_stall_release_cntl1 = (wr_ind_data & (didt_ind_index == `DIDT_TCP_STALL_RELEASE_CNTL1_DEC_ADDR));

   wire  wr_didt_TCP_didt_stall_pattern_1_2    = (wr_ind_data & (didt_ind_index == `DIDT_TCP_STALL_PATTERN_1_2_DEC_ADDR));
   wire  wr_didt_TCP_didt_stall_pattern_3_4    = (wr_ind_data & (didt_ind_index == `DIDT_TCP_STALL_PATTERN_3_4_DEC_ADDR));
   wire  wr_didt_TCP_didt_stall_pattern_5_6    = (wr_ind_data & (didt_ind_index == `DIDT_TCP_STALL_PATTERN_5_6_DEC_ADDR));
   wire  wr_didt_TCP_didt_stall_pattern_7      = (wr_ind_data & (didt_ind_index == `DIDT_TCP_STALL_PATTERN_7_DEC_ADDR));

   wire  wr_TCP_edc_ctrl    				= (wr_ind_data & (didt_ind_index == `DIDT_TCP_EDC_CTRL_DEC_ADDR));
   wire  wr_TCP_edc_threshold    		= (wr_ind_data & (didt_ind_index == `DIDT_TCP_EDC_THRESHOLD_DEC_ADDR));
   wire  wr_TCP_edc_stall_pattern_1_2    = (wr_ind_data & (didt_ind_index == `DIDT_TCP_EDC_STALL_PATTERN_1_2_DEC_ADDR));
   wire  wr_TCP_edc_stall_pattern_3_4    = (wr_ind_data & (didt_ind_index == `DIDT_TCP_EDC_STALL_PATTERN_3_4_DEC_ADDR));
   wire  wr_TCP_edc_stall_pattern_5_6    = (wr_ind_data & (didt_ind_index == `DIDT_TCP_EDC_STALL_PATTERN_5_6_DEC_ADDR));
   wire  wr_TCP_edc_stall_pattern_7    	= (wr_ind_data & (didt_ind_index == `DIDT_TCP_EDC_STALL_PATTERN_7_DEC_ADDR));
   wire  wr_TCP_edc_timer_period             = (wr_ind_data & (didt_ind_index == `DIDT_TCP_EDC_TIMER_PERIOD_DEC_ADDR));
   wire  wr_TCP_throttle_ctrl             = (wr_ind_data & (didt_ind_index == `DIDT_TCP_THROTTLE_CTRL_DEC_ADDR));
   wire  wr_TCP_edc_stall_delay_1    	= (wr_ind_data & (didt_ind_index == `DIDT_TCP_EDC_STALL_DELAY_1_DEC_ADDR));
   wire  wr_TCP_edc_stall_delay_2    = (wr_ind_data & (didt_ind_index == `DIDT_TCP_EDC_STALL_DELAY_2_DEC_ADDR));
   wire  wr_TCP_edc_stall_delay_3    = (wr_ind_data & (didt_ind_index == `DIDT_TCP_EDC_STALL_DELAY_3_DEC_ADDR));
   wire  wr_TCP_edc_grbm_wr_pwrdelta_en    = (wr_ind_data & (didt_ind_index == `DIDT_TCP_EDC_ROLLING_POWER_DELTA_DEC_ADDR));
   assign TCP_edc_grbm_wr_pwrdelta_en = wr_TCP_edc_grbm_wr_pwrdelta_en;
   assign TCP_edc_grbm_wr_pwrdelta_data = reg_wd;  //reg_wd only be sampled when wr_didt_edc_grbm_wr_pwrdelta_data_en is valid in DIDT_CNTL_CORE
   //write-clear
   wire  wr_TCP_edc_overflow_rolling_power_delta_overflow    = (wr_ind_data & (didt_ind_index == `DIDT_TCP_EDC_OVERFLOW_DEC_ADDR) & (reg_wd[`DIDT_TCP_EDC_OVERFLOW_EDC_ROLLING_POWER_DELTA_OVERFLOW] == 'd1));
   assign TCP_edc_grbm_clear_rolling_power_delta_ov = wr_TCP_edc_overflow_rolling_power_delta_overflow;

   wire [`DIDT_TCP_CTRL0_REG]      didt_TCP_ctrl0_d    = wr_didt_TCP_ctrl0    ? reg_wd : didt_TCP_ctrl0;
   wire [`DIDT_TCP_CTRL1_REG]      didt_TCP_ctrl1_d    = wr_didt_TCP_ctrl1    ? reg_wd : didt_TCP_ctrl1;
   wire [`DIDT_TCP_CTRL2_REG]      didt_TCP_ctrl2_d    = wr_didt_TCP_ctrl2    ? reg_wd : didt_TCP_ctrl2;
   wire [`DIDT_TCP_CTRL3_REG]      didt_TCP_ctrl3_d    = wr_didt_TCP_ctrl3    ? reg_wd[`DIDT_TCP_CTRL3_REG] : didt_TCP_ctrl3;
   wire [`DIDT_TCP_CTRL_OCP_REG]      didt_TCP_ctrl_ocp_d    = wr_didt_TCP_ctrl_ocp    ? reg_wd : didt_TCP_ctrl_ocp;
   wire [`DIDT_TCP_STALL_CTRL_REG]      didt_TCP_stall_ctrl_d    = wr_didt_TCP_stall_ctrl    ? reg_wd : didt_TCP_stall_ctrl;
   wire [`DIDT_TCP_TUNING_CTRL_REG]      didt_TCP_tuning_ctrl_d    = wr_didt_TCP_tuning_ctrl    ? reg_wd[`DIDT_TCP_TUNING_CTRL_REG] : didt_TCP_tuning_ctrl;
   wire [`DIDT_TCP_STALL_AUTO_RELEASE_CTRL_REG]      didt_TCP_stall_auto_release_ctrl_d    = wr_didt_TCP_stall_auto_release_ctrl    ? reg_wd[`DIDT_TCP_STALL_AUTO_RELEASE_CTRL_REG] : didt_TCP_stall_auto_release_ctrl;
   wire [`DIDT_TCP_WEIGHT0_3_REG]  didt_TCP_weight_0_d = wr_didt_TCP_weight_0 ? reg_wd : didt_TCP_weight_0;
   wire [`DIDT_TCP_WEIGHT4_7_REG]  didt_TCP_weight_1_d = wr_didt_TCP_weight_1 ? reg_wd : didt_TCP_weight_1;
   wire [`DIDT_TCP_WEIGHT8_11_REG] didt_TCP_weight_2_d = wr_didt_TCP_weight_2 ? reg_wd : didt_TCP_weight_2;
   wire [`DIDT_TCP_MPD_SCALE_FACTOR_REG] didt_TCP_mpd_scale_factor_d = wr_didt_TCP_mpd_scale_factor ? reg_wd : didt_TCP_mpd_scale_factor;
   wire [`DIDT_TCP_STALL_RELEASE_CNTL0_REG] didt_TCP_stall_release_cntl0_d = wr_didt_TCP_stall_release_cntl0 ? reg_wd : didt_TCP_stall_release_cntl0;
   wire [`DIDT_TCP_STALL_RELEASE_CNTL1_REG] didt_TCP_stall_release_cntl1_d = wr_didt_TCP_stall_release_cntl1 ? reg_wd : didt_TCP_stall_release_cntl1;   
   wire [`DIDT_TCP_STALL_PATTERN_1_2_REG]      didt_TCP_didt_stall_pattern_1_2_d    = wr_didt_TCP_didt_stall_pattern_1_2    ? reg_wd : didt_TCP_didt_stall_pattern_1_2;
   wire [`DIDT_TCP_STALL_PATTERN_3_4_REG]      didt_TCP_didt_stall_pattern_3_4_d    = wr_didt_TCP_didt_stall_pattern_3_4    ? reg_wd : didt_TCP_didt_stall_pattern_3_4;
   wire [`DIDT_TCP_STALL_PATTERN_5_6_REG]      didt_TCP_didt_stall_pattern_5_6_d    = wr_didt_TCP_didt_stall_pattern_5_6    ? reg_wd : didt_TCP_didt_stall_pattern_5_6;
   wire [`DIDT_TCP_STALL_PATTERN_7_REG]        didt_TCP_didt_stall_pattern_7_d    = wr_didt_TCP_didt_stall_pattern_7    ? reg_wd : didt_TCP_didt_stall_pattern_7;


   wire [`DIDT_TCP_EDC_CTRL_REG]      TCP_edc_ctrl_d    = wr_TCP_edc_ctrl    ? reg_wd : TCP_edc_ctrl;
   wire [`DIDT_TCP_EDC_THRESHOLD_REG]      TCP_edc_threshold_d    = wr_TCP_edc_threshold    ? reg_wd : TCP_edc_threshold_reg;
   wire [`DIDT_TCP_EDC_STALL_PATTERN_1_2_REG]      TCP_edc_stall_pattern_1_2_d    = wr_TCP_edc_stall_pattern_1_2    ? reg_wd : TCP_edc_stall_pattern_1_2;
   wire [`DIDT_TCP_EDC_STALL_PATTERN_3_4_REG]      TCP_edc_stall_pattern_3_4_d    = wr_TCP_edc_stall_pattern_3_4    ? reg_wd : TCP_edc_stall_pattern_3_4;
   wire [`DIDT_TCP_EDC_STALL_PATTERN_5_6_REG]      TCP_edc_stall_pattern_5_6_d    = wr_TCP_edc_stall_pattern_5_6    ? reg_wd : TCP_edc_stall_pattern_5_6;
   wire [`DIDT_TCP_EDC_STALL_PATTERN_7_REG]        TCP_edc_stall_pattern_7_d   	= wr_TCP_edc_stall_pattern_7      ? reg_wd : TCP_edc_stall_pattern_7;
   wire [`DIDT_TCP_EDC_TIMER_PERIOD_REG]          TCP_edc_timer_period_d   	= wr_TCP_edc_timer_period         ? reg_wd : TCP_edc_timer_period_reg;
   wire [`DIDT_TCP_THROTTLE_CTRL_REG]          TCP_throttle_ctrl_d   	= wr_TCP_throttle_ctrl         ? reg_wd : TCP_throttle_ctrl_reg;
   assign rd_TCP_edc_pcc_perf_counter		= (rd_ind_data & (didt_ind_index == `DIDT_TCP_EDC_PCC_PERF_COUNTER_DEC_ADDR));
   wire [`DIDT_TCP_EDC_STALL_DELAY_1_REG]      TCP_edc_stall_delay_1_d    = wr_TCP_edc_stall_delay_1    ? reg_wd[`DIDT_TCP_EDC_STALL_DELAY_1_REG] : TCP_edc_stall_delay_1;
   wire [`DIDT_TCP_EDC_STALL_DELAY_2_REG]      TCP_edc_stall_delay_2_d    = wr_TCP_edc_stall_delay_2    ? reg_wd : TCP_edc_stall_delay_2;
   wire [`DIDT_TCP_EDC_STALL_DELAY_3_REG]      TCP_edc_stall_delay_3_d    = wr_TCP_edc_stall_delay_3    ? reg_wd : TCP_edc_stall_delay_3;



// DIDT output assignments for the SQ block
   assign  didt_SQ_didt_ctrl_en         = didt_SQ_ctrl0[`DIDT_SQ_CTRL0_DIDT_CTRL_EN];
   assign  didt_SQ_phase_offset         = didt_SQ_ctrl0[`DIDT_SQ_CTRL0_PHASE_OFFSET];
   assign  didt_SQ_didt_ctrl_rst        = didt_SQ_ctrl0[`DIDT_SQ_CTRL0_DIDT_CTRL_RST];
   assign  didt_SQ_didt_clk_en_override = didt_SQ_ctrl0[`DIDT_SQ_CTRL0_DIDT_CLK_EN_OVERRIDE];
   assign  didt_SQ_stall_ctrl_enable           = didt_SQ_ctrl0[`DIDT_SQ_CTRL0_DIDT_STALL_CTRL_EN];
	assign didt_SQ_tuning_enable              = didt_SQ_ctrl0[`DIDT_SQ_CTRL0_DIDT_TUNING_CTRL_EN];
	assign didt_SQ_stall_auto_release_enable              = didt_SQ_ctrl0[`DIDT_SQ_CTRL0_DIDT_STALL_AUTO_RELEASE_EN];
   assign  didt_SQ_hi_power_threshold            = didt_SQ_ctrl0[`DIDT_SQ_CTRL0_DIDT_HI_POWER_THRESHOLD];
	assign didt_SQ_auto_mpd_enable              = didt_SQ_ctrl0[`DIDT_SQ_CTRL0_DIDT_AUTO_MPD_EN];
	assign didt_SQ_stall_event_enable           = didt_SQ_ctrl0[`DIDT_SQ_CTRL0_DIDT_STALL_EVENT_EN];
	assign didt_SQ_stall_event_counter_clear    = wr_didt_SQ_ctrl0 && (reg_wd[`DIDT_SQ_CTRL0_DIDT_STALL_EVENT_COUNTER_CLEAR] == 'd1);
    assign didt_SQ_didt_rlc_force_stall_en           = didt_SQ_ctrl0[`DIDT_SQ_CTRL0_DIDT_RLC_FORCE_STALL_EN];
    assign didt_SQ_didt_rlc_stall_level_sel          = didt_SQ_ctrl0[`DIDT_SQ_CTRL0_DIDT_RLC_STALL_LEVEL_SEL];
    assign didt_SQ_didt_throttle_mode          = didt_SQ_ctrl0[`DIDT_SQ_CTRL0_DIDT_THROTTLE_MODE];

   assign  didt_SQ_min_power            = didt_SQ_ctrl1[`DIDT_SQ_CTRL1_MIN_POWER];
   assign  didt_SQ_max_power            = didt_SQ_ctrl1[`DIDT_SQ_CTRL1_MAX_POWER];

   assign  didt_SQ_max_power_delta          = didt_SQ_ctrl2[`DIDT_SQ_CTRL2_MAX_POWER_DELTA];
   assign  didt_SQ_short_term_interval_size = didt_SQ_ctrl2[`DIDT_SQ_CTRL2_SHORT_TERM_INTERVAL_SIZE ];
   assign  didt_SQ_long_term_interval_ratio = didt_SQ_ctrl2[`DIDT_SQ_CTRL2_LONG_TERM_INTERVAL_RATIO];

	assign didt_SQ_ocp_max_power = didt_SQ_ctrl_ocp[`DIDT_SQ_CTRL_OCP_OCP_MAX_POWER];

   assign  didt_SQ_stall_delay_hi            = didt_SQ_stall_ctrl[`DIDT_SQ_STALL_CTRL_DIDT_STALL_DELAY_HI];
   assign  didt_SQ_stall_delay_lo            = didt_SQ_stall_ctrl[`DIDT_SQ_STALL_CTRL_DIDT_STALL_DELAY_LO];
   assign  didt_SQ_didt_max_stalls_allowed_hi = didt_SQ_stall_ctrl[`DIDT_SQ_STALL_CTRL_DIDT_MAX_STALLS_ALLOWED_HI];
   assign  didt_SQ_didt_max_stalls_allowed_lo = didt_SQ_stall_ctrl[`DIDT_SQ_STALL_CTRL_DIDT_MAX_STALLS_ALLOWED_LO];

	assign didt_SQ_max_power_delta_hi         = didt_SQ_tuning_ctrl[`DIDT_SQ_TUNING_CTRL_MAX_POWER_DELTA_HI];
	assign didt_SQ_max_power_delta_lo         = didt_SQ_tuning_ctrl[`DIDT_SQ_TUNING_CTRL_MAX_POWER_DELTA_LO];

	assign didt_SQ_stall_auto_release_time              = didt_SQ_stall_auto_release_ctrl[`DIDT_SQ_STALL_AUTO_RELEASE_CTRL_DIDT_STALL_AUTO_RELEASE_TIME];

   assign  didt_SQ_weight0 = didt_SQ_weight_0[`DIDT_SQ_WEIGHT0_3_WEIGHT0];
   assign  didt_SQ_weight1 = didt_SQ_weight_0[`DIDT_SQ_WEIGHT0_3_WEIGHT1];
   assign  didt_SQ_weight2 = didt_SQ_weight_0[`DIDT_SQ_WEIGHT0_3_WEIGHT2];
   assign  didt_SQ_weight3 = didt_SQ_weight_0[`DIDT_SQ_WEIGHT0_3_WEIGHT3];

   assign  didt_SQ_weight4 = didt_SQ_weight_1[`DIDT_SQ_WEIGHT4_7_WEIGHT4];
   assign  didt_SQ_weight5 = didt_SQ_weight_1[`DIDT_SQ_WEIGHT4_7_WEIGHT5];
   assign  didt_SQ_weight6 = didt_SQ_weight_1[`DIDT_SQ_WEIGHT4_7_WEIGHT6];
   assign  didt_SQ_weight7 = didt_SQ_weight_1[`DIDT_SQ_WEIGHT4_7_WEIGHT7];

   assign  didt_SQ_weight8  = didt_SQ_weight_2[`DIDT_SQ_WEIGHT8_11_WEIGHT8];
   assign  didt_SQ_weight9  = didt_SQ_weight_2[`DIDT_SQ_WEIGHT8_11_WEIGHT9];
   assign  didt_SQ_weight10 = didt_SQ_weight_2[`DIDT_SQ_WEIGHT8_11_WEIGHT10];
   assign  didt_SQ_weight11 = didt_SQ_weight_2[`DIDT_SQ_WEIGHT8_11_WEIGHT11];

   assign  didt_SQ_mpd_ratio_scale_level1 = didt_SQ_mpd_scale_factor[`DIDT_SQ_MPD_SCALE_FACTOR_MPD_RATIO_SCALE_LEVEL1];
   assign  didt_SQ_mpd_ratio_scale_level2 = didt_SQ_mpd_scale_factor[`DIDT_SQ_MPD_SCALE_FACTOR_MPD_RATIO_SCALE_LEVEL2];
   assign  didt_SQ_mpd_ratio_scale_level3 = didt_SQ_mpd_scale_factor[`DIDT_SQ_MPD_SCALE_FACTOR_MPD_RATIO_SCALE_LEVEL3];
   assign  didt_SQ_mpd_ratio_scale_level4 = didt_SQ_mpd_scale_factor[`DIDT_SQ_MPD_SCALE_FACTOR_MPD_RATIO_SCALE_LEVEL4];
   assign  didt_SQ_mpd_scale_level0 = didt_SQ_mpd_scale_factor[`DIDT_SQ_MPD_SCALE_FACTOR_MPD_SCALE_LEVEL0];
   assign  didt_SQ_mpd_scale_level1 = didt_SQ_mpd_scale_factor[`DIDT_SQ_MPD_SCALE_FACTOR_MPD_SCALE_LEVEL1];
   assign  didt_SQ_mpd_scale_level2 = didt_SQ_mpd_scale_factor[`DIDT_SQ_MPD_SCALE_FACTOR_MPD_SCALE_LEVEL2];
   assign  didt_SQ_mpd_scale_level3 = didt_SQ_mpd_scale_factor[`DIDT_SQ_MPD_SCALE_FACTOR_MPD_SCALE_LEVEL3];

   assign  didt_SQ_didt_stall_release_cntl_en = didt_SQ_stall_release_cntl0[`DIDT_SQ_STALL_RELEASE_CNTL0_DIDT_STALL_RELEASE_CNTL_EN];
   assign  didt_SQ_didt_stall_cntl_sel = didt_SQ_stall_release_cntl0[`DIDT_SQ_STALL_RELEASE_CNTL0_DIDT_STALL_CNTL_SEL];
   assign  didt_SQ_didt_release_delay_hi = didt_SQ_stall_release_cntl0[`DIDT_SQ_STALL_RELEASE_CNTL0_DIDT_RELEASE_DELAY_HI];
   assign  didt_SQ_didt_release_delay_lo = didt_SQ_stall_release_cntl0[`DIDT_SQ_STALL_RELEASE_CNTL0_DIDT_RELEASE_DELAY_LO];

   assign  didt_SQ_didt_base_release_allowed_hi = didt_SQ_stall_release_cntl1[`DIDT_SQ_STALL_RELEASE_CNTL1_DIDT_BASE_RELEASE_ALLOWED_HI];
   assign  didt_SQ_didt_incr_release_allowed_hi = didt_SQ_stall_release_cntl1[`DIDT_SQ_STALL_RELEASE_CNTL1_DIDT_INCR_RELEASE_ALLOWED_HI];
   assign  didt_SQ_didt_base_release_allowed_lo = didt_SQ_stall_release_cntl1[`DIDT_SQ_STALL_RELEASE_CNTL1_DIDT_BASE_RELEASE_ALLOWED_LO];
   assign  didt_SQ_didt_incr_release_allowed_lo = didt_SQ_stall_release_cntl1[`DIDT_SQ_STALL_RELEASE_CNTL1_DIDT_INCR_RELEASE_ALLOWED_LO];

   assign  didt_SQ_gc_didt_enable         = didt_SQ_ctrl3[`DIDT_SQ_CTRL3_GC_DIDT_ENABLE];
   assign  didt_SQ_gc_didt_clk_en_override         = didt_SQ_ctrl3[`DIDT_SQ_CTRL3_GC_DIDT_CLK_EN_OVERRIDE];
   assign  didt_SQ_throttle_policy         = didt_SQ_ctrl3[`DIDT_SQ_CTRL3_THROTTLE_POLICY];
   assign  didt_SQ_didt_trigger_throttle_lowbit        = didt_SQ_ctrl3[`DIDT_SQ_CTRL3_DIDT_TRIGGER_THROTTLE_LOWBIT];
   assign  didt_SQ_didt_power_level_lowbit        = didt_SQ_ctrl3[`DIDT_SQ_CTRL3_DIDT_POWER_LEVEL_LOWBIT];
   assign  didt_SQ_didt_stall_pattern_bit_nums         = didt_SQ_ctrl3[`DIDT_SQ_CTRL3_DIDT_STALL_PATTERN_BIT_NUMS];
   assign  didt_SQ_gc_didt_level_comb_en         = didt_SQ_ctrl3[`DIDT_SQ_CTRL3_GC_DIDT_LEVEL_COMB_EN];
   assign  didt_SQ_se_didt_level_comb_en         = didt_SQ_ctrl3[`DIDT_SQ_CTRL3_SE_DIDT_LEVEL_COMB_EN];
   assign  didt_SQ_qualify_stall_en         = didt_SQ_ctrl3[`DIDT_SQ_CTRL3_QUALIFY_STALL_EN];
   assign  didt_SQ_didt_stall_sel         = didt_SQ_ctrl3[`DIDT_SQ_CTRL3_DIDT_STALL_SEL];
   assign  didt_SQ_didt_force_stall       = didt_SQ_ctrl3[`DIDT_SQ_CTRL3_DIDT_FORCE_STALL];
   assign  didt_SQ_didt_stall_delay_en      = didt_SQ_ctrl3[`DIDT_SQ_CTRL3_DIDT_STALL_DELAY_EN];

   assign didt_SQ_stall_pattern1 = didt_SQ_didt_stall_pattern_1_2[`DIDT_SQ_STALL_PATTERN_1_2_DIDT_STALL_PATTERN_1];
   assign didt_SQ_stall_pattern2 = didt_SQ_didt_stall_pattern_1_2[`DIDT_SQ_STALL_PATTERN_1_2_DIDT_STALL_PATTERN_2];
   assign didt_SQ_stall_pattern3 = didt_SQ_didt_stall_pattern_3_4[`DIDT_SQ_STALL_PATTERN_3_4_DIDT_STALL_PATTERN_3];
   assign didt_SQ_stall_pattern4 = didt_SQ_didt_stall_pattern_3_4[`DIDT_SQ_STALL_PATTERN_3_4_DIDT_STALL_PATTERN_4];
   assign didt_SQ_stall_pattern5 = didt_SQ_didt_stall_pattern_5_6[`DIDT_SQ_STALL_PATTERN_5_6_DIDT_STALL_PATTERN_5];
   assign didt_SQ_stall_pattern6 = didt_SQ_didt_stall_pattern_5_6[`DIDT_SQ_STALL_PATTERN_5_6_DIDT_STALL_PATTERN_6];
   assign didt_SQ_stall_pattern7 = didt_SQ_didt_stall_pattern_7[`DIDT_SQ_STALL_PATTERN_7_DIDT_STALL_PATTERN_7];

   assign SQ_edc_en = SQ_edc_ctrl[`DIDT_SQ_EDC_CTRL_EDC_EN];
   assign SQ_edc_sw_rst = SQ_edc_ctrl[`DIDT_SQ_EDC_CTRL_EDC_SW_RST];
   assign SQ_edc_clk_en_override = SQ_edc_ctrl[`DIDT_SQ_EDC_CTRL_EDC_CLK_EN_OVERRIDE];
   assign SQ_edc_force_stall = SQ_edc_ctrl[`DIDT_SQ_EDC_CTRL_EDC_FORCE_STALL];
   assign SQ_edc_trigger_throttle_lowbit = SQ_edc_ctrl[`DIDT_SQ_EDC_CTRL_EDC_TRIGGER_THROTTLE_LOWBIT];
   assign SQ_edc_stall_pattern_bit_nums = SQ_edc_ctrl[`DIDT_SQ_EDC_CTRL_EDC_STALL_PATTERN_BIT_NUMS];
   assign SQ_edc_allow_write_pwrdelta = SQ_edc_ctrl[`DIDT_SQ_EDC_CTRL_EDC_ALLOW_WRITE_PWRDELTA];
   assign SQ_gc_edc_en = SQ_edc_ctrl[`DIDT_SQ_EDC_CTRL_GC_EDC_EN];
   assign SQ_gc_edc_stall_policy = SQ_edc_ctrl[`DIDT_SQ_EDC_CTRL_GC_EDC_STALL_POLICY];
   assign SQ_gc_edc_level_comb_en = SQ_edc_ctrl[`DIDT_SQ_EDC_CTRL_GC_EDC_LEVEL_COMB_EN];
   assign SQ_se_edc_level_comb_en = SQ_edc_ctrl[`DIDT_SQ_EDC_CTRL_SE_EDC_LEVEL_COMB_EN];
   assign SQ_edc_level_mode_sel = SQ_edc_ctrl[`DIDT_SQ_EDC_CTRL_EDC_LEVEL_MODE_SEL];
   assign SQ_edc_level_comb_adapt_mode_en = SQ_edc_ctrl[`DIDT_SQ_EDC_CTRL_EDC_LEVEL_COMB_ADAPT_MODE_EN];
   assign SQ_edc_grbm_clear_overflow_counter = rd_ind_data & (didt_ind_index == `DIDT_SQ_EDC_OVERFLOW_DEC_ADDR);
   assign SQ_edc_threshold = SQ_edc_threshold_reg[`DIDT_SQ_EDC_THRESHOLD_EDC_THRESHOLD];
   assign SQ_edc_stall_pattern1 = SQ_edc_stall_pattern_1_2[`DIDT_SQ_EDC_STALL_PATTERN_1_2_EDC_STALL_PATTERN_1];
   assign SQ_edc_stall_pattern2 = SQ_edc_stall_pattern_1_2[`DIDT_SQ_EDC_STALL_PATTERN_1_2_EDC_STALL_PATTERN_2];
   assign SQ_edc_stall_pattern3 = SQ_edc_stall_pattern_3_4[`DIDT_SQ_EDC_STALL_PATTERN_3_4_EDC_STALL_PATTERN_3];
   assign SQ_edc_stall_pattern4 = SQ_edc_stall_pattern_3_4[`DIDT_SQ_EDC_STALL_PATTERN_3_4_EDC_STALL_PATTERN_4];
   assign SQ_edc_stall_pattern5 = SQ_edc_stall_pattern_5_6[`DIDT_SQ_EDC_STALL_PATTERN_5_6_EDC_STALL_PATTERN_5];
   assign SQ_edc_stall_pattern6 = SQ_edc_stall_pattern_5_6[`DIDT_SQ_EDC_STALL_PATTERN_5_6_EDC_STALL_PATTERN_6];
   assign SQ_edc_stall_pattern7 = SQ_edc_stall_pattern_7[`DIDT_SQ_EDC_STALL_PATTERN_7_EDC_STALL_PATTERN_7];
   assign SQ_edc_timer_period   = SQ_edc_timer_period_reg[`DIDT_SQ_EDC_TIMER_PERIOD_EDC_TIMER_PERIOD];
   assign SQ_gc_edc_stall_en   = SQ_throttle_ctrl_reg[`DIDT_SQ_THROTTLE_CTRL_GC_EDC_STALL_EN];
   assign SQ_pcc_stall_en   = SQ_throttle_ctrl_reg[`DIDT_SQ_THROTTLE_CTRL_PCC_STALL_EN];
   assign SQ_pwrbrk_stall_en   = SQ_throttle_ctrl_reg[`DIDT_SQ_THROTTLE_CTRL_PWRBRK_STALL_EN];
   assign SQ_gc_edc_only_mode   = SQ_throttle_ctrl_reg[`DIDT_SQ_THROTTLE_CTRL_GC_EDC_ONLY_MODE];
   assign SQ_edc_stall_delay_sig0 = SQ_edc_stall_delay_1[`DIDT_SQ_EDC_STALL_DELAY_1_EDC_STALL_DELAY_SQ0];
   assign SQ_edc_stall_delay_sig1 = SQ_edc_stall_delay_1[`DIDT_SQ_EDC_STALL_DELAY_1_EDC_STALL_DELAY_SQ1];
   assign SQ_edc_stall_delay_sig2 = SQ_edc_stall_delay_1[`DIDT_SQ_EDC_STALL_DELAY_1_EDC_STALL_DELAY_SQ2];
   assign SQ_edc_stall_delay_sig3 = SQ_edc_stall_delay_1[`DIDT_SQ_EDC_STALL_DELAY_1_EDC_STALL_DELAY_SQ3];
   assign SQ_edc_stall_delay_sig4 = SQ_edc_stall_delay_2[`DIDT_SQ_EDC_STALL_DELAY_2_EDC_STALL_DELAY_SQ4];
   assign SQ_edc_stall_delay_sig5 = SQ_edc_stall_delay_2[`DIDT_SQ_EDC_STALL_DELAY_2_EDC_STALL_DELAY_SQ5];
   assign SQ_edc_stall_delay_sig6 = SQ_edc_stall_delay_2[`DIDT_SQ_EDC_STALL_DELAY_2_EDC_STALL_DELAY_SQ6];
   assign SQ_edc_stall_delay_sig7 = SQ_edc_stall_delay_2[`DIDT_SQ_EDC_STALL_DELAY_2_EDC_STALL_DELAY_SQ7];
   assign SQ_edc_stall_delay_sig8 = SQ_edc_stall_delay_3[`DIDT_SQ_EDC_STALL_DELAY_3_EDC_STALL_DELAY_SQ8];
   assign SQ_edc_stall_delay_sig9 = SQ_edc_stall_delay_3[`DIDT_SQ_EDC_STALL_DELAY_3_EDC_STALL_DELAY_SQ9];

// DIDT output assignments for the DB block
   assign  didt_DB_didt_ctrl_en         = didt_DB_ctrl0[`DIDT_DB_CTRL0_DIDT_CTRL_EN];
   assign  didt_DB_phase_offset         = didt_DB_ctrl0[`DIDT_DB_CTRL0_PHASE_OFFSET];
   assign  didt_DB_didt_ctrl_rst        = didt_DB_ctrl0[`DIDT_DB_CTRL0_DIDT_CTRL_RST];
   assign  didt_DB_didt_clk_en_override = didt_DB_ctrl0[`DIDT_DB_CTRL0_DIDT_CLK_EN_OVERRIDE];
   assign  didt_DB_stall_ctrl_enable           = didt_DB_ctrl0[`DIDT_DB_CTRL0_DIDT_STALL_CTRL_EN];
	assign didt_DB_tuning_enable              = didt_DB_ctrl0[`DIDT_DB_CTRL0_DIDT_TUNING_CTRL_EN];
	assign didt_DB_stall_auto_release_enable              = didt_DB_ctrl0[`DIDT_DB_CTRL0_DIDT_STALL_AUTO_RELEASE_EN];
   assign  didt_DB_hi_power_threshold            = didt_DB_ctrl0[`DIDT_DB_CTRL0_DIDT_HI_POWER_THRESHOLD];
	assign didt_DB_auto_mpd_enable              = didt_DB_ctrl0[`DIDT_DB_CTRL0_DIDT_AUTO_MPD_EN];
	assign didt_DB_stall_event_enable           = didt_DB_ctrl0[`DIDT_DB_CTRL0_DIDT_STALL_EVENT_EN];
	assign didt_DB_stall_event_counter_clear    = wr_didt_DB_ctrl0 && (reg_wd[`DIDT_DB_CTRL0_DIDT_STALL_EVENT_COUNTER_CLEAR] == 'd1);
    assign didt_DB_didt_rlc_force_stall_en           = didt_DB_ctrl0[`DIDT_DB_CTRL0_DIDT_RLC_FORCE_STALL_EN];
    assign didt_DB_didt_rlc_stall_level_sel          = didt_DB_ctrl0[`DIDT_DB_CTRL0_DIDT_RLC_STALL_LEVEL_SEL];
    assign didt_DB_didt_throttle_mode          = didt_DB_ctrl0[`DIDT_DB_CTRL0_DIDT_THROTTLE_MODE];

   assign  didt_DB_min_power            = didt_DB_ctrl1[`DIDT_DB_CTRL1_MIN_POWER];
   assign  didt_DB_max_power            = didt_DB_ctrl1[`DIDT_DB_CTRL1_MAX_POWER];

   assign  didt_DB_max_power_delta          = didt_DB_ctrl2[`DIDT_DB_CTRL2_MAX_POWER_DELTA];
   assign  didt_DB_short_term_interval_size = didt_DB_ctrl2[`DIDT_DB_CTRL2_SHORT_TERM_INTERVAL_SIZE ];
   assign  didt_DB_long_term_interval_ratio = didt_DB_ctrl2[`DIDT_DB_CTRL2_LONG_TERM_INTERVAL_RATIO];

	assign didt_DB_ocp_max_power = didt_DB_ctrl_ocp[`DIDT_DB_CTRL_OCP_OCP_MAX_POWER];

   assign  didt_DB_stall_delay_hi            = didt_DB_stall_ctrl[`DIDT_DB_STALL_CTRL_DIDT_STALL_DELAY_HI];
   assign  didt_DB_stall_delay_lo            = didt_DB_stall_ctrl[`DIDT_DB_STALL_CTRL_DIDT_STALL_DELAY_LO];
   assign  didt_DB_didt_max_stalls_allowed_hi = didt_DB_stall_ctrl[`DIDT_DB_STALL_CTRL_DIDT_MAX_STALLS_ALLOWED_HI];
   assign  didt_DB_didt_max_stalls_allowed_lo = didt_DB_stall_ctrl[`DIDT_DB_STALL_CTRL_DIDT_MAX_STALLS_ALLOWED_LO];

	assign didt_DB_max_power_delta_hi         = didt_DB_tuning_ctrl[`DIDT_DB_TUNING_CTRL_MAX_POWER_DELTA_HI];
	assign didt_DB_max_power_delta_lo         = didt_DB_tuning_ctrl[`DIDT_DB_TUNING_CTRL_MAX_POWER_DELTA_LO];

	assign didt_DB_stall_auto_release_time              = didt_DB_stall_auto_release_ctrl[`DIDT_DB_STALL_AUTO_RELEASE_CTRL_DIDT_STALL_AUTO_RELEASE_TIME];

   assign  didt_DB_weight0 = didt_DB_weight_0[`DIDT_DB_WEIGHT0_3_WEIGHT0];
   assign  didt_DB_weight1 = didt_DB_weight_0[`DIDT_DB_WEIGHT0_3_WEIGHT1];
   assign  didt_DB_weight2 = didt_DB_weight_0[`DIDT_DB_WEIGHT0_3_WEIGHT2];
   assign  didt_DB_weight3 = didt_DB_weight_0[`DIDT_DB_WEIGHT0_3_WEIGHT3];

   assign  didt_DB_weight4 = didt_DB_weight_1[`DIDT_DB_WEIGHT4_7_WEIGHT4];
   assign  didt_DB_weight5 = didt_DB_weight_1[`DIDT_DB_WEIGHT4_7_WEIGHT5];
   assign  didt_DB_weight6 = didt_DB_weight_1[`DIDT_DB_WEIGHT4_7_WEIGHT6];
   assign  didt_DB_weight7 = didt_DB_weight_1[`DIDT_DB_WEIGHT4_7_WEIGHT7];

   assign  didt_DB_weight8  = didt_DB_weight_2[`DIDT_DB_WEIGHT8_11_WEIGHT8];
   assign  didt_DB_weight9  = didt_DB_weight_2[`DIDT_DB_WEIGHT8_11_WEIGHT9];
   assign  didt_DB_weight10 = didt_DB_weight_2[`DIDT_DB_WEIGHT8_11_WEIGHT10];
   assign  didt_DB_weight11 = didt_DB_weight_2[`DIDT_DB_WEIGHT8_11_WEIGHT11];

   assign  didt_DB_mpd_ratio_scale_level1 = didt_DB_mpd_scale_factor[`DIDT_DB_MPD_SCALE_FACTOR_MPD_RATIO_SCALE_LEVEL1];
   assign  didt_DB_mpd_ratio_scale_level2 = didt_DB_mpd_scale_factor[`DIDT_DB_MPD_SCALE_FACTOR_MPD_RATIO_SCALE_LEVEL2];
   assign  didt_DB_mpd_ratio_scale_level3 = didt_DB_mpd_scale_factor[`DIDT_DB_MPD_SCALE_FACTOR_MPD_RATIO_SCALE_LEVEL3];
   assign  didt_DB_mpd_ratio_scale_level4 = didt_DB_mpd_scale_factor[`DIDT_DB_MPD_SCALE_FACTOR_MPD_RATIO_SCALE_LEVEL4];
   assign  didt_DB_mpd_scale_level0 = didt_DB_mpd_scale_factor[`DIDT_DB_MPD_SCALE_FACTOR_MPD_SCALE_LEVEL0];
   assign  didt_DB_mpd_scale_level1 = didt_DB_mpd_scale_factor[`DIDT_DB_MPD_SCALE_FACTOR_MPD_SCALE_LEVEL1];
   assign  didt_DB_mpd_scale_level2 = didt_DB_mpd_scale_factor[`DIDT_DB_MPD_SCALE_FACTOR_MPD_SCALE_LEVEL2];
   assign  didt_DB_mpd_scale_level3 = didt_DB_mpd_scale_factor[`DIDT_DB_MPD_SCALE_FACTOR_MPD_SCALE_LEVEL3];

   assign  didt_DB_didt_stall_release_cntl_en = didt_DB_stall_release_cntl0[`DIDT_DB_STALL_RELEASE_CNTL0_DIDT_STALL_RELEASE_CNTL_EN];
   assign  didt_DB_didt_stall_cntl_sel = didt_DB_stall_release_cntl0[`DIDT_DB_STALL_RELEASE_CNTL0_DIDT_STALL_CNTL_SEL];
   assign  didt_DB_didt_release_delay_hi = didt_DB_stall_release_cntl0[`DIDT_DB_STALL_RELEASE_CNTL0_DIDT_RELEASE_DELAY_HI];
   assign  didt_DB_didt_release_delay_lo = didt_DB_stall_release_cntl0[`DIDT_DB_STALL_RELEASE_CNTL0_DIDT_RELEASE_DELAY_LO];

   assign  didt_DB_didt_base_release_allowed_hi = didt_DB_stall_release_cntl1[`DIDT_DB_STALL_RELEASE_CNTL1_DIDT_BASE_RELEASE_ALLOWED_HI];
   assign  didt_DB_didt_incr_release_allowed_hi = didt_DB_stall_release_cntl1[`DIDT_DB_STALL_RELEASE_CNTL1_DIDT_INCR_RELEASE_ALLOWED_HI];
   assign  didt_DB_didt_base_release_allowed_lo = didt_DB_stall_release_cntl1[`DIDT_DB_STALL_RELEASE_CNTL1_DIDT_BASE_RELEASE_ALLOWED_LO];
   assign  didt_DB_didt_incr_release_allowed_lo = didt_DB_stall_release_cntl1[`DIDT_DB_STALL_RELEASE_CNTL1_DIDT_INCR_RELEASE_ALLOWED_LO];

   assign  didt_DB_gc_didt_enable         = didt_DB_ctrl3[`DIDT_DB_CTRL3_GC_DIDT_ENABLE];
   assign  didt_DB_gc_didt_clk_en_override         = didt_DB_ctrl3[`DIDT_DB_CTRL3_GC_DIDT_CLK_EN_OVERRIDE];
   assign  didt_DB_throttle_policy         = didt_DB_ctrl3[`DIDT_DB_CTRL3_THROTTLE_POLICY];
   assign  didt_DB_didt_trigger_throttle_lowbit        = didt_DB_ctrl3[`DIDT_DB_CTRL3_DIDT_TRIGGER_THROTTLE_LOWBIT];
   assign  didt_DB_didt_power_level_lowbit        = didt_DB_ctrl3[`DIDT_DB_CTRL3_DIDT_POWER_LEVEL_LOWBIT];
   assign  didt_DB_didt_stall_pattern_bit_nums         = didt_DB_ctrl3[`DIDT_DB_CTRL3_DIDT_STALL_PATTERN_BIT_NUMS];
   assign  didt_DB_gc_didt_level_comb_en         = didt_DB_ctrl3[`DIDT_DB_CTRL3_GC_DIDT_LEVEL_COMB_EN];
   assign  didt_DB_se_didt_level_comb_en         = didt_DB_ctrl3[`DIDT_DB_CTRL3_SE_DIDT_LEVEL_COMB_EN];
   assign  didt_DB_qualify_stall_en         = didt_DB_ctrl3[`DIDT_DB_CTRL3_QUALIFY_STALL_EN];
   assign  didt_DB_didt_stall_sel         = didt_DB_ctrl3[`DIDT_DB_CTRL3_DIDT_STALL_SEL];
   assign  didt_DB_didt_force_stall       = didt_DB_ctrl3[`DIDT_DB_CTRL3_DIDT_FORCE_STALL];
   assign  didt_DB_didt_stall_delay_en      = didt_DB_ctrl3[`DIDT_DB_CTRL3_DIDT_STALL_DELAY_EN];

   assign didt_DB_stall_pattern1 = didt_DB_didt_stall_pattern_1_2[`DIDT_DB_STALL_PATTERN_1_2_DIDT_STALL_PATTERN_1];
   assign didt_DB_stall_pattern2 = didt_DB_didt_stall_pattern_1_2[`DIDT_DB_STALL_PATTERN_1_2_DIDT_STALL_PATTERN_2];
   assign didt_DB_stall_pattern3 = didt_DB_didt_stall_pattern_3_4[`DIDT_DB_STALL_PATTERN_3_4_DIDT_STALL_PATTERN_3];
   assign didt_DB_stall_pattern4 = didt_DB_didt_stall_pattern_3_4[`DIDT_DB_STALL_PATTERN_3_4_DIDT_STALL_PATTERN_4];
   assign didt_DB_stall_pattern5 = didt_DB_didt_stall_pattern_5_6[`DIDT_DB_STALL_PATTERN_5_6_DIDT_STALL_PATTERN_5];
   assign didt_DB_stall_pattern6 = didt_DB_didt_stall_pattern_5_6[`DIDT_DB_STALL_PATTERN_5_6_DIDT_STALL_PATTERN_6];
   assign didt_DB_stall_pattern7 = didt_DB_didt_stall_pattern_7[`DIDT_DB_STALL_PATTERN_7_DIDT_STALL_PATTERN_7];

   assign DB_edc_en = DB_edc_ctrl[`DIDT_DB_EDC_CTRL_EDC_EN];
   assign DB_edc_sw_rst = DB_edc_ctrl[`DIDT_DB_EDC_CTRL_EDC_SW_RST];
   assign DB_edc_clk_en_override = DB_edc_ctrl[`DIDT_DB_EDC_CTRL_EDC_CLK_EN_OVERRIDE];
   assign DB_edc_force_stall = DB_edc_ctrl[`DIDT_DB_EDC_CTRL_EDC_FORCE_STALL];
   assign DB_edc_trigger_throttle_lowbit = DB_edc_ctrl[`DIDT_DB_EDC_CTRL_EDC_TRIGGER_THROTTLE_LOWBIT];
   assign DB_edc_stall_pattern_bit_nums = DB_edc_ctrl[`DIDT_DB_EDC_CTRL_EDC_STALL_PATTERN_BIT_NUMS];
   assign DB_edc_allow_write_pwrdelta = DB_edc_ctrl[`DIDT_DB_EDC_CTRL_EDC_ALLOW_WRITE_PWRDELTA];
   assign DB_gc_edc_en = DB_edc_ctrl[`DIDT_DB_EDC_CTRL_GC_EDC_EN];
   assign DB_gc_edc_stall_policy = DB_edc_ctrl[`DIDT_DB_EDC_CTRL_GC_EDC_STALL_POLICY];
   assign DB_gc_edc_level_comb_en = DB_edc_ctrl[`DIDT_DB_EDC_CTRL_GC_EDC_LEVEL_COMB_EN];
   assign DB_se_edc_level_comb_en = DB_edc_ctrl[`DIDT_DB_EDC_CTRL_SE_EDC_LEVEL_COMB_EN];
   assign DB_edc_level_mode_sel = DB_edc_ctrl[`DIDT_DB_EDC_CTRL_EDC_LEVEL_MODE_SEL];
   assign DB_edc_level_comb_adapt_mode_en = DB_edc_ctrl[`DIDT_DB_EDC_CTRL_EDC_LEVEL_COMB_ADAPT_MODE_EN];
   assign DB_edc_grbm_clear_overflow_counter = rd_ind_data & (didt_ind_index == `DIDT_DB_EDC_OVERFLOW_DEC_ADDR);
   assign DB_edc_threshold = DB_edc_threshold_reg[`DIDT_DB_EDC_THRESHOLD_EDC_THRESHOLD];
   assign DB_edc_stall_pattern1 = DB_edc_stall_pattern_1_2[`DIDT_DB_EDC_STALL_PATTERN_1_2_EDC_STALL_PATTERN_1];
   assign DB_edc_stall_pattern2 = DB_edc_stall_pattern_1_2[`DIDT_DB_EDC_STALL_PATTERN_1_2_EDC_STALL_PATTERN_2];
   assign DB_edc_stall_pattern3 = DB_edc_stall_pattern_3_4[`DIDT_DB_EDC_STALL_PATTERN_3_4_EDC_STALL_PATTERN_3];
   assign DB_edc_stall_pattern4 = DB_edc_stall_pattern_3_4[`DIDT_DB_EDC_STALL_PATTERN_3_4_EDC_STALL_PATTERN_4];
   assign DB_edc_stall_pattern5 = DB_edc_stall_pattern_5_6[`DIDT_DB_EDC_STALL_PATTERN_5_6_EDC_STALL_PATTERN_5];
   assign DB_edc_stall_pattern6 = DB_edc_stall_pattern_5_6[`DIDT_DB_EDC_STALL_PATTERN_5_6_EDC_STALL_PATTERN_6];
   assign DB_edc_stall_pattern7 = DB_edc_stall_pattern_7[`DIDT_DB_EDC_STALL_PATTERN_7_EDC_STALL_PATTERN_7];
   assign DB_edc_timer_period   = DB_edc_timer_period_reg[`DIDT_DB_EDC_TIMER_PERIOD_EDC_TIMER_PERIOD];
   assign DB_gc_edc_stall_en   = DB_throttle_ctrl_reg[`DIDT_DB_THROTTLE_CTRL_GC_EDC_STALL_EN];
   assign DB_pcc_stall_en   = DB_throttle_ctrl_reg[`DIDT_DB_THROTTLE_CTRL_PCC_STALL_EN];
   assign DB_pwrbrk_stall_en   = DB_throttle_ctrl_reg[`DIDT_DB_THROTTLE_CTRL_PWRBRK_STALL_EN];
   assign DB_gc_edc_only_mode   = DB_throttle_ctrl_reg[`DIDT_DB_THROTTLE_CTRL_GC_EDC_ONLY_MODE];
   assign DB_edc_stall_delay_sig0 = DB_edc_stall_delay_1[`DIDT_DB_EDC_STALL_DELAY_1_EDC_STALL_DELAY_DB0];
   assign DB_edc_stall_delay_sig1 = DB_edc_stall_delay_1[`DIDT_DB_EDC_STALL_DELAY_1_EDC_STALL_DELAY_DB1];
   assign DB_edc_stall_delay_sig2 = DB_edc_stall_delay_1[`DIDT_DB_EDC_STALL_DELAY_1_EDC_STALL_DELAY_DB2];
   assign DB_edc_stall_delay_sig3 = DB_edc_stall_delay_1[`DIDT_DB_EDC_STALL_DELAY_1_EDC_STALL_DELAY_DB3];

// DIDT output assignments for the TD block
   assign  didt_TD_didt_ctrl_en         = didt_TD_ctrl0[`DIDT_TD_CTRL0_DIDT_CTRL_EN];
   assign  didt_TD_phase_offset         = didt_TD_ctrl0[`DIDT_TD_CTRL0_PHASE_OFFSET];
   assign  didt_TD_didt_ctrl_rst        = didt_TD_ctrl0[`DIDT_TD_CTRL0_DIDT_CTRL_RST];
   assign  didt_TD_didt_clk_en_override = didt_TD_ctrl0[`DIDT_TD_CTRL0_DIDT_CLK_EN_OVERRIDE];
   assign  didt_TD_stall_ctrl_enable           = didt_TD_ctrl0[`DIDT_TD_CTRL0_DIDT_STALL_CTRL_EN];
	assign didt_TD_tuning_enable              = didt_TD_ctrl0[`DIDT_TD_CTRL0_DIDT_TUNING_CTRL_EN];
	assign didt_TD_stall_auto_release_enable              = didt_TD_ctrl0[`DIDT_TD_CTRL0_DIDT_STALL_AUTO_RELEASE_EN];
   assign  didt_TD_hi_power_threshold            = didt_TD_ctrl0[`DIDT_TD_CTRL0_DIDT_HI_POWER_THRESHOLD];
	assign didt_TD_auto_mpd_enable              = didt_TD_ctrl0[`DIDT_TD_CTRL0_DIDT_AUTO_MPD_EN];
	assign didt_TD_stall_event_enable           = didt_TD_ctrl0[`DIDT_TD_CTRL0_DIDT_STALL_EVENT_EN];
	assign didt_TD_stall_event_counter_clear    = wr_didt_TD_ctrl0 && (reg_wd[`DIDT_TD_CTRL0_DIDT_STALL_EVENT_COUNTER_CLEAR] == 'd1);
    assign didt_TD_didt_rlc_force_stall_en           = didt_TD_ctrl0[`DIDT_TD_CTRL0_DIDT_RLC_FORCE_STALL_EN];
    assign didt_TD_didt_rlc_stall_level_sel          = didt_TD_ctrl0[`DIDT_TD_CTRL0_DIDT_RLC_STALL_LEVEL_SEL];
    assign didt_TD_didt_throttle_mode          = didt_TD_ctrl0[`DIDT_TD_CTRL0_DIDT_THROTTLE_MODE];

   assign  didt_TD_min_power            = didt_TD_ctrl1[`DIDT_TD_CTRL1_MIN_POWER];
   assign  didt_TD_max_power            = didt_TD_ctrl1[`DIDT_TD_CTRL1_MAX_POWER];

   assign  didt_TD_max_power_delta          = didt_TD_ctrl2[`DIDT_TD_CTRL2_MAX_POWER_DELTA];
   assign  didt_TD_short_term_interval_size = didt_TD_ctrl2[`DIDT_TD_CTRL2_SHORT_TERM_INTERVAL_SIZE ];
   assign  didt_TD_long_term_interval_ratio = didt_TD_ctrl2[`DIDT_TD_CTRL2_LONG_TERM_INTERVAL_RATIO];

	assign didt_TD_ocp_max_power = didt_TD_ctrl_ocp[`DIDT_TD_CTRL_OCP_OCP_MAX_POWER];

   assign  didt_TD_stall_delay_hi            = didt_TD_stall_ctrl[`DIDT_TD_STALL_CTRL_DIDT_STALL_DELAY_HI];
   assign  didt_TD_stall_delay_lo            = didt_TD_stall_ctrl[`DIDT_TD_STALL_CTRL_DIDT_STALL_DELAY_LO];
   assign  didt_TD_didt_max_stalls_allowed_hi = didt_TD_stall_ctrl[`DIDT_TD_STALL_CTRL_DIDT_MAX_STALLS_ALLOWED_HI];
   assign  didt_TD_didt_max_stalls_allowed_lo = didt_TD_stall_ctrl[`DIDT_TD_STALL_CTRL_DIDT_MAX_STALLS_ALLOWED_LO];

	assign didt_TD_max_power_delta_hi         = didt_TD_tuning_ctrl[`DIDT_TD_TUNING_CTRL_MAX_POWER_DELTA_HI];
	assign didt_TD_max_power_delta_lo         = didt_TD_tuning_ctrl[`DIDT_TD_TUNING_CTRL_MAX_POWER_DELTA_LO];

	assign didt_TD_stall_auto_release_time              = didt_TD_stall_auto_release_ctrl[`DIDT_TD_STALL_AUTO_RELEASE_CTRL_DIDT_STALL_AUTO_RELEASE_TIME];

   assign  didt_TD_weight0 = didt_TD_weight_0[`DIDT_TD_WEIGHT0_3_WEIGHT0];
   assign  didt_TD_weight1 = didt_TD_weight_0[`DIDT_TD_WEIGHT0_3_WEIGHT1];
   assign  didt_TD_weight2 = didt_TD_weight_0[`DIDT_TD_WEIGHT0_3_WEIGHT2];
   assign  didt_TD_weight3 = didt_TD_weight_0[`DIDT_TD_WEIGHT0_3_WEIGHT3];

   assign  didt_TD_weight4 = didt_TD_weight_1[`DIDT_TD_WEIGHT4_7_WEIGHT4];
   assign  didt_TD_weight5 = didt_TD_weight_1[`DIDT_TD_WEIGHT4_7_WEIGHT5];
   assign  didt_TD_weight6 = didt_TD_weight_1[`DIDT_TD_WEIGHT4_7_WEIGHT6];
   assign  didt_TD_weight7 = didt_TD_weight_1[`DIDT_TD_WEIGHT4_7_WEIGHT7];

   assign  didt_TD_weight8  = didt_TD_weight_2[`DIDT_TD_WEIGHT8_11_WEIGHT8];
   assign  didt_TD_weight9  = didt_TD_weight_2[`DIDT_TD_WEIGHT8_11_WEIGHT9];
   assign  didt_TD_weight10 = didt_TD_weight_2[`DIDT_TD_WEIGHT8_11_WEIGHT10];
   assign  didt_TD_weight11 = didt_TD_weight_2[`DIDT_TD_WEIGHT8_11_WEIGHT11];

   assign  didt_TD_mpd_ratio_scale_level1 = didt_TD_mpd_scale_factor[`DIDT_TD_MPD_SCALE_FACTOR_MPD_RATIO_SCALE_LEVEL1];
   assign  didt_TD_mpd_ratio_scale_level2 = didt_TD_mpd_scale_factor[`DIDT_TD_MPD_SCALE_FACTOR_MPD_RATIO_SCALE_LEVEL2];
   assign  didt_TD_mpd_ratio_scale_level3 = didt_TD_mpd_scale_factor[`DIDT_TD_MPD_SCALE_FACTOR_MPD_RATIO_SCALE_LEVEL3];
   assign  didt_TD_mpd_ratio_scale_level4 = didt_TD_mpd_scale_factor[`DIDT_TD_MPD_SCALE_FACTOR_MPD_RATIO_SCALE_LEVEL4];
   assign  didt_TD_mpd_scale_level0 = didt_TD_mpd_scale_factor[`DIDT_TD_MPD_SCALE_FACTOR_MPD_SCALE_LEVEL0];
   assign  didt_TD_mpd_scale_level1 = didt_TD_mpd_scale_factor[`DIDT_TD_MPD_SCALE_FACTOR_MPD_SCALE_LEVEL1];
   assign  didt_TD_mpd_scale_level2 = didt_TD_mpd_scale_factor[`DIDT_TD_MPD_SCALE_FACTOR_MPD_SCALE_LEVEL2];
   assign  didt_TD_mpd_scale_level3 = didt_TD_mpd_scale_factor[`DIDT_TD_MPD_SCALE_FACTOR_MPD_SCALE_LEVEL3];

   assign  didt_TD_didt_stall_release_cntl_en = didt_TD_stall_release_cntl0[`DIDT_TD_STALL_RELEASE_CNTL0_DIDT_STALL_RELEASE_CNTL_EN];
   assign  didt_TD_didt_stall_cntl_sel = didt_TD_stall_release_cntl0[`DIDT_TD_STALL_RELEASE_CNTL0_DIDT_STALL_CNTL_SEL];
   assign  didt_TD_didt_release_delay_hi = didt_TD_stall_release_cntl0[`DIDT_TD_STALL_RELEASE_CNTL0_DIDT_RELEASE_DELAY_HI];
   assign  didt_TD_didt_release_delay_lo = didt_TD_stall_release_cntl0[`DIDT_TD_STALL_RELEASE_CNTL0_DIDT_RELEASE_DELAY_LO];

   assign  didt_TD_didt_base_release_allowed_hi = didt_TD_stall_release_cntl1[`DIDT_TD_STALL_RELEASE_CNTL1_DIDT_BASE_RELEASE_ALLOWED_HI];
   assign  didt_TD_didt_incr_release_allowed_hi = didt_TD_stall_release_cntl1[`DIDT_TD_STALL_RELEASE_CNTL1_DIDT_INCR_RELEASE_ALLOWED_HI];
   assign  didt_TD_didt_base_release_allowed_lo = didt_TD_stall_release_cntl1[`DIDT_TD_STALL_RELEASE_CNTL1_DIDT_BASE_RELEASE_ALLOWED_LO];
   assign  didt_TD_didt_incr_release_allowed_lo = didt_TD_stall_release_cntl1[`DIDT_TD_STALL_RELEASE_CNTL1_DIDT_INCR_RELEASE_ALLOWED_LO];

   assign  didt_TD_gc_didt_enable         = didt_TD_ctrl3[`DIDT_TD_CTRL3_GC_DIDT_ENABLE];
   assign  didt_TD_gc_didt_clk_en_override         = didt_TD_ctrl3[`DIDT_TD_CTRL3_GC_DIDT_CLK_EN_OVERRIDE];
   assign  didt_TD_throttle_policy         = didt_TD_ctrl3[`DIDT_TD_CTRL3_THROTTLE_POLICY];
   assign  didt_TD_didt_trigger_throttle_lowbit        = didt_TD_ctrl3[`DIDT_TD_CTRL3_DIDT_TRIGGER_THROTTLE_LOWBIT];
   assign  didt_TD_didt_power_level_lowbit        = didt_TD_ctrl3[`DIDT_TD_CTRL3_DIDT_POWER_LEVEL_LOWBIT];
   assign  didt_TD_didt_stall_pattern_bit_nums         = didt_TD_ctrl3[`DIDT_TD_CTRL3_DIDT_STALL_PATTERN_BIT_NUMS];
   assign  didt_TD_gc_didt_level_comb_en         = didt_TD_ctrl3[`DIDT_TD_CTRL3_GC_DIDT_LEVEL_COMB_EN];
   assign  didt_TD_se_didt_level_comb_en         = didt_TD_ctrl3[`DIDT_TD_CTRL3_SE_DIDT_LEVEL_COMB_EN];
   assign  didt_TD_qualify_stall_en         = didt_TD_ctrl3[`DIDT_TD_CTRL3_QUALIFY_STALL_EN];
   assign  didt_TD_didt_stall_sel         = didt_TD_ctrl3[`DIDT_TD_CTRL3_DIDT_STALL_SEL];
   assign  didt_TD_didt_force_stall       = didt_TD_ctrl3[`DIDT_TD_CTRL3_DIDT_FORCE_STALL];
   assign  didt_TD_didt_stall_delay_en      = didt_TD_ctrl3[`DIDT_TD_CTRL3_DIDT_STALL_DELAY_EN];

   assign didt_TD_stall_pattern1 = didt_TD_didt_stall_pattern_1_2[`DIDT_TD_STALL_PATTERN_1_2_DIDT_STALL_PATTERN_1];
   assign didt_TD_stall_pattern2 = didt_TD_didt_stall_pattern_1_2[`DIDT_TD_STALL_PATTERN_1_2_DIDT_STALL_PATTERN_2];
   assign didt_TD_stall_pattern3 = didt_TD_didt_stall_pattern_3_4[`DIDT_TD_STALL_PATTERN_3_4_DIDT_STALL_PATTERN_3];
   assign didt_TD_stall_pattern4 = didt_TD_didt_stall_pattern_3_4[`DIDT_TD_STALL_PATTERN_3_4_DIDT_STALL_PATTERN_4];
   assign didt_TD_stall_pattern5 = didt_TD_didt_stall_pattern_5_6[`DIDT_TD_STALL_PATTERN_5_6_DIDT_STALL_PATTERN_5];
   assign didt_TD_stall_pattern6 = didt_TD_didt_stall_pattern_5_6[`DIDT_TD_STALL_PATTERN_5_6_DIDT_STALL_PATTERN_6];
   assign didt_TD_stall_pattern7 = didt_TD_didt_stall_pattern_7[`DIDT_TD_STALL_PATTERN_7_DIDT_STALL_PATTERN_7];

   assign TD_edc_en = TD_edc_ctrl[`DIDT_TD_EDC_CTRL_EDC_EN];
   assign TD_edc_sw_rst = TD_edc_ctrl[`DIDT_TD_EDC_CTRL_EDC_SW_RST];
   assign TD_edc_clk_en_override = TD_edc_ctrl[`DIDT_TD_EDC_CTRL_EDC_CLK_EN_OVERRIDE];
   assign TD_edc_force_stall = TD_edc_ctrl[`DIDT_TD_EDC_CTRL_EDC_FORCE_STALL];
   assign TD_edc_trigger_throttle_lowbit = TD_edc_ctrl[`DIDT_TD_EDC_CTRL_EDC_TRIGGER_THROTTLE_LOWBIT];
   assign TD_edc_stall_pattern_bit_nums = TD_edc_ctrl[`DIDT_TD_EDC_CTRL_EDC_STALL_PATTERN_BIT_NUMS];
   assign TD_edc_allow_write_pwrdelta = TD_edc_ctrl[`DIDT_TD_EDC_CTRL_EDC_ALLOW_WRITE_PWRDELTA];
   assign TD_gc_edc_en = TD_edc_ctrl[`DIDT_TD_EDC_CTRL_GC_EDC_EN];
   assign TD_gc_edc_stall_policy = TD_edc_ctrl[`DIDT_TD_EDC_CTRL_GC_EDC_STALL_POLICY];
   assign TD_gc_edc_level_comb_en = TD_edc_ctrl[`DIDT_TD_EDC_CTRL_GC_EDC_LEVEL_COMB_EN];
   assign TD_se_edc_level_comb_en = TD_edc_ctrl[`DIDT_TD_EDC_CTRL_SE_EDC_LEVEL_COMB_EN];
   assign TD_edc_level_mode_sel = TD_edc_ctrl[`DIDT_TD_EDC_CTRL_EDC_LEVEL_MODE_SEL];
   assign TD_edc_level_comb_adapt_mode_en = TD_edc_ctrl[`DIDT_TD_EDC_CTRL_EDC_LEVEL_COMB_ADAPT_MODE_EN];
   assign TD_edc_grbm_clear_overflow_counter = rd_ind_data & (didt_ind_index == `DIDT_TD_EDC_OVERFLOW_DEC_ADDR);
   assign TD_edc_threshold = TD_edc_threshold_reg[`DIDT_TD_EDC_THRESHOLD_EDC_THRESHOLD];
   assign TD_edc_stall_pattern1 = TD_edc_stall_pattern_1_2[`DIDT_TD_EDC_STALL_PATTERN_1_2_EDC_STALL_PATTERN_1];
   assign TD_edc_stall_pattern2 = TD_edc_stall_pattern_1_2[`DIDT_TD_EDC_STALL_PATTERN_1_2_EDC_STALL_PATTERN_2];
   assign TD_edc_stall_pattern3 = TD_edc_stall_pattern_3_4[`DIDT_TD_EDC_STALL_PATTERN_3_4_EDC_STALL_PATTERN_3];
   assign TD_edc_stall_pattern4 = TD_edc_stall_pattern_3_4[`DIDT_TD_EDC_STALL_PATTERN_3_4_EDC_STALL_PATTERN_4];
   assign TD_edc_stall_pattern5 = TD_edc_stall_pattern_5_6[`DIDT_TD_EDC_STALL_PATTERN_5_6_EDC_STALL_PATTERN_5];
   assign TD_edc_stall_pattern6 = TD_edc_stall_pattern_5_6[`DIDT_TD_EDC_STALL_PATTERN_5_6_EDC_STALL_PATTERN_6];
   assign TD_edc_stall_pattern7 = TD_edc_stall_pattern_7[`DIDT_TD_EDC_STALL_PATTERN_7_EDC_STALL_PATTERN_7];
   assign TD_edc_timer_period   = TD_edc_timer_period_reg[`DIDT_TD_EDC_TIMER_PERIOD_EDC_TIMER_PERIOD];
   assign TD_gc_edc_stall_en   = TD_throttle_ctrl_reg[`DIDT_TD_THROTTLE_CTRL_GC_EDC_STALL_EN];
   assign TD_pcc_stall_en   = TD_throttle_ctrl_reg[`DIDT_TD_THROTTLE_CTRL_PCC_STALL_EN];
   assign TD_pwrbrk_stall_en   = TD_throttle_ctrl_reg[`DIDT_TD_THROTTLE_CTRL_PWRBRK_STALL_EN];
   assign TD_gc_edc_only_mode   = TD_throttle_ctrl_reg[`DIDT_TD_THROTTLE_CTRL_GC_EDC_ONLY_MODE];
   assign TD_edc_stall_delay_sig0 = TD_edc_stall_delay_1[`DIDT_TD_EDC_STALL_DELAY_1_EDC_STALL_DELAY_TD0];
   assign TD_edc_stall_delay_sig1 = TD_edc_stall_delay_1[`DIDT_TD_EDC_STALL_DELAY_1_EDC_STALL_DELAY_TD1];
   assign TD_edc_stall_delay_sig2 = TD_edc_stall_delay_1[`DIDT_TD_EDC_STALL_DELAY_1_EDC_STALL_DELAY_TD2];
   assign TD_edc_stall_delay_sig3 = TD_edc_stall_delay_1[`DIDT_TD_EDC_STALL_DELAY_1_EDC_STALL_DELAY_TD3];
   assign TD_edc_stall_delay_sig4 = TD_edc_stall_delay_2[`DIDT_TD_EDC_STALL_DELAY_2_EDC_STALL_DELAY_TD4];
   assign TD_edc_stall_delay_sig5 = TD_edc_stall_delay_2[`DIDT_TD_EDC_STALL_DELAY_2_EDC_STALL_DELAY_TD5];
   assign TD_edc_stall_delay_sig6 = TD_edc_stall_delay_2[`DIDT_TD_EDC_STALL_DELAY_2_EDC_STALL_DELAY_TD6];
   assign TD_edc_stall_delay_sig7 = TD_edc_stall_delay_2[`DIDT_TD_EDC_STALL_DELAY_2_EDC_STALL_DELAY_TD7];
   assign TD_edc_stall_delay_sig8 = TD_edc_stall_delay_3[`DIDT_TD_EDC_STALL_DELAY_3_EDC_STALL_DELAY_TD8];
   assign TD_edc_stall_delay_sig9 = TD_edc_stall_delay_3[`DIDT_TD_EDC_STALL_DELAY_3_EDC_STALL_DELAY_TD9];

// DIDT output assignments for the TCP block
   assign  didt_TCP_didt_ctrl_en         = didt_TCP_ctrl0[`DIDT_TCP_CTRL0_DIDT_CTRL_EN];
   assign  didt_TCP_phase_offset         = didt_TCP_ctrl0[`DIDT_TCP_CTRL0_PHASE_OFFSET];
   assign  didt_TCP_didt_ctrl_rst        = didt_TCP_ctrl0[`DIDT_TCP_CTRL0_DIDT_CTRL_RST];
   assign  didt_TCP_didt_clk_en_override = didt_TCP_ctrl0[`DIDT_TCP_CTRL0_DIDT_CLK_EN_OVERRIDE];
   assign  didt_TCP_stall_ctrl_enable           = didt_TCP_ctrl0[`DIDT_TCP_CTRL0_DIDT_STALL_CTRL_EN];
	assign didt_TCP_tuning_enable              = didt_TCP_ctrl0[`DIDT_TCP_CTRL0_DIDT_TUNING_CTRL_EN];
	assign didt_TCP_stall_auto_release_enable              = didt_TCP_ctrl0[`DIDT_TCP_CTRL0_DIDT_STALL_AUTO_RELEASE_EN];
   assign  didt_TCP_hi_power_threshold            = didt_TCP_ctrl0[`DIDT_TCP_CTRL0_DIDT_HI_POWER_THRESHOLD];
	assign didt_TCP_auto_mpd_enable              = didt_TCP_ctrl0[`DIDT_TCP_CTRL0_DIDT_AUTO_MPD_EN];
	assign didt_TCP_stall_event_enable           = didt_TCP_ctrl0[`DIDT_TCP_CTRL0_DIDT_STALL_EVENT_EN];
	assign didt_TCP_stall_event_counter_clear    = wr_didt_TCP_ctrl0 && (reg_wd[`DIDT_TCP_CTRL0_DIDT_STALL_EVENT_COUNTER_CLEAR] == 'd1);
    assign didt_TCP_didt_rlc_force_stall_en           = didt_TCP_ctrl0[`DIDT_TCP_CTRL0_DIDT_RLC_FORCE_STALL_EN];
    assign didt_TCP_didt_rlc_stall_level_sel          = didt_TCP_ctrl0[`DIDT_TCP_CTRL0_DIDT_RLC_STALL_LEVEL_SEL];
    assign didt_TCP_didt_throttle_mode          = didt_TCP_ctrl0[`DIDT_TCP_CTRL0_DIDT_THROTTLE_MODE];

   assign  didt_TCP_min_power            = didt_TCP_ctrl1[`DIDT_TCP_CTRL1_MIN_POWER];
   assign  didt_TCP_max_power            = didt_TCP_ctrl1[`DIDT_TCP_CTRL1_MAX_POWER];

   assign  didt_TCP_max_power_delta          = didt_TCP_ctrl2[`DIDT_TCP_CTRL2_MAX_POWER_DELTA];
   assign  didt_TCP_short_term_interval_size = didt_TCP_ctrl2[`DIDT_TCP_CTRL2_SHORT_TERM_INTERVAL_SIZE ];
   assign  didt_TCP_long_term_interval_ratio = didt_TCP_ctrl2[`DIDT_TCP_CTRL2_LONG_TERM_INTERVAL_RATIO];

	assign didt_TCP_ocp_max_power = didt_TCP_ctrl_ocp[`DIDT_TCP_CTRL_OCP_OCP_MAX_POWER];

   assign  didt_TCP_stall_delay_hi            = didt_TCP_stall_ctrl[`DIDT_TCP_STALL_CTRL_DIDT_STALL_DELAY_HI];
   assign  didt_TCP_stall_delay_lo            = didt_TCP_stall_ctrl[`DIDT_TCP_STALL_CTRL_DIDT_STALL_DELAY_LO];
   assign  didt_TCP_didt_max_stalls_allowed_hi = didt_TCP_stall_ctrl[`DIDT_TCP_STALL_CTRL_DIDT_MAX_STALLS_ALLOWED_HI];
   assign  didt_TCP_didt_max_stalls_allowed_lo = didt_TCP_stall_ctrl[`DIDT_TCP_STALL_CTRL_DIDT_MAX_STALLS_ALLOWED_LO];

	assign didt_TCP_max_power_delta_hi         = didt_TCP_tuning_ctrl[`DIDT_TCP_TUNING_CTRL_MAX_POWER_DELTA_HI];
	assign didt_TCP_max_power_delta_lo         = didt_TCP_tuning_ctrl[`DIDT_TCP_TUNING_CTRL_MAX_POWER_DELTA_LO];

	assign didt_TCP_stall_auto_release_time              = didt_TCP_stall_auto_release_ctrl[`DIDT_TCP_STALL_AUTO_RELEASE_CTRL_DIDT_STALL_AUTO_RELEASE_TIME];

   assign  didt_TCP_weight0 = didt_TCP_weight_0[`DIDT_TCP_WEIGHT0_3_WEIGHT0];
   assign  didt_TCP_weight1 = didt_TCP_weight_0[`DIDT_TCP_WEIGHT0_3_WEIGHT1];
   assign  didt_TCP_weight2 = didt_TCP_weight_0[`DIDT_TCP_WEIGHT0_3_WEIGHT2];
   assign  didt_TCP_weight3 = didt_TCP_weight_0[`DIDT_TCP_WEIGHT0_3_WEIGHT3];

   assign  didt_TCP_weight4 = didt_TCP_weight_1[`DIDT_TCP_WEIGHT4_7_WEIGHT4];
   assign  didt_TCP_weight5 = didt_TCP_weight_1[`DIDT_TCP_WEIGHT4_7_WEIGHT5];
   assign  didt_TCP_weight6 = didt_TCP_weight_1[`DIDT_TCP_WEIGHT4_7_WEIGHT6];
   assign  didt_TCP_weight7 = didt_TCP_weight_1[`DIDT_TCP_WEIGHT4_7_WEIGHT7];

   assign  didt_TCP_weight8  = didt_TCP_weight_2[`DIDT_TCP_WEIGHT8_11_WEIGHT8];
   assign  didt_TCP_weight9  = didt_TCP_weight_2[`DIDT_TCP_WEIGHT8_11_WEIGHT9];
   assign  didt_TCP_weight10 = didt_TCP_weight_2[`DIDT_TCP_WEIGHT8_11_WEIGHT10];
   assign  didt_TCP_weight11 = didt_TCP_weight_2[`DIDT_TCP_WEIGHT8_11_WEIGHT11];

   assign  didt_TCP_mpd_ratio_scale_level1 = didt_TCP_mpd_scale_factor[`DIDT_TCP_MPD_SCALE_FACTOR_MPD_RATIO_SCALE_LEVEL1];
   assign  didt_TCP_mpd_ratio_scale_level2 = didt_TCP_mpd_scale_factor[`DIDT_TCP_MPD_SCALE_FACTOR_MPD_RATIO_SCALE_LEVEL2];
   assign  didt_TCP_mpd_ratio_scale_level3 = didt_TCP_mpd_scale_factor[`DIDT_TCP_MPD_SCALE_FACTOR_MPD_RATIO_SCALE_LEVEL3];
   assign  didt_TCP_mpd_ratio_scale_level4 = didt_TCP_mpd_scale_factor[`DIDT_TCP_MPD_SCALE_FACTOR_MPD_RATIO_SCALE_LEVEL4];
   assign  didt_TCP_mpd_scale_level0 = didt_TCP_mpd_scale_factor[`DIDT_TCP_MPD_SCALE_FACTOR_MPD_SCALE_LEVEL0];
   assign  didt_TCP_mpd_scale_level1 = didt_TCP_mpd_scale_factor[`DIDT_TCP_MPD_SCALE_FACTOR_MPD_SCALE_LEVEL1];
   assign  didt_TCP_mpd_scale_level2 = didt_TCP_mpd_scale_factor[`DIDT_TCP_MPD_SCALE_FACTOR_MPD_SCALE_LEVEL2];
   assign  didt_TCP_mpd_scale_level3 = didt_TCP_mpd_scale_factor[`DIDT_TCP_MPD_SCALE_FACTOR_MPD_SCALE_LEVEL3];

   assign  didt_TCP_didt_stall_release_cntl_en = didt_TCP_stall_release_cntl0[`DIDT_TCP_STALL_RELEASE_CNTL0_DIDT_STALL_RELEASE_CNTL_EN];
   assign  didt_TCP_didt_stall_cntl_sel = didt_TCP_stall_release_cntl0[`DIDT_TCP_STALL_RELEASE_CNTL0_DIDT_STALL_CNTL_SEL];
   assign  didt_TCP_didt_release_delay_hi = didt_TCP_stall_release_cntl0[`DIDT_TCP_STALL_RELEASE_CNTL0_DIDT_RELEASE_DELAY_HI];
   assign  didt_TCP_didt_release_delay_lo = didt_TCP_stall_release_cntl0[`DIDT_TCP_STALL_RELEASE_CNTL0_DIDT_RELEASE_DELAY_LO];

   assign  didt_TCP_didt_base_release_allowed_hi = didt_TCP_stall_release_cntl1[`DIDT_TCP_STALL_RELEASE_CNTL1_DIDT_BASE_RELEASE_ALLOWED_HI];
   assign  didt_TCP_didt_incr_release_allowed_hi = didt_TCP_stall_release_cntl1[`DIDT_TCP_STALL_RELEASE_CNTL1_DIDT_INCR_RELEASE_ALLOWED_HI];
   assign  didt_TCP_didt_base_release_allowed_lo = didt_TCP_stall_release_cntl1[`DIDT_TCP_STALL_RELEASE_CNTL1_DIDT_BASE_RELEASE_ALLOWED_LO];
   assign  didt_TCP_didt_incr_release_allowed_lo = didt_TCP_stall_release_cntl1[`DIDT_TCP_STALL_RELEASE_CNTL1_DIDT_INCR_RELEASE_ALLOWED_LO];

   assign  didt_TCP_gc_didt_enable         = didt_TCP_ctrl3[`DIDT_TCP_CTRL3_GC_DIDT_ENABLE];
   assign  didt_TCP_gc_didt_clk_en_override         = didt_TCP_ctrl3[`DIDT_TCP_CTRL3_GC_DIDT_CLK_EN_OVERRIDE];
   assign  didt_TCP_throttle_policy         = didt_TCP_ctrl3[`DIDT_TCP_CTRL3_THROTTLE_POLICY];
   assign  didt_TCP_didt_trigger_throttle_lowbit        = didt_TCP_ctrl3[`DIDT_TCP_CTRL3_DIDT_TRIGGER_THROTTLE_LOWBIT];
   assign  didt_TCP_didt_power_level_lowbit        = didt_TCP_ctrl3[`DIDT_TCP_CTRL3_DIDT_POWER_LEVEL_LOWBIT];
   assign  didt_TCP_didt_stall_pattern_bit_nums         = didt_TCP_ctrl3[`DIDT_TCP_CTRL3_DIDT_STALL_PATTERN_BIT_NUMS];
   assign  didt_TCP_gc_didt_level_comb_en         = didt_TCP_ctrl3[`DIDT_TCP_CTRL3_GC_DIDT_LEVEL_COMB_EN];
   assign  didt_TCP_se_didt_level_comb_en         = didt_TCP_ctrl3[`DIDT_TCP_CTRL3_SE_DIDT_LEVEL_COMB_EN];
   assign  didt_TCP_qualify_stall_en         = didt_TCP_ctrl3[`DIDT_TCP_CTRL3_QUALIFY_STALL_EN];
   assign  didt_TCP_didt_stall_sel         = didt_TCP_ctrl3[`DIDT_TCP_CTRL3_DIDT_STALL_SEL];
   assign  didt_TCP_didt_force_stall       = didt_TCP_ctrl3[`DIDT_TCP_CTRL3_DIDT_FORCE_STALL];
   assign  didt_TCP_didt_stall_delay_en      = didt_TCP_ctrl3[`DIDT_TCP_CTRL3_DIDT_STALL_DELAY_EN];

   assign didt_TCP_stall_pattern1 = didt_TCP_didt_stall_pattern_1_2[`DIDT_TCP_STALL_PATTERN_1_2_DIDT_STALL_PATTERN_1];
   assign didt_TCP_stall_pattern2 = didt_TCP_didt_stall_pattern_1_2[`DIDT_TCP_STALL_PATTERN_1_2_DIDT_STALL_PATTERN_2];
   assign didt_TCP_stall_pattern3 = didt_TCP_didt_stall_pattern_3_4[`DIDT_TCP_STALL_PATTERN_3_4_DIDT_STALL_PATTERN_3];
   assign didt_TCP_stall_pattern4 = didt_TCP_didt_stall_pattern_3_4[`DIDT_TCP_STALL_PATTERN_3_4_DIDT_STALL_PATTERN_4];
   assign didt_TCP_stall_pattern5 = didt_TCP_didt_stall_pattern_5_6[`DIDT_TCP_STALL_PATTERN_5_6_DIDT_STALL_PATTERN_5];
   assign didt_TCP_stall_pattern6 = didt_TCP_didt_stall_pattern_5_6[`DIDT_TCP_STALL_PATTERN_5_6_DIDT_STALL_PATTERN_6];
   assign didt_TCP_stall_pattern7 = didt_TCP_didt_stall_pattern_7[`DIDT_TCP_STALL_PATTERN_7_DIDT_STALL_PATTERN_7];

   assign TCP_edc_en = TCP_edc_ctrl[`DIDT_TCP_EDC_CTRL_EDC_EN];
   assign TCP_edc_sw_rst = TCP_edc_ctrl[`DIDT_TCP_EDC_CTRL_EDC_SW_RST];
   assign TCP_edc_clk_en_override = TCP_edc_ctrl[`DIDT_TCP_EDC_CTRL_EDC_CLK_EN_OVERRIDE];
   assign TCP_edc_force_stall = TCP_edc_ctrl[`DIDT_TCP_EDC_CTRL_EDC_FORCE_STALL];
   assign TCP_edc_trigger_throttle_lowbit = TCP_edc_ctrl[`DIDT_TCP_EDC_CTRL_EDC_TRIGGER_THROTTLE_LOWBIT];
   assign TCP_edc_stall_pattern_bit_nums = TCP_edc_ctrl[`DIDT_TCP_EDC_CTRL_EDC_STALL_PATTERN_BIT_NUMS];
   assign TCP_edc_allow_write_pwrdelta = TCP_edc_ctrl[`DIDT_TCP_EDC_CTRL_EDC_ALLOW_WRITE_PWRDELTA];
   assign TCP_gc_edc_en = TCP_edc_ctrl[`DIDT_TCP_EDC_CTRL_GC_EDC_EN];
   assign TCP_gc_edc_stall_policy = TCP_edc_ctrl[`DIDT_TCP_EDC_CTRL_GC_EDC_STALL_POLICY];
   assign TCP_gc_edc_level_comb_en = TCP_edc_ctrl[`DIDT_TCP_EDC_CTRL_GC_EDC_LEVEL_COMB_EN];
   assign TCP_se_edc_level_comb_en = TCP_edc_ctrl[`DIDT_TCP_EDC_CTRL_SE_EDC_LEVEL_COMB_EN];
   assign TCP_edc_level_mode_sel = TCP_edc_ctrl[`DIDT_TCP_EDC_CTRL_EDC_LEVEL_MODE_SEL];
   assign TCP_edc_level_comb_adapt_mode_en = TCP_edc_ctrl[`DIDT_TCP_EDC_CTRL_EDC_LEVEL_COMB_ADAPT_MODE_EN];
   assign TCP_edc_grbm_clear_overflow_counter = rd_ind_data & (didt_ind_index == `DIDT_TCP_EDC_OVERFLOW_DEC_ADDR);
   assign TCP_edc_threshold = TCP_edc_threshold_reg[`DIDT_TCP_EDC_THRESHOLD_EDC_THRESHOLD];
   assign TCP_edc_stall_pattern1 = TCP_edc_stall_pattern_1_2[`DIDT_TCP_EDC_STALL_PATTERN_1_2_EDC_STALL_PATTERN_1];
   assign TCP_edc_stall_pattern2 = TCP_edc_stall_pattern_1_2[`DIDT_TCP_EDC_STALL_PATTERN_1_2_EDC_STALL_PATTERN_2];
   assign TCP_edc_stall_pattern3 = TCP_edc_stall_pattern_3_4[`DIDT_TCP_EDC_STALL_PATTERN_3_4_EDC_STALL_PATTERN_3];
   assign TCP_edc_stall_pattern4 = TCP_edc_stall_pattern_3_4[`DIDT_TCP_EDC_STALL_PATTERN_3_4_EDC_STALL_PATTERN_4];
   assign TCP_edc_stall_pattern5 = TCP_edc_stall_pattern_5_6[`DIDT_TCP_EDC_STALL_PATTERN_5_6_EDC_STALL_PATTERN_5];
   assign TCP_edc_stall_pattern6 = TCP_edc_stall_pattern_5_6[`DIDT_TCP_EDC_STALL_PATTERN_5_6_EDC_STALL_PATTERN_6];
   assign TCP_edc_stall_pattern7 = TCP_edc_stall_pattern_7[`DIDT_TCP_EDC_STALL_PATTERN_7_EDC_STALL_PATTERN_7];
   assign TCP_edc_timer_period   = TCP_edc_timer_period_reg[`DIDT_TCP_EDC_TIMER_PERIOD_EDC_TIMER_PERIOD];
   assign TCP_gc_edc_stall_en   = TCP_throttle_ctrl_reg[`DIDT_TCP_THROTTLE_CTRL_GC_EDC_STALL_EN];
   assign TCP_pcc_stall_en   = TCP_throttle_ctrl_reg[`DIDT_TCP_THROTTLE_CTRL_PCC_STALL_EN];
   assign TCP_pwrbrk_stall_en   = TCP_throttle_ctrl_reg[`DIDT_TCP_THROTTLE_CTRL_PWRBRK_STALL_EN];
   assign TCP_gc_edc_only_mode   = TCP_throttle_ctrl_reg[`DIDT_TCP_THROTTLE_CTRL_GC_EDC_ONLY_MODE];
   assign TCP_edc_stall_delay_sig0 = TCP_edc_stall_delay_1[`DIDT_TCP_EDC_STALL_DELAY_1_EDC_STALL_DELAY_TCP0];
   assign TCP_edc_stall_delay_sig1 = TCP_edc_stall_delay_1[`DIDT_TCP_EDC_STALL_DELAY_1_EDC_STALL_DELAY_TCP1];
   assign TCP_edc_stall_delay_sig2 = TCP_edc_stall_delay_1[`DIDT_TCP_EDC_STALL_DELAY_1_EDC_STALL_DELAY_TCP2];
   assign TCP_edc_stall_delay_sig3 = TCP_edc_stall_delay_1[`DIDT_TCP_EDC_STALL_DELAY_1_EDC_STALL_DELAY_TCP3];
   assign TCP_edc_stall_delay_sig4 = TCP_edc_stall_delay_2[`DIDT_TCP_EDC_STALL_DELAY_2_EDC_STALL_DELAY_TCP4];
   assign TCP_edc_stall_delay_sig5 = TCP_edc_stall_delay_2[`DIDT_TCP_EDC_STALL_DELAY_2_EDC_STALL_DELAY_TCP5];
   assign TCP_edc_stall_delay_sig6 = TCP_edc_stall_delay_2[`DIDT_TCP_EDC_STALL_DELAY_2_EDC_STALL_DELAY_TCP6];
   assign TCP_edc_stall_delay_sig7 = TCP_edc_stall_delay_2[`DIDT_TCP_EDC_STALL_DELAY_2_EDC_STALL_DELAY_TCP7];
   assign TCP_edc_stall_delay_sig8 = TCP_edc_stall_delay_3[`DIDT_TCP_EDC_STALL_DELAY_3_EDC_STALL_DELAY_TCP8];
   assign TCP_edc_stall_delay_sig9 = TCP_edc_stall_delay_3[`DIDT_TCP_EDC_STALL_DELAY_3_EDC_STALL_DELAY_TCP9];


// Index register updates
always @(posedge reg_clk)
begin
    if (~resetb)
      didt_ind_index <= 32'h0;
    else begin
      if (wr_ind_index)
        didt_ind_index <= reg_wd;
      else if ((didt_index_auto_incr_en) && (wr_ind_data || rd_ind_data) && (didt_ind_index < DIDT_IND_INDEX_MAX))
        didt_ind_index <= didt_ind_index + 1'b1;  
    end
end

//; for (my =0; <4; ++) {
//always @(posedge sclk) //use AON clock to catch power_delta_overflow all the time
//begin
//	if(~resetb) begin
//		SQ_edc_rolling_power_delta_overflow_q <= 1'b0;
//	end
//	else if(wr_SQ_edc_overflow_rolling_power_delta_overflow) begin
//		SQ_edc_rolling_power_delta_overflow_q <= 1'b0;
//	end
//	else if(SQ_edc_rolling_power_delta_overflow)begin
//		SQ_edc_rolling_power_delta_overflow_q <= 1'b1;
//	end
//end
//;}


always @(posedge reg_clk)
begin
    if (~resetb) begin
      didt_index_auto_incr_en      <= `DIDT_INDEX_AUTO_INCR_EN_DIDT_INDEX_AUTO_INCR_EN_DEFAULT;
      didt_SQ_ctrl0    <= `DIDT_SQ_CTRL0_DEFAULT;
      didt_SQ_ctrl1    <= `DIDT_SQ_CTRL1_DEFAULT;
      didt_SQ_ctrl2    <= `DIDT_SQ_CTRL2_DEFAULT;
      didt_SQ_ctrl3    <= `DIDT_SQ_CTRL3_DEFAULT;
      didt_SQ_ctrl_ocp    <= `DIDT_SQ_CTRL_OCP_DEFAULT;
      didt_SQ_stall_ctrl    <= `DIDT_SQ_STALL_CTRL_DEFAULT;
      didt_SQ_tuning_ctrl    <= `DIDT_SQ_TUNING_CTRL_DEFAULT;
      didt_SQ_stall_auto_release_ctrl    <= `DIDT_SQ_STALL_AUTO_RELEASE_CTRL_DEFAULT;

	  didt_SQ_weight_0 <= `DIDT_SQ_WEIGHT0_3_DEFAULT;
      didt_SQ_weight_1 <= `DIDT_SQ_WEIGHT4_7_DEFAULT;
      didt_SQ_weight_2 <= `DIDT_SQ_WEIGHT8_11_DEFAULT;

      didt_SQ_mpd_scale_factor <= `DIDT_SQ_MPD_SCALE_FACTOR_DEFAULT;

      didt_SQ_stall_release_cntl0 <= `DIDT_SQ_STALL_RELEASE_CNTL0_DEFAULT;
      didt_SQ_stall_release_cntl1 <= `DIDT_SQ_STALL_RELEASE_CNTL1_DEFAULT;

	  didt_SQ_didt_stall_pattern_1_2		<= `DIDT_SQ_STALL_PATTERN_1_2_DEFAULT;
	  didt_SQ_didt_stall_pattern_3_4		<= `DIDT_SQ_STALL_PATTERN_3_4_DEFAULT;
	  didt_SQ_didt_stall_pattern_5_6		<= `DIDT_SQ_STALL_PATTERN_5_6_DEFAULT;
	  didt_SQ_didt_stall_pattern_7		<= `DIDT_SQ_STALL_PATTERN_7_DEFAULT;


	  SQ_edc_ctrl					<= `DIDT_SQ_EDC_CTRL_DEFAULT;
	  SQ_edc_threshold_reg			<= `DIDT_SQ_EDC_THRESHOLD_DEFAULT;
	  SQ_edc_stall_pattern_1_2		<= `DIDT_SQ_EDC_STALL_PATTERN_1_2_DEFAULT;
	  SQ_edc_stall_pattern_3_4		<= `DIDT_SQ_EDC_STALL_PATTERN_3_4_DEFAULT;
	  SQ_edc_stall_pattern_5_6		<= `DIDT_SQ_EDC_STALL_PATTERN_5_6_DEFAULT;
	  SQ_edc_stall_pattern_7		<= `DIDT_SQ_EDC_STALL_PATTERN_7_DEFAULT;
	  SQ_edc_timer_period_reg		<= `DIDT_SQ_EDC_TIMER_PERIOD_DEFAULT;
	  SQ_throttle_ctrl_reg			<= `DIDT_SQ_THROTTLE_CTRL_DEFAULT;

	  SQ_edc_stall_delay_1			<= `DIDT_SQ_EDC_STALL_DELAY_1_DEFAULT;
	  SQ_edc_stall_delay_2		<= `DIDT_SQ_EDC_STALL_DELAY_2_DEFAULT;
	  SQ_edc_stall_delay_3		<= `DIDT_SQ_EDC_STALL_DELAY_3_DEFAULT;

      didt_DB_ctrl0    <= `DIDT_DB_CTRL0_DEFAULT;
      didt_DB_ctrl1    <= `DIDT_DB_CTRL1_DEFAULT;
      didt_DB_ctrl2    <= `DIDT_DB_CTRL2_DEFAULT;
      didt_DB_ctrl3    <= `DIDT_DB_CTRL3_DEFAULT;
      didt_DB_ctrl_ocp    <= `DIDT_DB_CTRL_OCP_DEFAULT;
      didt_DB_stall_ctrl    <= `DIDT_DB_STALL_CTRL_DEFAULT;
      didt_DB_tuning_ctrl    <= `DIDT_DB_TUNING_CTRL_DEFAULT;
      didt_DB_stall_auto_release_ctrl    <= `DIDT_DB_STALL_AUTO_RELEASE_CTRL_DEFAULT;

	  didt_DB_weight_0 <= `DIDT_DB_WEIGHT0_3_DEFAULT;
      didt_DB_weight_1 <= `DIDT_DB_WEIGHT4_7_DEFAULT;
      didt_DB_weight_2 <= `DIDT_DB_WEIGHT8_11_DEFAULT;

      didt_DB_mpd_scale_factor <= `DIDT_DB_MPD_SCALE_FACTOR_DEFAULT;

      didt_DB_stall_release_cntl0 <= `DIDT_DB_STALL_RELEASE_CNTL0_DEFAULT;
      didt_DB_stall_release_cntl1 <= `DIDT_DB_STALL_RELEASE_CNTL1_DEFAULT;

	  didt_DB_didt_stall_pattern_1_2		<= `DIDT_DB_STALL_PATTERN_1_2_DEFAULT;
	  didt_DB_didt_stall_pattern_3_4		<= `DIDT_DB_STALL_PATTERN_3_4_DEFAULT;
	  didt_DB_didt_stall_pattern_5_6		<= `DIDT_DB_STALL_PATTERN_5_6_DEFAULT;
	  didt_DB_didt_stall_pattern_7		<= `DIDT_DB_STALL_PATTERN_7_DEFAULT;


	  DB_edc_ctrl					<= `DIDT_DB_EDC_CTRL_DEFAULT;
	  DB_edc_threshold_reg			<= `DIDT_DB_EDC_THRESHOLD_DEFAULT;
	  DB_edc_stall_pattern_1_2		<= `DIDT_DB_EDC_STALL_PATTERN_1_2_DEFAULT;
	  DB_edc_stall_pattern_3_4		<= `DIDT_DB_EDC_STALL_PATTERN_3_4_DEFAULT;
	  DB_edc_stall_pattern_5_6		<= `DIDT_DB_EDC_STALL_PATTERN_5_6_DEFAULT;
	  DB_edc_stall_pattern_7		<= `DIDT_DB_EDC_STALL_PATTERN_7_DEFAULT;
	  DB_edc_timer_period_reg		<= `DIDT_DB_EDC_TIMER_PERIOD_DEFAULT;
	  DB_throttle_ctrl_reg			<= `DIDT_DB_THROTTLE_CTRL_DEFAULT;

	  DB_edc_stall_delay_1			<= `DIDT_DB_EDC_STALL_DELAY_1_DEFAULT;
`ifdef GC__NUM_RB_PER_SA__4_PRESENT
	  DB_edc_stall_delay_2		<= `DIDT_DB_EDC_STALL_DELAY_2_DEFAULT;
`endif
`ifdef GC__NUM_RB_PER_SA__8_PRESENT
	  DB_edc_stall_delay_3		<= `DIDT_DB_EDC_STALL_DELAY_3_DEFAULT;
`endif
`ifdef GC__NUM_RB_PER_SA__12_PRESENT
	  DB_edc_stall_delay_4		<= `DIDT_DB_EDC_STALL_DELAY_4_DEFAULT;
`endif

      didt_TD_ctrl0    <= `DIDT_TD_CTRL0_DEFAULT;
      didt_TD_ctrl1    <= `DIDT_TD_CTRL1_DEFAULT;
      didt_TD_ctrl2    <= `DIDT_TD_CTRL2_DEFAULT;
      didt_TD_ctrl3    <= `DIDT_TD_CTRL3_DEFAULT;
      didt_TD_ctrl_ocp    <= `DIDT_TD_CTRL_OCP_DEFAULT;
      didt_TD_stall_ctrl    <= `DIDT_TD_STALL_CTRL_DEFAULT;
      didt_TD_tuning_ctrl    <= `DIDT_TD_TUNING_CTRL_DEFAULT;
      didt_TD_stall_auto_release_ctrl    <= `DIDT_TD_STALL_AUTO_RELEASE_CTRL_DEFAULT;

	  didt_TD_weight_0 <= `DIDT_TD_WEIGHT0_3_DEFAULT;
      didt_TD_weight_1 <= `DIDT_TD_WEIGHT4_7_DEFAULT;
      didt_TD_weight_2 <= `DIDT_TD_WEIGHT8_11_DEFAULT;

      didt_TD_mpd_scale_factor <= `DIDT_TD_MPD_SCALE_FACTOR_DEFAULT;

      didt_TD_stall_release_cntl0 <= `DIDT_TD_STALL_RELEASE_CNTL0_DEFAULT;
      didt_TD_stall_release_cntl1 <= `DIDT_TD_STALL_RELEASE_CNTL1_DEFAULT;

	  didt_TD_didt_stall_pattern_1_2		<= `DIDT_TD_STALL_PATTERN_1_2_DEFAULT;
	  didt_TD_didt_stall_pattern_3_4		<= `DIDT_TD_STALL_PATTERN_3_4_DEFAULT;
	  didt_TD_didt_stall_pattern_5_6		<= `DIDT_TD_STALL_PATTERN_5_6_DEFAULT;
	  didt_TD_didt_stall_pattern_7		<= `DIDT_TD_STALL_PATTERN_7_DEFAULT;


	  TD_edc_ctrl					<= `DIDT_TD_EDC_CTRL_DEFAULT;
	  TD_edc_threshold_reg			<= `DIDT_TD_EDC_THRESHOLD_DEFAULT;
	  TD_edc_stall_pattern_1_2		<= `DIDT_TD_EDC_STALL_PATTERN_1_2_DEFAULT;
	  TD_edc_stall_pattern_3_4		<= `DIDT_TD_EDC_STALL_PATTERN_3_4_DEFAULT;
	  TD_edc_stall_pattern_5_6		<= `DIDT_TD_EDC_STALL_PATTERN_5_6_DEFAULT;
	  TD_edc_stall_pattern_7		<= `DIDT_TD_EDC_STALL_PATTERN_7_DEFAULT;
	  TD_edc_timer_period_reg		<= `DIDT_TD_EDC_TIMER_PERIOD_DEFAULT;
	  TD_throttle_ctrl_reg			<= `DIDT_TD_THROTTLE_CTRL_DEFAULT;

	  TD_edc_stall_delay_1			<= `DIDT_TD_EDC_STALL_DELAY_1_DEFAULT;
	  TD_edc_stall_delay_2		<= `DIDT_TD_EDC_STALL_DELAY_2_DEFAULT;
	  TD_edc_stall_delay_3		<= `DIDT_TD_EDC_STALL_DELAY_3_DEFAULT;

      didt_TCP_ctrl0    <= `DIDT_TCP_CTRL0_DEFAULT;
      didt_TCP_ctrl1    <= `DIDT_TCP_CTRL1_DEFAULT;
      didt_TCP_ctrl2    <= `DIDT_TCP_CTRL2_DEFAULT;
      didt_TCP_ctrl3    <= `DIDT_TCP_CTRL3_DEFAULT;
      didt_TCP_ctrl_ocp    <= `DIDT_TCP_CTRL_OCP_DEFAULT;
      didt_TCP_stall_ctrl    <= `DIDT_TCP_STALL_CTRL_DEFAULT;
      didt_TCP_tuning_ctrl    <= `DIDT_TCP_TUNING_CTRL_DEFAULT;
      didt_TCP_stall_auto_release_ctrl    <= `DIDT_TCP_STALL_AUTO_RELEASE_CTRL_DEFAULT;

	  didt_TCP_weight_0 <= `DIDT_TCP_WEIGHT0_3_DEFAULT;
      didt_TCP_weight_1 <= `DIDT_TCP_WEIGHT4_7_DEFAULT;
      didt_TCP_weight_2 <= `DIDT_TCP_WEIGHT8_11_DEFAULT;

      didt_TCP_mpd_scale_factor <= `DIDT_TCP_MPD_SCALE_FACTOR_DEFAULT;

      didt_TCP_stall_release_cntl0 <= `DIDT_TCP_STALL_RELEASE_CNTL0_DEFAULT;
      didt_TCP_stall_release_cntl1 <= `DIDT_TCP_STALL_RELEASE_CNTL1_DEFAULT;

	  didt_TCP_didt_stall_pattern_1_2		<= `DIDT_TCP_STALL_PATTERN_1_2_DEFAULT;
	  didt_TCP_didt_stall_pattern_3_4		<= `DIDT_TCP_STALL_PATTERN_3_4_DEFAULT;
	  didt_TCP_didt_stall_pattern_5_6		<= `DIDT_TCP_STALL_PATTERN_5_6_DEFAULT;
	  didt_TCP_didt_stall_pattern_7		<= `DIDT_TCP_STALL_PATTERN_7_DEFAULT;


	  TCP_edc_ctrl					<= `DIDT_TCP_EDC_CTRL_DEFAULT;
	  TCP_edc_threshold_reg			<= `DIDT_TCP_EDC_THRESHOLD_DEFAULT;
	  TCP_edc_stall_pattern_1_2		<= `DIDT_TCP_EDC_STALL_PATTERN_1_2_DEFAULT;
	  TCP_edc_stall_pattern_3_4		<= `DIDT_TCP_EDC_STALL_PATTERN_3_4_DEFAULT;
	  TCP_edc_stall_pattern_5_6		<= `DIDT_TCP_EDC_STALL_PATTERN_5_6_DEFAULT;
	  TCP_edc_stall_pattern_7		<= `DIDT_TCP_EDC_STALL_PATTERN_7_DEFAULT;
	  TCP_edc_timer_period_reg		<= `DIDT_TCP_EDC_TIMER_PERIOD_DEFAULT;
	  TCP_throttle_ctrl_reg			<= `DIDT_TCP_THROTTLE_CTRL_DEFAULT;

	  TCP_edc_stall_delay_1			<= `DIDT_TCP_EDC_STALL_DELAY_1_DEFAULT;
	  TCP_edc_stall_delay_2		<= `DIDT_TCP_EDC_STALL_DELAY_2_DEFAULT;
	  TCP_edc_stall_delay_3		<= `DIDT_TCP_EDC_STALL_DELAY_3_DEFAULT;

    end
    else begin
        didt_index_auto_incr_en      <= didt_index_auto_incr_en_d;
        didt_SQ_ctrl0    <= didt_SQ_ctrl0_d;
        didt_SQ_ctrl1    <= didt_SQ_ctrl1_d;
        didt_SQ_ctrl2    <= didt_SQ_ctrl2_d;
        didt_SQ_ctrl3    <= didt_SQ_ctrl3_d;
        didt_SQ_ctrl_ocp    <= didt_SQ_ctrl_ocp_d;
        didt_SQ_stall_ctrl    <= didt_SQ_stall_ctrl_d;
        didt_SQ_tuning_ctrl    <= didt_SQ_tuning_ctrl_d;
        didt_SQ_stall_auto_release_ctrl    <= didt_SQ_stall_auto_release_ctrl_d;

        didt_SQ_weight_0 <= didt_SQ_weight_0_d;
        didt_SQ_weight_1 <= didt_SQ_weight_1_d;
        didt_SQ_weight_2 <= didt_SQ_weight_2_d;

        didt_SQ_mpd_scale_factor <= didt_SQ_mpd_scale_factor_d;

        didt_SQ_stall_release_cntl0 <= didt_SQ_stall_release_cntl0_d;
        didt_SQ_stall_release_cntl1 <= didt_SQ_stall_release_cntl1_d;

	  didt_SQ_didt_stall_pattern_1_2		<= didt_SQ_didt_stall_pattern_1_2_d;
	  didt_SQ_didt_stall_pattern_3_4		<= didt_SQ_didt_stall_pattern_3_4_d;
	  didt_SQ_didt_stall_pattern_5_6		<= didt_SQ_didt_stall_pattern_5_6_d;
	  didt_SQ_didt_stall_pattern_7			<= didt_SQ_didt_stall_pattern_7_d;


	  SQ_edc_ctrl					<= SQ_edc_ctrl_d;
	  SQ_edc_threshold_reg			<= SQ_edc_threshold_d;
	  SQ_edc_stall_pattern_1_2		<= SQ_edc_stall_pattern_1_2_d;
	  SQ_edc_stall_pattern_3_4		<= SQ_edc_stall_pattern_3_4_d;
	  SQ_edc_stall_pattern_5_6		<= SQ_edc_stall_pattern_5_6_d;
	  SQ_edc_stall_pattern_7		<= SQ_edc_stall_pattern_7_d;
	  SQ_edc_timer_period_reg		<= SQ_edc_timer_period_d;
	  SQ_throttle_ctrl_reg			<= SQ_throttle_ctrl_d;
	  SQ_edc_stall_delay_1			<= SQ_edc_stall_delay_1_d;
	  SQ_edc_stall_delay_2		<= SQ_edc_stall_delay_2_d;
	  SQ_edc_stall_delay_3		<= SQ_edc_stall_delay_3_d;

        didt_DB_ctrl0    <= didt_DB_ctrl0_d;
        didt_DB_ctrl1    <= didt_DB_ctrl1_d;
        didt_DB_ctrl2    <= didt_DB_ctrl2_d;
        didt_DB_ctrl3    <= didt_DB_ctrl3_d;
        didt_DB_ctrl_ocp    <= didt_DB_ctrl_ocp_d;
        didt_DB_stall_ctrl    <= didt_DB_stall_ctrl_d;
        didt_DB_tuning_ctrl    <= didt_DB_tuning_ctrl_d;
        didt_DB_stall_auto_release_ctrl    <= didt_DB_stall_auto_release_ctrl_d;

        didt_DB_weight_0 <= didt_DB_weight_0_d;
        didt_DB_weight_1 <= didt_DB_weight_1_d;
        didt_DB_weight_2 <= didt_DB_weight_2_d;

        didt_DB_mpd_scale_factor <= didt_DB_mpd_scale_factor_d;

        didt_DB_stall_release_cntl0 <= didt_DB_stall_release_cntl0_d;
        didt_DB_stall_release_cntl1 <= didt_DB_stall_release_cntl1_d;

	  didt_DB_didt_stall_pattern_1_2		<= didt_DB_didt_stall_pattern_1_2_d;
	  didt_DB_didt_stall_pattern_3_4		<= didt_DB_didt_stall_pattern_3_4_d;
	  didt_DB_didt_stall_pattern_5_6		<= didt_DB_didt_stall_pattern_5_6_d;
	  didt_DB_didt_stall_pattern_7			<= didt_DB_didt_stall_pattern_7_d;


	  DB_edc_ctrl					<= DB_edc_ctrl_d;
	  DB_edc_threshold_reg			<= DB_edc_threshold_d;
	  DB_edc_stall_pattern_1_2		<= DB_edc_stall_pattern_1_2_d;
	  DB_edc_stall_pattern_3_4		<= DB_edc_stall_pattern_3_4_d;
	  DB_edc_stall_pattern_5_6		<= DB_edc_stall_pattern_5_6_d;
	  DB_edc_stall_pattern_7		<= DB_edc_stall_pattern_7_d;
	  DB_edc_timer_period_reg		<= DB_edc_timer_period_d;
	  DB_throttle_ctrl_reg			<= DB_throttle_ctrl_d;
	  DB_edc_stall_delay_1			<= DB_edc_stall_delay_1_d;
`ifdef GC__NUM_RB_PER_SA__4_PRESENT
	  DB_edc_stall_delay_2		<= DB_edc_stall_delay_2_d;
`endif
`ifdef GC__NUM_RB_PER_SA__8_PRESENT
	  DB_edc_stall_delay_3		<= DB_edc_stall_delay_3_d;
`endif
`ifdef GC__NUM_RB_PER_SA__12_PRESENT
	  DB_edc_stall_delay_4		<= DB_edc_stall_delay_4_d;
`endif

        didt_TD_ctrl0    <= didt_TD_ctrl0_d;
        didt_TD_ctrl1    <= didt_TD_ctrl1_d;
        didt_TD_ctrl2    <= didt_TD_ctrl2_d;
        didt_TD_ctrl3    <= didt_TD_ctrl3_d;
        didt_TD_ctrl_ocp    <= didt_TD_ctrl_ocp_d;
        didt_TD_stall_ctrl    <= didt_TD_stall_ctrl_d;
        didt_TD_tuning_ctrl    <= didt_TD_tuning_ctrl_d;
        didt_TD_stall_auto_release_ctrl    <= didt_TD_stall_auto_release_ctrl_d;

        didt_TD_weight_0 <= didt_TD_weight_0_d;
        didt_TD_weight_1 <= didt_TD_weight_1_d;
        didt_TD_weight_2 <= didt_TD_weight_2_d;

        didt_TD_mpd_scale_factor <= didt_TD_mpd_scale_factor_d;

        didt_TD_stall_release_cntl0 <= didt_TD_stall_release_cntl0_d;
        didt_TD_stall_release_cntl1 <= didt_TD_stall_release_cntl1_d;

	  didt_TD_didt_stall_pattern_1_2		<= didt_TD_didt_stall_pattern_1_2_d;
	  didt_TD_didt_stall_pattern_3_4		<= didt_TD_didt_stall_pattern_3_4_d;
	  didt_TD_didt_stall_pattern_5_6		<= didt_TD_didt_stall_pattern_5_6_d;
	  didt_TD_didt_stall_pattern_7			<= didt_TD_didt_stall_pattern_7_d;


	  TD_edc_ctrl					<= TD_edc_ctrl_d;
	  TD_edc_threshold_reg			<= TD_edc_threshold_d;
	  TD_edc_stall_pattern_1_2		<= TD_edc_stall_pattern_1_2_d;
	  TD_edc_stall_pattern_3_4		<= TD_edc_stall_pattern_3_4_d;
	  TD_edc_stall_pattern_5_6		<= TD_edc_stall_pattern_5_6_d;
	  TD_edc_stall_pattern_7		<= TD_edc_stall_pattern_7_d;
	  TD_edc_timer_period_reg		<= TD_edc_timer_period_d;
	  TD_throttle_ctrl_reg			<= TD_throttle_ctrl_d;
	  TD_edc_stall_delay_1			<= TD_edc_stall_delay_1_d;
	  TD_edc_stall_delay_2		<= TD_edc_stall_delay_2_d;
	  TD_edc_stall_delay_3		<= TD_edc_stall_delay_3_d;

        didt_TCP_ctrl0    <= didt_TCP_ctrl0_d;
        didt_TCP_ctrl1    <= didt_TCP_ctrl1_d;
        didt_TCP_ctrl2    <= didt_TCP_ctrl2_d;
        didt_TCP_ctrl3    <= didt_TCP_ctrl3_d;
        didt_TCP_ctrl_ocp    <= didt_TCP_ctrl_ocp_d;
        didt_TCP_stall_ctrl    <= didt_TCP_stall_ctrl_d;
        didt_TCP_tuning_ctrl    <= didt_TCP_tuning_ctrl_d;
        didt_TCP_stall_auto_release_ctrl    <= didt_TCP_stall_auto_release_ctrl_d;

        didt_TCP_weight_0 <= didt_TCP_weight_0_d;
        didt_TCP_weight_1 <= didt_TCP_weight_1_d;
        didt_TCP_weight_2 <= didt_TCP_weight_2_d;

        didt_TCP_mpd_scale_factor <= didt_TCP_mpd_scale_factor_d;

        didt_TCP_stall_release_cntl0 <= didt_TCP_stall_release_cntl0_d;
        didt_TCP_stall_release_cntl1 <= didt_TCP_stall_release_cntl1_d;

	  didt_TCP_didt_stall_pattern_1_2		<= didt_TCP_didt_stall_pattern_1_2_d;
	  didt_TCP_didt_stall_pattern_3_4		<= didt_TCP_didt_stall_pattern_3_4_d;
	  didt_TCP_didt_stall_pattern_5_6		<= didt_TCP_didt_stall_pattern_5_6_d;
	  didt_TCP_didt_stall_pattern_7			<= didt_TCP_didt_stall_pattern_7_d;


	  TCP_edc_ctrl					<= TCP_edc_ctrl_d;
	  TCP_edc_threshold_reg			<= TCP_edc_threshold_d;
	  TCP_edc_stall_pattern_1_2		<= TCP_edc_stall_pattern_1_2_d;
	  TCP_edc_stall_pattern_3_4		<= TCP_edc_stall_pattern_3_4_d;
	  TCP_edc_stall_pattern_5_6		<= TCP_edc_stall_pattern_5_6_d;
	  TCP_edc_stall_pattern_7		<= TCP_edc_stall_pattern_7_d;
	  TCP_edc_timer_period_reg		<= TCP_edc_timer_period_d;
	  TCP_throttle_ctrl_reg			<= TCP_throttle_ctrl_d;
	  TCP_edc_stall_delay_1			<= TCP_edc_stall_delay_1_d;
	  TCP_edc_stall_delay_2		<= TCP_edc_stall_delay_2_d;
	  TCP_edc_stall_delay_3		<= TCP_edc_stall_delay_3_d;

  end // else: !if(~resetb)
end // always @ (posedge reg_clk)


always @(*)
begin
  if (rd_ind_index)
    reg_read_data_d = didt_ind_index;
  else if (rd_ind_data)
case ({{24{1'b0}},didt_ind_index[7:0]})
      `DIDT_SQ_CTRL0_DEC_ADDR      : reg_read_data_d = {2'h0, didt_SQ_didt_throttle_mode, didt_SQ_didt_rlc_stall_level_sel, didt_SQ_didt_rlc_force_stall_en, didt_SQ_stall_event_counter_clear, didt_SQ_stall_event_enable, didt_SQ_auto_mpd_enable, didt_SQ_hi_power_threshold, didt_SQ_stall_auto_release_enable, didt_SQ_tuning_enable, didt_SQ_stall_ctrl_enable, didt_SQ_didt_clk_en_override, didt_SQ_didt_ctrl_rst, didt_SQ_phase_offset, didt_SQ_didt_ctrl_en};
      `DIDT_SQ_CTRL1_DEC_ADDR      : reg_read_data_d = didt_SQ_ctrl1;
      `DIDT_SQ_CTRL2_DEC_ADDR      : reg_read_data_d = {1'b0, didt_SQ_long_term_interval_ratio, 1'b0, didt_SQ_short_term_interval_size, 2'h0, didt_SQ_max_power_delta};
      `DIDT_SQ_CTRL3_DEC_ADDR      : reg_read_data_d = {3'h0, didt_SQ_didt_stall_delay_en, didt_SQ_didt_force_stall, didt_SQ_didt_stall_sel, didt_SQ_qualify_stall_en, didt_SQ_se_didt_level_comb_en, didt_SQ_gc_didt_level_comb_en, didt_SQ_didt_stall_pattern_bit_nums, didt_SQ_didt_power_level_lowbit, didt_SQ_didt_trigger_throttle_lowbit, didt_SQ_throttle_policy, didt_SQ_gc_didt_clk_en_override, didt_SQ_gc_didt_enable};
      `DIDT_SQ_CTRL_OCP_DEC_ADDR      : reg_read_data_d = {16'b0, didt_SQ_ocp_max_power};
      `DIDT_SQ_STALL_CTRL_DEC_ADDR      : reg_read_data_d = {8'h0, didt_SQ_didt_max_stalls_allowed_lo, didt_SQ_didt_max_stalls_allowed_hi, didt_SQ_stall_delay_lo, didt_SQ_stall_delay_hi};
      `DIDT_SQ_TUNING_CTRL_DEC_ADDR      : reg_read_data_d = didt_SQ_tuning_ctrl;
      `DIDT_SQ_STALL_AUTO_RELEASE_CTRL_DEC_ADDR      : reg_read_data_d = didt_SQ_stall_auto_release_ctrl;

      `DIDT_SQ_WEIGHT0_3_DEC_ADDR  : reg_read_data_d = didt_SQ_weight_0;
      `DIDT_SQ_WEIGHT4_7_DEC_ADDR  : reg_read_data_d = didt_SQ_weight_1;
      `DIDT_SQ_WEIGHT8_11_DEC_ADDR : reg_read_data_d = didt_SQ_weight_2;

      `DIDT_SQ_MPD_SCALE_FACTOR_DEC_ADDR : reg_read_data_d = didt_SQ_mpd_scale_factor;

      `DIDT_SQ_STALL_RELEASE_CNTL0_DEC_ADDR : reg_read_data_d = {8'b0, didt_SQ_didt_release_delay_lo, didt_SQ_didt_release_delay_hi, didt_SQ_didt_stall_cntl_sel, didt_SQ_didt_stall_release_cntl_en};
      `DIDT_SQ_STALL_RELEASE_CNTL1_DEC_ADDR : reg_read_data_d = {12'b0, didt_SQ_didt_incr_release_allowed_lo, didt_SQ_didt_base_release_allowed_lo, didt_SQ_didt_incr_release_allowed_hi, didt_SQ_didt_base_release_allowed_hi};
      `DIDT_SQ_STALL_RELEASE_CNTL_STATUS_DEC_ADDR      : reg_read_data_d = {{(32-`DIDT_SQ_STALL_RELEASE_CNTL_STATUS_DIDT_STALL_RELEASE_CNTL_FSM_STATE_SIZE){1'b0}}, didt_SQ_didt_stall_release_cntl_fsm_state};     

	  `DIDT_SQ_STALL_PATTERN_1_2_DEC_ADDR      : reg_read_data_d = {1'b0, didt_SQ_stall_pattern2, 1'b0, didt_SQ_stall_pattern1};
	  `DIDT_SQ_STALL_PATTERN_3_4_DEC_ADDR      : reg_read_data_d = {1'b0, didt_SQ_stall_pattern4, 1'b0, didt_SQ_stall_pattern3};
	  `DIDT_SQ_STALL_PATTERN_5_6_DEC_ADDR      : reg_read_data_d = {1'b0, didt_SQ_stall_pattern6, 1'b0, didt_SQ_stall_pattern5};
      `DIDT_SQ_STALL_PATTERN_7_DEC_ADDR        : reg_read_data_d = {17'h0, didt_SQ_stall_pattern7};

      `DIDT_SQ_STALL_EVENT_COUNTER_DEC_ADDR        : reg_read_data_d = didt_SQ_stall_event_counter;

      `DIDT_SQ_EDC_CTRL_DEC_ADDR      : reg_read_data_d = {7'h0, SQ_edc_level_comb_adapt_mode_en, SQ_edc_level_mode_sel, SQ_se_edc_level_comb_en,SQ_gc_edc_level_comb_en, SQ_gc_edc_stall_policy, SQ_gc_edc_en, SQ_edc_allow_write_pwrdelta, SQ_edc_stall_pattern_bit_nums, SQ_edc_trigger_throttle_lowbit, SQ_edc_force_stall, SQ_edc_clk_en_override, SQ_edc_sw_rst, SQ_edc_en};
      `DIDT_SQ_EDC_THRESHOLD_DEC_ADDR      : reg_read_data_d = SQ_edc_threshold_reg;
      `DIDT_SQ_EDC_STALL_PATTERN_1_2_DEC_ADDR      : reg_read_data_d = {1'b0, SQ_edc_stall_pattern2, 1'b0, SQ_edc_stall_pattern1};
      `DIDT_SQ_EDC_STALL_PATTERN_3_4_DEC_ADDR      : reg_read_data_d = {1'b0, SQ_edc_stall_pattern4, 1'b0, SQ_edc_stall_pattern3};
      `DIDT_SQ_EDC_STALL_PATTERN_5_6_DEC_ADDR      : reg_read_data_d = {1'b0, SQ_edc_stall_pattern6, 1'b0, SQ_edc_stall_pattern5};
      `DIDT_SQ_EDC_STALL_PATTERN_7_DEC_ADDR      : reg_read_data_d = {17'b0, SQ_edc_stall_pattern7};
      `DIDT_SQ_EDC_TIMER_PERIOD_DEC_ADDR         : reg_read_data_d = {18'b0, SQ_edc_timer_period_reg};
      `DIDT_SQ_THROTTLE_CTRL_DEC_ADDR         : reg_read_data_d = {28'b0, SQ_gc_edc_only_mode, SQ_pwrbrk_stall_en, SQ_pcc_stall_en, SQ_gc_edc_stall_en};
      `DIDT_SQ_EDC_STATUS_DEC_ADDR      : reg_read_data_d = {{(32-`DIDT_SQ_EDC_STATUS_EDC_THROTTLE_LEVEL_SIZE - `DIDT_SQ_EDC_STATUS_EDC_FSM_STATE_SIZE){1'b0}}, SQ_edc_throttle_level, SQ_edc_fsm_state};
      `DIDT_SQ_EDC_PCC_PERF_COUNTER_DEC_ADDR      : reg_read_data_d = {SQ_edc_pcc_perf_counter};

	  `DIDT_SQ_EDC_STALL_DELAY_1_DEC_ADDR	:	reg_read_data_d = {{`DIDT_SQ_EDC_STALL_DELAY_1_UNUSED_SIZE{1'b0}}, SQ_edc_stall_delay_1[(31 -`DIDT_SQ_EDC_STALL_DELAY_1_UNUSED_SIZE):0]};

	  `DIDT_SQ_EDC_STALL_DELAY_2_DEC_ADDR	:	reg_read_data_d = {{`DIDT_SQ_EDC_STALL_DELAY_2_UNUSED_SIZE{1'b0}}, SQ_edc_stall_delay_2[(31 -`DIDT_SQ_EDC_STALL_DELAY_2_UNUSED_SIZE):0]};

	  `DIDT_SQ_EDC_STALL_DELAY_3_DEC_ADDR	:	reg_read_data_d = {{`DIDT_SQ_EDC_STALL_DELAY_3_UNUSED_SIZE{1'b0}}, SQ_edc_stall_delay_3[(31 -`DIDT_SQ_EDC_STALL_DELAY_3_UNUSED_SIZE):0]};

      `DIDT_SQ_EDC_OVERFLOW_DEC_ADDR      : reg_read_data_d = {{(32-`DIDT_SQ_EDC_OVERFLOW_EDC_THROTTLE_LEVEL_OVERFLOW_COUNTER_SIZE - `DIDT_SQ_EDC_OVERFLOW_EDC_ROLLING_POWER_DELTA_OVERFLOW_SIZE){1'b0}}, SQ_edc_throttle_level_overflow_counter, SQ_edc_rolling_power_delta_overflow};
      `DIDT_SQ_EDC_ROLLING_POWER_DELTA_DEC_ADDR      : reg_read_data_d = SQ_edc_rolling_power_delta;

      `DIDT_DB_CTRL0_DEC_ADDR      : reg_read_data_d = {2'h0, didt_DB_didt_throttle_mode, didt_DB_didt_rlc_stall_level_sel, didt_DB_didt_rlc_force_stall_en, didt_DB_stall_event_counter_clear, didt_DB_stall_event_enable, didt_DB_auto_mpd_enable, didt_DB_hi_power_threshold, didt_DB_stall_auto_release_enable, didt_DB_tuning_enable, didt_DB_stall_ctrl_enable, didt_DB_didt_clk_en_override, didt_DB_didt_ctrl_rst, didt_DB_phase_offset, didt_DB_didt_ctrl_en};
      `DIDT_DB_CTRL1_DEC_ADDR      : reg_read_data_d = didt_DB_ctrl1;
      `DIDT_DB_CTRL2_DEC_ADDR      : reg_read_data_d = {1'b0, didt_DB_long_term_interval_ratio, 1'b0, didt_DB_short_term_interval_size, 2'h0, didt_DB_max_power_delta};
      `DIDT_DB_CTRL3_DEC_ADDR      : reg_read_data_d = {3'h0, didt_DB_didt_stall_delay_en, didt_DB_didt_force_stall, didt_DB_didt_stall_sel, didt_DB_qualify_stall_en, didt_DB_se_didt_level_comb_en, didt_DB_gc_didt_level_comb_en, didt_DB_didt_stall_pattern_bit_nums, didt_DB_didt_power_level_lowbit, didt_DB_didt_trigger_throttle_lowbit, didt_DB_throttle_policy, didt_DB_gc_didt_clk_en_override, didt_DB_gc_didt_enable};
      `DIDT_DB_CTRL_OCP_DEC_ADDR      : reg_read_data_d = {16'b0, didt_DB_ocp_max_power};
      `DIDT_DB_STALL_CTRL_DEC_ADDR      : reg_read_data_d = {8'h0, didt_DB_didt_max_stalls_allowed_lo, didt_DB_didt_max_stalls_allowed_hi, didt_DB_stall_delay_lo, didt_DB_stall_delay_hi};
      `DIDT_DB_TUNING_CTRL_DEC_ADDR      : reg_read_data_d = didt_DB_tuning_ctrl;
      `DIDT_DB_STALL_AUTO_RELEASE_CTRL_DEC_ADDR      : reg_read_data_d = didt_DB_stall_auto_release_ctrl;

      `DIDT_DB_WEIGHT0_3_DEC_ADDR  : reg_read_data_d = didt_DB_weight_0;
      `DIDT_DB_WEIGHT4_7_DEC_ADDR  : reg_read_data_d = didt_DB_weight_1;
      `DIDT_DB_WEIGHT8_11_DEC_ADDR : reg_read_data_d = didt_DB_weight_2;

      `DIDT_DB_MPD_SCALE_FACTOR_DEC_ADDR : reg_read_data_d = didt_DB_mpd_scale_factor;

      `DIDT_DB_STALL_RELEASE_CNTL0_DEC_ADDR : reg_read_data_d = {8'b0, didt_DB_didt_release_delay_lo, didt_DB_didt_release_delay_hi, didt_DB_didt_stall_cntl_sel, didt_DB_didt_stall_release_cntl_en};
      `DIDT_DB_STALL_RELEASE_CNTL1_DEC_ADDR : reg_read_data_d = {12'b0, didt_DB_didt_incr_release_allowed_lo, didt_DB_didt_base_release_allowed_lo, didt_DB_didt_incr_release_allowed_hi, didt_DB_didt_base_release_allowed_hi};
      `DIDT_DB_STALL_RELEASE_CNTL_STATUS_DEC_ADDR      : reg_read_data_d = {{(32-`DIDT_DB_STALL_RELEASE_CNTL_STATUS_DIDT_STALL_RELEASE_CNTL_FSM_STATE_SIZE){1'b0}}, didt_DB_didt_stall_release_cntl_fsm_state};     

	  `DIDT_DB_STALL_PATTERN_1_2_DEC_ADDR      : reg_read_data_d = {1'b0, didt_DB_stall_pattern2, 1'b0, didt_DB_stall_pattern1};
	  `DIDT_DB_STALL_PATTERN_3_4_DEC_ADDR      : reg_read_data_d = {1'b0, didt_DB_stall_pattern4, 1'b0, didt_DB_stall_pattern3};
	  `DIDT_DB_STALL_PATTERN_5_6_DEC_ADDR      : reg_read_data_d = {1'b0, didt_DB_stall_pattern6, 1'b0, didt_DB_stall_pattern5};
      `DIDT_DB_STALL_PATTERN_7_DEC_ADDR        : reg_read_data_d = {17'h0, didt_DB_stall_pattern7};

      `DIDT_DB_STALL_EVENT_COUNTER_DEC_ADDR        : reg_read_data_d = didt_DB_stall_event_counter;

      `DIDT_DB_EDC_CTRL_DEC_ADDR      : reg_read_data_d = {7'h0, DB_edc_level_comb_adapt_mode_en, DB_edc_level_mode_sel, DB_se_edc_level_comb_en,DB_gc_edc_level_comb_en, DB_gc_edc_stall_policy, DB_gc_edc_en, DB_edc_allow_write_pwrdelta, DB_edc_stall_pattern_bit_nums, DB_edc_trigger_throttle_lowbit, DB_edc_force_stall, DB_edc_clk_en_override, DB_edc_sw_rst, DB_edc_en};
      `DIDT_DB_EDC_THRESHOLD_DEC_ADDR      : reg_read_data_d = DB_edc_threshold_reg;
      `DIDT_DB_EDC_STALL_PATTERN_1_2_DEC_ADDR      : reg_read_data_d = {1'b0, DB_edc_stall_pattern2, 1'b0, DB_edc_stall_pattern1};
      `DIDT_DB_EDC_STALL_PATTERN_3_4_DEC_ADDR      : reg_read_data_d = {1'b0, DB_edc_stall_pattern4, 1'b0, DB_edc_stall_pattern3};
      `DIDT_DB_EDC_STALL_PATTERN_5_6_DEC_ADDR      : reg_read_data_d = {1'b0, DB_edc_stall_pattern6, 1'b0, DB_edc_stall_pattern5};
      `DIDT_DB_EDC_STALL_PATTERN_7_DEC_ADDR      : reg_read_data_d = {17'b0, DB_edc_stall_pattern7};
      `DIDT_DB_EDC_TIMER_PERIOD_DEC_ADDR         : reg_read_data_d = {18'b0, DB_edc_timer_period_reg};
      `DIDT_DB_THROTTLE_CTRL_DEC_ADDR         : reg_read_data_d = {28'b0, DB_gc_edc_only_mode, DB_pwrbrk_stall_en, DB_pcc_stall_en, DB_gc_edc_stall_en};
      `DIDT_DB_EDC_STATUS_DEC_ADDR      : reg_read_data_d = {{(32-`DIDT_DB_EDC_STATUS_EDC_THROTTLE_LEVEL_SIZE - `DIDT_DB_EDC_STATUS_EDC_FSM_STATE_SIZE){1'b0}}, DB_edc_throttle_level, DB_edc_fsm_state};
      `DIDT_DB_EDC_PCC_PERF_COUNTER_DEC_ADDR      : reg_read_data_d = {DB_edc_pcc_perf_counter};

	  `DIDT_DB_EDC_STALL_DELAY_1_DEC_ADDR	:	reg_read_data_d = {{`DIDT_DB_EDC_STALL_DELAY_1_UNUSED_SIZE{1'b0}}, DB_edc_stall_delay_1[(31 -`DIDT_DB_EDC_STALL_DELAY_1_UNUSED_SIZE):0]};

`ifdef GC__NUM_RB_PER_SA__4_PRESENT
	  `DIDT_DB_EDC_STALL_DELAY_2_DEC_ADDR	:	reg_read_data_d = {{`DIDT_DB_EDC_STALL_DELAY_2_UNUSED_SIZE{1'b0}}, DB_edc_stall_delay_2[(31 -`DIDT_DB_EDC_STALL_DELAY_2_UNUSED_SIZE):0]};
`endif

`ifdef GC__NUM_RB_PER_SA__8_PRESENT
	  `DIDT_DB_EDC_STALL_DELAY_3_DEC_ADDR	:	reg_read_data_d = {{`DIDT_DB_EDC_STALL_DELAY_3_UNUSED_SIZE{1'b0}}, DB_edc_stall_delay_3[(31 -`DIDT_DB_EDC_STALL_DELAY_3_UNUSED_SIZE):0]};
`endif

`ifdef GC__NUM_RB_PER_SA__12_PRESENT
	  `DIDT_DB_EDC_STALL_DELAY_4_DEC_ADDR	:	reg_read_data_d = {{`DIDT_DB_EDC_STALL_DELAY_4_UNUSED_SIZE{1'b0}}, DB_edc_stall_delay_4[(31 -`DIDT_DB_EDC_STALL_DELAY_4_UNUSED_SIZE):0]};
`endif
      `DIDT_DB_EDC_OVERFLOW_DEC_ADDR      : reg_read_data_d = {{(32-`DIDT_DB_EDC_OVERFLOW_EDC_THROTTLE_LEVEL_OVERFLOW_COUNTER_SIZE - `DIDT_DB_EDC_OVERFLOW_EDC_ROLLING_POWER_DELTA_OVERFLOW_SIZE){1'b0}}, DB_edc_throttle_level_overflow_counter, DB_edc_rolling_power_delta_overflow};
      `DIDT_DB_EDC_ROLLING_POWER_DELTA_DEC_ADDR      : reg_read_data_d = DB_edc_rolling_power_delta;

      `DIDT_TD_CTRL0_DEC_ADDR      : reg_read_data_d = {2'h0, didt_TD_didt_throttle_mode, didt_TD_didt_rlc_stall_level_sel, didt_TD_didt_rlc_force_stall_en, didt_TD_stall_event_counter_clear, didt_TD_stall_event_enable, didt_TD_auto_mpd_enable, didt_TD_hi_power_threshold, didt_TD_stall_auto_release_enable, didt_TD_tuning_enable, didt_TD_stall_ctrl_enable, didt_TD_didt_clk_en_override, didt_TD_didt_ctrl_rst, didt_TD_phase_offset, didt_TD_didt_ctrl_en};
      `DIDT_TD_CTRL1_DEC_ADDR      : reg_read_data_d = didt_TD_ctrl1;
      `DIDT_TD_CTRL2_DEC_ADDR      : reg_read_data_d = {1'b0, didt_TD_long_term_interval_ratio, 1'b0, didt_TD_short_term_interval_size, 2'h0, didt_TD_max_power_delta};
      `DIDT_TD_CTRL3_DEC_ADDR      : reg_read_data_d = {3'h0, didt_TD_didt_stall_delay_en, didt_TD_didt_force_stall, didt_TD_didt_stall_sel, didt_TD_qualify_stall_en, didt_TD_se_didt_level_comb_en, didt_TD_gc_didt_level_comb_en, didt_TD_didt_stall_pattern_bit_nums, didt_TD_didt_power_level_lowbit, didt_TD_didt_trigger_throttle_lowbit, didt_TD_throttle_policy, didt_TD_gc_didt_clk_en_override, didt_TD_gc_didt_enable};
      `DIDT_TD_CTRL_OCP_DEC_ADDR      : reg_read_data_d = {16'b0, didt_TD_ocp_max_power};
      `DIDT_TD_STALL_CTRL_DEC_ADDR      : reg_read_data_d = {8'h0, didt_TD_didt_max_stalls_allowed_lo, didt_TD_didt_max_stalls_allowed_hi, didt_TD_stall_delay_lo, didt_TD_stall_delay_hi};
      `DIDT_TD_TUNING_CTRL_DEC_ADDR      : reg_read_data_d = didt_TD_tuning_ctrl;
      `DIDT_TD_STALL_AUTO_RELEASE_CTRL_DEC_ADDR      : reg_read_data_d = didt_TD_stall_auto_release_ctrl;

      `DIDT_TD_WEIGHT0_3_DEC_ADDR  : reg_read_data_d = didt_TD_weight_0;
      `DIDT_TD_WEIGHT4_7_DEC_ADDR  : reg_read_data_d = didt_TD_weight_1;
      `DIDT_TD_WEIGHT8_11_DEC_ADDR : reg_read_data_d = didt_TD_weight_2;

      `DIDT_TD_MPD_SCALE_FACTOR_DEC_ADDR : reg_read_data_d = didt_TD_mpd_scale_factor;

      `DIDT_TD_STALL_RELEASE_CNTL0_DEC_ADDR : reg_read_data_d = {8'b0, didt_TD_didt_release_delay_lo, didt_TD_didt_release_delay_hi, didt_TD_didt_stall_cntl_sel, didt_TD_didt_stall_release_cntl_en};
      `DIDT_TD_STALL_RELEASE_CNTL1_DEC_ADDR : reg_read_data_d = {12'b0, didt_TD_didt_incr_release_allowed_lo, didt_TD_didt_base_release_allowed_lo, didt_TD_didt_incr_release_allowed_hi, didt_TD_didt_base_release_allowed_hi};
      `DIDT_TD_STALL_RELEASE_CNTL_STATUS_DEC_ADDR      : reg_read_data_d = {{(32-`DIDT_TD_STALL_RELEASE_CNTL_STATUS_DIDT_STALL_RELEASE_CNTL_FSM_STATE_SIZE){1'b0}}, didt_TD_didt_stall_release_cntl_fsm_state};     

	  `DIDT_TD_STALL_PATTERN_1_2_DEC_ADDR      : reg_read_data_d = {1'b0, didt_TD_stall_pattern2, 1'b0, didt_TD_stall_pattern1};
	  `DIDT_TD_STALL_PATTERN_3_4_DEC_ADDR      : reg_read_data_d = {1'b0, didt_TD_stall_pattern4, 1'b0, didt_TD_stall_pattern3};
	  `DIDT_TD_STALL_PATTERN_5_6_DEC_ADDR      : reg_read_data_d = {1'b0, didt_TD_stall_pattern6, 1'b0, didt_TD_stall_pattern5};
      `DIDT_TD_STALL_PATTERN_7_DEC_ADDR        : reg_read_data_d = {17'h0, didt_TD_stall_pattern7};

      `DIDT_TD_STALL_EVENT_COUNTER_DEC_ADDR        : reg_read_data_d = didt_TD_stall_event_counter;

      `DIDT_TD_EDC_CTRL_DEC_ADDR      : reg_read_data_d = {7'h0, TD_edc_level_comb_adapt_mode_en, TD_edc_level_mode_sel, TD_se_edc_level_comb_en,TD_gc_edc_level_comb_en, TD_gc_edc_stall_policy, TD_gc_edc_en, TD_edc_allow_write_pwrdelta, TD_edc_stall_pattern_bit_nums, TD_edc_trigger_throttle_lowbit, TD_edc_force_stall, TD_edc_clk_en_override, TD_edc_sw_rst, TD_edc_en};
      `DIDT_TD_EDC_THRESHOLD_DEC_ADDR      : reg_read_data_d = TD_edc_threshold_reg;
      `DIDT_TD_EDC_STALL_PATTERN_1_2_DEC_ADDR      : reg_read_data_d = {1'b0, TD_edc_stall_pattern2, 1'b0, TD_edc_stall_pattern1};
      `DIDT_TD_EDC_STALL_PATTERN_3_4_DEC_ADDR      : reg_read_data_d = {1'b0, TD_edc_stall_pattern4, 1'b0, TD_edc_stall_pattern3};
      `DIDT_TD_EDC_STALL_PATTERN_5_6_DEC_ADDR      : reg_read_data_d = {1'b0, TD_edc_stall_pattern6, 1'b0, TD_edc_stall_pattern5};
      `DIDT_TD_EDC_STALL_PATTERN_7_DEC_ADDR      : reg_read_data_d = {17'b0, TD_edc_stall_pattern7};
      `DIDT_TD_EDC_TIMER_PERIOD_DEC_ADDR         : reg_read_data_d = {18'b0, TD_edc_timer_period_reg};
      `DIDT_TD_THROTTLE_CTRL_DEC_ADDR         : reg_read_data_d = {28'b0, TD_gc_edc_only_mode, TD_pwrbrk_stall_en, TD_pcc_stall_en, TD_gc_edc_stall_en};
      `DIDT_TD_EDC_STATUS_DEC_ADDR      : reg_read_data_d = {{(32-`DIDT_TD_EDC_STATUS_EDC_THROTTLE_LEVEL_SIZE - `DIDT_TD_EDC_STATUS_EDC_FSM_STATE_SIZE){1'b0}}, TD_edc_throttle_level, TD_edc_fsm_state};
      `DIDT_TD_EDC_PCC_PERF_COUNTER_DEC_ADDR      : reg_read_data_d = {TD_edc_pcc_perf_counter};

	  `DIDT_TD_EDC_STALL_DELAY_1_DEC_ADDR	:	reg_read_data_d = {{`DIDT_TD_EDC_STALL_DELAY_1_UNUSED_SIZE{1'b0}}, TD_edc_stall_delay_1[(31 -`DIDT_TD_EDC_STALL_DELAY_1_UNUSED_SIZE):0]};

	  `DIDT_TD_EDC_STALL_DELAY_2_DEC_ADDR	:	reg_read_data_d = {{`DIDT_TD_EDC_STALL_DELAY_2_UNUSED_SIZE{1'b0}}, TD_edc_stall_delay_2[(31 -`DIDT_TD_EDC_STALL_DELAY_2_UNUSED_SIZE):0]};

	  `DIDT_TD_EDC_STALL_DELAY_3_DEC_ADDR	:	reg_read_data_d = {{`DIDT_TD_EDC_STALL_DELAY_3_UNUSED_SIZE{1'b0}}, TD_edc_stall_delay_3[(31 -`DIDT_TD_EDC_STALL_DELAY_3_UNUSED_SIZE):0]};

      `DIDT_TD_EDC_OVERFLOW_DEC_ADDR      : reg_read_data_d = {{(32-`DIDT_TD_EDC_OVERFLOW_EDC_THROTTLE_LEVEL_OVERFLOW_COUNTER_SIZE - `DIDT_TD_EDC_OVERFLOW_EDC_ROLLING_POWER_DELTA_OVERFLOW_SIZE){1'b0}}, TD_edc_throttle_level_overflow_counter, TD_edc_rolling_power_delta_overflow};
      `DIDT_TD_EDC_ROLLING_POWER_DELTA_DEC_ADDR      : reg_read_data_d = TD_edc_rolling_power_delta;

      `DIDT_TCP_CTRL0_DEC_ADDR      : reg_read_data_d = {2'h0, didt_TCP_didt_throttle_mode, didt_TCP_didt_rlc_stall_level_sel, didt_TCP_didt_rlc_force_stall_en, didt_TCP_stall_event_counter_clear, didt_TCP_stall_event_enable, didt_TCP_auto_mpd_enable, didt_TCP_hi_power_threshold, didt_TCP_stall_auto_release_enable, didt_TCP_tuning_enable, didt_TCP_stall_ctrl_enable, didt_TCP_didt_clk_en_override, didt_TCP_didt_ctrl_rst, didt_TCP_phase_offset, didt_TCP_didt_ctrl_en};
      `DIDT_TCP_CTRL1_DEC_ADDR      : reg_read_data_d = didt_TCP_ctrl1;
      `DIDT_TCP_CTRL2_DEC_ADDR      : reg_read_data_d = {1'b0, didt_TCP_long_term_interval_ratio, 1'b0, didt_TCP_short_term_interval_size, 2'h0, didt_TCP_max_power_delta};
      `DIDT_TCP_CTRL3_DEC_ADDR      : reg_read_data_d = {3'h0, didt_TCP_didt_stall_delay_en, didt_TCP_didt_force_stall, didt_TCP_didt_stall_sel, didt_TCP_qualify_stall_en, didt_TCP_se_didt_level_comb_en, didt_TCP_gc_didt_level_comb_en, didt_TCP_didt_stall_pattern_bit_nums, didt_TCP_didt_power_level_lowbit, didt_TCP_didt_trigger_throttle_lowbit, didt_TCP_throttle_policy, didt_TCP_gc_didt_clk_en_override, didt_TCP_gc_didt_enable};
      `DIDT_TCP_CTRL_OCP_DEC_ADDR      : reg_read_data_d = {16'b0, didt_TCP_ocp_max_power};
      `DIDT_TCP_STALL_CTRL_DEC_ADDR      : reg_read_data_d = {8'h0, didt_TCP_didt_max_stalls_allowed_lo, didt_TCP_didt_max_stalls_allowed_hi, didt_TCP_stall_delay_lo, didt_TCP_stall_delay_hi};
      `DIDT_TCP_TUNING_CTRL_DEC_ADDR      : reg_read_data_d = didt_TCP_tuning_ctrl;
      `DIDT_TCP_STALL_AUTO_RELEASE_CTRL_DEC_ADDR      : reg_read_data_d = didt_TCP_stall_auto_release_ctrl;

      `DIDT_TCP_WEIGHT0_3_DEC_ADDR  : reg_read_data_d = didt_TCP_weight_0;
      `DIDT_TCP_WEIGHT4_7_DEC_ADDR  : reg_read_data_d = didt_TCP_weight_1;
      `DIDT_TCP_WEIGHT8_11_DEC_ADDR : reg_read_data_d = didt_TCP_weight_2;

      `DIDT_TCP_MPD_SCALE_FACTOR_DEC_ADDR : reg_read_data_d = didt_TCP_mpd_scale_factor;

      `DIDT_TCP_STALL_RELEASE_CNTL0_DEC_ADDR : reg_read_data_d = {8'b0, didt_TCP_didt_release_delay_lo, didt_TCP_didt_release_delay_hi, didt_TCP_didt_stall_cntl_sel, didt_TCP_didt_stall_release_cntl_en};
      `DIDT_TCP_STALL_RELEASE_CNTL1_DEC_ADDR : reg_read_data_d = {12'b0, didt_TCP_didt_incr_release_allowed_lo, didt_TCP_didt_base_release_allowed_lo, didt_TCP_didt_incr_release_allowed_hi, didt_TCP_didt_base_release_allowed_hi};
      `DIDT_TCP_STALL_RELEASE_CNTL_STATUS_DEC_ADDR      : reg_read_data_d = {{(32-`DIDT_TCP_STALL_RELEASE_CNTL_STATUS_DIDT_STALL_RELEASE_CNTL_FSM_STATE_SIZE){1'b0}}, didt_TCP_didt_stall_release_cntl_fsm_state};     

	  `DIDT_TCP_STALL_PATTERN_1_2_DEC_ADDR      : reg_read_data_d = {1'b0, didt_TCP_stall_pattern2, 1'b0, didt_TCP_stall_pattern1};
	  `DIDT_TCP_STALL_PATTERN_3_4_DEC_ADDR      : reg_read_data_d = {1'b0, didt_TCP_stall_pattern4, 1'b0, didt_TCP_stall_pattern3};
	  `DIDT_TCP_STALL_PATTERN_5_6_DEC_ADDR      : reg_read_data_d = {1'b0, didt_TCP_stall_pattern6, 1'b0, didt_TCP_stall_pattern5};
      `DIDT_TCP_STALL_PATTERN_7_DEC_ADDR        : reg_read_data_d = {17'h0, didt_TCP_stall_pattern7};

      `DIDT_TCP_STALL_EVENT_COUNTER_DEC_ADDR        : reg_read_data_d = didt_TCP_stall_event_counter;

      `DIDT_TCP_EDC_CTRL_DEC_ADDR      : reg_read_data_d = {7'h0, TCP_edc_level_comb_adapt_mode_en, TCP_edc_level_mode_sel, TCP_se_edc_level_comb_en,TCP_gc_edc_level_comb_en, TCP_gc_edc_stall_policy, TCP_gc_edc_en, TCP_edc_allow_write_pwrdelta, TCP_edc_stall_pattern_bit_nums, TCP_edc_trigger_throttle_lowbit, TCP_edc_force_stall, TCP_edc_clk_en_override, TCP_edc_sw_rst, TCP_edc_en};
      `DIDT_TCP_EDC_THRESHOLD_DEC_ADDR      : reg_read_data_d = TCP_edc_threshold_reg;
      `DIDT_TCP_EDC_STALL_PATTERN_1_2_DEC_ADDR      : reg_read_data_d = {1'b0, TCP_edc_stall_pattern2, 1'b0, TCP_edc_stall_pattern1};
      `DIDT_TCP_EDC_STALL_PATTERN_3_4_DEC_ADDR      : reg_read_data_d = {1'b0, TCP_edc_stall_pattern4, 1'b0, TCP_edc_stall_pattern3};
      `DIDT_TCP_EDC_STALL_PATTERN_5_6_DEC_ADDR      : reg_read_data_d = {1'b0, TCP_edc_stall_pattern6, 1'b0, TCP_edc_stall_pattern5};
      `DIDT_TCP_EDC_STALL_PATTERN_7_DEC_ADDR      : reg_read_data_d = {17'b0, TCP_edc_stall_pattern7};
      `DIDT_TCP_EDC_TIMER_PERIOD_DEC_ADDR         : reg_read_data_d = {18'b0, TCP_edc_timer_period_reg};
      `DIDT_TCP_THROTTLE_CTRL_DEC_ADDR         : reg_read_data_d = {28'b0, TCP_gc_edc_only_mode, TCP_pwrbrk_stall_en, TCP_pcc_stall_en, TCP_gc_edc_stall_en};
      `DIDT_TCP_EDC_STATUS_DEC_ADDR      : reg_read_data_d = {{(32-`DIDT_TCP_EDC_STATUS_EDC_THROTTLE_LEVEL_SIZE - `DIDT_TCP_EDC_STATUS_EDC_FSM_STATE_SIZE){1'b0}}, TCP_edc_throttle_level, TCP_edc_fsm_state};
      `DIDT_TCP_EDC_PCC_PERF_COUNTER_DEC_ADDR      : reg_read_data_d = {TCP_edc_pcc_perf_counter};

	  `DIDT_TCP_EDC_STALL_DELAY_1_DEC_ADDR	:	reg_read_data_d = {{`DIDT_TCP_EDC_STALL_DELAY_1_UNUSED_SIZE{1'b0}}, TCP_edc_stall_delay_1[(31 -`DIDT_TCP_EDC_STALL_DELAY_1_UNUSED_SIZE):0]};

	  `DIDT_TCP_EDC_STALL_DELAY_2_DEC_ADDR	:	reg_read_data_d = {{`DIDT_TCP_EDC_STALL_DELAY_2_UNUSED_SIZE{1'b0}}, TCP_edc_stall_delay_2[(31 -`DIDT_TCP_EDC_STALL_DELAY_2_UNUSED_SIZE):0]};

	  `DIDT_TCP_EDC_STALL_DELAY_3_DEC_ADDR	:	reg_read_data_d = {{`DIDT_TCP_EDC_STALL_DELAY_3_UNUSED_SIZE{1'b0}}, TCP_edc_stall_delay_3[(31 -`DIDT_TCP_EDC_STALL_DELAY_3_UNUSED_SIZE):0]};

      `DIDT_TCP_EDC_OVERFLOW_DEC_ADDR      : reg_read_data_d = {{(32-`DIDT_TCP_EDC_OVERFLOW_EDC_THROTTLE_LEVEL_OVERFLOW_COUNTER_SIZE - `DIDT_TCP_EDC_OVERFLOW_EDC_ROLLING_POWER_DELTA_OVERFLOW_SIZE){1'b0}}, TCP_edc_throttle_level_overflow_counter, TCP_edc_rolling_power_delta_overflow};
      `DIDT_TCP_EDC_ROLLING_POWER_DELTA_DEC_ADDR      : reg_read_data_d = TCP_edc_rolling_power_delta;

      default:
       reg_read_data_d = `TARG_GRBM_READ_DATA_WIDTH'h0;
      endcase // case (didt_ind_index)
  else if (rd_didt_index_auto_incr_en)
       reg_read_data_d = {{(32- `DIDT_INDEX_AUTO_INCR_EN_REG_SIZE){1'b0}},didt_index_auto_incr_en};
  else
       reg_read_data_d = `TARG_GRBM_READ_DATA_WIDTH'h0;
end // always @ (*)


assign reg_read_valid_d = rd_ind_data | rd_ind_index | rd_didt_index_auto_incr_en;

//always @(posedge reg_clk)
//begin
//  if (~resetb) begin
//    reg_read_valid <= 1'b0;
//    reg_read_data  <= `TARG_GRBM_READ_DATA_WIDTH'h0;
//
//  end
//  else begin
//    reg_read_valid <= reg_read_valid_d;
//    reg_read_data  <= reg_read_data_d;
//  end
//end
assign reg_read_valid = reg_read_valid_d;
assign reg_read_data  = reg_read_data_d;

endmodule // didt_regs
